/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/

/*
	This is an optimized version of Dojo, built for deployment and not for
	development. To get sources and documentation, please visit:

		http://dojotoolkit.org
*/

(function(
	userConfig,
	defaultConfig
){
	// summary:
	//		This is the "source loader" and is the entry point for Dojo during development. You may also load Dojo with
	//		any AMD-compliant loader via the package main module dojo/main.
	// description:
	//		This is the "source loader" for Dojo. It provides an AMD-compliant loader that can be configured
	//		to operate in either synchronous or asynchronous modes. After the loader is defined, dojo is loaded
	//		IAW the package main module dojo/main. In the event you wish to use a foreign loader, you may load dojo as a package
	//		via the package main module dojo/main and this loader is not required; see dojo/package.json for details.
	//
	//		In order to keep compatibility with the v1.x line, this loader includes additional machinery that enables
	//		the dojo.provide, dojo.require et al API. This machinery is loaded by default, but may be dynamically removed
	//		via the has.js API and statically removed via the build system.
	//
	//		This loader includes sniffing machinery to determine the environment; the following environments are supported:
	//
	//		- browser
	//		- node.js
	//		- rhino
	//
	//		This is the so-called "source loader". As such, it includes many optional features that may be discarded by
	//		building a customized version with the build system.

	// Design and Implementation Notes
	//
	// This is a dojo-specific adaption of bdLoad, donated to the dojo foundation by Altoviso LLC.
	//
	// This function defines an AMD-compliant (http://wiki.commonjs.org/wiki/Modules/AsynchronousDefinition)
	// loader that can be configured to operate in either synchronous or asynchronous modes.
	//
	// Since this machinery implements a loader, it does not have the luxury of using a load system and/or
	// leveraging a utility library. This results in an unpleasantly long file; here is a road map of the contents:
	//
	//	 1. Small library for use implementing the loader.
	//	 2. Define the has.js API; this is used throughout the loader to bracket features.
	//	 3. Define the node.js and rhino sniffs and sniff.
	//	 4. Define the loader's data.
	//	 5. Define the configuration machinery.
	//	 6. Define the script element sniffing machinery and sniff for configuration data.
	//	 7. Configure the loader IAW the provided user, default, and sniffing data.
	//	 8. Define the global require function.
	//	 9. Define the module resolution machinery.
	//	10. Define the module and plugin module definition machinery
	//	11. Define the script injection machinery.
	//	12. Define the window load detection.
	//	13. Define the logging API.
	//	14. Define the tracing API.
	//	16. Define the AMD define function.
	//	17. Define the dojo v1.x provide/require machinery--so called "legacy" modes.
	//	18. Publish global variables.
	//
	// Language and Acronyms and Idioms
	//
	// moduleId: a CJS module identifier, (used for public APIs)
	// mid: moduleId (used internally)
	// packageId: a package identifier (used for public APIs)
	// pid: packageId (used internally); the implied system or default package has pid===""
	// pack: package is used internally to reference a package object (since javascript has reserved words including "package")
	// prid: plugin resource identifier
	// The integer constant 1 is used in place of true and 0 in place of false.

	// define a minimal library to help build the loader
	var	noop = function(){
		},

		isEmpty = function(it){
			for(var p in it){
				return 0;
			}
			return 1;
		},

		toString = {}.toString,

		isFunction = function(it){
			return toString.call(it) == "[object Function]";
		},

		isString = function(it){
			return toString.call(it) == "[object String]";
		},

		isArray = function(it){
			return toString.call(it) == "[object Array]";
		},

		forEach = function(vector, callback){
			if(vector){
				for(var i = 0; vector[i];){
					callback(vector[i++]);
				}
			}
		},

		mix = function(dest, src){
			for(var p in src){
				dest[p] = src[p];
			}
			return dest;
		},

		makeError = function(error, info){
			return mix(new Error(error), {src:"dojoLoader", info:info});
		},

		uidSeed = 1,

		uid = function(){
			// Returns a unique identifier (within the lifetime of the document) of the form /_d+/.
			return "_" + uidSeed++;
		},

		// FIXME: how to doc window.require() api

		// this will be the global require function; define it immediately so we can start hanging things off of it
		req = function(
			config,		  //(object, optional) hash of configuration properties
			dependencies, //(array of commonjs.moduleId, optional) list of modules to be loaded before applying callback
			callback	  //(function, optional) lambda expression to apply to module values implied by dependencies
		){
			return contextRequire(config, dependencies, callback, 0, req);
		},

		// the loader uses the has.js API to control feature inclusion/exclusion; define then use throughout
		global = this,

		doc = global.document,

		element = doc && doc.createElement("DiV"),

		has = req.has = function(name){
			return isFunction(hasCache[name]) ? (hasCache[name] = hasCache[name](global, doc, element)) : hasCache[name];
		},

		hasCache = has.cache = defaultConfig.hasCache;

	has.add = function(name, test, now, force){
		(hasCache[name]===undefined || force) && (hasCache[name] = test);
		return now && has(name);
	};

	 0 && has.add("host-node", userConfig.has && "host-node" in userConfig.has ?
		userConfig.has["host-node"] :
		(typeof process == "object" && process.versions && process.versions.node && process.versions.v8));
	if( 0 ){
		// fixup the default config for node.js environment
		require("./_base/configNode.js").config(defaultConfig);
		// remember node's require (with respect to baseUrl==dojo's root)
		defaultConfig.loaderPatch.nodeRequire = require;
	}

	 0 && has.add("host-rhino", userConfig.has && "host-rhino" in userConfig.has ?
		userConfig.has["host-rhino"] :
		(typeof load == "function" && (typeof Packages == "function" || typeof Packages == "object")));
	if( 0 ){
		// owing to rhino's lame feature that hides the source of the script, give the user a way to specify the baseUrl...
		for(var baseUrl = userConfig.baseUrl || ".", arg, rhinoArgs = this.arguments, i = 0; i < rhinoArgs.length;){
			arg = (rhinoArgs[i++] + "").split("=");
			if(arg[0] == "baseUrl"){
				baseUrl = arg[1];
				break;
			}
		}
		load(baseUrl + "/_base/configRhino.js");
		rhinoDojoConfig(defaultConfig, baseUrl, rhinoArgs);
	}

	// userConfig has tests override defaultConfig has tests; do this after the environment detection because
	// the environment detection usually sets some has feature values in the hasCache.
	for(var p in userConfig.has){
		has.add(p, userConfig.has[p], 0, 1);
	}

	//
	// define the loader data
	//

	// the loader will use these like symbols if the loader has the traceApi; otherwise
	// define magic numbers so that modules can be provided as part of defaultConfig
	var	requested = 1,
		arrived = 2,
		nonmodule = 3,
		executing = 4,
		executed = 5;

	if( 0 ){
		// these make debugging nice; but using strings for symbols is a gross rookie error; don't do it for production code
		requested = "requested";
		arrived = "arrived";
		nonmodule = "not-a-module";
		executing = "executing";
		executed = "executed";
	}

	var legacyMode = 0,
		sync = "sync",
		xd = "xd",
		syncExecStack = [],
		dojoRequirePlugin = 0,
		checkDojoRequirePlugin = noop,
		transformToAmd = noop,
		getXhr;
	if( 1 ){
		req.isXdUrl = noop;

		req.initSyncLoader = function(dojoRequirePlugin_, checkDojoRequirePlugin_, transformToAmd_){
			// the first dojo/_base/loader loaded gets to define these variables; they are designed to work
			// in the presence of zero to many mapped dojo/_base/loaders
			if(!dojoRequirePlugin){
				dojoRequirePlugin = dojoRequirePlugin_;
				checkDojoRequirePlugin = checkDojoRequirePlugin_;
				transformToAmd = transformToAmd_;
			}

			return {
				sync:sync,
				requested:requested,
				arrived:arrived,
				nonmodule:nonmodule,
				executing:executing,
				executed:executed,
				syncExecStack:syncExecStack,
				modules:modules,
				execQ:execQ,
				getModule:getModule,
				injectModule:injectModule,
				setArrived:setArrived,
				signal:signal,
				finishExec:finishExec,
				execModule:execModule,
				dojoRequirePlugin:dojoRequirePlugin,
				getLegacyMode:function(){return legacyMode;},
				guardCheckComplete:guardCheckComplete
			};
		};

		if( 1 ){
			// in legacy sync mode, the loader needs a minimal XHR library

			var locationProtocol = location.protocol,
				locationHost = location.host;
			req.isXdUrl = function(url){
				if(/^\./.test(url)){
					// begins with a dot is always relative to page URL; therefore not xdomain
					return false;
				}
				if(/^\/\//.test(url)){
					// for v1.6- backcompat, url starting with // indicates xdomain
					return true;
				}
				// get protocol and host
				// \/+ takes care of the typical file protocol that looks like file:///drive/path/to/file
				// locationHost is falsy if file protocol => if locationProtocol matches and is "file:", || will return false
				var match = url.match(/^([^\/\:]+\:)\/+([^\/]+)/);
				return match && (match[1] != locationProtocol || (locationHost && match[2] != locationHost));
			};


			// note: to get the file:// protocol to work in FF, you must set security.fileuri.strict_origin_policy to false in about:config
			 1 || has.add("dojo-xhr-factory", 1);
			has.add("dojo-force-activex-xhr",  1  && !doc.addEventListener && window.location.protocol == "file:");
			has.add("native-xhr", typeof XMLHttpRequest != "undefined");
			if(has("native-xhr") && !has("dojo-force-activex-xhr")){
				getXhr = function(){
					return new XMLHttpRequest();
				};
			}else{
				// if in the browser an old IE; find an xhr
				for(var XMLHTTP_PROGIDS = ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP', 'Msxml2.XMLHTTP.4.0'], progid, i = 0; i < 3;){
					try{
						progid = XMLHTTP_PROGIDS[i++];
						if(new ActiveXObject(progid)){
							// this progid works; therefore, use it from now on
							break;
						}
					}catch(e){
						// squelch; we're just trying to find a good ActiveX progid
						// if they all fail, then progid ends up as the last attempt and that will signal the error
						// the first time the client actually tries to exec an xhr
					}
				}
				getXhr = function(){
					return new ActiveXObject(progid);
				};
			}
			req.getXhr = getXhr;

			has.add("dojo-gettext-api", 1);
			req.getText = function(url, async, onLoad){
				var xhr = getXhr();
				xhr.open('GET', fixupUrl(url), false);
				xhr.send(null);
				if(xhr.status == 200 || (!location.host && !xhr.status)){
					if(onLoad){
						onLoad(xhr.responseText, async);
					}
				}else{
					throw makeError("xhrFailed", xhr.status);
				}
				return xhr.responseText;
			};
		}
	}else{
		req.async = 1;
	}

	//
	// loader eval
	//
	var eval_ =
		// use the function constructor so our eval is scoped close to (but not in) in the global space with minimal pollution
		new Function('return eval(arguments[0]);');

	req.eval =
		function(text, hint){
			return eval_(text + "\r\n//# sourceURL=" + hint);
		};

	//
	// loader micro events API
	//
	var listenerQueues = {},
		error = "error",
		signal = req.signal = function(type, args){
			var queue = listenerQueues[type];
			// notice we run a copy of the queue; this allows listeners to add/remove
			// other listeners without affecting this particular signal
			forEach(queue && queue.slice(0), function(listener){
				listener.apply(null, isArray(args) ? args : [args]);
			});
		},
		on = req.on = function(type, listener){
			// notice a queue is not created until a client actually connects
			var queue = listenerQueues[type] || (listenerQueues[type] = []);
			queue.push(listener);
			return {
				remove:function(){
					for(var i = 0; i<queue.length; i++){
						if(queue[i]===listener){
							queue.splice(i, 1);
							return;
						}
					}
				}
			};
		};

	// configuration machinery; with an optimized/built defaultConfig, all configuration machinery can be discarded
	// lexical variables hold key loader data structures to help with minification; these may be completely,
	// one-time initialized by defaultConfig for optimized/built versions
	var
		aliases
			// a vector of pairs of [regexs or string, replacement] => (alias, actual)
			= [],

		paths
			// CommonJS paths
			= {},

		pathsMapProg
			// list of (from-path, to-path, regex, length) derived from paths;
			// a "program" to apply paths; see computeMapProg
			= [],

		packs
			// a map from packageId to package configuration object; see fixupPackageInfo
			= {},

		map = req.map
			// AMD map config variable; dojo/_base/kernel needs req.map to figure out the scope map
			= {},

		mapProgs
			// vector of quads as described by computeMapProg; map-key is AMD map key, map-value is AMD map value
			= [],

		modules
			// A hash:(mid) --> (module-object) the module namespace
			//
			// pid: the package identifier to which the module belongs (e.g., "dojo"); "" indicates the system or default package
			// mid: the fully-resolved (i.e., mappings have been applied) module identifier without the package identifier (e.g., "dojo/io/script")
			// url: the URL from which the module was retrieved
			// pack: the package object of the package to which the module belongs
			// executed: 0 => not executed; executing => in the process of traversing deps and running factory; executed => factory has been executed
			// deps: the dependency vector for this module (vector of modules objects)
			// def: the factory for this module
			// result: the result of the running the factory for this module
			// injected: (0 | requested | arrived) the status of the module; nonmodule means the resource did not call define
			// load: plugin load function; applicable only for plugins
			//
			// Modules go through several phases in creation:
			//
			// 1. Requested: some other module's definition or a require application contained the requested module in
			//	  its dependency vector or executing code explicitly demands a module via req.require.
			//
			// 2. Injected: a script element has been appended to the insert-point element demanding the resource implied by the URL
			//
			// 3. Loaded: the resource injected in [2] has been evaluated.
			//
			// 4. Defined: the resource contained a define statement that advised the loader about the module. Notice that some
			//	  resources may just contain a bundle of code and never formally define a module via define
			//
			// 5. Evaluated: the module was defined via define and the loader has evaluated the factory and computed a result.
			= {},

		cacheBust
			// query string to append to module URLs to bust browser cache
			= "",

		cache
			// hash:(mid | url)-->(function | string)
			//
			// A cache of resources. The resources arrive via a config.cache object, which is a hash from either mid --> function or
			// url --> string. The url key is distinguished from the mid key by always containing the prefix "url:". url keys as provided
			// by config.cache always have a string value that represents the contents of the resource at the given url. mid keys as provided
			// by configl.cache always have a function value that causes the same code to execute as if the module was script injected.
			//
			// Both kinds of key-value pairs are entered into cache via the function consumePendingCache, which may relocate keys as given
			// by any mappings *iff* the config.cache was received as part of a module resource request.
			//
			// Further, for mid keys, the implied url is computed and the value is entered into that key as well. This allows mapped modules
			// to retrieve cached items that may have arrived consequent to another namespace.
			//
			 = {},

		urlKeyPrefix
			// the prefix to prepend to a URL key in the cache.
			= "url:",

		pendingCacheInsert
			// hash:(mid)-->(function)
			//
			// Gives a set of cache modules pending entry into cache. When cached modules are published to the loader, they are
			// entered into pendingCacheInsert; modules are then pressed into cache upon (1) AMD define or (2) upon receiving another
			// independent set of cached modules. (1) is the usual case, and this case allows normalizing mids given in the pending
			// cache for the local configuration, possibly relocating modules.
			 = {},

		dojoSniffConfig
			// map of configuration variables
			// give the data-dojo-config as sniffed from the document (if any)
			= {},

		insertPointSibling
			// the nodes used to locate where scripts are injected into the document
			= 0;

	if( 1 ){
		var consumePendingCacheInsert = function(referenceModule){
				var p, item, match, now, m;
				for(p in pendingCacheInsert){
					item = pendingCacheInsert[p];
					match = p.match(/^url\:(.+)/);
					if(match){
						cache[urlKeyPrefix + toUrl(match[1], referenceModule)] =  item;
					}else if(p=="*now"){
						now = item;
					}else if(p!="*noref"){
						m = getModuleInfo(p, referenceModule);
						cache[m.mid] = cache[urlKeyPrefix + m.url] = item;
					}
				}
				if(now){
					now(createRequire(referenceModule));
				}
				pendingCacheInsert = {};
			},

			escapeString = function(s){
				return s.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(c){ return "\\" + c; });
			},

			computeMapProg = function(map, dest){
				// This routine takes a map as represented by a JavaScript object and initializes dest, a vector of
				// quads of (map-key, map-value, refex-for-map-key, length-of-map-key), sorted decreasing by length-
				// of-map-key. The regex looks for the map-key followed by either "/" or end-of-string at the beginning
				// of a the search source. Notice the map-value is irrelevant to the algorithm
				dest.splice(0, dest.length);
				for(var p in map){
					dest.push([
						p,
						map[p],
						new RegExp("^" + escapeString(p) + "(\/|$)"),
						p.length]);
				}
				dest.sort(function(lhs, rhs){ return rhs[3] - lhs[3]; });
				return dest;
			},

			computeAliases = function(config, dest){
				forEach(config, function(pair){
					// take a fixed-up copy...
					dest.push([isString(pair[0]) ? new RegExp("^" + escapeString(pair[0]) + "$") : pair[0], pair[1]]);
				});
			},


			fixupPackageInfo = function(packageInfo){
				// calculate the precise (name, location, main, mappings) for a package
				var name = packageInfo.name;
				if(!name){
					// packageInfo must be a string that gives the name
					name = packageInfo;
					packageInfo = {name:name};
				}
				packageInfo = mix({main:"main"}, packageInfo);
				packageInfo.location = packageInfo.location ? packageInfo.location : name;

				// packageMap is deprecated in favor of AMD map
				if(packageInfo.packageMap){
					map[name] = packageInfo.packageMap;
				}

				if(!packageInfo.main.indexOf("./")){
					packageInfo.main = packageInfo.main.substring(2);
				}

				// now that we've got a fully-resolved package object, push it into the configuration
				packs[name] = packageInfo;
			},

			delayedModuleConfig
				// module config cannot be consumed until the loader is completely initialized; therefore, all
				// module config detected during booting is memorized and applied at the end of loader initialization
				// TODO: this is a bit of a kludge; all config should be moved to end of loader initialization, but
				// we'll delay this chore and do it with a final loader 1.x cleanup after the 2.x loader prototyping is complete
				= [],


			config = function(config, booting, referenceModule){
				for(var p in config){
					if(p=="waitSeconds"){
						req.waitms = (config[p] || 0) * 1000;
					}
					if(p=="cacheBust"){
						cacheBust = config[p] ? (isString(config[p]) ? config[p] : (new Date()).getTime() + "") : "";
					}
					if(p=="baseUrl" || p=="combo"){
						req[p] = config[p];
					}
					if( 1  && p=="async"){
						// falsy or "sync" => legacy sync loader
						// "xd" => sync but loading xdomain tree and therefore loading asynchronously (not configurable, set automatically by the loader)
						// "legacyAsync" => permanently in "xd" by choice
						// "debugAtAllCosts" => trying to load everything via script injection (not implemented)
						// otherwise, must be truthy => AMD
						// legacyMode: sync | legacyAsync | xd | false
						var mode = config[p];
						req.legacyMode = legacyMode = (isString(mode) && /sync|legacyAsync/.test(mode) ? mode : (!mode ? sync : false));
						req.async = !legacyMode;
					}
					if(config[p]!==hasCache){
						// accumulate raw config info for client apps which can use this to pass their own config
						req.rawConfig[p] = config[p];
						p!="has" && has.add("config-"+p, config[p], 0, booting);
					}
				}

				// make sure baseUrl exists
				if(!req.baseUrl){
					req.baseUrl = "./";
				}
				// make sure baseUrl ends with a slash
				if(!/\/$/.test(req.baseUrl)){
					req.baseUrl += "/";
				}

				// now do the special work for has, packages, packagePaths, paths, aliases, and cache

				for(p in config.has){
					has.add(p, config.has[p], 0, booting);
				}

				// for each package found in any packages config item, augment the packs map owned by the loader
				forEach(config.packages, fixupPackageInfo);

				// for each packagePath found in any packagePaths config item, augment the packageConfig
				// packagePaths is deprecated; remove in 2.0
				for(baseUrl in config.packagePaths){
					forEach(config.packagePaths[baseUrl], function(packageInfo){
						var location = baseUrl + "/" + packageInfo;
						if(isString(packageInfo)){
							packageInfo = {name:packageInfo};
						}
						packageInfo.location = location;
						fixupPackageInfo(packageInfo);
					});
				}

				// notice that computeMapProg treats the dest as a reference; therefore, if/when that variable
				// is published (see dojo-publish-privates), the published variable will always hold a valid value.

				// this must come after all package processing since package processing may mutate map
				computeMapProg(mix(map, config.map), mapProgs);
				forEach(mapProgs, function(item){
					item[1] = computeMapProg(item[1], []);
					if(item[0]=="*"){
						mapProgs.star = item;
					}
				});

				// push in any paths and recompute the internal pathmap
				computeMapProg(mix(paths, config.paths), pathsMapProg);

				// aliases
				computeAliases(config.aliases, aliases);

				if(booting){
					delayedModuleConfig.push({config:config.config});
				}else{
					for(p in config.config){
						var module = getModule(p, referenceModule);
						module.config = mix(module.config || {}, config.config[p]);
					}
				}

				// push in any new cache values
				if(config.cache){
					consumePendingCacheInsert();
					pendingCacheInsert = config.cache;
					if(config.cache["*noref"]){
						consumePendingCacheInsert();
					}
				}

				signal("config", [config, req.rawConfig]);
			};

		//
		// execute the various sniffs; userConfig can override and value
		//

		if(has("dojo-cdn") ||  1 ){
			// the sniff regex looks for a src attribute ending in dojo.js, optionally preceded with a path.
			// match[3] returns the path to dojo.js (if any) without the trailing slash. This is used for the
			// dojo location on CDN deployments and baseUrl when either/both of these are not provided
			// explicitly in the config data; this is the 1.6- behavior.

			var scripts = doc.getElementsByTagName("script"),
				i = 0,
				script, dojoDir, src, match;
			while(i < scripts.length){
				script = scripts[i++];
				if((src = script.getAttribute("src")) && (match = src.match(/(((.*)\/)|^)dojo\.js(\W|$)/i))){
					// sniff dojoDir and baseUrl
					dojoDir = match[3] || "";
					defaultConfig.baseUrl = defaultConfig.baseUrl || dojoDir;

					// remember an insertPointSibling
					insertPointSibling = script;
				}

				// sniff configuration on attribute in script element
				if((src = (script.getAttribute("data-dojo-config") || script.getAttribute("djConfig")))){
					dojoSniffConfig = req.eval("({ " + src + " })", "data-dojo-config");

					// remember an insertPointSibling
					insertPointSibling = script;
				}

				// sniff requirejs attribute
				if( 0 ){
					if((src = script.getAttribute("data-main"))){
						dojoSniffConfig.deps = dojoSniffConfig.deps || [src];
					}
				}
			}
		}

		if( 0 ){
			// pass down doh.testConfig from parent as if it were a data-dojo-config
			try{
				if(window.parent != window && window.parent.require){
					var doh = window.parent.require("doh");
					doh && mix(dojoSniffConfig, doh.testConfig);
				}
			}catch(e){}
		}

		// configure the loader; let the user override defaults
		req.rawConfig = {};
		config(defaultConfig, 1);

		// do this before setting userConfig/sniffConfig to allow userConfig/sniff overrides
		if(has("dojo-cdn")){
			packs.dojo.location = dojoDir;
			if(dojoDir){
				dojoDir += "/";
			}
			packs.dijit.location = dojoDir + "../dijit/";
			packs.dojox.location = dojoDir + "../dojox/";
		}

		config(userConfig, 1);
		config(dojoSniffConfig, 1);

	}else{
		// no config API, assume defaultConfig has everything the loader needs...for the entire lifetime of the application
		paths = defaultConfig.paths;
		pathsMapProg = defaultConfig.pathsMapProg;
		packs = defaultConfig.packs;
		aliases = defaultConfig.aliases;
		mapProgs = defaultConfig.mapProgs;
		modules = defaultConfig.modules;
		cache = defaultConfig.cache;
		cacheBust = defaultConfig.cacheBust;

		// remember the default config for other processes (e.g., dojo/config)
		req.rawConfig = defaultConfig;
	}


	if( 0 ){
		req.combo = req.combo || {add:noop};
		var	comboPending = 0,
			combosPending = [],
			comboPendingTimer = null;
	}


	// build the loader machinery iaw configuration, including has feature tests
	var	injectDependencies = function(module){
			// checkComplete!=0 holds the idle signal; we're not idle if we're injecting dependencies
			guardCheckComplete(function(){
				forEach(module.deps, injectModule);
				if( 0  && comboPending && !comboPendingTimer){
					comboPendingTimer = setTimeout(function() {
						comboPending = 0;
						comboPendingTimer = null;
						req.combo.done(function(mids, url) {
							var onLoadCallback= function(){
								// defQ is a vector of module definitions 1-to-1, onto mids
								runDefQ(0, mids);
								checkComplete();
							};
							combosPending.push(mids);
							injectingModule = mids;
							req.injectUrl(url, onLoadCallback, mids);
							injectingModule = 0;
						}, req);
					}, 0);
				}
			});
		},

		contextRequire = function(a1, a2, a3, referenceModule, contextRequire){
			var module, syntheticMid;
			if(isString(a1)){
				// signature is (moduleId)
				module = getModule(a1, referenceModule, true);
				if(module && module.executed){
					return module.result;
				}
				throw makeError("undefinedModule", a1);
			}
			if(!isArray(a1)){
				// a1 is a configuration
				config(a1, 0, referenceModule);

				// juggle args; (a2, a3) may be (dependencies, callback)
				a1 = a2;
				a2 = a3;
			}
			if(isArray(a1)){
				// signature is (requestList [,callback])
				if(!a1.length){
					a2 && a2();
				}else{
					syntheticMid = "require*" + uid();

					// resolve the request list with respect to the reference module
					for(var mid, deps = [], i = 0; i < a1.length;){
						mid = a1[i++];
						deps.push(getModule(mid, referenceModule));
					}

					// construct a synthetic module to control execution of the requestList, and, optionally, callback
					module = mix(makeModuleInfo("", syntheticMid, 0, ""), {
						injected: arrived,
						deps: deps,
						def: a2 || noop,
						require: referenceModule ? referenceModule.require : req,
						gc: 1 //garbage collect
					});
					modules[module.mid] = module;

					// checkComplete!=0 holds the idle signal; we're not idle if we're injecting dependencies
					injectDependencies(module);

					// try to immediately execute
					// if already traversing a factory tree, then strict causes circular dependency to abort the execution; maybe
					// it's possible to execute this require later after the current traversal completes and avoid the circular dependency.
					// ...but *always* insist on immediate in synch mode
					var strict = checkCompleteGuard && legacyMode!=sync;
					guardCheckComplete(function(){
						execModule(module, strict);
					});
					if(!module.executed){
						// some deps weren't on board or circular dependency detected and strict; therefore, push into the execQ
						execQ.push(module);
					}
					checkComplete();
				}
			}
			return contextRequire;
		},

		createRequire = function(module){
			if(!module){
				return req;
			}
			var result = module.require;
			if(!result){
				result = function(a1, a2, a3){
					return contextRequire(a1, a2, a3, module, result);
				};
				module.require = mix(result, req);
				result.module = module;
				result.toUrl = function(name){
					return toUrl(name, module);
				};
				result.toAbsMid = function(mid){
					return toAbsMid(mid, module);
				};
				if( 0 ){
					result.undef = function(mid){
						req.undef(mid, module);
					};
				}
				if( 1 ){
					result.syncLoadNls = function(mid){
						var nlsModuleInfo = getModuleInfo(mid, module),
							nlsModule = modules[nlsModuleInfo.mid];
						if(!nlsModule || !nlsModule.executed){
							cached = cache[nlsModuleInfo.mid] || cache[urlKeyPrefix + nlsModuleInfo.url];
							if(cached){
								evalModuleText(cached);
								nlsModule = modules[nlsModuleInfo.mid];
							}
						}
						return nlsModule && nlsModule.executed && nlsModule.result;
					};
				}

			}
			return result;
		},

		execQ =
			// The list of modules that need to be evaluated.
			[],

		defQ =
			// The queue of define arguments sent to loader.
			[],

		waiting =
			// The set of modules upon which the loader is waiting for definition to arrive
			{},

		setRequested = function(module){
			module.injected = requested;
			waiting[module.mid] = 1;
			if(module.url){
				waiting[module.url] = module.pack || 1;
			}
			startTimer();
		},

		setArrived = function(module){
			module.injected = arrived;
			delete waiting[module.mid];
			if(module.url){
				delete waiting[module.url];
			}
			if(isEmpty(waiting)){
				clearTimer();
				 1  && legacyMode==xd && (legacyMode = sync);
			}
		},

		execComplete = req.idle =
			// says the loader has completed (or not) its work
			function(){
				return !defQ.length && isEmpty(waiting) && !execQ.length && !checkCompleteGuard;
			},

		runMapProg = function(targetMid, map){
			// search for targetMid in map; return the map item if found; falsy otherwise
			if(map){
			for(var i = 0; i < map.length; i++){
				if(map[i][2].test(targetMid)){
					return map[i];
				}
			}
			}
			return 0;
		},

		compactPath = function(path){
			var result = [],
				segment, lastSegment;
			path = path.replace(/\\/g, '/').split('/');
			while(path.length){
				segment = path.shift();
				if(segment==".." && result.length && lastSegment!=".."){
					result.pop();
					lastSegment = result[result.length - 1];
				}else if(segment!="."){
					result.push(lastSegment= segment);
				} // else ignore "."
			}
			return result.join("/");
		},

		makeModuleInfo = function(pid, mid, pack, url){
			if( 1 ){
				var xd= req.isXdUrl(url);
				return {pid:pid, mid:mid, pack:pack, url:url, executed:0, def:0, isXd:xd, isAmd:!!(xd || (packs[pid] && packs[pid].isAmd))};
			}else{
				return {pid:pid, mid:mid, pack:pack, url:url, executed:0, def:0};
			}
		},

		getModuleInfo_ = function(mid, referenceModule, packs, modules, baseUrl, mapProgs, pathsMapProg, aliases, alwaysCreate){
			// arguments are passed instead of using lexical variables so that this function my be used independent of the loader (e.g., the builder)
			// alwaysCreate is useful in this case so that getModuleInfo never returns references to real modules owned by the loader
			var pid, pack, midInPackage, mapItem, url, result, isRelative, requestedMid;
			requestedMid = mid;
			isRelative = /^\./.test(mid);
			if(/(^\/)|(\:)|(\.js$)/.test(mid) || (isRelative && !referenceModule)){
				// absolute path or protocol of .js filetype, or relative path but no reference module and therefore relative to page
				// whatever it is, it's not a module but just a URL of some sort
				// note: pid===0 indicates the routine is returning an unmodified mid

				return makeModuleInfo(0, mid, 0, mid);
			}else{
				// relative module ids are relative to the referenceModule; get rid of any dots
				mid = compactPath(isRelative ? (referenceModule.mid + "/../" + mid) : mid);
				if(/^\./.test(mid)){
					throw makeError("irrationalPath", mid);
				}
				// at this point, mid is an absolute mid

				// map the mid
				if(referenceModule){
					mapItem = runMapProg(referenceModule.mid, mapProgs);
				}
				mapItem = mapItem || mapProgs.star;
				mapItem = mapItem && runMapProg(mid, mapItem[1]);

				if(mapItem){
					mid = mapItem[1] + mid.substring(mapItem[3]);
					}

				match = mid.match(/^([^\/]+)(\/(.+))?$/);
				pid = match ? match[1] : "";
				if((pack = packs[pid])){
					mid = pid + "/" + (midInPackage = (match[3] || pack.main));
				}else{
					pid = "";
				}

				// search aliases
				var candidateLength = 0,
					candidate = 0;
				forEach(aliases, function(pair){
					var match = mid.match(pair[0]);
					if(match && match.length>candidateLength){
						candidate = isFunction(pair[1]) ? mid.replace(pair[0], pair[1]) : pair[1];
					}
				});
				if(candidate){
					return getModuleInfo_(candidate, 0, packs, modules, baseUrl, mapProgs, pathsMapProg, aliases, alwaysCreate);
				}

				result = modules[mid];
				if(result){
					return alwaysCreate ? makeModuleInfo(result.pid, result.mid, result.pack, result.url) : modules[mid];
				}
			}
			// get here iff the sought-after module does not yet exist; therefore, we need to compute the URL given the
			// fully resolved (i.e., all relative indicators and package mapping resolved) module id

			// note: pid!==0 indicates the routine is returning a url that has .js appended unmodified mid
			mapItem = runMapProg(mid, pathsMapProg);
			if(mapItem){
				url = mapItem[1] + mid.substring(mapItem[3]);
			}else if(pid){
				url = pack.location + "/" + midInPackage;
			}else if(has("config-tlmSiblingOfDojo")){
				url = "../" + mid;
			}else{
				url = mid;
			}
			// if result is not absolute, add baseUrl
			if(!(/(^\/)|(\:)/.test(url))){
				url = baseUrl + url;
			}
			url += ".js";
			return makeModuleInfo(pid, mid, pack, compactPath(url));
		},

		getModuleInfo = function(mid, referenceModule){
			return getModuleInfo_(mid, referenceModule, packs, modules, req.baseUrl, mapProgs, pathsMapProg, aliases);
		},

		resolvePluginResourceId = function(plugin, prid, referenceModule){
			return plugin.normalize ? plugin.normalize(prid, function(mid){return toAbsMid(mid, referenceModule);}) : toAbsMid(prid, referenceModule);
		},

		dynamicPluginUidGenerator = 0,

		getModule = function(mid, referenceModule, immediate){
			// compute and optionally construct (if necessary) the module implied by the mid with respect to referenceModule
			var match, plugin, prid, result;
			match = mid.match(/^(.+?)\!(.*)$/);
			if(match){
				// name was <plugin-module>!<plugin-resource-id>
				plugin = getModule(match[1], referenceModule, immediate);

				if( 1  && legacyMode == sync && !plugin.executed){
					injectModule(plugin);
					if(plugin.injected===arrived && !plugin.executed){
						guardCheckComplete(function(){
							execModule(plugin);
						});
					}
					if(plugin.executed){
						promoteModuleToPlugin(plugin);
					}else{
						// we are in xdomain mode for some reason
						execQ.unshift(plugin);
					}
				}



				if(plugin.executed === executed && !plugin.load){
					// executed the module not knowing it was a plugin
					promoteModuleToPlugin(plugin);
				}

				// if the plugin has not been loaded, then can't resolve the prid and  must assume this plugin is dynamic until we find out otherwise
				if(plugin.load){
					prid = resolvePluginResourceId(plugin, match[2], referenceModule);
					mid = (plugin.mid + "!" + (plugin.dynamic ? ++dynamicPluginUidGenerator + "!" : "") + prid);
				}else{
					prid = match[2];
					mid = plugin.mid + "!" + (++dynamicPluginUidGenerator) + "!waitingForPlugin";
				}
				result = {plugin:plugin, mid:mid, req:createRequire(referenceModule), prid:prid};
			}else{
				result = getModuleInfo(mid, referenceModule);
			}
			return modules[result.mid] || (!immediate && (modules[result.mid] = result));
		},

		toAbsMid = req.toAbsMid = function(mid, referenceModule){
			return getModuleInfo(mid, referenceModule).mid;
		},

		toUrl = req.toUrl = function(name, referenceModule){
			var moduleInfo = getModuleInfo(name+"/x", referenceModule),
				url= moduleInfo.url;
			return fixupUrl(moduleInfo.pid===0 ?
				// if pid===0, then name had a protocol or absolute path; either way, toUrl is the identify function in such cases
				name :
				// "/x.js" since getModuleInfo automatically appends ".js" and we appended "/x" to make name look like a module id
				url.substring(0, url.length-5)
			);
		},

		nonModuleProps = {
			injected: arrived,
			executed: executed,
			def: nonmodule,
			result: nonmodule
		},

		makeCjs = function(mid){
			return modules[mid] = mix({mid:mid}, nonModuleProps);
		},

		cjsRequireModule = makeCjs("require"),
		cjsExportsModule = makeCjs("exports"),
		cjsModuleModule = makeCjs("module"),

		runFactory = function(module, args){
			req.trace("loader-run-factory", [module.mid]);
			var factory = module.def,
				result;
			 1  && syncExecStack.unshift(module);
			if(has("config-dojo-loader-catches")){
				try{
					result= isFunction(factory) ? factory.apply(null, args) : factory;
				}catch(e){
					signal(error, module.result = makeError("factoryThrew", [module, e]));
				}
			}else{
				result= isFunction(factory) ? factory.apply(null, args) : factory;
			}
			module.result = result===undefined && module.cjs ? module.cjs.exports : result;
			 1  && syncExecStack.shift(module);
		},

		abortExec = {},

		defOrder = 0,

		promoteModuleToPlugin = function(pluginModule){
			var plugin = pluginModule.result;
			pluginModule.dynamic = plugin.dynamic;
			pluginModule.normalize = plugin.normalize;
			pluginModule.load = plugin.load;
			return pluginModule;
		},

		resolvePluginLoadQ = function(plugin){
			// plugins is a newly executed module that has a loadQ waiting to run

			// step 1: traverse the loadQ and fixup the mid and prid; remember the map from original mid to new mid
			// recall the original mid was created before the plugin was on board and therefore it was impossible to
			// compute the final mid; accordingly, prid may or may not change, but the mid will definitely change
			var map = {};
			forEach(plugin.loadQ, function(pseudoPluginResource){
				// manufacture and insert the real module in modules
				var prid = resolvePluginResourceId(plugin, pseudoPluginResource.prid, pseudoPluginResource.req.module),
					mid = plugin.dynamic ? pseudoPluginResource.mid.replace(/waitingForPlugin$/, prid) : (plugin.mid + "!" + prid),
					pluginResource = mix(mix({}, pseudoPluginResource), {mid:mid, prid:prid, injected:0});
				if(!modules[mid]){
					// create a new (the real) plugin resource and inject it normally now that the plugin is on board
					injectPlugin(modules[mid] = pluginResource);
				} // else this was a duplicate request for the same (plugin, rid) for a nondynamic plugin

				// pluginResource is really just a placeholder with the wrong mid (because we couldn't calculate it until the plugin was on board)
				// mark is as arrived and delete it from modules; the real module was requested above
				map[pseudoPluginResource.mid] = modules[mid];
				setArrived(pseudoPluginResource);
				delete modules[pseudoPluginResource.mid];
			});
			plugin.loadQ = 0;

			// step2: replace all references to any placeholder modules with real modules
			var substituteModules = function(module){
				for(var replacement, deps = module.deps || [], i = 0; i<deps.length; i++){
					replacement = map[deps[i].mid];
					if(replacement){
						deps[i] = replacement;
					}
				}
			};
			for(var p in modules){
				substituteModules(modules[p]);
			}
			forEach(execQ, substituteModules);
		},

		finishExec = function(module){
			req.trace("loader-finish-exec", [module.mid]);
			module.executed = executed;
			module.defOrder = defOrder++;
			 1  && forEach(module.provides, function(cb){ cb(); });
			if(module.loadQ){
				// the module was a plugin
				promoteModuleToPlugin(module);
				resolvePluginLoadQ(module);
			}
			// remove all occurrences of this module from the execQ
			for(i = 0; i < execQ.length;){
				if(execQ[i] === module){
					execQ.splice(i, 1);
				}else{
					i++;
				}
			}
			// delete references to synthetic modules
			if (/^require\*/.test(module.mid)) {
				delete modules[module.mid];
			}
		},

		circleTrace = [],

		execModule = function(module, strict){
			// run the dependency vector, then run the factory for module
			if(module.executed === executing){
				req.trace("loader-circular-dependency", [circleTrace.concat(module.mid).join("->")]);
				return (!module.def || strict) ? abortExec :  (module.cjs && module.cjs.exports);
			}
			// at this point the module is either not executed or fully executed


			if(!module.executed){
				if(!module.def){
					return abortExec;
				}
				var mid = module.mid,
					deps = module.deps || [],
					arg, argResult,
					args = [],
					i = 0;

				if( 0 ){
					circleTrace.push(mid);
					req.trace("loader-exec-module", ["exec", circleTrace.length, mid]);
				}

				// for circular dependencies, assume the first module encountered was executed OK
				// modules that circularly depend on a module that has not run its factory will get
				// the pre-made cjs.exports===module.result. They can take a reference to this object and/or
				// add properties to it. When the module finally runs its factory, the factory can
				// read/write/replace this object. Notice that so long as the object isn't replaced, any
				// reference taken earlier while walking the deps list is still valid.
				module.executed = executing;
				while((arg = deps[i++])){
					argResult = ((arg === cjsRequireModule) ? createRequire(module) :
									((arg === cjsExportsModule) ? module.cjs.exports :
										((arg === cjsModuleModule) ? module.cjs :
											execModule(arg, strict))));
					if(argResult === abortExec){
						module.executed = 0;
						req.trace("loader-exec-module", ["abort", mid]);
						 0  && circleTrace.pop();
						return abortExec;
					}
					args.push(argResult);
				}
				runFactory(module, args);
				finishExec(module);
				 0  && circleTrace.pop();
			}
			// at this point the module is guaranteed fully executed

			return module.result;
		},


		checkCompleteGuard = 0,

		guardCheckComplete = function(proc){
			try{
				checkCompleteGuard++;
				proc();
			}finally{
				checkCompleteGuard--;
			}
			if(execComplete()){
				signal("idle", []);
			}
		},

		checkComplete = function(){
			// keep going through the execQ as long as at least one factory is executed
			// plugins, recursion, cached modules all make for many execution path possibilities
			if(checkCompleteGuard){
				return;
			}
			guardCheckComplete(function(){
				checkDojoRequirePlugin();
				for(var currentDefOrder, module, i = 0; i < execQ.length;){
					currentDefOrder = defOrder;
					module = execQ[i];
					execModule(module);
					if(currentDefOrder!=defOrder){
						// defOrder was bumped one or more times indicating something was executed (note, this indicates
						// the execQ was modified, maybe a lot (for example a later module causes an earlier module to execute)
						checkDojoRequirePlugin();
						i = 0;
					}else{
						// nothing happened; check the next module in the exec queue
						i++;
					}
				}
			});
		};


	if( 0 ){
		req.undef = function(moduleId, referenceModule){
			// In order to reload a module, it must be undefined (this routine) and then re-requested.
			// This is useful for testing frameworks (at least).
			var module = getModule(moduleId, referenceModule);
			setArrived(module);
			mix(module, {def:0, executed:0, injected:0, node:0});
		};
	}

	if( 1 ){
		if(has("dojo-loader-eval-hint-url")===undefined){
			has.add("dojo-loader-eval-hint-url", 1);
		}

		var fixupUrl= function(url){
				url += ""; // make sure url is a Javascript string (some paths may be a Java string)
				return url + (cacheBust ? ((/\?/.test(url) ? "&" : "?") + cacheBust) : "");
			},

			injectPlugin = function(
				module
			){
				// injects the plugin module given by module; may have to inject the plugin itself
				var plugin = module.plugin;

				if(plugin.executed === executed && !plugin.load){
					// executed the module not knowing it was a plugin
					promoteModuleToPlugin(plugin);
				}

				var onLoad = function(def){
						module.result = def;
						setArrived(module);
						finishExec(module);
						checkComplete();
					};

				if(plugin.load){
					plugin.load(module.prid, module.req, onLoad);
				}else if(plugin.loadQ){
					plugin.loadQ.push(module);
				}else{
					// the unshift instead of push is important: we don't want plugins to execute as
					// dependencies of some other module because this may cause circles when the plugin
					// loadQ is run; also, generally, we want plugins to run early since they may load
					// several other modules and therefore can potentially unblock many modules
					plugin.loadQ = [module];
					execQ.unshift(plugin);
					injectModule(plugin);
				}
			},

			// for IE, injecting a module may result in a recursive execution if the module is in the cache

			cached = 0,

			injectingModule = 0,

			injectingCachedModule = 0,

			evalModuleText = function(text, module){
				// see def() for the injectingCachedModule bracket; it simply causes a short, safe circuit
				if(has("config-stripStrict")){
					text = text.replace(/"use strict"/g, '');
				}
				injectingCachedModule = 1;
				if(has("config-dojo-loader-catches")){
					try{
						if(text===cached){
							cached.call(null);
						}else{
							req.eval(text, has("dojo-loader-eval-hint-url") ? module.url : module.mid);
						}
					}catch(e){
						signal(error, makeError("evalModuleThrew", module));
					}
				}else{
					if(text===cached){
						cached.call(null);
					}else{
						req.eval(text, has("dojo-loader-eval-hint-url") ? module.url : module.mid);
					}
				}
				injectingCachedModule = 0;
			},

			injectModule = function(module){
				// Inject the module. In the browser environment, this means appending a script element into
				// the document; in other environments, it means loading a file.
				//
				// If in synchronous mode, then get the module synchronously if it's not xdomainLoading.

				var mid = module.mid,
					url = module.url;
				if(module.executed || module.injected || waiting[mid] || (module.url && ((module.pack && waiting[module.url]===module.pack) || waiting[module.url]==1))){
					return;
				}
				setRequested(module);

				if( 0 ){
					var viaCombo = 0;
					if(module.plugin && module.plugin.isCombo){
						// a combo plugin; therefore, must be handled by combo service
						// the prid should have already been converted to a URL (if required by the plugin) during
						// the normalize process; in any event, there is no way for the loader to know how to
						// to the conversion; therefore the third argument is zero
						req.combo.add(module.plugin.mid, module.prid, 0, req);
						viaCombo = 1;
					}else if(!module.plugin){
						viaCombo = req.combo.add(0, module.mid, module.url, req);
					}
					if(viaCombo){
						comboPending= 1;
						return;
					}
				}

				if(module.plugin){
					injectPlugin(module);
					return;
				} // else a normal module (not a plugin)


				var onLoadCallback = function(){
					runDefQ(module);
					if(module.injected !== arrived){
						// the script that contained the module arrived and has been executed yet
						// nothing was added to the defQ (so it wasn't an AMD module) and the module
						// wasn't marked as arrived by dojo.provide (so it wasn't a v1.6- module);
						// therefore, it must not have been a module; adjust state accordingly
						if(has("dojo-enforceDefine")){
							signal(error, makeError("noDefine", module));
							return;
						}
						setArrived(module);
						mix(module, nonModuleProps);
						req.trace("loader-define-nonmodule", [module.url]);
					}

					if( 1  && legacyMode){
						// must call checkComplete even in for sync loader because we may be in xdomainLoading mode;
						// but, if xd loading, then don't call checkComplete until out of the current sync traversal
						// in order to preserve order of execution of the dojo.required modules
						!syncExecStack.length && checkComplete();
					}else{
						checkComplete();
					}
				};
				cached = cache[mid] || cache[urlKeyPrefix + module.url];
				if(cached){
					req.trace("loader-inject", ["cache", module.mid, url]);
					evalModuleText(cached, module);
					onLoadCallback();
					return;
				}
				if( 1  && legacyMode){
					if(module.isXd){
						// switch to async mode temporarily; if current legacyMode!=sync, then is must be one of {legacyAsync, xd, false}
						legacyMode==sync && (legacyMode = xd);
						// fall through and load via script injection
					}else if(module.isAmd && legacyMode!=sync){
						// fall through and load via script injection
					}else{
						// mode may be sync, xd/legacyAsync, or async; module may be AMD or legacy; but module is always located on the same domain
						var xhrCallback = function(text){
							if(legacyMode==sync){
								// the top of syncExecStack gives the current synchronously executing module; the loader needs
								// to know this if it has to switch to async loading in the middle of evaluating a legacy module
								// this happens when a modules dojo.require's a module that must be loaded async because it's xdomain
								// (using unshift/shift because there is no back() methods for Javascript arrays)
								syncExecStack.unshift(module);
								evalModuleText(text, module);
								syncExecStack.shift();

								// maybe the module was an AMD module
								runDefQ(module);

								// legacy modules never get to defineModule() => cjs and injected never set; also evaluation implies executing
								if(!module.cjs){
									setArrived(module);
									finishExec(module);
								}

								if(module.finish){
									// while synchronously evaluating this module, dojo.require was applied referencing a module
									// that had to be loaded async; therefore, the loader stopped answering all dojo.require
									// requests so they could be answered completely in the correct sequence; module.finish gives
									// the list of dojo.requires that must be re-applied once all target modules are available;
									// make a synthetic module to execute the dojo.require's in the correct order

									// compute a guaranteed-unique mid for the synthetic finish module; remember the finish vector; remove it from the reference module
									// TODO: can we just leave the module.finish...what's it hurting?
									var finishMid = mid + "*finish",
										finish = module.finish;
									delete module.finish;

									def(finishMid, ["dojo", ("dojo/require!" + finish.join(",")).replace(/\./g, "/")], function(dojo){
										forEach(finish, function(mid){ dojo.require(mid); });
									});
									// unshift, not push, which causes the current traversal to be reattempted from the top
									execQ.unshift(getModule(finishMid));
								}
								onLoadCallback();
							}else{
								text = transformToAmd(module, text);
								if(text){
									evalModuleText(text, module);
									onLoadCallback();
								}else{
									// if transformToAmd returned falsy, then the module was already AMD and it can be script-injected
									// do so to improve debugability(even though it means another download...which probably won't happen with a good browser cache)
									injectingModule = module;
									req.injectUrl(fixupUrl(url), onLoadCallback, module);
									injectingModule = 0;
								}
							}
						};

						req.trace("loader-inject", ["xhr", module.mid, url, legacyMode!=sync]);
						if(has("config-dojo-loader-catches")){
							try{
								req.getText(url, legacyMode!=sync, xhrCallback);
							}catch(e){
								signal(error, makeError("xhrInjectFailed", [module, e]));
							}
						}else{
							req.getText(url, legacyMode!=sync, xhrCallback);
						}
						return;
					}
				} // else async mode or fell through in xdomain loading mode; either way, load by script injection
				req.trace("loader-inject", ["script", module.mid, url]);
				injectingModule = module;
				req.injectUrl(fixupUrl(url), onLoadCallback, module);
				injectingModule = 0;
			},

			defineModule = function(module, deps, def){
				req.trace("loader-define-module", [module.mid, deps]);

				if( 0  && module.plugin && module.plugin.isCombo){
					// the module is a plugin resource loaded by the combo service
					// note: check for module.plugin should be enough since normal plugin resources should
					// not follow this path; module.plugin.isCombo is future-proofing belt and suspenders
					module.result = isFunction(def) ? def() : def;
					setArrived(module);
					finishExec(module);
					return module;
				}

				var mid = module.mid;
				if(module.injected === arrived){
					signal(error, makeError("multipleDefine", module));
					return module;
				}
				mix(module, {
					deps: deps,
					def: def,
					cjs: {
						id: module.mid,
						uri: module.url,
						exports: (module.result = {}),
						setExports: function(exports){
							module.cjs.exports = exports;
						},
						config:function(){
							return module.config;
						}
					}
				});

				// resolve deps with respect to this module
				for(var i = 0; deps[i]; i++){
					deps[i] = getModule(deps[i], module);
				}

				if( 1  && legacyMode && !waiting[mid]){
					// the module showed up without being asked for; it was probably in a <script> element
					injectDependencies(module);
					execQ.push(module);
					checkComplete();
				}
				setArrived(module);

				if(!isFunction(def) && !deps.length){
					module.result = def;
					finishExec(module);
				}

				return module;
			},

			runDefQ = function(referenceModule, mids){
				// defQ is an array of [id, dependencies, factory]
				// mids (if any) is a vector of mids given by a combo service
				var definedModules = [],
					module, args;
				while(defQ.length){
					args = defQ.shift();
					mids && (args[0]= mids.shift());
					// explicit define indicates possible multiple modules in a single file; delay injecting dependencies until defQ fully
					// processed since modules earlier in the queue depend on already-arrived modules that are later in the queue
					// TODO: what if no args[0] and no referenceModule
					module = (args[0] && getModule(args[0])) || referenceModule;
					definedModules.push([module, args[1], args[2]]);
				}
				consumePendingCacheInsert(referenceModule);
				forEach(definedModules, function(args){
					injectDependencies(defineModule.apply(null, args));
				});
			};
	}

	var timerId = 0,
		clearTimer = noop,
		startTimer = noop;
	if( 1 ){
		// Timer machinery that monitors how long the loader is waiting and signals an error when the timer runs out.
		clearTimer = function(){
			timerId && clearTimeout(timerId);
			timerId = 0;
		};

		startTimer = function(){
			clearTimer();
			if(req.waitms){
				timerId = window.setTimeout(function(){
					clearTimer();
					signal(error, makeError("timeout", waiting));
				}, req.waitms);
			}
		};
	}

	if ( 1 ) {
		// Test for IE's different way of signaling when scripts finish loading.  Note that according to
		// http://bugs.dojotoolkit.org/ticket/15096#comment:14, IE9 also needs to follow the
		// IE specific code path even though it has an addEventListener() method.
		// Unknown if special path needed on IE10+, which also has a document.attachEvent() method.
		// Should evaluate to false for Opera and Windows 8 apps, even though they document.attachEvent()
		//  is defined in both those environments.
		has.add("ie-event-behavior", doc.attachEvent && typeof Windows === "undefined" &&
			(typeof opera === "undefined" || opera.toString() != "[object Opera]"));
	}

	if( 1  && ( 1  ||  1 )){
		var domOn = function(node, eventName, ieEventName, handler){
				// Add an event listener to a DOM node using the API appropriate for the current browser;
				// return a function that will disconnect the listener.
				if(!has("ie-event-behavior")){
					node.addEventListener(eventName, handler, false);
					return function(){
						node.removeEventListener(eventName, handler, false);
					};
				}else{
					node.attachEvent(ieEventName, handler);
					return function(){
						node.detachEvent(ieEventName, handler);
					};
				}
			},
			windowOnLoadListener = domOn(window, "load", "onload", function(){
				req.pageLoaded = 1;
				doc.readyState!="complete" && (doc.readyState = "complete");
				windowOnLoadListener();
			});

		if( 1 ){
			// if the loader is on the page, there must be at least one script element
			// getting its parent and then doing insertBefore solves the "Operation Aborted"
			// error in IE from appending to a node that isn't properly closed; see
			// dojo/tests/_base/loader/requirejs/simple-badbase.html for an example
			// don't use scripts with type dojo/... since these may be removed; see #15809
			// prefer to use the insertPoint computed during the config sniff in case a script is removed; see #16958
			var scripts = doc.getElementsByTagName("script"),
				i = 0,
				script;
			while(!insertPointSibling){
				if(!/^dojo/.test((script = scripts[i++]) && script.type)){
					insertPointSibling= script;
				}
			}

			req.injectUrl = function(url, callback, owner){
				// insert a script element to the insert-point element with src=url;
				// apply callback upon detecting the script has loaded.

				var node = owner.node = doc.createElement("script"),
					onLoad = function(e){
						e = e || window.event;
						var node = e.target || e.srcElement;
						if(e.type === "load" || /complete|loaded/.test(node.readyState)){
							loadDisconnector();
							errorDisconnector();
							callback && callback();
						}
					},
					loadDisconnector = domOn(node, "load", "onreadystatechange", onLoad),
					errorDisconnector = domOn(node, "error", "onerror", function(e){
						loadDisconnector();
						errorDisconnector();
						signal(error, makeError("scriptError", [url, e]));
					});

				node.type = "text/javascript";
				node.charset = "utf-8";
				node.src = url;
				insertPointSibling.parentNode.insertBefore(node, insertPointSibling);
				return node;
			};
		}
	}

	if( 1 ){
		req.log = function(){
			try{
				for(var i = 0; i < arguments.length; i++){
					console.log(arguments[i]);
				}
			}catch(e){}
		};
	}else{
		req.log = noop;
	}

	if( 0 ){
		var trace = req.trace = function(
			group,	// the trace group to which this application belongs
			args	// the contents of the trace
		){
			///
			// Tracing interface by group.
			//
			// Sends the contents of args to the console iff (req.trace.on && req.trace[group])

			if(trace.on && trace.group[group]){
				signal("trace", [group, args]);
				for(var arg, dump = [], text= "trace:" + group + (args.length ? (":" + args[0]) : ""), i= 1; i<args.length;){
					arg = args[i++];
					if(isString(arg)){
						text += ", " + arg;
					}else{
						dump.push(arg);
					}
				}
				req.log(text);
				dump.length && dump.push(".");
				req.log.apply(req, dump);
			}
		};
		mix(trace, {
			on:1,
			group:{},
			set:function(group, value){
				if(isString(group)){
					trace.group[group]= value;
				}else{
					mix(trace.group, group);
				}
			}
		});
		trace.set(mix(mix(mix({}, defaultConfig.trace), userConfig.trace), dojoSniffConfig.trace));
		on("config", function(config){
			config.trace && trace.set(config.trace);
		});
	}else{
		req.trace = noop;
	}

	var def = function(
		mid,		  //(commonjs.moduleId, optional)
		dependencies, //(array of commonjs.moduleId, optional) list of modules to be loaded before running factory
		factory		  //(any)
	){
		///
		// Advises the loader of a module factory. //Implements http://wiki.commonjs.org/wiki/Modules/AsynchronousDefinition.
		///
		//note
		// CommonJS factory scan courtesy of http://requirejs.org

		var arity = arguments.length,
			defaultDeps = ["require", "exports", "module"],
			// the predominate signature...
			args = [0, mid, dependencies];
		if(arity==1){
			args = [0, (isFunction(mid) ? defaultDeps : []), mid];
		}else if(arity==2 && isString(mid)){
			args = [mid, (isFunction(dependencies) ? defaultDeps : []), dependencies];
		}else if(arity==3){
			args = [mid, dependencies, factory];
		}

		if( 0  && args[1]===defaultDeps){
			args[2].toString()
				.replace(/(\/\*([\s\S]*?)\*\/|\/\/(.*)$)/mg, "")
				.replace(/require\(["']([\w\!\-_\.\/]+)["']\)/g, function(match, dep){
				args[1].push(dep);
			});
		}

		req.trace("loader-define", args.slice(0, 2));
		var targetModule = args[0] && getModule(args[0]),
			module;
		if(targetModule && !waiting[targetModule.mid]){
			// given a mid that hasn't been requested; therefore, defined through means other than injecting
			// consequent to a require() or define() application; examples include defining modules on-the-fly
			// due to some code path or including a module in a script element. In any case,
			// there is no callback waiting to finish processing and nothing to trigger the defQ and the
			// dependencies are never requested; therefore, do it here.
			injectDependencies(defineModule(targetModule, args[1], args[2]));
		}else if(!has("ie-event-behavior") || ! 1  || injectingCachedModule){
			// not IE path: anonymous module and therefore must have been injected; therefore, onLoad will fire immediately
			// after script finishes being evaluated and the defQ can be run from that callback to detect the module id
			defQ.push(args);
		}else{
			// IE path: possibly anonymous module and therefore injected; therefore, cannot depend on 1-to-1,
			// in-order exec of onLoad with script eval (since it's IE) and must manually detect here
			targetModule = targetModule || injectingModule;
			if(!targetModule){
				for(mid in waiting){
					module = modules[mid];
					if(module && module.node && module.node.readyState === 'interactive'){
						targetModule = module;
						break;
					}
				}
				if( 0  && !targetModule){
					for(var i = 0; i<combosPending.length; i++){
						targetModule = combosPending[i];
						if(targetModule.node && targetModule.node.readyState === 'interactive'){
							break;
						}
						targetModule= 0;
					}
				}
			}
			if( 0  && isArray(targetModule)){
				injectDependencies(defineModule(getModule(targetModule.shift()), args[1], args[2]));
				if(!targetModule.length){
					combosPending.splice(i, 1);
				}
			}else if(targetModule){
				consumePendingCacheInsert(targetModule);
				injectDependencies(defineModule(targetModule, args[1], args[2]));
			}else{
				signal(error, makeError("ieDefineFailed", args[0]));
			}
			checkComplete();
		}
	};
	def.amd = {
		vendor:"dojotoolkit.org"
	};

	if( 0 ){
		req.def = def;
	}

	// allow config to override default implementation of named functions; this is useful for
	// non-browser environments, e.g., overriding injectUrl, getText, log, etc. in node.js, Rhino, etc.
	// also useful for testing and monkey patching loader
	mix(mix(req, defaultConfig.loaderPatch), userConfig.loaderPatch);

	// now that req is fully initialized and won't change, we can hook it up to the error signal
	on(error, function(arg){
		try{
			console.error(arg);
			if(arg instanceof Error){
				for(var p in arg){
					console.log(p + ":", arg[p]);
				}
				console.log(".");
			}
		}catch(e){}
	});

	// always publish these
	mix(req, {
		uid:uid,
		cache:cache,
		packs:packs
	});


	if( 0 ){
		mix(req, {
			// these may be interesting to look at when debugging
			paths:paths,
			aliases:aliases,
			modules:modules,
			legacyMode:legacyMode,
			execQ:execQ,
			defQ:defQ,
			waiting:waiting,

			// these are used for testing
			// TODO: move testing infrastructure to a different has feature
			packs:packs,
			mapProgs:mapProgs,
			pathsMapProg:pathsMapProg,
			listenerQueues:listenerQueues,

			// these are used by the builder (at least)
			computeMapProg:computeMapProg,
			computeAliases:computeAliases,
			runMapProg:runMapProg,
			compactPath:compactPath,
			getModuleInfo:getModuleInfo_
		});
	}

	// the loader can be defined exactly once; look for global define which is the symbol AMD loaders are
	// *required* to define (as opposed to require, which is optional)
	if(global.define){
		if( 1 ){
			signal(error, makeError("defineAlreadyDefined", 0));
		}
		return;
	}else{
		global.define = def;
		global.require = req;
		if( 0 ){
			require = req;
		}
	}

	if( 0  && req.combo && req.combo.plugins){
		var plugins = req.combo.plugins,
			pluginName;
		for(pluginName in plugins){
			mix(mix(getModule(pluginName), plugins[pluginName]), {isCombo:1, executed:"executed", load:1});
		}
	}

	if( 1 ){
		forEach(delayedModuleConfig, function(c){ config(c); });
		var bootDeps = dojoSniffConfig.deps ||	userConfig.deps || defaultConfig.deps,
			bootCallback = dojoSniffConfig.callback || userConfig.callback || defaultConfig.callback;
		req.boot = (bootDeps || bootCallback) ? [bootDeps || [], bootCallback] : 0;
	}
	if(! 1 ){
		!req.async && req(["dojo"]);
		req.boot && req.apply(null, req.boot);
	}
})
(this.dojoConfig || this.djConfig || this.require || {}, {
		async:0,
		hasCache:{
				'config-selectorEngine':"lite",
				'config-tlmSiblingOfDojo':1,
				'dojo-built':1,
				'dojo-loader':1,
				dom:1,
				'host-browser':1
		},
		packages:[
				{
					 location:".",
					 name:"dojo"
				},
				{
					 location:"../dijit",
					 name:"dijit"
				},
				{
					 location:"../dojox",
					 name:"dojox"
				},
				{
					 location:"../dgrid",
					 main:"OnDemandGrid",
					 name:"dgrid"
				},
				{
					 location:"../put-selector",
					 main:"put",
					 name:"put-selector"
				},
				{
					 location:"../xstyle",
					 main:"css",
					 name:"xstyle"
				},
				{
					 location:"../vsd",
					 name:"vsd"
				}
		]
});require({cache:{
'dojo/main':function(){
define([
	"./_base/kernel",	// kernel.isAsync
	"./has",
	"require",
	"./sniff",
	"./_base/lang",
	"./_base/array",
	"./_base/config",
	"./ready",
	"./_base/declare",
	"./_base/connect",
	"./_base/Deferred",
	"./_base/json",
	"./_base/Color",
	"./has!dojo-firebug?./_firebug/firebug",
	"./_base/browser",
	"./_base/loader"
], function(kernel, has, require, sniff, lang, array, config, ready){
	// module:
	//		dojo/main
	// summary:
	//		This is the package main module for the dojo package; it loads dojo base appropriate for the execution environment.

	// the preferred way to load the dojo firebug console is by setting has("dojo-firebug") true in dojoConfig
	// the isDebug config switch is for backcompat and will work fine in sync loading mode; it works in
	// async mode too, but there's no guarantee when the module is loaded; therefore, if you need a firebug
	// console guaranteed at a particular spot in an app, either set config.has["dojo-firebug"] true before
	// loading dojo.js or explicitly include dojo/_firebug/firebug in a dependency list.
	if(config.isDebug){
		require(["./_firebug/firebug"]);
	}

	// dojoConfig.require is deprecated; use the loader configuration property deps
	 1 || has.add("dojo-config-require", 1);
	if( 1 ){
		var deps= config.require;
		if(deps){
			// config.require may be dot notation
			deps= array.map(lang.isArray(deps) ? deps : [deps], function(item){ return item.replace(/\./g, "/"); });
			if(kernel.isAsync){
				require(deps);
			}else{
				// this is a bit janky; in 1.6- dojo is defined before these requires are applied; but in 1.7+
				// dojo isn't defined until returning from this module; this is only a problem in sync mode
				// since we're in sync mode, we know we've got our loader with its priority ready queue
				ready(1, function(){require(deps);});
			}
		}
	}

	return kernel;
});

},
'dojo/_base/kernel':function(){
define(["../has", "./config", "require", "module"], function(has, config, require, module){
	// module:
	//		dojo/_base/kernel

	// This module is the foundational module of the dojo boot sequence; it defines the dojo object.

	var
		// loop variables for this module
		i, p,

		// create dojo, dijit, and dojox
		// FIXME: in 2.0 remove dijit, dojox being created by dojo
		dijit = {},
		dojox = {},
		dojo = {
			// summary:
			//		This module is the foundational module of the dojo boot sequence; it defines the dojo object.

			// notice dojo takes ownership of the value of the config module
			config:config,
			global:this,
			dijit:dijit,
			dojox:dojox
		};


	// Configure the scope map. For a 100% AMD application, the scope map is not needed other than to provide
	// a _scopeName property for the dojo, dijit, and dojox root object so those packages can create
	// unique names in the global space.
	//
	// Built, legacy modules use the scope map to allow those modules to be expressed as if dojo, dijit, and dojox,
	// where global when in fact they are either global under different names or not global at all. In v1.6-, the
	// config variable "scopeMap" was used to map names as used within a module to global names. This has been
	// subsumed by the AMD map configuration variable which can relocate packages to different names. For backcompat,
	// only the "*" mapping is supported. See http://livedocs.dojotoolkit.org/developer/design/loader#legacy-cross-domain-mode for details.
	//
	// The following computations contort the packageMap for this dojo instance into a scopeMap.
	var scopeMap =
			// a map from a name used in a legacy module to the (global variable name, object addressed by that name)
			// always map dojo, dijit, and dojox
			{
				dojo:["dojo", dojo],
				dijit:["dijit", dijit],
				dojox:["dojox", dojox]
			},

		packageMap =
			// the package map for this dojo instance; note, a foreign loader or no pacakgeMap results in the above default config
			(require.map && require.map[module.id.match(/[^\/]+/)[0]]),

		item;


	// process all mapped top-level names for this instance of dojo
	for(p in packageMap){
		if(scopeMap[p]){
			// mapped dojo, dijit, or dojox
			scopeMap[p][0] = packageMap[p];
		}else{
			// some other top-level name
			scopeMap[p] = [packageMap[p], {}];
		}
	}

	// publish those names to _scopeName and, optionally, the global namespace
	for(p in scopeMap){
		item = scopeMap[p];
		item[1]._scopeName = item[0];
		if(!config.noGlobals){
			this[item[0]] = item[1];
		}
	}
	dojo.scopeMap = scopeMap;

	/*===== dojo.__docParserConfigureScopeMap(scopeMap); =====*/

	// FIXME: dojo.baseUrl and dojo.config.baseUrl should be deprecated
	dojo.baseUrl = dojo.config.baseUrl = require.baseUrl;
	dojo.isAsync = ! 1  || require.async;
	dojo.locale = config.locale;

	var rev = "$Rev: 43d05c6 $".match(/\d+/);
	dojo.version = {
		// summary:
		//		Version number of the Dojo Toolkit
		// description:
		//		Hash about the version, including
		//
		//		- major: Integer: Major version. If total version is "1.2.0beta1", will be 1
		//		- minor: Integer: Minor version. If total version is "1.2.0beta1", will be 2
		//		- patch: Integer: Patch version. If total version is "1.2.0beta1", will be 0
		//		- flag: String: Descriptor flag. If total version is "1.2.0beta1", will be "beta1"
		//		- revision: Number: The SVN rev from which dojo was pulled

		major: 1, minor: 9, patch: 1, flag: "",
		revision: rev ? +rev[0] : NaN,
		toString: function(){
			var v = dojo.version;
			return v.major + "." + v.minor + "." + v.patch + v.flag + " (" + v.revision + ")";	// String
		}
	};

	// If  1  is truthy, then as a dojo module is defined it should push it's definitions
	// into the dojo object, and conversely. In 2.0, it will likely be unusual to augment another object
	// as a result of defining a module. This has feature gives a way to force 2.0 behavior as the code
	// is migrated. Absent specific advice otherwise, set extend-dojo to truthy.
	 1 || has.add("extend-dojo", 1);


	(Function("d", "d.eval = function(){return d.global.eval ? d.global.eval(arguments[0]) : eval(arguments[0]);}"))(dojo);
	/*=====
	dojo.eval = function(scriptText){
		// summary:
		//		A legacy method created for use exclusively by internal Dojo methods. Do not use this method
		//		directly unless you understand its possibly-different implications on the platforms your are targeting.
		// description:
		//		Makes an attempt to evaluate scriptText in the global scope. The function works correctly for browsers
		//		that support indirect eval.
		//
		//		As usual, IE does not. On IE, the only way to implement global eval is to
		//		use execScript. Unfortunately, execScript does not return a value and breaks some current usages of dojo.eval.
		//		This implementation uses the technique of executing eval in the scope of a function that is a single scope
		//		frame below the global scope; thereby coming close to the global scope. Note carefully that
		//
		//		dojo.eval("var pi = 3.14;");
		//
		//		will define global pi in non-IE environments, but define pi only in a temporary local scope for IE. If you want
		//		to define a global variable using dojo.eval, write something like
		//
		//		dojo.eval("window.pi = 3.14;")
		// scriptText:
		//		The text to evaluation.
		// returns:
		//		The result of the evaluation. Often `undefined`
	};
	=====*/


	if( 0 ){
		dojo.exit = function(exitcode){
			quit(exitcode);
		};
	}else{
		dojo.exit = function(){
		};
	}

	 1 || has.add("dojo-guarantee-console",
		// ensure that console.log, console.warn, etc. are defined
		1
	);
	if( 1 ){
		typeof console != "undefined" || (console = {});
		//	Be careful to leave 'log' always at the end
		var cn = [
			"assert", "count", "debug", "dir", "dirxml", "error", "group",
			"groupEnd", "info", "profile", "profileEnd", "time", "timeEnd",
			"trace", "warn", "log"
		];
		var tn;
		i = 0;
		while((tn = cn[i++])){
			if(!console[tn]){
				(function(){
					var tcn = tn + "";
					console[tcn] = ('log' in console) ? function(){
						var a = Array.apply({}, arguments);
						a.unshift(tcn + ":");
						console["log"](a.join(" "));
					} : function(){};
					console[tcn]._fake = true;
				})();
			}
		}
	}

	has.add("dojo-debug-messages",
		// include dojo.deprecated/dojo.experimental implementations
		!!config.isDebug
	);
	dojo.deprecated = dojo.experimental =  function(){};
	if(has("dojo-debug-messages")){
		dojo.deprecated = function(/*String*/ behaviour, /*String?*/ extra, /*String?*/ removal){
			// summary:
			//		Log a debug message to indicate that a behavior has been
			//		deprecated.
			// behaviour: String
			//		The API or behavior being deprecated. Usually in the form
			//		of "myApp.someFunction()".
			// extra: String?
			//		Text to append to the message. Often provides advice on a
			//		new function or facility to achieve the same goal during
			//		the deprecation period.
			// removal: String?
			//		Text to indicate when in the future the behavior will be
			//		removed. Usually a version number.
			// example:
			//	| dojo.deprecated("myApp.getTemp()", "use myApp.getLocaleTemp() instead", "1.0");

			var message = "DEPRECATED: " + behaviour;
			if(extra){ message += " " + extra; }
			if(removal){ message += " -- will be removed in version: " + removal; }
			console.warn(message);
		};

		dojo.experimental = function(/* String */ moduleName, /* String? */ extra){
			// summary:
			//		Marks code as experimental.
			// description:
			//		This can be used to mark a function, file, or module as
			//		experimental.	 Experimental code is not ready to be used, and the
			//		APIs are subject to change without notice.	Experimental code may be
			//		completed deleted without going through the normal deprecation
			//		process.
			// moduleName: String
			//		The name of a module, or the name of a module file or a specific
			//		function
			// extra: String?
			//		some additional message for the user
			// example:
			//	| dojo.experimental("dojo.data.Result");
			// example:
			//	| dojo.experimental("dojo.weather.toKelvin()", "PENDING approval from NOAA");

			var message = "EXPERIMENTAL: " + moduleName + " -- APIs subject to change without notice.";
			if(extra){ message += " " + extra; }
			console.warn(message);
		};
	}

	 1 || has.add("dojo-modulePaths",
		// consume dojo.modulePaths processing
		1
	);
	if( 1 ){
		// notice that modulePaths won't be applied to any require's before the dojo/_base/kernel factory is run;
		// this is the v1.6- behavior.
		if(config.modulePaths){
			dojo.deprecated("dojo.modulePaths", "use paths configuration");
			var paths = {};
			for(p in config.modulePaths){
				paths[p.replace(/\./g, "/")] = config.modulePaths[p];
			}
			require({paths:paths});
		}
	}

	 1 || has.add("dojo-moduleUrl",
		// include dojo.moduleUrl
		1
	);
	if( 1 ){
		dojo.moduleUrl = function(/*String*/module, /*String?*/url){
			// summary:
			//		Returns a URL relative to a module.
			// example:
			//	|	var pngPath = dojo.moduleUrl("acme","images/small.png");
			//	|	console.dir(pngPath); // list the object properties
			//	|	// create an image and set it's source to pngPath's value:
			//	|	var img = document.createElement("img");
			//	|	img.src = pngPath;
			//	|	// add our image to the document
			//	|	dojo.body().appendChild(img);
			// example:
			//		you may de-reference as far as you like down the package
			//		hierarchy.  This is sometimes handy to avoid lenghty relative
			//		urls or for building portable sub-packages. In this example,
			//		the `acme.widget` and `acme.util` directories may be located
			//		under different roots (see `dojo.registerModulePath`) but the
			//		the modules which reference them can be unaware of their
			//		relative locations on the filesystem:
			//	|	// somewhere in a configuration block
			//	|	dojo.registerModulePath("acme.widget", "../../acme/widget");
			//	|	dojo.registerModulePath("acme.util", "../../util");
			//	|
			//	|	// ...
			//	|
			//	|	// code in a module using acme resources
			//	|	var tmpltPath = dojo.moduleUrl("acme.widget","templates/template.html");
			//	|	var dataPath = dojo.moduleUrl("acme.util","resources/data.json");

			dojo.deprecated("dojo.moduleUrl()", "use require.toUrl", "2.0");

			// require.toUrl requires a filetype; therefore, just append the suffix "/*.*" to guarantee a filetype, then
			// remove the suffix from the result. This way clients can request a url w/out a filetype. This should be
			// rare, but it maintains backcompat for the v1.x line (note: dojo.moduleUrl will be removed in v2.0).
			// Notice * is an illegal filename so it won't conflict with any real path map that may exist the paths config.
			var result = null;
			if(module){
				result = require.toUrl(module.replace(/\./g, "/") + (url ? ("/" + url) : "") + "/*.*").replace(/\/\*\.\*/, "") + (url ? "" : "/");
			}
			return result;
		};
	}

	dojo._hasResource = {}; // for backward compatibility with layers built with 1.6 tooling

	return dojo;
});

},
'dojo/has':function(){
define(["require", "module"], function(require, module){
	// module:
	//		dojo/has
	// summary:
	//		Defines the has.js API and several feature tests used by dojo.
	// description:
	//		This module defines the has API as described by the project has.js with the following additional features:
	//
	//		- the has test cache is exposed at has.cache.
	//		- the method has.add includes a forth parameter that controls whether or not existing tests are replaced
	//		- the loader's has cache may be optionally copied into this module's has cahce.
	//
	//		This module adopted from https://github.com/phiggins42/has.js; thanks has.js team!

	// try to pull the has implementation from the loader; both the dojo loader and bdLoad provide one
	// if using a foreign loader, then the has cache may be initialized via the config object for this module
	// WARNING: if a foreign loader defines require.has to be something other than the has.js API, then this implementation fail
	var has = require.has || function(){};
	if(! 1 ){
		var
			isBrowser =
				// the most fundamental decision: are we in the browser?
				typeof window != "undefined" &&
				typeof location != "undefined" &&
				typeof document != "undefined" &&
				window.location == location && window.document == document,

			// has API variables
			global = this,
			doc = isBrowser && document,
			element = doc && doc.createElement("DiV"),
			cache = (module.config && module.config()) || {};

		has = function(name){
			// summary:
			//		Return the current value of the named feature.
			//
			// name: String|Integer
			//		The name (if a string) or identifier (if an integer) of the feature to test.
			//
			// description:
			//		Returns the value of the feature named by name. The feature must have been
			//		previously added to the cache by has.add.

			return typeof cache[name] == "function" ? (cache[name] = cache[name](global, doc, element)) : cache[name]; // Boolean
		};

		has.cache = cache;

		has.add = function(name, test, now, force){
			// summary:
			//	 	Register a new feature test for some named feature.
			// name: String|Integer
			//	 	The name (if a string) or identifier (if an integer) of the feature to test.
			// test: Function
			//		 A test function to register. If a function, queued for testing until actually
			//		 needed. The test function should return a boolean indicating
			//	 	the presence of a feature or bug.
			// now: Boolean?
			//		 Optional. Omit if `test` is not a function. Provides a way to immediately
			//		 run the test and cache the result.
			// force: Boolean?
			//	 	Optional. If the test already exists and force is truthy, then the existing
			//	 	test will be replaced; otherwise, add does not replace an existing test (that
			//	 	is, by default, the first test advice wins).
			// example:
			//		A redundant test, testFn with immediate execution:
			//	|	has.add("javascript", function(){ return true; }, true);
			//
			// example:
			//		Again with the redundantness. You can do this in your tests, but we should
			//		not be doing this in any internal has.js tests
			//	|	has.add("javascript", true);
			//
			// example:
			//		Three things are passed to the testFunction. `global`, `document`, and a generic element
			//		from which to work your test should the need arise.
			//	|	has.add("bug-byid", function(g, d, el){
			//	|		// g	== global, typically window, yadda yadda
			//	|		// d	== document object
			//	|		// el == the generic element. a `has` element.
			//	|		return false; // fake test, byid-when-form-has-name-matching-an-id is slightly longer
			//	|	});

			(typeof cache[name]=="undefined" || force) && (cache[name]= test);
			return now && has(name);
		};

		// since we're operating under a loader that doesn't provide a has API, we must explicitly initialize
		// has as it would have otherwise been initialized by the dojo loader; use has.add to the builder
		// can optimize these away iff desired
		 1 || has.add("host-browser", isBrowser);
		 1 || has.add("dom", isBrowser);
		 1 || has.add("dojo-dom-ready-api", 1);
		 1 || has.add("dojo-sniff", 1);
	}

	if( 1 ){
		// Common application level tests
		has.add("dom-addeventlistener", !!document.addEventListener);
		has.add("touch", "ontouchstart" in document || window.navigator.msMaxTouchPoints > 0);
		// I don't know if any of these tests are really correct, just a rough guess
		has.add("device-width", screen.availWidth || innerWidth);

		// Tests for DOMNode.attributes[] behavior:
		//	 - dom-attributes-explicit - attributes[] only lists explicitly user specified attributes
		//	 - dom-attributes-specified-flag (IE8) - need to check attr.specified flag to skip attributes user didn't specify
		//	 - Otherwise, in IE6-7. attributes[] will list hundreds of values, so need to do outerHTML to get attrs instead.
		var form = document.createElement("form");
		has.add("dom-attributes-explicit", form.attributes.length == 0); // W3C
		has.add("dom-attributes-specified-flag", form.attributes.length > 0 && form.attributes.length < 40);	// IE8
	}

	has.clearElement = function(element){
		// summary:
		//	 Deletes the contents of the element passed to test functions.
		element.innerHTML= "";
		return element;
	};

	has.normalize = function(id, toAbsMid){
		// summary:
		//	 Resolves id into a module id based on possibly-nested tenary expression that branches on has feature test value(s).
		//
		// toAbsMid: Function
		//	 Resolves a relative module id into an absolute module id
		var
			tokens = id.match(/[\?:]|[^:\?]*/g), i = 0,
			get = function(skip){
				var term = tokens[i++];
				if(term == ":"){
					// empty string module name, resolves to 0
					return 0;
				}else{
					// postfixed with a ? means it is a feature to branch on, the term is the name of the feature
					if(tokens[i++] == "?"){
						if(!skip && has(term)){
							// matched the feature, get the first value from the options
							return get();
						}else{
							// did not match, get the second value, passing over the first
							get(true);
							return get(skip);
						}
					}
					// a module
					return term || 0;
				}
			};
		id = get();
		return id && toAbsMid(id);
	};

	has.load = function(id, parentRequire, loaded){
		// summary:
		//		Conditional loading of AMD modules based on a has feature test value.
		// id: String
		//		Gives the resolved module id to load.
		// parentRequire: Function
		//		The loader require function with respect to the module that contained the plugin resource in it's
		//		dependency list.
		// loaded: Function
		//	 Callback to loader that consumes result of plugin demand.

		if(id){
			parentRequire([id], loaded);
		}else{
			loaded();
		}
	};

	return has;
});

},
'dojo/_base/config':function(){
define(["../has", "require"], function(has, require){
	// module:
	//		dojo/_base/config

/*=====
return {
	// summary:
	//		This module defines the user configuration during bootstrap.
	// description:
	//		By defining user configuration as a module value, an entire configuration can be specified in a build,
	//		thereby eliminating the need for sniffing and or explicitly setting in the global variable dojoConfig.
	//		Also, when multiple instances of dojo exist in a single application, each will necessarily be located
	//		at an unique absolute module identifier as given by the package configuration. Implementing configuration
	//		as a module allows for specifying unique, per-instance configurations.
	// example:
	//		Create a second instance of dojo with a different, instance-unique configuration (assume the loader and
	//		dojo.js are already loaded).
	//		|	// specify a configuration that creates a new instance of dojo at the absolute module identifier "myDojo"
	//		|	require({
	//		|		packages:[{
	//		|			name:"myDojo",
	//		|			location:".", //assume baseUrl points to dojo.js
	//		|		}]
	//		|	});
	//		|
	//		|	// specify a configuration for the myDojo instance
	//		|	define("myDojo/config", {
	//		|		// normal configuration variables go here, e.g.,
	//		|		locale:"fr-ca"
	//		|	});
	//		|
	//		|	// load and use the new instance of dojo
	//		|	require(["myDojo"], function(dojo){
	//		|		// dojo is the new instance of dojo
	//		|		// use as required
	//		|	});

	// isDebug: Boolean
	//		Defaults to `false`. If set to `true`, ensures that Dojo provides
	//		extended debugging feedback via Firebug. If Firebug is not available
	//		on your platform, setting `isDebug` to `true` will force Dojo to
	//		pull in (and display) the version of Firebug Lite which is
	//		integrated into the Dojo distribution, thereby always providing a
	//		debugging/logging console when `isDebug` is enabled. Note that
	//		Firebug's `console.*` methods are ALWAYS defined by Dojo. If
	//		`isDebug` is false and you are on a platform without Firebug, these
	//		methods will be defined as no-ops.
	isDebug: false,

	// locale: String
	//		The locale to assume for loading localized resources in this page,
	//		specified according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt).
	//		Must be specified entirely in lowercase, e.g. `en-us` and `zh-cn`.
	//		See the documentation for `dojo.i18n` and `dojo.requireLocalization`
	//		for details on loading localized resources. If no locale is specified,
	//		Dojo assumes the locale of the user agent, according to `navigator.userLanguage`
	//		or `navigator.language` properties.
	locale: undefined,

	// extraLocale: Array
	//		No default value. Specifies additional locales whose
	//		resources should also be loaded alongside the default locale when
	//		calls to `dojo.requireLocalization()` are processed.
	extraLocale: undefined,

	// baseUrl: String
	//		The directory in which `dojo.js` is located. Under normal
	//		conditions, Dojo auto-detects the correct location from which it
	//		was loaded. You may need to manually configure `baseUrl` in cases
	//		where you have renamed `dojo.js` or in which `<base>` tags confuse
	//		some browsers (e.g. IE 6). The variable `dojo.baseUrl` is assigned
	//		either the value of `djConfig.baseUrl` if one is provided or the
	//		auto-detected root if not. Other modules are located relative to
	//		this path. The path should end in a slash.
	baseUrl: undefined,

	// modulePaths: [deprecated] Object
	//		A map of module names to paths relative to `dojo.baseUrl`. The
	//		key/value pairs correspond directly to the arguments which
	//		`dojo.registerModulePath` accepts. Specifying
	//		`djConfig.modulePaths = { "foo": "../../bar" }` is the equivalent
	//		of calling `dojo.registerModulePath("foo", "../../bar");`. Multiple
	//		modules may be configured via `djConfig.modulePaths`.
	modulePaths: {},

	// addOnLoad: Function|Array
	//		Adds a callback via dojo/ready. Useful when Dojo is added after
	//		the page loads and djConfig.afterOnLoad is true. Supports the same
	//		arguments as dojo/ready. When using a function reference, use
	//		`djConfig.addOnLoad = function(){};`. For object with function name use
	//		`djConfig.addOnLoad = [myObject, "functionName"];` and for object with
	//		function reference use
	//		`djConfig.addOnLoad = [myObject, function(){}];`
	addOnLoad: null,

	// parseOnLoad: Boolean
	//		Run the parser after the page is loaded
	parseOnLoad: false,

	// require: String[]
	//		An array of module names to be loaded immediately after dojo.js has been included
	//		in a page.
	require: [],

	// defaultDuration: Number
	//		Default duration, in milliseconds, for wipe and fade animations within dijits.
	//		Assigned to dijit.defaultDuration.
	defaultDuration: 200,

	// dojoBlankHtmlUrl: String
	//		Used by some modules to configure an empty iframe. Used by dojo/io/iframe and
	//		dojo/back, and dijit/popup support in IE where an iframe is needed to make sure native
	//		controls do not bleed through the popups. Normally this configuration variable
	//		does not need to be set, except when using cross-domain/CDN Dojo builds.
	//		Save dojo/resources/blank.html to your domain and set `djConfig.dojoBlankHtmlUrl`
	//		to the path on your domain your copy of blank.html.
	dojoBlankHtmlUrl: undefined,

	// ioPublish: Boolean?
	//		Set this to true to enable publishing of topics for the different phases of
	//		IO operations. Publishing is done via dojo/topic.publish(). See dojo/main.__IoPublish for a list
	//		of topics that are published.
	ioPublish: false,

	// useCustomLogger: Anything?
	//		If set to a value that evaluates to true such as a string or array and
	//		isDebug is true and Firebug is not available or running, then it bypasses
	//		the creation of Firebug Lite allowing you to define your own console object.
	useCustomLogger: undefined,

	// transparentColor: Array
	//		Array containing the r, g, b components used as transparent color in dojo.Color;
	//		if undefined, [255,255,255] (white) will be used.
	transparentColor: undefined,
	
	// deps: Function|Array
	//		Defines dependencies to be used before the loader has been loaded.
	//		When provided, they cause the loader to execute require(deps, callback) 
	//		once it has finished loading. Should be used with callback.
	deps: undefined,
	
	// callback: Function|Array
	//		Defines a callback to be used when dependencies are defined before 
	//		the loader has been loaded. When provided, they cause the loader to 
	//		execute require(deps, callback) once it has finished loading. 
	//		Should be used with deps.
	callback: undefined,
	
	// deferredInstrumentation: Boolean
	//		Whether deferred instrumentation should be loaded or included
	//		in builds.
	deferredInstrumentation: true,

	// useDeferredInstrumentation: Boolean|String
	//		Whether the deferred instrumentation should be used.
	//
	//		* `"report-rejections"`: report each rejection as it occurs.
	//		* `true` or `1` or `"report-unhandled-rejections"`: wait 1 second
	//			in an attempt to detect unhandled rejections.
	useDeferredInstrumentation: "report-unhandled-rejections"
};
=====*/

	var result = {};
	if( 1 ){
		// must be the dojo loader; take a shallow copy of require.rawConfig
		var src = require.rawConfig, p;
		for(p in src){
			result[p] = src[p];
		}
	}else{
		var adviseHas = function(featureSet, prefix, booting){
			for(p in featureSet){
				p!="has" && has.add(prefix + p, featureSet[p], 0, booting);
			}
		};
		result =  1  ?
			// must be a built version of the dojo loader; all config stuffed in require.rawConfig
			require.rawConfig :
			// a foreign loader
			this.dojoConfig || this.djConfig || {};
		adviseHas(result, "config", 1);
		adviseHas(result.has, "", 1);
	}

	if(!result.locale && typeof navigator != "undefined"){
		// Default locale for browsers.
		result.locale = (navigator.language || navigator.userLanguage).toLowerCase();
	}

	return result;
});


},
'dojo/sniff':function(){
define(["./has"], function(has){
	// module:
	//		dojo/sniff

	/*=====
	return function(){
		// summary:
		//		This module sets has() flags based on the current browser.
		//		It returns the has() function.
	};
	=====*/

	if( 1 ){
		var n = navigator,
			dua = n.userAgent,
			dav = n.appVersion,
			tv = parseFloat(dav);

		has.add("air", dua.indexOf("AdobeAIR") >= 0);
		has.add("msapp", parseFloat(dua.split("MSAppHost/")[1]) || undefined);
		has.add("khtml", dav.indexOf("Konqueror") >= 0 ? tv : undefined);
		has.add("webkit", parseFloat(dua.split("WebKit/")[1]) || undefined);
		has.add("chrome", parseFloat(dua.split("Chrome/")[1]) || undefined);
		has.add("safari", dav.indexOf("Safari")>=0 && !has("chrome") ? parseFloat(dav.split("Version/")[1]) : undefined);
		has.add("mac", dav.indexOf("Macintosh") >= 0);
		has.add("quirks", document.compatMode == "BackCompat");
		if(dua.match(/(iPhone|iPod|iPad)/)){
			var p = RegExp.$1.replace(/P/, "p");
			var v = dua.match(/OS ([\d_]+)/) ? RegExp.$1 : "1";
			var os = parseFloat(v.replace(/_/, ".").replace(/_/g, ""));
			has.add(p, os);		// "iphone", "ipad" or "ipod"
			has.add("ios", os);
		}
		has.add("android", parseFloat(dua.split("Android ")[1]) || undefined);
		has.add("bb", (dua.indexOf("BlackBerry") >= 0 || dua.indexOf("BB10") >= 0) && parseFloat(dua.split("Version/")[1]) || undefined);

		has.add("svg", typeof SVGAngle !== "undefined");

		if(!has("webkit")){
			// Opera
			if(dua.indexOf("Opera") >= 0){
				// see http://dev.opera.com/articles/view/opera-ua-string-changes and http://www.useragentstring.com/pages/Opera/
				// 9.8 has both styles; <9.8, 9.9 only old style
				has.add("opera", tv >= 9.8 ? parseFloat(dua.split("Version/")[1]) || tv : tv);
			}

			// Mozilla and firefox
			if(dua.indexOf("Gecko") >= 0 && !has("khtml") && !has("webkit")){
				has.add("mozilla", tv);
			}
			if(has("mozilla")){
				//We really need to get away from this. Consider a sane isGecko approach for the future.
				has.add("ff", parseFloat(dua.split("Firefox/")[1] || dua.split("Minefield/")[1]) || undefined);
			}

			// IE
			if(document.all && !has("opera")){
				var isIE = parseFloat(dav.split("MSIE ")[1]) || undefined;

				//In cases where the page has an HTTP header or META tag with
				//X-UA-Compatible, then it is in emulation mode.
				//Make sure isIE reflects the desired version.
				//document.documentMode of 5 means quirks mode.
				//Only switch the value if documentMode's major version
				//is different from isIE's major version.
				var mode = document.documentMode;
				if(mode && mode != 5 && Math.floor(isIE) != mode){
					isIE = mode;
				}

				has.add("ie", isIE);
			}

			// Wii
			has.add("wii", typeof opera != "undefined" && opera.wiiremote);
		}
	}

	return has;
});

},
'dojo/_base/lang':function(){
define(["./kernel", "../has", "../sniff"], function(dojo, has){
	// module:
	//		dojo/_base/lang

	has.add("bug-for-in-skips-shadowed", function(){
		// if true, the for-in iterator skips object properties that exist in Object's prototype (IE 6 - ?)
		for(var i in {toString: 1}){
			return 0;
		}
		return 1;
	});

	// Helper methods
	var _extraNames =
			has("bug-for-in-skips-shadowed") ?
				"hasOwnProperty.valueOf.isPrototypeOf.propertyIsEnumerable.toLocaleString.toString.constructor".split(".") : [],

		_extraLen = _extraNames.length,

		getProp = function(/*Array*/parts, /*Boolean*/create, /*Object*/context){
			var p, i = 0, dojoGlobal = dojo.global;
			if(!context){
				if(!parts.length){
					return dojoGlobal;
				}else{
					p = parts[i++];
					try{
						context = dojo.scopeMap[p] && dojo.scopeMap[p][1];
					}catch(e){}
					context = context || (p in dojoGlobal ? dojoGlobal[p] : (create ? dojoGlobal[p] = {} : undefined));
				}
			}
			while(context && (p = parts[i++])){
				context = (p in context ? context[p] : (create ? context[p] = {} : undefined));
			}
			return context; // mixed
		},

		opts = Object.prototype.toString,

		efficient = function(obj, offset, startWith){
			return (startWith||[]).concat(Array.prototype.slice.call(obj, offset||0));
		},

		_pattern = /\{([^\}]+)\}/g;

	// Module export
	var lang = {
		// summary:
		//		This module defines Javascript language extensions.

		// _extraNames: String[]
		//		Lists property names that must be explicitly processed during for-in iteration
		//		in environments that have has("bug-for-in-skips-shadowed") true.
		_extraNames:_extraNames,

		_mixin: function(dest, source, copyFunc){
			// summary:
			//		Copies/adds all properties of source to dest; returns dest.
			// dest: Object
			//		The object to which to copy/add all properties contained in source.
			// source: Object
			//		The object from which to draw all properties to copy into dest.
			// copyFunc: Function?
			//		The process used to copy/add a property in source; defaults to the Javascript assignment operator.
			// returns:
			//		dest, as modified
			// description:
			//		All properties, including functions (sometimes termed "methods"), excluding any non-standard extensions
			//		found in Object.prototype, are copied/added to dest. Copying/adding each particular property is
			//		delegated to copyFunc (if any); copyFunc defaults to the Javascript assignment operator if not provided.
			//		Notice that by default, _mixin executes a so-called "shallow copy" and aggregate types are copied/added by reference.
			var name, s, i, empty = {};
			for(name in source){
				// the (!(name in empty) || empty[name] !== s) condition avoids copying properties in "source"
				// inherited from Object.prototype.	 For example, if dest has a custom toString() method,
				// don't overwrite it with the toString() method that source inherited from Object.prototype
				s = source[name];
				if(!(name in dest) || (dest[name] !== s && (!(name in empty) || empty[name] !== s))){
					dest[name] = copyFunc ? copyFunc(s) : s;
				}
			}

			if(has("bug-for-in-skips-shadowed")){
				if(source){
					for(i = 0; i < _extraLen; ++i){
						name = _extraNames[i];
						s = source[name];
						if(!(name in dest) || (dest[name] !== s && (!(name in empty) || empty[name] !== s))){
							dest[name] = copyFunc ? copyFunc(s) : s;
						}
					}
				}
			}

			return dest; // Object
		},

		mixin: function(dest, sources){
			// summary:
			//		Copies/adds all properties of one or more sources to dest; returns dest.
			// dest: Object
			//		The object to which to copy/add all properties contained in source. If dest is falsy, then
			//		a new object is manufactured before copying/adding properties begins.
			// sources: Object...
			//		One of more objects from which to draw all properties to copy into dest. sources are processed
			//		left-to-right and if more than one of these objects contain the same property name, the right-most
			//		value "wins".
			// returns: Object
			//		dest, as modified
			// description:
			//		All properties, including functions (sometimes termed "methods"), excluding any non-standard extensions
			//		found in Object.prototype, are copied/added from sources to dest. sources are processed left to right.
			//		The Javascript assignment operator is used to copy/add each property; therefore, by default, mixin
			//		executes a so-called "shallow copy" and aggregate types are copied/added by reference.
			// example:
			//		make a shallow copy of an object
			//	|	var copy = lang.mixin({}, source);
			// example:
			//		many class constructors often take an object which specifies
			//		values to be configured on the object. In this case, it is
			//		often simplest to call `lang.mixin` on the `this` object:
			//	|	declare("acme.Base", null, {
			//	|		constructor: function(properties){
			//	|			// property configuration:
			//	|			lang.mixin(this, properties);
			//	|
			//	|			console.log(this.quip);
			//	|			//	...
			//	|		},
			//	|		quip: "I wasn't born yesterday, you know - I've seen movies.",
			//	|		// ...
			//	|	});
			//	|
			//	|	// create an instance of the class and configure it
			//	|	var b = new acme.Base({quip: "That's what it does!" });
			// example:
			//		copy in properties from multiple objects
			//	|	var flattened = lang.mixin(
			//	|		{
			//	|			name: "Frylock",
			//	|			braces: true
			//	|		},
			//	|		{
			//	|			name: "Carl Brutanananadilewski"
			//	|		}
			//	|	);
			//	|
			//	|	// will print "Carl Brutanananadilewski"
			//	|	console.log(flattened.name);
			//	|	// will print "true"
			//	|	console.log(flattened.braces);

			if(!dest){ dest = {}; }
			for(var i = 1, l = arguments.length; i < l; i++){
				lang._mixin(dest, arguments[i]);
			}
			return dest; // Object
		},

		setObject: function(name, value, context){
			// summary:
			//		Set a property from a dot-separated string, such as "A.B.C"
			// description:
			//		Useful for longer api chains where you have to test each object in
			//		the chain, or when you have an object reference in string format.
			//		Objects are created as needed along `path`. Returns the passed
			//		value if setting is successful or `undefined` if not.
			// name: String
			//		Path to a property, in the form "A.B.C".
			// value: anything
			//		value or object to place at location given by name
			// context: Object?
			//		Optional. Object to use as root of path. Defaults to
			//		`dojo.global`.
			// example:
			//		set the value of `foo.bar.baz`, regardless of whether
			//		intermediate objects already exist:
			//	| lang.setObject("foo.bar.baz", value);
			// example:
			//		without `lang.setObject`, we often see code like this:
			//	| // ensure that intermediate objects are available
			//	| if(!obj["parent"]){ obj.parent = {}; }
			//	| if(!obj.parent["child"]){ obj.parent.child = {}; }
			//	| // now we can safely set the property
			//	| obj.parent.child.prop = "some value";
			//		whereas with `lang.setObject`, we can shorten that to:
			//	| lang.setObject("parent.child.prop", "some value", obj);

			var parts = name.split("."), p = parts.pop(), obj = getProp(parts, true, context);
			return obj && p ? (obj[p] = value) : undefined; // Object
		},

		getObject: function(name, create, context){
			// summary:
			//		Get a property from a dot-separated string, such as "A.B.C"
			// description:
			//		Useful for longer api chains where you have to test each object in
			//		the chain, or when you have an object reference in string format.
			// name: String
			//		Path to an property, in the form "A.B.C".
			// create: Boolean?
			//		Optional. Defaults to `false`. If `true`, Objects will be
			//		created at any point along the 'path' that is undefined.
			// context: Object?
			//		Optional. Object to use as root of path. Defaults to
			//		'dojo.global'. Null may be passed.
			return getProp(name.split("."), create, context); // Object
		},

		exists: function(name, obj){
			// summary:
			//		determine if an object supports a given method
			// description:
			//		useful for longer api chains where you have to test each object in
			//		the chain. Useful for object and method detection.
			// name: String
			//		Path to an object, in the form "A.B.C".
			// obj: Object?
			//		Object to use as root of path. Defaults to
			//		'dojo.global'. Null may be passed.
			// example:
			//	| // define an object
			//	| var foo = {
			//	|		bar: { }
			//	| };
			//	|
			//	| // search the global scope
			//	| lang.exists("foo.bar"); // true
			//	| lang.exists("foo.bar.baz"); // false
			//	|
			//	| // search from a particular scope
			//	| lang.exists("bar", foo); // true
			//	| lang.exists("bar.baz", foo); // false
			return lang.getObject(name, false, obj) !== undefined; // Boolean
		},

		// Crockford (ish) functions

		isString: function(it){
			// summary:
			//		Return true if it is a String
			// it: anything
			//		Item to test.
			return (typeof it == "string" || it instanceof String); // Boolean
		},

		isArray: function(it){
			// summary:
			//		Return true if it is an Array.
			//		Does not work on Arrays created in other windows.
			// it: anything
			//		Item to test.
			return it && (it instanceof Array || typeof it == "array"); // Boolean
		},

		isFunction: function(it){
			// summary:
			//		Return true if it is a Function
			// it: anything
			//		Item to test.
			return opts.call(it) === "[object Function]";
		},

		isObject: function(it){
			// summary:
			//		Returns true if it is a JavaScript object (or an Array, a Function
			//		or null)
			// it: anything
			//		Item to test.
			return it !== undefined &&
				(it === null || typeof it == "object" || lang.isArray(it) || lang.isFunction(it)); // Boolean
		},

		isArrayLike: function(it){
			// summary:
			//		similar to isArray() but more permissive
			// it: anything
			//		Item to test.
			// returns:
			//		If it walks like a duck and quacks like a duck, return `true`
			// description:
			//		Doesn't strongly test for "arrayness".  Instead, settles for "isn't
			//		a string or number and has a length property". Arguments objects
			//		and DOM collections will return true when passed to
			//		isArrayLike(), but will return false when passed to
			//		isArray().
			return it && it !== undefined && // Boolean
				// keep out built-in constructors (Number, String, ...) which have length
				// properties
				!lang.isString(it) && !lang.isFunction(it) &&
				!(it.tagName && it.tagName.toLowerCase() == 'form') &&
				(lang.isArray(it) || isFinite(it.length));
		},

		isAlien: function(it){
			// summary:
			//		Returns true if it is a built-in function or some other kind of
			//		oddball that *should* report as a function but doesn't
			return it && !lang.isFunction(it) && /\{\s*\[native code\]\s*\}/.test(String(it)); // Boolean
		},

		extend: function(ctor, props){
			// summary:
			//		Adds all properties and methods of props to constructor's
			//		prototype, making them available to all instances created with
			//		constructor.
			// ctor: Object
			//		Target constructor to extend.
			// props: Object
			//		One or more objects to mix into ctor.prototype
			for(var i=1, l=arguments.length; i<l; i++){
				lang._mixin(ctor.prototype, arguments[i]);
			}
			return ctor; // Object
		},

		_hitchArgs: function(scope, method){
			var pre = lang._toArray(arguments, 2);
			var named = lang.isString(method);
			return function(){
				// arrayify arguments
				var args = lang._toArray(arguments);
				// locate our method
				var f = named ? (scope||dojo.global)[method] : method;
				// invoke with collected args
				return f && f.apply(scope || this, pre.concat(args)); // mixed
			}; // Function
		},

		hitch: function(scope, method){
			// summary:
			//		Returns a function that will only ever execute in the a given scope.
			//		This allows for easy use of object member functions
			//		in callbacks and other places in which the "this" keyword may
			//		otherwise not reference the expected scope.
			//		Any number of default positional arguments may be passed as parameters
			//		beyond "method".
			//		Each of these values will be used to "placehold" (similar to curry)
			//		for the hitched function.
			// scope: Object
			//		The scope to use when method executes. If method is a string,
			//		scope is also the object containing method.
			// method: Function|String...
			//		A function to be hitched to scope, or the name of the method in
			//		scope to be hitched.
			// example:
			//	|	lang.hitch(foo, "bar")();
			//		runs foo.bar() in the scope of foo
			// example:
			//	|	lang.hitch(foo, myFunction);
			//		returns a function that runs myFunction in the scope of foo
			// example:
			//		Expansion on the default positional arguments passed along from
			//		hitch. Passed args are mixed first, additional args after.
			//	|	var foo = { bar: function(a, b, c){ console.log(a, b, c); } };
			//	|	var fn = lang.hitch(foo, "bar", 1, 2);
			//	|	fn(3); // logs "1, 2, 3"
			// example:
			//	|	var foo = { bar: 2 };
			//	|	lang.hitch(foo, function(){ this.bar = 10; })();
			//		execute an anonymous function in scope of foo
			if(arguments.length > 2){
				return lang._hitchArgs.apply(dojo, arguments); // Function
			}
			if(!method){
				method = scope;
				scope = null;
			}
			if(lang.isString(method)){
				scope = scope || dojo.global;
				if(!scope[method]){ throw(['lang.hitch: scope["', method, '"] is null (scope="', scope, '")'].join('')); }
				return function(){ return scope[method].apply(scope, arguments || []); }; // Function
			}
			return !scope ? method : function(){ return method.apply(scope, arguments || []); }; // Function
		},

		delegate: (function(){
			// boodman/crockford delegation w/ cornford optimization
			function TMP(){}
			return function(obj, props){
				TMP.prototype = obj;
				var tmp = new TMP();
				TMP.prototype = null;
				if(props){
					lang._mixin(tmp, props);
				}
				return tmp; // Object
			};
		})(),
		/*=====
		delegate: function(obj, props){
			// summary:
			//		Returns a new object which "looks" to obj for properties which it
			//		does not have a value for. Optionally takes a bag of properties to
			//		seed the returned object with initially.
			// description:
			//		This is a small implementation of the Boodman/Crockford delegation
			//		pattern in JavaScript. An intermediate object constructor mediates
			//		the prototype chain for the returned object, using it to delegate
			//		down to obj for property lookup when object-local lookup fails.
			//		This can be thought of similarly to ES4's "wrap", save that it does
			//		not act on types but rather on pure objects.
			// obj: Object
			//		The object to delegate to for properties not found directly on the
			//		return object or in props.
			// props: Object...
			//		an object containing properties to assign to the returned object
			// returns:
			//		an Object of anonymous type
			// example:
			//	|	var foo = { bar: "baz" };
			//	|	var thinger = lang.delegate(foo, { thud: "xyzzy"});
			//	|	thinger.bar == "baz"; // delegated to foo
			//	|	foo.thud == undefined; // by definition
			//	|	thinger.thud == "xyzzy"; // mixed in from props
			//	|	foo.bar = "thonk";
			//	|	thinger.bar == "thonk"; // still delegated to foo's bar
		},
		=====*/

		_toArray: has("ie") ?
			(function(){
				function slow(obj, offset, startWith){
					var arr = startWith||[];
					for(var x = offset || 0; x < obj.length; x++){
						arr.push(obj[x]);
					}
					return arr;
				}
				return function(obj){
					return ((obj.item) ? slow : efficient).apply(this, arguments);
				};
			})() : efficient,
		/*=====
		 _toArray: function(obj, offset, startWith){
			 // summary:
			 //		Converts an array-like object (i.e. arguments, DOMCollection) to an
			 //		array. Returns a new Array with the elements of obj.
			 // obj: Object
			 //		the object to "arrayify". We expect the object to have, at a
			 //		minimum, a length property which corresponds to integer-indexed
			 //		properties.
			 // offset: Number?
			 //		the location in obj to start iterating from. Defaults to 0.
			 //		Optional.
			 // startWith: Array?
			 //		An array to pack with the properties of obj. If provided,
			 //		properties in obj are appended at the end of startWith and
			 //		startWith is the returned array.
		 },
		 =====*/

		partial: function(/*Function|String*/ method /*, ...*/){
			// summary:
			//		similar to hitch() except that the scope object is left to be
			//		whatever the execution context eventually becomes.
			// description:
			//		Calling lang.partial is the functional equivalent of calling:
			//		|	lang.hitch(null, funcName, ...);
			// method:
			//		The function to "wrap"
			var arr = [ null ];
			return lang.hitch.apply(dojo, arr.concat(lang._toArray(arguments))); // Function
		},

		clone: function(/*anything*/ src){
			// summary:
			//		Clones objects (including DOM nodes) and all children.
			//		Warning: do not clone cyclic structures.
			// src:
			//		The object to clone
			if(!src || typeof src != "object" || lang.isFunction(src)){
				// null, undefined, any non-object, or function
				return src;	// anything
			}
			if(src.nodeType && "cloneNode" in src){
				// DOM Node
				return src.cloneNode(true); // Node
			}
			if(src instanceof Date){
				// Date
				return new Date(src.getTime());	// Date
			}
			if(src instanceof RegExp){
				// RegExp
				return new RegExp(src);   // RegExp
			}
			var r, i, l;
			if(lang.isArray(src)){
				// array
				r = [];
				for(i = 0, l = src.length; i < l; ++i){
					if(i in src){
						r.push(lang.clone(src[i]));
					}
				}
				// we don't clone functions for performance reasons
				//		}else if(d.isFunction(src)){
				//			// function
				//			r = function(){ return src.apply(this, arguments); };
			}else{
				// generic objects
				r = src.constructor ? new src.constructor() : {};
			}
			return lang._mixin(r, src, lang.clone);
		},


		trim: String.prototype.trim ?
			function(str){ return str.trim(); } :
			function(str){ return str.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); },
		/*=====
		 trim: function(str){
			 // summary:
			 //		Trims whitespace from both sides of the string
			 // str: String
			 //		String to be trimmed
			 // returns: String
			 //		Returns the trimmed string
			 // description:
			 //		This version of trim() was selected for inclusion into the base due
			 //		to its compact size and relatively good performance
			 //		(see [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript)
			 //		Uses String.prototype.trim instead, if available.
			 //		The fastest but longest version of this function is located at
			 //		lang.string.trim()
		 },
		 =====*/

		replace: function(tmpl, map, pattern){
			// summary:
			//		Performs parameterized substitutions on a string. Throws an
			//		exception if any parameter is unmatched.
			// tmpl: String
			//		String to be used as a template.
			// map: Object|Function
			//		If an object, it is used as a dictionary to look up substitutions.
			//		If a function, it is called for every substitution with following parameters:
			//		a whole match, a name, an offset, and the whole template
			//		string (see https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/String/replace
			//		for more details).
			// pattern: RegEx?
			//		Optional regular expression objects that overrides the default pattern.
			//		Must be global and match one item. The default is: /\{([^\}]+)\}/g,
			//		which matches patterns like that: "{xxx}", where "xxx" is any sequence
			//		of characters, which doesn't include "}".
			// returns: String
			//		Returns the substituted string.
			// example:
			//	|	// uses a dictionary for substitutions:
			//	|	lang.replace("Hello, {name.first} {name.last} AKA {nick}!",
			//	|		{
			//	|			nick: "Bob",
			//	|			name: {
			//	|				first:	"Robert",
			//	|				middle: "X",
			//	|				last:		"Cringely"
			//	|			}
			//	|		});
			//	|	// returns: Hello, Robert Cringely AKA Bob!
			// example:
			//	|	// uses an array for substitutions:
			//	|	lang.replace("Hello, {0} {2}!",
			//	|		["Robert", "X", "Cringely"]);
			//	|	// returns: Hello, Robert Cringely!
			// example:
			//	|	// uses a function for substitutions:
			//	|	function sum(a){
			//	|		var t = 0;
			//	|		arrayforEach(a, function(x){ t += x; });
			//	|		return t;
			//	|	}
			//	|	lang.replace(
			//	|		"{count} payments averaging {avg} USD per payment.",
			//	|		lang.hitch(
			//	|			{ payments: [11, 16, 12] },
			//	|			function(_, key){
			//	|				switch(key){
			//	|					case "count": return this.payments.length;
			//	|					case "min":		return Math.min.apply(Math, this.payments);
			//	|					case "max":		return Math.max.apply(Math, this.payments);
			//	|					case "sum":		return sum(this.payments);
			//	|					case "avg":		return sum(this.payments) / this.payments.length;
			//	|				}
			//	|			}
			//	|		)
			//	|	);
			//	|	// prints: 3 payments averaging 13 USD per payment.
			// example:
			//	|	// uses an alternative PHP-like pattern for substitutions:
			//	|	lang.replace("Hello, ${0} ${2}!",
			//	|		["Robert", "X", "Cringely"], /\$\{([^\}]+)\}/g);
			//	|	// returns: Hello, Robert Cringely!

			return tmpl.replace(pattern || _pattern, lang.isFunction(map) ?
				map : function(_, k){ return lang.getObject(k, false, map); });
		}
	};

	 1  && lang.mixin(dojo, lang);

	return lang;
});


},
'dojo/_base/array':function(){
define(["./kernel", "../has", "./lang"], function(dojo, has, lang){
	// module:
	//		dojo/_base/array

	// our old simple function builder stuff
	var cache = {}, u;

	function buildFn(fn){
		return cache[fn] = new Function("item", "index", "array", fn); // Function
	}
	// magic snippet: if(typeof fn == "string") fn = cache[fn] || buildFn(fn);

	// every & some

	function everyOrSome(some){
		var every = !some;
		return function(a, fn, o){
			var i = 0, l = a && a.length || 0, result;
			if(l && typeof a == "string") a = a.split("");
			if(typeof fn == "string") fn = cache[fn] || buildFn(fn);
			if(o){
				for(; i < l; ++i){
					result = !fn.call(o, a[i], i, a);
					if(some ^ result){
						return !result;
					}
				}
			}else{
				for(; i < l; ++i){
					result = !fn(a[i], i, a);
					if(some ^ result){
						return !result;
					}
				}
			}
			return every; // Boolean
		};
	}

	// indexOf, lastIndexOf

	function index(up){
		var delta = 1, lOver = 0, uOver = 0;
		if(!up){
			delta = lOver = uOver = -1;
		}
		return function(a, x, from, last){
			if(last && delta > 0){
				// TODO: why do we use a non-standard signature? why do we need "last"?
				return array.lastIndexOf(a, x, from);
			}
			var l = a && a.length || 0, end = up ? l + uOver : lOver, i;
			if(from === u){
				i = up ? lOver : l + uOver;
			}else{
				if(from < 0){
					i = l + from;
					if(i < 0){
						i = lOver;
					}
				}else{
					i = from >= l ? l + uOver : from;
				}
			}
			if(l && typeof a == "string") a = a.split("");
			for(; i != end; i += delta){
				if(a[i] == x){
					return i; // Number
				}
			}
			return -1; // Number
		};
	}

	var array = {
		// summary:
		//		The Javascript v1.6 array extensions.

		every: everyOrSome(false),
		/*=====
		 every: function(arr, callback, thisObject){
			 // summary:
			 //		Determines whether or not every item in arr satisfies the
			 //		condition implemented by callback.
			 // arr: Array|String
			 //		the array to iterate on. If a string, operates on individual characters.
			 // callback: Function|String
			 //		a function is invoked with three arguments: item, index,
			 //		and array and returns true if the condition is met.
			 // thisObject: Object?
			 //		may be used to scope the call to callback
			 // returns: Boolean
			 // description:
			 //		This function corresponds to the JavaScript 1.6 Array.every() method, with one difference: when
			 //		run over sparse arrays, this implementation passes the "holes" in the sparse array to
			 //		the callback function with a value of undefined. JavaScript 1.6's every skips the holes in the sparse array.
			 //		For more details, see:
			 //		https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/every
			 // example:
			 //	|	// returns false
			 //	|	array.every([1, 2, 3, 4], function(item){ return item>1; });
			 // example:
			 //	|	// returns true
			 //	|	array.every([1, 2, 3, 4], function(item){ return item>0; });
		 },
		 =====*/

		some: everyOrSome(true),
		/*=====
		some: function(arr, callback, thisObject){
			// summary:
			//		Determines whether or not any item in arr satisfies the
			//		condition implemented by callback.
			// arr: Array|String
			//		the array to iterate over. If a string, operates on individual characters.
			// callback: Function|String
			//		a function is invoked with three arguments: item, index,
			//		and array and returns true if the condition is met.
			// thisObject: Object?
			//		may be used to scope the call to callback
			// returns: Boolean
			// description:
			//		This function corresponds to the JavaScript 1.6 Array.some() method, with one difference: when
			//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's some skips the holes in the sparse array.
			//		For more details, see:
			//		https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/some
			// example:
			//	| // is true
			//	| array.some([1, 2, 3, 4], function(item){ return item>1; });
			// example:
			//	| // is false
			//	| array.some([1, 2, 3, 4], function(item){ return item<1; });
		},
		=====*/

		indexOf: index(true),
		/*=====
		indexOf: function(arr, value, fromIndex, findLast){
			// summary:
			//		locates the first index of the provided value in the
			//		passed array. If the value is not found, -1 is returned.
			// description:
			//		This method corresponds to the JavaScript 1.6 Array.indexOf method, with two differences:
			//
			//		1. when run over sparse arrays, the Dojo function invokes the callback for every index
			//		   whereas JavaScript 1.6's indexOf skips the holes in the sparse array.
			//		2. uses equality (==) rather than strict equality (===)
			//
			//		For details on this method, see:
			//		https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/indexOf
			// arr: Array
			// value: Object
			// fromIndex: Integer?
			// findLast: Boolean?
			//		Makes indexOf() work like lastIndexOf().  Used internally; not meant for external usage.
			// returns: Number
		},
		=====*/

		lastIndexOf: index(false),
		/*=====
		lastIndexOf: function(arr, value, fromIndex){
			// summary:
			//		locates the last index of the provided value in the passed
			//		array. If the value is not found, -1 is returned.
			// description:
		 	//		This method corresponds to the JavaScript 1.6 Array.lastIndexOf method, with two differences:
		 	//
		 	//		1. when run over sparse arrays, the Dojo function invokes the callback for every index
		 	//		   whereas JavaScript 1.6's lasIndexOf skips the holes in the sparse array.
		 	//		2. uses equality (==) rather than strict equality (===)
		 	//
		 	//		For details on this method, see:
		 	//		https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/lastIndexOf
			// arr: Array,
			// value: Object,
			// fromIndex: Integer?
			// returns: Number
		},
		=====*/

		forEach: function(arr, callback, thisObject){
			// summary:
			//		for every item in arr, callback is invoked. Return values are ignored.
			//		If you want to break out of the loop, consider using array.every() or array.some().
			//		forEach does not allow breaking out of the loop over the items in arr.
			// arr:
			//		the array to iterate over. If a string, operates on individual characters.
			// callback:
			//		a function is invoked with three arguments: item, index, and array
			// thisObject:
			//		may be used to scope the call to callback
			// description:
			//		This function corresponds to the JavaScript 1.6 Array.forEach() method, with one difference: when
			//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's forEach skips the holes in the sparse array.
			//		For more details, see:
			//		https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/forEach
			// example:
			//	| // log out all members of the array:
			//	| array.forEach(
			//	|		[ "thinger", "blah", "howdy", 10 ],
			//	|		function(item){
			//	|			console.log(item);
			//	|		}
			//	| );
			// example:
			//	| // log out the members and their indexes
			//	| array.forEach(
			//	|		[ "thinger", "blah", "howdy", 10 ],
			//	|		function(item, idx, arr){
			//	|			console.log(item, "at index:", idx);
			//	|		}
			//	| );
			// example:
			//	| // use a scoped object member as the callback
			//	|
			//	| var obj = {
			//	|		prefix: "logged via obj.callback:",
			//	|		callback: function(item){
			//	|			console.log(this.prefix, item);
			//	|		}
			//	| };
			//	|
			//	| // specifying the scope function executes the callback in that scope
			//	| array.forEach(
			//	|		[ "thinger", "blah", "howdy", 10 ],
			//	|		obj.callback,
			//	|		obj
			//	| );
			//	|
			//	| // alternately, we can accomplish the same thing with lang.hitch()
			//	| array.forEach(
			//	|		[ "thinger", "blah", "howdy", 10 ],
			//	|		lang.hitch(obj, "callback")
			//	| );
			// arr: Array|String
			// callback: Function|String
			// thisObject: Object?

			var i = 0, l = arr && arr.length || 0;
			if(l && typeof arr == "string") arr = arr.split("");
			if(typeof callback == "string") callback = cache[callback] || buildFn(callback);
			if(thisObject){
				for(; i < l; ++i){
					callback.call(thisObject, arr[i], i, arr);
				}
			}else{
				for(; i < l; ++i){
					callback(arr[i], i, arr);
				}
			}
		},

		map: function(arr, callback, thisObject, Ctr){
			// summary:
			//		applies callback to each element of arr and returns
			//		an Array with the results
			// arr: Array|String
			//		the array to iterate on. If a string, operates on
			//		individual characters.
			// callback: Function|String
			//		a function is invoked with three arguments, (item, index,
			//		array),	 and returns a value
			// thisObject: Object?
			//		may be used to scope the call to callback
			// returns: Array
			// description:
			//		This function corresponds to the JavaScript 1.6 Array.map() method, with one difference: when
			//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's map skips the holes in the sparse array.
			//		For more details, see:
			//		https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/map
			// example:
			//	| // returns [2, 3, 4, 5]
			//	| array.map([1, 2, 3, 4], function(item){ return item+1 });

			// TODO: why do we have a non-standard signature here? do we need "Ctr"?
			var i = 0, l = arr && arr.length || 0, out = new (Ctr || Array)(l);
			if(l && typeof arr == "string") arr = arr.split("");
			if(typeof callback == "string") callback = cache[callback] || buildFn(callback);
			if(thisObject){
				for(; i < l; ++i){
					out[i] = callback.call(thisObject, arr[i], i, arr);
				}
			}else{
				for(; i < l; ++i){
					out[i] = callback(arr[i], i, arr);
				}
			}
			return out; // Array
		},

		filter: function(arr, callback, thisObject){
			// summary:
			//		Returns a new Array with those items from arr that match the
			//		condition implemented by callback.
			// arr: Array
			//		the array to iterate over.
			// callback: Function|String
			//		a function that is invoked with three arguments (item,
			//		index, array). The return of this function is expected to
			//		be a boolean which determines whether the passed-in item
			//		will be included in the returned array.
			// thisObject: Object?
			//		may be used to scope the call to callback
			// returns: Array
			// description:
			//		This function corresponds to the JavaScript 1.6 Array.filter() method, with one difference: when
			//		run over sparse arrays, this implementation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's filter skips the holes in the sparse array.
			//		For more details, see:
			//		https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/filter
			// example:
			//	| // returns [2, 3, 4]
			//	| array.filter([1, 2, 3, 4], function(item){ return item>1; });

			// TODO: do we need "Ctr" here like in map()?
			var i = 0, l = arr && arr.length || 0, out = [], value;
			if(l && typeof arr == "string") arr = arr.split("");
			if(typeof callback == "string") callback = cache[callback] || buildFn(callback);
			if(thisObject){
				for(; i < l; ++i){
					value = arr[i];
					if(callback.call(thisObject, value, i, arr)){
						out.push(value);
					}
				}
			}else{
				for(; i < l; ++i){
					value = arr[i];
					if(callback(value, i, arr)){
						out.push(value);
					}
				}
			}
			return out; // Array
		},

		clearCache: function(){
			cache = {};
		}
	};


	 1  && lang.mixin(dojo, array);

	return array;
});

},
'dojo/ready':function(){
define(["./_base/kernel", "./has", "require", "./domReady", "./_base/lang"], function(dojo, has, require, domReady, lang){
	// module:
	//		dojo/ready
	// note:
	//		This module should be unnecessary in dojo 2.0

	var
		// truthy if DOMContentLoaded or better (e.g., window.onload fired) has been achieved
		isDomReady = 0,

		// The queue of functions waiting to execute as soon as dojo.ready conditions satisfied
		loadQ = [],

		// prevent recursion in onLoad
		onLoadRecursiveGuard = 0,

		handleDomReady = function(){
			isDomReady = 1;
			dojo._postLoad = dojo.config.afterOnLoad = true;
			onEvent();
		},

		onEvent = function(){
			// Called when some state changes:
			//		- dom ready
			//		- dojo/domReady has finished processing everything in its queue
			//		- task added to loadQ
			//		- require() has finished loading all currently requested modules
			//
			// Run the functions queued with dojo.ready if appropriate.


			//guard against recursions into this function
			if(onLoadRecursiveGuard){
				return;
			}
			onLoadRecursiveGuard = 1;

			// Run tasks in queue if require() is finished loading modules, the dom is ready, and there are no
			// pending tasks registered via domReady().
			// The last step is necessary so that a user defined dojo.ready() callback is delayed until after the
			// domReady() calls inside of dojo.	  Failure can be seen on dijit/tests/robot/Dialog_ally.html on IE8
			// because the dijit/focus.js domReady() callback doesn't execute until after the test starts running.
			while(isDomReady && (!domReady || domReady._Q.length == 0) && (require.idle ? require.idle() : true) && loadQ.length){
				var f = loadQ.shift();
				try{
					f();
				}catch(e){
					// force the dojo.js on("error") handler do display the message
					e.info = e.message;
					if(require.signal){
						require.signal("error", e);
					}else{
						throw e;
					}
				}
			}

			onLoadRecursiveGuard = 0;
		};

	// Check if we should run the next queue operation whenever require() finishes loading modules or domReady
	// finishes processing it's queue.
	require.on && require.on("idle", onEvent);
	if(domReady){
		domReady._onQEmpty = onEvent;
	}

	var ready = dojo.ready = dojo.addOnLoad = function(priority, context, callback){
		// summary:
		//		Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.
		//		In most cases, the `domReady` plug-in should suffice and this method should not be needed.
		//
		//		When called in a non-browser environment, just checks that all requested modules have arrived and been
		//		evaluated.
		// priority: Integer?
		//		The order in which to exec this callback relative to other callbacks, defaults to 1000
		// context: Object?|Function
		//		The context in which to run execute callback, or a callback if not using context
		// callback: Function?
		//		The function to execute.
		//
		// example:
		//	Simple DOM and Modules ready syntax
		//	|	require(["dojo/ready"], function(ready){
		//	|		ready(function(){ alert("Dom ready!"); });
		//	|	});
		//
		// example:
		//	Using a priority
		//	|	require(["dojo/ready"], function(ready){
		//	|		ready(2, function(){ alert("low priority ready!"); })
		//	|	});
		//
		// example:
		//	Using context
		//	|	require(["dojo/ready"], function(ready){
		//	|		ready(foo, function(){
		//	|			// in here, this == foo
		//	|		});
		//	|	});
		//
		// example:
		//	Using dojo/hitch style args:
		//	|	require(["dojo/ready"], function(ready){
		//	|		var foo = { dojoReady: function(){ console.warn(this, "dojo dom and modules ready."); } };
		//	|		ready(foo, "dojoReady");
		//	|	});

		var hitchArgs = lang._toArray(arguments);
		if(typeof priority != "number"){
			callback = context;
			context = priority;
			priority = 1000;
		}else{
			hitchArgs.shift();
		}
		callback = callback ?
			lang.hitch.apply(dojo, hitchArgs) :
			function(){
				context();
			};
		callback.priority = priority;
		for(var i = 0; i < loadQ.length && priority >= loadQ[i].priority; i++){}
		loadQ.splice(i, 0, callback);
		onEvent();
	};

	 1 || has.add("dojo-config-addOnLoad", 1);
	if( 1 ){
		var dca = dojo.config.addOnLoad;
		if(dca){
			ready[(lang.isArray(dca) ? "apply" : "call")](dojo, dca);
		}
	}

	if( 1  && dojo.config.parseOnLoad && !dojo.isAsync){
		ready(99, function(){
			if(!dojo.parser){
				dojo.deprecated("Add explicit require(['dojo/parser']);", "", "2.0");
				require(["dojo/parser"]);
			}
		});
	}

	if(domReady){
		domReady(handleDomReady);
	}else{
		handleDomReady();
	}

	return ready;
});

},
'dojo/domReady':function(){
define(['./has'], function(has){
	var global = this,
		doc = document,
		readyStates = { 'loaded': 1, 'complete': 1 },
		fixReadyState = typeof doc.readyState != "string",
		ready = !!readyStates[doc.readyState],
		readyQ = [],
		recursiveGuard;

	function domReady(callback){
		// summary:
		//		Plugin to delay require()/define() callback from firing until the DOM has finished loading.
		readyQ.push(callback);
		if(ready){ processQ(); }
	}
	domReady.load = function(id, req, load){
		domReady(load);
	};

	// Export queue so that ready() can check if it's empty or not.
	domReady._Q = readyQ;
	domReady._onQEmpty = function(){
		// summary:
		//		Private method overridden by dojo/ready, to notify when everything in the
		//		domReady queue has been processed.  Do not use directly.
		//		Will be removed in 2.0, along with domReady._Q.
	};

	// For FF <= 3.5
	if(fixReadyState){ doc.readyState = "loading"; }

	function processQ(){
		// Calls all functions in the queue in order, unless processQ() is already running, in which case just return

		if(recursiveGuard){ return; }
		recursiveGuard = true;

		while(readyQ.length){
			try{
				(readyQ.shift())(doc);
			}catch(err){
				console.log("Error on domReady callback: " + err);
			}
		}

		recursiveGuard = false;

		// Notification for dojo/ready.  Remove for 2.0.
		// Note that this could add more tasks to the ready queue.
		domReady._onQEmpty();
	}

	if(!ready){
		var tests = [],
			detectReady = function(evt){
				evt = evt || global.event;
				if(ready || (evt.type == "readystatechange" && !readyStates[doc.readyState])){ return; }

				// For FF <= 3.5
				if(fixReadyState){ doc.readyState = "complete"; }

				ready = 1;
				processQ();
			},
			on = function(node, event){
				node.addEventListener(event, detectReady, false);
				readyQ.push(function(){ node.removeEventListener(event, detectReady, false); });
			};

		if(!has("dom-addeventlistener")){
			on = function(node, event){
				event = "on" + event;
				node.attachEvent(event, detectReady);
				readyQ.push(function(){ node.detachEvent(event, detectReady); });
			};

			var div = doc.createElement("div");
			try{
				if(div.doScroll && global.frameElement === null){
					// the doScroll test is only useful if we're in the top-most frame
					tests.push(function(){
						// Derived with permission from Diego Perini's IEContentLoaded
						// http://javascript.nwbox.com/IEContentLoaded/
						try{
							div.doScroll("left");
							return 1;
						}catch(e){}
					});
				}
			}catch(e){}
		}

		on(doc, "DOMContentLoaded");
		on(global, "load");

		if("onreadystatechange" in doc){
			on(doc, "readystatechange");
		}else if(!fixReadyState){
			// if the ready state property exists and there's
			// no readystatechange event, poll for the state
			// to change
			tests.push(function(){
				return readyStates[doc.readyState];
			});
		}

		if(tests.length){
			var poller = function(){
				if(ready){ return; }
				var i = tests.length;
				while(i--){
					if(tests[i]()){
						detectReady("poller");
						return;
					}
				}
				setTimeout(poller, 30);
			};
			poller();
		}
	}

	return domReady;
});

},
'dojo/_base/declare':function(){
define(["./kernel", "../has", "./lang"], function(dojo, has, lang){
	// module:
	//		dojo/_base/declare

	var mix = lang.mixin, op = Object.prototype, opts = op.toString,
		xtor = new Function, counter = 0, cname = "constructor";

	function err(msg, cls){ throw new Error("declare" + (cls ? " " + cls : "") + ": " + msg); }

	// C3 Method Resolution Order (see http://www.python.org/download/releases/2.3/mro/)
	function c3mro(bases, className){
		var result = [], roots = [{cls: 0, refs: []}], nameMap = {}, clsCount = 1,
			l = bases.length, i = 0, j, lin, base, top, proto, rec, name, refs;

		// build a list of bases naming them if needed
		for(; i < l; ++i){
			base = bases[i];
			if(!base){
				err("mixin #" + i + " is unknown. Did you use dojo.require to pull it in?", className);
			}else if(opts.call(base) != "[object Function]"){
				err("mixin #" + i + " is not a callable constructor.", className);
			}
			lin = base._meta ? base._meta.bases : [base];
			top = 0;
			// add bases to the name map
			for(j = lin.length - 1; j >= 0; --j){
				proto = lin[j].prototype;
				if(!proto.hasOwnProperty("declaredClass")){
					proto.declaredClass = "uniqName_" + (counter++);
				}
				name = proto.declaredClass;
				if(!nameMap.hasOwnProperty(name)){
					nameMap[name] = {count: 0, refs: [], cls: lin[j]};
					++clsCount;
				}
				rec = nameMap[name];
				if(top && top !== rec){
					rec.refs.push(top);
					++top.count;
				}
				top = rec;
			}
			++top.count;
			roots[0].refs.push(top);
		}

		// remove classes without external references recursively
		while(roots.length){
			top = roots.pop();
			result.push(top.cls);
			--clsCount;
			// optimization: follow a single-linked chain
			while(refs = top.refs, refs.length == 1){
				top = refs[0];
				if(!top || --top.count){
					// branch or end of chain => do not end to roots
					top = 0;
					break;
				}
				result.push(top.cls);
				--clsCount;
			}
			if(top){
				// branch
				for(i = 0, l = refs.length; i < l; ++i){
					top = refs[i];
					if(!--top.count){
						roots.push(top);
					}
				}
			}
		}
		if(clsCount){
			err("can't build consistent linearization", className);
		}

		// calculate the superclass offset
		base = bases[0];
		result[0] = base ?
			base._meta && base === result[result.length - base._meta.bases.length] ?
				base._meta.bases.length : 1 : 0;

		return result;
	}

	function inherited(args, a, f){
		var name, chains, bases, caller, meta, base, proto, opf, pos,
			cache = this._inherited = this._inherited || {};

		// crack arguments
		if(typeof args == "string"){
			name = args;
			args = a;
			a = f;
		}
		f = 0;

		caller = args.callee;
		name = name || caller.nom;
		if(!name){
			err("can't deduce a name to call inherited()", this.declaredClass);
		}

		meta = this.constructor._meta;
		bases = meta.bases;

		pos = cache.p;
		if(name != cname){
			// method
			if(cache.c !== caller){
				// cache bust
				pos = 0;
				base = bases[0];
				meta = base._meta;
				if(meta.hidden[name] !== caller){
					// error detection
					chains = meta.chains;
					if(chains && typeof chains[name] == "string"){
						err("calling chained method with inherited: " + name, this.declaredClass);
					}
					// find caller
					do{
						meta = base._meta;
						proto = base.prototype;
						if(meta && (proto[name] === caller && proto.hasOwnProperty(name) || meta.hidden[name] === caller)){
							break;
						}
					}while(base = bases[++pos]); // intentional assignment
					pos = base ? pos : -1;
				}
			}
			// find next
			base = bases[++pos];
			if(base){
				proto = base.prototype;
				if(base._meta && proto.hasOwnProperty(name)){
					f = proto[name];
				}else{
					opf = op[name];
					do{
						proto = base.prototype;
						f = proto[name];
						if(f && (base._meta ? proto.hasOwnProperty(name) : f !== opf)){
							break;
						}
					}while(base = bases[++pos]); // intentional assignment
				}
			}
			f = base && f || op[name];
		}else{
			// constructor
			if(cache.c !== caller){
				// cache bust
				pos = 0;
				meta = bases[0]._meta;
				if(meta && meta.ctor !== caller){
					// error detection
					chains = meta.chains;
					if(!chains || chains.constructor !== "manual"){
						err("calling chained constructor with inherited", this.declaredClass);
					}
					// find caller
					while(base = bases[++pos]){ // intentional assignment
						meta = base._meta;
						if(meta && meta.ctor === caller){
							break;
						}
					}
					pos = base ? pos : -1;
				}
			}
			// find next
			while(base = bases[++pos]){	// intentional assignment
				meta = base._meta;
				f = meta ? meta.ctor : base;
				if(f){
					break;
				}
			}
			f = base && f;
		}

		// cache the found super method
		cache.c = f;
		cache.p = pos;

		// now we have the result
		if(f){
			return a === true ? f : f.apply(this, a || args);
		}
		// intentionally no return if a super method was not found
	}

	function getInherited(name, args){
		if(typeof name == "string"){
			return this.__inherited(name, args, true);
		}
		return this.__inherited(name, true);
	}

	function inherited__debug(args, a1, a2){
		var f = this.getInherited(args, a1);
		if(f){ return f.apply(this, a2 || a1 || args); }
		// intentionally no return if a super method was not found
	}

	var inheritedImpl = dojo.config.isDebug ? inherited__debug : inherited;

	// emulation of "instanceof"
	function isInstanceOf(cls){
		var bases = this.constructor._meta.bases;
		for(var i = 0, l = bases.length; i < l; ++i){
			if(bases[i] === cls){
				return true;
			}
		}
		return this instanceof cls;
	}

	function mixOwn(target, source){
		// add props adding metadata for incoming functions skipping a constructor
		for(var name in source){
			if(name != cname && source.hasOwnProperty(name)){
				target[name] = source[name];
			}
		}
		if(has("bug-for-in-skips-shadowed")){
			for(var extraNames= lang._extraNames, i= extraNames.length; i;){
				name = extraNames[--i];
				if(name != cname && source.hasOwnProperty(name)){
					  target[name] = source[name];
				}
			}
		}
	}

	// implementation of safe mixin function
	function safeMixin(target, source){
		// summary:
		//		Mix in properties skipping a constructor and decorating functions
		//		like it is done by declare().
		// target: Object
		//		Target object to accept new properties.
		// source: Object
		//		Source object for new properties.
		// description:
		//		This function is used to mix in properties like lang.mixin does,
		//		but it skips a constructor property and decorates functions like
		//		declare() does.
		//
		//		It is meant to be used with classes and objects produced with
		//		declare. Functions mixed in with dojo.safeMixin can use
		//		this.inherited() like normal methods.
		//
		//		This function is used to implement extend() method of a constructor
		//		produced with declare().
		//
		// example:
		//	|	var A = declare(null, {
		//	|		m1: function(){
		//	|			console.log("A.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("A.m2");
		//	|		}
		//	|	});
		//	|	var B = declare(A, {
		//	|		m1: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("B.m1");
		//	|		}
		//	|	});
		//	|	B.extend({
		//	|		m2: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("B.m2");
		//	|		}
		//	|	});
		//	|	var x = new B();
		//	|	dojo.safeMixin(x, {
		//	|		m1: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("X.m1");
		//	|		},
		//	|		m2: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("X.m2");
		//	|		}
		//	|	});
		//	|	x.m2();
		//	|	// prints:
		//	|	// A.m1
		//	|	// B.m1
		//	|	// X.m1

		var name, t;
		// add props adding metadata for incoming functions skipping a constructor
		for(name in source){
			t = source[name];
			if((t !== op[name] || !(name in op)) && name != cname){
				if(opts.call(t) == "[object Function]"){
					// non-trivial function method => attach its name
					t.nom = name;
				}
				target[name] = t;
			}
		}
		if(has("bug-for-in-skips-shadowed")){
			for(var extraNames= lang._extraNames, i= extraNames.length; i;){
				name = extraNames[--i];
				t = source[name];
				if((t !== op[name] || !(name in op)) && name != cname){
					if(opts.call(t) == "[object Function]"){
						// non-trivial function method => attach its name
						  t.nom = name;
					}
					target[name] = t;
				}
			}
		}
		return target;
	}

	function extend(source){
		declare.safeMixin(this.prototype, source);
		return this;
	}

	function createSubclass(mixins, props){
		return declare([this].concat(mixins), props || {});
	}

	// chained constructor compatible with the legacy declare()
	function chainedConstructor(bases, ctorSpecial){
		return function(){
			var a = arguments, args = a, a0 = a[0], f, i, m,
				l = bases.length, preArgs;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original declare()
			// 1) call two types of the preamble
			if(ctorSpecial && (a0 && a0.preamble || this.preamble)){
				// full blown ritual
				preArgs = new Array(bases.length);
				// prepare parameters
				preArgs[0] = a;
				for(i = 0;;){
					// process the preamble of the 1st argument
					a0 = a[0];
					if(a0){
						f = a0.preamble;
						if(f){
							a = f.apply(this, a) || a;
						}
					}
					// process the preamble of this class
					f = bases[i].prototype;
					f = f.hasOwnProperty("preamble") && f.preamble;
					if(f){
						a = f.apply(this, a) || a;
					}
					// one peculiarity of the preamble:
					// it is called if it is not needed,
					// e.g., there is no constructor to call
					// let's watch for the last constructor
					// (see ticket #9795)
					if(++i == l){
						break;
					}
					preArgs[i] = a;
				}
			}
			// 2) call all non-trivial constructors using prepared arguments
			for(i = l - 1; i >= 0; --i){
				f = bases[i];
				m = f._meta;
				f = m ? m.ctor : f;
				if(f){
					f.apply(this, preArgs ? preArgs[i] : a);
				}
			}
			// 3) continue the original ritual: call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, args);
			}
		};
	}


	// chained constructor compatible with the legacy declare()
	function singleConstructor(ctor, ctorSpecial){
		return function(){
			var a = arguments, t = a, a0 = a[0], f;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original declare()
			// 1) call two types of the preamble
			if(ctorSpecial){
				// full blown ritual
				if(a0){
					// process the preamble of the 1st argument
					f = a0.preamble;
					if(f){
						t = f.apply(this, t) || t;
					}
				}
				f = this.preamble;
				if(f){
					// process the preamble of this class
					f.apply(this, t);
					// one peculiarity of the preamble:
					// it is called even if it is not needed,
					// e.g., there is no constructor to call
					// let's watch for the last constructor
					// (see ticket #9795)
				}
			}
			// 2) call a constructor
			if(ctor){
				ctor.apply(this, a);
			}
			// 3) continue the original ritual: call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, a);
			}
		};
	}

	// plain vanilla constructor (can use inherited() to call its base constructor)
	function simpleConstructor(bases){
		return function(){
			var a = arguments, i = 0, f, m;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original declare()
			// 1) do not call the preamble
			// 2) call the top constructor (it can use this.inherited())
			for(; f = bases[i]; ++i){ // intentional assignment
				m = f._meta;
				f = m ? m.ctor : f;
				if(f){
					f.apply(this, a);
					break;
				}
			}
			// 3) call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, a);
			}
		};
	}

	function chain(name, bases, reversed){
		return function(){
			var b, m, f, i = 0, step = 1;
			if(reversed){
				i = bases.length - 1;
				step = -1;
			}
			for(; b = bases[i]; i += step){ // intentional assignment
				m = b._meta;
				f = (m ? m.hidden : b.prototype)[name];
				if(f){
					f.apply(this, arguments);
				}
			}
		};
	}

	// forceNew(ctor)
	// return a new object that inherits from ctor.prototype but
	// without actually running ctor on the object.
	function forceNew(ctor){
		// create object with correct prototype using a do-nothing
		// constructor
		xtor.prototype = ctor.prototype;
		var t = new xtor;
		xtor.prototype = null;	// clean up
		return t;
	}

	// applyNew(args)
	// just like 'new ctor()' except that the constructor and its arguments come
	// from args, which must be an array or an arguments object
	function applyNew(args){
		// create an object with ctor's prototype but without
		// calling ctor on it.
		var ctor = args.callee, t = forceNew(ctor);
		// execute the real constructor on the new object
		ctor.apply(t, args);
		return t;
	}

	function declare(className, superclass, props){
		// summary:
		//		Create a feature-rich constructor from compact notation.
		// className: String?
		//		The optional name of the constructor (loosely, a "class")
		//		stored in the "declaredClass" property in the created prototype.
		//		It will be used as a global name for a created constructor.
		// superclass: Function|Function[]
		//		May be null, a Function, or an Array of Functions. This argument
		//		specifies a list of bases (the left-most one is the most deepest
		//		base).
		// props: Object
		//		An object whose properties are copied to the created prototype.
		//		Add an instance-initialization function by making it a property
		//		named "constructor".
		// returns: dojo/_base/declare.__DeclareCreatedObject
		//		New constructor function.
		// description:
		//		Create a constructor using a compact notation for inheritance and
		//		prototype extension.
		//
		//		Mixin ancestors provide a type of multiple inheritance.
		//		Prototypes of mixin ancestors are copied to the new class:
		//		changes to mixin prototypes will not affect classes to which
		//		they have been mixed in.
		//
		//		Ancestors can be compound classes created by this version of
		//		declare(). In complex cases all base classes are going to be
		//		linearized according to C3 MRO algorithm
		//		(see http://www.python.org/download/releases/2.3/mro/ for more
		//		details).
		//
		//		"className" is cached in "declaredClass" property of the new class,
		//		if it was supplied. The immediate super class will be cached in
		//		"superclass" property of the new class.
		//
		//		Methods in "props" will be copied and modified: "nom" property
		//		(the declared name of the method) will be added to all copied
		//		functions to help identify them for the internal machinery. Be
		//		very careful, while reusing methods: if you use the same
		//		function under different names, it can produce errors in some
		//		cases.
		//
		//		It is possible to use constructors created "manually" (without
		//		declare()) as bases. They will be called as usual during the
		//		creation of an instance, their methods will be chained, and even
		//		called by "this.inherited()".
		//
		//		Special property "-chains-" governs how to chain methods. It is
		//		a dictionary, which uses method names as keys, and hint strings
		//		as values. If a hint string is "after", this method will be
		//		called after methods of its base classes. If a hint string is
		//		"before", this method will be called before methods of its base
		//		classes.
		//
		//		If "constructor" is not mentioned in "-chains-" property, it will
		//		be chained using the legacy mode: using "after" chaining,
		//		calling preamble() method before each constructor, if available,
		//		and calling postscript() after all constructors were executed.
		//		If the hint is "after", it is chained as a regular method, but
		//		postscript() will be called after the chain of constructors.
		//		"constructor" cannot be chained "before", but it allows
		//		a special hint string: "manual", which means that constructors
		//		are not going to be chained in any way, and programmer will call
		//		them manually using this.inherited(). In the latter case
		//		postscript() will be called after the construction.
		//
		//		All chaining hints are "inherited" from base classes and
		//		potentially can be overridden. Be very careful when overriding
		//		hints! Make sure that all chained methods can work in a proposed
		//		manner of chaining.
		//
		//		Once a method was chained, it is impossible to unchain it. The
		//		only exception is "constructor". You don't need to define a
		//		method in order to supply a chaining hint.
		//
		//		If a method is chained, it cannot use this.inherited() because
		//		all other methods in the hierarchy will be called automatically.
		//
		//		Usually constructors and initializers of any kind are chained
		//		using "after" and destructors of any kind are chained as
		//		"before". Note that chaining assumes that chained methods do not
		//		return any value: any returned value will be discarded.
		//
		// example:
		//	|	declare("my.classes.bar", my.classes.foo, {
		//	|		// properties to be added to the class prototype
		//	|		someValue: 2,
		//	|		// initialization function
		//	|		constructor: function(){
		//	|			this.myComplicatedObject = new ReallyComplicatedObject();
		//	|		},
		//	|		// other functions
		//	|		someMethod: function(){
		//	|			doStuff();
		//	|		}
		//	|	});
		//
		// example:
		//	|	var MyBase = declare(null, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyClass1 = declare(MyBase, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyClass2 = declare(MyBase, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyDiamond = declare([MyClass1, MyClass2], {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//
		// example:
		//	|	var F = function(){ console.log("raw constructor"); };
		//	|	F.prototype.method = function(){
		//	|		console.log("raw method");
		//	|	};
		//	|	var A = declare(F, {
		//	|		constructor: function(){
		//	|			console.log("A.constructor");
		//	|		},
		//	|		method: function(){
		//	|			console.log("before calling F.method...");
		//	|			this.inherited(arguments);
		//	|			console.log("...back in A");
		//	|		}
		//	|	});
		//	|	new A().method();
		//	|	// will print:
		//	|	// raw constructor
		//	|	// A.constructor
		//	|	// before calling F.method...
		//	|	// raw method
		//	|	// ...back in A
		//
		// example:
		//	|	var A = declare(null, {
		//	|		"-chains-": {
		//	|			destroy: "before"
		//	|		}
		//	|	});
		//	|	var B = declare(A, {
		//	|		constructor: function(){
		//	|			console.log("B.constructor");
		//	|		},
		//	|		destroy: function(){
		//	|			console.log("B.destroy");
		//	|		}
		//	|	});
		//	|	var C = declare(B, {
		//	|		constructor: function(){
		//	|			console.log("C.constructor");
		//	|		},
		//	|		destroy: function(){
		//	|			console.log("C.destroy");
		//	|		}
		//	|	});
		//	|	new C().destroy();
		//	|	// prints:
		//	|	// B.constructor
		//	|	// C.constructor
		//	|	// C.destroy
		//	|	// B.destroy
		//
		// example:
		//	|	var A = declare(null, {
		//	|		"-chains-": {
		//	|			constructor: "manual"
		//	|		}
		//	|	});
		//	|	var B = declare(A, {
		//	|		constructor: function(){
		//	|			// ...
		//	|			// call the base constructor with new parameters
		//	|			this.inherited(arguments, [1, 2, 3]);
		//	|			// ...
		//	|		}
		//	|	});
		//
		// example:
		//	|	var A = declare(null, {
		//	|		"-chains-": {
		//	|			m1: "before"
		//	|		},
		//	|		m1: function(){
		//	|			console.log("A.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("A.m2");
		//	|		}
		//	|	});
		//	|	var B = declare(A, {
		//	|		"-chains-": {
		//	|			m2: "after"
		//	|		},
		//	|		m1: function(){
		//	|			console.log("B.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("B.m2");
		//	|		}
		//	|	});
		//	|	var x = new B();
		//	|	x.m1();
		//	|	// prints:
		//	|	// B.m1
		//	|	// A.m1
		//	|	x.m2();
		//	|	// prints:
		//	|	// A.m2
		//	|	// B.m2

		// crack parameters
		if(typeof className != "string"){
			props = superclass;
			superclass = className;
			className = "";
		}
		props = props || {};

		var proto, i, t, ctor, name, bases, chains, mixins = 1, parents = superclass;

		// build a prototype
		if(opts.call(superclass) == "[object Array]"){
			// C3 MRO
			bases = c3mro(superclass, className);
			t = bases[0];
			mixins = bases.length - t;
			superclass = bases[mixins];
		}else{
			bases = [0];
			if(superclass){
				if(opts.call(superclass) == "[object Function]"){
					t = superclass._meta;
					bases = bases.concat(t ? t.bases : superclass);
				}else{
					err("base class is not a callable constructor.", className);
				}
			}else if(superclass !== null){
				err("unknown base class. Did you use dojo.require to pull it in?", className);
			}
		}
		if(superclass){
			for(i = mixins - 1;; --i){
				proto = forceNew(superclass);
				if(!i){
					// stop if nothing to add (the last base)
					break;
				}
				// mix in properties
				t = bases[i];
				(t._meta ? mixOwn : mix)(proto, t.prototype);
				// chain in new constructor
				ctor = new Function;
				ctor.superclass = superclass;
				ctor.prototype = proto;
				superclass = proto.constructor = ctor;
			}
		}else{
			proto = {};
		}
		// add all properties
		declare.safeMixin(proto, props);
		// add constructor
		t = props.constructor;
		if(t !== op.constructor){
			t.nom = cname;
			proto.constructor = t;
		}

		// collect chains and flags
		for(i = mixins - 1; i; --i){ // intentional assignment
			t = bases[i]._meta;
			if(t && t.chains){
				chains = mix(chains || {}, t.chains);
			}
		}
		if(proto["-chains-"]){
			chains = mix(chains || {}, proto["-chains-"]);
		}

		// build ctor
		t = !chains || !chains.hasOwnProperty(cname);
		bases[0] = ctor = (chains && chains.constructor === "manual") ? simpleConstructor(bases) :
			(bases.length == 1 ? singleConstructor(props.constructor, t) : chainedConstructor(bases, t));

		// add meta information to the constructor
		ctor._meta  = {bases: bases, hidden: props, chains: chains,
			parents: parents, ctor: props.constructor};
		ctor.superclass = superclass && superclass.prototype;
		ctor.extend = extend;
		ctor.createSubclass = createSubclass;
		ctor.prototype = proto;
		proto.constructor = ctor;

		// add "standard" methods to the prototype
		proto.getInherited = getInherited;
		proto.isInstanceOf = isInstanceOf;
		proto.inherited    = inheritedImpl;
		proto.__inherited  = inherited;

		// add name if specified
		if(className){
			proto.declaredClass = className;
			lang.setObject(className, ctor);
		}

		// build chains and add them to the prototype
		if(chains){
			for(name in chains){
				if(proto[name] && typeof chains[name] == "string" && name != cname){
					t = proto[name] = chain(name, bases, chains[name] === "after");
					t.nom = name;
				}
			}
		}
		// chained methods do not return values
		// no need to chain "invisible" functions

		return ctor;	// Function
	}

	/*=====
	declare.__DeclareCreatedObject = {
		// summary:
		//		dojo/_base/declare() returns a constructor `C`.   `new C()` returns an Object with the following
		//		methods, in addition to the methods and properties specified via the arguments passed to declare().

		inherited: function(name, args, newArgs){
			// summary:
			//		Calls a super method.
			// name: String?
			//		The optional method name. Should be the same as the caller's
			//		name. Usually "name" is specified in complex dynamic cases, when
			//		the calling method was dynamically added, undecorated by
			//		declare(), and it cannot be determined.
			// args: Arguments
			//		The caller supply this argument, which should be the original
			//		"arguments".
			// newArgs: Object?
			//		If "true", the found function will be returned without
			//		executing it.
			//		If Array, it will be used to call a super method. Otherwise
			//		"args" will be used.
			// returns:
			//		Whatever is returned by a super method, or a super method itself,
			//		if "true" was specified as newArgs.
			// description:
			//		This method is used inside method of classes produced with
			//		declare() to call a super method (next in the chain). It is
			//		used for manually controlled chaining. Consider using the regular
			//		chaining, because it is faster. Use "this.inherited()" only in
			//		complex cases.
			//
			//		This method cannot me called from automatically chained
			//		constructors including the case of a special (legacy)
			//		constructor chaining. It cannot be called from chained methods.
			//
			//		If "this.inherited()" cannot find the next-in-chain method, it
			//		does nothing and returns "undefined". The last method in chain
			//		can be a default method implemented in Object, which will be
			//		called last.
			//
			//		If "name" is specified, it is assumed that the method that
			//		received "args" is the parent method for this call. It is looked
			//		up in the chain list and if it is found the next-in-chain method
			//		is called. If it is not found, the first-in-chain method is
			//		called.
			//
			//		If "name" is not specified, it will be derived from the calling
			//		method (using a methoid property "nom").
			//
			// example:
			//	|	var B = declare(A, {
			//	|		method1: function(a, b, c){
			//	|			this.inherited(arguments);
			//	|		},
			//	|		method2: function(a, b){
			//	|			return this.inherited(arguments, [a + b]);
			//	|		}
			//	|	});
			//	|	// next method is not in the chain list because it is added
			//	|	// manually after the class was created.
			//	|	B.prototype.method3 = function(){
			//	|		console.log("This is a dynamically-added method.");
			//	|		this.inherited("method3", arguments);
			//	|	};
			// example:
			//	|	var B = declare(A, {
			//	|		method: function(a, b){
			//	|			var super = this.inherited(arguments, true);
			//	|			// ...
			//	|			if(!super){
			//	|				console.log("there is no super method");
			//	|				return 0;
			//	|			}
			//	|			return super.apply(this, arguments);
			//	|		}
			//	|	});
			return	{};	// Object
		},

		getInherited: function(name, args){
			// summary:
			//		Returns a super method.
			// name: String?
			//		The optional method name. Should be the same as the caller's
			//		name. Usually "name" is specified in complex dynamic cases, when
			//		the calling method was dynamically added, undecorated by
			//		declare(), and it cannot be determined.
			// args: Arguments
			//		The caller supply this argument, which should be the original
			//		"arguments".
			// returns:
			//		Returns a super method (Function) or "undefined".
			// description:
			//		This method is a convenience method for "this.inherited()".
			//		It uses the same algorithm but instead of executing a super
			//		method, it returns it, or "undefined" if not found.
			//
			// example:
			//	|	var B = declare(A, {
			//	|		method: function(a, b){
			//	|			var super = this.getInherited(arguments);
			//	|			// ...
			//	|			if(!super){
			//	|				console.log("there is no super method");
			//	|				return 0;
			//	|			}
			//	|			return super.apply(this, arguments);
			//	|		}
			//	|	});
			return	{};	// Object
		},

		isInstanceOf: function(cls){
			// summary:
			//		Checks the inheritance chain to see if it is inherited from this
			//		class.
			// cls: Function
			//		Class constructor.
			// returns:
			//		"true", if this object is inherited from this class, "false"
			//		otherwise.
			// description:
			//		This method is used with instances of classes produced with
			//		declare() to determine of they support a certain interface or
			//		not. It models "instanceof" operator.
			//
			// example:
			//	|	var A = declare(null, {
			//	|		// constructor, properties, and methods go here
			//	|		// ...
			//	|	});
			//	|	var B = declare(null, {
			//	|		// constructor, properties, and methods go here
			//	|		// ...
			//	|	});
			//	|	var C = declare([A, B], {
			//	|		// constructor, properties, and methods go here
			//	|		// ...
			//	|	});
			//	|	var D = declare(A, {
			//	|		// constructor, properties, and methods go here
			//	|		// ...
			//	|	});
			//	|
			//	|	var a = new A(), b = new B(), c = new C(), d = new D();
			//	|
			//	|	console.log(a.isInstanceOf(A)); // true
			//	|	console.log(b.isInstanceOf(A)); // false
			//	|	console.log(c.isInstanceOf(A)); // true
			//	|	console.log(d.isInstanceOf(A)); // true
			//	|
			//	|	console.log(a.isInstanceOf(B)); // false
			//	|	console.log(b.isInstanceOf(B)); // true
			//	|	console.log(c.isInstanceOf(B)); // true
			//	|	console.log(d.isInstanceOf(B)); // false
			//	|
			//	|	console.log(a.isInstanceOf(C)); // false
			//	|	console.log(b.isInstanceOf(C)); // false
			//	|	console.log(c.isInstanceOf(C)); // true
			//	|	console.log(d.isInstanceOf(C)); // false
			//	|
			//	|	console.log(a.isInstanceOf(D)); // false
			//	|	console.log(b.isInstanceOf(D)); // false
			//	|	console.log(c.isInstanceOf(D)); // false
			//	|	console.log(d.isInstanceOf(D)); // true
			return	{};	// Object
		},

		extend: function(source){
			// summary:
			//		Adds all properties and methods of source to constructor's
			//		prototype, making them available to all instances created with
			//		constructor. This method is specific to constructors created with
			//		declare().
			// source: Object
			//		Source object which properties are going to be copied to the
			//		constructor's prototype.
			// description:
			//		Adds source properties to the constructor's prototype. It can
			//		override existing properties.
			//
			//		This method is similar to dojo.extend function, but it is specific
			//		to constructors produced by declare(). It is implemented
			//		using dojo.safeMixin, and it skips a constructor property,
			//		and properly decorates copied functions.
			//
			// example:
			//	|	var A = declare(null, {
			//	|		m1: function(){},
			//	|		s1: "Popokatepetl"
			//	|	});
			//	|	A.extend({
			//	|		m1: function(){},
			//	|		m2: function(){},
			//	|		f1: true,
			//	|		d1: 42
			//	|	});
		},
		
		createSubclass: function(mixins, props){
			// summary:
			//		Create a subclass of the declared class from a list of base classes.
			// mixins: Function[]
			//		Specifies a list of bases (the left-most one is the most deepest
			//		base).
			// props: Object?
			//		An optional object whose properties are copied to the created prototype.
			// returns: dojo/_base/declare.__DeclareCreatedObject
			//		New constructor function.
			// description:
			//		Create a constructor using a compact notation for inheritance and
			//		prototype extension.
			//
			//		Mixin ancestors provide a type of multiple inheritance.
			//		Prototypes of mixin ancestors are copied to the new class:
			//		changes to mixin prototypes will not affect classes to which
			//		they have been mixed in.
			//
			// example:
			//	|	var A = declare(null, {
			//	|		m1: function(){},
			//	|		s1: "bar"
			//	|	});
			//	|	var B = declare(null, {
			//	|		m2: function(){},
			//	|		s2: "foo"
			//	|	});
			//	|	var C = declare(null, {
			//	|	});
			//	|	var D1 = A.createSubclass([B, C], {
			//	|		m1: function(){},
			//	|		d1: 42
			//	|	});
			//	|	var d1 = new D1();
			//	|
			//	|	// this is equivalent to:
			//	|	var D2 = declare([A, B, C], {
			//	|		m1: function(){},
			//	|		d1: 42
			//	|	});
			//	|	var d2 = new D2();
		}
	};
	=====*/

	// For back-compat, remove for 2.0
	dojo.safeMixin = declare.safeMixin = safeMixin;
	dojo.declare = declare;

	return declare;
});

},
'dojo/_base/connect':function(){
define(["./kernel", "../on", "../topic", "../aspect", "./event", "../mouse", "./sniff", "./lang", "../keys"], function(dojo, on, hub, aspect, eventModule, mouse, has, lang){
// module:
//		dojo/_base/connect

has.add("events-keypress-typed", function(){ // keypresses should only occur a printable character is hit
	var testKeyEvent = {charCode: 0};
	try{
		testKeyEvent = document.createEvent("KeyboardEvent");
		(testKeyEvent.initKeyboardEvent || testKeyEvent.initKeyEvent).call(testKeyEvent, "keypress", true, true, null, false, false, false, false, 9, 3);
	}catch(e){}
	return testKeyEvent.charCode == 0 && !has("opera");
});

function connect_(obj, event, context, method, dontFix){
	method = lang.hitch(context, method);
	if(!obj || !(obj.addEventListener || obj.attachEvent)){
		// it is a not a DOM node and we are using the dojo.connect style of treating a
		// method like an event, must go right to aspect
		return aspect.after(obj || dojo.global, event, method, true);
	}
	if(typeof event == "string" && event.substring(0, 2) == "on"){
		event = event.substring(2);
	}
	if(!obj){
		obj = dojo.global;
	}
	if(!dontFix){
		switch(event){
			// dojo.connect has special handling for these event types
			case "keypress":
				event = keypress;
				break;
			case "mouseenter":
				event = mouse.enter;
				break;
			case "mouseleave":
				event = mouse.leave;
				break;
		}
	}
	return on(obj, event, method, dontFix);
}

var _punctMap = {
	106:42,
	111:47,
	186:59,
	187:43,
	188:44,
	189:45,
	190:46,
	191:47,
	192:96,
	219:91,
	220:92,
	221:93,
	222:39,
	229:113
};
var evtCopyKey = has("mac") ? "metaKey" : "ctrlKey";


var _synthesizeEvent = function(evt, props){
	var faux = lang.mixin({}, evt, props);
	setKeyChar(faux);
	// FIXME: would prefer to use lang.hitch: lang.hitch(evt, evt.preventDefault);
	// but it throws an error when preventDefault is invoked on Safari
	// does Event.preventDefault not support "apply" on Safari?
	faux.preventDefault = function(){ evt.preventDefault(); };
	faux.stopPropagation = function(){ evt.stopPropagation(); };
	return faux;
};
function setKeyChar(evt){
	evt.keyChar = evt.charCode ? String.fromCharCode(evt.charCode) : '';
	evt.charOrCode = evt.keyChar || evt.keyCode;
}
var keypress;
if(has("events-keypress-typed")){
	// this emulates Firefox's keypress behavior where every keydown can correspond to a keypress
	var _trySetKeyCode = function(e, code){
		try{
			// squelch errors when keyCode is read-only
			// (e.g. if keyCode is ctrl or shift)
			return (e.keyCode = code);
		}catch(e){
			return 0;
		}
	};
	keypress = function(object, listener){
		var keydownSignal = on(object, "keydown", function(evt){
			// munge key/charCode
			var k=evt.keyCode;
			// These are Windows Virtual Key Codes
			// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/WinUI/WindowsUserInterface/UserInput/VirtualKeyCodes.asp
			var unprintable = (k!=13) && k!=32 && (k!=27||!has("ie")) && (k<48||k>90) && (k<96||k>111) && (k<186||k>192) && (k<219||k>222) && k!=229;
			// synthesize keypress for most unprintables and CTRL-keys
			if(unprintable||evt.ctrlKey){
				var c = unprintable ? 0 : k;
				if(evt.ctrlKey){
					if(k==3 || k==13){
						return listener.call(evt.currentTarget, evt); // IE will post CTRL-BREAK, CTRL-ENTER as keypress natively
					}else if(c>95 && c<106){
						c -= 48; // map CTRL-[numpad 0-9] to ASCII
					}else if((!evt.shiftKey)&&(c>=65&&c<=90)){
						c += 32; // map CTRL-[A-Z] to lowercase
					}else{
						c = _punctMap[c] || c; // map other problematic CTRL combinations to ASCII
					}
				}
				// simulate a keypress event
				var faux = _synthesizeEvent(evt, {type: 'keypress', faux: true, charCode: c});
				listener.call(evt.currentTarget, faux);
				if(has("ie")){
					_trySetKeyCode(evt, faux.keyCode);
				}
			}
		});
		var keypressSignal = on(object, "keypress", function(evt){
			var c = evt.charCode;
			c = c>=32 ? c : 0;
			evt = _synthesizeEvent(evt, {charCode: c, faux: true});
			return listener.call(this, evt);
		});
		return {
			remove: function(){
				keydownSignal.remove();
				keypressSignal.remove();
			}
		};
	};
}else{
	if(has("opera")){
		keypress = function(object, listener){
			return on(object, "keypress", function(evt){
				var c = evt.which;
				if(c==3){
					c=99; // Mozilla maps CTRL-BREAK to CTRL-c
				}
				// can't trap some keys at all, like INSERT and DELETE
				// there is no differentiating info between DELETE and ".", or INSERT and "-"
				c = c<32 && !evt.shiftKey ? 0 : c;
				if(evt.ctrlKey && !evt.shiftKey && c>=65 && c<=90){
					// lowercase CTRL-[A-Z] keys
					c += 32;
				}
				return listener.call(this, _synthesizeEvent(evt, { charCode: c }));
			});
		};
	}else{
		keypress = function(object, listener){
			return on(object, "keypress", function(evt){
				setKeyChar(evt);
				return listener.call(this, evt);
			});
		};
	}
}

var connect = {
	// summary:
	//		This module defines the dojo.connect API.
	//		This modules also provides keyboard event handling helpers.
	//		This module exports an extension event for emulating Firefox's keypress handling.
	//		However, this extension event exists primarily for backwards compatibility and
	//		is not recommended. WebKit and IE uses an alternate keypress handling (only
	//		firing for printable characters, to distinguish from keydown events), and most
	//		consider the WebKit/IE behavior more desirable.

	_keypress:keypress,

	connect:function(obj, event, context, method, dontFix){
		// summary:
		//		`dojo.connect` is a deprecated event handling and delegation method in
		//		Dojo. It allows one function to "listen in" on the execution of
		//		any other, triggering the second whenever the first is called. Many
		//		listeners may be attached to a function, and source functions may
		//		be either regular function calls or DOM events.
		//
		// description:
		//		Connects listeners to actions, so that after event fires, a
		//		listener is called with the same arguments passed to the original
		//		function.
		//
		//		Since `dojo.connect` allows the source of events to be either a
		//		"regular" JavaScript function or a DOM event, it provides a uniform
		//		interface for listening to all the types of events that an
		//		application is likely to deal with though a single, unified
		//		interface. DOM programmers may want to think of it as
		//		"addEventListener for everything and anything".
		//
		//		When setting up a connection, the `event` parameter must be a
		//		string that is the name of the method/event to be listened for. If
		//		`obj` is null, `kernel.global` is assumed, meaning that connections
		//		to global methods are supported but also that you may inadvertently
		//		connect to a global by passing an incorrect object name or invalid
		//		reference.
		//
		//		`dojo.connect` generally is forgiving. If you pass the name of a
		//		function or method that does not yet exist on `obj`, connect will
		//		not fail, but will instead set up a stub method. Similarly, null
		//		arguments may simply be omitted such that fewer than 4 arguments
		//		may be required to set up a connection See the examples for details.
		//
		//		The return value is a handle that is needed to
		//		remove this connection with `dojo.disconnect`.
		//
		// obj: Object?
		//		The source object for the event function.
		//		Defaults to `kernel.global` if null.
		//		If obj is a DOM node, the connection is delegated
		//		to the DOM event manager (unless dontFix is true).
		//
		// event: String
		//		String name of the event function in obj.
		//		I.e. identifies a property `obj[event]`.
		//
		// context: Object|null
		//		The object that method will receive as "this".
		//
		//		If context is null and method is a function, then method
		//		inherits the context of event.
		//
		//		If method is a string then context must be the source
		//		object object for method (context[method]). If context is null,
		//		kernel.global is used.
		//
		// method: String|Function
		//		A function reference, or name of a function in context.
		//		The function identified by method fires after event does.
		//		method receives the same arguments as the event.
		//		See context argument comments for information on method's scope.
		//
		// dontFix: Boolean?
		//		If obj is a DOM node, set dontFix to true to prevent delegation
		//		of this connection to the DOM event manager.
		//
		// example:
		//		When obj.onchange(), do ui.update():
		//	|	dojo.connect(obj, "onchange", ui, "update");
		//	|	dojo.connect(obj, "onchange", ui, ui.update); // same
		//
		// example:
		//		Using return value for disconnect:
		//	|	var link = dojo.connect(obj, "onchange", ui, "update");
		//	|	...
		//	|	dojo.disconnect(link);
		//
		// example:
		//		When onglobalevent executes, watcher.handler is invoked:
		//	|	dojo.connect(null, "onglobalevent", watcher, "handler");
		//
		// example:
		//		When ob.onCustomEvent executes, customEventHandler is invoked:
		//	|	dojo.connect(ob, "onCustomEvent", null, "customEventHandler");
		//	|	dojo.connect(ob, "onCustomEvent", "customEventHandler"); // same
		//
		// example:
		//		When ob.onCustomEvent executes, customEventHandler is invoked
		//		with the same scope (this):
		//	|	dojo.connect(ob, "onCustomEvent", null, customEventHandler);
		//	|	dojo.connect(ob, "onCustomEvent", customEventHandler); // same
		//
		// example:
		//		When globalEvent executes, globalHandler is invoked
		//		with the same scope (this):
		//	|	dojo.connect(null, "globalEvent", null, globalHandler);
		//	|	dojo.connect("globalEvent", globalHandler); // same

		// normalize arguments
		var a=arguments, args=[], i=0;
		// if a[0] is a String, obj was omitted
		args.push(typeof a[0] == "string" ? null : a[i++], a[i++]);
		// if the arg-after-next is a String or Function, context was NOT omitted
		var a1 = a[i+1];
		args.push(typeof a1 == "string" || typeof a1 == "function" ? a[i++] : null, a[i++]);
		// absorb any additional arguments
		for(var l=a.length; i<l; i++){	args.push(a[i]); }
		return connect_.apply(this, args);
	},

	disconnect:function(handle){
		// summary:
		//		Remove a link created by dojo.connect.
		// description:
		//		Removes the connection between event and the method referenced by handle.
		// handle: Handle
		//		the return value of the dojo.connect call that created the connection.

		if(handle){
			handle.remove();
		}
	},

	subscribe:function(topic, context, method){
		// summary:
		//		Attach a listener to a named topic. The listener function is invoked whenever the
		//		named topic is published (see: dojo.publish).
		//		Returns a handle which is needed to unsubscribe this listener.
		// topic: String
		//		The topic to which to subscribe.
		// context: Object?
		//		Scope in which method will be invoked, or null for default scope.
		// method: String|Function
		//		The name of a function in context, or a function reference. This is the function that
		//		is invoked when topic is published.
		// example:
		//	|	dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); });
		//	|	dojo.publish("alerts", [ "read this", "hello world" ]);
		return hub.subscribe(topic, lang.hitch(context, method));
	},

	publish:function(topic, args){
		// summary:
		//		Invoke all listener method subscribed to topic.
		// topic: String
		//		The name of the topic to publish.
		// args: Array?
		//		An array of arguments. The arguments will be applied
		//		to each topic subscriber (as first class parameters, via apply).
		// example:
		//	|	dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); };
		//	|	dojo.publish("alerts", [ "read this", "hello world" ]);
		return hub.publish.apply(hub, [topic].concat(args));
	},

	connectPublisher:function(topic, obj, event){
		// summary:
		//		Ensure that every time obj.event() is called, a message is published
		//		on the topic. Returns a handle which can be passed to
		//		dojo.disconnect() to disable subsequent automatic publication on
		//		the topic.
		// topic: String
		//		The name of the topic to publish.
		// obj: Object?
		//		The source object for the event function. Defaults to kernel.global
		//		if null.
		// event: String
		//		The name of the event function in obj.
		//		I.e. identifies a property obj[event].
		// example:
		//	|	dojo.connectPublisher("/ajax/start", dojo, "xhrGet");
		var pf = function(){ connect.publish(topic, arguments); };
		return event ? connect.connect(obj, event, pf) : connect.connect(obj, pf); //Handle
	},

	isCopyKey: function(e){
		// summary:
		//		Checks an event for the copy key (meta on Mac, and ctrl anywhere else)
		// e: Event
		//		Event object to examine
		return e[evtCopyKey];	// Boolean
	}
};

connect.unsubscribe = connect.disconnect;
/*=====
 connect.unsubscribe = function(handle){
	 // summary:
	 //		Remove a topic listener.
	 // handle: Handle
	 //		The handle returned from a call to subscribe.
	 // example:
	 //	|	var alerter = dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); };
	 //	|	...
	 //	|	dojo.unsubscribe(alerter);
 };
 =====*/

 1  && lang.mixin(dojo, connect);
return connect;

});



},
'dojo/on':function(){
define(["./has!dom-addeventlistener?:./aspect", "./_base/kernel", "./sniff"], function(aspect, dojo, has){

	"use strict";
	if( 1 ){ // check to make sure we are in a browser, this module should work anywhere
		var major = window.ScriptEngineMajorVersion;
		has.add("jscript", major && (major() + ScriptEngineMinorVersion() / 10));
		has.add("event-orientationchange", has("touch") && !has("android")); // TODO: how do we detect this?
		has.add("event-stopimmediatepropagation", window.Event && !!window.Event.prototype && !!window.Event.prototype.stopImmediatePropagation);
		has.add("event-focusin", function(global, doc, element){
			// All browsers except firefox support focusin, but too hard to feature test webkit since element.onfocusin
			// is undefined.  Just return true for IE and use fallback path for other browsers.
			return !!element.attachEvent;
		});
	}
	var on = function(target, type, listener, dontFix){
		// summary:
		//		A function that provides core event listening functionality. With this function
		//		you can provide a target, event type, and listener to be notified of
		//		future matching events that are fired.
		// target: Element|Object
		//		This is the target object or DOM element that to receive events from
		// type: String|Function
		//		This is the name of the event to listen for or an extension event type.
		// listener: Function
		//		This is the function that should be called when the event fires.
		// returns: Object
		//		An object with a remove() method that can be used to stop listening for this
		//		event.
		// description:
		//		To listen for "click" events on a button node, we can do:
		//		|	define(["dojo/on"], function(listen){
		//		|		on(button, "click", clickHandler);
		//		|		...
		//		Evented JavaScript objects can also have their own events.
		//		|	var obj = new Evented;
		//		|	on(obj, "foo", fooHandler);
		//		And then we could publish a "foo" event:
		//		|	on.emit(obj, "foo", {key: "value"});
		//		We can use extension events as well. For example, you could listen for a tap gesture:
		//		|	define(["dojo/on", "dojo/gesture/tap", function(listen, tap){
		//		|		on(button, tap, tapHandler);
		//		|		...
		//		which would trigger fooHandler. Note that for a simple object this is equivalent to calling:
		//		|	obj.onfoo({key:"value"});
		//		If you use on.emit on a DOM node, it will use native event dispatching when possible.

		if(typeof target.on == "function" && typeof type != "function" && !target.nodeType){
			// delegate to the target's on() method, so it can handle it's own listening if it wants (unless it 
			// is DOM node and we may be dealing with jQuery or Prototype's incompatible addition to the
			// Element prototype 
			return target.on(type, listener);
		}
		// delegate to main listener code
		return on.parse(target, type, listener, addListener, dontFix, this);
	};
	on.pausable =  function(target, type, listener, dontFix){
		// summary:
		//		This function acts the same as on(), but with pausable functionality. The
		//		returned signal object has pause() and resume() functions. Calling the
		//		pause() method will cause the listener to not be called for future events. Calling the
		//		resume() method will cause the listener to again be called for future events.
		var paused;
		var signal = on(target, type, function(){
			if(!paused){
				return listener.apply(this, arguments);
			}
		}, dontFix);
		signal.pause = function(){
			paused = true;
		};
		signal.resume = function(){
			paused = false;
		};
		return signal;
	};
	on.once = function(target, type, listener, dontFix){
		// summary:
		//		This function acts the same as on(), but will only call the listener once. The 
		//		listener will be called for the first
		//		event that takes place and then listener will automatically be removed.
		var signal = on(target, type, function(){
			// remove this listener
			signal.remove();
			// proceed to call the listener
			return listener.apply(this, arguments);
		});
		return signal;
	};
	on.parse = function(target, type, listener, addListener, dontFix, matchesTarget){
		if(type.call){
			// event handler function
			// on(node, touch.press, touchListener);
			return type.call(matchesTarget, target, listener);
		}

		if(type.indexOf(",") > -1){
			// we allow comma delimited event names, so you can register for multiple events at once
			var events = type.split(/\s*,\s*/);
			var handles = [];
			var i = 0;
			var eventName;
			while(eventName = events[i++]){
				handles.push(addListener(target, eventName, listener, dontFix, matchesTarget));
			}
			handles.remove = function(){
				for(var i = 0; i < handles.length; i++){
					handles[i].remove();
				}
			};
			return handles;
		}
		return addListener(target, type, listener, dontFix, matchesTarget);
	};
	var touchEvents = /^touch/;
	function addListener(target, type, listener, dontFix, matchesTarget){
		// event delegation:
		var selector = type.match(/(.*):(.*)/);
		// if we have a selector:event, the last one is interpreted as an event, and we use event delegation
		if(selector){
			type = selector[2];
			selector = selector[1];
			// create the extension event for selectors and directly call it
			return on.selector(selector, type).call(matchesTarget, target, listener);
		}
		// test to see if it a touch event right now, so we don't have to do it every time it fires
		if(has("touch")){
			if(touchEvents.test(type)){
				// touch event, fix it
				listener = fixTouchListener(listener);
			}
			if(!has("event-orientationchange") && (type == "orientationchange")){
				//"orientationchange" not supported <= Android 2.1, 
				//but works through "resize" on window
				type = "resize"; 
				target = window;
				listener = fixTouchListener(listener);
			} 
		}
		if(addStopImmediate){
			// add stopImmediatePropagation if it doesn't exist
			listener = addStopImmediate(listener);
		}
		// normal path, the target is |this|
		if(target.addEventListener){
			// the target has addEventListener, which should be used if available (might or might not be a node, non-nodes can implement this method as well)
			// check for capture conversions
			var capture = type in captures,
				adjustedType = capture ? captures[type] : type;
			target.addEventListener(adjustedType, listener, capture);
			// create and return the signal
			return {
				remove: function(){
					target.removeEventListener(adjustedType, listener, capture);
				}
			};
		}
		type = "on" + type;
		if(fixAttach && target.attachEvent){
			return fixAttach(target, type, listener);
		}
		throw new Error("Target must be an event emitter");
	}

	on.selector = function(selector, eventType, children){
		// summary:
		//		Creates a new extension event with event delegation. This is based on
		//		the provided event type (can be extension event) that
		//		only calls the listener when the CSS selector matches the target of the event.
		//
		//		The application must require() an appropriate level of dojo/query to handle the selector.
		// selector:
		//		The CSS selector to use for filter events and determine the |this| of the event listener.
		// eventType:
		//		The event to listen for
		// children:
		//		Indicates if children elements of the selector should be allowed. This defaults to 
		//		true
		// example:
		// |	require(["dojo/on", "dojo/mouse", "dojo/query!css2"], function(listen, mouse){
		// |		on(node, on.selector(".my-class", mouse.enter), handlerForMyHover);
		return function(target, listener){
			// if the selector is function, use it to select the node, otherwise use the matches method
			var matchesTarget = typeof selector == "function" ? {matches: selector} : this,
				bubble = eventType.bubble;
			function select(eventTarget){
				// see if we have a valid matchesTarget or default to dojo.query
				matchesTarget = matchesTarget && matchesTarget.matches ? matchesTarget : dojo.query;
				// there is a selector, so make sure it matches
				while(!matchesTarget.matches(eventTarget, selector, target)){
					if(eventTarget == target || children === false || !(eventTarget = eventTarget.parentNode) || eventTarget.nodeType != 1){ // intentional assignment
						return;
					}
				}
				return eventTarget;
			}
			if(bubble){
				// the event type doesn't naturally bubble, but has a bubbling form, use that, and give it the selector so it can perform the select itself
				return on(target, bubble(select), listener);
			}
			// standard event delegation
			return on(target, eventType, function(event){
				// call select to see if we match
				var eventTarget = select(event.target);
				// if it matches we call the listener
				return eventTarget && listener.call(eventTarget, event);
			});
		};
	};

	function syntheticPreventDefault(){
		this.cancelable = false;
		this.defaultPrevented = true;
	}
	function syntheticStopPropagation(){
		this.bubbles = false;
	}
	var slice = [].slice,
		syntheticDispatch = on.emit = function(target, type, event){
		// summary:
		//		Fires an event on the target object.
		// target:
		//		The target object to fire the event on. This can be a DOM element or a plain 
		//		JS object. If the target is a DOM element, native event emitting mechanisms
		//		are used when possible.
		// type:
		//		The event type name. You can emulate standard native events like "click" and 
		//		"mouseover" or create custom events like "open" or "finish".
		// event:
		//		An object that provides the properties for the event. See https://developer.mozilla.org/en/DOM/event.initEvent 
		//		for some of the properties. These properties are copied to the event object.
		//		Of particular importance are the cancelable and bubbles properties. The
		//		cancelable property indicates whether or not the event has a default action
		//		that can be cancelled. The event is cancelled by calling preventDefault() on
		//		the event object. The bubbles property indicates whether or not the
		//		event will bubble up the DOM tree. If bubbles is true, the event will be called
		//		on the target and then each parent successively until the top of the tree
		//		is reached or stopPropagation() is called. Both bubbles and cancelable 
		//		default to false.
		// returns:
		//		If the event is cancelable and the event is not cancelled,
		//		emit will return true. If the event is cancelable and the event is cancelled,
		//		emit will return false.
		// details:
		//		Note that this is designed to emit events for listeners registered through
		//		dojo/on. It should actually work with any event listener except those
		//		added through IE's attachEvent (IE8 and below's non-W3C event emitting
		//		doesn't support custom event types). It should work with all events registered
		//		through dojo/on. Also note that the emit method does do any default
		//		action, it only returns a value to indicate if the default action should take
		//		place. For example, emitting a keypress event would not cause a character
		//		to appear in a textbox.
		// example:
		//		To fire our own click event
		//	|	on.emit(dojo.byId("button"), "click", {
		//	|		cancelable: true,
		//	|		bubbles: true,
		//	|		screenX: 33,
		//	|		screenY: 44
		//	|	});
		//		We can also fire our own custom events:
		//	|	on.emit(dojo.byId("slider"), "slide", {
		//	|		cancelable: true,
		//	|		bubbles: true,
		//	|		direction: "left-to-right"
		//	|	});
		var args = slice.call(arguments, 2);
		var method = "on" + type;
		if("parentNode" in target){
			// node (or node-like), create event controller methods
			var newEvent = args[0] = {};
			for(var i in event){
				newEvent[i] = event[i];
			}
			newEvent.preventDefault = syntheticPreventDefault;
			newEvent.stopPropagation = syntheticStopPropagation;
			newEvent.target = target;
			newEvent.type = type;
			event = newEvent;
		}
		do{
			// call any node which has a handler (note that ideally we would try/catch to simulate normal event propagation but that causes too much pain for debugging)
			target[method] && target[method].apply(target, args);
			// and then continue up the parent node chain if it is still bubbling (if started as bubbles and stopPropagation hasn't been called)
		}while(event && event.bubbles && (target = target.parentNode));
		return event && event.cancelable && event; // if it is still true (was cancelable and was cancelled), return the event to indicate default action should happen
	};
	var captures = has("event-focusin") ? {} : {focusin: "focus", focusout: "blur"};
	if(!has("event-stopimmediatepropagation")){
		var stopImmediatePropagation =function(){
			this.immediatelyStopped = true;
			this.modified = true; // mark it as modified so the event will be cached in IE
		};
		var addStopImmediate = function(listener){
			return function(event){
				if(!event.immediatelyStopped){// check to make sure it hasn't been stopped immediately
					event.stopImmediatePropagation = stopImmediatePropagation;
					return listener.apply(this, arguments);
				}
			};
		}
	} 
	if(has("dom-addeventlistener")){
		// emitter that works with native event handling
		on.emit = function(target, type, event){
			if(target.dispatchEvent && document.createEvent){
				// use the native event emitting mechanism if it is available on the target object
				// create a generic event				
				// we could create branch into the different types of event constructors, but 
				// that would be a lot of extra code, with little benefit that I can see, seems 
				// best to use the generic constructor and copy properties over, making it 
				// easy to have events look like the ones created with specific initializers
				var nativeEvent = target.ownerDocument.createEvent("HTMLEvents");
				nativeEvent.initEvent(type, !!event.bubbles, !!event.cancelable);
				// and copy all our properties over
				for(var i in event){
					if(!(i in nativeEvent)){
						nativeEvent[i] = event[i];
					}
				}
				return target.dispatchEvent(nativeEvent) && nativeEvent;
			}
			return syntheticDispatch.apply(on, arguments); // emit for a non-node
		};
	}else{
		// no addEventListener, basically old IE event normalization
		on._fixEvent = function(evt, sender){
			// summary:
			//		normalizes properties on the event object including event
			//		bubbling methods, keystroke normalization, and x/y positions
			// evt:
			//		native event object
			// sender:
			//		node to treat as "currentTarget"
			if(!evt){
				var w = sender && (sender.ownerDocument || sender.document || sender).parentWindow || window;
				evt = w.event;
			}
			if(!evt){return evt;}
			try{
				if(lastEvent && evt.type == lastEvent.type  && evt.srcElement == lastEvent.target){
					// should be same event, reuse event object (so it can be augmented);
					// accessing evt.srcElement rather than evt.target since evt.target not set on IE until fixup below
					evt = lastEvent;
				}
			}catch(e){
				// will occur on IE on lastEvent.type reference if lastEvent points to a previous event that already
				// finished bubbling, but the setTimeout() to clear lastEvent hasn't fired yet
			}
			if(!evt.target){ // check to see if it has been fixed yet
				evt.target = evt.srcElement;
				evt.currentTarget = (sender || evt.srcElement);
				if(evt.type == "mouseover"){
					evt.relatedTarget = evt.fromElement;
				}
				if(evt.type == "mouseout"){
					evt.relatedTarget = evt.toElement;
				}
				if(!evt.stopPropagation){
					evt.stopPropagation = stopPropagation;
					evt.preventDefault = preventDefault;
				}
				switch(evt.type){
					case "keypress":
						var c = ("charCode" in evt ? evt.charCode : evt.keyCode);
						if (c==10){
							// CTRL-ENTER is CTRL-ASCII(10) on IE, but CTRL-ENTER on Mozilla
							c=0;
							evt.keyCode = 13;
						}else if(c==13||c==27){
							c=0; // Mozilla considers ENTER and ESC non-printable
						}else if(c==3){
							c=99; // Mozilla maps CTRL-BREAK to CTRL-c
						}
						// Mozilla sets keyCode to 0 when there is a charCode
						// but that stops the event on IE.
						evt.charCode = c;
						_setKeyChar(evt);
						break;
				}
			}
			return evt;
		};
		var lastEvent, IESignal = function(handle){
			this.handle = handle;
		};
		IESignal.prototype.remove = function(){
			delete _dojoIEListeners_[this.handle];
		};
		var fixListener = function(listener){
			// this is a minimal function for closing on the previous listener with as few as variables as possible
			return function(evt){
				evt = on._fixEvent(evt, this);
				var result = listener.call(this, evt);
				if(evt.modified){
					// cache the last event and reuse it if we can
					if(!lastEvent){
						setTimeout(function(){
							lastEvent = null;
						});
					}
					lastEvent = evt;
				}
				return result;
			};
		};
		var fixAttach = function(target, type, listener){
			listener = fixListener(listener);
			if(((target.ownerDocument ? target.ownerDocument.parentWindow : target.parentWindow || target.window || window) != top || 
						has("jscript") < 5.8) && 
					!has("config-_allow_leaks")){
				// IE will leak memory on certain handlers in frames (IE8 and earlier) and in unattached DOM nodes for JScript 5.7 and below.
				// Here we use global redirection to solve the memory leaks
				if(typeof _dojoIEListeners_ == "undefined"){
					_dojoIEListeners_ = [];
				}
				var emitter = target[type];
				if(!emitter || !emitter.listeners){
					var oldListener = emitter;
					emitter = Function('event', 'var callee = arguments.callee; for(var i = 0; i<callee.listeners.length; i++){var listener = _dojoIEListeners_[callee.listeners[i]]; if(listener){listener.call(this,event);}}');
					emitter.listeners = [];
					target[type] = emitter;
					emitter.global = this;
					if(oldListener){
						emitter.listeners.push(_dojoIEListeners_.push(oldListener) - 1);
					}
				}
				var handle;
				emitter.listeners.push(handle = (emitter.global._dojoIEListeners_.push(listener) - 1));
				return new IESignal(handle);
			}
			return aspect.after(target, type, listener, true);
		};

		var _setKeyChar = function(evt){
			evt.keyChar = evt.charCode ? String.fromCharCode(evt.charCode) : '';
			evt.charOrCode = evt.keyChar || evt.keyCode;
		};
		// Called in Event scope
		var stopPropagation = function(){
			this.cancelBubble = true;
		};
		var preventDefault = on._preventDefault = function(){
			// Setting keyCode to 0 is the only way to prevent certain keypresses (namely
			// ctrl-combinations that correspond to menu accelerator keys).
			// Otoh, it prevents upstream listeners from getting this information
			// Try to split the difference here by clobbering keyCode only for ctrl
			// combinations. If you still need to access the key upstream, bubbledKeyCode is
			// provided as a workaround.
			this.bubbledKeyCode = this.keyCode;
			if(this.ctrlKey){
				try{
					// squelch errors when keyCode is read-only
					// (e.g. if keyCode is ctrl or shift)
					this.keyCode = 0;
				}catch(e){
				}
			}
			this.defaultPrevented = true;
			this.returnValue = false;
			this.modified = true; // mark it as modified  (for defaultPrevented flag) so the event will be cached in IE
		};
	}
	if(has("touch")){ 
		var Event = function(){};
		var windowOrientation = window.orientation; 
		var fixTouchListener = function(listener){ 
			return function(originalEvent){ 
				//Event normalization(for ontouchxxx and resize): 
				//1.incorrect e.pageX|pageY in iOS 
				//2.there are no "e.rotation", "e.scale" and "onorientationchange" in Android
				//3.More TBD e.g. force | screenX | screenX | clientX | clientY | radiusX | radiusY

				// see if it has already been corrected
				var event = originalEvent.corrected;
				if(!event){
					var type = originalEvent.type;
					try{
						delete originalEvent.type; // on some JS engines (android), deleting properties make them mutable
					}catch(e){} 
					if(originalEvent.type){
						// deleting properties doesn't work (older iOS), have to use delegation
						if(has('mozilla')){
							// Firefox doesn't like delegated properties, so we have to copy
							var event = {};
							for(var name in originalEvent){
								event[name] = originalEvent[name];
							}
						}else{
							// old iOS branch
							Event.prototype = originalEvent;
							var event = new Event;
						}
						// have to delegate methods to make them work
						event.preventDefault = function(){
							originalEvent.preventDefault();
						};
						event.stopPropagation = function(){
							originalEvent.stopPropagation();
						};
					}else{
						// deletion worked, use property as is
						event = originalEvent;
						event.type = type;
					}
					originalEvent.corrected = event;
					if(type == 'resize'){
						if(windowOrientation == window.orientation){ 
							return null;//double tap causes an unexpected 'resize' in Android
						} 
						windowOrientation = window.orientation;
						event.type = "orientationchange"; 
						return listener.call(this, event);
					}
					// We use the original event and augment, rather than doing an expensive mixin operation
					if(!("rotation" in event)){ // test to see if it has rotation
						event.rotation = 0; 
						event.scale = 1;
					}
					//use event.changedTouches[0].pageX|pageY|screenX|screenY|clientX|clientY|target
					var firstChangeTouch = event.changedTouches[0];
					for(var i in firstChangeTouch){ // use for-in, we don't need to have dependency on dojo/_base/lang here
						delete event[i]; // delete it first to make it mutable
						event[i] = firstChangeTouch[i];
					}
				}
				return listener.call(this, event); 
			}; 
		}; 
	}
	return on;
});

},
'dojo/topic':function(){
define(["./Evented"], function(Evented){

	// module:
	//		dojo/topic

	var hub = new Evented;
	return {
		// summary:
		//		Pubsub hub.
		// example:
		//		| 	topic.subscribe("some/topic", function(event){
		//		|	... do something with event
		//		|	});
		//		|	topic.publish("some/topic", {name:"some event", ...});

		publish: function(topic, event){
			// summary:
			//		Publishes a message to a topic on the pub/sub hub. All arguments after
			//		the first will be passed to the subscribers, so any number of arguments
			//		can be provided (not just event).
			// topic: String
			//		The name of the topic to publish to
			// event: Object
			//		An event to distribute to the topic listeners
			return hub.emit.apply(hub, arguments);
		},

		subscribe: function(topic, listener){
			// summary:
			//		Subscribes to a topic on the pub/sub hub
			// topic: String
			//		The topic to subscribe to
			// listener: Function
			//		A function to call when a message is published to the given topic
			return hub.on.apply(hub, arguments);
		}
	};
});

},
'dojo/Evented':function(){
define(["./aspect", "./on"], function(aspect, on){
	// module:
	//		dojo/Evented

 	"use strict";
 	var after = aspect.after;
	function Evented(){
		// summary:
		//		A class that can be used as a mixin or base class,
		//		to add on() and emit() methods to a class
		//		for listening for events and emitting events:
		//
		//		|	define(["dojo/Evented"], function(Evented){
		//		|		var EventedWidget = dojo.declare([Evented, dijit._Widget], {...});
		//		|		widget = new EventedWidget();
		//		|		widget.on("open", function(event){
		//		|		... do something with event
		//		|	 });
		//		|
		//		|	widget.emit("open", {name:"some event", ...});
	}
	Evented.prototype = {
		on: function(type, listener){
			return on.parse(this, type, listener, function(target, type){
				return after(target, 'on' + type, listener, true);
			});
		},
		emit: function(type, event){
			var args = [this];
			args.push.apply(args, arguments);
			return on.emit.apply(on, args);
		}
	};
	return Evented;
});

},
'dojo/aspect':function(){
define([], function(){

	// module:
	//		dojo/aspect

	"use strict";
	var undefined, nextId = 0;
	function advise(dispatcher, type, advice, receiveArguments){
		var previous = dispatcher[type];
		var around = type == "around";
		var signal;
		if(around){
			var advised = advice(function(){
				return previous.advice(this, arguments);
			});
			signal = {
				remove: function(){
					if(advised){
						advised = dispatcher = advice = null;
					}
				},
				advice: function(target, args){
					return advised ?
						advised.apply(target, args) :  // called the advised function
						previous.advice(target, args); // cancelled, skip to next one
				}
			};
		}else{
			// create the remove handler
			signal = {
				remove: function(){
					if(signal.advice){
						var previous = signal.previous;
						var next = signal.next;
						if(!next && !previous){
							delete dispatcher[type];
						}else{
							if(previous){
								previous.next = next;
							}else{
								dispatcher[type] = next;
							}
							if(next){
								next.previous = previous;
							}
						}

						// remove the advice to signal that this signal has been removed
						dispatcher = advice = signal.advice = null;
					}
				},
				id: nextId++,
				advice: advice,
				receiveArguments: receiveArguments
			};
		}
		if(previous && !around){
			if(type == "after"){
				// add the listener to the end of the list
				// note that we had to change this loop a little bit to workaround a bizarre IE10 JIT bug
				while(previous.next && (previous = previous.next)){}
				previous.next = signal;
				signal.previous = previous;
			}else if(type == "before"){
				// add to beginning
				dispatcher[type] = signal;
				signal.next = previous;
				previous.previous = signal;
			}
		}else{
			// around or first one just replaces
			dispatcher[type] = signal;
		}
		return signal;
	}
	function aspect(type){
		return function(target, methodName, advice, receiveArguments){
			var existing = target[methodName], dispatcher;
			if(!existing || existing.target != target){
				// no dispatcher in place
				target[methodName] = dispatcher = function(){
					var executionId = nextId;
					// before advice
					var args = arguments;
					var before = dispatcher.before;
					while(before){
						args = before.advice.apply(this, args) || args;
						before = before.next;
					}
					// around advice
					if(dispatcher.around){
						var results = dispatcher.around.advice(this, args);
					}
					// after advice
					var after = dispatcher.after;
					while(after && after.id < executionId){
						if(after.receiveArguments){
							var newResults = after.advice.apply(this, args);
							// change the return value only if a new value was returned
							results = newResults === undefined ? results : newResults;
						}else{
							results = after.advice.call(this, results, args);
						}
						after = after.next;
					}
					return results;
				};
				if(existing){
					dispatcher.around = {advice: function(target, args){
						return existing.apply(target, args);
					}};
				}
				dispatcher.target = target;
			}
			var results = advise((dispatcher || existing), type, advice, receiveArguments);
			advice = null;
			return results;
		};
	}

	// TODOC: after/before/around return object

	var after = aspect("after");
	/*=====
	after = function(target, methodName, advice, receiveArguments){
		// summary:
		//		The "after" export of the aspect module is a function that can be used to attach
		//		"after" advice to a method. This function will be executed after the original method
		//		is executed. By default the function will be called with a single argument, the return
		//		value of the original method, or the the return value of the last executed advice (if a previous one exists).
		//		The fourth (optional) argument can be set to true to so the function receives the original
		//		arguments (from when the original method was called) rather than the return value.
		//		If there are multiple "after" advisors, they are executed in the order they were registered.
		// target: Object
		//		This is the target object
		// methodName: String
		//		This is the name of the method to attach to.
		// advice: Function
		//		This is function to be called after the original method
		// receiveArguments: Boolean?
		//		If this is set to true, the advice function receives the original arguments (from when the original mehtod
		//		was called) rather than the return value of the original/previous method.
		// returns:
		//		A signal object that can be used to cancel the advice. If remove() is called on this signal object, it will
		//		stop the advice function from being executed.
	};
	=====*/

	var before = aspect("before");
	/*=====
	before = function(target, methodName, advice){
		// summary:
		//		The "before" export of the aspect module is a function that can be used to attach
		//		"before" advice to a method. This function will be executed before the original method
		//		is executed. This function will be called with the arguments used to call the method.
		//		This function may optionally return an array as the new arguments to use to call
		//		the original method (or the previous, next-to-execute before advice, if one exists).
		//		If the before method doesn't return anything (returns undefined) the original arguments
		//		will be preserved.
		//		If there are multiple "before" advisors, they are executed in the reverse order they were registered.
		// target: Object
		//		This is the target object
		// methodName: String
		//		This is the name of the method to attach to.
		// advice: Function
		//		This is function to be called before the original method
	};
	=====*/

	var around = aspect("around");
	/*=====
	 around = function(target, methodName, advice){
		// summary:
		//		The "around" export of the aspect module is a function that can be used to attach
		//		"around" advice to a method. The advisor function is immediately executed when
		//		the around() is called, is passed a single argument that is a function that can be
		//		called to continue execution of the original method (or the next around advisor).
		//		The advisor function should return a function, and this function will be called whenever
		//		the method is called. It will be called with the arguments used to call the method.
		//		Whatever this function returns will be returned as the result of the method call (unless after advise changes it).
		// example:
		//		If there are multiple "around" advisors, the most recent one is executed first,
		//		which can then delegate to the next one and so on. For example:
		//		|	around(obj, "foo", function(originalFoo){
		//		|		return function(){
		//		|			var start = new Date().getTime();
		//		|			var results = originalFoo.apply(this, arguments); // call the original
		//		|			var end = new Date().getTime();
		//		|			console.log("foo execution took " + (end - start) + " ms");
		//		|			return results;
		//		|		};
		//		|	});
		// target: Object
		//		This is the target object
		// methodName: String
		//		This is the name of the method to attach to.
		// advice: Function
		//		This is function to be called around the original method
	};
	=====*/

	return {
		// summary:
		//		provides aspect oriented programming functionality, allowing for
		//		one to add before, around, or after advice on existing methods.
		// example:
		//	|	define(["dojo/aspect"], function(aspect){
		//	|		var signal = aspect.after(targetObject, "methodName", function(someArgument){
		//	|			this will be called when targetObject.methodName() is called, after the original function is called
		//	|		});
		//
		// example:
		//	The returned signal object can be used to cancel the advice.
		//	|	signal.remove(); // this will stop the advice from being executed anymore
		//	|	aspect.before(targetObject, "methodName", function(someArgument){
		//	|		// this will be called when targetObject.methodName() is called, before the original function is called
		//	|	 });

		before: before,
		around: around,
		after: after
	};
});

},
'dojo/_base/event':function(){
define(["./kernel", "../on", "../has", "../dom-geometry"], function(dojo, on, has, dom){
	// module:
	//		dojo/_base/event

	if(on._fixEvent){
		var fixEvent = on._fixEvent;
		on._fixEvent = function(evt, se){
			// add some additional normalization for back-compat, this isn't in on.js because it is somewhat more expensive
			evt = fixEvent(evt, se);
			if(evt){
				dom.normalizeEvent(evt);
			}
			return evt;
		};		
	}
	
	var ret = {
		// summary:
		//		This module defines dojo DOM event API.   Usually you should use dojo/on, and evt.stopPropagation() +
		//		evt.preventDefault(), rather than this module.

		fix: function(/*Event*/ evt, /*DOMNode*/ sender){
			// summary:
			//		normalizes properties on the event object including event
			//		bubbling methods, keystroke normalization, and x/y positions
			// evt: Event
			//		native event object
			// sender: DOMNode
			//		node to treat as "currentTarget"
			if(on._fixEvent){
				return on._fixEvent(evt, sender);
			}
			return evt;	// Event
		},
	
		stop: function(/*Event*/ evt){
			// summary:
			//		prevents propagation and clobbers the default action of the
			//		passed event
			// evt: Event
			//		The event object. If omitted, window.event is used on IE.
			if(has("dom-addeventlistener") || (evt && evt.preventDefault)){
				evt.preventDefault();
				evt.stopPropagation();
			}else{
				evt = evt || window.event;
				evt.cancelBubble = true;
				on._preventDefault.call(evt);
			}
		}
	};

	if( 1 ){
		dojo.fixEvent = ret.fix;
		dojo.stopEvent = ret.stop;
	}

	return ret;
});

},
'dojo/dom-geometry':function(){
define(["./sniff", "./_base/window","./dom", "./dom-style"],
		function(has, win, dom, style){
	// module:
	//		dojo/dom-geometry

	// the result object
	var geom = {
		// summary:
		//		This module defines the core dojo DOM geometry API.
	};

	// Box functions will assume this model.
	// On IE/Opera, BORDER_BOX will be set if the primary document is in quirks mode.
	// Can be set to change behavior of box setters.

	// can be either:
	//	"border-box"
	//	"content-box" (default)
	geom.boxModel = "content-box";

	// We punt per-node box mode testing completely.
	// If anybody cares, we can provide an additional (optional) unit
	// that overrides existing code to include per-node box sensitivity.

	// Opera documentation claims that Opera 9 uses border-box in BackCompat mode.
	// but experiments (Opera 9.10.8679 on Windows Vista) indicate that it actually continues to use content-box.
	// IIRC, earlier versions of Opera did in fact use border-box.
	// Opera guys, this is really confusing. Opera being broken in quirks mode is not our fault.

	if(has("ie") /*|| has("opera")*/){
		// client code may have to adjust if compatMode varies across iframes
		geom.boxModel = document.compatMode == "BackCompat" ? "border-box" : "content-box";
	}

	geom.getPadExtents = function getPadExtents(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		Returns object with special values specifically useful for node
		//		fitting.
		// description:
		//		Returns an object with `w`, `h`, `l`, `t` properties:
		//	|		l/t/r/b = left/top/right/bottom padding (respectively)
		//	|		w = the total of the left and right padding
		//	|		h = the total of the top and bottom padding
		//		If 'node' has position, l/t forms the origin for child nodes.
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), px = style.toPixelValue,
			l = px(node, s.paddingLeft), t = px(node, s.paddingTop), r = px(node, s.paddingRight), b = px(node, s.paddingBottom);
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	var none = "none";

	geom.getBorderExtents = function getBorderExtents(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		returns an object with properties useful for noting the border
		//		dimensions.
		// description:
		//		- l/t/r/b = the sum of left/top/right/bottom border (respectively)
		//		- w = the sum of the left and right border
		//		- h = the sum of the top and bottom border
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var px = style.toPixelValue, s = computedStyle || style.getComputedStyle(node),
			l = s.borderLeftStyle != none ? px(node, s.borderLeftWidth) : 0,
			t = s.borderTopStyle != none ? px(node, s.borderTopWidth) : 0,
			r = s.borderRightStyle != none ? px(node, s.borderRightWidth) : 0,
			b = s.borderBottomStyle != none ? px(node, s.borderBottomWidth) : 0;
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	geom.getPadBorderExtents = function getPadBorderExtents(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		Returns object with properties useful for box fitting with
		//		regards to padding.
		// description:
		//		- l/t/r/b = the sum of left/top/right/bottom padding and left/top/right/bottom border (respectively)
		//		- w = the sum of the left and right padding and border
		//		- h = the sum of the top and bottom padding and border
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node),
			p = geom.getPadExtents(node, s),
			b = geom.getBorderExtents(node, s);
		return {
			l: p.l + b.l,
			t: p.t + b.t,
			r: p.r + b.r,
			b: p.b + b.b,
			w: p.w + b.w,
			h: p.h + b.h
		};
	};

	geom.getMarginExtents = function getMarginExtents(node, computedStyle){
		// summary:
		//		returns object with properties useful for box fitting with
		//		regards to box margins (i.e., the outer-box).
		//
		//		- l/t = marginLeft, marginTop, respectively
		//		- w = total width, margin inclusive
		//		- h = total height, margin inclusive
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), px = style.toPixelValue,
			l = px(node, s.marginLeft), t = px(node, s.marginTop), r = px(node, s.marginRight), b = px(node, s.marginBottom);
		return {l: l, t: t, r: r, b: b, w: l + r, h: t + b};
	};

	// Box getters work in any box context because offsetWidth/clientWidth
	// are invariant wrt box context
	//
	// They do *not* work for display: inline objects that have padding styles
	// because the user agent ignores padding (it's bogus styling in any case)
	//
	// Be careful with IMGs because they are inline or block depending on
	// browser and browser mode.

	// Although it would be easier to read, there are not separate versions of
	// _getMarginBox for each browser because:
	// 1. the branching is not expensive
	// 2. factoring the shared code wastes cycles (function call overhead)
	// 3. duplicating the shared code wastes bytes

	geom.getMarginBox = function getMarginBox(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		returns an object that encodes the width, height, left and top
		//		positions of the node's margin box.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), me = geom.getMarginExtents(node, s),
			l = node.offsetLeft - me.l, t = node.offsetTop - me.t, p = node.parentNode, px = style.toPixelValue, pcs;
		if(has("mozilla")){
			// Mozilla:
			// If offsetParent has a computed overflow != visible, the offsetLeft is decreased
			// by the parent's border.
			// We don't want to compute the parent's style, so instead we examine node's
			// computed left/top which is more stable.
			var sl = parseFloat(s.left), st = parseFloat(s.top);
			if(!isNaN(sl) && !isNaN(st)){
				l = sl;
				t = st;
			}else{
				// If child's computed left/top are not parseable as a number (e.g. "auto"), we
				// have no choice but to examine the parent's computed style.
				if(p && p.style){
					pcs = style.getComputedStyle(p);
					if(pcs.overflow != "visible"){
						l += pcs.borderLeftStyle != none ? px(node, pcs.borderLeftWidth) : 0;
						t += pcs.borderTopStyle != none ? px(node, pcs.borderTopWidth) : 0;
					}
				}
			}
		}else if(has("opera") || (has("ie") == 8 && !has("quirks"))){
			// On Opera and IE 8, offsetLeft/Top includes the parent's border
			if(p){
				pcs = style.getComputedStyle(p);
				l -= pcs.borderLeftStyle != none ? px(node, pcs.borderLeftWidth) : 0;
				t -= pcs.borderTopStyle != none ? px(node, pcs.borderTopWidth) : 0;
			}
		}
		return {l: l, t: t, w: node.offsetWidth + me.w, h: node.offsetHeight + me.h};
	};

	geom.getContentBox = function getContentBox(node, computedStyle){
		// summary:
		//		Returns an object that encodes the width, height, left and top
		//		positions of the node's content box, irrespective of the
		//		current box model.
		// node: DOMNode
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		// clientWidth/Height are important since the automatically account for scrollbars
		// fallback to offsetWidth/Height for special cases (see #3378)
		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), w = node.clientWidth, h,
			pe = geom.getPadExtents(node, s), be = geom.getBorderExtents(node, s);
		if(!w){
			w = node.offsetWidth;
			h = node.offsetHeight;
		}else{
			h = node.clientHeight;
			be.w = be.h = 0;
		}
		// On Opera, offsetLeft includes the parent's border
		if(has("opera")){
			pe.l += be.l;
			pe.t += be.t;
		}
		return {l: pe.l, t: pe.t, w: w - pe.w - be.w, h: h - pe.h - be.h};
	};

	// Box setters depend on box context because interpretation of width/height styles
	// vary wrt box context.
	//
	// The value of boxModel is used to determine box context.
	// boxModel can be set directly to change behavior.
	//
	// Beware of display: inline objects that have padding styles
	// because the user agent ignores padding (it's a bogus setup anyway)
	//
	// Be careful with IMGs because they are inline or block depending on
	// browser and browser mode.
	//
	// Elements other than DIV may have special quirks, like built-in
	// margins or padding, or values not detectable via computedStyle.
	// In particular, margins on TABLE do not seems to appear
	// at all in computedStyle on Mozilla.

	function setBox(/*DomNode*/ node, /*Number?*/ l, /*Number?*/ t, /*Number?*/ w, /*Number?*/ h, /*String?*/ u){
		// summary:
		//		sets width/height/left/top in the current (native) box-model
		//		dimensions. Uses the unit passed in u.
		// node:
		//		DOM Node reference. Id string not supported for performance
		//		reasons.
		// l:
		//		left offset from parent.
		// t:
		//		top offset from parent.
		// w:
		//		width in current box model.
		// h:
		//		width in current box model.
		// u:
		//		unit measure to use for other measures. Defaults to "px".
		u = u || "px";
		var s = node.style;
		if(!isNaN(l)){
			s.left = l + u;
		}
		if(!isNaN(t)){
			s.top = t + u;
		}
		if(w >= 0){
			s.width = w + u;
		}
		if(h >= 0){
			s.height = h + u;
		}
	}

	function isButtonTag(/*DomNode*/ node){
		// summary:
		//		True if the node is BUTTON or INPUT.type="button".
		return node.tagName.toLowerCase() == "button" ||
			node.tagName.toLowerCase() == "input" && (node.getAttribute("type") || "").toLowerCase() == "button"; // boolean
	}

	function usesBorderBox(/*DomNode*/ node){
		// summary:
		//		True if the node uses border-box layout.

		// We could test the computed style of node to see if a particular box
		// has been specified, but there are details and we choose not to bother.

		// TABLE and BUTTON (and INPUT type=button) are always border-box by default.
		// If you have assigned a different box to either one via CSS then
		// box functions will break.

		return geom.boxModel == "border-box" || node.tagName.toLowerCase() == "table" || isButtonTag(node); // boolean
	}

	geom.setContentSize = function setContentSize(/*DomNode*/ node, /*Object*/ box, /*Object*/ computedStyle){
		// summary:
		//		Sets the size of the node's contents, irrespective of margins,
		//		padding, or borders.
		// node: DOMNode
		// box: Object
		//		hash with optional "w", and "h" properties for "width", and "height"
		//		respectively. All specified properties should have numeric values in whole pixels.
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var w = box.w, h = box.h;
		if(usesBorderBox(node)){
			var pb = geom.getPadBorderExtents(node, computedStyle);
			if(w >= 0){
				w += pb.w;
			}
			if(h >= 0){
				h += pb.h;
			}
		}
		setBox(node, NaN, NaN, w, h);
	};

	var nilExtents = {l: 0, t: 0, w: 0, h: 0};

	geom.setMarginBox = function setMarginBox(/*DomNode*/ node, /*Object*/ box, /*Object*/ computedStyle){
		// summary:
		//		sets the size of the node's margin box and placement
		//		(left/top), irrespective of box model. Think of it as a
		//		passthrough to setBox that handles box-model vagaries for
		//		you.
		// node: DOMNode
		// box: Object
		//		hash with optional "l", "t", "w", and "h" properties for "left", "right", "width", and "height"
		//		respectively. All specified properties should have numeric values in whole pixels.
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var s = computedStyle || style.getComputedStyle(node), w = box.w, h = box.h,
		// Some elements have special padding, margin, and box-model settings.
		// To use box functions you may need to set padding, margin explicitly.
		// Controlling box-model is harder, in a pinch you might set dojo/dom-geometry.boxModel.
			pb = usesBorderBox(node) ? nilExtents : geom.getPadBorderExtents(node, s),
			mb = geom.getMarginExtents(node, s);
		if(has("webkit")){
			// on Safari (3.1.2), button nodes with no explicit size have a default margin
			// setting an explicit size eliminates the margin.
			// We have to swizzle the width to get correct margin reading.
			if(isButtonTag(node)){
				var ns = node.style;
				if(w >= 0 && !ns.width){
					ns.width = "4px";
				}
				if(h >= 0 && !ns.height){
					ns.height = "4px";
				}
			}
		}
		if(w >= 0){
			w = Math.max(w - pb.w - mb.w, 0);
		}
		if(h >= 0){
			h = Math.max(h - pb.h - mb.h, 0);
		}
		setBox(node, box.l, box.t, w, h);
	};

	// =============================
	// Positioning
	// =============================

	geom.isBodyLtr = function isBodyLtr(/*Document?*/ doc){
		// summary:
		//		Returns true if the current language is left-to-right, and false otherwise.
		// doc: Document?
		//		Optional document to query.   If unspecified, use win.doc.
		// returns: Boolean

		doc = doc || win.doc;
		return (win.body(doc).dir || doc.documentElement.dir || "ltr").toLowerCase() == "ltr"; // Boolean
	};

	geom.docScroll = function docScroll(/*Document?*/ doc){
		// summary:
		//		Returns an object with {node, x, y} with corresponding offsets.
		// doc: Document?
		//		Optional document to query.   If unspecified, use win.doc.
		// returns: Object

		doc = doc || win.doc;
		var node = win.doc.parentWindow || win.doc.defaultView;   // use UI window, not dojo.global window.   TODO: use dojo/window::get() except for circular dependency problem
		return "pageXOffset" in node ? {x: node.pageXOffset, y: node.pageYOffset } :
			(node = has("quirks") ? win.body(doc) : doc.documentElement) &&
				{x: geom.fixIeBiDiScrollLeft(node.scrollLeft || 0, doc), y: node.scrollTop || 0 };
	};

	if(has("ie")){
		geom.getIeDocumentElementOffset = function getIeDocumentElementOffset(/*Document?*/ doc){
			// summary:
			//		returns the offset in x and y from the document body to the
			//		visual edge of the page for IE
			// doc: Document?
			//		Optional document to query.   If unspecified, use win.doc.
			// description:
			//		The following values in IE contain an offset:
			//	|		event.clientX
			//	|		event.clientY
			//	|		node.getBoundingClientRect().left
			//	|		node.getBoundingClientRect().top
			//		But other position related values do not contain this offset,
			//		such as node.offsetLeft, node.offsetTop, node.style.left and
			//		node.style.top. The offset is always (2, 2) in LTR direction.
			//		When the body is in RTL direction, the offset counts the width
			//		of left scroll bar's width.  This function computes the actual
			//		offset.

			//NOTE: assumes we're being called in an IE browser

			doc = doc || win.doc;
			var de = doc.documentElement; // only deal with HTML element here, position() handles body/quirks

			if(has("ie") < 8){
				var r = de.getBoundingClientRect(), // works well for IE6+
					l = r.left, t = r.top;
				if(has("ie") < 7){
					l += de.clientLeft;	// scrollbar size in strict/RTL, or,
					t += de.clientTop;	// HTML border size in strict
				}
				return {
					x: l < 0 ? 0 : l, // FRAME element border size can lead to inaccurate negative values
					y: t < 0 ? 0 : t
				};
			}else{
				return {
					x: 0,
					y: 0
				};
			}
		};
	}

	geom.fixIeBiDiScrollLeft = function fixIeBiDiScrollLeft(/*Integer*/ scrollLeft, /*Document?*/ doc){
		// summary:
		//		In RTL direction, scrollLeft should be a negative value, but IE
		//		returns a positive one. All codes using documentElement.scrollLeft
		//		must call this function to fix this error, otherwise the position
		//		will offset to right when there is a horizontal scrollbar.
		// scrollLeft: Number
		// doc: Document?
		//		Optional document to query.   If unspecified, use win.doc.
		// returns: Number

		// In RTL direction, scrollLeft should be a negative value, but IE
		// returns a positive one. All codes using documentElement.scrollLeft
		// must call this function to fix this error, otherwise the position
		// will offset to right when there is a horizontal scrollbar.

		doc = doc || win.doc;
		var ie = has("ie");
		if(ie && !geom.isBodyLtr(doc)){
			var qk = has("quirks"),
				de = qk ? win.body(doc) : doc.documentElement,
				pwin = win.global;	// TODO: use winUtils.get(doc) after resolving circular dependency b/w dom-geometry.js and dojo/window.js
			if(ie == 6 && !qk && pwin.frameElement && de.scrollHeight > de.clientHeight){
				scrollLeft += de.clientLeft; // workaround ie6+strict+rtl+iframe+vertical-scrollbar bug where clientWidth is too small by clientLeft pixels
			}
			return (ie < 8 || qk) ? (scrollLeft + de.clientWidth - de.scrollWidth) : -scrollLeft; // Integer
		}
		return scrollLeft; // Integer
	};

	geom.position = function(/*DomNode*/ node, /*Boolean?*/ includeScroll){
		// summary:
		//		Gets the position and size of the passed element relative to
		//		the viewport (if includeScroll==false), or relative to the
		//		document root (if includeScroll==true).
		//
		// description:
		//		Returns an object of the form:
		//		`{ x: 100, y: 300, w: 20, h: 15 }`.
		//		If includeScroll==true, the x and y values will include any
		//		document offsets that may affect the position relative to the
		//		viewport.
		//		Uses the border-box model (inclusive of border and padding but
		//		not margin).  Does not act as a setter.
		// node: DOMNode|String
		// includeScroll: Boolean?
		// returns: Object

		node = dom.byId(node);
		var	db = win.body(node.ownerDocument),
			ret = node.getBoundingClientRect();
		ret = {x: ret.left, y: ret.top, w: ret.right - ret.left, h: ret.bottom - ret.top};

		if(has("ie") < 9){
			// On IE<9 there's a 2px offset that we need to adjust for, see dojo.getIeDocumentElementOffset()
			var offset = geom.getIeDocumentElementOffset(node.ownerDocument);

			// fixes the position in IE, quirks mode
			ret.x -= offset.x + (has("quirks") ? db.clientLeft + db.offsetLeft : 0);
			ret.y -= offset.y + (has("quirks") ? db.clientTop + db.offsetTop : 0);
		}

		// account for document scrolling
		// if offsetParent is used, ret value already includes scroll position
		// so we may have to actually remove that value if !includeScroll
		if(includeScroll){
			var scroll = geom.docScroll(node.ownerDocument);
			ret.x += scroll.x;
			ret.y += scroll.y;
		}

		return ret; // Object
	};

	// random "private" functions wildly used throughout the toolkit

	geom.getMarginSize = function getMarginSize(/*DomNode*/ node, /*Object*/ computedStyle){
		// summary:
		//		returns an object that encodes the width and height of
		//		the node's margin box
		// node: DOMNode|String
		// computedStyle: Object?
		//		This parameter accepts computed styles object.
		//		If this parameter is omitted, the functions will call
		//		dojo.getComputedStyle to get one. It is a better way, calling
		//		dojo.computedStyle once, and then pass the reference to this
		//		computedStyle parameter. Wherever possible, reuse the returned
		//		object of dojo/dom-style.getComputedStyle().

		node = dom.byId(node);
		var me = geom.getMarginExtents(node, computedStyle || style.getComputedStyle(node));
		var size = node.getBoundingClientRect();
		return {
			w: (size.right - size.left) + me.w,
			h: (size.bottom - size.top) + me.h
		};
	};

	geom.normalizeEvent = function(event){
		// summary:
		//		Normalizes the geometry of a DOM event, normalizing the pageX, pageY,
		//		offsetX, offsetY, layerX, and layerX properties
		// event: Object
		if(!("layerX" in event)){
			event.layerX = event.offsetX;
			event.layerY = event.offsetY;
		}
		if(!has("dom-addeventlistener")){
			// old IE version
			// FIXME: scroll position query is duped from dojo.html to
			// avoid dependency on that entire module. Now that HTML is in
			// Base, we should convert back to something similar there.
			var se = event.target;
			var doc = (se && se.ownerDocument) || document;
			// DO NOT replace the following to use dojo.body(), in IE, document.documentElement should be used
			// here rather than document.body
			var docBody = has("quirks") ? doc.body : doc.documentElement;
			var offset = geom.getIeDocumentElementOffset(doc);
			event.pageX = event.clientX + geom.fixIeBiDiScrollLeft(docBody.scrollLeft || 0, doc) - offset.x;
			event.pageY = event.clientY + (docBody.scrollTop || 0) - offset.y;
		}
	};

	// TODO: evaluate separate getters/setters for position and sizes?

	return geom;
});

},
'dojo/_base/window':function(){
define(["./kernel", "./lang", "../sniff"], function(dojo, lang, has){
// module:
//		dojo/_base/window

var ret = {
	// summary:
	//		API to save/set/restore the global/document scope.

	global: dojo.global,
	/*=====
	 global: {
		 // summary:
		 //		Alias for the current window. 'global' can be modified
		 //		for temporary context shifting. See also withGlobal().
		 // description:
		 //		Use this rather than referring to 'window' to ensure your code runs
		 //		correctly in managed contexts.
	 },
	 =====*/

	doc: this["document"] || null,
	/*=====
	doc: {
		// summary:
		//		Alias for the current document. 'doc' can be modified
		//		for temporary context shifting. See also withDoc().
		// description:
		//		Use this rather than referring to 'window.document' to ensure your code runs
		//		correctly in managed contexts.
		// example:
		//	|	n.appendChild(dojo.doc.createElement('div'));
	},
	=====*/

	body: function(/*Document?*/ doc){
		// summary:
		//		Return the body element of the specified document or of dojo/_base/window::doc.
		// example:
		//	|	win.body().appendChild(dojo.doc.createElement('div'));

		// Note: document.body is not defined for a strict xhtml document
		// Would like to memoize this, but dojo.doc can change vi dojo.withDoc().
		doc = doc || dojo.doc;
		return doc.body || doc.getElementsByTagName("body")[0]; // Node
	},

	setContext: function(/*Object*/ globalObject, /*DocumentElement*/ globalDocument){
		// summary:
		//		changes the behavior of many core Dojo functions that deal with
		//		namespace and DOM lookup, changing them to work in a new global
		//		context (e.g., an iframe). The varibles dojo.global and dojo.doc
		//		are modified as a result of calling this function and the result of
		//		`dojo.body()` likewise differs.
		dojo.global = ret.global = globalObject;
		dojo.doc = ret.doc = globalDocument;
	},

	withGlobal: function(	/*Object*/ globalObject,
							/*Function*/ callback,
							/*Object?*/ thisObject,
							/*Array?*/ cbArguments){
		// summary:
		//		Invoke callback with globalObject as dojo.global and
		//		globalObject.document as dojo.doc.
		// description:
		//		Invoke callback with globalObject as dojo.global and
		//		globalObject.document as dojo.doc. If provided, globalObject
		//		will be executed in the context of object thisObject
		//		When callback() returns or throws an error, the dojo.global
		//		and dojo.doc will be restored to its previous state.

		var oldGlob = dojo.global;
		try{
			dojo.global = ret.global = globalObject;
			return ret.withDoc.call(null, globalObject.document, callback, thisObject, cbArguments);
		}finally{
			dojo.global = ret.global = oldGlob;
		}
	},

	withDoc: function(	/*DocumentElement*/ documentObject,
						/*Function*/ callback,
						/*Object?*/ thisObject,
						/*Array?*/ cbArguments){
		// summary:
		//		Invoke callback with documentObject as dojo/_base/window::doc.
		// description:
		//		Invoke callback with documentObject as dojo/_base/window::doc. If provided,
		//		callback will be executed in the context of object thisObject
		//		When callback() returns or throws an error, the dojo/_base/window::doc will
		//		be restored to its previous state.

		var oldDoc = ret.doc,
			oldQ = has("quirks"),
			oldIE = has("ie"), isIE, mode, pwin;

		try{
			dojo.doc = ret.doc = documentObject;
			// update dojo.isQuirks and the value of the has feature "quirks".
			// remove setting dojo.isQuirks and dojo.isIE for 2.0
			dojo.isQuirks = has.add("quirks", dojo.doc.compatMode == "BackCompat", true, true); // no need to check for QuirksMode which was Opera 7 only

			if(has("ie")){
				if((pwin = documentObject.parentWindow) && pwin.navigator){
					// re-run IE detection logic and update dojo.isIE / has("ie")
					// (the only time parentWindow/navigator wouldn't exist is if we were not
					// passed an actual legitimate document object)
					isIE = parseFloat(pwin.navigator.appVersion.split("MSIE ")[1]) || undefined;
					mode = documentObject.documentMode;
					if(mode && mode != 5 && Math.floor(isIE) != mode){
						isIE = mode;
					}
					dojo.isIE = has.add("ie", isIE, true, true);
				}
			}

			if(thisObject && typeof callback == "string"){
				callback = thisObject[callback];
			}

			return callback.apply(thisObject, cbArguments || []);
		}finally{
			dojo.doc = ret.doc = oldDoc;
			dojo.isQuirks = has.add("quirks", oldQ, true, true);
			dojo.isIE = has.add("ie", oldIE, true, true);
		}
	}
};

 1  && lang.mixin(dojo, ret);

return ret;

});

},
'dojo/dom':function(){
define(["./sniff", "./_base/window"],
		function(has, win){
	// module:
	//		dojo/dom

	// FIXME: need to add unit tests for all the semi-public methods

	if(has("ie") <= 7){
		try{
			document.execCommand("BackgroundImageCache", false, true);
		}catch(e){
			// sane browsers don't have cache "issues"
		}
	}

	// =============================
	// DOM Functions
	// =============================

	// the result object
	var dom = {
		// summary:
		//		This module defines the core dojo DOM API.
	};

	if(has("ie")){
		dom.byId = function(id, doc){
			if(typeof id != "string"){
				return id;
			}
			var _d = doc || win.doc, te = id && _d.getElementById(id);
			// attributes.id.value is better than just id in case the
			// user has a name=id inside a form
			if(te && (te.attributes.id.value == id || te.id == id)){
				return te;
			}else{
				var eles = _d.all[id];
				if(!eles || eles.nodeName){
					eles = [eles];
				}
				// if more than 1, choose first with the correct id
				var i = 0;
				while((te = eles[i++])){
					if((te.attributes && te.attributes.id && te.attributes.id.value == id) || te.id == id){
						return te;
					}
				}
			}
		};
	}else{
		dom.byId = function(id, doc){
			// inline'd type check.
			// be sure to return null per documentation, to match IE branch.
			return ((typeof id == "string") ? (doc || win.doc).getElementById(id) : id) || null; // DOMNode
		};
	}
	/*=====
	 dom.byId = function(id, doc){
		 // summary:
		 //		Returns DOM node with matching `id` attribute or falsy value (ex: null or undefined)
		 //		if not found.  If `id` is a DomNode, this function is a no-op.
		 //
		 // id: String|DOMNode
		 //		A string to match an HTML id attribute or a reference to a DOM Node
		 //
		 // doc: Document?
		 //		Document to work in. Defaults to the current value of
		 //		dojo.doc.  Can be used to retrieve
		 //		node references from other documents.
		 //
		 // example:
		 //		Look up a node by ID:
		 //	|	var n = dojo.byId("foo");
		 //
		 // example:
		 //		Check if a node exists, and use it.
		 //	|	var n = dojo.byId("bar");
		 //	|	if(n){ doStuff() ... }
		 //
		 // example:
		 //		Allow string or DomNode references to be passed to a custom function:
		 //	|	var foo = function(nodeOrId){
		 //	|		nodeOrId = dojo.byId(nodeOrId);
		 //	|		// ... more stuff
		 //	|	}
	 };
	 =====*/

	dom.isDescendant = function(/*DOMNode|String*/ node, /*DOMNode|String*/ ancestor){
		// summary:
		//		Returns true if node is a descendant of ancestor
		// node: DOMNode|String
		//		string id or node reference to test
		// ancestor: DOMNode|String
		//		string id or node reference of potential parent to test against
		//
		// example:
		//		Test is node id="bar" is a descendant of node id="foo"
		//	|	if(dojo.isDescendant("bar", "foo")){ ... }

		try{
			node = dom.byId(node);
			ancestor = dom.byId(ancestor);
			while(node){
				if(node == ancestor){
					return true; // Boolean
				}
				node = node.parentNode;
			}
		}catch(e){ /* squelch, return false */ }
		return false; // Boolean
	};


	// TODO: do we need setSelectable in the base?

	// Add feature test for user-select CSS property
	// (currently known to work in all but IE < 10 and Opera)
	has.add("css-user-select", function(global, doc, element){
		// Avoid exception when dom.js is loaded in non-browser environments
		if(!element){ return false; }
		
		var style = element.style;
		var prefixes = ["Khtml", "O", "ms", "Moz", "Webkit"],
			i = prefixes.length,
			name = "userSelect",
			prefix;

		// Iterate prefixes from most to least likely
		do{
			if(typeof style[name] !== "undefined"){
				// Supported; return property name
				return name;
			}
		}while(i-- && (name = prefixes[i] + "UserSelect"));

		// Not supported if we didn't return before now
		return false;
	});

	/*=====
	dom.setSelectable = function(node, selectable){
		// summary:
		//		Enable or disable selection on a node
		// node: DOMNode|String
		//		id or reference to node
		// selectable: Boolean
		//		state to put the node in. false indicates unselectable, true
		//		allows selection.
		// example:
		//		Make the node id="bar" unselectable
		//	|	dojo.setSelectable("bar");
		// example:
		//		Make the node id="bar" selectable
		//	|	dojo.setSelectable("bar", true);
	};
	=====*/

	var cssUserSelect = has("css-user-select");
	dom.setSelectable = cssUserSelect ? function(node, selectable){
		// css-user-select returns a (possibly vendor-prefixed) CSS property name
		dom.byId(node).style[cssUserSelect] = selectable ? "" : "none";
	} : function(node, selectable){
		node = dom.byId(node);

		// (IE < 10 / Opera) Fall back to setting/removing the
		// unselectable attribute on the element and all its children
		var nodes = node.getElementsByTagName("*"),
			i = nodes.length;

		if(selectable){
			node.removeAttribute("unselectable");
			while(i--){
				nodes[i].removeAttribute("unselectable");
			}
		}else{
			node.setAttribute("unselectable", "on");
			while(i--){
				nodes[i].setAttribute("unselectable", "on");
			}
		}
	};

	return dom;
});

},
'dojo/dom-style':function(){
define(["./sniff", "./dom"], function(has, dom){
	// module:
	//		dojo/dom-style

	// =============================
	// Style Functions
	// =============================

	// getComputedStyle drives most of the style code.
	// Wherever possible, reuse the returned object.
	//
	// API functions below that need to access computed styles accept an
	// optional computedStyle parameter.
	// If this parameter is omitted, the functions will call getComputedStyle themselves.
	// This way, calling code can access computedStyle once, and then pass the reference to
	// multiple API functions.

	// Although we normally eschew argument validation at this
	// level, here we test argument 'node' for (duck)type,
	// by testing nodeType, ecause 'document' is the 'parentNode' of 'body'
	// it is frequently sent to this function even
	// though it is not Element.
	var getComputedStyle, style = {
		// summary:
		//		This module defines the core dojo DOM style API.
	};
	if(has("webkit")){
		getComputedStyle = function(/*DomNode*/ node){
			var s;
			if(node.nodeType == 1){
				var dv = node.ownerDocument.defaultView;
				s = dv.getComputedStyle(node, null);
				if(!s && node.style){
					node.style.display = "";
					s = dv.getComputedStyle(node, null);
				}
			}
			return s || {};
		};
	}else if(has("ie") && (has("ie") < 9 || has("quirks"))){
		getComputedStyle = function(node){
			// IE (as of 7) doesn't expose Element like sane browsers
			// currentStyle can be null on IE8!
			return node.nodeType == 1 /* ELEMENT_NODE*/ && node.currentStyle ? node.currentStyle : {};
		};
	}else{
		getComputedStyle = function(node){
			return node.nodeType == 1 /* ELEMENT_NODE*/ ?
				node.ownerDocument.defaultView.getComputedStyle(node, null) : {};
		};
	}
	style.getComputedStyle = getComputedStyle;
	/*=====
	style.getComputedStyle = function(node){
		// summary:
		//		Returns a "computed style" object.
		//
		// description:
		//		Gets a "computed style" object which can be used to gather
		//		information about the current state of the rendered node.
		//
		//		Note that this may behave differently on different browsers.
		//		Values may have different formats and value encodings across
		//		browsers.
		//
		//		Note also that this method is expensive.  Wherever possible,
		//		reuse the returned object.
		//
		//		Use the dojo.style() method for more consistent (pixelized)
		//		return values.
		//
		// node: DOMNode
		//		A reference to a DOM node. Does NOT support taking an
		//		ID string for speed reasons.
		// example:
		//	|	dojo.getComputedStyle(dojo.byId('foo')).borderWidth;
		//
		// example:
		//		Reusing the returned object, avoiding multiple lookups:
		//	|	var cs = dojo.getComputedStyle(dojo.byId("someNode"));
		//	|	var w = cs.width, h = cs.height;
		return; // CSS2Properties
	};
	=====*/

	var toPixel;
	if(!has("ie")){
		toPixel = function(element, value){
			// style values can be floats, client code may want
			// to round for integer pixels.
			return parseFloat(value) || 0;
		};
	}else{
		toPixel = function(element, avalue){
			if(!avalue){ return 0; }
			// on IE7, medium is usually 4 pixels
			if(avalue == "medium"){ return 4; }
			// style values can be floats, client code may
			// want to round this value for integer pixels.
			if(avalue.slice && avalue.slice(-2) == 'px'){ return parseFloat(avalue); }
			var s = element.style, rs = element.runtimeStyle, cs = element.currentStyle,
				sLeft = s.left, rsLeft = rs.left;
			rs.left = cs.left;
			try{
				// 'avalue' may be incompatible with style.left, which can cause IE to throw
				// this has been observed for border widths using "thin", "medium", "thick" constants
				// those particular constants could be trapped by a lookup
				// but perhaps there are more
				s.left = avalue;
				avalue = s.pixelLeft;
			}catch(e){
				avalue = 0;
			}
			s.left = sLeft;
			rs.left = rsLeft;
			return avalue;
		};
	}
	style.toPixelValue = toPixel;
	/*=====
	style.toPixelValue = function(node, value){
		// summary:
		//		converts style value to pixels on IE or return a numeric value.
		// node: DOMNode
		// value: String
		// returns: Number
	};
	=====*/

	// FIXME: there opacity quirks on FF that we haven't ported over. Hrm.

	var astr = "DXImageTransform.Microsoft.Alpha";
	var af = function(n, f){
		try{
			return n.filters.item(astr);
		}catch(e){
			return f ? {} : null;
		}
	};

	var _getOpacity =
		has("ie") < 9 || (has("ie") < 10 && has("quirks")) ? function(node){
			try{
				return af(node).Opacity / 100; // Number
			}catch(e){
				return 1; // Number
			}
		} :
		function(node){
			return getComputedStyle(node).opacity;
		};

	var _setOpacity =
		has("ie") < 9 || (has("ie") < 10 && has("quirks")) ? function(/*DomNode*/ node, /*Number*/ opacity){
			if(opacity === ""){ opacity = 1; }
			var ov = opacity * 100, fullyOpaque = opacity === 1;

			// on IE7 Alpha(Filter opacity=100) makes text look fuzzy so disable it altogether (bug #2661),
			// but still update the opacity value so we can get a correct reading if it is read later:
			// af(node, 1).Enabled = !fullyOpaque;

			if(fullyOpaque){
				node.style.zoom = "";
				if(af(node)){
					node.style.filter = node.style.filter.replace(
						new RegExp("\\s*progid:" + astr + "\\([^\\)]+?\\)", "i"), "");
				}
			}else{
				node.style.zoom = 1;
				if(af(node)){
					af(node, 1).Opacity = ov;
				}else{
					node.style.filter += " progid:" + astr + "(Opacity=" + ov + ")";
				}
				af(node, 1).Enabled = true;
			}

			if(node.tagName.toLowerCase() == "tr"){
				for(var td = node.firstChild; td; td = td.nextSibling){
					if(td.tagName.toLowerCase() == "td"){
						_setOpacity(td, opacity);
					}
				}
			}
			return opacity;
		} :
		function(node, opacity){
			return node.style.opacity = opacity;
		};

	var _pixelNamesCache = {
		left: true, top: true
	};
	var _pixelRegExp = /margin|padding|width|height|max|min|offset/; // |border
	function _toStyleValue(node, type, value){
		//TODO: should we really be doing string case conversion here? Should we cache it? Need to profile!
		type = type.toLowerCase();
		if(has("ie")){
			if(value == "auto"){
				if(type == "height"){ return node.offsetHeight; }
				if(type == "width"){ return node.offsetWidth; }
			}
			if(type == "fontweight"){
				switch(value){
					case 700: return "bold";
					case 400:
					default: return "normal";
				}
			}
		}
		if(!(type in _pixelNamesCache)){
			_pixelNamesCache[type] = _pixelRegExp.test(type);
		}
		return _pixelNamesCache[type] ? toPixel(node, value) : value;
	}

	var _floatAliases = {cssFloat: 1, styleFloat: 1, "float": 1};

	// public API

	style.get = function getStyle(/*DOMNode|String*/ node, /*String?*/ name){
		// summary:
		//		Accesses styles on a node.
		// description:
		//		Getting the style value uses the computed style for the node, so the value
		//		will be a calculated value, not just the immediate node.style value.
		//		Also when getting values, use specific style names,
		//		like "borderBottomWidth" instead of "border" since compound values like
		//		"border" are not necessarily reflected as expected.
		//		If you want to get node dimensions, use `dojo.marginBox()`,
		//		`dojo.contentBox()` or `dojo.position()`.
		// node: DOMNode|String
		//		id or reference to node to get style for
		// name: String?
		//		the style property to get
		// example:
		//		Passing only an ID or node returns the computed style object of
		//		the node:
		//	|	dojo.getStyle("thinger");
		// example:
		//		Passing a node and a style property returns the current
		//		normalized, computed value for that property:
		//	|	dojo.getStyle("thinger", "opacity"); // 1 by default

		var n = dom.byId(node), l = arguments.length, op = (name == "opacity");
		if(l == 2 && op){
			return _getOpacity(n);
		}
		name = _floatAliases[name] ? "cssFloat" in n.style ? "cssFloat" : "styleFloat" : name;
		var s = style.getComputedStyle(n);
		return (l == 1) ? s : _toStyleValue(n, name, s[name] || n.style[name]); /* CSS2Properties||String||Number */
	};

	style.set = function setStyle(/*DOMNode|String*/ node, /*String|Object*/ name, /*String?*/ value){
		// summary:
		//		Sets styles on a node.
		// node: DOMNode|String
		//		id or reference to node to set style for
		// name: String|Object
		//		the style property to set in DOM-accessor format
		//		("borderWidth", not "border-width") or an object with key/value
		//		pairs suitable for setting each property.
		// value: String?
		//		If passed, sets value on the node for style, handling
		//		cross-browser concerns.  When setting a pixel value,
		//		be sure to include "px" in the value. For instance, top: "200px".
		//		Otherwise, in some cases, some browsers will not apply the style.
		//
		// example:
		//		Passing a node, a style property, and a value changes the
		//		current display of the node and returns the new computed value
		//	|	dojo.setStyle("thinger", "opacity", 0.5); // == 0.5
		//
		// example:
		//		Passing a node, an object-style style property sets each of the values in turn and returns the computed style object of the node:
		//	|	dojo.setStyle("thinger", {
		//	|		"opacity": 0.5,
		//	|		"border": "3px solid black",
		//	|		"height": "300px"
		//	|	});
		//
		// example:
		//		When the CSS style property is hyphenated, the JavaScript property is camelCased.
		//		font-size becomes fontSize, and so on.
		//	|	dojo.setStyle("thinger",{
		//	|		fontSize:"14pt",
		//	|		letterSpacing:"1.2em"
		//	|	});
		//
		// example:
		//		dojo/NodeList implements .style() using the same syntax, omitting the "node" parameter, calling
		//		dojo.style() on every element of the list. See: `dojo.query()` and `dojo/NodeList`
		//	|	dojo.query(".someClassName").style("visibility","hidden");
		//	|	// or
		//	|	dojo.query("#baz > div").style({
		//	|		opacity:0.75,
		//	|		fontSize:"13pt"
		//	|	});

		var n = dom.byId(node), l = arguments.length, op = (name == "opacity");
		name = _floatAliases[name] ? "cssFloat" in n.style ? "cssFloat" : "styleFloat" : name;
		if(l == 3){
			return op ? _setOpacity(n, value) : n.style[name] = value; // Number
		}
		for(var x in name){
			style.set(node, x, name[x]);
		}
		return style.getComputedStyle(n);
	};

	return style;
});

},
'dojo/mouse':function(){
define(["./_base/kernel", "./on", "./has", "./dom", "./_base/window"], function(dojo, on, has, dom, win){

	// module:
	//		dojo/mouse

    has.add("dom-quirks", win.doc && win.doc.compatMode == "BackCompat");
	has.add("events-mouseenter", win.doc && "onmouseenter" in win.doc.createElement("div"));
	has.add("events-mousewheel", win.doc && 'onmousewheel' in win.doc);

	var mouseButtons;
	if((has("dom-quirks") && has("ie")) || !has("dom-addeventlistener")){
		mouseButtons = {
			LEFT:   1,
			MIDDLE: 4,
			RIGHT:  2,
			// helper functions
			isButton: function(e, button){ return e.button & button; },
			isLeft:   function(e){ return e.button & 1; },
			isMiddle: function(e){ return e.button & 4; },
			isRight:  function(e){ return e.button & 2; }
		};
	}else{
		mouseButtons = {
			LEFT:   0,
			MIDDLE: 1,
			RIGHT:  2,
			// helper functions
			isButton: function(e, button){ return e.button == button; },
			isLeft:   function(e){ return e.button == 0; },
			isMiddle: function(e){ return e.button == 1; },
			isRight:  function(e){ return e.button == 2; }
		};
	}
	dojo.mouseButtons = mouseButtons;

/*=====
	dojo.mouseButtons = {
		// LEFT: Number
		//		Numeric value of the left mouse button for the platform.
		LEFT:   0,
		// MIDDLE: Number
		//		Numeric value of the middle mouse button for the platform.
		MIDDLE: 1,
		// RIGHT: Number
		//		Numeric value of the right mouse button for the platform.
		RIGHT:  2,

		isButton: function(e, button){
			// summary:
			//		Checks an event object for a pressed button
			// e: Event
			//		Event object to examine
			// button: Number
			//		The button value (example: dojo.mouseButton.LEFT)
			return e.button == button; // Boolean
		},
		isLeft: function(e){
			// summary:
			//		Checks an event object for the pressed left button
			// e: Event
			//		Event object to examine
			return e.button == 0; // Boolean
		},
		isMiddle: function(e){
			// summary:
			//		Checks an event object for the pressed middle button
			// e: Event
			//		Event object to examine
			return e.button == 1; // Boolean
		},
		isRight: function(e){
			// summary:
			//		Checks an event object for the pressed right button
			// e: Event
			//		Event object to examine
			return e.button == 2; // Boolean
		}
	};
=====*/

	function eventHandler(type, selectHandler){
		// emulation of mouseenter/leave with mouseover/out using descendant checking
		var handler = function(node, listener){
			return on(node, type, function(evt){
				if(selectHandler){
					return selectHandler(evt, listener);
				}
				if(!dom.isDescendant(evt.relatedTarget, node)){
					return listener.call(this, evt);
				}
			});
		};
		handler.bubble = function(select){
			return eventHandler(type, function(evt, listener){
				// using a selector, use the select function to determine if the mouse moved inside the selector and was previously outside the selector
				var target = select(evt.target);
				var relatedTarget = evt.relatedTarget;
				if(target && (target != (relatedTarget && relatedTarget.nodeType == 1 && select(relatedTarget)))){
					return listener.call(target, evt);
				} 
			});
		};
		return handler;
	}
	var wheel;
	if(has("events-mousewheel")){
		wheel = 'mousewheel';
	}else{ //firefox
		wheel = function(node, listener){
			return on(node, 'DOMMouseScroll', function(evt){
				evt.wheelDelta = -evt.detail;
				listener.call(this, evt);
			});
		};
	}
	return {
		// summary:
		//		This module provide mouse event handling utility functions and exports
		//		mouseenter and mouseleave event emulation.
		// example:
		//		To use these events, you register a mouseenter like this:
		//		|	define(["dojo/on", dojo/mouse"], function(on, mouse){
		//		|		on(targetNode, mouse.enter, function(event){
		//		|			dojo.addClass(targetNode, "highlighted");
		//		|		});
		//		|		on(targetNode, mouse.leave, function(event){
		//		|			dojo.removeClass(targetNode, "highlighted");
		//		|		});

		_eventHandler: eventHandler,		// for dojo/touch

		// enter: Synthetic Event
		//		This is an extension event for the mouseenter that IE provides, emulating the
		//		behavior on other browsers.
		enter: eventHandler("mouseover"),

		// leave: Synthetic Event
		//		This is an extension event for the mouseleave that IE provides, emulating the
		//		behavior on other browsers.
		leave: eventHandler("mouseout"),

		// wheel: Normalized Mouse Wheel Event
		//		This is an extension event for the mousewheel that non-Mozilla browsers provide,
		//		emulating the behavior on Mozilla based browsers.
		wheel: wheel,

		isLeft: mouseButtons.isLeft,
		/*=====
		isLeft: function(){
			// summary:
			//		Test an event object (from a mousedown event) to see if the left button was pressed.
		},
		=====*/

		isMiddle: mouseButtons.isMiddle,
		/*=====
		 isMiddle: function(){
			 // summary:
			 //		Test an event object (from a mousedown event) to see if the middle button was pressed.
		 },
		 =====*/

		isRight: mouseButtons.isRight
		/*=====
		 , isRight: function(){
			 // summary:
			 //		Test an event object (from a mousedown event) to see if the right button was pressed.
		 }
		 =====*/
	};
});

},
'dojo/_base/sniff':function(){
define(["./kernel", "./lang", "../sniff"], function(dojo, lang, has){
	// module:
	//		dojo/_base/sniff

	/*=====
	return {
		// summary:
		//		Deprecated.   New code should use dojo/sniff.
		//		This module populates the dojo browser version sniffing properties like dojo.isIE.
	};
	=====*/

	if(! 1 ){
		return has;
	}

	// no idea what this is for, or if it's used
	dojo._name = "browser";

	lang.mixin(dojo, {
		// isBrowser: Boolean
		//		True if the client is a web-browser
		isBrowser: true,

		// isFF: Number|undefined
		//		Version as a Number if client is FireFox. undefined otherwise. Corresponds to
		//		major detected FireFox version (1.5, 2, 3, etc.)
		isFF: has("ff"),

		// isIE: Number|undefined
		//		Version as a Number if client is MSIE(PC). undefined otherwise. Corresponds to
		//		major detected IE version (6, 7, 8, etc.)
		isIE: has("ie"),

		// isKhtml: Number|undefined
		//		Version as a Number if client is a KHTML browser. undefined otherwise. Corresponds to major
		//		detected version.
		isKhtml: has("khtml"),

		// isWebKit: Number|undefined
		//		Version as a Number if client is a WebKit-derived browser (Konqueror,
		//		Safari, Chrome, etc.). undefined otherwise.
		isWebKit: has("webkit"),

		// isMozilla: Number|undefined
		//		Version as a Number if client is a Mozilla-based browser (Firefox,
		//		SeaMonkey). undefined otherwise. Corresponds to major detected version.
		isMozilla: has("mozilla"),
		// isMoz: Number|undefined
		//		Version as a Number if client is a Mozilla-based browser (Firefox,
		//		SeaMonkey). undefined otherwise. Corresponds to major detected version.
		isMoz: has("mozilla"),

		// isOpera: Number|undefined
		//		Version as a Number if client is Opera. undefined otherwise. Corresponds to
		//		major detected version.
		isOpera: has("opera"),

		// isSafari: Number|undefined
		//		Version as a Number if client is Safari or iPhone. undefined otherwise.
		isSafari: has("safari"),

		// isChrome: Number|undefined
		//		Version as a Number if client is Chrome browser. undefined otherwise.
		isChrome: has("chrome"),

		// isMac: Boolean
		//		True if the client runs on Mac
		isMac: has("mac"),

		// isIos: Number|undefined
		//		Version as a Number if client is iPhone, iPod, or iPad. undefined otherwise.
		isIos: has("ios"),

		// isAndroid: Number|undefined
		//		Version as a Number if client is android browser. undefined otherwise.
		isAndroid: has("android"),

		// isWii: Boolean
		//		True if client is Wii
		isWii: has("wii"),

		// isQuirks: Boolean
		//		Page is in quirks mode.
		isQuirks: has("quirks"),

		// isAir: Boolean
		//		True if client is Adobe Air
		isAir: has("air")
	});

	return has;
});

},
'dojo/keys':function(){
define(["./_base/kernel", "./sniff"], function(dojo, has){

	// module:
	//		dojo/keys

	return dojo.keys = {
		// summary:
		//		Definitions for common key values.  Client code should test keyCode against these named constants,
		//		as the actual codes can vary by browser.

		BACKSPACE: 8,
		TAB: 9,
		CLEAR: 12,
		ENTER: 13,
		SHIFT: 16,
		CTRL: 17,
		ALT: 18,
		META: has("webkit") ? 91 : 224,		// the apple key on macs
		PAUSE: 19,
		CAPS_LOCK: 20,
		ESCAPE: 27,
		SPACE: 32,
		PAGE_UP: 33,
		PAGE_DOWN: 34,
		END: 35,
		HOME: 36,
		LEFT_ARROW: 37,
		UP_ARROW: 38,
		RIGHT_ARROW: 39,
		DOWN_ARROW: 40,
		INSERT: 45,
		DELETE: 46,
		HELP: 47,
		LEFT_WINDOW: 91,
		RIGHT_WINDOW: 92,
		SELECT: 93,
		NUMPAD_0: 96,
		NUMPAD_1: 97,
		NUMPAD_2: 98,
		NUMPAD_3: 99,
		NUMPAD_4: 100,
		NUMPAD_5: 101,
		NUMPAD_6: 102,
		NUMPAD_7: 103,
		NUMPAD_8: 104,
		NUMPAD_9: 105,
		NUMPAD_MULTIPLY: 106,
		NUMPAD_PLUS: 107,
		NUMPAD_ENTER: 108,
		NUMPAD_MINUS: 109,
		NUMPAD_PERIOD: 110,
		NUMPAD_DIVIDE: 111,
		F1: 112,
		F2: 113,
		F3: 114,
		F4: 115,
		F5: 116,
		F6: 117,
		F7: 118,
		F8: 119,
		F9: 120,
		F10: 121,
		F11: 122,
		F12: 123,
		F13: 124,
		F14: 125,
		F15: 126,
		NUM_LOCK: 144,
		SCROLL_LOCK: 145,
		UP_DPAD: 175,
		DOWN_DPAD: 176,
		LEFT_DPAD: 177,
		RIGHT_DPAD: 178,
		// virtual key mapping
		copyKey: has("mac") && !has("air") ? (has("safari") ? 91 : 224 ) : 17
	};
});

},
'dojo/_base/Deferred':function(){
define([
	"./kernel",
	"../Deferred",
	"../promise/Promise",
	"../errors/CancelError",
	"../has",
	"./lang",
	"../when"
], function(dojo, NewDeferred, Promise, CancelError, has, lang, when){
	// module:
	//		dojo/_base/Deferred

	var mutator = function(){};
	var freeze = Object.freeze || function(){};
	// A deferred provides an API for creating and resolving a promise.
	var Deferred = dojo.Deferred = function(/*Function?*/ canceller){
		// summary:
		//		Deprecated.   This module defines the legacy dojo/_base/Deferred API.
		//		New code should use dojo/Deferred instead.
		// description:
		//		The Deferred API is based on the concept of promises that provide a
		//		generic interface into the eventual completion of an asynchronous action.
		//		The motivation for promises fundamentally is about creating a
		//		separation of concerns that allows one to achieve the same type of
		//		call patterns and logical data flow in asynchronous code as can be
		//		achieved in synchronous code. Promises allows one
		//		to be able to call a function purely with arguments needed for
		//		execution, without conflating the call with concerns of whether it is
		//		sync or async. One shouldn't need to alter a call's arguments if the
		//		implementation switches from sync to async (or vice versa). By having
		//		async functions return promises, the concerns of making the call are
		//		separated from the concerns of asynchronous interaction (which are
		//		handled by the promise).
		//
		//		The Deferred is a type of promise that provides methods for fulfilling the
		//		promise with a successful result or an error. The most important method for
		//		working with Dojo's promises is the then() method, which follows the
		//		CommonJS proposed promise API. An example of using a Dojo promise:
		//
		//		|	var resultingPromise = someAsyncOperation.then(function(result){
		//		|		... handle result ...
		//		|	},
		//		|	function(error){
		//		|		... handle error ...
		//		|	});
		//
		//		The .then() call returns a new promise that represents the result of the
		//		execution of the callback. The callbacks will never affect the original promises value.
		//
		//		The Deferred instances also provide the following functions for backwards compatibility:
		//
		//		- addCallback(handler)
		//		- addErrback(handler)
		//		- callback(result)
		//		- errback(result)
		//
		//		Callbacks are allowed to return promises themselves, so
		//		you can build complicated sequences of events with ease.
		//
		//		The creator of the Deferred may specify a canceller.  The canceller
		//		is a function that will be called if Deferred.cancel is called
		//		before the Deferred fires. You can use this to implement clean
		//		aborting of an XMLHttpRequest, etc. Note that cancel will fire the
		//		deferred with a CancelledError (unless your canceller returns
		//		another kind of error), so the errbacks should be prepared to
		//		handle that error for cancellable Deferreds.
		// example:
		//	|	var deferred = new Deferred();
		//	|	setTimeout(function(){ deferred.callback({success: true}); }, 1000);
		//	|	return deferred;
		// example:
		//		Deferred objects are often used when making code asynchronous. It
		//		may be easiest to write functions in a synchronous manner and then
		//		split code using a deferred to trigger a response to a long-lived
		//		operation. For example, instead of register a callback function to
		//		denote when a rendering operation completes, the function can
		//		simply return a deferred:
		//
		//		|	// callback style:
		//		|	function renderLotsOfData(data, callback){
		//		|		var success = false
		//		|		try{
		//		|			for(var x in data){
		//		|				renderDataitem(data[x]);
		//		|			}
		//		|			success = true;
		//		|		}catch(e){ }
		//		|		if(callback){
		//		|			callback(success);
		//		|		}
		//		|	}
		//
		//		|	// using callback style
		//		|	renderLotsOfData(someDataObj, function(success){
		//		|		// handles success or failure
		//		|		if(!success){
		//		|			promptUserToRecover();
		//		|		}
		//		|	});
		//		|	// NOTE: no way to add another callback here!!
		// example:
		//		Using a Deferred doesn't simplify the sending code any, but it
		//		provides a standard interface for callers and senders alike,
		//		providing both with a simple way to service multiple callbacks for
		//		an operation and freeing both sides from worrying about details
		//		such as "did this get called already?". With Deferreds, new
		//		callbacks can be added at any time.
		//
		//		|	// Deferred style:
		//		|	function renderLotsOfData(data){
		//		|		var d = new Deferred();
		//		|		try{
		//		|			for(var x in data){
		//		|				renderDataitem(data[x]);
		//		|			}
		//		|			d.callback(true);
		//		|		}catch(e){
		//		|			d.errback(new Error("rendering failed"));
		//		|		}
		//		|		return d;
		//		|	}
		//
		//		|	// using Deferred style
		//		|	renderLotsOfData(someDataObj).then(null, function(){
		//		|		promptUserToRecover();
		//		|	});
		//		|	// NOTE: addErrback and addCallback both return the Deferred
		//		|	// again, so we could chain adding callbacks or save the
		//		|	// deferred for later should we need to be notified again.
		// example:
		//		In this example, renderLotsOfData is synchronous and so both
		//		versions are pretty artificial. Putting the data display on a
		//		timeout helps show why Deferreds rock:
		//
		//		|	// Deferred style and async func
		//		|	function renderLotsOfData(data){
		//		|		var d = new Deferred();
		//		|		setTimeout(function(){
		//		|			try{
		//		|				for(var x in data){
		//		|					renderDataitem(data[x]);
		//		|				}
		//		|				d.callback(true);
		//		|			}catch(e){
		//		|				d.errback(new Error("rendering failed"));
		//		|			}
		//		|		}, 100);
		//		|		return d;
		//		|	}
		//
		//		|	// using Deferred style
		//		|	renderLotsOfData(someDataObj).then(null, function(){
		//		|		promptUserToRecover();
		//		|	});
		//
		//		Note that the caller doesn't have to change his code at all to
		//		handle the asynchronous case.

		var result, finished, canceled, fired, isError, head, nextListener;
		var promise = (this.promise = new Promise());

		function complete(value){
			if(finished){
				throw new Error("This deferred has already been resolved");
			}
			result = value;
			finished = true;
			notify();
		}
		function notify(){
			var mutated;
			while(!mutated && nextListener){
				var listener = nextListener;
				nextListener = nextListener.next;
				if((mutated = (listener.progress == mutator))){ // assignment and check
					finished = false;
				}

				var func = (isError ? listener.error : listener.resolved);
				if(has("config-useDeferredInstrumentation")){
					if(isError && NewDeferred.instrumentRejected){
						NewDeferred.instrumentRejected(result, !!func);
					}
				}
				if(func){
					try{
						var newResult = func(result);
						if (newResult && typeof newResult.then === "function"){
							newResult.then(lang.hitch(listener.deferred, "resolve"), lang.hitch(listener.deferred, "reject"), lang.hitch(listener.deferred, "progress"));
							continue;
						}
						var unchanged = mutated && newResult === undefined;
						if(mutated && !unchanged){
							isError = newResult instanceof Error;
						}
						listener.deferred[unchanged && isError ? "reject" : "resolve"](unchanged ? result : newResult);
					}catch(e){
						listener.deferred.reject(e);
					}
				}else{
					if(isError){
						listener.deferred.reject(result);
					}else{
						listener.deferred.resolve(result);
					}
				}
			}
		}

		this.isResolved = promise.isResolved = function(){
			// summary:
			//		Checks whether the deferred has been resolved.
			// returns: Boolean

			return fired == 0;
		};

		this.isRejected = promise.isRejected = function(){
			// summary:
			//		Checks whether the deferred has been rejected.
			// returns: Boolean

			return fired == 1;
		};

		this.isFulfilled = promise.isFulfilled = function(){
			// summary:
			//		Checks whether the deferred has been resolved or rejected.
			// returns: Boolean

			return fired >= 0;
		};

		this.isCanceled = promise.isCanceled = function(){
			// summary:
			//		Checks whether the deferred has been canceled.
			// returns: Boolean

			return canceled;
		};

		// calling resolve will resolve the promise
		this.resolve = this.callback = function(value){
			// summary:
			//		Fulfills the Deferred instance successfully with the provide value
			this.fired = fired = 0;
			this.results = [value, null];
			complete(value);
		};


		// calling error will indicate that the promise failed
		this.reject = this.errback = function(error){
			// summary:
			//		Fulfills the Deferred instance as an error with the provided error
			isError = true;
			this.fired = fired = 1;
			if(has("config-useDeferredInstrumentation")){
				if(NewDeferred.instrumentRejected){
					NewDeferred.instrumentRejected(error, !!nextListener);
				}
			}
			complete(error);
			this.results = [null, error];
		};
		// call progress to provide updates on the progress on the completion of the promise
		this.progress = function(update){
			// summary:
			//		Send progress events to all listeners
			var listener = nextListener;
			while(listener){
				var progress = listener.progress;
				progress && progress(update);
				listener = listener.next;
			}
		};
		this.addCallbacks = function(callback, errback){
			// summary:
			//		Adds callback and error callback for this deferred instance.
			// callback: Function?
			//		The callback attached to this deferred object.
			// errback: Function?
			//		The error callback attached to this deferred object.
			// returns:
			//		Returns this deferred object.
			this.then(callback, errback, mutator);
			return this;	// Deferred
		};
		// provide the implementation of the promise
		promise.then = this.then = function(/*Function?*/resolvedCallback, /*Function?*/errorCallback, /*Function?*/progressCallback){
			// summary:
			//		Adds a fulfilledHandler, errorHandler, and progressHandler to be called for
			//		completion of a promise. The fulfilledHandler is called when the promise
			//		is fulfilled. The errorHandler is called when a promise fails. The
			//		progressHandler is called for progress events. All arguments are optional
			//		and non-function values are ignored. The progressHandler is not only an
			//		optional argument, but progress events are purely optional. Promise
			//		providers are not required to ever create progress events.
			//
			//		This function will return a new promise that is fulfilled when the given
			//		fulfilledHandler or errorHandler callback is finished. This allows promise
			//		operations to be chained together. The value returned from the callback
			//		handler is the fulfillment value for the returned promise. If the callback
			//		throws an error, the returned promise will be moved to failed state.
			//
			// returns:
			//		Returns a new promise that represents the result of the
			//		execution of the callback. The callbacks will never affect the original promises value.
			// example:
			//		An example of using a CommonJS compliant promise:
			//		|	asyncComputeTheAnswerToEverything().
			//		|		then(addTwo).
			//		|		then(printResult, onError);
			//		|	>44
			//
			var returnDeferred = progressCallback == mutator ? this : new Deferred(promise.cancel);
			var listener = {
				resolved: resolvedCallback,
				error: errorCallback,
				progress: progressCallback,
				deferred: returnDeferred
			};
			if(nextListener){
				head = head.next = listener;
			}
			else{
				nextListener = head = listener;
			}
			if(finished){
				notify();
			}
			return returnDeferred.promise; // Promise
		};
		var deferred = this;
		promise.cancel = this.cancel = function(){
			// summary:
			//		Cancels the asynchronous operation
			if(!finished){
				var error = canceller && canceller(deferred);
				if(!finished){
					if (!(error instanceof Error)){
						error = new CancelError(error);
					}
					error.log = false;
					deferred.reject(error);
				}
			}
			canceled = true;
		};
		freeze(promise);
	};
	lang.extend(Deferred, {
		addCallback: function(/*Function*/ callback){
			// summary:
			//		Adds successful callback for this deferred instance.
			// returns:
			//		Returns this deferred object.
			return this.addCallbacks(lang.hitch.apply(dojo, arguments));	// Deferred
		},

		addErrback: function(/*Function*/ errback){
			// summary:
			//		Adds error callback for this deferred instance.
			// returns:
			//		Returns this deferred object.
			return this.addCallbacks(null, lang.hitch.apply(dojo, arguments));	// Deferred
		},

		addBoth: function(/*Function*/ callback){
			// summary:
			//		Add handler as both successful callback and error callback for this deferred instance.
			// returns:
			//		Returns this deferred object.
			var enclosed = lang.hitch.apply(dojo, arguments);
			return this.addCallbacks(enclosed, enclosed);	// Deferred
		},
		fired: -1
	});

	Deferred.when = dojo.when = when;

	return Deferred;
});

},
'dojo/Deferred':function(){
define([
	"./has",
	"./_base/lang",
	"./errors/CancelError",
	"./promise/Promise",
	"./promise/instrumentation"
], function(has, lang, CancelError, Promise, instrumentation){
	"use strict";

	// module:
	//		dojo/Deferred

	var PROGRESS = 0,
			RESOLVED = 1,
			REJECTED = 2;
	var FULFILLED_ERROR_MESSAGE = "This deferred has already been fulfilled.";

	var freezeObject = Object.freeze || function(){};

	var signalWaiting = function(waiting, type, result, rejection, deferred){
		if( 1 ){
			if(type === REJECTED && Deferred.instrumentRejected && waiting.length === 0){
				Deferred.instrumentRejected(result, false, rejection, deferred);
			}
		}

		for(var i = 0; i < waiting.length; i++){
			signalListener(waiting[i], type, result, rejection);
		}
	};

	var signalListener = function(listener, type, result, rejection){
		var func = listener[type];
		var deferred = listener.deferred;
		if(func){
			try{
				var newResult = func(result);
				if(type === PROGRESS){
					if(typeof newResult !== "undefined"){
						signalDeferred(deferred, type, newResult);
					}
				}else{
					if(newResult && typeof newResult.then === "function"){
						listener.cancel = newResult.cancel;
						newResult.then(
								// Only make resolvers if they're actually going to be used
								makeDeferredSignaler(deferred, RESOLVED),
								makeDeferredSignaler(deferred, REJECTED),
								makeDeferredSignaler(deferred, PROGRESS));
						return;
					}
					signalDeferred(deferred, RESOLVED, newResult);
				}
			}catch(error){
				signalDeferred(deferred, REJECTED, error);
			}
		}else{
			signalDeferred(deferred, type, result);
		}

		if( 1 ){
			if(type === REJECTED && Deferred.instrumentRejected){
				Deferred.instrumentRejected(result, !!func, rejection, deferred.promise);
			}
		}
	};

	var makeDeferredSignaler = function(deferred, type){
		return function(value){
			signalDeferred(deferred, type, value);
		};
	};

	var signalDeferred = function(deferred, type, result){
		if(!deferred.isCanceled()){
			switch(type){
				case PROGRESS:
					deferred.progress(result);
					break;
				case RESOLVED:
					deferred.resolve(result);
					break;
				case REJECTED:
					deferred.reject(result);
					break;
			}
		}
	};

	var Deferred = function(canceler){
		// summary:
		//		Creates a new deferred. This API is preferred over
		//		`dojo/_base/Deferred`.
		// description:
		//		Creates a new deferred, as an abstraction over (primarily)
		//		asynchronous operations. The deferred is the private interface
		//		that should not be returned to calling code. That's what the
		//		`promise` is for. See `dojo/promise/Promise`.
		// canceler: Function?
		//		Will be invoked if the deferred is canceled. The canceler
		//		receives the reason the deferred was canceled as its argument.
		//		The deferred is rejected with its return value, or a new
		//		`dojo/errors/CancelError` instance.

		// promise: dojo/promise/Promise
		//		The public promise object that clients can add callbacks to. 
		var promise = this.promise = new Promise();

		var deferred = this;
		var fulfilled, result, rejection;
		var canceled = false;
		var waiting = [];

		if( 1  && Error.captureStackTrace){
			Error.captureStackTrace(deferred, Deferred);
			Error.captureStackTrace(promise, Deferred);
		}

		this.isResolved = promise.isResolved = function(){
			// summary:
			//		Checks whether the deferred has been resolved.
			// returns: Boolean

			return fulfilled === RESOLVED;
		};

		this.isRejected = promise.isRejected = function(){
			// summary:
			//		Checks whether the deferred has been rejected.
			// returns: Boolean

			return fulfilled === REJECTED;
		};

		this.isFulfilled = promise.isFulfilled = function(){
			// summary:
			//		Checks whether the deferred has been resolved or rejected.
			// returns: Boolean

			return !!fulfilled;
		};

		this.isCanceled = promise.isCanceled = function(){
			// summary:
			//		Checks whether the deferred has been canceled.
			// returns: Boolean

			return canceled;
		};

		this.progress = function(update, strict){
			// summary:
			//		Emit a progress update on the deferred.
			// description:
			//		Emit a progress update on the deferred. Progress updates
			//		can be used to communicate updates about the asynchronous
			//		operation before it has finished.
			// update: any
			//		The progress update. Passed to progbacks.
			// strict: Boolean?
			//		If strict, will throw an error if the deferred has already
			//		been fulfilled and consequently no progress can be emitted.
			// returns: dojo/promise/Promise
			//		Returns the original promise for the deferred.

			if(!fulfilled){
				signalWaiting(waiting, PROGRESS, update, null, deferred);
				return promise;
			}else if(strict === true){
				throw new Error(FULFILLED_ERROR_MESSAGE);
			}else{
				return promise;
			}
		};

		this.resolve = function(value, strict){
			// summary:
			//		Resolve the deferred.
			// description:
			//		Resolve the deferred, putting it in a success state.
			// value: any
			//		The result of the deferred. Passed to callbacks.
			// strict: Boolean?
			//		If strict, will throw an error if the deferred has already
			//		been fulfilled and consequently cannot be resolved.
			// returns: dojo/promise/Promise
			//		Returns the original promise for the deferred.

			if(!fulfilled){
				// Set fulfilled, store value. After signaling waiting listeners unset
				// waiting.
				signalWaiting(waiting, fulfilled = RESOLVED, result = value, null, deferred);
				waiting = null;
				return promise;
			}else if(strict === true){
				throw new Error(FULFILLED_ERROR_MESSAGE);
			}else{
				return promise;
			}
		};

		var reject = this.reject = function(error, strict){
			// summary:
			//		Reject the deferred.
			// description:
			//		Reject the deferred, putting it in an error state.
			// error: any
			//		The error result of the deferred. Passed to errbacks.
			// strict: Boolean?
			//		If strict, will throw an error if the deferred has already
			//		been fulfilled and consequently cannot be rejected.
			// returns: dojo/promise/Promise
			//		Returns the original promise for the deferred.

			if(!fulfilled){
				if( 1  && Error.captureStackTrace){
					Error.captureStackTrace(rejection = {}, reject);
				}
				signalWaiting(waiting, fulfilled = REJECTED, result = error, rejection, deferred);
				waiting = null;
				return promise;
			}else if(strict === true){
				throw new Error(FULFILLED_ERROR_MESSAGE);
			}else{
				return promise;
			}
		};

		this.then = promise.then = function(callback, errback, progback){
			// summary:
			//		Add new callbacks to the deferred.
			// description:
			//		Add new callbacks to the deferred. Callbacks can be added
			//		before or after the deferred is fulfilled.
			// callback: Function?
			//		Callback to be invoked when the promise is resolved.
			//		Receives the resolution value.
			// errback: Function?
			//		Callback to be invoked when the promise is rejected.
			//		Receives the rejection error.
			// progback: Function?
			//		Callback to be invoked when the promise emits a progress
			//		update. Receives the progress update.
			// returns: dojo/promise/Promise
			//		Returns a new promise for the result of the callback(s).
			//		This can be used for chaining many asynchronous operations.

			var listener = [progback, callback, errback];
			// Ensure we cancel the promise we're waiting for, or if callback/errback
			// have returned a promise, cancel that one.
			listener.cancel = promise.cancel;
			listener.deferred = new Deferred(function(reason){
				// Check whether cancel is really available, returned promises are not
				// required to expose `cancel`
				return listener.cancel && listener.cancel(reason);
			});
			if(fulfilled && !waiting){
				signalListener(listener, fulfilled, result, rejection);
			}else{
				waiting.push(listener);
			}
			return listener.deferred.promise;
		};

		this.cancel = promise.cancel = function(reason, strict){
			// summary:
			//		Inform the deferred it may cancel its asynchronous operation.
			// description:
			//		Inform the deferred it may cancel its asynchronous operation.
			//		The deferred's (optional) canceler is invoked and the
			//		deferred will be left in a rejected state. Can affect other
			//		promises that originate with the same deferred.
			// reason: any
			//		A message that may be sent to the deferred's canceler,
			//		explaining why it's being canceled.
			// strict: Boolean?
			//		If strict, will throw an error if the deferred has already
			//		been fulfilled and consequently cannot be canceled.
			// returns: any
			//		Returns the rejection reason if the deferred was canceled
			//		normally.

			if(!fulfilled){
				// Cancel can be called even after the deferred is fulfilled
				if(canceler){
					var returnedReason = canceler(reason);
					reason = typeof returnedReason === "undefined" ? reason : returnedReason;
				}
				canceled = true;
				if(!fulfilled){
					// Allow canceler to provide its own reason, but fall back to a CancelError
					if(typeof reason === "undefined"){
						reason = new CancelError();
					}
					reject(reason);
					return reason;
				}else if(fulfilled === REJECTED && result === reason){
					return reason;
				}
			}else if(strict === true){
				throw new Error(FULFILLED_ERROR_MESSAGE);
			}
		};

		freezeObject(promise);
	};

	Deferred.prototype.toString = function(){
		// returns: String
		//		Returns `[object Deferred]`.

		return "[object Deferred]";
	};

	if(instrumentation){
		instrumentation(Deferred);
	}

	return Deferred;
});

},
'dojo/errors/CancelError':function(){
define(["./create"], function(create){
	// module:
	//		dojo/errors/CancelError

	/*=====
	return function(){
		// summary:
		//		Default error if a promise is canceled without a reason.
	};
	=====*/

	return create("CancelError", null, null, { dojoType: "cancel" });
});

},
'dojo/errors/create':function(){
define(["../_base/lang"], function(lang){
	return function(name, ctor, base, props){
		base = base || Error;

		var ErrorCtor = function(message){
			if(base === Error){
				if(Error.captureStackTrace){
					Error.captureStackTrace(this, ErrorCtor);
				}

				// Error.call() operates on the returned error
				// object rather than operating on |this|
				var err = Error.call(this, message),
					prop;

				// Copy own properties from err to |this|
				for(prop in err){
					if(err.hasOwnProperty(prop)){
						this[prop] = err[prop];
					}
				}

				// messsage is non-enumerable in ES5
				this.message = message;
				// stack is non-enumerable in at least Firefox
				this.stack = err.stack;
			}else{
				base.apply(this, arguments);
			}
			if(ctor){
				ctor.apply(this, arguments);
			}
		};

		ErrorCtor.prototype = lang.delegate(base.prototype, props);
		ErrorCtor.prototype.name = name;
		ErrorCtor.prototype.constructor = ErrorCtor;

		return ErrorCtor;
	};
});

},
'dojo/promise/Promise':function(){
define([
	"../_base/lang"
], function(lang){
	"use strict";

	// module:
	//		dojo/promise/Promise

	function throwAbstract(){
		throw new TypeError("abstract");
	}

	return lang.extend(function Promise(){
		// summary:
		//		The public interface to a deferred.
		// description:
		//		The public interface to a deferred. All promises in Dojo are
		//		instances of this class.
	}, {
		then: function(callback, errback, progback){
			// summary:
			//		Add new callbacks to the promise.
			// description:
			//		Add new callbacks to the deferred. Callbacks can be added
			//		before or after the deferred is fulfilled.
			// callback: Function?
			//		Callback to be invoked when the promise is resolved.
			//		Receives the resolution value.
			// errback: Function?
			//		Callback to be invoked when the promise is rejected.
			//		Receives the rejection error.
			// progback: Function?
			//		Callback to be invoked when the promise emits a progress
			//		update. Receives the progress update.
			// returns: dojo/promise/Promise
			//		Returns a new promise for the result of the callback(s).
			//		This can be used for chaining many asynchronous operations.

			throwAbstract();
		},

		cancel: function(reason, strict){
			// summary:
			//		Inform the deferred it may cancel its asynchronous operation.
			// description:
			//		Inform the deferred it may cancel its asynchronous operation.
			//		The deferred's (optional) canceler is invoked and the
			//		deferred will be left in a rejected state. Can affect other
			//		promises that originate with the same deferred.
			// reason: any
			//		A message that may be sent to the deferred's canceler,
			//		explaining why it's being canceled.
			// strict: Boolean?
			//		If strict, will throw an error if the deferred has already
			//		been fulfilled and consequently cannot be canceled.
			// returns: any
			//		Returns the rejection reason if the deferred was canceled
			//		normally.

			throwAbstract();
		},

		isResolved: function(){
			// summary:
			//		Checks whether the promise has been resolved.
			// returns: Boolean

			throwAbstract();
		},

		isRejected: function(){
			// summary:
			//		Checks whether the promise has been rejected.
			// returns: Boolean

			throwAbstract();
		},

		isFulfilled: function(){
			// summary:
			//		Checks whether the promise has been resolved or rejected.
			// returns: Boolean

			throwAbstract();
		},

		isCanceled: function(){
			// summary:
			//		Checks whether the promise has been canceled.
			// returns: Boolean

			throwAbstract();
		},

		always: function(callbackOrErrback){
			// summary:
			//		Add a callback to be invoked when the promise is resolved
			//		or rejected.
			// callbackOrErrback: Function?
			//		A function that is used both as a callback and errback.
			// returns: dojo/promise/Promise
			//		Returns a new promise for the result of the callback/errback.

			return this.then(callbackOrErrback, callbackOrErrback);
		},

		otherwise: function(errback){
			// summary:
			//		Add new errbacks to the promise.
			// errback: Function?
			//		Callback to be invoked when the promise is rejected.
			// returns: dojo/promise/Promise
			//		Returns a new promise for the result of the errback.

			return this.then(null, errback);
		},

		trace: function(){
			return this;
		},

		traceRejected: function(){
			return this;
		},

		toString: function(){
			// returns: string
			//		Returns `[object Promise]`.

			return "[object Promise]";
		}
	});
});

},
'dojo/promise/instrumentation':function(){
define([
	"./tracer",
	"../has",
	"../_base/lang",
	"../_base/array"
], function(tracer, has, lang, arrayUtil){
	function logError(error, rejection, deferred){
		var stack = "";
		if(error && error.stack){
			stack += error.stack;
		}
		if(rejection && rejection.stack){
			stack += "\n    ----------------------------------------\n    rejected" + rejection.stack.split("\n").slice(1).join("\n").replace(/^\s+/, " ");
		}
		if(deferred && deferred.stack){
			stack += "\n    ----------------------------------------\n" + deferred.stack;
		}
		console.error(error, stack);
	}

	function reportRejections(error, handled, rejection, deferred){
		if(!handled){
			logError(error, rejection, deferred);
		}
	}

	var errors = [];
	var activeTimeout = false;
	var unhandledWait = 1000;
	function trackUnhandledRejections(error, handled, rejection, deferred){
		if(handled){
			arrayUtil.some(errors, function(obj, ix){
				if(obj.error === error){
					errors.splice(ix, 1);
					return true;
				}
			});
		}else if(!arrayUtil.some(errors, function(obj){ return obj.error === error; })){
			errors.push({
				error: error,
				rejection: rejection,
				deferred: deferred,
				timestamp: new Date().getTime()
			});
		}

		if(!activeTimeout){
			activeTimeout = setTimeout(logRejected, unhandledWait);
		}
	}

	function logRejected(){
		var now = new Date().getTime();
		var reportBefore = now - unhandledWait;
		errors = arrayUtil.filter(errors, function(obj){
			if(obj.timestamp < reportBefore){
				logError(obj.error, obj.rejection, obj.deferred);
				return false;
			}
			return true;
		});

		if(errors.length){
			activeTimeout = setTimeout(logRejected, errors[0].timestamp + unhandledWait - now);
		}else{
			activeTimeout = false;
		}
	}

	return function(Deferred){
		// summary:
		//		Initialize instrumentation for the Deferred class.
		// description:
		//		Initialize instrumentation for the Deferred class.
		//		Done automatically by `dojo/Deferred` if the
		//		`deferredInstrumentation` and `useDeferredInstrumentation`
		//		config options are set.
		//
		//		Sets up `dojo/promise/tracer` to log to the console.
		//
		//		Sets up instrumentation of rejected deferreds so unhandled
		//		errors are logged to the console.

		var usage = has("config-useDeferredInstrumentation");
		if(usage){
			tracer.on("resolved", lang.hitch(console, "log", "resolved"));
			tracer.on("rejected", lang.hitch(console, "log", "rejected"));
			tracer.on("progress", lang.hitch(console, "log", "progress"));

			var args = [];
			if(typeof usage === "string"){
				args = usage.split(",");
				usage = args.shift();
			}
			if(usage === "report-rejections"){
				Deferred.instrumentRejected = reportRejections;
			}else if(usage === "report-unhandled-rejections" || usage === true || usage === 1){
				Deferred.instrumentRejected = trackUnhandledRejections;
				unhandledWait = parseInt(args[0], 10) || unhandledWait;
			}else{
				throw new Error("Unsupported instrumentation usage <" + usage + ">");
			}
		}
	};
});

},
'dojo/promise/tracer':function(){
define([
	"../_base/lang",
	"./Promise",
	"../Evented"
], function(lang, Promise, Evented){
	"use strict";

	// module:
	//		dojo/promise/tracer

	/*=====
	return {
		// summary:
		//		Trace promise fulfillment.
		// description:
		//		Trace promise fulfillment. Calling `.trace()` or `.traceError()` on a
		//		promise enables tracing. Will emit `resolved`, `rejected` or `progress`
		//		events.

		on: function(type, listener){
			// summary:
			//		Subscribe to traces.
			// description:
			//		See `dojo/Evented#on()`.
			// type: String
			//		`resolved`, `rejected`, or `progress`
			// listener: Function
			//		The listener is passed the traced value and any arguments
			//		that were used with the `.trace()` call.
		}
	};
	=====*/

	var evented = new Evented;
	var emit = evented.emit;
	evented.emit = null;
	// Emit events asynchronously since they should not change the promise state.
	function emitAsync(args){
		setTimeout(function(){
			emit.apply(evented, args);
		}, 0);
	}

	Promise.prototype.trace = function(){
		// summary:
		//		Trace the promise.
		// description:
		//		Tracing allows you to transparently log progress,
		//		resolution and rejection of promises, without affecting the
		//		promise itself. Any arguments passed to `trace()` are
		//		emitted in trace events. See `dojo/promise/tracer` on how
		//		to handle traces.
		// returns: dojo/promise/Promise
		//		The promise instance `trace()` is called on.

		var args = lang._toArray(arguments);
		this.then(
			function(value){ emitAsync(["resolved", value].concat(args)); },
			function(error){ emitAsync(["rejected", error].concat(args)); },
			function(update){ emitAsync(["progress", update].concat(args)); }
		);
		return this;
	};

	Promise.prototype.traceRejected = function(){
		// summary:
		//		Trace rejection of the promise.
		// description:
		//		Tracing allows you to transparently log progress,
		//		resolution and rejection of promises, without affecting the
		//		promise itself. Any arguments passed to `trace()` are
		//		emitted in trace events. See `dojo/promise/tracer` on how
		//		to handle traces.
		// returns: dojo/promise/Promise
		//		The promise instance `traceRejected()` is called on.

		var args = lang._toArray(arguments);
		this.otherwise(function(error){
			emitAsync(["rejected", error].concat(args));
		});
		return this;
	};

	return evented;
});

},
'dojo/when':function(){
define([
	"./Deferred",
	"./promise/Promise"
], function(Deferred, Promise){
	"use strict";

	// module:
	//		dojo/when

	return function when(valueOrPromise, callback, errback, progback){
		// summary:
		//		Transparently applies callbacks to values and/or promises.
		// description:
		//		Accepts promises but also transparently handles non-promises. If no
		//		callbacks are provided returns a promise, regardless of the initial
		//		value. Foreign promises are converted.
		//
		//		If callbacks are provided and the initial value is not a promise,
		//		the callback is executed immediately with no error handling. Returns
		//		a promise if the initial value is a promise, or the result of the
		//		callback otherwise.
		// valueOrPromise:
		//		Either a regular value or an object with a `then()` method that
		//		follows the Promises/A specification.
		// callback: Function?
		//		Callback to be invoked when the promise is resolved, or a non-promise
		//		is received.
		// errback: Function?
		//		Callback to be invoked when the promise is rejected.
		// progback: Function?
		//		Callback to be invoked when the promise emits a progress update.
		// returns: dojo/promise/Promise
		//		Promise, or if a callback is provided, the result of the callback.

		var receivedPromise = valueOrPromise && typeof valueOrPromise.then === "function";
		var nativePromise = receivedPromise && valueOrPromise instanceof Promise;

		if(!receivedPromise){
			if(arguments.length > 1){
				return callback ? callback(valueOrPromise) : valueOrPromise;
			}else{
				return new Deferred().resolve(valueOrPromise);
			}
		}else if(!nativePromise){
			var deferred = new Deferred(valueOrPromise.cancel);
			valueOrPromise.then(deferred.resolve, deferred.reject, deferred.progress);
			valueOrPromise = deferred.promise;
		}

		if(callback || errback || progback){
			return valueOrPromise.then(callback, errback, progback);
		}
		return valueOrPromise;
	};
});

},
'dojo/_base/json':function(){
define(["./kernel", "../json"], function(dojo, json){

// module:
//		dojo/_base/json

/*=====
return {
	// summary:
	//		This module defines the dojo JSON API.
};
=====*/

dojo.fromJson = function(/*String*/ js){
	// summary:
	//		Parses a JavaScript expression and returns a JavaScript value.
	// description:
	//		Throws for invalid JavaScript expressions. It does not use a strict JSON parser. It
	//		always delegates to eval(). The content passed to this method must therefore come
	//		from a trusted source.
	//		It is recommend that you use dojo/json's parse function for an
	//		implementation uses the (faster) native JSON parse when available.
	// js:
	//		a string literal of a JavaScript expression, for instance:
	//		`'{ "foo": [ "bar", 1, { "baz": "thud" } ] }'`

	return eval("(" + js + ")"); // Object
};

/*=====
dojo._escapeString = function(){
	// summary:
	//		Adds escape sequences for non-visual characters, double quote and
	//		backslash and surrounds with double quotes to form a valid string
	//		literal.
};
=====*/
dojo._escapeString = json.stringify; // just delegate to json.stringify

dojo.toJsonIndentStr = "\t";
dojo.toJson = function(/*Object*/ it, /*Boolean?*/ prettyPrint){
	// summary:
	//		Returns a [JSON](http://json.org) serialization of an object.
	// description:
	//		Returns a [JSON](http://json.org) serialization of an object.
	//		Note that this doesn't check for infinite recursion, so don't do that!
	//		It is recommend that you use dojo/json's stringify function for an lighter
	//		and faster implementation that matches the native JSON API and uses the
	//		native JSON serializer when available.
	// it:
	//		an object to be serialized. Objects may define their own
	//		serialization via a special "__json__" or "json" function
	//		property. If a specialized serializer has been defined, it will
	//		be used as a fallback.
	//		Note that in 1.6, toJson would serialize undefined, but this no longer supported
	//		since it is not supported by native JSON serializer.
	// prettyPrint:
	//		if true, we indent objects and arrays to make the output prettier.
	//		The variable `dojo.toJsonIndentStr` is used as the indent string --
	//		to use something other than the default (tab), change that variable
	//		before calling dojo.toJson().
	//		Note that if native JSON support is available, it will be used for serialization,
	//		and native implementations vary on the exact spacing used in pretty printing.
	// returns:
	//		A JSON string serialization of the passed-in object.
	// example:
	//		simple serialization of a trivial object
	//		|	var jsonStr = dojo.toJson({ howdy: "stranger!", isStrange: true });
	//		|	doh.is('{"howdy":"stranger!","isStrange":true}', jsonStr);
	// example:
	//		a custom serializer for an objects of a particular class:
	//		|	dojo.declare("Furby", null, {
	//		|		furbies: "are strange",
	//		|		furbyCount: 10,
	//		|		__json__: function(){
	//		|		},
	//		|	});

	// use dojo/json
	return json.stringify(it, function(key, value){
		if(value){
			var tf = value.__json__||value.json;
			if(typeof tf == "function"){
				return tf.call(value);
			}
		}
		return value;
	}, prettyPrint && dojo.toJsonIndentStr);	// String
};

return dojo;
});

},
'dojo/json':function(){
define(["./has"], function(has){
	"use strict";
	var hasJSON = typeof JSON != "undefined";
	has.add("json-parse", hasJSON); // all the parsers work fine
		// Firefox 3.5/Gecko 1.9 fails to use replacer in stringify properly https://bugzilla.mozilla.org/show_bug.cgi?id=509184
	has.add("json-stringify", hasJSON && JSON.stringify({a:0}, function(k,v){return v||1;}) == '{"a":1}');

	/*=====
	return {
		// summary:
		//		Functions to parse and serialize JSON

		parse: function(str, strict){
			// summary:
			//		Parses a [JSON](http://json.org) string to return a JavaScript object.
			// description:
			//		This function follows [native JSON API](https://developer.mozilla.org/en/JSON)
			//		Throws for invalid JSON strings. This delegates to eval() if native JSON
			//		support is not available. By default this will evaluate any valid JS expression.
			//		With the strict parameter set to true, the parser will ensure that only
			//		valid JSON strings are parsed (otherwise throwing an error). Without the strict
			//		parameter, the content passed to this method must come
			//		from a trusted source.
			// str:
			//		a string literal of a JSON item, for instance:
			//		`'{ "foo": [ "bar", 1, { "baz": "thud" } ] }'`
			// strict:
			//		When set to true, this will ensure that only valid, secure JSON is ever parsed.
			//		Make sure this is set to true for untrusted content. Note that on browsers/engines
			//		without native JSON support, setting this to true will run slower.
		},
		stringify: function(value, replacer, spacer){
			// summary:
			//		Returns a [JSON](http://json.org) serialization of an object.
			// description:
			//		Returns a [JSON](http://json.org) serialization of an object.
			//		This function follows [native JSON API](https://developer.mozilla.org/en/JSON)
			//		Note that this doesn't check for infinite recursion, so don't do that!
			// value:
			//		A value to be serialized.
			// replacer:
			//		A replacer function that is called for each value and can return a replacement
			// spacer:
			//		A spacer string to be used for pretty printing of JSON
			// example:
			//		simple serialization of a trivial object
			//	|	define(["dojo/json"], function(JSON){
			// 	|		var jsonStr = JSON.stringify({ howdy: "stranger!", isStrange: true });
			//	|		doh.is('{"howdy":"stranger!","isStrange":true}', jsonStr);
		}
	};
	=====*/

	if(has("json-stringify")){
		return JSON;
	}else{
		var escapeString = function(/*String*/str){
			// summary:
			//		Adds escape sequences for non-visual characters, double quote and
			//		backslash and surrounds with double quotes to form a valid string
			//		literal.
			return ('"' + str.replace(/(["\\])/g, '\\$1') + '"').
				replace(/[\f]/g, "\\f").replace(/[\b]/g, "\\b").replace(/[\n]/g, "\\n").
				replace(/[\t]/g, "\\t").replace(/[\r]/g, "\\r"); // string
		};
		return {
			parse: has("json-parse") ? JSON.parse : function(str, strict){
				if(strict && !/^([\s\[\{]*(?:"(?:\\.|[^"])*"|-?\d[\d\.]*(?:[Ee][+-]?\d+)?|null|true|false|)[\s\]\}]*(?:,|:|$))+$/.test(str)){
					throw new SyntaxError("Invalid characters in JSON");
				}
				return eval('(' + str + ')');
			},
			stringify: function(value, replacer, spacer){
				var undef;
				if(typeof replacer == "string"){
					spacer = replacer;
					replacer = null;
				}
				function stringify(it, indent, key){
					if(replacer){
						it = replacer(key, it);
					}
					var val, objtype = typeof it;
					if(objtype == "number"){
						return isFinite(it) ? it + "" : "null";
					}
					if(objtype == "boolean"){
						return it + "";
					}
					if(it === null){
						return "null";
					}
					if(typeof it == "string"){
						return escapeString(it);
					}
					if(objtype == "function" || objtype == "undefined"){
						return undef; // undefined
					}
					// short-circuit for objects that support "json" serialization
					// if they return "self" then just pass-through...
					if(typeof it.toJSON == "function"){
						return stringify(it.toJSON(key), indent, key);
					}
					if(it instanceof Date){
						return '"{FullYear}-{Month+}-{Date}T{Hours}:{Minutes}:{Seconds}Z"'.replace(/\{(\w+)(\+)?\}/g, function(t, prop, plus){
							var num = it["getUTC" + prop]() + (plus ? 1 : 0);
							return num < 10 ? "0" + num : num;
						});
					}
					if(it.valueOf() !== it){
						// primitive wrapper, try again unwrapped:
						return stringify(it.valueOf(), indent, key);
					}
					var nextIndent= spacer ? (indent + spacer) : "";
					/* we used to test for DOM nodes and throw, but FF serializes them as {}, so cross-browser consistency is probably not efficiently attainable */ 
				
					var sep = spacer ? " " : "";
					var newLine = spacer ? "\n" : "";
				
					// array
					if(it instanceof Array){
						var itl = it.length, res = [];
						for(key = 0; key < itl; key++){
							var obj = it[key];
							val = stringify(obj, nextIndent, key);
							if(typeof val != "string"){
								val = "null";
							}
							res.push(newLine + nextIndent + val);
						}
						return "[" + res.join(",") + newLine + indent + "]";
					}
					// generic object code path
					var output = [];
					for(key in it){
						var keyStr;
						if(it.hasOwnProperty(key)){
							if(typeof key == "number"){
								keyStr = '"' + key + '"';
							}else if(typeof key == "string"){
								keyStr = escapeString(key);
							}else{
								// skip non-string or number keys
								continue;
							}
							val = stringify(it[key], nextIndent, key);
							if(typeof val != "string"){
								// skip non-serializable values
								continue;
							}
							// At this point, the most non-IE browsers don't get in this branch 
							// (they have native JSON), so push is definitely the way to
							output.push(newLine + nextIndent + keyStr + ":" + sep + val);
						}
					}
					return "{" + output.join(",") + newLine + indent + "}"; // String
				}
				return stringify(value, "", "");
			}
		};
	}
});

},
'dojo/_base/Color':function(){
define(["./kernel", "./lang", "./array", "./config"], function(dojo, lang, ArrayUtil, config){

	var Color = dojo.Color = function(/*Array|String|Object*/ color){
		// summary:
		//		Takes a named string, hex string, array of rgb or rgba values,
		//		an object with r, g, b, and a properties, or another `Color` object
		//		and creates a new Color instance to work from.
		//
		// example:
		//		Work with a Color instance:
		//	 | var c = new Color();
		//	 | c.setColor([0,0,0]); // black
		//	 | var hex = c.toHex(); // #000000
		//
		// example:
		//		Work with a node's color:
		//	 | var color = dojo.style("someNode", "backgroundColor");
		//	 | var n = new Color(color);
		//	 | // adjust the color some
		//	 | n.r *= .5;
		//	 | console.log(n.toString()); // rgb(128, 255, 255);
		if(color){ this.setColor(color); }
	};

	// FIXME:
	// there's got to be a more space-efficient way to encode or discover
	// these!! Use hex?
	Color.named = {
		// summary:
		//		Dictionary list of all CSS named colors, by name. Values are 3-item arrays with corresponding RG and B values.
		"black":  [0,0,0],
		"silver": [192,192,192],
		"gray":	  [128,128,128],
		"white":  [255,255,255],
		"maroon": [128,0,0],
		"red":	  [255,0,0],
		"purple": [128,0,128],
		"fuchsia":[255,0,255],
		"green":  [0,128,0],
		"lime":	  [0,255,0],
		"olive":  [128,128,0],
		"yellow": [255,255,0],
		"navy":	  [0,0,128],
		"blue":	  [0,0,255],
		"teal":	  [0,128,128],
		"aqua":	  [0,255,255],
		"transparent": config.transparentColor || [0,0,0,0]
	};

	lang.extend(Color, {
		r: 255, g: 255, b: 255, a: 1,
		_set: function(r, g, b, a){
			var t = this; t.r = r; t.g = g; t.b = b; t.a = a;
		},
		setColor: function(/*Array|String|Object*/ color){
			// summary:
			//		Takes a named string, hex string, array of rgb or rgba values,
			//		an object with r, g, b, and a properties, or another `Color` object
			//		and sets this color instance to that value.
			//
			// example:
			//	|	var c = new Color(); // no color
			//	|	c.setColor("#ededed"); // greyish
			if(lang.isString(color)){
				Color.fromString(color, this);
			}else if(lang.isArray(color)){
				Color.fromArray(color, this);
			}else{
				this._set(color.r, color.g, color.b, color.a);
				if(!(color instanceof Color)){ this.sanitize(); }
			}
			return this;	// Color
		},
		sanitize: function(){
			// summary:
			//		Ensures the object has correct attributes
			// description:
			//		the default implementation does nothing, include dojo.colors to
			//		augment it with real checks
			return this;	// Color
		},
		toRgb: function(){
			// summary:
			//		Returns 3 component array of rgb values
			// example:
			//	|	var c = new Color("#000000");
			//	|	console.log(c.toRgb()); // [0,0,0]
			var t = this;
			return [t.r, t.g, t.b]; // Array
		},
		toRgba: function(){
			// summary:
			//		Returns a 4 component array of rgba values from the color
			//		represented by this object.
			var t = this;
			return [t.r, t.g, t.b, t.a];	// Array
		},
		toHex: function(){
			// summary:
			//		Returns a CSS color string in hexadecimal representation
			// example:
			//	|	console.log(new Color([0,0,0]).toHex()); // #000000
			var arr = ArrayUtil.map(["r", "g", "b"], function(x){
				var s = this[x].toString(16);
				return s.length < 2 ? "0" + s : s;
			}, this);
			return "#" + arr.join("");	// String
		},
		toCss: function(/*Boolean?*/ includeAlpha){
			// summary:
			//		Returns a css color string in rgb(a) representation
			// example:
			//	|	var c = new Color("#FFF").toCss();
			//	|	console.log(c); // rgb('255','255','255')
			var t = this, rgb = t.r + ", " + t.g + ", " + t.b;
			return (includeAlpha ? "rgba(" + rgb + ", " + t.a : "rgb(" + rgb) + ")";	// String
		},
		toString: function(){
			// summary:
			//		Returns a visual representation of the color
			return this.toCss(true); // String
		}
	});

	Color.blendColors = dojo.blendColors = function(
		/*Color*/ start,
		/*Color*/ end,
		/*Number*/ weight,
		/*Color?*/ obj
	){
		// summary:
		//		Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,
		//		can reuse a previously allocated Color object for the result
		var t = obj || new Color();
		ArrayUtil.forEach(["r", "g", "b", "a"], function(x){
			t[x] = start[x] + (end[x] - start[x]) * weight;
			if(x != "a"){ t[x] = Math.round(t[x]); }
		});
		return t.sanitize();	// Color
	};

	Color.fromRgb = dojo.colorFromRgb = function(/*String*/ color, /*Color?*/ obj){
		// summary:
		//		Returns a `Color` instance from a string of the form
		//		"rgb(...)" or "rgba(...)". Optionally accepts a `Color`
		//		object to update with the parsed value and return instead of
		//		creating a new object.
		// returns:
		//		A Color object. If obj is passed, it will be the return value.
		var m = color.toLowerCase().match(/^rgba?\(([\s\.,0-9]+)\)/);
		return m && Color.fromArray(m[1].split(/\s*,\s*/), obj);	// Color
	};

	Color.fromHex = dojo.colorFromHex = function(/*String*/ color, /*Color?*/ obj){
		// summary:
		//		Converts a hex string with a '#' prefix to a color object.
		//		Supports 12-bit #rgb shorthand. Optionally accepts a
		//		`Color` object to update with the parsed value.
		//
		// returns:
		//		A Color object. If obj is passed, it will be the return value.
		//
		// example:
		//	 | var thing = dojo.colorFromHex("#ededed"); // grey, longhand
		//
		// example:
		//	| var thing = dojo.colorFromHex("#000"); // black, shorthand
		var t = obj || new Color(),
			bits = (color.length == 4) ? 4 : 8,
			mask = (1 << bits) - 1;
		color = Number("0x" + color.substr(1));
		if(isNaN(color)){
			return null; // Color
		}
		ArrayUtil.forEach(["b", "g", "r"], function(x){
			var c = color & mask;
			color >>= bits;
			t[x] = bits == 4 ? 17 * c : c;
		});
		t.a = 1;
		return t;	// Color
	};

	Color.fromArray = dojo.colorFromArray = function(/*Array*/ a, /*Color?*/ obj){
		// summary:
		//		Builds a `Color` from a 3 or 4 element array, mapping each
		//		element in sequence to the rgb(a) values of the color.
		// example:
		//		| var myColor = dojo.colorFromArray([237,237,237,0.5]); // grey, 50% alpha
		// returns:
		//		A Color object. If obj is passed, it will be the return value.
		var t = obj || new Color();
		t._set(Number(a[0]), Number(a[1]), Number(a[2]), Number(a[3]));
		if(isNaN(t.a)){ t.a = 1; }
		return t.sanitize();	// Color
	};

	Color.fromString = dojo.colorFromString = function(/*String*/ str, /*Color?*/ obj){
		// summary:
		//		Parses `str` for a color value. Accepts hex, rgb, and rgba
		//		style color values.
		// description:
		//		Acceptable input values for str may include arrays of any form
		//		accepted by dojo.colorFromArray, hex strings such as "#aaaaaa", or
		//		rgb or rgba strings such as "rgb(133, 200, 16)" or "rgba(10, 10,
		//		10, 50)"
		// returns:
		//		A Color object. If obj is passed, it will be the return value.
		var a = Color.named[str];
		return a && Color.fromArray(a, obj) || Color.fromRgb(str, obj) || Color.fromHex(str, obj);	// Color
	};

	return Color;
});

},
'dojo/_base/browser':function(){
if(require.has){
	require.has.add("config-selectorEngine", "acme");
}
define([
	"../ready",
	"./kernel",
	"./connect", // until we decide if connect is going back into non-browser environments
	"./unload",
	"./window",
	"./event",
	"./html",
	"./NodeList",
	"../query",
	"./xhr",
	"./fx"], function(dojo){

	// module:
	//		dojo/_base/browser

	/*=====
	return {
		// summary:
		//		This module causes the browser-only base modules to be loaded.
	};
	=====*/

	return dojo;
});

},
'dojo/_base/unload':function(){
define(["./kernel", "./lang", "../on"], function(dojo, lang, on){

// module:
//		dojo/unload

var win = window;

var unload = {
	// summary:
	//		This module contains the document and window unload detection API.

	addOnWindowUnload: function(/*Object|Function?*/ obj, /*String|Function?*/ functionName){
		// summary:
		//		registers a function to be triggered when window.onunload
		//		fires.
		// description:
		//		The first time that addOnWindowUnload is called Dojo
		//		will register a page listener to trigger your unload
		//		handler with. Note that registering these handlers may
		//		destroy "fastback" page caching in browsers that support
		//		it. Be careful trying to modify the DOM or access
		//		JavaScript properties during this phase of page unloading:
		//		they may not always be available. Consider
		//		addOnUnload() if you need to modify the DOM or do
		//		heavy JavaScript work since it fires at the equivalent of
		//		the page's "onbeforeunload" event.
		// example:
		//	|	unload.addOnWindowUnload(functionPointer)
		//	|	unload.addOnWindowUnload(object, "functionName");
		//	|	unload.addOnWindowUnload(object, function(){ /* ... */});

		if (!dojo.windowUnloaded){
			on(win, "unload", (dojo.windowUnloaded = function(){
				// summary:
				//		signal fired by impending window destruction. You may use
				//		dojo.addOnWindowUnload() to register a listener for this
				//		event. NOTE: if you wish to dojo.connect() to this method
				//		to perform page/application cleanup, be aware that this
				//		event WILL NOT fire if no handler has been registered with
				//		addOnWindowUnload(). This behavior started in Dojo 1.3.
				//		Previous versions always triggered windowUnloaded(). See
				//		addOnWindowUnload for more info.
			}));
		}
		on(win, "unload", lang.hitch(obj, functionName));
	},

	addOnUnload: function(/*Object?|Function?*/ obj, /*String|Function?*/ functionName){
		// summary:
		//		registers a function to be triggered when the page unloads.
		// description:
		//		The first time that addOnUnload is called Dojo will
		//		register a page listener to trigger your unload handler
		//		with.
		//
		//		In a browser environment, the functions will be triggered
		//		during the window.onbeforeunload event. Be careful of doing
		//		too much work in an unload handler. onbeforeunload can be
		//		triggered if a link to download a file is clicked, or if
		//		the link is a javascript: link. In these cases, the
		//		onbeforeunload event fires, but the document is not
		//		actually destroyed. So be careful about doing destructive
		//		operations in a dojo.addOnUnload callback.
		//
		//		Further note that calling dojo.addOnUnload will prevent
		//		browsers from using a "fast back" cache to make page
		//		loading via back button instantaneous.
		// example:
		//	|	dojo.addOnUnload(functionPointer)
		//	|	dojo.addOnUnload(object, "functionName")
		//	|	dojo.addOnUnload(object, function(){ /* ... */});

		on(win, "beforeunload", lang.hitch(obj, functionName));
	}
};

dojo.addOnWindowUnload = unload.addOnWindowUnload;
dojo.addOnUnload = unload.addOnUnload;

return unload;

});

},
'dojo/_base/html':function(){
define(["./kernel", "../dom", "../dom-style", "../dom-attr", "../dom-prop", "../dom-class", "../dom-construct", "../dom-geometry"], function(dojo, dom, style, attr, prop, cls, ctr, geom){
	// module:
	//		dojo/dom

	/*=====
	return {
		// summary:
		//		This module is a stub for the core dojo DOM API.
	};
	=====*/

	// mix-in dom
	dojo.byId = dom.byId;
	dojo.isDescendant = dom.isDescendant;
	dojo.setSelectable = dom.setSelectable;

	// mix-in dom-attr
	dojo.getAttr = attr.get;
	dojo.setAttr = attr.set;
	dojo.hasAttr = attr.has;
	dojo.removeAttr = attr.remove;
	dojo.getNodeProp = attr.getNodeProp;

	dojo.attr = function(node, name, value){
		// summary:
		//		Gets or sets an attribute on an HTML element.
		// description:
		//		Handles normalized getting and setting of attributes on DOM
		//		Nodes. If 2 arguments are passed, and a the second argument is a
		//		string, acts as a getter.
		//
		//		If a third argument is passed, or if the second argument is a
		//		map of attributes, acts as a setter.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to get or set the attribute on
		// name: String|Object
		//		the name of the attribute to get or set.
		// value: String?
		//		The value to set for the attribute
		// returns:
		//		when used as a getter, the value of the requested attribute
		//		or null if that attribute does not have a specified or
		//		default value;
		//
		//		when used as a setter, the DOM node
		//
		// example:
		//	|	// get the current value of the "foo" attribute on a node
		//	|	dojo.attr(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.attr("nodeId", "foo");
		//
		// example:
		//	|	// use attr() to set the tab index
		//	|	dojo.attr("nodeId", "tabIndex", 3);
		//	|
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.attr("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.attr("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.attr("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);

		if(arguments.length == 2){
			return attr[typeof name == "string" ? "get" : "set"](node, name);
		}
		return attr.set(node, name, value);
	};

	// mix-in dom-class
	dojo.hasClass = cls.contains;
	dojo.addClass = cls.add;
	dojo.removeClass = cls.remove;
	dojo.toggleClass = cls.toggle;
	dojo.replaceClass = cls.replace;

	// mix-in dom-construct
	dojo._toDom = dojo.toDom = ctr.toDom;
	dojo.place = ctr.place;
	dojo.create = ctr.create;
	dojo.empty = function(node){ ctr.empty(node); };
	dojo._destroyElement = dojo.destroy = function(node){ ctr.destroy(node); };

	// mix-in dom-geometry
	dojo._getPadExtents = dojo.getPadExtents = geom.getPadExtents;
	dojo._getBorderExtents = dojo.getBorderExtents = geom.getBorderExtents;
	dojo._getPadBorderExtents = dojo.getPadBorderExtents = geom.getPadBorderExtents;
	dojo._getMarginExtents = dojo.getMarginExtents = geom.getMarginExtents;
	dojo._getMarginSize = dojo.getMarginSize = geom.getMarginSize;
	dojo._getMarginBox = dojo.getMarginBox = geom.getMarginBox;
	dojo.setMarginBox = geom.setMarginBox;
	dojo._getContentBox = dojo.getContentBox = geom.getContentBox;
	dojo.setContentSize = geom.setContentSize;
	dojo._isBodyLtr = dojo.isBodyLtr = geom.isBodyLtr;
	dojo._docScroll = dojo.docScroll = geom.docScroll;
	dojo._getIeDocumentElementOffset = dojo.getIeDocumentElementOffset = geom.getIeDocumentElementOffset;
	dojo._fixIeBiDiScrollLeft = dojo.fixIeBiDiScrollLeft = geom.fixIeBiDiScrollLeft;
	dojo.position = geom.position;

	dojo.marginBox = function marginBox(/*DomNode|String*/node, /*Object?*/box){
		// summary:
		//		Getter/setter for the margin-box of node.
		// description:
		//		Getter/setter for the margin-box of node.
		//		Returns an object in the expected format of box (regardless
		//		if box is passed). The object might look like:
		//		`{ l: 50, t: 200, w: 300: h: 150 }`
		//		for a node offset from its parent 50px to the left, 200px from
		//		the top with a margin width of 300px and a margin-height of
		//		150px.
		// node:
		//		id or reference to DOM Node to get/set box for
		// box:
		//		If passed, denotes that dojo.marginBox() should
		//		update/set the margin box for node. Box is an object in the
		//		above format. All properties are optional if passed.
		// example:
		//		Retrieve the margin box of a passed node
		//	|	var box = dojo.marginBox("someNodeId");
		//	|	console.dir(box);
		//
		// example:
		//		Set a node's margin box to the size of another node
		//	|	var box = dojo.marginBox("someNodeId");
		//	|	dojo.marginBox("someOtherNode", box);
		return box ? geom.setMarginBox(node, box) : geom.getMarginBox(node); // Object
	};

	dojo.contentBox = function contentBox(/*DomNode|String*/node, /*Object?*/box){
		// summary:
		//		Getter/setter for the content-box of node.
		// description:
		//		Returns an object in the expected format of box (regardless if box is passed).
		//		The object might look like:
		//		`{ l: 50, t: 200, w: 300: h: 150 }`
		//		for a node offset from its parent 50px to the left, 200px from
		//		the top with a content width of 300px and a content-height of
		//		150px. Note that the content box may have a much larger border
		//		or margin box, depending on the box model currently in use and
		//		CSS values set/inherited for node.
		//		While the getter will return top and left values, the
		//		setter only accepts setting the width and height.
		// node:
		//		id or reference to DOM Node to get/set box for
		// box:
		//		If passed, denotes that dojo.contentBox() should
		//		update/set the content box for node. Box is an object in the
		//		above format, but only w (width) and h (height) are supported.
		//		All properties are optional if passed.
		return box ? geom.setContentSize(node, box) : geom.getContentBox(node); // Object
	};

	dojo.coords = function(/*DomNode|String*/node, /*Boolean?*/includeScroll){
		// summary:
		//		Deprecated: Use position() for border-box x/y/w/h
		//		or marginBox() for margin-box w/h/l/t.
		//
		//		Returns an object that measures margin-box (w)idth/(h)eight
		//		and absolute position x/y of the border-box. Also returned
		//		is computed (l)eft and (t)op values in pixels from the
		//		node's offsetParent as returned from marginBox().
		//		Return value will be in the form:
		//|			{ l: 50, t: 200, w: 300: h: 150, x: 100, y: 300 }
		//		Does not act as a setter. If includeScroll is passed, the x and
		//		y params are affected as one would expect in dojo.position().
		dojo.deprecated("dojo.coords()", "Use dojo.position() or dojo.marginBox().");
		node = dom.byId(node);
		var s = style.getComputedStyle(node), mb = geom.getMarginBox(node, s);
		var abs = geom.position(node, includeScroll);
		mb.x = abs.x;
		mb.y = abs.y;
		return mb;	// Object
	};

	// mix-in dom-prop
	dojo.getProp = prop.get;
	dojo.setProp = prop.set;

	dojo.prop = function(/*DomNode|String*/node, /*String|Object*/name, /*String?*/value){
		// summary:
		//		Gets or sets a property on an HTML element.
		// description:
		//		Handles normalized getting and setting of properties on DOM
		//		Nodes. If 2 arguments are passed, and a the second argument is a
		//		string, acts as a getter.
		//
		//		If a third argument is passed, or if the second argument is a
		//		map of attributes, acts as a setter.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node:
		//		id or reference to the element to get or set the property on
		// name:
		//		the name of the property to get or set.
		// value:
		//		The value to set for the property
		// returns:
		//		when used as a getter, the value of the requested property
		//		or null if that attribute does not have a specified or
		//		default value;
		//
		//		when used as a setter, the DOM node
		//
		// example:
		//	|	// get the current value of the "foo" property on a node
		//	|	dojo.prop(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.prop("nodeId", "foo");
		//
		// example:
		//	|	// use prop() to set the tab index
		//	|	dojo.prop("nodeId", "tabIndex", 3);
		//	|
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.prop("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//		Style is s special case: Only set with an object hash of styles
		//	|	dojo.prop("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//		Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.prop("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);

		if(arguments.length == 2){
			return prop[typeof name == "string" ? "get" : "set"](node, name);
		}
		// setter
		return prop.set(node, name, value);
	};

	// mix-in dom-style
	dojo.getStyle = style.get;
	dojo.setStyle = style.set;
	dojo.getComputedStyle = style.getComputedStyle;
	dojo.__toPixelValue = dojo.toPixelValue = style.toPixelValue;

	dojo.style = function(node, name, value){
		// summary:
		//		Accesses styles on a node. If 2 arguments are
		//		passed, acts as a getter. If 3 arguments are passed, acts
		//		as a setter.
		// description:
		//		Getting the style value uses the computed style for the node, so the value
		//		will be a calculated value, not just the immediate node.style value.
		//		Also when getting values, use specific style names,
		//		like "borderBottomWidth" instead of "border" since compound values like
		//		"border" are not necessarily reflected as expected.
		//		If you want to get node dimensions, use `dojo.marginBox()`,
		//		`dojo.contentBox()` or `dojo.position()`.
		// node: DOMNode|String
		//		id or reference to node to get/set style for
		// name: String|Object?
		//		the style property to set in DOM-accessor format
		//		("borderWidth", not "border-width") or an object with key/value
		//		pairs suitable for setting each property.
		// value: String?
		//		If passed, sets value on the node for style, handling
		//		cross-browser concerns.  When setting a pixel value,
		//		be sure to include "px" in the value. For instance, top: "200px".
		//		Otherwise, in some cases, some browsers will not apply the style.
		// returns:
		//		when used as a getter, return the computed style of the node if passing in an ID or node,
		//		or return the normalized, computed value for the property when passing in a node and a style property
		// example:
		//		Passing only an ID or node returns the computed style object of
		//		the node:
		//	|	dojo.style("thinger");
		// example:
		//		Passing a node and a style property returns the current
		//		normalized, computed value for that property:
		//	|	dojo.style("thinger", "opacity"); // 1 by default
		//
		// example:
		//		Passing a node, a style property, and a value changes the
		//		current display of the node and returns the new computed value
		//	|	dojo.style("thinger", "opacity", 0.5); // == 0.5
		//
		// example:
		//		Passing a node, an object-style style property sets each of the values in turn and returns the computed style object of the node:
		//	|	dojo.style("thinger", {
		//	|		"opacity": 0.5,
		//	|		"border": "3px solid black",
		//	|		"height": "300px"
		//	|	});
		//
		// example:
		//		When the CSS style property is hyphenated, the JavaScript property is camelCased.
		//		font-size becomes fontSize, and so on.
		//	|	dojo.style("thinger",{
		//	|		fontSize:"14pt",
		//	|		letterSpacing:"1.2em"
		//	|	});
		//
		// example:
		//		dojo/NodeList implements .style() using the same syntax, omitting the "node" parameter, calling
		//		dojo.style() on every element of the list. See: `dojo/query` and `dojo/NodeList`
		//	|	dojo.query(".someClassName").style("visibility","hidden");
		//	|	// or
		//	|	dojo.query("#baz > div").style({
		//	|		opacity:0.75,
		//	|		fontSize:"13pt"
		//	|	});

		switch(arguments.length){
			case 1:
				return style.get(node);
			case 2:
				return style[typeof name == "string" ? "get" : "set"](node, name);
		}
		// setter
		return style.set(node, name, value);
	};

	return dojo;
});

},
'dojo/dom-attr':function(){
define(["exports", "./sniff", "./_base/lang", "./dom", "./dom-style", "./dom-prop"],
		function(exports, has, lang, dom, style, prop){
	// module:
	//		dojo/dom-attr
	// summary:
	//		This module defines the core dojo DOM attributes API.

	// TODOC: summary not showing up in output see https://github.com/csnover/js-doc-parse/issues/42

	// =============================
	// Element attribute Functions
	// =============================

	// This module will be obsolete soon. Use dojo/prop instead.

	// dojo.attr() should conform to http://www.w3.org/TR/DOM-Level-2-Core/

	// attribute-related functions (to be obsolete soon)

	var forcePropNames = {
			innerHTML:	1,
			className:	1,
			htmlFor:	has("ie"),
			value:		1
		},
		attrNames = {
			// original attribute names
			classname: "class",
			htmlfor: "for",
			// for IE
			tabindex: "tabIndex",
			readonly: "readOnly"
		};

	function _hasAttr(node, name){
		var attr = node.getAttributeNode && node.getAttributeNode(name);
		return attr && attr.specified; // Boolean
	}

	// There is a difference in the presence of certain properties and their default values
	// between browsers. For example, on IE "disabled" is present on all elements,
	// but it is value is "false"; "tabIndex" of <div> returns 0 by default on IE, yet other browsers
	// can return -1.

	exports.has = function hasAttr(/*DOMNode|String*/ node, /*String*/ name){
		// summary:
		//		Returns true if the requested attribute is specified on the
		//		given element, and false otherwise.
		// node: DOMNode|String
		//		id or reference to the element to check
		// name: String
		//		the name of the attribute
		// returns: Boolean
		//		true if the requested attribute is specified on the
		//		given element, and false otherwise

		var lc = name.toLowerCase();
		return forcePropNames[prop.names[lc] || name] || _hasAttr(dom.byId(node), attrNames[lc] || name);	// Boolean
	};

	exports.get = function getAttr(/*DOMNode|String*/ node, /*String*/ name){
		// summary:
		//		Gets an attribute on an HTML element.
		// description:
		//		Handles normalized getting of attributes on DOM Nodes.
		// node: DOMNode|String
		//		id or reference to the element to get the attribute on
		// name: String
		//		the name of the attribute to get.
		// returns:
		//		the value of the requested attribute or null if that attribute does not have a specified or
		//		default value;
		//
		// example:
		//	|	// get the current value of the "foo" attribute on a node
		//	|	dojo.getAttr(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.getAttr("nodeId", "foo");

		node = dom.byId(node);
		var lc = name.toLowerCase(),
			propName = prop.names[lc] || name,
			forceProp = forcePropNames[propName],
			value = node[propName];		// should we access this attribute via a property or via getAttribute()?

		if(forceProp && typeof value != "undefined"){
			// node's property
			return value;	// Anything
		}
		if(propName != "href" && (typeof value == "boolean" || lang.isFunction(value))){
			// node's property
			return value;	// Anything
		}
		// node's attribute
		// we need _hasAttr() here to guard against IE returning a default value
		var attrName = attrNames[lc] || name;
		return _hasAttr(node, attrName) ? node.getAttribute(attrName) : null; // Anything
	};

	exports.set = function setAttr(/*DOMNode|String*/ node, /*String|Object*/ name, /*String?*/ value){
		// summary:
		//		Sets an attribute on an HTML element.
		// description:
		//		Handles normalized setting of attributes on DOM Nodes.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to set the attribute on
		// name: String|Object
		//		the name of the attribute to set, or a hash of key-value pairs to set.
		// value: String?
		//		the value to set for the attribute, if the name is a string.
		// returns:
		//		the DOM node
		//
		// example:
		//	|	// use attr() to set the tab index
		//	|	dojo.setAttr("nodeId", "tabIndex", 3);
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.setAttr("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.setAttr("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.setAttr("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.setStyle("someNode", obj);

		node = dom.byId(node);
		if(arguments.length == 2){ // inline'd type check
			// the object form of setter: the 2nd argument is a dictionary
			for(var x in name){
				exports.set(node, x, name[x]);
			}
			return node; // DomNode
		}
		var lc = name.toLowerCase(),
			propName = prop.names[lc] || name,
			forceProp = forcePropNames[propName];
		if(propName == "style" && typeof value != "string"){ // inline'd type check
			// special case: setting a style
			style.set(node, value);
			return node; // DomNode
		}
		if(forceProp || typeof value == "boolean" || lang.isFunction(value)){
			return prop.set(node, name, value);
		}
		// node's attribute
		node.setAttribute(attrNames[lc] || name, value);
		return node; // DomNode
	};

	exports.remove = function removeAttr(/*DOMNode|String*/ node, /*String*/ name){
		// summary:
		//		Removes an attribute from an HTML element.
		// node: DOMNode|String
		//		id or reference to the element to remove the attribute from
		// name: String
		//		the name of the attribute to remove

		dom.byId(node).removeAttribute(attrNames[name.toLowerCase()] || name);
	};

	exports.getNodeProp = function getNodeProp(/*DomNode|String*/ node, /*String*/ name){
		// summary:
		//		Returns an effective value of a property or an attribute.
		// node: DOMNode|String
		//		id or reference to the element to remove the attribute from
		// name: String
		//		the name of the attribute
		// returns:
		//		the value of the attribute

		node = dom.byId(node);
		var lc = name.toLowerCase(), propName = prop.names[lc] || name;
		if((propName in node) && propName != "href"){
			// node's property
			return node[propName];	// Anything
		}
		// node's attribute
		var attrName = attrNames[lc] || name;
		return _hasAttr(node, attrName) ? node.getAttribute(attrName) : null; // Anything
	};
});

},
'dojo/dom-prop':function(){
define(["exports", "./_base/kernel", "./sniff", "./_base/lang", "./dom", "./dom-style", "./dom-construct", "./_base/connect"],
		function(exports, dojo, has, lang, dom, style, ctr, conn){
	// module:
	//		dojo/dom-prop
	// summary:
	//		This module defines the core dojo DOM properties API.

	// TODOC: summary not showing up in output, see https://github.com/csnover/js-doc-parse/issues/42

	// =============================
	// Element properties Functions
	// =============================

	// helper to connect events
	var _evtHdlrMap = {}, _ctr = 0, _attrId = dojo._scopeName + "attrid";

	exports.names = {
		// properties renamed to avoid clashes with reserved words
		"class": "className",
		"for": "htmlFor",
		// properties written as camelCase
		tabindex: "tabIndex",
		readonly: "readOnly",
		colspan: "colSpan",
		frameborder: "frameBorder",
		rowspan: "rowSpan",
		valuetype: "valueType"
	};

	exports.get = function getProp(/*DOMNode|String*/ node, /*String*/ name){
		// summary:
		//		Gets a property on an HTML element.
		// description:
		//		Handles normalized getting of properties on DOM nodes.
		//
		// node: DOMNode|String
		//		id or reference to the element to get the property on
		// name: String
		//		the name of the property to get.
		// returns:
		//		the value of the requested property or its default value
		//
		// example:
		//	|	// get the current value of the "foo" property on a node
		//	|	dojo.getProp(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.getProp("nodeId", "foo");

		node = dom.byId(node);
		var lc = name.toLowerCase(), propName = exports.names[lc] || name;
		return node[propName];	// Anything
	};

	exports.set = function setProp(/*DOMNode|String*/ node, /*String|Object*/ name, /*String?*/ value){
		// summary:
		//		Sets a property on an HTML element.
		// description:
		//		Handles normalized setting of properties on DOM nodes.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		// node: DOMNode|String
		//		id or reference to the element to set the property on
		// name: String|Object
		//		the name of the property to set, or a hash object to set
		//		multiple properties at once.
		// value: String?
		//		The value to set for the property
		// returns:
		//		the DOM node
		//
		// example:
		//	|	// use prop() to set the tab index
		//	|	dojo.setProp("nodeId", "tabIndex", 3);
		//	|
		//
		// example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.setProp("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		// example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.setProp("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		// example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.setProp("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);

		node = dom.byId(node);
		var l = arguments.length;
		if(l == 2 && typeof name != "string"){ // inline'd type check
			// the object form of setter: the 2nd argument is a dictionary
			for(var x in name){
				exports.set(node, x, name[x]);
			}
			return node; // DomNode
		}
		var lc = name.toLowerCase(), propName = exports.names[lc] || name;
		if(propName == "style" && typeof value != "string"){ // inline'd type check
			// special case: setting a style
			style.set(node, value);
			return node; // DomNode
		}
		if(propName == "innerHTML"){
			// special case: assigning HTML
			// the hash lists elements with read-only innerHTML on IE
			if(has("ie") && node.tagName.toLowerCase() in {col: 1, colgroup: 1,
						table: 1, tbody: 1, tfoot: 1, thead: 1, tr: 1, title: 1}){
				ctr.empty(node);
				node.appendChild(ctr.toDom(value, node.ownerDocument));
			}else{
				node[propName] = value;
			}
			return node; // DomNode
		}
		if(lang.isFunction(value)){
			// special case: assigning an event handler
			// clobber if we can
			var attrId = node[_attrId];
			if(!attrId){
				attrId = _ctr++;
				node[_attrId] = attrId;
			}
			if(!_evtHdlrMap[attrId]){
				_evtHdlrMap[attrId] = {};
			}
			var h = _evtHdlrMap[attrId][propName];
			if(h){
				//h.remove();
				conn.disconnect(h);
			}else{
				try{
					delete node[propName];
				}catch(e){}
			}
			// ensure that event objects are normalized, etc.
			if(value){
				//_evtHdlrMap[attrId][propName] = on(node, propName, value);
				_evtHdlrMap[attrId][propName] = conn.connect(node, propName, value);
			}else{
				node[propName] = null;
			}
			return node; // DomNode
		}
		node[propName] = value;
		return node;	// DomNode
	};
});

},
'dojo/dom-construct':function(){
define(["exports", "./_base/kernel", "./sniff", "./_base/window", "./dom", "./dom-attr"],
		function(exports, dojo, has, win, dom, attr){
	// module:
	//		dojo/dom-construct
	// summary:
	//		This module defines the core dojo DOM construction API.

	// TODOC: summary not showing up in output, see https://github.com/csnover/js-doc-parse/issues/42

	// support stuff for toDom()
	var tagWrap = {
			option: ["select"],
			tbody: ["table"],
			thead: ["table"],
			tfoot: ["table"],
			tr: ["table", "tbody"],
			td: ["table", "tbody", "tr"],
			th: ["table", "thead", "tr"],
			legend: ["fieldset"],
			caption: ["table"],
			colgroup: ["table"],
			col: ["table", "colgroup"],
			li: ["ul"]
		},
		reTag = /<\s*([\w\:]+)/,
		masterNode = {}, masterNum = 0,
		masterName = "__" + dojo._scopeName + "ToDomId";

	// generate start/end tag strings to use
	// for the injection for each special tag wrap case.
	for(var param in tagWrap){
		if(tagWrap.hasOwnProperty(param)){
			var tw = tagWrap[param];
			tw.pre = param == "option" ? '<select multiple="multiple">' : "<" + tw.join("><") + ">";
			tw.post = "</" + tw.reverse().join("></") + ">";
			// the last line is destructive: it reverses the array,
			// but we don't care at this point
		}
	}

	var html5domfix;
	if(has("ie") <= 8){
		html5domfix = function(doc){
			doc.__dojo_html5_tested = "yes";
			var div = create('div', {innerHTML: "<nav>a</nav>", style: {visibility: "hidden"}}, doc.body);
			if(div.childNodes.length !== 1){
				('abbr article aside audio canvas details figcaption figure footer header ' +
				'hgroup mark meter nav output progress section summary time video').replace(
					/\b\w+\b/g, function(n){
						doc.createElement(n);
					}
				);
			}
			destroy(div);
		}
	}

	function _insertBefore(/*DomNode*/ node, /*DomNode*/ ref){
		var parent = ref.parentNode;
		if(parent){
			parent.insertBefore(node, ref);
		}
	}

	function _insertAfter(/*DomNode*/ node, /*DomNode*/ ref){
		// summary:
		//		Try to insert node after ref
		var parent = ref.parentNode;
		if(parent){
			if(parent.lastChild == ref){
				parent.appendChild(node);
			}else{
				parent.insertBefore(node, ref.nextSibling);
			}
		}
	}

	exports.toDom = function toDom(frag, doc){
		// summary:
		//		instantiates an HTML fragment returning the corresponding DOM.
		// frag: String
		//		the HTML fragment
		// doc: DocumentNode?
		//		optional document to use when creating DOM nodes, defaults to
		//		dojo.doc if not specified.
		// returns:
		//		Document fragment, unless it's a single node in which case it returns the node itself
		// example:
		//		Create a table row:
		//	|	var tr = dojo.toDom("<tr><td>First!</td></tr>");

		doc = doc || win.doc;
		var masterId = doc[masterName];
		if(!masterId){
			doc[masterName] = masterId = ++masterNum + "";
			masterNode[masterId] = doc.createElement("div");
		}

		if(has("ie") <= 8){
			if(!doc.__dojo_html5_tested && doc.body){
				html5domfix(doc);
			}
		}

		// make sure the frag is a string.
		frag += "";

		// find the starting tag, and get node wrapper
		var match = frag.match(reTag),
			tag = match ? match[1].toLowerCase() : "",
			master = masterNode[masterId],
			wrap, i, fc, df;
		if(match && tagWrap[tag]){
			wrap = tagWrap[tag];
			master.innerHTML = wrap.pre + frag + wrap.post;
			for(i = wrap.length; i; --i){
				master = master.firstChild;
			}
		}else{
			master.innerHTML = frag;
		}

		// one node shortcut => return the node itself
		if(master.childNodes.length == 1){
			return master.removeChild(master.firstChild); // DOMNode
		}

		// return multiple nodes as a document fragment
		df = doc.createDocumentFragment();
		while((fc = master.firstChild)){ // intentional assignment
			df.appendChild(fc);
		}
		return df; // DocumentFragment
	};

	exports.place = function place(/*DOMNode|String*/ node, /*DOMNode|String*/ refNode, /*String|Number?*/ position){
		// summary:
		//		Attempt to insert node into the DOM, choosing from various positioning options.
		//		Returns the first argument resolved to a DOM node.
		// node: DOMNode|String
		//		id or node reference, or HTML fragment starting with "<" to place relative to refNode
		// refNode: DOMNode|String
		//		id or node reference to use as basis for placement
		// position: String|Number?
		//		string noting the position of node relative to refNode or a
		//		number indicating the location in the childNodes collection of refNode.
		//		Accepted string values are:
		//
		//		- before
		//		- after
		//		- replace
		//		- only
		//		- first
		//		- last
		//
		//		"first" and "last" indicate positions as children of refNode, "replace" replaces refNode,
		//		"only" replaces all children.  position defaults to "last" if not specified
		// returns: DOMNode
		//		Returned values is the first argument resolved to a DOM node.
		//
		//		.place() is also a method of `dojo/NodeList`, allowing `dojo.query` node lookups.
		// example:
		//		Place a node by string id as the last child of another node by string id:
		//	|	dojo.place("someNode", "anotherNode");
		// example:
		//		Place a node by string id before another node by string id
		//	|	dojo.place("someNode", "anotherNode", "before");
		// example:
		//		Create a Node, and place it in the body element (last child):
		//	|	dojo.place("<div></div>", dojo.body());
		// example:
		//		Put a new LI as the first child of a list by id:
		//	|	dojo.place("<li></li>", "someUl", "first");

		refNode = dom.byId(refNode);
		if(typeof node == "string"){ // inline'd type check
			node = /^\s*</.test(node) ? exports.toDom(node, refNode.ownerDocument) : dom.byId(node);
		}
		if(typeof position == "number"){ // inline'd type check
			var cn = refNode.childNodes;
			if(!cn.length || cn.length <= position){
				refNode.appendChild(node);
			}else{
				_insertBefore(node, cn[position < 0 ? 0 : position]);
			}
		}else{
			switch(position){
				case "before":
					_insertBefore(node, refNode);
					break;
				case "after":
					_insertAfter(node, refNode);
					break;
				case "replace":
					refNode.parentNode.replaceChild(node, refNode);
					break;
				case "only":
					exports.empty(refNode);
					refNode.appendChild(node);
					break;
				case "first":
					if(refNode.firstChild){
						_insertBefore(node, refNode.firstChild);
						break;
					}
					// else fallthrough...
				default: // aka: last
					refNode.appendChild(node);
			}
		}
		return node; // DomNode
	};

	var create = exports.create = function create(/*DOMNode|String*/ tag, /*Object*/ attrs, /*DOMNode|String?*/ refNode, /*String?*/ pos){
		// summary:
		//		Create an element, allowing for optional attribute decoration
		//		and placement.
		// description:
		//		A DOM Element creation function. A shorthand method for creating a node or
		//		a fragment, and allowing for a convenient optional attribute setting step,
		//		as well as an optional DOM placement reference.
		//
		//		Attributes are set by passing the optional object through `dojo.setAttr`.
		//		See `dojo.setAttr` for noted caveats and nuances, and API if applicable.
		//
		//		Placement is done via `dojo.place`, assuming the new node to be the action
		//		node, passing along the optional reference node and position.
		// tag: DOMNode|String
		//		A string of the element to create (eg: "div", "a", "p", "li", "script", "br"),
		//		or an existing DOM node to process.
		// attrs: Object
		//		An object-hash of attributes to set on the newly created node.
		//		Can be null, if you don't want to set any attributes/styles.
		//		See: `dojo.setAttr` for a description of available attributes.
		// refNode: DOMNode|String?
		//		Optional reference node. Used by `dojo.place` to place the newly created
		//		node somewhere in the dom relative to refNode. Can be a DomNode reference
		//		or String ID of a node.
		// pos: String?
		//		Optional positional reference. Defaults to "last" by way of `dojo.place`,
		//		though can be set to "first","after","before","last", "replace" or "only"
		//		to further control the placement of the new node relative to the refNode.
		//		'refNode' is required if a 'pos' is specified.
		// example:
		//		Create a DIV:
		//	|	var n = dojo.create("div");
		//
		// example:
		//		Create a DIV with content:
		//	|	var n = dojo.create("div", { innerHTML:"<p>hi</p>" });
		//
		// example:
		//		Place a new DIV in the BODY, with no attributes set
		//	|	var n = dojo.create("div", null, dojo.body());
		//
		// example:
		//		Create an UL, and populate it with LI's. Place the list as the first-child of a
		//		node with id="someId":
		//	|	var ul = dojo.create("ul", null, "someId", "first");
		//	|	var items = ["one", "two", "three", "four"];
		//	|	dojo.forEach(items, function(data){
		//	|		dojo.create("li", { innerHTML: data }, ul);
		//	|	});
		//
		// example:
		//		Create an anchor, with an href. Place in BODY:
		//	|	dojo.create("a", { href:"foo.html", title:"Goto FOO!" }, dojo.body());
		//
		// example:
		//		Create a `dojo/NodeList()` from a new element (for syntactic sugar):
		//	|	dojo.query(dojo.create('div'))
		//	|		.addClass("newDiv")
		//	|		.onclick(function(e){ console.log('clicked', e.target) })
		//	|		.place("#someNode"); // redundant, but cleaner.

		var doc = win.doc;
		if(refNode){
			refNode = dom.byId(refNode);
			doc = refNode.ownerDocument;
		}
		if(typeof tag == "string"){ // inline'd type check
			tag = doc.createElement(tag);
		}
		if(attrs){ attr.set(tag, attrs); }
		if(refNode){ exports.place(tag, refNode, pos); }
		return tag; // DomNode
	};

	function _empty(/*DomNode*/ node){
		if(node.canHaveChildren){
			try{
				// fast path
				node.innerHTML = "";
				return;
			}catch(e){
				// innerHTML is readOnly (e.g. TABLE (sub)elements in quirks mode)
				// Fall through (saves bytes)
			}
		}
		// SVG/strict elements don't support innerHTML/canHaveChildren, and OBJECT/APPLET elements in quirks node have canHaveChildren=false
		for(var c; c = node.lastChild;){ // intentional assignment
			_destroy(c, node); // destroy is better than removeChild so TABLE subelements are removed in proper order
		}
	}

	exports.empty = function empty(/*DOMNode|String*/ node){
		 // summary:
		 //		safely removes all children of the node.
		 // node: DOMNode|String
		 //		a reference to a DOM node or an id.
		 // example:
		 //		Destroy node's children byId:
		 //	|	dojo.empty("someId");
		 //
		 // example:
		 //		Destroy all nodes' children in a list by reference:
		 //	|	dojo.query(".someNode").forEach(dojo.empty);

		_empty(dom.byId(node));
	};


	function _destroy(/*DomNode*/ node, /*DomNode*/ parent){
		// in IE quirks, node.canHaveChildren can be false but firstChild can be non-null (OBJECT/APPLET)
		if(node.firstChild){
			_empty(node);
		}
		if(parent){
			// removeNode(false) doesn't leak in IE 6+, but removeChild() and removeNode(true) are known to leak under IE 8- while 9+ is TBD.
			// In IE quirks mode, PARAM nodes as children of OBJECT/APPLET nodes have a removeNode method that does nothing and
			// the parent node has canHaveChildren=false even though removeChild correctly removes the PARAM children.
			// In IE, SVG/strict nodes don't have a removeNode method nor a canHaveChildren boolean.
			has("ie") && parent.canHaveChildren && "removeNode" in node ? node.removeNode(false) : parent.removeChild(node);
		}
	}
	var destroy = exports.destroy = function destroy(/*DOMNode|String*/ node){
		// summary:
		//		Removes a node from its parent, clobbering it and all of its
		//		children.
		//
		// description:
		//		Removes a node from its parent, clobbering it and all of its
		//		children. Function only works with DomNodes, and returns nothing.
		//
		// node: DOMNode|String
		//		A String ID or DomNode reference of the element to be destroyed
		//
		// example:
		//		Destroy a node byId:
		//	|	dojo.destroy("someId");
		//
		// example:
		//		Destroy all nodes in a list by reference:
		//	|	dojo.query(".someNode").forEach(dojo.destroy);

		node = dom.byId(node);
		if(!node){ return; }
		_destroy(node, node.parentNode);
	};
});

},
'dojo/dom-class':function(){
define(["./_base/lang", "./_base/array", "./dom"], function(lang, array, dom){
	// module:
	//		dojo/dom-class

	var className = "className";

	/* Part I of classList-based implementation is preserved here for posterity
	var classList = "classList";
	has.add("dom-classList", function(){
		return classList in document.createElement("p");
	});
	*/

	// =============================
	// (CSS) Class Functions
	// =============================

	var cls, // exports object
		spaces = /\s+/, a1 = [""];

	function str2array(s){
		if(typeof s == "string" || s instanceof String){
			if(s && !spaces.test(s)){
				a1[0] = s;
				return a1;
			}
			var a = s.split(spaces);
			if(a.length && !a[0]){
				a.shift();
			}
			if(a.length && !a[a.length - 1]){
				a.pop();
			}
			return a;
		}
		// assumed to be an array
		if(!s){
			return [];
		}
		return array.filter(s, function(x){ return x; });
	}

	/* Part II of classList-based implementation is preserved here for posterity
	if(has("dom-classList")){
		// new classList version
		cls = {
			contains: function containsClass(node, classStr){
				var clslst = classStr && dom.byId(node)[classList];
				return clslst && clslst.contains(classStr); // Boolean
			},

			add: function addClass(node, classStr){
				node = dom.byId(node);
				classStr = str2array(classStr);
				for(var i = 0, len = classStr.length; i < len; ++i){
					node[classList].add(classStr[i]);
				}
			},

			remove: function removeClass(node, classStr){
				node = dom.byId(node);
				if(classStr === undefined){
					node[className] = "";
				}else{
					classStr = str2array(classStr);
					for(var i = 0, len = classStr.length; i < len; ++i){
						node[classList].remove(classStr[i]);
					}
				}
			},

			replace: function replaceClass(node, addClassStr, removeClassStr){
				node = dom.byId(node);
				if(removeClassStr === undefined){
					node[className] = "";
				}else{
					removeClassStr = str2array(removeClassStr);
					for(var i = 0, len = removeClassStr.length; i < len; ++i){
						node[classList].remove(removeClassStr[i]);
					}
				}
				addClassStr = str2array(addClassStr);
				for(i = 0, len = addClassStr.length; i < len; ++i){
					node[classList].add(addClassStr[i]);
				}
			},

			toggle: function toggleClass(node, classStr, condition){
				node = dom.byId(node);
				if(condition === undefined){
					classStr = str2array(classStr);
					for(var i = 0, len = classStr.length; i < len; ++i){
						node[classList].toggle(classStr[i]);
					}
				}else{
					cls[condition ? "add" : "remove"](node, classStr);
				}
				return condition;   // Boolean
			}
		}
	}
	*/

	// regular DOM version
	var fakeNode = {};  // for effective replacement
	cls = {
		// summary:
		//		This module defines the core dojo DOM class API.

		contains: function containsClass(/*DomNode|String*/ node, /*String*/ classStr){
			// summary:
			//		Returns whether or not the specified classes are a portion of the
			//		class list currently applied to the node.
			// node: String|DOMNode
			//		String ID or DomNode reference to check the class for.
			// classStr: String
			//		A string class name to look for.
			// example:
			//		Do something if a node with id="someNode" has class="aSillyClassName" present
			//	|	if(dojo.hasClass("someNode","aSillyClassName")){ ... }

			return ((" " + dom.byId(node)[className] + " ").indexOf(" " + classStr + " ") >= 0); // Boolean
		},

		add: function addClass(/*DomNode|String*/ node, /*String|Array*/ classStr){
			// summary:
			//		Adds the specified classes to the end of the class list on the
			//		passed node. Will not re-apply duplicate classes.
			//
			// node: String|DOMNode
			//		String ID or DomNode reference to add a class string too
			//
			// classStr: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			//
			// example:
			//		Add a class to some node:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.add("someNode", "anewClass");
			//	|	});
			//
			// example:
			//		Add two classes at once:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.add("someNode", "firstClass secondClass");
			//	|	});
			//
			// example:
			//		Add two classes at once (using array):
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.add("someNode", ["firstClass", "secondClass"]);
			//	|	});
			//
			// example:
			//		Available in `dojo/NodeList` for multiple additions
			//	|	require(["dojo/query"], function(query){
			//	|		query("ul > li").addClass("firstLevel");
			//	|	});

			node = dom.byId(node);
			classStr = str2array(classStr);
			var cls = node[className], oldLen;
			cls = cls ? " " + cls + " " : " ";
			oldLen = cls.length;
			for(var i = 0, len = classStr.length, c; i < len; ++i){
				c = classStr[i];
				if(c && cls.indexOf(" " + c + " ") < 0){
					cls += c + " ";
				}
			}
			if(oldLen < cls.length){
				node[className] = cls.substr(1, cls.length - 2);
			}
		},

		remove: function removeClass(/*DomNode|String*/ node, /*String|Array?*/ classStr){
			// summary:
			//		Removes the specified classes from node. No `contains()`
			//		check is required.
			//
			// node: String|DOMNode
			//		String ID or DomNode reference to remove the class from.
			//
			// classStr: String|Array
			//		An optional String class name to remove, or several space-separated
			//		class names, or an array of class names. If omitted, all class names
			//		will be deleted.
			//
			// example:
			//		Remove a class from some node:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.remove("someNode", "firstClass");
			//	|	});
			//
			// example:
			//		Remove two classes from some node:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.remove("someNode", "firstClass secondClass");
			//	|	});
			//
			// example:
			//		Remove two classes from some node (using array):
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.remove("someNode", ["firstClass", "secondClass"]);
			//	|	});
			//
			// example:
			//		Remove all classes from some node:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.remove("someNode");
			//	|	});
			//
			// example:
			//		Available in `dojo/NodeList` for multiple removal
			//	|	require(["dojo/query"], function(query){
			//	|		query("ul > li").removeClass("foo");
			//	|	});

			node = dom.byId(node);
			var cls;
			if(classStr !== undefined){
				classStr = str2array(classStr);
				cls = " " + node[className] + " ";
				for(var i = 0, len = classStr.length; i < len; ++i){
					cls = cls.replace(" " + classStr[i] + " ", " ");
				}
				cls = lang.trim(cls);
			}else{
				cls = "";
			}
			if(node[className] != cls){ node[className] = cls; }
		},

		replace: function replaceClass(/*DomNode|String*/ node, /*String|Array*/ addClassStr, /*String|Array?*/ removeClassStr){
			// summary:
			//		Replaces one or more classes on a node if not present.
			//		Operates more quickly than calling dojo.removeClass and dojo.addClass
			//
			// node: String|DOMNode
			//		String ID or DomNode reference to remove the class from.
			//
			// addClassStr: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			//
			// removeClassStr: String|Array?
			//		A String class name to remove, or several space-separated class names,
			//		or an array of class names.
			//
			// example:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.replace("someNode", "add1 add2", "remove1 remove2");
			//	|	});
			//
			// example:
			//	Replace all classes with addMe
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.replace("someNode", "addMe");
			//	|	});
			//
			// example:
			//	Available in `dojo/NodeList` for multiple toggles
			//	|	require(["dojo/query"], function(query){
			//	|		query(".findMe").replaceClass("addMe", "removeMe");
			//	|	});

			node = dom.byId(node);
			fakeNode[className] = node[className];
			cls.remove(fakeNode, removeClassStr);
			cls.add(fakeNode, addClassStr);
			if(node[className] !== fakeNode[className]){
				node[className] = fakeNode[className];
			}
		},

		toggle: function toggleClass(/*DomNode|String*/ node, /*String|Array*/ classStr, /*Boolean?*/ condition){
			// summary:
			//		Adds a class to node if not present, or removes if present.
			//		Pass a boolean condition if you want to explicitly add or remove.
			//		Returns the condition that was specified directly or indirectly.
			//
			// node: String|DOMNode
			//		String ID or DomNode reference to toggle a class string
			//
			// classStr: String|Array
			//		A String class name to toggle, or several space-separated class names,
			//		or an array of class names.
			//
			// condition:
			//		If passed, true means to add the class, false means to remove.
			//		Otherwise dojo.hasClass(node, classStr) is used to detect the class presence.
			//
			// example:
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.toggle("someNode", "hovered");
			//	|	});
			//
			// example:
			//		Forcefully add a class
			//	|	require(["dojo/dom-class"], function(domClass){
			//	|		domClass.toggle("someNode", "hovered", true);
			//	|	});
			//
			// example:
			//		Available in `dojo/NodeList` for multiple toggles
			//	|	require(["dojo/query"], function(query){
			//	|		query(".toggleMe").toggleClass("toggleMe");
			//	|	});

			node = dom.byId(node);
			if(condition === undefined){
				classStr = str2array(classStr);
				for(var i = 0, len = classStr.length, c; i < len; ++i){
					c = classStr[i];
					cls[cls.contains(node, c) ? "remove" : "add"](node, c);
				}
			}else{
				cls[condition ? "add" : "remove"](node, classStr);
			}
			return condition;   // Boolean
		}
	};

	return cls;
});

},
'dojo/_base/NodeList':function(){
define(["./kernel", "../query", "./array", "./html", "../NodeList-dom"], function(dojo, query, array){
	// module:
	//		dojo/_base/NodeList

	/*=====
	return {
		// summary:
		//		This module extends dojo/NodeList with the legacy connect(), coords(),
		//		blur(), focus(), change(), click(), error(), keydown(), keypress(),
		//		keyup(), load(), mousedown(), mouseenter(), mouseleave(), mousemove(),
		//		mouseout(), mouseover(), mouseup(), and submit() methods.
	};
	=====*/
 
	var NodeList = query.NodeList,
		nlp = NodeList.prototype;

	nlp.connect = NodeList._adaptAsForEach(function(){
		// don't bind early to dojo.connect since we no longer explicitly depend on it
		return dojo.connect.apply(this, arguments);
	});
	/*=====
	nlp.connect = function(methodName, objOrFunc, funcName){
		// summary:
		//		Attach event handlers to every item of the NodeList. Uses dojo.connect()
		//		so event properties are normalized.
		//
		//		Application must manually require() "dojo/_base/connect" before using this method.
		// methodName: String
		//		the name of the method to attach to. For DOM events, this should be
		//		the lower-case name of the event
		// objOrFunc: Object|Function|String
		//		if 2 arguments are passed (methodName, objOrFunc), objOrFunc should
		//		reference a function or be the name of the function in the global
		//		namespace to attach. If 3 arguments are provided
		//		(methodName, objOrFunc, funcName), objOrFunc must be the scope to
		//		locate the bound function in
		// funcName: String?
		//		optional. A string naming the function in objOrFunc to bind to the
		//		event. May also be a function reference.
		// example:
		//		add an onclick handler to every button on the page
		//		|	query("div:nth-child(odd)").connect("onclick", function(e){
		//		|		console.log("clicked!");
		//		|	});
		// example:
		//		attach foo.bar() to every odd div's onmouseover
		//		|	query("div:nth-child(odd)").connect("onmouseover", foo, "bar");

		return null;	// NodeList
	};
	=====*/

	nlp.coords = NodeList._adaptAsMap(dojo.coords);
	/*=====
	nlp.coords = function(){
		// summary:
		//		Deprecated: Use position() for border-box x/y/w/h
		//		or marginBox() for margin-box w/h/l/t.
		//		Returns the box objects of all elements in a node list as
		//		an Array (*not* a NodeList). Acts like `domGeom.coords`, though assumes
		//		the node passed is each node in this list.

		return []; // Array
	};
	=====*/

	NodeList.events = [
		// summary:
		//		list of all DOM events used in NodeList
		"blur", "focus", "change", "click", "error", "keydown", "keypress",
		"keyup", "load", "mousedown", "mouseenter", "mouseleave", "mousemove",
		"mouseout", "mouseover", "mouseup", "submit"
	];

	// FIXME: pseudo-doc the above automatically generated on-event functions

	// syntactic sugar for DOM events
	array.forEach(NodeList.events, function(evt){
			var _oe = "on" + evt;
			nlp[_oe] = function(a, b){
				return this.connect(_oe, a, b);
			};
				// FIXME: should these events trigger publishes?
				/*
				return (a ? this.connect(_oe, a, b) :
							this.forEach(function(n){
								// FIXME:
								//		listeners get buried by
								//		addEventListener and can't be dug back
								//		out to be triggered externally.
								// see:
								//		http://developer.mozilla.org/en/docs/DOM:element

								console.log(n, evt, _oe);

								// FIXME: need synthetic event support!
								var _e = { target: n, faux: true, type: evt };
								// dojo._event_listener._synthesizeEvent({}, { target: n, faux: true, type: evt });
								try{ n[evt](_e); }catch(e){ console.log(e); }
								try{ n[_oe](_e); }catch(e){ console.log(e); }
							})
				);
				*/
		}
	);

	dojo.NodeList = NodeList;
	return NodeList;
});

},
'dojo/query':function(){
define(["./_base/kernel", "./has", "./dom", "./on", "./_base/array", "./_base/lang", "./selector/_loader", "./selector/_loader!default"],
	function(dojo, has, dom, on, array, lang, loader, defaultEngine){

	"use strict";

	has.add("array-extensible", function(){
		// test to see if we can extend an array (not supported in old IE)
		return lang.delegate([], {length: 1}).length == 1 && !has("bug-for-in-skips-shadowed");
	});
	
	var ap = Array.prototype, aps = ap.slice, apc = ap.concat, forEach = array.forEach;

	var tnl = function(/*Array*/ a, /*dojo/NodeList?*/ parent, /*Function?*/ NodeListCtor){
		// summary:
		//		decorate an array to make it look like a `dojo/NodeList`.
		// a:
		//		Array of nodes to decorate.
		// parent:
		//		An optional parent NodeList that generated the current
		//		list of nodes. Used to call _stash() so the parent NodeList
		//		can be accessed via end() later.
		// NodeListCtor:
		//		An optional constructor function to use for any
		//		new NodeList calls. This allows a certain chain of
		//		NodeList calls to use a different object than dojo/NodeList.
		var nodeList = new (NodeListCtor || this._NodeListCtor || nl)(a);
		return parent ? nodeList._stash(parent) : nodeList;
	};

	var loopBody = function(f, a, o){
		a = [0].concat(aps.call(a, 0));
		o = o || dojo.global;
		return function(node){
			a[0] = node;
			return f.apply(o, a);
		};
	};

	// adapters

	var adaptAsForEach = function(f, o){
		// summary:
		//		adapts a single node function to be used in the forEach-type
		//		actions. The initial object is returned from the specialized
		//		function.
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			this.forEach(loopBody(f, arguments, o));
			return this;	// Object
		};
	};

	var adaptAsMap = function(f, o){
		// summary:
		//		adapts a single node function to be used in the map-type
		//		actions. The return is a new array of values, as via `dojo.map`
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			return this.map(loopBody(f, arguments, o));
		};
	};

	var adaptAsFilter = function(f, o){
		// summary:
		//		adapts a single node function to be used in the filter-type actions
		// f: Function
		//		a function to adapt
		// o: Object?
		//		an optional context for f
		return function(){
			return this.filter(loopBody(f, arguments, o));
		};
	};

	var adaptWithCondition = function(f, g, o){
		// summary:
		//		adapts a single node function to be used in the map-type
		//		actions, behaves like forEach() or map() depending on arguments
		// f: Function
		//		a function to adapt
		// g: Function
		//		a condition function, if true runs as map(), otherwise runs as forEach()
		// o: Object?
		//		an optional context for f and g
		return function(){
			var a = arguments, body = loopBody(f, a, o);
			if(g.call(o || dojo.global, a)){
				return this.map(body);	// self
			}
			this.forEach(body);
			return this;	// self
		};
	};

	var NodeList = function(array){
		// summary:
		//		Array-like object which adds syntactic
		//		sugar for chaining, common iteration operations, animation, and
		//		node manipulation. NodeLists are most often returned as the
		//		result of dojo.query() calls.
		// description:
		//		NodeList instances provide many utilities that reflect
		//		core Dojo APIs for Array iteration and manipulation, DOM
		//		manipulation, and event handling. Instead of needing to dig up
		//		functions in the dojo.* namespace, NodeLists generally make the
		//		full power of Dojo available for DOM manipulation tasks in a
		//		simple, chainable way.
		// example:
		//		create a node list from a node
		//		|	new query.NodeList(dojo.byId("foo"));
		// example:
		//		get a NodeList from a CSS query and iterate on it
		//		|	var l = dojo.query(".thinger");
		//		|	l.forEach(function(node, index, nodeList){
		//		|		console.log(index, node.innerHTML);
		//		|	});
		// example:
		//		use native and Dojo-provided array methods to manipulate a
		//		NodeList without needing to use dojo.* functions explicitly:
		//		|	var l = dojo.query(".thinger");
		//		|	// since NodeLists are real arrays, they have a length
		//		|	// property that is both readable and writable and
		//		|	// push/pop/shift/unshift methods
		//		|	console.log(l.length);
		//		|	l.push(dojo.create("span"));
		//		|
		//		|	// dojo's normalized array methods work too:
		//		|	console.log( l.indexOf(dojo.byId("foo")) );
		//		|	// ...including the special "function as string" shorthand
		//		|	console.log( l.every("item.nodeType == 1") );
		//		|
		//		|	// NodeLists can be [..] indexed, or you can use the at()
		//		|	// function to get specific items wrapped in a new NodeList:
		//		|	var node = l[3]; // the 4th element
		//		|	var newList = l.at(1, 3); // the 2nd and 4th elements
		// example:
		//		the style functions you expect are all there too:
		//		|	// style() as a getter...
		//		|	var borders = dojo.query(".thinger").style("border");
		//		|	// ...and as a setter:
		//		|	dojo.query(".thinger").style("border", "1px solid black");
		//		|	// class manipulation
		//		|	dojo.query("li:nth-child(even)").addClass("even");
		//		|	// even getting the coordinates of all the items
		//		|	var coords = dojo.query(".thinger").coords();
		// example:
		//		DOM manipulation functions from the dojo.* namespace area also available:
		//		|	// remove all of the elements in the list from their
		//		|	// parents (akin to "deleting" them from the document)
		//		|	dojo.query(".thinger").orphan();
		//		|	// place all elements in the list at the front of #foo
		//		|	dojo.query(".thinger").place("foo", "first");
		// example:
		//		Event handling couldn't be easier. `dojo.connect` is mapped in,
		//		and shortcut handlers are provided for most DOM events:
		//		|	// like dojo.connect(), but with implicit scope
		//		|	dojo.query("li").connect("onclick", console, "log");
		//		|
		//		|	// many common event handlers are already available directly:
		//		|	dojo.query("li").onclick(console, "log");
		//		|	var toggleHovered = dojo.hitch(dojo, "toggleClass", "hovered");
		//		|	dojo.query("p")
		//		|		.onmouseenter(toggleHovered)
		//		|		.onmouseleave(toggleHovered);
		// example:
		//		chainability is a key advantage of NodeLists:
		//		|	dojo.query(".thinger")
		//		|		.onclick(function(e){ /* ... */ })
		//		|		.at(1, 3, 8) // get a subset
		//		|			.style("padding", "5px")
		//		|			.forEach(console.log);
		var isNew = this instanceof nl && has("array-extensible");
		if(typeof array == "number"){
			array = Array(array);
		}
		var nodeArray = (array && "length" in array) ? array : arguments;
		if(isNew || !nodeArray.sort){
			// make sure it's a real array before we pass it on to be wrapped 
			var target = isNew ? this : [],
				l = target.length = nodeArray.length;
			for(var i = 0; i < l; i++){
				target[i] = nodeArray[i];
			}
			if(isNew){
				// called with new operator, this means we are going to use this instance and push
				// the nodes on to it. This is usually much faster since the NodeList properties
				//	don't need to be copied (unless the list of nodes is extremely large).
				return target;
			}
			nodeArray = target;
		}
		// called without new operator, use a real array and copy prototype properties,
		// this is slower and exists for back-compat. Should be removed in 2.0.
		lang._mixin(nodeArray, nlp);
		nodeArray._NodeListCtor = function(array){
			// call without new operator to preserve back-compat behavior
			return nl(array);
		};
		return nodeArray;
	};
	
	var nl = NodeList, nlp = nl.prototype = 
		has("array-extensible") ? [] : {};// extend an array if it is extensible

	// expose adapters and the wrapper as private functions

	nl._wrap = nlp._wrap = tnl;
	nl._adaptAsMap = adaptAsMap;
	nl._adaptAsForEach = adaptAsForEach;
	nl._adaptAsFilter  = adaptAsFilter;
	nl._adaptWithCondition = adaptWithCondition;

	// mass assignment

	// add array redirectors
	forEach(["slice", "splice"], function(name){
		var f = ap[name];
		//Use a copy of the this array via this.slice() to allow .end() to work right in the splice case.
		// CANNOT apply ._stash()/end() to splice since it currently modifies
		// the existing this array -- it would break backward compatibility if we copy the array before
		// the splice so that we can use .end(). So only doing the stash option to this._wrap for slice.
		nlp[name] = function(){ return this._wrap(f.apply(this, arguments), name == "slice" ? this : null); };
	});
	// concat should be here but some browsers with native NodeList have problems with it

	// add array.js redirectors
	forEach(["indexOf", "lastIndexOf", "every", "some"], function(name){
		var f = array[name];
		nlp[name] = function(){ return f.apply(dojo, [this].concat(aps.call(arguments, 0))); };
	});

	lang.extend(NodeList, {
		// copy the constructors
		constructor: nl,
		_NodeListCtor: nl,
		toString: function(){
			// Array.prototype.toString can't be applied to objects, so we use join
			return this.join(",");
		},
		_stash: function(parent){
			// summary:
			//		private function to hold to a parent NodeList. end() to return the parent NodeList.
			//
			// example:
			//		How to make a `dojo/NodeList` method that only returns the third node in
			//		the dojo/NodeList but allows access to the original NodeList by using this._stash:
			//	|	dojo.extend(NodeList, {
			//	|		third: function(){
			//	|			var newNodeList = NodeList(this[2]);
			//	|			return newNodeList._stash(this);
			//	|		}
			//	|	});
			//	|	// then see how _stash applies a sub-list, to be .end()'ed out of
			//	|	dojo.query(".foo")
			//	|		.third()
			//	|			.addClass("thirdFoo")
			//	|		.end()
			//	|		// access to the orig .foo list
			//	|		.removeClass("foo")
			//	|
			//
			this._parent = parent;
			return this; // dojo/NodeList
		},

		on: function(eventName, listener){
			// summary:
			//		Listen for events on the nodes in the NodeList. Basic usage is:
			//		| query(".my-class").on("click", listener);
			//		This supports event delegation by using selectors as the first argument with the event names as
			//		pseudo selectors. For example:
			//		| dojo.query("#my-list").on("li:click", listener);
			//		This will listen for click events within `<li>` elements that are inside the `#my-list` element.
			//		Because on supports CSS selector syntax, we can use comma-delimited events as well:
			//		| dojo.query("#my-list").on("li button:mouseover, li:click", listener);
			var handles = this.map(function(node){
				return on(node, eventName, listener); // TODO: apply to the NodeList so the same selector engine is used for matches
			});
			handles.remove = function(){
				for(var i = 0; i < handles.length; i++){
					handles[i].remove();
				}
			};
			return handles;
		},

		end: function(){
			// summary:
			//		Ends use of the current `NodeList` by returning the previous NodeList
			//		that generated the current NodeList.
			// description:
			//		Returns the `NodeList` that generated the current `NodeList`. If there
			//		is no parent NodeList, an empty NodeList is returned.
			// example:
			//	|	dojo.query("a")
			//	|		.filter(".disabled")
			//	|			// operate on the anchors that only have a disabled class
			//	|			.style("color", "grey")
			//	|		.end()
			//	|		// jump back to the list of anchors
			//	|		.style(...)
			//
			if(this._parent){
				return this._parent;
			}else{
				//Just return empty list.
				return new this._NodeListCtor(0);
			}
		},

		// http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array#Methods

		// FIXME: handle return values for #3244
		//		http://trac.dojotoolkit.org/ticket/3244

		// FIXME:
		//		need to wrap or implement:
		//			join (perhaps w/ innerHTML/outerHTML overload for toString() of items?)
		//			reduce
		//			reduceRight

		/*=====
		slice: function(begin, end){
			// summary:
			//		Returns a new NodeList, maintaining this one in place
			// description:
			//		This method behaves exactly like the Array.slice method
			//		with the caveat that it returns a dojo/NodeList and not a
			//		raw Array. For more details, see Mozilla's [slice
			//		documentation](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/slice)
			// begin: Integer
			//		Can be a positive or negative integer, with positive
			//		integers noting the offset to begin at, and negative
			//		integers denoting an offset from the end (i.e., to the left
			//		of the end)
			// end: Integer?
			//		Optional parameter to describe what position relative to
			//		the NodeList's zero index to end the slice at. Like begin,
			//		can be positive or negative.
			return this._wrap(a.slice.apply(this, arguments));
		},

		splice: function(index, howmany, item){
			// summary:
			//		Returns a new NodeList, manipulating this NodeList based on
			//		the arguments passed, potentially splicing in new elements
			//		at an offset, optionally deleting elements
			// description:
			//		This method behaves exactly like the Array.splice method
			//		with the caveat that it returns a dojo/NodeList and not a
			//		raw Array. For more details, see Mozilla's [splice
			//		documentation](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/splice)
			//		For backwards compatibility, calling .end() on the spliced NodeList
			//		does not return the original NodeList -- splice alters the NodeList in place.
			// index: Integer
			//		begin can be a positive or negative integer, with positive
			//		integers noting the offset to begin at, and negative
			//		integers denoting an offset from the end (i.e., to the left
			//		of the end)
			// howmany: Integer?
			//		Optional parameter to describe what position relative to
			//		the NodeList's zero index to end the slice at. Like begin,
			//		can be positive or negative.
			// item: Object...?
			//		Any number of optional parameters may be passed in to be
			//		spliced into the NodeList
			return this._wrap(a.splice.apply(this, arguments));	// dojo/NodeList
		},

		indexOf: function(value, fromIndex){
			// summary:
			//		see dojo.indexOf(). The primary difference is that the acted-on
			//		array is implicitly this NodeList
			// value: Object
			//		The value to search for.
			// fromIndex: Integer?
			//		The location to start searching from. Optional. Defaults to 0.
			// description:
			//		For more details on the behavior of indexOf, see Mozilla's
			//		[indexOf
			//		docs](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/indexOf)
			// returns:
			//		Positive Integer or 0 for a match, -1 of not found.
			return d.indexOf(this, value, fromIndex); // Integer
		},

		lastIndexOf: function(value, fromIndex){
			// summary:
			//		see dojo.lastIndexOf(). The primary difference is that the
			//		acted-on array is implicitly this NodeList
			// description:
			//		For more details on the behavior of lastIndexOf, see
			//		Mozilla's [lastIndexOf
			//		docs](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/lastIndexOf)
			// value: Object
			//		The value to search for.
			// fromIndex: Integer?
			//		The location to start searching from. Optional. Defaults to 0.
			// returns:
			//		Positive Integer or 0 for a match, -1 of not found.
			return d.lastIndexOf(this, value, fromIndex); // Integer
		},

		every: function(callback, thisObject){
			// summary:
			//		see `dojo.every()` and the [Array.every
			//		docs](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/every).
			//		Takes the same structure of arguments and returns as
			//		dojo.every() with the caveat that the passed array is
			//		implicitly this NodeList
			// callback: Function
			//		the callback
			// thisObject: Object?
			//		the context
			return d.every(this, callback, thisObject); // Boolean
		},

		some: function(callback, thisObject){
			// summary:
			//		Takes the same structure of arguments and returns as
			//		`dojo.some()` with the caveat that the passed array is
			//		implicitly this NodeList.  See `dojo.some()` and Mozilla's
			//		[Array.some
			//		documentation](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/some).
			// callback: Function
			//		the callback
			// thisObject: Object?
			//		the context
			return d.some(this, callback, thisObject); // Boolean
		},
		=====*/

		concat: function(item){
			// summary:
			//		Returns a new NodeList comprised of items in this NodeList
			//		as well as items passed in as parameters
			// description:
			//		This method behaves exactly like the Array.concat method
			//		with the caveat that it returns a `NodeList` and not a
			//		raw Array. For more details, see the [Array.concat
			//		docs](https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/concat)
			// item: Object?
			//		Any number of optional parameters may be passed in to be
			//		spliced into the NodeList

			//return this._wrap(apc.apply(this, arguments));
			// the line above won't work for the native NodeList, or for Dojo NodeLists either :-(

			// implementation notes:
			// Array.concat() doesn't recognize native NodeLists or Dojo NodeLists
			// as arrays, and so does not inline them into a unioned array, but
			// appends them as single entities. Both the original NodeList and the
			// items passed in as parameters must be converted to raw Arrays
			// and then the concatenation result may be re-_wrap()ed as a Dojo NodeList.

			var t = aps.call(this, 0),
				m = array.map(arguments, function(a){
					return aps.call(a, 0);
				});
			return this._wrap(apc.apply(t, m), this);	// dojo/NodeList
		},

		map: function(/*Function*/ func, /*Function?*/ obj){
			// summary:
			//		see dojo.map(). The primary difference is that the acted-on
			//		array is implicitly this NodeList and the return is a
			//		NodeList (a subclass of Array)
			return this._wrap(array.map(this, func, obj), this); // dojo/NodeList
		},

		forEach: function(callback, thisObj){
			// summary:
			//		see `dojo.forEach()`. The primary difference is that the acted-on
			//		array is implicitly this NodeList. If you want the option to break out
			//		of the forEach loop, use every() or some() instead.
			forEach(this, callback, thisObj);
			// non-standard return to allow easier chaining
			return this; // dojo/NodeList
		},
		filter: function(/*String|Function*/ filter){
			// summary:
			//		"masks" the built-in javascript filter() method (supported
			//		in Dojo via `dojo.filter`) to support passing a simple
			//		string filter in addition to supporting filtering function
			//		objects.
			// filter:
			//		If a string, a CSS rule like ".thinger" or "div > span".
			// example:
			//		"regular" JS filter syntax as exposed in dojo.filter:
			//		|	dojo.query("*").filter(function(item){
			//		|		// highlight every paragraph
			//		|		return (item.nodeName == "p");
			//		|	}).style("backgroundColor", "yellow");
			// example:
			//		the same filtering using a CSS selector
			//		|	dojo.query("*").filter("p").styles("backgroundColor", "yellow");

			var a = arguments, items = this, start = 0;
			if(typeof filter == "string"){ // inline'd type check
				items = query._filterResult(this, a[0]);
				if(a.length == 1){
					// if we only got a string query, pass back the filtered results
					return items._stash(this); // dojo/NodeList
				}
				// if we got a callback, run it over the filtered items
				start = 1;
			}
			return this._wrap(array.filter(items, a[start], a[start + 1]), this);	// dojo/NodeList
		},
		instantiate: function(/*String|Object*/ declaredClass, /*Object?*/ properties){
			// summary:
			//		Create a new instance of a specified class, using the
			//		specified properties and each node in the NodeList as a
			//		srcNodeRef.
			// example:
			//		Grabs all buttons in the page and converts them to dijit/form/Button's.
			//	|	var buttons = query("button").instantiate(Button, {showLabel: true});
			var c = lang.isFunction(declaredClass) ? declaredClass : lang.getObject(declaredClass);
			properties = properties || {};
			return this.forEach(function(node){
				new c(properties, node);
			});	// dojo/NodeList
		},
		at: function(/*===== index =====*/){
			// summary:
			//		Returns a new NodeList comprised of items in this NodeList
			//		at the given index or indices.
			//
			// index: Integer...
			//		One or more 0-based indices of items in the current
			//		NodeList. A negative index will start at the end of the
			//		list and go backwards.
			//
			// example:
			//	Shorten the list to the first, second, and third elements
			//	|	query("a").at(0, 1, 2).forEach(fn);
			//
			// example:
			//	Retrieve the first and last elements of a unordered list:
			//	|	query("ul > li").at(0, -1).forEach(cb);
			//
			// example:
			//	Do something for the first element only, but end() out back to
			//	the original list and continue chaining:
			//	|	query("a").at(0).onclick(fn).end().forEach(function(n){
			//	|		console.log(n); // all anchors on the page.
			//	|	})

			var t = new this._NodeListCtor(0);
			forEach(arguments, function(i){
				if(i < 0){ i = this.length + i; }
				if(this[i]){ t.push(this[i]); }
			}, this);
			return t._stash(this); // dojo/NodeList
		}
	});

	function queryForEngine(engine, NodeList){
		var query = function(/*String*/ query, /*String|DOMNode?*/ root){
			// summary:
			//		Returns nodes which match the given CSS selector, searching the
			//		entire document by default but optionally taking a node to scope
			//		the search by. Returns an instance of NodeList.
			if(typeof root == "string"){
				root = dom.byId(root);
				if(!root){
					return new NodeList([]);
				}
			}
			var results = typeof query == "string" ? engine(query, root) : query ? (query.end && query.on) ? query : [query] : [];
			if(results.end && results.on){
				// already wrapped
				return results;
			}
			return new NodeList(results);
		};
		query.matches = engine.match || function(node, selector, root){
			// summary:
			//		Test to see if a node matches a selector
			return query.filter([node], selector, root).length > 0;
		};
		// the engine provides a filtering function, use it to for matching
		query.filter = engine.filter || function(nodes, selector, root){
			// summary:
			//		Filters an array of nodes. Note that this does not guarantee to return a NodeList, just an array.
			return query(selector, root).filter(function(node){
				return array.indexOf(nodes, node) > -1;
			});
		};
		if(typeof engine != "function"){
			var search = engine.search;
			engine = function(selector, root){
				// Slick does it backwards (or everyone else does it backwards, probably the latter)
				return search(root || document, selector);
			};
		}
		return query;
	}
	var query = queryForEngine(defaultEngine, NodeList);
	/*=====
	query = function(selector, context){
		// summary:
		//		This modules provides DOM querying functionality. The module export is a function
		//		that can be used to query for DOM nodes by CSS selector and returns a NodeList
		//		representing the matching nodes.
		// selector: String
		//		A CSS selector to search for.
		// context: String|DomNode?
		//		An optional context to limit the searching scope. Only nodes under `context` will be
		//		scanned.
		// example:
		//		add an onclick handler to every submit button in the document
		//		which causes the form to be sent via Ajax instead:
		//	|	require(["dojo/query"], function(query){
		//	|		query("input[type='submit']").on("click", function(e){
		//	|			dojo.stopEvent(e); // prevent sending the form
		//	|			var btn = e.target;
		//	|			dojo.xhrPost({
		//	|				form: btn.form,
		//	|				load: function(data){
		//	|					// replace the form with the response
		//	|					var div = dojo.doc.createElement("div");
		//	|					dojo.place(div, btn.form, "after");
		//	|					div.innerHTML = data;
		//	|					dojo.style(btn.form, "display", "none");
		//	|				}
		//	|			});
		//	|		});
		// |	});
		//
		// description:
		//		dojo/query is responsible for loading the appropriate query engine and wrapping
		//		its results with a `NodeList`. You can use dojo/query with a specific selector engine
		//		by using it as a plugin. For example, if you installed the sizzle package, you could
		//		use it as the selector engine with:
		//		|	require(["dojo/query!sizzle"], function(query){
		//		|		query("div")...
		//
		//		The id after the ! can be a module id of the selector engine or one of the following values:
		//
		//		- acme: This is the default engine used by Dojo base, and will ensure that the full
		//		Acme engine is always loaded.
		//
		//		- css2: If the browser has a native selector engine, this will be used, otherwise a
		//		very minimal lightweight selector engine will be loaded that can do simple CSS2 selectors
		//		(by #id, .class, tag, and [name=value] attributes, with standard child or descendant (>)
		//		operators) and nothing more.
		//
		//		- css2.1: If the browser has a native selector engine, this will be used, otherwise the
		//		full Acme engine will be loaded.
		//
		//		- css3: If the browser has a native selector engine with support for CSS3 pseudo
		//		selectors (most modern browsers except IE8), this will be used, otherwise the
		//		full Acme engine will be loaded.
		//
		//		- Or the module id of a selector engine can be used to explicitly choose the selector engine
		//
		//		For example, if you are using CSS3 pseudo selectors in module, you can specify that
		//		you will need support them with:
		//		|	require(["dojo/query!css3"], function(query){
		//		|		query('#t > h3:nth-child(odd)')...
		//
		//		You can also choose the selector engine/load configuration by setting the query-selector:
		//		For example:
		//		|	<script data-dojo-config="query-selector:'css3'" src="dojo.js"></script>
		//
		return new NodeList(); // dojo/NodeList
	 };
	 =====*/

	// the query that is returned from this module is slightly different than dojo.query,
	// because dojo.query has to maintain backwards compatibility with returning a
	// true array which has performance problems. The query returned from the module
	// does not use true arrays, but rather inherits from Array, making it much faster to
	// instantiate.
	dojo.query = queryForEngine(defaultEngine, function(array){
		// call it without the new operator to invoke the back-compat behavior that returns a true array
		return NodeList(array);	// dojo/NodeList
	});

	query.load = function(id, parentRequire, loaded){
		// summary:
		//		can be used as AMD plugin to conditionally load new query engine
		// example:
		//	|	require(["dojo/query!custom"], function(qsa){
		//	|		// loaded selector/custom.js as engine
		//	|		qsa("#foobar").forEach(...);
		//	|	});
		loader.load(id, parentRequire, function(engine){
			loaded(queryForEngine(engine, NodeList));
		});
	};

	dojo._filterQueryResult = query._filterResult = function(nodes, selector, root){
		return new NodeList(query.filter(nodes, selector, root));
	};
	dojo.NodeList = query.NodeList = NodeList;
	return query;
});

},
'dojo/selector/_loader':function(){
define(["../has", "require"],
		function(has, require){

"use strict";
var testDiv = document.createElement("div");
has.add("dom-qsa2.1", !!testDiv.querySelectorAll);
has.add("dom-qsa3", function(){
			// test to see if we have a reasonable native selector engine available
			try{
				testDiv.innerHTML = "<p class='TEST'></p>"; // test kind of from sizzle
				// Safari can't handle uppercase or unicode characters when
				// in quirks mode, IE8 can't handle pseudos like :empty
				return testDiv.querySelectorAll(".TEST:empty").length == 1;
			}catch(e){}
		});
var fullEngine;
var acme = "./acme", lite = "./lite";
return {
	// summary:
	//		This module handles loading the appropriate selector engine for the given browser

	load: function(id, parentRequire, loaded, config){
		var req = require;
		// here we implement the default logic for choosing a selector engine
		id = id == "default" ? has("config-selectorEngine") || "css3" : id;
		id = id == "css2" || id == "lite" ? lite :
				id == "css2.1" ? has("dom-qsa2.1") ? lite : acme :
				id == "css3" ? has("dom-qsa3") ? lite : acme :
				id == "acme" ? acme : (req = parentRequire) && id;
		if(id.charAt(id.length-1) == '?'){
			id = id.substring(0,id.length - 1);
			var optionalLoad = true;
		}
		// the query engine is optional, only load it if a native one is not available or existing one has not been loaded
		if(optionalLoad && (has("dom-compliant-qsa") || fullEngine)){
			return loaded(fullEngine);
		}
		// load the referenced selector engine
		req([id], function(engine){
			if(id != "./lite"){
				fullEngine = engine;
			}
			loaded(engine);
		});
	}
};
});

},
'dojo/selector/lite':function(){
define(["../has", "../_base/kernel"], function(has, dojo){
"use strict";

var testDiv = document.createElement("div");
var matchesSelector = testDiv.matchesSelector || testDiv.webkitMatchesSelector || testDiv.mozMatchesSelector || testDiv.msMatchesSelector || testDiv.oMatchesSelector; // IE9, WebKit, Firefox have this, but not Opera yet
var querySelectorAll = testDiv.querySelectorAll;
var unionSplit = /([^\s,](?:"(?:\\.|[^"])+"|'(?:\\.|[^'])+'|[^,])*)/g;
has.add("dom-matches-selector", !!matchesSelector);
has.add("dom-qsa", !!querySelectorAll); 

// this is a simple query engine. It has handles basic selectors, and for simple
// common selectors is extremely fast
var liteEngine = function(selector, root){
	// summary:
	//		A small lightweight query selector engine that implements CSS2.1 selectors
	//		minus pseudo-classes and the sibling combinator, plus CSS3 attribute selectors

	if(combine && selector.indexOf(',') > -1){
		return combine(selector, root);
	}
	// use the root's ownerDocument if provided, otherwise try to use dojo.doc. Note 
	// that we don't use dojo/_base/window's doc to reduce dependencies, and 
	// fallback to plain document if dojo.doc hasn't been defined (by dojo/_base/window).
	// presumably we will have a better way to do this in 2.0 
	var doc = root ? root.ownerDocument || root : dojo.doc || document, 
		match = (querySelectorAll ? 
			/^([\w]*)#([\w\-]+$)|^(\.)([\w\-\*]+$)|^(\w+$)/ : // this one only matches on simple queries where we can beat qSA with specific methods
			/^([\w]*)#([\w\-]+)(?:\s+(.*))?$|(?:^|(>|.+\s+))([\w\-\*]+)(\S*$)/) // this one matches parts of the query that we can use to speed up manual filtering
			.exec(selector);
	root = root || doc;
	if(match){
		// fast path regardless of whether or not querySelectorAll exists
		if(match[2]){
			// an #id
			// use dojo.byId if available as it fixes the id retrieval in IE, note that we can't use the dojo namespace in 2.0, but if there is a conditional module use, we will use that
			var found = dojo.byId ? dojo.byId(match[2], doc) : doc.getElementById(match[2]);
			if(!found || (match[1] && match[1] != found.tagName.toLowerCase())){
				// if there is a tag qualifer and it doesn't match, no matches
				return [];
			}
			if(root != doc){
				// there is a root element, make sure we are a child of it
				var parent = found;
				while(parent != root){
					parent = parent.parentNode;
					if(!parent){
						return [];
					}
				}
			}
			return match[3] ?
					liteEngine(match[3], found) 
					: [found];
		}
		if(match[3] && root.getElementsByClassName){
			// a .class
			return root.getElementsByClassName(match[4]);
		}
		var found;
		if(match[5]){
			// a tag
			found = root.getElementsByTagName(match[5]);
			if(match[4] || match[6]){
				selector = (match[4] || "") + match[6];
			}else{
				// that was the entirety of the query, return results
				return found;
			}
		}
	}
	if(querySelectorAll){
		// qSA works strangely on Element-rooted queries
		// We can work around this by specifying an extra ID on the root
		// and working up from there (Thanks to Andrew Dupont for the technique)
		// IE 8 doesn't work on object elements
		if (root.nodeType === 1 && root.nodeName.toLowerCase() !== "object"){				
			return useRoot(root, selector, root.querySelectorAll);
		}else{
			// we can use the native qSA
			return root.querySelectorAll(selector);
		}
	}else if(!found){
		// search all children and then filter
		found = root.getElementsByTagName("*");
	}
	// now we filter the nodes that were found using the matchesSelector
	var results = [];
	for(var i = 0, l = found.length; i < l; i++){
		var node = found[i];
		if(node.nodeType == 1 && jsMatchesSelector(node, selector, root)){
			// keep the nodes that match the selector
			results.push(node);
		}
	}
	return results;
};
var useRoot = function(context, query, method){
	// this function creates a temporary id so we can do rooted qSA queries, this is taken from sizzle
	var oldContext = context,
		old = context.getAttribute("id"),
		nid = old || "__dojo__",
		hasParent = context.parentNode,
		relativeHierarchySelector = /^\s*[+~]/.test(query);

	if(relativeHierarchySelector && !hasParent){
		return [];
	}
	if(!old){
		context.setAttribute("id", nid);
	}else{
		nid = nid.replace(/'/g, "\\$&");
	}
	if(relativeHierarchySelector && hasParent){
		context = context.parentNode;
	}
	var selectors = query.match(unionSplit);
	for(var i = 0; i < selectors.length; i++){
		selectors[i] = "[id='" + nid + "'] " + selectors[i];
	}
	query = selectors.join(",");

	try{
		return method.call(context, query);
	}finally{
		if(!old){
			oldContext.removeAttribute("id");
		}
	}
};

if(!has("dom-matches-selector")){
	var jsMatchesSelector = (function(){
		// a JS implementation of CSS selector matching, first we start with the various handlers
		var caseFix = testDiv.tagName == "div" ? "toLowerCase" : "toUpperCase";
		var selectorTypes = {
			"": function(tagName){
				tagName = tagName[caseFix]();
				return function(node){
					return node.tagName == tagName;
				};
			},
			".": function(className){
				var classNameSpaced = ' ' + className + ' ';
				return function(node){
					return node.className.indexOf(className) > -1 && (' ' + node.className + ' ').indexOf(classNameSpaced) > -1;
				};
			},
			"#": function(id){
				return function(node){
					return node.id == id;
				};
			}
		};
		var attrComparators = {
			"^=": function(attrValue, value){
				return attrValue.indexOf(value) == 0;
			},
			"*=": function(attrValue, value){
				return attrValue.indexOf(value) > -1;
			},
			"$=": function(attrValue, value){
				return attrValue.substring(attrValue.length - value.length, attrValue.length) == value;
			},
			"~=": function(attrValue, value){
				return (' ' + attrValue + ' ').indexOf(' ' + value + ' ') > -1;
			},
			"|=": function(attrValue, value){
				return (attrValue + '-').indexOf(value + '-') == 0;
			},
			"=": function(attrValue, value){
				return attrValue == value;
			},
			"": function(attrValue, value){
				return true;
			}
		};
		function attr(name, value, type){
			var firstChar = value.charAt(0);
			if(firstChar == '"' || firstChar == "'"){
				// it is quoted, remove the quotes
				value = value.slice(1, -1);
			}
			value = value.replace(/\\/g,'');
			var comparator = attrComparators[type || ""];
			return function(node){
				var attrValue = node.getAttribute(name);
				return attrValue && comparator(attrValue, value);
			};
		}
		function ancestor(matcher){
			return function(node, root){
				while((node = node.parentNode) != root){
					if(matcher(node, root)){
						return true;
					}
				}
			};
		}
		function parent(matcher){
			return function(node, root){
				node = node.parentNode;
				return matcher ? 
					node != root && matcher(node, root)
					: node == root;
			};
		}
		var cache = {};
		function and(matcher, next){
			return matcher ?
				function(node, root){
					return next(node) && matcher(node, root);
				}
				: next;
		}
		return function(node, selector, root){
			// this returns true or false based on if the node matches the selector (optionally within the given root)
			var matcher = cache[selector]; // check to see if we have created a matcher function for the given selector
			if(!matcher){
				// create a matcher function for the given selector
				// parse the selectors
				if(selector.replace(/(?:\s*([> ])\s*)|(#|\.)?((?:\\.|[\w-])+)|\[\s*([\w-]+)\s*(.?=)?\s*("(?:\\.|[^"])+"|'(?:\\.|[^'])+'|(?:\\.|[^\]])*)\s*\]/g, function(t, combinator, type, value, attrName, attrType, attrValue){
					if(value){
						matcher = and(matcher, selectorTypes[type || ""](value.replace(/\\/g, '')));
					}
					else if(combinator){
						matcher = (combinator == " " ? ancestor : parent)(matcher);
					}
					else if(attrName){
						matcher = and(matcher, attr(attrName, attrValue, attrType));
					}
					return "";
				})){
					throw new Error("Syntax error in query");
				}
				if(!matcher){
					return true;
				}
				cache[selector] = matcher;
			}
			// now run the matcher function on the node
			return matcher(node, root);
		};
	})();
}
if(!has("dom-qsa")){
	var combine = function(selector, root){
		// combined queries
		var selectors = selector.match(unionSplit);
		var indexed = [];
		// add all results and keep unique ones, this only runs in IE, so we take advantage 
		// of known IE features, particularly sourceIndex which is unique and allows us to 
		// order the results 
		for(var i = 0; i < selectors.length; i++){
			selector = new String(selectors[i].replace(/\s*$/,''));
			selector.indexOf = escape; // keep it from recursively entering combine
			var results = liteEngine(selector, root);
			for(var j = 0, l = results.length; j < l; j++){
				var node = results[j];
				indexed[node.sourceIndex] = node;
			}
		}
		// now convert from a sparse array to a dense array
		var totalResults = [];
		for(i in indexed){
			totalResults.push(indexed[i]);
		}
		return totalResults;
	};
}

liteEngine.match = matchesSelector ? function(node, selector, root){
	if(root && root.nodeType != 9){
		// doesn't support three args, use rooted id trick
		return useRoot(root, selector, function(query){
			return matchesSelector.call(node, query);
		});
	}
	// we have a native matchesSelector, use that
	return matchesSelector.call(node, selector);
} : jsMatchesSelector; // otherwise use the JS matches impl

return liteEngine;
});

},
'dojo/NodeList-dom':function(){
define(["./_base/kernel", "./query", "./_base/array", "./_base/lang", "./dom-class", "./dom-construct", "./dom-geometry", "./dom-attr", "./dom-style"], function(dojo, query, array, lang, domCls, domCtr, domGeom, domAttr, domStyle){

	// module:
	//		dojo/NodeList-dom.js

	/*=====
	 return function(){
		 // summary:
		 //		Adds DOM related methods to NodeList, and returns NodeList constructor.
	 };
	 =====*/

	var magicGuard = function(a){
		// summary:
		//		the guard function for dojo.attr() and dojo.style()
		return a.length == 1 && (typeof a[0] == "string"); // inline'd type check
	};

	var orphan = function(node){
		// summary:
		//		function to orphan nodes
		var p = node.parentNode;
		if(p){
			p.removeChild(node);
		}
	};
	// FIXME: should we move orphan() to dojo.html?

	var NodeList = query.NodeList,
		awc = NodeList._adaptWithCondition,
		aafe = NodeList._adaptAsForEach,
		aam = NodeList._adaptAsMap;

	function getSet(module){
		return function(node, name, value){
			if(arguments.length == 2){
				return module[typeof name == "string" ? "get" : "set"](node, name);
			}
			// setter
			return module.set(node, name, value);
		};
	}

	lang.extend(NodeList, {
		_normalize: function(/*String||Element||Object||NodeList*/content, /*DOMNode?*/refNode){
			// summary:
			//		normalizes data to an array of items to insert.
			// description:
			//		If content is an object, it can have special properties "template" and
			//		"parse". If "template" is defined, then the template value is run through
			//		dojo.string.substitute (if dojo/string.substitute() has been dojo.required elsewhere),
			//		or if templateFunc is a function on the content, that function will be used to
			//		transform the template into a final string to be used for for passing to dojo/dom-construct.toDom().
			//		If content.parse is true, then it is remembered for later, for when the content
			//		nodes are inserted into the DOM. At that point, the nodes will be parsed for widgets
			//		(if dojo.parser has been dojo.required elsewhere).

			//Wanted to just use a DocumentFragment, but for the array/NodeList
			//case that meant using cloneNode, but we may not want that.
			//Cloning should only happen if the node operations span
			//multiple refNodes. Also, need a real array, not a NodeList from the
			//DOM since the node movements could change those NodeLists.

			var parse = content.parse === true;

			//Do we have an object that needs to be run through a template?
			if(typeof content.template == "string"){
				var templateFunc = content.templateFunc || (dojo.string && dojo.string.substitute);
				content = templateFunc ? templateFunc(content.template, content) : content;
			}

			var type = (typeof content);
			if(type == "string" || type == "number"){
				content = domCtr.toDom(content, (refNode && refNode.ownerDocument));
				if(content.nodeType == 11){
					//DocumentFragment. It cannot handle cloneNode calls, so pull out the children.
					content = lang._toArray(content.childNodes);
				}else{
					content = [content];
				}
			}else if(!lang.isArrayLike(content)){
				content = [content];
			}else if(!lang.isArray(content)){
				//To get to this point, content is array-like, but
				//not an array, which likely means a DOM NodeList. Convert it now.
				content = lang._toArray(content);
			}

			//Pass around the parse info
			if(parse){
				content._runParse = true;
			}
			return content; //Array
		},

		_cloneNode: function(/*DOMNode*/ node){
			// summary:
			//		private utility to clone a node. Not very interesting in the vanilla
			//		dojo/NodeList case, but delegates could do interesting things like
			//		clone event handlers if that is derivable from the node.
			return node.cloneNode(true);
		},

		_place: function(/*Array*/ary, /*DOMNode*/refNode, /*String*/position, /*Boolean*/useClone){
			// summary:
			//		private utility to handle placing an array of nodes relative to another node.
			// description:
			//		Allows for cloning the nodes in the array, and for
			//		optionally parsing widgets, if ary._runParse is true.

			//Avoid a disallowed operation if trying to do an innerHTML on a non-element node.
			if(refNode.nodeType != 1 && position == "only"){
				return;
			}
			var rNode = refNode, tempNode;

			//Always cycle backwards in case the array is really a
			//DOM NodeList and the DOM operations take it out of the live collection.
			var length = ary.length;
			for(var i = length - 1; i >= 0; i--){
				var node = (useClone ? this._cloneNode(ary[i]) : ary[i]);

				//If need widget parsing, use a temp node, instead of waiting after inserting into
				//real DOM because we need to start widget parsing at one node up from current node,
				//which could cause some already parsed widgets to be parsed again.
				if(ary._runParse && dojo.parser && dojo.parser.parse){
					if(!tempNode){
						tempNode = rNode.ownerDocument.createElement("div");
					}
					tempNode.appendChild(node);
					dojo.parser.parse(tempNode);
					node = tempNode.firstChild;
					while(tempNode.firstChild){
						tempNode.removeChild(tempNode.firstChild);
					}
				}

				if(i == length - 1){
					domCtr.place(node, rNode, position);
				}else{
					rNode.parentNode.insertBefore(node, rNode);
				}
				rNode = node;
			}
		},


		position: aam(domGeom.position),
		/*=====
		position: function(){
			// summary:
			//		Returns border-box objects (x/y/w/h) of all elements in a node list
			//		as an Array (*not* a NodeList). Acts like `dojo.position`, though
			//		assumes the node passed is each node in this list.

			return dojo.map(this, dojo.position); // Array
		},
		=====*/

		attr: awc(getSet(domAttr), magicGuard),
		/*=====
		attr: function(property, value){
			// summary:
			//		gets or sets the DOM attribute for every element in the
			//		NodeList. See also `dojo.attr`
			// property: String
			//		the attribute to get/set
			// value: String?
			//		optional. The value to set the property to
			// returns:
			//		if no value is passed, the result is an array of attribute values
			//		If a value is passed, the return is this NodeList
			// example:
			//		Make all nodes with a particular class focusable:
			//	|	dojo.query(".focusable").attr("tabIndex", -1);
			// example:
			//		Disable a group of buttons:
			//	|	dojo.query("button.group").attr("disabled", true);
			// example:
			//		innerHTML can be assigned or retrieved as well:
			//	|	// get the innerHTML (as an array) for each list item
			//	|	var ih = dojo.query("li.replaceable").attr("innerHTML");
			return; // dojo/NodeList|Array
		},
		=====*/

		style: awc(getSet(domStyle), magicGuard),
		/*=====
		style: function(property, value){
			// summary:
			//		gets or sets the CSS property for every element in the NodeList
			// property: String
			//		the CSS property to get/set, in JavaScript notation
			//		("lineHieght" instead of "line-height")
			// value: String?
			//		optional. The value to set the property to
			// returns:
			//		if no value is passed, the result is an array of strings.
			//		If a value is passed, the return is this NodeList
			return; // dojo/NodeList
			return; // Array
		},
		=====*/

		addClass: aafe(domCls.add),
		/*=====
		addClass: function(className){
			// summary:
			//		adds the specified class to every node in the list
			// className: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			return; // dojo/NodeList
		},
		=====*/

		removeClass: aafe(domCls.remove),
		/*=====
		removeClass: function(className){
			// summary:
			//		removes the specified class from every node in the list
			// className: String|Array?
			//		An optional String class name to remove, or several space-separated
			//		class names, or an array of class names. If omitted, all class names
			//		will be deleted.
			// returns:
			//		this list
			return; // dojo/NodeList
		},
		=====*/

		toggleClass: aafe(domCls.toggle),
		/*=====
		toggleClass: function(className, condition){
			// summary:
			//		Adds a class to node if not present, or removes if present.
			//		Pass a boolean condition if you want to explicitly add or remove.
			// condition: Boolean?
			//		If passed, true means to add the class, false means to remove.
			// className: String
			//		the CSS class to add
			return; // dojo/NodeList
		},
		=====*/

		replaceClass: aafe(domCls.replace),
		/*=====
		replaceClass: function(addClassStr, removeClassStr){
			// summary:
			//		Replaces one or more classes on a node if not present.
			//		Operates more quickly than calling `removeClass()` and `addClass()`
			// addClassStr: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			// removeClassStr: String|Array?
			//		A String class name to remove, or several space-separated class names,
			//		or an array of class names.
			return; // dojo/NodeList
		 },
		 =====*/

		empty: aafe(domCtr.empty),
		/*=====
		empty: function(){
			// summary:
			//		clears all content from each node in the list. Effectively
			//		equivalent to removing all child nodes from every item in
			//		the list.
			return this.forEach("item.innerHTML='';"); // dojo/NodeList
			// FIXME: should we be checking for and/or disposing of widgets below these nodes?
		},
		=====*/

		removeAttr: aafe(domAttr.remove),
		/*=====
		 removeAttr: function(name){
			// summary:
			//		Removes an attribute from each node in the list.
			// name: String
			//		the name of the attribute to remove
			return;		// dojo/NodeList
		},
		=====*/

		marginBox: aam(domGeom.getMarginBox),
		/*=====
		marginBox: function(){
			// summary:
			//		Returns margin-box size of nodes
		 	return; // dojo/NodeList
		 },
		 =====*/

		// FIXME: connectPublisher()? connectRunOnce()?

		/*
		destroy: function(){
			// summary:
			//		destroys every item in the list.
			this.forEach(d.destroy);
			// FIXME: should we be checking for and/or disposing of widgets below these nodes?
		},
		*/

		place: function(/*String||Node*/ queryOrNode, /*String*/ position){
			// summary:
			//		places elements of this node list relative to the first element matched
			//		by queryOrNode. Returns the original NodeList. See: `dojo.place`
			// queryOrNode:
			//		may be a string representing any valid CSS3 selector or a DOM node.
			//		In the selector case, only the first matching element will be used
			//		for relative positioning.
			// position:
			//		can be one of:
			//
			//		-	"last" (default)
			//		-	"first"
			//		-	"before"
			//		-	"after"
			//		-	"only"
			//		-	"replace"
			//
			//		or an offset in the childNodes property
			var item = query(queryOrNode)[0];
			return this.forEach(function(node){ domCtr.place(node, item, position); }); // dojo/NodeList
		},

		orphan: function(/*String?*/ filter){
			// summary:
			//		removes elements in this list that match the filter
			//		from their parents and returns them as a new NodeList.
			// filter:
			//		CSS selector like ".foo" or "div > span"
			// returns:
			//		NodeList containing the orphaned elements
			return (filter ? query._filterResult(this, filter) : this).forEach(orphan); // dojo/NodeList
		},

		adopt: function(/*String||Array||DomNode*/ queryOrListOrNode, /*String?*/ position){
			// summary:
			//		places any/all elements in queryOrListOrNode at a
			//		position relative to the first element in this list.
			//		Returns a dojo/NodeList of the adopted elements.
			// queryOrListOrNode:
			//		a DOM node or a query string or a query result.
			//		Represents the nodes to be adopted relative to the
			//		first element of this NodeList.
			// position:
			//		can be one of:
			//
			//		-	"last" (default)
			//		-	"first"
			//		-	"before"
			//		-	"after"
			//		-	"only"
			//		-	"replace"
			//
			//		or an offset in the childNodes property
			return query(queryOrListOrNode).place(this[0], position)._stash(this);	// dojo/NodeList
		},

		// FIXME: do we need this?
		query: function(/*String*/ queryStr){
			// summary:
			//		Returns a new list whose members match the passed query,
			//		assuming elements of the current NodeList as the root for
			//		each search.
			// example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo">
			//	|		<p>
			//	|			bacon is tasty, <span>dontcha think?</span>
			//	|		</p>
			//	|	</div>
			//	|	<div id="bar">
			//	|		<p>great comedians may not be funny <span>in person</span></p>
			//	|	</div>
			//		If we are presented with the following definition for a NodeList:
			//	|	var l = new NodeList(dojo.byId("foo"), dojo.byId("bar"));
			//		it's possible to find all span elements under paragraphs
			//		contained by these elements with this sub-query:
			//	|	var spans = l.query("p span");

			// FIXME: probably slow
			if(!queryStr){ return this; }
			var ret = new NodeList;
			this.map(function(node){
				// FIXME: why would we ever get undefined here?
				query(queryStr, node).forEach(function(subNode){
					if(subNode !== undefined){
						ret.push(subNode);
					}
				});
			});
			return ret._stash(this);	// dojo/NodeList
		},

		filter: function(/*String|Function*/ filter){
			// summary:
			//		"masks" the built-in javascript filter() method (supported
			//		in Dojo via `dojo.filter`) to support passing a simple
			//		string filter in addition to supporting filtering function
			//		objects.
			// filter:
			//		If a string, a CSS rule like ".thinger" or "div > span".
			// example:
			//		"regular" JS filter syntax as exposed in dojo.filter:
			//		|	dojo.query("*").filter(function(item){
			//		|		// highlight every paragraph
			//		|		return (item.nodeName == "p");
			//		|	}).style("backgroundColor", "yellow");
			// example:
			//		the same filtering using a CSS selector
			//		|	dojo.query("*").filter("p").styles("backgroundColor", "yellow");

			var a = arguments, items = this, start = 0;
			if(typeof filter == "string"){ // inline'd type check
				items = query._filterResult(this, a[0]);
				if(a.length == 1){
					// if we only got a string query, pass back the filtered results
					return items._stash(this); // dojo/NodeList
				}
				// if we got a callback, run it over the filtered items
				start = 1;
			}
			return this._wrap(array.filter(items, a[start], a[start + 1]), this);	// dojo/NodeList
		},

		/*
		// FIXME: should this be "copyTo" and include parenting info?
		clone: function(){
			// summary:
			//		creates node clones of each element of this list
			//		and returns a new list containing the clones
		},
		*/

		addContent: function(/*String||DomNode||Object||dojo/NodeList*/ content, /*String||Integer?*/ position){
			// summary:
			//		add a node, NodeList or some HTML as a string to every item in the
			//		list.  Returns the original list.
			// description:
			//		a copy of the HTML content is added to each item in the
			//		list, with an optional position argument. If no position
			//		argument is provided, the content is appended to the end of
			//		each item.
			// content:
			//		DOM node, HTML in string format, a NodeList or an Object. If a DOM node or
			//		NodeList, the content will be cloned if the current NodeList has more than one
			//		element. Only the DOM nodes are cloned, no event handlers. If it is an Object,
			//		it should be an object with at "template" String property that has the HTML string
			//		to insert. If dojo.string has already been dojo.required, then dojo.string.substitute
			//		will be used on the "template" to generate the final HTML string. Other allowed
			//		properties on the object are: "parse" if the HTML
			//		string should be parsed for widgets (dojo.require("dojo.parser") to get that
			//		option to work), and "templateFunc" if a template function besides dojo.string.substitute
			//		should be used to transform the "template".
			// position:
			//		can be one of:
			//
			//		-	"last"||"end" (default)
			//		-	"first||"start"
			//		-	"before"
			//		-	"after"
			//		-	"replace" (replaces nodes in this NodeList with new content)
			//		-	"only" (removes other children of the nodes so new content is the only child)
			//
			//		or an offset in the childNodes property
			// example:
			//		appends content to the end if the position is omitted
			//	|	dojo.query("h3 > p").addContent("hey there!");
			// example:
			//		add something to the front of each element that has a
			//		"thinger" property:
			//	|	dojo.query("[thinger]").addContent("...", "first");
			// example:
			//		adds a header before each element of the list
			//	|	dojo.query(".note").addContent("<h4>NOTE:</h4>", "before");
			// example:
			//		add a clone of a DOM node to the end of every element in
			//		the list, removing it from its existing parent.
			//	|	dojo.query(".note").addContent(dojo.byId("foo"));
			// example:
			//		Append nodes from a templatized string.
			// |	dojo.require("dojo.string");
			// |	dojo.query(".note").addContent({
			// |		template: '<b>${id}: </b><span>${name}</span>',
			// |		id: "user332",
			// |		name: "Mr. Anderson"
			// |	});
			// example:
			//		Append nodes from a templatized string that also has widgets parsed.
			// |	dojo.require("dojo.string");
			// |	dojo.require("dojo.parser");
			// |	var notes = dojo.query(".note").addContent({
			// |		template: '<button dojoType="dijit/form/Button">${text}</button>',
			// |		parse: true,
			// |		text: "Send"
			// |	});
			content = this._normalize(content, this[0]);
			for(var i = 0, node; (node = this[i]); i++){
				if(content.length){
					this._place(content, node, position, i > 0);
				}else{
					// if it is an empty array, we empty the target node
					domCtr.empty(node);
				}
			}
			return this; // dojo/NodeList
		}
	});

	return NodeList;
});

},
'dojo/_base/xhr':function(){
define([
	"./kernel",
	"./sniff",
	"require",
	"../io-query",
	/*===== "./declare", =====*/
	"../dom",
	"../dom-form",
	"./Deferred",
	"./config",
	"./json",
	"./lang",
	"./array",
	"../on",
	"../aspect",
	"../request/watch",
	"../request/xhr",
	"../request/util"
], function(dojo, has, require, ioq, /*===== declare, =====*/ dom, domForm, Deferred, config, json, lang, array, on, aspect, watch, _xhr, util){
	// module:
	//		dojo/_base/xhr

	/*=====
	dojo._xhrObj = function(){
		// summary:
		//		does the work of portably generating a new XMLHTTPRequest object.
	};
	=====*/
	dojo._xhrObj = _xhr._create;

	var cfg = dojo.config;

	// mix in io-query and dom-form
	dojo.objectToQuery = ioq.objectToQuery;
	dojo.queryToObject = ioq.queryToObject;
	dojo.fieldToObject = domForm.fieldToObject;
	dojo.formToObject = domForm.toObject;
	dojo.formToQuery = domForm.toQuery;
	dojo.formToJson = domForm.toJson;

	// need to block async callbacks from snatching this thread as the result
	// of an async callback might call another sync XHR, this hangs khtml forever
	// must checked by watchInFlight()

	dojo._blockAsync = false;

	// MOW: remove dojo._contentHandlers alias in 2.0
	var handlers = dojo._contentHandlers = dojo.contentHandlers = {
		// summary:
		//		A map of available XHR transport handle types. Name matches the
		//		`handleAs` attribute passed to XHR calls.
		// description:
		//		A map of available XHR transport handle types. Name matches the
		//		`handleAs` attribute passed to XHR calls. Each contentHandler is
		//		called, passing the xhr object for manipulation. The return value
		//		from the contentHandler will be passed to the `load` or `handle`
		//		functions defined in the original xhr call.
		// example:
		//		Creating a custom content-handler:
		//	|	xhr.contentHandlers.makeCaps = function(xhr){
		//	|		return xhr.responseText.toUpperCase();
		//	|	}
		//	|	// and later:
		//	|	dojo.xhrGet({
		//	|		url:"foo.txt",
		//	|		handleAs:"makeCaps",
		//	|		load: function(data){ /* data is a toUpper version of foo.txt */ }
		//	|	});

		"text": function(xhr){
			// summary:
			//		A contentHandler which simply returns the plaintext response data
			return xhr.responseText;
		},
		"json": function(xhr){
			// summary:
			//		A contentHandler which returns a JavaScript object created from the response data
			return json.fromJson(xhr.responseText || null);
		},
		"json-comment-filtered": function(xhr){
			// summary:
			//		A contentHandler which expects comment-filtered JSON.
			// description:
			//		A contentHandler which expects comment-filtered JSON.
			//		the json-comment-filtered option was implemented to prevent
			//		"JavaScript Hijacking", but it is less secure than standard JSON. Use
			//		standard JSON instead. JSON prefixing can be used to subvert hijacking.
			//
			//		Will throw a notice suggesting to use application/json mimetype, as
			//		json-commenting can introduce security issues. To decrease the chances of hijacking,
			//		use the standard `json` contentHandler, and prefix your "JSON" with: {}&&
			//
			//		use djConfig.useCommentedJson = true to turn off the notice
			if(!config.useCommentedJson){
				console.warn("Consider using the standard mimetype:application/json."
					+ " json-commenting can introduce security issues. To"
					+ " decrease the chances of hijacking, use the standard the 'json' handler and"
					+ " prefix your json with: {}&&\n"
					+ "Use djConfig.useCommentedJson=true to turn off this message.");
			}

			var value = xhr.responseText;
			var cStartIdx = value.indexOf("\/*");
			var cEndIdx = value.lastIndexOf("*\/");
			if(cStartIdx == -1 || cEndIdx == -1){
				throw new Error("JSON was not comment filtered");
			}
			return json.fromJson(value.substring(cStartIdx+2, cEndIdx));
		},
		"javascript": function(xhr){
			// summary:
			//		A contentHandler which evaluates the response data, expecting it to be valid JavaScript

			// FIXME: try Moz and IE specific eval variants?
			return dojo.eval(xhr.responseText);
		},
		"xml": function(xhr){
			// summary:
			//		A contentHandler returning an XML Document parsed from the response data
			var result = xhr.responseXML;

			if(result && has("dom-qsa2.1") && !result.querySelectorAll && has("dom-parser")){
				// http://bugs.dojotoolkit.org/ticket/15631
				// IE9 supports a CSS3 querySelectorAll implementation, but the DOM implementation 
				// returned by IE9 xhr.responseXML does not. Manually create the XML DOM to gain 
				// the fuller-featured implementation and avoid bugs caused by the inconsistency
				result = new DOMParser().parseFromString(xhr.responseText, "application/xml");
			}

			if(has("ie")){
				if((!result || !result.documentElement)){
					//WARNING: this branch used by the xml handling in dojo.io.iframe,
					//so be sure to test dojo.io.iframe if making changes below.
					var ms = function(n){ return "MSXML" + n + ".DOMDocument"; };
					var dp = ["Microsoft.XMLDOM", ms(6), ms(4), ms(3), ms(2)];
					array.some(dp, function(p){
						try{
							var dom = new ActiveXObject(p);
							dom.async = false;
							dom.loadXML(xhr.responseText);
							result = dom;
						}catch(e){ return false; }
						return true;
					});
				}
			}
			return result; // DOMDocument
		},
		"json-comment-optional": function(xhr){
			// summary:
			//		A contentHandler which checks the presence of comment-filtered JSON and
			//		alternates between the `json` and `json-comment-filtered` contentHandlers.
			if(xhr.responseText && /^[^{\[]*\/\*/.test(xhr.responseText)){
				return handlers["json-comment-filtered"](xhr);
			}else{
				return handlers["json"](xhr);
			}
		}
	};

	/*=====

	// kwargs function parameter definitions.   Assigning to dojo namespace rather than making them local variables
	// because they are used by dojo/io modules too

	dojo.__IoArgs = declare(null, {
		// url: String
		//		URL to server endpoint.
		// content: Object?
		//		Contains properties with string values. These
		//		properties will be serialized as name1=value2 and
		//		passed in the request.
		// timeout: Integer?
		//		Milliseconds to wait for the response. If this time
		//		passes, the then error callbacks are called.
		// form: DOMNode?
		//		DOM node for a form. Used to extract the form values
		//		and send to the server.
		// preventCache: Boolean?
		//		Default is false. If true, then a
		//		"dojo.preventCache" parameter is sent in the request
		//		with a value that changes with each request
		//		(timestamp). Useful only with GET-type requests.
		// handleAs: String?
		//		Acceptable values depend on the type of IO
		//		transport (see specific IO calls for more information).
		// rawBody: String?
		//		Sets the raw body for an HTTP request. If this is used, then the content
		//		property is ignored. This is mostly useful for HTTP methods that have
		//		a body to their requests, like PUT or POST. This property can be used instead
		//		of postData and putData for dojo/_base/xhr.rawXhrPost and dojo/_base/xhr.rawXhrPut respectively.
		// ioPublish: Boolean?
		//		Set this explicitly to false to prevent publishing of topics related to
		//		IO operations. Otherwise, if djConfig.ioPublish is set to true, topics
		//		will be published via dojo/topic.publish() for different phases of an IO operation.
		//		See dojo/main.__IoPublish for a list of topics that are published.

		load: function(response, ioArgs){
			// summary:
			//		This function will be
			//		called on a successful HTTP response code.
	 		// ioArgs: dojo/main.__IoCallbackArgs
			//		Provides additional information about the request.
			// response: Object
			//		The response in the format as defined with handleAs.
		},

		error: function(response, ioArgs){
			// summary:
			//		This function will
			//		be called when the request fails due to a network or server error, the url
			//		is invalid, etc. It will also be called if the load or handle callback throws an
			//		exception, unless djConfig.debugAtAllCosts is true.	 This allows deployed applications
			//		to continue to run even when a logic error happens in the callback, while making
			//		it easier to troubleshoot while in debug mode.
			// ioArgs: dojo/main.__IoCallbackArgs
			//		Provides additional information about the request.
			// response: Object
			//		The response in the format as defined with handleAs.
		},

		handle: function(loadOrError, response, ioArgs){
			// summary:
	 		//		This function will
	 		//		be called at the end of every request, whether or not an error occurs.
			// loadOrError: String
			//		Provides a string that tells you whether this function
			//		was called because of success (load) or failure (error).
			// response: Object
			//		The response in the format as defined with handleAs.
			// ioArgs: dojo/main.__IoCallbackArgs
			//		Provides additional information about the request.
		}
	});

	dojo.__IoCallbackArgs = declare(null, {
		// args: Object
		//		the original object argument to the IO call.
		// xhr: XMLHttpRequest
		//		For XMLHttpRequest calls only, the
		//		XMLHttpRequest object that was used for the
		//		request.
		// url: String
		//		The final URL used for the call. Many times it
		//		will be different than the original args.url
		//		value.
		// query: String
		//		For non-GET requests, the
		//		name1=value1&name2=value2 parameters sent up in
		//		the request.
		// handleAs: String
		//		The final indicator on how the response will be
		//		handled.
		// id: String
		//		For dojo/io/script calls only, the internal
		//		script ID used for the request.
		// canDelete: Boolean
		//		For dojo/io/script calls only, indicates
		//		whether the script tag that represents the
		//		request can be deleted after callbacks have
		//		been called. Used internally to know when
		//		cleanup can happen on JSONP-type requests.
		// json: Object
		//		For dojo/io/script calls only: holds the JSON
		//		response for JSONP-type requests. Used
		//		internally to hold on to the JSON responses.
		//		You should not need to access it directly --
		//		the same object should be passed to the success
		//		callbacks directly.
	});

	dojo.__IoPublish = declare(null, {
		// summary:
		//		This is a list of IO topics that can be published
		//		if djConfig.ioPublish is set to true. IO topics can be
		//		published for any Input/Output, network operation. So,
		//		dojo.xhr, dojo.io.script and dojo.io.iframe can all
		//		trigger these topics to be published.
		// start: String
		//		"/dojo/io/start" is sent when there are no outstanding IO
		//		requests, and a new IO request is started. No arguments
		//		are passed with this topic.
		// send: String
		//		"/dojo/io/send" is sent whenever a new IO request is started.
		//		It passes the dojo.Deferred for the request with the topic.
		// load: String
		//		"/dojo/io/load" is sent whenever an IO request has loaded
		//		successfully. It passes the response and the dojo.Deferred
		//		for the request with the topic.
		// error: String
		//		"/dojo/io/error" is sent whenever an IO request has errored.
		//		It passes the error and the dojo.Deferred
		//		for the request with the topic.
		// done: String
		//		"/dojo/io/done" is sent whenever an IO request has completed,
		//		either by loading or by erroring. It passes the error and
		//		the dojo.Deferred for the request with the topic.
		// stop: String
		//		"/dojo/io/stop" is sent when all outstanding IO requests have
		//		finished. No arguments are passed with this topic.
	});
	=====*/


	dojo._ioSetArgs = function(/*dojo/main.__IoArgs*/args,
			/*Function*/canceller,
			/*Function*/okHandler,
			/*Function*/errHandler){
		// summary:
		//		sets up the Deferred and ioArgs property on the Deferred so it
		//		can be used in an io call.
		// args:
		//		The args object passed into the public io call. Recognized properties on
		//		the args object are:
		// canceller:
		//		The canceller function used for the Deferred object. The function
		//		will receive one argument, the Deferred object that is related to the
		//		canceller.
		// okHandler:
		//		The first OK callback to be registered with Deferred. It has the opportunity
		//		to transform the OK response. It will receive one argument -- the Deferred
		//		object returned from this function.
		// errHandler:
		//		The first error callback to be registered with Deferred. It has the opportunity
		//		to do cleanup on an error. It will receive two arguments: error (the
		//		Error object) and dfd, the Deferred object returned from this function.

		var ioArgs = {args: args, url: args.url};

		//Get values from form if requested.
		var formObject = null;
		if(args.form){
			var form = dom.byId(args.form);
			//IE requires going through getAttributeNode instead of just getAttribute in some form cases,
			//so use it for all. See #2844
			var actnNode = form.getAttributeNode("action");
			ioArgs.url = ioArgs.url || (actnNode ? actnNode.value : null);
			formObject = domForm.toObject(form);
		}

		// set up the query params
		var miArgs = [{}];

		if(formObject){
			// potentially over-ride url-provided params w/ form values
			miArgs.push(formObject);
		}
		if(args.content){
			// stuff in content over-rides what's set by form
			miArgs.push(args.content);
		}
		if(args.preventCache){
			miArgs.push({"dojo.preventCache": new Date().valueOf()});
		}
		ioArgs.query = ioq.objectToQuery(lang.mixin.apply(null, miArgs));

		// .. and the real work of getting the deferred in order, etc.
		ioArgs.handleAs = args.handleAs || "text";
		var d = new Deferred(function(dfd){
			dfd.canceled = true;
			canceller && canceller(dfd);

			var err = dfd.ioArgs.error;
			if(!err){
				err = new Error("request cancelled");
				err.dojoType="cancel";
				dfd.ioArgs.error = err;
			}
			return err;
		});
		d.addCallback(okHandler);

		//Support specifying load, error and handle callback functions from the args.
		//For those callbacks, the "this" object will be the args object.
		//The callbacks will get the deferred result value as the
		//first argument and the ioArgs object as the second argument.
		var ld = args.load;
		if(ld && lang.isFunction(ld)){
			d.addCallback(function(value){
				return ld.call(args, value, ioArgs);
			});
		}
		var err = args.error;
		if(err && lang.isFunction(err)){
			d.addErrback(function(value){
				return err.call(args, value, ioArgs);
			});
		}
		var handle = args.handle;
		if(handle && lang.isFunction(handle)){
			d.addBoth(function(value){
				return handle.call(args, value, ioArgs);
			});
		}

		// Attach error handler last (not including topic publishing)
		// to catch any errors that may have been generated from load
		// or handle functions.
		d.addErrback(function(error){
			return errHandler(error, d);
		});

		//Plug in topic publishing, if dojo.publish is loaded.
		if(cfg.ioPublish && dojo.publish && ioArgs.args.ioPublish !== false){
			d.addCallbacks(
				function(res){
					dojo.publish("/dojo/io/load", [d, res]);
					return res;
				},
				function(res){
					dojo.publish("/dojo/io/error", [d, res]);
					return res;
				}
			);
			d.addBoth(function(res){
				dojo.publish("/dojo/io/done", [d, res]);
				return res;
			});
		}

		d.ioArgs = ioArgs;

		// FIXME: need to wire up the xhr object's abort method to something
		// analogous in the Deferred
		return d;
	};

	var _deferredOk = function(/*Deferred*/dfd){
		// summary:
		//		okHandler function for dojo._ioSetArgs call.

		var ret = handlers[dfd.ioArgs.handleAs](dfd.ioArgs.xhr);
		return ret === undefined ? null : ret;
	};
	var _deferError = function(/*Error*/error, /*Deferred*/dfd){
		// summary:
		//		errHandler function for dojo._ioSetArgs call.

		if(!dfd.ioArgs.args.failOk){
			console.error(error);
		}
		return error;
	};

	//Use a separate count for knowing if we are starting/stopping io calls.
	var _checkPubCount = function(dfd){
		if(_pubCount <= 0){
			_pubCount = 0;
			if(cfg.ioPublish && dojo.publish && (!dfd || dfd && dfd.ioArgs.args.ioPublish !== false)){
				dojo.publish("/dojo/io/stop");
			}
		}
	};

	var _pubCount = 0;
	aspect.after(watch, "_onAction", function(){
		_pubCount -= 1;
	});
	aspect.after(watch, "_onInFlight", _checkPubCount);

	dojo._ioCancelAll = watch.cancelAll;
	/*=====
	dojo._ioCancelAll = function(){
		// summary:
		//		Cancels all pending IO requests, regardless of IO type
		//		(xhr, script, iframe).
	};
	=====*/

	dojo._ioNotifyStart = function(/*Deferred*/dfd){
		// summary:
		//		If dojo.publish is available, publish topics
		//		about the start of a request queue and/or the
		//		the beginning of request.
		//
		//		Used by IO transports. An IO transport should
		//		call this method before making the network connection.
		if(cfg.ioPublish && dojo.publish && dfd.ioArgs.args.ioPublish !== false){
			if(!_pubCount){
				dojo.publish("/dojo/io/start");
			}
			_pubCount += 1;
			dojo.publish("/dojo/io/send", [dfd]);
		}
	};

	dojo._ioWatch = function(dfd, validCheck, ioCheck, resHandle){
		// summary:
		//		Watches the io request represented by dfd to see if it completes.
		// dfd: Deferred
		//		The Deferred object to watch.
		// validCheck: Function
		//		Function used to check if the IO request is still valid. Gets the dfd
		//		object as its only argument.
		// ioCheck: Function
		//		Function used to check if basic IO call worked. Gets the dfd
		//		object as its only argument.
		// resHandle: Function
		//		Function used to process response. Gets the dfd
		//		object as its only argument.

		var args = dfd.ioArgs.options = dfd.ioArgs.args;
		lang.mixin(dfd, {
			response: dfd.ioArgs,
			isValid: function(response){
				return validCheck(dfd);
			},
			isReady: function(response){
				return ioCheck(dfd);
			},
			handleResponse: function(response){
				return resHandle(dfd);
			}
		});
		watch(dfd);

		_checkPubCount(dfd);
	};

	var _defaultContentType = "application/x-www-form-urlencoded";

	dojo._ioAddQueryToUrl = function(/*dojo.__IoCallbackArgs*/ioArgs){
		// summary:
		//		Adds query params discovered by the io deferred construction to the URL.
		//		Only use this for operations which are fundamentally GET-type operations.
		if(ioArgs.query.length){
			ioArgs.url += (ioArgs.url.indexOf("?") == -1 ? "?" : "&") + ioArgs.query;
			ioArgs.query = null;
		}
	};

	/*=====
	dojo.__XhrArgs = declare(dojo.__IoArgs, {
		// summary:
		//		In addition to the properties listed for the dojo._IoArgs type,
		//		the following properties are allowed for dojo.xhr* methods.
		// handleAs: String?
		//		Acceptable values are: text (default), json, json-comment-optional,
		//		json-comment-filtered, javascript, xml. See `dojo/_base/xhr.contentHandlers`
	 	// sync: Boolean?
		//		false is default. Indicates whether the request should
		//		be a synchronous (blocking) request.
		// headers: Object?
		//		Additional HTTP headers to send in the request.
		// failOk: Boolean?
		//		false is default. Indicates whether a request should be
		//		allowed to fail (and therefore no console error message in
		//		the event of a failure)
		// contentType: String|Boolean
		//		"application/x-www-form-urlencoded" is default. Set to false to
		//		prevent a Content-Type header from being sent, or to a string
		//		to send a different Content-Type.
	 });
	=====*/

	dojo.xhr = function(/*String*/ method, /*dojo.__XhrArgs*/ args, /*Boolean?*/ hasBody){
		// summary:
		//		Deprecated.   Use dojo/request instead.
		// description:
		//		Sends an HTTP request with the given method.
		//		See also dojo.xhrGet(), xhrPost(), xhrPut() and dojo.xhrDelete() for shortcuts
		//		for those HTTP methods. There are also methods for "raw" PUT and POST methods
		//		via dojo.rawXhrPut() and dojo.rawXhrPost() respectively.
		// method:
		//		HTTP method to be used, such as GET, POST, PUT, DELETE. Should be uppercase.
		// hasBody:
		//		If the request has an HTTP body, then pass true for hasBody.

		var rDfd;
		//Make the Deferred object for this xhr request.
		var dfd = dojo._ioSetArgs(args, function(dfd){
			rDfd && rDfd.cancel();
		}, _deferredOk, _deferError);
		var ioArgs = dfd.ioArgs;

		//Allow for specifying the HTTP body completely.
		if("postData" in args){
			ioArgs.query = args.postData;
		}else if("putData" in args){
			ioArgs.query = args.putData;
		}else if("rawBody" in args){
			ioArgs.query = args.rawBody;
		}else if((arguments.length > 2 && !hasBody) || "POST|PUT".indexOf(method.toUpperCase()) === -1){
			//Check for hasBody being passed. If no hasBody,
			//then only append query string if not a POST or PUT request.
			dojo._ioAddQueryToUrl(ioArgs);
		}

		var options = {
			method: method,
			handleAs: "text",
			timeout: args.timeout,
			withCredentials: args.withCredentials,
			ioArgs: ioArgs
		};

		if(typeof args.headers !== 'undefined'){
			options.headers = args.headers;
		}
		if(typeof args.contentType !== 'undefined'){
			if(!options.headers){
				options.headers = {};
			}
			options.headers['Content-Type'] = args.contentType;
		}
		if(typeof ioArgs.query !== 'undefined'){
			options.data = ioArgs.query;
		}
		if(typeof args.sync !== 'undefined'){
			options.sync = args.sync;
		}

		dojo._ioNotifyStart(dfd);
		try{
			rDfd = _xhr(ioArgs.url, options, true);
		}catch(e){
			// If XHR creation fails, dojo/request/xhr throws
			// When this happens, cancel the deferred
			dfd.cancel();
			return dfd;
		}

		// sync ioArgs
		dfd.ioArgs.xhr = rDfd.response.xhr;

		rDfd.then(function(){
			dfd.resolve(dfd);
		}).otherwise(function(error){
			ioArgs.error = error;
			if(error.response){
				error.status = error.response.status;
				error.responseText = error.response.text;
				error.xhr = error.response.xhr;
			}
			dfd.reject(error);
		});
		return dfd; // dojo/_base/Deferred
	};

	dojo.xhrGet = function(/*dojo.__XhrArgs*/ args){
		// summary:
		//		Sends an HTTP GET request to the server.
		return dojo.xhr("GET", args); // dojo/_base/Deferred
	};

	dojo.rawXhrPost = dojo.xhrPost = function(/*dojo.__XhrArgs*/ args){
		// summary:
		//		Sends an HTTP POST request to the server. In addition to the properties
		//		listed for the dojo.__XhrArgs type, the following property is allowed:
		// postData:
		//		String. Send raw data in the body of the POST request.
		return dojo.xhr("POST", args, true); // dojo/_base/Deferred
	};

	dojo.rawXhrPut = dojo.xhrPut = function(/*dojo.__XhrArgs*/ args){
		// summary:
		//		Sends an HTTP PUT request to the server. In addition to the properties
		//		listed for the dojo.__XhrArgs type, the following property is allowed:
		// putData:
		//		String. Send raw data in the body of the PUT request.
		return dojo.xhr("PUT", args, true); // dojo/_base/Deferred
	};

	dojo.xhrDelete = function(/*dojo.__XhrArgs*/ args){
		// summary:
		//		Sends an HTTP DELETE request to the server.
		return dojo.xhr("DELETE", args); // dojo/_base/Deferred
	};

	/*
	dojo.wrapForm = function(formNode){
		// summary:
		//		A replacement for FormBind, but not implemented yet.

		// FIXME: need to think harder about what extensions to this we might
		// want. What should we allow folks to do w/ this? What events to
		// set/send?
		throw new Error("dojo.wrapForm not yet implemented");
	}
	*/

	dojo._isDocumentOk = function(x){
		return util.checkStatus(x.status);
	};

	dojo._getText = function(url){
		var result;
		dojo.xhrGet({url:url, sync:true, load:function(text){
			result = text;
		}});
		return result;
	};

	// Add aliases for static functions to dojo.xhr since dojo.xhr is what's returned from this module
	lang.mixin(dojo.xhr, {
		_xhrObj: dojo._xhrObj,
		fieldToObject: domForm.fieldToObject,
		formToObject: domForm.toObject,
		objectToQuery: ioq.objectToQuery,
		formToQuery: domForm.toQuery,
		formToJson: domForm.toJson,
		queryToObject: ioq.queryToObject,
		contentHandlers: handlers,
		_ioSetArgs: dojo._ioSetArgs,
		_ioCancelAll: dojo._ioCancelAll,
		_ioNotifyStart: dojo._ioNotifyStart,
		_ioWatch: dojo._ioWatch,
		_ioAddQueryToUrl: dojo._ioAddQueryToUrl,
		_isDocumentOk: dojo._isDocumentOk,
		_getText: dojo._getText,
		get: dojo.xhrGet,
		post: dojo.xhrPost,
		put: dojo.xhrPut,
		del: dojo.xhrDelete	// because "delete" is a reserved word
	});

	return dojo.xhr;
});

},
'dojo/io-query':function(){
define(["./_base/lang"], function(lang){

// module:
//		dojo/io-query

var backstop = {};

return {
// summary:
//		This module defines query string processing functions.

	objectToQuery: function objectToQuery(/*Object*/ map){
		// summary:
        //		takes a name/value mapping object and returns a string representing
        //		a URL-encoded version of that object.
        // example:
        //		this object:
        //
        //	|	{
        //	|		blah: "blah",
        //	|		multi: [
        //	|			"thud",
        //	|			"thonk"
        //	|		]
        //	|	};
        //
        //		yields the following query string:
        //
        //	|	"blah=blah&multi=thud&multi=thonk"

        // FIXME: need to implement encodeAscii!!
        var enc = encodeURIComponent, pairs = [];
        for(var name in map){
            var value = map[name];
            if(value != backstop[name]){
                var assign = enc(name) + "=";
                if(lang.isArray(value)){
                    for(var i = 0, l = value.length; i < l; ++i){
                        pairs.push(assign + enc(value[i]));
                    }
                }else{
                    pairs.push(assign + enc(value));
                }
            }
        }
        return pairs.join("&"); // String
    },

	queryToObject: function queryToObject(/*String*/ str){
        // summary:
        //		Create an object representing a de-serialized query section of a
        //		URL. Query keys with multiple values are returned in an array.
        //
        // example:
        //		This string:
        //
        //	|		"foo=bar&foo=baz&thinger=%20spaces%20=blah&zonk=blarg&"
        //
        //		results in this object structure:
        //
        //	|		{
        //	|			foo: [ "bar", "baz" ],
        //	|			thinger: " spaces =blah",
        //	|			zonk: "blarg"
        //	|		}
        //
        //		Note that spaces and other urlencoded entities are correctly
        //		handled.

        // FIXME: should we grab the URL string if we're not passed one?
        var dec = decodeURIComponent, qp = str.split("&"), ret = {}, name, val;
        for(var i = 0, l = qp.length, item; i < l; ++i){
            item = qp[i];
            if(item.length){
                var s = item.indexOf("=");
                if(s < 0){
                    name = dec(item);
                    val = "";
                }else{
                    name = dec(item.slice(0, s));
                    val  = dec(item.slice(s + 1));
                }
                if(typeof ret[name] == "string"){ // inline'd type check
                    ret[name] = [ret[name]];
                }

                if(lang.isArray(ret[name])){
                    ret[name].push(val);
                }else{
                    ret[name] = val;
                }
            }
        }
        return ret; // Object
    }
};
});
},
'dojo/dom-form':function(){
define(["./_base/lang", "./dom", "./io-query", "./json"], function(lang, dom, ioq, json){
	// module:
	//		dojo/dom-form

    function setValue(/*Object*/ obj, /*String*/ name, /*String*/ value){
        // summary:
        //		For the named property in object, set the value. If a value
        //		already exists and it is a string, convert the value to be an
        //		array of values.

        // Skip it if there is no value
        if(value === null){
            return;
        }

        var val = obj[name];
        if(typeof val == "string"){ // inline'd type check
            obj[name] = [val, value];
        }else if(lang.isArray(val)){
            val.push(value);
        }else{
            obj[name] = value;
        }
    }

	var exclude = "file|submit|image|reset|button";

	var form = {
		// summary:
		//		This module defines form-processing functions.

		fieldToObject: function fieldToObject(/*DOMNode|String*/ inputNode){
			// summary:
			//		Serialize a form field to a JavaScript object.
			// description:
			//		Returns the value encoded in a form field as
			//		as a string or an array of strings. Disabled form elements
			//		and unchecked radio and checkboxes are skipped.	Multi-select
			//		elements are returned as an array of string values.
			// inputNode: DOMNode|String
			// returns: Object

			var ret = null;
			inputNode = dom.byId(inputNode);
			if(inputNode){
				var _in = inputNode.name, type = (inputNode.type || "").toLowerCase();
				if(_in && type && !inputNode.disabled){
					if(type == "radio" || type == "checkbox"){
						if(inputNode.checked){
							ret = inputNode.value;
						}
					}else if(inputNode.multiple){
						ret = [];
						var nodes = [inputNode.firstChild];
						while(nodes.length){
							for(var node = nodes.pop(); node; node = node.nextSibling){
								if(node.nodeType == 1 && node.tagName.toLowerCase() == "option"){
									if(node.selected){
										ret.push(node.value);
									}
								}else{
									if(node.nextSibling){
										nodes.push(node.nextSibling);
									}
									if(node.firstChild){
										nodes.push(node.firstChild);
									}
									break;
								}
							}
						}
					}else{
						ret = inputNode.value;
					}
				}
			}
			return ret; // Object
		},

		toObject: function formToObject(/*DOMNode|String*/ formNode){
			// summary:
			//		Serialize a form node to a JavaScript object.
			// description:
			//		Returns the values encoded in an HTML form as
			//		string properties in an object which it then returns. Disabled form
			//		elements, buttons, and other non-value form elements are skipped.
			//		Multi-select elements are returned as an array of string values.
			// formNode: DOMNode|String
			// example:
			//		This form:
			//		|	<form id="test_form">
			//		|		<input type="text" name="blah" value="blah">
			//		|		<input type="text" name="no_value" value="blah" disabled>
			//		|		<input type="button" name="no_value2" value="blah">
			//		|		<select type="select" multiple name="multi" size="5">
			//		|			<option value="blah">blah</option>
			//		|			<option value="thud" selected>thud</option>
			//		|			<option value="thonk" selected>thonk</option>
			//		|		</select>
			//		|	</form>
			//
			//		yields this object structure as the result of a call to
			//		formToObject():
			//
			//		|	{
			//		|		blah: "blah",
			//		|		multi: [
			//		|			"thud",
			//		|			"thonk"
			//		|		]
			//		|	};

			var ret = {}, elems = dom.byId(formNode).elements;
			for(var i = 0, l = elems.length; i < l; ++i){
				var item = elems[i], _in = item.name, type = (item.type || "").toLowerCase();
				if(_in && type && exclude.indexOf(type) < 0 && !item.disabled){
					setValue(ret, _in, form.fieldToObject(item));
					if(type == "image"){
						ret[_in + ".x"] = ret[_in + ".y"] = ret[_in].x = ret[_in].y = 0;
					}
				}
			}
			return ret; // Object
		},

		toQuery: function formToQuery(/*DOMNode|String*/ formNode){
			// summary:
			//		Returns a URL-encoded string representing the form passed as either a
			//		node or string ID identifying the form to serialize
			// formNode: DOMNode|String
			// returns: String

			return ioq.objectToQuery(form.toObject(formNode)); // String
		},

		toJson: function formToJson(/*DOMNode|String*/ formNode, /*Boolean?*/ prettyPrint){
			// summary:
			//		Create a serialized JSON string from a form node or string
			//		ID identifying the form to serialize
			// formNode: DOMNode|String
			// prettyPrint: Boolean?
			// returns: String

			return json.stringify(form.toObject(formNode), null, prettyPrint ? 4 : 0); // String
		}
	};

    return form;
});

},
'dojo/request/watch':function(){
define([
	'./util',
	'../errors/RequestTimeoutError',
	'../errors/CancelError',
	'../_base/array',
	'../_base/window',
	'../has!host-browser?dom-addeventlistener?:../on:'
], function(util, RequestTimeoutError, CancelError, array, win, on){
	// avoid setting a timer per request. It degrades performance on IE
	// something fierece if we don't use unified loops.
	var _inFlightIntvl = null,
		_inFlight = [];

	function watchInFlight(){
		// summary:
		//		internal method that checks each inflight XMLHttpRequest to see
		//		if it has completed or if the timeout situation applies.

		var now = +(new Date);

		// we need manual loop because we often modify _inFlight (and therefore 'i') while iterating
		for(var i = 0, dfd; i < _inFlight.length && (dfd = _inFlight[i]); i++){
			var response = dfd.response,
				options = response.options;
			if((dfd.isCanceled && dfd.isCanceled()) || (dfd.isValid && !dfd.isValid(response))){
				_inFlight.splice(i--, 1);
				watch._onAction && watch._onAction();
			}else if(dfd.isReady && dfd.isReady(response)){
				_inFlight.splice(i--, 1);
				dfd.handleResponse(response);
				watch._onAction && watch._onAction();
			}else if(dfd.startTime){
				// did we timeout?
				if(dfd.startTime + (options.timeout || 0) < now){
					_inFlight.splice(i--, 1);
					// Cancel the request so the io module can do appropriate cleanup.
					dfd.cancel(new RequestTimeoutError('Timeout exceeded', response));
					watch._onAction && watch._onAction();
				}
			}
		}

		watch._onInFlight && watch._onInFlight(dfd);

		if(!_inFlight.length){
			clearInterval(_inFlightIntvl);
			_inFlightIntvl = null;
		}
	}

	function watch(dfd){
		// summary:
		//		Watches the io request represented by dfd to see if it completes.
		// dfd: Deferred
		//		The Deferred object to watch.
		// response: Object
		//		The object used as the value of the request promise.
		// validCheck: Function
		//		Function used to check if the IO request is still valid. Gets the dfd
		//		object as its only argument.
		// ioCheck: Function
		//		Function used to check if basic IO call worked. Gets the dfd
		//		object as its only argument.
		// resHandle: Function
		//		Function used to process response. Gets the dfd
		//		object as its only argument.
		if(dfd.response.options.timeout){
			dfd.startTime = +(new Date);
		}

		if(dfd.isFulfilled()){
			// bail out if the deferred is already fulfilled
			return;
		}

		_inFlight.push(dfd);
		if(!_inFlightIntvl){
			_inFlightIntvl = setInterval(watchInFlight, 50);
		}

		// handle sync requests separately from async:
		// http://bugs.dojotoolkit.org/ticket/8467
		if(dfd.response.options.sync){
			watchInFlight();
		}
	}

	watch.cancelAll = function cancelAll(){
		// summary:
		//		Cancels all pending IO requests, regardless of IO type
		try{
			array.forEach(_inFlight, function(dfd){
				try{
					dfd.cancel(new CancelError('All requests canceled.'));
				}catch(e){}
			});
		}catch(e){}
	};

	if(win && on && win.doc.attachEvent){
		// Automatically call cancel all io calls on unload in IE
		// http://bugs.dojotoolkit.org/ticket/2357
		on(win.global, 'unload', function(){
			watch.cancelAll();
		});
	}

	return watch;
});

},
'dojo/request/util':function(){
define([
	'exports',
	'../errors/RequestError',
	'../errors/CancelError',
	'../Deferred',
	'../io-query',
	'../_base/array',
	'../_base/lang',
	'../promise/Promise'
], function(exports, RequestError, CancelError, Deferred, ioQuery, array, lang, Promise){
	exports.deepCopy = function deepCopy(target, source){
		for(var name in source){
			var tval = target[name],
				sval = source[name];
			if(tval !== sval){
				if(tval && typeof tval === 'object' && sval && typeof sval === 'object'){
					exports.deepCopy(tval, sval);
				}else{
					target[name] = sval;
				}
			}
		}
		return target;
	};

	exports.deepCreate = function deepCreate(source, properties){
		properties = properties || {};
		var target = lang.delegate(source),
			name, value;

		for(name in source){
			value = source[name];

			if(value && typeof value === 'object'){
				target[name] = exports.deepCreate(value, properties[name]);
			}
		}
		return exports.deepCopy(target, properties);
	};

	var freeze = Object.freeze || function(obj){ return obj; };
	function okHandler(response){
		return freeze(response);
	}
	function dataHandler (response) {
		return response.data || response.text;
	}

	exports.deferred = function deferred(response, cancel, isValid, isReady, handleResponse, last){
		var def = new Deferred(function(reason){
			cancel && cancel(def, response);

			if(!reason || !(reason instanceof RequestError) && !(reason instanceof CancelError)){
				return new CancelError('Request canceled', response);
			}
			return reason;
		});

		def.response = response;
		def.isValid = isValid;
		def.isReady = isReady;
		def.handleResponse = handleResponse;

		function errHandler(error){
			error.response = response;
			throw error;
		}
		var responsePromise = def.then(okHandler).otherwise(errHandler);

		if(exports.notify){
			responsePromise.then(
				lang.hitch(exports.notify, 'emit', 'load'),
				lang.hitch(exports.notify, 'emit', 'error')
			);
		}

		var dataPromise = responsePromise.then(dataHandler);

		// http://bugs.dojotoolkit.org/ticket/16794
		// The following works around a leak in IE9 through the
		// prototype using lang.delegate on dataPromise and
		// assigning the result a property with a reference to
		// responsePromise.
		var promise = new Promise();
		for (var prop in dataPromise) {
			if (dataPromise.hasOwnProperty(prop)) {
				promise[prop] = dataPromise[prop];
			}
		}
		promise.response = responsePromise;
		freeze(promise);
		// End leak fix


		if(last){
			def.then(function(response){
				last.call(def, response);
			}, function(error){
				last.call(def, response, error);
			});
		}

		def.promise = promise;
		def.then = promise.then;

		return def;
	};

	exports.addCommonMethods = function addCommonMethods(provider, methods){
		array.forEach(methods||['GET', 'POST', 'PUT', 'DELETE'], function(method){
			provider[(method === 'DELETE' ? 'DEL' : method).toLowerCase()] = function(url, options){
				options = lang.delegate(options||{});
				options.method = method;
				return provider(url, options);
			};
		});
	};

	exports.parseArgs = function parseArgs(url, options, skipData){
		var data = options.data,
			query = options.query;
		
		if(data && !skipData){
			if(typeof data === 'object'){
				options.data = ioQuery.objectToQuery(data);
			}
		}

		if(query){
			if(typeof query === 'object'){
				query = ioQuery.objectToQuery(query);
			}
			if(options.preventCache){
				query += (query ? '&' : '') + 'request.preventCache=' + (+(new Date));
			}
		}else if(options.preventCache){
			query = 'request.preventCache=' + (+(new Date));
		}

		if(url && query){
			url += (~url.indexOf('?') ? '&' : '?') + query;
		}

		return {
			url: url,
			options: options,
			getHeader: function(headerName){ return null; }
		};
	};

	exports.checkStatus = function(stat){
		stat = stat || 0;
		return (stat >= 200 && stat < 300) || // allow any 2XX response code
			stat === 304 ||                 // or, get it out of the cache
			stat === 1223 ||                // or, Internet Explorer mangled the status code
			!stat;                         // or, we're Titanium/browser chrome/chrome extension requesting a local file
	};
});

},
'dojo/errors/RequestError':function(){
define(['./create'], function(create){
	// module:
	//		dojo/errors/RequestError

	/*=====
	 return function(){
		 // summary:
		 //		TODOC
	 };
	 =====*/

	return create("RequestError", function(message, response){
		this.response = response;
	});
});

},
'dojo/errors/RequestTimeoutError':function(){
define(['./create', './RequestError'], function(create, RequestError){
	// module:
	//		dojo/errors/RequestTimeoutError

	/*=====
	 return function(){
		 // summary:
		 //		TODOC
	 };
	 =====*/

	return create("RequestTimeoutError", null, RequestError, {
		dojoType: "timeout"
	});
});

},
'dojo/request/xhr':function(){
define([
	'../errors/RequestError',
	'./watch',
	'./handlers',
	'./util',
	'../has'/*=====,
	'../request',
	'../_base/declare' =====*/
], function(RequestError, watch, handlers, util, has/*=====, request, declare =====*/){
	has.add('native-xhr', function(){
		// if true, the environment has a native XHR implementation
		return typeof XMLHttpRequest !== 'undefined';
	});
	has.add('dojo-force-activex-xhr', function(){
		return has('activex') && !document.addEventListener && window.location.protocol === 'file:';
	});

	has.add('native-xhr2', function(){
		if(!has('native-xhr')){ return; }
		var x = new XMLHttpRequest();
		return typeof x['addEventListener'] !== 'undefined' &&
			(typeof opera === 'undefined' || typeof x['upload'] !== 'undefined');
	});

	has.add('native-formdata', function(){
		// if true, the environment has a native FormData implementation
		return typeof FormData === 'function';
	});

	function handleResponse(response, error){
		var _xhr = response.xhr;
		response.status = response.xhr.status;
		response.text = _xhr.responseText;

		if(response.options.handleAs === 'xml'){
			response.data = _xhr.responseXML;
		}

		if(!error){
			try{
				handlers(response);
			}catch(e){
				error = e;
			}
		}

		if(error){
			this.reject(error);
		}else if(util.checkStatus(_xhr.status)){
			this.resolve(response);
		}else{
			error = new RequestError('Unable to load ' + response.url + ' status: ' + _xhr.status, response);

			this.reject(error);
		}
	}

	var isValid, isReady, addListeners, cancel;
	if(has('native-xhr2')){
		// Any platform with XHR2 will only use the watch mechanism for timeout.

		isValid = function(response){
			// summary:
			//		Check to see if the request should be taken out of the watch queue
			return !this.isFulfilled();
		};
		cancel = function(dfd, response){
			// summary:
			//		Canceler for deferred
			response.xhr.abort();
		};
		addListeners = function(_xhr, dfd, response){
			// summary:
			//		Adds event listeners to the XMLHttpRequest object
			function onLoad(evt){
				dfd.handleResponse(response);
			}
			function onError(evt){
				var _xhr = evt.target;
				var error = new RequestError('Unable to load ' + response.url + ' status: ' + _xhr.status, response);
				dfd.handleResponse(response, error);
			}

			function onProgress(evt){
				if(evt.lengthComputable){
					response.loaded = evt.loaded;
					response.total = evt.total;
					dfd.progress(response);
				}
			}

			_xhr.addEventListener('load', onLoad, false);
			_xhr.addEventListener('error', onError, false);
			_xhr.addEventListener('progress', onProgress, false);

			return function(){
				_xhr.removeEventListener('load', onLoad, false);
				_xhr.removeEventListener('error', onError, false);
				_xhr.removeEventListener('progress', onProgress, false);
				_xhr = null;
			};
		};
	}else{
		isValid = function(response){
			return response.xhr.readyState; //boolean
		};
		isReady = function(response){
			return 4 === response.xhr.readyState; //boolean
		};
		cancel = function(dfd, response){
			// summary:
			//		canceller function for util.deferred call.
			var xhr = response.xhr;
			var _at = typeof xhr.abort;
			if(_at === 'function' || _at === 'object' || _at === 'unknown'){
				xhr.abort();
			}
		};
	}

	function getHeader(headerName){
		return this.xhr.getResponseHeader(headerName);
	}

	var undefined,
		defaultOptions = {
			data: null,
			query: null,
			sync: false,
			method: 'GET'
		};
	function xhr(url, options, returnDeferred){
		var response = util.parseArgs(
			url,
			util.deepCreate(defaultOptions, options),
			has('native-formdata') && options && options.data && options.data instanceof FormData
		);
		url = response.url;
		options = response.options;

		var remover,
			last = function(){
				remover && remover();
			};

		//Make the Deferred object for this xhr request.
		var dfd = util.deferred(
			response,
			cancel,
			isValid,
			isReady,
			handleResponse,
			last
		);
		var _xhr = response.xhr = xhr._create();

		if(!_xhr){
			// If XHR factory somehow returns nothings,
			// cancel the deferred.
			dfd.cancel(new RequestError('XHR was not created'));
			return returnDeferred ? dfd : dfd.promise;
		}

		response.getHeader = getHeader;

		if(addListeners){
			remover = addListeners(_xhr, dfd, response);
		}

		var data = options.data,
			async = !options.sync,
			method = options.method;

		try{
			// IE6 won't let you call apply() on the native function.
			_xhr.open(method, url, async, options.user || undefined, options.password || undefined);

			if(options.withCredentials){
				_xhr.withCredentials = options.withCredentials;
			}

			var headers = options.headers,
				contentType = 'application/x-www-form-urlencoded';
			if(headers){
				for(var hdr in headers){
					if(hdr.toLowerCase() === 'content-type'){
						contentType = headers[hdr];
					}else if(headers[hdr]){
						//Only add header if it has a value. This allows for instance, skipping
						//insertion of X-Requested-With by specifying empty value.
						_xhr.setRequestHeader(hdr, headers[hdr]);
					}
				}
			}

			if(contentType && contentType !== false){
				_xhr.setRequestHeader('Content-Type', contentType);
			}
			if(!headers || !('X-Requested-With' in headers)){
				_xhr.setRequestHeader('X-Requested-With', 'XMLHttpRequest');
			}

			if(util.notify){
				util.notify.emit('send', response, dfd.promise.cancel);
			}
			_xhr.send(data);
		}catch(e){
			dfd.reject(e);
		}

		watch(dfd);
		_xhr = null;

		return returnDeferred ? dfd : dfd.promise;
	}

	/*=====
	xhr = function(url, options){
		// summary:
		//		Sends a request using XMLHttpRequest with the given URL and options.
		// url: String
		//		URL to request
		// options: dojo/request/xhr.__Options?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	xhr.__BaseOptions = declare(request.__BaseOptions, {
		// sync: Boolean?
		//		Whether to make a synchronous request or not. Default
		//		is `false` (asynchronous).
		// data: String|Object|FormData?
		//		Data to transfer. This is ignored for GET and DELETE
		//		requests.
		// headers: Object?
		//		Headers to use for the request.
		// user: String?
		//		Username to use during the request.
		// password: String?
		//		Password to use during the request.
		// withCredentials: Boolean?
		//		For cross-site requests, whether to send credentials
		//		or not.
	});
	xhr.__MethodOptions = declare(null, {
		// method: String?
		//		The HTTP method to use to make the request. Must be
		//		uppercase. Default is `"GET"`.
	});
	xhr.__Options = declare([xhr.__BaseOptions, xhr.__MethodOptions]);

	xhr.get = function(url, options){
		// summary:
		//		Send an HTTP GET request using XMLHttpRequest with the given URL and options.
		// url: String
		//		URL to request
		// options: dojo/request/xhr.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	xhr.post = function(url, options){
		// summary:
		//		Send an HTTP POST request using XMLHttpRequest with the given URL and options.
		// url: String
		//		URL to request
		// options: dojo/request/xhr.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	xhr.put = function(url, options){
		// summary:
		//		Send an HTTP PUT request using XMLHttpRequest with the given URL and options.
		// url: String
		//		URL to request
		// options: dojo/request/xhr.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	xhr.del = function(url, options){
		// summary:
		//		Send an HTTP DELETE request using XMLHttpRequest with the given URL and options.
		// url: String
		//		URL to request
		// options: dojo/request/xhr.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	=====*/
	xhr._create = function(){
		// summary:
		//		does the work of portably generating a new XMLHTTPRequest object.
		throw new Error('XMLHTTP not available');
	};
	if(has('native-xhr') && !has('dojo-force-activex-xhr')){
		xhr._create = function(){
			return new XMLHttpRequest();
		};
	}else if(has('activex')){
		try{
			new ActiveXObject('Msxml2.XMLHTTP');
			xhr._create = function(){
				return new ActiveXObject('Msxml2.XMLHTTP');
			};
		}catch(e){
			try{
				new ActiveXObject('Microsoft.XMLHTTP');
				xhr._create = function(){
					return new ActiveXObject('Microsoft.XMLHTTP');
				};
			}catch(e){}
		}
	}

	util.addCommonMethods(xhr);

	return xhr;
});

},
'dojo/request/handlers':function(){
define([
	'../json',
	'../_base/kernel',
	'../_base/array',
	'../has',
	'../selector/_loader' // only included for has() qsa tests
], function(JSON, kernel, array, has){
	has.add('activex', typeof ActiveXObject !== 'undefined');
	has.add('dom-parser', function(global){
		return 'DOMParser' in global;
	});

	var handleXML;
	if(has('activex')){
		// GUIDs obtained from http://msdn.microsoft.com/en-us/library/ms757837(VS.85).aspx
		var dp = [
			'Msxml2.DOMDocument.6.0',
			'Msxml2.DOMDocument.4.0',
			'MSXML2.DOMDocument.3.0',
			'MSXML.DOMDocument' // 2.0
		];

		handleXML = function(response){
			var result = response.data;

			if(result && has('dom-qsa2.1') && !result.querySelectorAll && has('dom-parser')){
				// http://bugs.dojotoolkit.org/ticket/15631
				// IE9 supports a CSS3 querySelectorAll implementation, but the DOM implementation 
				// returned by IE9 xhr.responseXML does not. Manually create the XML DOM to gain 
				// the fuller-featured implementation and avoid bugs caused by the inconsistency
				result = new DOMParser().parseFromString(response.text, 'application/xml');
			}

			if(!result || !result.documentElement){
				var text = response.text;
				array.some(dp, function(p){
					try{
						var dom = new ActiveXObject(p);
						dom.async = false;
						dom.loadXML(text);
						result = dom;
					}catch(e){ return false; }
					return true;
				});
			}

			return result;
		};
	}

	var handlers = {
		'javascript': function(response){
			return kernel.eval(response.text || '');
		},
		'json': function(response){
			return JSON.parse(response.text || null);
		},
		'xml': handleXML
	};

	function handle(response){
		var handler = handlers[response.options.handleAs];

		response.data = handler ? handler(response) : (response.data || response.text);

		return response;
	}

	handle.register = function(name, handler){
		handlers[name] = handler;
	};

	return handle;
});

},
'dojo/_base/fx':function(){
define(["./kernel", "./config", /*===== "./declare", =====*/ "./lang", "../Evented", "./Color", "../aspect", "../sniff", "../dom", "../dom-style"],
	function(dojo, config, /*===== declare, =====*/ lang, Evented, Color, aspect, has, dom, style){
	// module:
	//		dojo/_base/fx
	// notes:
	//		Animation loosely package based on Dan Pupius' work, contributed under CLA; see
	//		http://pupius.co.uk/js/Toolkit.Drawing.js

	var _mixin = lang.mixin;

	// Module export
	var basefx = {
		// summary:
		//		This module defines the base dojo/_base/fx implementation.
	};

	var _Line = basefx._Line = function(/*int*/ start, /*int*/ end){
		// summary:
		//		Object used to generate values from a start value to an end value
		// start: int
		//		Beginning value for range
		// end: int
		//		Ending value for range
		this.start = start;
		this.end = end;
	};

	_Line.prototype.getValue = function(/*float*/ n){
		// summary:
		//		Returns the point on the line
		// n:
		//		a floating point number greater than 0 and less than 1
		return ((this.end - this.start) * n) + this.start; // Decimal
	};

	var Animation = basefx.Animation = function(args){
		// summary:
		//		A generic animation class that fires callbacks into its handlers
		//		object at various states.
		// description:
		//		A generic animation class that fires callbacks into its handlers
		//		object at various states. Nearly all dojo animation functions
		//		return an instance of this method, usually without calling the
		//		.play() method beforehand. Therefore, you will likely need to
		//		call .play() on instances of `Animation` when one is
		//		returned.
		// args: Object
		//		The 'magic argument', mixing all the properties into this
		//		animation instance.

		_mixin(this, args);
		if(lang.isArray(this.curve)){
			this.curve = new _Line(this.curve[0], this.curve[1]);
		}

	};
	Animation.prototype = new Evented();

	lang.extend(Animation, {
		// duration: Integer
		//		The time in milliseconds the animation will take to run
		duration: 350,

	/*=====
		// curve: _Line|Array
		//		A two element array of start and end values, or a `_Line` instance to be
		//		used in the Animation.
		curve: null,

		// easing: Function?
		//		A Function to adjust the acceleration (or deceleration) of the progress
		//		across a _Line
		easing: null,
	=====*/

		// repeat: Integer?
		//		The number of times to loop the animation
		repeat: 0,

		// rate: Integer?
		//		the time in milliseconds to wait before advancing to next frame
		//		(used as a fps timer: 1000/rate = fps)
		rate: 20 /* 50 fps */,

	/*=====
		// delay: Integer?
		//		The time in milliseconds to wait before starting animation after it
		//		has been .play()'ed
		delay: null,

		// beforeBegin: Event?
		//		Synthetic event fired before a Animation begins playing (synchronous)
		beforeBegin: null,

		// onBegin: Event?
		//		Synthetic event fired as a Animation begins playing (useful?)
		onBegin: null,

		// onAnimate: Event?
		//		Synthetic event fired at each interval of the Animation
		onAnimate: null,

		// onEnd: Event?
		//		Synthetic event fired after the final frame of the Animation
		onEnd: null,

		// onPlay: Event?
		//		Synthetic event fired any time the Animation is play()'ed
		onPlay: null,

		// onPause: Event?
		//		Synthetic event fired when the Animation is paused
		onPause: null,

		// onStop: Event
		//		Synthetic event fires when the Animation is stopped
		onStop: null,

	=====*/

		_percent: 0,
		_startRepeatCount: 0,

		_getStep: function(){
			var _p = this._percent,
				_e = this.easing
			;
			return _e ? _e(_p) : _p;
		},
		_fire: function(/*Event*/ evt, /*Array?*/ args){
			// summary:
			//		Convenience function.  Fire event "evt" and pass it the
			//		arguments specified in "args".
			// description:
			//		Convenience function.  Fire event "evt" and pass it the
			//		arguments specified in "args".
			//		Fires the callback in the scope of this Animation
			//		instance.
			// evt:
			//		The event to fire.
			// args:
			//		The arguments to pass to the event.
			var a = args||[];
			if(this[evt]){
				if(config.debugAtAllCosts){
					this[evt].apply(this, a);
				}else{
					try{
						this[evt].apply(this, a);
					}catch(e){
						// squelch and log because we shouldn't allow exceptions in
						// synthetic event handlers to cause the internal timer to run
						// amuck, potentially pegging the CPU. I'm not a fan of this
						// squelch, but hopefully logging will make it clear what's
						// going on
						console.error("exception in animation handler for:", evt);
						console.error(e);
					}
				}
			}
			return this; // Animation
		},

		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			// summary:
			//		Start the animation.
			// delay:
			//		How many milliseconds to delay before starting.
			// gotoStart:
			//		If true, starts the animation from the beginning; otherwise,
			//		starts it from its current position.
			// returns: Animation
			//		The instance to allow chaining.

			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			if(gotoStart){
				_t._stopTimer();
				_t._active = _t._paused = false;
				_t._percent = 0;
			}else if(_t._active && !_t._paused){
				return _t;
			}

			_t._fire("beforeBegin", [_t.node]);

			var de = delay || _t.delay,
				_p = lang.hitch(_t, "_play", gotoStart);

			if(de > 0){
				_t._delayTimer = setTimeout(_p, de);
				return _t;
			}
			_p();
			return _t;	// Animation
		},

		_play: function(gotoStart){
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			_t._startTime = new Date().valueOf();
			if(_t._paused){
				_t._startTime -= _t.duration * _t._percent;
			}

			_t._active = true;
			_t._paused = false;
			var value = _t.curve.getValue(_t._getStep());
			if(!_t._percent){
				if(!_t._startRepeatCount){
					_t._startRepeatCount = _t.repeat;
				}
				_t._fire("onBegin", [value]);
			}

			_t._fire("onPlay", [value]);

			_t._cycle();
			return _t; // Animation
		},

		pause: function(){
			// summary:
			//		Pauses a running animation.
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			_t._stopTimer();
			if(!_t._active){ return _t; /*Animation*/ }
			_t._paused = true;
			_t._fire("onPause", [_t.curve.getValue(_t._getStep())]);
			return _t; // Animation
		},

		gotoPercent: function(/*Decimal*/ percent, /*Boolean?*/ andPlay){
			// summary:
			//		Sets the progress of the animation.
			// percent:
			//		A percentage in decimal notation (between and including 0.0 and 1.0).
			// andPlay:
			//		If true, play the animation after setting the progress.
			var _t = this;
			_t._stopTimer();
			_t._active = _t._paused = true;
			_t._percent = percent;
			if(andPlay){ _t.play(); }
			return _t; // Animation
		},

		stop: function(/*boolean?*/ gotoEnd){
			// summary:
			//		Stops a running animation.
			// gotoEnd:
			//		If true, the animation will end.
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			if(!_t._timer){ return _t; /* Animation */ }
			_t._stopTimer();
			if(gotoEnd){
				_t._percent = 1;
			}
			_t._fire("onStop", [_t.curve.getValue(_t._getStep())]);
			_t._active = _t._paused = false;
			return _t; // Animation
		},

		status: function(){
			// summary:
			//		Returns a string token representation of the status of
			//		the animation, one of: "paused", "playing", "stopped"
			if(this._active){
				return this._paused ? "paused" : "playing"; // String
			}
			return "stopped"; // String
		},

		_cycle: function(){
			var _t = this;
			if(_t._active){
				var curr = new Date().valueOf();
				// Allow durations of 0 (instant) by setting step to 1 - see #13798
				var step = _t.duration === 0 ? 1 : (curr - _t._startTime) / (_t.duration);

				if(step >= 1){
					step = 1;
				}
				_t._percent = step;

				// Perform easing
				if(_t.easing){
					step = _t.easing(step);
				}

				_t._fire("onAnimate", [_t.curve.getValue(step)]);

				if(_t._percent < 1){
					_t._startTimer();
				}else{
					_t._active = false;

					if(_t.repeat > 0){
						_t.repeat--;
						_t.play(null, true);
					}else if(_t.repeat == -1){
						_t.play(null, true);
					}else{
						if(_t._startRepeatCount){
							_t.repeat = _t._startRepeatCount;
							_t._startRepeatCount = 0;
						}
					}
					_t._percent = 0;
					_t._fire("onEnd", [_t.node]);
					!_t.repeat && _t._stopTimer();
				}
			}
			return _t; // Animation
		},

		_clearTimer: function(){
			// summary:
			//		Clear the play delay timer
			clearTimeout(this._delayTimer);
			delete this._delayTimer;
		}

	});

	// the local timer, stubbed into all Animation instances
	var ctr = 0,
		timer = null,
		runner = {
			run: function(){}
		};

	lang.extend(Animation, {

		_startTimer: function(){
			if(!this._timer){
				this._timer = aspect.after(runner, "run", lang.hitch(this, "_cycle"), true);
				ctr++;
			}
			if(!timer){
				timer = setInterval(lang.hitch(runner, "run"), this.rate);
			}
		},

		_stopTimer: function(){
			if(this._timer){
				this._timer.remove();
				this._timer = null;
				ctr--;
			}
			if(ctr <= 0){
				clearInterval(timer);
				timer = null;
				ctr = 0;
			}
		}

	});

	var _makeFadeable =
		has("ie") ? function(node){
			// only set the zoom if the "tickle" value would be the same as the
			// default
			var ns = node.style;
			// don't set the width to auto if it didn't already cascade that way.
			// We don't want to f anyones designs
			if(!ns.width.length && style.get(node, "width") == "auto"){
				ns.width = "auto";
			}
		} :
		function(){};

	basefx._fade = function(/*Object*/ args){
		// summary:
		//		Returns an animation that will fade the node defined by
		//		args.node from the start to end values passed (args.start
		//		args.end) (end is mandatory, start is optional)

		args.node = dom.byId(args.node);
		var fArgs = _mixin({ properties: {} }, args),
			props = (fArgs.properties.opacity = {});

		props.start = !("start" in fArgs) ?
			function(){
				return +style.get(fArgs.node, "opacity")||0;
			} : fArgs.start;
		props.end = fArgs.end;

		var anim = basefx.animateProperty(fArgs);
		aspect.after(anim, "beforeBegin", lang.partial(_makeFadeable, fArgs.node), true);

		return anim; // Animation
	};

	/*=====
	var __FadeArgs = declare(null, {
		// node: DOMNode|String
		//		The node referenced in the animation
		// duration: Integer?
		//		Duration of the animation in milliseconds.
		// easing: Function?
		//		An easing function.
	});
	=====*/

	basefx.fadeIn = function(/*__FadeArgs*/ args){
		// summary:
		//		Returns an animation that will fade node defined in 'args' from
		//		its current opacity to fully opaque.
		return basefx._fade(_mixin({ end: 1 }, args)); // Animation
	};

	basefx.fadeOut = function(/*__FadeArgs*/ args){
		// summary:
		//		Returns an animation that will fade node defined in 'args'
		//		from its current opacity to fully transparent.
		return basefx._fade(_mixin({ end: 0 }, args)); // Animation
	};

	basefx._defaultEasing = function(/*Decimal?*/ n){
		// summary:
		//		The default easing function for Animation(s)
		return 0.5 + ((Math.sin((n + 1.5) * Math.PI)) / 2);	// Decimal
	};

	var PropLine = function(properties){
		// PropLine is an internal class which is used to model the values of
		// an a group of CSS properties across an animation lifecycle. In
		// particular, the "getValue" function handles getting interpolated
		// values between start and end for a particular CSS value.
		this._properties = properties;
		for(var p in properties){
			var prop = properties[p];
			if(prop.start instanceof Color){
				// create a reusable temp color object to keep intermediate results
				prop.tempColor = new Color();
			}
		}
	};

	PropLine.prototype.getValue = function(r){
		var ret = {};
		for(var p in this._properties){
			var prop = this._properties[p],
				start = prop.start;
			if(start instanceof Color){
				ret[p] = Color.blendColors(start, prop.end, r, prop.tempColor).toCss();
			}else if(!lang.isArray(start)){
				ret[p] = ((prop.end - start) * r) + start + (p != "opacity" ? prop.units || "px" : 0);
			}
		}
		return ret;
	};

	/*=====
	var __AnimArgs = declare(__FadeArgs, {
		// properties: Object?
		//		A hash map of style properties to Objects describing the transition,
		//		such as the properties of _Line with an additional 'units' property
		properties: {}

		//TODOC: add event callbacks
	});
	=====*/

	basefx.animateProperty = function(/*__AnimArgs*/ args){
		// summary:
		//		Returns an animation that will transition the properties of
		//		node defined in `args` depending how they are defined in
		//		`args.properties`
		//
		// description:
		//		Foundation of most `dojo/_base/fx`
		//		animations. It takes an object of "properties" corresponding to
		//		style properties, and animates them in parallel over a set
		//		duration.
		//
		// example:
		//		A simple animation that changes the width of the specified node.
		//	|	basefx.animateProperty({
		//	|		node: "nodeId",
		//	|		properties: { width: 400 },
		//	|	}).play();
		//		Dojo figures out the start value for the width and converts the
		//		integer specified for the width to the more expressive but
		//		verbose form `{ width: { end: '400', units: 'px' } }` which you
		//		can also specify directly. Defaults to 'px' if omitted.
		//
		// example:
		//		Animate width, height, and padding over 2 seconds... the
		//		pedantic way:
		//	|	basefx.animateProperty({ node: node, duration:2000,
		//	|		properties: {
		//	|			width: { start: '200', end: '400', units:"px" },
		//	|			height: { start:'200', end: '400', units:"px" },
		//	|			paddingTop: { start:'5', end:'50', units:"px" }
		//	|		}
		//	|	}).play();
		//		Note 'paddingTop' is used over 'padding-top'. Multi-name CSS properties
		//		are written using "mixed case", as the hyphen is illegal as an object key.
		//
		// example:
		//		Plug in a different easing function and register a callback for
		//		when the animation ends. Easing functions accept values between
		//		zero and one and return a value on that basis. In this case, an
		//		exponential-in curve.
		//	|	basefx.animateProperty({
		//	|		node: "nodeId",
		//	|		// dojo figures out the start value
		//	|		properties: { width: { end: 400 } },
		//	|		easing: function(n){
		//	|			return (n==0) ? 0 : Math.pow(2, 10 * (n - 1));
		//	|		},
		//	|		onEnd: function(node){
		//	|			// called when the animation finishes. The animation
		//	|			// target is passed to this function
		//	|		}
		//	|	}).play(500); // delay playing half a second
		//
		// example:
		//		Like all `Animation`s, animateProperty returns a handle to the
		//		Animation instance, which fires the events common to Dojo FX. Use `aspect.after`
		//		to access these events outside of the Animation definition:
		//	|	var anim = basefx.animateProperty({
		//	|		node:"someId",
		//	|		properties:{
		//	|			width:400, height:500
		//	|		}
		//	|	});
		//	|	aspect.after(anim, "onEnd", function(){
		//	|		console.log("animation ended");
		//	|	}, true);
		//	|	// play the animation now:
		//	|	anim.play();
		//
		// example:
		//		Each property can be a function whose return value is substituted along.
		//		Additionally, each measurement (eg: start, end) can be a function. The node
		//		reference is passed directly to callbacks.
		//	|	basefx.animateProperty({
		//	|		node:"mine",
		//	|		properties:{
		//	|			height:function(node){
		//	|				// shrink this node by 50%
		//	|				return domGeom.position(node).h / 2
		//	|			},
		//	|			width:{
		//	|				start:function(node){ return 100; },
		//	|				end:function(node){ return 200; }
		//	|			}
		//	|		}
		//	|	}).play();
		//

		var n = args.node = dom.byId(args.node);
		if(!args.easing){ args.easing = dojo._defaultEasing; }

		var anim = new Animation(args);
		aspect.after(anim, "beforeBegin", lang.hitch(anim, function(){
			var pm = {};
			for(var p in this.properties){
				// Make shallow copy of properties into pm because we overwrite
				// some values below. In particular if start/end are functions
				// we don't want to overwrite them or the functions won't be
				// called if the animation is reused.
				if(p == "width" || p == "height"){
					this.node.display = "block";
				}
				var prop = this.properties[p];
				if(lang.isFunction(prop)){
					prop = prop(n);
				}
				prop = pm[p] = _mixin({}, (lang.isObject(prop) ? prop: { end: prop }));

				if(lang.isFunction(prop.start)){
					prop.start = prop.start(n);
				}
				if(lang.isFunction(prop.end)){
					prop.end = prop.end(n);
				}
				var isColor = (p.toLowerCase().indexOf("color") >= 0);
				function getStyle(node, p){
					// domStyle.get(node, "height") can return "auto" or "" on IE; this is more reliable:
					var v = { height: node.offsetHeight, width: node.offsetWidth }[p];
					if(v !== undefined){ return v; }
					v = style.get(node, p);
					return (p == "opacity") ? +v : (isColor ? v : parseFloat(v));
				}
				if(!("end" in prop)){
					prop.end = getStyle(n, p);
				}else if(!("start" in prop)){
					prop.start = getStyle(n, p);
				}

				if(isColor){
					prop.start = new Color(prop.start);
					prop.end = new Color(prop.end);
				}else{
					prop.start = (p == "opacity") ? +prop.start : parseFloat(prop.start);
				}
			}
			this.curve = new PropLine(pm);
		}), true);
		aspect.after(anim, "onAnimate", lang.hitch(style, "set", anim.node), true);
		return anim; // Animation
	};

	basefx.anim = function(	/*DOMNode|String*/	node,
							/*Object*/			properties,
							/*Integer?*/		duration,
							/*Function?*/		easing,
							/*Function?*/		onEnd,
							/*Integer?*/		delay){
		// summary:
		//		A simpler interface to `animateProperty()`, also returns
		//		an instance of `Animation` but begins the animation
		//		immediately, unlike nearly every other Dojo animation API.
		// description:
		//		Simpler (but somewhat less powerful) version
		//		of `animateProperty`.  It uses defaults for many basic properties
		//		and allows for positional parameters to be used in place of the
		//		packed "property bag" which is used for other Dojo animation
		//		methods.
		//
		//		The `Animation` object returned will be already playing, so
		//		calling play() on it again is (usually) a no-op.
		// node:
		//		a DOM node or the id of a node to animate CSS properties on
		// duration:
		//		The number of milliseconds over which the animation
		//		should run. Defaults to the global animation default duration
		//		(350ms).
		// easing:
		//		An easing function over which to calculate acceleration
		//		and deceleration of the animation through its duration.
		//		A default easing algorithm is provided, but you may
		//		plug in any you wish. A large selection of easing algorithms
		//		are available in `dojo/fx/easing`.
		// onEnd:
		//		A function to be called when the animation finishes
		//		running.
		// delay:
		//		The number of milliseconds to delay beginning the
		//		animation by. The default is 0.
		// example:
		//		Fade out a node
		//	|	basefx.anim("id", { opacity: 0 });
		// example:
		//		Fade out a node over a full second
		//	|	basefx.anim("id", { opacity: 0 }, 1000);
		return basefx.animateProperty({ // Animation
			node: node,
			duration: duration || Animation.prototype.duration,
			properties: properties,
			easing: easing,
			onEnd: onEnd
		}).play(delay || 0);
	};


	if( 1 ){
		_mixin(dojo, basefx);
		// Alias to drop come 2.0:
		dojo._Animation = Animation;
	}

	return basefx;
});

},
'dojo/_base/loader':function(){
define(["./kernel", "../has", "require", "module", "../json", "./lang", "./array"], function(dojo, has, require, thisModule, json, lang, array) {
	// module:
	//		dojo/_base/loader

	//		This module defines the v1.x synchronous loader API.

	// signal the loader in sync mode...
	//>>pure-amd

	if (! 1 ){
		console.error("cannot load the Dojo v1.x loader with a foreign loader");
		return 0;
	}

	 1 || has.add("dojo-fast-sync-require", 1);


	var makeErrorToken = function(id){
			return {src:thisModule.id, id:id};
		},

		slashName = function(name){
			return name.replace(/\./g, "/");
		},

		buildDetectRe = /\/\/>>built/,

		dojoRequireCallbacks = [],
		dojoRequireModuleStack = [],

		dojoRequirePlugin = function(mid, require, loaded){
			dojoRequireCallbacks.push(loaded);
			array.forEach(mid.split(","), function(mid){
				var module = getModule(mid, require.module);
				dojoRequireModuleStack.push(module);
				injectModule(module);
			});
			checkDojoRequirePlugin();
		},

		checkDojoRequirePlugin = ( 1  ?
			// This version of checkDojoRequirePlugin makes the observation that all dojoRequireCallbacks can be released
			// when all *non-dojo/require!, dojo/loadInit!* modules are either executed, not requested, or arrived. This is
			// the case since there are no more modules the loader is waiting for, therefore, dojo/require! must have
			// everything it needs on board.
			//
			// The potential weakness of this algorithm is that dojo/require will not execute callbacks until *all* dependency
			// trees are ready. It is possible that some trees may be ready earlier than others, and this extra wait is non-optimal.
			// Still, for big projects, this seems better than the original algorithm below that proved slow in some cases.
			// Note, however, the original algorithm had the potential to execute partial trees,  but that potential was never enabled.
			// There are also other optimization available with the original algorithm that have not been explored.
			function(){
				var module, mid;
				for(mid in modules){
					module = modules[mid];
					if(module.noReqPluginCheck===undefined){
						// tag the module as either a loadInit or require plugin or not for future reference
						module.noReqPluginCheck = /loadInit\!/.test(mid) || /require\!/.test(mid) ? 1 : 0;
					}
					if(!module.executed && !module.noReqPluginCheck && module.injected==requested){
						return;
					}
				}

				guardCheckComplete(function(){
					var oldCallbacks = dojoRequireCallbacks;
					dojoRequireCallbacks = [];
					array.forEach(oldCallbacks, function(cb){cb(1);});
				});
		} : (function(){
			// Note: this is the original checkDojoRequirePlugin that is much slower than the algorithm above. However, we know it
			// works, so we leave it here in case the algorithm above fails in some corner case.
			//
			// checkDojoRequirePlugin inspects all of the modules demanded by a dojo/require!<module-list> dependency
			// to see if they have arrived. The loader does not release *any* of these modules to be instantiated
			// until *all* of these modules are on board, thereby preventing the evaluation of a module with dojo.require's
			// that reference modules that are not available.
			//
			// The algorithm works by traversing the dependency graphs (remember, there can be cycles so they are not trees)
			// of each module in the dojoRequireModuleStack array (which contains the list of modules demanded by dojo/require!).
			// The moment a single module is discovered that is missing, the algorithm gives up and indicates that not all
			// modules are on board. dojo/loadInit! and dojo/require! are ignored because there dependencies are inserted
			// directly in dojoRequireModuleStack. For example, if "your/module" module depends on "dojo/require!my/module", then
			// *both* "dojo/require!my/module" and "my/module" will be in dojoRequireModuleStack. Obviously, if "my/module"
			// is on board, then "dojo/require!my/module" is also satisfied, so the algorithm doesn't check for "dojo/require!my/module".
			//
			// Note: inserting a dojo/require!<some-module-list> dependency in the dojoRequireModuleStack achieves nothing
			// with the current algorithm; however, having such modules present makes it possible to optimize the algorithm
			//
			// Note: prior versions of this algorithm had an optimization that signaled loaded on dojo/require! dependencies
			// individually (rather than waiting for them all to be resolved). The implementation proved problematic with cycles
			// and plugins. However, it is possible to reattach that strategy in the future.

			// a set from module-id to {undefined | 1 | 0}, where...
			//	 undefined => the module has not been inspected
			//	 0 => the module or at least one of its dependencies has not arrived
			//	 1 => the module is a loadInit! or require! plugin resource, or is currently being traversed (therefore, assume
			//		  OK until proven otherwise), or has been completely traversed and all dependencies have arrived

			var touched,
			traverse = function(m){
				touched[m.mid] = 1;
				for(var t, module, deps = m.deps || [], i= 0; i<deps.length; i++){
					module = deps[i];
					if(!(t = touched[module.mid])){
						if(t===0 || !traverse(module)){
							touched[m.mid] = 0;
							return false;
						}
					}
				}
				return true;
			};

			return function(){
				// initialize the touched hash with easy-to-compute values that help short circuit recursive algorithm;
				// recall loadInit/require plugin modules are dependencies of modules in dojoRequireModuleStack...
				// which would cause a circular dependency chain that would never be resolved if checked here
				// notice all dependencies of any particular loadInit/require plugin module will already
				// be checked since those are pushed into dojoRequireModuleStack explicitly by the
				// plugin...so if a particular loadInitPlugin module's dependencies are not really
				// on board, that *will* be detected elsewhere in the traversal.
				var module, mid;
				touched = {};
				for(mid in modules){
					module = modules[mid];
					if(module.executed || module.noReqPluginCheck){
						touched[mid] = 1;
					}else{
						if(module.noReqPluginCheck!==0){
							// tag the module as either a loadInit or require plugin or not for future reference
							module.noReqPluginCheck = /loadInit\!/.test(mid) || /require\!/.test(mid) ? 1 : 0;
						}
						if(module.noReqPluginCheck){
							touched[mid] = 1;
						}else if(module.injected!==arrived){
							// not executed, has not arrived, and is not a loadInit or require plugin resource
							touched[mid] = 0;
						}// else, leave undefined and we'll traverse the dependencies
					}
				}
				for(var t, i = 0, end = dojoRequireModuleStack.length; i<end; i++){
					module = dojoRequireModuleStack[i];
					if(!(t = touched[module.mid])){
						if(t===0 || !traverse(module)){
							return;
						}
					}
				}
				guardCheckComplete(function(){
					var oldCallbacks = dojoRequireCallbacks;
					dojoRequireCallbacks = [];
					array.forEach(oldCallbacks, function(cb){cb(1);});
				});
			};
		})()),

		dojoLoadInitPlugin = function(mid, require, loaded){
			// mid names a module that defines a "dojo load init" bundle, an object with two properties:
			//
			//	 * names: a vector of module ids that give top-level names to define in the lexical scope of def
			//	 * def: a function that contains some some legacy loader API applications
			//
			// The point of def is to possibly cause some modules to be loaded (but not executed) by dojo/require! where the module
			// ids are possibly-determined at runtime. For example, here is dojox.gfx from v1.6 expressed as an AMD module using the dojo/loadInit
			// and dojo/require plugins.
			//
			// // dojox/gfx:
			//
			//	 define("*loadInit_12, {
			//	   names:["dojo", "dijit", "dojox"],
			//	   def: function(){
			//		 dojo.loadInit(function(){
			//		   var gfx = lang.getObject("dojox.gfx", true);
			//
			//		   //
			//		   // code required to set gfx properties ommitted...
			//		   //
			//
			//		   // now use the calculations to include the runtime-dependent module
			//		   dojo.require("dojox.gfx." + gfx.renderer);
			//		 });
			//	   }
			//	 });
			//
			//	 define(["dojo", "dojo/loadInit!" + id].concat("dojo/require!dojox/gfx/matric,dojox/gfx/_base"), function(dojo){
			//	   // when this AMD factory function is executed, the following modules are guaranteed downloaded but not executed:
			//	   //	"dojox.gfx." + gfx.renderer
			//	   //	dojox.gfx.matrix
			//	   //	dojox.gfx._base
			//	   dojo.provide("dojo.gfx");
			//	   dojo.require("dojox.gfx.matrix");
			//	   dojo.require("dojox.gfx._base");
			//	   dojo.require("dojox.gfx." + gfx.renderer);
			//	   return lang.getObject("dojo.gfx");
			//	 });
			//	})();
			//
			// The idea is to run the legacy loader API with global variables shadowed, which allows these variables to
			// be relocated. For example, dojox and dojo could be relocated to different names by giving a map and the code above will
			// execute properly (because the plugin below resolves the load init bundle.names module with respect to the module that demanded
			// the plugin resource).
			//
			// Note that the relocation is specified in the runtime configuration; relocated names need not be set at build-time.
			//
			// Warning: this is not the best way to express dojox.gfx as and AMD module. In fact, the module has been properly converted in
			// v1.7. However, this technique allows the builder to convert legacy modules into AMD modules and guarantee the codepath is the
			// same in the converted AMD module.
			require([mid], function(bundle){
				// notice how names is resolved with respect to the module that demanded the plugin resource
				require(bundle.names, function(){
					// bring the bundle names into scope
					for(var scopeText = "", args= [], i = 0; i<arguments.length; i++){
						scopeText+= "var " + bundle.names[i] + "= arguments[" + i + "]; ";
						args.push(arguments[i]);
					}
					eval(scopeText);

					var callingModule = require.module,
						// the list of modules that need to be downloaded but not executed before the callingModule can be executed
						requireList = [],

						// the list of i18n bundles that are xdomain; undefined if none
						i18nDeps,

						syncLoaderApi = {
							provide:function(moduleName){
								// mark modules that arrive consequent to multiple provides in this module as arrived since they can't be injected
								moduleName = slashName(moduleName);
								var providedModule = getModule(moduleName, callingModule);
								if(providedModule!==callingModule){
									setArrived(providedModule);
								}
							},
							require:function(moduleName, omitModuleCheck){
								moduleName = slashName(moduleName);
								omitModuleCheck && (getModule(moduleName, callingModule).result = nonmodule);
								requireList.push(moduleName);
							},
							requireLocalization:function(moduleName, bundleName, locale){
								// since we're going to need dojo/i8n, add it to i18nDeps if not already there
								if(!i18nDeps){
									// don't have to map since that will occur when the dependency is resolved
									i18nDeps = ["dojo/i18n"];
								}

								// figure out if the bundle is xdomain; if so, add it to the i18nDepsSet
								locale = (locale || dojo.locale).toLowerCase();
								moduleName = slashName(moduleName) + "/nls/" + (/root/i.test(locale) ? "" : locale + "/") + slashName(bundleName);
								if(getModule(moduleName, callingModule).isXd){
									// don't have to map since that will occur when the dependency is resolved
									i18nDeps.push("dojo/i18n!" + moduleName);
								}// else the bundle will be loaded synchronously when the module is evaluated
							},
							loadInit:function(f){
								f();
							}
						},

						hold = {},
						p;

					// hijack the correct dojo and apply bundle.def
					try{
						for(p in syncLoaderApi){
							hold[p] = dojo[p];
							dojo[p] = syncLoaderApi[p];
						}
						bundle.def.apply(null, args);
					}catch(e){
						signal("error", [makeErrorToken("failedDojoLoadInit"), e]);
					}finally{
						for(p in syncLoaderApi){
							dojo[p] = hold[p];
						}
					}

					if(i18nDeps){
						requireList = requireList.concat(i18nDeps);
					}

					if(requireList.length){
						dojoRequirePlugin(requireList.join(","), require, loaded);
					}else{
						loaded();
					}
				});
			});
		},

		extractApplication = function(
			text,			  // the text to search
			startSearch,	  // the position in text to start looking for the closing paren
			startApplication  // the position in text where the function application expression starts
		){
			// find end of the call by finding the matching end paren
			// Warning: as usual, this will fail in the presense of unmatched right parans contained in strings, regexs, or unremoved comments
			var parenRe = /\(|\)/g,
				matchCount = 1,
				match;
			parenRe.lastIndex = startSearch;
			while((match = parenRe.exec(text))){
				if(match[0] == ")"){
					matchCount -= 1;
				}else{
					matchCount += 1;
				}
				if(matchCount == 0){
					break;
				}
			}

			if(matchCount != 0){
				throw "unmatched paren around character " + parenRe.lastIndex + " in: " + text;
			}

			//Put the master matching string in the results.
			return [dojo.trim(text.substring(startApplication, parenRe.lastIndex))+";\n", parenRe.lastIndex];
		},

		// the following regex is taken from 1.6. It is a very poor technique to remove comments and
		// will fail in some cases; for example, consider the code...
		//
		//	  var message = "Category-1 */* Category-2";
		//
		// The regex that follows will see a /* comment and trash the code accordingly. In fact, there are all
		// kinds of cases like this with strings and regexs that will cause this design to fail miserably.
		//
		// Alternative regex designs exist that will result in less-likely failures, but will still fail in many cases.
		// The only solution guaranteed 100% correct is to parse the code and that seems overkill for this
		// backcompat/unbuilt-xdomain layer. In the end, since it's been this way for a while, we won't change it.
		// See the opening paragraphs of Chapter 7 or ECME-262 which describes the lexical abiguity further.
		removeCommentRe = /(\/\*([\s\S]*?)\*\/|\/\/(.*)$)/mg,

		syncLoaderApiRe = /(^|\s)dojo\.(loadInit|require|provide|requireLocalization|requireIf|requireAfterIf|platformRequire)\s*\(/mg,

		amdLoaderApiRe = /(^|\s)(require|define)\s*\(/m,

		extractLegacyApiApplications = function(text, noCommentText){
			// scan the noCommentText for any legacy loader API applications. Copy such applications into result (this is
			// used by the builder). Move dojo.loadInit applications to loadInitApplications string. Copy all other applications
			// to otherApplications string. If no applications were found, return 0, signalling an AMD module. Otherwise, return
			// loadInitApplications + otherApplications. Fixup text by replacing
			//
			//	 dojo.loadInit(// etc...
			//
			// with
			//
			//	 \n 0 && dojo.loadInit(// etc...
			//
			// Which results in the dojo.loadInit from *not* being applied. This design goes a long way towards protecting the
			// code from an over-agressive removeCommentRe. However...
			//
			// WARNING: the removeCommentRe will cause an error if a detected comment removes all or part of a legacy-loader application
			// that is not in a comment.

			var match, startSearch, startApplication, application,
				loadInitApplications = [],
				otherApplications = [],
				allApplications = [];

			// noCommentText may be provided by a build app with comments extracted by a better method than regex (hopefully)
			noCommentText = noCommentText || text.replace(removeCommentRe, function(match){
				// remove iff the detected comment has text that looks like a sync loader API application; this helps by
				// removing as little as possible, minimizing the changes the janky regex will kill the module
				syncLoaderApiRe.lastIndex = amdLoaderApiRe.lastIndex = 0;
				return (syncLoaderApiRe.test(match) || amdLoaderApiRe.test(match)) ? "" : match;
			});

			// find and extract all dojo.loadInit applications
			while((match = syncLoaderApiRe.exec(noCommentText))){
				startSearch = syncLoaderApiRe.lastIndex;
				startApplication = startSearch	- match[0].length;
				application = extractApplication(noCommentText, startSearch, startApplication);
				if(match[2]=="loadInit"){
					loadInitApplications.push(application[0]);
				}else{
					otherApplications.push(application[0]);
				}
				syncLoaderApiRe.lastIndex = application[1];
			}
			allApplications = loadInitApplications.concat(otherApplications);
			if(allApplications.length || !amdLoaderApiRe.test(noCommentText)){
				// either there were some legacy loader API applications or there were no AMD API applications
				return [text.replace(/(^|\s)dojo\.loadInit\s*\(/g, "\n0 && dojo.loadInit("), allApplications.join(""), allApplications];
			}else{
				// legacy loader API *was not* detected and AMD API *was* detected; therefore, assume it's an AMD module
				return 0;
			}
		},

		transformToAmd = function(module, text){
			// This is roughly the equivalent of dojo._xdCreateResource in 1.6-; however, it expresses a v1.6- dojo
			// module in terms of AMD define instead of creating the dojo proprietary xdomain module expression.
			// The module could have originated from several sources:
			//
			//	 * amd require() a module, e.g., require(["my/module"])
			//	 * amd require() a nonmodule, e.g., require(["my/resource.js"')
			//	 * amd define() deps vector (always a module)
			//	 * dojo.require() a module, e.g. dojo.require("my.module")
			//	 * dojo.require() a nonmodule, e.g., dojo.require("my.module", true)
			//	 * dojo.requireIf/requireAfterIf/platformRequire a module
			//
			// The module is scanned for legacy loader API applications; if none are found, then assume the module is an
			// AMD module and return 0. Otherwise, a synthetic dojo/loadInit plugin resource is created and the module text
			// is rewritten as an AMD module with the single dependency of this synthetic resource. When the dojo/loadInit
			// plugin loaded the synthetic resource, it will cause all dojo.loadInit's to be executed, find all dojo.require's
			// (either directly consequent to dojo.require or indirectly consequent to dojo.require[After]If or
			// dojo.platformRequire, and finally cause loading of all dojo.required modules with the dojo/require plugin. Thus,
			// when the dojo/loadInit plugin reports it has been loaded, all modules required by the given module are guaranteed
			// loaded (but not executed). This then allows the module to execute it's code path without interupts, thereby
			// following the synchronous code path.
			//
			// Notice that this function behaves the same whether or not it happens to be in a mapped dojo/loader module.

			var extractResult, id, names = [], namesAsStrings = [];
			if(buildDetectRe.test(text) || !(extractResult = extractLegacyApiApplications(text))){
				// buildDetectRe.test(text) => a built module, always AMD
				// extractResult==0 => no sync API
				return 0;
			}

			// manufacture a synthetic module id that can never be a real mdule id (just like require does)
			id = module.mid + "-*loadInit";

			// construct the dojo/loadInit names vector which causes any relocated names to be defined as lexical variables under their not-relocated name
			// the dojo/loadInit plugin assumes the first name in names is "dojo"

			for(var p in getModule("dojo", module).result.scopeMap){
				names.push(p);
				namesAsStrings.push('"' + p + '"');
			}

			// rewrite the module as a synthetic dojo/loadInit plugin resource + the module expressed as an AMD module that depends on this synthetic resource
			// don't have to map dojo/init since that will occur when the dependency is resolved
			return "// xdomain rewrite of " + module.mid + "\n" +
				"define('" + id + "',{\n" +
				"\tnames:" + json.stringify(names) + ",\n" +
				"\tdef:function(" + names.join(",") + "){" + extractResult[1] + "}" +
				"});\n\n" +
				"define(" + json.stringify(names.concat(["dojo/loadInit!"+id])) + ", function(" + names.join(",") + "){\n" + extractResult[0] + "});";
		},

		loaderVars = require.initSyncLoader(dojoRequirePlugin, checkDojoRequirePlugin, transformToAmd),

		sync =
			loaderVars.sync,

		requested =
			loaderVars.requested,

		arrived =
			loaderVars.arrived,

		nonmodule =
			loaderVars.nonmodule,

		executing =
			loaderVars.executing,

		executed =
			loaderVars.executed,

		syncExecStack =
			loaderVars.syncExecStack,

		modules =
			loaderVars.modules,

		execQ =
			loaderVars.execQ,

		getModule =
			loaderVars.getModule,

		injectModule =
			loaderVars.injectModule,

		setArrived =
			loaderVars.setArrived,

		signal =
			loaderVars.signal,

		finishExec =
			loaderVars.finishExec,

		execModule =
			loaderVars.execModule,

		getLegacyMode =
			loaderVars.getLegacyMode,

		guardCheckComplete =
			loaderVars.guardCheckComplete;

	// there is exactly one dojoRequirePlugin among possibly-many dojo/_base/loader's (owing to mapping)
	dojoRequirePlugin = loaderVars.dojoRequirePlugin;

	dojo.provide = function(mid){
		var executingModule = syncExecStack[0],
			module = lang.mixin(getModule(slashName(mid), require.module), {
				executed:executing,
				result:lang.getObject(mid, true)
			});
		setArrived(module);
		if(executingModule){
			(executingModule.provides || (executingModule.provides = [])).push(function(){
				module.result = lang.getObject(mid);
				delete module.provides;
				module.executed!==executed && finishExec(module);
			});
		}// else dojo.provide called not consequent to loading; therefore, give up trying to publish module value to loader namespace
		return module.result;
	};

	has.add("config-publishRequireResult", 1, 0, 0);

	dojo.require = function(moduleName, omitModuleCheck) {
		// summary:
		//		loads a Javascript module from the appropriate URI
		//
		// moduleName: String
		//		module name to load, using periods for separators,
		//		 e.g. "dojo.date.locale".  Module paths are de-referenced by dojo's
		//		internal mapping of locations to names and are disambiguated by
		//		longest prefix. See `dojo.registerModulePath()` for details on
		//		registering new modules.
		//
		// omitModuleCheck: Boolean?
		//		if `true`, omitModuleCheck skips the step of ensuring that the
		//		loaded file actually defines the symbol it is referenced by.
		//		For example if it called as `dojo.require("a.b.c")` and the
		//		file located at `a/b/c.js` does not define an object `a.b.c`,
		//		and exception will be throws whereas no exception is raised
		//		when called as `dojo.require("a.b.c", true)`
		//
		// description:
		//		Modules are loaded via dojo.require by using one of two loaders: the normal loader
		//		and the xdomain loader. The xdomain loader is used when dojo was built with a
		//		custom build that specified loader=xdomain and the module lives on a modulePath
		//		that is a whole URL, with protocol and a domain. The versions of Dojo that are on
		//		the Google and AOL CDNs use the xdomain loader.
		//
		//		If the module is loaded via the xdomain loader, it is an asynchronous load, since
		//		the module is added via a dynamically created script tag. This
		//		means that dojo.require() can return before the module has loaded. However, this
		//		should only happen in the case where you do dojo.require calls in the top-level
		//		HTML page, or if you purposely avoid the loader checking for dojo.require
		//		dependencies in your module by using a syntax like dojo["require"] to load the module.
		//
		//		Sometimes it is useful to not have the loader detect the dojo.require calls in the
		//		module so that you can dynamically load the modules as a result of an action on the
		//		page, instead of right at module load time.
		//
		//		Also, for script blocks in an HTML page, the loader does not pre-process them, so
		//		it does not know to download the modules before the dojo.require calls occur.
		//
		//		So, in those two cases, when you want on-the-fly module loading or for script blocks
		//		in the HTML page, special care must be taken if the dojo.required code is loaded
		//		asynchronously. To make sure you can execute code that depends on the dojo.required
		//		modules, be sure to add the code that depends on the modules in a dojo.addOnLoad()
		//		callback. dojo.addOnLoad waits for all outstanding modules to finish loading before
		//		executing.
		//
		//		This type of syntax works with both xdomain and normal loaders, so it is good
		//		practice to always use this idiom for on-the-fly code loading and in HTML script
		//		blocks. If at some point you change loaders and where the code is loaded from,
		//		it will all still work.
		//
		//		More on how dojo.require
		//		`dojo.require("A.B")` first checks to see if symbol A.B is
		//		defined. If it is, it is simply returned (nothing to do).
		//
		//		If it is not defined, it will look for `A/B.js` in the script root
		//		directory.
		//
		//		`dojo.require` throws an exception if it cannot find a file
		//		to load, or if the symbol `A.B` is not defined after loading.
		//
		//		It returns the object `A.B`, but note the caveats above about on-the-fly loading and
		//		HTML script blocks when the xdomain loader is loading a module.
		//
		//		`dojo.require()` does nothing about importing symbols into
		//		the current namespace.	It is presumed that the caller will
		//		take care of that.
		//
		// example:
		//		To use dojo.require in conjunction with dojo.ready:
		//
		//		|	dojo.require("foo");
		//		|	dojo.require("bar");
		//		|	dojo.addOnLoad(function(){
		//		|		//you can now safely do something with foo and bar
		//		|	});
		//
		// example:
		//		For example, to import all symbols into a local block, you might write:
		//
		//		|	with (dojo.require("A.B")) {
		//		|		...
		//		|	}
		//
		//		And to import just the leaf symbol to a local variable:
		//
		//		|	var B = dojo.require("A.B");
		//		|	...
		//
		// returns:
		//		the required namespace object
		function doRequire(mid, omitModuleCheck){
			var module = getModule(slashName(mid), require.module);
			if(syncExecStack.length && syncExecStack[0].finish){
				// switched to async loading in the middle of evaluating a legacy module; stop
				// applying dojo.require so the remaining dojo.requires are applied in order
				syncExecStack[0].finish.push(mid);
				return undefined;
			}

			// recall module.executed has values {0, executing, executed}; therefore, truthy indicates executing or executed
			if(module.executed){
				return module.result;
			}
			omitModuleCheck && (module.result = nonmodule);

			// rcg...why here and in two lines??
			var currentMode = getLegacyMode();

			// recall, in sync mode to inject is to *eval* the module text
			// if the module is a legacy module, this is the same as executing
			// but if the module is an AMD module, this means defining, not executing
			injectModule(module);
			// the inject may have changed the mode
			currentMode = getLegacyMode();

			// in sync mode to dojo.require is to execute
			if(module.executed!==executed && module.injected===arrived){
				// the module was already here before injectModule was called probably finishing up a xdomain
				// load, but maybe a module given to the loader directly rather than having the loader retrieve it

				loaderVars.guardCheckComplete(function(){
					execModule(module);
				});
			}
			if(module.executed){
				return module.result;
			}

			if(currentMode==sync){
				// the only way to get here is in sync mode and dojo.required a module that
				//	 * was loaded async in the injectModule application a few lines up
				//	 * was an AMD module that had deps that are being loaded async and therefore couldn't execute
				if(module.cjs){
					// the module was an AMD module; unshift, not push, which causes the current traversal to be reattempted from the top
					execQ.unshift(module);
				}else{
					// the module was a legacy module
					syncExecStack.length && (syncExecStack[0].finish= [mid]);
				}
			}else{
				// the loader wasn't in sync mode on entry; probably async mode; therefore, no expectation of getting
				// the module value synchronously; make sure it gets executed though
				execQ.push(module);
			}

			return undefined;
		}

		var result = doRequire(moduleName, omitModuleCheck);
		if(has("config-publishRequireResult") && !lang.exists(moduleName) && result!==undefined){
			lang.setObject(moduleName, result);
		}
		return result;
	};

	dojo.loadInit = function(f) {
		f();
	};

	dojo.registerModulePath = function(/*String*/moduleName, /*String*/prefix){
		// summary:
		//		Maps a module name to a path
		// description:
		//		An unregistered module is given the default path of ../[module],
		//		relative to Dojo root. For example, module acme is mapped to
		//		../acme.  If you want to use a different module name, use
		//		dojo.registerModulePath.
		// example:
		//		If your dojo.js is located at this location in the web root:
		//	|	/myapp/js/dojo/dojo/dojo.js
		//		and your modules are located at:
		//	|	/myapp/js/foo/bar.js
		//	|	/myapp/js/foo/baz.js
		//	|	/myapp/js/foo/thud/xyzzy.js
		//		Your application can tell Dojo to locate the "foo" namespace by calling:
		//	|	dojo.registerModulePath("foo", "../../foo");
		//		At which point you can then use dojo.require() to load the
		//		modules (assuming they provide() the same things which are
		//		required). The full code might be:
		//	|	<script type="text/javascript"
		//	|		src="/myapp/js/dojo/dojo/dojo.js"></script>
		//	|	<script type="text/javascript">
		//	|		dojo.registerModulePath("foo", "../../foo");
		//	|		dojo.require("foo.bar");
		//	|		dojo.require("foo.baz");
		//	|		dojo.require("foo.thud.xyzzy");
		//	|	</script>

		var paths = {};
		paths[moduleName.replace(/\./g, "/")] = prefix;
		require({paths:paths});
	};

	dojo.platformRequire = function(/*Object*/modMap){
		// summary:
		//		require one or more modules based on which host environment
		//		Dojo is currently operating in
		// description:
		//		This method takes a "map" of arrays which one can use to
		//		optionally load dojo modules. The map is indexed by the
		//		possible dojo.name_ values, with two additional values:
		//		"default" and "common". The items in the "default" array will
		//		be loaded if none of the other items have been choosen based on
		//		dojo.name_, set by your host environment. The items in the
		//		"common" array will *always* be loaded, regardless of which
		//		list is chosen.
		// example:
		//		|	dojo.platformRequire({
		//		|		browser: [
		//		|			"foo.sample", // simple module
		//		|			"foo.test",
		//		|			["foo.bar.baz", true] // skip object check in _loadModule (dojo.require)
		//		|		],
		//		|		default: [ "foo.sample._base" ],
		//		|		common: [ "important.module.common" ]
		//		|	});

		var result = (modMap.common || []).concat(modMap[dojo._name] || modMap["default"] || []),
			temp;
		while(result.length){
			if(lang.isArray(temp = result.shift())){
				dojo.require.apply(dojo, temp);
			}else{
				dojo.require(temp);
			}
		}
	};

	dojo.requireIf = dojo.requireAfterIf = function(/*Boolean*/ condition, /*String*/ moduleName, /*Boolean?*/omitModuleCheck){
		// summary:
		//		If the condition is true then call `dojo.require()` for the specified
		//		resource
		//
		// example:
		//	|	dojo.requireIf(dojo.isBrowser, "my.special.Module");

		if(condition){
			dojo.require(moduleName, omitModuleCheck);
		}
	};

	dojo.requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale){
		require(["../i18n"], function(i18n){
			i18n.getLocalization(moduleName, bundleName, locale);
		});
	};

	return {
		// summary:
		//		This module defines the v1.x synchronous loader API.

		extractLegacyApiApplications:extractLegacyApiApplications,
		require:dojoRequirePlugin,
		loadInit:dojoLoadInitPlugin
	};
});

}}});
require({cache:{
'dojo/vsd':function(){
define([], 1);

},
'dojo/i18n':function(){
define(["./_base/kernel", "require", "./has", "./_base/array", "./_base/config", "./_base/lang", "./_base/xhr", "./json", "module"],
	function(dojo, require, has, array, config, lang, xhr, json, module){

	// module:
	//		dojo/i18n

	has.add("dojo-preload-i18n-Api",
		// if true, define the preload localizations machinery
		1
	);

	 1 || has.add("dojo-v1x-i18n-Api",
		// if true, define the v1.x i18n functions
		1
	);

	var
		thisModule = dojo.i18n =
			{
				// summary:
				//		This module implements the dojo/i18n! plugin and the v1.6- i18n API
				// description:
				//		We choose to include our own plugin to leverage functionality already contained in dojo
				//		and thereby reduce the size of the plugin compared to various loader implementations. Also, this
				//		allows foreign AMD loaders to be used without their plugins.
			},

		nlsRe =
			// regexp for reconstructing the master bundle name from parts of the regexp match
			// nlsRe.exec("foo/bar/baz/nls/en-ca/foo") gives:
			// ["foo/bar/baz/nls/en-ca/foo", "foo/bar/baz/nls/", "/", "/", "en-ca", "foo"]
			// nlsRe.exec("foo/bar/baz/nls/foo") gives:
			// ["foo/bar/baz/nls/foo", "foo/bar/baz/nls/", "/", "/", "foo", ""]
			// so, if match[5] is blank, it means this is the top bundle definition.
			// courtesy of http://requirejs.org
			/(^.*(^|\/)nls)(\/|$)([^\/]*)\/?([^\/]*)/,

		getAvailableLocales = function(
			root,
			locale,
			bundlePath,
			bundleName
		){
			// summary:
			//		return a vector of module ids containing all available locales with respect to the target locale
			//		For example, assuming:
			//
			//		- the root bundle indicates specific bundles for "fr" and "fr-ca",
			//		-  bundlePath is "myPackage/nls"
			//		- bundleName is "myBundle"
			//
			//		Then a locale argument of "fr-ca" would return
			//
			//			["myPackage/nls/myBundle", "myPackage/nls/fr/myBundle", "myPackage/nls/fr-ca/myBundle"]
			//
			//		Notice that bundles are returned least-specific to most-specific, starting with the root.
			//
			//		If root===false indicates we're working with a pre-AMD i18n bundle that doesn't tell about the available locales;
			//		therefore, assume everything is available and get 404 errors that indicate a particular localization is not available

			for(var result = [bundlePath + bundleName], localeParts = locale.split("-"), current = "", i = 0; i<localeParts.length; i++){
				current += (current ? "-" : "") + localeParts[i];
				if(!root || root[current]){
					result.push(bundlePath + current + "/" + bundleName);
					result.specificity = current;
				}
			}
			return result;
		},

		cache = {},

		getBundleName = function(moduleName, bundleName, locale){
			locale = locale ? locale.toLowerCase() : dojo.locale;
			moduleName = moduleName.replace(/\./g, "/");
			bundleName = bundleName.replace(/\./g, "/");
			return (/root/i.test(locale)) ?
				(moduleName + "/nls/" + bundleName) :
				(moduleName + "/nls/" + locale + "/" + bundleName);
		},

		getL10nName = dojo.getL10nName = function(moduleName, bundleName, locale){
			return moduleName = module.id + "!" + getBundleName(moduleName, bundleName, locale);
		},

		doLoad = function(require, bundlePathAndName, bundlePath, bundleName, locale, load){
			// summary:
			//		get the root bundle which instructs which other bundles are required to construct the localized bundle
			require([bundlePathAndName], function(root){
				var current = lang.clone(root.root),
					availableLocales = getAvailableLocales(!root._v1x && root, locale, bundlePath, bundleName);
				require(availableLocales, function(){
					for (var i = 1; i<availableLocales.length; i++){
						current = lang.mixin(lang.clone(current), arguments[i]);
					}
					// target may not have been resolve (e.g., maybe only "fr" exists when "fr-ca" was requested)
					var target = bundlePathAndName + "/" + locale;
					cache[target] = current;
					current.$locale = availableLocales.specificity;
					load();
				});
			});
		},

		normalize = function(id, toAbsMid){
			// summary:
			//		id may be relative.
			//		preload has form `*preload*<path>/nls/<module>*<flattened locales>` and
			//		therefore never looks like a relative
			return /^\./.test(id) ? toAbsMid(id) : id;
		},

		getLocalesToLoad = function(targetLocale){
			var list = config.extraLocale || [];
			list = lang.isArray(list) ? list : [list];
			list.push(targetLocale);
			return list;
		},

		load = function(id, require, load){
			// summary:
			//		id is in one of the following formats
			//
			//		1. <path>/nls/<bundle>
			//			=> load the bundle, localized to config.locale; load all bundles localized to
			//			config.extraLocale (if any); return the loaded bundle localized to config.locale.
			//
			//		2. <path>/nls/<locale>/<bundle>
			//			=> load then return the bundle localized to <locale>
			//
			//		3. *preload*<path>/nls/<module>*<JSON array of available locales>
			//			=> for config.locale and all config.extraLocale, load all bundles found
			//			in the best-matching bundle rollup. A value of 1 is returned, which
			//			is meaningless other than to say the plugin is executing the requested
			//			preloads
			//
			//		In cases 1 and 2, <path> is always normalized to an absolute module id upon entry; see
			//		normalize. In case 3, it <path> is assumed to be absolute; this is arranged by the builder.
			//
			//		To load a bundle means to insert the bundle into the plugin's cache and publish the bundle
			//		value to the loader. Given <path>, <bundle>, and a particular <locale>, the cache key
			//
			//			<path>/nls/<bundle>/<locale>
			//
			//		will hold the value. Similarly, then plugin will publish this value to the loader by
			//
			//			define("<path>/nls/<bundle>/<locale>", <bundle-value>);
			//
			//		Given this algorithm, other machinery can provide fast load paths be preplacing
			//		values in the plugin's cache, which is public. When a load is demanded the
			//		cache is inspected before starting any loading. Explicitly placing values in the plugin
			//		cache is an advanced/experimental feature that should not be needed; use at your own risk.
			//
			//		For the normal AMD algorithm, the root bundle is loaded first, which instructs the
			//		plugin what additional localized bundles are required for a particular locale. These
			//		additional locales are loaded and a mix of the root and each progressively-specific
			//		locale is returned. For example:
			//
			//		1. The client demands "dojo/i18n!some/path/nls/someBundle
			//
			//		2. The loader demands load(some/path/nls/someBundle)
			//
			//		3. This plugin require's "some/path/nls/someBundle", which is the root bundle.
			//
			//		4. Assuming config.locale is "ab-cd-ef" and the root bundle indicates that localizations
			//		are available for "ab" and "ab-cd-ef" (note the missing "ab-cd", then the plugin
			//		requires "some/path/nls/ab/someBundle" and "some/path/nls/ab-cd-ef/someBundle"
			//
			//		5. Upon receiving all required bundles, the plugin constructs the value of the bundle
			//		ab-cd-ef as...
			//
			//				mixin(mixin(mixin({}, require("some/path/nls/someBundle"),
			//		  			require("some/path/nls/ab/someBundle")),
			//					require("some/path/nls/ab-cd-ef/someBundle"));
			//
			//		This value is inserted into the cache and published to the loader at the
			//		key/module-id some/path/nls/someBundle/ab-cd-ef.
			//
			//		The special preload signature (case 3) instructs the plugin to stop servicing all normal requests
			//		(further preload requests will be serviced) until all ongoing preloading has completed.
			//
			//		The preload signature instructs the plugin that a special rollup module is available that contains
			//		one or more flattened, localized bundles. The JSON array of available locales indicates which locales
			//		are available. Here is an example:
			//
			//			*preload*some/path/nls/someModule*["root", "ab", "ab-cd-ef"]
			//
			//		This indicates the following rollup modules are available:
			//
			//			some/path/nls/someModule_ROOT
			//			some/path/nls/someModule_ab
			//			some/path/nls/someModule_ab-cd-ef
			//
			//		Each of these modules is a normal AMD module that contains one or more flattened bundles in a hash.
			//		For example, assume someModule contained the bundles some/bundle/path/someBundle and
			//		some/bundle/path/someOtherBundle, then some/path/nls/someModule_ab would be expressed as follows:
			//
			//			define({
			//				some/bundle/path/someBundle:<value of someBundle, flattened with respect to locale ab>,
			//				some/bundle/path/someOtherBundle:<value of someOtherBundle, flattened with respect to locale ab>,
			//			});
			//
			//		E.g., given this design, preloading for locale=="ab" can execute the following algorithm:
			//
			//			require(["some/path/nls/someModule_ab"], function(rollup){
			//				for(var p in rollup){
			//					var id = p + "/ab",
			//					cache[id] = rollup[p];
			//					define(id, rollup[p]);
			//				}
			//			});
			//
			//		Similarly, if "ab-cd" is requested, the algorithm can determine that "ab" is the best available and
			//		load accordingly.
			//
			//		The builder will write such rollups for every layer if a non-empty localeList  profile property is
			//		provided. Further, the builder will include the following cache entry in the cache associated with
			//		any layer.
			//
			//			"*now":function(r){r(['dojo/i18n!*preload*<path>/nls/<module>*<JSON array of available locales>']);}
			//
			//		The *now special cache module instructs the loader to apply the provided function to context-require
			//		with respect to the particular layer being defined. This causes the plugin to hold all normal service
			//		requests until all preloading is complete.
			//
			//		Notice that this algorithm is rarely better than the standard AMD load algorithm. Consider the normal case
			//		where the target locale has a single segment and a layer depends on a single bundle:
			//
			//		Without Preloads:
			//
			//		1. Layer loads root bundle.
			//		2. bundle is demanded; plugin loads single localized bundle.
			//
			//		With Preloads:
			//
			//		1. Layer causes preloading of target bundle.
			//		2. bundle is demanded; service is delayed until preloading complete; bundle is returned.
			//
			//		In each case a single transaction is required to load the target bundle. In cases where multiple bundles
			//		are required and/or the locale has multiple segments, preloads still requires a single transaction whereas
			//		the normal path requires an additional transaction for each additional bundle/locale-segment. However all
			//		of these additional transactions can be done concurrently. Owing to this analysis, the entire preloading
			//		algorithm can be discard during a build by setting the has feature dojo-preload-i18n-Api to false.

			if(has("dojo-preload-i18n-Api")){
				var split = id.split("*"),
					preloadDemand = split[1] == "preload";
				if(preloadDemand){
					if(!cache[id]){
						// use cache[id] to prevent multiple preloads of the same preload; this shouldn't happen, but
						// who knows what over-aggressive human optimizers may attempt
						cache[id] = 1;
						preloadL10n(split[2], json.parse(split[3]), 1, require);
					}
					// don't stall the loader!
					load(1);
				}
				if(preloadDemand || waitForPreloads(id, require, load)){
					return;
				}
			}

			var match = nlsRe.exec(id),
				bundlePath = match[1] + "/",
				bundleName = match[5] || match[4],
				bundlePathAndName = bundlePath + bundleName,
				localeSpecified = (match[5] && match[4]),
				targetLocale =	localeSpecified || dojo.locale || "",
				loadTarget = bundlePathAndName + "/" + targetLocale,
				loadList = localeSpecified ? [targetLocale] : getLocalesToLoad(targetLocale),
				remaining = loadList.length,
				finish = function(){
					if(!--remaining){
						load(lang.delegate(cache[loadTarget]));
					}
				};
			array.forEach(loadList, function(locale){
				var target = bundlePathAndName + "/" + locale;
				if(has("dojo-preload-i18n-Api")){
					checkForLegacyModules(target);
				}
				if(!cache[target]){
					doLoad(require, bundlePathAndName, bundlePath, bundleName, locale, finish);
				}else{
					finish();
				}
			});
		};

	if(has("dojo-unit-tests")){
		var unitTests = thisModule.unitTests = [];
	}

	if(has("dojo-preload-i18n-Api") ||  1 ){
		var normalizeLocale = thisModule.normalizeLocale = function(locale){
				var result = locale ? locale.toLowerCase() : dojo.locale;
				return result == "root" ? "ROOT" : result;
			},

			isXd = function(mid, contextRequire){
				return ( 1  &&  1 ) ?
					contextRequire.isXdUrl(require.toUrl(mid + ".js")) :
					true;
			},

			preloading = 0,

			preloadWaitQueue = [],

			preloadL10n = thisModule._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated, /*boolean?*/ guaranteedAmdFormat, /*function?*/ contextRequire){
				// summary:
				//		Load available flattened resource bundles associated with a particular module for dojo/locale and all dojo/config.extraLocale (if any)
				// description:
				//		Only called by built layer files. The entire locale hierarchy is loaded. For example,
				//		if locale=="ab-cd", then ROOT, "ab", and "ab-cd" are loaded. This is different than v1.6-
				//		in that the v1.6- would only load ab-cd...which was *always* flattened.
				//
				//		If guaranteedAmdFormat is true, then the module can be loaded with require thereby circumventing the detection algorithm
				//		and the extra possible extra transaction.

				// If this function is called from legacy code, then guaranteedAmdFormat and contextRequire will be undefined. Since the function
				// needs a require in order to resolve module ids, fall back to the context-require associated with this dojo/i18n module, which
				// itself may have been mapped.
				contextRequire = contextRequire || require;

				function doRequire(mid, callback){
					if(isXd(mid, contextRequire) || guaranteedAmdFormat){
						contextRequire([mid], callback);
					}else{
						syncRequire([mid], callback, contextRequire);
					}
				}

				function forEachLocale(locale, func){
					// given locale= "ab-cd-ef", calls func on "ab-cd-ef", "ab-cd", "ab", "ROOT"; stops calling the first time func returns truthy
					var parts = locale.split("-");
					while(parts.length){
						if(func(parts.join("-"))){
							return;
						}
						parts.pop();
					}
					func("ROOT");
				}

				function preload(locale){
					locale = normalizeLocale(locale);
					forEachLocale(locale, function(loc){
						if(array.indexOf(localesGenerated, loc)>=0){
							var mid = bundlePrefix.replace(/\./g, "/")+"_"+loc;
							preloading++;
							doRequire(mid, function(rollup){
								for(var p in rollup){
									cache[require.toAbsMid(p) + "/" + loc] = rollup[p];
								}
								--preloading;
								while(!preloading && preloadWaitQueue.length){
									load.apply(null, preloadWaitQueue.shift());
								}
							});
							return true;
						}
						return false;
					});
				}

				preload();
				array.forEach(dojo.config.extraLocale, preload);
			},

			waitForPreloads = function(id, require, load){
				if(preloading){
					preloadWaitQueue.push([id, require, load]);
				}
				return preloading;
			},

			checkForLegacyModules = function()
				{};
	}

	if( 1 ){
		// this code path assumes the dojo loader and won't work with a standard AMD loader
		var amdValue = {},
			evalBundle =
				// use the function ctor to keep the minifiers away (also come close to global scope, but this is secondary)
				new Function(
					"__bundle",				   // the bundle to evalutate
					"__checkForLegacyModules", // a function that checks if __bundle defined __mid in the global space
					"__mid",				   // the mid that __bundle is intended to define
					"__amdValue",

					// returns one of:
					//		1 => the bundle was an AMD bundle
					//		a legacy bundle object that is the value of __mid
					//		instance of Error => could not figure out how to evaluate bundle

					  // used to detect when __bundle calls define
					  "var define = function(mid, factory){define.called = 1; __amdValue.result = factory || mid;},"
					+ "	   require = function(){define.called = 1;};"

					+ "try{"
					+		"define.called = 0;"
					+		"eval(__bundle);"
					+		"if(define.called==1)"
								// bundle called define; therefore signal it's an AMD bundle
					+			"return __amdValue;"

					+		"if((__checkForLegacyModules = __checkForLegacyModules(__mid)))"
								// bundle was probably a v1.6- built NLS flattened NLS bundle that defined __mid in the global space
					+			"return __checkForLegacyModules;"

					+ "}catch(e){}"
					// evaulating the bundle was *neither* an AMD *nor* a legacy flattened bundle
					// either way, re-eval *after* surrounding with parentheses

					+ "try{"
					+		"return eval('('+__bundle+')');"
					+ "}catch(e){"
					+		"return e;"
					+ "}"
				),

			syncRequire = function(deps, callback, require){
				var results = [];
				array.forEach(deps, function(mid){
					var url = require.toUrl(mid + ".js");

					function load(text){
						var result = evalBundle(text, checkForLegacyModules, mid, amdValue);
						if(result===amdValue){
							// the bundle was an AMD module; re-inject it through the normal AMD path
							// we gotta do this since it could be an anonymous module and simply evaluating
							// the text here won't provide the loader with the context to know what
							// module is being defined()'d. With browser caching, this should be free; further
							// this entire code path can be circumvented by using the AMD format to begin with
							results.push(cache[url] = amdValue.result);
						}else{
							if(result instanceof Error){
								console.error("failed to evaluate i18n bundle; url=" + url, result);
								result = {};
							}
							// nls/<locale>/<bundle-name> indicates not the root.
							results.push(cache[url] = (/nls\/[^\/]+\/[^\/]+$/.test(url) ? result : {root:result, _v1x:1}));
						}
					}

					if(cache[url]){
						results.push(cache[url]);
					}else{
						var bundle = require.syncLoadNls(mid);
						// don't need to check for legacy since syncLoadNls returns a module if the module
						// (1) was already loaded, or (2) was in the cache. In case 1, if syncRequire is called
						// from getLocalization --> load, then load will have called checkForLegacyModules() before
						// calling syncRequire; if syncRequire is called from preloadLocalizations, then we
						// don't care about checkForLegacyModules() because that will be done when a particular
						// bundle is actually demanded. In case 2, checkForLegacyModules() is never relevant
						// because cached modules are always v1.7+ built modules.
						if(bundle){
							results.push(bundle);
						}else{
							if(!xhr){
								try{
									require.getText(url, true, load);
								}catch(e){
									results.push(cache[url] = {});
								}
							}else{
								xhr.get({
									url:url,
									sync:true,
									load:load,
									error:function(){
										results.push(cache[url] = {});
									}
								});
							}
						}
					}
				});
				callback && callback.apply(null, results);
			};

		checkForLegacyModules = function(target){
			// legacy code may have already loaded [e.g] the raw bundle x/y/z at x.y.z; when true, push into the cache
			for(var result, names = target.split("/"), object = dojo.global[names[0]], i = 1; object && i<names.length-1; object = object[names[i++]]){}
			if(object){
				result = object[names[i]];
				if(!result){
					// fallback for incorrect bundle build of 1.6
					result = object[names[i].replace(/-/g,"_")];
				}
				if(result){
					cache[target] = result;
				}
			}
			return result;
		};

		thisModule.getLocalization = function(moduleName, bundleName, locale){
			var result,
				l10nName = getBundleName(moduleName, bundleName, locale);
			load(
				l10nName,

				// isXd() and syncRequire() need a context-require in order to resolve the mid with respect to a reference module.
				// Since this legacy function does not have the concept of a reference module, resolve with respect to this
				// dojo/i18n module, which, itself may have been mapped.
				(!isXd(l10nName, require) ? function(deps, callback){ syncRequire(deps, callback, require); } : require),

				function(result_){ result = result_; }
			);
			return result;
		};

		if(has("dojo-unit-tests")){
			unitTests.push(function(doh){
				doh.register("tests.i18n.unit", function(t){
					var check;

					check = evalBundle("{prop:1}", checkForLegacyModules, "nonsense", amdValue);
					t.is({prop:1}, check); t.is(undefined, check[1]);

					check = evalBundle("({prop:1})", checkForLegacyModules, "nonsense", amdValue);
					t.is({prop:1}, check); t.is(undefined, check[1]);

					check = evalBundle("{'prop-x':1}", checkForLegacyModules, "nonsense", amdValue);
					t.is({'prop-x':1}, check); t.is(undefined, check[1]);

					check = evalBundle("({'prop-x':1})", checkForLegacyModules, "nonsense", amdValue);
					t.is({'prop-x':1}, check); t.is(undefined, check[1]);

					check = evalBundle("define({'prop-x':1})", checkForLegacyModules, "nonsense", amdValue);
					t.is(amdValue, check); t.is({'prop-x':1}, amdValue.result);

					check = evalBundle("define('some/module', {'prop-x':1})", checkForLegacyModules, "nonsense", amdValue);
					t.is(amdValue, check); t.is({'prop-x':1}, amdValue.result);

					check = evalBundle("this is total nonsense and should throw an error", checkForLegacyModules, "nonsense", amdValue);
					t.is(check instanceof Error, true);
				});
			});
		}
	}

	return lang.mixin(thisModule, {
		dynamic:true,
		normalize:normalize,
		load:load,
		cache:cache,
		getL10nName: getL10nName
	});
});

},
'dijit/registry':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/sniff", // has("ie")
	"dojo/_base/window", // win.body
	"./main"	// dijit._scopeName
], function(array, has, win, dijit){

	// module:
	//		dijit/registry

	var _widgetTypeCtr = {}, hash = {};

	var registry =  {
		// summary:
		//		Registry of existing widget on page, plus some utility methods.

		// length: Number
		//		Number of registered widgets
		length: 0,

		add: function(widget){
			// summary:
			//		Add a widget to the registry. If a duplicate ID is detected, a error is thrown.
			// widget: dijit/_WidgetBase
			//		Any dijit/_WidgetBase subclass.
			if(hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from the registry. Does not destroy the widget; simply
			//		removes the reference.
			if(hash[id]){
				delete hash[id];
				this.length--;
			}
		},

		byId: function(/*String|Widget*/ id){
			// summary:
			//		Find a widget by it's id.
			//		If passed a widget then just returns the widget.
			return typeof id == "string" ? hash[id] : id;	// dijit/_WidgetBase
		},

		byNode: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget corresponding to the given DOMNode
			return hash[node.getAttribute("widgetId")]; // dijit/_WidgetBase
		},

		toArray: function(){
			// summary:
			//		Convert registry into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	array.map(registry.toArray(), function(w){ return w.domNode; });

			var ar = [];
			for(var id in hash){
				ar.push(hash[id]);
			}
			return ar;	// dijit/_WidgetBase[]
		},

		getUniqueId: function(/*String*/widgetType){
			// summary:
			//		Generates a unique id for a given widgetType

			var id;
			do{
				id = widgetType + "_" +
					(widgetType in _widgetTypeCtr ?
						++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
			}while(hash[id]);
			return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
		},

		findWidgets: function(root, skipNode){
			// summary:
			//		Search subtree under root returning widgets found.
			//		Doesn't search for nested widgets (ie, widgets inside other widgets).
			// root: DOMNode
			//		Node to search under.
			// skipNode: DOMNode
			//		If specified, don't search beneath this node (usually containerNode).

			var outAry = [];

			function getChildrenHelper(root){
				for(var node = root.firstChild; node; node = node.nextSibling){
					if(node.nodeType == 1){
						var widgetId = node.getAttribute("widgetId");
						if(widgetId){
							var widget = hash[widgetId];
							if(widget){	// may be null on page w/multiple dojo's loaded
								outAry.push(widget);
							}
						}else if(node !== skipNode){
							getChildrenHelper(node);
						}
					}
				}
			}

			getChildrenHelper(root);
			return outAry;
		},

		_destroyAll: function(){
			// summary:
			//		Code to destroy all widgets and do other cleanup on page unload

			// Clean up focus manager lingering references to widgets and nodes
			dijit._curFocus = null;
			dijit._prevFocus = null;
			dijit._activeStack = [];

			// Destroy all the widgets, top down
			array.forEach(registry.findWidgets(win.body()), function(widget){
				// Avoid double destroy of widgets like Menu that are attached to <body>
				// even though they are logically children of other widgets.
				if(!widget._destroyed){
					if(widget.destroyRecursive){
						widget.destroyRecursive();
					}else if(widget.destroy){
						widget.destroy();
					}
				}
			});
		},

		getEnclosingWidget: function(/*DOMNode*/ node){
			// summary:
			//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
			//		the node is not contained within the DOM tree of any widget
			while(node){
				var id = node.nodeType == 1 && node.getAttribute("widgetId");
				if(id){
					return hash[id];
				}
				node = node.parentNode;
			}
			return null;
		},

		// In case someone needs to access hash.
		// Actually, this is accessed from WidgetSet back-compatibility code
		_hash: hash
	};

	dijit.registry = registry;

	return registry;
});

},
'dijit/main':function(){
define([
	"dojo/_base/kernel"
], function(dojo){
	// module:
	//		dijit/main

/*=====
return {
	// summary:
	//		The dijit package main module.
	//		Deprecated.   Users should access individual modules (ex: dijit/registry) directly.
};
=====*/

	return dojo.dijit;
});

},
'dojo/_firebug/firebug':function(){
define([
	"../_base/kernel",
	"require",
	"../_base/html",
	"../sniff",
	"../_base/array",
	"../_base/lang",
	"../_base/event",
	"../_base/unload"], function(dojo, require, html, has){

	// module:
	//		dojo/_firebug/firebug
	// summary:
	//		Firebug Lite, the baby brother to Joe Hewitt's Firebug for Mozilla Firefox
	// description:
	//		Opens a console for logging, debugging, and error messages.
	//		Contains partial functionality to Firebug. See function list below.
	//
	//		NOTE:
	//		Firebug is a Firefox extension created by Joe Hewitt (see license). You do not need Dojo to run Firebug.
	//		Firebug Lite is included in Dojo by permission from Joe Hewitt
	//		If you are new to Firebug, or used to the Dojo 0.4 dojo.debug, you can learn Firebug
	//		functionality by reading the function comments below or visiting http://www.getfirebug.com/docs.html
	//
	//		NOTE:
	//		To test Firebug Lite in Firefox:
	//
	//		- FF2: set "console = null" before loading dojo and set djConfig.isDebug=true
	//		- FF3: disable Firebug and set djConfig.isDebug=true
	//
	// example:
	//		Supports inline objects in object inspector window (only simple trace of dom nodes, however)
	//	|	console.log("my object", {foo:"bar"})
	// example:
	//		Option for console to open in popup window
	//	|	var djConfig = {isDebug: true, popup:true };
	// example:
	//		Option for console height (ignored for popup)
	//	|	var djConfig = {isDebug: true, debugHeight:100 }


	var isNewIE = (/Trident/.test(window.navigator.userAgent));
	if(isNewIE){
		// Fixing IE's console
		// IE doesn't insert space between arguments. How annoying.
		var calls = ["log", "info", "debug", "warn", "error"];
		for(var i=0;i<calls.length;i++){
			var m = calls[i];
			if(!console[m] ||console[m]._fake){
				// IE9 doesn't have console.debug method, a fake one is added later
				continue;
			}
			var n = "_"+calls[i];
			console[n] = console[m];
			console[m] = (function(){
				var type = n;
				return function(){
					console[type](Array.prototype.join.call(arguments, " "));
				};
			})();
		}
		// clear the console on load. This is more than a convenience - too many logs crashes it.
		// If closed it throws an error
		try{ console.clear(); }catch(e){}
	}

	if(
		has("ff") ||								// Firefox has Firebug
		has("chrome") ||							// Chrome 3+ has a console
		has("safari") ||							// Safari 4 has a console
		isNewIE ||									// Has the new IE console
		window.firebug ||							// Testing for mozilla firebug lite
		(typeof console != "undefined" && console.firebug) || //The firebug console
		dojo.config.useCustomLogger ||				// Allow custom loggers
		has("air")									// isDebug triggers AIRInsector, not Firebug
	){
		return;
	}

	// don't build firebug in iframes
	try{
		if(window != window.parent){
			// but if we've got a parent logger, connect to it
			if(window.parent["console"]){
				window.console = window.parent.console;
			}
			return;
		}
	}catch(e){/*squelch*/}

	// ***************************************************************************
	// Placing these variables before the functions that use them to avoid a
	// shrinksafe bug where variable renaming does not happen correctly otherwise.

	// most of the objects in this script are run anonomously
	var _firebugDoc = document;
	var _firebugWin = window;
	var __consoleAnchorId__ = 0;

	var consoleFrame = null;
	var consoleBody = null;
	var consoleObjectInspector = null;
	var fireBugTabs = null;
	var commandLine = null;
	var consoleToolbar = null;

	var frameVisible = false;
	var messageQueue = [];
	var groupStack = [];
	var timeMap = {};
	var countMap = {};

	var consoleDomInspector = null;
	var _inspectionMoveConnection;
	var _inspectionClickConnection;
	var _inspectionEnabled = false;
	var _inspectionTimer = null;
	var _inspectTempNode = document.createElement("div");


	var _inspectCurrentNode;
	var _restoreBorderStyle;

	// ***************************************************************************

	window.console = {
		_connects: [],
		log: function(){
			// summary:
			//		Sends arguments to console.
			logFormatted(arguments, "");
		},

		debug: function(){
			// summary:
			//		Sends arguments to console. Missing finctionality to show script line of trace.
			logFormatted(arguments, "debug");
		},

		info: function(){
			// summary:
			//		Sends arguments to console, highlighted with (I) icon.
			logFormatted(arguments, "info");
		},

		warn: function(){
			// summary:
			//		Sends warning arguments to console, highlighted with (!) icon and blue style.
			logFormatted(arguments, "warning");
		},

		error: function(){
			// summary:
			//		Sends error arguments (object) to console, highlighted with (X) icon and yellow style
			//		NEW: error object now displays in object inspector
			logFormatted(arguments, "error");
		},

		assert: function(truth, message){
			// summary:
			//		Tests for true. Throws exception if false.
			if(!truth){
				var args = [];
				for(var i = 1; i < arguments.length; ++i){
					args.push(arguments[i]);
				}

				logFormatted(args.length ? args : ["Assertion Failure"], "error");
				throw message ? message : "Assertion Failure";
			}
		},

		dir: function(obj){
			var str = printObject( obj );
			str = str.replace(/\n/g, "<br />");
			str = str.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
			logRow([str], "dir");
		},

		dirxml: function(node){
			var html = [];
			appendNode(node, html);
			logRow(html, "dirxml");
		},

		group: function(){
			// summary:
			//		collects log messages into a group, starting with this call and ending with
			//		groupEnd(). Missing collapse functionality
			logRow(arguments, "group", pushGroup);
		},

		groupEnd: function(){
			// summary:
			//		Closes group. See above
			logRow(arguments, "", popGroup);
		},

		time: function(name){
			// summary:
			//		Starts timers assigned to name given in argument. Timer stops and displays on timeEnd(title);
			// example:
			//	|	console.time("load");
			//	|	console.time("myFunction");
			//	|	console.timeEnd("load");
			//	|	console.timeEnd("myFunction");
			timeMap[name] = new Date().getTime();
		},

		timeEnd: function(name){
			// summary:
			//		See above.
			if(name in timeMap){
				var delta = (new Date()).getTime() - timeMap[name];
				logFormatted([name+ ":", delta+"ms"]);
				delete timeMap[name];
			}
		},

		count: function(name){
			// summary:
			//		Not supported
			if(!countMap[name]) countMap[name] = 0;
			countMap[name]++;
			logFormatted([name+": "+countMap[name]]);
		},

		trace: function(_value){
			var stackAmt = _value || 3;
			var f = console.trace.caller; //function that called trace
			console.log(">>> console.trace(stack)");
			for(var i=0;i<stackAmt;i++){
				var func = f.toString();
				var args=[];
				for (var a = 0; a < f.arguments.length; a++){
					args.push(f.arguments[a]);
				}
				if(f.arguments.length){
					console.dir({"function":func, "arguments":args});
				}else{
					console.dir({"function":func});
				}

				f = f.caller;
			}
		},

		profile: function(){
			// summary:
			//		Not supported
			this.warn(["profile() not supported."]);
		},

		profileEnd: function(){ },

		clear: function(){
			// summary:
			//		Clears message console. Do not call this directly
			if(consoleBody){
				while(consoleBody.childNodes.length){
					dojo.destroy(consoleBody.firstChild);
				}
			}
			dojo.forEach(this._connects,dojo.disconnect);
		},

		open: function(){
			// summary:
			//		Opens message console. Do not call this directly
			toggleConsole(true);
		},

		close: function(){
			// summary:
			//		Closes message console. Do not call this directly
			if(frameVisible){
				toggleConsole();
			}
		},
		_restoreBorder: function(){
			if(_inspectCurrentNode){
				_inspectCurrentNode.style.border = _restoreBorderStyle;
			}
		},
		openDomInspector: function(){
			_inspectionEnabled = true;
			consoleBody.style.display = "none";
			consoleDomInspector.style.display = "block";
			consoleObjectInspector.style.display = "none";
			document.body.style.cursor = "pointer";
			_inspectionMoveConnection = dojo.connect(document, "mousemove", function(evt){
				if(!_inspectionEnabled){ return; }
				if(!_inspectionTimer){
					_inspectionTimer = setTimeout(function(){ _inspectionTimer = null; }, 50);
				}else{
					return;
				}
				var node = evt.target;
				if(node && (_inspectCurrentNode !== node)){
					var parent = true;

					console._restoreBorder();
					var html = [];
					appendNode(node, html);
					consoleDomInspector.innerHTML = html.join("");

					_inspectCurrentNode = node;
					_restoreBorderStyle = _inspectCurrentNode.style.border;
					_inspectCurrentNode.style.border = "#0000FF 1px solid";
				}
			});
			setTimeout(function(){
				_inspectionClickConnection = dojo.connect(document, "click", function(evt){
					document.body.style.cursor = "";
					_inspectionEnabled = !_inspectionEnabled;
					dojo.disconnect(_inspectionClickConnection);
					// console._restoreBorder();
				});
			}, 30);
		},
		_closeDomInspector: function(){
			document.body.style.cursor = "";
			dojo.disconnect(_inspectionMoveConnection);
			dojo.disconnect(_inspectionClickConnection);
			_inspectionEnabled = false;
			console._restoreBorder();
		},
		openConsole:function(){
			// summary:
			//		Closes object inspector and opens message console. Do not call this directly
			consoleBody.style.display = "block";
			consoleDomInspector.style.display = "none";
			consoleObjectInspector.style.display = "none";
			console._closeDomInspector();
		},
		openObjectInspector:function(){
			consoleBody.style.display = "none";
			consoleDomInspector.style.display = "none";
			consoleObjectInspector.style.display = "block";
			console._closeDomInspector();
		},
		recss: function(){
			// this is placed in dojo since the console is most likely
			// in another window and dojo is easilly accessible
			var i,a,s;a=document.getElementsByTagName('link');
			for(i=0;i<a.length;i++){
				s=a[i];
				if(s.rel.toLowerCase().indexOf('stylesheet')>=0&&s.href){
					var h=s.href.replace(/(&|%5C?)forceReload=\d+/,'');
					s.href=h+(h.indexOf('?')>=0?'&':'?')+'forceReload='+new Date().valueOf();
				}
			}
		}
	};

	// ***************************************************************************

	function toggleConsole(forceOpen){
		frameVisible = forceOpen || !frameVisible;
		if(consoleFrame){
			consoleFrame.style.display = frameVisible ? "block" : "none";
		}
	}

	function focusCommandLine(){
		toggleConsole(true);
		if(commandLine){
			commandLine.focus();
		}
	}

	function openWin(x,y,w,h){
		var win = window.open("","_firebug","status=0,menubar=0,resizable=1,top="+y+",left="+x+",width="+w+",height="+h+",scrollbars=1,addressbar=0");
		if(!win){
			var msg = "Firebug Lite could not open a pop-up window, most likely because of a blocker.\n" +
				"Either enable pop-ups for this domain, or change the djConfig to popup=false.";
			alert(msg);
		}
		createResizeHandler(win);
		var newDoc=win.document;
		//Safari needs an HTML height
		var HTMLstring=	'<html style="height:100%;"><head><title>Firebug Lite</title></head>\n' +
					'<body bgColor="#ccc" style="height:97%;" onresize="opener.onFirebugResize()">\n' +
					'<div id="fb"></div>' +
					'</body></html>';

		newDoc.write(HTMLstring);
		newDoc.close();
		return win;
	}

	function createResizeHandler(wn){
		// summary:
		//		Creates handle for onresize window. Called from script in popup's body tag (so that it will work with IE).
		//

		var d = new Date();
			d.setTime(d.getTime()+(60*24*60*60*1000)); // 60 days
			d = d.toUTCString();

			var dc = wn.document,
				getViewport;

			if (wn.innerWidth){
				getViewport = function(){
					return{w:wn.innerWidth, h:wn.innerHeight};
				};
			}else if (dc.documentElement && dc.documentElement.clientWidth){
				getViewport = function(){
					return{w:dc.documentElement.clientWidth, h:dc.documentElement.clientHeight};
				};
			}else if (dc.body){
				getViewport = function(){
					return{w:dc.body.clientWidth, h:dc.body.clientHeight};
				};
			}


		window.onFirebugResize = function(){

			//resize the height of the console log body
			layout(getViewport().h);

			clearInterval(wn._firebugWin_resize);
			wn._firebugWin_resize = setTimeout(function(){
				var x = wn.screenLeft,
					y = wn.screenTop,
					w = wn.outerWidth  || wn.document.body.offsetWidth,
					h = wn.outerHeight || wn.document.body.offsetHeight;

				document.cookie = "_firebugPosition=" + [x,y,w,h].join(",") + "; expires="+d+"; path=/";

			 }, 5000); //can't capture window.onMove - long timeout gives better chance of capturing a resize, then the move

		};
	}


	/*****************************************************************************/


	function createFrame(){
		if(consoleFrame){
			return;
		}
		toggleConsole(true);
		if(dojo.config.popup){
			var containerHeight = "100%";
			var cookieMatch = document.cookie.match(/(?:^|; )_firebugPosition=([^;]*)/);
			var p = cookieMatch ? cookieMatch[1].split(",") : [2,2,320,480];

			_firebugWin = openWin(p[0],p[1],p[2],p[3]);	// global
			_firebugDoc = _firebugWin.document;			// global

			dojo.config.debugContainerId = 'fb';

			// connecting popup
			_firebugWin.console = window.console;
			_firebugWin.dojo = window.dojo;
		}else{
			_firebugDoc = document;
			containerHeight = (dojo.config.debugHeight || 300) + "px";
		}

		var styleElement = _firebugDoc.createElement("link");
		styleElement.href = require.toUrl("./firebug.css");
		styleElement.rel = "stylesheet";
		styleElement.type = "text/css";
		var styleParent = _firebugDoc.getElementsByTagName("head");
		if(styleParent){
			styleParent = styleParent[0];
		}
		if(!styleParent){
			styleParent = _firebugDoc.getElementsByTagName("html")[0];
		}
		if(has("ie")){
			window.setTimeout(function(){ styleParent.appendChild(styleElement); }, 0);
		}else{
			styleParent.appendChild(styleElement);
		}

		if(dojo.config.debugContainerId){
			consoleFrame = _firebugDoc.getElementById(dojo.config.debugContainerId);
		}
		if(!consoleFrame){
			consoleFrame = _firebugDoc.createElement("div");
			_firebugDoc.body.appendChild(consoleFrame);
		}
		consoleFrame.className += " firebug";
		consoleFrame.id = "firebug";
		consoleFrame.style.height = containerHeight;
		consoleFrame.style.display = (frameVisible ? "block" : "none");

		var buildLink = function(label, title, method, _class){
			return '<li class="'+_class+'"><a href="javascript:void(0);" onclick="console.'+ method +'(); return false;" title="'+title+'">'+label+'</a></li>';
		};
		consoleFrame.innerHTML =
			  '<div id="firebugToolbar">'
			+ '  <ul id="fireBugTabs" class="tabs">'

			+ buildLink("Clear", "Remove All Console Logs", "clear", "")
			+ buildLink("ReCSS", "Refresh CSS without reloading page", "recss", "")

			+ buildLink("Console", "Show Console Logs", "openConsole", "gap")
			+ buildLink("DOM", "Show DOM Inspector", "openDomInspector", "")
			+ buildLink("Object", "Show Object Inspector", "openObjectInspector", "")
			+ ((dojo.config.popup) ? "" : buildLink("Close", "Close the console", "close", "gap"))

			+ '	</ul>'
			+ '</div>'
			+ '<input type="text" id="firebugCommandLine" />'
			+ '<div id="firebugLog"></div>'
			+ '<div id="objectLog" style="display:none;">Click on an object in the Log display</div>'
			+ '<div id="domInspect" style="display:none;">Hover over HTML elements in the main page. Click to hold selection.</div>';


		consoleToolbar = _firebugDoc.getElementById("firebugToolbar");

		commandLine = _firebugDoc.getElementById("firebugCommandLine");
		addEvent(commandLine, "keydown", onCommandLineKeyDown);

		addEvent(_firebugDoc, has("ie") || has("safari") ? "keydown" : "keypress", onKeyDown);

		consoleBody = _firebugDoc.getElementById("firebugLog");
		consoleObjectInspector = _firebugDoc.getElementById("objectLog");
		consoleDomInspector = _firebugDoc.getElementById("domInspect");
		fireBugTabs = _firebugDoc.getElementById("fireBugTabs");
		layout();
		flush();
	}

	dojo.addOnLoad(createFrame);

	function clearFrame(){
		_firebugDoc = null;

		if(_firebugWin.console){
			_firebugWin.console.clear();
		}
		_firebugWin = null;
		consoleFrame = null;
		consoleBody = null;
		consoleObjectInspector = null;
		consoleDomInspector = null;
		commandLine = null;
		messageQueue = [];
		groupStack = [];
		timeMap = {};
	}


	function evalCommandLine(){
		var text = commandLine.value;
		commandLine.value = "";

		logRow([">  ", text], "command");

		var value;
		try{
			value = eval(text);
		}catch(e){
			console.debug(e); // put exception on the console
		}

		console.log(value);
	}

	function layout(h){
		var tHeight = 25; //consoleToolbar.offsetHeight; // tab style not ready on load - throws off layout
		var height = h ?
			h  - (tHeight + commandLine.offsetHeight +25 + (h*.01)) + "px" :
			(consoleFrame.offsetHeight - tHeight - commandLine.offsetHeight) + "px";

		consoleBody.style.top = tHeight + "px";
		consoleBody.style.height = height;
		consoleObjectInspector.style.height = height;
		consoleObjectInspector.style.top = tHeight + "px";
		consoleDomInspector.style.height = height;
		consoleDomInspector.style.top = tHeight + "px";
		commandLine.style.bottom = 0;

		dojo.addOnWindowUnload(clearFrame);
	}

	function logRow(message, className, handler){
		if(consoleBody){
			writeMessage(message, className, handler);
		}else{
			messageQueue.push([message, className, handler]);
		}
	}

	function flush(){
		var queue = messageQueue;
		messageQueue = [];

		for(var i = 0; i < queue.length; ++i){
			writeMessage(queue[i][0], queue[i][1], queue[i][2]);
		}
	}

	function writeMessage(message, className, handler){
		var isScrolledToBottom =
			consoleBody.scrollTop + consoleBody.offsetHeight >= consoleBody.scrollHeight;

		handler = handler||writeRow;

		handler(message, className);

		if(isScrolledToBottom){
			consoleBody.scrollTop = consoleBody.scrollHeight - consoleBody.offsetHeight;
		}
	}

	function appendRow(row){
		var container = groupStack.length ? groupStack[groupStack.length-1] : consoleBody;
		container.appendChild(row);
	}

	function writeRow(message, className){
		var row = consoleBody.ownerDocument.createElement("div");
		row.className = "logRow" + (className ? " logRow-"+className : "");
		row.innerHTML = message.join("");
		appendRow(row);
	}

	function pushGroup(message, className){
		logFormatted(message, className);

		//var groupRow = consoleBody.ownerDocument.createElement("div");
		//groupRow.className = "logGroup";
		var groupRowBox = consoleBody.ownerDocument.createElement("div");
		groupRowBox.className = "logGroupBox";
		//groupRow.appendChild(groupRowBox);
		appendRow(groupRowBox);
		groupStack.push(groupRowBox);
	}

	function popGroup(){
		groupStack.pop();
	}

	// ***************************************************************************

	function logFormatted(objects, className){
		var html = [];

		var format = objects[0];
		var objIndex = 0;

		if(typeof(format) != "string"){
			format = "";
			objIndex = -1;
		}

		var parts = parseFormat(format);

		for(var i = 0; i < parts.length; ++i){
			var part = parts[i];
			if(part && typeof part == "object"){
				part.appender(objects[++objIndex], html);
			}else{
				appendText(part, html);
			}
		}


		var ids = [];
		var obs = [];
		for(i = objIndex+1; i < objects.length; ++i){
			appendText(" ", html);

			var object = objects[i];
			if(object === undefined || object === null ){
				appendNull(object, html);

			}else if(typeof(object) == "string"){
				appendText(object, html);

			}else if(object instanceof Date){
				appendText(object.toString(), html);

			}else if(object.nodeType == 9){
				appendText("[ XmlDoc ]", html);

			}else{
				// Create link for object inspector
				// need to create an ID for this link, since it is currently text
				var id = "_a" + __consoleAnchorId__++;
				ids.push(id);
				// need to save the object, so the arrays line up
				obs.push(object);
				var str = '<a id="'+id+'" href="javascript:void(0);">'+getObjectAbbr(object)+'</a>';

				appendLink( str , html);
			}
		}

		logRow(html, className);

		// Now that the row is inserted in the DOM, loop through all of the links that were just created
		for(i=0; i<ids.length; i++){
			var btn = _firebugDoc.getElementById(ids[i]);
			if(!btn){ continue; }

			// store the object in the dom btn for reference later
			// avoid parsing these objects unless necessary
			btn.obj = obs[i];

			_firebugWin.console._connects.push(dojo.connect(btn, "onclick", function(){

				console.openObjectInspector();

				try{
					printObject(this.obj);
				}catch(e){
					this.obj = e;
				}
				consoleObjectInspector.innerHTML = "<pre>" + printObject( this.obj ) + "</pre>";
			}));
		}
	}

	function parseFormat(format){
		var parts = [];

		var reg = /((^%|[^\\]%)(\d+)?(\.)([a-zA-Z]))|((^%|[^\\]%)([a-zA-Z]))/;
		var appenderMap = {s: appendText, d: appendInteger, i: appendInteger, f: appendFloat};

		for(var m = reg.exec(format); m; m = reg.exec(format)){
			var type = m[8] ? m[8] : m[5];
			var appender = type in appenderMap ? appenderMap[type] : appendObject;
			var precision = m[3] ? parseInt(m[3]) : (m[4] == "." ? -1 : 0);

			parts.push(format.substr(0, m[0][0] == "%" ? m.index : m.index+1));
			parts.push({appender: appender, precision: precision});

			format = format.substr(m.index+m[0].length);
		}

		parts.push(format);

		return parts;
	}

	function escapeHTML(value){
		function replaceChars(ch){
			switch(ch){
				case "<":
					return "&lt;";
				case ">":
					return "&gt;";
				case "&":
					return "&amp;";
				case "'":
					return "&#39;";
				case '"':
					return "&quot;";
			}
			return "?";
		}
		return String(value).replace(/[<>&"']/g, replaceChars);
	}

	function objectToString(object){
		try{
			return object+"";
		}catch(e){
			return null;
		}
	}

	// ***************************************************************************
	function appendLink(object, html){
		// needed for object links - no HTML escaping
		html.push( objectToString(object) );
	}

	function appendText(object, html){
		html.push(escapeHTML(objectToString(object)));
	}

	function appendNull(object, html){
		html.push('<span class="objectBox-null">', escapeHTML(objectToString(object)), '</span>');
	}

	function appendString(object, html){
		html.push('<span class="objectBox-string">&quot;', escapeHTML(objectToString(object)),
			'&quot;</span>');
	}

	function appendInteger(object, html){
		html.push('<span class="objectBox-number">', escapeHTML(objectToString(object)), '</span>');
	}

	function appendFloat(object, html){
		html.push('<span class="objectBox-number">', escapeHTML(objectToString(object)), '</span>');
	}

	function appendFunction(object, html){
		html.push('<span class="objectBox-function">', getObjectAbbr(object), '</span>');
	}

	function appendObject(object, html){
		try{
			if(object === undefined){
				appendNull("undefined", html);
			}else if(object === null){
				appendNull("null", html);
			}else if(typeof object == "string"){
				appendString(object, html);
			}else if(typeof object == "number"){
				appendInteger(object, html);
			}else if(typeof object == "function"){
				appendFunction(object, html);
			}else if(object.nodeType == 1){
				appendSelector(object, html);
			}else if(typeof object == "object"){
				appendObjectFormatted(object, html);
			}else{
				appendText(object, html);
			}
		}catch(e){
			/* squelch */
		}
	}

	function appendObjectFormatted(object, html){
		var text = objectToString(object);
		var reObject = /\[object (.*?)\]/;

		var m = reObject.exec(text);
		html.push('<span class="objectBox-object">', m ? m[1] : text, '</span>');
	}

	function appendSelector(object, html){
		html.push('<span class="objectBox-selector">');

		html.push('<span class="selectorTag">', escapeHTML(object.nodeName.toLowerCase()), '</span>');
		if(object.id){
			html.push('<span class="selectorId">#', escapeHTML(object.id), '</span>');
		}
		if(object.className){
			html.push('<span class="selectorClass">.', escapeHTML(object.className), '</span>');
		}

		html.push('</span>');
	}

	function appendNode(node, html){
		if(node.nodeType == 1){
			html.push(
				'<div class="objectBox-element">',
					'&lt;<span class="nodeTag">', node.nodeName.toLowerCase(), '</span>');

			for(var i = 0; i < node.attributes.length; ++i){
				var attr = node.attributes[i];
				if(!attr.specified){ continue; }

				html.push('&nbsp;<span class="nodeName">', attr.nodeName.toLowerCase(),
					'</span>=&quot;<span class="nodeValue">', escapeHTML(attr.nodeValue),
					'</span>&quot;');
			}

			if(node.firstChild){
				html.push('&gt;</div><div class="nodeChildren">');

				for(var child = node.firstChild; child; child = child.nextSibling){
					appendNode(child, html);
				}

				html.push('</div><div class="objectBox-element">&lt;/<span class="nodeTag">',
					node.nodeName.toLowerCase(), '&gt;</span></div>');
			}else{
				html.push('/&gt;</div>');
			}
		}else if (node.nodeType == 3){
			html.push('<div class="nodeText">', escapeHTML(node.nodeValue),
				'</div>');
		}
	}

	// ***************************************************************************

	function addEvent(object, name, handler){
		if(document.all){
			object.attachEvent("on"+name, handler);
		}else{
			object.addEventListener(name, handler, false);
		}
	}

	function removeEvent(object, name, handler){
		if(document.all){
			object.detachEvent("on"+name, handler);
		}else{
			object.removeEventListener(name, handler, false);
		}
	}

	function cancelEvent(event){
		if(document.all){
			event.cancelBubble = true;
		}else{
			event.stopPropagation();
		}
	}

	function onError(msg, href, lineNo){
		var lastSlash = href.lastIndexOf("/");
		var fileName = lastSlash == -1 ? href : href.substr(lastSlash+1);

		var html = [
			'<span class="errorMessage">', msg, '</span>',
			'<div class="objectBox-sourceLink">', fileName, ' (line ', lineNo, ')</div>'
		];

		logRow(html, "error");
	}


	//After converting to div instead of iframe, now getting two keydowns right away in IE 6.
	//Make sure there is a little bit of delay.
	var onKeyDownTime = new Date().getTime();

	function onKeyDown(event){
		var timestamp = (new Date()).getTime();
		if(timestamp > onKeyDownTime + 200){
			event = dojo.fixEvent(event);
			var keys = dojo.keys;
			var ekc = event.keyCode;
			onKeyDownTime = timestamp;
			if(ekc == keys.F12){
				toggleConsole();
			}else if(
				(ekc == keys.NUMPAD_ENTER || ekc == 76) &&
				event.shiftKey &&
				(event.metaKey || event.ctrlKey)
			){
				focusCommandLine();
			}else{
				return;
			}
			cancelEvent(event);
		}
	}

	function onCommandLineKeyDown(e){
		var dk = dojo.keys;
		if(e.keyCode == 13 && commandLine.value){
			addToHistory(commandLine.value);
			evalCommandLine();
		}else if(e.keyCode == 27){
			commandLine.value = "";
		}else if(e.keyCode == dk.UP_ARROW || e.charCode == dk.UP_ARROW){
			navigateHistory("older");
		}else if(e.keyCode == dk.DOWN_ARROW || e.charCode == dk.DOWN_ARROW){
			navigateHistory("newer");
		}else if(e.keyCode == dk.HOME || e.charCode == dk.HOME){
			historyPosition = 1;
			navigateHistory("older");
		}else if(e.keyCode == dk.END || e.charCode == dk.END){
			historyPosition = 999999;
			navigateHistory("newer");
		}
	}

	var historyPosition = -1;
	var historyCommandLine = null;

	function addToHistory(value){
		var history = cookie("firebug_history");
		history = (history) ? dojo.fromJson(history) : [];
		var pos = dojo.indexOf(history, value);
		if (pos != -1){
			history.splice(pos, 1);
		}
		history.push(value);
		cookie("firebug_history", dojo.toJson(history), 30);
		while(history.length && !cookie("firebug_history")){
			history.shift();
			cookie("firebug_history", dojo.toJson(history), 30);
		}
		historyCommandLine = null;
		historyPosition = -1;
	}

	function navigateHistory(direction){
		var history = cookie("firebug_history");
		history = (history) ? dojo.fromJson(history) : [];
		if(!history.length){
			return;
		}

		if(historyCommandLine === null){
			historyCommandLine = commandLine.value;
		}

		if(historyPosition == -1){
			historyPosition = history.length;
		}

		if(direction == "older"){
			--historyPosition;
			if(historyPosition < 0){
				historyPosition = 0;
			}
		}else if(direction == "newer"){
			++historyPosition;
			if(historyPosition > history.length){
				historyPosition = history.length;
			}
		}

		if(historyPosition == history.length){
			commandLine.value = historyCommandLine;
			historyCommandLine = null;
		}else{
			commandLine.value = history[historyPosition];
		}
	}

	function cookie(name, value){
		var c = document.cookie;
		if(arguments.length == 1){
			var matches = c.match(new RegExp("(?:^|; )" + name + "=([^;]*)"));
			return matches ? decodeURIComponent(matches[1]) : undefined; // String or undefined
		}else{
			var d = new Date();
			d.setMonth(d.getMonth()+1);
			document.cookie = name + "=" + encodeURIComponent(value) + ((d.toUtcString) ? "; expires=" + d.toUTCString() : "");
		}
	}

	function isArray(it){
		return it && it instanceof Array || typeof it == "array";
	}

	//***************************************************************************************************
	// Print Object Helpers
	function objectLength(o){
		var cnt = 0;
		for(var nm in o){
			cnt++;
		}
		return cnt;
	}

	function printObject(o, i, txt, used){
		// Recursively trace object, indenting to represent depth for display in object inspector
		var ind = " \t";
		txt = txt || "";
		i = i || ind;
		used = used || [];
		var opnCls;

		if(o && o.nodeType == 1){
			var html = [];
			appendNode(o, html);
			return html.join("");
		}

		var br=",\n", cnt = 0, length = objectLength(o);

		if(o instanceof Date){
			return i + o.toString() + br;
		}
		looking:
		for(var nm in o){
			cnt++;
			if(cnt==length){br = "\n";}
			if(o[nm] === window || o[nm] === document){
				// do nothing
			}else if(o[nm] === null){
				txt += i+nm + " : NULL" + br;
			}else if(o[nm] && o[nm].nodeType){
				if(o[nm].nodeType == 1){
					//txt += i+nm + " : < "+o[nm].tagName+" id=\""+ o[nm].id+"\" />" + br;
				}else if(o[nm].nodeType == 3){
					txt += i+nm + " : [ TextNode "+o[nm].data + " ]" + br;
				}

			}else if(typeof o[nm] == "object" && (o[nm] instanceof String || o[nm] instanceof Number || o[nm] instanceof Boolean)){
				txt += i+nm + " : " + o[nm] + "," + br;

			}else if(o[nm] instanceof Date){
				txt += i+nm + " : " + o[nm].toString() + br;

			}else if(typeof(o[nm]) == "object" && o[nm]){
				for(var j = 0, seen; seen = used[j]; j++){
					if(o[nm] === seen){
						txt += i+nm + " : RECURSION" + br;
						continue looking;
					}
				}
				used.push(o[nm]);

				opnCls = (isArray(o[nm]))?["[","]"]:["{","}"];
				txt += i+nm +" : " + opnCls[0] + "\n";//non-standard break, (no comma)
				txt += printObject(o[nm], i+ind, "", used);
				txt += i + opnCls[1] + br;

			}else if(typeof o[nm] == "undefined"){
				txt += i+nm + " : undefined" + br;
			}else if(nm == "toString" && typeof o[nm] == "function"){
				var toString = o[nm]();
				if(typeof toString == "string" && toString.match(/function ?(.*?)\(/)){
					toString = escapeHTML(getObjectAbbr(o[nm]));
				}
				txt += i+nm +" : " + toString + br;
			}else{
				txt += i+nm +" : "+ escapeHTML(getObjectAbbr(o[nm])) + br;
			}
		}
		return txt;
	}

	function getObjectAbbr(obj){
		// Gets an abbreviation of an object for display in log
		// X items in object, including id
		// X items in an array
		// TODO: Firebug Sr. actually goes by char count
		var isError = (obj instanceof Error);
		if(obj.nodeType == 1){
			return escapeHTML('< '+obj.tagName.toLowerCase()+' id=\"'+ obj.id+ '\" />');
		}
		if(obj.nodeType == 3){
			return escapeHTML('[TextNode: "'+obj.nodeValue+'"]');
		}
		var nm = (obj && (obj.id || obj.name || obj.ObjectID || obj.widgetId));
		if(!isError && nm){ return "{"+nm+"}";	}

		var obCnt = 2;
		var arCnt = 4;
		var cnt = 0;

		if(isError){
			nm = "[ Error: "+(obj.message || obj.description || obj)+" ]";
		}else if(isArray(obj)){
			nm = "[" + obj.slice(0,arCnt).join(",");
			if(obj.length > arCnt){
				nm += " ... ("+obj.length+" items)";
			}
			nm += "]";
		}else if(typeof obj == "function"){
			nm = obj + "";
			var reg = /function\s*([^\(]*)(\([^\)]*\))[^\{]*\{/;
			var m = reg.exec(nm);
			if(m){
				if(!m[1]){
					m[1] = "function";
				}
				nm = m[1] + m[2];
			}else{
				nm = "function()";
			}
		}else if(typeof obj != "object" || typeof obj == "string"){
			nm = obj + "";
		}else{
			nm = "{";
			for(var i in obj){
				cnt++;
				if(cnt > obCnt){ break; }
				nm += i+":"+escapeHTML(obj[i])+"  ";
			}
			nm+="}";
		}

		return nm;
	}

	//*************************************************************************************

	//window.onerror = onError;

	addEvent(document, has("ie") || has("safari") ? "keydown" : "keypress", onKeyDown);

	if(	(document.documentElement.getAttribute("debug") == "true")||
		(dojo.config.isDebug)
	){
		toggleConsole(true);
	}

	dojo.addOnWindowUnload(function(){
		// Erase the globals and event handlers I created, to prevent spurious leak warnings
		removeEvent(document, has("ie") || has("safari") ? "keydown" : "keypress", onKeyDown);
		window.onFirebugResize = null;
		window.console = null;
	});

});

},
'dojo/selector/acme':function(){
define([
	"../dom", "../sniff", "../_base/array", "../_base/lang", "../_base/window"
], function(dom, has, array, lang, win){

	// module:
	//		dojo/selector/acme

/*
	acme architectural overview:

		acme is a relatively full-featured CSS3 query library. It is
		designed to take any valid CSS3 selector and return the nodes matching
		the selector. To do this quickly, it processes queries in several
		steps, applying caching where profitable.

		The steps (roughly in reverse order of the way they appear in the code):
			1.) check to see if we already have a "query dispatcher"
				- if so, use that with the given parameterization. Skip to step 4.
			2.) attempt to determine which branch to dispatch the query to:
				- JS (optimized DOM iteration)
				- native (FF3.1+, Safari 3.1+, IE 8+)
			3.) tokenize and convert to executable "query dispatcher"
				- this is where the lion's share of the complexity in the
					system lies. In the DOM version, the query dispatcher is
					assembled as a chain of "yes/no" test functions pertaining to
					a section of a simple query statement (".blah:nth-child(odd)"
					but not "div div", which is 2 simple statements). Individual
					statement dispatchers are cached (to prevent re-definition)
					as are entire dispatch chains (to make re-execution of the
					same query fast)
			4.) the resulting query dispatcher is called in the passed scope
					(by default the top-level document)
				- for DOM queries, this results in a recursive, top-down
					evaluation of nodes based on each simple query section
				- for native implementations, this may mean working around spec
					bugs. So be it.
			5.) matched nodes are pruned to ensure they are unique (if necessary)
*/


	////////////////////////////////////////////////////////////////////////
	// Toolkit aliases
	////////////////////////////////////////////////////////////////////////

	// if you are extracting acme for use in your own system, you will
	// need to provide these methods and properties. No other porting should be
	// necessary, save for configuring the system to use a class other than
	// dojo/NodeList as the return instance instantiator
	var trim = 			lang.trim;
	var each = 			array.forEach;

	var getDoc = function(){ return win.doc; };
	// NOTE(alex): the spec is idiotic. CSS queries should ALWAYS be case-sensitive, but nooooooo
	var cssCaseBug = (getDoc().compatMode) == "BackCompat";

	////////////////////////////////////////////////////////////////////////
	// Global utilities
	////////////////////////////////////////////////////////////////////////


	var specials = ">~+";

	// global thunk to determine whether we should treat the current query as
	// case sensitive or not. This switch is flipped by the query evaluator
	// based on the document passed as the context to search.
	var caseSensitive = false;

	// how high?
	var yesman = function(){ return true; };

	////////////////////////////////////////////////////////////////////////
	// Tokenizer
	////////////////////////////////////////////////////////////////////////

	var getQueryParts = function(query){
		// summary:
		//		state machine for query tokenization
		// description:
		//		instead of using a brittle and slow regex-based CSS parser,
		//		acme implements an AST-style query representation. This
		//		representation is only generated once per query. For example,
		//		the same query run multiple times or under different root nodes
		//		does not re-parse the selector expression but instead uses the
		//		cached data structure. The state machine implemented here
		//		terminates on the last " " (space) character and returns an
		//		ordered array of query component structures (or "parts"). Each
		//		part represents an operator or a simple CSS filtering
		//		expression. The structure for parts is documented in the code
		//		below.


		// NOTE:
		//		this code is designed to run fast and compress well. Sacrifices
		//		to readability and maintainability have been made.  Your best
		//		bet when hacking the tokenizer is to put The Donnas on *really*
		//		loud (may we recommend their "Spend The Night" release?) and
		//		just assume you're gonna make mistakes. Keep the unit tests
		//		open and run them frequently. Knowing is half the battle ;-)
		if(specials.indexOf(query.slice(-1)) >= 0){
			// if we end with a ">", "+", or "~", that means we're implicitly
			// searching all children, so make it explicit
			query += " * ";
		}else{
			// if you have not provided a terminator, one will be provided for
			// you...
			query += " ";
		}

		var ts = function(/*Integer*/ s, /*Integer*/ e){
			// trim and slice.

			// take an index to start a string slice from and an end position
			// and return a trimmed copy of that sub-string
			return trim(query.slice(s, e));
		};

		// the overall data graph of the full query, as represented by queryPart objects
		var queryParts = [];


		// state keeping vars
		var inBrackets = -1, inParens = -1, inMatchFor = -1,
			inPseudo = -1, inClass = -1, inId = -1, inTag = -1, currentQuoteChar,
			lc = "", cc = "", pStart;

		// iteration vars
		var x = 0, // index in the query
			ql = query.length,
			currentPart = null, // data structure representing the entire clause
			_cp = null; // the current pseudo or attr matcher

		// several temporary variables are assigned to this structure during a
		// potential sub-expression match:
		//		attr:
		//			a string representing the current full attribute match in a
		//			bracket expression
		//		type:
		//			if there's an operator in a bracket expression, this is
		//			used to keep track of it
		//		value:
		//			the internals of parenthetical expression for a pseudo. for
		//			:nth-child(2n+1), value might be "2n+1"

		var endTag = function(){
			// called when the tokenizer hits the end of a particular tag name.
			// Re-sets state variables for tag matching and sets up the matcher
			// to handle the next type of token (tag or operator).
			if(inTag >= 0){
				var tv = (inTag == x) ? null : ts(inTag, x); // .toLowerCase();
				currentPart[ (specials.indexOf(tv) < 0) ? "tag" : "oper" ] = tv;
				inTag = -1;
			}
		};

		var endId = function(){
			// called when the tokenizer might be at the end of an ID portion of a match
			if(inId >= 0){
				currentPart.id = ts(inId, x).replace(/\\/g, "");
				inId = -1;
			}
		};

		var endClass = function(){
			// called when the tokenizer might be at the end of a class name
			// match. CSS allows for multiple classes, so we augment the
			// current item with another class in its list
			if(inClass >= 0){
				currentPart.classes.push(ts(inClass + 1, x).replace(/\\/g, ""));
				inClass = -1;
			}
		};

		var endAll = function(){
			// at the end of a simple fragment, so wall off the matches
			endId();
			endTag();
			endClass();
		};

		var endPart = function(){
			endAll();
			if(inPseudo >= 0){
				currentPart.pseudos.push({ name: ts(inPseudo + 1, x) });
			}
			// hint to the selector engine to tell it whether or not it
			// needs to do any iteration. Many simple selectors don't, and
			// we can avoid significant construction-time work by advising
			// the system to skip them
			currentPart.loops = (
					currentPart.pseudos.length ||
					currentPart.attrs.length ||
					currentPart.classes.length	);

			currentPart.oquery = currentPart.query = ts(pStart, x); // save the full expression as a string


			// otag/tag are hints to suggest to the system whether or not
			// it's an operator or a tag. We save a copy of otag since the
			// tag name is cast to upper-case in regular HTML matches. The
			// system has a global switch to figure out if the current
			// expression needs to be case sensitive or not and it will use
			// otag or tag accordingly
			currentPart.otag = currentPart.tag = (currentPart["oper"]) ? null : (currentPart.tag || "*");

			if(currentPart.tag){
				// if we're in a case-insensitive HTML doc, we likely want
				// the toUpperCase when matching on element.tagName. If we
				// do it here, we can skip the string op per node
				// comparison
				currentPart.tag = currentPart.tag.toUpperCase();
			}

			// add the part to the list
			if(queryParts.length && (queryParts[queryParts.length-1].oper)){
				// operators are always infix, so we remove them from the
				// list and attach them to the next match. The evaluator is
				// responsible for sorting out how to handle them.
				currentPart.infixOper = queryParts.pop();
				currentPart.query = currentPart.infixOper.query + " " + currentPart.query;
				/*
				console.debug(	"swapping out the infix",
								currentPart.infixOper,
								"and attaching it to",
								currentPart);
				*/
			}
			queryParts.push(currentPart);

			currentPart = null;
		};

		// iterate over the query, character by character, building up a
		// list of query part objects
		for(; lc=cc, cc=query.charAt(x), x < ql; x++){
			//		cc: the current character in the match
			//		lc: the last character (if any)

			// someone is trying to escape something, so don't try to match any
			// fragments. We assume we're inside a literal.
			if(lc == "\\"){ continue; }
			if(!currentPart){ // a part was just ended or none has yet been created
				// NOTE: I hate all this alloc, but it's shorter than writing tons of if's
				pStart = x;
				//	rules describe full CSS sub-expressions, like:
				//		#someId
				//		.className:first-child
				//	but not:
				//		thinger > div.howdy[type=thinger]
				//	the indidual components of the previous query would be
				//	split into 3 parts that would be represented a structure like:
				//		[
				//			{
				//				query: "thinger",
				//				tag: "thinger",
				//			},
				//			{
				//				query: "div.howdy[type=thinger]",
				//				classes: ["howdy"],
				//				infixOper: {
				//					query: ">",
				//					oper: ">",
				//				}
				//			},
				//		]
				currentPart = {
					query: null, // the full text of the part's rule
					pseudos: [], // CSS supports multiple pseud-class matches in a single rule
					attrs: [],	// CSS supports multi-attribute match, so we need an array
					classes: [], // class matches may be additive, e.g.: .thinger.blah.howdy
					tag: null,	// only one tag...
					oper: null, // ...or operator per component. Note that these wind up being exclusive.
					id: null,	// the id component of a rule
					getTag: function(){
						return caseSensitive ? this.otag : this.tag;
					}
				};

				// if we don't have a part, we assume we're going to start at
				// the beginning of a match, which should be a tag name. This
				// might fault a little later on, but we detect that and this
				// iteration will still be fine.
				inTag = x;
			}

			// Skip processing all quoted characters.
			// If we are inside quoted text then currentQuoteChar stores the character that began the quote,
			// thus that character that will end it.
			if(currentQuoteChar){
				if(cc == currentQuoteChar){
					currentQuoteChar = null;
				}
				continue;
			}else if (cc == "'" || cc == '"'){
				currentQuoteChar = cc;
				continue;
			}

			if(inBrackets >= 0){
				// look for a the close first
				if(cc == "]"){ // if we're in a [...] clause and we end, do assignment
					if(!_cp.attr){
						// no attribute match was previously begun, so we
						// assume this is an attribute existence match in the
						// form of [someAttributeName]
						_cp.attr = ts(inBrackets+1, x);
					}else{
						// we had an attribute already, so we know that we're
						// matching some sort of value, as in [attrName=howdy]
						_cp.matchFor = ts((inMatchFor||inBrackets+1), x);
					}
					var cmf = _cp.matchFor;
					if(cmf){
						// try to strip quotes from the matchFor value. We want
						// [attrName=howdy] to match the same
						//	as [attrName = 'howdy' ]
						if(	(cmf.charAt(0) == '"') || (cmf.charAt(0) == "'") ){
							_cp.matchFor = cmf.slice(1, -1);
						}
					}
					// remove backslash escapes from an attribute match, since DOM
					// querying will get attribute values without backslashes
					if(_cp.matchFor){
						_cp.matchFor = _cp.matchFor.replace(/\\/g, "");
					}

					// end the attribute by adding it to the list of attributes.
					currentPart.attrs.push(_cp);
					_cp = null; // necessary?
					inBrackets = inMatchFor = -1;
				}else if(cc == "="){
					// if the last char was an operator prefix, make sure we
					// record it along with the "=" operator.
					var addToCc = ("|~^$*".indexOf(lc) >=0 ) ? lc : "";
					_cp.type = addToCc+cc;
					_cp.attr = ts(inBrackets+1, x-addToCc.length);
					inMatchFor = x+1;
				}
				// now look for other clause parts
			}else if(inParens >= 0){
				// if we're in a parenthetical expression, we need to figure
				// out if it's attached to a pseudo-selector rule like
				// :nth-child(1)
				if(cc == ")"){
					if(inPseudo >= 0){
						_cp.value = ts(inParens+1, x);
					}
					inPseudo = inParens = -1;
				}
			}else if(cc == "#"){
				// start of an ID match
				endAll();
				inId = x+1;
			}else if(cc == "."){
				// start of a class match
				endAll();
				inClass = x;
			}else if(cc == ":"){
				// start of a pseudo-selector match
				endAll();
				inPseudo = x;
			}else if(cc == "["){
				// start of an attribute match.
				endAll();
				inBrackets = x;
				// provide a new structure for the attribute match to fill-in
				_cp = {
					/*=====
					attr: null, type: null, matchFor: null
					=====*/
				};
			}else if(cc == "("){
				// we really only care if we've entered a parenthetical
				// expression if we're already inside a pseudo-selector match
				if(inPseudo >= 0){
					// provide a new structure for the pseudo match to fill-in
					_cp = {
						name: ts(inPseudo+1, x),
						value: null
					};
					currentPart.pseudos.push(_cp);
				}
				inParens = x;
			}else if(
				(cc == " ") &&
				// if it's a space char and the last char is too, consume the
				// current one without doing more work
				(lc != cc)
			){
				endPart();
			}
		}
		return queryParts;
	};


	////////////////////////////////////////////////////////////////////////
	// DOM query infrastructure
	////////////////////////////////////////////////////////////////////////

	var agree = function(first, second){
		// the basic building block of the yes/no chaining system. agree(f1,
		// f2) generates a new function which returns the boolean results of
		// both of the passed functions to a single logical-anded result. If
		// either are not passed, the other is used exclusively.
		if(!first){ return second; }
		if(!second){ return first; }

		return function(){
			return first.apply(window, arguments) && second.apply(window, arguments);
		};
	};

	var getArr = function(i, arr){
		// helps us avoid array alloc when we don't need it
		var r = arr||[]; // FIXME: should this be 'new d._NodeListCtor()' ?
		if(i){ r.push(i); }
		return r;
	};

	var _isElement = function(n){ return (1 == n.nodeType); };

	// FIXME: need to coalesce _getAttr with defaultGetter
	var blank = "";
	var _getAttr = function(elem, attr){
		if(!elem){ return blank; }
		if(attr == "class"){
			return elem.className || blank;
		}
		if(attr == "for"){
			return elem.htmlFor || blank;
		}
		if(attr == "style"){
			return elem.style.cssText || blank;
		}
		return (caseSensitive ? elem.getAttribute(attr) : elem.getAttribute(attr, 2)) || blank;
	};

	var attrs = {
		"*=": function(attr, value){
			return function(elem){
				// E[foo*="bar"]
				//		an E element whose "foo" attribute value contains
				//		the substring "bar"
				return (_getAttr(elem, attr).indexOf(value)>=0);
			};
		},
		"^=": function(attr, value){
			// E[foo^="bar"]
			//		an E element whose "foo" attribute value begins exactly
			//		with the string "bar"
			return function(elem){
				return (_getAttr(elem, attr).indexOf(value)==0);
			};
		},
		"$=": function(attr, value){
			// E[foo$="bar"]
			//		an E element whose "foo" attribute value ends exactly
			//		with the string "bar"
			return function(elem){
				var ea = " "+_getAttr(elem, attr);
				var lastIndex = ea.lastIndexOf(value);
				return lastIndex > -1 && (lastIndex==(ea.length-value.length));
			};
		},
		"~=": function(attr, value){
			// E[foo~="bar"]
			//		an E element whose "foo" attribute value is a list of
			//		space-separated values, one of which is exactly equal
			//		to "bar"

			// return "[contains(concat(' ',@"+attr+",' '), ' "+ value +" ')]";
			var tval = " "+value+" ";
			return function(elem){
				var ea = " "+_getAttr(elem, attr)+" ";
				return (ea.indexOf(tval)>=0);
			};
		},
		"|=": function(attr, value){
			// E[hreflang|="en"]
			//		an E element whose "hreflang" attribute has a
			//		hyphen-separated list of values beginning (from the
			//		left) with "en"
			var valueDash = value+"-";
			return function(elem){
				var ea = _getAttr(elem, attr);
				return (
					(ea == value) ||
					(ea.indexOf(valueDash)==0)
				);
			};
		},
		"=": function(attr, value){
			return function(elem){
				return (_getAttr(elem, attr) == value);
			};
		}
	};

	// avoid testing for node type if we can. Defining this in the negative
	// here to avoid negation in the fast path.
	var _noNES = (typeof getDoc().firstChild.nextElementSibling == "undefined");
	var _ns = !_noNES ? "nextElementSibling" : "nextSibling";
	var _ps = !_noNES ? "previousElementSibling" : "previousSibling";
	var _simpleNodeTest = (_noNES ? _isElement : yesman);

	var _lookLeft = function(node){
		// look left
		while(node = node[_ps]){
			if(_simpleNodeTest(node)){ return false; }
		}
		return true;
	};

	var _lookRight = function(node){
		// look right
		while(node = node[_ns]){
			if(_simpleNodeTest(node)){ return false; }
		}
		return true;
	};

	var getNodeIndex = function(node){
		var root = node.parentNode;
		root = root.nodeType != 7 ? root : root.nextSibling; // PROCESSING_INSTRUCTION_NODE
		var i = 0,
			tret = root.children || root.childNodes,
			ci = (node["_i"]||node.getAttribute("_i")||-1),
			cl = (root["_l"]|| (typeof root.getAttribute !== "undefined" ? root.getAttribute("_l") : -1));

		if(!tret){ return -1; }
		var l = tret.length;

		// we calculate the parent length as a cheap way to invalidate the
		// cache. It's not 100% accurate, but it's much more honest than what
		// other libraries do
		if( cl == l && ci >= 0 && cl >= 0 ){
			// if it's legit, tag and release
			return ci;
		}

		// else re-key things
		if(has("ie") && typeof root.setAttribute !== "undefined"){
			root.setAttribute("_l", l);
		}else{
			root["_l"] = l;
		}
		ci = -1;
		for(var te = root["firstElementChild"]||root["firstChild"]; te; te = te[_ns]){
			if(_simpleNodeTest(te)){
				if(has("ie")){
					te.setAttribute("_i", ++i);
				}else{
					te["_i"] = ++i;
				}
				if(node === te){
					// NOTE:
					//	shortcutting the return at this step in indexing works
					//	very well for benchmarking but we avoid it here since
					//	it leads to potential O(n^2) behavior in sequential
					//	getNodexIndex operations on a previously un-indexed
					//	parent. We may revisit this at a later time, but for
					//	now we just want to get the right answer more often
					//	than not.
					ci = i;
				}
			}
		}
		return ci;
	};

	var isEven = function(elem){
		return !((getNodeIndex(elem)) % 2);
	};

	var isOdd = function(elem){
		return ((getNodeIndex(elem)) % 2);
	};

	var pseudos = {
		"checked": function(name, condition){
			return function(elem){
				return !!("checked" in elem ? elem.checked : elem.selected);
			};
		},
		"disabled": function(name, condition){
			return function(elem){
				return elem.disabled;
			};
		},
		"enabled": function(name, condition){
			return function(elem){
				return !elem.disabled;
			};
		},
		"first-child": function(){ return _lookLeft; },
		"last-child": function(){ return _lookRight; },
		"only-child": function(name, condition){
			return function(node){
				return _lookLeft(node) && _lookRight(node);
			};
		},
		"empty": function(name, condition){
			return function(elem){
				// DomQuery and jQuery get this wrong, oddly enough.
				// The CSS 3 selectors spec is pretty explicit about it, too.
				var cn = elem.childNodes;
				var cnl = elem.childNodes.length;
				// if(!cnl){ return true; }
				for(var x=cnl-1; x >= 0; x--){
					var nt = cn[x].nodeType;
					if((nt === 1)||(nt == 3)){ return false; }
				}
				return true;
			};
		},
		"contains": function(name, condition){
			var cz = condition.charAt(0);
			if( cz == '"' || cz == "'" ){ //remove quote
				condition = condition.slice(1, -1);
			}
			return function(elem){
				return (elem.innerHTML.indexOf(condition) >= 0);
			};
		},
		"not": function(name, condition){
			var p = getQueryParts(condition)[0];
			var ignores = { el: 1 };
			if(p.tag != "*"){
				ignores.tag = 1;
			}
			if(!p.classes.length){
				ignores.classes = 1;
			}
			var ntf = getSimpleFilterFunc(p, ignores);
			return function(elem){
				return (!ntf(elem));
			};
		},
		"nth-child": function(name, condition){
			var pi = parseInt;
			// avoid re-defining function objects if we can
			if(condition == "odd"){
				return isOdd;
			}else if(condition == "even"){
				return isEven;
			}
			// FIXME: can we shorten this?
			if(condition.indexOf("n") != -1){
				var tparts = condition.split("n", 2);
				var pred = tparts[0] ? ((tparts[0] == '-') ? -1 : pi(tparts[0])) : 1;
				var idx = tparts[1] ? pi(tparts[1]) : 0;
				var lb = 0, ub = -1;
				if(pred > 0){
					if(idx < 0){
						idx = (idx % pred) && (pred + (idx % pred));
					}else if(idx>0){
						if(idx >= pred){
							lb = idx - idx % pred;
						}
						idx = idx % pred;
					}
				}else if(pred<0){
					pred *= -1;
					// idx has to be greater than 0 when pred is negative;
					// shall we throw an error here?
					if(idx > 0){
						ub = idx;
						idx = idx % pred;
					}
				}
				if(pred > 0){
					return function(elem){
						var i = getNodeIndex(elem);
						return (i>=lb) && (ub<0 || i<=ub) && ((i % pred) == idx);
					};
				}else{
					condition = idx;
				}
			}
			var ncount = pi(condition);
			return function(elem){
				return (getNodeIndex(elem) == ncount);
			};
		}
	};

	var defaultGetter = (has("ie") < 9 || has("ie") == 9 && has("quirks")) ? function(cond){
		var clc = cond.toLowerCase();
		if(clc == "class"){ cond = "className"; }
		return function(elem){
			return (caseSensitive ? elem.getAttribute(cond) : elem[cond]||elem[clc]);
		};
	} : function(cond){
		return function(elem){
			return (elem && elem.getAttribute && elem.hasAttribute(cond));
		};
	};

	var getSimpleFilterFunc = function(query, ignores){
		// generates a node tester function based on the passed query part. The
		// query part is one of the structures generated by the query parser
		// when it creates the query AST. The "ignores" object specifies which
		// (if any) tests to skip, allowing the system to avoid duplicating
		// work where it may have already been taken into account by other
		// factors such as how the nodes to test were fetched in the first
		// place
		if(!query){ return yesman; }
		ignores = ignores||{};

		var ff = null;

		if(!("el" in ignores)){
			ff = agree(ff, _isElement);
		}

		if(!("tag" in ignores)){
			if(query.tag != "*"){
				ff = agree(ff, function(elem){
					return (elem && ((caseSensitive ? elem.tagName : elem.tagName.toUpperCase()) == query.getTag()));
				});
			}
		}

		if(!("classes" in ignores)){
			each(query.classes, function(cname, idx, arr){
				// get the class name
				/*
				var isWildcard = cname.charAt(cname.length-1) == "*";
				if(isWildcard){
					cname = cname.substr(0, cname.length-1);
				}
				// I dislike the regex thing, even if memoized in a cache, but it's VERY short
				var re = new RegExp("(?:^|\\s)" + cname + (isWildcard ? ".*" : "") + "(?:\\s|$)");
				*/
				var re = new RegExp("(?:^|\\s)" + cname + "(?:\\s|$)");
				ff = agree(ff, function(elem){
					return re.test(elem.className);
				});
				ff.count = idx;
			});
		}

		if(!("pseudos" in ignores)){
			each(query.pseudos, function(pseudo){
				var pn = pseudo.name;
				if(pseudos[pn]){
					ff = agree(ff, pseudos[pn](pn, pseudo.value));
				}
			});
		}

		if(!("attrs" in ignores)){
			each(query.attrs, function(attr){
				var matcher;
				var a = attr.attr;
				// type, attr, matchFor
				if(attr.type && attrs[attr.type]){
					matcher = attrs[attr.type](a, attr.matchFor);
				}else if(a.length){
					matcher = defaultGetter(a);
				}
				if(matcher){
					ff = agree(ff, matcher);
				}
			});
		}

		if(!("id" in ignores)){
			if(query.id){
				ff = agree(ff, function(elem){
					return (!!elem && (elem.id == query.id));
				});
			}
		}

		if(!ff){
			if(!("default" in ignores)){
				ff = yesman;
			}
		}
		return ff;
	};

	var _nextSibling = function(filterFunc){
		return function(node, ret, bag){
			while(node = node[_ns]){
				if(_noNES && (!_isElement(node))){ continue; }
				if(
					(!bag || _isUnique(node, bag)) &&
					filterFunc(node)
				){
					ret.push(node);
				}
				break;
			}
			return ret;
		};
	};

	var _nextSiblings = function(filterFunc){
		return function(root, ret, bag){
			var te = root[_ns];
			while(te){
				if(_simpleNodeTest(te)){
					if(bag && !_isUnique(te, bag)){
						break;
					}
					if(filterFunc(te)){
						ret.push(te);
					}
				}
				te = te[_ns];
			}
			return ret;
		};
	};

	// get an array of child *elements*, skipping text and comment nodes
	var _childElements = function(filterFunc){
		filterFunc = filterFunc||yesman;
		return function(root, ret, bag){
			// get an array of child elements, skipping text and comment nodes
			var te, x = 0, tret = root.children || root.childNodes;
			while(te = tret[x++]){
				if(
					_simpleNodeTest(te) &&
					(!bag || _isUnique(te, bag)) &&
					(filterFunc(te, x))
				){
					ret.push(te);
				}
			}
			return ret;
		};
	};

	// test to see if node is below root
	var _isDescendant = function(node, root){
		var pn = node.parentNode;
		while(pn){
			if(pn == root){
				break;
			}
			pn = pn.parentNode;
		}
		return !!pn;
	};

	var _getElementsFuncCache = {};

	var getElementsFunc = function(query){
		var retFunc = _getElementsFuncCache[query.query];
		// if we've got a cached dispatcher, just use that
		if(retFunc){ return retFunc; }
		// else, generate a new on

		// NOTE:
		//		this function returns a function that searches for nodes and
		//		filters them.  The search may be specialized by infix operators
		//		(">", "~", or "+") else it will default to searching all
		//		descendants (the " " selector). Once a group of children is
		//		found, a test function is applied to weed out the ones we
		//		don't want. Many common cases can be fast-pathed. We spend a
		//		lot of cycles to create a dispatcher that doesn't do more work
		//		than necessary at any point since, unlike this function, the
		//		dispatchers will be called every time. The logic of generating
		//		efficient dispatchers looks like this in pseudo code:
		//
		//		# if it's a purely descendant query (no ">", "+", or "~" modifiers)
		//		if infixOperator == " ":
		//			if only(id):
		//				return def(root):
		//					return d.byId(id, root);
		//
		//			elif id:
		//				return def(root):
		//					return filter(d.byId(id, root));
		//
		//			elif cssClass && getElementsByClassName:
		//				return def(root):
		//					return filter(root.getElementsByClassName(cssClass));
		//
		//			elif only(tag):
		//				return def(root):
		//					return root.getElementsByTagName(tagName);
		//
		//			else:
		//				# search by tag name, then filter
		//				return def(root):
		//					return filter(root.getElementsByTagName(tagName||"*"));
		//
		//		elif infixOperator == ">":
		//			# search direct children
		//			return def(root):
		//				return filter(root.children);
		//
		//		elif infixOperator == "+":
		//			# search next sibling
		//			return def(root):
		//				return filter(root.nextElementSibling);
		//
		//		elif infixOperator == "~":
		//			# search rightward siblings
		//			return def(root):
		//				return filter(nextSiblings(root));

		var io = query.infixOper;
		var oper = (io ? io.oper : "");
		// the default filter func which tests for all conditions in the query
		// part. This is potentially inefficient, so some optimized paths may
		// re-define it to test fewer things.
		var filterFunc = getSimpleFilterFunc(query, { el: 1 });
		var qt = query.tag;
		var wildcardTag = ("*" == qt);
		var ecs = getDoc()["getElementsByClassName"];

		if(!oper){
			// if there's no infix operator, then it's a descendant query. ID
			// and "elements by class name" variants can be accelerated so we
			// call them out explicitly:
			if(query.id){
				// testing shows that the overhead of yesman() is acceptable
				// and can save us some bytes vs. re-defining the function
				// everywhere.
				filterFunc = (!query.loops && wildcardTag) ?
					yesman :
					getSimpleFilterFunc(query, { el: 1, id: 1 });

				retFunc = function(root, arr){
					var te = dom.byId(query.id, (root.ownerDocument||root));
					if(!te || !filterFunc(te)){ return; }
					if(9 == root.nodeType){ // if root's a doc, we just return directly
						return getArr(te, arr);
					}else{ // otherwise check ancestry
						if(_isDescendant(te, root)){
							return getArr(te, arr);
						}
					}
				};
			}else if(
				ecs &&
				// isAlien check. Workaround for Prototype.js being totally evil/dumb.
				/\{\s*\[native code\]\s*\}/.test(String(ecs)) &&
				query.classes.length &&
				!cssCaseBug
			){
				// it's a class-based query and we've got a fast way to run it.

				// ignore class and ID filters since we will have handled both
				filterFunc = getSimpleFilterFunc(query, { el: 1, classes: 1, id: 1 });
				var classesString = query.classes.join(" ");
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					var tret = root.getElementsByClassName(classesString);
					while((te = tret[x++])){
						if(filterFunc(te, root) && _isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};

			}else if(!wildcardTag && !query.loops){
				// it's tag only. Fast-path it.
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					var tag = query.getTag(),
						tret = tag ? root.getElementsByTagName(tag) : [];
					while((te = tret[x++])){
						if(_isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};
			}else{
				// the common case:
				//		a descendant selector without a fast path. By now it's got
				//		to have a tag selector, even if it's just "*" so we query
				//		by that and filter
				filterFunc = getSimpleFilterFunc(query, { el: 1, tag: 1, id: 1 });
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					// we use getTag() to avoid case sensitivity issues
					var tag = query.getTag(),
						tret = tag ? root.getElementsByTagName(tag) : [];
					while((te = tret[x++])){
						if(filterFunc(te, root) && _isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};
			}
		}else{
			// the query is scoped in some way. Instead of querying by tag we
			// use some other collection to find candidate nodes
			var skipFilters = { el: 1 };
			if(wildcardTag){
				skipFilters.tag = 1;
			}
			filterFunc = getSimpleFilterFunc(query, skipFilters);
			if("+" == oper){
				retFunc = _nextSibling(filterFunc);
			}else if("~" == oper){
				retFunc = _nextSiblings(filterFunc);
			}else if(">" == oper){
				retFunc = _childElements(filterFunc);
			}
		}
		// cache it and return
		return _getElementsFuncCache[query.query] = retFunc;
	};

	var filterDown = function(root, queryParts){
		// NOTE:
		//		this is the guts of the DOM query system. It takes a list of
		//		parsed query parts and a root and finds children which match
		//		the selector represented by the parts
		var candidates = getArr(root), qp, x, te, qpl = queryParts.length, bag, ret;

		for(var i = 0; i < qpl; i++){
			ret = [];
			qp = queryParts[i];
			x = candidates.length - 1;
			if(x > 0){
				// if we have more than one root at this level, provide a new
				// hash to use for checking group membership but tell the
				// system not to post-filter us since we will already have been
				// guaranteed to be unique
				bag = {};
				ret.nozip = true;
			}
			var gef = getElementsFunc(qp);
			for(var j = 0; (te = candidates[j]); j++){
				// for every root, get the elements that match the descendant
				// selector, adding them to the "ret" array and filtering them
				// via membership in this level's bag. If there are more query
				// parts, then this level's return will be used as the next
				// level's candidates
				gef(te, ret, bag);
			}
			if(!ret.length){ break; }
			candidates = ret;
		}
		return ret;
	};

	////////////////////////////////////////////////////////////////////////
	// the query runner
	////////////////////////////////////////////////////////////////////////

	// these are the primary caches for full-query results. The query
	// dispatcher functions are generated then stored here for hash lookup in
	// the future
	var _queryFuncCacheDOM = {},
		_queryFuncCacheQSA = {};

	// this is the second level of splitting, from full-length queries (e.g.,
	// "div.foo .bar") into simple query expressions (e.g., ["div.foo",
	// ".bar"])
	var getStepQueryFunc = function(query){
		var qparts = getQueryParts(trim(query));

		// if it's trivial, avoid iteration and zipping costs
		if(qparts.length == 1){
			// we optimize this case here to prevent dispatch further down the
			// chain, potentially slowing things down. We could more elegantly
			// handle this in filterDown(), but it's slower for simple things
			// that need to be fast (e.g., "#someId").
			var tef = getElementsFunc(qparts[0]);
			return function(root){
				var r = tef(root, []);
				if(r){ r.nozip = true; }
				return r;
			};
		}

		// otherwise, break it up and return a runner that iterates over the parts recursively
		return function(root){
			return filterDown(root, qparts);
		};
	};

	// NOTES:
	//	* we can't trust QSA for anything but document-rooted queries, so
	//	  caching is split into DOM query evaluators and QSA query evaluators
	//	* caching query results is dirty and leak-prone (or, at a minimum,
	//	  prone to unbounded growth). Other toolkits may go this route, but
	//	  they totally destroy their own ability to manage their memory
	//	  footprint. If we implement it, it should only ever be with a fixed
	//	  total element reference # limit and an LRU-style algorithm since JS
	//	  has no weakref support. Caching compiled query evaluators is also
	//	  potentially problematic, but even on large documents the size of the
	//	  query evaluators is often < 100 function objects per evaluator (and
	//	  LRU can be applied if it's ever shown to be an issue).
	//	* since IE's QSA support is currently only for HTML documents and even
	//	  then only in IE 8's "standards mode", we have to detect our dispatch
	//	  route at query time and keep 2 separate caches. Ugg.

	// we need to determine if we think we can run a given query via
	// querySelectorAll or if we'll need to fall back on DOM queries to get
	// there. We need a lot of information about the environment and the query
	// to make the determination (e.g. does it support QSA, does the query in
	// question work in the native QSA impl, etc.).

	// IE QSA queries may incorrectly include comment nodes, so we throw the
	// zipping function into "remove" comments mode instead of the normal "skip
	// it" which every other QSA-clued browser enjoys
	var noZip = has("ie") ? "commentStrip" : "nozip";

	var qsa = "querySelectorAll";
	var qsaAvail = !!getDoc()[qsa];

	//Don't bother with n+3 type of matches, IE complains if we modify those.
	var infixSpaceRe = /\\[>~+]|n\+\d|([^ \\])?([>~+])([^ =])?/g;
	var infixSpaceFunc = function(match, pre, ch, post){
		return ch ? (pre ? pre + " " : "") + ch + (post ? " " + post : "") : /*n+3*/ match;
	};
	
	//Don't apply the infixSpaceRe to attribute value selectors
	var attRe = /([^[]*)([^\]]*])?/g;
	var attFunc = function(match, nonAtt, att){
		return nonAtt.replace(infixSpaceRe, infixSpaceFunc) + (att||"");
	};
	var getQueryFunc = function(query, forceDOM){
		//Normalize query. The CSS3 selectors spec allows for omitting spaces around
		//infix operators, >, ~ and +
		//Do the work here since detection for spaces is used as a simple "not use QSA"
		//test below.
		query = query.replace(attRe, attFunc);

		if(qsaAvail){
			// if we've got a cached variant and we think we can do it, run it!
			var qsaCached = _queryFuncCacheQSA[query];
			if(qsaCached && !forceDOM){ return qsaCached; }
		}

		// else if we've got a DOM cached variant, assume that we already know
		// all we need to and use it
		var domCached = _queryFuncCacheDOM[query];
		if(domCached){ return domCached; }

		// TODO:
		//		today we're caching DOM and QSA branches separately so we
		//		recalc useQSA every time. If we had a way to tag root+query
		//		efficiently, we'd be in good shape to do a global cache.

		var qcz = query.charAt(0);
		var nospace = (-1 == query.indexOf(" "));

		// byId searches are wicked fast compared to QSA, even when filtering
		// is required
		if( (query.indexOf("#") >= 0) && (nospace) ){
			forceDOM = true;
		}

		var useQSA = (
			qsaAvail && (!forceDOM) &&
			// as per CSS 3, we can't currently start w/ combinator:
			//		http://www.w3.org/TR/css3-selectors/#w3cselgrammar
			(specials.indexOf(qcz) == -1) &&
			// IE's QSA impl sucks on pseudos
			(!has("ie") || (query.indexOf(":") == -1)) &&

			(!(cssCaseBug && (query.indexOf(".") >= 0))) &&

			// FIXME:
			//		need to tighten up browser rules on ":contains" and "|=" to
			//		figure out which aren't good
			//		Latest webkit (around 531.21.8) does not seem to do well with :checked on option
			//		elements, even though according to spec, selected options should
			//		match :checked. So go nonQSA for it:
			//		http://bugs.dojotoolkit.org/ticket/5179
			(query.indexOf(":contains") == -1) && (query.indexOf(":checked") == -1) &&
			(query.indexOf("|=") == -1) // some browsers don't grok it
		);

		// TODO:
		//		if we've got a descendant query (e.g., "> .thinger" instead of
		//		just ".thinger") in a QSA-able doc, but are passed a child as a
		//		root, it should be possible to give the item a synthetic ID and
		//		trivially rewrite the query to the form "#synid > .thinger" to
		//		use the QSA branch


		if(useQSA){
			var tq = (specials.indexOf(query.charAt(query.length-1)) >= 0) ?
						(query + " *") : query;
			return _queryFuncCacheQSA[query] = function(root){
				try{
					// the QSA system contains an egregious spec bug which
					// limits us, effectively, to only running QSA queries over
					// entire documents.  See:
					//		http://ejohn.org/blog/thoughts-on-queryselectorall/
					//	despite this, we can also handle QSA runs on simple
					//	selectors, but we don't want detection to be expensive
					//	so we're just checking for the presence of a space char
					//	right now. Not elegant, but it's cheaper than running
					//	the query parser when we might not need to
					if(!((9 == root.nodeType) || nospace)){ throw ""; }
					var r = root[qsa](tq);
					// skip expensive duplication checks and just wrap in a NodeList
					r[noZip] = true;
					return r;
				}catch(e){
					// else run the DOM branch on this query, ensuring that we
					// default that way in the future
					return getQueryFunc(query, true)(root);
				}
			};
		}else{
			// DOM branch
			var parts = query.match(/([^\s,](?:"(?:\\.|[^"])+"|'(?:\\.|[^'])+'|[^,])*)/g);
			return _queryFuncCacheDOM[query] = ((parts.length < 2) ?
				// if not a compound query (e.g., ".foo, .bar"), cache and return a dispatcher
				getStepQueryFunc(query) :
				// if it *is* a complex query, break it up into its
				// constituent parts and return a dispatcher that will
				// merge the parts when run
				function(root){
					var pindex = 0, // avoid array alloc for every invocation
						ret = [],
						tp;
					while((tp = parts[pindex++])){
						ret = ret.concat(getStepQueryFunc(tp)(root));
					}
					return ret;
				}
			);
		}
	};

	var _zipIdx = 0;

	// NOTE:
	//		this function is Moo inspired, but our own impl to deal correctly
	//		with XML in IE
	var _nodeUID = has("ie") ? function(node){
		if(caseSensitive){
			// XML docs don't have uniqueID on their nodes
			return (node.getAttribute("_uid") || node.setAttribute("_uid", ++_zipIdx) || _zipIdx);

		}else{
			return node.uniqueID;
		}
	} :
	function(node){
		return (node._uid || (node._uid = ++_zipIdx));
	};

	// determine if a node in is unique in a "bag". In this case we don't want
	// to flatten a list of unique items, but rather just tell if the item in
	// question is already in the bag. Normally we'd just use hash lookup to do
	// this for us but IE's DOM is busted so we can't really count on that. On
	// the upside, it gives us a built in unique ID function.
	var _isUnique = function(node, bag){
		if(!bag){ return 1; }
		var id = _nodeUID(node);
		if(!bag[id]){ return bag[id] = 1; }
		return 0;
	};

	// attempt to efficiently determine if an item in a list is a dupe,
	// returning a list of "uniques", hopefully in document order
	var _zipIdxName = "_zipIdx";
	var _zip = function(arr){
		if(arr && arr.nozip){ return arr; }

		if(!arr || !arr.length){ return []; }
		if(arr.length < 2){ return [arr[0]]; }

		var ret = [];

		_zipIdx++;

		// we have to fork here for IE and XML docs because we can't set
		// expandos on their nodes (apparently). *sigh*
		var x, te;
		if(has("ie") && caseSensitive){
			var szidx = _zipIdx+"";
			for(x = 0; x < arr.length; x++){
				if((te = arr[x]) && te.getAttribute(_zipIdxName) != szidx){
					ret.push(te);
					te.setAttribute(_zipIdxName, szidx);
				}
			}
		}else if(has("ie") && arr.commentStrip){
			try{
				for(x = 0; x < arr.length; x++){
					if((te = arr[x]) && _isElement(te)){
						ret.push(te);
					}
				}
			}catch(e){ /* squelch */ }
		}else{
			for(x = 0; x < arr.length; x++){
				if((te = arr[x]) && te[_zipIdxName] != _zipIdx){
					ret.push(te);
					te[_zipIdxName] = _zipIdx;
				}
			}
		}
		return ret;
	};

	// the main executor
	var query = function(/*String*/ query, /*String|DOMNode?*/ root){
		// summary:
		//		Returns nodes which match the given CSS3 selector, searching the
		//		entire document by default but optionally taking a node to scope
		//		the search by. Returns an array.
		// description:
		//		dojo.query() is the swiss army knife of DOM node manipulation in
		//		Dojo. Much like Prototype's "$$" (bling-bling) function or JQuery's
		//		"$" function, dojo.query provides robust, high-performance
		//		CSS-based node selector support with the option of scoping searches
		//		to a particular sub-tree of a document.
		//
		//		Supported Selectors:
		//		--------------------
		//
		//		acme supports a rich set of CSS3 selectors, including:
		//
		//		- class selectors (e.g., `.foo`)
		//		- node type selectors like `span`
		//		- ` ` descendant selectors
		//		- `>` child element selectors
		//		- `#foo` style ID selectors
		//		- `*` universal selector
		//		- `~`, the preceded-by sibling selector
		//		- `+`, the immediately preceded-by sibling selector
		//		- attribute queries:
		//			- `[foo]` attribute presence selector
		//			- `[foo='bar']` attribute value exact match
		//			- `[foo~='bar']` attribute value list item match
		//			- `[foo^='bar']` attribute start match
		//			- `[foo$='bar']` attribute end match
		//			- `[foo*='bar']` attribute substring match
		//		- `:first-child`, `:last-child`, and `:only-child` positional selectors
		//		- `:empty` content emtpy selector
		//		- `:checked` pseudo selector
		//		- `:nth-child(n)`, `:nth-child(2n+1)` style positional calculations
		//		- `:nth-child(even)`, `:nth-child(odd)` positional selectors
		//		- `:not(...)` negation pseudo selectors
		//
		//		Any legal combination of these selectors will work with
		//		`dojo.query()`, including compound selectors ("," delimited).
		//		Very complex and useful searches can be constructed with this
		//		palette of selectors and when combined with functions for
		//		manipulation presented by dojo/NodeList, many types of DOM
		//		manipulation operations become very straightforward.
		//
		//		Unsupported Selectors:
		//		----------------------
		//
		//		While dojo.query handles many CSS3 selectors, some fall outside of
		//		what's reasonable for a programmatic node querying engine to
		//		handle. Currently unsupported selectors include:
		//
		//		- namespace-differentiated selectors of any form
		//		- all `::` pseduo-element selectors
		//		- certain pseudo-selectors which don't get a lot of day-to-day use:
		//			- `:root`, `:lang()`, `:target`, `:focus`
		//		- all visual and state selectors:
		//			- `:root`, `:active`, `:hover`, `:visited`, `:link`,
		//				  `:enabled`, `:disabled`
		//			- `:*-of-type` pseudo selectors
		//
		//		dojo.query and XML Documents:
		//		-----------------------------
		//
		//		`dojo.query` (as of dojo 1.2) supports searching XML documents
		//		in a case-sensitive manner. If an HTML document is served with
		//		a doctype that forces case-sensitivity (e.g., XHTML 1.1
		//		Strict), dojo.query() will detect this and "do the right
		//		thing". Case sensitivity is dependent upon the document being
		//		searched and not the query used. It is therefore possible to
		//		use case-sensitive queries on strict sub-documents (iframes,
		//		etc.) or XML documents while still assuming case-insensitivity
		//		for a host/root document.
		//
		//		Non-selector Queries:
		//		---------------------
		//
		//		If something other than a String is passed for the query,
		//		`dojo.query` will return a new `dojo/NodeList` instance
		//		constructed from that parameter alone and all further
		//		processing will stop. This means that if you have a reference
		//		to a node or NodeList, you can quickly construct a new NodeList
		//		from the original by calling `dojo.query(node)` or
		//		`dojo.query(list)`.
		//
		// query:
		//		The CSS3 expression to match against. For details on the syntax of
		//		CSS3 selectors, see <http://www.w3.org/TR/css3-selectors/#selectors>
		// root:
		//		A DOMNode (or node id) to scope the search from. Optional.
		// returns: Array
		// example:
		//		search the entire document for elements with the class "foo":
		//	|	dojo.query(".foo");
		//		these elements will match:
		//	|	<span class="foo"></span>
		//	|	<span class="foo bar"></span>
		//	|	<p class="thud foo"></p>
		// example:
		//		search the entire document for elements with the classes "foo" *and* "bar":
		//	|	dojo.query(".foo.bar");
		//		these elements will match:
		//	|	<span class="foo bar"></span>
		//		while these will not:
		//	|	<span class="foo"></span>
		//	|	<p class="thud foo"></p>
		// example:
		//		find `<span>` elements which are descendants of paragraphs and
		//		which have a "highlighted" class:
		//	|	dojo.query("p span.highlighted");
		//		the innermost span in this fragment matches:
		//	|	<p class="foo">
		//	|		<span>...
		//	|			<span class="highlighted foo bar">...</span>
		//	|		</span>
		//	|	</p>
		// example:
		//		set an "odd" class on all odd table rows inside of the table
		//		`#tabular_data`, using the `>` (direct child) selector to avoid
		//		affecting any nested tables:
		//	|	dojo.query("#tabular_data > tbody > tr:nth-child(odd)").addClass("odd");
		// example:
		//		remove all elements with the class "error" from the document
		//		and store them in a list:
		//	|	var errors = dojo.query(".error").orphan();
		// example:
		//		add an onclick handler to every submit button in the document
		//		which causes the form to be sent via Ajax instead:
		//	|	dojo.query("input[type='submit']").onclick(function(e){
		//	|		dojo.stopEvent(e); // prevent sending the form
		//	|		var btn = e.target;
		//	|		dojo.xhrPost({
		//	|			form: btn.form,
		//	|			load: function(data){
		//	|				// replace the form with the response
		//	|				var div = dojo.doc.createElement("div");
		//	|				dojo.place(div, btn.form, "after");
		//	|				div.innerHTML = data;
		//	|				dojo.style(btn.form, "display", "none");
		//	|			}
		//	|		});
		//	|	});

		root = root || getDoc();

		// throw the big case sensitivity switch
		var od = root.ownerDocument || root;	// root is either Document or a node inside the document
		caseSensitive = (od.createElement("div").tagName === "div");

		// NOTE:
		//		adding "true" as the 2nd argument to getQueryFunc is useful for
		//		testing the DOM branch without worrying about the
		//		behavior/performance of the QSA branch.
		var r = getQueryFunc(query)(root);

		// FIXME:
		//		need to investigate this branch WRT #8074 and #8075
		if(r && r.nozip){
			return r;
		}
		return _zip(r); // dojo/NodeList
	};
	query.filter = function(/*Node[]*/ nodeList, /*String*/ filter, /*String|DOMNode?*/ root){
		// summary:
		//		function for filtering a NodeList based on a selector, optimized for simple selectors
		var tmpNodeList = [],
			parts = getQueryParts(filter),
			filterFunc =
				(parts.length == 1 && !/[^\w#\.]/.test(filter)) ?
				getSimpleFilterFunc(parts[0]) :
				function(node){
					return array.indexOf(query(filter, dom.byId(root)), node) != -1;
				};
		for(var x = 0, te; te = nodeList[x]; x++){
			if(filterFunc(te)){ tmpNodeList.push(te); }
		}
		return tmpNodeList;
	};
	return query;
});

},
'xstyle/load-css':function(){
/**
 * This includes code from https://github.com/unscriptable/cssx
 * Copyright (c) 2010 unscriptable.com
 */

/*jslint browser:true, on:true, sub:true */

define([], function(){
"use strict";

/*
 * RequireJS css! plugin
 * This plugin will load and wait for css files.  This could be handy when
 * loading css files as part of a layer or as a way to apply a run-time theme.
 * Most browsers do not support the load event handler of the link element.
 * Therefore, we have to use other means to detect when a css file loads.
 * (The HTML5 spec states that the LINK element should have a load event, but
 * not even Chrome 8 or FF4b7 have it, yet.
 * http://www.w3.org/TR/html5/semantics.html#the-link-element)
 *
 * This plugin tries to use the load event and a universal work-around when
 * it is invoked the first time.  If the load event works, it is used on
 * every successive load.  Therefore, browsers that support the load event will
 * just work (i.e. no need for hacks!).  FYI, Feature-detecting the load
 * event is tricky since most browsers have a non-functional onload property.
 *
 * The universal work-around watches a stylesheet until its rules are
 * available (not null or undefined).  There are nuances, of course, between
 * the various browsers.  The isLinkReady function accounts for these.
 *
 * Note: it appears that all browsers load @import'ed stylesheets before
 * fully processing the rest of the importing stylesheet. Therefore, we
 * don't need to find and wait for any @import rules explicitly.
 *
 * Note #2: for Opera compatibility, stylesheets must have at least one rule.
 * AFAIK, there's no way to tell the difference between an empty sheet and
 * one that isn't finished loading in Opera (XD or same-domain).
 *
 * Options:
 *      !nowait - does not wait for the stylesheet to be parsed, just loads it
 *
 * Global configuration options:
 *
 * cssDeferLoad: Boolean. You can also instruct this plugin to not wait
 * for css resources. They'll get loaded asap, but other code won't wait
 * for them. This is just like using the !nowait option on every css file.
 *
 * cssWatchPeriod: if direct load-detection techniques fail, this option
 * determines the msec to wait between brute-force checks for rules. The
 * default is 50 msec.
 *
 * You may specify an alternate file extension:
 *      require('css!myproj/component.less') // --> myproj/component.less
 *      require('css!myproj/component.scss') // --> myproj/component.scss
 *
 * When using alternative file extensions, be sure to serve the files from
 * the server with the correct mime type (text/css) or some browsers won't
 * parse them, causing an error in the plugin.
 *
 * usage:
 *      require(['css!myproj/comp']); // load and wait for myproj/comp.css
 *      define(['css!some/folder/file'], {}); // wait for some/folder/file.css
 *      require(['css!myWidget!nowait']);
 *
 * Tested in:
 *      Firefox 1.5, 2.0, 3.0, 3.5, 3.6, and 4.0b6
 *      Safari 3.0.4, 3.2.1, 5.0
 *      Chrome 7 (8+ is partly b0rked)
 *      Opera 9.52, 10.63, and Opera 11.00
 *      IE 6, 7, and 8
 *      Netscape 7.2 (WTF? SRSLY!)
 * Does not work in Safari 2.x :(
 * In Chrome 8+, there's no way to wait for cross-domain (XD) stylesheets.
 * See comments in the code below.
 * TODO: figure out how to be forward-compatible when browsers support HTML5's
 *  load handler without breaking IE and Opera
*/


	var
		// compressibility shortcuts
		onreadystatechange = 'onreadystatechange',
		onload = 'onload',
		createElement = 'createElement',
		// failed is true if RequireJS threw an exception
		failed = false,
		doc = document,
		cache = typeof _css_cache == "undefined" ? {} : _css_cache,
		undef,
		features = {
			"event-link-onload": document.createElement("link").onload === null &&
				// safari lies about the onload event
				!navigator.userAgent.match(/AppleWebKit/),
			"dom-create-style-element": !document.createStyleSheet
		},
		// find the head element and set it to it's standard property if nec.
		head = doc.head || (doc.head = doc.getElementsByTagName('head')[0]);

	function has (feature) {
		return features[feature];
	}
	function createLink (doc, optHref) {
		var link = doc[createElement]('link');
		link.rel = "stylesheet";
		link.type = "text/css";
		if (optHref) {
			link.href = optHref;
		}
		return link;
	}
	function nameWithExt (name, defaultExt) {
		return name.lastIndexOf('.') <= name.lastIndexOf('/') ?
			name + '.' + defaultExt : name;
	}
	function parseSuffixes (name) {
		// creates a dual-structure: both an array and a hashmap
		// suffixes[0] is the actual name
		var parts = name.split('!'),
			suf, i = 1, pair;
		while ((suf = parts[i++])) { // double-parens to avoid jslint griping
			pair = suf.split('=', 2);
			parts[pair[0]] = pair.length == 2 ? pair[1] : true;
		}
		return parts;
	}
		

	if(!has("bundled-css")){ // if all the CSS is bundled, we don't need to the loader code
		var loadDetector = function(params, cb){ 
			// failure detection
			// we need to watch for onError when using RequireJS so we can shut off
			// our setTimeouts when it encounters an error.
			if (require.onError) {
				require.onError = (function (orig) {
					return function () {
						failed = true;
						orig.apply(this, arguments);
					}
				})(require.onError);
			}
		
			/***** load-detection functions *****/
		
			function loadHandler (params, cb) {
				// We're using 'readystatechange' because IE and Opera happily support both
				var link = params.link;
				link[onreadystatechange] = link[onload] = function () {
					if (!link.readyState || link.readyState == 'complete') {
						features["event-link-onload"] = true;
						cleanup(params);
						cb();
					}
				};
			}
		
		
			function isLinkReady (link) {
				// based on http://www.yearofmoo.com/2011/03/cross-browser-stylesheet-preloading.html
				// Therefore, we need
				// to continually test beta browsers until they all support the LINK load
				// event like IE and Opera.
				// webkit's and IE's sheet is null until the sheet is loaded
				var sheet = link.sheet || link.styleSheet;
				if(sheet){
					var styleSheets = document.styleSheets;
					for(var i = styleSheets.length; i > 0; i--){
						if(styleSheets[i-1] == sheet){
							return true;
						}
					}
				}
			}
		
			function ssWatcher (params, cb) {
				// watches a stylesheet for loading signs.
				if (isLinkReady(params.link)) {
					cleanup(params);
					cb();
				}
				else if (!failed) {
					setTimeout(function () { ssWatcher(params, cb); }, params.wait);
				}
			}
		
			function cleanup (params) {
				var link = params.link;
				link[onreadystatechange] = link[onload] = null;
			}
			// It would be nice to use onload everywhere, but the onload handler
			// only works in IE and Opera.
			// Detecting it cross-browser is completely impossible, too, since
			// THE BROWSERS ARE LIARS! DON'T TELL ME YOU HAVE AN ONLOAD PROPERTY
			// IF IT DOESN'T DO ANYTHING!
			var loaded;
			function cbOnce () {
				if (!loaded) {
					loaded = true;
					cb();
				}
			}
			loadHandler(params, cbOnce);
			if (!has("event-link-onload")) ssWatcher(params, cbOnce);
		
		};
	}
	function insertCss(css){
		if(has("dom-create-style-element")){
			// we can use standard <style> element creation
			styleSheet = document.createElement("style");
			styleSheet.setAttribute("type", "text/css");
			styleSheet.appendChild(document.createTextNode(css));
			head.insertBefore(styleSheet, head.firstChild);
			return styleSheet;
		}
		else{
			var styleSheet = document.createStyleSheet();
			styleSheet.cssText = css;
			return styleSheet.owningElement;
		}
	}
	/***** finally! the actual plugin *****/
	return function (resourceDef, callback, config) {
				var resources = resourceDef.split(","),
					loadingCount = resources.length,

				// all detector functions must ensure that this function only gets
				// called once per stylesheet!
					loaded = 
				function () {
					// load/error handler may have executed before stylesheet is
					// fully parsed / processed in Opera, so use setTimeout.
					// Opera will process before the it next enters the event loop
					// (so 0 msec is enough time).
					if(--loadingCount == 0){
						// TODO: move this setTimeout to loadHandler
						callback(link.sheet || link.styleSheet)
						// TODO: Is this need for Opera?
						//setTimeout(onCssLoaded,0);
					}
				}

				// after will become truthy once the loop executes a second time
				for(var i = 0, after; i < resources.length; i++, after = url){
					resourceDef = resources[i];
					var cached = cache[resourceDef]; 
					if(cached){
						link = insertCss(cached);
						return loaded();
					}
					var
						// TODO: this is a bit weird: find a better way to extract name?
						opts = parseSuffixes(resourceDef),
						name = opts.shift(),
						url = nameWithExt(name, "css"),
						link = createLink(doc),
						nowait = 'nowait' in opts ? opts.nowait != 'false' : !!(config && config.cssDeferLoad),
						params = {
							link: link,
							url: url,
							wait: config && config.cssWatchPeriod || 25
						};
					// hook up load detector(s)
					loadDetector(params, loaded);
					if (nowait) {
						callback(link);
					}

					// go!
					link.href = url;

					head.appendChild(link);
				}
			};

});

},
'xstyle/xstyle':function(){
if(typeof define == "undefined"){
	(function(){
		// pseudo passive loader
		var modules = {};
		define = function(id, deps, factory){
			for(var i = 0;i < deps.length; i++){
				deps[i] = modules[deps[i]];
			}
			modules[id] = factory.apply(this, deps);
		};
		require = function(deps){
			define("", deps, factory);
		};
	})();
}
define("xstyle/xstyle", ["require"], function (require) {
	"use strict";
	var undef, testDiv = document.createElement("div");
	function search(tag){
		var elements = document.getElementsByTagName(tag);
		for(var i = 0; i < elements.length; i++){
			checkImports(elements[i]);
		}
	}
	var ua = navigator.userAgent;
	var vendorPrefix = ua.indexOf("WebKit") > -1 ? "-webkit-" :
		ua.indexOf("Firefox") > -1 ? "-moz-" :
		ua.indexOf("MSIE") > -1 ? "-ms-" :
		ua.indexOf("Opera") > -1 ? "-o-" : "";
	function checkImports(element, callback, fixedImports){
		var sheet = element.sheet || element.styleSheet;
		var needsParsing = sheet.needsParsing, // load-imports can check for the need to parse when it does it's recursive look at imports 
			cssRules = sheet.rules || sheet.cssRules;
		function fixImports(){
			// need to fix imports, applying load-once semantics for all browsers, and flattening for IE to fix nested @import bugs
			require(["./load-imports"], function(load){
				load(element, function(){
					checkImports(element, callback, true);
				});
			});
		}
		if(sheet.imports && !fixedImports && sheet.imports.length){
			// this is how we check for imports in IE
			return fixImports();
		}
		if(!needsParsing){
			for(var i = 0; i < cssRules.length; i++){								
				var rule = cssRules[i];
				if(rule.href && !fixedImports){
					// it's an import (for non-IE browsers)
					return fixImports();
				}
				if(rule.selectorText && rule.selectorText.substring(0,2) == "x-"){
					// an extension is used, needs to be parsed
					needsParsing = true;
				}
			}
		}
		if(needsParsing){
			// ok, determined that CSS extensions are in the CSS, need to get the source and really parse it
			parse(sheet.source || sheet.ownerElement.innerHTML, sheet, callback);
		}
	}
	function parse(css, styleSheet, callback) {
		// normalize the stylesheet.
		if(!styleSheet.addRule){
			// only FF doesn't have this
			styleSheet.addRule = function(selector, style, index){
				return this.insertRule(selector + "{" + style + "}", index >= 0 ? index : this.cssRules.length);
			}
		}
		if(!styleSheet.deleteRule){
			styleSheet.deleteRule = sheet.removeRule;
		}
		var handlers = {property:{}};
		function addHandler(type, name, module){
			var handlersForType = handlers[type] || (handlers[type] = {});
			handlersForType[name] = module;
		}
		function addExtensionHandler(type){
			if(!handlers[type]){
				handlers[type] = {};
			}
			addHandler("selector", 'x-' + type, {
				onRule: function(rule){
					rule.eachProperty(function(name, value){
						do{
							var parts = value.match(/require\s*\((.+)\)|([^, ]+)([, ]+(.+))?/);
							if(parts[1]){
								return addHandler(type, name, parts[1]);
							}
							var first = parts[2];
							if(first == "default"){
								if((type == "property" && typeof testDiv.style[name] == "string")){
									return;
								}
								if(type == "pseudo"){
									try{
										document.querySelectorAll("x:" + name);
										return;
									}catch(e){}
								}
							}else if(first == "prefix"){
								if(typeof testDiv.style[vendorPrefix + name] == "string"){
									return addHandler(type, name, 'xstyle/xstyle');
								}
							}else{
								return addHandler(type, name, function(){
									return value;
								});
							}
						}while(value = parts[4]);
/*						var ifUnsupported = value.charAt(value.length - 1) == "?";
						value = value.replace(/require\s*\(|\)\??/g, '');
						if(!ifUnsupported || typeof testDiv.style[name] != "string"){ // if conditioned on support, test to see browser has that style
							// not supported as a standard property, now let's check to see if we can support it with vendor prefixing
							if(ifUnsupported && typeof testDiv.style[vendorPrefix + name] == "string"){
								// it does support vendor prefixing, fix it with that
								value = 'xstyle/xstyle';
							}
							addHandler(type, name, value);
						}*/
					});
				}
			});
		}
		addExtensionHandler("property");
		addExtensionHandler("value");
		addExtensionHandler("pseudo");
		var waiting = 1;
		var baseUrl = (styleSheet.href || location.href).replace(/[^\/]+$/,'');
		var properties = [], values = [];
		var valueModules = {};
		
		var convertedRules = [];
		var valueRegex = new RegExp("(?:^|\\W)(" + values.join("|") + ")(?:$|\\W)");
		function Rule () {}
		Rule.prototype = {
			eachProperty: function (onproperty, propertyRegex) {
				var selector, css;
				selector = this.selector; //(this.children ? onproperty(0, "layout", this.children) || this.selector : this.selector);
				this.cssText.replace(/\s*([^;:]+)\s*:\s*([^;]+)?/g, function (full, name, value) {
					onproperty(name, value);
				});
				if(this.children){
					for(var i = 0; i < this.children.length; i++){
						var child = this.children[i];
						if(!child.selector){ // it won't have a selector if it is property with nested properties
							onproperty(child.property, child);
						}
					}
				}
			},
			fullSelector: function(){
				return (this.parent ? this.parent.fullSelector() : "") + (this.selector || "") + " ";  
			},
			add: function(selector, cssText){
				if(cssText){
					styleSheet.addRule ?
						styleSheet.addRule(selector, cssText) :
						styleSheet.insertRule(selector + '{' + cssText + '}', styleSheet.cssRules.length);
				}
			},
			cssText: ""
		};
		
		var lastRule = new Rule;
		lastRule.css = css;
		
		function onProperty(name, value) {
			// this is called for each CSS property
			var propertyName = name;
			do{
				var handlerForName = handlers.property[name];
				if(handlerForName){
					return handler(handlerForName, "onProperty", propertyName, value);
				}
				// if we didn't match, we try to match property groups, for example "background-image" should match the "background" listener 
				name = name.substring(0, name.lastIndexOf("-"));
			}while(name);
		}
		function onIdentifier(identifier, name, value){
			var handlerForName = handlers.value[identifier];
			if(handlerForName){
				handler(handlerForName, "onIdentifier", name, value);
			}
		}
		function onRule(selector, rule){
			var handlerForName = handlers.selector[selector];
			if(handlerForName){
				handler(handlerForName, "onRule", rule);
			}
		}
		function onPseudo(pseudo, rule){
			var handlerForName = handlers.pseudo[pseudo];
			if(handlerForName){
				handler(handlerForName, "onPseudo", pseudo, rule);
			}
		}
		function handler(module, type, name, value){
			if(module){
				var rule = lastRule;
				var ruleHandled = function(text){
					console.log("loaded ", module, text);
					if(text){
						/* TODO: is the a way to determine the index deterministically?
						var cssRules = styleSheet.rules || styleSheet.cssRules;
						for(var index = rule.index || 0; index < cssRules.length; index++){
							if(cssRules[index].selectorText == rule.fullSelector(){
								break;
							}
						}*/
						/* TODO: merge IE filters
						if(isIE){
							var filters = [];
							convertedText = convertedText.replace(/filter: ([^;]+);/g, function(t, filter){
								filters.push(filter);
								return "";
							});
							if(filters.length){
								console.log("filters", filters);
								convertedText = "zoom: 1;filter: " + filters.join("") + ";" + convertedText;
							}
						}
						*/
						styleSheet.addRule(rule.fullSelector(), text);
					}
					finishedLoad();
				};
				
				waiting++;
				console.log("loading ", module, name, value);
				var onLoad = function(module){
					var result = module[type](name, value, rule, styleSheet);
					if(result && result.then){
							// a promise, return immediately defer handling
						result.then(ruleHandled);
					}else{
						ruleHandled(result);
					}
				}
				typeof module == "string" ? require([module], onLoad) : onLoad(module);					
			}
		}
		// parse the CSS, finding each rule
		css.replace(/\s*(?:([^{;\s]+)\s*{)?\s*([^{}]+;)?\s*(};?)?/g, function (full, selector, properties, close) {
			// called for each rule
			if (selector) {
				// a selector was found, start a new rule (note this can be nested inside another selector)
				var newRule = new Rule();
				(lastRule.children || (lastRule.children = [])).push(newRule); // add to the parent layout 
				newRule.parent = lastRule;
				if(selector.charAt(selector.length - 1) == ":"){
					// it is property style nesting
					newRule.property= selector.substring(0, selector.length - 1);
				}else{
					// just this segment of selector
					newRule.selector = selector; 
				}
				lastRule = newRule;
			}
			if (properties) {
				// some properties were found
				lastRule.cssText += properties;
			}
			if (close) {
				// rule was closed with }
				// TODO: use a specialized regex that only looks for registered properties
				lastRule.cssText.replace(/\s*([^;:]+)\s*:\s*([^;]+)?/g, function (full, name, value) {
					onProperty(name, value);
					value.replace(valueRegex, function(t, identifier){
						//onIdentifier(identifier, name, value);
					});
				});
				if(lastRule.children){
					for(var i = 0; i < lastRule.children.length; i++){
						var child = lastRule.children[i];
						if(!child.selector){ // it won't have a selector if it is property with nested properties
							onProperty(child.property, child);
						}
					}
				}
				onRule(lastRule.selector, lastRule);
				lastRule.selector && lastRule.selector.replace(/:([-\w]+)/, function(t, pseudo){
					return onPseudo(pseudo, lastRule);
				});
				lastRule = lastRule.parent;
			}
		});
		function finishedLoad(){
			if(--waiting == 0){
				if(callback){
					callback(styleSheet);
				}
			}
		}		
		finishedLoad();
	}
	search('link');
	search('style');
	var xstyle =  {
		process: checkImports,
		vendorPrefix: vendorPrefix,
		onProperty: function(name, value){
			// basically a noop for most operations, we rely on the vendor prefixing in the main property parser 
			if(name == "opacity" && vendorPrefix == "-ms-"){
				return 'filter: alpha(opacity=' + (value * 100) + '); zoom: 1;';
			}
			return vendorPrefix + name + ':' + value + ';';
		}
	};
	return xstyle;

});

},
'vsd/app':function(){
'use strict';

require(['vsd/dojoModules'], function (dojoModules) {

    angular.module('vsd', ['ngRoute', 'ngCookies', 'filters'])
        .config(function ($routeProvider) {
            $routeProvider
                .when('/', {
                    templateUrl: 'vsd/main/main.html',
                    controller: 'MainCtrl'
                })
                .when('/window/:id?', {
                    templateUrl: 'vsd/window/window.html',
                    controller: 'WindowCtrl',
                    resolve: {
                        operators: function (operatorSvc) {
                            return operatorSvc.getOperators();
                        },
                        window: function ($q, $location, $route, windowSvc, newWindowSvc, loadingMaskSvc, dojoModules) {
                            loadingMaskSvc.showLoadingMask(dojoModules.resources.main_openingWindow_msg_txt);
                            var newWindow = newWindowSvc.getWindow();
                            if ($route.current.params.id) {
                                return windowSvc.getWindow($route.current.params.id, true);
                            }
                            if (!newWindow) {
                                $location.path('/');
                                var defer = $q.defer();
                                defer.reject();
                                return defer.promise;
                            }

                            return newWindow;
                        }
                    }
                })
                .when('/scenario/:id?', {
                    templateUrl: 'vsd/scenario/scenario.html',
                    controller: 'ScenarioCtrl'
                })
                .when('/deployment/:id?', {
                    templateUrl: 'vsd/deployment/deployment.html',
                    controller: 'DeploymentCtrl'
                })
                .when('/deployment/:id/simulation', {
                    templateUrl: 'vsd/deployment/simulation/simulation.html',
                    controller: 'SimulationCtrl'
                })
                .otherwise({
                    redirectTo: '/'
                });
        })
        .factory('dojoDepends', function () {
            return dojoModules;
        })
        .factory('dojoModules', function () {
            return dojoModules;
        });


    require([
        'vsd/common/vsmCommonConst',
        'vsd/common/charts/d3DistribChart',
        'vsd/common/charts/d3DualSlider',
        'vsd/common/charts/d3ParallelCoordChart',
        'vsd/common/charts/d3PctGauge',
        'vsd/common/charts/d3StackedBarChart',
        'vsd/common/charts/d3UtilsSvc',
        'vsd/common/charts/SvgTooltip',
        'vsd/common/charts/vsdBoxplotChart',
        'vsd/common/charts/vsdD3VariableImportanceChart',
        'vsd/common/dialogs/ConfirmDialogCtrl',
        'vsd/common/dialogs/confirmDialogSvc',
        'vsd/common/dialogs/OpenDialogCtrl',
        'vsd/common/loadingMask/loadingMaskSvc',
        'vsd/common/message/messageSvc',
        'vsd/common/ng/directives/conditionBuilder/conditionBuilder',
        'vsd/common/ng/directives/conditionBuilder/conditionBuilderHierarchy',
        'vsd/common/ng/directives/conditionBuilder/conditionPackage',
        'vsd/common/ng/directives/dualSelector/sngDualSelector',
        'vsd/common/ng/directives/filereader',
        'vsd/common/ng/directives/sngCheckboxGrid',
        'vsd/common/ng/directives/sngDateTextBox',
        'vsd/common/ng/directives/sngDiagram',
        'vsd/common/ng/directives/sngDgrid',
        'vsd/common/ng/directives/sngDialog',
        'vsd/common/ng/directives/sngDijit',
        'vsd/common/ng/directives/sngDojoDndTargetArea',
        'vsd/common/ng/directives/sngDojoParse',
        'vsd/common/ng/directives/sngDragSort',
        'vsd/common/ng/directives/sngDropDownButton',
        'vsd/common/ng/directives/sngIndentedDropDown',
        'vsd/common/ng/directives/sngLayoutContainer',
        'vsd/common/ng/directives/sngListButton',
        'vsd/common/ng/directives/sngMultiSelect',
        'vsd/common/ng/directives/sngStringNumberSpinner',
        'vsd/common/ng/directives/sngTooltipDialog',
        'vsd/common/ng/directives/sngTree',
        'vsd/common/ng/services/arrayReorderSvc',
        'vsd/common/ng/services/columnNameValidationSvc',
        'vsd/common/ng/services/encoderSvc',
        'vsd/common/ng/services/fieldParserSvc',
        'vsd/common/ng/services/localeSvc',
        'vsd/common/ng/services/objectBrowseSvc',
        'vsd/common/ng/services/operatorSvc',
        'vsd/common/ng/services/configSvc',
        'vsd/common/ng/services/tableSelectionSvc',
        'vsd/common/ng/services/treeUtilSvc',
        'vsd/common/ng/services/Util',
        'vsd/common/ng/services/vsdUtil',
        'vsd/common/ng/currencyCodes',
        'vsd/common/ng/reservedWords',
        'vsd/common/ng/errorInterceptor',
        'vsd/import/ImportDialogCtrl',
        'vsd/import/ImportRenameDialogCtrl',
        'vsd/import/importRenameDialogSvc',

        'vsd/main/AppCtrl',
        'vsd/main/MainCtrl',
        'vsd/main/OverviewCtrl',

        'vsd/deployment/DeploymentCtrl',
        'vsd/deployment/DeploymentSvcs',
        'vsd/deployment/confirmRefresh/DeploymentRefreshCtrl',
        'vsd/deployment/editSelection/DeploymentEditSelectionCtrl',
        'vsd/deployment/esp/DeploymentESPCtrl',
        'vsd/deployment/new/NewDeploymentCtrl',
        'vsd/deployment/new/newDeploymentSvc',
        'vsd/deployment/simulation/RunSimulationDialogCtrl',
        'vsd/deployment/simulation/simParameterTree',
        'vsd/deployment/simulation/simulationCondition',
        'vsd/deployment/simulation/SimulationCtrl',
        'vsd/deployment/simulation/simulationSvc',
        'vsd/deployment/simulation/simulationTrialGrid',

        'vsd/scenario/ScenarioCtrl',
        'vsd/scenario/ScenarioSvc',
        'vsd/scenario/actionEditor/ScenarioActionColumnEditorCtrl',
        'vsd/scenario/actionEditor/actionEditorSvc',
        'vsd/scenario/actionEditor/actionInput',
        'vsd/scenario/decisionTable/directives/actionCell',
        'vsd/scenario/decisionTable/directives/conditionCell',
        'vsd/scenario/decisionTable/directives/contextMenu',
        'vsd/scenario/decisionTable/directives/decisionTable',
        'vsd/scenario/decisionTable/decisionTableFilters',
        'vsd/scenario/decisionTable/ScenarioCharDropCtrl',
        'vsd/scenario/decisionTable/ScenarioDecisionTableCtrl',
        'vsd/scenario/decisionTable/ScenarioNumericDropCtrl',
        'vsd/scenario/decisionTree/decisionTreeSvc',
        'vsd/scenario/decisionTree/ScenarioDecisionTreeCtrl',
        'vsd/scenario/decisionTree/scenarioInputTreeSvc',
        'vsd/scenario/decisionTree/vsdD3DecisionTree',
        'vsd/scenario/exploreDirect/pchartHoverMenuSvc',
        'vsd/scenario/exploreDirect/ScenarioDirectCtrl',
        'vsd/scenario/exploreDirect/ScenarioExplorerCtrl',
        'vsd/scenario/exploreDirect/scenarioPChart',
        'vsd/scenario/exploreDirect/ScenarioRebinCtrl',
        'vsd/scenario/new/NewScenarioCtrl',
        'vsd/scenario/new/newScenarioSvc',
        'vsd/scenario/saveAs/SaveAsCtrl',
        'vsd/scenario/saveAs/saveAsSvc',

        'vsd/window/WindowCtrl',
        'vsd/window/WindowSvc',
        'vsd/window/columnsEditor/columnProperties',
        'vsd/window/columnsEditor/WindowEditCtrl',
        'vsd/window/new/NewWindowCtrl',
        'vsd/window/new/newWindowSvc',

        'vsd/appConfig',
        'dojo/domReady!'], function () {
        angular.bootstrap(document.body, ['vsd']);
        document.title = dojoModules.resources.app_title_txt;
    });
});


},
'vsd/dojoModules':function(){
'use strict';
(function () {
    var dojoModConfig = {
        'tree': 'dgrid/tree',
        'registry': 'dijit/registry',
        'Tooltip': 'dijit/Tooltip',
        'Avatar':          'dojo/dnd/Avatar',
        'Manager':         'dojo/dnd/Manager',
        'domAttr':        'dojo/dom-attr',
        'Menu': 'dijit/Menu',
        'MenuItem': 'dijit/MenuItem',
        'PopupMenuItem': 'dijit/PopupMenuItem',
        'popup': 'dijit/popup',
        'FilteringSelect': 'dijit/form/FilteringSelect',
        'ComboBox': 'dijit/form/ComboBox',
        'MultiSelect': 'dijit/form/MultiSelect',
        'DateTextBox': 'dijit/form/DateTextBox',
        'declare': 'dojo/_base/declare',
        'lang': 'dojo/_base/lang',
        'aspect': 'dojo/aspect',
        'currency': 'dojo/currency',
        'locale': 'dojo/date/locale',
        'DojoDND': 'dojo/dnd/Source',
        'DojoDNDTarget': 'dojo/dnd/Target',
        'domClass': 'dojo/dom-class',
        'domStyle': 'dojo/dom-style',
        'domConstruct': 'dojo/dom-construct',
        'resources': 'dojo/i18n!vsd/nls/resources',
        'json': 'dojo/json',
        'number': 'dojo/number',
        'on': 'dojo/on',
        'parser': 'dojo/parser',
        'query': 'dojo/query',
        'Memory': 'dojo/store/Memory',
        'string': 'dojo/string',
        'put': 'put-selector/put',
        'VSDMessages': 'vsd/common/message/VSDMessages',
        'TooltipDialog': 'dijit/TooltipDialog',
        'DropDownButton': 'dijit/form/DropDownButton',
        'DropDownMenu': 'dijit/DropDownMenu',
        'MenuSeparator': 'dijit/MenuSeparator',
        'ResizeHandle': 'dojox/layout/ResizeHandle',
        'reqs': [
            'dojo/dnd/Avatar',
            'dojo/dnd/Manager',
            'dojo/dom-attr',
            'dgrid/editor',
            'dgrid/extensions/ColumnReorder',
            'dgrid/extensions/ColumnResizer',
            'dgrid/extensions/CompoundColumns',
            'dgrid/extensions/DijitRegistry',
            'dgrid/extensions/DnD',
            'dgrid/CellSelection',
            'dgrid/ColumnSet',
            'dgrid/Keyboard',
            'dgrid/OnDemandGrid',
            'dgrid/Selection',
            'dijit/dijit',
            'dijit/Dialog',
            'dijit/form/Button',
            'dijit/form/CheckBox',
            'dijit/Dialog',
            'dijit/Tree',
            'dijit/tree/ObjectStoreModel',
            'dijit/form/Form',
            'dijit/form/NumberSpinner',
            'dijit/form/SimpleTextarea',
            'dijit/form/Textarea',
            'dijit/form/TextBox',
            'dijit/form/NumberTextBox',
            'dijit/form/ValidationTextBox',
            'dijit/form/CurrencyTextBox',
            "dijit/form/HorizontalSlider",
            'dijit/layout/BorderContainer',
            'dijit/layout/ContentPane',
            'dijit/layout/TabContainer',
            'dijit/MenuSeparator',
            'dijit/Toolbar',
            'dijit/ToolbarSeparator',
            'dojo/_base/NodeList',
            'dojo/keys',
            'dijit/form/RadioButton',
            'dojo/store/Memory']
    };

    var modLocations = [];
    var modNames = [];
    for (var mod in dojoModConfig) {
        if (dojoModConfig.hasOwnProperty(mod) && mod !== 'reqs') {
            modLocations.push(dojoModConfig[mod]);
            modNames.push(mod);
        }
    }
    // Additional required dependencies
    modLocations = modLocations.concat(dojoModConfig['reqs']);

    define(modLocations, function () {
        var modules = {};
        for (var ii = 0; ii < modNames.length; ii++) {
            modules[modNames[ii]] = arguments[ii];
        }
        return modules;
    });

})();
},
'dgrid/tree':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/_base/Deferred",
	"dojo/query",
	"dojo/on",
	"dojo/aspect",
	"./util/has-css3",
	"./Grid",
	"dojo/has!touch?./util/touch",
	"put-selector/put"
], function(declare, arrayUtil, Deferred, querySelector, on, aspect, has, Grid, touchUtil, put){

function defaultRenderExpando(level, hasChildren, expanded, object){
	// summary:
	//		Provides default implementation for column.renderExpando.
	
	var dir = this.grid.isRTL ? "right" : "left",
		cls = ".dgrid-expando-icon",
		node;
	if(hasChildren){
		cls += ".ui-icon.ui-icon-triangle-1-" + (expanded ? "se" : "e");
	}
	node = put("div" + cls + "[style=margin-" + dir + ": " +
		(level * (this.indentWidth || 9)) + "px; float: " + dir + "]");
	node.innerHTML = "&nbsp;"; // for opera to space things properly
	return node;
}

function ontransitionend(event){
	var container = this,
		height = this.style.height;
	if(height){
		// After expansion, ensure display is correct;
		// after collapse, set display to none to improve performance
		this.style.display = height == "0px" ? "none" : "block";
	}
	
	// Reset height to be auto, so future height changes (from children
	// expansions, for example), will expand to the right height.
	if(event){
		// For browsers with CSS transition support, setting the height to
		// auto or "" will cause an animation to zero height for some
		// reason, so temporarily set the transition to be zero duration
		put(this, ".dgrid-tree-resetting");
		setTimeout(function(){
			// Turn off the zero duration transition after we have let it render
			put(container, "!dgrid-tree-resetting");
		});
	}
	// Now set the height to auto
	this.style.height = "";
}

function tree(column){
	// summary:
	//		Adds tree navigation capability to a column.
	
	var originalRenderCell = column.renderCell || Grid.defaultRenderCell;
	
	var currentLevel, // tracks last rendered item level (for aspected insertRow)
		clicked; // tracks row that was clicked (for expand dblclick event handling)
		
	if(!column){ column = {}; }
	
	column.shouldExpand = column.shouldExpand || function(row, level, previouslyExpanded){
		// summary:
		//		Function called after each row is inserted to determine whether
		//		expand(rowElement, true) should be automatically called.
		//		The default implementation re-expands any rows that were expanded
		//		the last time they were rendered (if applicable).
		
		return previouslyExpanded;
	};
	
	aspect.after(column, "init", function(){
		var grid = column.grid,
			colSelector = ".dgrid-content .dgrid-column-" + column.id,
			listeners = []; // to be removed when this column is destroyed

		// Turn off automatic cleanup of empty observers, to prevent confusion
		// due to observers operating at multiple hierarchy levels.
		grid.cleanEmptyObservers = false;
		
		if(!grid.store){
			throw new Error("dgrid tree column plugin requires a store to operate.");
		}
		
		if (!column.renderExpando){
			column.renderExpando = defaultRenderExpando;
		}
		
		// Set up the event listener once and use event delegation for better memory use.
		listeners.push(grid.on(
			column.expandOn || ".dgrid-expando-icon:click," + colSelector + ":dblclick," + colSelector + ":keydown",
			function(event){
				var row = grid.row(event);	
				if((!grid.store.mayHaveChildren || grid.store.mayHaveChildren(row.data)) &&
						(event.type != "keydown" || event.keyCode == 32) &&
						!(event.type == "dblclick" && clicked && clicked.count > 1 &&
							row.id == clicked.id && event.target.className.indexOf("dgrid-expando-icon") > -1)){
					grid.expand(row);
				}
				
				// If the expando icon was clicked, update clicked object to prevent
				// potential over-triggering on dblclick (all tested browsers but IE < 9).
				if(event.target.className.indexOf("dgrid-expando-icon") > -1){
					if(clicked && clicked.id == grid.row(event).id){
						clicked.count++;
					}else{
						clicked = {
							id: grid.row(event).id,
							count: 1
						};
					}
				}
			})
		);
		
		if(has("touch")){
			// Also listen on double-taps of the cell.
			listeners.push(grid.on(touchUtil.selector(colSelector, touchUtil.dbltap),
				function(){ grid.expand(this); }));
		}
		
		// Set up hash to store IDs of expanded rows
		if(!grid._expanded){ grid._expanded = {}; }
		
		listeners.push(aspect.after(grid, "insertRow", function(rowElement){
			// Auto-expand (shouldExpand) considerations
			var row = this.row(rowElement),
				expanded = column.shouldExpand(row, currentLevel, this._expanded[row.id]);
			
			if(expanded){ this.expand(rowElement, true, true); }
			return rowElement; // pass return value through
		}));
		
		listeners.push(aspect.before(grid, "removeRow", function(rowElement, justCleanup){
			var connected = rowElement.connected;
			if(connected){
				// if it has a connected expando node, we process the children
				querySelector(">.dgrid-row", connected).forEach(function(element){
					grid.removeRow(element, true);
				});
				// now remove the connected container node
				if(!justCleanup){
					put(connected, "!");
				}
			}
		}));
		
		if(column.collapseOnRefresh){
			// Clear out the _expanded hash on each call to cleanup
			// (which generally coincides with refreshes, as well as destroy).
			listeners.push(aspect.after(grid, "cleanup", function(){
				this._expanded = {};
			}));
		}
		
		grid._calcRowHeight = function(rowElement){
			// we override this method so we can provide row height measurements that
			// include the children of a row
			var connected = rowElement.connected;
			// if connected, need to consider this in the total row height
			return rowElement.offsetHeight + (connected ? connected.offsetHeight : 0); 
		};
		
		grid.expand = function(target, expand, noTransition){
			// summary:
			//		Expands the row corresponding to the given target.
			// target: Object
			//		Row object (or something resolvable to one) to expand/collapse.
			// expand: Boolean?
			//		If specified, designates whether to expand or collapse the row;
			//		if unspecified, toggles the current state.
			
			var row = target.element ? target : grid.row(target),
				hasTransitionend = has("transitionend");
			
			target = row.element;
			target = target.className.indexOf("dgrid-expando-icon") > -1 ? target :
				querySelector(".dgrid-expando-icon", target)[0];
			
			if(target && target.mayHaveChildren &&
					(noTransition || expand !== !!this._expanded[row.id])){
				// toggle or set expand/collapsed state based on optional 2nd argument
				var expanded = expand === undefined ? !this._expanded[row.id] : expand;
				
				// update the expando display
				put(target, ".ui-icon-triangle-1-" + (expanded ? "se" : "e") +
					"!ui-icon-triangle-1-" + (expanded ? "e" : "se"));
				
				var preloadNode = target.preloadNode,
					rowElement = row.element,
					container,
					containerStyle,
					scrollHeight,
					options = {
						originalQuery: this.query
					};
				
				if(!preloadNode){
					// if the children have not been created, create a container, a preload node and do the 
					// query for the children
					container = rowElement.connected = put('div.dgrid-tree-container');//put(rowElement, '+...
					preloadNode = target.preloadNode = put(rowElement, '+', container, 'div.dgrid-preload');
					var query = function(options){
						return grid.store.getChildren(row.data, options);
					};
					if(column.allowDuplicates){
						// If allowDuplicates is specified, include parentId in options
						// in order to facilitate unique IDs for each occurrence of the
						// same item under multiple different parents.
						options.parentId = row.id;
					}
					// Include level information on query for renderQuery case
					if("level" in target){
						query.level = target.level;
					}
					Deferred.when(
						grid.renderQuery ?
							grid._trackError(function(){
								return grid.renderQuery(query, preloadNode, options);
							}) :
							grid.renderArray(query(options), preloadNode,
								"level" in query ? { queryLevel: query.level } : {}),
						function(){
							// Expand once results are retrieved, if the row is still expanded.
							if(grid._expanded[row.id] && hasTransitionend){
								var scrollHeight = container.scrollHeight;
								container.style.height = scrollHeight ? scrollHeight + "px" : "auto";
							}
						}
					);
					
					if(hasTransitionend){
						on(container, hasTransitionend, ontransitionend);
					}else{
						ontransitionend.call(container);
					}
				}
				
				// Show or hide all the children.
				
				container = rowElement.connected;
				container.hidden = !expanded;
				containerStyle = container.style;
				
				// make sure it is visible so we can measure it
				if(!hasTransitionend || noTransition){
					containerStyle.display = expanded ? "block" : "none";
					containerStyle.height = "";
				}else{
					if(expanded){
						containerStyle.display = "block";
						scrollHeight = container.scrollHeight;
						containerStyle.height = "0px";
					}
					else{
						// if it will be hidden we need to be able to give a full height
						// without animating it, so it has the right starting point to animate to zero
						put(container, ".dgrid-tree-resetting");
						containerStyle.height = container.scrollHeight + "px";
					}
					// Perform a transition for the expand or collapse.
					setTimeout(function(){
						put(container, "!dgrid-tree-resetting");
						containerStyle.height =
							expanded ? (scrollHeight ? scrollHeight + "px" : "auto") : "0px";
					});
				}
				
				// Update _expanded map.
				if(expanded){
					this._expanded[row.id] = true;
				}else{
					delete this._expanded[row.id];
				}
			}
		}; // end function grid.expand
		
		// Set up a destroy function on column to tear down the listeners/aspects
		// established above if the grid's columns are redefined later.
		aspect.after(column, "destroy", function(){
			arrayUtil.forEach(listeners, function(l){ l.remove(); });
			// Delete methods we added/overrode on the instance.
			delete grid.expand;
			delete grid._calcRowHeight;
		});
	});
	
	column.renderCell = function(object, value, td, options){
		// summary:
		//		Renders a cell that can be expanded, creating more rows
		
		var grid = column.grid,
			level = Number(options && options.queryLevel) + 1,
			mayHaveChildren = !grid.store.mayHaveChildren || grid.store.mayHaveChildren(object),
			parentId = options.parentId,
			expando, node;
		
		level = currentLevel = isNaN(level) ? 0 : level;
		expando = column.renderExpando(level, mayHaveChildren,
			grid._expanded[(parentId ? parentId + "-" : "") + grid.store.getIdentity(object)], object);
		expando.level = level;
		expando.mayHaveChildren = mayHaveChildren;
		
		node = originalRenderCell.call(column, object, value, td, options);
		if(node && node.nodeType){
			put(td, expando);
			put(td, node);
		}else{
			td.insertBefore(expando, td.firstChild);
		}
	};
	return column;
}

tree.defaultRenderExpando = defaultRenderExpando;
return tree;
});

},
'dgrid/util/has-css3':function(){
define(["dojo/has", "xstyle/css!../css/has-transforms3d.css"],
function(has){
	// This module defines feature tests for CSS3 features such as transitions.
	// The css-transitions, css-transforms, and css-transforms3d has-features
	// can report either boolean or string:
	// * false indicates no support
	// * true indicates prefix-less support
	// * string indicates the vendor prefix under which the feature is supported

	var cssPrefixes = ["ms", "O", "Moz", "Webkit"];
	
	has.add("css-transitions", function(global, doc, element){
		var style = element.style,
			i;
		
		if(style.transitionProperty !== undefined){ // standard, no vendor prefix
			return true;
		}
		for (i = cssPrefixes.length; i--;) {
			if (style[cssPrefixes[i] + "TransitionProperty"] !== undefined) {
				return cssPrefixes[i]; // vendor-specific css property prefix
			}
		}
		
		return false; // otherwise, not supported
	});
	
	has.add("transitionend", function(){
		// Infer transitionend event name based on CSS transitions has-feature.
		var tpfx = has("css-transitions");
		if(!tpfx){ return false; }
		if(tpfx === true){ return "transitionend"; }
		return {
			ms: "MSTransitionEnd",
			O: "oTransitionEnd",
			Moz: "transitionend",
			Webkit: "webkitTransitionEnd"
		}[tpfx];
	});
	
	has.add("css-transforms", function(global, doc, element){
		var style = element.style, i;
		if (style.transformProperty !== undefined) {
			return true; // standard, no vendor prefix
		}
		for (i = cssPrefixes.length; i--;) {
			if (style[cssPrefixes[i] + "Transform"] !== undefined) {
				return cssPrefixes[i];
			}
		}
		
		return false; // otherwise, not supported
	});
	
	has.add("css-transforms3d", function(global, doc, element){
		var left, prefix;
		
		// Apply csstransforms3d class to test transform-3d media queries.
		element.className = "has-csstransforms3d";
		// Add to body to allow measurement.
		document.body.appendChild(element);
		left = element.offsetLeft;
		
		if (left === 9) {
			return true; // standard, no prefix
		} else if (left > 9){
			// Matched one of the vendor prefixes; offset indicates which.
			prefix = cssPrefixes[left - 10];
			return prefix || false;
		}
		document.body.removeChild(element);
		element.className = "";
		
		return false; // otherwise, not supported
	});
	
	return has;
});

},
'xstyle/css':function(){
define(["require"], function(moduleRequire){
"use strict";
var cssCache = window.cssCache || (window.cssCache = {});
/*
 * RequireJS css! plugin
 * This plugin will load and wait for css files.  This could be handy when
 * loading css files as part of a layer or as a way to apply a run-time theme. This
 * module checks to see if the CSS is already loaded before incurring the cost
 * of loading the full CSS loader codebase
 */
/* 	function search(tag, href){
		var elements = document.getElementsByTagName(tag);
		for(var i = 0; i < elements.length; i++){
			var element = elements[i];
			var sheet = alreadyLoaded(element.sheet || element.styleSheet, href)
			if(sheet){
				return sheet;
			}
		}
	}
	function alreadyLoaded(sheet, href){
		if(sheet){
			var importRules = sheet.imports || sheet.rules || sheet.cssRules;
			for(var i = 0; i < importRules.length; i++){								
				var importRule = importRules[i];
				if(importRule.href){
					sheet = importRule.styleSheet || importRule;
					if(importRule.href == href){
						return sheet;
					}
					sheet = alreadyLoaded(sheet, href);
					if(sheet){
						return sheet;
					}
				}
			}
		}
	}
	function nameWithExt (name, defaultExt) {
		return name.lastIndexOf('.') <= name.lastIndexOf('/') ?
			name + '.' + defaultExt : name;
	}*/
 	return {
		load: function (resourceDef, require, callback, config) {
			var url = require.toUrl(resourceDef);
			if(cssCache[url]){
				return createStyleSheet(cssCache[url]);
			}
/*			var cssIdTest = resourceDef.match(/(.+)\?(.+)/);
			if(cssIdTest){*/
				// if there is an id test available, see if the referenced rule is already loaded,
				// and if so we can completely avoid any dynamic CSS loading. If it is
				// not present, we need to use the dynamic CSS loader.
				var docElement = document.documentElement;
				var testDiv = docElement.insertBefore(document.createElement('div'), docElement.firstChild);
				testDiv.id = require.toAbsMid(resourceDef).replace(/\//g,'-').replace(/\..*/,'') + "-loaded";  //cssIdTest[2];
				var displayStyle = (testDiv.currentStyle || getComputedStyle(testDiv, null)).display;
				docElement.removeChild(testDiv);
				if(displayStyle == "none"){
					return callback();
				}
				//resourceDef = cssIdTest[1];
			//}
			// use dynamic loader
			/*if(search("link", url) || search("style", url)){
				callback();
			}else{*/
			moduleRequire(["./load-css"], function(load){
				load(url, callback);
			});
		},
		pluginBuilder: "xstyle/css-builder"

	};
});

},
'dgrid/Grid':function(){
define(["dojo/_base/kernel", "dojo/_base/declare", "dojo/on", "dojo/has", "put-selector/put", "./List", "./util/misc", "dojo/_base/sniff"],
function(kernel, declare, listen, has, put, List, miscUtil){
	var contentBoxSizing = has("ie") < 8 && !has("quirks");
	var invalidClassChars = /[^\._a-zA-Z0-9-]/g;
	function appendIfNode(parent, subNode){
		if(subNode && subNode.nodeType){
			parent.appendChild(subNode);
		}
	}
	
	var Grid = declare(List, {
		columns: null,
		// cellNavigation: Boolean
		//		This indicates that focus is at the cell level. This may be set to false to cause
		//		focus to be at the row level, which is useful if you want only want row-level
		//		navigation.
		cellNavigation: true,
		tabableHeader: true,
		showHeader: true,
		column: function(target){
			// summary:
			//		Get the column object by node, or event, or a columnId
			if(typeof target != "object"){
				return this.columns[target];
			}else{
				return this.cell(target).column;
			}
		},
		listType: "grid",
		cell: function(target, columnId){
			// summary:
			//		Get the cell object by node, or event, id, plus a columnId
			
			if(target.column && target.element){ return target; }
			
			if(target.target && target.target.nodeType){
				// event
				target = target.target;
			}
			var element;
			if(target.nodeType){
				var object;
				do{
					if(this._rowIdToObject[target.id]){
						break;
					}
					var colId = target.columnId;
					if(colId){
						columnId = colId;
						element = target;
						break;
					}
					target = target.parentNode;
				}while(target && target != this.domNode);
			}
			if(!element && typeof columnId != "undefined"){
				var row = this.row(target),
					rowElement = row && row.element;
				if(rowElement){
					var elements = rowElement.getElementsByTagName("td");
					for(var i = 0; i < elements.length; i++){
						if(elements[i].columnId == columnId){
							element = elements[i];
							break;
						}
					}
				}
			}
			if(target != null){
				return {
					row: row || this.row(target),
					column: columnId && this.column(columnId),
					element: element
				};
			}
		},
		
		createRowCells: function(tag, each, subRows){
			// summary:
			//		Generates the grid for each row (used by renderHeader and and renderRow)
			var row = put("table.dgrid-row-table[role=presentation]"),
				cellNavigation = this.cellNavigation,
				// IE < 9 needs an explicit tbody; other browsers do not
				tbody = (has("ie") < 9 || has("quirks")) ? put(row, "tbody") : row,
				tr,
				si, sl, i, l, // iterators
				subRow, column, id, extraClassName, cell, innerCell, colSpan, rowSpan; // used inside loops
			
			// Allow specification of custom/specific subRows, falling back to
			// those defined on the instance.
			subRows = subRows || this.subRows;
			
			for(si = 0, sl = subRows.length; si < sl; si++){
				subRow = subRows[si];
				// for single-subrow cases in modern browsers, TR can be skipped
				// http://jsperf.com/table-without-trs
				tr = put(tbody, "tr");
				if(subRow.className){
					put(tr, "." + subRow.className);
				}
				
				for(i = 0, l = subRow.length; i < l; i++){
					// iterate through the columns
					column = subRow[i];
					id = column.id;
					extraClassName = column.className || (column.field && "field-" + column.field);
					cell = put(tag + (
							".dgrid-cell.dgrid-cell-padding" +
							(id ? ".dgrid-column-" + id : "") +
							(extraClassName ? "." + extraClassName : "")
						).replace(invalidClassChars,"-") +
						"[role=" + (tag === "th" ? "columnheader" : "gridcell") + "]");
					cell.columnId = id;
					if(contentBoxSizing){
						// The browser (IE7-) does not support box-sizing: border-box, so we emulate it with a padding div
						innerCell = put(cell, "!dgrid-cell-padding div.dgrid-cell-padding");// remove the dgrid-cell-padding, and create a child with that class
						cell.contents = innerCell;
					}else{
						innerCell = cell;
					}
					colSpan = column.colSpan;
					if(colSpan){
						cell.colSpan = colSpan;
					}
					rowSpan = column.rowSpan;
					if(rowSpan){
						cell.rowSpan = rowSpan;
					}
					each(innerCell, column);
					// add the td to the tr at the end for better performance
					tr.appendChild(cell);
				}
			}
			return row;
		},
		
		left: function(cell, steps){
			if(!cell.element){ cell = this.cell(cell); }
			return this.cell(this._move(cell, -(steps || 1), "dgrid-cell"));
		},
		right: function(cell, steps){
			if(!cell.element){ cell = this.cell(cell); }
			return this.cell(this._move(cell, steps || 1, "dgrid-cell"));
		},
		
		renderRow: function(object, options){
			var self = this;
			var row = this.createRowCells("td", function(td, column){
				var data = object;
				// Support get function or field property (similar to DataGrid)
				if(column.get){
					data = column.get(object);
				}else if("field" in column && column.field != "_item"){
					data = data[column.field];
				}
				
				if(column.renderCell){
					// A column can provide a renderCell method to do its own DOM manipulation,
					// event handling, etc.
					appendIfNode(td, column.renderCell(object, data, td, options));
				}else{
					defaultRenderCell.call(column, object, data, td, options);
				}
			}, options && options.subRows);
			// row gets a wrapper div for a couple reasons:
			//	1. So that one can set a fixed height on rows (heights can't be set on <table>'s AFAICT)
			// 2. So that outline style can be set on a row when it is focused, and Safari's outline style is broken on <table>
			return put("div[role=row]>", row);
		},
		renderHeader: function(){
			// summary:
			//		Setup the headers for the grid
			var
				grid = this,
				columns = this.columns,
				headerNode = this.headerNode,
				i = headerNode.childNodes.length;
			
			headerNode.setAttribute("role", "row");
			
			// clear out existing header in case we're resetting
			while(i--){
				put(headerNode.childNodes[i], "!");
			}
			
			var row = this.createRowCells("th", function(th, column){
				var contentNode = column.headerNode = th;
				if(contentBoxSizing){
					// we're interested in the th, but we're passed the inner div
					th = th.parentNode;
				}
				var field = column.field;
				if(field){
					th.field = field;
				}
				// allow for custom header content manipulation
				if(column.renderHeaderCell){
					appendIfNode(contentNode, column.renderHeaderCell(contentNode));
				}else if("label" in column || column.field){
					contentNode.appendChild(document.createTextNode(
						"label" in column ? column.label : column.field));
				}
				if(column.sortable !== false && field && field != "_item"){
					th.sortable = true;
					th.className += " dgrid-sortable";
				}
			}, this.subRows && this.subRows.headerRows);
			this._rowIdToObject[row.id = this.id + "-header"] = this.columns;
			headerNode.appendChild(row);
			
			// If the columns are sortable, re-sort on clicks.
			// Use a separate listener property to be managed by renderHeader in case
			// of subsequent calls.
			if(this._sortListener){
				this._sortListener.remove();
			}
			this._sortListener = listen(row, "click,keydown", function(event){
				// respond to click, space keypress, or enter keypress
				if(event.type == "click" || event.keyCode == 32 /* space bar */ || (!has("opera") && event.keyCode == 13) /* enter */){
					var target = event.target,
						field, sort, newSort, eventObj;
					do{
						if(target.sortable){
							// If the click is on the same column as the active sort,
							// reverse sort direction
							newSort = [{
								attribute: (field = target.field || target.columnId),
								descending: (sort = grid._sort[0]) && sort.attribute == field &&
									!sort.descending
							}];
							
							// Emit an event with the new sort
							eventObj = {
								bubbles: true,
								cancelable: true,
								grid: grid,
								parentType: event.type,
								sort: newSort
							};
							
							if (listen.emit(event.target, "dgrid-sort", eventObj)){
								// Stash node subject to DOM manipulations,
								// to be referenced then removed by sort()
								grid._sortNode = target;
								grid.set("sort", newSort);
							}
							
							break;
						}
					}while((target = target.parentNode) && target != headerNode);
				}
			});
		},
		
		resize: function(){
			// extension of List.resize to allow accounting for
			// column sizes larger than actual grid area
			var
				headerTableNode = this.headerNode.firstChild,
				contentNode = this.contentNode,
				width;
			
			this.inherited(arguments);
			
			if(!has("ie") || (has("ie") > 7 && !has("quirks"))){
				// Force contentNode width to match up with header width.
				// (Old IEs don't have a problem due to how they layout.)
				
				contentNode.style.width = ""; // reset first
				
				if(contentNode && headerTableNode){
					if((width = headerTableNode.offsetWidth) != contentNode.offsetWidth){
						// update size of content node if necessary (to match size of rows)
						// (if headerTableNode can't be found, there isn't much we can do)
						contentNode.style.width = width + "px";
					}
				}
			}
		},
		
		destroy: function(){
			// Run _destroyColumns first to perform any column plugin tear-down logic.
			this._destroyColumns();
			if(this._sortListener){
				this._sortListener.remove();
			}
			
			this.inherited(arguments);
		},
		
		_setSort: function(property, descending){
			// summary:
			//		Extension of List.js sort to update sort arrow in UI
			
			// Normalize _sort first via inherited logic, then update the sort arrow
			this.inherited(arguments);
			this.updateSortArrow(this._sort);
		},
		
		updateSortArrow: function(sort, updateSort){
			// summary:
			//		Method responsible for updating the placement of the arrow in the
			//		appropriate header cell.  Typically this should not be called (call
			//		set("sort", ...) when actually updating sort programmatically), but
			//		this method may be used by code which is customizing sort (e.g.
			//		by reacting to the dgrid-sort event, canceling it, then
			//		performing logic and calling this manually).
			// sort: Array
			//		Standard sort parameter - array of object(s) containing attribute
			//		and optionally descending property
			// updateSort: Boolean?
			//		If true, will update this._sort based on the passed sort array
			//		(i.e. to keep it in sync when custom logic is otherwise preventing
			//		it from being updated); defaults to false
			
			// Clean up UI from any previous sort
			if(this._lastSortedArrow){
				// Remove the sort classes from the parent node
				put(this._lastSortedArrow, "<!dgrid-sort-up!dgrid-sort-down");
				// Destroy the lastSortedArrow node
				put(this._lastSortedArrow, "!");
				delete this._lastSortedArrow;
			}
			
			if(updateSort){ this._sort = sort; }
			if(!sort[0]){ return; } // nothing to do if no sort is specified
			
			var prop = sort[0].attribute,
				desc = sort[0].descending,
				target = this._sortNode, // stashed if invoked from header click
				columns, column, i;
			
			delete this._sortNode;
			
			if(!target){
				columns = this.columns;
				for(i in columns){
					column = columns[i];
					if(column.field == prop){
						target = column.headerNode;
						break;
					}
				}
			}
			// skip this logic if field being sorted isn't actually displayed
			if(target){
				target = target.contents || target;
				// place sort arrow under clicked node, and add up/down sort class
				this._lastSortedArrow = put(target.firstChild, "-div.dgrid-sort-arrow.ui-icon[role=presentation]");
				this._lastSortedArrow.innerHTML = "&nbsp;";
				put(target, desc ? ".dgrid-sort-down" : ".dgrid-sort-up");
				// call resize in case relocation of sort arrow caused any height changes
				this.resize();
			}
		},
		
		styleColumn: function(colId, css){
			// summary:
			//		Dynamically creates a stylesheet rule to alter a column's style.
			
			return this.addCssRule("#" + miscUtil.escapeCssIdentifier(this.domNode.id) +
				" .dgrid-column-" + colId, css);
		},
		
		/*=====
		_configColumn: function(column, columnId, rowColumns, prefix){
			// summary:
			//		Method called when normalizing base configuration of a single
			//		column.  Can be used as an extension point for behavior requiring
			//		access to columns when a new configuration is applied.
		},=====*/
		
		_configColumns: function(prefix, rowColumns){
			// configure the current column
			var subRow = [],
				isArray = rowColumns instanceof Array;
			
			function configColumn(column, columnId){
				if(typeof column == "string"){
					rowColumns[columnId] = column = {label:column};
				}
				if(!isArray && !column.field){
					column.field = columnId;
				}
				columnId = column.id = column.id || (isNaN(columnId) ? columnId : (prefix + columnId));
				if(isArray){ this.columns[columnId] = column; }
				
				// allow further base configuration in subclasses
				if(this._configColumn){
					this._configColumn(column, columnId, rowColumns, prefix);
				}
				
				// add grid reference to each column object for potential use by plugins
				column.grid = this;
				if(typeof column.init === "function"){ column.init(); }
				
				subRow.push(column); // make sure it can be iterated on
			}
			
			miscUtil.each(rowColumns, configColumn, this);
			return isArray ? rowColumns : subRow;
		},
		
		_destroyColumns: function(){
			// summary:
			//		Iterates existing subRows looking for any column definitions with
			//		destroy methods (defined by plugins) and calls them.  This is called
			//		immediately before configuring a new column structure.
			
			var subRows = this.subRows,
				// if we have column sets, then we don't need to do anything with the missing subRows, ColumnSet will handle it
				subRowsLength = subRows && subRows.length,
				i, j, column, len;
			
			// First remove rows (since they'll be refreshed after we're done),
			// so that anything aspected onto removeRow by plugins can run.
			// (cleanup will end up running again, but with nothing to iterate.)
			this.cleanup();
			
			for(i = 0; i < subRowsLength; i++){
				for(j = 0, len = subRows[i].length; j < len; j++){
					column = subRows[i][j];
					if(typeof column.destroy === "function"){ column.destroy(); }
				}
			}
		},
		
		configStructure: function(){
			// configure the columns and subRows
			var subRows = this.subRows,
				columns = this._columns = this.columns;
			
			// Reset this.columns unless it was already passed in as an object
			this.columns = !columns || columns instanceof Array ? {} : columns;
			
			if(subRows){
				// Process subrows, which will in turn populate the this.columns object
				for(var i = 0; i < subRows.length; i++){
					subRows[i] = this._configColumns(i + "-", subRows[i]);
				}
			}else{
				this.subRows = [this._configColumns("", columns)];
			}
		},
		
		_getColumns: function(){
			// _columns preserves what was passed to set("columns"), but if subRows
			// was set instead, columns contains the "object-ified" version, which
			// was always accessible in the past, so maintain that accessibility going
			// forward.
			return this._columns || this.columns;
		},
		_setColumns: function(columns){
			this._destroyColumns();
			// reset instance variables
			this.subRows = null;
			this.columns = columns;
			// re-run logic
			this._updateColumns();
		},
		
		_setSubRows: function(subrows){
			this._destroyColumns();
			this.subRows = subrows;
			this._updateColumns();
		},
		
		setColumns: function(columns){
			kernel.deprecated("setColumns(...)", 'use set("columns", ...) instead', "dgrid 0.4");
			this.set("columns", columns);
		},
		setSubRows: function(subrows){
			kernel.deprecated("setSubRows(...)", 'use set("subRows", ...) instead', "dgrid 0.4");
			this.set("subRows", subrows);
		},
		
		_updateColumns: function(){
			// summary:
			//		Called when columns, subRows, or columnSets are reset
			
			this.configStructure();
			this.renderHeader();
			
			this.refresh();
			// re-render last collection if present
			this._lastCollection && this.renderArray(this._lastCollection);
			
			// After re-rendering the header, re-apply the sort arrow if needed.
			if(this._started){
				if(this._sort && this._sort.length){
					this.updateSortArrow(this._sort);
				} else {
					// Only call resize directly if we didn't call updateSortArrow,
					// since that calls resize itself when it updates.
					this.resize();
				}
			}
		}
	});
	
	function defaultRenderCell(object, data, td, options){
		if(this.formatter){
			// Support formatter, with or without formatterScope
			var formatter = this.formatter,
				formatterScope = this.grid.formatterScope;
			td.innerHTML = typeof formatter === "string" && formatterScope ?
				formatterScope[formatter](data, object) : formatter(data, object);
		}else if(data != null){
			td.appendChild(document.createTextNode(data)); 
		}
	}
	
	// expose appendIfNode and default implementation of renderCell,
	// e.g. for use by column plugins
	Grid.appendIfNode = appendIfNode;
	Grid.defaultRenderCell = defaultRenderCell;
	
	return Grid;
});

},
'put-selector/put':function(){
(function(define){
var forDocument, fragmentFasterHeuristic = /[-+,> ]/; // if it has any of these combinators, it is probably going to be faster with a document fragment 
define([], forDocument = function(doc, newFragmentFasterHeuristic){
"use strict";
	// module:
	//		put-selector/put
	// summary:
	//		This module defines a fast lightweight function for updating and creating new elements
	//		terse, CSS selector-based syntax. The single function from this module creates
	// 		new DOM elements and updates existing elements. See README.md for more information.
	//	examples:
	//		To create a simple div with a class name of "foo":
	//		|	put("div.foo");
	fragmentFasterHeuristic = newFragmentFasterHeuristic || fragmentFasterHeuristic;
	var selectorParse = /(?:\s*([-+ ,<>]))?\s*(\.|!\.?|#)?([-\w%$]+)?(?:\[([^\]=]+)=?['"]?([^\]'"]*)['"]?\])?/g,
		undefined,
		doc = doc || document,
		ieCreateElement = typeof doc.createElement == "object"; // telltale sign of the old IE behavior with createElement that does not support later addition of name 
	function insertTextNode(element, text){
		element.appendChild(doc.createTextNode(text));
	}
	function put(topReferenceElement){
		var fragment, lastSelectorArg, nextSibling, referenceElement, current,
			args = arguments,
			returnValue = args[0]; // use the first argument as the default return value in case only an element is passed in
		function insertLastElement(){
			// we perform insertBefore actions after the element is fully created to work properly with 
			// <input> tags in older versions of IE that require type attributes
			//	to be set before it is attached to a parent.
			// We also handle top level as a document fragment actions in a complex creation 
			// are done on a detached DOM which is much faster
			// Also if there is a parse error, we generally error out before doing any DOM operations (more atomic) 
			if(current && referenceElement && current != referenceElement){
				(referenceElement == topReferenceElement &&
					// top level, may use fragment for faster access 
					(fragment || 
						// fragment doesn't exist yet, check to see if we really want to create it 
						(fragment = fragmentFasterHeuristic.test(argument) && doc.createDocumentFragment()))
							// any of the above fails just use the referenceElement  
							|| referenceElement).
								insertBefore(current, nextSibling || null); // do the actual insertion
			}
		}
		for(var i = 0; i < args.length; i++){
			var argument = args[i];
			if(typeof argument == "object"){
				lastSelectorArg = false;
				if(argument instanceof Array){
					// an array
					current = doc.createDocumentFragment();
					for(var key = 0; key < argument.length; key++){
						current.appendChild(put(argument[key]));
					}
					argument = current;
				}
				if(argument.nodeType){
					current = argument;
					insertLastElement();
					referenceElement = argument;
					nextSibling = 0;
				}else{
					// an object hash
					for(var key in argument){
						current[key] = argument[key];
					}				
				}
			}else if(lastSelectorArg){
				// a text node should be created
				// take a scalar value, use createTextNode so it is properly escaped
				// createTextNode is generally several times faster than doing an escaped innerHTML insertion: http://jsperf.com/createtextnode-vs-innerhtml/2
				lastSelectorArg = false;
				insertTextNode(current, argument);
			}else{
				if(i < 1){
					// if we are starting with a selector, there is no top element
					topReferenceElement = null;
				}
				lastSelectorArg = true;
				var leftoverCharacters = argument.replace(selectorParse, function(t, combinator, prefix, value, attrName, attrValue){
					if(combinator){
						// insert the last current object
						insertLastElement();
						if(combinator == '-' || combinator == '+'){
							// + or - combinator, 
							// TODO: add support for >- as a means of indicating before the first child?
							referenceElement = (nextSibling = (current || referenceElement)).parentNode;
							current = null;
							if(combinator == "+"){
								nextSibling = nextSibling.nextSibling;
							}// else a - operator, again not in CSS, but obvious in it's meaning (create next element before the current/referenceElement)
						}else{
							if(combinator == "<"){
								// parent combinator (not really in CSS, but theorized, and obvious in it's meaning)
								referenceElement = current = (current || referenceElement).parentNode;
							}else{
								if(combinator == ","){
									// comma combinator, start a new selector
									referenceElement = topReferenceElement;
								}else if(current){
									// else descendent or child selector (doesn't matter, treated the same),
									referenceElement = current;
								}
								current = null;
							}
							nextSibling = 0;
						}
						if(current){
							referenceElement = current;
						}
					}
					var tag = !prefix && value;
					if(tag || (!current && (prefix || attrName))){
						if(tag == "$"){
							// this is a variable to be replaced with a text node
							insertTextNode(referenceElement, args[++i]);
						}else{
							// Need to create an element
							tag = tag || put.defaultTag;
							var ieInputName = ieCreateElement && args[i +1] && args[i +1].name;
							if(ieInputName){
								// in IE, we have to use the crazy non-standard createElement to create input's that have a name 
								tag = '<' + tag + ' name="' + ieInputName + '">';
							}
							current = doc.createElement(tag);
						}
					}
					if(prefix){
						if(value == "$"){
							value = args[++i];
						}
						if(prefix == "#"){
							// #id was specified
							current.id = value;
						}else{
							// we are in the className addition and removal branch
							var currentClassName = current.className;
							// remove the className (needed for addition or removal)
							// see http://jsperf.com/remove-class-name-algorithm/2 for some tests on this
							var removed = currentClassName && (" " + currentClassName + " ").replace(" " + value + " ", " ");
							if(prefix == "."){
								// addition, add the className
								current.className = currentClassName ? (removed + value).substring(1) : value;
							}else{
								// else a '!' class removal
								if(argument == "!"){
									// special signal to delete this element
									// use the ol' innerHTML trick to get IE to do some cleanup
									put("div", current, '<').innerHTML = "";
								}else{
									// we already have removed the class, just need to trim
									removed = removed.substring(1, removed.length - 1);
									// only assign if it changed, this can save a lot of time
									if(removed != currentClassName){
										current.className = removed;
									}
								}
							}
							// CSS class removal
						}
					}
					if(attrName){
						if(attrValue == "$"){
							attrValue = args[++i];
						}
						// [name=value]
						if(attrName == "style"){
							// handle the special case of setAttribute not working in old IE
							current.style.cssText = attrValue;
						}else{
							current[attrName.charAt(0) == "!" ? (attrName = attrName.substring(1)) && 'removeAttribute' : 'setAttribute'](attrName, attrValue === '' ? attrName : attrValue);
						}
					}
					return '';
				});
				if(leftoverCharacters){
					throw new SyntaxError("Unexpected char " + leftoverCharacters + " in " + argument);
				}
				insertLastElement();
				referenceElement = returnValue = current || referenceElement;
			}
		}
		if(topReferenceElement && fragment){
			// we now insert the top level elements for the fragment if it exists
			topReferenceElement.appendChild(fragment);
		}
		return returnValue;
	}
	put.defaultTag = "div";
	put.forDocument = forDocument;
	return put;
});
})(typeof define == "undefined" ? function(deps, factory){
	if(typeof window == "undefined"){
		// server side JavaScript, probably (hopefully) NodeJS
		require("./node-html")(module, factory);
	}else{
		// plain script in a browser
		put = factory();
	}
} : define);

},
'dgrid/List':function(){
define(["dojo/_base/kernel", "dojo/_base/declare", "dojo/on", "dojo/has", "./util/misc", "dojo/has!touch?./TouchScroll", "xstyle/has-class", "put-selector/put", "dojo/_base/sniff", "xstyle/css!./css/dgrid.css"], 
function(kernel, declare, listen, has, miscUtil, TouchScroll, hasClass, put){
	// Add user agent/feature CSS classes 
	hasClass("mozilla", "opera", "webkit", "ie", "ie-6", "ie-6-7", "quirks", "no-quirks", "touch");
	
	var oddClass = "dgrid-row-odd",
		evenClass = "dgrid-row-even",
		scrollbarWidth, scrollbarHeight;
	
	function byId(id){
		return document.getElementById(id);
	}
	
	function getScrollbarSize(node, dimension){
		// Used by has tests for scrollbar width/height
		var body = document.body,
			size;
		
		put(body, node, ".dgrid-scrollbar-measure");
		size = node["offset" + dimension] - node["client" + dimension];
		
		put(node, "!dgrid-scrollbar-measure");
		body.removeChild(node);
		
		return size;
	}
	has.add("dom-scrollbar-width", function(global, doc, element){
		return getScrollbarSize(element, "Width");
	});
	has.add("dom-scrollbar-height", function(global, doc, element){
		return getScrollbarSize(element, "Height");
	});
	
	// var and function for autogenerating ID when one isn't provided
	var autogen = 0;
	function generateId(){
		return "dgrid_" + autogen++;
	}
	
	// common functions for class and className setters/getters
	// (these are run in instance context)
	var spaceRx = / +/g;
	function setClass(cls){
		// Format input appropriately for use with put...
		var putClass = cls ? "." + cls.replace(spaceRx, ".") : "";
		
		// Remove any old classes, and add new ones.
		if(this._class){
			putClass = "!" + this._class.replace(spaceRx, "!") + putClass;
		}
		put(this.domNode, putClass);
		
		// Store for later retrieval/removal.
		this._class = cls;
	}
	function getClass(){
		return this._class;
	}
	
	// window resize event handler, run in context of List instance
	var winResizeHandler = has("ie") < 7 && !has("quirks") ? function(){
		// IE6 triggers window.resize on any element resize;
		// avoid useless calls (and infinite loop if height: auto).
		// The measurement logic here is based on dojo/window logic.
		var root, w, h, dims;
		
		if(!this._started){ return; } // no sense calling resize yet
		
		root = document.documentElement;
		w = root.clientWidth;
		h = root.clientHeight;
		dims = this._prevWinDims || [];
		if(dims[0] !== w || dims[1] !== h){
			this.resize();
			this._prevWinDims = [w, h];
		}
	} :
	function(){
		if(this._started){ this.resize(); }
	};
	
	// Desktop versions of functions, deferred to when there is no touch support,
	// or when the useTouchScroll instance property is set to false
	
	function desktopGetScrollPosition(){
		return {
			x: this.bodyNode.scrollLeft,
			y: this.bodyNode.scrollTop
		};
	}
	
	function desktopScrollTo(options){
		if(typeof options.x !== "undefined"){
			this.bodyNode.scrollLeft = options.x;
		}
		if(typeof options.y !== "undefined"){
			this.bodyNode.scrollTop = options.y;
		}
	}
	
	return declare(has("touch") ? TouchScroll : null, {
		tabableHeader: false,
		// showHeader: Boolean
		//		Whether to render header (sub)rows.
		showHeader: false,
		
		// showFooter: Boolean
		//		Whether to render footer area.  Extensions which display content
		//		in the footer area should set this to true.
		showFooter: false,
		
		// maintainOddEven: Boolean
		//		Whether to maintain the odd/even classes when new rows are inserted.
		//		This can be disabled to improve insertion performance if odd/even styling is not employed.
		maintainOddEven: true,
		
		// cleanAddedRules: Boolean
		//		Whether to track rules added via the addCssRule method to be removed
		//		when the list is destroyed.  Note this is effective at the time of
		//		the call to addCssRule, not at the time of destruction.
		cleanAddedRules: true,
		
		// useTouchScroll: Boolean
		//		If touch support is available, this determines whether to
		//		incorporate logic from the TouchScroll module (at the expense of
		//		normal desktop/mouse or native mobile scrolling functionality).
		useTouchScroll: true,

		// cleanEmptyObservers: Boolean
		//		Whether to clean up observers for empty result sets.
		cleanEmptyObservers: true,

		// highlightDuration: Integer
		//		The amount of time (in milliseconds) that a row should remain
		//		highlighted after it has been updated.
		highlightDuration: 250,
		
		postscript: function(params, srcNodeRef){
			// perform setup and invoke create in postScript to allow descendants to
			// perform logic before create/postCreate happen (a la dijit/_WidgetBase)
			var grid = this;
			
			(this._Row = function(id, object, element){
				this.id = id;
				this.data = object;
				this.element = element;
			}).prototype.remove = function(){
				grid.removeRow(this.element);
			};
			
			if(srcNodeRef){
				// normalize srcNodeRef and store on instance during create process.
				// Doing this in postscript is a bit earlier than dijit would do it,
				// but allows subclasses to access it pre-normalized during create.
				this.srcNodeRef = srcNodeRef =
					srcNodeRef.nodeType ? srcNodeRef : byId(srcNodeRef);
			}
			this.create(params, srcNodeRef);
		},
		listType: "list",
		
		create: function(params, srcNodeRef){
			var domNode = this.domNode = srcNodeRef || put("div"),
				cls;
			
			if(params){
				this.params = params;
				declare.safeMixin(this, params);
				
				// Check for initial class or className in params or on domNode
				cls = params["class"] || params.className || domNode.className;
				
				// handle sort param - TODO: revise @ 0.4 when _sort -> sort
				this._sort = params.sort || [];
				delete this.sort; // ensure back-compat method isn't shadowed
			}else{
				this._sort = [];
			}
			
			// ensure arrays and hashes are initialized
			this.observers = [];
			this._numObservers = 0;
			this._listeners = [];
			this._rowIdToObject = {};
			
			this.postMixInProperties && this.postMixInProperties();
			
			// Apply id to widget and domNode,
			// from incoming node, widget params, or autogenerated.
			this.id = domNode.id = domNode.id || this.id || generateId();
			
			// Perform initial rendering, and apply classes if any were specified.
			this.buildRendering();
			if(cls){ setClass.call(this, cls); }
			
			this.postCreate();
			
			// remove srcNodeRef instance property post-create
			delete this.srcNodeRef;
			// to preserve "it just works" behavior, call startup if we're visible
			if(this.domNode.offsetHeight){
				this.startup();
			}
		},
		buildRendering: function(){
			var domNode = this.domNode,
				self = this,
				headerNode, spacerNode, bodyNode, footerNode, isRTL;
			
			// Detect RTL on html/body nodes; taken from dojo/dom-geometry
			isRTL = this.isRTL = (document.body.dir || document.documentElement.dir ||
				document.body.style.direction).toLowerCase() == "rtl";
			
			// Clear out className (any pre-applied classes will be re-applied via the
			// class / className setter), then apply standard classes/attributes
			domNode.className = "";
			
			put(domNode, "[role=grid].ui-widget.dgrid.dgrid-" + this.listType);
			
			// Place header node (initially hidden if showHeader is false).
			headerNode = this.headerNode = put(domNode, 
				"div.dgrid-header.dgrid-header-row.ui-widget-header" +
				(this.showHeader ? "" : ".dgrid-header-hidden"));
			if(has("quirks") || has("ie") < 8){
				spacerNode = put(domNode, "div.dgrid-spacer");
			}
			bodyNode = this.bodyNode = put(domNode, "div.dgrid-scroller");
			
			// firefox 4 until at least 10 adds overflow: auto elements to the tab index by default for some
			// reason; force them to be not tabbable
			bodyNode.tabIndex = -1;
			
			this.headerScrollNode = put(domNode, "div.dgrid-header-scroll.dgrid-scrollbar-width.ui-widget-header");
			
			// Place footer node (initially hidden if showFooter is false).
			footerNode = this.footerNode = put("div.dgrid-footer" +
				(this.showFooter ? "" : ".dgrid-footer-hidden"));
			put(domNode, footerNode);
			
			if(isRTL){
				domNode.className += " dgrid-rtl" + (has("webkit") ? "" : " dgrid-rtl-nonwebkit");
			}
			
			listen(bodyNode, "scroll", function(event){
				if(self.showHeader){
					// keep the header aligned with the body
					headerNode.scrollLeft = event.scrollLeft || bodyNode.scrollLeft;
				}
				// re-fire, since browsers are not consistent about propagation here
				event.stopPropagation();
				listen.emit(domNode, "scroll", {scrollTarget: bodyNode});
			});
			this.configStructure();
			this.renderHeader();
			
			this.contentNode = this.touchNode = put(this.bodyNode, "div.dgrid-content.ui-widget-content");
			// add window resize handler, with reference for later removal if needed
			this._listeners.push(this._resizeHandle = listen(window, "resize",
				miscUtil.throttleDelayed(winResizeHandler, this)));
		},
		
		postCreate: has("touch") ? function(){
			if(this.useTouchScroll){
				this.inherited(arguments);
			}
		} : function(){},
		
		startup: function(){
			// summary:
			//		Called automatically after postCreate if the component is already
			//		visible; otherwise, should be called manually once placed.
			
			if(this._started){ return; } // prevent double-triggering
			this.inherited(arguments);
			this._started = true;
			this.resize();
			// apply sort (and refresh) now that we're ready to render
			this.set("sort", this._sort);
		},
		
		configStructure: function(){
			// does nothing in List, this is more of a hook for the Grid
		},
		resize: function(){
			var
				bodyNode = this.bodyNode,
				headerNode = this.headerNode,
				footerNode = this.footerNode,
				headerHeight = headerNode.offsetHeight,
				footerHeight = this.showFooter ? footerNode.offsetHeight : 0,
				quirks = has("quirks") || has("ie") < 7;
			
			this.headerScrollNode.style.height = bodyNode.style.marginTop = headerHeight + "px";
			bodyNode.style.marginBottom = footerHeight + "px";
			
			if(quirks){
				// in IE6 and quirks mode, the "bottom" CSS property is ignored.
				// We guard against negative values in case of issues with external CSS.
				bodyNode.style.height = ""; // reset first
				bodyNode.style.height =
					Math.max((this.domNode.offsetHeight - headerHeight - footerHeight), 0) + "px";
				if (footerHeight) {
					// Work around additional glitch where IE 6 / quirks fails to update
					// the position of the bottom-aligned footer; this jogs its memory.
					footerNode.style.bottom = '1px';
					setTimeout(function(){ footerNode.style.bottom = ''; }, 0);
				}
			}
			
			if(!scrollbarWidth){
				// Measure the browser's scrollbar width using a DIV we'll delete right away
				scrollbarWidth = has("dom-scrollbar-width");
				scrollbarHeight = has("dom-scrollbar-height");
				
				// Avoid issues with certain widgets inside in IE7, and
				// ColumnSet scroll issues with all supported IE versions
				if(has("ie")){
					scrollbarWidth++;
					scrollbarHeight++;
				}
				
				// add rules that can be used where scrollbar width/height is needed
				miscUtil.addCssRule(".dgrid-scrollbar-width", "width: " + scrollbarWidth + "px");
				miscUtil.addCssRule(".dgrid-scrollbar-height", "height: " + scrollbarHeight + "px");
				
				if(scrollbarWidth != 17 && !quirks){
					// for modern browsers, we can perform a one-time operation which adds
					// a rule to account for scrollbar width in all grid headers.
					miscUtil.addCssRule(".dgrid-header", "right: " + scrollbarWidth + "px");
					// add another for RTL grids
					miscUtil.addCssRule(".dgrid-rtl-nonwebkit .dgrid-header", "left: " + scrollbarWidth + "px");
				}
			}
			
			if(quirks){
				// old IE doesn't support left + right + width:auto; set width directly
				headerNode.style.width = bodyNode.clientWidth + "px";
				setTimeout(function(){
					// sync up (after the browser catches up with the new width)
					headerNode.scrollLeft = bodyNode.scrollLeft;
				}, 0);
			}
		},
		
		addCssRule: function(selector, css){
			// summary:
			//		Version of util/misc.addCssRule which tracks added rules and removes
			//		them when the List is destroyed.
			
			var rule = miscUtil.addCssRule(selector, css);
			if(this.cleanAddedRules){
				// Although this isn't a listener, it shares the same remove contract
				this._listeners.push(rule);
			}
			return rule;
		},
		
		on: function(eventType, listener){
			// delegate events to the domNode
			var signal = listen(this.domNode, eventType, listener);
			if(!has("dom-addeventlistener")){
				this._listeners.push(signal);
			}
			return signal;
		},
		
		cleanup: function(){
			// summary:
			//		Clears out all rows currently in the list.
			
			var observers = this.observers,
				i;
			for(i in this._rowIdToObject){
				if(this._rowIdToObject[i] != this.columns){
					var rowElement = byId(i);
					if(rowElement){
						this.removeRow(rowElement, true);
					}
				}
			}
			// remove any store observers
			for(i = 0;i < observers.length; i++){
				var observer = observers[i];
				observer && observer.cancel();
			}
			this.observers = [];
			this._numObservers = 0;
			this.preload = null;
		},
		destroy: function(){
			// summary:
			//		Destroys this grid
			
			// Remove any event listeners and other such removables
			if(this._listeners){ // Guard against accidental subsequent calls to destroy
				for(var i = this._listeners.length; i--;){
					this._listeners[i].remove();
				}
				delete this._listeners;
			}
			
			this.cleanup();
			// destroy DOM
			put(this.domNode, "!");
			this.inherited(arguments);
		},
		refresh: function(){
			// summary:
			//		refreshes the contents of the grid
			this.cleanup();
			this._rowIdToObject = {};
			this._autoId = 0;
			
			// make sure all the content has been removed so it can be recreated
			this.contentNode.innerHTML = "";
			// Ensure scroll position always resets (especially for TouchScroll).
			this.scrollTo({ x: 0, y: 0 });
		},
		
		newRow: function(object, parentNode, beforeNode, i, options){
			if(parentNode){
				var row = this.insertRow(object, parentNode, beforeNode, i, options);
				put(row, ".ui-state-highlight");
				setTimeout(function(){
					put(row, "!ui-state-highlight");
				}, this.highlightDuration);
				return row;
			}
		},
		adjustRowIndices: function(firstRow){
			// this traverses through rows to maintain odd/even classes on the rows when indexes shift;
			var next = firstRow;
			var rowIndex = next.rowIndex;
			if(rowIndex > -1){ // make sure we have a real number in case this is called on a non-row
				do{
					// Skip non-numeric, non-rows
					if(next.rowIndex > -1){
						if(this.maintainOddEven){
							if((next.className + ' ').indexOf("dgrid-row ") > -1){
								put(next, '.' + (rowIndex % 2 == 1 ? oddClass : evenClass) + '!' + (rowIndex % 2 == 0 ? oddClass : evenClass));
							}
						}
						next.rowIndex = rowIndex++;
					}
				}while((next = next.nextSibling) && next.rowIndex != rowIndex);
			}
		},
		renderArray: function(results, beforeNode, options){
			// summary:
			//		This renders an array or collection of objects as rows in the grid, before the
			//		given node. This will listen for changes in the collection if an observe method
			//		is available (as it should be if it comes from an Observable data store).
			options = options || {};
			var self = this,
				start = options.start || 0,
				observers = this.observers,
				rows, container, observerIndex;
			
			if(!beforeNode){
				this._lastCollection = results;
			}
			if(results.observe){
				// observe the results for changes
				self._numObservers++;
				observerIndex = observers.push(results.observe(function(object, from, to){
					var row, firstRow, nextNode, parentNode;
					
					function advanceNext() {
						nextNode = (nextNode.connected || nextNode).nextSibling;
					}
					
					// a change in the data took place
					if(from > -1 && rows[from]){
						// remove from old slot
						row = rows.splice(from, 1)[0];
						// check to make the sure the node is still there before we try to remove it, (in case it was moved to a different place in the DOM)
						if(row.parentNode == container){
							firstRow = row.nextSibling;
							if(firstRow){ // it's possible for this to have been already removed if it is in overlapping query results
								if(from != to){ // if from and to are identical, it is an in-place update and we don't want to alter the rowIndex at all
									firstRow.rowIndex--; // adjust the rowIndex so adjustRowIndices has the right starting point
								}
							}
							self.removeRow(row); // now remove
						}
						// the removal of rows could cause us to need to page in more items
						if(self._processScroll){
							self._processScroll();
						}
					}
					if(to > -1){
						// Add to new slot (either before an existing row, or at the end)
						// First determine the DOM node that this should be placed before.
						if(rows.length){
							nextNode = rows[to];
							if(!nextNode){
								nextNode = rows[to - 1];
								if(nextNode){
									// Make sure to skip connected nodes, so we don't accidentally
									// insert a row in between a parent and its children.
									advanceNext();
								}
							}
						}else{
							// There are no rows.  Allow for subclasses to insert new rows somewhere other than
							// at the end of the parent node.
							nextNode = self._getFirstRowSibling && self._getFirstRowSibling(container);
						}
						// Make sure we don't trip over a stale reference to a
						// node that was removed, or try to place a node before
						// itself (due to overlapped queries)
						if(row && nextNode && row.id === nextNode.id){
							advanceNext();
						}
						if(nextNode && !nextNode.parentNode){
							nextNode = byId(nextNode.id);
						}
						parentNode = (beforeNode && beforeNode.parentNode) ||
							(nextNode && nextNode.parentNode) || self.contentNode;
						row = self.newRow(object, parentNode, nextNode, options.start + to, options);
						
						if(row){
							row.observerIndex = observerIndex;
							rows.splice(to, 0, row);
							if(!firstRow || to < from){
								// the inserted row is first, so we update firstRow to point to it
								var previous = row.previousSibling;
								// if we are not in sync with the previous row, roll the firstRow back one so adjustRowIndices can sync everything back up.
								firstRow = !previous || previous.rowIndex + 1 == row.rowIndex || row.rowIndex == 0 ?
									row : previous;
							}
						}
						options.count++;
					}
					from != to && firstRow && self.adjustRowIndices(firstRow);
					self._onNotification(rows, object, from, to);
				}, true)) - 1;
			}
			var rowsFragment = document.createDocumentFragment(),
				lastRow;
			
			function mapEach(object){
				lastRow = self.insertRow(object, rowsFragment, null, start++, options);
				lastRow.observerIndex = observerIndex;
				return lastRow;
			}
			function whenError(error){
				if(typeof observerIndex !== "undefined"){
					observers[observerIndex].cancel();
					observers[observerIndex] = 0;
					self._numObservers--;
				}
				if(error){
					throw error;
				}
			}
			function whenDone(resolvedRows){
				container = beforeNode ? beforeNode.parentNode : self.contentNode;
				if(container && container.parentNode &&
						(container !== self.contentNode || resolvedRows.length)){
					container.insertBefore(rowsFragment, beforeNode || null);
					lastRow = resolvedRows[resolvedRows.length - 1];
					lastRow && self.adjustRowIndices(lastRow);
				}else if(observers[observerIndex] && self.cleanEmptyObservers){
					// Remove the observer and don't bother inserting;
					// rows are already out of view or there were none to track
					whenError();
				}
				return (rows = resolvedRows);
			}
			
			// now render the results
			if(results.map){
				rows = results.map(mapEach, console.error);
				if(rows.then){
					return rows.then(whenDone, whenError);
				}
			}else{
				rows = [];
				for(var i = 0, l = results.length; i < l; i++){
					rows[i] = mapEach(results[i]);
				}
			}
			
			return whenDone(rows);
		},

		_onNotification: function(rows, object, from, to){
			// summary:
			//		Protected method called whenever a store notification is observed.
			//		Intended to be extended as necessary by mixins/extensions.
		},

		renderHeader: function(){
			// no-op in a plain list
		},
		
		_autoId: 0,
		insertRow: function(object, parent, beforeNode, i, options){
			// summary:
			//		Creates a single row in the grid.
			
			// Include parentId within row identifier if one was specified in options.
			// (This is used by tree to allow the same object to appear under
			// multiple parents.)
			var parentId = options.parentId,
				id = this.id + "-row-" + (parentId ? parentId + "-" : "") + 
					((this.store && this.store.getIdentity) ? 
						this.store.getIdentity(object) : this._autoId++),
				row = byId(id),
				previousRow = row && row.previousSibling;
			
			if(row){// if it existed elsewhere in the DOM, we will remove it, so we can recreate it
				this.removeRow(row);
			}
			row = this.renderRow(object, options);
			row.className = (row.className || "") + " ui-state-default dgrid-row " + (i % 2 == 1 ? oddClass : evenClass);
			// get the row id for easy retrieval
			this._rowIdToObject[row.id = id] = object;
			parent.insertBefore(row, beforeNode || null);
			if(previousRow){
				// in this case, we are pulling the row from another location in the grid, and we need to readjust the rowIndices from the point it was removed
				this.adjustRowIndices(previousRow);
			}
			row.rowIndex = i;
			return row;
		},
		renderRow: function(value, options){
			// summary:
			//		Responsible for returning the DOM for a single row in the grid.
			
			return put("div", "" + value);
		},
		removeRow: function(rowElement, justCleanup){
			// summary:
			//		Simply deletes the node in a plain List.
			//		Column plugins may aspect this to implement their own cleanup routines.
			// rowElement: Object|DOMNode
			//		Object or element representing the row to be removed.
			// justCleanup: Boolean
			//		If true, the row element will not be removed from the DOM; this can
			//		be used by extensions/plugins in cases where the DOM will be
			//		massively cleaned up at a later point in time.
			
			rowElement = rowElement.element || rowElement;
			delete this._rowIdToObject[rowElement.id];
			if(!justCleanup){
				put(rowElement, "!");
			}
		},
		
		row: function(target){
			// summary:
			//		Get the row object by id, object, node, or event
			var id;
			
			if(target instanceof this._Row){ return target; } // no-op; already a row
			
			if(target.target && target.target.nodeType){
				// event
				target = target.target;
			}
			if(target.nodeType){
				var object;
				do{
					var rowId = target.id;
					if((object = this._rowIdToObject[rowId])){
						return new this._Row(rowId.substring(this.id.length + 5), object, target); 
					}
					target = target.parentNode;
				}while(target && target != this.domNode);
				return;
			}
			if(typeof target == "object"){
				// assume target represents a store item
				id = this.store.getIdentity(target);
			}else{
				// assume target is a row ID
				id = target;
				target = this._rowIdToObject[this.id + "-row-" + id];
			}
			return new this._Row(id, target, byId(this.id + "-row-" + id));
		},
		cell: function(target){
			// this doesn't do much in a plain list
			return {
				row: this.row(target)
			};
		},
		
		_move: function(item, steps, targetClass, visible){
			var nextSibling, current, element;
			// Start at the element indicated by the provided row or cell object.
			element = current = item.element;
			steps = steps || 1;
			
			do{
				// Outer loop: move in the appropriate direction.
				if((nextSibling = current[steps < 0 ? "previousSibling" : "nextSibling"])){
					do{
						// Inner loop: advance, and dig into children if applicable.
						current = nextSibling;
						if(current && (current.className + " ").indexOf(targetClass + " ") > -1){
							// Element with the appropriate class name; count step, stop digging.
							element = current;
							steps += steps < 0 ? 1 : -1;
							break;
						}
						// If the next sibling isn't a match, drill down to search, unless
						// visible is true and children are hidden.
					}while((nextSibling = (!visible || !current.hidden) && current[steps < 0 ? "lastChild" : "firstChild"]));
				}else{
					current = current.parentNode;
					if(current === this.bodyNode || current === this.headerNode){
						// Break out if we step out of the navigation area entirely.
						break;
					}
				}
			}while(steps);
			// Return the final element we arrived at, which might still be the
			// starting element if we couldn't navigate further in that direction.
			return element;
		},
		
		up: function(row, steps, visible){
			// summary:
			//		Returns the row that is the given number of steps (1 by default)
			//		above the row represented by the given object.
			// row:
			//		The row to navigate upward from.
			// steps:
			//		Number of steps to navigate up from the given row; default is 1.
			// visible:
			//		If true, rows that are currently hidden (i.e. children of
			//		collapsed tree rows) will not be counted in the traversal.
			// returns:
			//		A row object representing the appropriate row.  If the top of the
			//		list is reached before the given number of steps, the first row will
			//		be returned.
			if(!row.element){ row = this.row(row); }
			return this.row(this._move(row, -(steps || 1), "dgrid-row", visible));
		},
		down: function(row, steps, visible){
			// summary:
			//		Returns the row that is the given number of steps (1 by default)
			//		below the row represented by the given object.
			// row:
			//		The row to navigate downward from.
			// steps:
			//		Number of steps to navigate down from the given row; default is 1.
			// visible:
			//		If true, rows that are currently hidden (i.e. children of
			//		collapsed tree rows) will not be counted in the traversal.
			// returns:
			//		A row object representing the appropriate row.  If the bottom of the
			//		list is reached before the given number of steps, the last row will
			//		be returned.
			if(!row.element){ row = this.row(row); }
			return this.row(this._move(row, steps || 1, "dgrid-row", visible));
		},
		
		scrollTo: has("touch") ? function(options){
			// If TouchScroll is the superclass, defer to its implementation.
			return this.useTouchScroll ? this.inherited(arguments) :
				desktopScrollTo.call(this, options);
		} : desktopScrollTo,
		
		getScrollPosition: has("touch") ? function(){
			// If TouchScroll is the superclass, defer to its implementation.
			return this.useTouchScroll ? this.inherited(arguments) :
				desktopGetScrollPosition.call(this);
		} : desktopGetScrollPosition,
		
		get: function(/*String*/ name /*, ... */){
			// summary:
			//		Get a property on a List instance.
			//	name:
			//		The property to get.
			//	returns:
			//		The property value on this List instance.
			// description:
			//		Get a named property on a List object. The property may
			//		potentially be retrieved via a getter method in subclasses. In the base class
			//		this just retrieves the object's property.
			
			var fn = "_get" + name.charAt(0).toUpperCase() + name.slice(1);
			
			if(typeof this[fn] === "function"){
				return this[fn].apply(this, [].slice.call(arguments, 1));
			}
			
			// Alert users that try to use Dijit-style getter/setters so they don’t get confused
			// if they try to use them and it does not work
			if(! 1  && typeof this[fn + "Attr"] === "function"){
				console.warn("dgrid: Use " + fn + " instead of " + fn + "Attr for getting " + name);
			}
			
			return this[name];
		},
		
		set: function(/*String*/ name, /*Object*/ value /*, ... */){
			//	summary:
			//		Set a property on a List instance
			//	name:
			//		The property to set.
			//	value:
			//		The value to set in the property.
			//	returns:
			//		The function returns this List instance.
			//	description:
			//		Sets named properties on a List object.
			//		A programmatic setter may be defined in subclasses.
			//
			//		set() may also be called with a hash of name/value pairs, ex:
			//	|	myObj.set({
			//	|		foo: "Howdy",
			//	|		bar: 3
			//	|	})
			//		This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
			
			if(typeof name === "object"){
				for(var k in name){
					this.set(k, name[k]);
				}
			}else{
				var fn = "_set" + name.charAt(0).toUpperCase() + name.slice(1);
				
				if(typeof this[fn] === "function"){
					this[fn].apply(this, [].slice.call(arguments, 1));
				}else{
					// Alert users that try to use Dijit-style getter/setters so they don’t get confused
					// if they try to use them and it does not work
					if(! 1  && typeof this[fn + "Attr"] === "function"){
						console.warn("dgrid: Use " + fn + " instead of " + fn + "Attr for setting " + name);
					}
					
					this[name] = value;
				}
			}
			
			return this;
		},
		
		// Accept both class and className programmatically to set domNode class.
		_getClass: getClass,
		_setClass: setClass,
		_getClassName: getClass,
		_setClassName: setClass,
		
		_setSort: function(property, descending){
			// summary:
			//		Sort the content
			// property: String|Array
			//		String specifying field to sort by, or actual array of objects
			//		with attribute and descending properties
			// descending: boolean
			//		In the case where property is a string, this argument
			//		specifies whether to sort ascending (false) or descending (true)
			
			this._sort = typeof property != "string" ? property :
				[{attribute: property, descending: descending}];
			
			this.refresh();
			
			if(this._lastCollection){
				if(property.length){
					// if an array was passed in, flatten to just first sort attribute
					// for default array sort logic
					if(typeof property != "string"){
						descending = property[0].descending;
						property = property[0].attribute;
					}
					
					this._lastCollection.sort(function(a,b){
						var aVal = a[property], bVal = b[property];
						// fall back undefined values to "" for more consistent behavior
						if(aVal === undefined){ aVal = ""; }
						if(bVal === undefined){ bVal = ""; }
						return aVal == bVal ? 0 : (aVal > bVal == !descending ? 1 : -1);
					});
				}
				this.renderArray(this._lastCollection);
			}
		},
		// TODO: remove the following two (and rename _sort to sort) in 0.4
		sort: function(property, descending){
			kernel.deprecated("sort(...)", 'use set("sort", ...) instead', "dgrid 0.4");
			this.set("sort", property, descending);
		},
		_getSort: function(){
			return this._sort;
		},
		
		_setShowHeader: function(show){
			// this is in List rather than just in Grid, primarily for two reasons:
			// (1) just in case someone *does* want to show a header in a List
			// (2) helps address IE < 8 header display issue in List
			
			var headerNode = this.headerNode;
			
			this.showHeader = show;
			
			// add/remove class which has styles for "hiding" header
			put(headerNode, (show ? "!" : ".") + "dgrid-header-hidden");
			
			this.renderHeader();
			this.resize(); // resize to account for (dis)appearance of header
			
			if(show){
				// Update scroll position of header to make sure it's in sync.
				headerNode.scrollLeft = this.getScrollPosition().x;
			}
		},
		setShowHeader: function(show){
			kernel.deprecated("setShowHeader(...)", 'use set("showHeader", ...) instead', "dgrid 0.4");
			this.set("showHeader", show);
		},
		
		_setShowFooter: function(show){
			this.showFooter = show;
			
			// add/remove class which has styles for hiding footer
			put(this.footerNode, (show ? "!" : ".") + "dgrid-footer-hidden");
			
			this.resize(); // to account for (dis)appearance of footer
		}
	});
});

},
'dgrid/util/misc':function(){
define(["put-selector/put"], function(put){
	// summary:
	//		This module defines miscellaneous utility methods for purposes of
	//		adding styles, and throttling/debouncing function calls.
	
	// establish an extra stylesheet which addCssRule calls will use,
	// plus an array to track actual indices in stylesheet for removal
	var extraRules = [],
		extraSheet,
		removeMethod,
		rulesProperty,
		invalidCssChars = /([^A-Za-z0-9_\u00A0-\uFFFF-])/g;
	
	function removeRule(index){
		// Function called by the remove method on objects returned by addCssRule.
		var realIndex = extraRules[index],
			i, l;
		if (realIndex === undefined) { return; } // already removed
		
		// remove rule indicated in internal array at index
		extraSheet[removeMethod](realIndex);
		
		// Clear internal array item representing rule that was just deleted.
		// NOTE: we do NOT splice, since the point of this array is specifically
		// to negotiate the splicing that occurs in the stylesheet itself!
		extraRules[index] = undefined;
		
		// Then update array items as necessary to downshift remaining rule indices.
		// Can start at index + 1, since array is sparse but strictly increasing.
		for(i = index + 1, l = extraRules.length; i < l; i++){
			if(extraRules[i] > realIndex){ extraRules[i]--; }
		}
	}
	
	var util = {
		// Throttle/debounce functions
		
		defaultDelay: 15,
		throttle: function(cb, context, delay){
			// summary:
			//		Returns a function which calls the given callback at most once per
			//		delay milliseconds.  (Inspired by plugd)
			var ran = false;
			delay = delay || util.defaultDelay;
			return function(){
				if(ran){ return; }
				ran = true;
				cb.apply(context, arguments);
				setTimeout(function(){ ran = false; }, delay);
			};
		},
		throttleDelayed: function(cb, context, delay){
			// summary:
			//		Like throttle, except that the callback runs after the delay,
			//		rather than before it.
			var ran = false;
			delay = delay || util.defaultDelay;
			return function(){
				if(ran){ return; }
				ran = true;
				var a = arguments;
				setTimeout(function(){
					ran = false;
					cb.apply(context, a);
				}, delay);
			};
		},
		debounce: function(cb, context, delay){
			// summary:
			//		Returns a function which calls the given callback only after a
			//		certain time has passed without successive calls.  (Inspired by plugd)
			var timer;
			delay = delay || util.defaultDelay;
			return function(){
				if(timer){
					clearTimeout(timer);
					timer = null;
				}
				var a = arguments;
				timer = setTimeout(function(){
					cb.apply(context, a);
				}, delay);
			};
		},
		
		// Iterative functions
		
		each: function(arrayOrObject, callback, context){
			// summary:
			//		Given an array or object, iterates through its keys.
			//		Does not use hasOwnProperty (since even Dojo does not
			//		consistently use it), but will iterate using a for or for-in
			//		loop as appropriate.
			
			var i, len;
			
			if(!arrayOrObject){
				return;
			}
			
			if(typeof arrayOrObject.length === "number"){
				for(i = 0, len = arrayOrObject.length; i < len; i++){
					callback.call(context, arrayOrObject[i], i, arrayOrObject);
				}
			}else{
				for(i in arrayOrObject){
					callback.call(context, arrayOrObject[i], i, arrayOrObject);
				}
			}
		},
		
		// CSS-related functions
		
		addCssRule: function(selector, css){
			// summary:
			//		Dynamically adds a style rule to the document.  Returns an object
			//		with a remove method which can be called to later remove the rule.
			
			if(!extraSheet){
				// First time, create an extra stylesheet for adding rules
				extraSheet = put(document.getElementsByTagName("head")[0], "style");
				// Keep reference to actual StyleSheet object (`styleSheet` for IE < 9)
				extraSheet = extraSheet.sheet || extraSheet.styleSheet;
				// Store name of method used to remove rules (`removeRule` for IE < 9)
				removeMethod = extraSheet.deleteRule ? "deleteRule" : "removeRule";
				// Store name of property used to access rules (`rules` for IE < 9)
				rulesProperty = extraSheet.cssRules ? "cssRules" : "rules";
			}
			
			var index = extraRules.length;
			extraRules[index] = (extraSheet.cssRules || extraSheet.rules).length;
			extraSheet.addRule ?
				extraSheet.addRule(selector, css) :
				extraSheet.insertRule(selector + '{' + css + '}', extraRules[index]);
			
			return {
				get: function(prop) {
					return extraSheet[rulesProperty][extraRules[index]].style[prop];
				},
				set: function(prop, value) {
					if (typeof extraRules[index] !== "undefined") {
						extraSheet[rulesProperty][extraRules[index]].style[prop] = value;
					}
				},
				remove: function(){
					removeRule(index);
				}
			};
		},
		
		escapeCssIdentifier: function(id){
			// summary:
			//		Escapes normally-invalid characters in a CSS identifier (such as .);
			//		see http://www.w3.org/TR/CSS2/syndata.html#value-def-identifier
			// id: String
			//		CSS identifier (e.g. tag name, class, or id) to be escaped
			
			return id.replace(invalidCssChars, "\\$1");
		}
	};
	return util;
});
},
'xstyle/has-class':function(){
define(["dojo/has"], function(has){
	var tested = {};
	return function(){
		var test, args = arguments;
		for(var i = 0; i < args.length; i++){
			var test = args[i];
			if(!tested[test]){
				tested[test] = true;
				var parts = test.match(/^(no-)?(.+?)((-[\d\.]+)(-[\d\.]+)?)?$/), // parse the class name
					hasResult = has(parts[2]), // the actual has test
					lower = -parts[4]; // lower bound if it is in the form of test-4 or test-4-6 (would be 4)
				if((lower > 0 ? lower <= hasResult && (-parts[5] || lower) >= hasResult :  // if it has a range boundary, compare to see if we are in it
						!!hasResult) == !parts[1]){ // parts[1] is the no- prefix that can negate the result
					document.documentElement.className += ' has-' + test;
				}
			}
		}
	}
});
},
'dijit/Tooltip':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/dom", // dom.byId
	"dojo/dom-class", // domClass.add
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/_base/lang", // lang.hitch lang.isArrayLike
	"dojo/mouse",
	"dojo/on",
	"dojo/sniff", // has("ie")
	"./_base/manager",	// manager.defaultDuration
	"./place",
	"./_Widget",
	"./_TemplatedMixin",
	"./BackgroundIframe",
	"dojo/text!./templates/Tooltip.html",
	"./main"		// sets dijit.showTooltip etc. for back-compat
], function(array, declare, fx, dom, domClass, domGeometry, domStyle, lang, mouse, on, has,
			manager, place, _Widget, _TemplatedMixin, BackgroundIframe, template, dijit){

	// module:
	//		dijit/Tooltip


	// TODO: Tooltip should really share more positioning code with TooltipDialog, like:
	//		- the orient() method
	//		- the connector positioning code in show()
	//		- the dijitTooltip[Dialog] class
	//
	// The problem is that Tooltip's implementation supplies it's own <iframe> and interacts directly
	// with dijit/place, rather than going through dijit/popup like TooltipDialog and other popups (ex: Menu).

	var MasterTooltip = declare("dijit._MasterTooltip", [_Widget, _TemplatedMixin], {
		// summary:
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup
		// tags:
		//		protected

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: manager.defaultDuration,

		templateString: template,

		postCreate: function(){
			this.ownerDocumentBody.appendChild(this.domNode);

			this.bgIframe = new BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = fx.fadeIn({ node: this.domNode, duration: this.duration, onEnd: lang.hitch(this, "_onShow") });
			this.fadeOut = fx.fadeOut({ node: this.domNode, duration: this.duration, onEnd: lang.hitch(this, "_onHide") });
		},

		show: function(innerHTML, aroundNode, position, rtl, textDir){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if rtl == true)
			// innerHTML: String
			//		Contents of the tooltip
			// aroundNode: DomNode|dijit/place.__Rectangle
			//		Specifies that tooltip should be next to this node / area
			// position: String[]?
			//		List of positions to try to position tooltip (ex: ["right", "above"])
			// rtl: Boolean?
			//		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
			//		means "rtl"; specifies GUI direction, not text direction.
			// textDir: String?
			//		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.


			if(this.aroundNode && this.aroundNode === aroundNode && this.containerNode.innerHTML == innerHTML){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			if(textDir){
				this.set("textDir", textDir);
			}

			this.containerNode.align = rtl? "right" : "left"; //fix the text alignment

			var pos = place.around(this.domNode, aroundNode,
				position && position.length ? position : Tooltip.defaultPosition, !rtl, lang.hitch(this, "orient"));

			// Position the tooltip connector for middle alignment.
			// This could not have been done in orient() since the tooltip wasn't positioned at that time.
			var aroundNodeCoords = pos.aroundNodePos;
			if(pos.corner.charAt(0) == 'M' && pos.aroundCorner.charAt(0) == 'M'){
				this.connectorNode.style.top = aroundNodeCoords.y + ((aroundNodeCoords.h - this.connectorNode.offsetHeight) >> 1) - pos.y + "px";
				this.connectorNode.style.left = "";
			}else if(pos.corner.charAt(1) == 'M' && pos.aroundCorner.charAt(1) == 'M'){
				this.connectorNode.style.left = aroundNodeCoords.x + ((aroundNodeCoords.w - this.connectorNode.offsetWidth) >> 1) - pos.x + "px";
			}else{
				// Not *-centered, but just above/below/after/before
				this.connectorNode.style.left = "";
				this.connectorNode.style.top = "";
			}

			// show it
			domStyle.set(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ tooltipCorner, /*Object*/ spaceAvailable, /*Object*/ aroundNodeCoords){
			// summary:
			//		Private function to set CSS for tooltip node based on which position it's in.
			//		This is called by the dijit popup code.   It will also reduce the tooltip's
			//		width to whatever width is available
			// tags:
			//		protected

			this.connectorNode.style.top = ""; //reset to default

			var heightAvailable = spaceAvailable.h,
				widthAvailable = spaceAvailable.w;

			node.className = "dijitTooltip " +
				{
					"MR-ML": "dijitTooltipRight",
					"ML-MR": "dijitTooltipLeft",
					"TM-BM": "dijitTooltipAbove",
					"BM-TM": "dijitTooltipBelow",
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft"
				}[aroundCorner + "-" + tooltipCorner];

			// reset width; it may have been set by orient() on a previous tooltip show()
			this.domNode.style.width = "auto";

			// Reduce tooltip's width to the amount of width available, so that it doesn't overflow screen.
			// Note that sometimes widthAvailable is negative, but we guard against setting style.width to a
			// negative number since that causes an exception on IE.
			var size = domGeometry.position(this.domNode);
			if(has("ie") == 9){
				// workaround strange IE9 bug where setting width to offsetWidth causes words to wrap
				size.w += 2;
			}

			var width = Math.min((Math.max(widthAvailable,1)), size.w);

			domGeometry.setMarginBox(this.domNode, {w: width});

			// Reposition the tooltip connector.
			if(tooltipCorner.charAt(0) == 'B' && aroundCorner.charAt(0) == 'B'){
				var bb = domGeometry.position(node);
				var tooltipConnectorHeight = this.connectorNode.offsetHeight;
				if(bb.h > heightAvailable){
					// The tooltip starts at the top of the page and will extend past the aroundNode
					var aroundNodePlacement = heightAvailable - ((aroundNodeCoords.h + tooltipConnectorHeight) >> 1);
					this.connectorNode.style.top = aroundNodePlacement + "px";
					this.connectorNode.style.bottom = "";
				}else{
					// Align center of connector with center of aroundNode, except don't let bottom
					// of connector extend below bottom of tooltip content, or top of connector
					// extend past top of tooltip content
					this.connectorNode.style.bottom = Math.min(
						Math.max(aroundNodeCoords.h/2 - tooltipConnectorHeight/2, 0),
						bb.h - tooltipConnectorHeight) + "px";
					this.connectorNode.style.top = "";
				}
			}else{
				// reset the tooltip back to the defaults
				this.connectorNode.style.top = "";
				this.connectorNode.style.bottom = "";
			}

			return Math.max(0, size.w - widthAvailable);
		},

		_onShow: function(){
			// summary:
			//		Called at end of fade-in operation
			// tags:
			//		protected
			if(has("ie")){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary:
			//		Hide the tooltip

			if(this._onDeck && this._onDeck[1] == aroundNode){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
			}else if(this.aroundNode === aroundNode){
				// this hide request is for the currently displayed tooltip
				this.fadeIn.stop();
				this.isShowingNow = false;
				this.aroundNode = null;
				this.fadeOut.play();
			}else{
				// just ignore the call, it's for a tooltip that has already been erased
			}
		},

		_onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}
	});

	if(has("dojo-bidi")){
		MasterTooltip.extend({
			_setAutoTextDir: function(/*Object*/node){
				// summary:
				//		Resolve "auto" text direction for children nodes
				// tags:
				//		private

				this.applyTextDir(node);
				array.forEach(node.children, function(child){ this._setAutoTextDir(child); }, this);
			},

			_setTextDirAttr: function(/*String*/ textDir){
				// summary:
				//		Setter for textDir.
				// description:
				//		Users shouldn't call this function; they should be calling
				//		set('textDir', value)
				// tags:
				//		private

				this._set("textDir", textDir);

				if (textDir == "auto"){
					this._setAutoTextDir(this.containerNode);
				}else{
					this.containerNode.dir = this.textDir;
				}
			}
		});
	}

	dijit.showTooltip = function(innerHTML, aroundNode, position, rtl, textDir){
		// summary:
		//		Static method to display tooltip w/specified contents in specified position.
		//		See description of dijit/Tooltip.defaultPosition for details on position parameter.
		//		If position is not specified then dijit/Tooltip.defaultPosition is used.
		// innerHTML: String
		//		Contents of the tooltip
		// aroundNode: place.__Rectangle
		//		Specifies that tooltip should be next to this node / area
		// position: String[]?
		//		List of positions to try to position tooltip (ex: ["right", "above"])
		// rtl: Boolean?
		//		Corresponds to `WidgetBase.dir` attribute, where false means "ltr" and true
		//		means "rtl"; specifies GUI direction, not text direction.
		// textDir: String?
		//		Corresponds to `WidgetBase.textdir` attribute; specifies direction of text.

		// After/before don't work, but for back-compat convert them to the working after-centered, before-centered.
		// Possibly remove this in 2.0.   Alternately, get before/after to work.
		if(position){
			position = array.map(position, function(val){
				return {after: "after-centered", before: "before-centered"}[val] || val;
			});
		}

		if(!Tooltip._masterTT){ dijit._masterTT = Tooltip._masterTT = new MasterTooltip(); }
		return Tooltip._masterTT.show(innerHTML, aroundNode, position, rtl, textDir);
	};

	dijit.hideTooltip = function(aroundNode){
		// summary:
		//		Static method to hide the tooltip displayed via showTooltip()
		return Tooltip._masterTT && Tooltip._masterTT.hide(aroundNode);
	};

	var Tooltip = declare("dijit.Tooltip", _Widget, {
		// summary:
		//		Pops up a tooltip (a help message) when you hover over a node.
		//		Also provides static show() and hide() methods that can be used without instantiating a dijit/Tooltip.

		// label: String
		//		HTML to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String|String[]|DomNode|DomNode[]
		//		Id of domNode(s) to attach the tooltip to.
		//		When user hovers over specified dom node(s), the tooltip will appear.
		connectId: [],

		// position: String[]
		//		See description of `dijit/Tooltip.defaultPosition` for details on position parameter.
		position: [],

		// selector: String?
		//		CSS expression to apply this Tooltip to descendants of connectIds, rather than to
		//		the nodes specified by connectIds themselves.    Useful for applying a Tooltip to
		//		a range of rows in a table, tree, etc.   Use in conjunction with getContent() parameter.
		//		Ex: connectId: myTable, selector: "tr", getContent: function(node){ return ...; }
		//
		//		The application must require() an appropriate level of dojo/query to handle the selector.
		selector: "",

		// TODO: in 2.0 remove support for multiple connectIds.   selector gives the same effect.
		// So, change connectId to a "", remove addTarget()/removeTarget(), etc.

		_setConnectIdAttr: function(/*String|String[]|DomNode|DomNode[]*/ newId){
			// summary:
			//		Connect to specified node(s)

			// Remove connections to old nodes (if there are any)
			array.forEach(this._connections || [], function(nested){
				array.forEach(nested, function(handle){ handle.remove(); });
			}, this);

			// Make array of id's to connect to, excluding entries for nodes that don't exist yet, see startup()
			this._connectIds = array.filter(lang.isArrayLike(newId) ? newId : (newId ? [newId] : []),
					function(id){ return dom.byId(id, this.ownerDocument); }, this);

			// Make connections
			this._connections = array.map(this._connectIds, function(id){
				var node = dom.byId(id, this.ownerDocument),
					selector = this.selector,
					delegatedEvent = selector ?
						function(eventType){ return on.selector(selector, eventType); } :
						function(eventType){ return eventType; },
					self = this;
				return [
					on(node, delegatedEvent(mouse.enter), function(){
						self._onHover(this);
					}),
					on(node, delegatedEvent("focusin"), function(){
						self._onHover(this);
					}),
					on(node, delegatedEvent(mouse.leave), lang.hitch(self, "_onUnHover")),
					on(node, delegatedEvent("focusout"), lang.hitch(self, "_onUnHover"))
				];
			}, this);

			this._set("connectId", newId);
		},

		addTarget: function(/*OomNode|String*/ node){
			// summary:
			//		Attach tooltip to specified node if it's not already connected

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node;
			if(array.indexOf(this._connectIds, id) == -1){
				this.set("connectId", this._connectIds.concat(id));
			}
		},

		removeTarget: function(/*DomNode|String*/ node){
			// summary:
			//		Detach tooltip from specified node

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node,	// map from DOMNode back to plain id string
				idx = array.indexOf(this._connectIds, id);
			if(idx >= 0){
				// remove id (modifies original this._connectIds but that's OK in this case)
				this._connectIds.splice(idx, 1);
				this.set("connectId", this._connectIds);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode,"dijitTooltipData");
		},

		startup: function(){
			this.inherited(arguments);

			// If this tooltip was created in a template, or for some other reason the specified connectId[s]
			// didn't exist during the widget's initialization, then connect now.
			var ids = this.connectId;
			array.forEach(lang.isArrayLike(ids) ? ids : [ids], this.addTarget, this);
		},

		getContent: function(/*DomNode*/ node){
			// summary:
			//		User overridable function that return the text to display in the tooltip.
			// tags:
			//		extension
			return this.label || this.domNode.innerHTML;
		},

		_onHover: function(/*DomNode*/ target){
			// summary:
			//		Despite the name of this method, it actually handles both hover and focus
			//		events on the target node, setting a timer to show the tooltip.
			// tags:
			//		private
			if(!this._showTimer){
				this._showTimer = this.defer(function(){ this.open(target); }, this.showDelay);
			}
		},

		_onUnHover: function(){
			// summary:
			//		Despite the name of this method, it actually handles both mouseleave and blur
			//		events on the target node, hiding the tooltip.
			// tags:
			//		private

			if(this._showTimer){
				this._showTimer.remove();
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
			// summary:
			//		Display the tooltip; usually not called directly.
			// tags:
			//		private

			if(this._showTimer){
				this._showTimer.remove();
				delete this._showTimer;
			}

			var content = this.getContent(target);
			if(!content){
				return;
			}
			Tooltip.show(content, target, this.position, !this.isLeftToRight(), this.textDir);

			this._connectNode = target;		// _connectNode means "tooltip currently displayed for this node"
			this.onShow(target, this.position);
		},

		close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				Tooltip.hide(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				this._showTimer.remove();
				delete this._showTimer;
			}
		},

		onShow: function(/*===== target, position =====*/){
			// summary:
			//		Called when the tooltip is shown
			// tags:
			//		callback
		},

		onHide: function(){
			// summary:
			//		Called when the tooltip is hidden
			// tags:
			//		callback
		},

		destroy: function(){
			this.close();

			// Remove connections manually since they aren't registered to be removed by _WidgetBase
			array.forEach(this._connections || [], function(nested){
				array.forEach(nested, function(handle){ handle.remove(); });
			}, this);

			this.inherited(arguments);
		}
	});

	Tooltip._MasterTooltip = MasterTooltip;		// for monkey patching
	Tooltip.show = dijit.showTooltip;		// export function through module return value
	Tooltip.hide = dijit.hideTooltip;		// export function through module return value

	Tooltip.defaultPosition = ["after-centered", "before-centered"];

	/*=====
	lang.mixin(Tooltip, {
		 // defaultPosition: String[]
		 //		This variable controls the position of tooltips, if the position is not specified to
		 //		the Tooltip widget or *TextBox widget itself.  It's an array of strings with the values
		 //		possible for `dijit/place.around()`.   The recommended values are:
		 //
		 //		- before-centered: centers tooltip to the left of the anchor node/widget, or to the right
		 //		  in the case of RTL scripts like Hebrew and Arabic
		 //		- after-centered: centers tooltip to the right of the anchor node/widget, or to the left
		 //		  in the case of RTL scripts like Hebrew and Arabic
		 //		- above-centered: tooltip is centered above anchor node
		 //		- below-centered: tooltip is centered above anchor node
		 //
		 //		The list is positions is tried, in order, until a position is found where the tooltip fits
		 //		within the viewport.
		 //
		 //		Be careful setting this parameter.  A value of "above-centered" may work fine until the user scrolls
		 //		the screen so that there's no room above the target node.   Nodes with drop downs, like
		 //		DropDownButton or FilteringSelect, are especially problematic, in that you need to be sure
		 //		that the drop down and tooltip don't overlap, even when the viewport is scrolled so that there
		 //		is only room below (or above) the target node, but not both.
	 });
	=====*/
	return Tooltip;
});

},
'dijit/_base/manager':function(){
define([
	"dojo/_base/array",
	"dojo/_base/config", // defaultDuration
	"dojo/_base/lang",
	"../registry",
	"../main"	// for setting exports to dijit namespace
], function(array, config, lang, registry, dijit){

	// module:
	//		dijit/_base/manager

	var exports = {
		// summary:
		//		Deprecated.  Shim to methods on registry, plus a few other declarations.
		//		New code should access dijit/registry directly when possible.
	};

	array.forEach(["byId", "getUniqueId", "findWidgets", "_destroyAll", "byNode", "getEnclosingWidget"], function(name){
		exports[name] = registry[name];
	});

	 lang.mixin(exports, {
		 // defaultDuration: Integer
		 //		The default fx.animation speed (in ms) to use for all Dijit
		 //		transitional fx.animations, unless otherwise specified
		 //		on a per-instance basis. Defaults to 200, overrided by
		 //		`djConfig.defaultDuration`
		 defaultDuration: config["defaultDuration"] || 200
	 });

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dijit/place':function(){
define([
	"dojo/_base/array", // array.forEach array.map array.some
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/window", // win.body
	"./Viewport", // getEffectiveBox
	"./main"	// dijit (defining dijit.place to match API doc)
], function(array, domGeometry, domStyle, kernel, win, Viewport, dijit){

	// module:
	//		dijit/place


	function _place(/*DomNode*/ node, choices, layoutNode, aroundNodeCoords){
		// summary:
		//		Given a list of spots to put node, put it at the first spot where it fits,
		//		of if it doesn't fit anywhere then the place with the least overflow
		// choices: Array
		//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
		//		Above example says to put the top-left corner of the node at (10,20)
		// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
		//		for things like tooltip, they are displayed differently (and have different dimensions)
		//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
		//		It also passes in the available size for the popup, which is useful for tooltips to
		//		tell them that their width is limited to a certain amount.	 layoutNode() may return a value expressing
		//		how much the popup had to be modified to fit into the available space.	 This is used to determine
		//		what the best placement is.
		// aroundNodeCoords: Object
		//		Size of aroundNode, ex: {w: 200, h: 50}

		// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
		// viewport over document
		var view = Viewport.getEffectiveBox(node.ownerDocument);

		// This won't work if the node is inside a <div style="position: relative">,
		// so reattach it to <body>.	 (Otherwise, the positioning will be wrong
		// and also it might get cutoff.)
		if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
			win.body(node.ownerDocument).appendChild(node);
		}

		var best = null;
		array.some(choices, function(choice){
			var corner = choice.corner;
			var pos = choice.pos;
			var overflow = 0;

			// calculate amount of space available given specified position of node
			var spaceAvailable = {
				w: {
					'L': view.l + view.w - pos.x,
					'R': pos.x - view.l,
					'M': view.w
				}[corner.charAt(1)],
				h: {
					'T': view.t + view.h - pos.y,
					'B': pos.y - view.t,
					'M': view.h
				}[corner.charAt(0)]
			};

			// Clear left/right position settings set earlier so they don't interfere with calculations,
			// specifically when layoutNode() (a.k.a. Tooltip.orient()) measures natural width of Tooltip
			var s = node.style;
			s.left = s.right = "auto";

			// configure node to be displayed in given position relative to button
			// (need to do this in order to get an accurate size for the node, because
			// a tooltip's size changes based on position, due to triangle)
			if(layoutNode){
				var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
				overflow = typeof res == "undefined" ? 0 : res;
			}

			// get node's size
			var style = node.style;
			var oldDisplay = style.display;
			var oldVis = style.visibility;
			if(style.display == "none"){
				style.visibility = "hidden";
				style.display = "";
			}
			var bb = domGeometry.position(node);
			style.display = oldDisplay;
			style.visibility = oldVis;

			// coordinates and size of node with specified corner placed at pos,
			// and clipped by viewport
			var
				startXpos = {
					'L': pos.x,
					'R': pos.x - bb.w,
					'M': Math.max(view.l, Math.min(view.l + view.w, pos.x + (bb.w >> 1)) - bb.w) // M orientation is more flexible
				}[corner.charAt(1)],
				startYpos = {
					'T': pos.y,
					'B': pos.y - bb.h,
					'M': Math.max(view.t, Math.min(view.t + view.h, pos.y + (bb.h >> 1)) - bb.h)
				}[corner.charAt(0)],
				startX = Math.max(view.l, startXpos),
				startY = Math.max(view.t, startYpos),
				endX = Math.min(view.l + view.w, startXpos + bb.w),
				endY = Math.min(view.t + view.h, startYpos + bb.h),
				width = endX - startX,
				height = endY - startY;

			overflow += (bb.w - width) + (bb.h - height);

			if(best == null || overflow < best.overflow){
				best = {
					corner: corner,
					aroundCorner: choice.aroundCorner,
					x: startX,
					y: startY,
					w: width,
					h: height,
					overflow: overflow,
					spaceAvailable: spaceAvailable
				};
			}

			return !overflow;
		});

		// In case the best position is not the last one we checked, need to call
		// layoutNode() again.
		if(best.overflow && layoutNode){
			layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
		}

		// And then position the node.  Do this last, after the layoutNode() above
		// has sized the node, due to browser quirks when the viewport is scrolled
		// (specifically that a Tooltip will shrink to fit as though the window was
		// scrolled to the left).
		//
		// In RTL mode, set style.right rather than style.left so in the common case,
		// window resizes move the popup along with the aroundNode.

		var l = domGeometry.isBodyLtr(node.ownerDocument),
			top = best.y,
			side = l ? best.x : view.w - best.x - best.w;

		if(/relative|absolute/.test(domStyle.get(win.body(node.ownerDocument), "position"))){
			// compensate for margin on <body>, see #16148
			top -= domStyle.get(win.body(node.ownerDocument), "marginTop");
			side -= (l ? 1 : -1) * domStyle.get(win.body(node.ownerDocument), l ? "marginLeft" : "marginRight");
		}

		var s = node.style;
		s.top = top + "px";
		s[l ? "left" : "right"] = side + "px";
		s[l ? "right" : "left"] = "auto";	// needed for FF or else tooltip goes to far left

		return best;
	}

	var reverse = {
		// Map from corner to kitty-corner
		"TL": "BR",
		"TR": "BL",
		"BL": "TR",
		"BR": "TL"
	};

	var place = {
		// summary:
		//		Code to place a DOMNode relative to another DOMNode.
		//		Load using require(["dijit/place"], function(place){ ... }).

		at: function(node, pos, corners, padding, layoutNode){
			// summary:
			//		Positions node kitty-corner to the rectangle centered at (pos.x, pos.y) with width and height of
			//		padding.x * 2 and padding.y * 2, or zero if padding not specified.  Picks first corner in corners[]
			//		where node is fully visible, or the corner where it's most visible.
			//
			//		Node is assumed to be absolutely or relatively positioned.
			// node: DOMNode
			//		The node to position
			// pos: dijit/place.__Position
			//		Object like {x: 10, y: 20}
			// corners: String[]
			//		Array of Strings representing order to try corners of the node in, like ["TR", "BL"].
			//		Possible values are:
			//
			//		- "BL" - bottom left
			//		- "BR" - bottom right
			//		- "TL" - top left
			//		- "TR" - top right
			// padding: dijit/place.__Position?
			//		Optional param to set padding, to put some buffer around the element you want to position.
			//		Defaults to zero.
			// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
			//		For things like tooltip, they are displayed differently (and have different dimensions)
			//		based on their orientation relative to the parent.  This adjusts the popup based on orientation.
			// example:
			//		Try to place node's top right corner at (10,20).
			//		If that makes node go (partially) off screen, then try placing
			//		bottom left corner at (10,20).
			//	|	place(node, {x: 10, y: 20}, ["TR", "BL"])
			var choices = array.map(corners, function(corner){
				var c = {
					corner: corner,
					aroundCorner: reverse[corner],	// so TooltipDialog.orient() gets aroundCorner argument set
					pos: {x: pos.x,y: pos.y}
				};
				if(padding){
					c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
					c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
				}
				return c;
			});

			return _place(node, choices, layoutNode);
		},

		around: function(
			/*DomNode*/		node,
			/*DomNode|dijit/place.__Rectangle*/ anchor,
			/*String[]*/	positions,
			/*Boolean*/		leftToRight,
			/*Function?*/	layoutNode){

			// summary:
			//		Position node adjacent or kitty-corner to anchor
			//		such that it's fully visible in viewport.
			// description:
			//		Place node such that corner of node touches a corner of
			//		aroundNode, and that node is fully visible.
			// anchor:
			//		Either a DOMNode or a rectangle (object with x, y, width, height).
			// positions:
			//		Ordered list of positions to try matching up.
			//
			//		- before: places drop down to the left of the anchor node/widget, or to the right in the case
			//			of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//			with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//		- after: places drop down to the right of the anchor node/widget, or to the left in the case
			//			of RTL scripts like Hebrew and Arabic; aligns either the top of the drop down
			//			with the top of the anchor, or the bottom of the drop down with bottom of the anchor.
			//		- before-centered: centers drop down to the left of the anchor node/widget, or to the right
			//			in the case of RTL scripts like Hebrew and Arabic
			//		- after-centered: centers drop down to the right of the anchor node/widget, or to the left
			//			in the case of RTL scripts like Hebrew and Arabic
			//		- above-centered: drop down is centered above anchor node
			//		- above: drop down goes above anchor node, left sides aligned
			//		- above-alt: drop down goes above anchor node, right sides aligned
			//		- below-centered: drop down is centered above anchor node
			//		- below: drop down goes below anchor node
			//		- below-alt: drop down goes below anchor node, right sides aligned
			// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
			//		For things like tooltip, they are displayed differently (and have different dimensions)
			//		based on their orientation relative to the parent.	 This adjusts the popup based on orientation.
			// leftToRight:
			//		True if widget is LTR, false if widget is RTL.   Affects the behavior of "above" and "below"
			//		positions slightly.
			// example:
			//	|	placeAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
			//		This will try to position node such that node's top-left corner is at the same position
			//		as the bottom left corner of the aroundNode (ie, put node below
			//		aroundNode, with left edges aligned).	If that fails it will try to put
			//		the bottom-right corner of node where the top right corner of aroundNode is
			//		(ie, put node above aroundNode, with right edges aligned)
			//

			// If around is a DOMNode (or DOMNode id), convert to coordinates.
			var aroundNodePos;
			if(typeof anchor == "string" || "offsetWidth" in anchor){
				aroundNodePos = domGeometry.position(anchor, true);

				// For above and below dropdowns, subtract width of border so that popup and aroundNode borders
				// overlap, preventing a double-border effect.  Unfortunately, difficult to measure the border
				// width of either anchor or popup because in both cases the border may be on an inner node.
				if(/^(above|below)/.test(positions[0])){
					var anchorBorder = domGeometry.getBorderExtents(anchor),
						anchorChildBorder = anchor.firstChild ? domGeometry.getBorderExtents(anchor.firstChild) : {t:0,l:0,b:0,r:0},
						nodeBorder =  domGeometry.getBorderExtents(node),
						nodeChildBorder = node.firstChild ? domGeometry.getBorderExtents(node.firstChild) : {t:0,l:0,b:0,r:0};
					aroundNodePos.y += Math.min(anchorBorder.t + anchorChildBorder.t, nodeBorder.t + nodeChildBorder.t);
					aroundNodePos.h -=  Math.min(anchorBorder.t + anchorChildBorder.t, nodeBorder.t+ nodeChildBorder.t) +
						Math.min(anchorBorder.b + anchorChildBorder.b, nodeBorder.b + nodeChildBorder.b);
				}
			}else{
				aroundNodePos = anchor;
			}

			// Compute position and size of visible part of anchor (it may be partially hidden by ancestor nodes w/scrollbars)
			if(anchor.parentNode){
				// ignore nodes between position:relative and position:absolute
				var sawPosAbsolute = domStyle.getComputedStyle(anchor).position == "absolute";
				var parent = anchor.parentNode;
				while(parent && parent.nodeType == 1 && parent.nodeName != "BODY"){  //ignoring the body will help performance
					var parentPos = domGeometry.position(parent, true),
						pcs = domStyle.getComputedStyle(parent);
					if(/relative|absolute/.test(pcs.position)){
						sawPosAbsolute = false;
					}
					if(!sawPosAbsolute && /hidden|auto|scroll/.test(pcs.overflow)){
						var bottomYCoord = Math.min(aroundNodePos.y + aroundNodePos.h, parentPos.y + parentPos.h);
						var rightXCoord = Math.min(aroundNodePos.x + aroundNodePos.w, parentPos.x + parentPos.w);
						aroundNodePos.x = Math.max(aroundNodePos.x, parentPos.x);
						aroundNodePos.y = Math.max(aroundNodePos.y, parentPos.y);
						aroundNodePos.h = bottomYCoord - aroundNodePos.y;
						aroundNodePos.w = rightXCoord - aroundNodePos.x;
					}
					if(pcs.position == "absolute"){
						sawPosAbsolute = true;
					}
					parent = parent.parentNode;
				}
			}			

			var x = aroundNodePos.x,
				y = aroundNodePos.y,
				width = "w" in aroundNodePos ? aroundNodePos.w : (aroundNodePos.w = aroundNodePos.width),
				height = "h" in aroundNodePos ? aroundNodePos.h : (kernel.deprecated("place.around: dijit/place.__Rectangle: { x:"+x+", y:"+y+", height:"+aroundNodePos.height+", width:"+width+" } has been deprecated.  Please use { x:"+x+", y:"+y+", h:"+aroundNodePos.height+", w:"+width+" }", "", "2.0"), aroundNodePos.h = aroundNodePos.height);

			// Convert positions arguments into choices argument for _place()
			var choices = [];
			function push(aroundCorner, corner){
				choices.push({
					aroundCorner: aroundCorner,
					corner: corner,
					pos: {
						x: {
							'L': x,
							'R': x + width,
							'M': x + (width >> 1)
						}[aroundCorner.charAt(1)],
						y: {
							'T': y,
							'B': y + height,
							'M': y + (height >> 1)
						}[aroundCorner.charAt(0)]
					}
				})
			}
			array.forEach(positions, function(pos){
				var ltr =  leftToRight;
				switch(pos){
					case "above-centered":
						push("TM", "BM");
						break;
					case "below-centered":
						push("BM", "TM");
						break;
					case "after-centered":
						ltr = !ltr;
						// fall through
					case "before-centered":
						push(ltr ? "ML" : "MR", ltr ? "MR" : "ML");
						break;
					case "after":
						ltr = !ltr;
						// fall through
					case "before":
						push(ltr ? "TL" : "TR", ltr ? "TR" : "TL");
						push(ltr ? "BL" : "BR", ltr ? "BR" : "BL");
						break;
					case "below-alt":
						ltr = !ltr;
						// fall through
					case "below":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "BL" : "BR", ltr ? "TL" : "TR");
						push(ltr ? "BR" : "BL", ltr ? "TR" : "TL");
						break;
					case "above-alt":
						ltr = !ltr;
						// fall through
					case "above":
						// first try to align left borders, next try to align right borders (or reverse for RTL mode)
						push(ltr ? "TL" : "TR", ltr ? "BL" : "BR");
						push(ltr ? "TR" : "TL", ltr ? "BR" : "BL");
						break;
					default:
						// To assist dijit/_base/place, accept arguments of type {aroundCorner: "BL", corner: "TL"}.
						// Not meant to be used directly.  Remove for 2.0.
						push(pos.aroundCorner, pos.corner);
				}
			});

			var position = _place(node, choices, layoutNode, {w: width, h: height});
			position.aroundNodePos = aroundNodePos;

			return position;
		}
	};

	/*=====
	place.__Position = {
		// x: Integer
		//		horizontal coordinate in pixels, relative to document body
		// y: Integer
		//		vertical coordinate in pixels, relative to document body
	};
	place.__Rectangle = {
		// x: Integer
		//		horizontal offset in pixels, relative to document body
		// y: Integer
		//		vertical offset in pixels, relative to document body
		// w: Integer
		//		width in pixels.   Can also be specified as "width" for backwards-compatibility.
		// h: Integer
		//		height in pixels.   Can also be specified as "height" for backwards-compatibility.
	};
	=====*/

	return dijit.place = place;	// setting dijit.place for back-compat, remove for 2.0
});

},
'dijit/Viewport':function(){
define([
	"dojo/Evented",
	"dojo/on",
	"dojo/domReady",
	"dojo/sniff",	// has("ie"), has("ios")
	"dojo/window" // getBox()
], function(Evented, on, domReady, has, winUtils){

	// module:
	//		dijit/Viewport

	/*=====
	return {
		// summary:
		//		Utility singleton to watch for viewport resizes, avoiding duplicate notifications
		//		which can lead to infinite loops.
		// description:
		//		Usage: Viewport.on("resize", myCallback).
		//
		//		myCallback() is called without arguments in case it's _WidgetBase.resize(),
		//		which would interpret the argument as the size to make the widget.
	};
	=====*/

	var Viewport = new Evented();

	var focusedNode;

	domReady(function(){
		var oldBox = winUtils.getBox();
		Viewport._rlh = on(window, "resize", function(){
			var newBox = winUtils.getBox();
			if(oldBox.h == newBox.h && oldBox.w == newBox.w){ return; }
			oldBox = newBox;
			Viewport.emit("resize");
		});

		// Also catch zoom changes on IE8, since they don't naturally generate resize events
		if(has("ie") == 8){
			var deviceXDPI = screen.deviceXDPI;
			setInterval(function(){
				if(screen.deviceXDPI != deviceXDPI){
					deviceXDPI = screen.deviceXDPI;
					Viewport.emit("resize");
				}
			}, 500);
		}

		// On iOS, keep track of the focused node so we can guess when the keyboard is/isn't being displayed.
		if(has("ios")){
			on(document, "focusin", function(evt){
				focusedNode = evt.target;
			});
			on(document, "focusout", function(evt){
				focusedNode = null;
			});
		}
	});

	Viewport.getEffectiveBox = function(/*Document*/ doc){
		// summary:
		//		Get the size of the viewport, or on mobile devices, the part of the viewport not obscured by the
		//		virtual keyboard.

		var box = winUtils.getBox(doc);

		// Account for iOS virtual keyboard, if it's being shown.  Unfortunately no direct way to check or measure.
		var tag = focusedNode && focusedNode.tagName && focusedNode.tagName.toLowerCase();
		if(has("ios") && focusedNode && !focusedNode.readOnly && (tag == "textarea" || (tag == "input" &&
			/^(color|email|number|password|search|tel|text|url)$/.test(focusedNode.type)))){

			// Box represents the size of the viewport.  Some of the viewport is likely covered by the keyboard.
			// Estimate height of visible viewport assuming viewport goes to bottom of screen, but is covered by keyboard.
			box.h *= (orientation == 0 || orientation == 180 ? 0.66 : 0.40);

			// Above measurement will be inaccurate if viewport was scrolled up so far that it ends before the bottom
			// of the screen.   In this case, keyboard isn't covering as much of the viewport as we thought.
			// We know the visible size is at least the distance from the top of the viewport to the focused node.
			var rect = focusedNode.getBoundingClientRect();
			box.h = Math.max(box.h, rect.top + rect.height);
		}

		return box;
	};

	return Viewport;
});

},
'dojo/window':function(){
define(["./_base/lang", "./sniff", "./_base/window", "./dom", "./dom-geometry", "./dom-style", "./dom-construct"],
	function(lang, has, baseWindow, dom, geom, style, domConstruct){

	// feature detection
	/* not needed but included here for future reference
	has.add("rtl-innerVerticalScrollBar-on-left", function(win, doc){
		var	body = baseWindow.body(doc),
			scrollable = domConstruct.create('div', {
				style: {overflow:'scroll', overflowX:'hidden', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', width:'64px', height:'64px'}
			}, body, "last"),
			center = domConstruct.create('center', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last"),
			inner = domConstruct.create('div', {
				style: {overflow:'visible', display:'inline' }
			}, center, "last");
		inner.innerHTML="&nbsp;";
		var midPoint = Math.max(inner.offsetLeft, geom.position(inner).x);
		var ret = midPoint >= 32;
		center.removeChild(inner);
		scrollable.removeChild(center);
		body.removeChild(scrollable);
		return ret;
	});
	*/
	has.add("rtl-adjust-position-for-verticalScrollBar", function(win, doc){
		var	body = baseWindow.body(doc),
			scrollable = domConstruct.create('div', {
				style: {overflow:'scroll', overflowX:'visible', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', top:'0', width:'64px', height:'64px'}
			}, body, "last"),
			div = domConstruct.create('div', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last"),
			ret = geom.position(div).x != 0;
		scrollable.removeChild(div);
		body.removeChild(scrollable);
		return ret;
	});

	has.add("position-fixed-support", function(win, doc){
		// IE6, IE7+quirks, and some older mobile browsers don't support position:fixed
		var	body = baseWindow.body(doc),
			outer = domConstruct.create('span', {
				style: {visibility:'hidden', position:'fixed', left:'1px', top:'1px'}
			}, body, "last"),
			inner = domConstruct.create('span', {
				style: {position:'fixed', left:'0', top:'0'}
			}, outer, "last"),
			ret = geom.position(inner).x != geom.position(outer).x;
		outer.removeChild(inner);
		body.removeChild(outer);
		return ret;
	});

	// module:
	//		dojo/window

	var window = {
		// summary:
		//		TODOC

		getBox: function(/*Document?*/ doc){
			// summary:
			//		Returns the dimensions and scroll position of the viewable area of a browser window

			doc = doc || baseWindow.doc;

			var
				scrollRoot = (doc.compatMode == 'BackCompat') ? baseWindow.body(doc) : doc.documentElement,
				// get scroll position
				scroll = geom.docScroll(doc), // scrollRoot.scrollTop/Left should work
				w, h;

			if(has("touch")){ // if(scrollbars not supported)
				var uiWindow = window.get(doc);   // use UI window, not dojo.global window
				// on mobile, scrollRoot.clientHeight <= uiWindow.innerHeight <= scrollRoot.offsetHeight, return uiWindow.innerHeight
				w = uiWindow.innerWidth || scrollRoot.clientWidth; // || scrollRoot.clientXXX probably never evaluated
				h = uiWindow.innerHeight || scrollRoot.clientHeight;
			}else{
				// on desktops, scrollRoot.clientHeight <= scrollRoot.offsetHeight <= uiWindow.innerHeight, return scrollRoot.clientHeight
				// uiWindow.innerWidth/Height includes the scrollbar and cannot be used
				w = scrollRoot.clientWidth;
				h = scrollRoot.clientHeight;
			}
			return {
				l: scroll.x,
				t: scroll.y,
				w: w,
				h: h
			};
		},

		get: function(/*Document*/ doc){
			// summary:
			//		Get window object associated with document doc.
			// doc:
			//		The document to get the associated window for.

			// In some IE versions (at least 6.0), document.parentWindow does not return a
			// reference to the real window object (maybe a copy), so we must fix it as well
			// We use IE specific execScript to attach the real window reference to
			// document._parentWindow for later use
			if(has("ie") && window !== document.parentWindow){
				/*
				In IE 6, only the variable "window" can be used to connect events (others
				may be only copies).
				*/
				doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
				//to prevent memory leak, unset it after use
				//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
				var win = doc._parentWindow;
				doc._parentWindow = null;
				return win;	//	Window
			}

			return doc.parentWindow || doc.defaultView;	//	Window
		},

		scrollIntoView: function(/*DomNode*/ node, /*Object?*/ pos){
			// summary:
			//		Scroll the passed node into view using minimal movement, if it is not already.

			// Don't rely on node.scrollIntoView working just because the function is there since
			// it forces the node to the page's bottom or top (and left or right in IE) without consideration for the minimal movement.
			// WebKit's node.scrollIntoViewIfNeeded doesn't work either for inner scrollbars in right-to-left mode
			// and when there's a fixed position scrollable element

			try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
				node = dom.byId(node);
				var	doc = node.ownerDocument || baseWindow.doc,	// TODO: why baseWindow.doc?  Isn't node.ownerDocument always defined?
					body = baseWindow.body(doc),
					html = doc.documentElement || body.parentNode,
					isIE = has("ie"),
					isWK = has("webkit");
				// if an untested browser, then use the native method
				if(node == body || node == html){ return; }
				if(!(has("mozilla") || isIE || isWK || has("opera")) && ("scrollIntoView" in node)){
					node.scrollIntoView(false); // short-circuit to native if possible
					return;
				}
				var	backCompat = doc.compatMode == 'BackCompat',
					rootWidth = Math.min(body.clientWidth || html.clientWidth, html.clientWidth || body.clientWidth),
					rootHeight = Math.min(body.clientHeight || html.clientHeight, html.clientHeight || body.clientHeight),
					scrollRoot = (isWK || backCompat) ? body : html,
					nodePos = pos || geom.position(node),
					el = node.parentNode,
					isFixed = function(el){
						return (isIE <= 6 || (isIE == 7 && backCompat))
							? false
							: (has("position-fixed-support") && (style.get(el, 'position').toLowerCase() == "fixed"));
					};
				if(isFixed(node)){ return; } // nothing to do
				while(el){
					if(el == body){ el = scrollRoot; }
					var	elPos = geom.position(el),
						fixedPos = isFixed(el),
						rtl = style.getComputedStyle(el).direction.toLowerCase() == "rtl";

					if(el == scrollRoot){
						elPos.w = rootWidth; elPos.h = rootHeight;
						if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
						if(elPos.x < 0 || !isIE || isIE >= 9){ elPos.x = 0; } // older IE can have values > 0
						if(elPos.y < 0 || !isIE || isIE >= 9){ elPos.y = 0; }
					}else{
						var pb = geom.getPadBorderExtents(el);
						elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
						var clientSize = el.clientWidth,
							scrollBarSize = elPos.w - clientSize;
						if(clientSize > 0 && scrollBarSize > 0){
							if(rtl && has("rtl-adjust-position-for-verticalScrollBar")){
								elPos.x += scrollBarSize;
							}
							elPos.w = clientSize;
						}
						clientSize = el.clientHeight;
						scrollBarSize = elPos.h - clientSize;
						if(clientSize > 0 && scrollBarSize > 0){
							elPos.h = clientSize;
						}
					}
					if(fixedPos){ // bounded by viewport, not parents
						if(elPos.y < 0){
							elPos.h += elPos.y; elPos.y = 0;
						}
						if(elPos.x < 0){
							elPos.w += elPos.x; elPos.x = 0;
						}
						if(elPos.y + elPos.h > rootHeight){
							elPos.h = rootHeight - elPos.y;
						}
						if(elPos.x + elPos.w > rootWidth){
							elPos.w = rootWidth - elPos.x;
						}
					}
					// calculate overflow in all 4 directions
					var	l = nodePos.x - elPos.x, // beyond left: < 0
//						t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
						t = nodePos.y - elPos.y, // beyond top: < 0
						r = l + nodePos.w - elPos.w, // beyond right: > 0
						bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
					var s, old;
					if(r * l > 0 && (!!el.scrollLeft || el == scrollRoot || el.scrollWidth > el.offsetHeight)){
						s = Math[l < 0? "max" : "min"](l, r);
						if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9)){ s = -s; }
						old = el.scrollLeft;
						el.scrollLeft += s;
						s = el.scrollLeft - old;
						nodePos.x -= s;
					}
					if(bot * t > 0 && (!!el.scrollTop || el == scrollRoot || el.scrollHeight > el.offsetHeight)){
						s = Math.ceil(Math[t < 0? "max" : "min"](t, bot));
						old = el.scrollTop;
						el.scrollTop += s;
						s = el.scrollTop - old;
						nodePos.y -= s;
					}
					el = (el != scrollRoot) && !fixedPos && el.parentNode;
				}
			}catch(error){
				console.error('scrollIntoView: ' + error);
				node.scrollIntoView(false);
			}
		}
	};

	 1  && lang.setObject("dojo.window", window);

	return window;
});

},
'dijit/_Widget':function(){
define([
	"dojo/aspect",	// aspect.around
	"dojo/_base/config",	// config.isDebug
	"dojo/_base/connect",	// connect.connect
	"dojo/_base/declare", // declare
	"dojo/has",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/query",
	"dojo/ready",
	"./registry",	// registry.byNode
	"./_WidgetBase",
	"./_OnDijitClickMixin",
	"./_FocusMixin",
	"dojo/uacss",		// browser sniffing (included for back-compat; subclasses may be using)
	"./hccss"		// high contrast mode sniffing (included to set CSS classes on <body>, module ret value unused)
], function(aspect, config, connect, declare, has, kernel, lang, query, ready,
			registry, _WidgetBase, _OnDijitClickMixin, _FocusMixin){


// module:
//		dijit/_Widget


function connectToDomNode(){
	// summary:
	//		If user connects to a widget method === this function, then they will
	//		instead actually be connecting the equivalent event on this.domNode
}

// Trap dojo.connect() calls to connectToDomNode methods, and redirect to _Widget.on()
function aroundAdvice(originalConnect){
	return function(obj, event, scope, method){
		if(obj && typeof event == "string" && obj[event] == connectToDomNode){
			return obj.on(event.substring(2).toLowerCase(), lang.hitch(scope, method));
		}
		return originalConnect.apply(connect, arguments);
	};
}
aspect.around(connect, "connect", aroundAdvice);
if(kernel.connect){
	aspect.around(kernel, "connect", aroundAdvice);
}

var _Widget = declare("dijit._Widget", [_WidgetBase, _OnDijitClickMixin, _FocusMixin], {
	// summary:
	//		Old base class for widgets.   New widgets should extend `dijit/_WidgetBase` instead
	// description:
	//		Old Base class for Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//
	//		- declaratively/programatically specifying widget initialization parameters like
	//			onMouseMove="foo" that call foo when this.domNode gets a mousemove event
	//		- ondijitclick:
	//			Support new data-dojo-attach-event="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//		- focus related functions:
	//			In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//			dijit/_base/focus.js.
	//		- deprecated methods
	//		- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//
	//		- browser sniffing (putting browser class like `dj_ie` on `<html>` node)
	//		- high contrast mode sniffing (add `dijit_a11y` class to `<body>` if machine is in high contrast mode)


	////////////////// DEFERRED CONNECTS ///////////////////

	onClick: connectToDomNode,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: connectToDomNode,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: connectToDomNode,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: connectToDomNode,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: connectToDomNode,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: connectToDomNode,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: connectToDomNode,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: connectToDomNode,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: connectToDomNode,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: connectToDomNode,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: connectToDomNode,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: connectToDomNode,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	constructor: function(params /*===== ,srcNodeRef =====*/){
		// summary:
		//		Create the widget.
		// params: Object|null
		//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
		//		and functions, typically callbacks like onClick.
		//		The hash can contain any of the widget's properties, excluding read-only properties.
		// srcNodeRef: DOMNode|String?
		//		If a srcNodeRef (DOM node) is specified:
		//
		//		- use srcNodeRef.innerHTML as my contents
		//		- if this is a behavioral widget then apply behavior to that srcNodeRef
		//		- otherwise, replace srcNodeRef with my generated DOM tree

		// extract parameters like onMouseMove that should connect directly to this.domNode
		this._toConnect = {};
		for(var name in params){
			if(this[name] === connectToDomNode){
				this._toConnect[name.replace(/^on/, "").toLowerCase()] = params[name];
				delete params[name];
			}
		}
	},

	postCreate: function(){
		this.inherited(arguments);

		// perform connection from this.domNode to user specified handlers (ex: onMouseMove)
		for(var name in this._toConnect){
			this.on(name, this._toConnect[name]);
		}
		delete this._toConnect;
	},

	on: function(/*String|Function*/ type, /*Function*/ func){
		if(this[this._onMap(type)] === connectToDomNode){
			// Use connect.connect() rather than on() to get handling for "onmouseenter" on non-IE,
			// normalization of onkeypress/onkeydown to behave like firefox, etc.
			// Also, need to specify context as "this" rather than the default context of the DOMNode
			// Remove in 2.0.
			return connect.connect(this.domNode, type.toLowerCase(), this, func);
		}
		return this.inherited(arguments);
	},

	_setFocusedAttr: function(val){
		// Remove this method in 2.0 (or sooner), just here to set _focused == focused, for back compat
		// (but since it's a private variable we aren't required to keep supporting it).
		this._focused = val;
		this._set("focused", val);
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		kernel.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		// name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		// value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				kernel.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		kernel.deprecated(this.declaredClass+"::getDescendants() is deprecated. Use getChildren() instead.", "", "2.0");
		return this.containerNode ? query('[widgetId]', this.containerNode).map(registry.byNode) : []; // dijit/_WidgetBase[]
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit/layout/TabContainer`, `dijit/layout/StackContainer`,
		//		`dijit/layout/AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
		//		Called when another widget becomes the selected pane in a
		//		`dijit/layout/TabContainer`, `dijit/layout/StackContainer`,
		//		`dijit/layout/AccordionContainer`, etc.
		//
		//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit/layout/StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

// For back-compat, remove in 2.0.
if(has("dijit-legacy-requires")){
	ready(0, function(){
		var requires = ["dijit/_base"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}
return _Widget;
});

},
'dijit/_WidgetBase':function(){
define([
	"require", // require.toUrl
	"dojo/_base/array", // array.forEach array.map
	"dojo/aspect",
	"dojo/_base/config", // config.blankGif
	"dojo/_base/connect", // connect.connect
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.set domAttr.remove
	"dojo/dom-class", // domClass.add domClass.replace
	"dojo/dom-construct", // domConstruct.destroy domConstruct.place
	"dojo/dom-geometry", // isBodyLtr
	"dojo/dom-style", // domStyle.set, domStyle.get
	"dojo/has",
	"dojo/_base/kernel",
	"dojo/_base/lang", // mixin(), isArray(), etc.
	"dojo/on",
	"dojo/ready",
	"dojo/Stateful", // Stateful
	"dojo/topic",
	"dojo/_base/window", // win.body()
	"./Destroyable",
	"dojo/has!dojo-bidi?./_BidiMixin",
	"./registry"    // registry.getUniqueId(), registry.findWidgets()
], function(require, array, aspect, config, connect, declare,
			dom, domAttr, domClass, domConstruct, domGeometry, domStyle, has, kernel,
			lang, on, ready, Stateful, topic, win, Destroyable, _BidiMixin, registry){

	// module:
	//		dijit/_WidgetBase

	// Flag to make dijit load modules the app didn't explicitly request, for backwards compatibility
	has.add("dijit-legacy-requires", !kernel.isAsync);

	// Flag to enable support for textdir attribute
	has.add("dojo-bidi", false);


	// For back-compat, remove in 2.0.
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/_base/manager"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	// Nested hash listing attributes for each tag, all strings in lowercase.
	// ex: {"div": {"style": true, "tabindex" true}, "form": { ...
	var tagAttrs = {};

	function getAttrs(obj){
		var ret = {};
		for(var attr in obj){
			ret[attr.toLowerCase()] = true;
		}
		return ret;
	}

	function nonEmptyAttrToDom(attr){
		// summary:
		//		Returns a setter function that copies the attribute to this.domNode,
		//		or removes the attribute from this.domNode, depending on whether the
		//		value is defined or not.
		return function(val){
			domAttr[val ? "set" : "remove"](this.domNode, attr, val);
			this._set(attr, val);
		};
	}

	var _WidgetBase = declare("dijit._WidgetBase", [Stateful, Destroyable], {
		// summary:
		//		Future base class for all Dijit widgets.
		// description:
		//		Future base class for all Dijit widgets.
		//		_Widget extends this class adding support for various features needed by desktop.
		//
		//		Provides stubs for widget lifecycle methods for subclasses to extend, like postMixInProperties(), buildRendering(),
		//		postCreate(), startup(), and destroy(), and also public API methods like set(), get(), and watch().
		//
		//		Widgets can provide custom setters/getters for widget attributes, which are called automatically by set(name, value).
		//		For an attribute XXX, define methods _setXXXAttr() and/or _getXXXAttr().
		//
		//		_setXXXAttr can also be a string/hash/array mapping from a widget attribute XXX to the widget's DOMNodes:
		//
		//		- DOM node attribute
		// |		_setFocusAttr: {node: "focusNode", type: "attribute"}
		// |		_setFocusAttr: "focusNode"	(shorthand)
		// |		_setFocusAttr: ""		(shorthand, maps to this.domNode)
		//		Maps this.focus to this.focusNode.focus, or (last example) this.domNode.focus
		//
		//		- DOM node innerHTML
		//	|		_setTitleAttr: { node: "titleNode", type: "innerHTML" }
		//		Maps this.title to this.titleNode.innerHTML
		//
		//		- DOM node innerText
		//	|		_setTitleAttr: { node: "titleNode", type: "innerText" }
		//		Maps this.title to this.titleNode.innerText
		//
		//		- DOM node CSS class
		// |		_setMyClassAttr: { node: "domNode", type: "class" }
		//		Maps this.myClass to this.domNode.className
		//
		//		If the value of _setXXXAttr is an array, then each element in the array matches one of the
		//		formats of the above list.
		//
		//		If the custom setter is null, no action is performed other than saving the new value
		//		in the widget (in this).
		//
		//		If no custom setter is defined for an attribute, then it will be copied
		//		to this.focusNode (if the widget defines a focusNode), or this.domNode otherwise.
		//		That's only done though for attributes that match DOMNode attributes (title,
		//		alt, aria-labelledby, etc.)

		// id: [const] String
		//		A unique, opaque ID string that can be assigned by users or by the
		//		system. If the developer passes an ID which is known not to be
		//		unique, the specified ID is ignored and the system-generated ID is
		//		used instead.
		id: "",
		_setIdAttr: "domNode", // to copy to this.domNode even for auto-generated id's

		// lang: [const] String
		//		Rarely used.  Overrides the default Dojo locale used to render this widget,
		//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
		//		Value must be among the list of locales specified during by the Dojo bootstrap,
		//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
		lang: "",
		// set on domNode even when there's a focus node.	but don't set lang="", since that's invalid.
		_setLangAttr: nonEmptyAttrToDom("lang"),

		// dir: [const] String
		//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
		//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
		//		default direction.
		dir: "",
		// set on domNode even when there's a focus node.	but don't set dir="", since that's invalid.
		_setDirAttr: nonEmptyAttrToDom("dir"), // to set on domNode even when there's a focus node

		// class: String
		//		HTML class attribute
		"class": "",
		_setClassAttr: { node: "domNode", type: "class" },

		// style: String||Object
		//		HTML style attributes as cssText string or name/value hash
		style: "",

		// title: String
		//		HTML title attribute.
		//
		//		For form widgets this specifies a tooltip to display when hovering over
		//		the widget (just like the native HTML title attribute).
		//
		//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
		//		etc., it's used to specify the tab label, accordion pane title, etc.  In this case it's
		//		interpreted as HTML.
		title: "",

		// tooltip: String
		//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
		//		this specifies the tooltip to appear when the mouse is hovered over that text.
		tooltip: "",

		// baseClass: [protected] String
		//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
		//		widget state.
		baseClass: "",

		// srcNodeRef: [readonly] DomNode
		//		pointer to original DOM node
		srcNodeRef: null,

		// domNode: [readonly] DomNode
		//		This is our visible representation of the widget! Other DOM
		//		Nodes may by assigned to other properties, usually through the
		//		template system's data-dojo-attach-point syntax, but the domNode
		//		property is the canonical "top level" node in widget UI.
		domNode: null,

		// containerNode: [readonly] DomNode
		//		Designates where children of the source DOM node will be placed.
		//		"Children" in this case refers to both DOM nodes and widgets.
		//		For example, for myWidget:
		//
		//		|	<div data-dojo-type=myWidget>
		//		|		<b> here's a plain DOM node
		//		|		<span data-dojo-type=subWidget>and a widget</span>
		//		|		<i> and another plain DOM node </i>
		//		|	</div>
		//
		//		containerNode would point to:
		//
		//		|		<b> here's a plain DOM node
		//		|		<span data-dojo-type=subWidget>and a widget</span>
		//		|		<i> and another plain DOM node </i>
		//
		//		In templated widgets, "containerNode" is set via a
		//		data-dojo-attach-point assignment.
		//
		//		containerNode must be defined for any widget that accepts innerHTML
		//		(like ContentPane or BorderContainer or even Button), and conversely
		//		is null for widgets that don't, like TextBox.
		containerNode: null,

		// ownerDocument: [const] Document?
		//		The document this widget belongs to.  If not specified to constructor, will default to
		//		srcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global
		ownerDocument: null,
		_setOwnerDocumentAttr: function(val){
			// this setter is merely to avoid automatically trying to set this.domNode.ownerDocument
			this._set("ownerDocument", val);
		},

		/*=====
		// _started: [readonly] Boolean
		//		startup() has completed.
		_started: false,
		=====*/

		// attributeMap: [protected] Object
		//		Deprecated.	Instead of attributeMap, widget should have a _setXXXAttr attribute
		//		for each XXX attribute to be mapped to the DOM.
		//
		//		attributeMap sets up a "binding" between attributes (aka properties)
		//		of the widget and the widget's DOM.
		//		Changes to widget attributes listed in attributeMap will be
		//		reflected into the DOM.
		//
		//		For example, calling set('title', 'hello')
		//		on a TitlePane will automatically cause the TitlePane's DOM to update
		//		with the new title.
		//
		//		attributeMap is a hash where the key is an attribute of the widget,
		//		and the value reflects a binding to a:
		//
		//		- DOM node attribute
		// |		focus: {node: "focusNode", type: "attribute"}
		//		Maps this.focus to this.focusNode.focus
		//
		//		- DOM node innerHTML
		//	|		title: { node: "titleNode", type: "innerHTML" }
		//		Maps this.title to this.titleNode.innerHTML
		//
		//		- DOM node innerText
		//	|		title: { node: "titleNode", type: "innerText" }
		//		Maps this.title to this.titleNode.innerText
		//
		//		- DOM node CSS class
		// |		myClass: { node: "domNode", type: "class" }
		//		Maps this.myClass to this.domNode.className
		//
		//		If the value is an array, then each element in the array matches one of the
		//		formats of the above list.
		//
		//		There are also some shorthands for backwards compatibility:
		//
		//		- string --> { node: string, type: "attribute" }, for example:
		//
		//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
		//
		//		- "" --> { node: "domNode", type: "attribute" }
		attributeMap: {},

		// _blankGif: [protected] String
		//		Path to a blank 1x1 image.
		//		Used by `<img>` nodes in templates that really get their image via CSS background-image.
		_blankGif: config.blankGif || require.toUrl("dojo/resources/blank.gif"),

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		/*=====
		constructor: function(params, srcNodeRef){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified:
			//
			//		- use srcNodeRef.innerHTML as my contents
			//		- if this is a behavioral widget then apply behavior to that srcNodeRef
			//		- otherwise, replace srcNodeRef with my generated DOM tree
		},
		=====*/

		_introspect: function(){
			// summary:
			//		Collect metadata about this widget (only once per class, not once per instance):
			//
			//			- list of attributes with custom setters, storing in this.constructor._setterAttrs
			//			- generate this.constructor._onMap, mapping names like "mousedown" to functions like onMouseDown

			var ctor = this.constructor;
			if(!ctor._setterAttrs){
				var proto = ctor.prototype,
					attrs = ctor._setterAttrs = [], // attributes with custom setters
					onMap = (ctor._onMap = {});

				// Items in this.attributeMap are like custom setters.  For back-compat, remove for 2.0.
				for(var name in proto.attributeMap){
					attrs.push(name);
				}

				// Loop over widget properties, collecting properties with custom setters and filling in ctor._onMap.
				for(name in proto){
					if(/^on/.test(name)){
						onMap[name.substring(2).toLowerCase()] = name;
					}

					if(/^_set[A-Z](.*)Attr$/.test(name)){
						name = name.charAt(4).toLowerCase() + name.substr(5, name.length - 9);
						if(!proto.attributeMap || !(name in proto.attributeMap)){
							attrs.push(name);
						}
					}
				}

				// Note: this isn't picking up info on properties like aria-label and role, that don't have custom setters
				// but that set() maps to attributes on this.domNode or this.focusNode
			}
		},

		postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
			// summary:
			//		Kicks off widget instantiation.  See create() for details.
			// tags:
			//		private

			// Note that we skip calling this.inherited(), i.e. dojo/Stateful::postscript(), because 1.x widgets don't
			// expect their custom setters to get called until after buildRendering().  Consider changing for 2.0.

			this.create(params, srcNodeRef);
		},

		create: function(params, srcNodeRef){
			// summary:
			//		Kick off the life-cycle of a widget
			// description:
			//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
			//		etc.), some of which of you'll want to override. See http://dojotoolkit.org/reference-guide/dijit/_WidgetBase.html
			//		for a discussion of the widget creation lifecycle.
			//
			//		Of course, adventurous developers could override create entirely, but this should
			//		only be done as a last resort.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified:
			//
			//		- use srcNodeRef.innerHTML as my contents
			//		- if this is a behavioral widget then apply behavior to that srcNodeRef
			//		- otherwise, replace srcNodeRef with my generated DOM tree
			// tags:
			//		private

			// First time widget is instantiated, scan prototype to figure out info about custom setters etc.
			this._introspect();

			// store pointer to original DOM tree
			this.srcNodeRef = dom.byId(srcNodeRef);

			// No longer used, remove for 2.0.
			this._connects = [];
			this._supportingWidgets = [];

			// this is here for back-compat, remove in 2.0 (but check NodeList-instantiate.html test)
			if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){
				this.id = this.srcNodeRef.id;
			}

			// mix in our passed parameters
			if(params){
				this.params = params;
				lang.mixin(this, params);
			}
			this.postMixInProperties();

			// Generate an id for the widget if one wasn't specified, or it was specified as id: undefined.
			// Do this before buildRendering() because it might expect the id to be there.
			if(!this.id){
				this.id = registry.getUniqueId(this.declaredClass.replace(/\./g, "_"));
				if(this.params){
					// if params contains {id: undefined}, prevent _applyAttributes() from processing it
					delete this.params.id;
				}
			}

			// The document and <body> node this widget is associated with
			this.ownerDocument = this.ownerDocument || (this.srcNodeRef ? this.srcNodeRef.ownerDocument : document);
			this.ownerDocumentBody = win.body(this.ownerDocument);

			registry.add(this);

			this.buildRendering();

			var deleteSrcNodeRef;

			if(this.domNode){
				// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
				// Also calls custom setters for all attributes with custom setters.
				this._applyAttributes();

				// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
				// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
				// widget being attached to the DOM since it isn't when a widget is created programmatically like
				// new MyWidget({}).	See #11635.
				var source = this.srcNodeRef;
				if(source && source.parentNode && this.domNode !== source){
					source.parentNode.replaceChild(this.domNode, source);
					deleteSrcNodeRef = true;
				}

				// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
				// assuming that dojo._scopeName even exists in 2.0
				this.domNode.setAttribute("widgetId", this.id);
			}
			this.postCreate();

			// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
			// I think for back-compatibility it isn't deleting srcNodeRef until after postCreate() has run.
			if(deleteSrcNodeRef){
				delete this.srcNodeRef;
			}

			this._created = true;
		},

		_applyAttributes: function(){
			// summary:
			//		Step during widget creation to copy  widget attributes to the
			//		DOM according to attributeMap and _setXXXAttr objects, and also to call
			//		custom _setXXXAttr() methods.
			//
			//		Skips over blank/false attribute values, unless they were explicitly specified
			//		as parameters to the widget, since those are the default anyway,
			//		and setting tabIndex="" is different than not setting tabIndex at all.
			//
			//		For backwards-compatibility reasons attributeMap overrides _setXXXAttr when
			//		_setXXXAttr is a hash/string/array, but _setXXXAttr as a functions override attributeMap.
			// tags:
			//		private

			// Call this.set() for each property that was either specified as parameter to constructor,
			// or is in the list found above.	For correlated properties like value and displayedValue, the one
			// specified as a parameter should take precedence.
			// Particularly important for new DateTextBox({displayedValue: ...}) since DateTextBox's default value is
			// NaN and thus is not ignored like a default value of "".

			// Step 1: Save the current values of the widget properties that were specified as parameters to the constructor.
			// Generally this.foo == this.params.foo, except if postMixInProperties() changed the value of this.foo.
			var params = {};
			for(var key in this.params || {}){
				params[key] = this._get(key);
			}

			// Step 2: Call set() for each property with a non-falsy value that wasn't passed as a parameter to the constructor
			array.forEach(this.constructor._setterAttrs, function(key){
				if(!(key in params)){
					var val = this._get(key);
					if(val){
						this.set(key, val);
					}
				}
			}, this);

			// Step 3: Call set() for each property that was specified as parameter to constructor.
			// Use params hash created above to ignore side effects from step #2 above.
			for(key in params){
				this.set(key, params[key]);
			}
		},

		postMixInProperties: function(){
			// summary:
			//		Called after the parameters to the widget have been read-in,
			//		but before the widget template is instantiated. Especially
			//		useful to set properties that are referenced in the widget
			//		template.
			// tags:
			//		protected
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget, setting this.domNode.
			//		Most widgets will mixin `dijit._TemplatedMixin`, which implements this method.
			// tags:
			//		protected

			if(!this.domNode){
				// Create root node if it wasn't created by _TemplatedMixin
				this.domNode = this.srcNodeRef || this.ownerDocument.createElement("div");
			}

			// baseClass is a single class name or occasionally a space-separated list of names.
			// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
			// TODO: make baseClass custom setter
			if(this.baseClass){
				var classes = this.baseClass.split(" ");
				if(!this.isLeftToRight()){
					classes = classes.concat(array.map(classes, function(name){
						return name + "Rtl";
					}));
				}
				domClass.add(this.domNode, classes);
			}
		},

		postCreate: function(){
			// summary:
			//		Processing after the DOM fragment is created
			// description:
			//		Called after the DOM fragment has been created, but not necessarily
			//		added to the document.  Do not include any operations which rely on
			//		node dimensions or placement.
			// tags:
			//		protected
		},

		startup: function(){
			// summary:
			//		Processing after the DOM fragment is added to the document
			// description:
			//		Called after a widget and its children have been created and added to the page,
			//		and all related widgets have finished their create() cycle, up through postCreate().
			//
			//		Note that startup() may be called while the widget is still hidden, for example if the widget is
			//		inside a hidden dijit/Dialog or an unselected tab of a dijit/layout/TabContainer.
			//		For widgets that need to do layout, it's best to put that layout code inside resize(), and then
			//		extend dijit/layout/_LayoutWidget so that resize() is called when the widget is visible.
			if(this._started){
				return;
			}
			this._started = true;
			array.forEach(this.getChildren(), function(obj){
				if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			});
		},

		//////////// DESTROY FUNCTIONS ////////////////////////////////

		destroyRecursive: function(/*Boolean?*/ preserveDom){
			// summary:
			//		Destroy this widget and its descendants
			// description:
			//		This is the generic "destructor" function that all widget users
			//		should call to cleanly discard with a widget. Once a widget is
			//		destroyed, it is removed from the manager object.
			// preserveDom:
			//		If true, this method will leave the original DOM structure
			//		alone of descendant Widgets. Note: This will NOT work with
			//		dijit._TemplatedMixin widgets.

			this._beingDestroyed = true;
			this.destroyDescendants(preserveDom);
			this.destroy(preserveDom);
		},

		destroy: function(/*Boolean*/ preserveDom){
			// summary:
			//		Destroy this widget, but not its descendants.  Descendants means widgets inside of
			//		this.containerNode.   Will also destroy any resources (including widgets) registered via this.own().
			//
			//		This method will also destroy internal widgets such as those created from a template,
			//		assuming those widgets exist inside of this.domNode but outside of this.containerNode.
			//
			//		For 2.0 it's planned that this method will also destroy descendant widgets, so apps should not
			//		depend on the current ability to destroy a widget without destroying its descendants.   Generally
			//		they should use destroyRecursive() for widgets with children.
			// preserveDom: Boolean
			//		If true, this method will leave the original DOM structure alone.
			//		Note: This will not yet work with _TemplatedMixin widgets

			this._beingDestroyed = true;
			this.uninitialize();

			function destroy(w){
				if(w.destroyRecursive){
					w.destroyRecursive(preserveDom);
				}else if(w.destroy){
					w.destroy(preserveDom);
				}
			}

			// Back-compat, remove for 2.0
			array.forEach(this._connects, lang.hitch(this, "disconnect"));
			array.forEach(this._supportingWidgets, destroy);

			// Destroy supporting widgets, but not child widgets under this.containerNode (for 2.0, destroy child widgets
			// here too).   if() statement is to guard against exception if destroy() called multiple times (see #15815).
			if(this.domNode){
				array.forEach(registry.findWidgets(this.domNode, this.containerNode), destroy);
			}

			this.destroyRendering(preserveDom);
			registry.remove(this.id);
			this._destroyed = true;
		},

		destroyRendering: function(/*Boolean?*/ preserveDom){
			// summary:
			//		Destroys the DOM nodes associated with this widget.
			// preserveDom:
			//		If true, this method will leave the original DOM structure alone
			//		during tear-down. Note: this will not work with _Templated
			//		widgets yet.
			// tags:
			//		protected

			if(this.bgIframe){
				this.bgIframe.destroy(preserveDom);
				delete this.bgIframe;
			}

			if(this.domNode){
				if(preserveDom){
					domAttr.remove(this.domNode, "widgetId");
				}else{
					domConstruct.destroy(this.domNode);
				}
				delete this.domNode;
			}

			if(this.srcNodeRef){
				if(!preserveDom){
					domConstruct.destroy(this.srcNodeRef);
				}
				delete this.srcNodeRef;
			}
		},

		destroyDescendants: function(/*Boolean?*/ preserveDom){
			// summary:
			//		Recursively destroy the children of this widget and their
			//		descendants.
			// preserveDom:
			//		If true, the preserveDom attribute is passed to all descendant
			//		widget's .destroy() method. Not for use with _Templated
			//		widgets.

			// get all direct descendants and destroy them recursively
			array.forEach(this.getChildren(), function(widget){
				if(widget.destroyRecursive){
					widget.destroyRecursive(preserveDom);
				}
			});
		},

		uninitialize: function(){
			// summary:
			//		Deprecated. Override destroy() instead to implement custom widget tear-down
			//		behavior.
			// tags:
			//		protected
			return false;
		},

		////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

		_setStyleAttr: function(/*String||Object*/ value){
			// summary:
			//		Sets the style attribute of the widget according to value,
			//		which is either a hash like {height: "5px", width: "3px"}
			//		or a plain string
			// description:
			//		Determines which node to set the style on based on style setting
			//		in attributeMap.
			// tags:
			//		protected

			var mapNode = this.domNode;

			// Note: technically we should revert any style setting made in a previous call
			// to his method, but that's difficult to keep track of.

			if(lang.isObject(value)){
				domStyle.set(mapNode, value);
			}else{
				if(mapNode.style.cssText){
					mapNode.style.cssText += "; " + value;
				}else{
					mapNode.style.cssText = value;
				}
			}

			this._set("style", value);
		},

		_attrToDom: function(/*String*/ attr, /*String*/ value, /*Object?*/ commands){
			// summary:
			//		Reflect a widget attribute (title, tabIndex, duration etc.) to
			//		the widget DOM, as specified by commands parameter.
			//		If commands isn't specified then it's looked up from attributeMap.
			//		Note some attributes like "type"
			//		cannot be processed this way as they are not mutable.
			// attr:
			//		Name of member variable (ex: "focusNode" maps to this.focusNode) pointing
			//		to DOMNode inside the widget, or alternately pointing to a subwidget
			// tags:
			//		private

			commands = arguments.length >= 3 ? commands : this.attributeMap[attr];

			array.forEach(lang.isArray(commands) ? commands : [commands], function(command){

				// Get target node and what we are doing to that node
				var mapNode = this[command.node || command || "domNode"];	// DOM node
				var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

				switch(type){
					case "attribute":
						if(lang.isFunction(value)){ // functions execute in the context of the widget
							value = lang.hitch(this, value);
						}

						// Get the name of the DOM node attribute; usually it's the same
						// as the name of the attribute in the widget (attr), but can be overridden.
						// Also maps handler names to lowercase, like onSubmit --> onsubmit
						var attrName = command.attribute ? command.attribute :
							(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

						if(mapNode.tagName){
							// Normal case, mapping to a DOMNode.  Note that modern browsers will have a mapNode.set()
							// method, but for consistency we still call domAttr
							domAttr.set(mapNode, attrName, value);
						}else{
							// mapping to a sub-widget
							mapNode.set(attrName, value);
						}
						break;
					case "innerText":
						mapNode.innerHTML = "";
						mapNode.appendChild(this.ownerDocument.createTextNode(value));
						break;
					case "innerHTML":
						mapNode.innerHTML = value;
						break;
					case "class":
						domClass.replace(mapNode, value, this[attr]);
						break;
				}
			}, this);
		},

		get: function(name){
			// summary:
			//		Get a property from a widget.
			// name:
			//		The property to get.
			// description:
			//		Get a named property from a widget. The property may
			//		potentially be retrieved via a getter method. If no getter is defined, this
			//		just retrieves the object's property.
			//
			//		For example, if the widget has properties `foo` and `bar`
			//		and a method named `_getFooAttr()`, calling:
			//		`myWidget.get("foo")` would be equivalent to calling
			//		`widget._getFooAttr()` and `myWidget.get("bar")`
			//		would be equivalent to the expression
			//		`widget.bar2`
			var names = this._getAttrNames(name);
			return this[names.g] ? this[names.g]() : this._get(name);
		},

		set: function(name, value){
			// summary:
			//		Set a property on a widget
			// name:
			//		The property to set.
			// value:
			//		The value to set in the property.
			// description:
			//		Sets named properties on a widget which may potentially be handled by a
			//		setter in the widget.
			//
			//		For example, if the widget has properties `foo` and `bar`
			//		and a method named `_setFooAttr()`, calling
			//		`myWidget.set("foo", "Howdy!")` would be equivalent to calling
			//		`widget._setFooAttr("Howdy!")` and `myWidget.set("bar", 3)`
			//		would be equivalent to the statement `widget.bar = 3;`
			//
			//		set() may also be called with a hash of name/value pairs, ex:
			//
			//	|	myWidget.set({
			//	|		foo: "Howdy",
			//	|		bar: 3
			//	|	});
			//
			//	This is equivalent to calling `set(foo, "Howdy")` and `set(bar, 3)`

			if(typeof name === "object"){
				for(var x in name){
					this.set(x, name[x]);
				}
				return this;
			}
			var names = this._getAttrNames(name),
				setter = this[names.s];
			if(lang.isFunction(setter)){
				// use the explicit setter
				var result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
			}else{
				// Mapping from widget attribute to DOMNode/subwidget attribute/value/etc.
				// Map according to:
				//		1. attributeMap setting, if one exists (TODO: attributeMap deprecated, remove in 2.0)
				//		2. _setFooAttr: {...} type attribute in the widget (if one exists)
				//		3. apply to focusNode or domNode if standard attribute name, excluding funcs like onClick.
				// Checks if an attribute is a "standard attribute" by whether the DOMNode JS object has a similar
				// attribute name (ex: accept-charset attribute matches jsObject.acceptCharset).
				// Note also that Tree.focusNode() is a function not a DOMNode, so test for that.
				var defaultNode = this.focusNode && !lang.isFunction(this.focusNode) ? "focusNode" : "domNode",
					tag = this[defaultNode] && this[defaultNode].tagName,
					attrsForTag = tag && (tagAttrs[tag] || (tagAttrs[tag] = getAttrs(this[defaultNode]))),
					map = name in this.attributeMap ? this.attributeMap[name] :
						names.s in this ? this[names.s] :
							((attrsForTag && names.l in attrsForTag && typeof value != "function") ||
								/^aria-|^data-|^role$/.test(name)) ? defaultNode : null;
				if(map != null){
					this._attrToDom(name, value, map);
				}
				this._set(name, value);
			}
			return result || this;
		},

		_attrPairNames: {}, // shared between all widgets
		_getAttrNames: function(name){
			// summary:
			//		Helper function for get() and set().
			//		Caches attribute name values so we don't do the string ops every time.
			// tags:
			//		private

			var apn = this._attrPairNames;
			if(apn[name]){
				return apn[name];
			}
			var uc = name.replace(/^[a-z]|-[a-zA-Z]/g, function(c){
				return c.charAt(c.length - 1).toUpperCase();
			});
			return (apn[name] = {
				n: name + "Node",
				s: "_set" + uc + "Attr", // converts dashes to camel case, ex: accept-charset --> _setAcceptCharsetAttr
				g: "_get" + uc + "Attr",
				l: uc.toLowerCase()        // lowercase name w/out dashes, ex: acceptcharset
			});
		},

		_set: function(/*String*/ name, /*anything*/ value){
			// summary:
			//		Helper function to set new value for specified property, and call handlers
			//		registered with watch() if the value has changed.
			var oldValue = this[name];
			this[name] = value;
			if(this._created && value !== oldValue){
				if(this._watchCallbacks){
					this._watchCallbacks(name, oldValue, value);
				}
				this.emit("attrmodified-" + name, {
					detail: {
						prevValue: oldValue,
						newValue: value
					}
				});
			}
		},

		_get: function(/*String*/ name){
			// summary:
			//		Helper function to get value for specified property stored by this._set(),
			//		i.e. for properties with custom setters.  Used mainly by custom getters.
			//
			//		For example, CheckBox._getValueAttr() calls this._get("value").

			// future: return name in this.props ? this.props[name] : this[name];
			return this[name];
		},

		emit: function(/*String*/ type, /*Object?*/ eventObj, /*Array?*/ callbackArgs){
			// summary:
			//		Used by widgets to signal that a synthetic event occurred, ex:
			//	|	myWidget.emit("attrmodified-selectedChildWidget", {}).
			//
			//		Emits an event on this.domNode named type.toLowerCase(), based on eventObj.
			//		Also calls onType() method, if present, and returns value from that method.
			//		By default passes eventObj to callback, but will pass callbackArgs instead, if specified.
			//		Modifies eventObj by adding missing parameters (bubbles, cancelable, widget).
			// tags:
			//		protected

			// Specify fallback values for bubbles, cancelable in case they are not set in eventObj.
			// Also set pointer to widget, although since we can't add a pointer to the widget for native events
			// (see #14729), maybe we shouldn't do it here?
			eventObj = eventObj || {};
			if(eventObj.bubbles === undefined){
				eventObj.bubbles = true;
			}
			if(eventObj.cancelable === undefined){
				eventObj.cancelable = true;
			}
			if(!eventObj.detail){
				eventObj.detail = {};
			}
			eventObj.detail.widget = this;

			var ret, callback = this["on" + type];
			if(callback){
				ret = callback.apply(this, callbackArgs ? callbackArgs : [eventObj]);
			}

			// Emit event, but avoid spurious emit()'s as parent sets properties on child during startup/destroy
			if(this._started && !this._beingDestroyed){
				on.emit(this.domNode, type.toLowerCase(), eventObj);
			}

			return ret;
		},

		on: function(/*String|Function*/ type, /*Function*/ func){
			// summary:
			//		Call specified function when event occurs, ex: myWidget.on("click", function(){ ... }).
			// type:
			//		Name of event (ex: "click") or extension event like touch.press.
			// description:
			//		Call specified function when event `type` occurs, ex: `myWidget.on("click", function(){ ... })`.
			//		Note that the function is not run in any particular scope, so if (for example) you want it to run in the
			//		widget's scope you must do `myWidget.on("click", lang.hitch(myWidget, func))`.

			// For backwards compatibility, if there's an onType() method in the widget then connect to that.
			// Remove in 2.0.
			var widgetMethod = this._onMap(type);
			if(widgetMethod){
				return aspect.after(this, widgetMethod, func, true);
			}

			// Otherwise, just listen for the event on this.domNode.
			return this.own(on(this.domNode, type, func))[0];
		},

		_onMap: function(/*String|Function*/ type){
			// summary:
			//		Maps on() type parameter (ex: "mousemove") to method name (ex: "onMouseMove").
			//		If type is a synthetic event like touch.press then returns undefined.
			var ctor = this.constructor, map = ctor._onMap;
			if(!map){
				map = (ctor._onMap = {});
				for(var attr in ctor.prototype){
					if(/^on/.test(attr)){
						map[attr.replace(/^on/, "").toLowerCase()] = attr;
					}
				}
			}
			return map[typeof type == "string" && type.toLowerCase()];	// String
		},

		toString: function(){
			// summary:
			//		Returns a string that represents the widget.
			// description:
			//		When a widget is cast to a string, this method will be used to generate the
			//		output. Currently, it does not implement any sort of reversible
			//		serialization.
			return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
		},

		getChildren: function(){
			// summary:
			//		Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent
			//		is this widget.   Note that it does not return all descendants, but rather just direct children.
			//		Analogous to [Node.childNodes](https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes),
			//		except containing widgets rather than DOMNodes.
			//
			//		The result intentionally excludes internally created widgets (a.k.a. supporting widgets)
			//		outside of this.containerNode.
			//
			//		Note that the array returned is a simple array.  Application code should not assume
			//		existence of methods like forEach().

			return this.containerNode ? registry.findWidgets(this.containerNode) : []; // dijit/_WidgetBase[]
		},

		getParent: function(){
			// summary:
			//		Returns the parent widget of this widget.

			return registry.getEnclosingWidget(this.domNode.parentNode);
		},

		connect: function(/*Object|null*/ obj, /*String|Function*/ event, /*String|Function*/ method){
			// summary:
			//		Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.
			//
			//		Connects specified obj/event to specified method of this object
			//		and registers for disconnect() on widget destroy.
			//
			//		Provide widget-specific analog to dojo.connect, except with the
			//		implicit use of this widget as the target object.
			//		Events connected with `this.connect` are disconnected upon
			//		destruction.
			// returns:
			//		A handle that can be passed to `disconnect` in order to disconnect before
			//		the widget is destroyed.
			// example:
			//	|	var btn = new Button();
			//	|	// when foo.bar() is called, call the listener we're going to
			//	|	// provide in the scope of btn
			//	|	btn.connect(foo, "bar", function(){
			//	|		console.debug(this.toString());
			//	|	});
			// tags:
			//		protected

			return this.own(connect.connect(obj, event, this, method))[0];	// handle
		},

		disconnect: function(handle){
			// summary:
			//		Deprecated, will be removed in 2.0, use handle.remove() instead.
			//
			//		Disconnects handle created by `connect`.
			// tags:
			//		protected

			handle.remove();
		},

		subscribe: function(t, method){
			// summary:
			//		Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.
			//
			//		Subscribes to the specified topic and calls the specified method
			//		of this object and registers for unsubscribe() on widget destroy.
			//
			//		Provide widget-specific analog to dojo.subscribe, except with the
			//		implicit use of this widget as the target object.
			// t: String
			//		The topic
			// method: Function
			//		The callback
			// example:
			//	|	var btn = new Button();
			//	|	// when /my/topic is published, this button changes its label to
			//	|	// be the parameter of the topic.
			//	|	btn.subscribe("/my/topic", function(v){
			//	|		this.set("label", v);
			//	|	});
			// tags:
			//		protected
			return this.own(topic.subscribe(t, lang.hitch(this, method)))[0];	// handle
		},

		unsubscribe: function(/*Object*/ handle){
			// summary:
			//		Deprecated, will be removed in 2.0, use handle.remove() instead.
			//
			//		Unsubscribes handle created by this.subscribe.
			//		Also removes handle from this widget's list of subscriptions
			// tags:
			//		protected

			handle.remove();
		},

		isLeftToRight: function(){
			// summary:
			//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
			// tags:
			//		protected
			return this.dir ? (this.dir == "ltr") : domGeometry.isBodyLtr(this.ownerDocument); //Boolean
		},

		isFocusable: function(){
			// summary:
			//		Return true if this widget can currently be focused
			//		and false if not
			return this.focus && (domStyle.get(this.domNode, "display") != "none");
		},

		placeAt: function(/* String|DomNode|_Widget */ reference, /* String|Int? */ position){
			// summary:
			//		Place this widget somewhere in the DOM based
			//		on standard domConstruct.place() conventions.
			// description:
			//		A convenience function provided in all _Widgets, providing a simple
			//		shorthand mechanism to put an existing (or newly created) Widget
			//		somewhere in the dom, and allow chaining.
			// reference:
			//		Widget, DOMNode, or id of widget or DOMNode
			// position:
			//		If reference is a widget (or id of widget), and that widget has an ".addChild" method,
			//		it will be called passing this widget instance into that method, supplying the optional
			//		position index passed.  In this case position (if specified) should be an integer.
			//
			//		If reference is a DOMNode (or id matching a DOMNode but not a widget),
			//		the position argument can be a numeric index or a string
			//		"first", "last", "before", or "after", same as dojo/dom-construct::place().
			// returns: dijit/_WidgetBase
			//		Provides a useful return of the newly created dijit._Widget instance so you
			//		can "chain" this function by instantiating, placing, then saving the return value
			//		to a variable.
			// example:
			//	|	// create a Button with no srcNodeRef, and place it in the body:
			//	|	var button = new Button({ label:"click" }).placeAt(win.body());
			//	|	// now, 'button' is still the widget reference to the newly created button
			//	|	button.on("click", function(e){ console.log('click'); }));
			// example:
			//	|	// create a button out of a node with id="src" and append it to id="wrapper":
			//	|	var button = new Button({},"src").placeAt("wrapper");
			// example:
			//	|	// place a new button as the first element of some div
			//	|	var button = new Button({ label:"click" }).placeAt("wrapper","first");
			// example:
			//	|	// create a contentpane and add it to a TabContainer
			//	|	var tc = dijit.byId("myTabs");
			//	|	new ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

			var refWidget = !reference.tagName && registry.byId(reference);
			if(refWidget && refWidget.addChild && (!position || typeof position === "number")){
				// Adding this to refWidget and can use refWidget.addChild() to handle everything.
				refWidget.addChild(this, position);
			}else{
				// "reference" is a plain DOMNode, or we can't use refWidget.addChild().   Use domConstruct.place() and
				// target refWidget.containerNode for nested placement (position==number, "first", "last", "only"), and
				// refWidget.domNode otherwise ("after"/"before"/"replace").  (But not supported officially, see #14946.)
				var ref = refWidget ?
					(refWidget.containerNode && !/after|before|replace/.test(position || "") ?
						refWidget.containerNode : refWidget.domNode) : dom.byId(reference, this.ownerDocument);
				domConstruct.place(this.domNode, ref, position);

				// Start this iff it has a parent widget that's already started.
				// TODO: for 2.0 maybe it should also start the widget when this.getParent() returns null??
				if(!this._started && (this.getParent() || {})._started){
					this.startup();
				}
			}
			return this;
		},

		defer: function(fcn, delay){
			// summary:
			//		Wrapper to setTimeout to avoid deferred functions executing
			//		after the originating widget has been destroyed.
			//		Returns an object handle with a remove method (that returns null) (replaces clearTimeout).
			// fcn: function reference
			// delay: Optional number (defaults to 0)
			// tags:
			//		protected.
			var timer = setTimeout(lang.hitch(this,
				function(){
					if(!timer){
						return;
					}
					timer = null;
					if(!this._destroyed){
						lang.hitch(this, fcn)();
					}
				}),
				delay || 0
			);
			return {
				remove: function(){
					if(timer){
						clearTimeout(timer);
						timer = null;
					}
					return null; // so this works well: handle = handle.remove();
				}
			};
		}
	});

	if(has("dojo-bidi")){
		_WidgetBase.extend(_BidiMixin);
	}

	return _WidgetBase;
});

},
'dojo/Stateful':function(){
define(["./_base/declare", "./_base/lang", "./_base/array", "./when"], function(declare, lang, array, when){
	// module:
	//		dojo/Stateful

return declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	//
	//		The class also provides the functionality to auto-magically manage getters
	//		and setters for object attributes/properties.
	//		
	//		Getters and Setters should follow the format of _xxxGetter or _xxxSetter where 
	//		the xxx is a name of the attribute to handle.  So an attribute of "foo" 
	//		would have a custom getter of _fooGetter and a custom setter of _fooSetter.
	//
	// example:
	//	|	var obj = new dojo.Stateful();
	//	|	obj.watch("foo", function(){
	//	|		console.log("foo changed to " + this.get("foo"));
	//	|	});
	//	|	obj.set("foo","bar");

	// _attrPairNames: Hash
	//		Used across all instances a hash to cache attribute names and their getter 
	//		and setter names.
	_attrPairNames: {},

	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		return (apn[name] = {
			s: "_" + name + "Setter",
			g: "_" + name + "Getter"
		});
	},

	postscript: function(/*Object?*/ params){
		// Automatic setting of params during construction
		if (params){ this.set(params); }
	},

	_get: function(name, names){
		// summary:
		//		Private function that does a get based off a hash of names
		// names:
		//		Hash of names of custom attributes
		return typeof this[names.g] === "function" ? this[names.g]() : this[name];
	},
	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		// name:
		//		The property to get.
		// returns:
		//		The property value on this Stateful instance.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		//		this just retrieves the object's property.
		//		For example:
		//	|	stateful = new dojo.Stateful({foo: 3});
		//	|	stateful.get("foo") // returns 3
		//	|	stateful.foo // returns 3

		return this._get(name, this._getAttrNames(name)); //Any
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		// name:
		//		The property to set.
		// value:
		//		The value to set in the property.
		// returns:
		//		The function returns this dojo.Stateful instance.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		//		the property. A programmatic setter may be defined in subclasses.
		//		For example:
		//	|	stateful = new dojo.Stateful();
		//	|	stateful.watch(function(name, oldValue, value){
		//	|		// this will be called on the set below
		//	|	}
		//	|	stateful.set(foo, 5);
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myObj.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		// If an object is used, iterate through object
		if(typeof name === "object"){
			for(var x in name){
				if(name.hasOwnProperty(x) && x !="_watchCallbacks"){
					this.set(x, name[x]);
				}
			}
			return this;
		}

		var names = this._getAttrNames(name),
			oldValue = this._get(name, names),
			setter = this[names.s],
			result;
		if(typeof setter === "function"){
			// use the explicit setter
			result = setter.apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// no setter so set attribute directly
			this[name] = value;
		}
		if(this._watchCallbacks){
			var self = this;
			// If setter returned a promise, wait for it to complete, otherwise call watches immediatly
			when(result, function(){
				self._watchCallbacks(name, oldValue, value);
			});
		}
		return this; // dojo/Stateful
	},
	_changeAttrValue: function(name, value){
		// summary:
		//		Internal helper for directly changing an attribute value.
		//
		// name: String
		//		The property to set.
		// value: Mixed
		//		The value to set in the property.
		//
		// description:
		//		Directly change the value of an attribute on an object, bypassing any 
		//		accessor setter.  Also handles the calling of watch and emitting events. 
		//		It is designed to be used by descendent class when there are two values 
		//		of attributes that are linked, but calling .set() is not appropriate.

		var oldValue = this.get(name);
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this; // dojo/Stateful
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		// name:
		//		Indicates the property to watch. This is optional (the callback may be the
		//		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		//		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		// callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		//		second argument as the old value and the third argument as the new value.

		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
						propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							propertyCallbacks[i].call(self, name, oldValue, value);
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);

		// TODO: Remove unwatch in 2.0
		var handle = {};
		handle.unwatch = handle.remove = function(){
			var index = array.indexOf(propertyCallbacks, callback);
			if(index > -1){
				propertyCallbacks.splice(index, 1);
			}
		};
		return handle; //Object
	}

});

});

},
'dijit/Destroyable':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/aspect",
	"dojo/_base/declare"
], function(array, aspect, declare){

	// module:
	//		dijit/Destroyable

	return declare("dijit.Destroyable", null, {
		// summary:
		//		Mixin to track handles and release them when instance is destroyed.
		// description:
		//		Call this.own(...) on list of handles (returned from dojo/aspect, dojo/on,
		//		dojo/Stateful::watch, or any class (including widgets) with a destroyRecursive() or destroy() method.
		//		Then call destroy() later to destroy this instance and release the resources.

		destroy: function(/*Boolean*/ preserveDom){
			// summary:
			//		Destroy this class, releasing any resources registered via own().
			this._destroyed = true;
		},

		own: function(){
			// summary:
			//		Track specified handles and remove/destroy them when this instance is destroyed, unless they were
			//		already removed/destroyed manually.
			// tags:
			//		protected
			// returns:
			//		The array of specified handles, so you can do for example:
			//	|		var handle = this.own(on(...))[0];

			array.forEach(arguments, function(handle){
				var destroyMethodName =
					"destroyRecursive" in handle ? "destroyRecursive" : // remove "destroyRecursive" for 2.0
						"destroy" in handle ? "destroy" :
							"remove";

				// When this.destroy() is called, destroy handle.  Since I'm using aspect.before(),
				// the handle will be destroyed before a subclass's destroy() method starts running, before it calls
				// this.inherited() or even if it doesn't call this.inherited() at all.  If that's an issue, make an
				// onDestroy() method and connect to that instead.
				var odh = aspect.before(this, "destroy", function(preserveDom){
					handle[destroyMethodName](preserveDom);
				});

				// If handle is destroyed manually before this.destroy() is called, remove the listener set directly above.
				var hdh = aspect.after(handle, destroyMethodName, function(){
					odh.remove();
					hdh.remove();
				}, true);
			}, this);

			return arguments;		// handle
		}
	});
});

},
'dijit/_OnDijitClickMixin':function(){
define([
	"dojo/on",
	"dojo/_base/array", // array.forEach
	"dojo/keys", // keys.ENTER keys.SPACE
	"dojo/_base/declare", // declare
	"dojo/has", // has("dom-addeventlistener")
	"./a11yclick"
], function(on, array, keys, declare, has, a11yclick){

	// module:
	//		dijit/_OnDijitClickMixin

	var ret = declare("dijit._OnDijitClickMixin", null, {
		// summary:
		//		Deprecated.   New code should access the dijit/a11yclick event directly, ex:
		//		|	this.own(on(node, a11yclick, function(){ ... }));
		//
		//		Mixing in this class will make _WidgetBase.connect(node, "ondijitclick", ...) work.
		//		It also used to be necessary to make templates with ondijitclick work, but now you can just require
		//		dijit/a11yclick.

		connect: function(obj, event, method){
			// override _WidgetBase.connect() to make this.connect(node, "ondijitclick", ...) work
			return this.inherited(arguments, [obj, event == "ondijitclick" ? a11yclick : event, method]);
		}
	});

	ret.a11yclick = a11yclick;	// back compat

	return ret;
});

},
'dijit/a11yclick':function(){
define([
	"dojo/keys", // keys.ENTER keys.SPACE
	"dojo/mouse",
	"dojo/on",
	"dojo/touch" // touch support for click is now there
], function(keys, mouse, on, touch){

	// module:
	//		dijit/a11yclick

	/*=====
	return {
		// summary:
		//		Custom press, release, and click synthetic events
		//		which trigger on a left mouse click, touch, or space/enter keyup.

		click: function(node, listener){
			// summary:
			//		Logical click operation for mouse, touch, or keyboard (space/enter key)
		},
		press: function(node, listener){
			// summary:
			//		Mousedown (left button), touchstart, or keydown (space or enter) corresponding to logical click operation.
		},
		release: function(node, listener){
			// summary:
			//		Mouseup (left button), touchend, or keyup (space or enter) corresponding to logical click operation.
		},
		move: function(node, listener){
			// summary:
			//		Mouse cursor or a finger is dragged over the given node.
		}
	};
	=====*/

	function clickKey(/*Event*/ e){
		// Test if this keyboard event should be tracked as the start (if keydown) or end (if keyup) of a click event.
		// Only track for nodes marked to be tracked, and not for buttons or inputs,
		// since buttons handle keyboard click natively, and text inputs should not
		// prevent typing spaces or newlines.
		if((e.keyCode === keys.ENTER || e.keyCode === keys.SPACE) && !/input|button|textarea/i.test(e.target.nodeName)){

			// Test if a node or its ancestor has been marked with the dojoClick property to indicate special processing
			for(var node = e.target; node; node = node.parentNode){
				if(node.dojoClick){ return true; }
			}
		}
	}

	var lastKeyDownNode;

	on(document, "keydown", function(e){
		//console.log("a11yclick: onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
		if(clickKey(e)){
			// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
			lastKeyDownNode = e.target;

			// Prevent viewport scrolling on space key in IE<9.
			// (Reproducible on test_Button.html on any of the first dijit/form/Button examples)
			e.preventDefault();
		}else{
			lastKeyDownNode = null;
		}
	});

	on(document, "keyup", function(e){
		//console.log("a11yclick: onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", lastKeyDownNode, ", equality is ", (e.target === lastKeyDownNode));
		if(clickKey(e) && e.target == lastKeyDownNode){	// === breaks greasemonkey
			//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
			lastKeyDownNode = null;

			on.emit(e.target, "click", {
				cancelable: true,
				bubbles: true,
				ctrlKey: e.ctrlKey,
				shiftKey: e.shiftKey,
				metaKey: e.metaKey,
				altKey: e.altKey,
				_origType: e.type
			});
		}
	});

	// I want to return a hash of the synthetic events, but for backwards compatibility the main return value
	// needs to be the click event.   Change for 2.0.

	var click = function(node, listener){
		// Set flag on node so that keydown/keyup above emits click event
		node.dojoClick = true;

		return on(node, "click", listener);
	};
	click.click = click;	// forward compatibility with 2.0

	click.press =  function(node, listener){
		var touchListener = on(node, touch.press, function(evt){
			if(evt.type == "mousedown" && !mouse.isLeft(evt)){
				// Ignore right click
				return;
			}
			listener(evt);
		}), keyListener = on(node, "keydown", function(evt){
			if(evt.keyCode === keys.ENTER || evt.keyCode === keys.SPACE){
				listener(evt);
			}
		});
		return {
			remove: function(){
				touchListener.remove();
				keyListener.remove();
			}
		};
	};

	click.release =  function(node, listener){
		var touchListener = on(node, touch.release, function(evt){
			if(evt.type == "mouseup" && !mouse.isLeft(evt)){
				// Ignore right click
				return;
			}
			listener(evt);
		}), keyListener = on(node, "keyup", function(evt){
			if(evt.keyCode === keys.ENTER || evt.keyCode === keys.SPACE){
				listener(evt);
			}
		});
		return {
			remove: function(){
				touchListener.remove();
				keyListener.remove();
			}
		};
	};

	click.move = touch.move;	// just for convenience

	return click;
});

},
'dojo/touch':function(){
define(["./_base/kernel", "./aspect", "./dom", "./dom-class", "./_base/lang", "./on", "./has", "./mouse", "./domReady", "./_base/window"],
function(dojo, aspect, dom, domClass, lang, on, has, mouse, domReady, win){

	// module:
	//		dojo/touch

	var hasTouch = has("touch");

	var ios4 = has("ios") < 5;
	
	var msPointer = navigator.msPointerEnabled;

	// Click generation variables
	var clicksInited, clickTracker, clickTarget, clickX, clickY, clickDx, clickDy, clickTime;

	// Time of most recent touchstart, touchmove, or touchend event
	var lastTouch;

	function dualEvent(mouseType, touchType, msPointerType){
		// Returns synthetic event that listens for both the specified mouse event and specified touch event.
		// But ignore fake mouse events that were generated due to the user touching the screen.
		if(msPointer && msPointerType){
			// IE10+: MSPointer* events are designed to handle both mouse and touch in a uniform way,
			// so just use that regardless of hasTouch.
			return function(node, listener){
				return on(node, msPointerType, listener);
			}
		}else if(hasTouch){
			return function(node, listener){
				var handle1 = on(node, touchType, listener),
					handle2 = on(node, mouseType, function(evt){
						if(!lastTouch || (new Date()).getTime() > lastTouch + 1000){
							listener.call(this, evt);
						}
					});
				return {
					remove: function(){
						handle1.remove();
						handle2.remove();
					}
				};
			};
		}else{
			// Avoid creating listeners for touch events on performance sensitive older browsers like IE6
			return function(node, listener){
				return on(node, mouseType, listener);
			}
		}
	}

	function marked(/*DOMNode*/ node){
		// Test if a node or its ancestor has been marked with the dojoClick property to indicate special processing,
		do{
			if(node.dojoClick){ return node.dojoClick; }
		}while(node = node.parentNode);
	}
	
	function doClicks(e, moveType, endType){
		// summary:
		//		Setup touch listeners to generate synthetic clicks immediately (rather than waiting for the browser
		//		to generate clicks after the double-tap delay) and consistently (regardless of whether event.preventDefault()
		//		was called in an event listener. Synthetic clicks are generated only if a node or one of its ancestors has
		//		its dojoClick property set to truthy.
		
		clickTracker  = !e.target.disabled && marked(e.target); // click threshold = true, number or x/y object
		if(clickTracker){
			clickTarget = e.target;
			clickX = e.touches ? e.touches[0].pageX : e.clientX;
			clickY = e.touches ? e.touches[0].pageY : e.clientY;
			clickDx = (typeof clickTracker == "object" ? clickTracker.x : (typeof clickTracker == "number" ? clickTracker : 0)) || 4;
			clickDy = (typeof clickTracker == "object" ? clickTracker.y : (typeof clickTracker == "number" ? clickTracker : 0)) || 4;

			// add move/end handlers only the first time a node with dojoClick is seen,
			// so we don't add too much overhead when dojoClick is never set.
			if(!clicksInited){
				clicksInited = true;

				win.doc.addEventListener(moveType, function(e){
					clickTracker = clickTracker &&
						e.target == clickTarget &&
						Math.abs((e.touches ? e.touches[0].pageX : e.clientX) - clickX) <= clickDx &&
						Math.abs((e.touches ? e.touches[0].pageY : e.clientY) - clickY) <= clickDy;
				}, true);

				win.doc.addEventListener(endType, function(e){
					if(clickTracker){
						clickTime = (new Date()).getTime();
						var target = e.target;
						if(target.tagName === "LABEL"){
							// when clicking on a label, forward click to its associated input if any
							target = dom.byId(target.getAttribute("for")) || target;
						}
						setTimeout(function(){
							on.emit(target, "click", {
								bubbles : true,
								cancelable : true,
								_dojo_click : true
							});
						});
					}
				}, true);

				function stopNativeEvents(type){
					win.doc.addEventListener(type, function(e){
						// Stop native events when we emitted our own click event.  Note that the native click may occur
						// on a different node than the synthetic click event was generated on.  For example,
						// click on a menu item, causing the menu to disappear, and then (~300ms later) the browser
						// sends a click event to the node that was *underneath* the menu.  So stop all native events
						// sent shortly after ours, similar to what is done in dualEvent.
						// The INPUT.dijitOffScreen test is for offscreen inputs used in dijit/form/Button, on which
						// we call click() explicitly, we don't want to stop this event.
						if(!e._dojo_click &&
								(new Date()).getTime() <= clickTime + 1000 &&
								!(e.target.tagName == "INPUT" && domClass.contains(e.target, "dijitOffScreen"))){
							e.stopPropagation();
							e.stopImmediatePropagation && e.stopImmediatePropagation();
							if(type == "click" && (e.target.tagName != "INPUT" || e.target.type == "radio" || e.target.type == "checkbox")
								&& e.target.tagName != "TEXTAREA" && e.target.tagName != "AUDIO" && e.target.tagName != "VIDEO"){
								 // preventDefault() breaks textual <input>s on android, keyboard doesn't popup,
								 // but it is still needed for checkboxes and radio buttons, otherwise in some cases
								 // the checked state becomes inconsistent with the widget's state
								e.preventDefault();
							}
						}
					}, true);
				}

				stopNativeEvents("click");

				// We also stop mousedown/up since these would be sent well after with our "fast" click (300ms),
				// which can confuse some dijit widgets.
				stopNativeEvents("mousedown");
				stopNativeEvents("mouseup");
			}
		}
	}

	var hoveredNode;

	if(hasTouch){
		if(msPointer){
			 // MSPointer (IE10+) already has support for over and out, so we just need to init click support
			domReady(function(){
				win.doc.addEventListener("MSPointerDown", function(evt){
					doClicks(evt, "MSPointerMove", "MSPointerUp");
				}, true);
			});		
		}else{
			domReady(function(){
				// Keep track of currently hovered node
				hoveredNode = win.body();	// currently hovered node

				win.doc.addEventListener("touchstart", function(evt){
					lastTouch = (new Date()).getTime();

					// Precede touchstart event with touch.over event.  DnD depends on this.
					// Use addEventListener(cb, true) to run cb before any touchstart handlers on node run,
					// and to ensure this code runs even if the listener on the node does event.stop().
					var oldNode = hoveredNode;
					hoveredNode = evt.target;
					on.emit(oldNode, "dojotouchout", {
						relatedTarget: hoveredNode,
						bubbles: true
					});
					on.emit(hoveredNode, "dojotouchover", {
						relatedTarget: oldNode,
						bubbles: true
					});
				
					doClicks(evt, "touchmove", "touchend"); // init click generation
				}, true);

				function copyEventProps(evt){
					// Make copy of event object and also set bubbles:true.  Used when calling on.emit().
					var props = lang.delegate(evt, {
						bubbles: true
					});

					if(has("ios") >= 6){
						// On iOS6 "touches" became a non-enumerable property, which 
						// is not hit by for...in.  Ditto for the other properties below.
						props.touches = evt.touches;
						props.altKey = evt.altKey;
						props.changedTouches = evt.changedTouches;
						props.ctrlKey = evt.ctrlKey;
						props.metaKey = evt.metaKey;
						props.shiftKey = evt.shiftKey;
						props.targetTouches = evt.targetTouches;
					}

					return props;
				}
				
				on(win.doc, "touchmove", function(evt){
					lastTouch = (new Date()).getTime();

					var newNode = win.doc.elementFromPoint(
						evt.pageX - (ios4 ? 0 : win.global.pageXOffset), // iOS 4 expects page coords
						evt.pageY - (ios4 ? 0 : win.global.pageYOffset)
					);

					if(newNode){
						// Fire synthetic touchover and touchout events on nodes since the browser won't do it natively.
						if(hoveredNode !== newNode){
							// touch out on the old node
							on.emit(hoveredNode, "dojotouchout", {
								relatedTarget: newNode,
								bubbles: true
							});

							// touchover on the new node
							on.emit(newNode, "dojotouchover", {
								relatedTarget: hoveredNode,
								bubbles: true
							});

							hoveredNode = newNode;
						}

						// Unlike a listener on "touchmove", on(node, "dojotouchmove", listener) fires when the finger
						// drags over the specified node, regardless of which node the touch started on.
						on.emit(newNode, "dojotouchmove", copyEventProps(evt));
					}
				});

				// Fire a dojotouchend event on the node where the finger was before it was removed from the screen.
				// This is different than the native touchend, which fires on the node where the drag started.
				on(win.doc, "touchend", function(evt){
					lastTouch = (new Date()).getTime();
					var node = win.doc.elementFromPoint(
						evt.pageX - (ios4 ? 0 : win.global.pageXOffset), // iOS 4 expects page coords
						evt.pageY - (ios4 ? 0 : win.global.pageYOffset)
					) || win.body(); // if out of the screen

					on.emit(node, "dojotouchend", copyEventProps(evt));
				});
			});
		}
	}

	//device neutral events - touch.press|move|release|cancel/over/out
	var touch = {
		press: dualEvent("mousedown", "touchstart", "MSPointerDown"),
		move: dualEvent("mousemove", "dojotouchmove", "MSPointerMove"),
		release: dualEvent("mouseup", "dojotouchend", "MSPointerUp"),
		cancel: dualEvent(mouse.leave, "touchcancel", hasTouch?"MSPointerCancel":null),
		over: dualEvent("mouseover", "dojotouchover", "MSPointerOver"),
		out: dualEvent("mouseout", "dojotouchout", "MSPointerOut"),
		enter: mouse._eventHandler(dualEvent("mouseover","dojotouchover", "MSPointerOver")),
		leave: mouse._eventHandler(dualEvent("mouseout", "dojotouchout", "MSPointerOut"))
	};

	/*=====
	touch = {
		// summary:
		//		This module provides unified touch event handlers by exporting
		//		press, move, release and cancel which can also run well on desktop.
		//		Based on http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html
		//		Also, if the dojoClick property is set to true on a DOM node, dojo/touch generates
		//		click events immediately for this node and its descendants, to avoid the
		//		delay before native browser click events, and regardless of whether evt.preventDefault()
		//		was called in a touch.press event listener.
		//
		// example:
		//		Used with dojo.on
		//		|	define(["dojo/on", "dojo/touch"], function(on, touch){
		//		|		on(node, touch.press, function(e){});
		//		|		on(node, touch.move, function(e){});
		//		|		on(node, touch.release, function(e){});
		//		|		on(node, touch.cancel, function(e){});
		// example:
		//		Used with touch.* directly
		//		|	touch.press(node, function(e){});
		//		|	touch.move(node, function(e){});
		//		|	touch.release(node, function(e){});
		//		|	touch.cancel(node, function(e){});
		// example:
		//		Have dojo/touch generate clicks without delay, with a default move threshold of 4 pixels
		//		|	node.dojoClick = true;
		// example:
		//		Have dojo/touch generate clicks without delay, with a move threshold of 10 pixels horizontally and vertically
		//		|	node.dojoClick = 10;
		// example:
		//		Have dojo/touch generate clicks without delay, with a move threshold of 50 pixels horizontally and 10 pixels vertically
		//		|	node.dojoClick = {x:50, y:5};
		

		press: function(node, listener){
			// summary:
			//		Register a listener to 'touchstart'|'mousedown' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		move: function(node, listener){
			// summary:
			//		Register a listener that fires when the mouse cursor or a finger is dragged over the given node.
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		release: function(node, listener){
			// summary:
			//		Register a listener to releasing the mouse button while the cursor is over the given node
			//		(i.e. "mouseup") or for removing the finger from the screen while touching the given node.
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		cancel: function(node, listener){
			// summary:
			//		Register a listener to 'touchcancel'|'mouseleave' for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		over: function(node, listener){
			// summary:
			//		Register a listener to 'mouseover' or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		out: function(node, listener){
			// summary:
			//		Register a listener to 'mouseout' or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		enter: function(node, listener){
			// summary:
			//		Register a listener to mouse.enter or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		},
		leave: function(node, listener){
			// summary:
			//		Register a listener to mouse.leave or touch equivalent for the given node
			// node: Dom
			//		Target node to listen to
			// listener: Function
			//		Callback function
			// returns:
			//		A handle which will be used to remove the listener by handle.remove()
		}
	};
	=====*/

	 1  && (dojo.touch = touch);

	return touch;
});

},
'dijit/_FocusMixin':function(){
define([
	"./focus",
	"./_WidgetBase",
	"dojo/_base/declare", // declare
	"dojo/_base/lang" // lang.extend
], function(focus, _WidgetBase, declare, lang){

	// module:
	//		dijit/_FocusMixin

	// We don't know where _FocusMixin will occur in the inheritance chain, but we need the _onFocus()/_onBlur() below
	// to be last in the inheritance chain, so mixin to _WidgetBase.
	lang.extend(_WidgetBase, {
		// focused: [readonly] Boolean
		//		This widget or a widget it contains has focus, or is "active" because
		//		it was recently clicked.
		focused: false,

		onFocus: function(){
			// summary:
			//		Called when the widget becomes "active" because
			//		it or a widget inside of it either has focus, or has recently
			//		been clicked.
			// tags:
			//		callback
		},

		onBlur: function(){
			// summary:
			//		Called when the widget stops being "active" because
			//		focus moved to something outside of it, or the user
			//		clicked somewhere outside of it, or the widget was
			//		hidden.
			// tags:
			//		callback
		},

		_onFocus: function(){
			// summary:
			//		This is where widgets do processing for when they are active,
			//		such as changing CSS classes.  See onFocus() for more details.
			// tags:
			//		protected
			this.onFocus();
		},

		_onBlur: function(){
			// summary:
			//		This is where widgets do processing for when they stop being active,
			//		such as changing CSS classes.  See onBlur() for more details.
			// tags:
			//		protected
			this.onBlur();
		}
	});

	return declare("dijit._FocusMixin", null, {
		// summary:
		//		Mixin to widget to provide _onFocus() and _onBlur() methods that
		//		fire when a widget or its descendants get/lose focus

		// flag that I want _onFocus()/_onBlur() notifications from focus manager
		_focusManager: focus
	});

});

},
'dijit/focus':function(){
define([
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/dom", // domAttr.get dom.isDescendant
	"dojo/dom-attr", // domAttr.get dom.isDescendant
	"dojo/dom-construct", // connect to domConstruct.empty, domConstruct.destroy
	"dojo/Evented",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/domReady",
	"dojo/sniff", // has("ie")
	"dojo/Stateful",
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.get
	"./a11y",	// a11y.isTabNavigable
	"./registry",	// registry.byId
	"./main"		// to set dijit.focus
], function(aspect, declare, dom, domAttr, domConstruct, Evented, lang, on, domReady, has, Stateful, win, winUtils,
			a11y, registry, dijit){

	// module:
	//		dijit/focus

	var lastFocusin;

	var FocusManager = declare([Stateful, Evented], {
		// summary:
		//		Tracks the currently focused node, and which widgets are currently "active".
		//		Access via require(["dijit/focus"], function(focus){ ... }).
		//
		//		A widget is considered active if it or a descendant widget has focus,
		//		or if a non-focusable node of this widget or a descendant was recently clicked.
		//
		//		Call focus.watch("curNode", callback) to track the current focused DOMNode,
		//		or focus.watch("activeStack", callback) to track the currently focused stack of widgets.
		//
		//		Call focus.on("widget-blur", func) or focus.on("widget-focus", ...) to monitor when
		//		when widgets become active/inactive
		//
		//		Finally, focus(node) will focus a node, suppressing errors if the node doesn't exist.

		// curNode: DomNode
		//		Currently focused item on screen
		curNode: null,

		// activeStack: dijit/_WidgetBase[]
		//		List of currently active widgets (focused widget and it's ancestors)
		activeStack: [],

		constructor: function(){
			// Don't leave curNode/prevNode pointing to bogus elements
			var check = lang.hitch(this, function(node){
				if(dom.isDescendant(this.curNode, node)){
					this.set("curNode", null);
				}
				if(dom.isDescendant(this.prevNode, node)){
					this.set("prevNode", null);
				}
			});
			aspect.before(domConstruct, "empty", check);
			aspect.before(domConstruct, "destroy", check);
		},

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the `<iframe>` itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle with remove() method to deregister.
			return this.registerWin(iframe.contentWindow, iframe);
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle with remove() method to deregister.

			// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

			// Listen for blur and focus events on targetWindow's document.
			var _this = this,
				body = targetWindow.document && targetWindow.document.body;

			if(body){
				var mdh = on(targetWindow.document, 'mousedown, touchstart', function(evt){
					_this._justMouseDowned = true;
					setTimeout(function(){ _this._justMouseDowned = false; }, 0);

					// workaround weird IE bug where the click is on an orphaned node
					// (first time clicking a Select/DropDownButton inside a TooltipDialog).
					// actually, strangely this is happening on latest chrome too.
					if(evt && evt.target && evt.target.parentNode == null){
						return;
					}

					_this._onTouchNode(effectiveNode || evt.target, "mouse");
				});

				var fih = on(body, 'focusin', function(evt){

					lastFocusin = (new Date()).getTime();

					// When you refocus the browser window, IE gives an event with an empty srcElement
					if(!evt.target.tagName) { return; }

					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// ignore those events
					var tag = evt.target.tagName.toLowerCase();
					if(tag == "#document" || tag == "body"){ return; }

					if(a11y.isTabNavigable(evt.target)){
						// If condition doesn't seem quite right, but it is correctly preventing focus events for
						// clicks on disabled buttons.  (TODO: it doesn't register clicks on TabContainer tabs because
						// they are tabIndex="-1")
						_this._onFocusNode(effectiveNode || evt.target);
					}else{
						// Previous code called _onTouchNode() for any activate event on a non-focusable node.   Can
						// probably just ignore such an event as it will be handled by onmousedown handler above, but
						// leaving the code for now.
						_this._onTouchNode(effectiveNode || evt.target);
					}
				});

				var foh = on(body, 'focusout', function(evt){
					// IE9+ has a problem where focusout events come after the corresponding focusin event.  At least
					// when moving focus from the Editor's <iframe> to a normal DOMNode.
					if((new Date()).getTime() < lastFocusin + 100){
						return;
					}

					_this._onBlurNode(effectiveNode || evt.target);
				});

				return {
					remove: function(){
						mdh.remove();
						fih.remove();
						foh.remove();
						mdh = fih = foh = null;
						body = null;	// prevent memory leak (apparent circular reference via closure)
					}
				};
			}
		},

		_onBlurNode: function(/*DomNode*/ node){
			// summary:
			//		Called when focus leaves a node.
			//		Usually ignored, _unless_ it *isn't* followed by touching another node,
			//		which indicates that we tabbed off the last field on the page,
			//		in which case every widget is marked inactive

			// If the blur event isn't followed by a focus event, it means the user clicked on something unfocusable,
			// so clear focus.
			if(this._clearFocusTimer){
				clearTimeout(this._clearFocusTimer);
			}
			this._clearFocusTimer = setTimeout(lang.hitch(this, function(){
				this.set("prevNode", this.curNode);
				this.set("curNode", null);
			}), 0);

			if(this._justMouseDowned){
				// the mouse down caused a new widget to be marked as active; this blur event
				// is coming late, so ignore it.
				return;
			}

			// If the blur event isn't followed by a focus or touch event then mark all widgets as inactive.
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
			}
			this._clearActiveWidgetsTimer = setTimeout(lang.hitch(this, function(){
				delete this._clearActiveWidgetsTimer;
				this._setStack([]);
			}), 0);
		},

		_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
			// summary:
			//		Callback when node is focused or mouse-downed
			// node:
			//		The node that was touched.
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			// ignore the recent blurNode event
			if(this._clearActiveWidgetsTimer){
				clearTimeout(this._clearActiveWidgetsTimer);
				delete this._clearActiveWidgetsTimer;
			}

			// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
			var newStack=[];
			try{
				while(node){
					var popupParent = domAttr.get(node, "dijitPopupParent");
					if(popupParent){
						node=registry.byId(popupParent).domNode;
					}else if(node.tagName && node.tagName.toLowerCase() == "body"){
						// is this the root of the document or just the root of an iframe?
						if(node === win.body()){
							// node is the root of the main document
							break;
						}
						// otherwise, find the iframe this node refers to (can't access it via parentNode,
						// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
						node=winUtils.get(node.ownerDocument).frameElement;
					}else{
						// if this node is the root node of a widget, then add widget id to stack,
						// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
						// to support MenuItem)
						var id = node.getAttribute && node.getAttribute("widgetId"),
							widget = id && registry.byId(id);
						if(widget && !(by == "mouse" && widget.get("disabled"))){
							newStack.unshift(id);
						}
						node=node.parentNode;
					}
				}
			}catch(e){ /* squelch */ }

			this._setStack(newStack, by);
		},

		_onFocusNode: function(/*DomNode*/ node){
			// summary:
			//		Callback when node is focused

			if(!node){
				return;
			}

			if(node.nodeType == 9){
				// Ignore focus events on the document itself.  This is here so that
				// (for example) clicking the up/down arrows of a spinner
				// (which don't get focus) won't cause that widget to blur. (FF issue)
				return;
			}

			// There was probably a blur event right before this event, but since we have a new focus, don't
			// do anything with the blur
			if(this._clearFocusTimer){
				clearTimeout(this._clearFocusTimer);
				delete this._clearFocusTimer;
			}

			this._onTouchNode(node);

			if(node == this.curNode){ return; }
			this.set("prevNode", this.curNode);
			this.set("curNode", node);
		},

		_setStack: function(/*String[]*/ newStack, /*String*/ by){
			// summary:
			//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
			// newStack:
			//		array of widget id's, starting from the top (outermost) widget
			// by:
			//		"mouse" if the focus/touch was caused by a mouse down event

			var oldStack = this.activeStack, lastOldIdx = oldStack.length - 1, lastNewIdx = newStack.length - 1;

			if(newStack[lastNewIdx] == oldStack[lastOldIdx]){
				// no changes, return now to avoid spurious notifications about changes to activeStack
				return;
			}

			this.set("activeStack", newStack);

			var widget, i;

			// for all elements that have gone out of focus, set focused=false
			for(i = lastOldIdx; i >= 0 && oldStack[i] != newStack[i]; i--){
				widget = registry.byId(oldStack[i]);
				if(widget){
					widget._hasBeenBlurred = true;		// TODO: used by form widgets, should be moved there
					widget.set("focused", false);
					if(widget._focusManager == this){
						widget._onBlur(by);
					}
					this.emit("widget-blur", widget, by);
				}
			}

			// for all element that have come into focus, set focused=true
			for(i++; i <= lastNewIdx; i++){
				widget = registry.byId(newStack[i]);
				if(widget){
					widget.set("focused", true);
					if(widget._focusManager == this){
						widget._onFocus(by);
					}
					this.emit("widget-focus", widget, by);
				}
			}
		},

		focus: function(node){
			// summary:
			//		Focus the specified node, suppressing errors if they occur
			if(node){
				try{ node.focus(); }catch(e){/*quiet*/}
			}
		}
	});

	var singleton = new FocusManager();

	// register top window and all the iframes it contains
	domReady(function(){
		var handle = singleton.registerWin(winUtils.get(document));
		if(has("ie")){
			on(window, "unload", function(){
				if(handle){	// because this gets called twice when doh.robot is running
					handle.remove();
					handle = null;
				}
			});
		}
	});

	// Setup dijit.focus as a pointer to the singleton but also (for backwards compatibility)
	// as a function to set focus.   Remove for 2.0.
	dijit.focus = function(node){
		singleton.focus(node);	// indirection here allows dijit/_base/focus.js to override behavior
	};
	for(var attr in singleton){
		if(!/^_/.test(attr)){
			dijit.focus[attr] = typeof singleton[attr] == "function" ? lang.hitch(singleton, attr) : singleton[attr];
		}
	}
	singleton.watch(function(attr, oldVal, newVal){
		dijit.focus[attr] = newVal;
	});

	return singleton;
});

},
'dijit/a11y':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/dom",			// dom.byId
	"dojo/dom-attr", // domAttr.attr domAttr.has
	"dojo/dom-style", // domStyle.style
	"dojo/_base/lang", // lang.mixin()
	"dojo/sniff", // has("ie")  1 
	"./main"	// for exporting methods to dijit namespace
], function(array, dom, domAttr, domStyle, lang, has, dijit){

	// module:
	//		dijit/a11y

	var a11y = {
		// summary:
		//		Accessibility utility functions (keyboard, tab stops, etc.)

		_isElementShown: function(/*Element*/ elem){
			var s = domStyle.get(elem);
			return (s.visibility != "hidden")
				&& (s.visibility != "collapsed")
				&& (s.display != "none")
				&& (domAttr.get(elem, "type") != "hidden");
		},

		hasDefaultTabStop: function(/*Element*/ elem){
			// summary:
			//		Tests if element is tab-navigable even without an explicit tabIndex setting

			// No explicit tabIndex setting, need to investigate node type
			switch(elem.nodeName.toLowerCase()){
				case "a":
					// An <a> w/out a tabindex is only navigable if it has an href
					return domAttr.has(elem, "href");
				case "area":
				case "button":
				case "input":
				case "object":
				case "select":
				case "textarea":
					// These are navigable by default
					return true;
				case "iframe":
					// If it's an editor <iframe> then it's tab navigable.
					var body;
					try{
						// non-IE
						var contentDocument = elem.contentDocument;
						if("designMode" in contentDocument && contentDocument.designMode == "on"){
							return true;
						}
						body = contentDocument.body;
					}catch(e1){
						// contentWindow.document isn't accessible within IE7/8
						// if the iframe.src points to a foreign url and this
						// page contains an element, that could get focus
						try{
							body = elem.contentWindow.document.body;
						}catch(e2){
							return false;
						}
					}
					return body && (body.contentEditable == 'true' ||
						(body.firstChild && body.firstChild.contentEditable == 'true'));
				default:
					return elem.contentEditable == 'true';
			}
		},

		isTabNavigable: function(/*Element*/ elem){
			// summary:
			//		Tests if an element is tab-navigable

			// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
			if(domAttr.get(elem, "disabled")){
				return false;
			}else if(domAttr.has(elem, "tabIndex")){
				// Explicit tab index setting
				return domAttr.get(elem, "tabIndex") >= 0; // boolean
			}else{
				// No explicit tabIndex setting, so depends on node type
				return a11y.hasDefaultTabStop(elem);
			}
		},

		_getTabNavigable: function(/*DOMNode*/ root){
			// summary:
			//		Finds descendants of the specified root node.
			// description:
			//		Finds the following descendants of the specified root node:
			//
			//		- the first tab-navigable element in document order
			//		  without a tabIndex or with tabIndex="0"
			//		- the last tab-navigable element in document order
			//		  without a tabIndex or with tabIndex="0"
			//		- the first element in document order with the lowest
			//		  positive tabIndex value
			//		- the last element in document order with the highest
			//		  positive tabIndex value
			var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};

			function radioName(node){
				// If this element is part of a radio button group, return the name for that group.
				return node && node.tagName.toLowerCase() == "input" &&
					node.type && node.type.toLowerCase() == "radio" &&
					node.name && node.name.toLowerCase();
			}

			var shown = a11y._isElementShown, isTabNavigable = a11y.isTabNavigable;
			var walkTree = function(/*DOMNode*/ parent){
				for(var child = parent.firstChild; child; child = child.nextSibling){
					// Skip text elements, hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
					// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
					if(child.nodeType != 1 || (has("ie") <= 9 && child.scopeName !== "HTML") || !shown(child)){
						continue;
					}

					if(isTabNavigable(child)){
						var tabindex = +domAttr.get(child, "tabIndex");	// + to convert string --> number
						if(!domAttr.has(child, "tabIndex") || tabindex == 0){
							if(!first){
								first = child;
							}
							last = child;
						}else if(tabindex > 0){
							if(!lowest || tabindex < lowestTabindex){
								lowestTabindex = tabindex;
								lowest = child;
							}
							if(!highest || tabindex >= highestTabindex){
								highestTabindex = tabindex;
								highest = child;
							}
						}
						var rn = radioName(child);
						if(domAttr.get(child, "checked") && rn){
							radioSelected[rn] = child;
						}
					}
					if(child.nodeName.toUpperCase() != 'SELECT'){
						walkTree(child);
					}
				}
			};
			if(shown(root)){
				walkTree(root);
			}
			function rs(node){
				// substitute checked radio button for unchecked one, if there is a checked one with the same name.
				return radioSelected[radioName(node)] || node;
			}

			return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
		},

		getFirstInTabbingOrder: function(/*String|DOMNode*/ root, /*Document?*/ doc){
			// summary:
			//		Finds the descendant of the specified root node
			//		that is first in the tabbing order
			var elems = a11y._getTabNavigable(dom.byId(root, doc));
			return elems.lowest ? elems.lowest : elems.first; // DomNode
		},

		getLastInTabbingOrder: function(/*String|DOMNode*/ root, /*Document?*/ doc){
			// summary:
			//		Finds the descendant of the specified root node
			//		that is last in the tabbing order
			var elems = a11y._getTabNavigable(dom.byId(root, doc));
			return elems.last ? elems.last : elems.highest; // DomNode
		}
	};

	 1  && lang.mixin(dijit, a11y);

	return a11y;
});

},
'dojo/uacss':function(){
define(["./dom-geometry", "./_base/lang", "./domReady", "./sniff", "./_base/window"],
	function(geometry, lang, domReady, has, baseWindow){

	// module:
	//		dojo/uacss

	/*=====
	return {
		// summary:
		//		Applies pre-set CSS classes to the top-level HTML node, based on:
		//
		//		- browser (ex: dj_ie)
		//		- browser version (ex: dj_ie6)
		//		- box model (ex: dj_contentBox)
		//		- text direction (ex: dijitRtl)
		//
		//		In addition, browser, browser version, and box model are
		//		combined with an RTL flag when browser text is RTL. ex: dj_ie-rtl.
		//
		//		Returns the has() method.
	};
	=====*/

	var
		html = baseWindow.doc.documentElement,
		ie = has("ie"),
		opera = has("opera"),
		maj = Math.floor,
		ff = has("ff"),
		boxModel = geometry.boxModel.replace(/-/,''),

		classes = {
			"dj_quirks": has("quirks"),

			// NOTE: Opera not supported by dijit
			"dj_opera": opera,

			"dj_khtml": has("khtml"),

			"dj_webkit": has("webkit"),
			"dj_safari": has("safari"),
			"dj_chrome": has("chrome"),

			"dj_gecko": has("mozilla"),

			"dj_ios": has("ios"),
			"dj_android": has("android")
		}; // no dojo unsupported browsers

	if(ie){
		classes["dj_ie"] = true;
		classes["dj_ie" + maj(ie)] = true;
		classes["dj_iequirks"] = has("quirks");
	}
	if(ff){
		classes["dj_ff" + maj(ff)] = true;
	}

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = lang.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	domReady(function(){
		if(!geometry.isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ");
			html.className = lang.trim(html.className + " " + rtlClassStr + "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl "));
		}
	});
	return has;
});

},
'dijit/hccss':function(){
define(["dojo/dom-class", "dojo/hccss", "dojo/domReady", "dojo/_base/window"], function(domClass, has, domReady, win){

	// module:
	//		dijit/hccss

	/*=====
	return function(){
		// summary:
		//		Test if computer is in high contrast mode, and sets `dijit_a11y` flag on `<body>` if it is.
		//		Deprecated, use ``dojo/hccss`` instead.
	};
	=====*/

	domReady(function(){
		if(has("highcontrast")){
			domClass.add(win.body(), "dijit_a11y");
		}
	});

	return has;
});

},
'dojo/hccss':function(){
define([
	"require",			// require, require.toUrl
	"./_base/config", // config.blankGif
	"./dom-class", // domClass.add
	"./dom-style", // domStyle.getComputedStyle
	"./has",
	"./domReady",
	"./_base/window" // win.body
], function(require, config, domClass, domStyle, has, domReady, win){

	// module:
	//		dojo/hccss

	/*=====
	return function(){
		// summary:
		//		Test if computer is in high contrast mode (i.e. if browser is not displaying background images).
		//		Defines `has("highcontrast")` and sets `dj_a11y` CSS class on `<body>` if machine is in high contrast mode.
		//		Returns `has()` method;
	};
	=====*/

	// Has() test for when background images aren't displayed.  Don't call has("highcontrast") before dojo/domReady!.
	has.add("highcontrast", function(){
		// note: if multiple documents, doesn't matter which one we use
		var div = win.doc.createElement("div");
		div.style.cssText = "border: 1px solid; border-color:red green; position: absolute; height: 5px; top: -999px;" +
			"background-image: url(" + (config.blankGif || require.toUrl("./resources/blank.gif")) + ");";
		win.body().appendChild(div);

		var cs = domStyle.getComputedStyle(div),
			bkImg = cs.backgroundImage,
			hc = (cs.borderTopColor == cs.borderRightColor) ||
				(bkImg && (bkImg == "none" || bkImg == "url(invalid-url:)" ));

		if(has("ie") <= 8){
			div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
		}else{
			win.body().removeChild(div);
		}

		return hc;
	});

	domReady(function(){
		if(has("highcontrast")){
			domClass.add(win.body(), "dj_a11y");
		}
	});

	return has;
});

},
'dijit/_TemplatedMixin':function(){
define([
	"dojo/cache",	// dojo.cache
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.destroy, domConstruct.toDom
	"dojo/_base/lang", // lang.getObject
	"dojo/on",
	"dojo/sniff", // has("ie")
	"dojo/string", // string.substitute string.trim
	"./_AttachMixin"
], function(cache, declare, domConstruct, lang, on, has, string, _AttachMixin){

	// module:
	//		dijit/_TemplatedMixin

	var _TemplatedMixin = declare("dijit._TemplatedMixin", _AttachMixin, {
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template.
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with require([... "dojo/text!..."], ...) instead
		templatePath: null,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template nodes poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

/*=====
		// _rendered: Boolean
		//		Not normally use, but this flag can be set by the app if the server has already rendered the template,
		//		i.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to
		//		just function like _AttachMixin.
		_rendered: false,
=====*/

		// Set _AttachMixin.searchContainerNode to true for back-compat for widgets that have data-dojo-attach-point's
		// and events inside this.containerNode.   Remove for 2.0.
		searchContainerNode: true,

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = lang.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			if(!this._rendered){
				if(!this.templateString){
					this.templateString = cache(this.templatePath, {sanitize: true});
				}

				// Lookup cached version of template, and download to cache if it
				// isn't there already.  Returns either a DomNode or a string, depending on
				// whether or not the template contains ${foo} replacement parameters.
				var cached = _TemplatedMixin.getCachedTemplate(this.templateString, this._skipNodeCache, this.ownerDocument);

				var node;
				if(lang.isString(cached)){
					node = domConstruct.toDom(this._stringRepl(cached), this.ownerDocument);
					if(node.nodeType != 1){
						// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
						throw new Error("Invalid template: " + cached);
					}
				}else{
					// if it's a node, all we have to do is clone it
					node = cached.cloneNode(true);
				}

				this.domNode = node;
			}

			// Call down to _WidgetBase.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to _setBaseClassAttr
			this.inherited(arguments);

			if(!this._rendered){
				this._fillContent(this.srcNodeRef);
			}

			this._rendered = true;
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		}

	});

	// key is templateString; object is either string or DOM tree
	_TemplatedMixin._templateCache = {};

	_TemplatedMixin.getCachedTemplate = function(templateString, alwaysUseString, doc){
		// summary:
		//		Static method to get a template based on the templatePath or
		//		templateString key
		// templateString: String
		//		The template
		// alwaysUseString: Boolean
		//		Don't cache the DOM tree for this template, even if it doesn't have any variables
		// doc: Document?
		//		The target document.   Defaults to document global if unspecified.
		// returns: Mixed
		//		Either string (if there are ${} variables that need to be replaced) or just
		//		a DOM tree (if the node can be cloned directly)

		// is it already cached?
		var tmplts = _TemplatedMixin._templateCache;
		var key = templateString;
		var cached = tmplts[key];
		if(cached){
			try{
				// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the
				// current document, then use the current cached value
				if(!cached.ownerDocument || cached.ownerDocument == (doc || document)){
					// string or node of the same document
					return cached;
				}
			}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
			domConstruct.destroy(cached);
		}

		templateString = string.trim(templateString);

		if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
			// there are variables in the template so all we can do is cache the string
			return (tmplts[key] = templateString); //String
		}else{
			// there are no variables in the template so we can cache the DOM tree
			var node = domConstruct.toDom(templateString, doc);
			if(node.nodeType != 1){
				throw new Error("Invalid template: " + templateString);
			}
			return (tmplts[key] = node); //Node
		}
	};

	if(has("ie")){
		on(window, "unload", function(){
			var cache = _TemplatedMixin._templateCache;
			for(var key in cache){
				var value = cache[key];
				if(typeof value == "object"){ // value is either a string or a DOM node template
					domConstruct.destroy(value);
				}
				delete cache[key];
			}
		});
	}

	return _TemplatedMixin;
});

},
'dojo/cache':function(){
define(["./_base/kernel", "./text"], function(dojo){
	// module:
	//		dojo/cache

	// dojo.cache is defined in dojo/text
	return dojo.cache;
});

},
'dojo/text':function(){
define(["./_base/kernel", "require", "./has", "./request"], function(dojo, require, has, request){
	// module:
	//		dojo/text

	var getText;
	if( 1 ){
		getText= function(url, sync, load){
			request(url, {sync:!!sync}).then(load);
		};
	}else{
		// Path for node.js and rhino, to load from local file system.
		// TODO: use node.js native methods rather than depending on a require.getText() method to exist.
		if(require.getText){
			getText= require.getText;
		}else{
			console.error("dojo/text plugin failed to load because loader does not support getText");
		}
	}

	var
		theCache = {},

		strip= function(text){
			//Strips <?xml ...?> declarations so that external SVG and XML
			//documents can be added to a document without worry. Also, if the string
			//is an HTML document, only the part inside the body tag is returned.
			if(text){
				text= text.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
				var matches= text.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
				if(matches){
					text= matches[1];
				}
			}else{
				text = "";
			}
			return text;
		},

		notFound = {},

		pending = {};

	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		//		A getter and setter for storing the string content associated with the
		//		module and url arguments.
		// description:
		//		If module is a string that contains slashes, then it is interpretted as a fully
		//		resolved path (typically a result returned by require.toUrl), and url should not be
		//		provided. This is the preferred signature. If module is a string that does not
		//		contain slashes, then url must also be provided and module and url are used to
		//		call `dojo.moduleUrl()` to generate a module URL. This signature is deprecated.
		//		If value is specified, the cache value for the moduleUrl will be set to
		//		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		//		in its internal cache and return that cached value for the URL. To clear
		//		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		//		the URL contents, only modules on the same domain of the page can use this capability.
		//		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		//		If a String with slashes, a fully resolved path; if a String without slashes, the
		//		module name to use for the base part of the URL, similar to module argument
		//		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		//		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		//		The rest of the path to append to the path derived from the module argument. If
		//		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		//		If a String, the value to use in the cache for the module/url combination.
		//		If an Object, it can have two properties: value and sanitize. The value property
		//		should be the value to use in the cache, and sanitize can be set to true or false,
		//		to indicate if XML declarations should be removed from the value and if the HTML
		//		inside a body tag in the value should be extracted as the real value. The value argument
		//		or the value property on the value argument are usually only used by the build system
		//		as it inlines cache content.
		// example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		//		of call is used to avoid an issue with the build system erroneously trying to intern
		//		this example. To get the build system to intern your dojo.cache calls, use the
		//		"dojo.cache" style of call):
		//		| //If template.html contains "<h1>Hello</h1>" that will be
		//		| //the value for the text variable.
		//		| var text = dojo["cache"]("my.module", "template.html");
		// example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		//		 (the dojo["cache"] style of call is used to avoid an issue with the build system
		//		erroneously trying to intern this example. To get the build system to intern your
		//		dojo.cache calls, use the "dojo.cache" style of call):
		//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		//		| //text variable will contain just "<h1>Hello</h1>".
		//		| var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		// example:
		//		Same example as previous, but demonstrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		//		| //If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		//		| //text variable will contain just "<h1>Hello</h1>".
		//		| var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//	 * (string string [value]) => (module, url, value)
		//	 * (object [value])        => (module, value), url defaults to ""
		//
		//	 * if module is an object, then it must be convertable to a string
		//	 * (module, url) module + (url ? ("/" + url) : "") must be a legal argument to require.toUrl
		//	 * value may be a string or an object; if an object then may have the properties "value" and/or "sanitize"
		var key;
		if(typeof module=="string"){
			if(/\//.test(module)){
				// module is a version 1.7+ resolved path
				key = module;
				value = url;
			}else{
				// module is a version 1.6- argument to dojo.moduleUrl
				key = require.toUrl(module.replace(/\./g, "/") + (url ? ("/" + url) : ""));
			}
		}else{
			key = module + "";
			value = url;
		}
		var
			val = (value != undefined && typeof value != "string") ? value.value : value,
			sanitize = value && value.sanitize;

		if(typeof val == "string"){
			//We have a string, set cache value
			theCache[key] = val;
			return sanitize ? strip(val) : val;
		}else if(val === null){
			//Remove cached value
			delete theCache[key];
			return null;
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in theCache)){
				getText(key, true, function(text){
					theCache[key]= text;
				});
			}
			return sanitize ? strip(theCache[key]) : theCache[key];
		}
	};

	return {
		// summary:
		//		This module implements the dojo/text! plugin and the dojo.cache API.
		// description:
		//		We choose to include our own plugin to leverage functionality already contained in dojo
		//		and thereby reduce the size of the plugin compared to various foreign loader implementations.
		//		Also, this allows foreign AMD loaders to be used without their plugins.
		//
		//		CAUTION: this module is designed to optionally function synchronously to support the dojo v1.x synchronous
		//		loader. This feature is outside the scope of the CommonJS plugins specification.

		// the dojo/text caches it's own resources because of dojo.cache
		dynamic: true,

		normalize: function(id, toAbsMid){
			// id is something like (path may be relative):
			//
			//	 "path/to/text.html"
			//	 "path/to/text.html!strip"
			var parts= id.split("!"),
				url= parts[0];
			return (/^\./.test(url) ? toAbsMid(url) : url) + (parts[1] ? "!" + parts[1] : "");
		},

		load: function(id, require, load){
			// id: String
			//		Path to the resource.
			// require: Function
			//		Object that include the function toUrl with given id returns a valid URL from which to load the text.
			// load: Function
			//		Callback function which will be called, when the loading finished.

			// id is something like (path is always absolute):
			//
			//	 "path/to/text.html"
			//	 "path/to/text.html!strip"
			var
				parts= id.split("!"),
				stripFlag= parts.length>1,
				absMid= parts[0],
				url = require.toUrl(parts[0]),
				requireCacheUrl = "url:" + url,
				text = notFound,
				finish = function(text){
					load(stripFlag ? strip(text) : text);
				};
			if(absMid in theCache){
				text = theCache[absMid];
			}else if(require.cache && requireCacheUrl in require.cache){
				text = require.cache[requireCacheUrl];
			}else if(url in theCache){
				text = theCache[url];
			}
			if(text===notFound){
				if(pending[url]){
					pending[url].push(finish);
				}else{
					var pendingList = pending[url] = [finish];
					getText(url, !require.async, function(text){
						theCache[absMid]= theCache[url]= text;
						for(var i = 0; i<pendingList.length;){
							pendingList[i++](text);
						}
						delete pending[url];
					});
				}
			}else{
				finish(text);
			}
		}
	};

});


},
'dojo/request':function(){
define([
	'./request/default!'/*=====,
	'./_base/declare',
	'./promise/Promise' =====*/
], function(request/*=====, declare, Promise =====*/){
	/*=====
	request = function(url, options){
		// summary:
		//		Send a request using the default transport for the current platform.
		// url: String
		//		The URL to request.
		// options: dojo/request.__Options?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	request.__Promise = declare(Promise, {
		// response: dojo/promise/Promise
		//		A promise resolving to an object representing
		//		the response from the server.
	});
	request.__BaseOptions = declare(null, {
		// query: String|Object?
		//		Query parameters to append to the URL.
		// data: String|Object?
		//		Data to transfer.  This is ignored for GET and DELETE
		//		requests.
		// preventCache: Boolean?
		//		Whether to append a cache-busting parameter to the URL.
		// timeout: Integer?
		//		Milliseconds to wait for the response.  If this time
		//		passes, the then the promise is rejected.
		// handleAs: String?
		//		How to handle the response from the server.  Default is
		//		'text'.  Other values are 'json', 'javascript', and 'xml'.
	});
	request.__MethodOptions = declare(null, {
		// method: String?
		//		The HTTP method to use to make the request.  Must be
		//		uppercase.
	});
	request.__Options = declare([request.__BaseOptions, request.__MethodOptions]);

	request.get = function(url, options){
		// summary:
		//		Send an HTTP GET request using the default transport for the current platform.
		// url: String
		//		URL to request
		// options: dojo/request.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	request.post = function(url, options){
		// summary:
		//		Send an HTTP POST request using the default transport for the current platform.
		// url: String
		//		URL to request
		// options: dojo/request.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	request.put = function(url, options){
		// summary:
		//		Send an HTTP POST request using the default transport for the current platform.
		// url: String
		//		URL to request
		// options: dojo/request.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	request.del = function(url, options){
		// summary:
		//		Send an HTTP DELETE request using the default transport for the current platform.
		// url: String
		//		URL to request
		// options: dojo/request.__BaseOptions?
		//		Options for the request.
		// returns: dojo/request.__Promise
	};
	=====*/
	return request;
});

},
'dojo/request/default':function(){
define([
	'exports',
	'require',
	'../has'
], function(exports, require, has){
	var defId = has('config-requestProvider'),
		platformId;

	if( 1 ){
		platformId = './xhr';
	}else if( 0 ){
		platformId = './node';
	/* TODO:
	}else if( 0 ){
		platformId = './rhino';
   */
	}

	if(!defId){
		defId = platformId;
	}

	exports.getPlatformDefaultId = function(){
		return platformId;
	};

	exports.load = function(id, parentRequire, loaded, config){
		require([id == 'platform' ? platformId : defId], function(provider){
			loaded(provider);
		});
	};
});

},
'dojo/string':function(){
define([
	"./_base/kernel",	// kernel.global
	"./_base/lang"
], function(kernel, lang){

// module:
//		dojo/string

var string = {
	// summary:
	//		String utilities for Dojo
};
lang.setObject("dojo.string", string);

string.rep = function(/*String*/str, /*Integer*/num){
	// summary:
	//		Efficiently replicate a string `n` times.
	// str:
	//		the string to replicate
	// num:
	//		number of times to replicate the string

	if(num <= 0 || !str){ return ""; }

	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	// text:
	//		the string to pad
	// size:
	//		length to provide padding
	// ch:
	//		character to pad, defaults to '0'
	// end:
	//		adds padding at the end if true, otherwise pads at start
	// example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

string.substitute = function(	/*String*/		template,
									/*Object|Array*/map,
									/*Function?*/	transform,
									/*Object?*/		thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// template:
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive.
	// map:
	//		hash to search for substitutions
	// transform:
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	// thisObject:
	//		where to look for optional format function; default to the global
	//		namespace
	// example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	// example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	// example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || kernel.global;
	transform = transform ?
		lang.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = lang.getObject(key, false, map);
			if(format){
				value = lang.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

string.trim = String.prototype.trim ?
	lang.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

/*=====
 string.trim = function(str){
	 // summary:
	 //		Trims whitespace from both sides of the string
	 // str: String
	 //		String to be trimmed
	 // returns: String
	 //		Returns the trimmed string
	 // description:
	 //		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	 //		The short yet performant version of this function is dojo.trim(),
	 //		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	 return "";	// String
 };
 =====*/

	return string;
});

},
'dijit/_AttachMixin':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach
	"dojo/_base/connect",	// remove for 2.0
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.getObject
	"dojo/mouse",
	"dojo/on",
	"dojo/touch",
	"./_WidgetBase"
], function(require, array, connect, declare, lang, mouse, on, touch, _WidgetBase){

	// module:
	//		dijit/_AttachMixin

	// Map from string name like "mouseenter" to synthetic event like mouse.enter
	var synthEvents = lang.delegate(touch, {
		"mouseenter": mouse.enter,
		"mouseleave": mouse.leave,
		"keypress": connect._keypress	// remove for 2.0
	});

	// To be lightweight, _AttachMixin doesn't require() dijit/a11yclick.
	// If the subclass has a template using "ondijitclick", it must load dijit/a11yclick itself.
	// In that case, the a11yclick variable below will get set to point to that synthetic event.
	var a11yclick;

	var _AttachMixin = declare("dijit._AttachMixin", null, {
		// summary:
		//		Mixin for widgets to attach to dom nodes and setup events via
		//		convenient data-dojo-attach-point and data-dojo-attach-event DOM attributes.
		//
		//		Superclass of _TemplatedMixin, and can also be used standalone when templates are pre-rendered on the
		//		server.
		//
		//		Does not [yet] handle widgets like ContentPane with this.containerNode set.   It should skip
		//		scanning for data-dojo-attach-point and data-dojo-attach-event inside this.containerNode, but it
		//		doesn't.

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with data-dojo-attach-point=... in the
		//		template, ex: ["containerNode", "labelNode"]
		_attachPoints: [],

		// _attachEvents: [private] Handle[]
		//		List of connections associated with data-dojo-attach-event=... in the
		//		template
		_attachEvents: [],

		// attachScope: [public] Object
		//		Object to which attach points and events will be scoped.  Defaults
		//		to 'this'.
		attachScope: undefined,

		// searchContainerNode: [protected] Boolean
		//		Search descendants of this.containerNode for data-dojo-attach-point and data-dojo-attach-event.
		//		Should generally be left false (the default value) both for performance and to avoid failures when
		//		this.containerNode holds other _AttachMixin instances with their own attach points and events.
 		searchContainerNode: false,
 =====*/

		constructor: function(/*===== params, srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree.

			this._attachPoints = [];
			this._attachEvents = [];
		},


		buildRendering: function(){
			// summary:
			//		Attach to DOM nodes marked with special attributes.
			// tags:
			//		protected

			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(this.domNode);

			this._beforeFillContent();		// hook for _WidgetsInTemplateMixin
		},

		_beforeFillContent: function(){
		},

		_attachTemplateNodes: function(rootNode){
			// summary:
			//		Iterate through the dom nodes and attach functions and nodes accordingly.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//
			//		- dojoAttachPoint/data-dojo-attach-point
			//		- dojoAttachEvent/data-dojo-attach-event
			// rootNode: DomNode
			//		The node to search for properties. All descendants will be searched.
			// tags:
			//		private

			// DFS to process all nodes except those inside of this.containerNode
			var node = rootNode;
			while(true){
				if(node.nodeType == 1 && (this._processTemplateNode(node, function(n,p){ return n.getAttribute(p); },
						this._attach) || this.searchContainerNode) && node.firstChild){
					node = node.firstChild;
				}else{
					if(node == rootNode){ return; }
					while(!node.nextSibling){
						node = node.parentNode;
						if(node == rootNode){ return; }
					}
					node = node.nextSibling;
				}
			}
		},

		_processTemplateNode: function(/*DOMNode|Widget*/ baseNode, getAttrFunc, attachFunc){
			// summary:
			//		Process data-dojo-attach-point and data-dojo-attach-event for given node or widget.
			//		Returns true if caller should process baseNode's children too.

			var ret = true;

			// Process data-dojo-attach-point
			var _attachScope = this.attachScope || this,
				attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
			if(attachPoint){
				var point, points = attachPoint.split(/\s*,\s*/);
				while((point = points.shift())){
					if(lang.isArray(_attachScope[point])){
						_attachScope[point].push(baseNode);
					}else{
						_attachScope[point] = baseNode;
					}
					ret = (point != "containerNode");
					this._attachPoints.push(point);
				}
			}

			// Process data-dojo-attach-event
			var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");
			if(attachEvent){
				// NOTE: we want to support attributes that have the form
				// "domEvent: nativeEvent; ..."
				var event, events = attachEvent.split(/\s*,\s*/);
				var trim = lang.trim;
				while((event = events.shift())){
					if(event){
						var thisFunc = null;
						if(event.indexOf(":") != -1){
							// oh, if only JS had tuple assignment
							var funcNameArr = event.split(":");
							event = trim(funcNameArr[0]);
							thisFunc = trim(funcNameArr[1]);
						}else{
							event = trim(event);
						}
						if(!thisFunc){
							thisFunc = event;
						}

						this._attachEvents.push(attachFunc(baseNode, event, lang.hitch(_attachScope, thisFunc)));
					}
				}
			}

			return ret;
		},

		_attach: function(node, type, func){
			// summary:
			//		Roughly corresponding to dojo/on, this is the default function for processing a
			//		data-dojo-attach-event.  Meant to attach to DOMNodes, not to widgets.
			// node: DOMNode
			//		The node to setup a listener on.
			// type: String
			//		Event name like "click".
			// getAttrFunc: Function
			//		Function to get the specified property for a given DomNode/Widget.
			// attachFunc: Function?
			//		Attaches an event handler from the specified node/widget to specified function.

			// Map special type names like "mouseenter" to synthetic events.
			// Subclasses are responsible to require() dijit/a11yclick if they want to use it.
			type = type.replace(/^on/, "").toLowerCase();
			if(type == "dijitclick"){
				type = a11yclick || (a11yclick = require("./a11yclick"));
			}else{
				type = synthEvents[type] || type;
			}

			return on(node, type, func);
		},

		_detachTemplateNodes: function() {
			// summary:
			//		Detach and clean up the attachments made in _attachtempalteNodes.

			// Delete all attach points to prevent IE6 memory leaks.
			var _attachScope = this.attachScope || this;
			array.forEach(this._attachPoints, function(point){
				delete _attachScope[point];
			});
			this._attachPoints = [];

			// And same for event handlers
			array.forEach(this._attachEvents, function(handle){ handle.remove(); });
			this._attachEvents = [];
		},

		destroyRendering: function(){
			this._detachTemplateNodes();
			this.inherited(arguments);
		}
	});

	// These arguments can be specified for widgets which are used in templates.
	// Since any widget can be specified as sub widgets in template, mix it
	// into the base widget class.  (This is a hack, but it's effective.).
	// Remove for 2.0.   Also, hide from API doc parser.
	lang.extend(_WidgetBase, /*===== {} || =====*/ {
		dojoAttachEvent: "",
		dojoAttachPoint: ""
	});
	
	return _AttachMixin;
});

},
'dijit/BackgroundIframe':function(){
define([
	"require",			// require.toUrl
	"./main",	// to export dijit.BackgroundIframe
	"dojo/_base/config",
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.set
	"dojo/_base/lang", // lang.extend lang.hitch
	"dojo/on",
	"dojo/sniff" // has("ie"), has("mozilla"), has("quirks")
], function(require, dijit, config, domConstruct, domStyle, lang, on, has){

	// module:
	//		dijit/BackgroundIFrame

	// Flag for whether to create background iframe behind popups like Menus and Dialog.
	// A background iframe is useful to prevent problems with popups appearing behind applets/pdf files,
	// and is also useful on older versions of IE (IE6 and IE7) to prevent the "bleed through select" problem.
	// TODO: For 2.0, make this false by default.  Also, possibly move definition to has.js so that this module can be
	// conditionally required via  dojo/has!bgIfame?dijit/BackgroundIframe
	has.add("config-bgIframe", !has("touch"));

	// TODO: remove _frames, it isn't being used much, since popups never release their
	// iframes (see [22236])
	var _frames = new function(){
		// summary:
		//		cache of iframes

		var queue = [];

		this.pop = function(){
			var iframe;
			if(queue.length){
				iframe = queue.pop();
				iframe.style.display="";
			}else{
				// transparency needed for DialogUnderlay and for tooltips on IE (to see screen near connector)
				if(has("ie") < 9){
					var burl = config["dojoBlankHtmlUrl"] || require.toUrl("dojo/resources/blank.html") || "javascript:\"\"";
					var html="<iframe src='" + burl + "' role='presentation'"
						+ " style='position: absolute; left: 0px; top: 0px;"
						+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
					iframe = document.createElement(html);
				}else{
					iframe = domConstruct.create("iframe");
					iframe.src = 'javascript:""';
					iframe.className = "dijitBackgroundIframe";
					iframe.setAttribute("role", "presentation");
					domStyle.set(iframe, "opacity", 0.1);
				}
				iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			}
			return iframe;
		};

		this.push = function(iframe){
			iframe.style.display="none";
			queue.push(iframe);
		}
	}();


	dijit.BackgroundIframe = function(/*DomNode*/ node){
		// summary:
		//		For IE/FF z-index shenanigans. id attribute is required.
		//
		// description:
		//		new dijit.BackgroundIframe(node).
		//
		//		Makes a background iframe as a child of node, that fills
		//		area (and position) of node

		if(!node.id){ throw new Error("no id"); }
		if(has("config-bgIframe")){
			var iframe = (this.iframe = _frames.pop());
			node.appendChild(iframe);
			if(has("ie")<7 || has("quirks")){
				this.resize(node);
				this._conn = on(node, 'resize', lang.hitch(this, "resize", node));
			}else{
				domStyle.set(iframe, {
					width: '100%',
					height: '100%'
				});
			}
		}
	};

	lang.extend(dijit.BackgroundIframe, {
		resize: function(node){
			// summary:
			//		Resize the iframe so it's the same size as node.
			//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
			if(this.iframe){
				domStyle.set(this.iframe, {
					width: node.offsetWidth + 'px',
					height: node.offsetHeight + 'px'
				});
			}
		},
		destroy: function(){
			// summary:
			//		destroy the iframe
			if(this._conn){
				this._conn.remove();
				this._conn = null;
			}
			if(this.iframe){
				_frames.push(this.iframe);
				delete this.iframe;
			}
		}
	});

	return dijit.BackgroundIframe;
});

},
'dojo/dnd/Avatar':function(){
define([
	"../_base/declare",
	"../_base/window",
	"../dom",
	"../dom-attr",
	"../dom-class",
	"../dom-construct",
	"../hccss",
	"../query"
], function(declare, win, dom, domAttr, domClass, domConstruct, has, query){

// module:
//		dojo/dnd/Avatar

return declare("dojo.dnd.Avatar", null, {
	// summary:
	//		Object that represents transferred DnD items visually
	// manager: Object
	//		a DnD manager object

	constructor: function(manager){
		this.manager = manager;
		this.construct();
	},

	// methods
	construct: function(){
		// summary:
		//		constructor function;
		//		it is separate so it can be (dynamically) overwritten in case of need

		var a = domConstruct.create("table", {
				"class": "dojoDndAvatar",
				style: {
					position: "absolute",
					zIndex:   "1999",
					margin:   "0px"
				}
			}),
			source = this.manager.source, node,
			b = domConstruct.create("tbody", null, a),
			tr = domConstruct.create("tr", null, b),
			td = domConstruct.create("td", null, tr),
			k = Math.min(5, this.manager.nodes.length), i = 0;

		if(has("highcontrast")){
			domConstruct.create("span", {
				id : "a11yIcon",
				innerHTML : this.manager.copy ? '+' : "<"
			}, td)
		}
		domConstruct.create("span", {
			innerHTML: source.generateText ? this._generateText() : ""
		}, td);

		// we have to set the opacity on IE only after the node is live
		domAttr.set(tr, {
			"class": "dojoDndAvatarHeader",
			style: {opacity: 0.9}
		});
		for(; i < k; ++i){
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[i].cloneNode(true);
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					var table = domConstruct.create("table"),
						tbody = domConstruct.create("tbody", null, table);
					tbody.appendChild(node);
					node = table;
				}
			}
			node.id = "";
			tr = domConstruct.create("tr", null, b);
			td = domConstruct.create("td", null, tr);
			td.appendChild(node);
			domAttr.set(tr, {
				"class": "dojoDndAvatarItem",
				style: {opacity: (9 - i) / 10}
			});
		}
		this.node = a;
	},
	destroy: function(){
		// summary:
		//		destructor for the avatar; called to remove all references so it can be garbage-collected
		domConstruct.destroy(this.node);
		this.node = false;
	},
	update: function(){
		// summary:
		//		updates the avatar to reflect the current DnD state
		domClass.toggle(this.node, "dojoDndAvatarCanDrop", this.manager.canDropFlag);
		if(has("highcontrast")){
			var icon = dom.byId("a11yIcon");
			var text = '+';   // assume canDrop && copy
			if (this.manager.canDropFlag && !this.manager.copy){
				text = '< '; // canDrop && move
			}else if (!this.manager.canDropFlag && !this.manager.copy){
				text = "o"; //!canDrop && move
			}else if(!this.manager.canDropFlag){
				text = 'x';  // !canDrop && copy
			}
			icon.innerHTML=text;
		}
		// replace text
		query(("tr.dojoDndAvatarHeader td span" +(has("highcontrast") ? " span" : "")), this.node).forEach(
			function(node){
				node.innerHTML = this.manager.source.generateText ? this._generateText() : "";
			}, this);
	},
	_generateText: function(){
		// summary:
		//		generates a proper text to reflect copying or moving of items
		return this.manager.nodes.length.toString();
	}
});

});

},
'dojo/dnd/Manager':function(){
define([
	"../_base/array",  "../_base/declare", "../_base/lang", "../_base/window",
	"../dom-class", "../Evented", "../has", "../keys", "../on", "../topic", "../touch",
	"./common", "./autoscroll", "./Avatar"
], function(array, declare, lang, win, domClass, Evented, has, keys, on, topic, touch,
	dnd, autoscroll, Avatar){

// module:
//		dojo/dnd/Manager

var Manager = declare("dojo.dnd.Manager", [Evented], {
	// summary:
	//		the manager of DnD operations (usually a singleton)
	constructor: function(){
		this.avatar  = null;
		this.source = null;
		this.nodes = [];
		this.copy  = true;
		this.target = null;
		this.canDropFlag = false;
		this.events = [];
	},

	// avatar's offset from the mouse
	OFFSET_X: has("touch") ? 0 : 16,
	OFFSET_Y: has("touch") ? -64 : 16,

	// methods
	overSource: function(source){
		// summary:
		//		called when a source detected a mouse-over condition
		// source: Object
		//		the reporter
		if(this.avatar){
			this.target = (source && source.targetState != "Disabled") ? source : null;
			this.canDropFlag = Boolean(this.target);
			this.avatar.update();
		}
		topic.publish("/dnd/source/over", source);
	},
	outSource: function(source){
		// summary:
		//		called when a source detected a mouse-out condition
		// source: Object
		//		the reporter
		if(this.avatar){
			if(this.target == source){
				this.target = null;
				this.canDropFlag = false;
				this.avatar.update();
				topic.publish("/dnd/source/over", null);
			}
		}else{
			topic.publish("/dnd/source/over", null);
		}
	},
	startDrag: function(source, nodes, copy){
		// summary:
		//		called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		// Tell autoscroll that a drag is starting
		autoscroll.autoScrollStart(win.doc);

		this.source = source;
		this.nodes  = nodes;
		this.copy   = Boolean(copy); // normalizing to true boolean
		this.avatar = this.makeAvatar();
		win.body().appendChild(this.avatar.node);
		topic.publish("/dnd/start", source, nodes, this.copy);

		function stopEvent(e){
			e.preventDefault();
			e.stopPropagation();
		}

		this.events = [
			on(win.doc, touch.move, lang.hitch(this, "onMouseMove")),
			on(win.doc, touch.release,   lang.hitch(this, "onMouseUp")),
			on(win.doc, "keydown",   lang.hitch(this, "onKeyDown")),
			on(win.doc, "keyup",     lang.hitch(this, "onKeyUp")),

			// cancel text selection and text dragging
			on(win.doc, "dragstart",   stopEvent),
			on(win.body(), "selectstart", stopEvent)
		];
		var c = "dojoDnd" + (copy ? "Copy" : "Move");
		domClass.add(win.body(), c);
	},
	canDrop: function(flag){
		// summary:
		//		called to notify if the current target can accept items
		var canDropFlag = Boolean(this.target && flag);
		if(this.canDropFlag != canDropFlag){
			this.canDropFlag = canDropFlag;
			this.avatar.update();
		}
	},
	stopDrag: function(){
		// summary:
		//		stop the DnD in progress
		domClass.remove(win.body(), ["dojoDndCopy", "dojoDndMove"]);
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.events = [];
		this.avatar.destroy();
		this.avatar = null;
		this.source = this.target = null;
		this.nodes = [];
	},
	makeAvatar: function(){
		// summary:
		//		makes the avatar; it is separate to be overwritten dynamically, if needed
		return new Avatar(this);
	},
	updateAvatar: function(){
		// summary:
		//		updates the avatar; it is separate to be overwritten dynamically, if needed
		this.avatar.update();
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		var a = this.avatar;
		if(a){
			autoscroll.autoScrollNodes(e);
			//autoscroll.autoScroll(e);
			var s = a.node.style;
			s.left = (e.pageX + this.OFFSET_X) + "px";
			s.top  = (e.pageY + this.OFFSET_Y) + "px";
			var copy = Boolean(this.source.copyState(dnd.getCopyKeyState(e)));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
		if(has("touch")){
			// Prevent page from scrolling so that user can drag instead.
			e.preventDefault();
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.avatar){
			if(this.target && this.canDropFlag){
				var copy = Boolean(this.source.copyState(dnd.getCopyKeyState(e)));
				topic.publish("/dnd/drop/before", this.source, this.nodes, copy, this.target, e);
				topic.publish("/dnd/drop", this.source, this.nodes, copy, this.target, e);
			}else{
				topic.publish("/dnd/cancel");
			}
			this.stopDrag();
		}
	},

	// keyboard event processors
	onKeyDown: function(e){
		// summary:
		//		event processor for onkeydown:
		//		watching for CTRL for copy/move status, watching for ESCAPE to cancel the drag
		// e: Event
		//		keyboard event
		if(this.avatar){
			switch(e.keyCode){
				case keys.CTRL:
					var copy = Boolean(this.source.copyState(true));
					if(this.copy != copy){
						this._setCopyStatus(copy);
					}
					break;
				case keys.ESCAPE:
					topic.publish("/dnd/cancel");
					this.stopDrag();
					break;
			}
		}
	},
	onKeyUp: function(e){
		// summary:
		//		event processor for onkeyup, watching for CTRL for copy/move status
		// e: Event
		//		keyboard event
		if(this.avatar && e.keyCode == keys.CTRL){
			var copy = Boolean(this.source.copyState(false));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
	},

	// utilities
	_setCopyStatus: function(copy){
		// summary:
		//		changes the copy status
		// copy: Boolean
		//		the copy status
		this.copy = copy;
		this.source._markDndStatus(this.copy);
		this.updateAvatar();
		domClass.replace(win.body(),
			"dojoDnd" + (this.copy ? "Copy" : "Move"),
			"dojoDnd" + (this.copy ? "Move" : "Copy"));
	}
});

// dnd._manager:
//		The manager singleton variable. Can be overwritten if needed.
dnd._manager = null;

Manager.manager = dnd.manager = function(){
	// summary:
	//		Returns the current DnD manager.  Creates one if it is not created yet.
	if(!dnd._manager){
		dnd._manager = new Manager();
	}
	return dnd._manager;	// Object
};

// TODO: for 2.0, store _manager and manager in Manager only.   Don't access dnd or dojo.dnd.

return Manager;
});

},
'dojo/dnd/common':function(){
define(["../sniff", "../_base/kernel", "../_base/lang", "../dom"],
	function(has, kernel, lang, dom){

// module:
//		dojo/dnd/common

var exports = lang.getObject("dojo.dnd", true);
/*=====
// TODO: for 2.0, replace line above with this code.
var exports = {
	// summary:
	//		TODOC
};
=====*/

exports.getCopyKeyState = function(evt){
	return evt[has("mac") ? "metaKey" : "ctrlKey"]
};

exports._uniqueId = 0;
exports.getUniqueId = function(){
	// summary:
	//		returns a unique string for use with any DOM element
	var id;
	do{
		id = kernel._scopeName + "Unique" + (++exports._uniqueId);
	}while(dom.byId(id));
	return id;
};

exports._empty = {};

exports.isFormElement = function(/*Event*/ e){
	// summary:
	//		returns true if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

return exports;
});

},
'dojo/dnd/autoscroll':function(){
define(["../_base/lang", "../sniff", "../_base/window", "../dom-geometry", "../dom-style", "../window"],
	function(lang, has, win, domGeom, domStyle, winUtils){

// module:
//		dojo/dnd/autoscroll

var exports = {
	// summary:
	//		Used by dojo/dnd/Manager to scroll document or internal node when the user
	//		drags near the edge of the viewport or a scrollable node
};
lang.setObject("dojo.dnd.autoscroll", exports);

exports.getViewport = winUtils.getBox;

exports.V_TRIGGER_AUTOSCROLL = 32;
exports.H_TRIGGER_AUTOSCROLL = 32;

exports.V_AUTOSCROLL_VALUE = 16;
exports.H_AUTOSCROLL_VALUE = 16;

// These are set by autoScrollStart().
// Set to default values in case autoScrollStart() isn't called. (back-compat, remove for 2.0)
var viewport,
	doc = win.doc,
	maxScrollTop = Infinity,
	maxScrollLeft = Infinity;

exports.autoScrollStart = function(d){
	// summary:
	//		Called at the start of a drag.
	// d: Document
	//		The document of the node being dragged.

	doc = d;
	viewport = winUtils.getBox(doc);

	// Save height/width of document at start of drag, before it gets distorted by a user dragging an avatar past
	// the document's edge
	var html = win.body(doc).parentNode;
	maxScrollTop = Math.max(html.scrollHeight - viewport.h, 0);
	maxScrollLeft = Math.max(html.scrollWidth - viewport.w, 0);	// usually 0
};

exports.autoScroll = function(e){
	// summary:
	//		a handler for mousemove and touchmove events, which scrolls the window, if
	//		necessary
	// e: Event
	//		mousemove/touchmove event

	// FIXME: needs more docs!
	var v = viewport || winUtils.getBox(doc), // getBox() call for back-compat, in case autoScrollStart() wasn't called
		html = win.body(doc).parentNode,
		dx = 0, dy = 0;
	if(e.clientX < exports.H_TRIGGER_AUTOSCROLL){
		dx = -exports.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - exports.H_TRIGGER_AUTOSCROLL){
		dx = Math.min(exports.H_AUTOSCROLL_VALUE, maxScrollLeft - html.scrollLeft);	// don't scroll past edge of doc
	}
	if(e.clientY < exports.V_TRIGGER_AUTOSCROLL){
		dy = -exports.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - exports.V_TRIGGER_AUTOSCROLL){
		dy = Math.min(exports.V_AUTOSCROLL_VALUE, maxScrollTop - html.scrollTop);	// don't scroll past edge of doc
	}
	window.scrollBy(dx, dy);
};

exports._validNodes = {"div": 1, "p": 1, "td": 1};
exports._validOverflow = {"auto": 1, "scroll": 1};

exports.autoScrollNodes = function(e){
	// summary:
	//		a handler for mousemove and touchmove events, which scrolls the first available
	//		Dom element, it falls back to exports.autoScroll()
	// e: Event
	//		mousemove/touchmove event

	// FIXME: needs more docs!

	var b, t, w, h, rx, ry, dx = 0, dy = 0, oldLeft, oldTop;

	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in exports._validNodes)){
			var s = domStyle.getComputedStyle(n),
				overflow = (s.overflow.toLowerCase() in exports._validOverflow),
				overflowX = (s.overflowX.toLowerCase() in exports._validOverflow),
				overflowY = (s.overflowY.toLowerCase() in exports._validOverflow);
			if(overflow || overflowX || overflowY){
				b = domGeom.getContentBox(n, s);
				t = domGeom.position(n, true);
			}
			// overflow-x
			if(overflow || overflowX){
				w = Math.min(exports.H_TRIGGER_AUTOSCROLL, b.w / 2);
				rx = e.pageX - t.x;
				if(has("webkit") || has("opera")){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the domGeom.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					rx += win.body().scrollLeft;
				}
				dx = 0;
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -w;
					}else if(rx > b.w - w){
						dx = w;
					}
					oldLeft = n.scrollLeft;
					n.scrollLeft = n.scrollLeft + dx;
				}
			}
			// overflow-y
			if(overflow || overflowY){
				//console.log(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				h = Math.min(exports.V_TRIGGER_AUTOSCROLL, b.h / 2);
				ry = e.pageY - t.y;
				if(has("webkit") || has("opera")){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the domGeom.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					ry += win.body().scrollTop;
				}
				dy = 0;
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -h;
					}else if(ry > b.h - h){
						dy = h;
					}
					oldTop = n.scrollTop;
					n.scrollTop  = n.scrollTop  + dy;
				}
			}
			if(dx || dy){ return; }
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	exports.autoScroll(e);
};

return exports;

});

},
'dijit/Menu':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId dom.isDescendant
	"dojo/dom-attr", // domAttr.get domAttr.set domAttr.has domAttr.remove
	"dojo/dom-geometry", // domStyle.getComputedStyle domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/keys", // keys.F10
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie"), has("quirks")
	"dojo/_base/window", // win.body
	"dojo/window", // winUtils.get
	"./popup",
	"./DropDownMenu",
	"dojo/ready"
], function(require, array, declare, dom, domAttr, domGeometry, domStyle, keys, lang, on, has, win, winUtils, pm, DropDownMenu, ready){

	// module:
	//		dijit/Menu

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/MenuItem", "dijit/PopupMenuItem", "dijit/CheckedMenuItem", "dijit/MenuSeparator"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.Menu", DropDownMenu, {
		// summary:
		//		A context menu you can assign to multiple elements

		constructor: function(/*===== params, srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified:
			//
			//		- use srcNodeRef.innerHTML as my contents
			//		- replace srcNodeRef with my generated DOM tree

			this._bindings = [];
		},

		// targetNodeIds: [const] String[]
		//		Array of dom node ids of nodes to attach to.
		//		Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
		targetNodeIds: [],

		// selector: String?
		//		CSS expression to apply this Menu to descendants of targetNodeIds, rather than to
		//		the nodes specified by targetNodeIds themselves.    Useful for applying a Menu to
		//		a range of rows in a table, tree, etc.
		//
		//		The application must require() an appropriate level of dojo/query to handle the selector.
		selector: "",

		// TODO: in 2.0 remove support for multiple targetNodeIds.   selector gives the same effect.
		// So, change targetNodeIds to a targetNodeId: "", remove bindDomNode()/unBindDomNode(), etc.

		/*=====
		// currentTarget: [readonly] DOMNode
		//		For context menus, set to the current node that the Menu is being displayed for.
		//		Useful so that the menu actions can be tailored according to the node
		currentTarget: null,
		=====*/

		// contextMenuForWindow: [const] Boolean
		//		If true, right clicking anywhere on the window will cause this context menu to open.
		//		If false, must specify targetNodeIds.
		contextMenuForWindow: false,

		// leftClickToOpen: [const] Boolean
		//		If true, menu will open on left click instead of right click, similar to a file menu.
		leftClickToOpen: false,

		// refocus: Boolean
		//		When this menu closes, re-focus the element which had focus before it was opened.
		refocus: true,

		postCreate: function(){
			if(this.contextMenuForWindow){
				this.bindDomNode(this.ownerDocumentBody);
			}else{
				array.forEach(this.targetNodeIds, this.bindDomNode, this);
			}
			this.inherited(arguments);
		},

		// thanks burstlib!
		_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el){
			// summary:
			//		Returns the window reference of the passed iframe
			// tags:
			//		private
			return winUtils.get(this._iframeContentDocument(iframe_el)) ||
				// Moz. TODO: is this available when defaultView isn't?
				this._iframeContentDocument(iframe_el)['__parent__'] ||
				(iframe_el.name && document.frames[iframe_el.name]) || null;	//	Window
		},

		_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
			// summary:
			//		Returns a reference to the document object inside iframe_el
			// tags:
			//		protected
			return iframe_el.contentDocument // W3
				|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
				|| (iframe_el.name && document.frames[iframe_el.name] && document.frames[iframe_el.name].document)
				|| null;	//	HTMLDocument
		},

		bindDomNode: function(/*String|DomNode*/ node){
			// summary:
			//		Attach menu to given node
			node = dom.byId(node, this.ownerDocument);

			var cn;	// Connect node

			// Support context menus on iframes.  Rather than binding to the iframe itself we need
			// to bind to the <body> node inside the iframe.
			if(node.tagName.toLowerCase() == "iframe"){
				var iframe = node,
					window = this._iframeContentWindow(iframe);
				cn = win.body(window.document);
			}else{
				// To capture these events at the top level, attach to <html>, not <body>.
				// Otherwise right-click context menu just doesn't work.
				cn = (node == win.body(this.ownerDocument) ? this.ownerDocument.documentElement : node);
			}


			// "binding" is the object to track our connection to the node (ie, the parameter to bindDomNode())
			var binding = {
				node: node,
				iframe: iframe
			};

			// Save info about binding in _bindings[], and make node itself record index(+1) into
			// _bindings[] array.  Prefix w/_dijitMenu to avoid setting an attribute that may
			// start with a number, which fails on FF/safari.
			domAttr.set(node, "_dijitMenu" + this.id, this._bindings.push(binding));

			// Setup the connections to monitor click etc., unless we are connecting to an iframe which hasn't finished
			// loading yet, in which case we need to wait for the onload event first, and then connect
			// On linux Shift-F10 produces the oncontextmenu event, but on Windows it doesn't, so
			// we need to monitor keyboard events in addition to the oncontextmenu event.
			var doConnects = lang.hitch(this, function(cn){
				var selector = this.selector,
					delegatedEvent = selector ?
						function(eventType){
							return on.selector(selector, eventType);
						} :
						function(eventType){
							return eventType;
						},
					self = this;
				return [
					// TODO: when leftClickToOpen is true then shouldn't space/enter key trigger the menu,
					// rather than shift-F10?
					on(cn, delegatedEvent(this.leftClickToOpen ? "click" : "contextmenu"), function(evt){
						// Schedule context menu to be opened unless it's already been scheduled from onkeydown handler
						evt.stopPropagation();
						evt.preventDefault();
						self._scheduleOpen(this, iframe, {x: evt.pageX, y: evt.pageY});
					}),
					on(cn, delegatedEvent("keydown"), function(evt){
						if(evt.shiftKey && evt.keyCode == keys.F10){
							evt.stopPropagation();
							evt.preventDefault();
							self._scheduleOpen(this, iframe);	// no coords - open near target node
						}
					})
				];
			});
			binding.connects = cn ? doConnects(cn) : [];

			if(iframe){
				// Setup handler to [re]bind to the iframe when the contents are initially loaded,
				// and every time the contents change.
				// Need to do this b/c we are actually binding to the iframe's <body> node.
				// Note: can't use connect.connect(), see #9609.

				binding.onloadHandler = lang.hitch(this, function(){
					// want to remove old connections, but IE throws exceptions when trying to
					// access the <body> node because it's already gone, or at least in a state of limbo

					var window = this._iframeContentWindow(iframe),
						cn = win.body(window.document);
					binding.connects = doConnects(cn);
				});
				if(iframe.addEventListener){
					iframe.addEventListener("load", binding.onloadHandler, false);
				}else{
					iframe.attachEvent("onload", binding.onloadHandler);
				}
			}
		},

		unBindDomNode: function(/*String|DomNode*/ nodeName){
			// summary:
			//		Detach menu from given node

			var node;
			try{
				node = dom.byId(nodeName, this.ownerDocument);
			}catch(e){
				// On IE the dom.byId() call will get an exception if the attach point was
				// the <body> node of an <iframe> that has since been reloaded (and thus the
				// <body> node is in a limbo state of destruction.
				return;
			}

			// node["_dijitMenu" + this.id] contains index(+1) into my _bindings[] array
			var attrName = "_dijitMenu" + this.id;
			if(node && domAttr.has(node, attrName)){
				var bid = domAttr.get(node, attrName) - 1, b = this._bindings[bid], h;
				while((h = b.connects.pop())){
					h.remove();
				}

				// Remove listener for iframe onload events
				var iframe = b.iframe;
				if(iframe){
					if(iframe.removeEventListener){
						iframe.removeEventListener("load", b.onloadHandler, false);
					}else{
						iframe.detachEvent("onload", b.onloadHandler);
					}
				}

				domAttr.remove(node, attrName);
				delete this._bindings[bid];
			}
		},

		_scheduleOpen: function(/*DomNode?*/ target, /*DomNode?*/ iframe, /*Object?*/ coords){
			// summary:
			//		Set timer to display myself.  Using a timer rather than displaying immediately solves
			//		two problems:
			//
			//		1. IE: without the delay, focus work in "open" causes the system
			//		context menu to appear in spite of stopEvent.
			//
			//		2. Avoid double-shows on linux, where shift-F10 generates an oncontextmenu event
			//		even after a evt.preventDefault().  (Shift-F10 on windows doesn't generate the
			//		oncontextmenu event.)

			if(!this._openTimer){
				this._openTimer = this.defer(function(){
					delete this._openTimer;
					this._openMyself({
						target: target,
						iframe: iframe,
						coords: coords
					});
				}, 1);
			}
		},

		_openMyself: function(args){
			// summary:
			//		Internal function for opening myself when the user does a right-click or something similar.
			// args:
			//		This is an Object containing:
			//
			//		- target: The node that is being clicked
			//		- iframe: If an `<iframe>` is being clicked, iframe points to that iframe
			//		- coords: Put menu at specified x/y position in viewport, or if iframe is
			//		  specified, then relative to iframe.
			//
			//		_openMyself() formerly took the event object, and since various code references
			//		evt.target (after connecting to _openMyself()), using an Object for parameters
			//		(so that old code still works).

			var target = args.target,
				iframe = args.iframe,
				coords = args.coords,
				byKeyboard = !coords;

			// To be used by MenuItem event handlers to tell which node the menu was opened on
			this.currentTarget = target;

			// Get coordinates to open menu, either at specified (mouse) position or (if triggered via keyboard)
			// then near the node the menu is assigned to.
			if(coords){
				if(iframe){
					// Specified coordinates are on <body> node of an <iframe>, convert to match main document
					var ifc = domGeometry.position(iframe, true),
						window = this._iframeContentWindow(iframe),
						scroll = domGeometry.docScroll(window.document);

					var cs = domStyle.getComputedStyle(iframe),
						tp = domStyle.toPixelValue,
						left = (has("ie") && has("quirks") ? 0 : tp(iframe, cs.paddingLeft)) + (has("ie") && has("quirks") ? tp(iframe, cs.borderLeftWidth) : 0),
						top = (has("ie") && has("quirks") ? 0 : tp(iframe, cs.paddingTop)) + (has("ie") && has("quirks") ? tp(iframe, cs.borderTopWidth) : 0);

					coords.x += ifc.x + left - scroll.x;
					coords.y += ifc.y + top - scroll.y;
				}
			}else{
				coords = domGeometry.position(target, true);
				coords.x += 10;
				coords.y += 10;
			}

			var self = this;
			var prevFocusNode = this._focusManager.get("prevNode");
			var curFocusNode = this._focusManager.get("curNode");
			var savedFocusNode = !curFocusNode || (dom.isDescendant(curFocusNode, this.domNode)) ? prevFocusNode : curFocusNode;

			function closeAndRestoreFocus(){
				// user has clicked on a menu or popup
				if(self.refocus && savedFocusNode){
					savedFocusNode.focus();
				}
				pm.close(self);
			}

			pm.open({
				popup: this,
				x: coords.x,
				y: coords.y,
				onExecute: closeAndRestoreFocus,
				onCancel: closeAndRestoreFocus,
				orient: this.isLeftToRight() ? 'L' : 'R'
			});

			// Focus the menu even when opened by mouse, so that a click on blank area of screen will close it
			this.focus();
			if(!byKeyboard){
				// But then (when opened by mouse), mark Menu as passive, so that the first item isn't highlighted.
				// On IE9+ this needs to be on a delay because the focus is asynchronous.
				this.defer(function(){
					this._cleanUp(true);
				});
			}

			this._onBlur = function(){
				this.inherited('_onBlur', arguments);
				// Usually the parent closes the child widget but if this is a context
				// menu then there is no parent
				pm.close(this);
				// don't try to restore focus; user has clicked another part of the screen
				// and set focus there
			};
		},

		destroy: function(){
			array.forEach(this._bindings, function(b){
				if(b){
					this.unBindDomNode(b.node);
				}
			}, this);
			this.inherited(arguments);
		}
	});
});

},
'dijit/popup':function(){
define([
	"dojo/_base/array", // array.forEach array.some
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-construct", // domConstruct.create domConstruct.destroy
	"dojo/dom-geometry", // domGeometry.isBodyLtr
	"dojo/dom-style", // domStyle.set
	"dojo/has", // has("config-bgIframe")
	"dojo/keys",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"./place",
	"./BackgroundIframe",
	"./Viewport",
	"./main"    // dijit (defining dijit.popup to match API doc)
], function(array, aspect, declare, dom, domAttr, domConstruct, domGeometry, domStyle, has, keys, lang, on,
			place, BackgroundIframe, Viewport, dijit){

	// module:
	//		dijit/popup

	/*=====
	 var __OpenArgs = {
		 // popup: Widget
		 //		widget to display
		 // parent: Widget
		 //		the button etc. that is displaying this popup
		 // around: DomNode
		 //		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
		 // x: Integer
		 //		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		 // y: Integer
		 //		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
		 // orient: Object|String
		 //		When the around parameter is specified, orient should be a list of positions to try, ex:
		 //	|	[ "below", "above" ]
		 //		For backwards compatibility it can also be an (ordered) hash of tuples of the form
		 //		(around-node-corner, popup-node-corner), ex:
		 //	|	{ "BL": "TL", "TL": "BL" }
		 //		where BL means "bottom left" and "TL" means "top left", etc.
		 //
		 //		dijit/popup.open() tries to position the popup according to each specified position, in order,
		 //		until the popup appears fully within the viewport.
		 //
		 //		The default value is ["below", "above"]
		 //
		 //		When an (x,y) position is specified rather than an around node, orient is either
		 //		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
		 //		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
		 //		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
		 //		and the top-right corner.
		 // onCancel: Function
		 //		callback when user has canceled the popup by:
		 //
		 //		1. hitting ESC or
		 //		2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		 //		   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
		 // onClose: Function
		 //		callback whenever this popup is closed
		 // onExecute: Function
		 //		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		 // padding: place.__Position
		 //		adding a buffer around the opening position. This is only useful when around is not set.
		 // maxHeight: Integer
		 //		The max height for the popup.  Any popup taller than this will have scrollbars.
		 //		Set to Infinity for no max height.  Default is to limit height to available space in viewport,
		 //		above or below the aroundNode or specified x/y position.
	 };
	 =====*/

	function destroyWrapper(){
		// summary:
		//		Function to destroy wrapper when popup widget is destroyed.
		//		Left in this scope to avoid memory leak on IE8 on refresh page, see #15206.
		if(this._popupWrapper){
			domConstruct.destroy(this._popupWrapper);
			delete this._popupWrapper;
		}
	}

	var PopupManager = declare(null, {
		// summary:
		//		Used to show drop downs (ex: the select list of a ComboBox)
		//		or popups (ex: right-click context menus).

		// _stack: dijit/_WidgetBase[]
		//		Stack of currently popped up widgets.
		//		(someone opened _stack[0], and then it opened _stack[1], etc.)
		_stack: [],

		// _beginZIndex: Number
		//		Z-index of the first popup.   (If first popup opens other
		//		popups they get a higher z-index.)
		_beginZIndex: 1000,

		_idGen: 1,

		_repositionAll: function(){
			// summary:
			//		If screen has been scrolled, reposition all the popups in the stack.
			//		Then set timer to check again later.

			if(this._firstAroundNode){	// guard for when clearTimeout() on IE doesn't work
				var oldPos = this._firstAroundPosition,
					newPos = domGeometry.position(this._firstAroundNode, true),
					dx = newPos.x - oldPos.x,
					dy = newPos.y - oldPos.y;

				if(dx || dy){
					this._firstAroundPosition = newPos;
					for(var i = 0; i < this._stack.length; i++){
						var style = this._stack[i].wrapper.style;
						style.top = (parseInt(style.top, 10) + dy) + "px";
						if(style.right == "auto"){
							style.left = (parseInt(style.left, 10) + dx) + "px";
						}else{
							style.right = (parseInt(style.right, 10) - dx) + "px";
						}
					}
				}

				this._aroundMoveListener = setTimeout(lang.hitch(this, "_repositionAll"), dx || dy ? 10 : 50);
			}
		},

		_createWrapper: function(/*Widget*/ widget){
			// summary:
			//		Initialization for widgets that will be used as popups.
			//		Puts widget inside a wrapper DIV (if not already in one),
			//		and returns pointer to that wrapper DIV.

			var wrapper = widget._popupWrapper,
				node = widget.domNode;

			if(!wrapper){
				// Create wrapper <div> for when this widget [in the future] will be used as a popup.
				// This is done early because of IE bugs where creating/moving DOM nodes causes focus
				// to go wonky, see tests/robot/Toolbar.html to reproduce
				wrapper = domConstruct.create("div", {
					"class": "dijitPopup",
					style: { display: "none"},
					role: "region",
					"aria-label": widget["aria-label"] || widget.label || widget.name || widget.id
				}, widget.ownerDocumentBody);
				wrapper.appendChild(node);

				var s = node.style;
				s.display = "";
				s.visibility = "";
				s.position = "";
				s.top = "0px";

				widget._popupWrapper = wrapper;
				aspect.after(widget, "destroy", destroyWrapper, true);
			}

			return wrapper;
		},

		moveOffScreen: function(/*Widget*/ widget){
			// summary:
			//		Moves the popup widget off-screen.
			//		Do not use this method to hide popups when not in use, because
			//		that will create an accessibility issue: the offscreen popup is
			//		still in the tabbing order.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			// Besides setting visibility:hidden, move it out of the viewport, see #5776, #10111, #13604
			var ltr = domGeometry.isBodyLtr(widget.ownerDocument),
				style = {
					visibility: "hidden",
					top: "-9999px",
					display: ""
				};
			style[ltr ? "left" : "right"] = "-9999px";
			style[ltr ? "right" : "left"] = "auto";
			domStyle.set(wrapper, style);

			return wrapper;
		},

		hide: function(/*Widget*/ widget){
			// summary:
			//		Hide this popup widget (until it is ready to be shown).
			//		Initialization for widgets that will be used as popups
			//
			//		Also puts widget inside a wrapper DIV (if not already in one)
			//
			//		If popup widget needs to layout it should
			//		do so when it is made visible, and popup._onShow() is called.

			// Create wrapper if not already there
			var wrapper = this._createWrapper(widget);

			domStyle.set(wrapper, {
				display: "none",
				height: "auto",		// Open may have limited the height to fit in the viewport
				overflow: "visible",
				border: ""			// Open() may have moved border from popup to wrapper.
			});

			// Open() may have moved border from popup to wrapper.  Move it back.
			var node = widget.domNode;
			if("_originalStyle" in node){
				node.style.cssText = node._originalStyle;
			}
		},

		getTopPopup: function(){
			// summary:
			//		Compute the closest ancestor popup that's *not* a child of another popup.
			//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
			var stack = this._stack;
			for(var pi = stack.length - 1; pi > 0 && stack[pi].parent === stack[pi - 1].widget; pi--){
				/* do nothing, just trying to get right value for pi */
			}
			return stack[pi];
		},

		open: function(/*__OpenArgs*/ args){
			// summary:
			//		Popup the widget at the specified position
			//
			// example:
			//		opening at the mouse position
			//		|		popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
			//
			// example:
			//		opening the widget as a dropdown
			//		|		popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
			//
			//		Note that whatever widget called dijit/popup.open() should also listen to its own _onBlur callback
			//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

			var stack = this._stack,
				widget = args.popup,
				node = widget.domNode,
				orient = args.orient || ["below", "below-alt", "above", "above-alt"],
				ltr = args.parent ? args.parent.isLeftToRight() : domGeometry.isBodyLtr(widget.ownerDocument),
				around = args.around,
				id = (args.around && args.around.id) ? (args.around.id + "_dropdown") : ("popup_" + this._idGen++);

			// If we are opening a new popup that isn't a child of a currently opened popup, then
			// close currently opened popup(s).   This should happen automatically when the old popups
			// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
			while(stack.length && (!args.parent || !dom.isDescendant(args.parent.domNode, stack[stack.length - 1].widget.domNode))){
				this.close(stack[stack.length - 1].widget);
			}

			// Get pointer to popup wrapper, and create wrapper if it doesn't exist.  Remove display:none (but keep
			// off screen) so we can do sizing calculations.
			var wrapper = this.moveOffScreen(widget);

			if(widget.startup && !widget._started){
				widget.startup(); // this has to be done after being added to the DOM
			}

			// Limit height to space available in viewport either above or below aroundNode (whichever side has more
			// room), adding scrollbar if necessary. Can't add scrollbar to widget because it may be a <table> (ex:
			// dijit/Menu), so add to wrapper, and then move popup's border to wrapper so scroll bar inside border.
			var maxHeight, popupSize = domGeometry.position(node);
			if("maxHeight" in args && args.maxHeight != -1){
				maxHeight = args.maxHeight || Infinity;	// map 0 --> infinity for back-compat of _HasDropDown.maxHeight
			}else{
				var viewport = Viewport.getEffectiveBox(this.ownerDocument),
					aroundPos = around ? domGeometry.position(around, false) : {y: args.y - (args.padding||0), h: (args.padding||0) * 2};
				maxHeight = Math.floor(Math.max(aroundPos.y, viewport.h - (aroundPos.y + aroundPos.h)));
			}
			if(popupSize.h > maxHeight){
				// Get style of popup's border.  Unfortunately domStyle.get(node, "border") doesn't work on FF or IE,
				// and domStyle.get(node, "borderColor") etc. doesn't work on FF, so need to use fully qualified names.
				var cs = domStyle.getComputedStyle(node),
					borderStyle = cs.borderLeftWidth + " " + cs.borderLeftStyle + " " + cs.borderLeftColor;
				domStyle.set(wrapper, {
					overflowY: "scroll",
					height: maxHeight + "px",
					border: borderStyle	// so scrollbar is inside border
				});
				node._originalStyle = node.style.cssText;
				node.style.border = "none";
			}

			domAttr.set(wrapper, {
				id: id,
				style: {
					zIndex: this._beginZIndex + stack.length
				},
				"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] + "Popup",
				dijitPopupParent: args.parent ? args.parent.id : ""
			});

			if(stack.length == 0 && around){
				// First element on stack. Save position of aroundNode and setup listener for changes to that position.
				this._firstAroundNode = around;
				this._firstAroundPosition = domGeometry.position(around, true);
				this._aroundMoveListener = setTimeout(lang.hitch(this, "_repositionAll"), 50);
			}

			if(has("config-bgIframe") && !widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _WidgetBase.destroyRendering()
				widget.bgIframe = new BackgroundIframe(wrapper);
			}

			// position the wrapper node and make it visible
			var layoutFunc = widget.orient ? lang.hitch(widget, "orient") : null,
				best = around ?
					place.around(wrapper, around, orient, ltr, layoutFunc) :
					place.at(wrapper, args, orient == 'R' ? ['TR', 'BR', 'TL', 'BL'] : ['TL', 'BL', 'TR', 'BR'], args.padding,
						layoutFunc);

			wrapper.style.visibility = "visible";
			node.style.visibility = "visible";	// counteract effects from _HasDropDown

			var handlers = [];

			// provide default escape and tab key handling
			// (this will work for any widget, not just menu)
			handlers.push(on(wrapper, "keydown", lang.hitch(this, function(evt){
				if(evt.keyCode == keys.ESCAPE && args.onCancel){
					evt.stopPropagation();
					evt.preventDefault();
					args.onCancel();
				}else if(evt.keyCode == keys.TAB){
					evt.stopPropagation();
					evt.preventDefault();
					var topPopup = this.getTopPopup();
					if(topPopup && topPopup.onCancel){
						topPopup.onCancel();
					}
				}
			})));

			// watch for cancel/execute events on the popup and notify the caller
			// (for a menu, "execute" means clicking an item)
			if(widget.onCancel && args.onCancel){
				handlers.push(widget.on("cancel", args.onCancel));
			}

			handlers.push(widget.on(widget.onExecute ? "execute" : "change", lang.hitch(this, function(){
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onExecute){
					topPopup.onExecute();
				}
			})));

			stack.push({
				widget: widget,
				wrapper: wrapper,
				parent: args.parent,
				onExecute: args.onExecute,
				onCancel: args.onCancel,
				onClose: args.onClose,
				handlers: handlers
			});

			if(widget.onOpen){
				// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
				widget.onOpen(best);
			}

			return best;
		},

		close: function(/*Widget?*/ popup){
			// summary:
			//		Close specified popup and any popups that it parented.
			//		If no popup is specified, closes all popups.

			var stack = this._stack;

			// Basically work backwards from the top of the stack closing popups
			// until we hit the specified popup, but IIRC there was some issue where closing
			// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
			// closing C might close B indirectly and then the while() condition will run where stack==[A]...
			// so the while condition is constructed defensively.
			while((popup && array.some(stack, function(elem){
				return elem.widget == popup;
			})) ||
				(!popup && stack.length)){
				var top = stack.pop(),
					widget = top.widget,
					onClose = top.onClose;

				if(widget.onClose){
					// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here).
					// Actually, StackContainer also calls onClose(), but to mean that the pane is being deleted
					// (i.e. that the TabContainer's tab's [x] icon was clicked)
					widget.onClose();
				}

				var h;
				while(h = top.handlers.pop()){
					h.remove();
				}

				// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
				if(widget && widget.domNode){
					this.hide(widget);
				}

				if(onClose){
					onClose();
				}
			}

			if(stack.length == 0 && this._aroundMoveListener){
				clearTimeout(this._aroundMoveListener);
				this._firstAroundNode = this._firstAroundPosition = this._aroundMoveListener = null;
			}
		}
	});

	return (dijit.popup = new PopupManager());
});

},
'dijit/DropDownMenu':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/keys", // keys
	"dojo/text!./templates/Menu.html",
	"./_OnDijitClickMixin",
	"./_MenuBase"
], function(declare, keys, template, _OnDijitClickMixin, _MenuBase){

	// module:
	//		dijit/DropDownMenu

	return declare("dijit.DropDownMenu", [_MenuBase, _OnDijitClickMixin], {
		// summary:
		//		A menu, without features for context menu (Meaning, drop down menu)

		templateString: template,

		baseClass: "dijitMenu",

		// Arrow key navigation
		_onUpArrow: function(){
			this.focusPrev();
		},
		_onDownArrow: function(){
			this.focusNext();
		},
		_onRightArrow: function(/*Event*/ evt){
			this._moveToPopup(evt);
			evt.stopPropagation();
			evt.preventDefault();
		},
		_onLeftArrow: function(){
			if(this.parentMenu){
				if(this.parentMenu._isMenuBar){
					this.parentMenu.focusPrev();
				}else{
					this.onCancel(false);
				}
			}else{
				evt.stopPropagation();
				evt.preventDefault();
			}
		}
	});
});

},
'dijit/_MenuBase':function(){
define([
	"dojo/_base/array", // array.indexOf
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant domClass.replace
	"dojo/dom-attr",
	"dojo/dom-class", // domClass.replace
	"dojo/_base/lang", // lang.hitch
	"dojo/mouse", // mouse.enter, mouse.leave
	"dojo/on",
	"dojo/window",
	"./a11yclick",
	"./registry",
	"./_Widget",
	"./_CssStateMixin",
	"./_KeyNavContainer",
	"./_TemplatedMixin"
], function(array, declare, dom, domAttr, domClass, lang, mouse, on, winUtils, a11yclick,
			registry, _Widget, _CssStateMixin, _KeyNavContainer, _TemplatedMixin){

	// module:
	//		dijit/_MenuBase

	return declare("dijit._MenuBase", [_Widget, _TemplatedMixin, _KeyNavContainer, _CssStateMixin], {
		// summary:
		//		Abstract base class for Menu and MenuBar.
		//		Subclass should implement _onUpArrow(), _onDownArrow(), _onLeftArrow(), and _onRightArrow().

		// selected: dijit/MenuItem
		//		Currently selected (a.k.a. highlighted) MenuItem, or null if no MenuItem is selected.
		//		If a submenu is open, will be set to MenuItem that displayed the submenu.   OTOH, if
		//		this Menu is in passive mode (i.e. hasn't been clicked yet), will be null, because
		//		"selected" is not merely "hovered".
		selected: null,
		_setSelectedAttr: function(item){
			if(this.selected != item){
				if(this.selected){
					this.selected._setSelected(false);
					this._onChildDeselect(this.selected);
				}
				if(item){
					item._setSelected(true);
				}
				this._set("selected", item);
			}
		},

		// activated: [readonly] Boolean
		//		This Menu has been clicked (mouse or via space/arrow key) or opened as a submenu,
		//		so mere mouseover will open submenus.  Focusing a menu via TAB does NOT automatically make it active
		//		since TAB is a navigation operation and not a selection one.
		//		For Windows apps, pressing the ALT key focuses the menubar menus (similar to TAB navigation) but the
		//		menu is not active (ie no dropdown) until an item is clicked.
		activated: false,
		_setActivatedAttr: function(val){
			domClass.toggle(this.domNode, "dijitMenuActive", val);
			domClass.toggle(this.domNode, "dijitMenuPassive", !val);
			this._set("activated", val);
		},

		// parentMenu: [readonly] Widget
		//		pointer to menu that displayed me
		parentMenu: null,

		// popupDelay: Integer
		//		After a menu has been activated (by clicking on it etc.), number of milliseconds before hovering
		//		(without clicking) another MenuItem causes that MenuItem's popup to automatically open.
		popupDelay: 500,

		// passivePopupDelay: Integer
		//		For a passive (unclicked) Menu, number of milliseconds before hovering (without clicking) will cause
		//		the popup to open.  Default is Infinity, meaning you need to click the menu to open it.
		passivePopupDelay: Infinity,

		// autoFocus: Boolean
		//		A toggle to control whether or not a Menu gets focused when opened as a drop down from a MenuBar
		//		or DropDownButton/ComboButton.   Note though that it always get focused when opened via the keyboard.
		autoFocus: false,

		childSelector: function(/*DOMNode*/ node){
			// summary:
			//		Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children
			//		like MenuSeparator.  If subclass overrides to a string (ex: "> *"), the subclass must require dojo/query.
			// tags:
			//		protected

			var widget = registry.byNode(node);
			return node.parentNode == this.containerNode && widget && widget.focus;
		},

		postCreate: function(){
			var self = this,
				matches = typeof this.childSelector == "string" ? this.childSelector : lang.hitch(this, "childSelector");
			this.own(
				on(this.containerNode, on.selector(matches, mouse.enter), function(){
					self.onItemHover(registry.byNode(this));
				}),
				on(this.containerNode, on.selector(matches, mouse.leave), function(){
					self.onItemUnhover(registry.byNode(this));
				}),
				on(this.containerNode, on.selector(matches, a11yclick), function(evt){
					self.onItemClick(registry.byNode(this), evt);
					evt.stopPropagation();
					evt.preventDefault();
				})
			);
			this.inherited(arguments);
		},

		onKeyboardSearch: function(/*MenuItem*/ item, /*Event*/ evt, /*String*/ searchString, /*Number*/ numMatches){
			// summary:
			//		Attach point for notification about when a menu item has been searched for
			//		via the keyboard search mechanism.
			// tags:
			//		protected
			this.inherited(arguments);
			if(!!item && (numMatches == -1 || (!!item.popup && numMatches == 1))){
				this.onItemClick(item, evt);
			}
		},

		_keyboardSearchCompare: function(/*dijit/_WidgetBase*/ item, /*String*/ searchString){
			// summary:
			//		Compares the searchString to the widget's text label, returning:
			//		-1: a high priority match and stop searching
			//		 0: no match
			//		 1: a match but keep looking for a higher priority match
			// tags:
			//		private
			if(!!item.shortcutKey){
				// accessKey matches have priority
				return searchString == item.shortcutKey.toLowerCase() ? -1 : 0;
			}
			return this.inherited(arguments) ? 1 : 0; // change return value of -1 to 1 so that searching continues
		},

		onExecute: function(){
			// summary:
			//		Attach point for notification about when a menu item has been executed.
			//		This is an internal mechanism used for Menus to signal to their parent to
			//		close them, because they are about to execute the onClick handler.  In
			//		general developers should not attach to or override this method.
			// tags:
			//		protected
		},

		onCancel: function(/*Boolean*/ /*===== closeAll =====*/){
			// summary:
			//		Attach point for notification about when the user cancels the current menu
			//		This is an internal mechanism used for Menus to signal to their parent to
			//		close them.  In general developers should not attach to or override this method.
			// tags:
			//		protected
		},

		_moveToPopup: function(/*Event*/ evt){
			// summary:
			//		This handles the right arrow key (left arrow key on RTL systems),
			//		which will either open a submenu, or move to the next item in the
			//		ancestor MenuBar
			// tags:
			//		private

			if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
				this.onItemClick(this.focusedChild, evt);
			}else{
				var topMenu = this._getTopMenu();
				if(topMenu && topMenu._isMenuBar){
					topMenu.focusNext();
				}
			}
		},

		_onPopupHover: function(/*Event*/ /*===== evt =====*/){
			// summary:
			//		This handler is called when the mouse moves over the popup.
			// tags:
			//		private

			// if the mouse hovers over a menu popup that is in pending-close state,
			// then stop the close operation.
			// This can't be done in onItemHover since some popup targets don't have MenuItems (e.g. ColorPicker)

			// highlight the parent menu item pointing to this popup (in case user temporarily moused over another MenuItem)
			this.set("selected", this.currentPopupItem);

			// cancel the pending close (if there is one) (in case user temporarily moused over another MenuItem)
			this._stopPendingCloseTimer();
		},

		onItemHover: function(/*MenuItem*/ item){
			// summary:
			//		Called when cursor is over a MenuItem.
			// tags:
			//		protected

			// Don't do anything unless user has "activated" the menu by:
			//		1) clicking it
			//		2) opening it from a parent menu (which automatically activates it)

			if(this.activated){
				this.set("selected", item);
				if(item.popup && !item.disabled && !this.hover_timer){
					this.hover_timer = this.defer(function(){
						this._openItemPopup(item);
					}, this.popupDelay);
				}
			}else if(this.passivePopupDelay < Infinity){
				if(this.passive_hover_timer){
					this.passive_hover_timer.remove();
				}
				this.passive_hover_timer = this.defer(function(){
					this.onItemClick(item, {type: "click"});
				}, this.passivePopupDelay);
			}

			this._hoveredChild = item;

			item._set("hovering", true);
		},

		_onChildDeselect: function(item){
			// summary:
			//		Called when a child MenuItem becomes deselected.   Setup timer to close its popup.

			this._stopPopupTimer();

			// Setup timer to close all popups that are open and descendants of this menu.
			// Will be canceled if user quickly moves the mouse over the popup.
			if(this.currentPopupItem == item){
				this._stopPendingCloseTimer();
				this._pendingClose_timer = this.defer(function(){
					this._pendingClose_timer = null;
					this.currentPopupItem = null;
					item._closePopup(); // this calls onClose
				}, this.popupDelay);
			}
		},

		onItemUnhover: function(/*MenuItem*/ item){
			// summary:
			//		Callback fires when mouse exits a MenuItem
			// tags:
			//		protected

			if(this._hoveredChild == item){
				this._hoveredChild = null;
			}

			if(this.passive_hover_timer){
				this.passive_hover_timer.remove();
				this.passive_hover_timer = null;
			}

			item._set("hovering", false);
		},

		_stopPopupTimer: function(){
			// summary:
			//		Cancels the popup timer because the user has stop hovering
			//		on the MenuItem, etc.
			// tags:
			//		private

			if(this.hover_timer){
				this.hover_timer = this.hover_timer.remove();
			}
		},

		_stopPendingCloseTimer: function(){
			// summary:
			//		Cancels the pending-close timer because the close has been preempted
			// tags:
			//		private
			if(this._pendingClose_timer){
				this._pendingClose_timer = this._pendingClose_timer.remove();
			}
		},

		_getTopMenu: function(){
			// summary:
			//		Returns the top menu in this chain of Menus
			// tags:
			//		private
			for(var top = this; top.parentMenu; top = top.parentMenu){}
			return top;
		},

		onItemClick: function(/*dijit/_WidgetBase*/ item, /*Event*/ evt){
			// summary:
			//		Handle clicks on an item.
			// tags:
			//		private

			if(this.passive_hover_timer){
				this.passive_hover_timer.remove();
			}

			this.focusChild(item);

			if(item.disabled){
				return false;
			}

			if(item.popup){
				this.set("selected", item);
				this.set("activated", true);
				var byKeyboard = /^key/.test(evt._origType || evt.type) ||
					(evt.clientX == 0 && evt.clientY == 0);	// detects accessKey like ALT+SHIFT+F, where type is "click"
				this._openItemPopup(item, byKeyboard);
			}else{
				// before calling user defined handler, close hierarchy of menus
				// and restore focus to place it was when menu was opened
				this.onExecute();

				// user defined handler for click
				item._onClick ? item._onClick(evt) : item.onClick(evt);
			}
		},

		_openItemPopup: function(/*dijit/MenuItem*/ from_item, /*Boolean*/ focus){
			// summary:
			//		Open the popup to the side of/underneath the current menu item, and optionally focus first item
			// tags:
			//		protected

			if(from_item == this.currentPopupItem){
				// Specified popup is already being shown, so just return
				return;
			}
			if(this.currentPopupItem){
				// If another popup is currently shown, then close it
				this._stopPendingCloseTimer();
				this.currentPopupItem._closePopup();
			}
			this._stopPopupTimer();

			var popup = from_item.popup;
			popup.parentMenu = this;

			// detect mouseover of the popup to handle lazy mouse movements that temporarily focus other menu items\c
			this.own(this._mouseoverHandle = on.once(popup.domNode, "mouseover", lang.hitch(this, "_onPopupHover")));

			var self = this;
			from_item._openPopup({
				parent: this,
				orient: this._orient || ["after", "before"],
				onCancel: function(){ // called when the child menu is canceled
					if(focus){
						// put focus back on my node before focused node is hidden
						self.focusChild(from_item);
					}

					// close the submenu (be sure this is done _after_ focus is moved)
					self._cleanUp();
				},
				onExecute: lang.hitch(this, "_cleanUp", true),
				onClose: function(){
					// Remove handler created by onItemHover
					if(self._mouseoverHandle){
						self._mouseoverHandle.remove();
						delete self._mouseoverHandle;
					}
				}
			}, focus);

			this.currentPopupItem = from_item;

			// TODO: focusing a popup should clear tabIndex on Menu (and it's child MenuItems), so that neither
			// TAB nor SHIFT-TAB returns to the menu.  Only ESC or ENTER should return to the menu.
		},

		onOpen: function(/*Event*/ /*===== e =====*/){
			// summary:
			//		Callback when this menu is opened.
			//		This is called by the popup manager as notification that the menu
			//		was opened.
			// tags:
			//		private

			this.isShowingNow = true;
			this.set("activated", true);
		},

		onClose: function(){
			// summary:
			//		Callback when this menu is closed.
			//		This is called by the popup manager as notification that the menu
			//		was closed.
			// tags:
			//		private

			this.set("activated", false);
			this.set("selected", null);
			this.isShowingNow = false;
			this.parentMenu = null;
		},

		_closeChild: function(){
			// summary:
			//		Called when submenu is clicked or focus is lost.  Close hierarchy of menus.
			// tags:
			//		private
			this._stopPopupTimer();

			if(this.currentPopupItem){
				// If focus is on a descendant MenuItem then move focus to me,
				// because IE doesn't like it when you display:none a node with focus,
				// and also so keyboard users don't lose control.
				// Likely, immediately after a user defined onClick handler will move focus somewhere
				// else, like a Dialog.
				if(this.focused){
					domAttr.set(this.selected.focusNode, "tabIndex", this.tabIndex);
					this.selected.focusNode.focus();
				}

				// Close all popups that are open and descendants of this menu
				this.currentPopupItem._closePopup();
				this.currentPopupItem = null;
			}
		},

		_onItemFocus: function(/*MenuItem*/ item){
			// summary:
			//		Called when child of this Menu gets focus from:
			//
			//		1. clicking it
			//		2. tabbing into it
			//		3. being opened by a parent menu.
			//
			//		This is not called just from mouse hover.

			if(this._hoveredChild && this._hoveredChild != item){
				this.onItemUnhover(this._hoveredChild);	// any previous mouse movement is trumped by focus selection
			}
			this.set("selected", item);
		},

		_onBlur: function(){
			// summary:
			//		Called when focus is moved away from this Menu and it's submenus.
			// tags:
			//		protected

			this._cleanUp(true);
			this.inherited(arguments);
		},

		_cleanUp: function(/*Boolean*/ clearSelectedItem){
			// summary:
			//		Called when the user is done with this menu.  Closes hierarchy of menus.
			// tags:
			//		private

			this._closeChild(); // don't call this.onClose since that's incorrect for MenuBar's that never close
			if(typeof this.isShowingNow == 'undefined'){ // non-popup menu doesn't call onClose
				this.set("activated", false);
			}

			if(clearSelectedItem){
				this.set("selected", null);
			}
		}
	});
});

},
'dijit/_CssStateMixin':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.isDescendant()
	"dojo/dom-class", // domClass.toggle
	"dojo/has",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/domReady",
	"dojo/touch",
	"dojo/_base/window", // win.body
	"./a11yclick",
	"./registry"
], function(array, declare, dom, domClass, has, lang, on, domReady, touch, win, a11yclick, registry){

	// module:
	//		dijit/_CssStateMixin

	var CssStateMixin = declare("dijit._CssStateMixin", [], {
		// summary:
		//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
		//		state changes, and also higher-level state changes such becoming disabled or selected.
		//
		// description:
		//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
		//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
		//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
		//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
		//
		//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
		//
		//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
		//		within the widget).

		/*=====
		 // cssStateNodes: [protected] Object
		 //		Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that
		 //		need CSS classes applied on mouse hover/press and focus.
		 //
		 //		Each entry in this optional hash is a an attach-point name (like "upArrowButton") mapped to a CSS class name
		 //		(like "dijitUpArrowButton"). Example:
		 //	|		{
		 //	|			"upArrowButton": "dijitUpArrowButton",
		 //	|			"downArrowButton": "dijitDownArrowButton"
		 //	|		}
		 //		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
		 //		is hovered, etc.
		 cssStateNodes: {},
		 =====*/

		// hovering: [readonly] Boolean
		//		True if cursor is over this widget
		hovering: false,

		// active: [readonly] Boolean
		//		True if mouse was pressed while over this widget, and hasn't been released yet
		active: false,

		_applyAttributes: function(){
			// This code would typically be in postCreate(), but putting in _applyAttributes() for
			// performance: so the class changes happen before DOM is inserted into the document.
			// Change back to postCreate() in 2.0.  See #11635.

			this.inherited(arguments);

			// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
			array.forEach(["disabled", "readOnly", "checked", "selected", "focused", "state", "hovering", "active", "_opened"], function(attr){
				this.watch(attr, lang.hitch(this, "_setStateClass"));
			}, this);

			// Track hover and active mouse events on widget root node, plus possibly on subnodes
			for(var ap in this.cssStateNodes || {}){
				this._trackMouseState(this[ap], this.cssStateNodes[ap]);
			}
			this._trackMouseState(this.domNode, this.baseClass);

			// Set state initially; there's probably no hover/active/focus state but widget might be
			// disabled/readonly/checked/selected so we want to set CSS classes for those conditions.
			this._setStateClass();
		},

		_cssMouseEvent: function(/*Event*/ event){
			// summary:
			//		Handler for CSS event on this.domNode. Sets hovering and active properties depending on mouse state,
			//		which triggers _setStateClass() to set appropriate CSS classes for this.domNode.

			if(!this.disabled){
				switch(event.type){
					case "mouseover":
					case "MSPointerOver":
						this._set("hovering", true);
						this._set("active", this._mouseDown);
						break;
					case "mouseout":
					case "MSPointerOut":
						this._set("hovering", false);
						this._set("active", false);
						break;
					case "mousedown":
					case "touchstart":
					case "MSPointerDown":
					case "keydown":
						this._set("active", true);
						break;
					case "mouseup":
					case "dojotouchend":
					case "keyup":
						this._set("active", false);
						break;
				}
			}
		},

		_setStateClass: function(){
			// summary:
			//		Update the visual state of the widget by setting the css classes on this.domNode
			//		(or this.stateNode if defined) by combining this.baseClass with
			//		various suffixes that represent the current widget state(s).
			//
			// description:
			//		In the case where a widget has multiple
			//		states, it sets the class based on all possible
			//		combinations.  For example, an invalid form widget that is being hovered
			//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
			//
			//		The widget may have one or more of the following states, determined
			//		by this.state, this.checked, this.valid, and this.selected:
			//
			//		- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
			//		- Incomplete - ValidationTextBox sets this.state to "Incomplete" if the current input value is not finished yet
			//		- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
			//		- Selected - ex: currently selected tab will have this.selected==true
			//
			//		In addition, it may have one or more of the following states,
			//		based on this.disabled and flags set in _onMouse (this.active, this.hovering) and from focus manager (this.focused):
			//
			//		- Disabled	- if the widget is disabled
			//		- Active		- if the mouse (or space/enter key?) is being pressed down
			//		- Focused		- if the widget has focus
			//		- Hover		- if the mouse is over the widget

			// Compute new set of classes
			var newStateClasses = this.baseClass.split(" ");

			function multiply(modifier){
				newStateClasses = newStateClasses.concat(array.map(newStateClasses, function(c){
					return c + modifier;
				}), "dijit" + modifier);
			}

			if(!this.isLeftToRight()){
				// For RTL mode we need to set an addition class like dijitTextBoxRtl.
				multiply("Rtl");
			}

			var checkedState = this.checked == "mixed" ? "Mixed" : (this.checked ? "Checked" : "");
			if(this.checked){
				multiply(checkedState);
			}
			if(this.state){
				multiply(this.state);
			}
			if(this.selected){
				multiply("Selected");
			}
			if(this._opened){
				multiply("Opened");
			}

			if(this.disabled){
				multiply("Disabled");
			}else if(this.readOnly){
				multiply("ReadOnly");
			}else{
				if(this.active){
					multiply("Active");
				}else if(this.hovering){
					multiply("Hover");
				}
			}

			if(this.focused){
				multiply("Focused");
			}

			// Remove old state classes and add new ones.
			// For performance concerns we only write into domNode.className once.
			var tn = this.stateNode || this.domNode,
				classHash = {};	// set of all classes (state and otherwise) for node

			array.forEach(tn.className.split(" "), function(c){
				classHash[c] = true;
			});

			if("_stateClasses" in this){
				array.forEach(this._stateClasses, function(c){
					delete classHash[c];
				});
			}

			array.forEach(newStateClasses, function(c){
				classHash[c] = true;
			});

			var newClasses = [];
			for(var c in classHash){
				newClasses.push(c);
			}
			tn.className = newClasses.join(" ");

			this._stateClasses = newStateClasses;
		},

		_subnodeCssMouseEvent: function(node, clazz, evt){
			// summary:
			//		Handler for hover/active mouse event on widget's subnode
			if(this.disabled || this.readOnly){
				return;
			}

			function hover(isHovering){
				domClass.toggle(node, clazz + "Hover", isHovering);
			}

			function active(isActive){
				domClass.toggle(node, clazz + "Active", isActive);
			}

			function focused(isFocused){
				domClass.toggle(node, clazz + "Focused", isFocused);
			}

			switch(evt.type){
				case "mouseover":
				case "MSPointerOver":
					hover(true);
					break;
				case "mouseout":
				case "MSPointerOut":
					hover(false);
					active(false);
					break;
				case "mousedown":
				case "touchstart":
				case "MSPointerDown":
				case "keydown":
					active(true);
					break;
				case "mouseup":
				case "MSPointerUp":
				case "dojotouchend":
				case "keyup":
					active(false);
					break;
				case "focus":
				case "focusin":
					focused(true);
					break;
				case "blur":
				case "focusout":
					focused(false);
					break;
			}
		},

		_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
			// summary:
			//		Track mouse/focus events on specified node and set CSS class on that node to indicate
			//		current state.   Usually not called directly, but via cssStateNodes attribute.
			// description:
			//		Given class=foo, will set the following CSS class on the node
			//
			//		- fooActive: if the user is currently pressing down the mouse button while over the node
			//		- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
			//		- fooFocus: if the node is focused
			//
			//		Note that it won't set any classes if the widget is disabled.
			// node: DomNode
			//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
			//		is handled specially and automatically just by mixing in this class.
			// clazz: String
			//		CSS class name (ex: dijitSliderUpArrow)

			// Flag for listener code below to call this._cssMouseEvent() or this._subnodeCssMouseEvent()
			// when node is hovered/active
			node._cssState = clazz;
		}
	});

	domReady(function(){
		// Document level listener to catch hover etc. events on widget root nodes and subnodes.
		// Note that when the mouse is moved quickly, a single onmouseenter event could signal that multiple widgets
		// have been hovered or unhovered (try test_Accordion.html)

		function pointerHandler(evt, target, relatedTarget){
			// Handler for mouseover, mouseout, a11yclick.press and a11click.release events

			// Poor man's event propagation.  Don't propagate event to ancestors of evt.relatedTarget,
			// to avoid processing mouseout events moving from a widget's domNode to a descendant node;
			// such events shouldn't be interpreted as a mouseleave on the widget.
			if(relatedTarget && dom.isDescendant(relatedTarget, target)){
				return;
			}

			for(var node = target; node && node != relatedTarget; node = node.parentNode){
				// Process any nodes with _cssState property.   They are generally widget root nodes,
				// but could also be sub-nodes within a widget
				if(node._cssState){
					var widget = registry.getEnclosingWidget(node);
					if(widget){
						if(node == widget.domNode){
							// event on the widget's root node
							widget._cssMouseEvent(evt);
						}else{
							// event on widget's sub-node
							widget._subnodeCssMouseEvent(node, node._cssState, evt);
						}
					}
				}
			}
		}

		var body = win.body(), activeNode;

		// Handle pointer related events (i.e. mouse or touch)
		on(body, touch.over, function(evt){
			// Using touch.over rather than mouseover mainly to ignore phantom mouse events on iOS.
			pointerHandler(evt, evt.target, evt.relatedTarget);
		});
		on(body, touch.out, function(evt){
			// Using touch.out rather than mouseout mainly to ignore phantom mouse events on iOS.
			pointerHandler(evt, evt.target, evt.relatedTarget);
		});
		on(body, a11yclick.press, function(evt){
			// Save the a11yclick.press target to reference when the a11yclick.release comes.
			activeNode = evt.target;
			pointerHandler(evt, activeNode)
		});
		on(body, a11yclick.release, function(evt){
			// The release event could come on a separate node than the press event, if for example user slid finger.
			// Reference activeNode to reset the state of the node that got state set in the a11yclick.press handler.
			pointerHandler(evt, activeNode);
			activeNode = null;
		});

		// Track focus events on widget sub-nodes that have been registered via _trackMouseState().
		// However, don't track focus events on the widget root nodes, because focus is tracked via the
		// focus manager (and it's not really tracking focus, but rather tracking that focus is on one of the widget's
		// nodes or a subwidget's node or a popup node, etc.)
		// Remove for 2.0 (if focus CSS needed, just use :focus pseudo-selector).
		on(body, "focusin, focusout", function(evt){
			var node = evt.target;
			if(node._cssState && !node.getAttribute("widgetId")){
				var widget = registry.getEnclosingWidget(node);
				if(widget){
					widget._subnodeCssMouseEvent(node, node._cssState, evt);
				}
			}
		});
	});

	return CssStateMixin;
});

},
'dijit/_KeyNavContainer':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // keys.END keys.HOME
	"dojo/_base/lang", // lang.hitch
	"./registry",
	"./_Container",
	"./_FocusMixin",
	"./_KeyNavMixin"
], function(array, declare, domAttr, kernel, keys, lang, registry, _Container, _FocusMixin, _KeyNavMixin){


	// module:
	//		dijit/_KeyNavContainer

	return declare("dijit._KeyNavContainer", [_FocusMixin, _KeyNavMixin, _Container], {
		// summary:
		//		A _Container with keyboard navigation of its children.
		// description:
		//		Provides normalized keyboard and focusing code for Container widgets.
		//		To use this mixin, call connectKeyNavHandlers() in postCreate().
		//		Also, child widgets must implement a focus() method.

		connectKeyNavHandlers: function(/*keys[]*/ prevKeyCodes, /*keys[]*/ nextKeyCodes){
			// summary:
			//		Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,
			//		but the preferred method is to override _onLeftArrow() and _onRightArrow(), or
			//		_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().
			// prevKeyCodes: keys[]
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: keys[]
			//		Key codes for navigating to the next child.
			// tags:
			//		protected

			// TODO: remove for 2.0, and make subclasses override _onLeftArrow, _onRightArrow etc. instead.

			var keyCodes = (this._keyNavCodes = {});
			var prev = lang.hitch(this, "focusPrev");
			var next = lang.hitch(this, "focusNext");
			array.forEach(prevKeyCodes, function(code){
				keyCodes[code] = prev;
			});
			array.forEach(nextKeyCodes, function(code){
				keyCodes[code] = next;
			});
			keyCodes[keys.HOME] = lang.hitch(this, "focusFirstChild");
			keyCodes[keys.END] = lang.hitch(this, "focusLastChild");
		},

		startupKeyNavChildren: function(){
			kernel.deprecated("startupKeyNavChildren() call no longer needed", "", "2.0");
		},

		startup: function(){
			this.inherited(arguments);
			array.forEach(this.getChildren(), lang.hitch(this, "_startupChild"));
		},

		addChild: function(/*dijit/_WidgetBase*/ widget, /*int?*/ insertIndex){
			this.inherited(arguments);
			this._startupChild(widget);
		},

		_startupChild: function(/*dijit/_WidgetBase*/ widget){
			// summary:
			//		Setup for each child widget.
			// description:
			//		Sets tabIndex=-1 on each child, so that the tab key will
			//		leave the container rather than visiting each child.
			//
			//		Note: if you add children by a different method than addChild(), then need to call this manually
			//		or at least make sure the child's tabIndex is -1.
			//
			//		Note: see also _LayoutWidget.setupChild(), which is also called for each child widget.
			// tags:
			//		private

			widget.set("tabIndex", "-1");
		},

		_getFirst: function(){
			// summary:
			//		Returns the first child.
			// tags:
			//		abstract extension
			var children = this.getChildren();
			return children.length ? children[0] : null;
		},

		_getLast: function(){
			// summary:
			//		Returns the last descendant.
			// tags:
			//		abstract extension
			var children = this.getChildren();
			return children.length ? children[children.length - 1] : null;
		},

		focusNext: function(){
			// summary:
			//		Focus the next widget
			// tags:
			//		protected
			this.focusChild(this._getNextFocusableChild(this.focusedChild, 1));
		},

		focusPrev: function(){
			// summary:
			//		Focus the last focusable node in the previous widget
			//		(ex: go to the ComboButton icon section rather than button section)
			// tags:
			//		protected
			this.focusChild(this._getNextFocusableChild(this.focusedChild, -1), true);
		},

		childSelector: function(/*DOMNode*/ node){
			// Implement _KeyNavMixin.childSelector, to identify focusable child nodes.
			// If we allowed a dojo/query dependency from this module this could more simply be a string "> *"
			// instead of this function.

			var node = registry.byNode(node);
			return node && node.getParent() == this;
		}
	});
});

},
'dijit/_Container':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.place
	"dojo/_base/kernel" // kernel.deprecated
], function(array, declare, domConstruct, kernel){

	// module:
	//		dijit/_Container

	return declare("dijit._Container", null, {
		// summary:
		//		Mixin for widgets that contain HTML and/or a set of widget children.

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// All widgets with descendants must set containerNode.
				// NB: this code doesn't quite work right because for TabContainer it runs before
				// _TemplatedMixin::buildRendering(), and thus
				// sets this.containerNode to this.domNode, later to be overridden by the assignment in the template.
				this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit/_WidgetBase*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).

			// I want to just call domConstruct.place(widget.domNode, this.containerNode, insertIndex), but the counting
			// is thrown off by text nodes and comment nodes that show up when constructed by markup.
			// In the future consider stripping those nodes on construction, either in the parser or this widget code.
			var refNode = this.containerNode;
			if(insertIndex > 0){
				// Old-school way to get nth child; dojo.query would be easier but _Container was weened from dojo.query
				// in #10087 to minimize download size.   Not sure if that's still and issue with new smaller dojo/query.
				refNode = refNode.firstChild;
				while(insertIndex > 0){
					if(refNode.nodeType == 1){ insertIndex--; }
					refNode = refNode.nextSibling;
				}
				if(refNode){
					insertIndex = "before";
				}else{
					// to support addChild(child, n-1) where there are n children (should add child at end)
					refNode = this.containerNode;
					insertIndex = "last";
				}
			}

			domConstruct.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget|int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove (ie, removeChild(5) removes the sixth widget).

			if(typeof widget == "number"){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.
			return this.getChildren().length > 0;	// Boolean
		},

		_getSiblingOfChild: function(/*dijit/_WidgetBase*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//		private
			kernel.deprecated(this.declaredClass+"::_getSiblingOfChild() is deprecated. Use _KeyNavMixin::_getNext() instead.", "", "2.0");
			var children = this.getChildren(),
				idx = array.indexOf(children, child);	// int
			return children[idx + dir];
		},

		getIndexOfChild: function(/*dijit/_WidgetBase*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return array.indexOf(this.getChildren(), child);	// int
		}
	});
});

},
'dijit/_KeyNavMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/keys", // keys.END keys.HOME, keys.LEFT_ARROW etc.
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dijit/registry",
	"dijit/_FocusMixin"        // to make _onBlur() work
], function(array, declare, domAttr, keys, lang, on, registry, _FocusMixin){

	// module:
	//		dijit/_KeyNavMixin

	return declare("dijit._KeyNavMixin", _FocusMixin, {
		// summary:
		//		A mixin to allow arrow key and letter key navigation of child or descendant widgets.
		//		It can be used by dijit/_Container based widgets with a flat list of children,
		//		or more complex widgets like dijit/Tree.
		//
		//		To use this mixin, the subclass must:
		//
		//			- Implement  _getNext(), _getFirst(), _getLast(), _onLeftArrow(), _onRightArrow()
		//			  _onDownArrow(), _onUpArrow() methods to handle home/end/left/right/up/down keystrokes.
		//			  Next and previous in this context refer to a linear ordering of the descendants used
		//			  by letter key search.
		//			- Set all descendants' initial tabIndex to "-1"; both initial descendants and any
		//			  descendants added later, by for example addChild()
		//			- Define childSelector to a function or string that identifies focusable descendant widgets
		//
		//		Also, child widgets must implement a focus() method.

		/*=====
		 // focusedChild: [protected readonly] Widget
		 //		The currently focused child widget, or null if there isn't one
		 focusedChild: null,

		 // _keyNavCodes: Object
		 //		Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.
		 //		Usually not used directly, as subclasses can instead override _onLeftArrow() etc.
		 _keyNavCodes: {},
		 =====*/

		// tabIndex: String
		//		Tab index of the container; same as HTML tabIndex attribute.
		//		Note then when user tabs into the container, focus is immediately
		//		moved to the first item in the container.
		tabIndex: "0",

		// childSelector: [protected abstract] Function||String
		//		Selector (passed to on.selector()) used to identify what to treat as a child widget.   Used to monitor
		//		focus events and set this.focusedChild.   Must be set by implementing class.   If this is a string
		//		(ex: "> *") then the implementing class must require dojo/query.
		childSelector: null,

		postCreate: function(){
			this.inherited(arguments);

			// Set tabIndex on this.domNode.  Will be automatic after #7381 is fixed.
			domAttr.set(this.domNode, "tabIndex", this.tabIndex);

			if(!this._keyNavCodes){
				var keyCodes = this._keyNavCodes = {};
				keyCodes[keys.HOME] = lang.hitch(this, "focusFirstChild");
				keyCodes[keys.END] = lang.hitch(this, "focusLastChild");
				keyCodes[this.isLeftToRight() ? keys.LEFT_ARROW : keys.RIGHT_ARROW] = lang.hitch(this, "_onLeftArrow");
				keyCodes[this.isLeftToRight() ? keys.RIGHT_ARROW : keys.LEFT_ARROW] = lang.hitch(this, "_onRightArrow");
				keyCodes[keys.UP_ARROW] = lang.hitch(this, "_onUpArrow");
				keyCodes[keys.DOWN_ARROW] = lang.hitch(this, "_onDownArrow");
			}

			var self = this,
				childSelector = typeof this.childSelector == "string"
					? this.childSelector
					: lang.hitch(this, "childSelector");
			this.own(
				on(this.domNode, "keypress", lang.hitch(this, "_onContainerKeypress")),
				on(this.domNode, "keydown", lang.hitch(this, "_onContainerKeydown")),
				on(this.domNode, "focus", lang.hitch(this, "_onContainerFocus")),
				on(this.containerNode, on.selector(childSelector, "focusin"), function(evt){
					self._onChildFocus(registry.getEnclosingWidget(this), evt);
				})
			);
		},

		_onLeftArrow: function(){
			// summary:
			//		Called on left arrow key, or right arrow key if widget is in RTL mode.
			//		Should go back to the previous child in horizontal container widgets like Toolbar.
			// tags:
			//		extension
		},

		_onRightArrow: function(){
			// summary:
			//		Called on right arrow key, or left arrow key if widget is in RTL mode.
			//		Should go to the next child in horizontal container widgets like Toolbar.
			// tags:
			//		extension
		},

		_onUpArrow: function(){
			// summary:
			//		Called on up arrow key. Should go to the previous child in vertical container widgets like Menu.
			// tags:
			//		extension
		},

		_onDownArrow: function(){
			// summary:
			//		Called on down arrow key. Should go to the next child in vertical container widgets like Menu.
			// tags:
			//		extension
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		_getFirstFocusableChild: function(){
			// summary:
			//		Returns first child that can be focused.

			// Leverage _getNextFocusableChild() to skip disabled children
			return this._getNextFocusableChild(null, 1);	// dijit/_WidgetBase
		},

		_getLastFocusableChild: function(){
			// summary:
			//		Returns last child that can be focused.

			// Leverage _getNextFocusableChild() to skip disabled children
			return this._getNextFocusableChild(null, -1);	// dijit/_WidgetBase
		},

		focusFirstChild: function(){
			// summary:
			//		Focus the first focusable child in the container.
			// tags:
			//		protected

			this.focusChild(this._getFirstFocusableChild());
		},

		focusLastChild: function(){
			// summary:
			//		Focus the last focusable child in the container.
			// tags:
			//		protected

			this.focusChild(this._getLastFocusableChild());
		},

		focusChild: function(/*dijit/_WidgetBase*/ widget, /*Boolean*/ last){
			// summary:
			//		Focus specified child widget.
			// widget:
			//		Reference to container's child widget
			// last:
			//		If true and if widget has multiple focusable nodes, focus the
			//		last one instead of the first one
			// tags:
			//		protected

			if(!widget){
				return;
			}

			if(this.focusedChild && widget !== this.focusedChild){
				this._onChildBlur(this.focusedChild);	// used to be used by _MenuBase
			}
			widget.set("tabIndex", this.tabIndex);	// for IE focus outline to appear, must set tabIndex before focus
			widget.focus(last ? "end" : "start");

			// Don't set focusedChild here, because the focus event should trigger a call to _onChildFocus(), which will
			// set it.   More importantly, _onChildFocus(), which may be executed asynchronously (after this function
			// returns) needs to know the old focusedChild to set its tabIndex to -1.
		},

		_onContainerFocus: function(evt){
			// summary:
			//		Handler for when the container itself gets focus.
			// description:
			//		Initially the container itself has a tabIndex, but when it gets
			//		focus, switch focus to first child...
			// tags:
			//		private

			// Note that we can't use _onFocus() because switching focus from the
			// _onFocus() handler confuses the focus.js code
			// (because it causes _onFocusNode() to be called recursively).
			// Also, _onFocus() would fire when focus went directly to a child widget due to mouse click.

			// Ignore spurious focus events:
			//	1. focus on a child widget bubbles on FF
			//	2. on IE, clicking the scrollbar of a select dropdown moves focus from the focused child item to me
			if(evt.target !== this.domNode || this.focusedChild){
				return;
			}

			this.focus();
		},

		_onFocus: function(){
			// When the container gets focus by being tabbed into, or a descendant gets focus by being clicked,
			// set the container's tabIndex to -1 (don't remove as that breaks Safari 4) so that tab or shift-tab
			// will go to the fields after/before the container, rather than the container itself
			domAttr.set(this.domNode, "tabIndex", "-1");

			this.inherited(arguments);
		},

		_onBlur: function(evt){
			// When focus is moved away the container, and its descendant (popup) widgets,
			// then restore the container's tabIndex so that user can tab to it again.
			// Note that using _onBlur() so that this doesn't happen when focus is shifted
			// to one of my child widgets (typically a popup)

			// TODO: for 2.0 consider changing this to blur whenever the container blurs, to be truthful that there is
			// no focused child at that time.

			domAttr.set(this.domNode, "tabIndex", this.tabIndex);
			if(this.focusedChild){
				this.focusedChild.set("tabIndex", "-1");
				this.lastFocusedChild = this.focusedChild;
				this._set("focusedChild", null);
			}
			this.inherited(arguments);
		},

		_onChildFocus: function(/*dijit/_WidgetBase*/ child){
			// summary:
			//		Called when a child widget gets focus, either by user clicking
			//		it, or programatically by arrow key handling code.
			// description:
			//		It marks that the current node is the selected one, and the previously
			//		selected node no longer is.

			if(child && child != this.focusedChild){
				if(this.focusedChild && !this.focusedChild._destroyed){
					// mark that the previously focusable node is no longer focusable
					this.focusedChild.set("tabIndex", "-1");
				}

				// mark that the new node is the currently selected one
				child.set("tabIndex", this.tabIndex);
				this.lastFocused = child;		// back-compat for Tree, remove for 2.0
				this._set("focusedChild", child);
			}
		},

		_searchString: "",
		// multiCharSearchDuration: Number
		//		If multiple characters are typed where each keystroke happens within
		//		multiCharSearchDuration of the previous keystroke,
		//		search for nodes matching all the keystrokes.
		//
		//		For example, typing "ab" will search for entries starting with
		//		"ab" unless the delay between "a" and "b" is greater than multiCharSearchDuration.
		multiCharSearchDuration: 1000,

		onKeyboardSearch: function(/*dijit/_WidgetBase*/ item, /*Event*/ evt, /*String*/ searchString, /*Number*/ numMatches){
			// summary:
			//		When a key is pressed that matches a child item,
			//		this method is called so that a widget can take appropriate action is necessary.
			// tags:
			//		protected
			if(item){
				this.focusChild(item);
			}
		},

		_keyboardSearchCompare: function(/*dijit/_WidgetBase*/ item, /*String*/ searchString){
			// summary:
			//		Compares the searchString to the widget's text label, returning:
			//
			//			* -1: a high priority match  and stop searching
			//		 	* 0: not a match
			//		 	* 1: a match but keep looking for a higher priority match
			// tags:
			//		private

			var element = item.domNode,
				text = item.label || (element.focusNode ? element.focusNode.label : '') || element.innerText || element.textContent || "",
				currentString = text.replace(/^\s+/, '').substr(0, searchString.length).toLowerCase();

			return (!!searchString.length && currentString == searchString) ? -1 : 0; // stop searching after first match by default
		},

		_onContainerKeydown: function(evt){
			// summary:
			//		When a key is pressed, if it's an arrow key etc. then it's handled here.
			// tags:
			//		private

			var func = this._keyNavCodes[evt.keyCode];
			if(func){
				func(evt, this.focusedChild);
				evt.stopPropagation();
				evt.preventDefault();
				this._searchString = ''; // so a DOWN_ARROW b doesn't search for ab
			}else if(evt.keyCode == keys.SPACE && this._searchTimer && !(evt.ctrlKey || evt.altKey || evt.metaKey)){
				evt.stopImmediatePropagation(); // stop a11yclick and _HasDropdown from seeing SPACE if we're doing keyboard searching
				evt.preventDefault(); // stop IE from scrolling, and most browsers (except FF) from sending keypress
				this._keyboardSearch(evt, ' ');
			}
		},

		_onContainerKeypress: function(evt){
			// summary:
			//		When a printable key is pressed, it's handled here, searching by letter.
			// tags:
			//		private

			if(evt.charCode < keys.SPACE || evt.ctrlKey || evt.altKey || evt.metaKey ||
					(evt.charCode == keys.SPACE && this._searchTimer)){
				// Avoid duplicate events on firefox (ex: arrow key that will be handled by keydown handler),
				// and also control sequences like CMD-Q
				return;
			}
			evt.preventDefault();
			evt.stopPropagation();

			this._keyboardSearch(evt, String.fromCharCode(evt.charCode).toLowerCase());
		},

		_keyboardSearch: function(/*Event*/ evt, /*String*/ keyChar){
			// summary:
			//		Perform a search of the widget's options based on the user's keyboard activity
			// description:
			//		Called on keypress (and sometimes keydown), searches through this widget's children
			//		looking for items that match the user's typed search string.  Multiple characters
			//		typed within 1 sec of each other are combined for multicharacter searching.
			// tags:
			//		private
			var
				matchedItem = null,
				searchString,
				numMatches = 0,
				search = lang.hitch(this, function(){
					if(this._searchTimer){
						this._searchTimer.remove();
					}
					this._searchString += keyChar;
					var allSameLetter = /^(.)\1*$/.test(this._searchString);
					var searchLen = allSameLetter ? 1 : this._searchString.length;
					searchString = this._searchString.substr(0, searchLen);
					// commented out code block to search again if the multichar search fails after a smaller timeout
					//this._searchTimer = this.defer(function(){ // this is the "failure" timeout
					//	this._typingSlowly = true; // if the search fails, then treat as a full timeout
					//	this._searchTimer = this.defer(function(){ // this is the "success" timeout
					//		this._searchTimer = null;
					//		this._searchString = '';
					//	}, this.multiCharSearchDuration >> 1);
					//}, this.multiCharSearchDuration >> 1);
					this._searchTimer = this.defer(function(){ // this is the "success" timeout
						this._searchTimer = null;
						this._searchString = '';
					}, this.multiCharSearchDuration);
					var currentItem = this.focusedChild || null;
					if(searchLen == 1 || !currentItem){
						currentItem = this._getNextFocusableChild(currentItem, 1); // skip current
						if(!currentItem){
							return;
						} // no items
					}
					var stop = currentItem;
					do{
						var rc = this._keyboardSearchCompare(currentItem, searchString);
						if(!!rc && numMatches++ == 0){
							matchedItem = currentItem;
						}
						if(rc == -1){ // priority match
							numMatches = -1;
							break;
						}
						currentItem = this._getNextFocusableChild(currentItem, 1);
					}while(currentItem != stop);
					// commented out code block to search again if the multichar search fails after a smaller timeout
					//if(!numMatches && (this._typingSlowly || searchLen == 1)){
					//	this._searchString = '';
					//	if(searchLen > 1){
					//		// if no matches and they're typing slowly, then go back to first letter searching
					//		search();
					//	}
					//}
				});

			search();
			// commented out code block to search again if the multichar search fails after a smaller timeout
			//this._typingSlowly = false;
			this.onKeyboardSearch(matchedItem, evt, searchString, numMatches);
		},

		_onChildBlur: function(/*dijit/_WidgetBase*/ /*===== widget =====*/){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
			//		Used to be used by MenuBase.js (remove for 2.0)
			// tags:
			//		protected
		},

		_getNextFocusableChild: function(child, dir){
			// summary:
			//		Returns the next or previous focusable descendant, compared to "child".
			//		Implements and extends _KeyNavMixin._getNextFocusableChild() for a _Container.
			// child: Widget
			//		The current widget
			// dir: Integer
			//		- 1 = after
			//		- -1 = before
			// tags:
			//		abstract extension

			var wrappedValue = child;
			do{
				if(!child){
					child = this[dir > 0 ? "_getFirst" : "_getLast"]();
					if(!child){ break; }
				}else{
					child = this._getNext(child, dir);
				}
				if(child != null && child != wrappedValue && child.isFocusable()){
					return child;	// dijit/_WidgetBase
				}
			}while(child != wrappedValue);
			// no focusable child found
			return null;	// dijit/_WidgetBase
		},

		_getFirst: function(){
			// summary:
			//		Returns the first child.
			// tags:
			//		abstract extension

			return null;	// dijit/_WidgetBase
		},

		_getLast: function(){
			// summary:
			//		Returns the last descendant.
			// tags:
			//		abstract extension

			return null;	// dijit/_WidgetBase
		},

		_getNext: function(child, dir){
			// summary:
			//		Returns the next descendant, compared to "child".
			// child: Widget
			//		The current widget
			// dir: Integer
			//		- 1 = after
			//		- -1 = before
			// tags:
			//		abstract extension

			if(child){
				child = child.domNode;
				while(child){
					child = child[dir < 0 ? "previousSibling" : "nextSibling"];
					if(child  && "getAttribute" in child){
						var w = registry.byNode(child);
						if(w){
							return w; // dijit/_WidgetBase
						}
					}
				}
			}
			return null;	// dijit/_WidgetBase
		}
	});
});

},
'dijit/MenuItem':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.toggle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/sniff", // has("ie")
	"dojo/_base/lang", // lang.hitch
	"./_Widget",
	"./_TemplatedMixin",
	"./_Contained",
	"./_CssStateMixin",
	"dojo/text!./templates/MenuItem.html"
], function(declare, dom, domAttr, domClass, kernel, has, lang,
			_Widget, _TemplatedMixin, _Contained, _CssStateMixin, template){

	// module:
	//		dijit/MenuItem

	var MenuItem = declare("dijit.MenuItem" + (has("dojo-bidi") ? "_NoBidi" : ""),
		[_Widget, _TemplatedMixin, _Contained, _CssStateMixin], {
		// summary:
		//		A line item in a Menu Widget

		// Make 3 columns
		// icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString: template,

		baseClass: "dijitMenuItem",

		// label: String
		//		Menu text as HTML
		label: "",
		_setLabelAttr: function(val){
			this._set("label", val);
			var shortcutKey = "";
			var text;
			var ndx = val.search(/{\S}/);
			if(ndx >= 0){
				shortcutKey = val.charAt(ndx + 1);
				var prefix = val.substr(0, ndx);
				var suffix = val.substr(ndx + 3);
				text = prefix + shortcutKey + suffix;
				val = prefix + '<span class="dijitMenuItemShortcutKey">' + shortcutKey + '</span>' + suffix;
			}else{
				text = val;
			}
			this.domNode.setAttribute("aria-label", text + " " + this.accelKey);
			this.containerNode.innerHTML = val;
			this._set('shortcutKey', shortcutKey);
		},

		/*=====
		// shortcutKey: [readonly] String
		//		Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,
		//		also known as [a mnemonic](http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29).
		//		This is denoted in the label by surrounding the single character with {}.
		//		For example, if label="{F}ile", then shortcutKey="F".
		shortcutKey: "",
		=====*/

		// iconClass: String
		//		Class to apply to DOMNode to make it display an icon.
		iconClass: "dijitNoIcon",
		_setIconClassAttr: { node: "iconNode", type: "class" },

		// accelKey: String
		//		Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to
		//		execute the menu item regardless of where the focus is on the page.
		//
		//		Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and
		//		execute those accelerators.
		accelKey: "",

		// disabled: Boolean
		//		If true, the menu item is disabled.
		//		If false, the menu item is enabled.
		disabled: false,

		_fillContent: function(/*DomNode*/ source){
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			if(source && !("label" in this.params)){
				this._set('label', source.innerHTML);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			var label = this.id + "_text";
			domAttr.set(this.containerNode, "id", label); // only needed for backward compat
			if(this.accelKeyNode){
				domAttr.set(this.accelKeyNode, "id", this.id + "_accel"); // only needed for backward compat
			}
			dom.setSelectable(this.domNode, false);
		},

		onClick: function(/*Event*/){
			// summary:
			//		User defined function to handle clicks
			// tags:
			//		callback
		},

		focus: function(){
			// summary:
			//		Focus on this MenuItem
			try{
				if(has("ie") == 8){
					// needed for IE8 which won't scroll TR tags into view on focus yet calling scrollIntoView creates flicker (#10275)
					this.containerNode.focus();
				}
				this.focusNode.focus();
			}catch(e){
				// this throws on IE (at least) in some scenarios
			}
		},

		_onFocus: function(){
			// summary:
			//		This is called by the focus manager when focus
			//		goes to this MenuItem or a child menu.
			// tags:
			//		protected

			this.getParent()._onItemFocus(this);

			this.inherited(arguments);
		},

		_setSelected: function(selected){
			// summary:
			//		Indicate that this node is the currently selected one
			// tags:
			//		private

			domClass.toggle(this.domNode, "dijitMenuItemSelected", selected);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.   Use set('label', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.MenuItem.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.   Use set('disabled', bool) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.Menu.setDisabled() is deprecated.  Use set('disabled', bool) instead.", "", "2.0");
			this.set('disabled', disabled);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			// summary:
			//		Hook for attr('disabled', ...) to work.
			//		Enable or disable this menu item.

			this.focusNode.setAttribute('aria-disabled', value ? 'true' : 'false');
			this._set("disabled", value);
		},

		_setAccelKeyAttr: function(/*String*/ value){
			// summary:
			//		Hook for attr('accelKey', ...) to work.
			//		Set accelKey on this menu item.

			if(this.accelKeyNode){
				this.accelKeyNode.style.display = value ? "" : "none";
				this.accelKeyNode.innerHTML = value;
				//have to use colSpan to make it work in IE
				domAttr.set(this.containerNode, 'colSpan', value ? "1" : "2");
			}
			this._set("accelKey", value);
		}
	});

	if(has("dojo-bidi")){
		MenuItem = declare("dijit.MenuItem", MenuItem, {
			_setLabelAttr: function(val){
				this.inherited(arguments);
				if(this.textDir === "auto"){
					this.applyTextDir(this.textDirNode);
				}
			}
		});
	}

	return MenuItem;
});

},
'dijit/_Contained':function(){
define([
	"dojo/_base/declare", // declare
	"./registry"	// registry.getEnclosingWidget(), registry.byNode()
], function(declare, registry){

	// module:
	//		dijit/_Contained

	return declare("dijit._Contained", null, {
		// summary:
		//		Mixin for widgets that are children of a container widget
		// example:
		//	|	// make a basic custom widget that knows about its parents
		//	|	declare("my.customClass",[dijit._WidgetBase, dijit._Contained],{});

		_getSibling: function(/*String*/ which){
			// summary:
			//		Returns next or previous sibling
			// which:
			//		Either "next" or "previous"
			// tags:
			//		private
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			return node && registry.byNode(node);	// dijit/_WidgetBase
		},

		getPreviousSibling: function(){
			// summary:
			//		Returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous"); // dijit/_WidgetBase
		},

		getNextSibling: function(){
			// summary:
			//		Returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next"); // dijit/_WidgetBase
		},

		getIndexInParent: function(){
			// summary:
			//		Returns the index of this widget within its container parent.
			//		It returns -1 if the parent does not exist, or if the parent
			//		is not a dijit/_Container

			var p = this.getParent();
			if(!p || !p.getIndexOfChild){
				return -1; // int
			}
			return p.getIndexOfChild(this); // int
		}
	});
});

},
'dijit/PopupMenuItem':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-style", // domStyle.set
	"dojo/_base/lang",
	"dojo/query", // query
	"./popup",
	"./registry",	// registry.byNode
	"./MenuItem",
	"./hccss"
], function(declare, domStyle, lang, query, pm, registry, MenuItem){

	// module:
	//		dijit/PopupMenuItem

	return declare("dijit.PopupMenuItem", MenuItem, {
		// summary:
		//		An item in a Menu that spawn a drop down (usually a drop down menu)

		_fillContent: function(){
			// summary:
			//		When Menu is declared in markup, this code gets the menu label and
			//		the popup widget from the srcNodeRef.
			// description:
			//		srcNodeRef.innerHTML contains both the menu item text and a popup widget
			//		The first part holds the menu item text and the second part is the popup
			// example:
			// |	<div data-dojo-type="dijit/PopupMenuItem">
			// |		<span>pick me</span>
			// |		<popup> ... </popup>
			// |	</div>
			// tags:
			//		protected

			if(this.srcNodeRef){
				var nodes = query("*", this.srcNodeRef);
				this.inherited(arguments, [nodes[0]]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		_openPopup: function(/*Object*/ params, /*Boolean*/ focus){
			// summary:
			//		Open the popup to the side of/underneath this MenuItem, and optionally focus first item
			// tags:
			//		protected

			var popup = this.popup;

			pm.open(lang.delegate(params, {
				popup: this.popup,
				around: this.domNode
			}));

			if(focus && popup.focus){
				popup.focus();
			}
		},

		_closePopup: function(){
			pm.close(this.popup);
			this.popup.parentMenu = null;
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);

			// We didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
			// land now.  Move it to <body>.
			if(!this.popup){
				var node = query("[widgetId]", this.dropDownContainer)[0];
				this.popup = registry.byNode(node);
			}
			this.ownerDocumentBody.appendChild(this.popup.domNode);
			this.popup.domNode.setAttribute("aria-labelledby", this.containerNode.id);
			this.popup.startup();

			this.popup.domNode.style.display="none";
			if(this.arrowWrapper){
				domStyle.set(this.arrowWrapper, "visibility", "");
			}
			this.focusNode.setAttribute("aria-haspopup", "true");
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			if(this.popup){
				// Destroy the popup, unless it's already been destroyed.  This can happen because
				// the popup is a direct child of <body> even though it's logically my child.
				if(!this.popup._destroyed){
					this.popup.destroyRecursive(preserveDom);
				}
				delete this.popup;
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/FilteringSelect':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.mixin
	"dojo/when",
	"./MappedTextBox",
	"./ComboBoxMixin"
], function(declare, lang, when, MappedTextBox, ComboBoxMixin){

	// module:
	//		dijit/form/FilteringSelect

	return declare("dijit.form.FilteringSelect", [MappedTextBox, ComboBoxMixin], {
		// summary:
		//		An enhanced version of the HTML SELECT tag, populated dynamically
		//
		// description:
		//		An enhanced version of the HTML SELECT tag, populated dynamically. It works
		//		very nicely with very large data sets because it can load and page data as needed.
		//		It also resembles ComboBox, but does not allow values outside of the provided ones.
		//		If OPTION tags are used as the data provider via markup, then the
		//		OPTION tag's child text node is used as the displayed value when selected
		//		while the OPTION tag's value attribute is used as the widget value on form submit.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Similar features:
		//
		//		- There is a drop down list of possible values.
		//		- You can only enter a value from the drop down list.  (You can't
		//			enter an arbitrary value.)
		//		- The value submitted with the form is the hidden value (ex: CA),
		//			not the displayed value a.k.a. label (ex: California)
		//
		//		Enhancements over plain HTML version:
		//
		//		- If you type in some text then it will filter down the list of
		//			possible values in the drop down list.
		//		- List can be specified either as a static list or via a javascript
		//			function (that can get the list from a server)

		// required: Boolean
		//		True (default) if user is required to enter a value into this field.
		required: true,

		_lastDisplayedValue: "",

		_isValidSubset: function(){
			return this._opened;
		},

		isValid: function(){
			// Overrides ValidationTextBox.isValid()
			return !!this.item || (!this.required && this.get('displayedValue') == ""); // #5974
		},

		_refreshState: function(){
			if(!this.searchTimer){ // state will be refreshed after results are returned
				this.inherited(arguments);
			}
		},

		_callbackSetLabel: function(
						/*Array*/ result,
						/*Object*/ query,
						/*Object*/ options,
						/*Boolean?*/ priorityChange){
			// summary:
			//		Callback from dojo.store after lookup of user entered value finishes

			// setValue does a synchronous lookup,
			// so it calls _callbackSetLabel directly,
			// and so does not pass dataObject
			// still need to test against _lastQuery in case it came too late
			if((query && query[this.searchAttr] !== this._lastQuery) || (!query && result.length && this.store.getIdentity(result[0]) != this._lastQuery)){
				return;
			}
			if(!result.length){
				//#3268: don't modify display value on bad input
				//#3285: change CSS to indicate error
				this.set("value", '', priorityChange || (priorityChange === undefined && !this.focused), this.textbox.value, null);
			}else{
				this.set('item', result[0], priorityChange);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ query, /*Object*/ options){
			// Callback when a data store query completes.
			// Overrides ComboBox._openResultList()

			// #3285: tap into search callback to see if user's query resembles a match
			if(query[this.searchAttr] !== this._lastQuery){
				return;
			}
			this.inherited(arguments);

			if(this.item === undefined){ // item == undefined for keyboard search
				// If the search returned no items that means that the user typed
				// in something invalid (and they can't make it valid by typing more characters),
				// so flag the FilteringSelect as being in an invalid state
				this.validate(true);
			}
		},

		_getValueAttr: function(){
			// summary:
			//		Hook for get('value') to work.

			// don't get the textbox value but rather the previously set hidden value.
			// Use this.valueNode.value which isn't always set for other MappedTextBox widgets until blur
			return this.valueNode.value;
		},

		_getValueField: function(){
			// Overrides ComboBox._getValueField()
			return "value";
		},

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange, /*String?*/ displayedValue, /*item?*/ item){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the select.
			//		Also sets the label to the corresponding value by reverse lookup.
			if(!this._onChangeActive){ priorityChange = null; }

			if(item === undefined){
				if(value === null || value === ''){
					value = '';
					if(!lang.isString(displayedValue)){
						this._setDisplayedValueAttr(displayedValue||'', priorityChange);
						return;
					}
				}

				var self = this;
				this._lastQuery = value;
				when(this.store.get(value), function(item){
					self._callbackSetLabel(item? [item] : [], undefined, undefined, priorityChange);
				});
			}else{
				this.valueNode.value = value;
				this.inherited(arguments);
			}
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Set the displayed valued in the input box, and the hidden value
			//		that gets submitted, based on a dojo.data store item.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('item', value)
			// tags:
			//		private
			this.inherited(arguments);
			this._lastDisplayedValue = this.textbox.value;
		},

		_getDisplayQueryString: function(/*String*/ text){
			return text.replace(/([\\\*\?])/g, "\\$1");
		},

		_setDisplayedValueAttr: function(/*String*/ label, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('displayedValue', label) works.
			// description:
			//		Sets textbox to display label. Also performs reverse lookup
			//		to set the hidden value.  label should corresponding to item.searchAttr.

			if(label == null){ label = ''; }

			// This is called at initialization along with every custom setter.
			// Usually (or always?) the call can be ignored.   If it needs to be
			// processed then at least make sure that the XHR request doesn't trigger an onChange()
			// event, even if it returns after creation has finished
			if(!this._created){
				if(!("displayedValue" in this.params)){
					return;
				}
				priorityChange = false;
			}

			// Do a reverse lookup to map the specified displayedValue to the hidden value.
			// Note that if there's a custom labelFunc() this code
			if(this.store){
				this.closeDropDown();
				var query = lang.clone(this.query); // #6196: populate query with user-specifics

				// Generate query
				var qs = this._getDisplayQueryString(label), q;
				if(this.store._oldAPI){
					// remove this branch for 2.0
					q = qs;
				}else{
					// Query on searchAttr is a regex for benefit of dojo/store/Memory,
					// but with a toString() method to help dojo/store/JsonRest.
					// Search string like "Co*" converted to regex like /^Co.*$/i.
					q = this._patternToRegExp(qs);
					q.toString = function(){ return qs; };
				}
				this._lastQuery = query[this.searchAttr] = q;

				// If the label is not valid, the callback will never set it,
				// so the last valid value will get the warning textbox.   Set the
				// textbox value now so that the impending warning will make
				// sense to the user
				this.textbox.value = label;
				this._lastDisplayedValue = label;
				this._set("displayedValue", label);	// for watch("displayedValue") notification
				var _this = this;
				var options = {
					ignoreCase: this.ignoreCase,
					deep: true
				};
				lang.mixin(options, this.fetchProperties);
				this._fetchHandle = this.store.query(query, options);
				when(this._fetchHandle, function(result){
					_this._fetchHandle = null;
					_this._callbackSetLabel(result || [], query, options, priorityChange);
				}, function(err){
					_this._fetchHandle = null;
					if(!_this._cancelingQuery){	// don't treat canceled query as an error
						console.error('dijit.form.FilteringSelect: ' + err.toString());
					}
				});
			}
		},

		undo: function(){
			this.set('displayedValue', this._lastDisplayedValue);
		}
	});
});

},
'dijit/form/MappedTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/sniff", // has("msapp")
	"dojo/dom-construct", // domConstruct.place
	"./ValidationTextBox"
], function(declare, has, domConstruct, ValidationTextBox){

	// module:
	//		dijit/form/MappedTextBox

	return declare("dijit.form.MappedTextBox", ValidationTextBox, {
		// summary:
		//		A dijit/form/ValidationTextBox subclass which provides a base class for widgets that have
		//		a visible formatted display value, and a serializable
		//		value in a hidden input field which is actually sent to the server.
		// description:
		//		The visible display may
		//		be locale-dependent and interactive.  The value sent to the server is stored in a hidden
		//		input field which uses the `name` attribute declared by the original widget.  That value sent
		//		to the server is defined by the dijit/form/MappedTextBox.serialize() method and is typically
		//		locale-neutral.
		// tags:
		//		protected

		postMixInProperties: function(){
			this.inherited(arguments);

			// We want the name attribute to go to the hidden <input>, not the displayed <input>,
			// so override _FormWidget.postMixInProperties() setting of nameAttrSetting for IE.
			this.nameAttrSetting = "";
		},

		// Remap name attribute to be mapped to hidden node created in buildRendering(), rather than this.focusNode
		_setNameAttr: "valueNode",

		serialize: function(val /*=====, options =====*/){
			// summary:
			//		Overridable function used to convert the get('value') result to a canonical
			//		(non-localized) string.  For example, will print dates in ISO format, and
			//		numbers the same way as they are represented in javascript.
			// val: anything
			// options: Object?
			// tags:
			//		protected extension
			return val.toString ? val.toString() : ""; // String
		},

		toString: function(){
			// summary:
			//		Returns widget as a printable string using the widget's value
			// tags:
			//		protected
			var val = this.filter(this.get('value')); // call filter in case value is nonstring and filter has been customized
			return val != null ? (typeof val == "string" ? val : this.serialize(val, this.constraints)) : ""; // String
		},

		validate: function(){
			// Overrides `dijit/form/TextBox.validate`
			this.valueNode.value = this.toString();
			return this.inherited(arguments);
		},

		buildRendering: function(){
			// Overrides `dijit/_TemplatedMixin/buildRendering`

			this.inherited(arguments);

			// Create a hidden <input> node with the serialized value used for submit
			// (as opposed to the displayed value).
			// Passing in name as markup rather than relying on _setNameAttr custom setter above
			// to make query(input[name=...]) work on IE. (see #8660).
			// But not doing that for Windows 8 Store apps because it causes a security exception (see #16452).
			this.valueNode = domConstruct.place("<input type='hidden'" +
				((this.name && !has("msapp")) ? ' name="' + this.name.replace(/"/g, "&quot;") + '"' : "") + "/>",
				this.textbox, "after");
		},

		reset: function(){
			// Overrides `dijit/form/ValidationTextBox.reset` to
			// reset the hidden textbox value to ''
			this.valueNode.value = '';
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/ValidationTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/i18n", // i18n.getLocalization
	"./TextBox",
	"../Tooltip",
	"dojo/text!./templates/ValidationTextBox.html",
	"dojo/i18n!./nls/validate"
], function(declare, kernel, i18n, TextBox, Tooltip, template){

	// module:
	//		dijit/form/ValidationTextBox


	/*=====
	var __Constraints = {
		// locale: String
		//		locale used for validation, picks up value from this widget's lang attribute
		// _flags_: anything
		//		various flags passed to pattern function
	};
	=====*/

	var ValidationTextBox;
	return ValidationTextBox = declare("dijit.form.ValidationTextBox", TextBox, {
		// summary:
		//		Base class for textbox widgets with the ability to validate content of various types and provide user feedback.

		templateString: template,

		// required: Boolean
		//		User is required to enter data into this field.
		required: false,

		// promptMessage: String
		//		If defined, display this hint string immediately on focus to the textbox, if empty.
		//		Also displays if the textbox value is Incomplete (not yet valid but will be with additional input).
		//		Think of this like a tooltip that tells the user what to do, not an error message
		//		that tells the user what they've done wrong.
		//
		//		Message disappears when user starts typing.
		promptMessage: "",

		// invalidMessage: String
		//		The message to display if value is invalid.
		//		The translated string value is read from the message file by default.
		//		Set to "" to use the promptMessage instead.
		invalidMessage: "$_unset_$",

		// missingMessage: String
		//		The message to display if value is empty and the field is required.
		//		The translated string value is read from the message file by default.
		//		Set to "" to use the invalidMessage instead.
		missingMessage: "$_unset_$",

		// message: String
		//		Currently error/prompt message.
		//		When using the default tooltip implementation, this will only be
		//		displayed when the field is focused.
		message: "",

		// constraints: __Constraints
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},

		// pattern: [extension protected] String|Function(constraints) returning a string.
		//		This defines the regular expression used to validate the input.
		//		Do not add leading ^ or $ characters since the widget adds these.
		//		A function may be used to generate a valid pattern when dependent on constraints or other runtime factors.
		//		set('pattern', String|Function).
		pattern: ".*",

		// regExp: Deprecated [extension protected] String.  Use "pattern" instead.
		regExp: "",

		regExpGen: function(/*__Constraints*/ /*===== constraints =====*/){
			// summary:
			//		Deprecated.  Use set('pattern', Function) instead.
		},

		// state: [readonly] String
		//		Shows current state (ie, validation result) of input (""=Normal, Incomplete, or Error)
		state: "",

		// tooltipPosition: String[]
		//		See description of `dijit/Tooltip.defaultPosition` for details on this parameter.
		tooltipPosition: [],

		_deprecateRegExp: function(attr, value){
			if(value != ValidationTextBox.prototype[attr]){
				kernel.deprecated("ValidationTextBox id="+this.id+", set('" + attr + "', ...) is deprecated.  Use set('pattern', ...) instead.", "", "2.0");
				this.set('pattern', value);
			}
		},
		_setRegExpGenAttr: function(/*Function*/ newFcn){
			this._deprecateRegExp("regExpGen", newFcn);
			this._set("regExpGen", this._computeRegexp); // backward compat with this.regExpGen(this.constraints)
		},
		_setRegExpAttr: function(/*String*/ value){
			this._deprecateRegExp("regExp", value);
		},

		_setValueAttr: function(){
			// summary:
			//		Hook so set('value', ...) works.
			this.inherited(arguments);
			this._refreshState();
		},

		validator: function(/*anything*/ value, /*__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the text input against the regular expression.
			// tags:
			//		protected
			return (new RegExp("^(?:" + this._computeRegexp(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== undefined); // Boolean
		},

		_isValidSubset: function(){
			// summary:
			//		Returns true if the value is either already valid or could be made valid by appending characters.
			//		This is used for validation while the user [may be] still typing.
			return this.textbox.value.search(this._partialre) == 0;
		},

		isValid: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Tests if value is valid.
			//		Can override with your own routine in a subclass.
			// tags:
			//		protected
			return this.validator(this.textbox.value, this.get('constraints'));
		},

		_isEmpty: function(value){
			// summary:
			//		Checks for whitespace
			return (this.trim ? /^\s*$/ : /^$/).test(value); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Return an error message to show if appropriate
			// tags:
			//		protected
			var invalid = this.invalidMessage == "$_unset_$" ? this.messages.invalidMessage :
				!this.invalidMessage ? this.promptMessage : this.invalidMessage;
			var missing = this.missingMessage == "$_unset_$" ? this.messages.missingMessage :
				!this.missingMessage ? invalid : this.missingMessage;
			return (this.required && this._isEmpty(this.textbox.value)) ? missing : invalid; // String
		},

		getPromptMessage: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Return a hint message to show when widget is first focused
			// tags:
			//		protected
			return this.promptMessage; // String
		},

		_maskValidSubsetError: true,
		validate: function(/*Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			// tags:
			//		protected
			var message = "";
			var isValid = this.disabled || this.isValid(isFocused);
			if(isValid){ this._maskValidSubsetError = true; }
			var isEmpty = this._isEmpty(this.textbox.value);
			var isValidSubset = !isValid && isFocused && this._isValidSubset();
			this._set("state", isValid ? "" : (((((!this._hasBeenBlurred || isFocused) && isEmpty) || isValidSubset) && (this._maskValidSubsetError || (isValidSubset && !this._hasBeenBlurred && isFocused))) ? "Incomplete" : "Error"));
			this.focusNode.setAttribute("aria-invalid", isValid ? "false" : "true");

			if(this.state == "Error"){
				this._maskValidSubsetError = isFocused && isValidSubset; // we want the error to show up after a blur and refocus
				message = this.getErrorMessage(isFocused);
			}else if(this.state == "Incomplete"){
				message = this.getPromptMessage(isFocused); // show the prompt whenever the value is not yet complete
				this._maskValidSubsetError = !this._hasBeenBlurred || isFocused; // no Incomplete warnings while focused
			}else if(isEmpty){
				message = this.getPromptMessage(isFocused); // show the prompt whenever there's no error and no text
			}
			this.set("message", message);

			return isValid;
		},

		displayMessage: function(/*String*/ message){
			// summary:
			//		Overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			// tags:
			//		extension
			if(message && this.focused){
				Tooltip.show(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
			}else{
				Tooltip.hide(this.domNode);
			}
		},

		_refreshState: function(){
			// Overrides TextBox._refreshState()
			if(this._created){ // should instead be this._started but that would require all programmatic ValidationTextBox instantiations to call startup()
				this.validate(this.focused);
			}
			this.inherited(arguments);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree.

			this.constraints = {};
			this.baseClass += ' dijitValidationTextBox';
		},

		startup: function(){
			this.inherited(arguments);
			this._refreshState(); // after all _set* methods have run
		},

		_setConstraintsAttr: function(/*__Constraints*/ constraints){
			if(!constraints.locale && this.lang){
				constraints.locale = this.lang;
			}
			this._set("constraints", constraints);
			this._refreshState();
		},

		_setPatternAttr: function(/*String|Function*/ pattern){
			this._set("pattern", pattern); // don't set on INPUT to avoid native HTML5 validation
		},

		_computeRegexp: function(/*__Constraints*/ constraints){
			// summary:
			//		Hook to get the current regExp and to compute the partial validation RE.

			var p = this.pattern;
			if(typeof p == "function"){
				p = p.call(this, constraints);
			}
			if(p != this._lastRegExp){
				var partialre = "";
				this._lastRegExp = p;
				// parse the regexp and produce a new regexp that matches valid subsets
				// if the regexp is .* then there's no use in matching subsets since everything is valid
				if(p != ".*"){
					p.replace(/\\.|\[\]|\[.*?[^\\]{1}\]|\{.*?\}|\(\?[=:!]|./g,
					function(re){
						switch(re.charAt(0)){
							case '{':
							case '+':
							case '?':
							case '*':
							case '^':
							case '$':
							case '|':
							case '(':
								partialre += re;
								break;
							case ")":
								partialre += "|$)";
								break;
							 default:
								partialre += "(?:"+re+"|$)";
								break;
						}
					});
				}
				try{ // this is needed for now since the above regexp parsing needs more test verification
					"".search(partialre);
				}catch(e){ // should never be here unless the original RE is bad or the parsing is bad
					partialre = this.pattern;
					console.warn('RegExp error in ' + this.declaredClass + ': ' + this.pattern);
				} // should never be here unless the original RE is bad or the parsing is bad
				this._partialre = "^(?:" + partialre + ")$";
			}
			return p;
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls attachPoints
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this.inherited(arguments);	// call FormValueWidget._setDisabledAttr()
			this._refreshState();
		},

		_setRequiredAttr: function(/*Boolean*/ value){
			this._set("required", value);
			this.focusNode.setAttribute("aria-required", value);
			this._refreshState();
		},

		_setMessageAttr: function(/*String*/ message){
			this._set("message", message);
			this.displayMessage(message);
		},

		reset:function(){
			// Overrides dijit/form/TextBox.reset() by also
			// hiding errors about partial matches
			this._maskValidSubsetError = true;
			this.inherited(arguments);
		},

		_onBlur: function(){
			// the message still exists but for back-compat, and to erase the tooltip
			// (if the message is being displayed as a tooltip), call displayMessage('')
			this.displayMessage('');

			this.inherited(arguments);
		}
	});
});

},
'dijit/form/TextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.create
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie") has("mozilla")
	"./_FormValueWidget",
	"./_TextBoxMixin",
	"dojo/text!./templates/TextBox.html",
	"../main"	// to export dijit._setSelectionRange, remove in 2.0
], function(declare, domConstruct, domStyle, kernel, lang, on, has,
			_FormValueWidget, _TextBoxMixin, template, dijit){

	// module:
	//		dijit/form/TextBox

	var TextBox = declare("dijit.form.TextBox" + (has("dojo-bidi") ? "_NoBidi" : ""), [_FormValueWidget, _TextBoxMixin], {
		// summary:
		//		A base class for textbox form inputs

		templateString: template,
		_singleNodeTemplate: '<input class="dijit dijitReset dijitLeft dijitInputField" data-dojo-attach-point="textbox,focusNode" autocomplete="off" type="${type}" ${!nameAttrSetting} />',

		_buttonInputDisabled: has("ie") ? "disabled" : "", // allows IE to disallow focus, but Firefox cannot be disabled for mousedown events

		baseClass: "dijitTextBox",

		postMixInProperties: function(){
			var type = this.type.toLowerCase();
			if(this.templateString && this.templateString.toLowerCase() == "input" || ((type == "hidden" || type == "file") && this.templateString == this.constructor.prototype.templateString)){
				this.templateString = this._singleNodeTemplate;
			}
			this.inherited(arguments);
		},

		postCreate: function(){
			this.inherited(arguments);

			if(has("ie") < 9){
				// IE INPUT tag fontFamily has to be set directly using STYLE
				// the defer gives IE a chance to render the TextBox and to deal with font inheritance
				this.defer(function(){
					try{
						var s = domStyle.getComputedStyle(this.domNode); // can throw an exception if widget is immediately destroyed
						if(s){
							var ff = s.fontFamily;
							if(ff){
								var inputs = this.domNode.getElementsByTagName("INPUT");
								if(inputs){
									for(var i=0; i < inputs.length; i++){
										inputs[i].style.fontFamily = ff;
									}
								}
							}
						}
					}catch(e){/*when used in a Dialog, and this is called before the dialog is
					 shown, s.fontFamily would trigger "Invalid Argument" error.*/}
				});
			}
		},

		_setPlaceHolderAttr: function(v){
			this._set("placeHolder", v);
			if(!this._phspan){
				this._attachPoints.push('_phspan');
				// dijitInputField class gives placeHolder same padding as the input field
				// parent node already has dijitInputField class but it doesn't affect this <span>
				// since it's position: absolute.
				this._phspan = domConstruct.create('span',{ onmousedown:function(e){ e.preventDefault(); }, className:'dijitPlaceHolder dijitInputField'},this.textbox,'after');
				this.own(on(this._phspan, "touchend, MSPointerUp", lang.hitch(this, function(){
					// If the user clicks placeholder rather than the <input>, need programmatic focus.  Normally this
					// is done in _FormWidgetMixin._onFocus() but after [30663] it's done on a delay, which is ineffective.
					this.focus();
				})));
			}
			this._phspan.innerHTML="";
			this._phspan.appendChild(this._phspan.ownerDocument.createTextNode(v));
			this._updatePlaceHolder();
		},

		_onInput: function(/*Event*/ evt){
			// summary:
			//		Called AFTER the input event has happened
			//		See if the placeHolder text should be removed or added while editing.
			this.inherited(arguments);
			this._updatePlaceHolder();
		},

		_updatePlaceHolder: function(){
			if(this._phspan){
				this._phspan.style.display = (this.placeHolder && !this.textbox.value) ? "" : "none";
			}
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			this.inherited(arguments);
			this._updatePlaceHolder();
		},

		getDisplayedValue: function(){
			// summary:
			//		Deprecated.  Use get('displayedValue') instead.
			// tags:
			//		deprecated
			kernel.deprecated(this.declaredClass+"::getDisplayedValue() is deprecated. Use get('displayedValue') instead.", "", "2.0");
			return this.get('displayedValue');
		},

		setDisplayedValue: function(/*String*/ value){
			// summary:
			//		Deprecated.  Use set('displayedValue', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated(this.declaredClass+"::setDisplayedValue() is deprecated. Use set('displayedValue', ...) instead.", "", "2.0");
			this.set('displayedValue', value);
		},

		_onBlur: function(e){
			if(this.disabled){ return; }
			this.inherited(arguments);
			this._updatePlaceHolder();

			if(has("mozilla")){
				if(this.selectOnClick){
					// clear selection so that the next mouse click doesn't reselect
					this.textbox.selectionStart = this.textbox.selectionEnd = undefined;
				}
			}
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){ return; }
			this.inherited(arguments);
			this._updatePlaceHolder();
		}
	});

	if(has("ie")){
		TextBox.prototype._isTextSelected = function(){
			var range = this.ownerDocument.selection.createRange();
			var parent = range.parentElement();
			return parent == this.textbox && range.text.length > 0;
		};

		// Overrides definition of _setSelectionRange from _TextBoxMixin (TODO: move to _TextBoxMixin.js?)
		dijit._setSelectionRange = _TextBoxMixin._setSelectionRange = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
			if(element.createTextRange){
				var r = element.createTextRange();
				r.collapse(true);
				r.moveStart("character", -99999); // move to 0
				r.moveStart("character", start); // delta from 0 is the correct position
				r.moveEnd("character", stop-start);
				r.select();
			}
		}
	}

	if(has("dojo-bidi")){
		TextBox = declare("dijit.form.TextBox", TextBox, {
			_setPlaceHolderAttr: function(v){
				this.inherited(arguments);
				this.applyTextDir(this._phspan);
			}
		});
	}

	return TextBox;
});

},
'dijit/form/_FormValueWidget':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/sniff", // has("ie")
	"./_FormWidget",
	"./_FormValueMixin"
], function(declare, has, _FormWidget, _FormValueMixin){

	// module:
	//		dijit/form/_FormValueWidget

	return declare("dijit.form._FormValueWidget", [_FormWidget, _FormValueMixin], {
		// summary:
		//		Base class for widgets corresponding to native HTML elements such as `<input>` or `<select>`
		//		that have user changeable values.
		// description:
		//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) `<input>` element,
		//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
		//		works as expected.

		// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
		// directly in the template as read by the parser in order to function. IE is known to specifically
		// require the 'name' attribute at element creation time.  See #8484, #8660.

		_layoutHackIE7: function(){
			// summary:
			//		Work around table sizing bugs on IE7 by forcing redraw

			if(has("ie") == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
				var domNode = this.domNode;
				var parent = domNode.parentNode;
				var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
				var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
				var _this = this;
				while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
					(function ping(){
						var disconnectHandle = _this.connect(parent, "onscroll",
							function(){
								_this.disconnect(disconnectHandle); // only call once
								pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
								_this.defer(function(){
									pingNode.style.filter = origFilter;
								}); // restore custom filter, if any
							}
						);
					})();
					parent = parent.parentNode;
				}
			}
		}
	});
});

},
'dijit/form/_FormWidget':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/sniff", // has("dijit-legacy-requires"), has("msapp")
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/ready",
	"../_Widget",
	"../_CssStateMixin",
	"../_TemplatedMixin",
	"./_FormWidgetMixin"
], function(declare, has, kernel, ready, _Widget, _CssStateMixin, _TemplatedMixin, _FormWidgetMixin){

	// module:
	//		dijit/form/_FormWidget

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/form/_FormValueWidget"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.form._FormWidget", [_Widget, _TemplatedMixin, _CssStateMixin, _FormWidgetMixin], {
		// summary:
		//		Base class for widgets corresponding to native HTML elements such as `<checkbox>` or `<button>`,
		//		which can be children of a `<form>` node or a `dijit/form/Form` widget.
		//
		// description:
		//		Represents a single HTML element.
		//		All these widgets should have these attributes just like native HTML input elements.
		//		You can set them during widget construction or afterwards, via `dijit/_WidgetBase.set()`.
		//
		//		They also share some common methods.

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.  Use set('disabled', ...) instead.
			kernel.deprecated("setDisabled(" + disabled + ") is deprecated. Use set('disabled'," + disabled + ") instead.", "", "2.0");
			this.set('disabled', disabled);
		},

		setValue: function(/*String*/ value){
			// summary:
			//		Deprecated.  Use set('value', ...) instead.
			kernel.deprecated("dijit.form._FormWidget:setValue(" + value + ") is deprecated.  Use set('value'," + value + ") instead.", "", "2.0");
			this.set('value', value);
		},

		getValue: function(){
			// summary:
			//		Deprecated.  Use get('value') instead.
			kernel.deprecated(this.declaredClass + "::getValue() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},

		postMixInProperties: function(){
			// Setup name=foo string to be referenced from the template (but only if a name has been specified).
			// Unfortunately we can't use _setNameAttr to set the name in IE due to IE limitations, see #8484, #8660.
			// But when IE6 and IE7 are desupported, then we probably don't need this anymore, so should remove it in 2.0.
			// Also, don't do this for Windows 8 Store Apps because it causes a security exception (see #16452).
			// Regarding escaping, see heading "Attribute values" in
			// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
			this.nameAttrSetting = (this.name && !has("msapp")) ? ('name="' + this.name.replace(/"/g, "&quot;") + '"') : '';
			this.inherited(arguments);
		},

		// Override automatic assigning type --> focusNode, it causes exception on IE.
		// Instead, type must be specified as ${type} in the template, as part of the original DOM
		_setTypeAttr: null
	});
});

},
'dijit/form/_FormWidgetMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-style", // domStyle.get
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/mouse", // mouse.isLeft
	"dojo/on",
	"dojo/sniff", // has("webkit")
	"dojo/window", // winUtils.scrollIntoView
	"../a11y"    // a11y.hasDefaultTabStop
], function(array, declare, domAttr, domStyle, lang, mouse, on, has, winUtils, a11y){

	// module:
	//		dijit/form/_FormWidgetMixin

	return declare("dijit.form._FormWidgetMixin", null, {
		// summary:
		//		Mixin for widgets corresponding to native HTML elements such as `<checkbox>` or `<button>`,
		//		which can be children of a `<form>` node or a `dijit/form/Form` widget.
		//
		// description:
		//		Represents a single HTML element.
		//		All these widgets should have these attributes just like native HTML input elements.
		//		You can set them during widget construction or afterwards, via `dijit/_WidgetBase.set()`.
		//
		//		They also share some common methods.

		// name: [const] String
		//		Name used when submitting form; same as "name" attribute or plain HTML elements
		name: "",

		// alt: String
		//		Corresponds to the native HTML `<input>` element's attribute.
		alt: "",

		// value: String
		//		Corresponds to the native HTML `<input>` element's attribute.
		value: "",

		// type: [const] String
		//		Corresponds to the native HTML `<input>` element's attribute.
		type: "text",

		// type: String
		//		Apply aria-label in markup to the widget's focusNode
		"aria-label": "focusNode",

		// tabIndex: String
		//		Order fields are traversed when user hits the tab key
		tabIndex: "0",
		_setTabIndexAttr: "focusNode", // force copy even when tabIndex default value, needed since Button is <span>

		// disabled: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "disabled='disabled'", or just "disabled".
		disabled: false,

		// intermediateChanges: Boolean
		//		Fires onChange for each value change or only on demand
		intermediateChanges: false,

		// scrollOnFocus: Boolean
		//		On focus, should this widget scroll into view?
		scrollOnFocus: true,

		// Override _WidgetBase mapping id to this.domNode, needs to be on focusNode so <label> etc.
		// works with screen reader
		_setIdAttr: "focusNode",

		_setDisabledAttr: function(/*Boolean*/ value){
			this._set("disabled", value);
			domAttr.set(this.focusNode, 'disabled', value);
			if(this.valueNode){
				domAttr.set(this.valueNode, 'disabled', value);
			}
			this.focusNode.setAttribute("aria-disabled", value ? "true" : "false");

			if(value){
				// reset these, because after the domNode is disabled, we can no longer receive
				// mouse related events, see #4200
				this._set("hovering", false);
				this._set("active", false);

				// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
				var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex :
					("_setTabIndexAttr" in this) ? this._setTabIndexAttr : "focusNode";
				array.forEach(lang.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
					var node = this[attachPointName];
					// complex code because tabIndex=-1 on a <div> doesn't work on FF
					if(has("webkit") || a11y.hasDefaultTabStop(node)){    // see #11064 about webkit bug
						node.setAttribute('tabIndex', "-1");
					}else{
						node.removeAttribute('tabIndex');
					}
				}, this);
			}else{
				if(this.tabIndex != ""){
					this.set('tabIndex', this.tabIndex);
				}
			}
		},

		_onFocus: function(/*String*/ by){
			// If user clicks on the widget, even if the mouse is released outside of it,
			// this widget's focusNode should get focus (to mimic native browser behavior).
			// Browsers often need help to make sure the focus via mouse actually gets to the focusNode.
			// TODO: consider removing all of this for 2.0 or sooner, see #16622 etc.
			if(by == "mouse" && this.isFocusable()){
				// IE exhibits strange scrolling behavior when refocusing a node so only do it when !focused.
				var focusHandle = this.own(on(this.focusNode, "focus", function(){
					mouseUpHandle.remove();
					focusHandle.remove();
				}))[0];
				// Set a global event to handle mouseup, so it fires properly
				// even if the cursor leaves this.domNode before the mouse up event.
				var mouseUpHandle = this.own(on(this.ownerDocumentBody, "mouseup, touchend", lang.hitch(this, function(evt){
					mouseUpHandle.remove();
					focusHandle.remove();
					// if here, then the mousedown did not focus the focusNode as the default action
					if(this.focused){
						if(evt.type == "touchend"){
							this.defer("focus"); // native focus hasn't occurred yet
						}else{
							this.focus(); // native focus already occurred on mousedown
						}
					}
				})))[0];
			}
			if(this.scrollOnFocus){
				this.defer(function(){
					winUtils.scrollIntoView(this.domNode);
				}); // without defer, the input caret position can change on mouse click
			}
			this.inherited(arguments);
		},

		isFocusable: function(){
			// summary:
			//		Tells if this widget is focusable or not.  Used internally by dijit.
			// tags:
			//		protected
			return !this.disabled && this.focusNode && (domStyle.get(this.domNode, "display") != "none");
		},

		focus: function(){
			// summary:
			//		Put focus on this widget
			if(!this.disabled && this.focusNode.focus){
				try{
					this.focusNode.focus();
				}catch(e){
				}
				/*squelch errors from hidden nodes*/
			}
		},

		compare: function(/*anything*/ val1, /*anything*/ val2){
			// summary:
			//		Compare 2 values (as returned by get('value') for this widget).
			// tags:
			//		protected
			if(typeof val1 == "number" && typeof val2 == "number"){
				return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
			}else if(val1 > val2){
				return 1;
			}else if(val1 < val2){
				return -1;
			}else{
				return 0;
			}
		},

		onChange: function(/*===== newValue =====*/){
			// summary:
			//		Callback when this widget's value is changed.
			// tags:
			//		callback
		},

		// _onChangeActive: [private] Boolean
		//		Indicates that changes to the value should call onChange() callback.
		//		This is false during widget initialization, to avoid calling onChange()
		//		when the initial value is set.
		_onChangeActive: false,

		_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Called when the value of the widget is set.  Calls onChange() if appropriate
			// newValue:
			//		the new value
			// priorityChange:
			//		For a slider, for example, dragging the slider is priorityChange==false,
			//		but on mouse up, it's priorityChange==true.  If intermediateChanges==false,
			//		onChange is only called form priorityChange=true events.
			// tags:
			//		private
			if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
				// this block executes not for a change, but during initialization,
				// and is used to store away the original value (or for ToggleButton, the original checked state)
				this._resetValue = this._lastValueReported = newValue;
			}
			this._pendingOnChange = this._pendingOnChange
				|| (typeof newValue != typeof this._lastValueReported)
				|| (this.compare(newValue, this._lastValueReported) != 0);
			if((this.intermediateChanges || priorityChange || priorityChange === undefined) && this._pendingOnChange){
				this._lastValueReported = newValue;
				this._pendingOnChange = false;
				if(this._onChangeActive){
					if(this._onChangeHandle){
						this._onChangeHandle.remove();
					}
					// defer allows hidden value processing to run and
					// also the onChange handler can safely adjust focus, etc
					this._onChangeHandle = this.defer(
						function(){
							this._onChangeHandle = null;
							this.onChange(newValue);
						}); // try to collapse multiple onChange's fired faster than can be processed
				}
			}
		},

		create: function(){
			// Overrides _Widget.create()
			this.inherited(arguments);
			this._onChangeActive = true;
		},

		destroy: function(){
			if(this._onChangeHandle){ // destroy called before last onChange has fired
				this._onChangeHandle.remove();
				this.onChange(this._lastValueReported);
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/_FormValueMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/keys", // keys.ESCAPE
	"dojo/_base/lang",
	"dojo/on",
	"dojo/sniff", // has("ie"), has("quirks")
	"./_FormWidgetMixin"
], function(declare, domAttr, keys, lang, on, has, _FormWidgetMixin){

	// module:
	//		dijit/form/_FormValueMixin

	return declare("dijit.form._FormValueMixin", _FormWidgetMixin, {
		// summary:
		//		Mixin for widgets corresponding to native HTML elements such as `<input>` or `<select>`
		//		that have user changeable values.
		// description:
		//		Each _FormValueMixin represents a single input value, and has a (possibly hidden) `<input>` element,
		//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
		//		works as expected.

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,

		_setReadOnlyAttr: function(/*Boolean*/ value){
			domAttr.set(this.focusNode, 'readOnly', value);
			this._set("readOnly", value);
		},

		postCreate: function(){
			this.inherited(arguments);

			// Update our reset value if it hasn't yet been set (because this.set()
			// is only called when there *is* a value)
			if(this._resetValue === undefined){
				this._lastValueReported = this._resetValue = this.value;
			}
		},

		_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the widget.
			//		If the value has changed, then fire onChange event, unless priorityChange
			//		is specified as null (or false?)
			this._handleOnChange(newValue, priorityChange);
		},

		_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
			// summary:
			//		Called when the value of the widget has changed.  Saves the new value in this.value,
			//		and calls onChange() if appropriate.   See _FormWidget._handleOnChange() for details.
			this._set("value", newValue);
			this.inherited(arguments);
		},

		undo: function(){
			// summary:
			//		Restore the value to the last value passed to onChange
			this._setValueAttr(this._lastValueReported, false);
		},

		reset: function(){
			// summary:
			//		Reset the widget's value to what it was at initialization time
			this._hasBeenBlurred = false;
			this._setValueAttr(this._resetValue, true);
		}
	});
});

},
'dijit/form/_TextBoxMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/has",
	"dojo/keys", // keys.ALT keys.CAPS_LOCK keys.CTRL keys.META keys.SHIFT
	"dojo/_base/lang", // lang.mixin
	"dojo/on", // on
	"../main"    // for exporting dijit._setSelectionRange, dijit.selectInputText
], function(array, declare, dom, has, keys, lang, on, dijit){

	// module:
	//		dijit/form/_TextBoxMixin

	var _TextBoxMixin = declare("dijit.form._TextBoxMixin" + (has("dojo-bidi") ? "_NoBidi" : ""), null, {
		// summary:
		//		A mixin for textbox form input widgets

		// trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		// uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		// lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		// propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		// maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		// selectOnClick: [const] Boolean
		//		If true, all text will be selected when focused with mouse
		selectOnClick: false,

		// placeHolder: String
		//		Defines a hint to help users fill out the input field (as defined in HTML 5).
		//		This should only contain plain text (no html markup).
		placeHolder: "",

		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works as we like.
			// description:
			//		For `dijit/form/TextBox` this basically returns the value of the `<input>`.
			//
			//		For `dijit/form/MappedTextBox` subclasses, which have both
			//		a "displayed value" and a separate "submit value",
			//		This treats the "displayed value" as the master value, computing the
			//		submit value from it via this.parse().
			return this.parse(this.get('displayedValue'), this.constraints);
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Hook so set('value', ...) works.
			//
			// description:
			//		Sets the value of the widget to "value" which can be of
			//		any type as determined by the widget.
			//
			// value:
			//		The visual element value is also set to a corresponding,
			//		but not necessarily the same, value.
			//
			// formattedValue:
			//		If specified, used to set the visual element value,
			//		otherwise a computed visual value is used.
			//
			// priorityChange:
			//		If true, an onChange event is fired immediately instead of
			//		waiting for the next blur event.

			var filteredValue;
			if(value !== undefined){
				// TODO: this is calling filter() on both the display value and the actual value.
				// I added a comment to the filter() definition about this, but it should be changed.
				filteredValue = this.filter(value);
				if(typeof formattedValue != "string"){
					if(filteredValue !== null && ((typeof filteredValue != "number") || !isNaN(filteredValue))){
						formattedValue = this.filter(this.format(filteredValue, this.constraints));
					}else{
						formattedValue = '';
					}
				}
			}
			if(formattedValue != null /* and !undefined */ && ((typeof formattedValue) != "number" || !isNaN(formattedValue)) && this.textbox.value != formattedValue){
				this.textbox.value = formattedValue;
				this._set("displayedValue", this.get("displayedValue"));
			}

			this.inherited(arguments, [filteredValue, priorityChange]);
		},

		// displayedValue: String
		//		For subclasses like ComboBox where the displayed value
		//		(ex: Kentucky) and the serialized value (ex: KY) are different,
		//		this represents the displayed value.
		//
		//		Setting 'displayedValue' through set('displayedValue', ...)
		//		updates 'value', and vice-versa.  Otherwise 'value' is updated
		//		from 'displayedValue' periodically, like onBlur etc.
		//
		//		TODO: move declaration to MappedTextBox?
		//		Problem is that ComboBox references displayedValue,
		//		for benefit of FilteringSelect.
		displayedValue: "",

		_getDisplayedValueAttr: function(){
			// summary:
			//		Hook so get('displayedValue') works.
			// description:
			//		Returns the displayed value (what the user sees on the screen),
			//		after filtering (ie, trimming spaces etc.).
			//
			//		For some subclasses of TextBox (like ComboBox), the displayed value
			//		is different from the serialized value that's actually
			//		sent to the server (see `dijit/form/ValidationTextBox.serialize()`)

			// TODO: maybe we should update this.displayedValue on every keystroke so that we don't need
			// this method
			// TODO: this isn't really the displayed value when the user is typing
			return this.filter(this.textbox.value);
		},

		_setDisplayedValueAttr: function(/*String*/ value){
			// summary:
			//		Hook so set('displayedValue', ...) works.
			// description:
			//		Sets the value of the visual element to the string "value".
			//		The widget value is also set to a corresponding,
			//		but not necessarily the same, value.

			if(value == null /* or undefined */){
				value = ''
			}
			else if(typeof value != "string"){
				value = String(value)
			}

			this.textbox.value = value;

			// sets the serialized value to something corresponding to specified displayedValue
			// (if possible), and also updates the textbox.value, for example converting "123"
			// to "123.00"
			this._setValueAttr(this.get('value'), undefined);

			this._set("displayedValue", this.get('displayedValue'));
		},

		format: function(value /*=====, constraints =====*/){
			// summary:
			//		Replaceable function to convert a value to a properly formatted string.
			// value: String
			// constraints: Object
			// tags:
			//		protected extension
			return value == null /* or undefined */ ? "" : (value.toString ? value.toString() : value);
		},

		parse: function(value /*=====, constraints =====*/){
			// summary:
			//		Replaceable function to convert a formatted string to a value
			// value: String
			// constraints: Object
			// tags:
			//		protected extension

			return value;	// String
		},

		_refreshState: function(){
			// summary:
			//		After the user types some characters, etc., this method is
			//		called to check the field for validity etc.  The base method
			//		in `dijit/form/TextBox` does nothing, but subclasses override.
			// tags:
			//		protected
		},

		 onInput: function(/*===== event =====*/){
			 // summary:
			 //		Connect to this function to receive notifications of various user data-input events.
			 //		Return false to cancel the event and prevent it from being processed.
			 // event:
			 //		keydown | keypress | cut | paste | input
			 // tags:
			 //		callback
		 },

		__skipInputEvent: false,
		_onInput: function(/*Event*/ evt){
			// summary:
			//		Called AFTER the input event has happened

			this._processInput(evt);

			if(this.intermediateChanges){
				// allow the key to post to the widget input box
				this.defer(function(){
					this._handleOnChange(this.get('value'), false);
				});
			}
		},

		_processInput: function(/*Event*/ evt){
			// summary:
			//		Default action handler for user input events

			this._refreshState();

			// In case someone is watch()'ing for changes to displayedValue
			this._set("displayedValue", this.get("displayedValue"));
		},

		postCreate: function(){
			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			this.textbox.setAttribute("value", this.textbox.value); // DOM and JS values should be the same

			this.inherited(arguments);

			// normalize input events to reduce spurious event processing
			//	onkeydown: do not forward modifier keys
			//		       set charOrCode to numeric keycode
			//	onkeypress: do not forward numeric charOrCode keys (already sent through onkeydown)
			//	onpaste & oncut: set charOrCode to 229 (IME)
			//	oninput: if primary event not already processed, set charOrCode to 229 (IME), else do not forward
			var handleEvent = function(e){
				var charOrCode;
				if(e.type == "keydown"){
					charOrCode = e.keyCode;
					switch(charOrCode){ // ignore state keys
						case keys.SHIFT:
						case keys.ALT:
						case keys.CTRL:
						case keys.META:
						case keys.CAPS_LOCK:
						case keys.NUM_LOCK:
						case keys.SCROLL_LOCK:
							return;
					}
					if(!e.ctrlKey && !e.metaKey && !e.altKey){ // no modifiers
						switch(charOrCode){ // ignore location keys
							case keys.NUMPAD_0:
							case keys.NUMPAD_1:
							case keys.NUMPAD_2:
							case keys.NUMPAD_3:
							case keys.NUMPAD_4:
							case keys.NUMPAD_5:
							case keys.NUMPAD_6:
							case keys.NUMPAD_7:
							case keys.NUMPAD_8:
							case keys.NUMPAD_9:
							case keys.NUMPAD_MULTIPLY:
							case keys.NUMPAD_PLUS:
							case keys.NUMPAD_ENTER:
							case keys.NUMPAD_MINUS:
							case keys.NUMPAD_PERIOD:
							case keys.NUMPAD_DIVIDE:
								return;
						}
						if((charOrCode >= 65 && charOrCode <= 90) || (charOrCode >= 48 && charOrCode <= 57) || charOrCode == keys.SPACE){
							return; // keypress will handle simple non-modified printable keys
						}
						var named = false;
						for(var i in keys){
							if(keys[i] === e.keyCode){
								named = true;
								break;
							}
						}
						if(!named){
							return;
						} // only allow named ones through
					}
				}
				charOrCode = e.charCode >= 32 ? String.fromCharCode(e.charCode) : e.charCode;
				if(!charOrCode){
					charOrCode = (e.keyCode >= 65 && e.keyCode <= 90) || (e.keyCode >= 48 && e.keyCode <= 57) || e.keyCode == keys.SPACE ? String.fromCharCode(e.keyCode) : e.keyCode;
				}
				if(!charOrCode){
					charOrCode = 229; // IME
				}
				if(e.type == "keypress"){
					if(typeof charOrCode != "string"){
						return;
					}
					if((charOrCode >= 'a' && charOrCode <= 'z') || (charOrCode >= 'A' && charOrCode <= 'Z') || (charOrCode >= '0' && charOrCode <= '9') || (charOrCode === ' ')){
						if(e.ctrlKey || e.metaKey || e.altKey){
							return;
						} // can only be stopped reliably in keydown
					}
				}
				if(e.type == "input"){
					if(this.__skipInputEvent){ // duplicate event
						this.__skipInputEvent = false;
						return;
					}
				}else{
					this.__skipInputEvent = true;
				}
				// create fake event to set charOrCode and to know if preventDefault() was called
				var faux = { faux: true }, attr;
				for(attr in e){
					if(attr != "layerX" && attr != "layerY"){ // prevent WebKit warnings
						var v = e[attr];
						if(typeof v != "function" && typeof v != "undefined"){
							faux[attr] = v;
						}
					}
				}
				lang.mixin(faux, {
					charOrCode: charOrCode,
					_wasConsumed: false,
					preventDefault: function(){
						faux._wasConsumed = true;
						e.preventDefault();
					},
					stopPropagation: function(){
						e.stopPropagation();
					}
				});
				// give web page author a chance to consume the event
				//console.log(faux.type + ', charOrCode = (' + (typeof charOrCode) + ') ' + charOrCode + ', ctrl ' + !!faux.ctrlKey + ', alt ' + !!faux.altKey + ', meta ' + !!faux.metaKey + ', shift ' + !!faux.shiftKey);
				if(this.onInput(faux) === false){ // return false means stop
					faux.preventDefault();
					faux.stopPropagation();
				}
				if(faux._wasConsumed){
					return;
				} // if preventDefault was called
				this.defer(function(){
					this._onInput(faux);
				}); // widget notification after key has posted
				if(e.type == "keypress"){
					e.stopPropagation(); // don't allow parents to stop printables from being typed
				}
			};
			this.own(on(this.textbox, "keydown, keypress, paste, cut, input, compositionend", lang.hitch(this, handleEvent)));
		},

		_blankValue: '', // if the textbox is blank, what value should be reported
		filter: function(val){
			// summary:
			//		Auto-corrections (such as trimming) that are applied to textbox
			//		value on blur or form submit.
			// description:
			//		For MappedTextBox subclasses, this is called twice
			//
			//		- once with the display value
			//		- once the value as set/returned by set('value', ...)
			//
			//		and get('value'), ex: a Number for NumberTextBox.
			//
			//		In the latter case it does corrections like converting null to NaN.  In
			//		the former case the NumberTextBox.filter() method calls this.inherited()
			//		to execute standard trimming code in TextBox.filter().
			//
			//		TODO: break this into two methods in 2.0
			//
			// tags:
			//		protected extension
			if(val === null){
				return this._blankValue;
			}
			if(typeof val != "string"){
				return val;
			}
			if(this.trim){
				val = lang.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0, 1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		_setBlurValue: function(){
			// Format the displayed value, for example (for NumberTextBox) convert 1.4 to 1.400,
			// or (for CurrencyTextBox) 2.50 to $2.50

			this._setValueAttr(this.get('value'), true);
		},

		_onBlur: function(e){
			if(this.disabled){
				return;
			}
			this._setBlurValue();
			this.inherited(arguments);
		},

		_isTextSelected: function(){
			return this.textbox.selectionStart != this.textbox.selectionEnd;
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){
				return;
			}

			// Select all text on focus via click if nothing already selected.
			// Since mouse-up will clear the selection, need to defer selection until after mouse-up.
			// Don't do anything on focus by tabbing into the widget since there's no associated mouse-up event.
			if(this.selectOnClick && by == "mouse"){
				// Use on.once() to only select all text on first click only; otherwise users would have no way to clear
				// the selection.
				this._selectOnClickHandle = on.once(this.domNode, "mouseup, touchend", lang.hitch(this, function(evt){
					// Check if the user selected some text manually (mouse-down, mouse-move, mouse-up)
					// and if not, then select all the text
					if(!this._isTextSelected()){
						_TextBoxMixin.selectInputText(this.textbox);
					}
				}));
				this.own(this._selectOnClickHandle);

				// in case the mouseup never comes
				this.defer(function(){
					if(this._selectOnClickHandle){
						this._selectOnClickHandle.remove();
						this._selectOnClickHandle = null;
					}
				}, 500); // if mouseup not received soon, then treat it as some gesture
			}
			// call this.inherited() before refreshState(), since this.inherited() will possibly scroll the viewport
			// (to scroll the TextBox into view), which will affect how _refreshState() positions the tooltip
			this.inherited(arguments);

			this._refreshState();
		},

		reset: function(){
			// Overrides `dijit/_FormWidget/reset()`.
			// Additionally resets the displayed textbox value to ''
			this.textbox.value = '';
			this.inherited(arguments);
		}
	});

	if(has("dojo-bidi")){
		_TextBoxMixin = declare("dijit.form._TextBoxMixin", _TextBoxMixin, {
			_setValueAttr: function(){
				this.inherited(arguments);
				this.applyTextDir(this.focusNode);
			},
			_setDisplayedValueAttr: function(){
				this.inherited(arguments);
				this.applyTextDir(this.focusNode);
			},
			_onInput: function(){
				this.applyTextDir(this.focusNode);
				this.inherited(arguments);
			}
		});
	}

	_TextBoxMixin._setSelectionRange = dijit._setSelectionRange = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
		if(element.setSelectionRange){
			element.setSelectionRange(start, stop);
		}
	};

	_TextBoxMixin.selectInputText = dijit.selectInputText = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
		// summary:
		//		Select text in the input element argument, from start (default 0), to stop (default end).

		// TODO: use functions in _editor/selection.js?
		element = dom.byId(element);
		if(isNaN(start)){
			start = 0;
		}
		if(isNaN(stop)){
			stop = element.value ? element.value.length : 0;
		}
		try{
			element.focus();
			_TextBoxMixin._setSelectionRange(element, start, stop);
		}catch(e){ /* squelch random errors (esp. on IE) from unexpected focus changes or DOM nodes being hidden */
		}
	};

	return _TextBoxMixin;
});

},
'dijit/form/ComboBoxMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/Deferred",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin
	"dojo/store/util/QueryResults",
	"./_AutoCompleterMixin",
	"./_ComboBoxMenu",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownBox.html"
], function(declare, Deferred, kernel, lang, QueryResults, _AutoCompleterMixin, _ComboBoxMenu, _HasDropDown, template){


	// module:
	//		dijit/form/ComboBoxMixin

	return declare("dijit.form.ComboBoxMixin", [_HasDropDown, _AutoCompleterMixin], {
		// summary:
		//		Provides main functionality of ComboBox widget

		// dropDownClass: [protected extension] Function String
		//		Dropdown widget class used to select a date/time.
		//		Subclasses should specify this.
		dropDownClass: _ComboBoxMenu,

		// hasDownArrow: Boolean
		//		Set this textbox to have a down arrow button, to display the drop down list.
		//		Defaults to true.
		hasDownArrow: true,

		templateString: template,

		baseClass: "dijitTextBox dijitComboBox",

		/*=====
		// store: [const] dojo/store/api/Store|dojo/data/api/Read
		//		Reference to data provider object used by this ComboBox.
		//
		//		Should be dojo/store/api/Store, but dojo/data/api/Read supported
		//		for backwards compatibility.
		store: null,
		=====*/

		// Set classes like dijitDownArrowButtonHover depending on
		// mouse action over button node
		cssStateNodes: {
			"_buttonNode": "dijitDownArrowButton"
		},

		_setHasDownArrowAttr: function(/*Boolean*/ val){
			this._set("hasDownArrow", val);
			this._buttonNode.style.display = val ? "" : "none";
		},

		_showResultList: function(){
			// hide the tooltip
			this.displayMessage("");
			this.inherited(arguments);
		},

		_setStoreAttr: function(store){
			// For backwards-compatibility, accept dojo.data store in addition to dojo/store/api/Store.  Remove in 2.0.
			if(!store.get){
				lang.mixin(store, {
					_oldAPI: true,
					get: function(id){
						// summary:
						//		Retrieves an object by it's identity. This will trigger a fetchItemByIdentity.
						//		Like dojo/store/DataStore.get() except returns native item.
						var deferred = new Deferred();
						this.fetchItemByIdentity({
							identity: id,
							onItem: function(object){
								deferred.resolve(object);
							},
							onError: function(error){
								deferred.reject(error);
							}
						});
						return deferred.promise;
					},
					query: function(query, options){
						// summary:
						//		Queries the store for objects.   Like dojo/store/DataStore.query()
						//		except returned Deferred contains array of native items.
						var deferred = new Deferred(function(){ fetchHandle.abort && fetchHandle.abort(); });
						deferred.total = new Deferred();
						var fetchHandle = this.fetch(lang.mixin({
							query: query,
							onBegin: function(count){
								deferred.total.resolve(count);
							},
							onComplete: function(results){
								deferred.resolve(results);
							},
							onError: function(error){
								deferred.reject(error);
							}
						}, options));
						return QueryResults(deferred);
					}
				});
			}
			this._set("store", store);
		},

		postMixInProperties: function(){
			// Since _setValueAttr() depends on this.store, _setStoreAttr() needs to execute first.
			// Unfortunately, without special code, it ends up executing second.
			var store = this.params.store || this.store;
			if(store){
				this._setStoreAttr(store);
			}

			this.inherited(arguments);

			// User may try to access this.store.getValue() etc.  in a custom labelFunc() function.
			// It's not available with the new data store for handling inline <option> tags, so add it.
			if(!this.params.store && !this.store._oldAPI){
				var clazz = this.declaredClass;
				lang.mixin(this.store, {
					getValue: function(item, attr){
						kernel.deprecated(clazz + ".store.getValue(item, attr) is deprecated for builtin store.  Use item.attr directly", "", "2.0");
						return item[attr];
					},
					getLabel: function(item){
						kernel.deprecated(clazz + ".store.getLabel(item) is deprecated for builtin store.  Use item.label directly", "", "2.0");
						return item.name;
					},
					fetch: function(args){
						kernel.deprecated(clazz + ".store.fetch() is deprecated for builtin store.", "Use store.query()", "2.0");
						var shim = ["dojo/data/ObjectStore"];	// indirection so it doesn't get rolled into a build
						require(shim, lang.hitch(this, function(ObjectStore){
							new ObjectStore({objectStore: this}).fetch(args);
						}));
					}
				});
			}
		}
	});
});

},
'dojo/store/util/QueryResults':function(){
define(["../../_base/array", "../../_base/lang", "../../when"
], function(array, lang, when){

// module:
//		dojo/store/util/QueryResults

var QueryResults = function(results){
	// summary:
	//		A function that wraps the results of a store query with additional
	//		methods.
	// description:
	//		QueryResults is a basic wrapper that allows for array-like iteration
	//		over any kind of returned data from a query.  While the simplest store
	//		will return a plain array of data, other stores may return deferreds or
	//		promises; this wrapper makes sure that *all* results can be treated
	//		the same.
	//
	//		Additional methods include `forEach`, `filter` and `map`.
	// results: Array|dojo/promise/Promise
	//		The result set as an array, or a promise for an array.
	// returns:
	//		An array-like object that can be used for iterating over.
	// example:
	//		Query a store and iterate over the results.
	//
	//	|	store.query({ prime: true }).forEach(function(item){
	//	|		//	do something
	//	|	});

	if(!results){
		return results;
	}
	// if it is a promise it may be frozen
	if(results.then){
		results = lang.delegate(results);
	}
	function addIterativeMethod(method){
		if(!results[method]){
			results[method] = function(){
				var args = arguments;
				return when(results, function(results){
					Array.prototype.unshift.call(args, results);
					return QueryResults(array[method].apply(array, args));
				});
			};
		}
	}
	addIterativeMethod("forEach");
	addIterativeMethod("filter");
	addIterativeMethod("map");
	if(!results.total){
		results.total = when(results, function(results){
			return results.length;
		});
	}
	return results; // Object
};

lang.setObject("dojo.store.util.QueryResults", QueryResults);

return QueryResults;

});

},
'dijit/form/_AutoCompleterMixin':function(){
define([
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.get
	"dojo/keys",
	"dojo/_base/lang", // lang.clone lang.hitch
	"dojo/query", // query
	"dojo/regexp", // regexp.escapeString
	"dojo/sniff", // has("ie")
	"./DataList",
	"./_TextBoxMixin", // defines _TextBoxMixin.selectInputText
	"./_SearchMixin"
], function(aspect, declare, domAttr, keys, lang, query, regexp, has, DataList, _TextBoxMixin, SearchMixin){

	// module:
	//		dijit/form/_AutoCompleterMixin

	var AutoCompleterMixin = declare("dijit.form._AutoCompleterMixin", SearchMixin, {
		// summary:
		//		A mixin that implements the base functionality for `dijit/form/ComboBox`/`dijit/form/FilteringSelect`
		// description:
		//		All widgets that mix in dijit/form/_AutoCompleterMixin must extend `dijit/form/_FormValueWidget`.
		// tags:
		//		protected

		// item: Object
		//		This is the item returned by the dojo/store/api/Store implementation that
		//		provides the data for this ComboBox, it's the currently selected item.
		item: null,

		// autoComplete: Boolean
		//		If user types in a partial string, and then tab out of the `<input>` box,
		//		automatically copy the first entry displayed in the drop down list to
		//		the `<input>` field
		autoComplete: true,

		// highlightMatch: String
		//		One of: "first", "all" or "none".
		//
		//		If the ComboBox/FilteringSelect opens with the search results and the searched
		//		string can be found, it will be highlighted.  If set to "all"
		//		then will probably want to change `queryExpr` parameter to '*${0}*'
		//
		//		Highlighting is only performed when `labelType` is "text", so as to not
		//		interfere with any HTML markup an HTML label might contain.
		highlightMatch: "first",

		// labelAttr: String?
		//		The entries in the drop down list come from this attribute in the
		//		dojo.data items.
		//		If not specified, the searchAttr attribute is used instead.
		labelAttr: "",

		// labelType: String
		//		Specifies how to interpret the labelAttr in the data store items.
		//		Can be "html" or "text".
		labelType: "text",

		// Flags to _HasDropDown to limit height of drop down to make it fit in viewport
		maxHeight: -1,

		// For backwards compatibility let onClick events propagate, even clicks on the down arrow button
		_stopClickEvents: false,

		_getCaretPos: function(/*DomNode*/ element){
			// khtml 3.5.2 has selection* methods as does webkit nightlies from 2005-06-22
			var pos = 0;
			if(typeof(element.selectionStart) == "number"){
				// FIXME: this is totally borked on Moz < 1.3. Any recourse?
				pos = element.selectionStart;
			}else if(has("ie")){
				// in the case of a mouse click in a popup being handled,
				// then the document.selection is not the textarea, but the popup
				// var r = document.selection.createRange();
				// hack to get IE 6 to play nice. What a POS browser.
				var tr = element.ownerDocument.selection.createRange().duplicate();
				var ntr = element.createTextRange();
				tr.move("character", 0);
				ntr.move("character", 0);
				try{
					// If control doesn't have focus, you get an exception.
					// Seems to happen on reverse-tab, but can also happen on tab (seems to be a race condition - only happens sometimes).
					// There appears to be no workaround for this - googled for quite a while.
					ntr.setEndPoint("EndToEnd", tr);
					pos = String(ntr.text).replace(/\r/g, "").length;
				}catch(e){
					// If focus has shifted, 0 is fine for caret pos.
				}
			}
			return pos;
		},

		_setCaretPos: function(/*DomNode*/ element, /*Number*/ location){
			location = parseInt(location);
			_TextBoxMixin.selectInputText(element, location, location);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			// Additional code to set disabled state of ComboBox node.
			// Overrides _FormValueWidget._setDisabledAttr() or ValidationTextBox._setDisabledAttr().
			this.inherited(arguments);
			this.domNode.setAttribute("aria-disabled", value ? "true" : "false");
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles keyboard events

			if(evt.charCode >= 32){
				return;
			} // alphanumeric reserved for searching

			var key = evt.charCode || evt.keyCode;

			// except for cutting/pasting case - ctrl + x/v
			if(key == keys.ALT || key == keys.CTRL || key == keys.META || key == keys.SHIFT){
				return; // throw out spurious events
			}

			var pw = this.dropDown;
			var highlighted = null;
			this._abortQuery();

			// _HasDropDown will do some of the work:
			//
			//	1. when drop down is not yet shown:
			//		- if user presses the down arrow key, call loadDropDown()
			//	2. when drop down is already displayed:
			//		- on ESC key, call closeDropDown()
			//		- otherwise, call dropDown.handleKey() to process the keystroke
			this.inherited(arguments);

			if(evt.altKey || evt.ctrlKey || evt.metaKey){
				return;
			} // don't process keys with modifiers  - but we want shift+TAB

			if(this._opened){
				highlighted = pw.getHighlightedOption();
			}
			switch(key){
				case keys.PAGE_DOWN:
				case keys.DOWN_ARROW:
				case keys.PAGE_UP:
				case keys.UP_ARROW:
					// Keystroke caused ComboBox_menu to move to a different item.
					// Copy new item to <input> box.
					if(this._opened){
						this._announceOption(highlighted);
					}
					evt.stopPropagation();
					evt.preventDefault();
					break;

				case keys.ENTER:
					// prevent submitting form if user presses enter. Also
					// prevent accepting the value if either Next or Previous
					// are selected
					if(highlighted){
						// only stop event on prev/next
						if(highlighted == pw.nextButton){
							this._nextSearch(1);
							// prevent submit
							evt.stopPropagation();
							evt.preventDefault();
							break;
						}else if(highlighted == pw.previousButton){
							this._nextSearch(-1);
							// prevent submit
							evt.stopPropagation();
							evt.preventDefault();
							break;
						}
						// prevent submit if ENTER was to choose an item
						evt.stopPropagation();
						evt.preventDefault();
					}else{
						// Update 'value' (ex: KY) according to currently displayed text
						this._setBlurValue(); // set value if needed
						this._setCaretPos(this.focusNode, this.focusNode.value.length); // move cursor to end and cancel highlighting
					}
				// fall through

				case keys.TAB:
					var newvalue = this.get('displayedValue');
					//	if the user had More Choices selected fall into the
					//	_onBlur handler
					if(pw && (newvalue == pw._messages["previousMessage"] || newvalue == pw._messages["nextMessage"])){
						break;
					}
					if(highlighted){
						this._selectOption(highlighted);
					}
				// fall through

				case keys.ESCAPE:
					if(this._opened){
						this._lastQuery = null; // in case results come back later
						this.closeDropDown();
					}
					break;
			}
		},

		_autoCompleteText: function(/*String*/ text){
			// summary:
			//		Fill in the textbox with the first item from the drop down
			//		list, and highlight the characters that were
			//		auto-completed. For example, if user typed "CA" and the
			//		drop down list appeared, the textbox would be changed to
			//		"California" and "ifornia" would be highlighted.

			var fn = this.focusNode;

			// IE7: clear selection so next highlight works all the time
			_TextBoxMixin.selectInputText(fn, fn.value.length);
			// does text autoComplete the value in the textbox?
			var caseFilter = this.ignoreCase ? 'toLowerCase' : 'substr';
			if(text[caseFilter](0).indexOf(this.focusNode.value[caseFilter](0)) == 0){
				var cpos = this.autoComplete ? this._getCaretPos(fn) : fn.value.length;
				// only try to extend if we added the last character at the end of the input
				if((cpos + 1) > fn.value.length){
					// only add to input node as we would overwrite Capitalisation of chars
					// actually, that is ok
					fn.value = text;//.substr(cpos);
					// visually highlight the autocompleted characters
					_TextBoxMixin.selectInputText(fn, cpos);
				}
			}else{
				// text does not autoComplete; replace the whole value and highlight
				fn.value = text;
				_TextBoxMixin.selectInputText(fn);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ query, /*Object*/ options){
			// summary:
			//		Callback when a search completes.
			// description:
			//		1. generates drop-down list and calls _showResultList() to display it
			//		2. if this result list is from user pressing "more choices"/"previous choices"
			//			then tell screen reader to announce new option
			var wasSelected = this.dropDown.getHighlightedOption();
			this.dropDown.clearResultList();
			if(!results.length && options.start == 0){ // if no results and not just the previous choices button
				this.closeDropDown();
				return;
			}
			this._nextSearch = this.dropDown.onPage = lang.hitch(this, function(direction){
				results.nextPage(direction !== -1);
				this.focus();
			});

			// Fill in the textbox with the first item from the drop down list,
			// and highlight the characters that were auto-completed. For
			// example, if user typed "CA" and the drop down list appeared, the
			// textbox would be changed to "California" and "ifornia" would be
			// highlighted.

			this.dropDown.createOptions(
				results,
				options,
				lang.hitch(this, "_getMenuLabelFromItem")
			);

			// show our list (only if we have content, else nothing)
			this._showResultList();

			// #4091:
			//		tell the screen reader that the paging callback finished by
			//		shouting the next choice
			if("direction" in options){
				if(options.direction){
					this.dropDown.highlightFirstOption();
				}else if(!options.direction){
					this.dropDown.highlightLastOption();
				}
				if(wasSelected){
					this._announceOption(this.dropDown.getHighlightedOption());
				}
			}else if(this.autoComplete && !this._prev_key_backspace
				// when the user clicks the arrow button to show the full list,
				// startSearch looks for "*".
				// it does not make sense to autocomplete
				// if they are just previewing the options available.
				&& !/^[*]+$/.test(query[this.searchAttr].toString())){
				this._announceOption(this.dropDown.containerNode.firstChild.nextSibling); // 1st real item
			}
		},

		_showResultList: function(){
			// summary:
			//		Display the drop down if not already displayed, or if it is displayed, then
			//		reposition it if necessary (reposition may be necessary if drop down's height changed).
			this.closeDropDown(true);
			this.openDropDown();
			this.domNode.setAttribute("aria-expanded", "true");
		},

		loadDropDown: function(/*Function*/ /*===== callback =====*/){
			// Overrides _HasDropDown.loadDropDown().
			// This is called when user has pressed button icon or pressed the down arrow key
			// to open the drop down.
			this._startSearchAll();
		},

		isLoaded: function(){
			// signal to _HasDropDown that it needs to call loadDropDown() to load the
			// drop down asynchronously before displaying it
			return false;
		},

		closeDropDown: function(){
			// Overrides _HasDropDown.closeDropDown().  Closes the drop down (assuming that it's open).
			// This method is the callback when the user types ESC or clicking
			// the button icon while the drop down is open.  It's also called by other code.
			this._abortQuery();
			if(this._opened){
				this.inherited(arguments);
				this.domNode.setAttribute("aria-expanded", "false");
			}
		},

		_setBlurValue: function(){
			// if the user clicks away from the textbox OR tabs away, set the
			// value to the textbox value
			// #4617:
			//		if value is now more choices or previous choices, revert
			//		the value
			var newvalue = this.get('displayedValue');
			var pw = this.dropDown;
			if(pw && (newvalue == pw._messages["previousMessage"] || newvalue == pw._messages["nextMessage"])){
				this._setValueAttr(this._lastValueReported, true);
			}else if(typeof this.item == "undefined"){
				// Update 'value' (ex: KY) according to currently displayed text
				this.item = null;
				this.set('displayedValue', newvalue);
			}else{
				if(this.value != this._lastValueReported){
					this._handleOnChange(this.value, true);
				}
				this._refreshState();
			}
			// Remove aria-activedescendant since it may not be removed if they select with arrows then blur with mouse
			this.focusNode.removeAttribute("aria-activedescendant");
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Set the displayed valued in the input box, and the hidden value
			//		that gets submitted, based on a dojo.data store item.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('item', value)
			// tags:
			//		private
			var value = '';
			if(item){
				if(!displayedValue){
					displayedValue = this.store._oldAPI ? // remove getValue() for 2.0 (old dojo.data API)
						this.store.getValue(item, this.searchAttr) : item[this.searchAttr];
				}
				value = this._getValueField() != this.searchAttr ? this.store.getIdentity(item) : displayedValue;
			}
			this.set('value', value, priorityChange, displayedValue, item);
		},

		_announceOption: function(/*Node*/ node){
			// summary:
			//		a11y code that puts the highlighted option in the textbox.
			//		This way screen readers will know what is happening in the
			//		menu.

			if(!node){
				return;
			}
			// pull the text value from the item attached to the DOM node
			var newValue;
			if(node == this.dropDown.nextButton ||
				node == this.dropDown.previousButton){
				newValue = node.innerHTML;
				this.item = undefined;
				this.value = '';
			}else{
				var item = this.dropDown.items[node.getAttribute("item")];
				newValue = (this.store._oldAPI ? // remove getValue() for 2.0 (old dojo.data API)
					this.store.getValue(item, this.searchAttr) : item[this.searchAttr]).toString();
				this.set('item', item, false, newValue);
			}
			// get the text that the user manually entered (cut off autocompleted text)
			this.focusNode.value = this.focusNode.value.substring(0, this._lastInput.length);
			// set up ARIA activedescendant
			this.focusNode.setAttribute("aria-activedescendant", domAttr.get(node, "id"));
			// autocomplete the rest of the option to announce change
			this._autoCompleteText(newValue);
		},

		_selectOption: function(/*DomNode*/ target){
			// summary:
			//		Menu callback function, called when an item in the menu is selected.
			this.closeDropDown();
			if(target){
				this._announceOption(target);
			}
			this._setCaretPos(this.focusNode, this.focusNode.value.length);
			this._handleOnChange(this.value, true);
			// Remove aria-activedescendant since the drop down is no loner visible
			// after closeDropDown() but _announceOption() adds it back in
			this.focusNode.removeAttribute("aria-activedescendant");
		},

		_startSearchAll: function(){
			this._startSearch('');
		},

		_startSearchFromInput: function(){
			this.item = undefined; // undefined means item needs to be set
			this.inherited(arguments);
		},

		_startSearch: function(/*String*/ key){
			// summary:
			//		Starts a search for elements matching key (key=="" means to return all items),
			//		and calls _openResultList() when the search completes, to display the results.
			if(!this.dropDown){
				var popupId = this.id + "_popup",
					dropDownConstructor = lang.isString(this.dropDownClass) ?
						lang.getObject(this.dropDownClass, false) : this.dropDownClass;
				this.dropDown = new dropDownConstructor({
					onChange: lang.hitch(this, this._selectOption),
					id: popupId,
					dir: this.dir,
					textDir: this.textDir
				});
			}
			this._lastInput = key; // Store exactly what was entered by the user.
			this.inherited(arguments);
		},

		_getValueField: function(){
			// summary:
			//		Helper for postMixInProperties() to set this.value based on data inlined into the markup.
			//		Returns the attribute name in the item (in dijit/form/_ComboBoxDataStore) to use as the value.
			return this.searchAttr;
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		postMixInProperties: function(){
			this.inherited(arguments);
			if(!this.store){
				var srcNodeRef = this.srcNodeRef;
				// if user didn't specify store, then assume there are option tags
				this.store = new DataList({}, srcNodeRef);

				// if there is no value set and there is an option list, set
				// the value to the first value to be consistent with native Select
				// Firefox and Safari set value
				// IE6 and Opera set selectedIndex, which is automatically set
				// by the selected attribute of an option tag
				// IE6 does not set value, Opera sets value = selectedIndex
				if(!("value" in this.params)){
					var item = (this.item = this.store.fetchSelectedItem());
					if(item){
						var valueField = this._getValueField();
						// remove getValue() for 2.0 (old dojo.data API)
						this.value = this.store._oldAPI ? this.store.getValue(item, valueField) : item[valueField];
					}
				}
			}
		},

		postCreate: function(){
			// summary:
			//		Subclasses must call this method from their postCreate() methods
			// tags:
			//		protected

			// find any associated label element and add to ComboBox node.
			var label = query('label[for="' + this.id + '"]');
			if(label.length){
				if(!label[0].id){
					label[0].id = this.id + "_label";
				}
				this.domNode.setAttribute("aria-labelledby", label[0].id);

			}
			this.inherited(arguments);
			aspect.after(this, "onSearch", lang.hitch(this, "_openResultList"), true);
		},

		_getMenuLabelFromItem: function(/*Item*/ item){
			var label = this.labelFunc(item, this.store),
				labelType = this.labelType;
			// If labelType is not "text" we don't want to screw any markup ot whatever.
			if(this.highlightMatch != "none" && this.labelType == "text" && this._lastInput){
				label = this.doHighlight(label, this._lastInput);
				labelType = "html";
			}
			return {html: labelType == "html", label: label};
		},

		doHighlight: function(/*String*/ label, /*String*/ find){
			// summary:
			//		Highlights the string entered by the user in the menu.  By default this
			//		highlights the first occurrence found. Override this method
			//		to implement your custom highlighting.
			// tags:
			//		protected

			var
			// Add (g)lobal modifier when this.highlightMatch == "all" and (i)gnorecase when this.ignoreCase == true
				modifiers = (this.ignoreCase ? "i" : "") + (this.highlightMatch == "all" ? "g" : ""),
				i = this.queryExpr.indexOf("${0}");
			find = regexp.escapeString(find); // escape regexp special chars
			//If < appears in label, and user presses t, we don't want to highlight the t in the escaped "&lt;"
			//first find out every occurences of "find", wrap each occurence in a pair of "\uFFFF" characters (which
			//should not appear in any string). then html escape the whole string, and replace '\uFFFF" with the
			//HTML highlight markup. 
			return this._escapeHtml(label.replace(
				new RegExp((i == 0 ? "^" : "") + "(" + find + ")" + (i == (this.queryExpr.length - 4) ? "$" : ""), modifiers),
				'\uFFFF$1\uFFFF')).replace(
				/\uFFFF([^\uFFFF]+)\uFFFF/g, '<span class="dijitComboBoxHighlightMatch">$1</span>'
			); // returns String, (almost) valid HTML (entities encoded)
		},

		_escapeHtml: function(/*String*/ str){
			// TODO Should become dojo.html.entities(), when exists use instead
			// summary:
			//		Adds escape sequences for special characters in XML: `&<>"'`
			str = String(str).replace(/&/gm, "&amp;").replace(/</gm, "&lt;")
				.replace(/>/gm, "&gt;").replace(/"/gm, "&quot;"); //balance"
			return str; // string
		},

		reset: function(){
			// Overrides the _FormWidget.reset().
			// Additionally reset the .item (to clean up).
			this.item = null;
			this.inherited(arguments);
		},

		labelFunc: function(item, store){
			// summary:
			//		Computes the label to display based on the dojo.data store item.
			// item: Object
			//		The item from the store
			// store: dojo/store/api/Store
			//		The store.
			// returns:
			//		The label that the ComboBox should display
			// tags:
			//		private

			// Use toString() because XMLStore returns an XMLItem whereas this
			// method is expected to return a String (#9354).
			// Remove getValue() for 2.0 (old dojo.data API)
			return (store._oldAPI ? store.getValue(item, this.labelAttr || this.searchAttr) :
				item[this.labelAttr || this.searchAttr]).toString(); // String
		},

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange, /*String?*/ displayedValue, /*item?*/ item){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the select.
			this._set("item", item || null); // value not looked up in store
			if(value == null /* or undefined */){
				value = '';
			} // null translates to blank
			this.inherited(arguments);
		}
	});

	if(has("dojo-bidi")){
		AutoCompleterMixin.extend({
			_setTextDirAttr: function(/*String*/ textDir){
				// summary:
				//		Setter for textDir, needed for the dropDown's textDir update.
				// description:
				//		Users shouldn't call this function; they should be calling
				//		set('textDir', value)
				// tags:
				//		private
				this.inherited(arguments);
				// update the drop down also (_ComboBoxMenuMixin)
				if(this.dropDown){
					this.dropDown._set("textDir", textDir);
				}
			}
		});
	}

	return AutoCompleterMixin;
});

},
'dojo/regexp':function(){
define(["./_base/kernel", "./_base/lang"], function(dojo, lang){

// module:
//		dojo/regexp

var regexp = {
	// summary:
	//		Regular expressions and Builder resources
};
lang.setObject("dojo.regexp", regexp);

regexp.escapeString = function(/*String*/str, /*String?*/except){
	// summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
};

regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	// summary:
	//		Builds a regular expression that groups subexpressions
	// description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression.
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return regexp.group(b.join("|"), nonCapture); // String
};

regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression.
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
};

return regexp;
});

},
'dijit/form/DataList':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.byId
	"dojo/_base/lang", // lang.trim
	"dojo/query", // query
	"dojo/store/Memory",
	"../registry"	// registry.add registry.remove
], function(declare, dom, lang, query, MemoryStore, registry){

	// module:
	//		dijit/form/DataList

	function toItem(/*DOMNode*/ option){
		// summary:
		//		Convert `<option>` node to hash
		return {
			id: option.value,
			value: option.value,
			name: lang.trim(option.innerText || option.textContent || '')
		};
	}

	return declare("dijit.form.DataList", MemoryStore, {
		// summary:
		//		Inefficient but small data store specialized for inlined data via OPTION tags
		//
		// description:
		//		Provides a store for inlined data like:
		//
		//	|	<datalist>
		//	|		<option value="AL">Alabama</option>
		//	|		...

		constructor: function(params, srcNodeRef){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String
			//		Attach widget to this DOM node.

			// store pointer to original DOM tree
			this.domNode = dom.byId(srcNodeRef);

			lang.mixin(this, params);
			if(this.id){
				registry.add(this); // add to registry so it can be easily found by id
			}
			this.domNode.style.display = "none";

			this.inherited(arguments, [{
				data: query("option", this.domNode).map(toItem)
			}]);
		},

		destroy: function(){
			registry.remove(this.id);
		},

		fetchSelectedItem: function(){
			// summary:
			//		Get the option marked as selected, like `<option selected>`.
			//		Not part of dojo.data API.
			var option = query("> option[selected]", this.domNode)[0] || query("> option", this.domNode)[0];
			return option && toItem(option);
		}
	});
});

},
'dojo/store/Memory':function(){
define(["../_base/declare", "./util/QueryResults", "./util/SimpleQueryEngine" /*=====, "./api/Store" =====*/],
function(declare, QueryResults, SimpleQueryEngine /*=====, Store =====*/){

// module:
//		dojo/store/Memory

// No base class, but for purposes of documentation, the base class is dojo/store/api/Store
var base = null;
/*===== base = Store; =====*/

return declare("dojo.store.Memory", base, {
	// summary:
	//		This is a basic in-memory object store. It implements dojo/store/api/Store.
	constructor: function(options){
		// summary:
		//		Creates a memory object store.
		// options: dojo/store/Memory
		//		This provides any configuration information that will be mixed into the store.
		//		This should generally include the data property to provide the starting set of data.
		for(var i in options){
			this[i] = options[i];
		}
		this.setData(this.data || []);
	},
	// data: Array
	//		The array of all the objects in the memory store
	data:null,

	// idProperty: String
	//		Indicates the property to use as the identity property. The values of this
	//		property should be unique.
	idProperty: "id",

	// index: Object
	//		An index of data indices into the data array by id
	index:null,

	// queryEngine: Function
	//		Defines the query engine to use for querying the data store
	queryEngine: SimpleQueryEngine,
	get: function(id){
		// summary:
		//		Retrieves an object by its identity
		// id: Number
		//		The identity to use to lookup the object
		// returns: Object
		//		The object in the store that matches the given id.
		return this.data[this.index[id]];
	},
	getIdentity: function(object){
		// summary:
		//		Returns an object's identity
		// object: Object
		//		The object to get the identity from
		// returns: Number
		return object[this.idProperty];
	},
	put: function(object, options){
		// summary:
		//		Stores an object
		// object: Object
		//		The object to store.
		// options: dojo/store/api/Store.PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		// returns: Number
		var data = this.data,
			index = this.index,
			idProperty = this.idProperty;
		var id = object[idProperty] = (options && "id" in options) ? options.id : idProperty in object ? object[idProperty] : Math.random();
		if(id in index){
			// object exists
			if(options && options.overwrite === false){
				throw new Error("Object already exists");
			}
			// replace the entry in data
			data[index[id]] = object;
		}else{
			// add the new object
			index[id] = data.push(object) - 1;
		}
		return id;
	},
	add: function(object, options){
		// summary:
		//		Creates an object, throws an error if the object already exists
		// object: Object
		//		The object to store.
		// options: dojo/store/api/Store.PutDirectives?
		//		Additional metadata for storing the data.  Includes an "id"
		//		property if a specific id is to be used.
		// returns: Number
		(options = options || {}).overwrite = false;
		// call put with overwrite being false
		return this.put(object, options);
	},
	remove: function(id){
		// summary:
		//		Deletes an object by its identity
		// id: Number
		//		The identity to use to delete the object
		// returns: Boolean
		//		Returns true if an object was removed, falsy (undefined) if no object matched the id
		var index = this.index;
		var data = this.data;
		if(id in index){
			data.splice(index[id], 1);
			// now we have to reindex
			this.setData(data);
			return true;
		}
	},
	query: function(query, options){
		// summary:
		//		Queries the store for objects.
		// query: Object
		//		The query to use for retrieving objects from the store.
		// options: dojo/store/api/Store.QueryOptions?
		//		The optional arguments to apply to the resultset.
		// returns: dojo/store/api/Store.QueryResults
		//		The results of the query, extended with iterative methods.
		//
		// example:
		//		Given the following store:
		//
		// 	|	var store = new Memory({
		// 	|		data: [
		// 	|			{id: 1, name: "one", prime: false },
		//	|			{id: 2, name: "two", even: true, prime: true},
		//	|			{id: 3, name: "three", prime: true},
		//	|			{id: 4, name: "four", even: true, prime: false},
		//	|			{id: 5, name: "five", prime: true}
		//	|		]
		//	|	});
		//
		//	...find all items where "prime" is true:
		//
		//	|	var results = store.query({ prime: true });
		//
		//	...or find all items where "even" is true:
		//
		//	|	var results = store.query({ even: true });
		return QueryResults(this.queryEngine(query, options)(this.data));
	},
	setData: function(data){
		// summary:
		//		Sets the given data as the source for this store, and indexes it
		// data: Object[]
		//		An array of objects to use as the source of data.
		if(data.items){
			// just for convenience with the data format IFRS expects
			this.idProperty = data.identifier;
			data = this.data = data.items;
		}else{
			this.data = data;
		}
		this.index = {};
		for(var i = 0, l = data.length; i < l; i++){
			this.index[data[i][this.idProperty]] = i;
		}
	}
});

});

},
'dojo/store/util/SimpleQueryEngine':function(){
define(["../../_base/array" /*=====, "../api/Store" =====*/], function(arrayUtil /*=====, Store =====*/){

// module:
//		dojo/store/util/SimpleQueryEngine

return function(query, options){
	// summary:
	//		Simple query engine that matches using filter functions, named filter
	//		functions or objects by name-value on a query object hash
	//
	// description:
	//		The SimpleQueryEngine provides a way of getting a QueryResults through
	//		the use of a simple object hash as a filter.  The hash will be used to
	//		match properties on data objects with the corresponding value given. In
	//		other words, only exact matches will be returned.
	//
	//		This function can be used as a template for more complex query engines;
	//		for example, an engine can be created that accepts an object hash that
	//		contains filtering functions, or a string that gets evaluated, etc.
	//
	//		When creating a new dojo.store, simply set the store's queryEngine
	//		field as a reference to this function.
	//
	// query: Object
	//		An object hash with fields that may match fields of items in the store.
	//		Values in the hash will be compared by normal == operator, but regular expressions
	//		or any object that provides a test() method are also supported and can be
	//		used to match strings by more complex expressions
	//		(and then the regex's or object's test() method will be used to match values).
	//
	// options: dojo/store/api/Store.QueryOptions?
	//		An object that contains optional information such as sort, start, and count.
	//
	// returns: Function
	//		A function that caches the passed query under the field "matches".  See any
	//		of the "query" methods on dojo.stores.
	//
	// example:
	//		Define a store with a reference to this engine, and set up a query method.
	//
	//	|	var myStore = function(options){
	//	|		//	...more properties here
	//	|		this.queryEngine = SimpleQueryEngine;
	//	|		//	define our query method
	//	|		this.query = function(query, options){
	//	|			return QueryResults(this.queryEngine(query, options)(this.data));
	//	|		};
	//	|	};

	// create our matching query function
	switch(typeof query){
		default:
			throw new Error("Can not query with a " + typeof query);
		case "object": case "undefined":
			var queryObject = query;
			query = function(object){
				for(var key in queryObject){
					var required = queryObject[key];
					if(required && required.test){
						// an object can provide a test method, which makes it work with regex
						if(!required.test(object[key], object)){
							return false;
						}
					}else if(required != object[key]){
						return false;
					}
				}
				return true;
			};
			break;
		case "string":
			// named query
			if(!this[query]){
				throw new Error("No filter function " + query + " was found in store");
			}
			query = this[query];
			// fall through
		case "function":
			// fall through
	}
	function execute(array){
		// execute the whole query, first we filter
		var results = arrayUtil.filter(array, query);
		// next we sort
		var sortSet = options && options.sort;
		if(sortSet){
			results.sort(typeof sortSet == "function" ? sortSet : function(a, b){
				for(var sort, i=0; sort = sortSet[i]; i++){
					var aValue = a[sort.attribute];
					var bValue = b[sort.attribute];
					if (aValue != bValue){
						return !!sort.descending == (aValue == null || aValue > bValue) ? -1 : 1;
					}
				}
				return 0;
			});
		}
		// now we paginate
		if(options && (options.start || options.count)){
			var total = results.length;
			results = results.slice(options.start || 0, (options.start || 0) + (options.count || Infinity));
			results.total = total;
		}
		return results;
	}
	execute.matches = query;
	return execute;
};

});

},
'dijit/form/_SearchMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.clone lang.hitch
	"dojo/query", // query
	"dojo/string", // string.substitute
	"dojo/when",
	"../registry"	// registry.byId
], function(declare, keys, lang, query, string, when, registry){

	// module:
	//		dijit/form/_SearchMixin


	return declare("dijit.form._SearchMixin", null, {
		// summary:
		//		A mixin that implements the base functionality to search a store based upon user-entered text such as
		//		with `dijit/form/ComboBox` or `dijit/form/FilteringSelect`
		// tags:
		//		protected

		// pageSize: Integer
		//		Argument to data provider.
		//		Specifies maximum number of search results to return per query
		pageSize: Infinity,

		// store: [const] dojo/store/api/Store
		//		Reference to data provider object used by this ComboBox.
		//		The store must accept an object hash of properties for its query. See `query` and `queryExpr` for details.
		store: null,

		// fetchProperties: Object
		//		Mixin to the store's fetch.
		//		For example, to set the sort order of the ComboBox menu, pass:
		//	|	{ sort: [{attribute:"name",descending: true}] }
		//		To override the default queryOptions so that deep=false, do:
		//	|	{ queryOptions: {ignoreCase: true, deep: false} }
		fetchProperties:{},

		// query: Object
		//		A query that can be passed to `store` to initially filter the items.
		//		ComboBox overwrites any reference to the `searchAttr` and sets it to the `queryExpr` with the user's input substituted.
		query: {},

		// searchDelay: Integer
		//		Delay in milliseconds between when user types something and we start
		//		searching based on that value
		searchDelay: 200,

		// searchAttr: String
		//		Search for items in the data store where this attribute (in the item)
		//		matches what the user typed
		searchAttr: "name",

		// queryExpr: String
		//		This specifies what query is sent to the data store,
		//		based on what the user has typed.  Changing this expression will modify
		//		whether the results are only exact matches, a "starting with" match,
		//		etc.
		//		`${0}` will be substituted for the user text.
		//		`*` is used for wildcards.
		//		`${0}*` means "starts with", `*${0}*` means "contains", `${0}` means "is"
		queryExpr: "${0}*",

		// ignoreCase: Boolean
		//		Set true if the query should ignore case when matching possible items
		ignoreCase: true,

		_patternToRegExp: function(pattern){
			// summary:
			//		Helper function to convert a simple pattern to a regular expression for matching.
			// description:
			//		Returns a regular expression object that conforms to the defined conversion rules.
			//		For example:
			//
			//		- ca*   -> /^ca.*$/
			//		- *ca*  -> /^.*ca.*$/
			//		- *c\*a*  -> /^.*c\*a.*$/
			//		- *c\*a?*  -> /^.*c\*a..*$/
			//
			//		and so on.
			// pattern: string
			//		A simple matching pattern to convert that follows basic rules:
			//
			//		- * Means match anything, so ca* means match anything starting with ca
			//		- ? Means match single character.  So, b?b will match to bob and bab, and so on.
			//		- \ is an escape character.  So for example, \* means do not treat * as a match, but literal character *.
			//
			//		To use a \ as a character in the string, it must be escaped.  So in the pattern it should be
			//		represented by \\ to be treated as an ordinary \ character instead of an escape.

			return new RegExp("^" + pattern.replace(/(\\.)|(\*)|(\?)|\W/g, function(str, literal, star, question){
				return star ? ".*" : question ? "." : literal ? literal : "\\" + str;
			}) + "$", this.ignoreCase ? "mi" : "m");
		},

		_abortQuery: function(){
			// stop in-progress query
			if(this.searchTimer){
				this.searchTimer = this.searchTimer.remove();
			}
			if(this._queryDeferHandle){
				this._queryDeferHandle = this._queryDeferHandle.remove();
			}
			if(this._fetchHandle){
				if(this._fetchHandle.abort){
					this._cancelingQuery = true;
					this._fetchHandle.abort();
					this._cancelingQuery = false;
				}
				if(this._fetchHandle.cancel){
					this._cancelingQuery = true;
					this._fetchHandle.cancel();
					this._cancelingQuery = false;
				}
				this._fetchHandle = null;
			}
		},

		_processInput: function(/*Event*/ evt){
			// summary:
			//		Handles input (keyboard/paste) events
			if(this.disabled || this.readOnly){ return; }
			var key = evt.charOrCode;

			// except for cutting/pasting case - ctrl + x/v
			if("type" in evt && evt.type.substring(0,3) == "key" && (evt.altKey || ((evt.ctrlKey || evt.metaKey) && (key != 'x' && key != 'v')) || key == keys.SHIFT)){
				return; // throw out weird key combinations and spurious events
			}

			var doSearch = false;
			this._prev_key_backspace = false;

			switch(key){
				case keys.DELETE:
				case keys.BACKSPACE:
					this._prev_key_backspace = true;
					this._maskValidSubsetError = true;
					doSearch = true;
					break;

				default:
					// Non char keys (F1-F12 etc..) shouldn't start a search..
					// Ascii characters and IME input (Chinese, Japanese etc.) should.
					//IME input produces keycode == 229.
					doSearch = typeof key == 'string' || key == 229;
			}
			if(doSearch){
				// need to wait a tad before start search so that the event
				// bubbles through DOM and we have value visible
				if(!this.store){
					this.onSearch();
				}else{
					this.searchTimer = this.defer("_startSearchFromInput", 1);
				}
			}
		},

		onSearch: function(/*===== results, query, options =====*/){
			// summary:
			//		Callback when a search completes.
			//
			// results: Object
			//		An array of items from the originating _SearchMixin's store.
			//
			// query: Object
			//		A copy of the originating _SearchMixin's query property.
			//
			// options: Object
			//		The additional parameters sent to the originating _SearchMixin's store, including: start, count, queryOptions.
			//
			// tags:
			//		callback
		},

		_startSearchFromInput: function(){
			this._startSearch(this.focusNode.value);
		},

		_startSearch: function(/*String*/ text){
			// summary:
			//		Starts a search for elements matching text (text=="" means to return all items),
			//		and calls onSearch(...) when the search completes, to display the results.

			this._abortQuery();
			var
				_this = this,
				// Setup parameters to be passed to store.query().
				// Create a new query to prevent accidentally querying for a hidden
				// value from FilteringSelect's keyField
				query = lang.clone(this.query), // #5970
				options = {
					start: 0,
					count: this.pageSize,
					queryOptions: {		// remove for 2.0
						ignoreCase: this.ignoreCase,
						deep: true
					}
				},
				qs = string.substitute(this.queryExpr, [text.replace(/([\\\*\?])/g, "\\$1")]),
				q,
				startQuery = function(){
					var resPromise = _this._fetchHandle = _this.store.query(query, options);
					if(_this.disabled || _this.readOnly || (q !== _this._lastQuery)){
						return;
					} // avoid getting unwanted notify
					when(resPromise, function(res){
						_this._fetchHandle = null;
						if(!_this.disabled && !_this.readOnly && (q === _this._lastQuery)){ // avoid getting unwanted notify
							when(resPromise.total, function(total){
								res.total = total;
								var pageSize = _this.pageSize;
								if(isNaN(pageSize) || pageSize > res.total){ pageSize = res.total; }
								// Setup method to fetching the next page of results
								res.nextPage = function(direction){
									//	tell callback the direction of the paging so the screen
									//	reader knows which menu option to shout
									options.direction = direction = direction !== false;
									options.count = pageSize;
									if(direction){
										options.start += res.length;
										if(options.start >= res.total){
											options.count = 0;
										}
									}else{
										options.start -= pageSize;
										if(options.start < 0){
											options.count = Math.max(pageSize + options.start, 0);
											options.start = 0;
										}
									}
									if(options.count <= 0){
										res.length = 0;
										_this.onSearch(res, query, options);
									}else{
										startQuery();
									}
								};
								_this.onSearch(res, query, options);
							});
						}
					}, function(err){
						_this._fetchHandle = null;
						if(!_this._cancelingQuery){	// don't treat canceled query as an error
							console.error(_this.declaredClass + ' ' + err.toString());
						}
					});
				};

			lang.mixin(options, this.fetchProperties);

			// Generate query
			if(this.store._oldAPI){
				// remove this branch for 2.0
				q = qs;
			}else{
				// Query on searchAttr is a regex for benefit of dojo/store/Memory,
				// but with a toString() method to help dojo/store/JsonRest.
				// Search string like "Co*" converted to regex like /^Co.*$/i.
				q = this._patternToRegExp(qs);
				q.toString = function(){ return qs; };
			}

			// set _lastQuery, *then* start the timeout
			// otherwise, if the user types and the last query returns before the timeout,
			// _lastQuery won't be set and their input gets rewritten
			this._lastQuery = query[this.searchAttr] = q;
			this._queryDeferHandle = this.defer(startQuery, this.searchDelay);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(){
			this.query={};
			this.fetchProperties={};
		},

		postMixInProperties: function(){
			if(!this.store){
				var list = this.list;
				if(list){
					this.store = registry.byId(list);
				}
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/_ComboBoxMenu':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-style", // domStyle.get
	"dojo/keys", // keys.DOWN_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.UP_ARROW
	"../_WidgetBase",
	"../_TemplatedMixin",
	"./_ComboBoxMenuMixin",
	"./_ListMouseMixin"
], function(declare, domClass, domStyle, keys,
			_WidgetBase, _TemplatedMixin, _ComboBoxMenuMixin, _ListMouseMixin){


	// module:
	//		dijit/form/_ComboBoxMenu

	return declare("dijit.form._ComboBoxMenu",[_WidgetBase, _TemplatedMixin, _ListMouseMixin, _ComboBoxMenuMixin], {
		// summary:
		//		Focus-less menu for internal use in `dijit/form/ComboBox`
		//		Abstract methods that must be defined externally:
		//
		//		- onChange: item was explicitly chosen (mousedown somewhere on the menu and mouseup somewhere on the menu)
		//		- onPage: next(1) or previous(-1) button pressed
		// tags:
		//		private

		// TODO for 2.0 or earlier: stop putting stuff inside this.containerNode.   Switch to using this.domNode
		// or a different attach point.    See _TemplatedMixin::searchContainerNode.
		templateString: "<div class='dijitReset dijitMenu' data-dojo-attach-point='containerNode' style='overflow: auto; overflow-x: hidden;' role='listbox'>"
				+"<div class='dijitMenuItem dijitMenuPreviousButton' data-dojo-attach-point='previousButton' role='option'></div>"
				+"<div class='dijitMenuItem dijitMenuNextButton' data-dojo-attach-point='nextButton' role='option'></div>"
				+"</div>",

		baseClass: "dijitComboBoxMenu",

		postCreate: function(){
			this.inherited(arguments);
			if(!this.isLeftToRight()){
				domClass.add(this.previousButton, "dijitMenuItemRtl");
				domClass.add(this.nextButton, "dijitMenuItemRtl");
			}
			this.containerNode.setAttribute("role","listbox");
		},

		_createMenuItem: function(){
			// note: not using domConstruct.create() because need to specify document
			var item = this.ownerDocument.createElement("div");
			item.className = "dijitReset dijitMenuItem" +(this.isLeftToRight() ? "" : " dijitMenuItemRtl");
			item.setAttribute("role", "option");
			return item;
		},

		onHover: function(/*DomNode*/ node){
			// summary:
			//		Add hover CSS
			domClass.add(node, "dijitMenuItemHover");
		},

		onUnhover: function(/*DomNode*/ node){
			// summary:
			//		Remove hover CSS
			domClass.remove(node, "dijitMenuItemHover");
		},

		onSelect: function(/*DomNode*/ node){
			// summary:
			//		Add selected CSS
			domClass.add(node, "dijitMenuItemSelected");
		},

		onDeselect: function(/*DomNode*/ node){
			// summary:
			//		Remove selected CSS
			domClass.remove(node, "dijitMenuItemSelected");
		},

		_page: function(/*Boolean*/ up){
			// summary:
			//		Handles page-up and page-down keypresses

			var scrollamount = 0;
			var oldscroll = this.domNode.scrollTop;
			var height = domStyle.get(this.domNode, "height");
			// if no item is highlighted, highlight the first option
			if(!this.getHighlightedOption()){
				this.selectNextNode();
			}
			while(scrollamount<height){
				var highlighted_option = this.getHighlightedOption();
				if(up){
					// stop at option 1
					if(!highlighted_option.previousSibling ||
						highlighted_option.previousSibling.style.display == "none"){
						break;
					}
					this.selectPreviousNode();
				}else{
					// stop at last option
					if(!highlighted_option.nextSibling ||
						highlighted_option.nextSibling.style.display == "none"){
						break;
					}
					this.selectNextNode();
				}
				// going backwards
				var newscroll = this.domNode.scrollTop;
				scrollamount += (newscroll-oldscroll)*(up ? -1:1);
				oldscroll = newscroll;
			}
		},

		handleKey: function(evt){
			// summary:
			//		Handle keystroke event forwarded from ComboBox, returning false if it's
			//		a keystroke I recognize and process, true otherwise.
			switch(evt.keyCode){
				case keys.DOWN_ARROW:
					this.selectNextNode();
					return false;
				case keys.PAGE_DOWN:
					this._page(false);
					return false;
				case keys.UP_ARROW:
					this.selectPreviousNode();
					return false;
				case keys.PAGE_UP:
					this._page(true);
					return false;
				default:
					return true;
			}
		}
	});
});

},
'dijit/form/_ComboBoxMenuMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/has",
	"dojo/i18n", // i18n.getLocalization
	"dojo/i18n!./nls/ComboBox"
], function(array, declare, domAttr, has, i18n){

	// module:
	//		dijit/form/_ComboBoxMenuMixin

	var ComboBoxMenuMixin = declare("dijit.form._ComboBoxMenuMixin" + (has("dojo-bidi") ? "_NoBidi" : ""), null, {
		// summary:
		//		Focus-less menu for internal use in `dijit/form/ComboBox`
		// tags:
		//		private

		// _messages: Object
		//		Holds "next" and "previous" text for paging buttons on drop down
		_messages: null,

		postMixInProperties: function(){
			this.inherited(arguments);
			this._messages = i18n.getLocalization("dijit.form", "ComboBox", this.lang);
		},

		buildRendering: function(){
			this.inherited(arguments);

			// fill in template with i18n messages
			this.previousButton.innerHTML = this._messages["previousMessage"];
			this.nextButton.innerHTML = this._messages["nextMessage"];
		},

		_setValueAttr: function(/*Object*/ value){
			this._set("value", value);
			this.onChange(value);
		},

		onClick: function(/*DomNode*/ node){
			if(node == this.previousButton){
				this._setSelectedAttr(null);
				this.onPage(-1);
			}else if(node == this.nextButton){
				this._setSelectedAttr(null);
				this.onPage(1);
			}else{
				this.onChange(node);
			}
		},

		// stubs
		onChange: function(/*Number*/ /*===== direction =====*/){
			// summary:
			//		Notifies ComboBox/FilteringSelect that user selected an option.
			// tags:
			//		callback
		},

		onPage: function(/*Number*/ /*===== direction =====*/){
			// summary:
			//		Notifies ComboBox/FilteringSelect that user clicked to advance to next/previous page.
			// tags:
			//		callback
		},

		onClose: function(){
			// summary:
			//		Callback from dijit.popup code to this widget, notifying it that it closed
			// tags:
			//		private
			this._setSelectedAttr(null);
		},

		_createOption: function(/*Object*/ item, labelFunc){
			// summary:
			//		Creates an option to appear on the popup menu subclassed by
			//		`dijit/form/FilteringSelect`.

			var menuitem = this._createMenuItem();
			var labelObject = labelFunc(item);
			if(labelObject.html){
				menuitem.innerHTML = labelObject.label;
			}else{
				menuitem.appendChild(
					menuitem.ownerDocument.createTextNode(labelObject.label)
				);
			}
			// #3250: in blank options, assign a normal height
			if(menuitem.innerHTML == ""){
				menuitem.innerHTML = "&#160;";	// &nbsp;
			}

			return menuitem;
		},

		createOptions: function(results, options, labelFunc){
			// summary:
			//		Fills in the items in the drop down list
			// results:
			//		Array of items
			// options:
			//		The options to the query function of the store
			//
			// labelFunc:
			//		Function to produce a label in the drop down list from a dojo.data item

			this.items = results;

			// display "Previous . . ." button
			this.previousButton.style.display = (options.start == 0) ? "none" : "";
			domAttr.set(this.previousButton, "id", this.id + "_prev");
			// create options using _createOption function defined by parent
			// ComboBox (or FilteringSelect) class
			// #2309:
			//		iterate over cache nondestructively
			array.forEach(results, function(item, i){
				var menuitem = this._createOption(item, labelFunc);
				menuitem.setAttribute("item", i);	// index to this.items; use indirection to avoid mem leak
				domAttr.set(menuitem, "id", this.id + i);
				this.nextButton.parentNode.insertBefore(menuitem, this.nextButton);
			}, this);
			// display "Next . . ." button
			var displayMore = false;
			// Try to determine if we should show 'more'...
			if(results.total && !results.total.then && results.total != -1){
				if((options.start + options.count) < results.total){
					displayMore = true;
				}else if((options.start + options.count) > results.total && options.count == results.length){
					// Weird return from a data store, where a start + count > maxOptions
					// implies maxOptions isn't really valid and we have to go into faking it.
					// And more or less assume more if count == results.length
					displayMore = true;
				}
			}else if(options.count == results.length){
				//Don't know the size, so we do the best we can based off count alone.
				//So, if we have an exact match to count, assume more.
				displayMore = true;
			}

			this.nextButton.style.display = displayMore ? "" : "none";
			domAttr.set(this.nextButton, "id", this.id + "_next");
		},

		clearResultList: function(){
			// summary:
			//		Clears the entries in the drop down list, but of course keeps the previous and next buttons.
			var container = this.containerNode;
			while(container.childNodes.length > 2){
				container.removeChild(container.childNodes[container.childNodes.length - 2]);
			}
			this._setSelectedAttr(null);
		},

		highlightFirstOption: function(){
			// summary:
			//		Highlight the first real item in the list (not Previous Choices).
			this.selectFirstNode();
		},

		highlightLastOption: function(){
			// summary:
			//		Highlight the last real item in the list (not More Choices).
			this.selectLastNode();
		},

		selectFirstNode: function(){
			this.inherited(arguments);
			if(this.getHighlightedOption() == this.previousButton){
				this.selectNextNode();
			}
		},

		selectLastNode: function(){
			this.inherited(arguments);
			if(this.getHighlightedOption() == this.nextButton){
				this.selectPreviousNode();
			}
		},

		getHighlightedOption: function(){
			return this.selected;
		}
	});

	if(has("dojo-bidi")){
		ComboBoxMenuMixin = declare("dijit.form._ComboBoxMenuMixin", ComboBoxMenuMixin, {
			_createOption: function(){
				var menuitem = this.inherited(arguments);

				// update menuitem.dir if BidiSupport was required
				this.applyTextDir(menuitem);

				return menuitem;
			}
		});
	}

	return ComboBoxMenuMixin;
});

},
'dijit/form/_ListMouseMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/on",
	"dojo/touch",
	"./_ListBase"
], function(declare, on, touch, _ListBase){

	// module:
	//		dijit/form/_ListMouseMixin

	return declare("dijit.form._ListMouseMixin", _ListBase, {
		// summary:
		//		A mixin to handle mouse or touch events for a focus-less menu
		//		Abstract methods that must be defined externally:
		//
		//		- onClick: item was chosen (mousedown somewhere on the menu and mouseup somewhere on the menu)
		// tags:
		//		private

		postCreate: function(){
			this.inherited(arguments);

			// Add flag to use normalized click handling from dojo/touch
			this.domNode.dojoClick = true;

			this.own(on(this.domNode, "mousedown", function(evt){
				evt.preventDefault();
			})); // prevent focus shift on list scrollbar press
			this._listConnect("click", "_onClick");
			this._listConnect(touch.press, "_onMouseDown");
			this._listConnect(touch.release, "_onMouseUp");
			this._listConnect(touch.over, "_onMouseOver");
			this._listConnect(touch.out, "_onMouseOut");
		},

		_onClick: function(/*Event*/ evt, /*DomNode*/ target){
			this._setSelectedAttr(target);
			if(this._deferredClick){
				this._deferredClick.remove();
			}
			this._deferredClick = this.defer(function(){
				this._deferredClick = null;
				this.onClick(target);
			});
		},

		_onMouseDown: function(/*Event*/ evt, /*DomNode*/ target){
			if(this._hoveredNode){
				this.onUnhover(this._hoveredNode);
				this._hoveredNode = null;
			}
			this._isDragging = true;
			this._setSelectedAttr(target);
		},

		_onMouseUp: function(/*Event*/ evt, /*DomNode*/ target){
			this._isDragging = false;
			var selectedNode = this.selected;
			var hoveredNode = this._hoveredNode;
			if(selectedNode && target == selectedNode){
				this.defer(function(){
					this._onClick(evt, selectedNode);
				});
			}else if(hoveredNode){ // drag to select
				this.defer(function(){
					this._onClick(evt, hoveredNode);
				});
			}
		},

		_onMouseOut: function(/*Event*/ evt, /*DomNode*/ target){
			if(this._hoveredNode){
				this.onUnhover(this._hoveredNode);
				this._hoveredNode = null;
			}
			if(this._isDragging){
				this._cancelDrag = (new Date()).getTime() + 1000; // cancel in 1 second if no _onMouseOver fires
			}
		},

		_onMouseOver: function(/*Event*/ evt, /*DomNode*/ target){
			if(this._cancelDrag){
				var time = (new Date()).getTime();
				if(time > this._cancelDrag){
					this._isDragging = false;
				}
				this._cancelDrag = null;
			}
			this._hoveredNode = target;
			this.onHover(target);
			if(this._isDragging){
				this._setSelectedAttr(target);
			}
		}
	});
});

},
'dijit/form/_ListBase':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/on",
	"dojo/window" // winUtils.scrollIntoView
], function(declare, on, winUtils){

	// module:
	//		dijit/form/_ListBase

	return declare("dijit.form._ListBase", null, {
		// summary:
		//		Focus-less menu to handle UI events consistently.
		//		Abstract methods that must be defined externally:
		//
		//		- onSelect: item is active (mousedown but not yet mouseup, or keyboard arrow selected but no Enter)
		//		- onDeselect:  cancels onSelect
		// tags:
		//		private

		// selected: DOMNode
		//		currently selected node
		selected: null,

		_listConnect: function(/*String|Function*/ eventType, /*String*/ callbackFuncName){
			// summary:
			//		Connects 'containerNode' to specified method of this object
			//		and automatically registers for 'disconnect' on widget destroy.
			// description:
			//		Provide widget-specific analog to 'connect'.
			//		The callback function is called with the normal event object,
			//		but also a second parameter is passed that indicates which list item
			//		actually received the event.
			// returns:
			//		A handle that can be passed to `disconnect` in order to disconnect
			//		before the widget is destroyed.
			// tags:
			//		private

			var self = this;
			return self.own(on(self.containerNode,
				on.selector(
					function(eventTarget, selector, target){
						return eventTarget.parentNode == target;
					},
					eventType
				),
				function(evt){
					evt.preventDefault();
					self[callbackFuncName](evt, this);
				}
			));
		},

		selectFirstNode: function(){
			// summary:
			//		Select the first displayed item in the list.
			var first = this.containerNode.firstChild;
			while(first && first.style.display == "none"){
				first = first.nextSibling;
			}
			this._setSelectedAttr(first);
		},

		selectLastNode: function(){
			// summary:
			//		Select the last displayed item in the list
			var last = this.containerNode.lastChild;
			while(last && last.style.display == "none"){
				last = last.previousSibling;
			}
			this._setSelectedAttr(last);
		},

		selectNextNode: function(){
			// summary:
			//		Select the item just below the current selection.
			//		If nothing selected, select first node.
			var selectedNode = this.selected;
			if(!selectedNode){
				this.selectFirstNode();
			}else{
				var next = selectedNode.nextSibling;
				while(next && next.style.display == "none"){
					next = next.nextSibling;
				}
				if(!next){
					this.selectFirstNode();
				}else{
					this._setSelectedAttr(next);
				}
			}
		},

		selectPreviousNode: function(){
			// summary:
			//		Select the item just above the current selection.
			//		If nothing selected, select last node (if
			//		you select Previous and try to keep scrolling up the list).
			var selectedNode = this.selected;
			if(!selectedNode){
				this.selectLastNode();
			}else{
				var prev = selectedNode.previousSibling;
				while(prev && prev.style.display == "none"){
					prev = prev.previousSibling;
				}
				if(!prev){
					this.selectLastNode();
				}else{
					this._setSelectedAttr(prev);
				}
			}
		},

		_setSelectedAttr: function(/*DomNode*/ node){
			// summary:
			//		Does the actual select.
			if(this.selected != node){
				var selectedNode = this.selected;
				if(selectedNode){
					this.onDeselect(selectedNode);
				}
				if(node){
					winUtils.scrollIntoView(node);
					this.onSelect(node);
				}
				this._set("selected", node);
			}else if(node){
				this.onSelect(node);
			}
		}
	});
});

},
'dijit/_HasDropDown':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred",
	"dojo/dom", // dom.isDescendant
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-class", // domClass.add domClass.contains domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/has", // has("touch")
	"dojo/keys", // keys.DOWN_ARROW keys.ENTER keys.ESCAPE
	"dojo/_base/lang", // lang.hitch lang.isFunction
	"dojo/on",
	"dojo/touch",
	"./registry", // registry.byNode()
	"./focus",
	"./popup",
	"./_FocusMixin"
], function(declare, Deferred, dom, domAttr, domClass, domGeometry, domStyle, has, keys, lang, on, touch,
			registry, focus, popup, _FocusMixin){


	// module:
	//		dijit/_HasDropDown

	return declare("dijit._HasDropDown", _FocusMixin, {
		// summary:
		//		Mixin for widgets that need drop down ability.

		// _buttonNode: [protected] DomNode
		//		The button/icon/node to click to display the drop down.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then either focusNode or domNode (if focusNode is also missing) will be used.
		_buttonNode: null,

		// _arrowWrapperNode: [protected] DomNode
		//		Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending
		//		on where the drop down is set to be positioned.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then _buttonNode will be used.
		_arrowWrapperNode: null,

		// _popupStateNode: [protected] DomNode
		//		The node to set the aria-expanded class on.
		//		Also sets popupActive class but that will be removed in 2.0.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then focusNode or _buttonNode (if focusNode is missing) will be used.
		_popupStateNode: null,

		// _aroundNode: [protected] DomNode
		//		The node to display the popup around.
		//		Can be set via a data-dojo-attach-point assignment.
		//		If missing, then domNode will be used.
		_aroundNode: null,

		// dropDown: [protected] Widget
		//		The widget to display as a popup.  This widget *must* be
		//		defined before the startup function is called.
		dropDown: null,

		// autoWidth: [protected] Boolean
		//		Set to true to make the drop down at least as wide as this
		//		widget.  Set to false if the drop down should just be its
		//		default width.
		autoWidth: true,

		// forceWidth: [protected] Boolean
		//		Set to true to make the drop down exactly as wide as this
		//		widget.  Overrides autoWidth.
		forceWidth: false,

		// maxHeight: [protected] Integer
		//		The max height for our dropdown.
		//		Any dropdown taller than this will have scrollbars.
		//		Set to 0 for no max height, or -1 to limit height to available space in viewport
		maxHeight: -1,

		// dropDownPosition: [const] String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//		- before: places drop down to the left of the target node/widget, or to the right in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- after: places drop down to the right of the target node/widget, or to the left in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- above: drop down goes above target node
		//		- below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		dropDownPosition: ["below", "above"],

		// _stopClickEvents: Boolean
		//		When set to false, the click events will not be stopped, in
		//		case you want to use them in your subclass
		_stopClickEvents: true,

		_onDropDownMouseDown: function(/*Event*/ e){
			// summary:
			//		Callback when the user mousedown/touchstart on the arrow icon.
			if(this.disabled || this.readOnly){
				return;
			}

			// Prevent default to stop things like text selection, but don't stop propagation, so that:
			//		1. TimeTextBox etc. can focus the <input> on mousedown
			//		2. dropDownButtonActive class applied by _CssStateMixin (on button depress)
			//		3. user defined onMouseDown handler fires

            // Fixme: The next line of code was commented out in in SAS VSD's d4vsm63 branch in order to fix this defect:
            // Dojo 1.9.1 defect: DropDown and Combo buttons do not close reliably in IE10" : https://bugs.dojotoolkit.org/ticket/17262
            // Fixme: This file will only be used for d4vsm63. We need to upgrade to at least 1.9.2 for d4vsm63 M1 release.
            //e.preventDefault();

			this._docHandler = this.own(on(this.ownerDocument, touch.release, lang.hitch(this, "_onDropDownMouseUp")))[0];

			this.toggleDropDown();
		},

		_onDropDownMouseUp: function(/*Event?*/ e){
			// summary:
			//		Callback on mouseup/touchend after mousedown/touchstart on the arrow icon.
			//		Note that this function is called regardless of what node the event occurred on (but only after
			//		a mousedown/touchstart on the arrow).
			//
			//		If the drop down is a simple menu and the cursor is over the menu, we execute it, otherwise, we focus our
			//		drop down widget.  If the event is missing, then we are not
			//		a mouseup event.
			//
			//		This is useful for the common mouse movement pattern
			//		with native browser `<select>` nodes:
			//
			//		1. mouse down on the select node (probably on the arrow)
			//		2. move mouse to a menu item while holding down the mouse button
			//		3. mouse up.  this selects the menu item as though the user had clicked it.
			if(e && this._docHandler){
				this._docHandler.remove();
				this._docHandler = null;
			}
			var dropDown = this.dropDown, overMenu = false;

			if(e && this._opened){
				// This code deals with the corner-case when the drop down covers the original widget,
				// because it's so large.  In that case mouse-up shouldn't select a value from the menu.
				// Find out if our target is somewhere in our dropdown widget,
				// but not over our _buttonNode (the clickable node)
				var c = domGeometry.position(this._buttonNode, true);
				if(!(e.pageX >= c.x && e.pageX <= c.x + c.w) || !(e.pageY >= c.y && e.pageY <= c.y + c.h)){
					var t = e.target;
					while(t && !overMenu){
						if(domClass.contains(t, "dijitPopup")){
							overMenu = true;
						}else{
							t = t.parentNode;
						}
					}
					if(overMenu){
						t = e.target;
						if(dropDown.onItemClick){
							var menuItem;
							while(t && !(menuItem = registry.byNode(t))){
								t = t.parentNode;
							}
							if(menuItem && menuItem.onClick && menuItem.getParent){
								menuItem.getParent().onItemClick(menuItem, e);
							}
						}
						return;
					}
				}
			}
			if(this._opened){
				if(dropDown.focus && dropDown.autoFocus !== false){
					// Focus the dropdown widget - do it on a delay so that we
					// don't steal back focus from the dropdown.
					this._focusDropDownTimer = this.defer(function(){
						dropDown.focus();
						delete this._focusDropDownTimer;
					});
				}
			}else{
				// The drop down arrow icon probably can't receive focus, but widget itself should get focus.
				// defer() needed to make it work on IE (test DateTextBox)
				if(this.focus){
					this.defer("focus");
				}
			}
		},

		_onDropDownClick: function(/*Event*/ e){
			// The drop down was already opened on mousedown/keydown; just need to stop the event
			if(this._stopClickEvents){
				e.stopPropagation();
				e.preventDefault();
			}
		},

		buildRendering: function(){
			this.inherited(arguments);

			this._buttonNode = this._buttonNode || this.focusNode || this.domNode;
			this._popupStateNode = this._popupStateNode || this.focusNode || this._buttonNode;

			// Add a class to the "dijitDownArrowButton" type class to _buttonNode so theme can set direction of arrow
			// based on where drop down will normally appear
			var defaultPos = {
				"after": this.isLeftToRight() ? "Right" : "Left",
				"before": this.isLeftToRight() ? "Left" : "Right",
				"above": "Up",
				"below": "Down",
				"left": "Left",
				"right": "Right"
			}[this.dropDownPosition[0]] || this.dropDownPosition[0] || "Down";
			domClass.add(this._arrowWrapperNode || this._buttonNode, "dijit" + defaultPos + "ArrowButton");
		},

		postCreate: function(){
			// summary:
			//		set up nodes and connect our mouse and keyboard events

			this.inherited(arguments);

			var keyboardEventNode = this.focusNode || this.domNode;
			this.own(
				on(this._buttonNode, touch.press, lang.hitch(this, "_onDropDownMouseDown")),
				on(this._buttonNode, "click", lang.hitch(this, "_onDropDownClick")),
				on(keyboardEventNode, "keydown", lang.hitch(this, "_onKey")),
				on(keyboardEventNode, "keyup", lang.hitch(this, "_onKeyUp"))
			);
		},

		destroy: function(){
			if(this.dropDown){
				// Destroy the drop down, unless it's already been destroyed.  This can happen because
				// the drop down is a direct child of <body> even though it's logically my child.
				if(!this.dropDown._destroyed){
					this.dropDown.destroyRecursive();
				}
				delete this.dropDown;
			}
			this.inherited(arguments);
		},

		_onKey: function(/*Event*/ e){
			// summary:
			//		Callback when the user presses a key while focused on the button node

			if(this.disabled || this.readOnly){
				return;
			}
			var d = this.dropDown, target = e.target;
			if(d && this._opened && d.handleKey){
				if(d.handleKey(e) === false){
					/* false return code means that the drop down handled the key */
					e.stopPropagation();
					e.preventDefault();
					return;
				}
			}
			if(d && this._opened && e.keyCode == keys.ESCAPE){
				this.closeDropDown();
				e.stopPropagation();
				e.preventDefault();
			}else if(!this._opened &&
				(e.keyCode == keys.DOWN_ARROW ||
					// ignore unmodified SPACE if _KeyNavMixin has active searching in progress
					( (e.keyCode == keys.ENTER || (e.keyCode == keys.SPACE && (!this._searchTimer || (e.ctrlKey || e.altKey || e.metaKey)))) &&
						//ignore enter and space if the event is for a text input
						((target.tagName || "").toLowerCase() !== 'input' ||
							(target.type && target.type.toLowerCase() !== 'text'))))){
				// Toggle the drop down, but wait until keyup so that the drop down doesn't
				// get a stray keyup event, or in the case of key-repeat (because user held
				// down key for too long), stray keydown events
				this._toggleOnKeyUp = true;
				e.stopPropagation();
				e.preventDefault();
			}
		},

		_onKeyUp: function(){
			if(this._toggleOnKeyUp){
				delete this._toggleOnKeyUp;
				this.toggleDropDown();
				var d = this.dropDown;	// drop down may not exist until toggleDropDown() call
				if(d && d.focus){
					this.defer(lang.hitch(d, "focus"), 1);
				}
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown

			// Close dropdown but don't focus my <input>.  User may have focused somewhere else (ex: clicked another
			// input), and even if they just clicked a blank area of the screen, focusing my <input> will unwantedly
			// popup the keyboard on mobile.
			this.closeDropDown(false);

			this.inherited(arguments);
		},

		isLoaded: function(){
			// summary:
			//		Returns true if the dropdown exists and it's data is loaded.  This can
			//		be overridden in order to force a call to loadDropDown().
			// tags:
			//		protected

			return true;
		},

		loadDropDown: function(/*Function*/ loadCallback){
			// summary:
			//		Creates the drop down if it doesn't exist, loads the data
			//		if there's an href and it hasn't been loaded yet, and then calls
			//		the given callback.
			// tags:
			//		protected

			// TODO: for 2.0, change API to return a Deferred, instead of calling loadCallback?
			loadCallback();
		},

		loadAndOpenDropDown: function(){
			// summary:
			//		Creates the drop down if it doesn't exist, loads the data
			//		if there's an href and it hasn't been loaded yet, and
			//		then opens the drop down.  This is basically a callback when the
			//		user presses the down arrow button to open the drop down.
			// returns: Deferred
			//		Deferred for the drop down widget that
			//		fires when drop down is created and loaded
			// tags:
			//		protected
			var d = new Deferred(),
				afterLoad = lang.hitch(this, function(){
					this.openDropDown();
					d.resolve(this.dropDown);
				});
			if(!this.isLoaded()){
				this.loadDropDown(afterLoad);
			}else{
				afterLoad();
			}
			return d;
		},

		toggleDropDown: function(){
			// summary:
			//		Callback when the user presses the down arrow button or presses
			//		the down arrow key to open/close the drop down.
			//		Toggle the drop-down widget; if it is up, close it, if not, open it
			// tags:
			//		protected

			if(this.disabled || this.readOnly){
				return;
			}
			if(!this._opened){
				this.loadAndOpenDropDown();
			}else{
				this.closeDropDown(true);	// refocus button to avoid hiding node w/focus
			}
		},

		openDropDown: function(){
			// summary:
			//		Opens the dropdown for this widget.   To be called only when this.dropDown
			//		has been created and is ready to display (ie, it's data is loaded).
			// returns:
			//		return value of dijit/popup.open()
			// tags:
			//		protected

			var dropDown = this.dropDown,
				ddNode = dropDown.domNode,
				aroundNode = this._aroundNode || this.domNode,
				self = this;

			var retVal = popup.open({
				parent: this,
				popup: dropDown,
				around: aroundNode,
				orient: this.dropDownPosition,
				maxHeight: this.maxHeight,
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					domAttr.set(self._popupStateNode, "popupActive", false);
					domClass.remove(self._popupStateNode, "dijitHasDropDownOpen");
					self._set("_opened", false);	// use set() because _CssStateMixin is watching
				}
			});

			// Set width of drop down if necessary, so that dropdown width + width of scrollbar (from popup wrapper)
			// matches width of aroundNode
			if(this.forceWidth || (this.autoWidth && aroundNode.offsetWidth > dropDown._popupWrapper.offsetWidth)){
				var resizeArgs = {
					w: aroundNode.offsetWidth - (dropDown._popupWrapper.offsetWidth - dropDown.domNode.offsetWidth)
				};
				if(lang.isFunction(dropDown.resize)){
					dropDown.resize(resizeArgs);
				}else{
					domGeometry.setMarginBox(ddNode, resizeArgs);
				}
			}

			domAttr.set(this._popupStateNode, "popupActive", "true");
			domClass.add(this._popupStateNode, "dijitHasDropDownOpen");
			this._set("_opened", true);	// use set() because _CssStateMixin is watching

			this._popupStateNode.setAttribute("aria-expanded", "true");
			this._popupStateNode.setAttribute("aria-owns", dropDown.id);

			// Set aria-labelledby on dropdown if it's not already set to something more meaningful
			if(ddNode.getAttribute("role") !== "presentation" && !ddNode.getAttribute("aria-labelledby")){
				ddNode.setAttribute("aria-labelledby", this.id);
			}

			return retVal;
		},

		closeDropDown: function(/*Boolean*/ focus){
			// summary:
			//		Closes the drop down on this widget
			// focus:
			//		If true, refocuses the button widget
			// tags:
			//		protected

			if(this._focusDropDownTimer){
				this._focusDropDownTimer.remove();
				delete this._focusDropDownTimer;
			}

			if(this._opened){
				this._popupStateNode.setAttribute("aria-expanded", "false");
				if(focus){
					this.focus();
				}
				popup.close(this.dropDown);
				this._opened = false;
			}
		}

	});
});

},
'dijit/form/ComboBox':function(){
define([
	"dojo/_base/declare", // declare
	"./ValidationTextBox",
	"./ComboBoxMixin"
], function(declare, ValidationTextBox, ComboBoxMixin){

	// module:
	//		dijit/form/ComboBox

	return declare("dijit.form.ComboBox", [ValidationTextBox, ComboBoxMixin], {
		// summary:
		//		Auto-completing text box
		//
		// description:
		//		The drop down box's values are populated from an class called
		//		a data provider, which returns a list of values based on the characters
		//		that the user has typed into the input box.
		//		If OPTION tags are used as the data provider via markup,
		//		then the OPTION tag's child text node is used as the widget value
		//		when selected.  The OPTION tag's value attribute is ignored.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Some of the options to the ComboBox are actually arguments to the data
		//		provider.
	});
});

},
'dijit/form/MultiSelect':function(){
define([
	"dojo/_base/array", // array.indexOf, array.map
	"dojo/_base/declare", // declare
	"dojo/dom-geometry", // domGeometry.setMarginBox
	"dojo/has",
	"dojo/query", // query
	"./_FormValueWidget"
], function(array, declare, domGeometry, has, query, _FormValueWidget){

	// module:
	//		dijit/form/MultiSelect

	var MultiSelect = declare("dijit.form.MultiSelect" + (has("dojo-bidi") ? "_NoBidi" : ""), _FormValueWidget, {
		// summary:
		//		Widget version of a `<select multiple=true>` element,
		//		for selecting multiple options.

		// size: Number
		//		Number of elements to display on a page
		//		NOTE: may be removed in version 2.0, since elements may have variable height;
		//		set the size via style="..." or CSS class names instead.
		size: 7,

		baseClass: "dijitMultiSelect",

		templateString: "<select multiple='true' ${!nameAttrSetting} data-dojo-attach-point='containerNode,focusNode' data-dojo-attach-event='onchange: _onChange'></select>",

		addSelected: function(/*dijit/form/MultiSelect*/ select){
			// summary:
			//		Move the selected nodes of a passed Select widget
			//		instance to this Select widget.
			//
			// example:
			// |	// move all the selected values from "bar" to "foo"
			// |	dijit.byId("foo").addSelected(dijit.byId("bar"));

			select.getSelected().forEach(function(n){
				this.containerNode.appendChild(n);
				// scroll to bottom to see item
				// cannot use scrollIntoView since <option> tags don't support all attributes
				// does not work on IE due to a bug where <select> always shows scrollTop = 0
				this.domNode.scrollTop = this.domNode.offsetHeight; // overshoot will be ignored
				// scrolling the source select is trickier esp. on safari who forgets to change the scrollbar size
				var oldscroll = select.domNode.scrollTop;
				select.domNode.scrollTop = 0;
				select.domNode.scrollTop = oldscroll;
			}, this);
			this._set('value', this.get('value'));
		},

		getSelected: function(){
			// summary:
			//		Access the NodeList of the selected options directly
			return query("option", this.containerNode).filter(function(n){
				return n.selected; // Boolean
			}); // dojo/NodeList
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			// description:
			//		Returns an array of the selected options' values.

			// Don't call getSelect.map() because it doesn't return a real array,
			// and that messes up dojo.toJson() calls like in the Form.html test
			return array.map(this.getSelected(), function(n){
				return n.value;
			});
		},

		multiple: true, // for Form

		_setValueAttr: function(/*Array*/ values, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', values) works.
			// description:
			//		Set the value(s) of this Select based on passed values
			query("option", this.containerNode).forEach(function(n){
				n.selected = (array.indexOf(values, n.value) != -1);
			});
			this.inherited(arguments);
		},

		invertSelection: function(/*Boolean?*/ onChange){
			// summary:
			//		Invert the selection
			// onChange: Boolean
			//		If false, onChange is not fired.
			var val = [];
			query("option", this.containerNode).forEach(function(n){
				if(!n.selected){
					val.push(n.value);
				}
			});
			this._setValueAttr(val, !(onChange === false || onChange == null));
		},

		_onChange: function(/*Event*/){
			this._handleOnChange(this.get('value'), true);
		},

		// for layout widgets:
		resize: function(/*Object*/ size){
			if(size){
				domGeometry.setMarginBox(this.domNode, size);
			}
		},

		postCreate: function(){
			this._set('value', this.get('value'));
			this.inherited(arguments);
		}
	});

	if(has("dojo-bidi")){
		MultiSelect = declare("dijit.form.MultiSelect", MultiSelect, {
			addSelected: function(/*dijit/form/MultiSelect*/ select){
				select.getSelected().forEach(function(n){
					n.text = this.enforceTextDirWithUcc(this.restoreOriginalText(n), n.text);
				}, this);
				this.inherited(arguments);
			},

			_setTextDirAttr: function(textDir){
				// to insure the code executed only when _BidiSupport loaded, and only
				// when there was a change in textDir
				if((this.textDir != textDir || !this._created) && this.enforceTextDirWithUcc){
					this._set("textDir", textDir);

					query("option", this.containerNode).forEach(function(option){
						// If the value wasn't defined explicitly, it the same object as
						// option.text. Since the option.text will be modified (by wrapping of UCC)
						// we want to save the original option.value for form submission.
						if(!this._created && option.value === option.text){
							option.value = option.text;
						}
						// apply the bidi support
						option.text = this.enforceTextDirWithUcc(option, option.originalText || option.text);
					}, this);
				}
			}
		});
	}

	return MultiSelect;
});

},
'dijit/form/DateTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"../Calendar",
	"./_DateTimeTextBox"
], function(declare, Calendar, _DateTimeTextBox){

	// module:
	//		dijit/form/DateTextBox

	return declare("dijit.form.DateTextBox", _DateTimeTextBox, {
		// summary:
		//		A validating, serializable, range-bound date text box with a drop down calendar
		// example:
		// |	new DateTextBox({value: new Date(2009, 0, 20)})
		// example:
		// |	<input data-dojo-type='dijit/form/DateTextBox' value='2009-01-20'>

		baseClass: "dijitTextBox dijitComboBox dijitDateTextBox",
		popupClass: Calendar,
		_selector: "date",

		// Prevent scrollbar on Calendar dropdown.  On iPad it often gets a scrollbar unnecessarily because Viewport
		// thinks the keyboard is showing.  Even if the keyboard is showing, it disappears when the calendar gets focus.
		maxHeight: Infinity,

		// value: Date
		//		The value of this widget as a JavaScript Date object, with only year/month/day specified.
		//		If specified in markup, use the format specified in `stamp.fromISOString`.
		//		set("value", ...) accepts either a Date object or a string.
		value: new Date("")	// value.toString()="NaN"
	});
});

},
'dijit/Calendar':function(){
define([
	"dojo/_base/array", // array.map
	"dojo/date",
	"dojo/date/locale",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.get
	"dojo/dom-class", // domClass.add domClass.contains domClass.remove domClass.toggle
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie")
	"./CalendarLite",
	"./_Widget",
	"./_CssStateMixin",
	"./_TemplatedMixin",
	"./form/DropDownButton"
], function(array, date, local, declare, domAttr, domClass, kernel, keys, lang, on, has, CalendarLite, _Widget, _CssStateMixin, _TemplatedMixin, DropDownButton){

	// module:
	//		dijit/Calendar

	// _Widget for deprecated methods like setAttribute()
	var Calendar = declare("dijit.Calendar", [CalendarLite, _Widget, _CssStateMixin], {
		// summary:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//
		// description:
		//		See CalendarLite for general description.   Calendar extends CalendarLite, adding:
		//
		//		- month drop down list
		//		- keyboard navigation
		//		- CSS classes for hover/mousepress on date, month, and year nodes
		//		- support of deprecated methods (will be removed in 2.0)

		// Set node classes for various mouse events, see dijit._CssStateMixin for more details
		cssStateNodes: {
			"decrementMonth": "dijitCalendarArrow",
			"incrementMonth": "dijitCalendarArrow",
			"previousYearLabelNode": "dijitCalendarPreviousYear",
			"nextYearLabelNode": "dijitCalendarNextYear"
		},

		setValue: function(/*Date*/ value){
			// summary:
			//		Deprecated.   Use set('value', ...) instead.
			// tags:
			//		deprecated
			kernel.deprecated("dijit.Calendar:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_createMonthWidget: function(){
			// summary:
			//		Creates the drop down button that displays the current month and lets user pick a new one

			return new Calendar._MonthDropDownButton({
				id: this.id + "_mddb",
				tabIndex: -1,
				onMonthSelect: lang.hitch(this, "_onMonthSelect"),
				lang: this.lang,
				dateLocaleModule: this.dateLocaleModule
			}, this.monthNode);
		},

		postCreate: function(){
			this.inherited(arguments);

			// Events specific to Calendar, not used in CalendarLite
			this.own(
				on(this.domNode, "keydown", lang.hitch(this, "_onKeyDown")),
				on(this.dateRowsNode, "mouseover", lang.hitch(this, "_onDayMouseOver")),
				on(this.dateRowsNode, "mouseout", lang.hitch(this, "_onDayMouseOut")),
				on(this.dateRowsNode, "mousedown", lang.hitch(this, "_onDayMouseDown")),
				on(this.dateRowsNode, "mouseup", lang.hitch(this, "_onDayMouseUp"))
			);
		},

		_onMonthSelect: function(/*Number*/ newMonth){
			// summary:
			//		Handler for when user selects a month from the drop down list
			// tags:
			//		protected

			// move to selected month, bounding by the number of days in the month
			// (ex: jan 31 --> feb 28, not feb 31)
			var date = new this.dateClassObj(this.currentFocus);
			date.setDate(1);
			date.setMonth(newMonth);
			var daysInMonth = this.dateModule.getDaysInMonth(date);
			var currentDate = this.currentFocus.getDate();
			date.setDate(Math.min(currentDate, daysInMonth));
			this._setCurrentFocusAttr(date);
		},

		_onDayMouseOver: function(/*Event*/ evt){
			// summary:
			//		Handler for mouse over events on days, sets hovered style
			// tags:
			//		protected

			// event can occur on <td> or the <span> inside the td,
			// set node to the <td>.
			var node =
				domClass.contains(evt.target, "dijitCalendarDateLabel") ?
					evt.target.parentNode :
					evt.target;

			if(node && (
				(node.dijitDateValue && !domClass.contains(node, "dijitCalendarDisabledDate"))
					|| node == this.previousYearLabelNode || node == this.nextYearLabelNode
				)){
				domClass.add(node, "dijitCalendarHoveredDate");
				this._currentNode = node;
			}
		},

		_onDayMouseOut: function(/*Event*/ evt){
			// summary:
			//		Handler for mouse out events on days, clears hovered style
			// tags:
			//		protected

			if(!this._currentNode){
				return;
			}

			// if mouse out occurs moving from <td> to <span> inside <td>, ignore it
			if(evt.relatedTarget && evt.relatedTarget.parentNode == this._currentNode){
				return;
			}
			var cls = "dijitCalendarHoveredDate";
			if(domClass.contains(this._currentNode, "dijitCalendarActiveDate")){
				cls += " dijitCalendarActiveDate";
			}
			domClass.remove(this._currentNode, cls);
			this._currentNode = null;
		},

		_onDayMouseDown: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue && !domClass.contains(node, "dijitCalendarDisabledDate")){
				domClass.add(node, "dijitCalendarActiveDate");
				this._currentNode = node;
			}
		},

		_onDayMouseUp: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue){
				domClass.remove(node, "dijitCalendarActiveDate");
			}
		},

		handleKey: function(/*Event*/ evt){
			// summary:
			//		Provides keyboard navigation of calendar.
			// description:
			//		Called from _onKeyDown() to handle keydown on a stand alone Calendar,
			//		and also from `dijit/form/_DateTimeTextBox` to pass a keydown event
			//		from the `dijit/form/DateTextBox` to be handled in this widget
			// returns:
			//		False if the key was recognized as a navigation key,
			//		to indicate that the event was handled by Calendar and shouldn't be propagated
			// tags:
			//		protected
			var increment = -1,
				interval,
				newValue = this.currentFocus;
			switch(evt.keyCode){
				case keys.RIGHT_ARROW:
					increment = 1;
				//fallthrough...
				case keys.LEFT_ARROW:
					interval = "day";
					if(!this.isLeftToRight()){
						increment *= -1;
					}
					break;
				case keys.DOWN_ARROW:
					increment = 1;
				//fallthrough...
				case keys.UP_ARROW:
					interval = "week";
					break;
				case keys.PAGE_DOWN:
					increment = 1;
				//fallthrough...
				case keys.PAGE_UP:
					interval = evt.ctrlKey || evt.altKey ? "year" : "month";
					break;
				case keys.END:
					// go to the next month
					newValue = this.dateModule.add(newValue, "month", 1);
					// subtract a day from the result when we're done
					interval = "day";
				//fallthrough...
				case keys.HOME:
					newValue = new this.dateClassObj(newValue);
					newValue.setDate(1);
					break;
				default:
					return true;
			}

			if(interval){
				newValue = this.dateModule.add(newValue, interval, increment);
			}

			this._setCurrentFocusAttr(newValue);

			return false;
		},

		_onKeyDown: function(/*Event*/ evt){
			// summary:
			//		For handling keydown events on a stand alone calendar
			if(!this.handleKey(evt)){
				evt.stopPropagation();
				evt.preventDefault();
			}
		},

		onValueSelected: function(/*Date*/ /*===== date =====*/){
			// summary:
			//		Deprecated.   Notification that a date cell was selected.  It may be the same as the previous value.
			// description:
			//		Formerly used by `dijit/form/_DateTimeTextBox` (and thus `dijit/form/DateTextBox`)
			//		to get notification when the user has clicked a date.  Now onExecute() (above) is used.
			// tags:
			//		protected
		},

		onChange: function(value){
			this.onValueSelected(value);	// remove in 2.0
		},

		getClassForDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension

			/*=====
			 return ""; // String
			 =====*/
		}
	});

	Calendar._MonthDropDownButton = declare("dijit.Calendar._MonthDropDownButton", DropDownButton, {
		// summary:
		//		DropDownButton for the current month.    Displays name of current month
		//		and a list of month names in the drop down

		onMonthSelect: function(){
		},

		postCreate: function(){
			this.inherited(arguments);
			this.dropDown = new Calendar._MonthDropDown({
				id: this.id + "_mdd", //do not change this id because it is referenced in the template
				onChange: this.onMonthSelect
			});
		},
		_setMonthAttr: function(month){
			// summary:
			//		Set the current month to display as a label
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month);
			this.dropDown.set("months", monthNames);

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.containerNode.innerHTML =
				(has("ie") == 6 ? "" : "<div class='dijitSpacer'>" + this.dropDown.domNode.innerHTML + "</div>") +
					"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" + monthNames[month.getMonth()] + "</div>";
		}
	});

	Calendar._MonthDropDown = declare("dijit.Calendar._MonthDropDown", [_Widget, _TemplatedMixin], {
		// summary:
		//		The list-of-months drop down from the MonthDropDownButton

		// months: String[]
		//		List of names of months, possibly w/some undefined entries for Hebrew leap months
		//		(ex: ["January", "February", undefined, "April", ...])
		months: [],

		templateString: "<div class='dijitCalendarMonthMenu dijitMenu' " +
			"data-dojo-attach-event='onclick:_onClick,onmouseover:_onMenuHover,onmouseout:_onMenuHover'></div>",

		_setMonthsAttr: function(/*String[]*/ months){
			this.domNode.innerHTML = array.map(months,function(month, idx){
				return month ? "<div class='dijitCalendarMonthLabel' month='" + idx + "'>" + month + "</div>" : "";
			}).join("");
		},

		_onClick: function(/*Event*/ evt){
			this.onChange(domAttr.get(evt.target, "month"));
		},

		onChange: function(/*Number*/ /*===== month =====*/){
			// summary:
			//		Callback when month is selected from drop down
		},

		_onMenuHover: function(evt){
			domClass.toggle(evt.target, "dijitCalendarMonthLabelHover", evt.type == "mouseover");
		}
	});

	return Calendar;
});

},
'dojo/date':function(){
define(["./has", "./_base/lang"], function(has, lang){
// module:
//		dojo/date

var date = {
	// summary:
	//		Date manipulation utilities
};

date.getDaysInMonth = function(/*Date*/dateObject){
	// summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
};

date.isLeapYear = function(/*Date*/dateObject){
	// summary:
	//		Determines if the year of the dateObject is a leap year
	// description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
};

// FIXME: This is not localized
date.getTimezoneName = function(/*Date*/dateObject){
	// summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	// description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string,
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
};

// Utility methods to do arithmetic calculations with Dates

date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	// summary:
	//		Compare two date objects by date, time, or both.
	// description:
	//		Returns 0 if equal, positive if a > b, else negative.
	// date1:
	//		Date object
	// date2:
	//		Date object.  If not specified, the current Date is used.
	// portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(+date1);
	date2 = new Date(+(date2 || new Date()));

	if(portion == "date"){
		// Ignore times and compare dates.
		date1.setHours(0, 0, 0, 0);
		date2.setHours(0, 0, 0, 0);
	}else if(portion == "time"){
		// Ignore dates and compare times.
		date1.setFullYear(0, 0, 0);
		date2.setFullYear(0, 0, 0);
	}

	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	// summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	// date: Date
	//		Date object to start with
	// interval:
	//		A string representing the interval.  One of the following:
	//		"year", "month", "day", "hour", "minute", "second",
	//		"millisecond", "quarter", "week", "weekday"
	// amount:
	//		How much to add to the date.

	var sum = new Date(+date); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but this is not always true.  see dojo/cldr/supplemental

			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var days, weeks;
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			var adj = 0;
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = (7 * weeks) + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
//		case "hour":
//		case "minute":
//		case "second":
//		case "millisecond":
		default:
			property = "UTC"+interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	// summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	// date1:
	//		Date object
	// date2:
	//		Date object.  If not specified, the current Date is used.
	// interval:
	//		A string representing the interval.  One of the following:
	//		"year", "month", "day", "hour", "minute", "second",
	//		"millisecond", "quarter", "week", "weekday"
	//
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(date.difference(date1, date2, "day"));
			var weeks = parseInt(date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

// Don't use setObject() because it may overwrite dojo/date/stamp (if that has already been loaded)
 1  && lang.mixin(lang.getObject("dojo.date", true), date);

return date;
});

},
'dojo/date/locale':function(){
define([
	"../_base/lang",
	"../_base/array",
	"../date",
	/*===== "../_base/declare", =====*/
	"../cldr/supplemental",
	"../i18n",
	"../regexp",
	"../string",
	"../i18n!../cldr/nls/gregorian",
	"module"
], function(lang, array, date, /*===== declare, =====*/ supplemental, i18n, regexp, string, gregorian, module){

// module:
//		dojo/date/locale

var exports = {
	// summary:
	//		This modules defines dojo/date/locale, localization methods for Date.
};
lang.setObject(module.id.replace(/\//g, "."), exports);

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.

// Load the bundles containing localization information for
// names and formats

//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

	// Format a pattern without literals
	function formatPattern(dateObject, bundle, options, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad,
				c = match.charAt(0),
				l = match.length,
				widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							if(!options.fullYear){
								s = String(s); s = s.substr(s.length - 2);
								break;
							}
							// fallthrough
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
				case 'L':
					var m = dateObject.getMonth();
					if(l<3){
						s = m+1; pad = true;
					}else{
						var propM = [
							"months",
							c == 'L' ? "standAlone" : "format",
							widthList[l-3]
						].join("-");
						s = bundle[propM][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = exports._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = exports._getDayOfYear(dateObject); pad = true;
					break;
				case 'e':
				case 'c':
					var d = dateObject.getDay();
					if(l<2){
						s = (d - supplemental.getFirstDayOfWeek(options.locale) + 8) % 7
						break;
					}
					// fallthrough
				case 'E':
					d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = [
							"days",
							c == 'c' ? "standAlone" : "format",
							widthList[l-3]
						].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = dateObject.getHours() < 12 ? 'am' : 'pm';
					s = options[timePeriod] || bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = exports._getZone(dateObject, true, options);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = exports._getZone(dateObject, false, options);
					var tz = [
						(offset<=0 ? "+" : "-"),
						string.pad(Math.floor(Math.abs(offset)/60), 2),
						string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A':
//					console.log(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = string.pad(s, l); }
			return s;
		});
	}

/*=====
var __FormatOptions = exports.__FormatOptions = declare(null, {
	// selector: String
	//		choice of 'time','date' (default: date and time)
	// formatLength: String
	//		choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	// datePattern:String
	//		override pattern with this string
	// timePattern:String
	//		override pattern with this string
	// am: String
	//		override strings for am in times
	// pm: String
	//		override strings for pm in times
	// locale: String
	//		override the locale used to determine formatting rules
	// fullYear: Boolean
	//		(format only) use 4 digit years whenever 2 digit years are called for
	// strict: Boolean
	//		(parse only) strict parsing, off by default
});
=====*/

exports._getZone = function(/*Date*/ dateObject, /*boolean*/ getName, /*__FormatOptions?*/ options){
	// summary:
	//		Returns the zone (or offset) for the given date and options.  This
	//		is broken out into a separate function so that it can be overridden
	//		by timezone-aware code.
	//
	// dateObject:
	//		the date and/or time being formatted.
	//
	// getName:
	//		Whether to return the timezone string (if true), or the offset (if false)
	//
	// options:
	//		The options being used for formatting
	if(getName){
		return date.getTimezoneName(dateObject);
	}else{
		return dateObject.getTimezoneOffset();
	}
};


exports.format = function(/*Date*/ dateObject, /*__FormatOptions?*/ options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo/date/locale.addCustomFormats() method.
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.

	options = options || {};

	var locale = i18n.normalizeLocale(options.locale),
		formatLength = options.formatLength || 'short',
		bundle = exports._getGregorianBundle(locale),
		str = [],
		sauce = lang.hitch(this, formatPattern, dateObject, bundle, options);
	if(options.selector == "year"){
		return _processPattern(bundle["dateFormatItem-yyyy"] || "yyyy", sauce);
	}
	var pattern;
	if(options.selector != "date"){
		pattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}
	if(options.selector != "time"){
		pattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}

	return str.length == 1 ? str[0] : bundle["dateTimeFormat-"+formatLength].replace(/\'/g,'').replace(/\{(\d+)\}/g,
		function(match, key){ return str[key]; }); // String
};

exports.regexp = function(/*__FormatOptions?*/ options){
	// summary:
	//		Builds the regular needed to parse a localized date

	return exports._parseInfo(options).regexp; // String
};

exports._parseInfo = function(/*__FormatOptions?*/ options){
	options = options || {};
	var locale = i18n.normalizeLocale(options.locale),
		bundle = exports._getGregorianBundle(locale),
		formatLength = options.formatLength || 'short',
		datePattern = options.datePattern || bundle["dateFormat-" + formatLength],
		timePattern = options.timePattern || bundle["timeFormat-" + formatLength],
		pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
			function(match, key){ return [timePattern, datePattern][key]; });
	}

	var tokens = [],
		re = _processPattern(pattern, lang.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

exports.parse = function(/*String*/ value, /*__FormatOptions?*/ options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo/date/locale.addCustomFormats() method.
	//
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//		When two digit years are used, a century is chosen according to a sliding
	//		window of 80 years before and 20 years after present year, for both `yy` and `yyyy` patterns.
	//		year < 100CE requires strict mode.
	//
	// value:
	//		A string representation of a date

	// remove non-printing bidi control chars from input and pattern
	var controlChars = /[\u200E\u200F\u202A\u202E]/g,
		info = exports._parseInfo(options),
		tokens = info.tokens, bundle = info.bundle,
		re = new RegExp("^" + info.regexp.replace(controlChars, "") + "$",
			info.strict ? "" : "i"),
		match = re.exec(value && value.replace(controlChars, ""));

	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'],
		result = [1970,0,1,0,0,0,0], // will get converted to a Date at the end
		amPm = "",
		valid = array.every(match, function(v, i){
		if(!i){return true;}
		var token = tokens[i-1],
			l = token.length,
			c = token.charAt(0);
		switch(c){
			case 'y':
				if(l != 2 && options.strict){
					//interpret year literally, so '5' would be 5 A.D.
					result[0] = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear(),
							century = year.substring(0, 2) * 100,
							cutoff = Math.min(Number(year.substring(2, 4)) + 20, 99);
						result[0] = (v < cutoff) ? century + v : century - 100 + v;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return false;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result[0] = v;
					}
				}
				break;
			case 'M':
			case 'L':
				if(l>2){
					var months = bundle['months-' +
							    (c == 'L' ? 'standAlone' : 'format') +
							    '-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = array.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = array.indexOf(months, v);
					if(v == -1){
//						console.log("dojo/date/locale.parse: Could not parse month name: '" + v + "'.");
						return false;
					}
				}else{
					v--;
				}
				result[1] = v;
				break;
			case 'E':
			case 'e':
			case 'c':
				var days = bundle['days-' +
						  (c == 'c' ? 'standAlone' : 'format') +
						  '-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = array.map(days, function(d){return d.toLowerCase();});
				}
				v = array.indexOf(days, v);
				if(v == -1){
//					console.log("dojo/date/locale.parse: Could not parse weekday name: '" + v + "'.");
					return false;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
				result[2] = v;
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.log("dojo/date/locale.parse: Could not parse am/pm part.");
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.log("dojo/date/locale.parse: Illegal hours value");
					return false;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = v;
				break;
			case 'm': //minutes
				result[4] = v;
				break;
			case 's': //seconds
				result[5] = v;
				break;
			case 'S': //milliseconds
				result[6] = v;
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.log("dojo/date/locale.parse: unsupported pattern char=" + token.charAt(0));
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}

	//TODO: implement a getWeekday() method in order to test
	//validity of input strings containing 'EEE' or 'EEEE'...

	var dateObject = new Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]); // Date
	if(options.strict){
		dateObject.setFullYear(result[0]);
	}

	// Check for overflow.  The Date() constructor normalizes things like April 32nd...
	//TODO: why isn't this done for times as well?
	var allTokens = tokens.join(""),
		dateToken = allTokens.indexOf('d') != -1,
		monthToken = allTokens.indexOf('M') != -1;

	if(!valid ||
		(monthToken && dateObject.getMonth() > result[1]) ||
		(dateToken && dateObject.getDate() > result[2])){
		return null;
	}

	// Check for underflow, due to DST shifts.  See #9366
	// This assumes a 1 hour dst shift correction at midnight
	// We could compare the timezone offset after the shift and add the difference instead.
	if((monthToken && dateObject.getMonth() < result[1]) ||
		(dateToken && dateObject.getDate() < result[2])){
		dateObject = date.add(dateObject, "hour", 1);
	}

	return dateObject; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g),
		literal = pattern.charAt(0) == "'";

	array.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk.replace(/''/g, "'"));
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s,
			c = match.charAt(0),
			l = match.length,
			p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
			case 'L':
				s = (l>2) ? '\\S+?' : '1[0-2]|'+p2+'[1-9]';
				break;
			case 'D':
				s = '[12][0-9][0-9]|3[0-5][0-9]|36[0-6]|'+p2+'[1-9][0-9]|'+p3+'[1-9]';
				break;
			case 'd':
				s = '3[01]|[12]\\d|'+p2+'[1-9]';
				break;
			case 'w':
				s = '[1-4][0-9]|5[0-3]|'+p2+'[1-9]';
				break;
			case 'E':
			case 'e':
			case 'c':
				s = '.+?'; // match anything including spaces until the first pattern delimiter is found such as a comma or space
				break;
			case 'h': //hour (1-12)
				s = '1[0-2]|'+p2+'[1-9]';
				break;
			case 'k': //hour (0-11)
				s = '1[01]|'+p2+'\\d';
				break;
			case 'H': //hour (0-23)
				s = '1\\d|2[0-3]|'+p2+'\\d';
				break;
			case 'K': //hour (1-24)
				s = '1\\d|2[0-4]|'+p2+'[1-9]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
					s = am + '|' + pm;
				if(!options.strict){
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					if(s.indexOf('.') != -1){ s += '|' + s.replace(/\./g, ""); }
				}
				s = s.replace(/\./g, "\\.");
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.log("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}

var _customFormats = [];
exports.addCustomFormats = function(/*String*/ packageName, /*String*/ bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo.cldr: `dateFormat-xxxx` / `timeFormat-xxxx`
	//		The pattern string should match the format used by the CLDR.
	//		See dojo/date/locale.format() for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

exports._getGregorianBundle = function(/*String*/ locale){
	var gregorian = {};
	array.forEach(_customFormats, function(desc){
		var bundle = i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = lang.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};

exports.addCustomFormats(module.id.replace(/\/date\/locale$/, ".cldr"),"gregorian");

exports.getNames = function(/*String*/ item, /*String*/ type, /*String?*/ context, /*String?*/ locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item:
	//	'months' || 'days'
	// type:
	//	'wide' || 'abbr' || 'narrow' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// context:
	//	'standAlone' || 'format' (default)
	// locale:
	//	override locale used to find the names

	var label,
		lookup = exports._getGregorianBundle(locale),
		props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

exports.isWeekend = function(/*Date?*/ dateObject, /*String?*/ locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = supplemental.getWeekend(locale),
		day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

exports._getDayOfYear = function(/*Date*/ dateObject){
	// summary:
	//		gets the day of the year as represented by dateObject
	return date.difference(new Date(dateObject.getFullYear(), 0, 1, dateObject.getHours()), dateObject) + 1; // Number
};

exports._getWeekOfYear = function(/*Date*/ dateObject, /*Number*/ firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay(),
		adj = (firstDayOfYear - firstDayOfWeek + 7) % 7,
		week = Math.floor((exports._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

return exports;
});

},
'dojo/cldr/supplemental':function(){
define(["../_base/lang", "../i18n"], function(lang, i18n){

// module:
//		dojo/cldr/supplemental


var supplemental = {
	// summary:
	//		TODOC
};
lang.setObject("dojo.cldr.supplemental", supplemental);

supplemental.getFirstDayOfWeek = function(/*String?*/locale){
	// summary:
	//		Returns a zero-based index for first day of the week
	// description:
	//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
	//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		bd:5,mv:5,
		ae:6,af:6,bh:6,dj:6,dz:6,eg:6,iq:6,ir:6,jo:6,kw:6,
		ly:6,ma:6,om:6,qa:6,sa:6,sd:6,sy:6,ye:6,
		ag:0,ar:0,as:0,au:0,br:0,bs:0,bt:0,bw:0,by:0,bz:0,ca:0,cn:0,
		co:0,dm:0,'do':0,et:0,gt:0,gu:0,hk:0,hn:0,id:0,ie:0,il:0,'in':0,
		jm:0,jp:0,ke:0,kh:0,kr:0,la:0,mh:0,mm:0,mo:0,mt:0,mx:0,mz:0,
		ni:0,np:0,nz:0,pa:0,pe:0,ph:0,pk:0,pr:0,py:0,sg:0,sv:0,th:0,
		tn:0,tt:0,tw:0,um:0,us:0,ve:0,vi:0,ws:0,za:0,zw:0
	};

	var country = supplemental._region(locale);
	var dow = firstDay[country];
	return (dow === undefined) ? 1 : dow; /*Number*/
};

supplemental._region = function(/*String?*/locale){
	locale = i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
		region = {
			aa:"et", ab:"ge", af:"za", ak:"gh", am:"et", ar:"eg", as:"in", av:"ru", ay:"bo", az:"az", ba:"ru",
			be:"by", bg:"bg", bi:"vu", bm:"ml", bn:"bd", bo:"cn", br:"fr", bs:"ba", ca:"es", ce:"ru", ch:"gu",
			co:"fr", cr:"ca", cs:"cz", cv:"ru", cy:"gb", da:"dk", de:"de", dv:"mv", dz:"bt", ee:"gh", el:"gr",
			en:"us", es:"es", et:"ee", eu:"es", fa:"ir", ff:"sn", fi:"fi", fj:"fj", fo:"fo", fr:"fr", fy:"nl",
			ga:"ie", gd:"gb", gl:"es", gn:"py", gu:"in", gv:"gb", ha:"ng", he:"il", hi:"in", ho:"pg", hr:"hr",
			ht:"ht", hu:"hu", hy:"am", ia:"fr", id:"id", ig:"ng", ii:"cn", ik:"us", "in":"id", is:"is", it:"it",
			iu:"ca", iw:"il", ja:"jp", ji:"ua", jv:"id", jw:"id", ka:"ge", kg:"cd", ki:"ke", kj:"na", kk:"kz",
			kl:"gl", km:"kh", kn:"in", ko:"kr", ks:"in", ku:"tr", kv:"ru", kw:"gb", ky:"kg", la:"va", lb:"lu",
			lg:"ug", li:"nl", ln:"cd", lo:"la", lt:"lt", lu:"cd", lv:"lv", mg:"mg", mh:"mh", mi:"nz", mk:"mk",
			ml:"in", mn:"mn", mo:"ro", mr:"in", ms:"my", mt:"mt", my:"mm", na:"nr", nb:"no", nd:"zw", ne:"np",
			ng:"na", nl:"nl", nn:"no", no:"no", nr:"za", nv:"us", ny:"mw", oc:"fr", om:"et", or:"in", os:"ge",
			pa:"in", pl:"pl", ps:"af", pt:"br", qu:"pe", rm:"ch", rn:"bi", ro:"ro", ru:"ru", rw:"rw", sa:"in",
			sd:"in", se:"no", sg:"cf", si:"lk", sk:"sk", sl:"si", sm:"ws", sn:"zw", so:"so", sq:"al", sr:"rs",
			ss:"za", st:"za", su:"id", sv:"se", sw:"tz", ta:"in", te:"in", tg:"tj", th:"th", ti:"et", tk:"tm",
			tl:"ph", tn:"za", to:"to", tr:"tr", ts:"za", tt:"ru", ty:"pf", ug:"cn", uk:"ua", ur:"pk", uz:"uz",
			ve:"za", vi:"vn", wa:"be", wo:"sn", xh:"za", yi:"il", yo:"ng", za:"cn", zh:"cn", zu:"za",
			ace:"id", ady:"ru", agq:"cm", alt:"ru", amo:"ng", asa:"tz", ast:"es", awa:"in", bal:"pk",
			ban:"id", bas:"cm", bax:"cm", bbc:"id", bem:"zm", bez:"tz", bfq:"in", bft:"pk", bfy:"in",
			bhb:"in", bho:"in", bik:"ph", bin:"ng", bjj:"in", bku:"ph", bqv:"ci", bra:"in", brx:"in",
			bss:"cm", btv:"pk", bua:"ru", buc:"yt", bug:"id", bya:"id", byn:"er", cch:"ng", ccp:"in",
			ceb:"ph", cgg:"ug", chk:"fm", chm:"ru", chp:"ca", chr:"us", cja:"kh", cjm:"vn", ckb:"iq",
			crk:"ca", csb:"pl", dar:"ru", dav:"ke", den:"ca", dgr:"ca", dje:"ne", doi:"in", dsb:"de",
			dua:"cm", dyo:"sn", dyu:"bf", ebu:"ke", efi:"ng", ewo:"cm", fan:"gq", fil:"ph", fon:"bj",
			fur:"it", gaa:"gh", gag:"md", gbm:"in", gcr:"gf", gez:"et", gil:"ki", gon:"in", gor:"id",
			grt:"in", gsw:"ch", guz:"ke", gwi:"ca", haw:"us", hil:"ph", hne:"in", hnn:"ph", hoc:"in",
			hoj:"in", ibb:"ng", ilo:"ph", inh:"ru", jgo:"cm", jmc:"tz", kaa:"uz", kab:"dz", kaj:"ng",
			kam:"ke", kbd:"ru", kcg:"ng", kde:"tz", kdt:"th", kea:"cv", ken:"cm", kfo:"ci", kfr:"in",
			kha:"in", khb:"cn", khq:"ml", kht:"in", kkj:"cm", kln:"ke", kmb:"ao", koi:"ru", kok:"in",
			kos:"fm", kpe:"lr", krc:"ru", kri:"sl", krl:"ru", kru:"in", ksb:"tz", ksf:"cm", ksh:"de",
			kum:"ru", lag:"tz", lah:"pk", lbe:"ru", lcp:"cn", lep:"in", lez:"ru", lif:"np", lis:"cn",
			lki:"ir", lmn:"in", lol:"cd", lua:"cd", luo:"ke", luy:"ke", lwl:"th", mad:"id", mag:"in",
			mai:"in", mak:"id", man:"gn", mas:"ke", mdf:"ru", mdh:"ph", mdr:"id", men:"sl", mer:"ke",
			mfe:"mu", mgh:"mz", mgo:"cm", min:"id", mni:"in", mnk:"gm", mnw:"mm", mos:"bf", mua:"cm",
			mwr:"in", myv:"ru", nap:"it", naq:"na", nds:"de", "new":"np", niu:"nu", nmg:"cm", nnh:"cm",
			nod:"th", nso:"za", nus:"sd", nym:"tz", nyn:"ug", pag:"ph", pam:"ph", pap:"bq", pau:"pw",
			pon:"fm", prd:"ir", raj:"in", rcf:"re", rej:"id", rjs:"np", rkt:"in", rof:"tz", rwk:"tz",
			saf:"gh", sah:"ru", saq:"ke", sas:"id", sat:"in", saz:"in", sbp:"tz", scn:"it", sco:"gb",
			sdh:"ir", seh:"mz", ses:"ml", shi:"ma", shn:"mm", sid:"et", sma:"se", smj:"se", smn:"fi",
			sms:"fi", snk:"ml", srn:"sr", srr:"sn", ssy:"er", suk:"tz", sus:"gn", swb:"yt", swc:"cd",
			syl:"bd", syr:"sy", tbw:"ph", tcy:"in", tdd:"cn", tem:"sl", teo:"ug", tet:"tl", tig:"er",
			tiv:"ng", tkl:"tk", tmh:"ne", tpi:"pg", trv:"tw", tsg:"ph", tts:"th", tum:"mw", tvl:"tv",
			twq:"ne", tyv:"ru", tzm:"ma", udm:"ru", uli:"fm", umb:"ao", unr:"in", unx:"in", vai:"lr",
			vun:"tz", wae:"ch", wal:"et", war:"ph", xog:"ug", xsr:"np", yao:"mz", yap:"fm", yav:"cm", zza:"tr"
		}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
};

supplemental.getWeekend = function(/*String?*/locale){
	// summary:
	//		Returns a hash containing the start and end days of the weekend
	// description:
	//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
	//		or by default in the user's locale.
	//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
			'in':0,
			af:4,dz:4,ir:4,om:4,sa:4,ye:4,
			ae:5,bh:5,eg:5,il:5,iq:5,jo:5,kw:5,ly:5,ma:5,qa:5,sd:5,sy:5,tn:5
		},

		weekendEnd = {/*default is 0=Sunday*/
			af:5,dz:5,ir:5,om:5,sa:5,ye:5,
			ae:6,bh:5,eg:6,il:6,iq:6,jo:6,kw:6,ly:6,ma:6,qa:6,sd:6,sy:6,tn:6
		},

		country = supplemental._region(locale),
		start = weekendStart[country],
		end = weekendEnd[country];

	if(start === undefined){start=6;}
	if(end === undefined){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

return supplemental;
});

},
'dijit/CalendarLite':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/cldr/supplemental", // cldrSupplemental.getFirstDayOfWeek
	"dojo/date", // date
	"dojo/date/locale",
	"dojo/date/stamp", // stamp.fromISOString
	"dojo/dom", // dom.setSelectable
	"dojo/dom-class", // domClass.contains
	"dojo/_base/lang", // lang.getObject, lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie") has("webkit")
	"dojo/string", // string.substitute
	"./_WidgetBase",
	"./_TemplatedMixin",
	"dojo/text!./templates/Calendar.html",
	"./a11yclick",	// not used directly, but template has ondijitclick in it
	"./hccss"    // not used directly, but sets CSS class on <body>
], function(array, declare, cldrSupplemental, date, locale, stamp, dom, domClass, lang, on, has, string, _WidgetBase, _TemplatedMixin, template){


	// module:
	//		dijit/CalendarLite

	var CalendarLite = declare("dijit.CalendarLite", [_WidgetBase, _TemplatedMixin], {
		// summary:
		//		Lightweight version of Calendar widget aimed towards mobile use
		//
		// description:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//		This widget can't be used in a form because it doesn't serialize the date to an
		//		`<input>` field.  For a form element, use dijit/form/DateTextBox instead.
		//
		//		Note that the parser takes all dates attributes passed in the
		//		[RFC 3339 format](http://www.faqs.org/rfcs/rfc3339.html), e.g. `2005-06-30T08:05:00-07:00`
		//		so that they are serializable and locale-independent.
		//
		//		Also note that this widget isn't keyboard accessible; use dijit.Calendar for that
		// example:
		//	|	var calendar = new dijit.CalendarLite({}, dojo.byId("calendarNode"));
		//
		// example:
		//	|	<div data-dojo-type="dijit/CalendarLite"></div>

		// Template for main calendar
		templateString: template,

		// Template for cell for a day of the week (ex: M)
		dowTemplateString: '<th class="dijitReset dijitCalendarDayLabelTemplate" role="columnheader" scope="col"><span class="dijitCalendarDayLabel">${d}</span></th>',

		// Templates for a single date (ex: 13), and for a row for a week (ex: 20 21 22 23 24 25 26)
		dateTemplateString: '<td class="dijitReset" role="gridcell" data-dojo-attach-point="dateCells"><span class="dijitCalendarDateLabel" data-dojo-attach-point="dateLabels"></span></td>',
		weekTemplateString: '<tr class="dijitReset dijitCalendarWeekTemplate" role="row">${d}${d}${d}${d}${d}${d}${d}</tr>',

		// value: Date
		//		The currently selected Date, initially set to invalid date to indicate no selection.
		value: new Date(""),
		// TODO: for 2.0 make this a string (ISO format) rather than a Date

		// datePackage: String
		//		JavaScript namespace to find calendar routines.	 If unspecified, uses Gregorian calendar routines
		//		at dojo/date and dojo/date/locale.
		datePackage: "",
		//		TODO: for 2.0, replace datePackage with dateModule and dateLocalModule attributes specifying MIDs,
		//		or alternately just get rid of this completely and tell user to use module ID remapping
		//		via require

		// dayWidth: String
		//		How to represent the days of the week in the calendar header. See locale
		dayWidth: "narrow",

		// tabIndex: String
		//		Order fields are traversed when user hits the tab key
		tabIndex: "0",

		// currentFocus: Date
		//		Date object containing the currently focused date, or the date which would be focused
		//		if the calendar itself was focused.   Also indicates which year and month to display,
		//		i.e. the current "page" the calendar is on.
		currentFocus: new Date(),

		// Put the summary to the node with role=grid
		_setSummaryAttr: "gridNode",

		baseClass: "dijitCalendar",

		_isValidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking that it's a valid date, rather
			//		than blank or NaN.
			// tags:
			//		private
			return value && !isNaN(value) && typeof value == "object" &&
				value.toString() != this.constructor.prototype.value.toString();
		},

		_getValueAttr: function(){
			// summary:
			//		Support get('value')

			// this.value is set to 1AM, but return midnight, local time for back-compat
			var storedVal = this._get("value");
			if(storedVal && !isNaN(storedVal)){
				var value = new this.dateClassObj(storedVal);
				value.setHours(0, 0, 0, 0);

				// If daylight savings pushes midnight to the previous date, fix the Date
				// object to point at 1am so it will represent the correct day. See #9366
				if(value.getDate() < storedVal.getDate()){
					value = this.dateModule.add(value, "hour", 1);
				}
				return value;
			}else{
				return null;
			}
		},

		_setValueAttr: function(/*Date|Number*/ value, /*Boolean*/ priorityChange){
			// summary:
			//		Support set("value", ...)
			// description:
			//		Set the current date and update the UI.  If the date is disabled, the value will
			//		not change, but the display will change to the corresponding month.
			// value:
			//		Either a Date or the number of seconds since 1970.
			// tags:
			//		protected
			if(typeof value == "string"){
				value = stamp.fromISOString(value);
			}
			value = this._patchDate(value);

			if(this._isValidDate(value) && !this.isDisabledDate(value, this.lang)){
				this._set("value", value);

				// Set focus cell to the new value.   Arguably this should only happen when there isn't a current
				// focus point.   This will also repopulate the grid to new month/year if necessary.
				this.set("currentFocus", value);

				// Mark the selected date
				this._markSelectedDates([value]);

				if(this._created && (priorityChange || typeof priorityChange == "undefined")){
					this.onChange(this.get('value'));
				}
			}else{
				// clear value, and mark all dates as unselected
				this._set("value", null);
				this._markSelectedDates([]);
			}
		},

		_patchDate: function(/*Date|Number*/ value){
			// summary:
			//		Convert Number into Date, or copy Date object.   Then, round to nearest day,
			//		setting to 1am to avoid issues when DST shift occurs at midnight, see #8521, #9366)
			if(value){
				value = new this.dateClassObj(value);
				value.setHours(1, 0, 0, 0);
			}
			return value;
		},

		_setText: function(node, text){
			// summary:
			//		This just sets the content of node to the specified text.
			//		Can't do "node.innerHTML=text" because of an IE bug w/tables, see #3434.
			// tags:
			//		private
			while(node.firstChild){
				node.removeChild(node.firstChild);
			}
			node.appendChild(node.ownerDocument.createTextNode(text));
		},

		_populateGrid: function(){
			// summary:
			//		Fills in the calendar grid with each day (1-31).
			//		Call this on creation, when moving to a new month.
			// tags:
			//		private

			var month = new this.dateClassObj(this.currentFocus);
			month.setDate(1);

			var firstDay = month.getDay(),
				daysInMonth = this.dateModule.getDaysInMonth(month),
				daysInPreviousMonth = this.dateModule.getDaysInMonth(this.dateModule.add(month, "month", -1)),
				today = new this.dateClassObj(),
				dayOffset = cldrSupplemental.getFirstDayOfWeek(this.lang);
			if(dayOffset > firstDay){
				dayOffset -= 7;
			}

			// If they didn't provide a summary, change the default summary to match with the new month
			if(!this.summary){
				var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month)
				this.gridNode.setAttribute("summary", monthNames[month.getMonth()]);
			}

			// Mapping from date (as specified by number returned from Date.valueOf()) to corresponding <td>
			this._date2cell = {};

			// Iterate through dates in the calendar and fill in date numbers and style info
			array.forEach(this.dateCells, function(template, idx){
				var i = idx + dayOffset;
				var date = new this.dateClassObj(month),
					number, clazz = "dijitCalendar", adj = 0;

				if(i < firstDay){
					number = daysInPreviousMonth - firstDay + i + 1;
					adj = -1;
					clazz += "Previous";
				}else if(i >= (firstDay + daysInMonth)){
					number = i - firstDay - daysInMonth + 1;
					adj = 1;
					clazz += "Next";
				}else{
					number = i - firstDay + 1;
					clazz += "Current";
				}

				if(adj){
					date = this.dateModule.add(date, "month", adj);
				}
				date.setDate(number);

				if(!this.dateModule.compare(date, today, "date")){
					clazz = "dijitCalendarCurrentDate " + clazz;
				}

				if(this.isDisabledDate(date, this.lang)){
					clazz = "dijitCalendarDisabledDate " + clazz;
					template.setAttribute("aria-disabled", "true");
				}else{
					clazz = "dijitCalendarEnabledDate " + clazz;
					template.removeAttribute("aria-disabled");
					template.setAttribute("aria-selected", "false");
				}

				var clazz2 = this.getClassForDate(date, this.lang);
				if(clazz2){
					clazz = clazz2 + " " + clazz;
				}

				template.className = clazz + "Month dijitCalendarDateTemplate";

				// Each cell has an associated integer value representing it's date
				var dateVal = date.valueOf();
				this._date2cell[dateVal] = template;
				template.dijitDateValue = dateVal;

				// Set Date string (ex: "13").
				this._setText(this.dateLabels[idx], date.getDateLocalized ? date.getDateLocalized(this.lang) : date.getDate());
			}, this);
		},

		_populateControls: function(){
			// summary:
			//		Fill in localized month, and prev/current/next years
			// tags:
			//		protected

			var month = new this.dateClassObj(this.currentFocus);
			month.setDate(1);

			// set name of this month
			this.monthWidget.set("month", month);

			var y = month.getFullYear() - 1;
			var d = new this.dateClassObj();
			array.forEach(["previous", "current", "next"], function(name){
				d.setFullYear(y++);
				this._setText(this[name + "YearLabelNode"],
					this.dateLocaleModule.format(d, {selector: 'year', locale: this.lang}));
			}, this);
		},

		goToToday: function(){
			// summary:
			//		Sets calendar's value to today's date
			this.set('value', new this.dateClassObj());
		},

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree

			this.dateModule = params.datePackage ? lang.getObject(params.datePackage, false) : date;
			this.dateClassObj = this.dateModule.Date || Date;
			this.dateLocaleModule = params.datePackage ? lang.getObject(params.datePackage + ".locale", false) : locale;
		},

		_createMonthWidget: function(){
			// summary:
			//		Creates the drop down button that displays the current month and lets user pick a new one

			return CalendarLite._MonthWidget({
				id: this.id + "_mddb",
				lang: this.lang,
				dateLocaleModule: this.dateLocaleModule
			}, this.monthNode);
		},

		buildRendering: function(){
			// Markup for days of the week (referenced from template)
			var d = this.dowTemplateString,
				dayNames = this.dateLocaleModule.getNames('days', this.dayWidth, 'standAlone', this.lang),
				dayOffset = cldrSupplemental.getFirstDayOfWeek(this.lang);
			this.dayCellsHtml = string.substitute([d, d, d, d, d, d, d].join(""), {d: ""}, function(){
				return dayNames[dayOffset++ % 7];
			});

			// Markup for dates of the month (referenced from template), but without numbers filled in
			var r = string.substitute(this.weekTemplateString, {d: this.dateTemplateString});
			this.dateRowsHtml = [r, r, r, r, r, r].join("");

			// Instantiate from template.
			// dateCells and dateLabels arrays filled when _Templated parses my template.
			this.dateCells = [];
			this.dateLabels = [];
			this.inherited(arguments);

			dom.setSelectable(this.domNode, false);

			var dateObj = new this.dateClassObj(this.currentFocus);

			this.monthWidget = this._createMonthWidget();

			this.set('currentFocus', dateObj, false);	// draw the grid to the month specified by currentFocus
		},

		postCreate: function(){
			this.inherited(arguments);
			this._connectControls();
		},

		_connectControls: function(){
			// summary:
			//		Set up connects for increment/decrement of months/years
			// tags:
			//		protected

			var connect = lang.hitch(this, function(nodeProp, part, amount){
				return on(this[nodeProp], "click", lang.hitch(this, function(){
					this._setCurrentFocusAttr(this.dateModule.add(this.currentFocus, part, amount));
				}));
			});

			this.own(
				connect("incrementMonth", "month", 1),
				connect("decrementMonth", "month", -1),
				connect("nextYearLabelNode", "year", 1),
				connect("previousYearLabelNode", "year", -1)
			);
		},

		_setCurrentFocusAttr: function(/*Date*/ date, /*Boolean*/ forceFocus){
			// summary:
			//		If the calendar currently has focus, then focuses specified date,
			//		changing the currently displayed month/year if necessary.
			//		If the calendar doesn't have focus, updates currently
			//		displayed month/year, and sets the cell that will get focus
			//		when Calendar is focused.
			// forceFocus:
			//		If true, will focus() the cell even if calendar itself doesn't have focus

			var oldFocus = this.currentFocus,
				oldCell = this._getNodeByDate(oldFocus);
			date = this._patchDate(date);

			this._set("currentFocus", date);

			// If the focus is on a different month than the current calendar month, switch the displayed month.
			// Also will populate the grid initially, on Calendar creation.
			if(!this._date2cell || this.dateModule.difference(oldFocus, date, "month") != 0){
				this._populateGrid();
				this._populateControls();
				this._markSelectedDates([this.value]);
			}

			// set tabIndex=0 on new cell, and focus it (but only if Calendar itself is focused)
			var newCell = this._getNodeByDate(date);
			newCell.setAttribute("tabIndex", this.tabIndex);
			if(this.focused || forceFocus){
				newCell.focus();
			}

			// set tabIndex=-1 on old focusable cell
			if(oldCell && oldCell != newCell){
				if(has("webkit")){    // see #11064 about webkit bug
					oldCell.setAttribute("tabIndex", "-1");
				}else{
					oldCell.removeAttribute("tabIndex");
				}
			}
		},

		focus: function(){
			// summary:
			//		Focus the calendar by focusing one of the calendar cells
			this._setCurrentFocusAttr(this.currentFocus, true);
		},

		_onDayClick: function(/*Event*/ evt){
			// summary:
			//		Handler for day clicks, selects the date if appropriate
			// tags:
			//		protected
			evt.stopPropagation();
			evt.preventDefault();
			for(var node = evt.target; node && !node.dijitDateValue; node = node.parentNode){
				;
			}
			if(node && !domClass.contains(node, "dijitCalendarDisabledDate")){
				this.set('value', node.dijitDateValue);
			}
		},

		_getNodeByDate: function(/*Date*/ value){
			// summary:
			//		Returns the cell corresponding to the date, or null if the date is not within the currently
			//		displayed month.
			value = this._patchDate(value);
			return value && this._date2cell ? this._date2cell[value.valueOf()] : null;
		},

		_markSelectedDates: function(/*Date[]*/ dates){
			// summary:
			//		Marks the specified cells as selected, and clears cells previously marked as selected.
			//		For CalendarLite at most one cell is selected at any point, but this allows an array
			//		for easy subclassing.

			// Function to mark a cell as selected or unselected
			function mark(/*Boolean*/ selected, /*DomNode*/ cell){
				domClass.toggle(cell, "dijitCalendarSelectedDate", selected);
				cell.setAttribute("aria-selected", selected ? "true" : "false");
			}

			// Clear previously selected cells.
			array.forEach(this._selectedCells || [], lang.partial(mark, false));

			// Mark newly selected cells.  Ignore dates outside the currently displayed month.
			this._selectedCells = array.filter(array.map(dates, this._getNodeByDate, this), function(n){
				return n;
			});
			array.forEach(this._selectedCells, lang.partial(mark, true));
		},

		onChange: function(/*Date*/ /*===== date =====*/){
			// summary:
			//		Called only when the selected date has changed
		},

		isDisabledDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to disable certain dates in the calendar e.g. `isDisabledDate=dojo.date.locale.isWeekend`
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension
			/*=====
			 return false; // Boolean
			 =====*/
		},

		getClassForDate: function(/*===== dateObject, locale =====*/){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// dateObject: Date
			// locale: String?
			// tags:
			//		extension

			/*=====
			 return ""; // String
			 =====*/
		}
	});

	CalendarLite._MonthWidget = declare("dijit.CalendarLite._MonthWidget", _WidgetBase, {
		// summary:
		//		Displays name of current month padded to the width of the month
		//		w/the longest name, so that changing months doesn't change width.
		//
		//		Create as:
		// |	new Calendar._MonthWidget({
		// |			lang: ...,
		// |			dateLocaleModule: ...
		// |		})

		_setMonthAttr: function(month){
			// summary:
			//		Set the current month to display as a label
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month),
				spacer =
					(has("ie") == 6 ? "" : "<div class='dijitSpacer'>" +
						array.map(monthNames,function(s){
							return "<div>" + s + "</div>";
						}).join("") + "</div>");

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.domNode.innerHTML =
				spacer +
					"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" +
					monthNames[month.getMonth()] + "</div>";
		}
	});

	return CalendarLite;
});

},
'dojo/date/stamp':function(){
define(["../_base/lang", "../_base/array"], function(lang, array){

// module:
//		dojo/date/stamp

var stamp = {
	// summary:
	//		TODOC
};
lang.setObject("dojo.date.stamp", stamp);

// Methods to convert dates to or from a wire (string) format using well-known conventions

stamp.fromISOString = function(/*String*/ formattedString, /*Number?*/ defaultTime){
	// summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	// description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//		- dates only
	//			- yyyy
	//			- yyyy-MM
	//			- yyyy-MM-dd
	//		- times only, with an optional time zone appended
	//			- THH:mm
	//			- THH:mm:ss
	//			- THH:mm:ss.SSS
	//		- and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	//		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
  	// formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	// defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!stamp._isoRegExp){
		stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			array.forEach(array.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
};

/*=====
var __Options = {
	// selector: String
	//		"date" or "time" for partial formatting of the Date object.
	//		Both date and time will be formatted by default.
	// zulu: Boolean
	//		if true, UTC/GMT is used for a timezone
	// milliseconds: Boolean
	//		if true, output milliseconds
};
=====*/

stamp.toISOString = function(/*Date*/ dateObject, /*__Options?*/ options){
	// summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	// description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	// dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") +
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
};

return stamp;
});

},
'dijit/form/DropDownButton':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // hitch
	"dojo/query", // query
	"../registry", // registry.byNode
	"../popup", // dijit.popup2.hide
	"./Button",
	"../_Container",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownButton.html"
], function(declare, lang, query, registry, popup, Button, _Container, _HasDropDown, template){

	// module:
	//		dijit/form/DropDownButton

	return declare("dijit.form.DropDownButton", [Button, _Container, _HasDropDown], {
		// summary:
		//		A button with a drop down
		//
		// example:
		// |	<button data-dojo-type="dijit/form/DropDownButton">
		// |		Hello world
		// |		<div data-dojo-type="dijit/Menu">...</div>
		// |	</button>
		//
		// example:
		// |	var button1 = new DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
		// |	win.body().appendChild(button1);
		//

		baseClass: "dijitDropDownButton",

		templateString: template,

		_fillContent: function(){
			// Overrides Button._fillContent().
			//
			// My inner HTML contains both the button contents and a drop down widget, like
			// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
			// The first node is assumed to be the button content. The widget is the popup.

			if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
				//FIXME: figure out how to filter out the widget and use all remaining nodes as button
				//	content, not just nodes[0]
				var nodes = query("*", this.srcNodeRef);
				this.inherited(arguments, [nodes[0]]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		startup: function(){
			if(this._started){
				return;
			}

			// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
			// make it invisible, and store a reference to pass to the popup code.
			if(!this.dropDown && this.dropDownContainer){
				var dropDownNode = query("[widgetId]", this.dropDownContainer)[0];
				if(dropDownNode){
					this.dropDown = registry.byNode(dropDownNode);
				}
				delete this.dropDownContainer;
			}
			if(this.dropDown){
				popup.hide(this.dropDown);
			}

			this.inherited(arguments);
		},

		isLoaded: function(){
			// Returns whether or not we are loaded - if our dropdown has an href,
			// then we want to check that.
			var dropDown = this.dropDown;
			return (!!dropDown && (!dropDown.href || dropDown.isLoaded));
		},

		loadDropDown: function(/*Function*/ callback){
			// Default implementation assumes that drop down already exists,
			// but hasn't loaded it's data (ex: ContentPane w/href).
			// App must override if the drop down is lazy-created.
			var dropDown = this.dropDown;
			var handler = dropDown.on("load", lang.hitch(this, function(){
				handler.remove();
				callback();
			}));
			dropDown.refresh();		// tell it to load
		},

		isFocusable: function(){
			// Overridden so that focus is handled by the _HasDropDown mixin, not by
			// the _FormWidget mixin.
			return this.inherited(arguments) && !this._mouseDown;
		}
	});
});

},
'dijit/form/Button':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.toggle
	"dojo/has", // has("dijit-legacy-requires")
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.trim
	"dojo/ready",
	"./_FormWidget",
	"./_ButtonMixin",
	"dojo/text!./templates/Button.html"
], function(require, declare, domClass, has, kernel, lang, ready, _FormWidget, _ButtonMixin, template){

	// module:
	//		dijit/form/Button

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/form/DropDownButton", "dijit/form/ComboButton", "dijit/form/ToggleButton"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	var Button = declare("dijit.form.Button" + (has("dojo-bidi") ? "_NoBidi" : ""), [_FormWidget, _ButtonMixin], {
		// summary:
		//		Basically the same thing as a normal HTML button, but with special styling.
		// description:
		//		Buttons can display a label, an icon, or both.
		//		A label should always be specified (through innerHTML) or the label
		//		attribute.  It can be hidden via showLabel=false.
		// example:
		// |	<button data-dojo-type="dijit/form/Button" onClick="...">Hello world</button>
		//
		// example:
		// |	var button1 = new Button({label: "hello world", onClick: foo});
		// |	dojo.body().appendChild(button1.domNode);

		// showLabel: Boolean
		//		Set this to true to hide the label text and display only the icon.
		//		(If showLabel=false then iconClass must be specified.)
		//		Especially useful for toolbars.
		//		If showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.
		//
		//		The exception case is for computers in high-contrast mode, where the label
		//		will still be displayed, since the icon doesn't appear.
		showLabel: true,

		// iconClass: String
		//		Class to apply to DOMNode in button to make it display an icon
		iconClass: "dijitNoIcon",
		_setIconClassAttr: { node: "iconNode", type: "class" },

		baseClass: "dijitButton",

		templateString: template,

		// Map widget attributes to DOMNode attributes.
		_setValueAttr: "valueNode",
		_setNameAttr: function(name){
			// avoid breaking existing subclasses where valueNode undefined.  Perhaps in 2.0 require it to be defined?
			if(this.valueNode){
				this.valueNode.setAttribute("name", name);
			}
		},

		_fillContent: function(/*DomNode*/ source){
			// Overrides _Templated._fillContent().
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			// TODO: remove the method in 2.0, parser will do it all for me
			if(source && (!this.params || !("label" in this.params))){
				var sourceLabel = lang.trim(source.innerHTML);
				if(sourceLabel){
					this.label = sourceLabel; // _applyAttributes will be called after buildRendering completes to update the DOM
				}
			}
		},

		_setShowLabelAttr: function(val){
			if(this.containerNode){
				domClass.toggle(this.containerNode, "dijitDisplayNone", !val);
			}
			this._set("showLabel", val);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.  Use set('label', ...) instead.
			kernel.deprecated("dijit.form.Button.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		_setLabelAttr: function(/*String*/ content){
			// summary:
			//		Hook for set('label', ...) to work.
			// description:
			//		Set the label (text) of the button; takes an HTML string.
			//		If the label is hidden (showLabel=false) then and no title has
			//		been specified, then label is also set as title attribute of icon.
			this.inherited(arguments);
			if(!this.showLabel && !("title" in this.params)){
				this.titleNode.title = lang.trim(this.containerNode.innerText || this.containerNode.textContent || '');
			}
		}
	});

	if(has("dojo-bidi")){
		Button = declare("dijit.form.Button", Button, {
			_setLabelAttr: function(/*String*/ content){
				this.inherited(arguments);
				if(this.titleNode.title){
					this.applyTextDir(this.titleNode, this.titleNode.title);
				}
			},

			_setTextDirAttr: function(/*String*/ textDir){
				if(this._created && this.textDir != textDir){
					this._set("textDir", textDir);
					this._setLabelAttr(this.label); // call applyTextDir on both focusNode and titleNode
				}
			}
		});
	}

	return Button;
});

},
'dijit/form/_ButtonMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/has",
	"../registry"        // registry.byNode
], function(declare, dom, has, registry){

	// module:
	//		dijit/form/_ButtonMixin

	var ButtonMixin = declare("dijit.form._ButtonMixin" + (has("dojo-bidi") ? "_NoBidi" : ""), null, {
		// summary:
		//		A mixin to add a thin standard API wrapper to a normal HTML button
		// description:
		//		A label should always be specified (through innerHTML) or the label attribute.
		//
		//		Attach points:
		//
		//		- focusNode (required): this node receives focus
		//		- valueNode (optional): this node's value gets submitted with FORM elements
		//		- containerNode (optional): this node gets the innerHTML assignment for label
		// example:
		// |	<button data-dojo-type="dijit/form/Button" onClick="...">Hello world</button>
		// example:
		// |	var button1 = new Button({label: "hello world", onClick: foo});
		// |	dojo.body().appendChild(button1.domNode);

		// label: HTML String
		//		Content to display in button.
		label: "",

		// type: [const] String
		//		Type of button (submit, reset, button, checkbox, radio)
		type: "button",

		__onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to divert the real click onto the hidden INPUT that has a native default action associated with it
			// type:
			//		private
			e.stopPropagation();
			e.preventDefault();
			if(!this.disabled){
				// cannot use on.emit since button default actions won't occur
				this.valueNode.click(e);
			}
			return false;
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions
			if(this.disabled){
				e.stopPropagation();
				e.preventDefault();
				return false;
			}
			if(this.onClick(e) === false){
				e.preventDefault();
			}
			cancelled = e.defaultPrevented;

			// Signal Form/Dialog to submit/close.  For 2.0, consider removing this code and instead making the Form/Dialog
			// listen for bubbled click events where evt.target.type == "submit" && !evt.defaultPrevented.
			if(!cancelled && this.type == "submit" && !(this.valueNode || this.focusNode).form){
				for(var node = this.domNode; node.parentNode; node = node.parentNode){
					var widget = registry.byNode(node);
					if(widget && typeof widget._onSubmit == "function"){
						widget._onSubmit(e);
						e.preventDefault(); // action has already occurred
						cancelled = true;
						break;
					}
				}
			}

			return !cancelled;
		},

		postCreate: function(){
			this.inherited(arguments);
			dom.setSelectable(this.focusNode, false);
		},

		onClick: function(/*Event*/ /*===== e =====*/){
			// summary:
			//		Callback for when button is clicked.
			//		If type="submit", return true to perform submit, or false to cancel it.
			// type:
			//		callback
			return true;		// Boolean
		},

		_setLabelAttr: function(/*String*/ content){
			// summary:
			//		Hook for set('label', ...) to work.
			// description:
			//		Set the label (text) of the button; takes an HTML string.
			this._set("label", content);
			var labelNode = this.containerNode || this.focusNode;
			labelNode.innerHTML = content;
		}
	});

	if(has("dojo-bidi")){
		ButtonMixin = declare("dijit.form._ButtonMixin", ButtonMixin, {
			_setLabelAttr: function(){
				this.inherited(arguments);
				var labelNode = this.containerNode || this.focusNode;
				this.applyTextDir(labelNode);
			}
		});
	}

	return ButtonMixin;
});

},
'dijit/form/_DateTimeTextBox':function(){
define([
	"dojo/date", // date date.compare
	"dojo/date/locale", // locale.regexp
	"dojo/date/stamp", // stamp.fromISOString stamp.toISOString
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.getObject
	"./RangeBoundTextBox",
	"../_HasDropDown",
	"dojo/text!./templates/DropDownBox.html"
], function(date, locale, stamp, declare, lang, RangeBoundTextBox, _HasDropDown, template){

	// module:
	//		dijit/form/_DateTimeTextBox

	new Date("X"); // workaround for #11279, new Date("") == NaN

	var _DateTimeTextBox = declare("dijit.form._DateTimeTextBox", [RangeBoundTextBox, _HasDropDown], {
		// summary:
		//		Base class for validating, serializable, range-bound date or time text box.

		templateString: template,

		// hasDownArrow: [const] Boolean
		//		Set this textbox to display a down arrow button, to open the drop down list.
		hasDownArrow: true,

		// Set classes like dijitDownArrowButtonHover depending on mouse action over button node
		cssStateNodes: {
			"_buttonNode": "dijitDownArrowButton"
		},

		/*=====
		// constraints: _DateTimeTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including starting/ending dates/times allowed) as well as
		//		formatting options like whether the date is displayed in long (ex: December 25, 2005)
		//		or short (ex: 12/25/2005) format.  See `dijit/form/_DateTimeTextBox.__Constraints` for details.
		constraints: {},
		======*/

		// Override ValidationTextBox.pattern.... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale  (plus formatting options too?)
		pattern: locale.regexp,

		// datePackage: String
		//		JavaScript namespace to find calendar routines.	 If unspecified, uses Gregorian calendar routines
		//		at dojo/date and dojo/date/locale.
		datePackage: "",
		//		TODO: for 2.0, replace datePackage with dateModule and dateLocalModule attributes specifying MIDs,
		//		or alternately just get rid of this completely and tell user to use module ID remapping
		//		via require

		postMixInProperties: function(){
			this.inherited(arguments);
			this._set("type", "text"); // in case type="date"|"time" was specified which messes up parse/format
		},

		// Override _FormWidget.compare() to work for dates/times
		compare: function(/*Date*/ val1, /*Date*/ val2){
			var isInvalid1 = this._isInvalidDate(val1);
			var isInvalid2 = this._isInvalidDate(val2);
			return isInvalid1 ? (isInvalid2 ? 0 : -1) : (isInvalid2 ? 1 : date.compare(val1, val2, this._selector));
		},

		// flag to _HasDropDown to make drop down Calendar width == <input> width
		autoWidth: true,

		format: function(/*Date*/ value, /*locale.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Date, according to specified locale (second argument)
			// tags:
			//		protected
			if(!value){ return ''; }
			return this.dateLocaleModule.format(value, constraints);
		},

		"parse": function(/*String*/ value, /*locale.__FormatOptions*/ constraints){
			// summary:
			//		Parses as string as a Date, according to constraints
			// tags:
			//		protected

			return this.dateLocaleModule.parse(value, constraints) || (this._isEmpty(value) ? null : undefined);	 // Date
		},

		// Overrides ValidationTextBox.serialize() to serialize a date in canonical ISO format.
		serialize: function(/*anything*/ val, /*Object?*/ options){
			if(val.toGregorian){
				val = val.toGregorian();
			}
			return stamp.toISOString(val, options);
		},

		// dropDownDefaultValue: Date
		//		The default value to focus in the popupClass widget when the textbox value is empty.
		dropDownDefaultValue : new Date(),

		// value: Date
		//		The value of this widget as a JavaScript Date object.  Use get("value") / set("value", val) to manipulate.
		//		When passed to the parser in markup, must be specified according to `dojo/date/stamp.fromISOString()`
		value: new Date(""),	// value.toString()="NaN"

		_blankValue: null,	// used by filter() when the textbox is blank

		// popupClass: [protected extension] String
		//		Name of the popup widget class used to select a date/time.
		//		Subclasses should specify this.
		popupClass: "", // default is no popup = text only


		// _selector: [protected extension] String
		//		Specifies constraints.selector passed to dojo.date functions, should be either
		//		"date" or "time".
		//		Subclass must specify this.
		_selector: "",

		constructor: function(params /*===== , srcNodeRef =====*/){
			// summary:
			//		Create the widget.
			// params: Object|null
			//		Hash of initialization parameters for widget, including scalar values (like title, duration etc.)
			//		and functions, typically callbacks like onClick.
			//		The hash can contain any of the widget's properties, excluding read-only properties.
			// srcNodeRef: DOMNode|String?
			//		If a srcNodeRef (DOM node) is specified, replace srcNodeRef with my generated DOM tree

			this.dateModule = params.datePackage ? lang.getObject(params.datePackage, false) : date;
			this.dateClassObj = this.dateModule.Date || Date;
			this.dateLocaleModule = params.datePackage ? lang.getObject(params.datePackage+".locale", false) : locale;
			this._set('pattern', this.dateLocaleModule.regexp);
			this._invalidDate = this.constructor.prototype.value.toString();
		},

		buildRendering: function(){
			this.inherited(arguments);

			if(!this.hasDownArrow){
				this._buttonNode.style.display = "none";
			}

			// If hasDownArrow is false, we basically just want to treat the whole widget as the
			// button.
			if(!this.hasDownArrow){
				this._buttonNode = this.domNode;
				this.baseClass += " dijitComboBoxOpenOnClick";
			}
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			constraints.selector = this._selector;
			constraints.fullYear = true; // see #5465 - always format with 4-digit years
			var fromISO = stamp.fromISOString;
			if(typeof constraints.min == "string"){ constraints.min = fromISO(constraints.min); }
			if(typeof constraints.max == "string"){ constraints.max = fromISO(constraints.max); }
			this.inherited(arguments);
		},

		_isInvalidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking for invalid conditions
			// tags:
			//		private
			return !value || isNaN(value) || typeof value != "object" || value.toString() == this._invalidDate;
		},

		_setValueAttr: function(/*Date|String*/ value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Sets the date on this textbox. Note: value can be a JavaScript Date literal or a string to be parsed.
			if(value !== undefined){
				if(typeof value == "string"){
					value = stamp.fromISOString(value);
				}
				if(this._isInvalidDate(value)){
					value = null;
				}
				if(value instanceof Date && !(this.dateClassObj instanceof Date)){
					value = new this.dateClassObj(value);
				}
			}
			this.inherited(arguments);
			if(this.value instanceof Date){
				this.filterString = "";
			}
			if(this.dropDown){
				this.dropDown.set('value', value, false);
			}
		},

		_set: function(attr, value){
			// Avoid spurious watch() notifications when value is changed to new Date object w/the same value
			var oldValue = this._get("value");
			if(attr == "value" && oldValue instanceof Date && this.compare(value, oldValue) == 0){
				return;
			}
			this.inherited(arguments);
		},

		_setDropDownDefaultValueAttr: function(/*Date*/ val){
			if(this._isInvalidDate(val)){
				// convert null setting into today's date, since there needs to be *some* default at all times.
				 val = new this.dateClassObj();
			}
			this._set("dropDownDefaultValue", val);
		},

		openDropDown: function(/*Function*/ callback){
			// rebuild drop down every time, so that constraints get copied (#6002)
			if(this.dropDown){
				this.dropDown.destroy();
			}
			var PopupProto = lang.isString(this.popupClass) ? lang.getObject(this.popupClass, false) : this.popupClass,
				textBox = this,
				value = this.get("value");
			this.dropDown = new PopupProto({
				onChange: function(value){
					// this will cause InlineEditBox and other handlers to do stuff so make sure it's last
					textBox.set('value', value, true);
				},
				id: this.id + "_popup",
				dir: textBox.dir,
				lang: textBox.lang,
				value: value,
				textDir: textBox.textDir,
				currentFocus: !this._isInvalidDate(value) ? value : this.dropDownDefaultValue,
				constraints: textBox.constraints,
				filterString: textBox.filterString, // for TimeTextBox, to filter times shown
				datePackage: textBox.params.datePackage,
				isDisabledDate: function(/*Date*/ date){
					// summary:
					//		disables dates outside of the min/max of the _DateTimeTextBox
					return !textBox.rangeCheck(date, textBox.constraints);
				}
			});

			this.inherited(arguments);
		},

		_getDisplayedValueAttr: function(){
			return this.textbox.value;
		},

		_setDisplayedValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange){
			this._setValueAttr(this.parse(value, this.constraints), priorityChange, value);
		}
	});


	/*=====
	 _DateTimeTextBox.__Constraints = declare([RangeBoundTextBox.__Constraints, locale.__FormatOptions], {
		 // summary:
		 //		Specifies both the rules on valid/invalid values (first/last date/time allowed),
		 //		and also formatting options for how the date/time is displayed.
		 // example:
		 //		To restrict to dates within 2004, displayed in a long format like "December 25, 2005":
		 //	|		{min:'2004-01-01',max:'2004-12-31', formatLength:'long'}
	 });
	 =====*/

	return _DateTimeTextBox;
});

},
'dijit/form/RangeBoundTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/i18n", // i18n.getLocalization
	"./MappedTextBox"
], function(declare, i18n, MappedTextBox){

	// module:
	//		dijit/form/RangeBoundTextBox


	var RangeBoundTextBox = declare("dijit.form.RangeBoundTextBox", MappedTextBox, {
		// summary:
		//		Base class for textbox form widgets which defines a range of valid values.

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		/*=====
		// constraints: RangeBoundTextBox.__Constraints
		constraints: {},
		======*/

		rangeCheck: function(/*Number*/ primitive, /*dijit/form/RangeBoundTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the range of the numeric input value.
			// tags:
			//		protected
			return	("min" in constraints? (this.compare(primitive,constraints.min) >= 0) : true) &&
				("max" in constraints? (this.compare(primitive,constraints.max) <= 0) : true); // Boolean
		},

		isInRange: function(/*Boolean*/ /*===== isFocused =====*/){
			// summary:
			//		Tests if the value is in the min/max range specified in constraints
			// tags:
			//		protected
			return this.rangeCheck(this.get('value'), this.constraints);
		},

		_isDefinitelyOutOfRange: function(){
			// summary:
			//		Returns true if the value is out of range and will remain
			//		out of range even if the user types more characters
			var val = this.get('value');
			if(val == null){ return false; } // not yet valid enough to compare to
			var outOfRange = false;
			if("min" in this.constraints){
				var min = this.constraints.min;
				outOfRange = this.compare(val, ((typeof min == "number") && min >= 0 && val != 0) ? 0 : min) < 0;
			}
			if(!outOfRange && ("max" in this.constraints)){
				var max = this.constraints.max;
				outOfRange = this.compare(val, ((typeof max != "number") || max > 0) ? max : 0) > 0;
			}
			return outOfRange;
		},

		_isValidSubset: function(){
			// summary:
			//		Overrides `dijit/form/ValidationTextBox._isValidSubset()`.
			//		Returns true if the input is syntactically valid, and either within
			//		range or could be made in range by more typing.
			return this.inherited(arguments) && !this._isDefinitelyOutOfRange();
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit/form/ValidationTextBox.isValid() to check that the value is also in range.
			return this.inherited(arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused)); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ isFocused){
			// Overrides dijit/form/ValidationTextBox.getErrorMessage() to print "out of range" message if appropriate
			var v = this.get('value');
			if(v != null /* and !undefined */ && v !== '' && (typeof v != "number" || !isNaN(v)) && !this.isInRange(isFocused)){ // don't check isInRange w/o a real value
				return this.rangeMessage; // String
			}
			return this.inherited(arguments);
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			if(!this.rangeMessage){
				this.messages = i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		}
	});
	/*=====
	RangeBoundTextBox.__Constraints = declare(null, {
		// min: Number
		//		Minimum signed value.  Default is -Infinity
		// max: Number
		//		Maximum signed value.  Default is +Infinity
	});
	=====*/
	return RangeBoundTextBox;
});

},
'dojo/currency':function(){
define([
	"./_base/array",
	"./_base/lang",
	/*===== "./_base/declare", =====*/
	"./number",
	"./i18n", "./i18n!./cldr/nls/currency",
	"./cldr/monetary"
], function(darray, lang, /*===== declare, =====*/ dnumber, i18n, nlsCurrency, cldrMonetary){

// module:
//		dojo/currency

var currency = {
	// summary:
	//		localized formatting and parsing routines for currencies
	// description:
	//		extends dojo.number to provide culturally-appropriate formatting of values
	//		in various world currencies, including use of a currency symbol.  The currencies are specified
	//		by a three-letter international symbol in all uppercase, and support for the currencies is
	//		provided by the data in `dojo.cldr`.  The scripts generating dojo.cldr specify which
	//		currency support is included.  A fixed number of decimal places is determined based
	//		on the currency type and is not determined by the 'pattern' argument.  The fractional
	//		portion is optional, by default, and variable length decimals are not supported.
};
lang.setObject("dojo.currency", currency);

currency._mixInDefaults = function(options){
	options = options || {};
	options.type = "currency";

	// Get locale-dependent currency data, like the symbol
	var bundle = i18n.getLocalization("dojo.cldr", "currency", options.locale) || {};

	// Mixin locale-independent currency data, like # of places
	var iso = options.currency;
	var data = cldrMonetary.getData(iso);

	darray.forEach(["displayName","symbol","group","decimal"], function(prop){
		data[prop] = bundle[iso+"_"+prop];
	});

	data.fractional = [true, false];

	// Mixin with provided options
	return lang.mixin(data, options);
};

/*=====
currency.__FormatOptions = declare([dnumber.__FormatOptions], {
	// type: String?
	//		Should not be set.  Value is assumed to be "currency".
	// symbol: String?
	//		localized currency symbol. The default will be looked up in table of supported currencies in `dojo.cldr`
	//		A [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code will be used if not found.
	// currency: String?
	//		an [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code, a three letter sequence like "USD".
	//		For use with dojo.currency only.
	// places: Number?
	//		number of decimal places to show.  Default is defined based on which currency is used.
	type: "",
	symbol: "",
	currency: "",
	places: ""
});
=====*/

currency.format = function(/*Number*/ value, /*__FormatOptions?*/ options){
	// summary:
	//		Format a Number as a currency, using locale-specific settings
	//
	// description:
	//		Create a string from a Number using a known, localized pattern.
	//		[Formatting patterns](http://www.unicode.org/reports/tr35/#Number_Elements)
	//		appropriate to the locale are chosen from the [CLDR](http://unicode.org/cldr)
	//		as well as the appropriate symbols and delimiters and number of decimal places.
	//
	// value:
	//		the number to be formatted.

	return dnumber.format(value, currency._mixInDefaults(options));
};

currency.regexp = function(/*dnumber.__RegexpOptions?*/ options){
	//
	// summary:
	//		Builds the regular needed to parse a currency value
	//
	// description:
	//		Returns regular expression with positive and negative match, group and decimal separators
	//		Note: the options.places default, the number of decimal places to accept, is defined by the currency type.
	return dnumber.regexp(currency._mixInDefaults(options)); // String
};

/*=====
var __ParseOptions = currency.__ParseOptions = declare(dnumber.__ParseOptions, {
	// type: String?
	//		Should not be set.  Value is assumed to be currency.
	// currency: String?
	//		an [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code, a three letter sequence like "USD".
	//		For use with dojo.currency only.
	// symbol: String?
	//		localized currency symbol. The default will be looked up in table of supported currencies in `dojo.cldr`
	//		A [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code will be used if not found.
	// places: Number?
	//		fixed number of decimal places to accept.  The default is determined based on which currency is used.
	// fractional: Boolean|Array?
	//		Whether to include the fractional portion, where the number of decimal places are implied by the currency
	//		or explicit 'places' parameter.  The value [true,false] makes the fractional portion optional.
	//		By default for currencies, it the fractional portion is optional.
});
=====*/

currency.parse = function(/*String*/ expression, /*__ParseOptions?*/ options){
	//
	// summary:
	//		Convert a properly formatted currency string to a primitive Number,
	//		using locale-specific settings.
	// description:
	//		Create a Number from a string using a known, localized pattern.
	//		[Formatting patterns](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		are chosen appropriate to the locale, as well as the appropriate symbols and delimiters
	//		and number of decimal places.
	// expression:
	//		A string representation of a currency value

	return dnumber.parse(expression, currency._mixInDefaults(options));
};

return currency;
});

},
'dojo/number':function(){
define([/*===== "./_base/declare", =====*/ "./_base/lang", "./i18n", "./i18n!./cldr/nls/number", "./string", "./regexp"],
	function(/*===== declare, =====*/ lang, i18n, nlsNumber, dstring, dregexp){

// module:
//		dojo/number

var number = {
	// summary:
	//		localized formatting and parsing routines for Number
};
lang.setObject("dojo.number", number);

/*=====
number.__FormatOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// places: Number?
	//		fixed number of decimal places to show.  This overrides any
	//		information in the provided pattern.
	// round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means do not round.
	// locale: String?
	//		override the locale used to determine formatting rules
	// fractional: Boolean?
	//		If false, show no decimal places, overriding places and pattern settings.
});
=====*/

number.format = function(/*Number*/ value, /*number.__FormatOptions?*/ options){
	// summary:
	//		Format a Number as a String, using locale-specific settings
	// description:
	//		Create a string from a Number using a known localized pattern.
	//		Formatting patterns appropriate to the locale are chosen from the
	//		[Common Locale Data Repository](http://unicode.org/cldr) as well as the appropriate symbols and
	//		delimiters.
	//		If value is Infinity, -Infinity, or is not a valid JavaScript number, return null.
	// value:
	//		the number to be formatted

	options = lang.mixin({}, options || {});
	var locale = i18n.normalizeLocale(options.locale),
		bundle = i18n.getLocalization("dojo.cldr", "number", locale);
	options.customs = bundle;
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
	if(isNaN(value) || Math.abs(value) == Infinity){ return null; } // null
	return number._applyPattern(value, pattern, options); // String
};

//number._numberPatternRE = /(?:[#0]*,?)*[#0](?:\.0*#*)?/; // not precise, but good enough
number._numberPatternRE = /[#0,]*[#0](?:\.0*#*)?/; // not precise, but good enough

number._applyPattern = function(/*Number*/ value, /*String*/ pattern, /*number.__FormatOptions?*/ options){
	// summary:
	//		Apply pattern to format value as a string using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted.
	// pattern:
	//		a pattern string as described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	// options: number.__FormatOptions?
	//		_applyPattern is usually called via `dojo/number.format()` which
	//		populates an extra property in the options parameter, "customs".
	//		The customs object specifies group and decimal parameters if set.

	//TODO: support escapes
	options = options || {};
	var group = options.customs.group,
		decimal = options.customs.decimal,
		patternList = pattern.split(';'),
		positivePattern = patternList[0];
	pattern = patternList[(value < 0) ? 1 : 0] || ("-" + positivePattern);

	//TODO: only test against unescaped
	if(pattern.indexOf('%') != -1){
		value *= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		value *= 1000; // per mille
	}else if(pattern.indexOf('\u00a4') != -1){
		group = options.customs.currencyGroup || group;//mixins instead?
		decimal = options.customs.currencyDecimal || decimal;// Should these be mixins instead?
		pattern = pattern.replace(/\u00a4{1,3}/, function(match){
			var prop = ["symbol", "currency", "displayName"][match.length-1];
			return options[prop] || options.currency || "";
		});
	}else if(pattern.indexOf('E') != -1){
		throw new Error("exponential notation not supported");
	}

	//TODO: support @ sig figs?
	var numberPatternRE = number._numberPatternRE;
	var numberPattern = positivePattern.match(numberPatternRE);
	if(!numberPattern){
		throw new Error("unable to find a number expression in pattern: "+pattern);
	}
	if(options.fractional === false){ options.places = 0; }
	return pattern.replace(numberPatternRE,
		number._formatAbsolute(value, numberPattern[0], {decimal: decimal, group: group, places: options.places, round: options.round}));
};

number.round = function(/*Number*/ value, /*Number?*/ places, /*Number?*/ increment){
	// summary:
	//		Rounds to the nearest value with the given number of decimal places, away from zero
	// description:
	//		Rounds to the nearest value with the given number of decimal places, away from zero if equal.
	//		Similar to Number.toFixed(), but compensates for browser quirks. Rounding can be done by
	//		fractional increments also, such as the nearest quarter.
	//		NOTE: Subject to floating point errors.  See dojox/math/round for experimental workaround.
	// value:
	//		The number to round
	// places:
	//		The number of decimal places where rounding takes place.  Defaults to 0 for whole rounding.
	//		Must be non-negative.
	// increment:
	//		Rounds next place to nearest value of increment/10.  10 by default.
	// example:
	// |	>>> number.round(-0.5)
	// |	-1
	// |	>>> number.round(162.295, 2)
	// |	162.29  // note floating point error.  Should be 162.3
	// |	>>> number.round(10.71, 0, 2.5)
	// |	10.75
	var factor = 10 / (increment || 10);
	return (factor * +value).toFixed(places) / factor; // Number
};

if((0.9).toFixed() == 0){
	// (isIE) toFixed() bug workaround: Rounding fails on IE when most significant digit
	// is just after the rounding place and is >=5
	var round = number.round;
	number.round = function(v, p, m){
		var d = Math.pow(10, -p || 0), a = Math.abs(v);
		if(!v || a >= d){
			d = 0;
		}else{
			a /= d;
			if(a < 0.5 || a >= 0.95){
				d = 0;
			}
		}
		return round(v, p, m) + (v > 0 ? d : -d);
	};

	// Use "doc hint" so the doc parser ignores this new definition of round(), and uses the one above.
	/*===== number.round = round; =====*/
}

/*=====
number.__FormatAbsoluteOptions = declare(null, {
	// decimal: String?
	//		the decimal separator
	// group: String?
	//		the group separator
	// places: Number|String?
	//		number of decimal places.  the range "n,m" will format to m places.
	// round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
});
=====*/

number._formatAbsolute = function(/*Number*/ value, /*String*/ pattern, /*number.__FormatAbsoluteOptions?*/ options){
	// summary:
	//		Apply numeric pattern to absolute value using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted, ignores sign
	// pattern:
	//		the number portion of a pattern (e.g. `#,##0.00`)
	options = options || {};
	if(options.places === true){options.places=0;}
	if(options.places === Infinity){options.places=6;} // avoid a loop; pick a limit

	var patternParts = pattern.split("."),
		comma = typeof options.places == "string" && options.places.indexOf(","),
		maxPlaces = options.places;
	if(comma){
		maxPlaces = options.places.substring(comma + 1);
	}else if(!(maxPlaces >= 0)){
		maxPlaces = (patternParts[1] || []).length;
	}
	if(!(options.round < 0)){
		value = number.round(value, maxPlaces, options.round);
	}

	var valueParts = String(Math.abs(value)).split("."),
		fractional = valueParts[1] || "";
	if(patternParts[1] || options.places){
		if(comma){
			options.places = options.places.substring(0, comma);
		}
		// Pad fractional with trailing zeros
		var pad = options.places !== undefined ? options.places : (patternParts[1] && patternParts[1].lastIndexOf("0") + 1);
		if(pad > fractional.length){
			valueParts[1] = dstring.pad(fractional, pad, '0', true);
		}

		// Truncate fractional
		if(maxPlaces < fractional.length){
			valueParts[1] = fractional.substr(0, maxPlaces);
		}
	}else{
		if(valueParts[1]){ valueParts.pop(); }
	}

	// Pad whole with leading zeros
	var patternDigits = patternParts[0].replace(',', '');
	pad = patternDigits.indexOf("0");
	if(pad != -1){
		pad = patternDigits.length - pad;
		if(pad > valueParts[0].length){
			valueParts[0] = dstring.pad(valueParts[0], pad);
		}

		// Truncate whole
		if(patternDigits.indexOf("#") == -1){
			valueParts[0] = valueParts[0].substr(valueParts[0].length - pad);
		}
	}

	// Add group separators
	var index = patternParts[0].lastIndexOf(','),
		groupSize, groupSize2;
	if(index != -1){
		groupSize = patternParts[0].length - index - 1;
		var remainder = patternParts[0].substr(0, index);
		index = remainder.lastIndexOf(',');
		if(index != -1){
			groupSize2 = remainder.length - index - 1;
		}
	}
	var pieces = [];
	for(var whole = valueParts[0]; whole;){
		var off = whole.length - groupSize;
		pieces.push((off > 0) ? whole.substr(off) : whole);
		whole = (off > 0) ? whole.slice(0, off) : "";
		if(groupSize2){
			groupSize = groupSize2;
			delete groupSize2;
		}
	}
	valueParts[0] = pieces.reverse().join(options.group || ",");

	return valueParts.join(options.decimal || ".");
};

/*=====
number.__RegexpOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// locale: String?
	//		override the locale used to determine formatting rules
	// strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	// places: Number|String?
	//		number of decimal places to accept: Infinity, a positive number, or
	//		a range "n,m".  Defined by pattern or Infinity if pattern not provided.
});
=====*/
number.regexp = function(/*number.__RegexpOptions?*/ options){
	// summary:
	//		Builds the regular needed to parse a number
	// description:
	//		Returns regular expression with positive and negative match, group
	//		and decimal separators
	return number._parseInfo(options).regexp; // String
};

number._parseInfo = function(/*Object?*/ options){
	options = options || {};
	var locale = i18n.normalizeLocale(options.locale),
		bundle = i18n.getLocalization("dojo.cldr", "number", locale),
		pattern = options.pattern || bundle[(options.type || "decimal") + "Format"],
//TODO: memoize?
		group = bundle.group,
		decimal = bundle.decimal,
		factor = 1;

	if(pattern.indexOf('%') != -1){
		factor /= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		factor /= 1000; // per mille
	}else{
		var isCurrency = pattern.indexOf('\u00a4') != -1;
		if(isCurrency){
			group = bundle.currencyGroup || group;
			decimal = bundle.currencyDecimal || decimal;
		}
	}

	//TODO: handle quoted escapes
	var patternList = pattern.split(';');
	if(patternList.length == 1){
		patternList.push("-" + patternList[0]);
	}

	var re = dregexp.buildGroupRE(patternList, function(pattern){
		pattern = "(?:"+dregexp.escapeString(pattern, '.')+")";
		return pattern.replace(number._numberPatternRE, function(format){
			var flags = {
				signed: false,
				separator: options.strict ? group : [group,""],
				fractional: options.fractional,
				decimal: decimal,
				exponent: false
				},

				parts = format.split('.'),
				places = options.places;

			// special condition for percent (factor != 1)
			// allow decimal places even if not specified in pattern
			if(parts.length == 1 && factor != 1){
			    parts[1] = "###";
			}
			if(parts.length == 1 || places === 0){
				flags.fractional = false;
			}else{
				if(places === undefined){ places = options.pattern ? parts[1].lastIndexOf('0') + 1 : Infinity; }
				if(places && options.fractional == undefined){flags.fractional = true;} // required fractional, unless otherwise specified
				if(!options.places && (places < parts[1].length)){ places += "," + parts[1].length; }
				flags.places = places;
			}
			var groups = parts[0].split(',');
			if(groups.length > 1){
				flags.groupSize = groups.pop().length;
				if(groups.length > 1){
					flags.groupSize2 = groups.pop().length;
				}
			}
			return "("+number._realNumberRegexp(flags)+")";
		});
	}, true);

	if(isCurrency){
		// substitute the currency symbol for the placeholder in the pattern
		re = re.replace(/([\s\xa0]*)(\u00a4{1,3})([\s\xa0]*)/g, function(match, before, target, after){
			var prop = ["symbol", "currency", "displayName"][target.length-1],
				symbol = dregexp.escapeString(options[prop] || options.currency || "");
			before = before ? "[\\s\\xa0]" : "";
			after = after ? "[\\s\\xa0]" : "";
			if(!options.strict){
				if(before){before += "*";}
				if(after){after += "*";}
				return "(?:"+before+symbol+after+")?";
			}
			return before+symbol+after;
		});
	}

//TODO: substitute localized sign/percent/permille/etc.?

	// normalize whitespace and return
	return {regexp: re.replace(/[\xa0 ]/g, "[\\s\\xa0]"), group: group, decimal: decimal, factor: factor}; // Object
};

/*=====
number.__ParseOptions = declare(null, {
	// pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	// type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	// locale: String?
	//		override the locale used to determine formatting rules
	// strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	// fractional: Boolean|Array?
	//		Whether to include the fractional portion, where the number of decimal places are implied by pattern
	//		or explicit 'places' parameter.  The value [true,false] makes the fractional portion optional.
});
=====*/
number.parse = function(/*String*/ expression, /*number.__ParseOptions?*/ options){
	// summary:
	//		Convert a properly formatted string to a primitive Number, using
	//		locale-specific settings.
	// description:
	//		Create a Number from a string using a known localized pattern.
	//		Formatting patterns are chosen appropriate to the locale
	//		and follow the syntax described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
    	//		Note that literal characters in patterns are not supported.
	// expression:
	//		A string representation of a Number
	var info = number._parseInfo(options),
		results = (new RegExp("^"+info.regexp+"$")).exec(expression);
	if(!results){
		return NaN; //NaN
	}
	var absoluteMatch = results[1]; // match for the positive expression
	if(!results[1]){
		if(!results[2]){
			return NaN; //NaN
		}
		// matched the negative pattern
		absoluteMatch =results[2];
		info.factor *= -1;
	}

	// Transform it to something Javascript can parse as a number.  Normalize
	// decimal point and strip out group separators or alternate forms of whitespace
	absoluteMatch = absoluteMatch.
		replace(new RegExp("["+info.group + "\\s\\xa0"+"]", "g"), "").
		replace(info.decimal, ".");
	// Adjust for negative sign, percent, etc. as necessary
	return absoluteMatch * info.factor; //Number
};

/*=====
number.__RealNumberRegexpFlags = declare(null, {
	// places: Number?
	//		The integer number of decimal places or a range given as "n,m".  If
	//		not given, the decimal part is optional and the number of places is
	//		unlimited.
	// decimal: String?
	//		A string for the character used as the decimal point.  Default
	//		is ".".
	// fractional: Boolean|Array?
	//		Whether decimal places are used.  Can be true, false, or [true,
	//		false].  Default is [true, false] which means optional.
	// exponent: Boolean|Array?
	//		Express in exponential notation.  Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if the
	//		exponential part is present are not).
	// eSigned: Boolean|Array?
	//		The leading plus-or-minus sign on the exponent.  Can be true,
	//		false, or [true, false].  Default is [true, false], (i.e. will
	//		match if it is signed or unsigned).  flags in regexp.integer can be
	//		applied.
});
=====*/

number._realNumberRegexp = function(/*__RealNumberRegexpFlags?*/ flags){
	// summary:
	//		Builds a regular expression to match a real number in exponential
	//		notation

	// assign default values to missing parameters
	flags = flags || {};
	//TODO: use mixin instead?
	if(!("places" in flags)){ flags.places = Infinity; }
	if(typeof flags.decimal != "string"){ flags.decimal = "."; }
	if(!("fractional" in flags) || /^0/.test(flags.places)){ flags.fractional = [true, false]; }
	if(!("exponent" in flags)){ flags.exponent = [true, false]; }
	if(!("eSigned" in flags)){ flags.eSigned = [true, false]; }

	var integerRE = number._integerRegexp(flags),
		decimalRE = dregexp.buildGroupRE(flags.fractional,
		function(q){
			var re = "";
			if(q && (flags.places!==0)){
				re = "\\" + flags.decimal;
				if(flags.places == Infinity){
					re = "(?:" + re + "\\d+)?";
				}else{
					re += "\\d{" + flags.places + "}";
				}
			}
			return re;
		},
		true
	);

	var exponentRE = dregexp.buildGroupRE(flags.exponent,
		function(q){
			if(q){ return "([eE]" + number._integerRegexp({ signed: flags.eSigned}) + ")"; }
			return "";
		}
	);

	var realRE = integerRE + decimalRE;
	// allow for decimals without integers, e.g. .25
	if(decimalRE){realRE = "(?:(?:"+ realRE + ")|(?:" + decimalRE + "))";}
	return realRE + exponentRE; // String
};

/*=====
number.__IntegerRegexpFlags = declare(null, {
	// signed: Boolean?
	//		The leading plus-or-minus sign. Can be true, false, or `[true,false]`.
	//		Default is `[true, false]`, (i.e. will match if it is signed
	//		or unsigned).
	// separator: String?
	//		The character used as the thousands separator. Default is no
	//		separator. For more than one symbol use an array, e.g. `[",", ""]`,
	//		makes ',' optional.
	// groupSize: Number?
	//		group size between separators
	// groupSize2: Number?
	//		second grouping, where separators 2..n have a different interval than the first separator (for India)
});
=====*/

number._integerRegexp = function(/*number.__IntegerRegexpFlags?*/ flags){
	// summary:
	//		Builds a regular expression that matches an integer

	// assign default values to missing parameters
	flags = flags || {};
	if(!("signed" in flags)){ flags.signed = [true, false]; }
	if(!("separator" in flags)){
		flags.separator = "";
	}else if(!("groupSize" in flags)){
		flags.groupSize = 3;
	}

	var signRE = dregexp.buildGroupRE(flags.signed,
		function(q){ return q ? "[-+]" : ""; },
		true
	);

	var numberRE = dregexp.buildGroupRE(flags.separator,
		function(sep){
			if(!sep){
				return "(?:\\d+)";
			}

			sep = dregexp.escapeString(sep);
			if(sep == " "){ sep = "\\s"; }
			else if(sep == "\xa0"){ sep = "\\s\\xa0"; }

			var grp = flags.groupSize, grp2 = flags.groupSize2;
			//TODO: should we continue to enforce that numbers with separators begin with 1-9?  See #6933
			if(grp2){
				var grp2RE = "(?:0|[1-9]\\d{0," + (grp2-1) + "}(?:[" + sep + "]\\d{" + grp2 + "})*[" + sep + "]\\d{" + grp + "})";
				return ((grp-grp2) > 0) ? "(?:" + grp2RE + "|(?:0|[1-9]\\d{0," + (grp-1) + "}))" : grp2RE;
			}
			return "(?:0|[1-9]\\d{0," + (grp-1) + "}(?:[" + sep + "]\\d{" + grp + "})*)";
		},
		true
	);

	return signRE + numberRE; // String
};

return number;
});

},
'dojo/cldr/monetary':function(){
define(["../_base/kernel", "../_base/lang"], function(dojo, lang){

// module:
//		dojo/cldr/monetary

var monetary = {
	// summary:
	//		TODOC
};
lang.setObject("dojo.cldr.monetary", monetary);

monetary.getData = function(/*String*/ code){
	// summary:
	//		A mapping of currency code to currency-specific formatting information. Returns a unique object with properties: places, round.
	// code:
	//		an [ISO 4217](http://en.wikipedia.org/wiki/ISO_4217) currency code

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/currencyData/fractions

	var placesData = {
		ADP:0,AFN:0,ALL:0,AMD:0,BHD:3,BIF:0,BYR:0,CLF:0,CLP:0,
		COP:0,CRC:0,DJF:0,ESP:0,GNF:0,GYD:0,HUF:0,IDR:0,IQD:0,
		IRR:3,ISK:0,ITL:0,JOD:3,JPY:0,KMF:0,KPW:0,KRW:0,KWD:3,
		LAK:0,LBP:0,LUF:0,LYD:3,MGA:0,MGF:0,MMK:0,MNT:0,MRO:0,
		MUR:0,OMR:3,PKR:0,PYG:0,RSD:0,RWF:0,SLL:0,SOS:0,STD:0,
		SYP:0,TMM:0,TND:3,TRL:0,TZS:0,UGX:0,UZS:0,VND:0,VUV:0,
		XAF:0,XOF:0,XPF:0,YER:0,ZMK:0,ZWD:0
	};

	var roundingData = {};

	var places = placesData[code], round = roundingData[code];
	if(typeof places == "undefined"){ places = 2; }
	if(typeof round == "undefined"){ round = 0; }

	return {places: places, round: round}; // Object
};

return monetary;
});

},
'dojo/dnd/Source':function(){
define([
	"../_base/array", "../_base/declare", "../_base/kernel", "../_base/lang",
	"../dom-class", "../dom-geometry", "../mouse", "../ready", "../topic",
	"./common", "./Selector", "./Manager"
], function(array, declare, kernel, lang, domClass, domGeom, mouse, ready, topic,
			dnd, Selector, Manager){

// module:
//		dojo/dnd/Source

/*
	Container property:
		"Horizontal"- if this is the horizontal container
	Source states:
		""			- normal state
		"Moved"		- this source is being moved
		"Copied"	- this source is being copied
	Target states:
		""			- normal state
		"Disabled"	- the target cannot accept an avatar
	Target anchor state:
		""			- item is not selected
		"Before"	- insert point is before the anchor
		"After"		- insert point is after the anchor
*/

/*=====
var __SourceArgs = {
	// summary:
	//		a dict of parameters for DnD Source configuration. Note that any
	//		property on Source elements may be configured, but this is the
	//		short-list
	// isSource: Boolean?
	//		can be used as a DnD source. Defaults to true.
	// accept: Array?
	//		list of accepted types (text strings) for a target; defaults to
	//		["text"]
	// autoSync: Boolean
	//		if true refreshes the node list on every operation; false by default
	// copyOnly: Boolean?
	//		copy items, if true, use a state of Ctrl key otherwise,
	//		see selfCopy and selfAccept for more details
	// delay: Number
	//		the move delay in pixels before detecting a drag; 0 by default
	// horizontal: Boolean?
	//		a horizontal container, if true, vertical otherwise or when omitted
	// selfCopy: Boolean?
	//		copy items by default when dropping on itself,
	//		false by default, works only if copyOnly is true
	// selfAccept: Boolean?
	//		accept its own items when copyOnly is true,
	//		true by default, works only if copyOnly is true
	// withHandles: Boolean?
	//		allows dragging only by handles, false by default
	// generateText: Boolean?
	//		generate text node for drag and drop, true by default
};
=====*/

// For back-compat, remove in 2.0.
if(!kernel.isAsync){
	ready(0, function(){
		var requires = ["dojo/dnd/AutoSource", "dojo/dnd/Target"];
		require(requires);	// use indirection so modules not rolled into a build
	});
}

var Source = declare("dojo.dnd.Source", Selector, {
	// summary:
	//		a Source object, which can be used as a DnD source, or a DnD target

	// object attributes (for markup)
	isSource: true,
	horizontal: false,
	copyOnly: false,
	selfCopy: false,
	selfAccept: true,
	skipForm: false,
	withHandles: false,
	autoSync: false,
	delay: 0, // pixels
	accept: ["text"],
	generateText: true,

	constructor: function(/*DOMNode|String*/ node, /*__SourceArgs?*/ params){
		// summary:
		//		a constructor of the Source
		// node:
		//		node or node's id to build the source on
		// params:
		//		any property of this class may be configured via the params
		//		object which is mixed-in to the `dojo/dnd/Source` instance
		lang.mixin(this, lang.mixin({}, params));
		var type = this.accept;
		if(type.length){
			this.accept = {};
			for(var i = 0; i < type.length; ++i){
				this.accept[type[i]] = 1;
			}
		}
		// class-specific variables
		this.isDragging = false;
		this.mouseDown = false;
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
		this._lastX = 0;
		this._lastY = 0;
		// states
		this.sourceState  = "";
		if(this.isSource){
			domClass.add(this.node, "dojoDndSource");
		}
		this.targetState  = "";
		if(this.accept){
			domClass.add(this.node, "dojoDndTarget");
		}
		if(this.horizontal){
			domClass.add(this.node, "dojoDndHorizontal");
		}
		// set up events
		this.topics = [
			topic.subscribe("/dnd/source/over", lang.hitch(this, "onDndSourceOver")),
			topic.subscribe("/dnd/start",  lang.hitch(this, "onDndStart")),
			topic.subscribe("/dnd/drop",   lang.hitch(this, "onDndDrop")),
			topic.subscribe("/dnd/cancel", lang.hitch(this, "onDndCancel"))
		];
	},

	// methods
	checkAcceptance: function(source, nodes){
		// summary:
		//		checks if the target can accept nodes from this source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		if(this == source){
			return !this.copyOnly || this.selfAccept;
		}
		for(var i = 0; i < nodes.length; ++i){
			var type = source.getItem(nodes[i].id).type;
			// type instanceof Array
			var flag = false;
			for(var j = 0; j < type.length; ++j){
				if(type[j] in this.accept){
					flag = true;
					break;
				}
			}
			if(!flag){
				return false;	// Boolean
			}
		}
		return true;	// Boolean
	},
	copyState: function(keyPressed, self){
		// summary:
		//		Returns true if we need to copy items, false to move.
		//		It is separated to be overwritten dynamically, if needed.
		// keyPressed: Boolean
		//		the "copy" key was pressed
		// self: Boolean?
		//		optional flag that means that we are about to drop on itself

		if(keyPressed){ return true; }
		if(arguments.length < 2){
			self = this == Manager.manager().target;
		}
		if(self){
			if(this.copyOnly){
				return this.selfCopy;
			}
		}else{
			return this.copyOnly;
		}
		return false;	// Boolean
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		Source.superclass.destroy.call(this);
		array.forEach(this.topics, function(t){t.remove();});
		this.targetAnchor = null;
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		Source.superclass.onMouseMove.call(this, e);
		var m = Manager.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dnd.getCopyKeyState(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = domGeom.position(this.current, true);
				}
				if(this.horizontal){
					// In LTR mode, the left part of the object means "before", but in RTL mode it means "after".
					before = (e.pageX - this.targetBox.x < this.targetBox.w / 2) == domGeom.isBodyLtr(this.current.ownerDocument);
				}else{
					before = (e.pageY - this.targetBox.y) < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	},
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(!this.mouseDown && this._legalMouseDown(e) && (!this.skipForm || !dnd.isFormElement(e))){
			this.mouseDown = true;
			this._lastX = e.pageX;
			this._lastY = e.pageY;
			Source.superclass.onMouseDown.call(this, e);
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.mouseDown){
			this.mouseDown = false;
			Source.superclass.onMouseUp.call(this, e);
		}
	},

	// topic event processors
	onDndSourceOver: function(source){
		// summary:
		//		topic event processor for /dnd/source/over, called when detected a current source
		// source: Object
		//		the source which has the mouse over it
		if(this !== source){
			this.mouseDown = false;
			if(this.targetAnchor){
				this._unmarkTargetAnchor();
			}
		}else if(this.isDragging){
			var m = Manager.manager();
			m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
		}
	},
	onDndStart: function(source, nodes, copy){
		// summary:
		//		topic event processor for /dnd/start, called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		if(this.autoSync){ this.sync(); }
		if(this.isSource){
			this._changeState("Source", this == source ? (copy ? "Copied" : "Moved") : "");
		}
		var accepted = this.accept && this.checkAcceptance(source, nodes);
		this._changeState("Target", accepted ? "" : "Disabled");
		if(this == source){
			Manager.manager().overSource(this);
		}
		this.isDragging = true;
	},
	onDndDrop: function(source, nodes, copy, target){
		// summary:
		//		topic event processor for /dnd/drop, called to finish the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		// target: Object
		//		the target which accepts items
		if(this == target){
			// this one is for us => move nodes!
			this.onDrop(source, nodes, copy);
		}
		this.onDndCancel();
	},
	onDndCancel: function(){
		// summary:
		//		topic event processor for /dnd/cancel, called to cancel the DnD operation
		if(this.targetAnchor){
			this._unmarkTargetAnchor();
			this.targetAnchor = null;
		}
		this.before = true;
		this.isDragging = false;
		this.mouseDown = false;
		this._changeState("Source", "");
		this._changeState("Target", "");
	},

	// local events
	onDrop: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		if(this != source){
			this.onDropExternal(source, nodes, copy);
		}else{
			this.onDropInternal(nodes, copy);
		}
	},
	onDropExternal: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from an external source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		var oldCreator = this._normalizedCreator;
		// transferring nodes from the source to the target
		if(this.creator){
			// use defined creator
			this._normalizedCreator = function(node, hint){
				return oldCreator.call(this, source.getItem(node.id).data, hint);
			};
		}else{
			// we have no creator defined => move/clone nodes
			if(copy){
				// clone nodes
				this._normalizedCreator = function(node /*=====, hint =====*/){
					var t = source.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}else{
				// move nodes
				this._normalizedCreator = function(node /*=====, hint =====*/){
					var t = source.getItem(node.id);
					source.delItem(node.id);
					return {node: node, data: t.data, type: t.type};
				};
			}
		}
		this.selectNone();
		if(!copy && !this.creator){
			source.selectNone();
		}
		this.insertNodes(true, nodes, this.before, this.current);
		if(!copy && this.creator){
			source.deleteSelectedNodes();
		}
		this._normalizedCreator = oldCreator;
	},
	onDropInternal: function(nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from the same target/source
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise

		var oldCreator = this._normalizedCreator;
		// transferring nodes within the single source
		if(this.current && this.current.id in this.selection){
			// do nothing
			return;
		}
		if(copy){
			if(this.creator){
				// create new copies of data items
				this._normalizedCreator = function(node, hint){
					return oldCreator.call(this, this.getItem(node.id).data, hint);
				};
			}else{
				// clone nodes
				this._normalizedCreator = function(node/*=====, hint =====*/){
					var t = this.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}
		}else{
			// move nodes
			if(!this.current){
				// do nothing
				return;
			}
			this._normalizedCreator = function(node /*=====, hint =====*/){
				var t = this.getItem(node.id);
				return {node: node, data: t.data, type: t.type};
			};
		}
		this._removeSelection();
		this.insertNodes(true, nodes, this.before, this.current);
		this._normalizedCreator = oldCreator;
	},
	onDraggingOver: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged over this target, and it is not disabled
	},
	onDraggingOut: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged away from this target, and it is not disabled
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		Source.superclass.onOverEvent.call(this);
		Manager.manager().overSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOver();
		}
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		Source.superclass.onOutEvent.call(this);
		Manager.manager().outSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOut();
		}
	},
	_markTargetAnchor: function(before){
		// summary:
		//		assigns a class to the current target anchor based on "before" status
		// before: Boolean
		//		insert before, if true, after otherwise
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
	},
	_unmarkTargetAnchor: function(){
		// summary:
		//		removes a class of the current target anchor based on "before" status
		if(!this.targetAnchor){ return; }
		this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
	},
	_markDndStatus: function(copy){
		// summary:
		//		changes source's state based on "copy" status
		this._changeState("Source", copy ? "Copied" : "Moved");
	},
	_legalMouseDown: function(e){
		// summary:
		//		checks if user clicked on "approved" items
		// e: Event
		//		mouse event

		// accept only the left mouse button, or the left finger
		if(e.type != "touchstart" && !mouse.isLeft(e)){ return false; }

		if(!this.withHandles){ return true; }

		// check for handles
		for(var node = e.target; node && node !== this.node; node = node.parentNode){
			if(domClass.contains(node, "dojoDndHandle")){ return true; }
			if(domClass.contains(node, "dojoDndItem") || domClass.contains(node, "dojoDndIgnore")){ break; }
		}
		return false;	// Boolean
	}
});

return Source;

});

},
'dojo/dnd/Selector':function(){
define([
	"../_base/array", "../_base/declare", "../_base/kernel", "../_base/lang",
	"../dom", "../dom-construct", "../mouse", "../_base/NodeList", "../on", "../touch", "./common", "./Container"
], function(array, declare, kernel, lang, dom, domConstruct, mouse, NodeList, on, touch, dnd, Container){

// module:
//		dojo/dnd/Selector

/*
	Container item states:
		""			- an item is not selected
		"Selected"	- an item is selected
		"Anchor"	- an item is selected, and is an anchor for a "shift" selection
*/

/*=====
var __SelectorArgs = declare([Container.__ContainerArgs], {
	// singular: Boolean
	//		allows selection of only one element, if true
	singular: false,

	// autoSync: Boolean
	//		autosynchronizes the source with its list of DnD nodes,
	autoSync: false
});
=====*/

var Selector = declare("dojo.dnd.Selector", Container, {
	// summary:
	//		a Selector object, which knows how to select its children

	/*=====
	// selection: Set<String>
	//		The set of id's that are currently selected, such that this.selection[id] == 1
	//		if the node w/that id is selected.  Can iterate over selected node's id's like:
	//	|		for(var id in this.selection)
	selection: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		constructor of the Selector
		// node: Node||String
		//		node or node's id to build the selector on
		// params: __SelectorArgs?
		//		a dictionary of parameters
		if(!params){ params = {}; }
		this.singular = params.singular;
		this.autoSync = params.autoSync;
		// class-specific variables
		this.selection = {};
		this.anchor = null;
		this.simpleSelection = false;
		// set up events
		this.events.push(
			on(this.node, touch.press, lang.hitch(this, "onMouseDown")),
			on(this.node, touch.release, lang.hitch(this, "onMouseUp"))
		);
	},

	// object attributes (for markup)
	singular: false,	// is singular property

	// methods
	getSelectedNodes: function(){
		// summary:
		//		returns a list (an array) of selected nodes
		var t = new NodeList();
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			t.push(dom.byId(i));
		}
		return t;	// NodeList
	},
	selectNone: function(){
		// summary:
		//		unselects all items
		return this._removeSelection()._removeAnchor();	// self
	},
	selectAll: function(){
		// summary:
		//		selects all items
		this.forInItems(function(data, id){
			this._addItemClass(dom.byId(id), "Selected");
			this.selection[id] = 1;
		}, this);
		return this._removeAnchor();	// self
	},
	deleteSelectedNodes: function(){
		// summary:
		//		deletes all selected items
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var n = dom.byId(i);
			this.delItem(i);
			domConstruct.destroy(n);
		}
		this.anchor = null;
		this.selection = {};
		return this;	// self
	},
	forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over selected items;
		//		see `dojo/dnd/Container.forInItems()` for details
		o = o || kernel.global;
		var s = this.selection, e = dnd._empty;
		for(var i in s){
			if(i in e){ continue; }
			f.call(o, this.getItem(i), i, this);
		}
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map

		Selector.superclass.sync.call(this);

		// fix the anchor
		if(this.anchor){
			if(!this.getItem(this.anchor.id)){
				this.anchor = null;
			}
		}

		// fix the selection
		var t = [], e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			if(!this.getItem(i)){
				t.push(i);
			}
		}
		array.forEach(t, function(i){
			delete this.selection[i];
		}, this);

		return this;	// self
	},
	insertNodes: function(addSelected, data, before, anchor){
		// summary:
		//		inserts new data items (see `dojo/dnd/Container.insertNodes()` method for details)
		// addSelected: Boolean
		//		all new nodes will be added to selected items, if true, no selection change otherwise
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		var oldCreator = this._normalizedCreator;
		this._normalizedCreator = function(item, hint){
			var t = oldCreator.call(this, item, hint);
			if(addSelected){
				if(!this.anchor){
					this.anchor = t.node;
					this._removeItemClass(t.node, "Selected");
					this._addItemClass(this.anchor, "Anchor");
				}else if(this.anchor != t.node){
					this._removeItemClass(t.node, "Anchor");
					this._addItemClass(t.node, "Selected");
				}
				this.selection[t.node.id] = 1;
			}else{
				this._removeItemClass(t.node, "Selected");
				this._removeItemClass(t.node, "Anchor");
			}
			return t;
		};
		Selector.superclass.insertNodes.call(this, data, before, anchor);
		this._normalizedCreator = oldCreator;
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		Selector.superclass.destroy.call(this);
		this.selection = this.anchor = null;
	},

	// mouse events
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(this.autoSync){ this.sync(); }
		if(!this.current){ return; }
		if(!this.singular && !dnd.getCopyKeyState(e) && !e.shiftKey && (this.current.id in this.selection)){
			this.simpleSelection = true;
			if(mouse.isLeft(e)){
				// Accept the left button and stop the event.   Stopping the event prevents text selection while
				// dragging.   However, don't stop the event on mobile because that prevents a click event,
				// and also prevents scroll (see #15838).
				// For IE we don't stop event when multiple buttons are pressed.
				e.stopPropagation();
				e.preventDefault();
			}
			return;
		}
		if(!this.singular && e.shiftKey){
			if(!dnd.getCopyKeyState(e)){
				this._removeSelection();
			}
			var c = this.getAllNodes();
			if(c.length){
				if(!this.anchor){
					this.anchor = c[0];
					this._addItemClass(this.anchor, "Anchor");
				}
				this.selection[this.anchor.id] = 1;
				if(this.anchor != this.current){
					var i = 0, node;
					for(; i < c.length; ++i){
						node = c[i];
						if(node == this.anchor || node == this.current){ break; }
					}
					for(++i; i < c.length; ++i){
						node = c[i];
						if(node == this.anchor || node == this.current){ break; }
						this._addItemClass(node, "Selected");
						this.selection[node.id] = 1;
					}
					this._addItemClass(this.current, "Selected");
					this.selection[this.current.id] = 1;
				}
			}
		}else{
			if(this.singular){
				if(this.anchor == this.current){
					if(dnd.getCopyKeyState(e)){
						this.selectNone();
					}
				}else{
					this.selectNone();
					this.anchor = this.current;
					this._addItemClass(this.anchor, "Anchor");
					this.selection[this.current.id] = 1;
				}
			}else{
				if(dnd.getCopyKeyState(e)){
					if(this.anchor == this.current){
						delete this.selection[this.anchor.id];
						this._removeAnchor();
					}else{
						if(this.current.id in this.selection){
							this._removeItemClass(this.current, "Selected");
							delete this.selection[this.current.id];
						}else{
							if(this.anchor){
								this._removeItemClass(this.anchor, "Anchor");
								this._addItemClass(this.anchor, "Selected");
							}
							this.anchor = this.current;
							this._addItemClass(this.current, "Anchor");
							this.selection[this.current.id] = 1;
						}
					}
				}else{
					if(!(this.current.id in this.selection)){
						this.selectNone();
						this.anchor = this.current;
						this._addItemClass(this.current, "Anchor");
						this.selection[this.current.id] = 1;
					}
				}
			}
		}
		e.stopPropagation();
		e.preventDefault();
	},
	onMouseUp: function(/*===== e =====*/){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(!this.simpleSelection){ return; }
		this.simpleSelection = false;
		this.selectNone();
		if(this.current){
			this.anchor = this.current;
			this._addItemClass(this.anchor, "Anchor");
			this.selection[this.current.id] = 1;
		}
	},
	onMouseMove: function(/*===== e =====*/){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		this.simpleSelection = false;
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		this.onmousemoveEvent = on(this.node, touch.move, lang.hitch(this, "onMouseMove"));
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		if(this.onmousemoveEvent){
			this.onmousemoveEvent.remove();
			delete this.onmousemoveEvent;
		}
	},
	_removeSelection: function(){
		// summary:
		//		unselects all items
		var e = dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var node = dom.byId(i);
			if(node){ this._removeItemClass(node, "Selected"); }
		}
		this.selection = {};
		return this;	// self
	},
	_removeAnchor: function(){
		if(this.anchor){
			this._removeItemClass(this.anchor, "Anchor");
			this.anchor = null;
		}
		return this;	// self
	}
});

return Selector;

});

},
'dojo/dnd/Container':function(){
define([
	"../_base/array",
	"../_base/declare",
	"../_base/kernel",
	"../_base/lang",
	"../_base/window",
	"../dom",
	"../dom-class",
	"../dom-construct",
	"../Evented",
	"../has",
	"../on",
	"../query",
	"../touch",
	"./common"
], function(
	array, declare, kernel, lang, win,
	dom, domClass, domConstruct, Evented, has, on, query, touch, dnd){

// module:
//		dojo/dnd/Container

/*
	Container states:
		""		- normal state
		"Over"	- mouse over a container
	Container item states:
		""		- normal state
		"Over"	- mouse over a container item
*/



var Container = declare("dojo.dnd.Container", Evented, {
	// summary:
	//		a Container object, which knows when mouse hovers over it,
	//		and over which element it hovers

	// object attributes (for markup)
	skipForm: false,
	// allowNested: Boolean
	//		Indicates whether to allow dnd item nodes to be nested within other elements.
	//		By default this is false, indicating that only direct children of the container can
	//		be draggable dnd item nodes
	allowNested: false,
	/*=====
	// current: DomNode
	//		The DOM node the mouse is currently hovered over
	current: null,

	// map: Hash<String, Container.Item>
	//		Map from an item's id (which is also the DOMNode's id) to
	//		the dojo/dnd/Container.Item itself.
	map: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		a constructor of the Container
		// node: Node
		//		node or node's id to build the container on
		// params: Container.__ContainerArgs
		//		a dictionary of parameters
		this.node = dom.byId(node);
		if(!params){ params = {}; }
		this.creator = params.creator || null;
		this.skipForm = params.skipForm;
		this.parent = params.dropParent && dom.byId(params.dropParent);

		// class-specific variables
		this.map = {};
		this.current = null;

		// states
		this.containerState = "";
		domClass.add(this.node, "dojoDndContainer");

		// mark up children
		if(!(params && params._skipStartup)){
			this.startup();
		}

		// set up events
		this.events = [
			on(this.node, touch.over, lang.hitch(this, "onMouseOver")),
			on(this.node, touch.out,  lang.hitch(this, "onMouseOut")),
			// cancel text selection and text dragging
			on(this.node, "dragstart",   lang.hitch(this, "onSelectStart")),
			on(this.node, "selectstart", lang.hitch(this, "onSelectStart"))
		];
	},

	// object attributes (for markup)
	creator: function(){
		// summary:
		//		creator function, dummy at the moment
	},

	// abstract access to the map
	getItem: function(/*String*/ key){
		// summary:
		//		returns a data item by its key (id)
		return this.map[key];	// Container.Item
	},
	setItem: function(/*String*/ key, /*Container.Item*/ data){
		// summary:
		//		associates a data item with its key (id)
		this.map[key] = data;
	},
	delItem: function(/*String*/ key){
		// summary:
		//		removes a data item from the map by its key (id)
		delete this.map[key];
	},
	forInItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over a data map skipping members that
		//		are present in the empty object (IE and/or 3rd-party libraries).
		o = o || kernel.global;
		var m = this.map, e = dnd._empty;
		for(var i in m){
			if(i in e){ continue; }
			f.call(o, m[i], i, this);
		}
		return o;	// Object
	},
	clearItems: function(){
		// summary:
		//		removes all data items from the map
		this.map = {};
	},

	// methods
	getAllNodes: function(){
		// summary:
		//		returns a list (an array) of all valid child nodes
		return query((this.allowNested ? "" : "> ") + ".dojoDndItem", this.parent);	// NodeList
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map
		var map = {};
		this.getAllNodes().forEach(function(node){
			if(node.id){
				var item = this.getItem(node.id);
				if(item){
					map[node.id] = item;
					return;
				}
			}else{
				node.id = dnd.getUniqueId();
			}
			var type = node.getAttribute("dndType"),
				data = node.getAttribute("dndData");
			map[node.id] = {
				data: data || node.innerHTML,
				type: type ? type.split(/\s*,\s*/) : ["text"]
			};
		}, this);
		this.map = map;
		return this;	// self
	},
	insertNodes: function(data, before, anchor){
		// summary:
		//		inserts an array of new nodes before/after an anchor node
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		if(!this.parent.firstChild){
			anchor = null;
		}else if(before){
			if(!anchor){
				anchor = this.parent.firstChild;
			}
		}else{
			if(anchor){
				anchor = anchor.nextSibling;
			}
		}
		var i, t;
		if(anchor){
			for(i = 0; i < data.length; ++i){
				t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				anchor.parentNode.insertBefore(t.node, anchor);
			}
		}else{
			for(i = 0; i < data.length; ++i){
				t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.appendChild(t.node);
			}
		}
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares this object to be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.clearItems();
		this.node = this.parent = this.current = null;
	},

	// markup methods
	markupFactory: function(params, node, Ctor){
		params._skipStartup = true;
		return new Ctor(node, params);
	},
	startup: function(){
		// summary:
		//		collects valid child items and populate the map

		// set up the real parent node
		if(!this.parent){
			// use the standard algorithm, if not assigned
			this.parent = this.node;
			if(this.parent.tagName.toLowerCase() == "table"){
				var c = this.parent.getElementsByTagName("tbody");
				if(c && c.length){ this.parent = c[0]; }
			}
		}
		this.defaultCreator = dnd._defaultCreator(this.parent);

		// process specially marked children
		this.sync();
	},

	// mouse events
	onMouseOver: function(e){
		// summary:
		//		event processor for onmouseover or touch, to mark that element as the current element
		// e: Event
		//		mouse event
		var n = e.relatedTarget;
		while(n){
			if(n == this.node){ break; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(!n){
			this._changeState("Container", "Over");
			this.onOverEvent();
		}
		n = this._getChildByEvent(e);
		if(this.current == n){ return; }
		if(this.current){ this._removeItemClass(this.current, "Over"); }
		if(n){ this._addItemClass(n, "Over"); }
		this.current = n;
	},
	onMouseOut: function(e){
		// summary:
		//		event processor for onmouseout
		// e: Event
		//		mouse event
		for(var n = e.relatedTarget; n;){
			if(n == this.node){ return; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(this.current){
			this._removeItemClass(this.current, "Over");
			this.current = null;
		}
		this._changeState("Container", "");
		this.onOutEvent();
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skipForm || !dnd.isFormElement(e)){
			e.stopPropagation();
			e.preventDefault();
		}
	},

	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
	},
	_changeState: function(type, newState){
		// summary:
		//		changes a named state to new state value
		// type: String
		//		a name of the state to change
		// newState: String
		//		new state
		var prefix = "dojoDnd" + type;
		var state  = type.toLowerCase() + "State";
		//domClass.replace(this.node, prefix + newState, prefix + this[state]);
		domClass.replace(this.node, prefix + newState, prefix + this[state]);
		this[state] = newState;
	},
	_addItemClass: function(node, type){
		// summary:
		//		adds a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		domClass.add(node, "dojoDndItem" + type);
	},
	_removeItemClass: function(node, type){
		// summary:
		//		removes a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		domClass.remove(node, "dojoDndItem" + type);
	},
	_getChildByEvent: function(e){
		// summary:
		//		gets a child, which is under the mouse at the moment, or null
		// e: Event
		//		a mouse event
		var node = e.target;
		if(node){
			for(var parent = node.parentNode; parent; node = parent, parent = node.parentNode){
				if((parent == this.parent || this.allowNested) && domClass.contains(node, "dojoDndItem")){ return node; }
			}
		}
		return null;
	},
	_normalizedCreator: function(/*Container.Item*/ item, /*String*/ hint){
		// summary:
		//		adds all necessary data to the output of the user-supplied creator function
		var t = (this.creator || this.defaultCreator).call(this, item, hint);
		if(!lang.isArray(t.type)){ t.type = ["text"]; }
		if(!t.node.id){ t.node.id = dnd.getUniqueId(); }
		domClass.add(t.node, "dojoDndItem");
		return t;
	}
});

dnd._createNode = function(tag){
	// summary:
	//		returns a function, which creates an element of given tag
	//		(SPAN by default) and sets its innerHTML to given text
	// tag: String
	//		a tag name or empty for SPAN
	if(!tag){ return dnd._createSpan; }
	return function(text){	// Function
		return domConstruct.create(tag, {innerHTML: text});	// Node
	};
};

dnd._createTrTd = function(text){
	// summary:
	//		creates a TR/TD structure with given text as an innerHTML of TD
	// text: String
	//		a text for TD
	var tr = domConstruct.create("tr");
	domConstruct.create("td", {innerHTML: text}, tr);
	return tr;	// Node
};

dnd._createSpan = function(text){
	// summary:
	//		creates a SPAN element with given text as its innerHTML
	// text: String
	//		a text for SPAN
	return domConstruct.create("span", {innerHTML: text});	// Node
};

// dnd._defaultCreatorNodes: Object
//		a dictionary that maps container tag names to child tag names
dnd._defaultCreatorNodes = {ul: "li", ol: "li", div: "div", p: "div"};

dnd._defaultCreator = function(node){
	// summary:
	//		takes a parent node, and returns an appropriate creator function
	// node: Node
	//		a container node
	var tag = node.tagName.toLowerCase();
	var c = tag == "tbody" || tag == "thead" ? dnd._createTrTd :
			dnd._createNode(dnd._defaultCreatorNodes[tag]);
	return function(item, hint){	// Function
		var isObj = item && lang.isObject(item), data, type, n;
		if(isObj && item.tagName && item.nodeType && item.getAttribute){
			// process a DOM node
			data = item.getAttribute("dndData") || item.innerHTML;
			type = item.getAttribute("dndType");
			type = type ? type.split(/\s*,\s*/) : ["text"];
			n = item;	// this node is going to be moved rather than copied
		}else{
			// process a DnD item object or a string
			data = (isObj && item.data) ? item.data : item;
			type = (isObj && item.type) ? item.type : ["text"];
			n = (hint == "avatar" ? dnd._createSpan : c)(String(data));
		}
		if(!n.id){
			n.id = dnd.getUniqueId();
		}
		return {node: n, data: data, type: type};
	};
};

/*=====
Container.__ContainerArgs = declare([], {
	creator: function(){
		// summary:
		//		a creator function, which takes a data item, and returns an object like that:
		//		{node: newNode, data: usedData, type: arrayOfStrings}
	},

	// skipForm: Boolean
	//		don't start the drag operation, if clicked on form elements
	skipForm: false,

	// dropParent: Node||String
	//		node or node's id to use as the parent node for dropped items
	//		(must be underneath the 'node' parameter in the DOM)
	dropParent: null,

	// _skipStartup: Boolean
	//		skip startup(), which collects children, for deferred initialization
	//		(this is used in the markup mode)
	_skipStartup: false
});

Container.Item = function(){
	// summary:
	//		Represents (one of) the source node(s) being dragged.
	//		Contains (at least) the "type" and "data" attributes.
	// type: String[]
	//		Type(s) of this item, by default this is ["text"]
	// data: Object
	//		Logical representation of the object being dragged.
	//		If the drag object's type is "text" then data is a String,
	//		if it's another type then data could be a different Object,
	//		perhaps a name/value hash.

	this.type = type;
	this.data = data;
};
=====*/

return Container;
});

},
'dojo/dnd/Target':function(){
define([ "../_base/declare", "../dom-class", "./Source" ], function(declare, domClass, Source){
	return declare("dojo.dnd.Target", Source, {
		// summary:
		//		a Target object, which can be used as a DnD target

		constructor: function(/*===== node, params =====*/){
			// summary:
			//		a constructor of the Target --- see the `dojo/dnd/Source` constructor for details
			this.isSource = false;
			domClass.remove(this.node, "dojoDndSource");
		}
	});
});

},
'dojo/parser':function(){
define([
	"require", "./_base/kernel", "./_base/lang", "./_base/array", "./_base/config", "./dom", "./_base/window",
		"./_base/url", "./aspect", "./promise/all", "./date/stamp", "./Deferred", "./has", "./query", "./on", "./ready"
], function(require, dojo, dlang, darray, config, dom, dwindow, _Url, aspect, all, dates, Deferred, has, query, don, ready){

	// module:
	//		dojo/parser

	new Date("X"); // workaround for #11279, new Date("") == NaN

	// data-dojo-props etc. is not restricted to JSON, it can be any javascript
	function myEval(text){
		return eval("(" + text + ")");
	}

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	var extendCnt = 0;
	aspect.after(dlang, "extend", function(){
		extendCnt++;
	}, true);

	function getNameMap(ctor){
		// summary:
		//		Returns map from lowercase name to attribute name in class, ex: {onclick: "onClick"}
		var map = ctor._nameCaseMap, proto = ctor.prototype;

		// Create the map if it's undefined.
		// Refresh the map if a superclass was possibly extended with new methods since the map was created.
		if(!map || map._extendCnt < extendCnt){
			map = ctor._nameCaseMap = {};
			for(var name in proto){
				if(name.charAt(0) === "_"){
					continue;
				}	// skip internal properties
				map[name.toLowerCase()] = name;
			}
			map._extendCnt = extendCnt;
		}
		return map;
	}

	// Map from widget name or list of widget names(ex: "dijit/form/Button,acme/MyMixin") to a constructor.
	var _ctorMap = {};

	function getCtor(/*String[]*/ types, /*Function?*/ contextRequire){
		// summary:
		//		Retrieves a constructor.  If the types array contains more than one class/MID then the
		//		subsequent classes will be mixed into the first class and a unique constructor will be
		//		returned for that array.

		var ts = types.join();
		if(!_ctorMap[ts]){
			var mixins = [];
			for(var i = 0, l = types.length; i < l; i++){
				var t = types[i];
				// TODO: Consider swapping getObject and require in the future
				mixins[mixins.length] = (_ctorMap[t] = _ctorMap[t] || (dlang.getObject(t) || (~t.indexOf('/') &&
					(contextRequire ? contextRequire(t) : require(t)))));
			}
			var ctor = mixins.shift();
			_ctorMap[ts] = mixins.length ? (ctor.createSubclass ? ctor.createSubclass(mixins) : ctor.extend.apply(ctor, mixins)) : ctor;
		}

		return _ctorMap[ts];
	}

	var parser = {
		// summary:
		//		The Dom/Widget parsing package

		_clearCache: function(){
			// summary:
			//		Clear cached data.   Used mainly for benchmarking.
			extendCnt++;
			_ctorMap = {};
		},

		_functionFromScript: function(script, attrData){
			// summary:
			//		Convert a `<script type="dojo/method" args="a, b, c"> ... </script>`
			//		into a function
			// script: DOMNode
			//		The `<script>` DOMNode
			// attrData: String
			//		For HTML5 compliance, searches for attrData + "args" (typically
			//		"data-dojo-args") instead of "args"
			var preamble = "",
				suffix = "",
				argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args")),
				withStr = script.getAttribute("with");

			// Convert any arguments supplied in script tag into an array to be passed to the
			var fnArgs = (argsStr || "").split(/\s*,\s*/);

			if(withStr && withStr.length){
				darray.forEach(withStr.split(/\s*,\s*/), function(part){
					preamble += "with(" + part + "){";
					suffix += "}";
				});
			}

			return new Function(fnArgs, preamble + script.innerHTML + suffix);
		},

		instantiate: function(nodes, mixin, options){
			// summary:
			//		Takes array of nodes, and turns them into class instances and
			//		potentially calls a startup method to allow them to connect with
			//		any children.
			// nodes: Array
			//		Array of DOM nodes
			// mixin: Object?
			//		An object that will be mixed in with each node in the array.
			//		Values in the mixin will override values in the node, if they
			//		exist.
			// options: Object?
			//		An object used to hold kwArgs for instantiation.
			//		See parse.options argument for details.
			// returns:
			//		Array of instances.

			mixin = mixin || {};
			options = options || {};

			var dojoType = (options.scope || dojo._scopeName) + "Type", // typically "dojoType"
				attrData = "data-" + (options.scope || dojo._scopeName) + "-", // typically "data-dojo-"
				dataDojoType = attrData + "type", // typically "data-dojo-type"
				dataDojoMixins = attrData + "mixins";					// typically "data-dojo-mixins"

			var list = [];
			darray.forEach(nodes, function(node){
				var type = dojoType in mixin ? mixin[dojoType] : node.getAttribute(dataDojoType) || node.getAttribute(dojoType);
				if(type){
					var mixinsValue = node.getAttribute(dataDojoMixins),
						types = mixinsValue ? [type].concat(mixinsValue.split(/\s*,\s*/)) : [type];

					list.push({
						node: node,
						types: types
					});
				}
			});

			// Instantiate the nodes and return the list of instances.
			return this._instantiate(list, mixin, options);
		},

		_instantiate: function(nodes, mixin, options, returnPromise){
			// summary:
			//		Takes array of objects representing nodes, and turns them into class instances and
			//		potentially calls a startup method to allow them to connect with
			//		any children.
			// nodes: Array
			//		Array of objects like
			//	|		{
			//	|			ctor: Function (may be null)
			//	|			types: ["dijit/form/Button", "acme/MyMixin"] (used if ctor not specified)
			//	|			node: DOMNode,
			//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
			//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
			//	|		}
			// mixin: Object
			//		An object that will be mixed in with each node in the array.
			//		Values in the mixin will override values in the node, if they
			//		exist.
			// options: Object
			//		An options object used to hold kwArgs for instantiation.
			//		See parse.options argument for details.
			// returnPromise: Boolean
			//		Return a Promise rather than the instance; supports asynchronous widget creation.
			// returns:
			//		Array of instances, or if returnPromise is true, a promise for array of instances
			//		that resolves when instances have finished initializing.

			// Call widget constructors.   Some may be asynchronous and return promises.
			var thelist = darray.map(nodes, function(obj){
				var ctor = obj.ctor || getCtor(obj.types, options.contextRequire);
				// If we still haven't resolved a ctor, it is fatal now
				if(!ctor){
					throw new Error("Unable to resolve constructor for: '" + obj.types.join() + "'");
				}
				return this.construct(ctor, obj.node, mixin, options, obj.scripts, obj.inherited);
			}, this);

			// After all widget construction finishes, call startup on each top level instance if it makes sense (as for
			// widgets).  Parent widgets will recursively call startup on their (non-top level) children
			function onConstruct(thelist){
				if(!mixin._started && !options.noStart){
					darray.forEach(thelist, function(instance){
						if(typeof instance.startup === "function" && !instance._started){
							instance.startup();
						}
					});
				}

				return thelist;
			}

			if(returnPromise){
				return all(thelist).then(onConstruct);
			}else{
				// Back-compat path, remove for 2.0
				return onConstruct(thelist);
			}
		},

		construct: function(ctor, node, mixin, options, scripts, inherited){
			// summary:
			//		Calls new ctor(params, node), where params is the hash of parameters specified on the node,
			//		excluding data-dojo-type and data-dojo-mixins.   Does not call startup().
			// ctor: Function
			//		Widget constructor.
			// node: DOMNode
			//		This node will be replaced/attached to by the widget.  It also specifies the arguments to pass to ctor.
			// mixin: Object?
			//		Attributes in this object will be passed as parameters to ctor,
			//		overriding attributes specified on the node.
			// options: Object?
			//		An options object used to hold kwArgs for instantiation.   See parse.options argument for details.
			// scripts: DomNode[]?
			//		Array of `<script type="dojo/*">` DOMNodes.  If not specified, will search for `<script>` tags inside node.
			// inherited: Object?
			//		Settings from dir=rtl or lang=... on a node above this node.   Overrides options.inherited.
			// returns:
			//		Instance or Promise for the instance, if markupFactory() itself returned a promise

			var proto = ctor && ctor.prototype;
			options = options || {};

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};

			if(options.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				dlang.mixin(params, options.defaults);
			}
			if(inherited){
				// settings from dir=rtl or lang=... on a node above this node
				dlang.mixin(params, inherited);
			}

			// Get list of attributes explicitly listed in the markup
			var attributes;
			if(has("dom-attributes-explicit")){
				// Standard path to get list of user specified attributes
				attributes = node.attributes;
			}else if(has("dom-attributes-specified-flag")){
				// Special processing needed for IE8, to skip a few faux values in attributes[]
				attributes = darray.filter(node.attributes, function(a){
					return a.specified;
				});
			}else{
				// Special path for IE6-7, avoid (sometimes >100) bogus entries in node.attributes
				var clone = /^input$|^img$/i.test(node.nodeName) ? node : node.cloneNode(false),
					attrs = clone.outerHTML.replace(/=[^\s"']+|="[^"]*"|='[^']*'/g, "").replace(/^\s*<[a-zA-Z0-9]*\s*/, "").replace(/\s*>.*$/, "");

				attributes = darray.map(attrs.split(/\s+/), function(name){
					var lcName = name.toLowerCase();
					return {
						name: name,
						// getAttribute() doesn't work for button.value, returns innerHTML of button.
						// but getAttributeNode().value doesn't work for the form.encType or li.value
						value: (node.nodeName == "LI" && name == "value") || lcName == "enctype" ?
							node.getAttribute(lcName) : node.getAttributeNode(lcName).value
					};
				});
			}

			// Hash to convert scoped attribute name (ex: data-dojo17-params) to something friendly (ex: data-dojo-params)
			// TODO: remove scope for 2.0
			var scope = options.scope || dojo._scopeName,
				attrData = "data-" + scope + "-", // typically "data-dojo-"
				hash = {};
			if(scope !== "dojo"){
				hash[attrData + "props"] = "data-dojo-props";
				hash[attrData + "type"] = "data-dojo-type";
				hash[attrData + "mixins"] = "data-dojo-mixins";
				hash[scope + "type"] = "dojoType";
				hash[attrData + "id"] = "data-dojo-id";
			}

			// Read in attributes and process them, including data-dojo-props, data-dojo-type,
			// dojoAttachPoint, etc., as well as normal foo=bar attributes.
			var i = 0, item, funcAttrs = [], jsname, extra;
			while(item = attributes[i++]){
				var name = item.name,
					lcName = name.toLowerCase(),
					value = item.value;

				switch(hash[lcName] || lcName){
				// Already processed, just ignore
				case "data-dojo-type":
				case "dojotype":
				case "data-dojo-mixins":
					break;

				// Data-dojo-props.   Save for later to make sure it overrides direct foo=bar settings
				case "data-dojo-props":
					extra = value;
					break;

				// data-dojo-id or jsId. TODO: drop jsId in 2.0
				case "data-dojo-id":
				case "jsid":
					jsname = value;
					break;

				// For the benefit of _Templated
				case "data-dojo-attach-point":
				case "dojoattachpoint":
					params.dojoAttachPoint = value;
					break;
				case "data-dojo-attach-event":
				case "dojoattachevent":
					params.dojoAttachEvent = value;
					break;

				// Special parameter handling needed for IE
				case "class":
					params["class"] = node.className;
					break;
				case "style":
					params["style"] = node.style && node.style.cssText;
					break;
				default:
					// Normal attribute, ex: value="123"

					// Find attribute in widget corresponding to specified name.
					// May involve case conversion, ex: onclick --> onClick
					if(!(name in proto)){
						var map = getNameMap(ctor);
						name = map[lcName] || name;
					}

					// Set params[name] to value, doing type conversion
					if(name in proto){
						switch(typeof proto[name]){
						case "string":
							params[name] = value;
							break;
						case "number":
							params[name] = value.length ? Number(value) : NaN;
							break;
						case "boolean":
							// for checked/disabled value might be "" or "checked".	 interpret as true.
							params[name] = value.toLowerCase() != "false";
							break;
						case "function":
							if(value === "" || value.search(/[^\w\.]+/i) != -1){
								// The user has specified some text for a function like "return x+5"
								params[name] = new Function(value);
							}else{
								// The user has specified the name of a global function like "myOnClick"
								// or a single word function "return"
								params[name] = dlang.getObject(value, false) || new Function(value);
							}
							funcAttrs.push(name);	// prevent "double connect", see #15026
							break;
						default:
							var pVal = proto[name];
							params[name] =
								(pVal && "length" in pVal) ? (value ? value.split(/\s*,\s*/) : []) :	// array
									(pVal instanceof Date) ?
										(value == "" ? new Date("") :	// the NaN of dates
										value == "now" ? new Date() :	// current date
										dates.fromISOString(value)) :
								(pVal instanceof _Url) ? (dojo.baseUrl + value) :
								myEval(value);
						}
					}else{
						params[name] = value;
					}
				}
			}

			// Remove function attributes from DOMNode to prevent "double connect" problem, see #15026.
			// Do this as a separate loop since attributes[] is often a live collection (depends on the browser though).
			for(var j = 0; j < funcAttrs.length; j++){
				var lcfname = funcAttrs[j].toLowerCase();
				node.removeAttribute(lcfname);
				node[lcfname] = null;
			}

			// Mix things found in data-dojo-props into the params, overriding any direct settings
			if(extra){
				try{
					extra = myEval.call(options.propsThis, "{" + extra + "}");
					dlang.mixin(params, extra);
				}catch(e){
					// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
					throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
				}
			}

			// Any parameters specified in "mixin" override everything else.
			dlang.mixin(params, mixin);

			// Get <script> nodes associated with this widget, if they weren't specified explicitly
			if(!scripts){
				scripts = (ctor && (ctor._noScript || proto._noScript) ? [] : query("> script[type^='dojo/']", node));
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" data-dojo-event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" data-dojo-event="foo"> tags are dojo.connected after instantiation,
			// and likewise with <script type="dojo/aspect" data-dojo-method="foo">
			// <script type="dojo/watch" data-dojo-prop="foo"> tags are dojo.watch after instantiation
			// <script type="dojo/on" data-dojo-event="foo"> tags are dojo.on after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var aspects = [],	// aspects to connect after instantiation
				calls = [],		// functions to call after instantiation
				watches = [],  // functions to watch after instantiation
				ons = []; // functions to on after instantiation

			if(scripts){
				for(i = 0; i < scripts.length; i++){
					var script = scripts[i];
					node.removeChild(script);
					// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
					var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
						prop = script.getAttribute(attrData + "prop"),
						method = script.getAttribute(attrData + "method"),
						advice = script.getAttribute(attrData + "advice"),
						scriptType = script.getAttribute("type"),
						nf = this._functionFromScript(script, attrData);
					if(event){
						if(scriptType == "dojo/connect"){
							aspects.push({ method: event, func: nf });
						}else if(scriptType == "dojo/on"){
							ons.push({ event: event, func: nf });
						}else{
							// <script type="dojo/method" data-dojo-event="foo">
							// TODO for 2.0: use data-dojo-method="foo" instead (also affects dijit/Declaration)
							params[event] = nf;
						}
					}else if(scriptType == "dojo/aspect"){
						aspects.push({ method: method, advice: advice, func: nf });
					}else if(scriptType == "dojo/watch"){
						watches.push({ prop: prop, func: nf });
					}else{
						calls.push(nf);
					}
				}
			}

			// create the instance
			var markupFactory = ctor.markupFactory || proto.markupFactory;
			var instance = markupFactory ? markupFactory(params, node, ctor) : new ctor(params, node);

			function onInstantiate(instance){
				// map it to the JS namespace if that makes sense
				if(jsname){
					dlang.setObject(jsname, instance);
				}

				// process connections and startup functions
				for(i = 0; i < aspects.length; i++){
					aspect[aspects[i].advice || "after"](instance, aspects[i].method, dlang.hitch(instance, aspects[i].func), true);
				}
				for(i = 0; i < calls.length; i++){
					calls[i].call(instance);
				}
				for(i = 0; i < watches.length; i++){
					instance.watch(watches[i].prop, watches[i].func);
				}
				for(i = 0; i < ons.length; i++){
					don(instance, ons[i].event, ons[i].func);
				}

				return instance;
			}

			if(instance.then){
				return instance.then(onInstantiate);
			}else{
				return onInstantiate(instance);
			}
		},

		scan: function(root, options){
			// summary:
			//		Scan a DOM tree and return an array of objects representing the DOMNodes
			//		that need to be turned into widgets.
			// description:
			//		Search specified node (or document root node) recursively for class instances
			//		and return an array of objects that represent potential widgets to be
			//		instantiated. Searches for either data-dojo-type="MID" or dojoType="MID" where
			//		"MID" is a module ID like "dijit/form/Button" or a fully qualified Class name
			//		like "dijit/form/Button".  If the MID is not currently available, scan will
			//		attempt to require() in the module.
			//
			//		See parser.parse() for details of markup.
			// root: DomNode?
			//		A default starting root node from which to start the parsing. Can be
			//		omitted, defaulting to the entire document. If omitted, the `options`
			//		object can be passed in this place. If the `options` object has a
			//		`rootNode` member, that is used.
			// options: Object
			//		a kwArgs options object, see parse() for details
			//
			// returns: Promise
			//		A promise that is resolved with the nodes that have been parsed.

			var list = [], // Output List
				mids = [], // An array of modules that are not yet loaded
				midsHash = {}; // Used to keep the mids array unique

			var dojoType = (options.scope || dojo._scopeName) + "Type", // typically "dojoType"
				attrData = "data-" + (options.scope || dojo._scopeName) + "-", // typically "data-dojo-"
				dataDojoType = attrData + "type", // typically "data-dojo-type"
				dataDojoTextDir = attrData + "textdir", // typically "data-dojo-textdir"
				dataDojoMixins = attrData + "mixins";					// typically "data-dojo-mixins"

			// Info on DOMNode currently being processed
			var node = root.firstChild;

			// Info on parent of DOMNode currently being processed
			//	- inherited: dir, lang, and textDir setting of parent, or inherited by parent
			//	- parent: pointer to identical structure for my parent (or null if no parent)
			//	- scripts: if specified, collects <script type="dojo/..."> type nodes from children
			var inherited = options.inherited;
			if(!inherited){
				function findAncestorAttr(node, attr){
					return (node.getAttribute && node.getAttribute(attr)) ||
						(node.parentNode && findAncestorAttr(node.parentNode, attr));
				}

				inherited = {
					dir: findAncestorAttr(root, "dir"),
					lang: findAncestorAttr(root, "lang"),
					textDir: findAncestorAttr(root, dataDojoTextDir)
				};
				for(var key in inherited){
					if(!inherited[key]){
						delete inherited[key];
					}
				}
			}

			// Metadata about parent node
			var parent = {
				inherited: inherited
			};

			// For collecting <script type="dojo/..."> type nodes (when null, we don't need to collect)
			var scripts;

			// when true, only look for <script type="dojo/..."> tags, and don't recurse to children
			var scriptsOnly;

			function getEffective(parent){
				// summary:
				//		Get effective dir, lang, textDir settings for specified obj
				//		(matching "parent" object structure above), and do caching.
				//		Take care not to return null entries.
				if(!parent.inherited){
					parent.inherited = {};
					var node = parent.node,
						grandparent = getEffective(parent.parent);
					var inherited = {
						dir: node.getAttribute("dir") || grandparent.dir,
						lang: node.getAttribute("lang") || grandparent.lang,
						textDir: node.getAttribute(dataDojoTextDir) || grandparent.textDir
					};
					for(var key in inherited){
						if(inherited[key]){
							parent.inherited[key] = inherited[key];
						}
					}
				}
				return parent.inherited;
			}

			// DFS on DOM tree, collecting nodes with data-dojo-type specified.
			while(true){
				if(!node){
					// Finished this level, continue to parent's next sibling
					if(!parent || !parent.node){
						break;
					}
					node = parent.node.nextSibling;
					scriptsOnly = false;
					parent = parent.parent;
					scripts = parent.scripts;
					continue;
				}

				if(node.nodeType != 1){
					// Text or comment node, skip to next sibling
					node = node.nextSibling;
					continue;
				}

				if(scripts && node.nodeName.toLowerCase() == "script"){
					// Save <script type="dojo/..."> for parent, then continue to next sibling
					type = node.getAttribute("type");
					if(type && /^dojo\/\w/i.test(type)){
						scripts.push(node);
					}
					node = node.nextSibling;
					continue;
				}
				if(scriptsOnly){
					// scriptsOnly flag is set, we have already collected scripts if the parent wants them, so now we shouldn't
					// continue further analysis of the node and will continue to the next sibling
					node = node.nextSibling;
					continue;
				}

				// Check for data-dojo-type attribute, fallback to backward compatible dojoType
				// TODO: Remove dojoType in 2.0
				var type = node.getAttribute(dataDojoType) || node.getAttribute(dojoType);

				// Short circuit for leaf nodes containing nothing [but text]
				var firstChild = node.firstChild;
				if(!type && (!firstChild || (firstChild.nodeType == 3 && !firstChild.nextSibling))){
					node = node.nextSibling;
					continue;
				}

				// Meta data about current node
				var current;

				var ctor = null;
				if(type){
					// If dojoType/data-dojo-type specified, add to output array of nodes to instantiate.
					var mixinsValue = node.getAttribute(dataDojoMixins),
						types = mixinsValue ? [type].concat(mixinsValue.split(/\s*,\s*/)) : [type];

					// Note: won't find classes declared via dojo/Declaration or any modules that haven't been
					// loaded yet so use try/catch to avoid throw from require()
					try{
						ctor = getCtor(types, options.contextRequire);
					}catch(e){}

					// If the constructor was not found, check to see if it has modules that can be loaded
					if(!ctor){
						darray.forEach(types, function(t){
							if(~t.indexOf('/') && !midsHash[t]){
								// If the type looks like a MID and it currently isn't in the array of MIDs to load, add it.
								midsHash[t] = true;
								mids[mids.length] = t;
							}
						});
					}

					var childScripts = ctor && !ctor.prototype._noScript ? [] : null; // <script> nodes that are parent's children

					// Setup meta data about this widget node, and save it to list of nodes to instantiate
					current = {
						types: types,
						ctor: ctor,
						parent: parent,
						node: node,
						scripts: childScripts
					};
					current.inherited = getEffective(current); // dir & lang settings for current node, explicit or inherited
					list.push(current);
				}else{
					// Meta data about this non-widget node
					current = {
						node: node,
						scripts: scripts,
						parent: parent
					};
				}

				// Recurse, collecting <script type="dojo/..."> children, and also looking for
				// descendant nodes with dojoType specified (unless the widget has the stopParser flag).
				// When finished with children, go to my next sibling.
				scripts = childScripts;
				scriptsOnly = node.stopParser || (ctor && ctor.prototype.stopParser && !(options.template));
				parent = current;
				node = firstChild;
			}

			var d = new Deferred();

			// If there are modules to load then require them in
			if(mids.length){
				// Warn that there are modules being auto-required
				if(has("dojo-debug-messages")){
					console.warn("WARNING: Modules being Auto-Required: " + mids.join(", "));
				}
				var r = options.contextRequire || require;
				r(mids, function(){
					// Go through list of widget nodes, filling in missing constructors, and filtering out nodes that shouldn't
					// be instantiated due to a stopParser flag on an ancestor that we belatedly learned about due to
					// auto-require of a module like ContentPane.   Assumes list is in DFS order.
					d.resolve(darray.filter(list, function(widget){
						if(!widget.ctor){
							// Attempt to find the constructor again.   Still won't find classes defined via
							// dijit/Declaration so need to try/catch.
							try{
								widget.ctor = getCtor(widget.types, options.contextRequire);
							}catch(e){}
						}

						// Get the parent widget
						var parent = widget.parent;
						while(parent && !parent.types){
							parent = parent.parent;
						}

						// Return false if this node should be skipped due to stopParser on an ancestor.
						// Since list[] is in DFS order, this loop will always set parent.instantiateChildren before
						// trying to compute widget.instantiate.
						var proto = widget.ctor && widget.ctor.prototype;
						widget.instantiateChildren = !(proto && proto.stopParser && !(options.template));
						widget.instantiate = !parent || (parent.instantiate && parent.instantiateChildren);
						return widget.instantiate;
					}));
				});
			}else{
				// There were no modules to load, so just resolve with the parsed nodes.   This separate code path is for
				// efficiency, to avoid running the require() and the callback code above.
				d.resolve(list);
			}

			// Return the promise
			return d.promise;
		},

		_require: function(/*DOMNode*/ script, /*Object?*/ options){
			// summary:
			//		Helper for _scanAMD().  Takes a `<script type=dojo/require>bar: "acme/bar", ...</script>` node,
			//		calls require() to load the specified modules and (asynchronously) assign them to the specified global
			//		variables, and returns a Promise for when that operation completes.
			//
			//		In the example above, it is effectively doing a require(["acme/bar", ...], function(a){ bar = a; }).

			var hash = myEval("{" + script.innerHTML + "}"), // can't use dojo/json::parse() because maybe no quotes
				vars = [],
				mids = [],
				d = new Deferred();

			var contextRequire = (options && options.contextRequire) || require;

			for(var name in hash){
				vars.push(name);
				mids.push(hash[name]);
			}

			contextRequire(mids, function(){
				for(var i = 0; i < vars.length; i++){
					dlang.setObject(vars[i], arguments[i]);
				}
				d.resolve(arguments);
			});

			return d.promise;
		},

		_scanAmd: function(root, options){
			// summary:
			//		Scans the DOM for any declarative requires and returns their values.
			// description:
			//		Looks for `<script type=dojo/require>bar: "acme/bar", ...</script>` node, calls require() to load the
			//		specified modules and (asynchronously) assign them to the specified global variables,
			//		and returns a Promise for when those operations complete.
			// root: DomNode
			//		The node to base the scan from.
			// options: Object?
			//		a kwArgs options object, see parse() for details

			// Promise that resolves when all the <script type=dojo/require> nodes have finished loading.
			var deferred = new Deferred(),
				promise = deferred.promise;
			deferred.resolve(true);

			var self = this;
			query("script[type='dojo/require']", root).forEach(function(node){
				// Fire off require() call for specified modules.  Chain this require to fire after
				// any previous requires complete, so that layers can be loaded before individual module require()'s fire.
				promise = promise.then(function(){
					return self._require(node, options);
				});

				// Remove from DOM so it isn't seen again
				node.parentNode.removeChild(node);
			});

			return promise;
		},

		parse: function(rootNode, options){
			// summary:
			//		Scan the DOM for class instances, and instantiate them.
			// description:
			//		Search specified node (or root node) recursively for class instances,
			//		and instantiate them. Searches for either data-dojo-type="Class" or
			//		dojoType="Class" where "Class" is a a fully qualified class name,
			//		like `dijit/form/Button`
			//
			//		Using `data-dojo-type`:
			//		Attributes using can be mixed into the parameters used to instantiate the
			//		Class by using a `data-dojo-props` attribute on the node being converted.
			//		`data-dojo-props` should be a string attribute to be converted from JSON.
			//
			//		Using `dojoType`:
			//		Attributes are read from the original domNode and converted to appropriate
			//		types by looking up the Class prototype values. This is the default behavior
			//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
			//		go away in Dojo 2.0.
			// rootNode: DomNode?
			//		A default starting root node from which to start the parsing. Can be
			//		omitted, defaulting to the entire document. If omitted, the `options`
			//		object can be passed in this place. If the `options` object has a
			//		`rootNode` member, that is used.
			// options: Object?
			//		A hash of options.
			//
			//		- noStart: Boolean?:
			//			when set will prevent the parser from calling .startup()
			//			when locating the nodes.
			//		- rootNode: DomNode?:
			//			identical to the function's `rootNode` argument, though
			//			allowed to be passed in via this `options object.
			//		- template: Boolean:
			//			If true, ignores ContentPane's stopParser flag and parses contents inside of
			//			a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
			//			nested inside the ContentPane to work.
			//		- inherited: Object:
			//			Hash possibly containing dir and lang settings to be applied to
			//			parsed widgets, unless there's another setting on a sub-node that overrides
			//		- scope: String:
			//			Root for attribute names to search for.   If scopeName is dojo,
			//			will search for data-dojo-type (or dojoType).   For backwards compatibility
			//			reasons defaults to dojo._scopeName (which is "dojo" except when
			//			multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			//		- propsThis: Object:
			//			If specified, "this" referenced from data-dojo-props will refer to propsThis.
			//			Intended for use from the widgets-in-template feature of `dijit._WidgetsInTemplateMixin`
			//		- contextRequire: Function:
			//			If specified, this require is utilised for looking resolving modules instead of the
			//			`dojo/parser` context `require()`.  Intended for use from the widgets-in-template feature of
			//			`dijit._WidgetsInTemplateMixin`.
			// returns: Mixed
			//		Returns a blended object that is an array of the instantiated objects, but also can include
			//		a promise that is resolved with the instantiated objects.  This is done for backwards
			//		compatibility.  If the parser auto-requires modules, it will always behave in a promise
			//		fashion and `parser.parse().then(function(instances){...})` should be used.
			// example:
			//		Parse all widgets on a page:
			//	|		parser.parse();
			// example:
			//		Parse all classes within the node with id="foo"
			//	|		parser.parse(dojo.byId('foo'));
			// example:
			//		Parse all classes in a page, but do not call .startup() on any
			//		child
			//	|		parser.parse({ noStart: true })
			// example:
			//		Parse all classes in a node, but do not call .startup()
			//	|		parser.parse(someNode, { noStart:true });
			//	|		// or
			//	|		parser.parse({ noStart:true, rootNode: someNode });

			// determine the root node and options based on the passed arguments.
			var root;
			if(!options && rootNode && rootNode.rootNode){
				options = rootNode;
				root = options.rootNode;
			}else if(rootNode && dlang.isObject(rootNode) && !("nodeType" in rootNode)){
				options = rootNode;
			}else{
				root = rootNode;
			}
			root = root ? dom.byId(root) : dwindow.body();

			options = options || {};

			var mixin = options.template ? { template: true } : {},
				instances = [],
				self = this;

			// First scan for any <script type=dojo/require> nodes, and execute.
			// Then scan for all nodes with data-dojo-type, and load any unloaded modules.
			// Then build the object instances.  Add instances to already existing (but empty) instances[] array,
			// which may already have been returned to caller.  Also, use otherwise to collect and throw any errors
			// that occur during the parse().
			var p =
				this._scanAmd(root, options).then(function(){
					return self.scan(root, options);
				}).then(function(parsedNodes){
					return self._instantiate(parsedNodes, mixin, options, true);
				}).then(function(_instances){
					// Copy the instances into the instances[] array we declared above, and are accessing as
					// our return value.
					return instances = instances.concat(_instances);
				}).otherwise(function(e){
					// TODO Modify to follow better pattern for promise error management when available
					console.error("dojo/parser::parse() error", e);
					throw e;
				});

			// Blend the array with the promise
			dlang.mixin(instances, p);
			return instances;
		}
	};

	if( 1 ){
		dojo.parser = parser;
	}

	// Register the parser callback. It should be the first callback
	// after the a11y test.
	if(config.parseOnLoad){
		ready(100, parser, "parse");
	}

	return parser;
});

},
'dojo/_base/url':function(){
define(["./kernel"], function(dojo){
	// module:
	//		dojo/url

	var
		ore = new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$"),
		ire = new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$"),
		_Url = function(){
			var n = null,
				_a = arguments,
				uri = [_a[0]];
			// resolve uri components relative to each other
			for(var i = 1; i<_a.length; i++){
				if(!_a[i]){ continue; }

				// Safari doesn't support this.constructor so we have to be explicit
				// FIXME: Tracked (and fixed) in Webkit bug 3537.
				//		http://bugs.webkit.org/show_bug.cgi?id=3537
				var relobj = new _Url(_a[i]+""),
					uriobj = new _Url(uri[0]+"");

				if(
					relobj.path == "" &&
					!relobj.scheme &&
					!relobj.authority &&
					!relobj.query
				){
					if(relobj.fragment != n){
						uriobj.fragment = relobj.fragment;
					}
					relobj = uriobj;
				}else if(!relobj.scheme){
					relobj.scheme = uriobj.scheme;

					if(!relobj.authority){
						relobj.authority = uriobj.authority;

						if(relobj.path.charAt(0) != "/"){
							var path = uriobj.path.substring(0,
								uriobj.path.lastIndexOf("/") + 1) + relobj.path;

							var segs = path.split("/");
							for(var j = 0; j < segs.length; j++){
								if(segs[j] == "."){
									// flatten "./" references
									if(j == segs.length - 1){
										segs[j] = "";
									}else{
										segs.splice(j, 1);
										j--;
									}
								}else if(j > 0 && !(j == 1 && segs[0] == "") &&
									segs[j] == ".." && segs[j-1] != ".."){
									// flatten "../" references
									if(j == (segs.length - 1)){
										segs.splice(j, 1);
										segs[j - 1] = "";
									}else{
										segs.splice(j - 1, 2);
										j -= 2;
									}
								}
							}
							relobj.path = segs.join("/");
						}
					}
				}

				uri = [];
				if(relobj.scheme){
					uri.push(relobj.scheme, ":");
				}
				if(relobj.authority){
					uri.push("//", relobj.authority);
				}
				uri.push(relobj.path);
				if(relobj.query){
					uri.push("?", relobj.query);
				}
				if(relobj.fragment){
					uri.push("#", relobj.fragment);
				}
			}

			this.uri = uri.join("");

			// break the uri into its main components
			var r = this.uri.match(ore);

			this.scheme = r[2] || (r[1] ? "" : n);
			this.authority = r[4] || (r[3] ? "" : n);
			this.path = r[5]; // can never be undefined
			this.query = r[7] || (r[6] ? "" : n);
			this.fragment	 = r[9] || (r[8] ? "" : n);

			if(this.authority != n){
				// server based naming authority
				r = this.authority.match(ire);

				this.user = r[3] || n;
				this.password = r[4] || n;
				this.host = r[6] || r[7]; // ipv6 || ipv4
				this.port = r[9] || n;
			}
		};
	_Url.prototype.toString = function(){ return this.uri; };

	return dojo._Url = _Url;
});

},
'dojo/promise/all':function(){
define([
	"../_base/array",
	"../Deferred",
	"../when"
], function(array, Deferred, when){
	"use strict";

	// module:
	//		dojo/promise/all

	var some = array.some;

	return function all(objectOrArray){
		// summary:
		//		Takes multiple promises and returns a new promise that is fulfilled
		//		when all promises have been fulfilled.
		// description:
		//		Takes multiple promises and returns a new promise that is fulfilled
		//		when all promises have been fulfilled. If one of the promises is rejected,
		//		the returned promise is also rejected. Canceling the returned promise will
		//		*not* cancel any passed promises.
		// objectOrArray: Object|Array?
		//		The promise will be fulfilled with a list of results if invoked with an
		//		array, or an object of results when passed an object (using the same
		//		keys). If passed neither an object or array it is resolved with an
		//		undefined value.
		// returns: dojo/promise/Promise

		var object, array;
		if(objectOrArray instanceof Array){
			array = objectOrArray;
		}else if(objectOrArray && typeof objectOrArray === "object"){
			object = objectOrArray;
		}

		var results;
		var keyLookup = [];
		if(object){
			array = [];
			for(var key in object){
				if(Object.hasOwnProperty.call(object, key)){
					keyLookup.push(key);
					array.push(object[key]);
				}
			}
			results = {};
		}else if(array){
			results = [];
		}

		if(!array || !array.length){
			return new Deferred().resolve(results);
		}

		var deferred = new Deferred();
		deferred.promise.always(function(){
			results = keyLookup = null;
		});
		var waiting = array.length;
		some(array, function(valueOrPromise, index){
			if(!object){
				keyLookup.push(index);
			}
			when(valueOrPromise, function(value){
				if(!deferred.isFulfilled()){
					results[keyLookup[index]] = value;
					if(--waiting === 0){
						deferred.resolve(results);
					}
				}
			}, deferred.reject);
			return deferred.isFulfilled();
		});
		return deferred.promise;	// dojo/promise/Promise
	};
});

},
'vsd/common/message/VSDMessages':function(){
/**
 * A widget that displays status, informational, warning, and error messages.
 */
define([
	'dojo/_base/declare',
	"dojo/parser",
	'dijit/_WidgetBase',
	'dijit/_TemplatedMixin',
	'dijit/_WidgetsInTemplateMixin',
	'dojo/text!./VSDMessagesTemplate.html',
	"dojo/i18n!vsd/nls/resources",
	'dojo/_base/lang',
	'dojo/_base/fx',
	'dojo/aspect',
	'dojo/dom-class',
	'dojo/dom-construct',
	'dojo/dom-style',
	'dojo/dom-geometry',
	'dojo/fx',
	"dojo/fx/easing",
	'dojo/has',
	'dojo/query',
	"put-selector/put",
	'dijit/form/Button',
	'dojo/window'

], function (declare, parser, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, templateString, resources, lang,
	baseFx, aspect, domClass, domConstruct, domStyle, domGeom, coreFx, easing, has, query, put, Button, window) {

	return declare('Messages.VSDMessages', [  _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {

		widgetsInTemplate: true,
		templateString:    templateString,
		resources: resources,

		MessageTypes:     {
			MSG_ERROR:   0,
			MSG_WARNING: 1,
			MSG_INFO:    2,
			MSG_STATUS:  3
		},

		// width: Integer
		//    Width of the widget
		width:            '600px',

		// slideOutDuration: Integer
		//   How long the "slide out" animation takes, in milliseconds
		slideOutDuration: 1000,

		// fadeOut: boolean
		//   If true, this entire dijit, including infos, warnings, and errors,
		//   will fade away after a delay of fadeOutDelay.
		fadeOut:          false,

		// fadeOutDelay: Integer
		//   How long the this dijit should display before fading out.  If fadeOut
		//   is false then this parameter will only affect "status" messages.
		fadeOutDelay:     3000,

		// fadeOutDuration: Integer
		//   How long the "fade out" animation takes (if any), in milliseconds.
		//   If fadeOut is false then this parameter will only affect "status"
		//   messages.
		fadeOutDuration:  500,

		// closeDuration: Integer
		//   How long the "close section" animation takes, in milliseconds.
		closeDuration:    600,

		// forceBulleting: boolean
		//   If true, bullets are used no matter how many messages there are.
		forceBulleting:   true,

		// roundedCorners: boolean
		//   If true, the bottom corners of the dropdown should be rounded.
		roundedCorners:   false,

		attachTo: null,

		postCreate: function () {
			this.msgType = this.MessageTypes.MSG_INFO;
			this.inherited(arguments);
			this.hide();

			this.iconClasses = ["messageErrorIcon", "messageWarningIcon", "messageInfoIcon", "messageStatusIcon"];
			this.msgTypeClasses = ["messageErrors", "messageWarnings", "messageInfos", "messageStatuses"];

			// Set up the message sections to render
			var bullet = this.forceBulleting || this._getMaxMessageCount() > 1;

			domStyle.set(this.domNode, 'width', this.width);
//			domConstruct.place(this.domNode,this.attachTo,"first" );
			document.body.appendChild(this.domNode);
		},

		_prepareMessageSection: function (msgType, messages, bullet) {
			if (typeof messages === 'string') {
				messages = [ messages ];
			}

			var msg = null;
			this.msgType = msgType;
			if (this.prevIconClass) {
				dojo.toggleClass(this.messageIcon, this.prevIconClass);
				dojo.toggleClass(this.messageDiv, this.prevDivClass);
			}
			dojo.toggleClass(this.messageIcon, this.iconClasses[msgType]);
			dojo.toggleClass(this.messageDiv, this.msgTypeClasses[msgType]);
			this.prevIconClass = this.iconClasses[msgType];
			this.prevDivClass = this.msgTypeClasses[msgType];

			//if prev message delete it first
			dojo.query(".messageText > * ", this.domNode).forEach(dojo.destroy);

			if (bullet) {
				msg = put(this.messageText, "ul.msgList");
			}
			else {
				msg = put(this.messageText, "div");
			}

			//  domStyle.set(msg,'width', this.width *.9);

			for (var i = 0; i < messages.length; i++) {
				if (bullet) {
					put(msg, 'li', messages[i]);
				}
				else {
					put(msg, 'div', messages[i]);
				}
			}

			if (msgType == this.MessageTypes.MSG_STATUS) {
				domStyle.set(this.closeButton.domNode, 'display', 'none');
			}
			else {
				domStyle.set(this.closeButton.domNode, 'display', 'inline');
			}

//			if (!bullet) {
//				domStyle.set(msg, 'textAlign', 'center');
//			}
//			else {
//				domStyle.set(msg, 'textAlign', 'left');
//			}

			this.showMessages();
		},

		// summary:
		//   Displays this dijit, using the messages passed into the constructor.
		showMessages:           function (slideOutDuration) {
			var me = this;
			slideOutDuration = slideOutDuration || this.slideOutDuration;
			domStyle.set(this.messageDiv, 'display', 'block');
			// Place our container node at its initial position
			this.show();

			var delay = this.fadeOutDelay || 3500;
			var nodeSize = domGeom.getMarginBox(this.containerNode);
			var style = this.clipNode.style;
			style.left = ((window.getBox().w - nodeSize.w) / 2) + 'px';
			style.top = (0 - nodeSize.h) + 'px';
			style.height = nodeSize.h + 'px';
			style.width = nodeSize.w + 'px';
			var messageRemove = function () {
				if (me.msgType != me.MessageTypes.MSG_ERROR) {
					// coreFx.combine([
//					coreFx.wipeOut({ node: me.clipNode,
//							duration:        me.closeDuration,
//							easing:          easing.sineOut,
//							onEnd:           function () {
//								me.hide();
//							}}
//					).play();
					coreFx.slideTo({ node: me.clipNode,
							duration:        slideOutDuration,
							easing:          easing.sineIn,
							left:            style.left,
							top:             -nodeSize.h,
							onEnd:           function () {
								me.hide();
							}}
					).play();
				}
			};

			this.slideAnim = coreFx.slideTo({
				node:     this.clipNode,
				left:     style.left,
				top:      0,
				units:    'px',
				duration: slideOutDuration,
				onEnd:    function () {
					setTimeout(messageRemove, delay);
				}
			}).play();
		},

		show: function () {
			domStyle.set(this.domNode, 'display', 'block');
		},

		hide: function () {
			domStyle.set(this.domNode, 'display', 'none');
		},


		closeMessage: function () {
			var me = this;

//			coreFx.wipeOut({ node: me.messageDiv,
//					duration:        this.closeDuration,
//					easing:          easing.sineOut,
//					onEnd:           function () {
//						me.hide();
//					}
//				}
//			).play();
//			debugger
            if(domStyle.get(this.domNode,'display') !== 'none') {
                coreFx.slideTo({ node: me.clipNode,
                        duration:        this.closeDuration,
//					easing:          easing.sineIn,
                        left:            domGeom.position(me.domNode).x,
//					top:             -me.domNode.offsetHeight,
                        top:             -window.getBox().h,
                        onEnd:           function () {
                            me.hide();
                        }}
                ).play();
            }

		},

		clearMessage: function (e) {

			var me = this;
			if (this.domNode.style.display == 'none') {
				return;
			}
			//  if ( this.domNode.style.height == 0 ) return;
			this.closeMessage();
		},

		showError: function (message) {
			this._prepareMessageSection(this.MessageTypes.MSG_ERROR, message, false);

		},

		showStatus: function (message) {
			this._prepareMessageSection(this.MessageTypes.MSG_STATUS, message, false);

		},

		showInfo: function (message) {
			this._prepareMessageSection(this.MessageTypes.MSG_INFO, message, false);

		},

		showWarning: function (message) {
			this._prepareMessageSection(this.MessageTypes.MSG_WARNING, message, false);

		}

	});


});

},
'dijit/_WidgetsInTemplateMixin':function(){
define([
	"dojo/_base/array", // forEach()
	"dojo/aspect", // after()
	"dojo/_base/declare", // declare()
	"dojo/_base/lang",	// hitch()
	"dojo/parser" // parse()
], function(array, aspect, declare, lang, parser){

	// module:
	//		dijit/_WidgetsInTemplateMixin

	return declare("dijit._WidgetsInTemplateMixin", null, {
		// summary:
		//		Mixin to supplement _TemplatedMixin when template contains widgets

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  (Remove for 2.0 and assume true)
		widgetsInTemplate: true,

		// contextRequire: Function
		//		Used to provide a context require to the dojo/parser in order to be
		//		able to use relative MIDs (e.g. `./Widget`) in the widget's template.
		contextRequire: null,

		_beforeFillContent: function(){
			if(this.widgetsInTemplate){
				// Before copying over content, instantiate widgets in template
				var node = this.domNode;

				if(this.containerNode && !this.searchContainerNode){
					// Tell parse call below not to look for widgets inside of this.containerNode
					this.containerNode.stopParser = true;
				}

				parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang, textDir: this.textDir},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					contextRequire: this.contextRequire,
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}).then(lang.hitch(this, function(widgets){
					this._startupWidgets = widgets;

					// _WidgetBase::destroy() will destroy any supporting widgets under this.domNode.
					// If we wanted to, we could call this.own() on anything in this._startupWidgets that was moved outside
					// of this.domNode (like Dialog, which is moved to <body>).

					// Hook up attach points and events for nodes that were converted to widgets
					for(var i = 0; i < widgets.length; i++){
						this._processTemplateNode(widgets[i], function(n,p){
							// callback to get a property of a widget
							return n[p];
						}, function(widget, type, callback){
							// callback to do data-dojo-attach-event to a widget
							if(type in widget){
								// back-compat, remove for 2.0
								return widget.connect(widget, type, callback);
							}else{
								// 1.x may never hit this branch, but it's the default for 2.0
								return widget.on(type, callback, true);
							}
						});
					}

					// Cleanup flag set above, just in case
					if(this.containerNode && this.containerNode.stopParser){
						delete this.containerNode.stopParser;
					}
				}));

				if(!this._startupWidgets){
					throw new Error(this.declaredClass + ": parser returned unfilled promise (probably waiting for module auto-load), " +
						"unsupported by _WidgetsInTemplateMixin.   Must pre-load all supporting widgets before instantiation.");
				}
			}
		},

		_processTemplateNode: function(/*DOMNode|Widget*/ baseNode, getAttrFunc, attachFunc){
			// Override _AttachMixin._processNode to skip DOMNodes with data-dojo-type set.   They are handled separately
			// in the _beforeFillContent() code above.

			if(getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type")){
				return true;
			}

			return this.inherited(arguments);
		},

		startup: function(){
			array.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this._startupWidgets = null;
			this.inherited(arguments);
		}
	});
});

},
'dojo/fx':function(){
define([
	"./_base/lang",
	"./Evented",
	"./_base/kernel",
	"./_base/array",
	"./aspect",
	"./_base/fx",
	"./dom",
	"./dom-style",
	"./dom-geometry",
	"./ready",
	"require" // for context sensitive loading of Toggler
], function(lang, Evented, dojo, arrayUtil, aspect, baseFx, dom, domStyle, geom, ready, require){

	// module:
	//		dojo/fx
	
	// For back-compat, remove in 2.0.
	if(!dojo.isAsync){
		ready(0, function(){
			var requires = ["./fx/Toggler"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	var coreFx = dojo.fx = {
		// summary:
		//		Effects library on top of Base animations
	};

	var _baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		arrayUtil.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	_chain.prototype = new Evented();
	lang.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			this._onAnimateCtx.remove();
			this._onEndCtx.remove();
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = aspect.after(this._current, "onAnimate", lang.hitch(this, "_onAnimate"), true);
				this._onEndCtx = aspect.after(this._current, "onEnd", lang.hitch(this, "_onEnd"), true);
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = aspect.after(this._current, "beforeBegin", lang.hitch(this, function(){
					this._fire("beforeBegin");
				}), true),
				onBegin = aspect.after(this._current, "onBegin", lang.hitch(this, function(arg){
					this._fire("onBegin", arguments);
				}), true),
				onPlay = aspect.after(this._current, "onPlay", lang.hitch(this, function(arg){
					this._fire("onPlay", arguments);
					beforeBegin.remove();
					onBegin.remove();
					onPlay.remove();
				}));
			if(this._onAnimateCtx){
				this._onAnimateCtx.remove();
			}
			this._onAnimateCtx = aspect.after(this._current, "onAnimate", lang.hitch(this, "_onAnimate"), true);
			if(this._onEndCtx){
				this._onEndCtx.remove();
			}
			this._onEndCtx = aspect.after(this._current, "onEnd", lang.hitch(this, "_onEnd"), true);
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = aspect.after(this._current, "onPause", lang.hitch(this, function(arg){
						this._fire("onPause", arguments);
						e.remove();
					}), true);
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			arrayUtil.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = aspect.after(this._current, "onStop", lang.hitch(this, function(arg){
						this._fire("onStop", arguments);
						e.remove();
					}), true);
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ this._onAnimateCtx.remove(); }
			if(this._onEndCtx){ this._onEndCtx.remove(); }
		}
	});
	lang.extend(_chain, _baseObj);

	coreFx.chain = function(/*dojo/_base/fx.Animation[]*/ animations){
		// summary:
		//		Chain a list of `dojo.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo.Animation` which will play all passed
		//		`dojo.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain,
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations); // dojo/_base/fx.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		arrayUtil.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(aspect.after(a, "onEnd", lang.hitch(this, "_onEnd"), true));
		}, this);

		this._pseudoAnimation = new baseFx.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		arrayUtil.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"],
			function(evt){
				self._connects.push(aspect.after(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); },
				true));
			}
		);
	};
	lang.extend(_combine, {
		_doAction: function(action, args){
			arrayUtil.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			arrayUtil.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			arrayUtil.forEach(this._connects, function(handle){
				handle.remove();
			});
		}
	});
	lang.extend(_combine, _baseObj);

	coreFx.combine = function(/*dojo/_base/fx.Animation[]*/ animations){
		// summary:
		//		Combine a list of `dojo.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo.Animation`s to run in parallel,
		//		providing a new `dojo.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	var anim = fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	aspect.after(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	}, true);
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo/_base/fx.Animation
	};

	coreFx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	fx.wipeIn({
		//	|		node:"someId"
		//	|	}).play()
		var node = args.node = dom.byId(args.node), s = node.style, o;

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = domStyle.get(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		var fini = function(){
			s.height = "auto";
			s.overflow = o;
		};
		aspect.after(anim, "onStop", fini, true);
		aspect.after(anim, "onEnd", fini, true);

		return anim; // dojo/_base/fx.Animation
	};

	coreFx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it.
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	fx.wipeOut({ node:"someId" }).play()

		var node = args.node = dom.byId(args.node), s = node.style, o;

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		aspect.after(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		}, true);
		var fini = function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		};
		aspect.after(anim, "onStop", fini, true);
		aspect.after(anim, "onEnd", fini, true);

		return anim; // dojo/_base/fx.Animation
	};

	coreFx.slideTo = function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node"
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = dom.byId(args.node),
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = domStyle.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = geom.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = baseFx.animateProperty(lang.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		aspect.after(anim, "beforeBegin", init, true);

		return anim; // dojo/_base/fx.Animation
	};

	return coreFx;
});

},
'dojo/fx/easing':function(){
define(["../_base/lang"], function(lang){

// module:
//		dojo/fx/easing

var easingFuncs = {
	// summary:
	//		Collection of easing functions to use beyond the default
	//		`dojo._defaultEasing` function.
	// description:
	//		Easing functions are used to manipulate the iteration through
	//		an `dojo.Animation`s _Line. _Line being the properties of an Animation,
	//		and the easing function progresses through that Line determining
	//		how quickly (or slowly) it should go. Or more accurately: modify
	//		the value of the _Line based on the percentage of animation completed.
	//
	//		All functions follow a simple naming convention of "ease type" + "when".
	//		If the name of the function ends in Out, the easing described appears
	//		towards the end of the animation. "In" means during the beginning,
	//		and InOut means both ranges of the Animation will applied, both
	//		beginning and end.
	//
	//		One does not call the easing function directly, it must be passed to
	//		the `easing` property of an animation.
	// example:
	//	|	dojo.require("dojo.fx.easing");
	//	|	var anim = dojo.fadeOut({
	//	|		node: 'node',
	//	|		duration: 2000,
	//	|		//	note there is no ()
	//	|		easing: dojo.fx.easing.quadIn
	//	|	}).play();
	//

	linear: function(/* Decimal? */n){
		// summary:
		//		A linear easing function
		return n;
	},

	quadIn: function(/* Decimal? */n){
		return Math.pow(n, 2);
	},

	quadOut: function(/* Decimal? */n){
		return n * (n - 2) * -1;
	},

	quadInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 2) / 2; }
		return -1 * ((--n) * (n - 2) - 1) / 2;
	},

	cubicIn: function(/* Decimal? */n){
		return Math.pow(n, 3);
	},

	cubicOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 3) + 1;
	},

	cubicInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 3) / 2; }
		n -= 2;
		return (Math.pow(n, 3) + 2) / 2;
	},

	quartIn: function(/* Decimal? */n){
		return Math.pow(n, 4);
	},

	quartOut: function(/* Decimal? */n){
		return -1 * (Math.pow(n - 1, 4) - 1);
	},

	quartInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 4) / 2; }
		n -= 2;
		return -1 / 2 * (Math.pow(n, 4) - 2);
	},

	quintIn: function(/* Decimal? */n){
		return Math.pow(n, 5);
	},

	quintOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 5) + 1;
	},

	quintInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 5) / 2; }
		n -= 2;
		return (Math.pow(n, 5) + 2) / 2;
	},

	sineIn: function(/* Decimal? */n){
		return -1 * Math.cos(n * (Math.PI / 2)) + 1;
	},

	sineOut: function(/* Decimal? */n){
		return Math.sin(n * (Math.PI / 2));
	},

	sineInOut: function(/* Decimal? */n){
		return -1 * (Math.cos(Math.PI * n) - 1) / 2;
	},

	expoIn: function(/* Decimal? */n){
		return (n == 0) ? 0 : Math.pow(2, 10 * (n - 1));
	},

	expoOut: function(/* Decimal? */n){
		return (n == 1) ? 1 : (-1 * Math.pow(2, -10 * n) + 1);
	},

	expoInOut: function(/* Decimal? */n){
		if(n == 0){ return 0; }
		if(n == 1){ return 1; }
		n = n * 2;
		if(n < 1){ return Math.pow(2, 10 * (n - 1)) / 2; }
		--n;
		return (-1 * Math.pow(2, -10 * n) + 2) / 2;
	},

	circIn: function(/* Decimal? */n){
		return -1 * (Math.sqrt(1 - Math.pow(n, 2)) - 1);
	},

	circOut: function(/* Decimal? */n){
		n = n - 1;
		return Math.sqrt(1 - Math.pow(n, 2));
	},

	circInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return -1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) - 1); }
		n -= 2;
		return 1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) + 1);
	},

	backIn: function(/* Decimal? */n){
		// summary:
		//		An easing function that starts away from the target,
		//		and quickly accelerates towards the end value.
		//
		//		Use caution when the easing will cause values to become
		//		negative as some properties cannot be set to negative values.
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n - s);
	},

	backOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that pops past the range briefly, and slowly comes back.
		// description:
		//		An easing function that pops past the range briefly, and slowly comes back.
		//
		//		Use caution when the easing will cause values to become negative as some
		//		properties cannot be set to negative values.

		n = n - 1;
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n + s) + 1;
	},

	backInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function combining the effects of `backIn` and `backOut`
		// description:
		//		An easing function combining the effects of `backIn` and `backOut`.
		//		Use caution when the easing will cause values to become negative
		//		as some properties cannot be set to negative values.
		var s = 1.70158 * 1.525;
		n = n * 2;
		if(n < 1){ return (Math.pow(n, 2) * ((s + 1) * n - s)) / 2; }
		n-=2;
		return (Math.pow(n, 2) * ((s + 1) * n + s) + 2) / 2;
	},

	elasticIn: function(/* Decimal? */n){
		// summary:
		//		An easing function the elastically snaps from the start value
		// description:
		//		An easing function the elastically snaps from the start value
		//
		//		Use caution when the elasticity will cause values to become negative
		//		as some properties cannot be set to negative values.
		if(n == 0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		n = n - 1;
		return -1 * Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p);
	},

	elasticOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		// description:
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		//
		//		Use caution when the elasticity will cause values to become
		//		negative as some properties cannot be set to negative values.
		if(n==0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		return Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p) + 1;
	},

	elasticInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		// description:
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		//
		//		Use caution when the elasticity will cause values to become
		//		negative as some properties cannot be set to negative values.
		if(n == 0) return 0;
		n = n * 2;
		if(n == 2) return 1;
		var p = .3 * 1.5;
		var s = p / 4;
		if(n < 1){
			n -= 1;
			return -.5 * (Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p));
		}
		n -= 1;
		return .5 * (Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p)) + 1;
	},

	bounceIn: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' near the beginning of an Animation
		return (1 - easingFuncs.bounceOut(1 - n)); // Decimal
	},

	bounceOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' near the end of an Animation
		var s = 7.5625;
		var p = 2.75;
		var l;
		if(n < (1 / p)){
			l = s * Math.pow(n, 2);
		}else if(n < (2 / p)){
			n -= (1.5 / p);
			l = s * Math.pow(n, 2) + .75;
		}else if(n < (2.5 / p)){
			n -= (2.25 / p);
			l = s * Math.pow(n, 2) + .9375;
		}else{
			n -= (2.625 / p);
			l = s * Math.pow(n, 2) + .984375;
		}
		return l;
	},

	bounceInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' at the beginning and end of the Animation
		if(n < 0.5){ return easingFuncs.bounceIn(n * 2) / 2; }
		return (easingFuncs.bounceOut(n * 2 - 1) / 2) + 0.5; // Decimal
	}
};

lang.setObject("dojo.fx.easing", easingFuncs);

return easingFuncs;
});

},
'dijit/TooltipDialog':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.replace
	"dojo/has",
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"./focus",
	"./layout/ContentPane",
	"./_DialogMixin",
	"./form/_FormMixin",
	"./_TemplatedMixin",
	"dojo/text!./templates/TooltipDialog.html",
	"./main"        // exports methods to dijit global
], function(declare, domClass, has, keys, lang, on, focus, ContentPane, _DialogMixin, _FormMixin, _TemplatedMixin, template, dijit){

	// module:
	//		dijit/TooltipDialog


	var TooltipDialog = declare("dijit.TooltipDialog",
		[ContentPane, _TemplatedMixin, _FormMixin, _DialogMixin], {
			// summary:
			//		Pops up a dialog that appears like a Tooltip

			// title: String
			//		Description of tooltip dialog (required for a11y)
			title: "",

			// doLayout: [protected] Boolean
			//		Don't change this parameter from the default value.
			//		This ContentPane parameter doesn't make sense for TooltipDialog, since TooltipDialog
			//		is never a child of a layout container, nor can you specify the size of
			//		TooltipDialog in order to control the size of an inner widget.
			doLayout: false,

			// autofocus: Boolean
			//		A Toggle to modify the default focus behavior of a Dialog, which
			//		is to focus on the first dialog element after opening the dialog.
			//		False will disable autofocusing.  Default: true.
			autofocus: true,

			// baseClass: [protected] String
			//		The root className to use for the various states of this widget
			baseClass: "dijitTooltipDialog",

			// _firstFocusItem: [private readonly] DomNode
			//		The pointer to the first focusable node in the dialog.
			//		Set by `dijit/_DialogMixin._getFocusItems()`.
			_firstFocusItem: null,

			// _lastFocusItem: [private readonly] DomNode
			//		The pointer to which node has focus prior to our dialog.
			//		Set by `dijit/_DialogMixin._getFocusItems()`.
			_lastFocusItem: null,

			templateString: template,

			_setTitleAttr: "containerNode",

			postCreate: function(){
				this.inherited(arguments);
				this.own(on(this.containerNode, "keydown", lang.hitch(this, "_onKey")));
			},

			orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ tooltipCorner){
				// summary:
				//		Configure widget to be displayed in given position relative to the button.
				//		This is called from the dijit.popup code, and should not be called
				//		directly.
				// tags:
				//		protected

				// Note: intentionally not using dijitTooltip class since that sets position:absolute, which
				// confuses dijit/popup trying to get the size of the tooltip.
				var newC = {
					// Real around node
					"MR-ML": "dijitTooltipRight",
					"ML-MR": "dijitTooltipLeft",
					"TM-BM": "dijitTooltipAbove",
					"BM-TM": "dijitTooltipBelow",
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft",

					// Positioning "around" a point, ex: mouse position
					"BR-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"BL-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TL-BR": "dijitTooltipAbove dijitTooltipABRight",
					"TR-BL": "dijitTooltipAbove dijitTooltipABLeft"
				}[aroundCorner + "-" + tooltipCorner];

				domClass.replace(this.domNode, newC, this._currentOrientClass || "");
				this._currentOrientClass = newC;

				// Tooltip.orient() has code to reposition connector for when Tooltip is before/after anchor.
				// Not putting here to avoid code bloat, and since TooltipDialogs are generally above/below.
				// Should combine code from Tooltip and TooltipDialog.
			},

			focus: function(){
				// summary:
				//		Focus on first field
				this._getFocusItems(this.containerNode);
				focus.focus(this._firstFocusItem);
			},

			onOpen: function(/*Object*/ pos){
				// summary:
				//		Called when dialog is displayed.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected

				this.orient(this.domNode, pos.aroundCorner, pos.corner);

				// Position the tooltip connector for middle alignment.
				// This could not have been done in orient() since the tooltip wasn't positioned at that time.
				var aroundNodeCoords = pos.aroundNodePos;
				if(pos.corner.charAt(0) == 'M' && pos.aroundCorner.charAt(0) == 'M'){
					this.connectorNode.style.top = aroundNodeCoords.y + ((aroundNodeCoords.h - this.connectorNode.offsetHeight) >> 1) - pos.y + "px";
					this.connectorNode.style.left = "";
				}else if(pos.corner.charAt(1) == 'M' && pos.aroundCorner.charAt(1) == 'M'){
					this.connectorNode.style.left = aroundNodeCoords.x + ((aroundNodeCoords.w - this.connectorNode.offsetWidth) >> 1) - pos.x + "px";
				}

				this._onShow(); // lazy load trigger  (TODO: shouldn't we load before positioning?)
			},

			onClose: function(){
				// summary:
				//		Called when dialog is hidden.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected
				this.onHide();
			},

			_onKey: function(/*Event*/ evt){
				// summary:
				//		Handler for keydown events
				// description:
				//		Keep keyboard focus in dialog; close dialog on escape key
				// tags:
				//		private

				if(evt.keyCode == keys.ESCAPE){
					// Use defer to avoid crash on IE, see #10396.  Not sure if this is still needed or not.
					// If this if() wasn't here, presumably dijit/popup would catch the ESCAPE key and close the popup.
					this.defer("onCancel");
					evt.stopPropagation();
					evt.preventDefault();
				}else if(evt.keyCode == keys.TAB){
					var node = evt.target;
					this._getFocusItems(this.containerNode);
					if(this._firstFocusItem == this._lastFocusItem){
						evt.stopPropagation();
						evt.preventDefault();
					}else if(node == this._firstFocusItem && evt.shiftKey){
						focus.focus(this._lastFocusItem); // send focus to last item in dialog
						evt.stopPropagation();
						evt.preventDefault();
					}else if(node == this._lastFocusItem && !evt.shiftKey){
						focus.focus(this._firstFocusItem); // send focus to first item in dialog
						evt.stopPropagation();
						evt.preventDefault();
					}else{
						// we want the browser's default tab handling to move focus
						// but we don't want the tab to propagate upwards
						evt.stopPropagation();
					}
				}
			}
		});

	if(has("dojo-bidi")){
		TooltipDialog.extend({
			_setTitleAttr: function(/*String*/ title){
				this.containerNode.title = (this.textDir && this.enforceTextDirWithUcc) ? this.enforceTextDirWithUcc(null, title) : title;
				this._set("title", title);
			},

			_setTextDirAttr: function(/*String*/ textDir){
				if(!this._created || this.textDir != textDir){
					this._set("textDir", textDir);
					if(this.textDir && this.title){
						this.containerNode.title = this.enforceTextDirWithUcc(null, this.title);
					}
				}
			}
		});
	}

	return TooltipDialog;
});

},
'dijit/layout/ContentPane':function(){
define([
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.mixin lang.delegate lang.hitch lang.isFunction lang.isObject
	"../_Widget",
	"../_Container",
	"./_ContentPaneResizeMixin",
	"dojo/string", // string.substitute
	"dojo/html", // html._ContentSetter
	"dojo/i18n!../nls/loading",
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/_base/Deferred", // Deferred
	"dojo/dom", // dom.byId
	"dojo/dom-attr", // domAttr.attr
	"dojo/dom-construct", // empty()
	"dojo/_base/xhr", // xhr.get
	"dojo/i18n", // i18n.getLocalization
	"dojo/when"
], function(kernel, lang, _Widget, _Container, _ContentPaneResizeMixin, string, html, nlsLoading, array, declare,
			Deferred, dom, domAttr, domConstruct, xhr, i18n, when){

	// module:
	//		dijit/layout/ContentPane

	return declare("dijit.layout.ContentPane", [_Widget, _Container, _ContentPaneResizeMixin], {
		// summary:
		//		A widget containing an HTML fragment, specified inline
		//		or by uri.  Fragment may include widgets.
		//
		// description:
		//		This widget embeds a document fragment in the page, specified
		//		either by uri, javascript generated markup or DOM reference.
		//		Any widgets within this content are instantiated and managed,
		//		but laid out according to the HTML structure.  Unlike IFRAME,
		//		ContentPane embeds a document fragment as would be found
		//		inside the BODY tag of a full HTML document.  It should not
		//		contain the HTML, HEAD, or BODY tags.
		//		For more advanced functionality with scripts and
		//		stylesheets, see dojox/layout/ContentPane.  This widget may be
		//		used stand alone or as a base class for other widgets.
		//		ContentPane is useful as a child of other layout containers
		//		such as BorderContainer or TabContainer, but note that those
		//		widgets can contain any widget as a child.
		//
		// example:
		//		Some quick samples:
		//		To change the innerHTML:
		// |		cp.set('content', '<b>new content</b>')`
		//		Or you can send it a NodeList:
		// |		cp.set('content', dojo.query('div [class=selected]', userSelection))
		//		To do an ajax update:
		// |		cp.set('href', url)

		// href: String
		//		The href of the content that displays now.
		//		Set this at construction if you want to load data externally when the
		//		pane is shown.  (Set preload=true to load it immediately.)
		//		Changing href after creation doesn't have any effect; Use set('href', ...);
		href: "",

		// content: String|DomNode|NodeList|dijit/_Widget
		//		The innerHTML of the ContentPane.
		//		Note that the initialization parameter / argument to set("content", ...)
		//		can be a String, DomNode, Nodelist, or _Widget.
		content: "",

		// extractContent: Boolean
		//		Extract visible content from inside of `<body> .... </body>`.
		//		I.e., strip `<html>` and `<head>` (and it's contents) from the href
		extractContent: false,

		// parseOnLoad: Boolean
		//		Parse content and create the widgets, if any.
		parseOnLoad: true,

		// parserScope: String
		//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
		//		will search for data-dojo-type (or dojoType).  For backwards compatibility
		//		reasons defaults to dojo._scopeName (which is "dojo" except when
		//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
		parserScope: kernel._scopeName,

		// preventCache: Boolean
		//		Prevent caching of data from href's by appending a timestamp to the href.
		preventCache: false,

		// preload: Boolean
		//		Force load of data on initialization even if pane is hidden.
		preload: false,

		// refreshOnShow: Boolean
		//		Refresh (re-download) content when pane goes from hidden to shown
		refreshOnShow: false,

		// loadingMessage: String
		//		Message that shows while downloading
		loadingMessage: "<span class='dijitContentPaneLoading'><span class='dijitInline dijitIconLoading'></span>${loadingState}</span>",

		// errorMessage: String
		//		Message that shows if an error occurs
		errorMessage: "<span class='dijitContentPaneError'><span class='dijitInline dijitIconError'></span>${errorState}</span>",

		// isLoaded: [readonly] Boolean
		//		True if the ContentPane has data in it, either specified
		//		during initialization (via href or inline content), or set
		//		via set('content', ...) / set('href', ...)
		//
		//		False if it doesn't have any content, or if ContentPane is
		//		still in the process of downloading href.
		isLoaded: false,

		baseClass: "dijitContentPane",

		/*======
		 // ioMethod: dojo/_base/xhr.get|dojo._base/xhr.post
		 //		Function that should grab the content specified via href.
		 ioMethod: dojo.xhrGet,
		 ======*/

		// ioArgs: Object
		//		Parameters to pass to xhrGet() request, for example:
		// |	<div data-dojo-type="dijit/layout/ContentPane" data-dojo-props="href: './bar', ioArgs: {timeout: 500}">
		ioArgs: {},

		// onLoadDeferred: [readonly] dojo.Deferred
		//		This is the `dojo.Deferred` returned by set('href', ...) and refresh().
		//		Calling onLoadDeferred.then() registers your
		//		callback to be called only once, when the prior set('href', ...) call or
		//		the initial href parameter to the constructor finishes loading.
		//
		//		This is different than an onLoad() handler which gets called any time any href
		//		or content is loaded.
		onLoadDeferred: null,

		// Cancel _WidgetBase's _setTitleAttr because we don't want the title attribute (used to specify
		// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
		// entire pane.
		_setTitleAttr: null,

		// Flag to parser that I'll parse my contents, so it shouldn't.
		stopParser: true,

		// template: [private] Boolean
		//		Flag from the parser that this ContentPane is inside a template
		//		so the contents are pre-parsed.
		// TODO: this declaration can be commented out in 2.0
		template: false,

		markupFactory: function(params, node, ctor){
			var self = new ctor(params, node);

			// If a parse has started but is waiting for modules to load, then return a Promise for when the parser
			// finishes.  Don't return a promise though for the case when content hasn't started loading because the
			// ContentPane is hidden and it has an href (ex: hidden pane of a TabContainer).   In that case we consider
			// that initialization has already finished.
			return !self.href && self._contentSetter && self._contentSetter.parseDeferred && !self._contentSetter.parseDeferred.isFulfilled() ?
				self._contentSetter.parseDeferred.then(function(){
					return self;
				}) : self;
		},

		create: function(params, srcNodeRef){
			// Convert a srcNodeRef argument into a content parameter, so that the original contents are
			// processed in the same way as contents set via set("content", ...), calling the parser etc.
			// Avoid modifying original params object since that breaks NodeList instantiation, see #11906.
			if((!params || !params.template) && srcNodeRef && !("href" in params) && !("content" in params)){
				srcNodeRef = dom.byId(srcNodeRef);
				var df = srcNodeRef.ownerDocument.createDocumentFragment();
				while(srcNodeRef.firstChild){
					df.appendChild(srcNodeRef.firstChild);
				}
				params = lang.delegate(params, {content: df});
			}
			this.inherited(arguments, [params, srcNodeRef]);
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			var messages = i18n.getLocalization("dijit", "loading", this.lang);
			this.loadingMessage = string.substitute(this.loadingMessage, messages);
			this.errorMessage = string.substitute(this.errorMessage, messages);
		},

		buildRendering: function(){
			this.inherited(arguments);

			// Since we have no template we need to set this.containerNode ourselves, to make getChildren() work.
			// For subclasses of ContentPane that do have a template, does nothing.
			if(!this.containerNode){
				this.containerNode = this.domNode;
			}

			// remove the title attribute so it doesn't show up when hovering
			// over a node  (TODO: remove in 2.0, no longer needed after #11490)
			this.domNode.removeAttribute("title");
		},

		startup: function(){
			// summary:
			//		Call startup() on all children including non _Widget ones like dojo/dnd/Source objects

			// This starts all the widgets
			this.inherited(arguments);

			// And this catches stuff like dojo/dnd/Source
			if(this._contentSetter){
				array.forEach(this._contentSetter.parseResults, function(obj){
					if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
						obj.startup();
						obj._started = true;
					}
				}, this);
			}
		},

		_startChildren: function(){
			// summary:
			//		Called when content is loaded.   Calls startup on each child widget.   Similar to ContentPane.startup()
			//		itself, but avoids marking the ContentPane itself as "restarted" (see #15581).

			// This starts all the widgets
			array.forEach(this.getChildren(), function(obj){
				if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			});

			// And this catches stuff like dojo/dnd/Source
			if(this._contentSetter){
				array.forEach(this._contentSetter.parseResults, function(obj){
					if(!obj._started && !obj._destroyed && lang.isFunction(obj.startup)){
						obj.startup();
						obj._started = true;
					}
				}, this);
			}
		},

		setHref: function(/*String|Uri*/ href){
			// summary:
			//		Deprecated.   Use set('href', ...) instead.
			kernel.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
			return this.set("href", href);
		},
		_setHrefAttr: function(/*String|Uri*/ href){
			// summary:
			//		Hook so set("href", ...) works.
			// description:
			//		Reset the (external defined) content of this pane and replace with new url
			//		Note: It delays the download until widget is shown if preload is false.
			// href:
			//		url to the page you want to get, must be within the same domain as your mainpage

			// Cancel any in-flight requests (a set('href', ...) will cancel any in-flight set('href', ...))
			this.cancel();

			this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
			this.onLoadDeferred.then(lang.hitch(this, "onLoad"));

			this._set("href", href);

			// _setHrefAttr() is called during creation and by the user, after creation.
			// Assuming preload == false, only in the second case do we actually load the URL;
			// otherwise it's done in startup(), and only if this widget is shown.
			if(this.preload || (this._created && this._isShown())){
				this._load();
			}else{
				// Set flag to indicate that href needs to be loaded the next time the
				// ContentPane is made visible
				this._hrefChanged = true;
			}

			return this.onLoadDeferred;		// Deferred
		},

		setContent: function(/*String|DomNode|Nodelist*/data){
			// summary:
			//		Deprecated.   Use set('content', ...) instead.
			kernel.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
			this.set("content", data);
		},
		_setContentAttr: function(/*String|DomNode|Nodelist*/data){
			// summary:
			//		Hook to make set("content", ...) work.
			//		Replaces old content with data content, include style classes from old content
			// data:
			//		the new Content may be String, DomNode or NodeList
			//
			//		if data is a NodeList (or an array of nodes) nodes are copied
			//		so you can import nodes from another document implicitly

			// clear href so we can't run refresh and clear content
			// refresh should only work if we downloaded the content
			this._set("href", "");

			// Cancel any in-flight requests (a set('content', ...) will cancel any in-flight set('href', ...))
			this.cancel();

			// Even though user is just setting content directly, still need to define an onLoadDeferred
			// because the _onLoadHandler() handler is still getting called from setContent()
			this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
			if(this._created){
				// For back-compat reasons, call onLoad() for set('content', ...)
				// calls but not for content specified in srcNodeRef (ie: <div data-dojo-type=ContentPane>...</div>)
				// or as initialization parameter (ie: new ContentPane({content: ...})
				this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
			}

			this._setContent(data || "");

			this._isDownloaded = false; // mark that content is from a set('content') not a set('href')

			return this.onLoadDeferred;	// Deferred
		},
		_getContentAttr: function(){
			// summary:
			//		Hook to make get("content") work
			return this.containerNode.innerHTML;
		},

		cancel: function(){
			// summary:
			//		Cancels an in-flight download of content
			if(this._xhrDfd && (this._xhrDfd.fired == -1)){
				this._xhrDfd.cancel();
			}
			delete this._xhrDfd; // garbage collect

			this.onLoadDeferred = null;
		},

		destroy: function(){
			this.cancel();
			this.inherited(arguments);
		},

		destroyRecursive: function(/*Boolean*/ preserveDom){
			// summary:
			//		Destroy the ContentPane and its contents

			// if we have multiple controllers destroying us, bail after the first
			if(this._beingDestroyed){
				return;
			}
			this.inherited(arguments);
		},

		_onShow: function(){
			// summary:
			//		Called when the ContentPane is made visible
			// description:
			//		For a plain ContentPane, this is called on initialization, from startup().
			//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
			//		called whenever the pane is made visible.
			//
			//		Does necessary processing, including href download and layout/resize of
			//		child widget(s)

			this.inherited(arguments);

			if(this.href){
				if(!this._xhrDfd && // if there's an href that isn't already being loaded
					(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
					){
					return this.refresh();	// If child has an href, promise that fires when the load is complete
				}
			}
		},

		refresh: function(){
			// summary:
			//		[Re]download contents of href and display
			// description:
			//		1. cancels any currently in-flight requests
			//		2. posts "loading..." message
			//		3. sends XHR to download new data

			// Cancel possible prior in-flight request
			this.cancel();

			this.onLoadDeferred = new Deferred(lang.hitch(this, "cancel"));
			this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
			this._load();
			return this.onLoadDeferred;		// If child has an href, promise that fires when refresh is complete
		},

		_load: function(){
			// summary:
			//		Load/reload the href specified in this.href

			// display loading message
			this._setContent(this.onDownloadStart(), true);

			var self = this;
			var getArgs = {
				preventCache: (this.preventCache || this.refreshOnShow),
				url: this.href,
				handleAs: "text"
			};
			if(lang.isObject(this.ioArgs)){
				lang.mixin(getArgs, this.ioArgs);
			}

			var hand = (this._xhrDfd = (this.ioMethod || xhr.get)(getArgs)),
				returnedHtml;

			hand.then(
				function(html){
					returnedHtml = html;
					try{
						self._isDownloaded = true;
						return self._setContent(html, false);
					}catch(err){
						self._onError('Content', err); // onContentError
					}
				},
				function(err){
					if(!hand.canceled){
						// show error message in the pane
						self._onError('Download', err); // onDownloadError
					}
					delete self._xhrDfd;
					return err;
				}
			).then(function(){
					self.onDownloadEnd();
					delete self._xhrDfd;
					return returnedHtml;
				});

			// Remove flag saying that a load is needed
			delete this._hrefChanged;
		},

		_onLoadHandler: function(data){
			// summary:
			//		This is called whenever new content is being loaded
			this._set("isLoaded", true);
			try{
				this.onLoadDeferred.resolve(data);
			}catch(e){
				console.error('Error ' + this.widgetId + ' running custom onLoad code: ' + e.message);
			}
		},

		_onUnloadHandler: function(){
			// summary:
			//		This is called whenever the content is being unloaded
			this._set("isLoaded", false);
			try{
				this.onUnload();
			}catch(e){
				console.error('Error ' + this.widgetId + ' running custom onUnload code: ' + e.message);
			}
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			// summary:
			//		Destroy all the widgets inside the ContentPane and empty containerNode

			// Make sure we call onUnload (but only when the ContentPane has real content)
			if(this.isLoaded){
				this._onUnloadHandler();
			}

			// Even if this.isLoaded == false there might still be a "Loading..." message
			// to erase, so continue...

			// For historical reasons we need to delete all widgets under this.containerNode,
			// even ones that the user has created manually.
			var setter = this._contentSetter;
			array.forEach(this.getChildren(), function(widget){
				if(widget.destroyRecursive){
					// All widgets will hit this branch
					widget.destroyRecursive(preserveDom);
				}else if(widget.destroy){
					// Things like dojo/dnd/Source have destroy(), not destroyRecursive()
					widget.destroy(preserveDom);
				}
				widget._destroyed = true;
			});
			if(setter){
				// Most of the widgets in setter.parseResults have already been destroyed, but
				// things like Menu that have been moved to <body> haven't yet
				array.forEach(setter.parseResults, function(widget){
					if(!widget._destroyed){
						if(widget.destroyRecursive){
							// All widgets will hit this branch
							widget.destroyRecursive(preserveDom);
						}else if(widget.destroy){
							// Things like dojo/dnd/Source have destroy(), not destroyRecursive()
							widget.destroy(preserveDom);
						}
						widget._destroyed = true;
					}
				});
				delete setter.parseResults;
			}

			// And then clear away all the DOM nodes
			if(!preserveDom){
				domConstruct.empty(this.containerNode);
			}

			// Delete any state information we have about current contents
			delete this._singleChild;
		},

		_setContent: function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
			// summary:
			//		Insert the content into the container node
			// returns:
			//		Returns a Deferred promise that is resolved when the content is parsed.

			// first get rid of child widgets
			this.destroyDescendants();

			// html.set will take care of the rest of the details
			// we provide an override for the error handling to ensure the widget gets the errors
			// configure the setter instance with only the relevant widget instance properties
			// NOTE: unless we hook into attr, or provide property setters for each property,
			// we need to re-configure the ContentSetter with each use
			var setter = this._contentSetter;
			if(!(setter && setter instanceof html._ContentSetter)){
				setter = this._contentSetter = new html._ContentSetter({
					node: this.containerNode,
					_onError: lang.hitch(this, this._onError),
					onContentError: lang.hitch(this, function(e){
						// fires if a domfault occurs when we are appending this.errorMessage
						// like for instance if domNode is a UL and we try append a DIV
						var errMess = this.onContentError(e);
						try{
							this.containerNode.innerHTML = errMess;
						}catch(e){
							console.error('Fatal ' + this.id + ' could not change content due to ' + e.message, e);
						}
					})/*,
					 _onError */
				});
			}

			var setterParams = lang.mixin({
				cleanContent: this.cleanContent,
				extractContent: this.extractContent,
				parseContent: !cont.domNode && this.parseOnLoad,
				parserScope: this.parserScope,
				startup: false,
				dir: this.dir,
				lang: this.lang,
				textDir: this.textDir
			}, this._contentSetterParams || {});

			var p = setter.set((lang.isObject(cont) && cont.domNode) ? cont.domNode : cont, setterParams);

			// dojox/layout/html/_base::_ContentSetter.set() returns a Promise that indicates when everything is completed.
			// dojo/html::_ContentSetter.set() currently returns the DOMNode, but that will be changed for 2.0.
			// So, if set() returns a promise then use it, otherwise fallback to waiting on setter.parseDeferred
			var self = this;
			return when(p && p.then ? p : setter.parseDeferred, function(){
				// setter params must be pulled afresh from the ContentPane each time
				delete self._contentSetterParams;

				if(!isFakeContent){
					if(self._started){
						// Startup each top level child widget (and they will start their children, recursively)
						self._startChildren();

						// Call resize() on each of my child layout widgets,
						// or resize() on my single child layout widget...
						// either now (if I'm currently visible) or when I become visible
						self._scheduleLayout();
					}
					self._onLoadHandler(cont);
				}
			});
		},

		_onError: function(type, err, consoleText){
			this.onLoadDeferred.reject(err);

			// shows user the string that is returned by on[type]Error
			// override on[type]Error and return your own string to customize
			var errText = this['on' + type + 'Error'].call(this, err);
			if(consoleText){
				console.error(consoleText, err);
			}else if(errText){// a empty string won't change current content
				this._setContent(errText, true);
			}
		},

		// EVENT's, should be overide-able
		onLoad: function(/*===== data =====*/){
			// summary:
			//		Event hook, is called after everything is loaded and widgetified
			// tags:
			//		callback
		},

		onUnload: function(){
			// summary:
			//		Event hook, is called before old content is cleared
			// tags:
			//		callback
		},

		onDownloadStart: function(){
			// summary:
			//		Called before download starts.
			// description:
			//		The string returned by this function will be the html
			//		that tells the user we are loading something.
			//		Override with your own function if you want to change text.
			// tags:
			//		extension
			return this.loadingMessage;
		},

		onContentError: function(/*Error*/ /*===== error =====*/){
			// summary:
			//		Called on DOM faults, require faults etc. in content.
			//
			//		In order to display an error message in the pane, return
			//		the error message from this method, as an HTML string.
			//
			//		By default (if this method is not overriden), it returns
			//		nothing, so the error message is just printed to the console.
			// tags:
			//		extension
		},

		onDownloadError: function(/*Error*/ /*===== error =====*/){
			// summary:
			//		Called when download error occurs.
			//
			//		In order to display an error message in the pane, return
			//		the error message from this method, as an HTML string.
			//
			//		Default behavior (if this method is not overriden) is to display
			//		the error message inside the pane.
			// tags:
			//		extension
			return this.errorMessage;
		},

		onDownloadEnd: function(){
			// summary:
			//		Called when download is finished.
			// tags:
			//		callback
		}
	});
});

},
'dijit/layout/_ContentPaneResizeMixin':function(){
define([
	"dojo/_base/array", // array.filter array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.contains domClass.toggle
	"dojo/dom-geometry", // domGeometry.contentBox domGeometry.marginBox
	"dojo/dom-style",
	"dojo/_base/lang", // lang.mixin
	"dojo/query", // query
	"dojo/sniff", // has("ie")
	"../registry", // registry.byId
	"../Viewport",
	"./utils" // marginBox2contextBox
], function(array, declare, domClass, domGeometry, domStyle, lang, query, has,
			registry, Viewport, layoutUtils){

	// module:
	//		dijit/layout/_ContentPaneResizeMixin

	return declare("dijit.layout._ContentPaneResizeMixin", null, {
		// summary:
		//		Resize() functionality of ContentPane.   If there's a single layout widget
		//		child then it will call resize() with the same dimensions as the ContentPane.
		//		Otherwise just calls resize on each child.
		//
		//		Also implements basic startup() functionality, where starting the parent
		//		will start the children

		// doLayout: Boolean
		//		- false - don't adjust size of children
		//		- true - if there is a single visible child widget, set it's size to however big the ContentPane is
		doLayout: true,

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget will call resize() on it's child widgets
		//		when they become visible.
		isLayoutContainer: true,

		startup: function(){
			// summary:
			//		See `dijit/layout/_LayoutWidget.startup()` for description.
			//		Although ContentPane doesn't extend _LayoutWidget, it does implement
			//		the same API.

			if(this._started){
				return;
			}

			var parent = this.getParent();
			this._childOfLayoutWidget = parent && parent.isLayoutContainer;

			// I need to call resize() on my child/children (when I become visible), unless
			// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
			this._needLayout = !this._childOfLayoutWidget;

			this.inherited(arguments);

			if(this._isShown()){
				this._onShow();
			}

			if(!this._childOfLayoutWidget){
				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when viewport size changes so that I can re-layout.
				// This is more for subclasses of ContentPane than ContentPane itself, although it
				// could be useful for a ContentPane if it has a single child widget inheriting ContentPane's size.
				this.own(Viewport.on("resize", lang.hitch(this, "resize")));
			}
		},

		_checkIfSingleChild: function(){
			// summary:
			//		Test if we have exactly one visible widget as a child,
			//		and if so assume that we are a container for that widget,
			//		and should propagate startup() and resize() calls to it.
			//		Skips over things like data stores since they aren't visible.

			var candidateWidgets = [],
				otherVisibleNodes = false;

			query("> *", this.containerNode).some(function(node){
				var widget = registry.byNode(node);
				if(widget && widget.resize){
					candidateWidgets.push(widget);
				}else if(!/script|link|style/i.test(node.nodeName) && node.offsetHeight){
					otherVisibleNodes = true;
				}
			});

			this._singleChild = candidateWidgets.length == 1 && !otherVisibleNodes ?
				candidateWidgets[0] : null;

			// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
			domClass.toggle(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		See `dijit/layout/_LayoutWidget.resize()` for description.
			//		Although ContentPane doesn't extend _LayoutWidget, it does implement
			//		the same API.

			this._resizeCalled = true;

			this._scheduleLayout(changeSize, resultSize);
		},

		_scheduleLayout: function(changeSize, resultSize){
			// summary:
			//		Resize myself, and call resize() on each of my child layout widgets, either now
			//		(if I'm currently visible) or when I become visible
			if(this._isShown()){
				this._layout(changeSize, resultSize);
			}else{
				this._needLayout = true;
				this._changeSize = changeSize;
				this._resultSize = resultSize;
			}
		},

		_layout: function(changeSize, resultSize){
			// summary:
			//		Resize myself according to optional changeSize/resultSize parameters, like a layout widget.
			//		Also, since I am an isLayoutContainer widget, each of my children expects me to
			//		call resize() or layout() on it.
			//
			//		Should be called on initialization and also whenever we get new content
			//		(from an href, or from set('content', ...))... but deferred until
			//		the ContentPane is visible

			delete this._needLayout;

			// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
			// never called directly, so resize() is our trigger to do the initial href download (see [20099]).
			// However, don't load href for closed TitlePanes.
			if(!this._wasShown && this.open !== false){
				this._onShow();
			}

			// Set margin box size, unless it wasn't specified, in which case use current size.
			if(changeSize){
				domGeometry.setMarginBox(this.domNode, changeSize);
			}

			// Compute content box size of containerNode in case we [later] need to size our single child.
			var cn = this.containerNode;
			if(cn === this.domNode){
				// If changeSize or resultSize was passed to this method and this.containerNode ==
				// this.domNode then we can compute the content-box size without querying the node,
				// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
				var mb = resultSize || {};
				lang.mixin(mb, changeSize || {}); // changeSize overrides resultSize
				if(!("h" in mb) || !("w" in mb)){
					mb = lang.mixin(domGeometry.getMarginBox(cn), mb); // just use domGeometry.setMarginBox() to fill in missing values
				}
				this._contentBox = layoutUtils.marginBox2contentBox(cn, mb);
			}else{
				this._contentBox = domGeometry.getContentBox(cn);
			}

			this._layoutChildren();
		},

		_layoutChildren: function(){
			// Call _checkIfSingleChild() again in case app has manually mucked w/the content
			// of the ContentPane (rather than changing it through the set("content", ...) API.
			if(this.doLayout){
				this._checkIfSingleChild();
			}

			if(this._singleChild && this._singleChild.resize){
				var cb = this._contentBox || domGeometry.getContentBox(this.containerNode);

				// note: if widget has padding this._contentBox will have l and t set,
				// but don't pass them to resize() or it will doubly-offset the child
				this._singleChild.resize({w: cb.w, h: cb.h});
			}else{
				// All my child widgets are independently sized (rather than matching my size),
				// but I still need to call resize() on each child to make it layout.
				var children = this.getChildren(),
					widget,
					i = 0;
				while(widget = children[i++]){
					if(widget.resize){
						widget.resize();
					}
				}
			}
		},

		_isShown: function(){
			// summary:
			//		Returns true if the content is currently shown.
			// description:
			//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
			//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
			//		tree every call, and at least solves the performance problem on page load by deferring loading
			//		hidden ContentPanes until they are first shown

			if(this._childOfLayoutWidget){
				// If we are TitlePane, etc - we return that only *IF* we've been resized
				if(this._resizeCalled && "open" in this){
					return this.open;
				}
				return this._resizeCalled;
			}else if("open" in this){
				return this.open;		// for TitlePane, etc.
			}else{
				var node = this.domNode, parent = this.domNode.parentNode;
				return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !domClass.contains(node, "dijitHidden") &&
					parent && parent.style && (parent.style.display != 'none');
			}
		},

		_onShow: function(){
			// summary:
			//		Called when the ContentPane is made visible
			// description:
			//		For a plain ContentPane, this is called on initialization, from startup().
			//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
			//		called whenever the pane is made visible.
			//
			//		Does layout/resize of child widget(s)

			// Need to keep track of whether ContentPane has been shown (which is different than
			// whether or not it's currently visible).
			this._wasShown = true;

			if(this._needLayout){
				// If a layout has been scheduled for when we become visible, do it now
				this._layout(this._changeSize, this._resultSize);
			}

			this.inherited(arguments);
		}
	});
});

},
'dijit/layout/utils':function(){
define([
	"dojo/_base/array", // array.filter array.forEach
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/_base/lang" // lang.mixin, lang.setObject
], function(array, domClass, domGeometry, domStyle, lang){

	// module:
	//		dijit/layout/utils

	function capitalize(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	}

	function size(widget, dim){
		// size the child
		var newSize = widget.resize ? widget.resize(dim) : domGeometry.setMarginBox(widget.domNode, dim);

		// record child's size
		if(newSize){
			// if the child returned it's new size then use that
			lang.mixin(widget, newSize);
		}else{
			// otherwise, call getMarginBox(), but favor our own numbers when we have them.
			// the browser lies sometimes
			lang.mixin(widget, domGeometry.getMarginBox(widget.domNode));
			lang.mixin(widget, dim);
		}
	}

	var utils = {
		// summary:
		//		Utility functions for doing layout

		marginBox2contentBox: function(/*DomNode*/ node, /*Object*/ mb){
			// summary:
			//		Given the margin-box size of a node, return its content box size.
			//		Functions like domGeometry.contentBox() but is more reliable since it doesn't have
			//		to wait for the browser to compute sizes.
			var cs = domStyle.getComputedStyle(node);
			var me = domGeometry.getMarginExtents(node, cs);
			var pb = domGeometry.getPadBorderExtents(node, cs);
			return {
				l: domStyle.toPixelValue(node, cs.paddingLeft),
				t: domStyle.toPixelValue(node, cs.paddingTop),
				w: mb.w - (me.w + pb.w),
				h: mb.h - (me.h + pb.h)
			};
		},


		layoutChildren: function(/*DomNode*/ container, /*Object*/ dim, /*Widget[]*/ children,
				/*String?*/ changedRegionId, /*Number?*/ changedRegionSize){
			// summary:
			//		Layout a bunch of child dom nodes within a parent dom node
			// container:
			//		parent node
			// dim:
			//		{l, t, w, h} object specifying dimensions of container into which to place children
			// children:
			//		An array of Widgets or at least objects containing:
			//
			//		- domNode: pointer to DOM node to position
			//		- region or layoutAlign: position to place DOM node
			//		- resize(): (optional) method to set size of node
			//		- id: (optional) Id of widgets, referenced from resize object, below.
			//
			//		The widgets in this array should be ordered according to how they should be laid out
			//		(each element will be processed in order, and take up as much remaining space as needed),
			//		with the center widget last.
			// changedRegionId:
			//		If specified, the slider for the region with the specified id has been dragged, and thus
			//		the region's height or width should be adjusted according to changedRegionSize
			// changedRegionSize:
			//		See changedRegionId.

			// copy dim because we are going to modify it
			dim = lang.mixin({}, dim);

			domClass.add(container, "dijitLayoutContainer");

			// Move "client" elements to the end of the array for layout.  a11y dictates that the author
			// needs to be able to put them in the document in tab-order, but this algorithm requires that
			// client be last.    TODO: remove for 2.0, all dijit client code already sends children as last item.
			children = array.filter(children, function(item){ return item.region != "center" && item.layoutAlign != "client"; })
				.concat(array.filter(children, function(item){ return item.region == "center" || item.layoutAlign == "client"; }));

			// set positions/sizes
			array.forEach(children, function(child){
				var elm = child.domNode,
					pos = (child.region || child.layoutAlign);
				if(!pos){
					throw new Error("No region setting for " + child.id)
				}

				// set elem to upper left corner of unused space; may move it later
				var elmStyle = elm.style;
				elmStyle.left = dim.l+"px";
				elmStyle.top = dim.t+"px";
				elmStyle.position = "absolute";

				domClass.add(elm, "dijitAlign" + capitalize(pos));

				// Size adjustments to make to this child widget
				var sizeSetting = {};

				// Check for optional size adjustment due to splitter drag (height adjustment for top/bottom align
				// panes and width adjustment for left/right align panes.
				if(changedRegionId && changedRegionId == child.id){
					sizeSetting[child.region == "top" || child.region == "bottom" ? "h" : "w"] = changedRegionSize;
				}

				if(pos == "leading"){
					pos = child.isLeftToRight() ? "left" : "right";
				}
				if(pos == "trailing"){
					pos = child.isLeftToRight() ? "right" : "left";
				}

				// set size && adjust record of remaining space.
				// note that setting the width of a <div> may affect its height.
				if(pos == "top" || pos == "bottom"){
					sizeSetting.w = dim.w;
					size(child, sizeSetting);
					dim.h -= child.h;
					if(pos == "top"){
						dim.t += child.h;
					}else{
						elmStyle.top = dim.t + dim.h + "px";
					}
				}else if(pos == "left" || pos == "right"){
					sizeSetting.h = dim.h;
					size(child, sizeSetting);
					dim.w -= child.w;
					if(pos == "left"){
						dim.l += child.w;
					}else{
						elmStyle.left = dim.l + dim.w + "px";
					}
				}else if(pos == "client" || pos == "center"){
					size(child, dim);
				}
			});
		}
	};

	lang.setObject("dijit.layout.utils", utils);	// remove for 2.0

	return utils;
});

},
'dojo/html':function(){
define(["./_base/kernel", "./_base/lang", "./_base/array", "./_base/declare", "./dom", "./dom-construct", "./parser"],
	function(kernel, lang, darray, declare, dom, domConstruct, parser){
	// module:
	//		dojo/html

	var html = {
		// summary:
		//		TODOC
	};
	lang.setObject("dojo.html", html);

	// the parser might be needed..

	// idCounter is incremented with each instantiation to allow assignment of a unique id for tracking, logging purposes
	var idCounter = 0;

	html._secureForInnerHtml = function(/*String*/ cont){
		// summary:
		//		removes !DOCTYPE and title elements from the html string.
		//
		//		khtml is picky about dom faults, you can't attach a style or `<title>` node as child of body
		//		must go into head, so we need to cut out those tags
		// cont:
		//		An html string for insertion into the dom
		//
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
	};

	html._emptyNode = domConstruct.empty;
	/*=====
	 dojo.html._emptyNode = function(node){
		 // summary:
		 //		Removes all child nodes from the given node.   Deprecated, should use dojo/dom-constuct.empty() directly
		 //		instead.
		 // node: DOMNode
		 //		the parent element
	 };
	 =====*/

		html._setNodeContent = function(/*DomNode*/ node, /*String|DomNode|NodeList*/ cont){
		// summary:
		//		inserts the given content into the given node
		// node:
		//		the parent element
		// content:
		//		the content to be set on the parent element.
		//		This can be an html string, a node reference or a NodeList, dojo/NodeList, Array or other enumerable list of nodes

		// always empty
		domConstruct.empty(node);

		if(cont){
			if(typeof cont == "string"){
				cont = domConstruct.toDom(cont, node.ownerDocument);
			}
			if(!cont.nodeType && lang.isArrayLike(cont)){
				// handle as enumerable, but it may shrink as we enumerate it
				for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0){
					domConstruct.place( cont[i], node, "last");
				}
			}else{
				// pass nodes, documentFragments and unknowns through to dojo.place
				domConstruct.place(cont, node, "last");
			}
		}

		// return DomNode
		return node;
	};

	// we wrap up the content-setting operation in a object
	html._ContentSetter = declare("dojo.html._ContentSetter", null,
		{
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",

			// id: String?
			//		Usually only used internally, and auto-generated with each instance
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,

			// extractContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of `<html> <body>` wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,

			// parserScope: String
			//		Flag passed to parser.	Root for attribute names to search for.	  If scopeName is dojo,
			//		will search for data-dojo-type (or dojoType).  For backwards compatibility
			//		reasons defaults to dojo._scopeName (which is "dojo" except when
			//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			parserScope: kernel._scopeName,

			// startup: Boolean
			//		Start the child widgets after parsing them.	  Only obeyed if parseContent is true.
			startup: true,

			// lifecycle methods
			constructor: function(/*Object*/ params, /*String|DomNode*/ node){
				// summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..

				// the original params are mixed directly into the instance "this"
				lang.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dom.byId( this.node || node );

				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "",
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /*Object?*/ params){
				// summary:
				//		front-end to the set-content sequence
				// cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();

				var ret = this.onEnd();

				if(ret && ret.then){
					// Make dojox/html/_ContentSetter.set() return a Promise that resolves when load and parse complete.
					return ret;
				}else{
					// Vanilla dojo/html._ContentSetter.set() returns a DOMNode for back compat.   For 2.0, switch it to
					// return a Deferred like above.
					return this.node;
				}
			},

			setContent: function(){
				// summary:
				//		sets the content on the node

				var node = this.node;
				if(!node){
					// can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV

					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e);
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},

			empty: function(){
				// summary:
				//		cleanly empty out existing content
				
				// If there is a parse in progress, cancel it.
				if(this.parseDeferred){
					if(!this.parseDeferred.isResolved()){
						this.parseDeferred.cancel();
					}
					delete this.parseDeferred;
				}

				// destroy any widgets from a previous run
				// NOTE: if you don't want this you'll need to empty
				// the parseResults array property yourself to avoid bad things happening
				if(this.parseResults && this.parseResults.length){
					darray.forEach(this.parseResults, function(w){
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could
				// override empty to skip this step
				domConstruct.empty(this.node);
			},

			onBegin: function(){
				// summary:
				//		Called after instantiation, but before set();
				//		It allows modification of any of the object properties -
				//		including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to
				//		optionally pre-process html string content
				var cont = this.content;

				if(lang.isString(cont)){
					if(this.cleanContent){
						cont = html._secureForInnerHtml(cont);
					}

					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();

				this.content = cont;
				return this.node; // DomNode
			},

			onEnd: function(){
				// summary:
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults and this.parseDeferred if you need those..
					this._parse();
				}
				return this.node; // DomNode
				// TODO: for 2.0 return a Promise indicating that the parse completed.
			},

			tearDown: function(){
				// summary:
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description:
				//		tearDown() is not called automatically.
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults;
				delete this.parseDeferred;
				delete this.node;
				delete this.content;
			},

			onContentError: function(err){
				return "Error occurred setting content: " + err;
			},

			onExecError: function(err){
				return "Error occurred executing scripts: " + err;
			},

			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we don't consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key];
				}
			},
			_parse: function(){
				// summary:
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		and the parse promise in this.parseDeferred
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					var inherited = {};
					darray.forEach(["dir", "lang", "textDir"], function(name){
						if(this[name]){
							inherited[name] = this[name];
						}
					}, this);
					var self = this;
					this.parseDeferred = parser.parse({
						rootNode: rootNode,
						noStart: !this.startup,
						inherited: inherited,
						scope: this.parserScope
					}).then(function(results){
						return self.parseResults = results;
					}, function(e){
						self._onError('Content', e, "Error parsing in _ContentSetter#" + this.id);
					});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#" + this.id);
				}
			},

			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		override/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					html._setNodeContent(this.node, errText, true);
				}
			}
	}); // end declare()

	html.set = function(/*DomNode*/ node, /*String|DomNode|NodeList*/ cont, /*Object?*/ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo.place(cont, node, "only"). dojo.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			// node:
			//		the parent element that will receive the content
			// cont:
			//		the content to be set on the parent element.
			//		This can be an html string, a node reference or a NodeList, dojo/NodeList, Array or other enumerable list of nodes
			// params:
			//		Optional flags/properties to configure the content-setting. See dojo/html/_ContentSetter
			// example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage:
			//	|	html.set(node, "some string");
			//	|	html.set(node, contentNode, {options});
			//	|	html.set(node, myNode.childNodes, {options});
		if(undefined == cont){
			console.warn("dojo.html.set: no cont argument provided, using empty string");
			cont = "";
		}
		if(!params){
			// simple and fast
			return html._setNodeContent(node, cont, true);
		}else{
			// more options but slower
			// note the arguments are reversed in order, to match the convention for instantiation via the parser
			var op = new html._ContentSetter(lang.mixin(
					params,
					{ content: cont, node: node }
			));
			return op.set();
		}
	};

	return html;
});

},
'dijit/_DialogMixin':function(){
define([
	"dojo/_base/declare", // declare
	"./a11y"	// _getTabNavigable
], function(declare, a11y){

	// module:
	//		dijit/_DialogMixin

	return declare("dijit._DialogMixin", null, {
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		execute: function(/*Object*/ /*===== formContents =====*/){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//		Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//		Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit/form/DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//		Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//		Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit/form/DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(){
			// summary:
			//		Finds focusable items in dialog,
			//		and sets this._firstFocusItem and this._lastFocusItem
			// tags:
			//		protected

			var elems = a11y._getTabNavigable(this.containerNode);
			this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
		}
	});
});

},
'dijit/form/_FormMixin':function(){
define([
	"dojo/_base/array", // array.every array.filter array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/_base/lang", // lang.hitch lang.isArray
	"dojo/on",
	"dojo/window" // winUtils.scrollIntoView
], function(array, declare, kernel, lang, on, winUtils){

	// module:
	//		dijit/form/_FormMixin

	return declare("dijit.form._FormMixin", null, {
		// summary:
		//		Mixin for containers of form widgets (i.e. widgets that represent a single value
		//		and can be children of a `<form>` node or `dijit/form/Form` widget)
		// description:
		//		Can extract all the form widgets
		//		values and combine them into a single javascript object, or alternately
		//		take such an object and set the values for all the contained
		//		form widgets

	/*=====
		// value: Object
		//		Name/value hash for each child widget with a name and value.
		//		Child widgets without names are not part of the hash.
		//
		//		If there are multiple child widgets w/the same name, value is an array,
		//		unless they are radio buttons in which case value is a scalar (since only
		//		one radio button can be checked at a time).
		//
		//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
		//
		//		Example:
		//	|	{ name: "John Smith", interests: ["sports", "movies"] }
	=====*/

		// state: [readonly] String
		//		Will be "Error" if one or more of the child widgets has an invalid value,
		//		"Incomplete" if not all of the required child widgets are filled in.  Otherwise, "",
		//		which indicates that the form is ready to be submitted.
		state: "",

		// TODO:
		//	* Repeater
		//	* better handling for arrays.  Often form elements have names with [] like
		//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])


		_getDescendantFormWidgets: function(/*dijit/_WidgetBase[]?*/ children){
			// summary:
			//		Returns all form widget descendants, searching through non-form child widgets like BorderContainer
			var res = [];
			array.forEach(children || this.getChildren(), function(child){
				if("value" in child){
					res.push(child);
				}else{
					res = res.concat(this._getDescendantFormWidgets(child.getChildren()));
				}
			}, this);
			return res;
		},

		reset: function(){
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//		provides a few additional (ui-specific) features:
			//
			//		1. it will highlight any sub-widgets that are not valid
			//		2. it will call focus() on the first invalid sub-widget
			var didFocus = false;
			return array.every(array.map(this._getDescendantFormWidgets(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					winUtils.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
				return valid;
			}), function(item){ return item; });
		},

		setValues: function(val){
			kernel.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*Object*/ obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by get('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = lang.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				values = [].concat(values);
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					array.forEach(widgets, function(w){
						w.set('value', array.indexOf(values, w._get('value')) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					array.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 *	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			array.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call set('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							array.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						array.forEach(element.options, function(option){
							option.selected = array.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						array.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
			});
			*/

			// Note: no need to call this._set("value", ...) as the child updates will trigger onChange events
			// which I am monitoring.
		},

		getValues: function(){
			kernel.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			//		Returns Object representing form values.   See description of `value` for details.
			// description:

			// The value is updated into this.value every time a child has an onChange event,
			// so in the common case this function could just return this.value.   However,
			// that wouldn't work when:
			//
			// 1. User presses return key to submit a form.  That doesn't fire an onchange event,
			// and even if it did it would come too late due to the defer(...) in _handleOnChange()
			//
			// 2. app for some reason calls this.get("value") while the user is typing into a
			// form field.   Not sure if that case needs to be supported or not.

			// get widget values
			var obj = { };
			array.forEach(this._getDescendantFormWidgets(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget)
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							lang.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = lang.getObject(name, false, obj);
							if(value === undefined){
								lang.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=lang.getObject(name, false, obj);
						if(!ary){
							ary=[];
							lang.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=lang.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(lang.isArray(prev)){
							prev.push(value);
						}else{
							lang.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						lang.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also domForm.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			array.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					}else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					}else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					}else{
						// can not set value when there is no name
					}
				}else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				}else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

		isValid: function(){
			// summary:
			//		Returns true if all of the widgets are valid.
			//		Deprecated, will be removed in 2.0.  Use get("state") instead.

			return this.state == "";
		},

		onValidStateChange: function(/*Boolean*/ /*===== isValid =====*/){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
			//
			//		Deprecated.  Will be removed in 2.0.  Use watch("state", ...) instead.
		},

		_getState: function(){
			// summary:
			//		Compute what this.state should be based on state of children
			var states = array.map(this._descendants, function(w){
				return w.get("state") || "";
			});

			return array.indexOf(states, "Error") >= 0 ? "Error" :
				array.indexOf(states, "Incomplete") >= 0 ? "Incomplete" : "";
		},

		disconnectChildren: function(){
			// summary:
			//		Deprecated method.   Applications no longer need to call this.   Remove for 2.0.
		},

		connectChildren: function(/*Boolean*/ inStartup){
			// summary:
			//		You can call this function directly, ex. in the event that you
			//		programmatically add a widget to the form *after* the form has been
			//		initialized.

			// TODO: rename for 2.0

			this._descendants = this._getDescendantFormWidgets();

			// To get notifications from children they need to be started.   Children didn't used to need to be started,
			// so for back-compat, start them here
			array.forEach(this._descendants, function(child){
				if(!child._started){ child.startup(); }
			});

			if(!inStartup){
				this._onChildChange();
			}
		},

		_onChildChange: function(/*String*/ attr){
			// summary:
			//		Called when child's value or disabled state changes

			// The unit tests expect state update to be synchronous, so update it immediately.
			if(!attr || attr == "state" || attr == "disabled"){
				this._set("state", this._getState());
			}

			// Use defer() to collapse value changes in multiple children into a single
			// update to my value.   Multiple updates will occur on:
			//	1. Form.set()
			//	2. Form.reset()
			//	3. user selecting a radio button (which will de-select another radio button,
			//		 causing two onChange events)
			if(!attr || attr == "value" || attr == "disabled" || attr == "checked"){
				if(this._onChangeDelayTimer){
					this._onChangeDelayTimer.remove();
				}
				this._onChangeDelayTimer = this.defer(function(){
					delete this._onChangeDelayTimer;
					this._set("value", this.get("value"));
				}, 10);
			}
		},

		startup: function(){
			this.inherited(arguments);

			// Set initial this.value and this.state.   Don't emit watch() notifications.
			this._descendants = this._getDescendantFormWidgets();
			this.value = this.get("value");
			this.state = this._getState();

			// Initialize value and valid/invalid state tracking.
			var self = this;
			this.own(
				on(
					this.containerNode,
					"attrmodified-state, attrmodified-disabled, attrmodified-value, attrmodified-checked",
					function(evt){
						if(evt.target == self.domNode){
							return;	// ignore events that I fire on myself because my children changed
						}
						self._onChildChange(evt.type.replace("attrmodified-", ""));
					}
				)
			);

			// Make state change call onValidStateChange(), will be removed in 2.0
			this.watch("state", function(attr, oldVal, newVal){ this.onValidStateChange(newVal == ""); });
		},

		destroy: function(){
			this.inherited(arguments);
		}

	});
});

},
'dijit/MenuSeparator':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"./_WidgetBase",
	"./_TemplatedMixin",
	"./_Contained",
	"dojo/text!./templates/MenuSeparator.html"
], function(declare, dom, _WidgetBase, _TemplatedMixin, _Contained, template){

	// module:
	//		dijit/MenuSeparator

	return declare("dijit.MenuSeparator", [_WidgetBase, _TemplatedMixin, _Contained], {
		// summary:
		//		A line between two menu items

		templateString: template,

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		Override to always return false
			// tags:
			//		protected

			return false; // Boolean
		}
	});
});

},
'dojox/layout/ResizeHandle':function(){
define(["dojo/_base/kernel","dojo/_base/lang","dojo/_base/connect","dojo/_base/array","dojo/_base/event",
	"dojo/_base/fx","dojo/_base/window","dojo/fx","dojo/window","dojo/dom","dojo/dom-class",
	"dojo/dom-geometry","dojo/dom-style","dijit/_base/manager","dijit/_Widget","dijit/_TemplatedMixin",
	"dojo/_base/declare"], function (
	kernel, lang, connect, arrayUtil, eventUtil, fxBase, windowBase, fxUtil, windowUtil, 
	domUtil, domClass, domGeometry, domStyle, manager, Widget, TemplatedMixin, declare) {

kernel.experimental("dojox.layout.ResizeHandle");

var ResizeHandle = declare("dojox.layout.ResizeHandle",[Widget, TemplatedMixin],
	{
	// summary:
	//		A draggable handle used to resize an attached node.
	//
	// description:
	//		The handle on the bottom-right corner of FloatingPane or other widgets that allows
	//		the widget to be resized.
	//		Typically not used directly.

	// targetId: String
	//		id of the Widget OR DomNode that I will size
	targetId: "",
	
	// targetContainer: DomNode
	//		over-ride targetId and attch this handle directly to a reference of a DomNode
	targetContainer: null,
	
	// resizeAxis: String
	//		one of: x|y|xy limit resizing to a single axis, default to xy ...
	resizeAxis: "xy",
	
	// activeResize: Boolean
	//		if true, node will size realtime with mouse movement,
	//		if false, node will create virtual node, and only resize target on mouseUp
	activeResize: false,
	
	// activeResizeClass: String
	//		css class applied to virtual resize node.
	activeResizeClass: "dojoxResizeHandleClone",
	
	// animateSizing: Boolean
	//		only applicable if activeResize = false. onMouseup, animate the node to the
	//		new size
	animateSizing: true,
	
	// animateMethod: String
	//		one of "chain" or "combine" ... visual effect only. combine will "scale"
	//		node to size, "chain" will alter width, then height
	animateMethod: "chain",

	// animateDuration: Integer
	//		time in MS to run sizing animation. if animateMethod="chain", total animation
	//		playtime is 2*animateDuration
	animateDuration: 225,

	// minHeight: Integer
	//		smallest height in px resized node can be
	minHeight: 100,

	// minWidth: Integer
	//		smallest width in px resize node can be
	minWidth: 100,

	// constrainMax: Boolean
	//		Toggle if this widget cares about the maxHeight and maxWidth
	//		parameters.
	constrainMax: false,

	// maxHeight: Integer
	//		Largest height size in px the resize node can become.
	maxHeight:0,
	
	// maxWidth: Integer
	//		Largest width size in px the resize node can become.
	maxWidth:0,

	// fixedAspect: Boolean
	//		Toggle to enable this widget to maintain the aspect
	//		ratio of the attached node.
	fixedAspect: false,

	// intermediateChanges: Boolean
	//		Toggle to enable/disable this widget from firing onResize
	//		events at every step of a resize. If `activeResize` is true,
	//		and this is false, onResize only fires _after_ the drop
	//		operation. Animated resizing is not affected by this setting.
	intermediateChanges: false,

	// startTopic: String
	//		The name of the topic this resizehandle publishes when resize is starting
	startTopic: "/dojo/resize/start",
	
	// endTopic: String
	//		The name of the topic this resizehandle publishes when resize is complete
	endTopic:"/dojo/resize/stop",

	templateString: '<div dojoAttachPoint="resizeHandle" class="dojoxResizeHandle"><div></div></div>',

	postCreate: function(){
		// summary:
		//		setup our one major listener upon creation
		this.connect(this.resizeHandle, "onmousedown", "_beginSizing");
		if(!this.activeResize){
			// there shall be only a single resize rubberbox that at the top
			// level so that we can overlay it on anything whenever the user
			// resizes something. Since there is only one mouse pointer he
			// can't at once resize multiple things interactively.
			this._resizeHelper = manager.byId('dojoxGlobalResizeHelper');
			if(!this._resizeHelper){
				this._resizeHelper = new _ResizeHelper({
						id: 'dojoxGlobalResizeHelper'
				}).placeAt(windowBase.body());
				domClass.add(this._resizeHelper.domNode, this.activeResizeClass);
			}
		}else{ this.animateSizing = false; }

		if(!this.minSize){
			this.minSize = { w: this.minWidth, h: this.minHeight };
		}
		
		if(this.constrainMax){
			this.maxSize = { w: this.maxWidth, h: this.maxHeight }
		}
		
		// should we modify the css for the cursor hover to n-resize nw-resize and w-resize?
		this._resizeX = this._resizeY = false;
		var addClass = lang.partial(domClass.add, this.resizeHandle);
		switch(this.resizeAxis.toLowerCase()){
			case "xy" :
				this._resizeX = this._resizeY = true;
				// FIXME: need logic to determine NW or NE class to see
				// based on which [todo] corner is clicked
				addClass("dojoxResizeNW");
				break;
			case "x" :
				this._resizeX = true;
				addClass("dojoxResizeW");
				break;
			case "y" :
				this._resizeY = true;
				addClass("dojoxResizeN");
				break;
		}
	},

	_beginSizing: function(/*Event*/ e){
		// summary:
		//		setup movement listeners and calculate initial size
		
		if(this._isSizing){ return; }

		connect.publish(this.startTopic, [ this ]);
		this.targetWidget = manager.byId(this.targetId);

		this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : domUtil.byId(this.targetId);
		if(this.targetContainer){ this.targetDomNode = this.targetContainer; }
		if(!this.targetDomNode){ return; }

		if(!this.activeResize){
			var c = domGeometry.position(this.targetDomNode, true);
			this._resizeHelper.resize({l: c.x, t: c.y, w: c.w, h: c.h});
			this._resizeHelper.show();
			if(!this.isLeftToRight()){
				this._resizeHelper.startPosition = {l: c.x, t: c.y};
			}
		}

		this._isSizing = true;
		this.startPoint  = { x:e.clientX, y:e.clientY };

		// widget.resize() or setting style.width/height expects native box model dimension 
		// (in most cases content-box, but it may be border-box if in backcompact mode)
		var style = domStyle.getComputedStyle(this.targetDomNode), 
			borderModel = domGeometry.boxModel==='border-model',
			padborder = borderModel?{w:0,h:0}:domGeometry.getPadBorderExtents(this.targetDomNode, style),
			margin = domGeometry.getMarginExtents(this.targetDomNode, style),
			mb;
		mb = this.startSize = { 
				w: domStyle.get(this.targetDomNode, 'width', style), 
				h: domStyle.get(this.targetDomNode, 'height', style),
				//ResizeHelper.resize expects a bounding box of the
				//border box, so let's keep track of padding/border
				//width/height as well
				pbw: padborder.w, pbh: padborder.h,
				mw: margin.w, mh: margin.h};
		if(!this.isLeftToRight() && dojo.style(this.targetDomNode, "position") == "absolute"){
			var p = domGeometry.position(this.targetDomNode, true);
			this.startPosition = {l: p.x, t: p.y};
		}
		
		this._pconnects = [
			connect.connect(windowBase.doc,"onmousemove",this,"_updateSizing"),
			connect.connect(windowBase.doc,"onmouseup", this, "_endSizing")
		];
		
		eventUtil.stop(e);
	},

	_updateSizing: function(/*Event*/ e){
		// summary:
		//		called when moving the ResizeHandle ... determines
		//		new size based on settings/position and sets styles.

		if(this.activeResize){
			this._changeSizing(e);
		}else{
			var tmp = this._getNewCoords(e, 'border', this._resizeHelper.startPosition);
			if(tmp === false){ return; }
			this._resizeHelper.resize(tmp);
		}
		e.preventDefault();
	},

	_getNewCoords: function(/* Event */ e, /* String */ box, /* Object */startPosition){
		
		// On IE, if you move the mouse above/to the left of the object being resized,
		// sometimes clientX/Y aren't set, apparently.  Just ignore the event.
		try{
			if(!e.clientX  || !e.clientY){ return false; }
		}catch(e){
			// sometimes you get an exception accessing above fields...
			return false;
		}
		this._activeResizeLastEvent = e;

		var dx = (this.isLeftToRight()?1:-1) * (this.startPoint.x - e.clientX),
			dy = this.startPoint.y - e.clientY,
			newW = this.startSize.w - (this._resizeX ? dx : 0),
			newH = this.startSize.h - (this._resizeY ? dy : 0),
			r = this._checkConstraints(newW, newH)
		;
		
		startPosition = (startPosition || this.startPosition);
		if(startPosition && this._resizeX){
			// adjust x position for RtoL
			r.l = startPosition.l + dx;
			if(r.w != newW){
				r.l += (newW - r.w);
			}
			r.t = startPosition.t;
		}

		switch(box){
			case 'margin':
				r.w += this.startSize.mw;
				r.h += this.startSize.mh;
				//pass through
			case "border":
				r.w += this.startSize.pbw;
				r.h += this.startSize.pbh;
				break;
			//default: //native, do nothing
		}

		return r; // Object
	},
	
	_checkConstraints: function(newW, newH){
		// summary:
		//		filter through the various possible constaint possibilities.
				
		// minimum size check
		if(this.minSize){
			var tm = this.minSize;
			if(newW < tm.w){
				newW = tm.w;
			}
			if(newH < tm.h){
				newH = tm.h;
			}
		}
		
		// maximum size check:
		if(this.constrainMax && this.maxSize){
			var ms = this.maxSize;
			if(newW > ms.w){
				newW = ms.w;
			}
			if(newH > ms.h){
				newH = ms.h;
			}
		}
		
		if(this.fixedAspect){
			var w = this.startSize.w, h = this.startSize.h,
				delta = w * newH - h * newW;
			if(delta<0){
				newW = newH * w / h;
			}else if(delta>0){
				newH = newW * h / w;
			}
		}
		
		return { w: newW, h: newH }; // Object
	},
		
	_changeSizing: function(/*Event*/ e){
		// summary:
		//		apply sizing information based on information in (e) to attached node
		
		var isWidget = this.targetWidget && lang.isFunction(this.targetWidget.resize),
			tmp = this._getNewCoords(e, isWidget && 'margin');
		if(tmp === false){ return; }

		if(isWidget){
			this.targetWidget.resize(tmp);
		}else{
			if(this.animateSizing){
				var anim = fxUtil[this.animateMethod]([
					fxBase.animateProperty({
						node: this.targetDomNode,
						properties: {
							width: { start: this.startSize.w, end: tmp.w }
						},
						duration: this.animateDuration
					}),
					fxBase.animateProperty({
						node: this.targetDomNode,
						properties: {
							height: { start: this.startSize.h, end: tmp.h }
						},
						duration: this.animateDuration
					})
				]);
				anim.play();
			}else{
				domStyle.set(this.targetDomNode,{
					width: tmp.w + "px",
					height: tmp.h + "px"
				});
			}
		}
		if(this.intermediateChanges){
			this.onResize(e);
		}
	},

	_endSizing: function(/*Event*/ e){
		// summary:
		//		disconnect listenrs and cleanup sizing
		arrayUtil.forEach(this._pconnects, connect.disconnect);
		var pub = lang.partial(connect.publish, this.endTopic, [ this ]);
		if(!this.activeResize){
			this._resizeHelper.hide();
			this._changeSizing(e);
			setTimeout(pub, this.animateDuration + 15);
		}else{
			pub();
		}
		this._isSizing = false;
		this.onResize(e);
	},
	
	onResize: function(e){
		// summary:
		//		Stub fired when sizing is done. Fired once
		//		after resize, or often when `intermediateChanges` is
		//		set to true.
	}
	
});

var _ResizeHelper = dojo.declare("dojox.layout._ResizeHelper", Widget, {
	// summary:
	//		A global private resize helper shared between any
	//		`dojox.layout.ResizeHandle` with activeSizing off.
	
	show: function(){
		// summary:
		//		show helper to start resizing
		domStyle.set(this.domNode, "display", "");
	},
	
	hide: function(){
		// summary:
		//		hide helper after resizing is complete
		domStyle.set(this.domNode, "display", "none");
	},
	
	resize: function(/* Object */dim){
		// summary:
		//		size the widget and place accordingly
		domGeometry.setMarginBox(this.domNode, dim);
	}
	
});
return ResizeHandle;
});

},
'dgrid/editor':function(){
define([
	"dojo/_base/kernel",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/Deferred",
	"dojo/on",
	"dojo/aspect",
	"dojo/has",
	"dojo/query",
	"./Grid",
	"put-selector/put",
	"dojo/_base/sniff"
], function(kernel, lang, arrayUtil, Deferred, on, aspect, has, query, Grid, put){

// Variables to track info for cell currently being edited (editOn only).
var activeCell, activeValue, activeOptions;

function updateInputValue(input, value){
	// common code for updating value of a standard input
	input.value = value;
	if(input.type == "radio" || input.type == "checkbox"){
		input.checked = input.defaultChecked = !!value;
	}
}

function dataFromValue(value, oldValue){
	// Default logic for translating values from editors;
	// tries to preserve type if possible.
	if(typeof oldValue == "number"){
		value = isNaN(value) ? value : parseFloat(value);
	}else if(typeof oldValue == "boolean"){
		value = value == "true" ? true : value == "false" ? false : value;
	}else if(oldValue instanceof Date){
		var asDate = new Date(value);
		value = isNaN(asDate.getTime()) ? value : asDate;
	}
	return value;
}

// intermediary frontend to dataFromValue for HTML and widget editors
function dataFromEditor(column, cmp){
	if(typeof cmp.get == "function"){ // widget
		return dataFromValue(cmp.get("value"));
	}else{ // HTML input
		return dataFromValue(
			cmp[cmp.type == "checkbox" || cmp.type == "radio"  ? "checked" : "value"]);
	}
}

function setProperty(grid, cellElement, oldValue, value, triggerEvent){
	// Updates dirty hash and fires dgrid-datachange event for a changed value.
	var cell, row, column, eventObject;
	// test whether old and new values are inequal, with coercion (e.g. for Dates)
	if((oldValue && oldValue.valueOf()) != (value && value.valueOf())){
		cell = grid.cell(cellElement);
		row = cell.row;
		column = cell.column;
		if(column.field && row){
			// TODO: remove rowId in lieu of cell (or grid.row/grid.cell)
			// (keeping for the moment for back-compat, but will note in changes)
			eventObject = {
				grid: grid,
				cell: cell,
				rowId: row.id,
				oldValue: oldValue,
				value: value,
				bubbles: true,
				cancelable: true
			};
			if(triggerEvent && triggerEvent.type){
				eventObject.parentType = triggerEvent.type;
			}
			
			if(on.emit(cellElement, "dgrid-datachange", eventObject)){
				if(grid.updateDirty){
					// for OnDemandGrid: update dirty data, and save if autoSave is true
					grid.updateDirty(row.id, column.field, value);
					// perform auto-save (if applicable) in next tick to avoid
					// unintentional mishaps due to order of handler execution
					column.autoSave && setTimeout(function(){ grid._trackError("save"); }, 0);
				}else{
					// update store-less grid
					row.data[column.field] = value;
				}
			}else{
				// Otherwise keep the value the same
				// For the sake of always-on editors, need to manually reset the value
				var cmp;
				if(cmp = cellElement.widget){
					// set _dgridIgnoreChange to prevent an infinite loop in the
					// onChange handler and prevent dgrid-datachange from firing
					// a second time
					cmp._dgridIgnoreChange = true;
					cmp.set("value", oldValue);
					setTimeout(function(){ cmp._dgridIgnoreChange = false; }, 0);
				}else if(cmp = cellElement.input){
					updateInputValue(cmp, oldValue);
				}
				
				return oldValue;
			}
		}
	}
	return value;
}

// intermediary frontend to setProperty for HTML and widget editors
function setPropertyFromEditor(grid, column, cmp, triggerEvent) {
	var value, id, editedRow;
	if(!cmp.isValid || cmp.isValid()){
		value = setProperty(grid, (cmp.domNode || cmp).parentNode,
			activeCell ? activeValue : cmp._dgridLastValue,
			dataFromEditor(column, cmp), triggerEvent);
		
		if(activeCell){ // for editors with editOn defined
			activeValue = value;
		}else{ // for always-on editors, update _dgridLastValue immediately
			cmp._dgridLastValue = value;
		}

		if(cmp.type === "radio" && cmp.name && !column.editOn && column.field){
			editedRow = grid.row(cmp);
			
			// Update all other rendered radio buttons in the group
			query("input[type=radio][name=" + cmp.name + "]", grid.contentNode).forEach(function(radioBtn){
				var row = grid.row(radioBtn);
				// Only update _dgridLastValue and the dirty data if it exists
				// and is not already false
				if(radioBtn !== cmp && radioBtn._dgridLastValue){
					radioBtn._dgridLastValue = false;
					if(grid.updateDirty){
						grid.updateDirty(row.id, column.field, false);
					}else{
						// update store-less grid
						row.data[column.field] = false;
					}
				}
			});
			
			// Also update dirty data for rows that are not currently rendered
			for(id in grid.dirty){
				if(editedRow.id !== id && grid.dirty[id][column.field]){
					grid.updateDirty(id, column.field, false);
				}
			}
		}
	}
}

// editor creation/hookup/placement logic

function createEditor(column){
	// Creates an editor instance based on column definition properties,
	// and hooks up events.
	var editor = column.editor,
		editOn = column.editOn,
		grid = column.grid,
		isWidget = typeof editor != "string", // string == standard HTML input
		args, cmp, node, putstr, handleChange;
	
	args = column.editorArgs || {};
	if(typeof args == "function"){ args = args.call(grid, column); }
	
	if(isWidget){
		cmp = new editor(args);
		node = cmp.focusNode || cmp.domNode;
		
		// Add dgrid-input to className to make consistent with HTML inputs.
		node.className += " dgrid-input";
		
		// For editOn editors, connect to onBlur rather than onChange, since
		// the latter is delayed by setTimeouts in Dijit and will fire too late.
		cmp.connect(cmp, editOn ? "onBlur" : "onChange", function(){
			if(!cmp._dgridIgnoreChange){
				setPropertyFromEditor(grid, column, this, {type: "widget"});
			}
		});
	}else{
		handleChange = function(evt){
			var target = evt.target;
			if("_dgridLastValue" in target && target.className.indexOf("dgrid-input") > -1){
				setPropertyFromEditor(grid, column, target, evt);
			}
		};

		// considerations for standard HTML form elements
		if(!column.grid._hasInputListener){
			// register one listener at the top level that receives events delegated
			grid._hasInputListener = true;
			grid.on("change", function(evt){ handleChange(evt); });
			// also register a focus listener
		}
		
		putstr = editor == "textarea" ? "textarea" :
			"input[type=" + editor + "]";
		cmp = node = put(putstr + ".dgrid-input", lang.mixin({
			name: column.field,
			tabIndex: isNaN(column.tabIndex) ? -1 : column.tabIndex
		}, args));
		
		if(has("ie") < 9 || (has("ie") && has("quirks"))){
			// IE<9 / quirks doesn't fire change events for all the right things,
			// and it doesn't bubble.
			if(editor == "radio" || editor == "checkbox"){
				// listen for clicks since IE doesn't fire change events properly for checks/radios
				on(cmp, "click", function(evt){ handleChange(evt); });
			}else{
				on(cmp, "change", function(evt){ handleChange(evt); });
			}
		}
	}
	
	// XXX: stop mousedown propagation to prevent confusing Keyboard mixin logic
	// with certain widgets; perhaps revising KB's `handledEvent` would be better.
	on(node, "mousedown", function(evt){ evt.stopPropagation(); });
	
	return cmp;
}

function createSharedEditor(column, originalRenderCell){
	// Creates an editor instance with additional considerations for
	// shared usage across an entire column (for columns with editOn specified).
	
	var cmp = createEditor(column),
		grid = column.grid,
		isWidget = cmp.domNode,
		node = cmp.domNode || cmp,
		focusNode = cmp.focusNode || node,
		reset = isWidget ?
			function(){ cmp.set("value", cmp._dgridLastValue); } :
			function(){
				updateInputValue(cmp, cmp._dgridLastValue);
				// call setProperty again in case we need to revert a previous change
				setPropertyFromEditor(column.grid, column, cmp);
			},
		keyHandle;
	
	function blur(){
		var element = activeCell;
		focusNode.blur();
		
		if(typeof grid.focus === "function"){
			// Dijit form widgets don't end up dismissed until the next turn,
			// so wait before calling focus (otherwise Keyboard will focus the
			// input again).  IE<9 needs to wait longer, otherwise the cell loses
			// focus after we've set it.
			setTimeout(function(){
				grid.focus(element);
			}, isWidget && has("ie") < 9 ? 15 : 0);
		}
	}
	
	function onblur(){
		var parentNode = node.parentNode,
			i = parentNode.children.length - 1,
			options = { alreadyHooked: true },
			cell = grid.cell(node);
		
		// emit an event immediately prior to removing an editOn editor
		on.emit(cell.element, "dgrid-editor-hide", {
			grid: grid,
			cell: cell,
			column: column,
			editor: cmp,
			bubbles: true,
			cancelable: false
		});
		column._editorBlurHandle.pause();
		// Remove the editor from the cell, to be reused later.
		parentNode.removeChild(node);
		
		put(cell.element, "!dgrid-cell-editing");
		
		// Clear out the rest of the cell's contents, then re-render with new value.
		while(i--){ put(parentNode.firstChild, "!"); }
		Grid.appendIfNode(parentNode, column.renderCell(
			column.grid.row(parentNode).data, activeValue, parentNode,
			activeOptions ? lang.delegate(options, activeOptions) : options));
		
		// reset state now that editor is deactivated
		activeCell = activeValue = activeOptions = null;
	}
	
	function dismissOnKey(evt){
		// Contains logic for reacting to enter/escape keypresses to save/cancel edits.
		// Calls `focusNode.blur()` in cases where field should be dismissed.
		var key = evt.keyCode || evt.which;
		
		if(key == 27){ // escape: revert + dismiss
			reset();
			activeValue = cmp._dgridLastValue;
			blur();
		}else if(key == 13 && column.dismissOnEnter !== false){ // enter: dismiss
			// FIXME: Opera is "reverting" even in this case
			blur();
		}
	}
	
	// hook up enter/esc key handling
	keyHandle = on(focusNode, "keydown", dismissOnKey);
	
	// hook up blur handler, but don't activate until widget is activated
	(column._editorBlurHandle = on.pausable(cmp, "blur", onblur)).pause();
	
	return cmp;
}

function showEditor(cmp, column, cellElement, value){
	// Places a shared editor into the newly-active cell in the column.
	// Also called when rendering an editor in an "always-on" editor column.
	
	var isWidget = cmp.domNode,
		grid = column.grid;
	
	// for regular inputs, we can update the value before even showing it
	if(!isWidget){ updateInputValue(cmp, value); }
	
	cellElement.innerHTML = "";
	put(cellElement, ".dgrid-cell-editing");
	put(cellElement, cmp.domNode || cmp);
	
	if(isWidget){
		// For widgets, ensure startup is called before setting value,
		// to maximize compatibility with flaky widgets like dijit/form/Select.
		if(!cmp._started){ cmp.startup(); }
		
		// Set value, but ensure it isn't processed as a user-generated change.
		// (Clear flag on a timeout to wait for delayed onChange to fire first)
		cmp._dgridIgnoreChange = true;
		cmp.set("value", value);
		setTimeout(function(){ cmp._dgridIgnoreChange = false; }, 0);
	}
	// track previous value for short-circuiting or in case we need to revert
	cmp._dgridLastValue = value;
	// if this is an editor with editOn, also update activeValue
	// (activeOptions will have been updated previously)
	if(activeCell){ 
		activeValue = value; 
		// emit an event immediately prior to placing a shared editor
		on.emit(cellElement, "dgrid-editor-show", {
			grid: grid,
			cell: grid.cell(cellElement),
			column: column,
			editor: cmp,
			bubbles: true,
			cancelable: false
		});
	}
}

function edit(cell) {
	// summary:
	//		Method to be mixed into grid instances, which will show/focus the
	//		editor for a given grid cell.  Also used by renderCell.
	// cell: Object
	//		Cell (or something resolvable by grid.cell) to activate editor on.
	// returns:
	//		If the cell is editable, returns a promise resolving to the editor
	//		input/widget when the cell editor is focused.
	//		If the cell is not editable, returns null.
	
	var row, column, cellElement, dirty, field, value, cmp, dfd;
	
	if(!cell.column){ cell = this.cell(cell); }
	if(!cell || !cell.element){ return null; }
	
	column = cell.column;
	field = column.field;
	cellElement = cell.element.contents || cell.element;
	
	if((cmp = column.editorInstance)){ // shared editor (editOn used)
		if(activeCell != cellElement &&
				(!column.canEdit || column.canEdit(cell.row.data, value))){
			activeCell = cellElement;
			row = cell.row;
			dirty = this.dirty && this.dirty[row.id];
			value = (dirty && field in dirty) ? dirty[field] :
				column.get ? column.get(row.data) : row.data[field];
			
			showEditor(column.editorInstance, column, cellElement, value);
			
			// focus / blur-handler-resume logic is surrounded in a setTimeout
			// to play nice with Keyboard's dgrid-cellfocusin as an editOn event
			dfd = new Deferred();
			setTimeout(function(){
				// focus the newly-placed control (supported by form widgets and HTML inputs)
				if(cmp.focus){ cmp.focus(); }
				// resume blur handler once editor is focused
				if(column._editorBlurHandle){ column._editorBlurHandle.resume(); }
				dfd.resolve(cmp);
			}, 0);
			
			return dfd.promise;
		}
	}else if(column.editor){ // editor but not shared; always-on
		cmp = cellElement.widget || cellElement.input;
		if(cmp){
			dfd = new Deferred();
			if(cmp.focus){ cmp.focus(); }
			dfd.resolve(cmp);
			return dfd.promise;
		}
	}
	return null;
}

// editor column plugin function

return function(column, editor, editOn){
	// summary:
	//		Adds editing capability to a column's cells.
	
	var originalRenderCell = column.renderCell || Grid.defaultRenderCell,
		listeners = [],
		isWidget;
	
	if(!column){ column = {}; }
	
	// accept arguments as parameters to editor function, or from column def,
	// but normalize to column def.
	column.editor = editor = editor || column.editor || "text";
	column.editOn = editOn = editOn || column.editOn;
	
	isWidget = typeof editor != "string";
	
	// warn for widgetArgs -> editorArgs; TODO: remove @ 0.4
	if(column.widgetArgs){
		kernel.deprecated("column.widgetArgs", "use column.editorArgs instead",
			"dgrid 0.4");
		column.editorArgs = column.widgetArgs;
	}
	
	aspect.after(column, "init", editOn ? function(){
		var grid = column.grid;
		if(!grid.edit){ grid.edit = edit; }
		
		// Create one shared widget/input to be swapped into the active cell.
		column.editorInstance = createSharedEditor(column, originalRenderCell);
	} : function(){
		var grid = column.grid;
		if(!grid.edit){ grid.edit = edit; }
		
		if(isWidget){
			// add advice for cleaning up widgets in this column
			listeners.push(aspect.before(grid, "removeRow", function(rowElement){
				// destroy our widget during the row removal operation,
				// but don't trip over loading nodes from incomplete requests
				var cellElement = grid.cell(rowElement, column.id).element,
					widget = cellElement && (cellElement.contents || cellElement).widget;
				if(widget){ widget.destroyRecursive(); }
			}));
		}
	});
	
	aspect.after(column, "destroy", function(){
		arrayUtil.forEach(listeners, function(l){ l.remove(); });
		if(column._editorBlurHandle){ column._editorBlurHandle.remove(); }
		
		if(editOn && isWidget){ column.editorInstance.destroyRecursive(); }
	});
	
	column.renderCell = editOn ? function(object, value, cell, options){
		// TODO: Consider using event delegation
		// (Would require using dgrid's focus events for activating on focus,
		// which we already advocate in README for optimal use)
		
		if(!options || !options.alreadyHooked){
			// in IE<8, cell is the child of the td due to the extra padding node
			on(cell.tagName == "TD" ? cell : cell.parentNode, editOn, function(){
				activeOptions = options;
				column.grid.edit(this);
			});
		}
		
		// initially render content in non-edit mode
		return originalRenderCell.call(column, object, value, cell, options);
		
	} : function(object, value, cell, options){
		// always-on: create editor immediately upon rendering each cell
		if(!column.canEdit || column.canEdit(object, value)){
			var cmp = createEditor(column);
			showEditor(cmp, column, cell, value);
			// Maintain reference for later use.
			cell[isWidget ? "widget" : "input"] = cmp;
		}else{
			return originalRenderCell.call(column, object, value, cell, options);
		}
	};
	
	return column;
};
});

},
'dgrid/extensions/ColumnReorder':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/_base/array",
	"dojo/on",
	"dojo/query",
	"dojo/dnd/Source",
	"put-selector/put",
	"xstyle/css!../css/extensions/ColumnReorder.css"
], function(lang, declare, arrayUtil, on, query, DndSource, put){
	var dndTypeRx = /(\d+)(?:-(\d+))?$/; // used to determine subrow from dndType
	
	// The following 2 functions are used by onDropInternal logic for
	// retrieving/modifying a given subRow.  The `match` variable in each is
	// expected to be the result of executing dndTypeRx on a subRow ID.
	
	function getMatchingSubRow(grid, match) {
		var hasColumnSets = match[2],
			rowOrSet = grid[hasColumnSets ? "columnSets" : "subRows"][match[1]];
		
		return hasColumnSets ? rowOrSet[match[2]] : rowOrSet;
	}
	
	function setMatchingSubRow(grid, match, subRow) {
		if(match[2]){
			grid.columnSets[match[1]][match[2]] = subRow;
		}else{
			grid.subRows[match[1]] = subRow;
		}
	}

	// Builds a prefix for a dndtype value based on a grid id.
	function makeDndTypePrefix(gridId) {
		return "dgrid-" + gridId + '-';
	}

	// Removes the grid id prefix from a dndtype value.  This allows the grid id to contain
	// a dash-number suffix.  This works only if a column is dropped on the grid from which it
	// originated.  Otherwise, a dash-number suffix will cause the regex to match on the wrong values.
	function stripIdPrefix(gridId, dndtype) {
		return dndtype.slice(makeDndTypePrefix(gridId).length);
	}
	
	var ColumnDndSource = declare(DndSource, {
		// summary:
		//		Custom dojo/dnd source extension configured specifically for
		//		dgrid column reordering.
		
		copyState: function(){ return false; }, // never copy
		
		checkAcceptance: function(source, nodes){
			return source == this; // self-accept only
		},
		
		_legalMouseDown: function(evt){
			// Overridden to prevent blocking ColumnResizer resize handles.
			return evt.target.className.indexOf("dgrid-resize-handle") > -1 ? false :
				this.inherited(arguments);
		},
		
		onDropInternal: function(nodes){
			var grid = this.grid,
				match = dndTypeRx.exec(stripIdPrefix(grid.id, nodes[0].getAttribute("dndType"))),
				structureProperty = match[2] ? "columnSets" : "subRows",
				oldSubRow = getMatchingSubRow(grid, match),
				columns = grid.columns;
			
			// First, allow original DnD logic to place node in new location.
			this.inherited(arguments);
			
			if(!match){ return; }
			
			// Then, iterate through the header cells in their new order,
			// to populate a new row array to assign as a new sub-row to the grid.
			// (Wait until the next turn to avoid errors in Opera.)
			setTimeout(function(){
				var newSubRow = arrayUtil.map(nodes[0].parentNode.childNodes, function(col) {
						return columns[col.columnId];
					}),
					eventObject;
				
				setMatchingSubRow(grid, match, newSubRow);
				
				eventObject = {
					grid: grid,
					subRow: newSubRow,
					column: columns[nodes[0].columnId],
					bubbles: true,
					cancelable: true,
					// Set parentType to indicate this is the result of user interaction.
					parentType: "dnd"
				};
				// Set columnSets or subRows depending on which the grid is using.
				eventObject[structureProperty] = grid[structureProperty];
				
				// Emit a custom event which passes the new structure.
				// Allow calling preventDefault() to cancel the reorder operation.
				if(on.emit(grid.domNode, "dgrid-columnreorder", eventObject)){
					// Event was not canceled - force processing of modified structure.
					grid.set(structureProperty, grid[structureProperty]);
				}else{
					// Event was canceled - revert the structure and re-render the header
					// (since the inherited logic invoked above will have shifted cells).
					setMatchingSubRow(grid, match, oldSubRow);
					grid.renderHeader();
					// After re-rendering the header, re-apply the sort arrow if needed.
					if (this._sort && this._sort.length){
						this.updateSortArrow(this._sort);
					}
				}
			}, 0);
		}
	});
	
	var ColumnReorder = declare(null, {
		// summary:
		//		Extension allowing reordering of columns in a grid via drag'n'drop.
		//		Reordering of columns within the same subrow or columnset is also
		//		supported; between different ones is not.
		
		// columnDndConstructor: Function
		//		Constructor to call for instantiating DnD sources within the grid's
		//		header.
		columnDndConstructor: ColumnDndSource,
		
		_initSubRowDnd: function(subRow, dndType){
			// summary:
			//		Initializes a dojo/dnd source for one subrow of a grid;
			//		this could be its only subrow, one of several, or a subrow within a
			//		columnset.
			
			var dndParent, c, len, col, th;
			
			for(c = 0, len = subRow.length; c < len; c++){
				col = subRow[c];
				if(col.reorderable === false){ continue; }
				
				th = col.headerNode;
				if(th.tagName != "TH"){ th = th.parentNode; } // from IE < 8 padding
				// Add dojoDndItem class, and a dndType unique to this subrow.
				put(th, ".dojoDndItem[dndType=" + dndType + "]");
				
				if(!dndParent){ dndParent = th.parentNode; }
			}
			
			if(dndParent){ // (if dndParent wasn't set, no columns are draggable!)
				this._columnDndSources.push(new this.columnDndConstructor(dndParent, {
					horizontal: true,
					grid: this
				}));
			}
		},
		
		renderHeader: function(){
			var dndTypePrefix = makeDndTypePrefix(this.id),
				csLength, cs;
			
			this.inherited(arguments);
			
			// After header is rendered, set up a dnd source on each of its subrows.
			
			this._columnDndSources = [];
			
			if(this.columnSets){
				// Iterate columnsets->subrows->columns.
				for(cs = 0, csLength = this.columnSets.length; cs < csLength; cs++){
					arrayUtil.forEach(this.columnSets[cs], function(subRow, sr){
						this._initSubRowDnd(subRow, dndTypePrefix + cs + "-" + sr);
					}, this);
				}
			}else{
				// Iterate subrows->columns.
				arrayUtil.forEach(this.subRows, function(subRow, sr){
					this._initSubRowDnd(subRow, dndTypePrefix + sr);
				}, this);
			}
		},
		
		_destroyColumns: function(){
			if(this._columnDndSources){
				// Destroy old dnd sources.
				arrayUtil.forEach(this._columnDndSources, function(source){
					source.destroy();
				});
			}
			
			this.inherited(arguments);
		}
	});
	
	ColumnReorder.ColumnDndSource = ColumnDndSource;
	return ColumnReorder;
});

},
'dgrid/extensions/ColumnResizer':function(){
define(["dojo/_base/declare", "dojo/on", "dojo/query", "dojo/_base/lang", "dojo/dom", "dojo/dom-geometry", "dojo/has", "../util/misc", "put-selector/put", "dojo/_base/html", "xstyle/css!../css/extensions/ColumnResizer.css"],
function(declare, listen, query, lang, dom, geom, has, miscUtil, put){

function addRowSpan(table, span, startRow, column, id){
	// loop through the rows of the table and add this column's id to
	// the rows' column
	for(var i=1; i<span; i++){
		table[startRow+i][column] = id;
	}
}
function subRowAssoc(subRows){
	// Take a sub-row structure and output an object with key=>value pairs
	// The keys will be the column id's; the values will be the first-row column
	// that column's resizer should be associated with.

	var i = subRows.length,
		l = i,
		numCols = subRows[0].length,
		table = new Array(i);

	// create table-like structure in an array so it can be populated
	// with row-spans and col-spans
	while(i--){
		table[i] = new Array(numCols);
	}

	var associations = {};

	for(i=0; i<l; i++){
		var row = table[i],
			subRow = subRows[i];

		// j: counter for table columns
		// js: counter for subrow structure columns
		for(var j=0, js=0; j<numCols; j++){
			var cell = subRow[js], k;

			// if something already exists in the table (row-span), skip this
			// spot and go to the next
			if(typeof row[j] != "undefined"){
				continue;
			}
			row[j] = cell.id;

			if(cell.rowSpan && cell.rowSpan > 1){
				addRowSpan(table, cell.rowSpan, i, j, cell.id);
			}

			// colSpans are only applicable in the second or greater rows
			// and only if the colSpan is greater than 1
			if(i>0 && cell.colSpan && cell.colSpan > 1){
				for(k=1; k<cell.colSpan; k++){
					// increment j and assign the id since this is a span
					row[++j] = cell.id;
					if(cell.rowSpan && cell.rowSpan > 1){
						addRowSpan(table, cell.rowSpan, i, j, cell.id);
					}
				}
			}
			associations[cell.id] = subRows[0][j].id;
			js++;
		}
	}

	return associations;
}

function resizeColumnWidth(grid, colId, width, parentType){
	// don't react to widths <= 0, e.g. for hidden columns
	if(width <= 0){ return; }

	var column = grid.columns[colId],
		event = {
			grid: grid,
			columnId: colId,
			width: width,
			bubbles: true,
			cancelable: true
		},
		rule;
	
	if(parentType){
		event.parentType = parentType;
	}
	
	if(!grid._resizedColumns || listen.emit(grid.headerNode, "dgrid-columnresize", event)){
		// Update width on column object, then convert value for CSS
		if(width === "auto"){
			delete column.width;
		}else{
			column.width = width;
			width += "px";
		}
		
		rule = grid._columnSizes[colId];
		
		if(rule){
			// Modify existing, rather than deleting + adding
			rule.set("width", width);
		}else{
			// Use miscUtil function directly, since we clean these up ourselves anyway
			rule = miscUtil.addCssRule(
				"#" + miscUtil.escapeCssIdentifier(grid.domNode.id) + " .dgrid-column-" + colId, "width: " + width + ";");
		}

		// keep a reference for future removal
		grid._columnSizes[colId] = rule;
		grid.resize();
		return true;
	}
}

// Functions for shared resizer node

var resizerNode, // DOM node for resize indicator, reused between instances
	resizableCount = 0; // Number of ColumnResizer-enabled grid instances
var resizer = {
	// This object contains functions for manipulating the shared resizerNode
	create: function(){
		resizerNode = put("div.dgrid-column-resizer");
	},
	destroy: function(){
		put(resizerNode, "!");
		resizerNode = null;
	},
	show: function(grid){
		var pos = geom.position(grid.domNode, true);
		resizerNode.style.top = pos.y + "px";
		resizerNode.style.height = pos.h + "px";
		put(document.body, resizerNode);
	},
	move: function(x){
		resizerNode.style.left = x + "px";
	},
	hide: function(){
		resizerNode.parentNode.removeChild(resizerNode);
	}
};

return declare(null, {
	resizeNode: null,
	
	// minWidth: Number
	//		Minimum column width, in px.
	minWidth: 40,
	
	// adjustLastColumn: Boolean
	//		If true, adjusts the last column's width to "auto" at times where the
	//		browser would otherwise stretch all columns to span the grid.
	adjustLastColumn: true,
	
	_resizedColumns: false, // flag indicating if resizer has converted column widths to px
	
	buildRendering: function(){
		this.inherited(arguments);
		
		// Create resizerNode when first grid w/ ColumnResizer is created
		if(!resizableCount++){
			resizer.create();
		}
	},
	
	destroy: function(){
		this.inherited(arguments);
		
		// Remove any applied column size styles since we're tracking them directly
		for(var name in this._columnSizes){
			this._columnSizes[name].remove();
		}
		
		// If this is the last grid on the page with ColumnResizer, destroy the
		// shared resizerNode
		if(!--resizableCount){
			resizer.destroy();
		}
	},
	
	resizeColumnWidth: function(colId, width){
		// Summary:
		//      calls grid's styleColumn function to add a style for the column
		// colId: String
		//      column id
		// width: Integer
		//      new width of the column
		return resizeColumnWidth(this, colId, width);
	},
	
	configStructure: function(){
		var oldSizes = this._oldColumnSizes = lang.mixin({}, this._columnSizes), // shallow clone
			k;
		
		this._resizedColumns = false;
		this._columnSizes = {};
		
		this.inherited(arguments);
		
		// Remove old column styles that are no longer relevant; this is specifically
		// done *after* calling inherited so that _columnSizes will contain keys
		// for all columns in the new structure that were assigned widths.
		for(k in oldSizes){
			if(!(k in this._columnSizes)){
				oldSizes[k].remove();
			}
		}
		delete this._oldColumnSizes;
	},
	
	_configColumn: function(column){
		this.inherited(arguments);
		
		var colId = column.id,
			rule;
		
		if("width" in column){
			// Update or add a style rule for the specified width
			if((rule = this._oldColumnSizes[colId])){
				rule.set("width", column.width + "px");
			}else{
				rule = miscUtil.addCssRule(
					"#" + this.domNode.id + " .dgrid-column-" + colId, "width: " + column.width + "px;");
			}
			this._columnSizes[colId] = rule;
		}
	},
	
	renderHeader: function(){
		this.inherited(arguments);
		
		var grid = this;
		
		var assoc;
		if(this.columnSets && this.columnSets.length){
			var csi = this.columnSets.length;
			while(csi--){
				assoc = lang.mixin(assoc||{}, subRowAssoc(this.columnSets[csi]));
			}
		}else if(this.subRows && this.subRows.length > 1){
			assoc = subRowAssoc(this.subRows);
		}

		var colNodes = query(".dgrid-cell", grid.headerNode),
			i = colNodes.length;
		while(i--){
			var colNode = colNodes[i],
				id = colNode.columnId,
				col = grid.columns[id],
				childNodes = colNode.childNodes;

			if(!col || col.resizable === false){ continue; }

			var headerTextNode = put("div.dgrid-resize-header-container");
			colNode.contents = headerTextNode;

			// move all the children to the header text node
			while(childNodes.length > 0){
				put(headerTextNode, childNodes[0]);
			}

			put(colNode, headerTextNode, "div.dgrid-resize-handle.resizeNode-"+id).columnId = 
				assoc ? assoc[id] : id;
		}

		if(!grid.mouseMoveListen){
			// establish listeners for initiating, dragging, and finishing resize
			listen(grid.headerNode,
				".dgrid-resize-handle:mousedown" +
					(has("touch") ? ",.dgrid-resize-handle:touchstart" : ""),
				function(e){
					grid._resizeMouseDown(e, this);
					grid.mouseMoveListen.resume();
					grid.mouseUpListen.resume();
				}
			);
			grid._listeners.push(grid.mouseMoveListen = listen.pausable(document,
				"mousemove" + (has("touch") ? ",touchmove" : ""),
				miscUtil.throttleDelayed(function(e){ grid._updateResizerPosition(e); })
			));
			grid._listeners.push(grid.mouseUpListen = listen.pausable(document,
				"mouseup" + (has("touch") ? ",touchend" : ""),
				function(e){
					grid._resizeMouseUp(e);
					grid.mouseMoveListen.pause();
					grid.mouseUpListen.pause();
				}
			));
			// initially pause the move/up listeners until a drag happens
			grid.mouseMoveListen.pause();
			grid.mouseUpListen.pause();
		}
	}, // end renderHeader

	_resizeMouseDown: function(e, target){
		// Summary:
		//      called when mouse button is pressed on the header
		// e: Object
		//      mousedown event object
		
		// preventDefault actually seems to be enough to prevent browser selection
		// in all but IE < 9.  setSelectable works for those.
		e.preventDefault();
		dom.setSelectable(this.domNode, false);
		this._startX = this._getResizeMouseLocation(e); //position of the target
		
		this._targetCell = query(".dgrid-column-" + target.columnId, this.headerNode)[0];

		// Show resizerNode after initializing its x position
		this._updateResizerPosition(e);
		resizer.show(this);
	},
	_resizeMouseUp: function(e){
		// Summary:
		//      called when mouse button is released
		// e: Object
		//      mouseup event object
		
		var columnSizes = this._columnSizes,
			colNodes, colWidths, gridWidth;
		
		if(this.adjustLastColumn){
			// For some reason, total column width needs to be 1 less than this
			gridWidth = this.headerNode.clientWidth - 1;
		}
		
		//This is used to set all the column widths to a static size
		if(!this._resizedColumns){
			colNodes = query(".dgrid-cell", this.headerNode);
			
			if(this.columnSets && this.columnSets.length){
				colNodes = colNodes.filter(function(node){
					var idx = node.columnId.split("-");
					return idx[0] == "0" && !(node.columnId in columnSizes);
				});
			}else if(this.subRows && this.subRows.length > 1){
				colNodes = colNodes.filter(function(node){
					return node.columnId.charAt(0) == "0" && !(node.columnId in columnSizes);
				});
			}
			
			// Get a set of sizes before we start mutating, to avoid
			// weird disproportionate measures if the grid has set
			// column widths, but no full grid width set
			colWidths = colNodes.map(function(colNode){
				return colNode.offsetWidth;
			});
			
			// Set a baseline size for each column based on
			// its original measure
			colNodes.forEach(function(colNode, i){
				this.resizeColumnWidth(colNode.columnId, colWidths[i]);
			}, this);
			
			this._resizedColumns = true;
		}
		dom.setSelectable(this.domNode, true);
		
		var cell = this._targetCell,
			delta = this._getResizeMouseLocation(e) - this._startX, //final change in position of resizer
			newWidth = cell.offsetWidth + delta, //the new width after resize
			obj = this._getResizedColumnWidths(),//get current total column widths before resize
			totalWidth = obj.totalWidth,
			lastCol = obj.lastColId,
			lastColWidth = query(".dgrid-column-"+lastCol, this.headerNode)[0].offsetWidth;
		
		if(newWidth < this.minWidth){
			//enforce minimum widths
			newWidth = this.minWidth;
		}
		
		if(resizeColumnWidth(this, cell.columnId, newWidth, e.type)){
			if(cell.columnId != lastCol && this.adjustLastColumn){
				if(totalWidth + delta < gridWidth) {
					//need to set last column's width to auto
					resizeColumnWidth(this, lastCol, "auto", e.type);
				}else if(lastColWidth-delta <= this.minWidth) {
					//change last col width back to px, unless it is the last column itself being resized...
					resizeColumnWidth(this, lastCol, this.minWidth, e.type);
				}
			}
		}
		resizer.hide();
		
		// Clean up after the resize operation
		delete this._startX;
		delete this._targetCell;
	},
	
	_updateResizerPosition: function(e){
		// Summary:
		//      updates position of resizer bar as mouse moves
		// e: Object
		//      mousemove event object

		if(!this._targetCell){ return; } // Release event was already processed
		
		var mousePos = this._getResizeMouseLocation(e),
			delta = mousePos - this._startX, //change from where user clicked to where they drag
			width = this._targetCell.offsetWidth,
			left = mousePos;
		if(width + delta < this.minWidth){ 
			left = this._startX - (width - this.minWidth); 
		}
		resizer.move(left);
	},

	_getResizeMouseLocation: function(e){
		//Summary:
		//      returns position of mouse relative to the left edge
		// e: event object
		//      mouse move event object
		var posX = 0;
		if(e.pageX){
			posX = e.pageX;
		}else if(e.clientX){
			posX = e.clientX + document.body.scrollLeft +
				document.documentElement.scrollLeft;
		}
		return posX;
	},
	_getResizedColumnWidths: function (){
		//Summary:
		//      returns object containing new column width and column id
		var totalWidth = 0,
			colNodes = query(".dgrid-cell", this.headerNode);

		// For ColumnSets and subRows, only the top row of columns matters
		if(this.columnSets && this.columnSets.length){
			colNodes = colNodes.filter(function(node){
				var idx = node.columnId.split("-");
				return idx[1] == "0";
			});
		}else if(this.subRows && this.subRows.length > 1){
			colNodes = colNodes.filter(function(node){
				return node.columnId.charAt(0) == "0";
			});
		}

		var i = colNodes.length;
		if(!i){ return {}; }

		var lastColId = colNodes[i-1].columnId;

		while(i--){
			totalWidth += colNodes[i].offsetWidth;
		}
		return {totalWidth: totalWidth, lastColId: lastColId};
	}
});
});

},
'dgrid/extensions/CompoundColumns':function(){
define([
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dgrid/util/misc",
	"xstyle/css!../css/extensions/CompoundColumns.css"
], function(lang, declare, miscUtil){
	return declare(null, {
		// summary:
		//		Extension allowing for specification of columns with additional
		//		header rows spanning multiple columns for strictly display purposes.
		//		Only works on `columns` arrays, not `columns` objects or `subRows`
		//		(nor ColumnSets).
		// description:
		//		CompoundColumns allows nested header cell configurations, wherein the
		//		higher-level headers may span multiple columns and are for
		//		display purposes only.
		//		These nested header cells are configured using a special recursive
		//		`children` property in the column definition, where only the deepest
		//		children are ultimately rendered in the grid as actual columns.
		//		In addition, the deepest child columns may be rendered without
		//		individual headers by specifying `showChildHeaders: false` on the parent.
		
		configStructure: function(){
			// create a set of sub rows for the header row so we can do compound columns
			// the first row is a special spacer row
			var columns = (this.subRows && this.subRows[0]) || this.columns,
				headerRows = [[]],
				contentColumns = [];
			// This first row is spacer row that will be made invisible (zero height)
			// with CSS, but it must be rendered as the first row since that is what
			// the table layout is driven by.
			headerRows[0].className = "dgrid-spacer-row";
			
			function processColumns(columns, level, hasLabel){
				var numColumns = 0,
					noop = function(){},
					column, children, hasChildLabels;
				
				function processColumn(column, i){
					children = column.children;
					hasChildLabels = column.children && (column.showChildHeaders !== false);
					if(children){
						// it has children, recursively process the children
						numColumns += (column.colSpan = processColumns(children, level + 1, hasChildLabels));
					}else{
						// it has no children, it is a normal header, add it to the content columns
						contentColumns.push(column);
						// add each one to the first spacer header row for proper layout of the header cells
						headerRows[0].push(lang.delegate(column, {renderHeaderCell: noop}));
						numColumns++;
					}
					if(!hasChildLabels){
						// create a header version of the column where we can define a specific rowSpan
						// we define the rowSpan as a negative, the number of levels less than the total number of rows, which we don't know yet
						column = lang.delegate(column, {rowSpan: -level});
					}
					// add the column to the header rows at the appropriate level
					if(hasLabel){
						(headerRows[level] || (headerRows[level] = [])).push(column);
					}
				}
				
				miscUtil.each(columns, processColumn, this);
				return numColumns;
			}
			processColumns(columns, 1, true);
			
			var numHeaderRows = headerRows.length,
				i, j, headerRow, headerColumn;
			// Now go back through and increase the rowSpans of the headers to be
			// total rows minus the number of levels they are at.
			for(i = 0; i < numHeaderRows; i++){
				headerRow = headerRows[i];
				for(j = 0; j < headerRow.length; j++){
					headerColumn = headerRow[j];
					if(headerColumn.rowSpan < 1){
						headerColumn.rowSpan += numHeaderRows;
					}
				}
			}
			// we need to set this to be used for subRows, so we make it a single row
			contentColumns = [contentColumns];
			// set our header rows so that the grid will use the alternate header row 
			// configuration for rendering the headers
			contentColumns.headerRows = headerRows;  
			this.subRows = contentColumns;
			this.inherited(arguments);
		}
	});
});

},
'dgrid/extensions/DijitRegistry':function(){
define(["dojo/_base/declare", "dojo/dom-geometry", "dijit/registry"],
function(declare, domGeometry, registry){
	return declare(null, {
		// summary:
		//		A dgrid extension which will add the grid to the dijit registry,
		//		so that startup() will be successfully called by dijit layout widgets
		//		with dgrid children.
		
		// Defaults normally imposed on _WidgetBase by container widget modules:
		minSize: 0, // BorderContainer
		maxSize: Infinity, // BorderContainer
		layoutPriority: 0, // BorderContainer
		showTitle: true, // StackContainer
		
		buildRendering: function(){
			registry.add(this);
			this.inherited(arguments);
			// Note: for dojo 2.0 may rename widgetId to dojo._scopeName + "_widgetId"
			this.domNode.setAttribute("widgetId", this.id);
		},
		
		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);
			
			var widget = registry.getEnclosingWidget(this.domNode.parentNode);
			// If we have a parent layout container widget, it will handle resize,
			// so remove the window resize listener added by List.
			if(widget && widget.isLayoutContainer){
				this._resizeHandle.remove();
			}
		},

		destroyRecursive: function() {
			this.destroy();
		},
		
		destroy: function(){
			this.inherited(arguments);
			registry.remove(this.id);
		},
		
		getChildren: function(){
			// provide hollow implementation for logic which assumes its existence
			// (e.g. dijit/form/_FormMixin)
			return [];
		},
		
		isLeftToRight: function(){
			// Implement method expected by Dijit layout widgets
			return !this.isRTL;
		},
		
		resize: function(changeSize){
			// Honor changeSize parameter used by layout widgets, and resize grid
			if(changeSize){
				domGeometry.setMarginBox(this.domNode, changeSize);
			}
			
			this.inherited(arguments);
		},
		
		_set: function(prop, value){
			// summary:
			//		Simple analogue of _WidgetBase#_set for compatibility with some
			//		Dijit layout widgets which assume its existence.
			this[prop] = value;
		},
		
		watch: function(){
			// summary:
			//		dgrid doesn't support watch; this is a no-op for compatibility with
			//		some Dijit layout widgets which assume its existence.
		},
		
		getParent: function(){
			// summary:
			//		Analogue of _WidgetBase#getParent for compatibility with for example
			//		dijit._KeyNavContainer.
			return registry.getEnclosingWidget(this.domNode.parentNode);
		}
	});
});

},
'dgrid/extensions/DnD':function(){
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/Deferred",
	"dojo/aspect",
	"dojo/on",
	"dojo/topic",
	"dojo/has",
	"dojo/dnd/Source",
	"dojo/dnd/Manager",
	"dojo/_base/NodeList",
	"put-selector/put",
	"../Selection",
	"dojo/has!touch?../util/touch",
	"dojo/has!touch?./_DnD-touch-autoscroll",
	"xstyle/css!dojo/resources/dnd.css"
], function(declare, lang, arrayUtil, Deferred, aspect, on, topic, has, DnDSource, DnDManager, NodeList, put, Selection, touchUtil){
	// Requirements
	// * requires a store (sounds obvious, but not all Lists/Grids have stores...)
	// * must support options.before in put calls
	//   (if undefined, put at end)
	// * should support copy
	//   (copy should also support options.before as above)
	
	// TODOs
	// * consider sending items rather than nodes to onDropExternal/Internal
	// * consider emitting store errors via OnDemandList._trackError
	
	var GridDnDSource = declare(DnDSource, {
		grid: null,
		
		getObject: function(node){
			// summary:
			//		getObject is a method which should be defined on any source intending
			//		on interfacing with dgrid DnD.
			
			var grid = this.grid;
			// Extract item id from row node id (gridID-row-*).
			return grid.store.get(node.id.slice(grid.id.length + 5));
		},
		_legalMouseDown: function(evt){
			// Fix _legalMouseDown to only allow starting drag from an item
			// (not from bodyNode outside contentNode).
			var legal = this.inherited(arguments);
			return legal && evt.target != this.grid.bodyNode;
		},

		// DnD method overrides
		onDrop: function(sourceSource, nodes, copy){
			var targetSource = this,
				targetRow = this._targetAnchor = this.targetAnchor, // save for Internal
				grid = this.grid,
				store = grid.store;
			
			if(!this.before && targetRow){
				// target before next node if dropped within bottom half of this node
				// (unless there's no node to target at all)
				targetRow = targetRow.nextSibling;
			}
			targetRow = targetRow && grid.row(targetRow);
			
			Deferred.when(targetRow && store.get(targetRow.id), function(target){
				// Note: if dropping after the last row, or into an empty grid,
				// target will be undefined.  Thus, it is important for store to place
				// item last in order if options.before is undefined.
				
				// Delegate to onDropInternal or onDropExternal for rest of logic.
				// These are passed the target item as an additional argument.
				if(targetSource != sourceSource){
					targetSource.onDropExternal(sourceSource, nodes, copy, target);
				}else{
					targetSource.onDropInternal(nodes, copy, target);
				}
			});
		},
		onDropInternal: function(nodes, copy, targetItem){
			var store = this.grid.store,
				targetSource = this,
				grid = this.grid,
				anchor = targetSource._targetAnchor,
				targetRow;
			
			if(anchor){ // (falsy if drop occurred in empty space after rows)
				targetRow = this.before ? anchor.previousSibling : anchor.nextSibling;
			}
			
			// Don't bother continuing if the drop is really not moving anything.
			// (Don't need to worry about edge first/last cases since dropping
			// directly on self doesn't fire onDrop, but we do have to worry about
			// dropping last node into empty space beyond rendered rows.)
			if(!copy && (targetRow === nodes[0] ||
					(!targetItem && grid.down(grid.row(nodes[0])).element == nodes[0]))){
				return;
			}
			
			nodes.forEach(function(node){
				Deferred.when(targetSource.getObject(node), function(object){
					// For copy DnD operations, copy object, if supported by store;
					// otherwise settle for put anyway.
					// (put will relocate an existing item with the same id, i.e. move).
					store[copy && store.copy ? "copy" : "put"](object, {
						before: targetItem
					});
				});
			});
		},
		onDropExternal: function(sourceSource, nodes, copy, targetItem){
			// Note: this default implementation expects that two grids do not
			// share the same store.  There may be more ideal implementations in the
			// case of two grids using the same store (perhaps differentiated by
			// query), dragging to each other.
			var store = this.grid.store,
				sourceGrid = sourceSource.grid;
			
			// TODO: bail out if sourceSource.getObject isn't defined?
			nodes.forEach(function(node, i){
				Deferred.when(sourceSource.getObject(node), function(object){
					if(!copy){
						if(sourceGrid){
							// Remove original in the case of inter-grid move.
							// (Also ensure dnd source is cleaned up properly)
							Deferred.when(sourceGrid.store.getIdentity(object), function(id){
								!i && sourceSource.selectNone(); // deselect all, one time
								sourceSource.delItem(node.id);
								sourceGrid.store.remove(id);
							});
						}else{
							sourceSource.deleteSelectedNodes();
						}
					}
					// Copy object, if supported by store; otherwise settle for put
					// (put will relocate an existing item with the same id).
					// Note that we use store.copy if available even for non-copy dnd:
					// since this coming from another dnd source, always behave as if
					// it is a new store item if possible, rather than replacing existing.
					store[store.copy ? "copy" : "put"](object, {
						before: targetItem
					});
				});
			});
		},
		
		onDndStart: function(source, nodes, copy){
			// Listen for start events to apply style change to avatar.
			
			this.inherited(arguments); // DnDSource.prototype.onDndStart.apply(this, arguments);
			if(source == this){
				// If TouchScroll is in use, cancel any pending scroll operation.
				if(this.grid.cancelTouchScroll){ this.grid.cancelTouchScroll(); }
				
				// Set avatar width to half the grid's width.
				// Kind of a naive default, but prevents ridiculously wide avatars.
				DnDManager.manager().avatar.node.style.width =
					this.grid.domNode.offsetWidth / 2 + "px";
			}
		},
		
		onMouseDown: function(evt){
			// Cancel the drag operation on presence of more than one contact point.
			// (This check will evaluate to false under non-touch circumstances.)
			if(has("touch") && this.isDragging &&
					touchUtil.countCurrentTouches(evt, this.grid.touchNode) > 1){
				topic.publish("/dnd/cancel");
				DnDManager.manager().stopDrag();
			}else{
				this.inherited(arguments);
			}
		},
		
		onMouseMove: function(evt){
			// If we're handling touchmove, only respond to single-contact events.
			if(!has("touch") || touchUtil.countCurrentTouches(evt, this.grid.touchNode) === 1){
				this.inherited(arguments);
			}
		},
		
		checkAcceptance: function(source, nodes){
			// Augment checkAcceptance to block drops from sources without getObject.
			return source.getObject &&
				DnDSource.prototype.checkAcceptance.apply(this, arguments);
		},
		getSelectedNodes: function(){
			// If dgrid's Selection mixin is in use, synchronize with it, using a
			// map of node references (updated on dgrid-[de]select events).
			
			if(!this.grid.selection){
				return this.inherited(arguments);
			}
			var t = new NodeList(),
				id;
			for(id in this.grid.selection){
				t.push(this._selectedNodes[id]);
			}
			return t;	// NodeList
		}
		// TODO: could potentially also implement copyState to jive with default
		// onDrop* implementations (checking whether store.copy is available);
		// not doing that just yet until we're sure about default impl.
	});
	
	// Mix in Selection for more resilient dnd handling, particularly when part
	// of the selection is scrolled out of view and unrendered (which we
	// handle below).
	var DnD = declare(Selection, {
		// dndSourceType: String
		//		Specifies the type which will be set for DnD items in the grid,
		//		as well as what will be accepted by it by default.
		dndSourceType: "dgrid-row",
		
		// dndParams: Object
		//		Object containing params to be passed to the DnD Source constructor.
		dndParams: null,
		
		// dndConstructor: Function
		//		Constructor from which to instantiate the DnD Source.
		//		Defaults to the GridSource constructor defined/exposed by this module.
		dndConstructor: GridDnDSource,
		
		postMixInProperties: function(){
			this.inherited(arguments);
			// ensure dndParams is initialized
			this.dndParams = lang.mixin({ accept: [this.dndSourceType] }, this.dndParams);
		},
		
		postCreate: function(){
			this.inherited(arguments);
			
			// Make the grid's content a DnD source/target.
			this.dndSource = new (this.dndConstructor || GridDnDSource)(
				this.bodyNode,
				lang.mixin(this.dndParams, {
					// add cross-reference to grid for potential use in inter-grid drop logic
					grid: this,
					dropParent: this.contentNode
				})
			);
			
			// Set up select/deselect handlers to maintain references, in case selected
			// rows are scrolled out of view and unrendered, but then dragged.
			var selectedNodes = this.dndSource._selectedNodes = {};
			
			function selectRow(row){
				selectedNodes[row.id] = row.element;
			}
			function deselectRow(row){
				delete selectedNodes[row.id];
			}
			
			this.on("dgrid-select", function(event){
				arrayUtil.forEach(event.rows, selectRow);
			});
			this.on("dgrid-deselect", function(event){
				arrayUtil.forEach(event.rows, deselectRow);
			});
			
			aspect.after(this, "destroy", function(){
				delete this.dndSource._selectedNodes;
				selectedNodes = null;
				this.dndSource.destroy();
			}, true);
		},
		
		insertRow: function(object){
			// override to add dojoDndItem class to make the rows draggable
			var row = this.inherited(arguments),
				type = typeof this.getObjectDndType == "function" ?
					this.getObjectDndType(object) : [this.dndSourceType];
			
			put(row, ".dojoDndItem");
			this.dndSource.setItem(row.id, {
				data: object,
				type: type instanceof Array ? type : [type]
			});
			return row;
		},
		
		removeRow: function (rowElement) {
			this.dndSource.delItem(this.row(rowElement));
			this.inherited(arguments);
		}
	});
	DnD.GridSource = GridDnDSource;
	
	return DnD;
});

},
'dgrid/Selection':function(){
define(["dojo/_base/kernel", "dojo/_base/declare", "dojo/_base/Deferred", "dojo/on", "dojo/has", "dojo/aspect", "./List", "dojo/has!touch?./util/touch", "put-selector/put", "dojo/query", "dojo/_base/sniff"],
function(kernel, declare, Deferred, on, has, aspect, List, touchUtil, put){

has.add("mspointer", function(global, doc, element){
	return "onmspointerdown" in element;
});

// Add feature test for user-select CSS property for optionally disabling
// text selection.
// (Can't use dom.setSelectable prior to 1.8.2 because of bad sniffs, see #15990)
has.add("css-user-select", function(global, doc, element){
	var style = element.style,
		prefixes = ["Khtml", "O", "ms", "Moz", "Webkit"],
		i = prefixes.length,
		name = "userSelect";

	// Iterate prefixes from most to least likely
	do{
		if(typeof style[name] !== "undefined"){
			// Supported; return property name
			return name;
		}
	}while(i-- && (name = prefixes[i] + "UserSelect"));

	// Not supported if we didn't return before now
	return false;
});

// Also add a feature test for the onselectstart event, which offers a more
// graceful fallback solution than node.unselectable.
has.add("dom-selectstart", typeof document.onselectstart !== "undefined");

var ctrlEquiv = has("mac") ? "metaKey" : "ctrlKey",
	hasUserSelect = has("css-user-select"),
	downType = has("mspointer") ? "MSPointerDown" : "mousedown",
	upType = has("mspointer") ? "MSPointerUp" : "mouseup";

function makeUnselectable(node, unselectable){
	// Utility function used in fallback path for recursively setting unselectable
	var value = node.unselectable = unselectable ? "on" : "",
		elements = node.getElementsByTagName("*"),
		i = elements.length;
	
	while(--i){
		if(elements[i].tagName === "INPUT" || elements[i].tagName === "TEXTAREA"){
			continue; // Don't prevent text selection in text input fields.
		}
		elements[i].unselectable = value;
	}
}

function setSelectable(grid, selectable){
	// Alternative version of dojo/dom.setSelectable based on feature detection.
	
	// For FF < 21, use -moz-none, which will respect -moz-user-select: text on
	// child elements (e.g. form inputs).  In FF 21, none behaves the same.
	// See https://developer.mozilla.org/en-US/docs/CSS/user-select
	var node = grid.bodyNode,
		value = selectable ? "text" : has("ff") < 21 ? "-moz-none" : "none";
	
	if(hasUserSelect){
		node.style[hasUserSelect] = value;
	}else if(has("dom-selectstart")){
		// For browsers that don't support user-select but support selectstart (IE<10),
		// we can hook up an event handler as necessary.  Since selectstart bubbles,
		// it will handle any child elements as well.
		// Note, however, that both this and the unselectable fallback below are
		// incapable of preventing text selection from outside the targeted node.
		if(!selectable && !grid._selectstartHandle){
			grid._selectstartHandle = on(node, "selectstart", function(evt){
				var tag = evt.target && evt.target.tagName;
				
				// Prevent selection except where a text input field is involved.
				if(tag !== "INPUT" && tag !== "TEXTAREA"){
					evt.preventDefault();
				}
			});
		}else if(selectable && grid._selectstartHandle){
			grid._selectstartHandle.remove();
			delete grid._selectstartHandle;
		}
	}else{
		// For browsers that don't support either user-select or selectstart (Opera),
		// we need to resort to setting the unselectable attribute on all nodes
		// involved.  Since this doesn't automatically apply to child nodes, we also
		// need to re-apply it whenever rows are rendered.
		makeUnselectable(node, !selectable);
		if(!selectable && !grid._unselectableHandle){
			grid._unselectableHandle = aspect.after(grid, "renderRow", function(row){
				makeUnselectable(row, true);
				return row;
			});
		}else if(selectable && grid._unselectableHandle){
			grid._unselectableHandle.remove();
			delete grid._unselectableHandle;
		}
	}
}

return declare(null, {
	// summary:
	//		Add selection capabilities to a grid. The grid will have a selection property and
	//		fire "dgrid-select" and "dgrid-deselect" events.
	
	// selectionDelegate: String
	//		Selector to delegate to as target of selection events.
	selectionDelegate: ".dgrid-row",
	
	// selectionEvents: String
	//		Event (or events, comma-delimited) to listen on to trigger select logic.
	//		Note: this is ignored in the case of touch devices.
	selectionEvents: downType + "," + upType + ",dgrid-cellfocusin",
	
	// deselectOnRefresh: Boolean
	//		If true, the selection object will be cleared when refresh is called.
	deselectOnRefresh: true,
	
	// allowSelectAll: Boolean
	//		If true, allow ctrl/cmd+A to select all rows.
	//		Also consulted by the selector plugin for showing select-all checkbox.
	allowSelectAll: false,
	
	// selection:
	//		An object where the property names correspond to 
	//		object ids and values are true or false depending on whether an item is selected
	selection: {},
	
	// selectionMode: String
	//		The selection mode to use, can be "none", "multiple", "single", or "extended".
	selectionMode: "extended",
	
	// allowTextSelection: Boolean
	//		Whether to still allow text within cells to be selected.  The default
	//		behavior is to allow text selection only when selectionMode is none;
	//		setting this property to either true or false will explicitly set the
	//		behavior regardless of selectionMode.
	allowTextSelection: undefined,
	
	create: function(){
		this.selection = {};
		return this.inherited(arguments);
	},
	postCreate: function(){
		this.inherited(arguments);
		
		// Force selectionMode setter to run.
		var selectionMode = this.selectionMode;
		this.selectionMode = "";
		this._setSelectionMode(selectionMode);
		
		this._initSelectionEvents(); // first time; set up event hooks
	},
	
	destroy: function(){
		this.inherited(arguments);
		
		// Remove any extra handles added by Selection.
		if(this._selectstartHandle){ this._selectstartHandle.remove(); }
		if(this._unselectableHandle){ this._unselectableHandle.remove(); }
		if(this._removeDeselectSignals){ this._removeDeselectSignals(); }
	},
	
	_setSelectionMode: function(mode){
		// summary:
		//		Updates selectionMode, resetting necessary variables.
		if(mode == this.selectionMode){ return; } // prevent unnecessary spinning
		
		// Start selection fresh when switching mode.
		this.clearSelection();
		
		this.selectionMode = mode;
		
		// Compute name of selection handler for this mode once
		// (in the form of _fooSelectionHandler)
		this._selectionHandlerName = "_" + mode + "SelectionHandler";
		
		// Also re-run allowTextSelection setter in case it is in automatic mode.
		this._setAllowTextSelection(this.allowTextSelection);
	},
	setSelectionMode: function(mode){
		kernel.deprecated("setSelectionMode(...)", 'use set("selectionMode", ...) instead', "dgrid 0.4");
		this.set("selectionMode", mode);
	},
	
	_setAllowTextSelection: function(allow){
		if(typeof allow !== "undefined"){
			setSelectable(this, allow);
		}else{
			setSelectable(this, this.selectionMode === "none");
		}
		this.allowTextSelection = allow;
	},
	
	_handleSelect: function(event, target){
		// Don't run if selection mode doesn't have a handler (incl. "none"),
		// or if coming from a dgrid-cellfocusin from a mousedown
		if(!this[this._selectionHandlerName] ||
				(event.type === "dgrid-cellfocusin" && event.parentType === "mousedown") ||
				(event.type === upType && target != this._waitForMouseUp)){
			return;
		}
		this._waitForMouseUp = null;
		this._selectionTriggerEvent = event;
		
		// Don't call select handler for ctrl+navigation
		if(!event.keyCode || !event.ctrlKey || event.keyCode == 32){
			// If clicking a selected item, wait for mouseup so that drag n' drop
			// is possible without losing our selection
			if(!event.shiftKey && event.type === downType && this.isSelected(target)){
				this._waitForMouseUp = target;
			}else{
				this[this._selectionHandlerName](event, target);
			}
		}
		this._selectionTriggerEvent = null;
	},
	
	_singleSelectionHandler: function(event, target){
		// summary:
		//		Selection handler for "single" mode, where only one target may be
		//		selected at a time.
		
		var ctrlKey = event.keyCode ? event.ctrlKey : event[ctrlEquiv];
		if(this._lastSelected === target){
			// Allow ctrl to toggle selection, even within single select mode.
			this.select(target, null, !ctrlKey || !this.isSelected(target));
		}else{
			this.clearSelection();
			this.select(target);
			this._lastSelected = target;
		}
	},
	
	_multipleSelectionHandler: function(event, target){
		// summary:
		//		Selection handler for "multiple" mode, where shift can be held to
		//		select ranges, ctrl/cmd can be held to toggle, and clicks/keystrokes
		//		without modifier keys will add to the current selection.
		
		var lastRow = this._lastSelected,
			ctrlKey = event.keyCode ? event.ctrlKey : event[ctrlEquiv],
			value;
		
		if(!event.shiftKey){
			// Toggle if ctrl is held; otherwise select
			value = ctrlKey ? null : true;
			lastRow = null;
		}
		this.select(target, lastRow, value);

		if(!lastRow){
			// Update reference for potential subsequent shift+select
			// (current row was already selected above)
			this._lastSelected = target;
		}
	},
	
	_extendedSelectionHandler: function(event, target){
		// summary:
		//		Selection handler for "extended" mode, which is like multiple mode
		//		except that clicks/keystrokes without modifier keys will clear
		//		the previous selection.
		
		// Clear selection first for right-clicks outside selection and non-ctrl-clicks;
		// otherwise, extended mode logic is identical to multiple mode
		if(event.button === 2 ? !this.isSelected(target) :
				!(event.keyCode ? event.ctrlKey : event[ctrlEquiv])){
			this.clearSelection(null, true);
		}
		this._multipleSelectionHandler(event, target);
	},
	
	_toggleSelectionHandler: function(event, target){
		// summary:
		//		Selection handler for "toggle" mode which simply toggles the selection
		//		of the given target.  Primarily useful for touch input.
		
		this.select(target, null, null);
	},

	_initSelectionEvents: function(){
		// summary:
		//		Performs first-time hookup of event handlers containing logic
		//		required for selection to operate.
		
		var grid = this,
			selector = this.selectionDelegate;
		
		if(has("touch")){
			// listen for touch taps if available
			on(this.contentNode, touchUtil.selector(selector, touchUtil.tap), function(evt){
				grid._handleSelect(evt, this);
			});
		}else{
			// listen for actions that should cause selections
			on(this.contentNode, on.selector(selector, this.selectionEvents), function(event){
				grid._handleSelect(event, this);
			});
		}
		
		// Also hook up spacebar (for ctrl+space)
		if(this.addKeyHandler){
			this.addKeyHandler(32, function(event){
				grid._handleSelect(event, event.target);
			});
		}
		
		// If allowSelectAll is true, bind ctrl/cmd+A to (de)select all rows,
		// unless the event was received from an editor component.
		// (Handler further checks against _allowSelectAll, which may be updated
		// if selectionMode is changed post-init.)
		if(this.allowSelectAll){
			this.on("keydown", function(event) {
				if(event[ctrlEquiv] && event.keyCode == 65 &&
						!/\bdgrid-input\b/.test(event.target.className)){
					event.preventDefault();
					grid[grid.allSelected ? "clearSelection" : "selectAll"]();
				}
			});
		}
		
		// Update aspects if there is a store change
		if(this._setStore){
			aspect.after(this, "_setStore", function(){
				grid._updateDeselectionAspect();
			});
		}
		this._updateDeselectionAspect();
	},
	
	_updateDeselectionAspect: function(){
		// summary:
		//		Hooks up logic to handle deselection of removed items.
		//		Aspects to an observable store's notify method if applicable,
		//		or to the list/grid's removeRow method otherwise.
		
		var self = this,
			store = this.store,
			beforeSignal,
			afterSignal;

		function ifSelected(object, idToUpdate, methodName){
			// Calls a method if the row corresponding to the object is selected.
			var id = idToUpdate || (object && object[self.idProperty || "id"]);
			if(id != null){
				var row = self.row(id),
					selection = row && self.selection[row.id];
				// Is the row currently in the selection list.
				if(selection){
					self[methodName](row, null, selection);
				}
			}
		}
		
		// Remove anything previously configured
		if(this._removeDeselectSignals){
			this._removeDeselectSignals();
		}

		// Is there currently an observable store?
		if(store && store.notify){
			beforeSignal = aspect.before(store, "notify", function(object, idToUpdate){
				if(!object){
					// Call deselect on the row if the object is being removed.  This allows the
					// deselect event to reference the row element while it still exists in the DOM.
					ifSelected(object, idToUpdate, "deselect");
				}
			});
			afterSignal = aspect.after(store, "notify", function(object, idToUpdate){
				// When List updates an item, the row element is removed and a new one inserted.
				// If at this point the object is still in grid.selection, then call select on the row so the
				// element's CSS is updated.  If the object was removed then the aspect-before has already deselected it.
				ifSelected(object, idToUpdate, "select");
			}, true);
			
			this._removeDeselectSignals = function(){
				beforeSignal.remove();
				afterSignal.remove();
			};
		}else{
			beforeSignal = aspect.before(this, "removeRow", function(rowElement, justCleanup){
				var row;
				if(!justCleanup){
					row = this.row(rowElement);
					// if it is a real row removal for a selected item, deselect it
					if(row && (row.id in this.selection)){
						this.deselect(row);
					}
				}
			});
			this._removeDeselectSignals = function(){
				beforeSignal.remove();
			};
		}
	},
	
	allowSelect: function(row){
		// summary:
		//		A method that can be overriden to determine whether or not a row (or 
		//		cell) can be selected. By default, all rows (or cells) are selectable.
		return true;
	},
	
	_selectionEventQueue: function(value, type){
		var grid = this,
			event = "dgrid-" + (value ? "select" : "deselect"),
			rows = this[event], // current event queue (actually cells for CellSelection)
			trigger = this._selectionTriggerEvent;
		
		if(trigger) {
			// If selection was triggered by another event, we want to know its type
			// to report later.  Grab it ahead of the timeout to avoid
			// "member not found" errors in IE < 9.
			trigger = trigger.type;
		}
		
		if(rows){ return rows; } // return existing queue, allowing to push more
		
		// Create a timeout to fire an event for the accumulated rows once everything is done.
		// We expose the callback in case the event needs to be fired immediately.
		setTimeout(this._fireSelectionEvent = function(){
			if(!rows){ return; } // rows will be set only the first time this is called
			
			var eventObject = {
				bubbles: true,
				grid: grid
			};
			if(trigger){ eventObject.parentType = trigger; }
			eventObject[type] = rows;
			on.emit(grid.contentNode, event, eventObject);
			rows = null;
			// clear the queue, so we create a new one as needed
			delete grid[event];
		}, 0);
		return (rows = this[event] = []);
	},
	select: function(row, toRow, value){
		if(value === undefined){
			// default to true
			value = true;
		} 
		if(!row.element){
			row = this.row(row);
		}
		
		// Check whether we're allowed to select the given row before proceeding.
		// If a deselect operation is being performed, this check is skipped,
		// to avoid errors when changing column definitions, and since disabled
		// rows shouldn't ever be selected anyway.
		if(value === false || this.allowSelect(row)){
			var selection = this.selection;
			var previousValue = selection[row.id];
			if(value === null){
				// indicates a toggle
				value = !previousValue;
			}
			var element = row.element;
			if(!value && !this.allSelected){
				delete this.selection[row.id];
			}else{
				selection[row.id] = value;
			}
			if(element){
				// add or remove classes as appropriate
				if(value){
					put(element, ".dgrid-selected.ui-state-active");
				}else{
					put(element, "!dgrid-selected!ui-state-active");
				}
			}
			if(value != previousValue && element){
				// add to the queue of row events
				this._selectionEventQueue(value, "rows").push(row);
			}
			
			if(toRow){
				if(!toRow.element){
					toRow = this.row(toRow);
				}
				var toElement = toRow.element;
				var fromElement = row.element;
				// find if it is earlier or later in the DOM
				var traverser = (toElement && (toElement.compareDocumentPosition ? 
					toElement.compareDocumentPosition(fromElement) == 2 :
					toElement.sourceIndex > fromElement.sourceIndex)) ? "down" : "up";
				while(row.element != toElement && (row = this[traverser](row))){
					this.select(row, null, value);
				}
			}
		}
	},
	deselect: function(row, toRow){
		this.select(row, toRow, false);
	},
	clearSelection: function(exceptId, dontResetLastSelected){
		// summary:
		//		Deselects any currently-selected items.
		// exceptId: Mixed?
		//		If specified, the given id will not be deselected.
		
		this.allSelected = false;
		for(var id in this.selection){
			if(exceptId !== id){
				this.deselect(id);
			}
		}
		if(!dontResetLastSelected){
			this._lastSelected = null;
		}
	},
	selectAll: function(){
		this.allSelected = true;
		this.selection = {}; // we do this to clear out pages from previous sorts
		for(var i in this._rowIdToObject){
			var row = this.row(this._rowIdToObject[i]);
			this.select(row.id);
		}
	},
	isSelected: function(object){
		// summary:
		//		Returns true if the indicated row is selected.
		
		if(typeof object === "undefined" || object === null){
			return false;
		}
		if(!object.element){
			object = this.row(object);
		}
		
		// First check whether the given row is indicated in the selection hash;
		// failing that, check if allSelected is true (testing against the
		// allowSelect method if possible)
		return (object.id in this.selection) ? !!this.selection[object.id] :
			this.allSelected && (!object.data || this.allowSelect(object));
	},
	
	refresh: function(){
		if(this.deselectOnRefresh){
			this.clearSelection();
			// Need to fire the selection event now because after the refresh,
			// the nodes that we will fire for will be gone.
			this._fireSelectionEvent && this._fireSelectionEvent();
		}
		this._lastSelected = null;
		return this.inherited(arguments);
	},
	
	renderArray: function(){
		var grid = this,
			rows = this.inherited(arguments);
		
		Deferred.when(rows, function(rows){
			var selection = grid.selection,
				i, row, selected;
			for(i = 0; i < rows.length; i++){
				row = grid.row(rows[i]);
				selected = row.id in selection ? selection[row.id] : grid.allSelected;
				if(selected){
					grid.select(row, null, selected);
				}
			}
		});
		return rows;
	}
});

});

},
'dgrid/CellSelection':function(){
define([
	"dojo/_base/declare",
	"dojo/aspect",
	"dojo/on",
	"dojo/has",
	"./Selection",
	"put-selector/put"
], function(declare, aspect, listen, has, Selection, put){

return declare(Selection, {
	// summary:
	//		Add cell level selection capabilities to a grid. The grid will have a selection property and
	//		fire "dgrid-select" and "dgrid-deselect" events.
	
	// ensure we don't select when an individual cell is not identifiable
	selectionDelegate: ".dgrid-cell",
	
	select: function(cell, toCell, value){
		var i, id;
		if(value === undefined){
			// default to true
			value = true;
		}
		if(typeof cell != "object" || !("element" in cell)){
			cell = this.cell(cell);
		}else if(!cell.row){
			// Row object was passed instead of cell
			if(value && typeof value === "object"){
				// value is a hash of true/false values
				for(id in value){
					this.select(this.cell(cell.id, id), null, value[id]);
				}
			}else{
				// Select/deselect all columns in row
				for(id in this.columns){
					this.select(this.cell(cell.id, id), null, value);
				}
			}
			return;
		}
		if(this.allowSelect(cell)){
			var selection = this.selection,
				rowId = cell.row.id,
				previousRow = selection[rowId];
			if(!cell.column){
				for(i in this.columns){
					this.select(this.cell(rowId, i), null, value);
				}
				return;
			}
			var previous = previousRow && previousRow[cell.column.id];
			if(value === null){
				// indicates a toggle
				value = !previous;
			}
			var element = cell.element;
			previousRow = previousRow || {};
			previousRow[cell.column.id] = value;
			this.selection[rowId] = previousRow;
			
			// Check for all-false objects to see if it can be deleted.
			// This prevents build-up of unnecessary iterations later.
			var hasSelected = false;
			for(i in previousRow){
				if(previousRow[i] === true){
					hasSelected = true;
					break;
				}
			}
			if(!hasSelected){ delete this.selection[rowId]; }
			
			if(element){
				// add or remove classes as appropriate
				if(value){
					put(element, ".dgrid-selected.ui-state-active");
				}else{
					put(element, "!dgrid-selected!ui-state-active");
				}
			}
			if(value != previous && element){
				this._selectionEventQueue(value, "cells").push(cell);
			}
			if(toCell){
				// a range
				if(!toCell.element){
					toCell = this.cell(toCell);
				}
				var toElement = toCell.element;
				var fromElement = cell.element;
				// find if it is earlier or later in the DOM
				var traverser = (toElement && (toElement.compareDocumentPosition ? 
					toElement.compareDocumentPosition(fromElement) == 2 :
					toElement.sourceIndex > fromElement.sourceIndex)) ? "nextSibling" : "previousSibling";
				// now we determine which columns are in the range 
				var idFrom = cell.column.id, idTo = toCell.column.id, started, columnIds = [];
				for(id in this.columns){
					if(started){
						columnIds.push(id);				
					}
					if(id == idFrom && (idFrom = columnIds) || // once found, we mark it off so we don't hit it again
						id == idTo && (idTo = columnIds)){
						columnIds.push(id);
						if(started || // last id, we are done 
							(idFrom == columnIds && id == idTo)){ // the ids are the same, we are done
							break;
						}
						started = true;
					}
				}
				// now we iterate over rows
				var row = cell.row, nextNode = row.element;
				toElement = toCell.row.element;
				do{
					// looping through each row..
					// and now loop through each column to be selected
					for(i = 0; i < columnIds.length; i++){
						cell = this.cell(nextNode, columnIds[i]);
						this.select(cell, null, value);
					}
					if(nextNode == toElement){
						break;
					}
				}while((nextNode = cell.row.element[traverser]));
			}
		}
	},
	isSelected: function(object, columnId){
		// summary:
		//		Returns true if the indicated cell is selected.
		
		if(typeof object === "undefined" || object === null){
			return false;
		}
		if(!object.element){
			object = this.cell(object, columnId);
		}
		
		// First check whether the given cell is indicated in the selection hash;
		// failing that, check if allSelected is true (testing against the
		// allowSelect method if possible)
		var rowId = object.row.id;
		if(rowId in this.selection){
			return !!this.selection[rowId][object.column.id];
		}else{
			return this.allSelected && (!object.row.data || this.allowSelect(object));
		}
	},
	clearSelection: function(exceptId){
		// disable exceptId in cell selection, since it would require double parameters
		exceptId = false;
		this.inherited(arguments);
	}
});
});

},
'dgrid/ColumnSet':function(){
define(["dojo/_base/kernel", "dojo/_base/declare", "dojo/_base/lang", "dojo/_base/Deferred", "dojo/on", "dojo/aspect", "dojo/query", "dojo/has", "./util/misc", "put-selector/put", "xstyle/has-class", "./Grid", "dojo/_base/sniff", "xstyle/css!./css/columnset.css"],
function(kernel, declare, lang, Deferred, listen, aspect, query, has, miscUtil, put, hasClass, Grid){
	has.add("event-mousewheel", function(global, document, element){
		return typeof element.onmousewheel !== "undefined";
	});
	has.add("event-wheel", function(global, document, element){
		var supported = false;
		// From https://developer.mozilla.org/en-US/docs/Mozilla_event_reference/wheel
		try{
			WheelEvent("wheel");
			supported = true;
		}catch(e){ // empty catch block; prevent debuggers from snagging
		}finally{
			return supported;
		}
	});

	var colsetidAttr = "data-dgrid-column-set-id";
	
	hasClass("safari", "ie-7");
	
	function adjustScrollLeft(grid, row){
		var scrollLefts = grid._columnSetScrollLefts;
		function doAdjustScrollLeft(){
			query(".dgrid-column-set", row).forEach(function(element){
				element.scrollLeft = scrollLefts[element.getAttribute(colsetidAttr)];
			});
		}
		if(has("ie") < 8 || has("quirks")){
			setTimeout(doAdjustScrollLeft, 1);
		}else{
			doAdjustScrollLeft();
		}
	}
	
	function scrollColumnSet(grid, columnSetNode, amount){
		var id = columnSetNode.getAttribute(colsetidAttr),
			scroller = grid._columnSetScrollers[id],
			scrollLeft = scroller.scrollLeft + amount;

		scroller.scrollLeft = scrollLeft < 0 ? 0 : scrollLeft;
	}
	
	var horizMouseWheel;
	if(!has("touch")){
		horizMouseWheel = has("event-mousewheel") || has("event-wheel") ? function(grid){
			return function(target, listener){
				return listen(target, has("event-wheel") ? "wheel" : "mousewheel", function(event){
					var node = event.target, deltaX;
					// WebKit will invoke mousewheel handlers with an event target of a text
					// node; check target and if it's not an element node, start one node higher
					// in the tree
					if(node.nodeType !== 1){
						node = node.parentNode;
					}
					while(!query.matches(node, ".dgrid-column-set[" + colsetidAttr + "]", target)){
						if(node === target || !(node = node.parentNode)){
							return;
						}
					}
					
					// Normalize reported delta value:
					// wheelDeltaX (webkit, mousewheel) needs to be negated and divided by 3
					// deltaX (FF17+, wheel) can be used exactly as-is
					deltaX = event.deltaX || -event.wheelDeltaX / 3;
					if(deltaX){
						// only respond to horizontal movement
						listener.call(null, grid, node, deltaX);
					}
				});
			};
		} : function(grid){
			return function(target, listener){
				return listen(target, ".dgrid-column-set[" + colsetidAttr + "]:MozMousePixelScroll", function(event){
					if(event.axis === 1){
						// only respond to horizontal movement
						listener.call(null, grid, this, event.detail);
					}
				});
			};
		};
	}
	
	return declare(null, {
		// summary:
		//		Provides column sets to isolate horizontal scroll of sets of 
		//		columns from each other. This mainly serves the purpose of allowing for
		//		column locking.
		
		postCreate: function(){
			this.inherited(arguments);
			
			if(!has("touch")){
				this.on(horizMouseWheel(this), function(grid, colsetNode, amount){
					var id = colsetNode.getAttribute(colsetidAttr),
						scroller = grid._columnSetScrollers[id],
						scrollLeft = scroller.scrollLeft + amount;
					
					scroller.scrollLeft = scrollLeft < 0 ? 0 : scrollLeft;
				});
			}
		},
		columnSets: [],
		createRowCells: function(tag, each){
			var row = put("table.dgrid-row-table");
			var tr = put(row, "tbody tr");
			for(var i = 0, l = this.columnSets.length; i < l; i++){
				// iterate through the columnSets
				var cell = put(tr, tag + ".dgrid-column-set-cell.dgrid-column-set-" + i +
					" div.dgrid-column-set[" + colsetidAttr + "=" + i + "]");
				cell.appendChild(this.inherited(arguments, [tag, each, this.columnSets[i]]));
			}
			return row;
		},
		renderArray: function(){
			var grid = this,
				rows = this.inherited(arguments);

			Deferred.when(rows, function(rows){
				for(var i = 0; i < rows.length; i++){
					adjustScrollLeft(grid, rows[i]);
				}
			});
			return rows;
		},

		renderHeader: function(){
			// summary:
			//		Setup the headers for the grid
			this.inherited(arguments);
			
			var columnSets = this.columnSets,
				domNode = this.domNode,
				scrollers = this._columnSetScrollers,
				scrollerContents = this._columnSetScrollerContents = {},
				scrollLefts = this._columnSetScrollLefts = {},
				grid = this,
				i, l;
			
			function reposition(){
				grid._positionScrollers();
			}
			
			if (scrollers) {
				// this isn't the first time; destroy existing scroller nodes first
				for(i in scrollers){
					put(scrollers[i], "!");
				}
			} else {
				// first-time-only operations: hook up event/aspected handlers
				aspect.after(this, "resize", reposition, true);
				aspect.after(this, "styleColumn", reposition, true);
				listen(domNode, ".dgrid-column-set:dgrid-cellfocusin", lang.hitch(this, '_onColumnSetScroll'));
			}
			
			// reset to new object to be populated in loop below
			scrollers = this._columnSetScrollers = {};
			
			for(i = 0, l = columnSets.length; i < l; i++){
				this._putScroller(columnSets[i], i);
			}
			
			this._positionScrollers();
		},
		
		styleColumnSet: function(colsetId, css){
			// summary:
			//		Dynamically creates a stylesheet rule to alter a columnset's style.
			
			var rule = this.addCssRule("#" + miscUtil.escapeCssIdentifier(this.domNode.id) + " .dgrid-column-set-" + colsetId, css);
			this._positionScrollers();
			return rule;
		},
		
		_destroyColumns: function(){
			var columnSetsLength = this.columnSets.length,
				i, j, k, subRowsLength, len, columnSet, subRow, column;
			for(i = 0; i < columnSetsLength; i++){
				columnSet = this.columnSets[i];
				for(j = 0, subRowsLength = columnSet.length; j < subRowsLength; j++){
					subRow = columnSet[j];
					for(k = 0, len = subRow.length; k < len; k++){
						column = subRow[k];
						if(typeof column.destroy === "function"){ column.destroy(); }
					}
				}
			}
			this.inherited(arguments);
		},
		configStructure: function(){
			this.columns = {};
			for(var i = 0, l = this.columnSets.length; i < l; i++){
				// iterate through the columnSets
				var columnSet = this.columnSets[i];
				for(var j = 0; j < columnSet.length; j++){
					columnSet[j] = this._configColumns(i + "-" + j + "-", columnSet[j]);
				}
			}
		},

		_positionScrollers: function (){
			var domNode = this.domNode,
				scrollers = this._columnSetScrollers,
				scrollerContents = this._columnSetScrollerContents,
				columnSets = this.columnSets,
				left = 0,
				scrollerWidth = 0,
				numScrollers = 0, // tracks number of visible scrollers (sets w/ overflow)
				i, l, columnSetElement, contentWidth;
			
			for(i = 0, l = columnSets.length; i < l; i++){
				// iterate through the columnSets
				left += scrollerWidth;
				columnSetElement = query('.dgrid-column-set[' + colsetidAttr + '="' + i +'"]', domNode)[0];
				scrollerWidth = columnSetElement.offsetWidth;
				contentWidth = columnSetElement.firstChild.offsetWidth;
				scrollerContents[i].style.width = contentWidth + "px";
				scrollers[i].style.width = scrollerWidth + "px";
				scrollers[i].style.bottom = this.showFooter ? this.footerNode.offsetHeight + "px" : "0px";
				// IE seems to need scroll to be set explicitly
				scrollers[i].style.overflowX = contentWidth > scrollerWidth ? "scroll" : "auto";
				scrollers[i].style.left = left + "px";
				// Keep track of how many scrollbars we're showing
				if(contentWidth > scrollerWidth){ numScrollers++; }
			}
			
			// Align bottom of body node depending on whether there are scrollbars
			this.bodyNode.style.bottom = numScrollers ?
				(has("dom-scrollbar-height") + (has("ie") ? 1 : 0) + "px") :
				"0";
		},

		_putScroller: function (columnSet, i){
			// function called for each columnSet
			var scroller = this._columnSetScrollers[i] =
				put(this.domNode, "div.dgrid-column-set-scroller.dgrid-column-set-scroller-" + i +
					"[" + colsetidAttr + "=" + i +"]");
			this._columnSetScrollerContents[i] = put(scroller, "div.dgrid-column-set-scroller-content");
			listen(scroller, "scroll", lang.hitch(this, '_onColumnSetScroll'));
		},

		_onColumnSetScroll: function (evt){
			var scrollLeft = evt.target.scrollLeft,
				colSetId = evt.target.getAttribute(colsetidAttr),
				newScrollLeft;

			if(this._columnSetScrollLefts[colSetId] != scrollLeft){
				query('.dgrid-column-set[' + colsetidAttr + '="' + colSetId + '"],.dgrid-column-set-scroller[' + colsetidAttr + '="' + colSetId + '"]', this.domNode).
					forEach(function(element, i){
						element.scrollLeft = scrollLeft;
						if(!i){
							// Compute newScrollLeft based on actual resulting
							// value of scrollLeft, which may be different than
							// what we assigned under certain circumstances
							// (e.g. Chrome under 33% / 67% / 90% zoom).
							// Only need to compute this once, as it will be the
							// same for every row.
							newScrollLeft = element.scrollLeft;
						}
					});
				this._columnSetScrollLefts[colSetId] = newScrollLeft;
			}
		},
		
		_setColumnSets: function(columnSets){
			this._destroyColumns();
			this.columnSets = columnSets;
			this._updateColumns();
		},
		setColumnSets: function(columnSets){
			kernel.deprecated("setColumnSets(...)", 'use set("columnSets", ...) instead', "dgrid 0.4");
			this.set("columnSets", columnSets);
		}
	});
});

},
'dgrid/Keyboard':function(){
define([
	"dojo/_base/declare",
	"dojo/aspect",
	"dojo/on",
	"dojo/_base/lang",
	"dojo/has",
	"put-selector/put",
	"dojo/_base/Deferred",
	"dojo/_base/sniff"
], function(declare, aspect, on, lang, has, put, Deferred){

var delegatingInputTypes = {
		checkbox: 1,
		radio: 1,
		button: 1
	},
	hasGridCellClass = /\bdgrid-cell\b/,
	hasGridRowClass = /\bdgrid-row\b/;

has.add("dom-contains", function(global, doc, element){
	return !!element.contains; // not supported by FF < 9
});

function contains(parent, node){
	// summary:
	//		Checks to see if an element is contained by another element.
	
	if(has("dom-contains")){
		return parent.contains(node);
	}else{
		return parent.compareDocumentPosition(node) & 8 /* DOCUMENT_POSITION_CONTAINS */;
	}
}

var Keyboard = declare(null, {
	// summary:
	//		Adds keyboard navigation capability to a list or grid.
	
	// pageSkip: Number
	//		Number of rows to jump by when page up or page down is pressed.
	pageSkip: 10,
	
	tabIndex: 0,
	
	// keyMap: Object
	//		Hash which maps key codes to functions to be executed (in the context
	//		of the instance) for key events within the grid's body.
	keyMap: null,
	
	// headerKeyMap: Object
	//		Hash which maps key codes to functions to be executed (in the context
	//		of the instance) for key events within the grid's header row.
	headerKeyMap: null,
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		if(!this.keyMap){
			this.keyMap = lang.mixin({}, Keyboard.defaultKeyMap);
		}
		if(!this.headerKeyMap){
			this.headerKeyMap = lang.mixin({}, Keyboard.defaultHeaderKeyMap);
		}
	},
	
	postCreate: function(){
		this.inherited(arguments);
		var grid = this;
		
		function handledEvent(event){
			// text boxes and other inputs that can use direction keys should be ignored and not affect cell/row navigation
			var target = event.target;
			return target.type && (!delegatingInputTypes[target.type] || event.keyCode == 32);
		}
		
		function enableNavigation(areaNode){
			var cellNavigation = grid.cellNavigation,
				isFocusableClass = cellNavigation ? hasGridCellClass : hasGridRowClass,
				isHeader = areaNode === grid.headerNode,
				initialNode = areaNode;
			
			function initHeader(){
				grid._focusedHeaderNode = initialNode =
					cellNavigation ? grid.headerNode.getElementsByTagName("th")[0] : grid.headerNode;
				if(initialNode){ initialNode.tabIndex = grid.tabIndex; }
			}
			
			if(isHeader){
				// Initialize header now (since it's already been rendered),
				// and aspect after future renderHeader calls to reset focus.
				initHeader();
				aspect.after(grid, "renderHeader", initHeader, true);
			}else{
				aspect.after(grid, "renderArray", function(ret){
					// summary:
					//		Ensures the first element of a grid is always keyboard selectable after data has been
					//		retrieved if there is not already a valid focused element.
					
					return Deferred.when(ret, function(ret){
						var focusedNode = grid._focusedNode || initialNode;
						
						// do not update the focused element if we already have a valid one
						if(isFocusableClass.test(focusedNode.className) && contains(areaNode, focusedNode)){
							return ret;
						}
						
						// ensure that the focused element is actually a grid cell, not a
						// dgrid-preload or dgrid-content element, which should not be focusable,
						// even when data is loaded asynchronously
						for(var i = 0, elements = areaNode.getElementsByTagName("*"), element; (element = elements[i]); ++i){
							if(isFocusableClass.test(element.className)){
								focusedNode = grid._focusedNode = element;
								break;
							}
						}
						
						focusedNode.tabIndex = grid.tabIndex;
						return ret;
					});
				});
			}
			
			grid._listeners.push(on(areaNode, "mousedown", function(event){
				if(!handledEvent(event)){
					grid._focusOnNode(event.target, isHeader, event);
				}
			}));
			
			grid._listeners.push(on(areaNode, "keydown", function(event){
				// For now, don't squash browser-specific functionalities by letting
				// ALT and META function as they would natively
				if(event.metaKey || event.altKey) {
					return;
				}
				
				var handler = grid[isHeader ? "headerKeyMap" : "keyMap"][event.keyCode];
				
				// Text boxes and other inputs that can use direction keys should be ignored and not affect cell/row navigation
				if(handler && !handledEvent(event)){
					handler.call(grid, event);
				}
			}));
		}
		
		if(this.tabableHeader){
			enableNavigation(this.headerNode);
			on(this.headerNode, "dgrid-cellfocusin", function(){
				grid.scrollTo({ x: this.scrollLeft });
			});
		}
		enableNavigation(this.contentNode);
	},
	
	removeRow: function(rowElement){
		if(!this._focusedNode){
			// Nothing special to do if we have no record of anything focused
			return this.inherited(arguments);
		}
		
		var self = this,
			isActive = document.activeElement === this._focusedNode,
			focusedTarget = this[this.cellNavigation ? "cell" : "row"](this._focusedNode),
			focusedRow = focusedTarget.row || focusedTarget,
			sibling;
		rowElement = rowElement.element || rowElement;
		
		// If removed row previously had focus, temporarily store information
		// to be handled in an immediately-following insertRow call, or next turn
		if(rowElement === focusedRow.element){
			sibling = this.down(focusedRow, true);
			
			// Check whether down call returned the same row, or failed to return
			// any (e.g. during a partial unrendering)
			if (!sibling || sibling.element === rowElement) {
				sibling = this.up(focusedRow, true);
			}
			
			this._removedFocus = {
				active: isActive,
				rowId: focusedRow.id,
				columnId: focusedTarget.column && focusedTarget.column.id,
				siblingId: !sibling || sibling.element === rowElement ? undefined : sibling.id
			};
			
			// Call _restoreFocus on next turn, to restore focus to sibling
			// if no replacement row was immediately inserted.
			// Pass original row's id in case it was re-inserted in a renderArray
			// call (and thus was found, but couldn't be focused immediately)
			setTimeout(function() {
				if(self._removedFocus){
					self._restoreFocus(focusedRow.id);
				}
			}, 0);
			
			// Clear _focusedNode until _restoreFocus is called, to avoid
			// needlessly re-running this logic
			this._focusedNode = null;
		}
		
		this.inherited(arguments);
	},
	
	insertRow: function(object){
		var rowElement = this.inherited(arguments);
		if(this._removedFocus && !this._removedFocus.wait){
			this._restoreFocus(rowElement);
		}
		return rowElement;
	},
	
	_restoreFocus: function(row) {
		// summary:
		//		Restores focus to the newly inserted row if it matches the
		//		previously removed row, or to the nearest sibling otherwise.
		
		var focusInfo = this._removedFocus,
			newTarget;
		
		row = row && this.row(row);
		newTarget = row && row.element && row.id === focusInfo.rowId ? row :
			typeof focusInfo.siblingId !== "undefined" && this.row(focusInfo.siblingId);
		
		if(newTarget && newTarget.element){
			if(!newTarget.element.parentNode.parentNode){
				// This was called from renderArray, so the row hasn't
				// actually been placed in the DOM yet; handle it on the next
				// turn (called from removeRow).
				focusInfo.wait = true;
				return;
			}
			newTarget = typeof focusInfo.columnId !== "undefined" ?
				this.cell(newTarget, focusInfo.columnId) : newTarget;
			if(focusInfo.active){
				// Row/cell was previously focused, so focus the new one immediately
				this.focus(newTarget);
			}else{
				// Row/cell was not focused, but we still need to update tabIndex
				// and the element's class to be consistent with the old one
				put(newTarget.element, ".dgrid-focus");
				newTarget.element.tabIndex = this.tabIndex;
			}
		}
		
		delete this._removedFocus;
	},
	
	addKeyHandler: function(key, callback, isHeader){
		// summary:
		//		Adds a handler to the keyMap on the instance.
		//		Supports binding additional handlers to already-mapped keys.
		// key: Number
		//		Key code representing the key to be handled.
		// callback: Function
		//		Callback to be executed (in instance context) when the key is pressed.
		// isHeader: Boolean
		//		Whether the handler is to be added for the grid body (false, default)
		//		or the header (true).
		
		// Aspects may be about 10% slower than using an array-based appraoch,
		// but there is significantly less code involved (here and above).
		return aspect.after( // Handle
			this[isHeader ? "headerKeyMap" : "keyMap"], key, callback, true);
	},
	
	_focusOnNode: function(element, isHeader, event){
		var focusedNodeProperty = "_focused" + (isHeader ? "Header" : "") + "Node",
			focusedNode = this[focusedNodeProperty],
			cellOrRowType = this.cellNavigation ? "cell" : "row",
			cell = this[cellOrRowType](element),
			inputs,
			input,
			numInputs,
			inputFocused,
			i;
		
		element = cell && cell.element;
		if(!element){ return; }
		
		if(this.cellNavigation){
			inputs = element.getElementsByTagName("input");
			for(i = 0, numInputs = inputs.length; i < numInputs; i++){
				input = inputs[i];
				if((input.tabIndex != -1 || "lastValue" in input) && !input.disabled){
					// Employ workaround for focus rectangle in IE < 8
					if(has("ie") < 8){ input.style.position = "relative"; }
					input.focus();
					if(has("ie") < 8){ input.style.position = ""; }
					inputFocused = true;
					break;
				}
			}
		}
		
		event = lang.mixin({ grid: this }, event);
		if(event.type){
			event.parentType = event.type;
		}
		if(!event.bubbles){
			// IE doesn't always have a bubbles property already true.
			// Opera throws if you try to set it to true if it is already true.
			event.bubbles = true;
		}
		if(focusedNode){
			// Clean up previously-focused element
			// Remove the class name and the tabIndex attribute
			put(focusedNode, "!dgrid-focus[!tabIndex]");
			if(has("ie") < 8){
				// Clean up after workaround below (for non-input cases)
				focusedNode.style.position = "";
			}
			
			// Expose object representing focused cell or row losing focus, via
			// event.cell or event.row; which is set depends on cellNavigation.
			event[cellOrRowType] = this[cellOrRowType](focusedNode);
			on.emit(element, "dgrid-cellfocusout", event);
		}
		focusedNode = this[focusedNodeProperty] = element;
		
		// Expose object representing focused cell or row gaining focus, via
		// event.cell or event.row; which is set depends on cellNavigation.
		// Note that yes, the same event object is being reused; on.emit
		// performs a shallow copy of properties into a new event object.
		event[cellOrRowType] = cell;
		
		if(!inputFocused){
			if(has("ie") < 8){
				// setting the position to relative magically makes the outline
				// work properly for focusing later on with old IE.
				// (can't be done a priori with CSS or screws up the entire table)
				element.style.position = "relative";
			}
			element.tabIndex = this.tabIndex;
			element.focus();
		}
		put(element, ".dgrid-focus");
		on.emit(focusedNode, "dgrid-cellfocusin", event);
	},
	
	focusHeader: function(element){
		this._focusOnNode(element || this._focusedHeaderNode, true);
	},
	
	focus: function(element){
		this._focusOnNode(element || this._focusedNode, false);
	}
});

// Common functions used in default keyMap (called in instance context)

var moveFocusVertical = Keyboard.moveFocusVertical = function(event, steps){
	var cellNavigation = this.cellNavigation,
		target = this[cellNavigation ? "cell" : "row"](event),
		columnId = cellNavigation && target.column.id,
		next = this.down(this._focusedNode, steps, true);
	
	// Navigate within same column if cell navigation is enabled
	if(cellNavigation){ next = this.cell(next, columnId); }
	this._focusOnNode(next, false, event);
	
	event.preventDefault();
};

var moveFocusUp = Keyboard.moveFocusUp = function(event){
	moveFocusVertical.call(this, event, -1);
};

var moveFocusDown = Keyboard.moveFocusDown = function(event){
	moveFocusVertical.call(this, event, 1);
};

var moveFocusPageUp = Keyboard.moveFocusPageUp = function(event){
	moveFocusVertical.call(this, event, -this.pageSkip);
};

var moveFocusPageDown = Keyboard.moveFocusPageDown = function(event){
	moveFocusVertical.call(this, event, this.pageSkip);
};

var moveFocusHorizontal = Keyboard.moveFocusHorizontal = function(event, steps){
	if(!this.cellNavigation){ return; }
	var isHeader = !this.row(event), // header reports row as undefined
		currentNode = this["_focused" + (isHeader ? "Header" : "") + "Node"];
	
	this._focusOnNode(this.right(currentNode, steps), isHeader, event);
	event.preventDefault();
};

var moveFocusLeft = Keyboard.moveFocusLeft = function(event){
	moveFocusHorizontal.call(this, event, -1);
};

var moveFocusRight = Keyboard.moveFocusRight = function(event){
	moveFocusHorizontal.call(this, event, 1);
};

var moveHeaderFocusEnd = Keyboard.moveHeaderFocusEnd = function(event, scrollToBeginning){
	// Header case is always simple, since all rows/cells are present
	var nodes;
	if(this.cellNavigation){
		nodes = this.headerNode.getElementsByTagName("th");
		this._focusOnNode(nodes[scrollToBeginning ? 0 : nodes.length - 1], true, event);
	}
	// In row-navigation mode, there's nothing to do - only one row in header
	
	// Prevent browser from scrolling entire page
	event.preventDefault();
};

var moveHeaderFocusHome = Keyboard.moveHeaderFocusHome = function(event){
	moveHeaderFocusEnd.call(this, event, true);
};

var moveFocusEnd = Keyboard.moveFocusEnd = function(event, scrollToTop){
	// summary:
	//		Handles requests to scroll to the beginning or end of the grid.
	
	// Assume scrolling to top unless event is specifically for End key
	var self = this,
		cellNavigation = this.cellNavigation,
		contentNode = this.contentNode,
		contentPos = scrollToTop ? 0 : contentNode.scrollHeight,
		scrollPos = contentNode.scrollTop + contentPos,
		endChild = contentNode[scrollToTop ? "firstChild" : "lastChild"],
		hasPreload = endChild.className.indexOf("dgrid-preload") > -1,
		endTarget = hasPreload ? endChild[(scrollToTop ? "next" : "previous") + "Sibling"] : endChild,
		endPos = endTarget.offsetTop + (scrollToTop ? 0 : endTarget.offsetHeight),
		handle;
	
	if(hasPreload){
		// Find the nearest dgrid-row to the relevant end of the grid
		while(endTarget && endTarget.className.indexOf("dgrid-row") < 0){
			endTarget = endTarget[(scrollToTop ? "next" : "previous") + "Sibling"];
		}
		// If none is found, there are no rows, and nothing to navigate
		if(!endTarget){ return; }
	}
	
	// Grid content may be lazy-loaded, so check if content needs to be
	// loaded first
	if(!hasPreload || endChild.offsetHeight < 1){
		// End row is loaded; focus the first/last row/cell now
		if(cellNavigation){
			// Preserve column that was currently focused
			endTarget = this.cell(endTarget, this.cell(event).column.id);
		}
		this._focusOnNode(endTarget, false, event);
	}else{
		// In IE < 9, the event member references will become invalid by the time
		// _focusOnNode is called, so make a (shallow) copy up-front
		if(!has("dom-addeventlistener")){
			event = lang.mixin({}, event);
		}
		
		// If the topmost/bottommost row rendered doesn't reach the top/bottom of
		// the contentNode, we are using OnDemandList and need to wait for more
		// data to render, then focus the first/last row in the new content.
		handle = aspect.after(this, "renderArray", function(rows){
			handle.remove();
			return Deferred.when(rows, function(rows){
				var target = rows[scrollToTop ? 0 : rows.length - 1];
				if(cellNavigation){
					// Preserve column that was currently focused
					target = self.cell(target, self.cell(event).column.id);
				}
				self._focusOnNode(target, false, event);
			});
		});
	}
	
	if(scrollPos === endPos){
		// Grid body is already scrolled to end; prevent browser from scrolling
		// entire page instead
		event.preventDefault();
	}
};

var moveFocusHome = Keyboard.moveFocusHome = function(event){
	moveFocusEnd.call(this, event, true);
};

function preventDefault(event){
	event.preventDefault();
}

Keyboard.defaultKeyMap = {
	32: preventDefault, // space
	33: moveFocusPageUp, // page up
	34: moveFocusPageDown, // page down
	35: moveFocusEnd, // end
	36: moveFocusHome, // home
	37: moveFocusLeft, // left
	38: moveFocusUp, // up
	39: moveFocusRight, // right
	40: moveFocusDown // down
};

// Header needs fewer default bindings (no vertical), so bind it separately
Keyboard.defaultHeaderKeyMap = {
	32: preventDefault, // space
	35: moveHeaderFocusEnd, // end
	36: moveHeaderFocusHome, // home
	37: moveFocusLeft, // left
	39: moveFocusRight // right
};

return Keyboard;
});
},
'dgrid/OnDemandGrid':function(){
define(["dojo/_base/declare", "./Grid", "./OnDemandList"], function(declare, Grid, OnDemandList){
	return declare([Grid, OnDemandList], {});
});
},
'dgrid/OnDemandList':function(){
define(["./List", "./_StoreMixin", "dojo/_base/declare", "dojo/_base/lang", "dojo/_base/Deferred", "dojo/on", "./util/misc", "put-selector/put"],
function(List, _StoreMixin, declare, lang, Deferred, listen, miscUtil, put){

return declare([List, _StoreMixin], {
	// summary:
	//		Extends List to include virtual scrolling functionality, querying a
	//		dojo/store instance for the appropriate range when the user scrolls.
	
	// minRowsPerPage: Integer
	//		The minimum number of rows to request at one time.
	minRowsPerPage: 25,
	
	// maxRowsPerPage: Integer
	//		The maximum number of rows to request at one time.
	maxRowsPerPage: 250,
	
	// maxEmptySpace: Integer
	//		Defines the maximum size (in pixels) of unrendered space below the
	//		currently-rendered rows. Setting this to less than Infinity can be useful if you
	//		wish to limit the initial vertical scrolling of the grid so that the scrolling is
	// 		not excessively sensitive. With very large grids of data this may make scrolling
	//		easier to use, albiet it can limit the ability to instantly scroll to the end.
	maxEmptySpace: Infinity,	
	
	// bufferRows: Integer
	//	  The number of rows to keep ready on each side of the viewport area so that the user can
	//	  perform local scrolling without seeing the grid being built. Increasing this number can
	//	  improve perceived performance when the data is being retrieved over a slow network.
	bufferRows: 10,
	
	// farOffRemoval: Integer
	//		Defines the minimum distance (in pixels) from the visible viewport area
	//		rows must be in order to be removed.  Setting to Infinity causes rows
	//		to never be removed.
	farOffRemoval: 2000,
	
	// queryRowsOverlap: Integer
	//		Indicates the number of rows to overlap queries. This helps keep
	//		continuous data when underlying data changes (and thus pages don't
	//		exactly align)
	queryRowsOverlap: 1,
	
	// pagingMethod: String
	//		Method (from dgrid/util/misc) to use to either throttle or debounce
	//		requests.  Default is "debounce" which will cause the grid to wait until
	//		the user pauses scrolling before firing any requests; can be set to
	//		"throttleDelayed" instead to progressively request as the user scrolls,
	//		which generally incurs more overhead but might appear more responsive.
	pagingMethod: "debounce",
	
	// pagingDelay: Integer
	//		Indicates the delay (in milliseconds) imposed upon pagingMethod, to wait
	//		before paging in more data on scroll events. This can be increased to
	//		reduce client-side overhead or the number of requests sent to a server.
	pagingDelay: miscUtil.defaultDelay,
	
	// keepScrollPosition: Boolean
	//		When refreshing the list, controls whether the scroll position is
	//		preserved, or reset to the top.  This can also be overridden for
	//		specific calls to refresh.
	keepScrollPosition: false,
	
	rowHeight: 22,
	
	postCreate: function(){
		this.inherited(arguments);
		var self = this;
		// check visibility on scroll events
		listen(this.bodyNode, "scroll",
			miscUtil[this.pagingMethod](function(event){ self._processScroll(event); },
				null, this.pagingDelay));
	},
	
	renderQuery: function(query, preloadNode, options){
		// summary:
		//		Creates a preload node for rendering a query into, and executes the query
		//		for the first page of data. Subsequent data will be downloaded as it comes
		//		into view.
		var self = this,
			preload = {
				query: query,
				count: 0,
				node: preloadNode,
				options: options
			},
			priorPreload = this.preload,
			results;
		
		if(!preloadNode){
			// Initial query; set up top and bottom preload nodes
			var topPreload = {
				node: put(this.contentNode, "div.dgrid-preload", {
					rowIndex: 0
				}),
				count: 0,
				query: query,
				next: preload,
				options: options
			};
			topPreload.node.style.height = "0";
			preload.node = preloadNode = put(this.contentNode, "div.dgrid-preload");
			preload.previous = topPreload;
		}
		// this preload node is used to represent the area of the grid that hasn't been
		// downloaded yet
		preloadNode.rowIndex = this.minRowsPerPage;

		if(priorPreload){
			// the preload nodes (if there are multiple) are represented as a linked list, need to insert it
			if((preload.next = priorPreload.next) && 
					// check to make sure that the current scroll position is below this preload
					this.bodyNode.scrollTop >= priorPreload.node.offsetTop){ 
				// the prior preload is above/before in the linked list
				preload.previous = priorPreload;
			}else{
				// the prior preload is below/after in the linked list
				preload.next = priorPreload;
				preload.previous = priorPreload.previous;
			}
			// adjust the previous and next links so the linked list is proper
			preload.previous.next = preload;
			preload.next.previous = preload; 
		}else{
			this.preload = preload;
		}
		
		var loadingNode = put(preloadNode, "-div.dgrid-loading"),
			innerNode = put(loadingNode, "div.dgrid-below");
		innerNode.innerHTML = this.loadingMessage;

		function errback(err) {
			// Used as errback for when calls;
			// remove the loadingNode and re-throw if an error was passed
			put(loadingNode, "!");
			
			if(err){
				if(self._refreshDeferred){
					self._refreshDeferred.reject(err);
					delete self._refreshDeferred;
				}
				throw err;
			}
		}

		// Establish query options, mixing in our own.
		// (The getter returns a delegated object, so simply using mixin is safe.)
		options = lang.mixin(this.get("queryOptions"), options, 
			{ start: 0, count: this.minRowsPerPage },
			"level" in query ? { queryLevel: query.level } : null);
		
		// Protect the query within a _trackError call, but return the QueryResults
		this._trackError(function(){ return results = query(options); });
		
		if(typeof results === "undefined"){
			// Synchronous error occurred (but was caught by _trackError)
			errback();
			return;
		}
		
		// Render the result set
		Deferred.when(self.renderArray(results, preloadNode, options), function(trs){
			var total = typeof results.total === "undefined" ?
				results.length : results.total;
			return Deferred.when(total, function(total){
				var trCount = trs.length,
					parentNode = preloadNode.parentNode,
					noDataNode = self.noDataNode;
				
				put(loadingNode, "!");
				if(!("queryLevel" in options)){
					self._total = total;
				}
				// now we need to adjust the height and total count based on the first result set
				if(total === 0){
					if(noDataNode){
						put(noDataNode, "!");
						delete self.noDataNode;
					}
					self.noDataNode = noDataNode = put("div.dgrid-no-data");
					parentNode.insertBefore(noDataNode, self._getFirstRowSibling(parentNode));
					noDataNode.innerHTML = self.noDataMessage;
				}
				var height = 0;
				for(var i = 0; i < trCount; i++){
					height += self._calcRowHeight(trs[i]);
				}
				// only update rowHeight if we actually got results and are visible
				if(trCount && height){ self.rowHeight = height / trCount; }
				
				total -= trCount;
				preload.count = total;
				preloadNode.rowIndex = trCount;
				if(total){
					preloadNode.style.height = Math.min(total * self.rowHeight, self.maxEmptySpace) + "px";
				}else{
					// if total is 0, IE quirks mode can't handle 0px height for some reason, I don't know why, but we are setting display: none for now
					preloadNode.style.display = "none";
				}
				
				if (self._previousScrollPosition) {
					// Restore position after a refresh operation w/ keepScrollPosition
					self.scrollTo(self._previousScrollPosition);
					delete self._previousScrollPosition;
				}
				
				// Redo scroll processing in case the query didn't fill the screen,
				// or in case scroll position was restored
				self._processScroll();
				
				// If _refreshDeferred is still defined after calling _processScroll,
				// resolve it now (_processScroll will remove it and resolve it itself
				// otherwise)
				if(self._refreshDeferred){
					self._refreshDeferred.resolve(results);
					delete self._refreshDeferred;
				}
				
				return trs;
			}, errback);
		}, errback);
		
		return results;
	},
	
	refresh: function(options){
		// summary:
		//		Refreshes the contents of the grid.
		// options: Object?
		//		Optional object, supporting the following parameters:
		//		* keepScrollPosition: like the keepScrollPosition instance property;
		//			specifying it in the options here will override the instance
		//			property's value for this specific refresh call only.
		
		var self = this,
			keep = (options && options.keepScrollPosition),
			dfd, results;
		
		// Fall back to instance property if option is not defined
		if(typeof keep === "undefined"){ keep = this.keepScrollPosition; }
		
		// Store scroll position to be restored after new total is received
		if(keep){ this._previousScrollPosition = this.getScrollPosition(); }
		
		this.inherited(arguments);
		if(this.store){
			// render the query
			dfd = this._refreshDeferred = new Deferred();
			
			// renderQuery calls _trackError internally
			results = self.renderQuery(function(queryOptions){
				return self.store.query(self.query, queryOptions);
			});
			if(typeof results === "undefined"){
				// Synchronous error occurred; reject the refresh promise.
				dfd.reject();
			}
			
			// Internally, _refreshDeferred will always be resolved with an object
			// containing `results` (QueryResults) and `rows` (the rendered rows);
			// externally the promise will resolve simply with the QueryResults, but
			// the event will be emitted with both under respective properties.
			return dfd.then(function(results){
				// Emit on a separate turn to enable event to be used consistently for
				// initial render, regardless of whether the backing store is async
				setTimeout(function() {
					listen.emit(self.domNode, "dgrid-refresh-complete", {
						bubbles: true,
						cancelable: false,
						grid: self,
						results: results // QueryResults object (may be a wrapped promise)
					});
				}, 0);
				
				// Delete the Deferred immediately so nothing tries to re-resolve
				delete self._refreshDeferred;
				
				// Resolve externally with just the QueryResults
				return results;
			}, function(err){
				delete self._refreshDeferred;
				throw err;
			});
		}
	},
	
	resize: function(){
		this.inherited(arguments);
		this._processScroll();
	},

	_getFirstRowSibling: function(container){
		// summary:
		//		Returns the DOM node that a new row should be inserted before
		//		when there are no other rows in the current result set.
		//		In the case of OnDemandList, this will always be the last child
		//		of the container (which will be a trailing preload node).
		return container.lastChild;
	},
	
	_calcRowHeight: function(rowElement){
		// summary:
		//		Calculate the height of a row. This is a method so it can be overriden for
		//		plugins that add connected elements to a row, like the tree
		
		var sibling = rowElement.previousSibling;
		sibling = sibling && !/\bdgrid-preload\b/.test(sibling.className) && sibling;
		
		// If a previous row exists, compare the top of this row with the
		// previous one (in case "rows" are actually rendering side-by-side).
		// If no previous row exists, this is either the first or only row,
		// in which case we count its own height.
		return sibling ? rowElement.offsetTop - sibling.offsetTop :
			rowElement.offsetHeight;
	},
	
	lastScrollTop: 0,
	_processScroll: function(evt){
		// summary:
		//		Checks to make sure that everything in the viewable area has been
		//		downloaded, and triggering a request for the necessary data when needed.
		var grid = this,
			scrollNode = grid.bodyNode,
			// grab current visible top from event if provided, otherwise from node
			visibleTop = (evt && evt.scrollTop) || this.getScrollPosition().y,
			visibleBottom = scrollNode.offsetHeight + visibleTop,
			priorPreload, preloadNode, preload = grid.preload,
			lastScrollTop = grid.lastScrollTop,
			requestBuffer = grid.bufferRows * grid.rowHeight,
			searchBuffer = requestBuffer - grid.rowHeight, // Avoid rounding causing multiple queries
			// References related to emitting dgrid-refresh-complete if applicable
			refreshDfd,
			lastResults,
			lastRows,
			preloadSearchNext = true;
		
		// XXX: I do not know why this happens.
		// munging the actual location of the viewport relative to the preload node by a few pixels in either
		// direction is necessary because at least WebKit on Windows seems to have an error that causes it to
		// not quite get the entire element being focused in the viewport during keyboard navigation,
		// which means it becomes impossible to load more data using keyboard navigation because there is
		// no more data to scroll to to trigger the fetch.
		// 1 is arbitrary and just gets it to work correctly with our current test cases; don’t wanna go
		// crazy and set it to a big number without understanding more about what is going on.
		// wondering if it has to do with border-box or something, but changing the border widths does not
		// seem to make it break more or less, so I do not know…
		var mungeAmount = 1;
		
		grid.lastScrollTop = visibleTop;

		function removeDistantNodes(preload, distanceOff, traversal, below){
			// we check to see the the nodes are "far off"
			var farOffRemoval = grid.farOffRemoval,
				preloadNode = preload.node;
			// by checking to see if it is the farOffRemoval distance away
			if(distanceOff > 2 * farOffRemoval){
				// ok, there is preloadNode that is far off, let's remove rows until we get to in the current viewpoint
				var row, nextRow = preloadNode[traversal];
				var reclaimedHeight = 0;
				var count = 0;
				var toDelete = [];
				while((row = nextRow)){
					var rowHeight = grid._calcRowHeight(row);
					if(reclaimedHeight + rowHeight + farOffRemoval > distanceOff || (nextRow.className.indexOf("dgrid-row") < 0 && nextRow.className.indexOf("dgrid-loading") < 0)){
						// we have reclaimed enough rows or we have gone beyond grid rows, let's call it good
						break;
					}
					var nextRow = row[traversal]; // have to do this before removing it
					reclaimedHeight += rowHeight;
					count += row.count || 1;
					// we just do cleanup here, as we will do a more efficient node destruction in the setTimeout below
					grid.removeRow(row, true);
					toDelete.push(row);
				}
				// now adjust the preloadNode based on the reclaimed space
				preload.count += count;
				if(below){
					preloadNode.rowIndex -= count;
					adjustHeight(preload);
				}else{
					// if it is above, we can calculate the change in exact row changes, which we must do to not mess with the scrolling
					preloadNode.style.height = (preloadNode.offsetHeight + reclaimedHeight) + "px";
				}
				// we remove the elements after expanding the preload node so that the contraction doesn't alter the scroll position
				var trashBin = put("div", toDelete);
				setTimeout(function(){
					// we can defer the destruction until later
					put(trashBin, "!");
				},1);
			}
		}
		
		function adjustHeight(preload, noMax){
			preload.node.style.height = Math.min(preload.count * grid.rowHeight, noMax ? Infinity : grid.maxEmptySpace) + "px";
		}
		function traversePreload(preload, moveNext){
			do{
				preload = moveNext ? preload.next : preload.previous;
			}while(preload && !preload.node.offsetWidth);// skip past preloads that are not currently connected
			return preload;
		}
		while(preload && !preload.node.offsetWidth){
			// skip past preloads that are not currently connected
			preload = preload.previous;
		}
		// there can be multiple preloadNodes (if they split, or multiple queries are created),
		//	so we can traverse them until we find whatever is in the current viewport, making
		//	sure we don't backtrack
		while(preload && preload != priorPreload){
			priorPreload = grid.preload;
			grid.preload = preload;
			preloadNode = preload.node;
			var preloadTop = preloadNode.offsetTop;
			var preloadHeight;
			
			if(visibleBottom + mungeAmount + searchBuffer < preloadTop){
				// the preload is below the line of sight
				preload = traversePreload(preload, (preloadSearchNext = false));
			}else if(visibleTop - mungeAmount - searchBuffer > (preloadTop + (preloadHeight = preloadNode.offsetHeight))){
				// the preload is above the line of sight
				preload = traversePreload(preload, (preloadSearchNext = true));
			}else{
				// the preload node is visible, or close to visible, better show it
				var offset = ((preloadNode.rowIndex ? visibleTop - requestBuffer : visibleBottom) - preloadTop) / grid.rowHeight;
				var count = (visibleBottom - visibleTop + 2 * requestBuffer) / grid.rowHeight;
				// utilize momentum for predictions
				var momentum = Math.max(Math.min((visibleTop - lastScrollTop) * grid.rowHeight, grid.maxRowsPerPage/2), grid.maxRowsPerPage/-2);
				count += Math.min(Math.abs(momentum), 10);
				if(preloadNode.rowIndex == 0){
					// at the top, adjust from bottom to top
					offset -= count;
				}
				offset = Math.max(offset, 0);
				if(offset < 10 && offset > 0 && count + offset < grid.maxRowsPerPage){
					// connect to the top of the preloadNode if possible to avoid excessive adjustments
					count += Math.max(0, offset);
					offset = 0;
				}
				count = Math.min(Math.max(count, grid.minRowsPerPage),
									grid.maxRowsPerPage, preload.count);
				
				if(count == 0){
					preload = traversePreload(preload, preloadSearchNext);
					continue;
				}
				
				count = Math.ceil(count);
				offset = Math.min(Math.floor(offset), preload.count - count);
				var options = lang.mixin(grid.get("queryOptions"), preload.options);
				preload.count -= count;
				var beforeNode = preloadNode,
					keepScrollTo, queryRowsOverlap = grid.queryRowsOverlap,
					below = preloadNode.rowIndex > 0 && preload; 
				if(below){
					// add new rows below
					var previous = preload.previous;
					if(previous){
						removeDistantNodes(previous, visibleTop - (previous.node.offsetTop + previous.node.offsetHeight), 'nextSibling');
						if(offset > 0 && previous.node == preloadNode.previousSibling){
							// all of the nodes above were removed
							offset = Math.min(preload.count, offset);
							preload.previous.count += offset;
							adjustHeight(preload.previous, true);
							preloadNode.rowIndex += offset;
							queryRowsOverlap = 0;
						}else{
							count += offset;
						}
						preload.count -= offset;
					}
					options.start = preloadNode.rowIndex - queryRowsOverlap;
					options.count = Math.min(count + queryRowsOverlap, grid.maxRowsPerPage);
					preloadNode.rowIndex = options.start + options.count;
				}else{
					// add new rows above
					if(preload.next){
						// remove out of sight nodes first
						removeDistantNodes(preload.next, preload.next.node.offsetTop - visibleBottom, 'previousSibling', true);
						var beforeNode = preloadNode.nextSibling;
						if(beforeNode == preload.next.node){
							// all of the nodes were removed, can position wherever we want
							preload.next.count += preload.count - offset;
							preload.next.node.rowIndex = offset + count;
							adjustHeight(preload.next);
							preload.count = offset;
							queryRowsOverlap = 0;
						}else{
							keepScrollTo = true;
						}
						
					}
					options.start = preload.count;
					options.count = Math.min(count + queryRowsOverlap, grid.maxRowsPerPage);
				}
				if(keepScrollTo && beforeNode && beforeNode.offsetWidth){
					keepScrollTo = beforeNode.offsetTop;
				}

				adjustHeight(preload);
				// create a loading node as a placeholder while the data is loaded
				var loadingNode = put(beforeNode, "-div.dgrid-loading[style=height:" + count * grid.rowHeight + "px]"),
					innerNode = put(loadingNode, "div.dgrid-" + (below ? "below" : "above"));
				innerNode.innerHTML = grid.loadingMessage;
				loadingNode.count = count;
				// use the query associated with the preload node to get the next "page"
				if("level" in preload.query){
					options.queryLevel = preload.query.level;
				}
				
				// Avoid spurious queries (ideally this should be unnecessary...)
				if(!("queryLevel" in options) && (options.start > grid._total || options.count < 0)){
					continue;
				}
				
				// Query now to fill in these rows.
				// Keep _trackError-wrapped results separate, since if results is a
				// promise, it will lose QueryResults functions when chained by `when`
				var results = preload.query(options),
					trackedResults = grid._trackError(function(){ return results; });
				
				if(trackedResults === undefined){
					// Sync query failed
					put(loadingNode, "!");
					return;
				}

				// Isolate the variables in case we make multiple requests
				// (which can happen if we need to render on both sides of an island of already-rendered rows)
				(function(loadingNode, scrollNode, below, keepScrollTo, results){
					lastRows = Deferred.when(grid.renderArray(results, loadingNode, options), function(rows){
						lastResults = results;
						
						// can remove the loading node now
						beforeNode = loadingNode.nextSibling;
						put(loadingNode, "!");
						if(keepScrollTo && beforeNode && beforeNode.offsetWidth){ // beforeNode may have been removed if the query results loading node was a removed as a distant node before rendering 
							// if the preload area above the nodes is approximated based on average
							// row height, we may need to adjust the scroll once they are filled in
							// so we don't "jump" in the scrolling position
							var pos = grid.getScrollPosition();
							grid.scrollTo({
								// Since we already had to query the scroll position,
								// include x to avoid TouchScroll querying it again on its end.
								x: pos.x,
								y: pos.y + beforeNode.offsetTop - keepScrollTo,
								// Don't kill momentum mid-scroll (for TouchScroll only).
								preserveMomentum: true
							});
						}
						
						Deferred.when(results.total || results.length, function(total){
							if(!("queryLevel" in options)){
								grid._total = total;
							}
							if(below){
								// if it is below, we will use the total from the results to update
								// the count of the last preload in case the total changes as later pages are retrieved
								// (not uncommon when total counts are estimated for db perf reasons)
								
								// recalculate the count
								below.count = total - below.node.rowIndex;
								// readjust the height
								adjustHeight(below);
							}
						});
						
						// make sure we have covered the visible area
						grid._processScroll();
						return rows;
					}, function (e) {
						put(loadingNode, "!");
						throw e;
					});
				}).call(this, loadingNode, scrollNode, below, keepScrollTo, results);
				preload = preload.previous;
			}
		}
		
		// After iterating, if additional requests have been made mid-refresh,
		// resolve the refresh promise based on the latest results obtained
		if (lastRows && (refreshDfd = this._refreshDeferred)) {
			delete this._refreshDeferred;
			Deferred.when(lastRows, function() {
				refreshDfd.resolve(lastResults);
			});
		}
	},

	removeRow: function(rowElement, justCleanup){
		function chooseIndex(index1, index2){
			return index1 != null ? index1 : index2;
		}

		if(rowElement){
			// Clean up observers that need to be cleaned up.
			var previousNode = rowElement.previousSibling,
				nextNode = rowElement.nextSibling,
				prevIndex = previousNode && chooseIndex(previousNode.observerIndex, previousNode.previousObserverIndex),
				nextIndex = nextNode && chooseIndex(nextNode.observerIndex, nextNode.nextObserverIndex),
				thisIndex = rowElement.observerIndex;

			// Clear the observerIndex on the node being removed so it will not be considered any longer.
			rowElement.observerIndex = undefined;
			if(justCleanup){
				// Save the indexes from the siblings for future calls to removeRow.
				rowElement.nextObserverIndex = nextIndex;
				rowElement.previousObserverIndex = prevIndex;
			}

			// Is this row's observer index different than those on either side?
			if(this.cleanEmptyObservers && thisIndex > -1 && thisIndex !== prevIndex && thisIndex !== nextIndex){
				// This is the last row that references the observer index.  Cancel the observer.
				var observers = this.observers;
				var observer = observers[thisIndex];
				if(observer){
					observer.cancel();
					this._numObservers--;
					observers[thisIndex] = 0; // remove it so we don't call cancel twice
				}
			}
		}
		// Finish the row removal.
		this.inherited(arguments);
	}
});

});

},
'dgrid/_StoreMixin':function(){
define(["dojo/_base/kernel", "dojo/_base/declare", "dojo/_base/lang", "dojo/_base/Deferred", "dojo/on", "dojo/aspect", "put-selector/put"],
function(kernel, declare, lang, Deferred, listen, aspect, put){
	// This module isolates the base logic required by store-aware list/grid
	// components, e.g. OnDemandList/Grid and the Pagination extension.
	
	// Noop function, needed for _trackError when callback due to a bug in 1.8
	// (see http://bugs.dojotoolkit.org/ticket/16667)
	function noop(value){ return value; }
	
	function emitError(err){
		// called by _trackError in context of list/grid, if an error is encountered
		if(typeof err !== "object"){
			// Ensure we actually have an error object, so we can attach a reference.
			err = new Error(err);
		}else if(err.dojoType === "cancel"){
			// Don't fire dgrid-error events for errors due to canceled requests
			// (unfortunately, the Deferred instrumentation will still log them)
			return;
		}
		// TODO: remove this @ 0.4 (prefer grid property directly on event object)
		err.grid = this;
		
		if(listen.emit(this.domNode, "dgrid-error", {
				grid: this,
				error: err,
				cancelable: true,
				bubbles: true })){
			console.error(err);
		}
	}
	
	return declare(null, {
		// store: Object
		//		The object store (implementing the dojo/store API) from which data is
		//		to be fetched.
		store: null,
		
		// query: Object
		//		Specifies query parameter(s) to pass to store.query calls.
		query: null,
		
		// queryOptions: Object
		//		Specifies additional query options to mix in when calling store.query;
		//		sort, start, and count are already handled.
		queryOptions: null,
		
		// getBeforePut: boolean
		//		If true, a get request will be performed to the store before each put
		//		as a baseline when saving; otherwise, existing row data will be used.
		getBeforePut: true,
		
		// noDataMessage: String
		//		Message to be displayed when no results exist for a query, whether at
		//		the time of the initial query or upon subsequent observed changes.
		//		Defined by _StoreMixin, but to be implemented by subclasses.
		noDataMessage: "",
		
		// loadingMessage: String
		//		Message displayed when data is loading.
		//		Defined by _StoreMixin, but to be implemented by subclasses.
		loadingMessage: "",
		
		constructor: function(){
			// Create empty objects on each instance, not the prototype
			this.query = {};
			this.queryOptions = {};
			this.dirty = {};
			this._updating = {}; // Tracks rows that are mid-update
			this._columnsWithSet = {};

			// Reset _columnsWithSet whenever column configuration is reset
			aspect.before(this, "configStructure", lang.hitch(this, function(){
				this._columnsWithSet = {};
			}));
		},
		
		postCreate: function(){
			this.inherited(arguments);
			if(this.store){
				this._updateNotifyHandle(this.store);
			}
		},
		
		destroy: function(){
			this.inherited(arguments);
			if(this._notifyHandle){
				this._notifyHandle.remove();
			}
		},
		
		_configColumn: function(column){
			// summary:
			//		Implements extension point provided by Grid to store references to
			//		any columns with `set` methods, for use during `save`.
			if (column.set){
				this._columnsWithSet[column.field] = column;
			}
		},
		
		_updateNotifyHandle: function(store){
			// summary:
			//		Unhooks any previously-existing store.notify handle, and
			//		hooks up a new one for the given store.
			
			if(this._notifyHandle){
				// Unhook notify handler from previous store
				this._notifyHandle.remove();
				delete this._notifyHandle;
			}
			if(store && typeof store.notify === "function"){
				this._notifyHandle = aspect.after(store, "notify",
					lang.hitch(this, "_onNotify"), true);
			}
		},
		
		_setStore: function(store, query, queryOptions){
			// summary:
			//		Assigns a new store (and optionally query/queryOptions) to the list,
			//		and tells it to refresh.
			
			this._updateNotifyHandle(store);
			
			this.store = store;
			this.dirty = {}; // discard dirty map, as it applied to a previous store
			this.set("query", query, queryOptions);
		},
		_setQuery: function(query, queryOptions){
			// summary:
			//		Assigns a new query (and optionally queryOptions) to the list,
			//		and tells it to refresh.
			
			var sort = queryOptions && queryOptions.sort;
			
			this.query = query !== undefined ? query : this.query;
			this.queryOptions = queryOptions || this.queryOptions;
			
			// If we have new sort criteria, pass them through sort
			// (which will update _sort and call refresh in itself).
			// Otherwise, just refresh.
			sort ? this.set("sort", sort) : this.refresh();
		},
		setStore: function(store, query, queryOptions){
			kernel.deprecated("setStore(...)", 'use set("store", ...) instead', "dgrid 0.4");
			this.set("store", store, query, queryOptions);
		},
		setQuery: function(query, queryOptions){
			kernel.deprecated("setQuery(...)", 'use set("query", ...) instead', "dgrid 0.4");
			this.set("query", query, queryOptions);
		},
		
		_getQueryOptions: function(){
			// summary:
			//		Get a fresh queryOptions object, also including the current sort
			var options = lang.delegate(this.queryOptions, {});
			if(this._sort.length){
				// Prevents SimpleQueryEngine from doing unnecessary "null" sorts (which can
				// change the ordering in browsers that don't use a stable sort algorithm, eg Chrome)
				options.sort = this._sort;
			}
			return options;
		},
		_getQuery: function(){
			// summary:
			//		Implemented consistent with _getQueryOptions so that if query is
			//		an object, this returns a protected (delegated) object instead of
			//		the original.
			var q = this.query;
			return typeof q == "object" && q != null ? lang.delegate(q, {}) : q;
		},
		
		_setSort: function(property, descending){
			// summary:
			//		Sort the content
			
			// prevent default storeless sort logic as long as we have a store
			if(this.store){ this._lastCollection = null; }
			this.inherited(arguments);
		},
		
		_onNotify: function(object, existingId){
			// summary:
			//		Method called when the store's notify method is called.
			
			// Call inherited in case anything was mixed in earlier
			this.inherited(arguments);
			
			// For adds/puts, check whether any observers are hooked up;
			// if not, force a refresh to properly hook one up now that there is data
			if(object && this._numObservers < 1){
				this.refresh({ keepScrollPosition: true });
			}
		},
		
		insertRow: function(object, parent, beforeNode, i, options){
			var store = this.store,
				dirty = this.dirty,
				id = store && store.getIdentity(object),
				dirtyObj;
			
			if(id in dirty && !(id in this._updating)){ dirtyObj = dirty[id]; }
			if(dirtyObj){
				// restore dirty object as delegate on top of original object,
				// to provide protection for subsequent changes as well
				object = lang.delegate(object, dirtyObj);
			}
			return this.inherited(arguments);
		},
		
		updateDirty: function(id, field, value){
			// summary:
			//		Updates dirty data of a field for the item with the specified ID.
			var dirty = this.dirty,
				dirtyObj = dirty[id];
			
			if(!dirtyObj){
				dirtyObj = dirty[id] = {};
			}
			dirtyObj[field] = value;
		},
		setDirty: function(id, field, value){
			kernel.deprecated("setDirty(...)", "use updateDirty() instead", "dgrid 0.4");
			this.updateDirty(id, field, value);
		},
		
		save: function() {
			// Keep track of the store and puts
			var self = this,
				store = this.store,
				dirty = this.dirty,
				dfd = new Deferred(), promise = dfd.promise,
				getFunc = function(id){
					// returns a function to pass as a step in the promise chain,
					// with the id variable closured
					var data;
					return (self.getBeforePut || !(data = self.row(id).data)) ?
						function(){ return store.get(id); } :
						function(){ return data; };
				};
			
			// function called within loop to generate a function for putting an item
			function putter(id, dirtyObj) {
				// Return a function handler
				return function(object) {
					var colsWithSet = self._columnsWithSet,
						updating = self._updating,
						key, data;

					if (typeof object.set === "function") {
						object.set(dirtyObj);
					} else {
						// Copy dirty props to the original, applying setters if applicable
						for(key in dirtyObj){
							object[key] = dirtyObj[key];
						}
					}

					// Apply any set methods in column definitions.
					// Note that while in the most common cases column.set is intended
					// to return transformed data for the key in question, it is also
					// possible to directly modify the object to be saved.
					for(key in colsWithSet){
						data = colsWithSet[key].set(object);
						if(data !== undefined){ object[key] = data; }
					}
					
					updating[id] = true;
					// Put it in the store, returning the result/promise
					return Deferred.when(store.put(object), function() {
						// Clear the item now that it's been confirmed updated
						delete dirty[id];
						delete updating[id];
					});
				};
			}
			
			// For every dirty item, grab the ID
			for(var id in dirty) {
				// Create put function to handle the saving of the the item
				var put = putter(id, dirty[id]);
				
				// Add this item onto the promise chain,
				// getting the item from the store first if desired.
				promise = promise.then(getFunc(id)).then(put);
			}
			
			// Kick off and return the promise representing all applicable get/put ops.
			// If the success callback is fired, all operations succeeded; otherwise,
			// save will stop at the first error it encounters.
			dfd.resolve();
			return promise;
		},
		
		revert: function(){
			// summary:
			//		Reverts any changes since the previous save.
			this.dirty = {};
			this.refresh();
		},
		
		_trackError: function(func){
			// summary:
			//		Utility function to handle emitting of error events.
			// func: Function|String
			//		A function which performs some store operation, or a String identifying
			//		a function to be invoked (sans arguments) hitched against the instance.
			//		If sync, it can return a value, but may throw an error on failure.
			//		If async, it should return a promise, which would fire the error
			//		callback on failure.
			// tags:
			//		protected
			
			var result;
			
			if(typeof func == "string"){ func = lang.hitch(this, func); }
			
			try{
				result = func();
			}catch(err){
				// report sync error
				emitError.call(this, err);
			}
			
			// wrap in when call to handle reporting of potential async error
			return Deferred.when(result, noop, lang.hitch(this, emitError));
		},
		
		newRow: function(){
			// Override to remove no data message when a new row appears.
			// Run inherited logic first to prevent confusion due to noDataNode
			// no longer being present as a sibling.
			var row = this.inherited(arguments);
			if(this.noDataNode){
				put(this.noDataNode, "!");
				delete this.noDataNode;
			}
			return row;
		},
		removeRow: function(rowElement, justCleanup){
			var row = {element: rowElement};
			// Check to see if we are now empty...
			if(!justCleanup && this.noDataMessage &&
					(this.up(row).element === rowElement) &&
					(this.down(row).element === rowElement)){
				// ...we are empty, so show the no data message.
				this.noDataNode = put(this.contentNode, "div.dgrid-no-data");
				this.noDataNode.innerHTML = this.noDataMessage;
			}
			return this.inherited(arguments);
		}
	});
});

},
'dijit/dijit':function(){
define([
	"./main",
	"./_base",
	"dojo/parser",
	"./_Widget",
	"./_TemplatedMixin",
	"./_Container",
	"./layout/_LayoutWidget",
	"./form/_FormWidget",
	"./form/_FormValueWidget"
], function(dijit){

	// module:
	//		dijit/dijit

	/*=====
	return {
		// summary:
		//		A roll-up for common dijit methods
		//		All the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)
		//		And some other stuff that we tend to pull in all the time anyway
	};
	=====*/

	return dijit;
});

},
'dijit/_base':function(){
define([
	"./main",
	"./a11y",	// used to be in dijit/_base/manager
	"./WidgetSet",	// used to be in dijit/_base/manager
	"./_base/focus",
	"./_base/manager",
	"./_base/place",
	"./_base/popup",
	"./_base/scroll",
	"./_base/sniff",
	"./_base/typematic",
	"./_base/wai",
	"./_base/window"
], function(dijit){

	// module:
	//		dijit/_base

	/*=====
	return {
		// summary:
		//		Includes all the modules in dijit/_base
	};
	=====*/

	return dijit._base;
});

},
'dijit/WidgetSet':function(){
define([
	"dojo/_base/array", // array.forEach array.map
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.global
	"./registry"	// to add functions to dijit.registry
], function(array, declare, kernel, registry){

	// module:
	//		dijit/WidgetSet

	var WidgetSet = declare("dijit.WidgetSet", null, {
		// summary:
		//		A set of widgets indexed by id.
		//		Deprecated, will be removed in 2.0.
		//
		// example:
		//		Create a small list of widgets:
		//		|	require(["dijit/WidgetSet", "dijit/registry"],
		//		|		function(WidgetSet, registry){
		//		|		var ws = new WidgetSet();
		//		|		ws.add(registry.byId("one"));
		//		|		ws.add(registry.byId("two"));
		//		|		// destroy both:
		//		|		ws.forEach(function(w){ w.destroy(); });
		//		|	});

		constructor: function(){
			this._hash = {};
			this.length = 0;
		},

		add: function(/*dijit/_WidgetBase*/ widget){
			// summary:
			//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
			//
			// widget: dijit/_WidgetBase
			//		Any dijit/_WidgetBase subclass.
			if(this._hash[widget.id]){
				throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
			}
			this._hash[widget.id] = widget;
			this.length++;
		},

		remove: function(/*String*/ id){
			// summary:
			//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
			//		removes the reference.
			if(this._hash[id]){
				delete this._hash[id];
				this.length--;
			}
		},

		forEach: function(/*Function*/ func, /* Object? */thisObj){
			// summary:
			//		Call specified function for each widget in this set.
			//
			// func:
			//		A callback function to run for each item. Is passed the widget, the index
			//		in the iteration, and the full hash, similar to `array.forEach`.
			//
			// thisObj:
			//		An optional scope parameter
			//
			// example:
			//		Using the default `dijit.registry` instance:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.forEach(function(widget){
			//		|			console.log(widget.declaredClass);
			//		|		});
			//		|	});
			//
			// returns:
			//		Returns self, in order to allow for further chaining.

			thisObj = thisObj || kernel.global;
			var i = 0, id;
			for(id in this._hash){
				func.call(thisObj, this._hash[id], i++, this._hash);
			}
			return this;	// dijit/WidgetSet
		},

		filter: function(/*Function*/ filter, /* Object? */thisObj){
			// summary:
			//		Filter down this WidgetSet to a smaller new WidgetSet
			//		Works the same as `array.filter` and `NodeList.filter`
			//
			// filter:
			//		Callback function to test truthiness. Is passed the widget
			//		reference and the pseudo-index in the object.
			//
			// thisObj: Object?
			//		Option scope to use for the filter function.
			//
			// example:
			//		Arbitrary: select the odd widgets in this list
			//		|	
			//		|		
			//		|	
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.filter(function(w, i){
			//		|			return i % 2 == 0;
			//		|		}).forEach(function(w){ /* odd ones */ });
			//		|	});

			thisObj = thisObj || kernel.global;
			var res = new WidgetSet(), i = 0, id;
			for(id in this._hash){
				var w = this._hash[id];
				if(filter.call(thisObj, w, i++, this._hash)){
					res.add(w);
				}
			}
			return res; // dijit/WidgetSet
		},

		byId: function(/*String*/ id){
			// summary:
			//		Find a widget in this list by it's id.
			// example:
			//		Test if an id is in a particular WidgetSet
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		var ws = new WidgetSet();
			//		|		ws.add(registry.byId("bar"));
			//		|		var t = ws.byId("bar") // returns a widget
			//		|		var x = ws.byId("foo"); // returns undefined
			//		|	});

			return this._hash[id];	// dijit/_WidgetBase
		},

		byClass: function(/*String*/ cls){
			// summary:
			//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
			//
			// cls: String
			//		The Class to scan for. Full dot-notated string.
			//
			// example:
			//		Find all `dijit.TitlePane`s in a page:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });
			//		|	});

			var res = new WidgetSet(), id, widget;
			for(id in this._hash){
				widget = this._hash[id];
				if(widget.declaredClass == cls){
					res.add(widget);
				}
			 }
			 return res; // dijit/WidgetSet
		},

		toArray: function(){
			// summary:
			//		Convert this WidgetSet into a true Array
			//
			// example:
			//		Work with the widget .domNodes in a real Array
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		array.map(registry.toArray(), function(w){ return w.domNode; });
			//		|	});


			var ar = [];
			for(var id in this._hash){
				ar.push(this._hash[id]);
			}
			return ar;	// dijit/_WidgetBase[]
		},

		map: function(/* Function */func, /* Object? */thisObj){
			// summary:
			//		Create a new Array from this WidgetSet, following the same rules as `array.map`
			// example:
			//		|	require(["dijit/WidgetSet", "dijit/registry"],
			//		|		function(WidgetSet, registry){
			//		|		var nodes = registry.map(function(w){ return w.domNode; });
			//		|	});
			//
			// returns:
			//		A new array of the returned values.
			return array.map(this.toArray(), func, thisObj); // Array
		},

		every: function(func, thisObj){
			// summary:
			//		A synthetic clone of `array.every` acting explicitly on this WidgetSet
			//
			// func: Function
			//		A callback function run for every widget in this list. Exits loop
			//		when the first false return is encountered.
			//
			// thisObj: Object?
			//		Optional scope parameter to use for the callback

			thisObj = thisObj || kernel.global;
			var x = 0, i;
			for(i in this._hash){
				if(!func.call(thisObj, this._hash[i], x++, this._hash)){
					return false; // Boolean
				}
			}
			return true; // Boolean
		},

		some: function(func, thisObj){
			// summary:
			//		A synthetic clone of `array.some` acting explicitly on this WidgetSet
			//
			// func: Function
			//		A callback function run for every widget in this list. Exits loop
			//		when the first true return is encountered.
			//
			// thisObj: Object?
			//		Optional scope parameter to use for the callback

			thisObj = thisObj || kernel.global;
			var x = 0, i;
			for(i in this._hash){
				if(func.call(thisObj, this._hash[i], x++, this._hash)){
					return true; // Boolean
				}
			}
			return false; // Boolean
		}

	});

	// Add in 1.x compatibility methods to dijit/registry.
	// These functions won't show up in the API doc but since they are deprecated anyway,
	// that's probably for the best.
	array.forEach(["forEach", "filter", "byClass", "map", "every", "some"], function(func){
		registry[func] = WidgetSet.prototype[func];
	});


	return WidgetSet;
});

},
'dijit/_base/focus':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/dom", // dom.isDescendant
	"dojo/_base/lang", // lang.isArray
	"dojo/topic", // publish
	"dojo/_base/window", // win.doc win.doc.selection win.global win.global.getSelection win.withGlobal
	"../focus",
	"../selection",
	"../main"	// for exporting symbols to dijit
], function(array, dom, lang, topic, win, focus, selection, dijit){

	// module:
	//		dijit/_base/focus

	var exports = {
		// summary:
		//		Deprecated module to monitor currently focused node and stack of currently focused widgets.
		//		New code should access dijit/focus directly.

		// _curFocus: DomNode
		//		Currently focused item on screen
		_curFocus: null,

		// _prevFocus: DomNode
		//		Previously focused item on screen
		_prevFocus: null,

		isCollapsed: function(){
			// summary:
			//		Returns true if there is no text selected
			return dijit.getBookmark().isCollapsed;
		},

		getBookmark: function(){
			// summary:
			//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
			var sel = win.global == window ? selection : new selection.SelectionManager(win.global);
			return sel.getBookmark();
		},

		moveToBookmark: function(/*Object*/ bookmark){
			// summary:
			//		Moves current selection to a bookmark
			// bookmark:
			//		This should be a returned object from dijit.getBookmark()

			var sel = win.global == window ? selection : new selection.SelectionManager(win.global);
			return sel.moveToBookmark(bookmark);
		},

		getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
			// summary:
			//		Called as getFocus(), this returns an Object showing the current focus
			//		and selected text.
			//
			//		Called as getFocus(widget), where widget is a (widget representing) a button
			//		that was just pressed, it returns where focus was before that button
			//		was pressed.   (Pressing the button may have either shifted focus to the button,
			//		or removed focus altogether.)   In this case the selected text is not returned,
			//		since it can't be accurately determined.
			//
			// menu: dijit/_WidgetBase|{domNode: DomNode} structure
			//		The button that was just pressed.  If focus has disappeared or moved
			//		to this button, returns the previous focus.  In this case the bookmark
			//		information is already lost, and null is returned.
			//
			// openedForWindow:
			//		iframe in which menu was opened
			//
			// returns:
			//		A handle to restore focus/selection, to be passed to `dijit.focus`
			var node = !focus.curNode || (menu && dom.isDescendant(focus.curNode, menu.domNode)) ? dijit._prevFocus : focus.curNode;
			return {
				node: node,
				bookmark: node && (node == focus.curNode) && win.withGlobal(openedForWindow || win.global, dijit.getBookmark),
				openedForWindow: openedForWindow
			}; // Object
		},

		// _activeStack: dijit/_WidgetBase[]
		//		List of currently active widgets (focused widget and it's ancestors)
		_activeStack: [],

		registerIframe: function(/*DomNode*/ iframe){
			// summary:
			//		Registers listeners on the specified iframe so that any click
			//		or focus event on that iframe (or anything in it) is reported
			//		as a focus/click event on the `<iframe>` itself.
			// description:
			//		Currently only used by editor.
			// returns:
			//		Handle to pass to unregisterIframe()
			return focus.registerIframe(iframe);
		},

		unregisterIframe: function(/*Object*/ handle){
			// summary:
			//		Unregisters listeners on the specified iframe created by registerIframe.
			//		After calling be sure to delete or null out the handle itself.
			// handle:
			//		Handle returned by registerIframe()

			handle && handle.remove();
		},

		registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
			// summary:
			//		Registers listeners on the specified window (either the main
			//		window or an iframe's window) to detect when the user has clicked somewhere
			//		or focused somewhere.
			// description:
			//		Users should call registerIframe() instead of this method.
			// targetWindow:
			//		If specified this is the window associated with the iframe,
			//		i.e. iframe.contentWindow.
			// effectiveNode:
			//		If specified, report any focus events inside targetWindow as
			//		an event on effectiveNode, rather than on evt.target.
			// returns:
			//		Handle to pass to unregisterWin()

			return focus.registerWin(targetWindow, effectiveNode);
		},

		unregisterWin: function(/*Handle*/ handle){
			// summary:
			//		Unregisters listeners on the specified window (either the main
			//		window or an iframe's window) according to handle returned from registerWin().
			//		After calling be sure to delete or null out the handle itself.

			handle && handle.remove();
		}
	};

	// Override focus singleton's focus function so that dijit.focus()
	// has backwards compatible behavior of restoring selection (although
	// probably no one is using that).
	focus.focus = function(/*Object|DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			focus._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && win.withGlobal(openedForWindow || win.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				win.withGlobal(openedForWindow || win.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	};

	// For back compatibility, monitor changes to focused node and active widget stack,
	// publishing events and copying changes from focus manager variables into dijit (top level) variables
	focus.watch("curNode", function(name, oldVal, newVal){
		dijit._curFocus = newVal;
		dijit._prevFocus = oldVal;
		if(newVal){
			topic.publish("focusNode", newVal);	// publish
		}
	});
	focus.watch("activeStack", function(name, oldVal, newVal){
		dijit._activeStack = newVal;
	});

	focus.on("widget-blur", function(widget, by){
		topic.publish("widgetBlur", widget, by);	// publish
	});
	focus.on("widget-focus", function(widget, by){
		topic.publish("widgetFocus", widget, by);	// publish
	});

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dijit/selection':function(){
define([
	"dojo/_base/array",
	"dojo/dom", // dom.byId
	"dojo/_base/lang",
	"dojo/sniff", // has("ie") has("opera")
	"dojo/_base/window",
	"dijit/focus"
], function(array, dom, lang, has, baseWindow, focus){

	// module:
	//		dijit/selection

	// Note that this class is using feature detection, but doesn't use has() because sometimes on IE the outer window
	// may be running in standards mode (ie, IE9 mode) but an iframe may be in compatibility mode.   So the code path
	// used will vary based on the window.

	var SelectionManager = function(win){
		// summary:
		//		Class for monitoring / changing the selection (typically highlighted text) in a given window
		// win: Window
		//		The window to monitor/adjust the selection on.

		var doc = win.document;

		this.getType = function(){
			// summary:
			//		Get the selection type (like doc.select.type in IE).
			if(doc.getSelection){
				// W3C path
				var stype = "text";

				// Check if the actual selection is a CONTROL (IMG, TABLE, HR, etc...).
				var oSel;
				try{
					oSel = win.getSelection();
				}catch(e){ /*squelch*/ }

				if(oSel && oSel.rangeCount == 1){
					var oRange = oSel.getRangeAt(0);
					if(	(oRange.startContainer == oRange.endContainer) &&
						((oRange.endOffset - oRange.startOffset) == 1) &&
						(oRange.startContainer.nodeType != 3 /* text node*/)
						){
						stype = "control";
					}
				}
				return stype; //String
			}else{
				// IE6-8
				return doc.selection.type.toLowerCase();
			}
		};

		this.getSelectedText = function(){
			// summary:
			//		Return the text (no html tags) included in the current selection or null if no text is selected
			if(doc.getSelection){
				// W3C path
				var selection = win.getSelection();
				return selection ? selection.toString() : ""; //String
			}else{
				// IE6-8
				if(this.getType() == 'control'){
					return null;
				}
				return doc.selection.createRange().text;
			}
		};

		this.getSelectedHtml = function(){
			// summary:
			//		Return the html text of the current selection or null if unavailable
			if(doc.getSelection){
				// W3C path
				var selection = win.getSelection();
				if(selection && selection.rangeCount){
					var i;
					var html = "";
					for(i = 0; i < selection.rangeCount; i++){
						//Handle selections spanning ranges, such as Opera
						var frag = selection.getRangeAt(i).cloneContents();
						var div = doc.createElement("div");
						div.appendChild(frag);
						html += div.innerHTML;
					}
					return html; //String
				}
				return null;
			}else{
				// IE6-8
				if(this.getType() == 'control'){
					return null;
				}
				return doc.selection.createRange().htmlText;
			}
		};

		this.getSelectedElement = function(){
			// summary:
			//		Retrieves the selected element (if any), just in the case that
			//		a single element (object like and image or a table) is
			//		selected.
			if(this.getType() == "control"){
				if(doc.getSelection){
					// W3C path
					var selection = win.getSelection();
					return selection.anchorNode.childNodes[ selection.anchorOffset ];
				}else{
					// IE6-8
					var range = doc.selection.createRange();
					if(range && range.item){
						return doc.selection.createRange().item(0);
					}
				}
			}
			return null;
		};

		this.getParentElement = function(){
			// summary:
			//		Get the parent element of the current selection
			if(this.getType() == "control"){
				var p = this.getSelectedElement();
				if(p){ return p.parentNode; }
			}else{
				if(doc.getSelection){
					var selection = doc.getSelection();
					if(selection){
						var node = selection.anchorNode;
						while(node && (node.nodeType != 1)){ // not an element
							node = node.parentNode;
						}
						return node;
					}
				}else{
					var r = doc.selection.createRange();
					r.collapse(true);
					return r.parentElement();
				}
			}
			return null;
		};

		this.hasAncestorElement = function(/*String*/ tagName /* ... */){
			// summary:
			//		Check whether current selection has a  parent element which is
			//		of type tagName (or one of the other specified tagName)
			// tagName: String
			//		The tag name to determine if it has an ancestor of.
			return this.getAncestorElement.apply(this, arguments) != null; //Boolean
		};

		this.getAncestorElement = function(/*String*/ tagName /* ... */){
			// summary:
			//		Return the parent element of the current selection which is of
			//		type tagName (or one of the other specified tagName)
			// tagName: String
			//		The tag name to determine if it has an ancestor of.
			var node = this.getSelectedElement() || this.getParentElement();
			return this.getParentOfType(node, arguments); //DOMNode
		};

		this.isTag = function(/*DomNode*/ node, /*String[]*/ tags){
			// summary:
			//		Function to determine if a node is one of an array of tags.
			// node:
			//		The node to inspect.
			// tags:
			//		An array of tag name strings to check to see if the node matches.
			if(node && node.tagName){
				var _nlc = node.tagName.toLowerCase();
				for(var i=0; i<tags.length; i++){
					var _tlc = String(tags[i]).toLowerCase();
					if(_nlc == _tlc){
						return _tlc; // String
					}
				}
			}
			return "";
		};

		this.getParentOfType = function(/*DomNode*/ node, /*String[]*/ tags){
			// summary:
			//		Function to locate a parent node that matches one of a set of tags
			// node:
			//		The node to inspect.
			// tags:
			//		An array of tag name strings to check to see if the node matches.
			while(node){
				if(this.isTag(node, tags).length){
					return node; // DOMNode
				}
				node = node.parentNode;
			}
			return null;
		};

		this.collapse = function(/*Boolean*/ beginning){
			// summary:
			//		Function to collapse (clear), the current selection
			// beginning: Boolean
			//		Indicates whether to collapse the cursor to the beginning of the selection or end.
			if(doc.getSelection){
				// W3C path
				var selection = win.getSelection();
				if(selection.removeAllRanges){ // Mozilla
					if(beginning){
						selection.collapseToStart();
					}else{
						selection.collapseToEnd();
					}
				}else{ // Safari
					// pulled from WebCore/ecma/kjs_window.cpp, line 2536
					selection.collapse(beginning);
				}
			}else{
				// IE6-8
				var range = doc.selection.createRange();
				range.collapse(beginning);
				range.select();
			}
		};

		this.remove = function(){
			// summary:
			//		Function to delete the currently selected content from the document.
			var sel = doc.selection;
			if(doc.getSelection){
				// W3C path
				sel = win.getSelection();
				sel.deleteFromDocument();
				return sel; //Selection
			}else{
				// IE6-8
				if(sel.type.toLowerCase() != "none"){
					sel.clear();
				}
				return sel; //Selection
			}
		};

		this.selectElementChildren = function(/*DomNode*/ element, /*Boolean?*/ nochangefocus){
			// summary:
			//		clear previous selection and select the content of the node
			//		(excluding the node itself)
			// element: DOMNode
			//		The element you wish to select the children content of.
			// nochangefocus: Boolean
			//		Indicates if the focus should change or not.

			var range;
			element = dom.byId(element);
			if(doc.getSelection){
				// W3C
				var selection = win.getSelection();
				if(has("opera")){
					//Opera's selectAllChildren doesn't seem to work right
					//against <body> nodes and possibly others ... so
					//we use the W3C range API
					if(selection.rangeCount){
						range = selection.getRangeAt(0);
					}else{
						range = doc.createRange();
					}
					range.setStart(element, 0);
					range.setEnd(element,(element.nodeType == 3) ? element.length : element.childNodes.length);
					selection.addRange(range);
				}else{
					selection.selectAllChildren(element);
				}
			}else{
				// IE6-8
				range = element.ownerDocument.body.createTextRange();
				range.moveToElementText(element);
				if(!nochangefocus){
					try{
						range.select(); // IE throws an exception here if the widget is hidden.  See #5439
					}catch(e){ /* squelch */}
				}
			}
		};

		this.selectElement = function(/*DomNode*/ element, /*Boolean?*/ nochangefocus){
			// summary:
			//		clear previous selection and select element (including all its children)
			// element: DOMNode
			//		The element to select.
			// nochangefocus: Boolean
			//		Boolean indicating if the focus should be changed.  IE only.
			var range;
			element = dom.byId(element);	// TODO: remove for 2.0 or sooner, spec listed above doesn't allow for string
			if(doc.getSelection){
				// W3C path
				var selection = doc.getSelection();
				range = doc.createRange();
				if(selection.removeAllRanges){ // Mozilla
					// FIXME: does this work on Safari?
					if(has("opera")){
						//Opera works if you use the current range on
						//the selection if present.
						if(selection.getRangeAt(0)){
							range = selection.getRangeAt(0);
						}
					}
					range.selectNode(element);
					selection.removeAllRanges();
					selection.addRange(range);
				}
			}else{
				// IE6-8
				try{
					var tg = element.tagName ? element.tagName.toLowerCase() : "";
					if(tg === "img" || tg === "table"){
						range = baseWindow.body(doc).createControlRange();
					}else{
						range = baseWindow.body(doc).createRange();
					}
					range.addElement(element);
					if(!nochangefocus){
						range.select();
					}
				}catch(e){
					this.selectElementChildren(element, nochangefocus);
				}
			}
		};

		this.inSelection = function(node){
			// summary:
			//		This function determines if 'node' is
			//		in the current selection.
			// tags:
			//		public
			if(node){
				var newRange;
				var range;

				if(doc.getSelection){
					// WC3
					var sel = win.getSelection();
					if(sel && sel.rangeCount > 0){
						range = sel.getRangeAt(0);
					}
					if(range && range.compareBoundaryPoints && doc.createRange){
						try{
							newRange = doc.createRange();
							newRange.setStart(node, 0);
							if(range.compareBoundaryPoints(range.START_TO_END, newRange) === 1){
								return true;
							}
						}catch(e){ /* squelch */}
					}
				}else{
					// IE6-8, so we can't use the range object as the pseudo
					// range doesn't implement the boundary checking, we have to
					// use IE specific crud.
					range = doc.selection.createRange();
					try{
						newRange = node.ownerDocument.body.createTextRange();
						newRange.moveToElementText(node);
					}catch(e2){/* squelch */}
					if(range && newRange){
						// We can finally compare similar to W3C
						if(range.compareEndPoints("EndToStart", newRange) === 1){
							return true;
						}
					}
				}
			}
			return false; // Boolean
		},

		this.getBookmark = function(){
			// summary:
			//		Retrieves a bookmark that can be used with moveToBookmark to reselect the currently selected range.

			// TODO: merge additional code from Editor._getBookmark into this method

			var bm, rg, tg, sel = doc.selection, cf = focus.curNode;

			if(doc.getSelection){
				// W3C Range API for selections.
				sel = win.getSelection();
				if(sel){
					if(sel.isCollapsed){
						tg = cf? cf.tagName : "";
						if(tg){
							// Create a fake rangelike item to restore selections.
							tg = tg.toLowerCase();
							if(tg == "textarea" ||
								(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
								sel = {
									start: cf.selectionStart,
									end: cf.selectionEnd,
									node: cf,
									pRange: true
								};
								return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
							}
						}
						bm = {isCollapsed:true};
						if(sel.rangeCount){
							bm.mark = sel.getRangeAt(0).cloneRange();
						}
					}else{
						rg = sel.getRangeAt(0);
						bm = {isCollapsed: false, mark: rg.cloneRange()};
					}
				}
			}else if(sel){
				// If the current focus was a input of some sort and no selection, don't bother saving
				// a native bookmark.  This is because it causes issues with dialog/page selection restore.
				// So, we need to create pseudo bookmarks to work with.
				tg = cf ? cf.tagName : "";
				tg = tg.toLowerCase();
				if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
					if(sel.type && sel.type.toLowerCase() == "none"){
						return {
							isCollapsed: true,
							mark: null
						}
					}else{
						rg = sel.createRange();
						return {
							isCollapsed: rg.text && rg.text.length?false:true,
							mark: {
								range: rg,
								pRange: true
							}
						};
					}
				}
				bm = {};

				//'IE' way for selections.
				try{
					// createRange() throws exception when dojo in iframe
					// and nothing selected, see #9632
					rg = sel.createRange();
					bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
				}catch(e){
					bm.isCollapsed = true;
					return bm;
				}
				if(sel.type.toUpperCase() == 'CONTROL'){
					if(rg.length){
						bm.mark=[];
						var i=0,len=rg.length;
						while(i<len){
							bm.mark.push(rg.item(i++));
						}
					}else{
						bm.isCollapsed = true;
						bm.mark = null;
					}
				}else{
					bm.mark = rg.getBookmark();
				}
			}else{
				console.warn("No idea how to store the current selection for this browser!");
			}
			return bm; // Object
		};

		this.moveToBookmark = function(/*Object*/ bookmark){
			// summary:
			//		Moves current selection to a bookmark.
			// bookmark:
			//		This should be a returned object from getBookmark().

			// TODO: merge additional code from Editor._moveToBookmark into this method

			var mark = bookmark.mark;
			if(mark){
				if(doc.getSelection){
					// W3C Range API (FF, WebKit, Opera, etc)
					var sel = win.getSelection();
					if(sel && sel.removeAllRanges){
						if(mark.pRange){
							var n = mark.node;
							n.selectionStart = mark.start;
							n.selectionEnd = mark.end;
						}else{
							sel.removeAllRanges();
							sel.addRange(mark);
						}
					}else{
						console.warn("No idea how to restore selection for this browser!");
					}
				}else if(doc.selection && mark){
					//'IE' way.
					var rg;
					if(mark.pRange){
						rg = mark.range;
					}else if(lang.isArray(mark)){
						rg = doc.body.createControlRange();
						//rg.addElement does not have call/apply method, so can not call it directly
						//rg is not available in "range.addElement(item)", so can't use that either
						array.forEach(mark, function(n){
							rg.addElement(n);
						});
					}else{
						rg = doc.body.createTextRange();
						rg.moveToBookmark(mark);
					}
					rg.select();
				}
			}
		};

		this.isCollapsed = function(){
			// summary:
			//		Returns true if there is no text selected
			return this.getBookmark().isCollapsed;
		};
	};

	// singleton on the main window
	var selection = new SelectionManager(window);

	// hook for editor to use class
	selection.SelectionManager = SelectionManager;

	return selection;
});

},
'dijit/_base/place':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/lang", // lang.isArray, lang.mixin
	"dojo/window", // windowUtils.getBox
	"../place",
	"../main"	// export to dijit namespace
], function(array, lang, windowUtils, place, dijit){

	// module:
	//		dijit/_base/place


	var exports = {
		// summary:
		//		Deprecated back compatibility module, new code should use dijit/place directly instead of using this module.
	};

	exports.getViewport = function(){
		// summary:
		//		Deprecated method to return the dimensions and scroll position of the viewable area of a browser window.
		//		New code should use windowUtils.getBox()

		return windowUtils.getBox();
	};

	exports.placeOnScreen = place.at;

	exports.placeOnScreenAroundElement = function(node, aroundNode, aroundCorners, layoutNode){
		// summary:
		//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
		//		for the "around" argument and finds a proper processor to place a node.
		//		Deprecated, new code should use dijit/place.around() instead.

		// Convert old style {"BL": "TL", "BR": "TR"} type argument
		// to style needed by dijit.place code:
		//		[
		//			{aroundCorner: "BL", corner: "TL" },
		//			{aroundCorner: "BR", corner: "TR" }
		//		]
		var positions;
		if(lang.isArray(aroundCorners)){
			positions = aroundCorners;
		}else{
			positions = [];
			for(var key in aroundCorners){
				positions.push({aroundCorner: key, corner: aroundCorners[key]});
			}
		}

		return place.around(node, aroundNode, positions, true, layoutNode);
	};

	exports.placeOnScreenAroundNode = exports.placeOnScreenAroundElement;
	/*=====
	exports.placeOnScreenAroundNode = function(node, aroundNode, aroundCorners, layoutNode){
		// summary:
		//		Position node adjacent or kitty-corner to aroundNode
		//		such that it's fully visible in viewport.
		//		Deprecated, new code should use dijit/place.around() instead.
	};
	=====*/

	exports.placeOnScreenAroundRectangle = exports.placeOnScreenAroundElement;
	/*=====
	exports.placeOnScreenAroundRectangle = function(node, aroundRect, aroundCorners, layoutNode){
		// summary:
		//		Like dijit.placeOnScreenAroundNode(), except that the "around"
		//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
		//		instead of a dom node.
		//		Deprecated, new code should use dijit/place.around() instead.
	};
	=====*/

	exports.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
		// summary:
		//		Deprecated method, unneeded when using dijit/place directly.
		//		Transforms the passed array of preferred positions into a format suitable for
		//		passing as the aroundCorners argument to dijit/place.placeOnScreenAroundElement.
		// position: String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//		- before: places drop down to the left of the target node/widget, or to the right in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- after: places drop down to the right of the target node/widget, or to the left in
		//		  the case of RTL scripts like Hebrew and Arabic
		//		- above: drop down goes above target node
		//		- below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		// leftToRight: Boolean
		//		Whether the popup will be displaying in leftToRight mode.

		var align = {};
		array.forEach(position, function(pos){
			var ltr = leftToRight;
			switch(pos){
				case "after":
					align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
					break;
				case "before":
					align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
					break;
				case "below-alt":
					ltr = !ltr;
					// fall through
				case "below":
					// first try to align left borders, next try to align right borders (or reverse for RTL mode)
					align[ltr ? "BL" : "BR"] = ltr ? "TL" : "TR";
					align[ltr ? "BR" : "BL"] = ltr ? "TR" : "TL";
					break;
				case "above-alt":
					ltr = !ltr;
					// fall through
				case "above":
				default:
					// first try to align left borders, next try to align right borders (or reverse for RTL mode)
					align[ltr ? "TL" : "TR"] = ltr ? "BL" : "BR";
					align[ltr ? "TR" : "TL"] = ltr ? "BR" : "BL";
					break;
			}
		});
		return align;
	};

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dijit/_base/popup':function(){
define([
	"dojo/dom-class", // domClass.contains
	"dojo/_base/window",
	"../popup",
	"../BackgroundIframe"	// just loading for back-compat, in case client code is referencing it
], function(domClass, win, popup){

// module:
//		dijit/_base/popup

/*=====
return {
	// summary:
	//		Deprecated.   Old module for popups, new code should use dijit/popup directly.
};
=====*/


// Hack support for old API passing in node instead of a widget (to various methods)
var origCreateWrapper = popup._createWrapper;
popup._createWrapper = function(widget){
	if(!widget.declaredClass){
		// make fake widget to pass to new API
		widget = {
			_popupWrapper: (widget.parentNode && domClass.contains(widget.parentNode, "dijitPopup")) ?
				widget.parentNode : null,
			domNode: widget,
			destroy: function(){},
			ownerDocument: widget.ownerDocument,
			ownerDocumentBody: win.body(widget.ownerDocument)
		};
	}
	return origCreateWrapper.call(this, widget);
};

// Support old format of orient parameter
var origOpen = popup.open;
popup.open = function(/*__OpenArgs*/ args){
	// Convert old hash structure (ex: {"BL": "TL", ...}) of orient to format compatible w/new popup.open() API.
	// Don't do conversion for:
	//		- null parameter (that means to use the default positioning)
	//		- "R" or "L" strings used to indicate positioning for context menus (when there is no around node)
	//		- new format, ex: ["below", "above"]
	//		- return value from deprecated dijit.getPopupAroundAlignment() method,
	//			ex: ["below", "above"]
	if(args.orient && typeof args.orient != "string" && !("length" in args.orient)){
		var ary = [];
		for(var key in args.orient){
			ary.push({aroundCorner: key, corner: args.orient[key]});
		}
		args.orient = ary;
	}

	return origOpen.call(this, args);
};

return popup;
});

},
'dijit/_base/scroll':function(){
define([
	"dojo/window", // windowUtils.scrollIntoView
	"../main"	// export symbol to dijit
], function(windowUtils, dijit){
	// module:
	//		dijit/_base/scroll

	/*=====
	return {
		// summary:
		//		Back compatibility module, new code should use windowUtils directly instead of using this module.
	};
	=====*/

	dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
		// summary:
		//		Scroll the passed node into view, if it is not already.
		//		Deprecated, use `windowUtils.scrollIntoView` instead.

		windowUtils.scrollIntoView(node, pos);
	};
});

},
'dijit/_base/sniff':function(){
define([ "dojo/uacss" ], function(){

	// module:
	//		dijit/_base/sniff

	/*=====
	return {
		// summary:
		//		Deprecated, back compatibility module, new code should require dojo/uacss directly instead of this module.
	};
	=====*/
});

},
'dijit/_base/typematic':function(){
define(["../typematic"], function(){

	/*=====
	return {
		// summary:
		//		Deprecated, for back-compat, just loads top level module
	};
	=====*/

});

},
'dijit/typematic':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/connect", // connect._keyPress
	"dojo/_base/lang", // lang.mixin, lang.hitch
	"dojo/on",
	"dojo/sniff", // has("ie")
	"./main"        // setting dijit.typematic global
], function(array, connect, lang, on, has, dijit){

	// module:
	//		dijit/typematic

	var typematic = (dijit.typematic = {
		// summary:
		//		These functions are used to repetitively call a user specified callback
		//		method when a specific key or mouse click over a specific DOM node is
		//		held down for a specific amount of time.
		//		Only 1 such event is allowed to occur on the browser page at 1 time.

		_fireEventAndReload: function(){
			this._timer = null;
			this._callback(++this._count, this._node, this._evt);

			// Schedule next event, timer is at most minDelay (default 10ms) to avoid
			// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
			this._currentTimeout = Math.max(
				this._currentTimeout < 0 ? this._initialDelay :
					(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
				this._minDelay);
			this._timer = setTimeout(lang.hitch(this, "_fireEventAndReload"), this._currentTimeout);
		},

		trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number?*/ subsequentDelay, /*Number?*/ initialDelay, /*Number?*/ minDelay){
			// summary:
			//		Start a timed, repeating callback sequence.
			//		If already started, the function call is ignored.
			//		This method is not normally called by the user but can be
			//		when the normal listener code is insufficient.
			// evt:
			//		key or mouse event object to pass to the user callback
			// _this:
			//		pointer to the user's widget space.
			// node:
			//		the DOM node object to pass the the callback function
			// callback:
			//		function to call until the sequence is stopped called with 3 parameters:
			// count:
			//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
			// node:
			//		the DOM node object passed in
			// evt:
			//		key or mouse event object
			// obj:
			//		user space object used to uniquely identify each typematic sequence
			// subsequentDelay:
			//		if > 1, the number of milliseconds until the 3->n events occur
			//		or else the fractional time multiplier for the next event's delay, default=0.9
			// initialDelay:
			//		the number of milliseconds until the 2nd event occurs, default=500ms
			// minDelay:
			//		the maximum delay in milliseconds for event to fire, default=10ms
			if(obj != this._obj){
				this.stop();
				this._initialDelay = initialDelay || 500;
				this._subsequentDelay = subsequentDelay || 0.90;
				this._minDelay = minDelay || 10;
				this._obj = obj;
				this._node = node;
				this._currentTimeout = -1;
				this._count = -1;
				this._callback = lang.hitch(_this, callback);
				this._evt = { faux: true };
				for(var attr in evt){
					if(attr != "layerX" && attr != "layerY"){ // prevent WebKit warnings
						var v = evt[attr];
						if(typeof v != "function" && typeof v != "undefined"){
							this._evt[attr] = v
						}
					}
				}
				this._fireEventAndReload();
			}
		},

		stop: function(){
			// summary:
			//		Stop an ongoing timed, repeating callback sequence.
			if(this._timer){
				clearTimeout(this._timer);
				this._timer = null;
			}
			if(this._obj){
				this._callback(-1, this._node, this._evt);
				this._obj = null;
			}
		},

		addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
			// summary:
			//		Start listening for a specific typematic key.
			//		See also the trigger method for other parameters.
			// keyObject:
			//		an object defining the key to listen for:
			//
			//		- keyCode: the keyCode (number) to listen for, used for non-printable keys
			//		- charCode: the charCode (number) to listen for, used for printable keys
			//		- charOrCode: deprecated, use keyCode or charCode
			//		- ctrlKey: desired ctrl key state to initiate the callback sequence:
			//			- pressed (true)
			//			- released (false)
			//			- either (unspecified)
			//		- altKey: same as ctrlKey but for the alt key
			//		- shiftKey: same as ctrlKey but for the shift key
			// returns:
			//		a connection handle

			// Setup keydown or keypress listener depending on whether keyCode or charCode was specified.
			// If charOrCode is specified use deprecated connect._keypress synthetic event (remove for 2.0)
			var type = "keyCode" in keyObject ? "keydown" : "charCode" in keyObject ? "keypress" : connect._keypress,
				attr = "keyCode" in keyObject ? "keyCode" : "charCode" in keyObject ? "charCode" : "charOrCode";

			var handles = [
				on(node, type, lang.hitch(this, function(evt){
					if(evt[attr] == keyObject[attr] &&
						(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
						(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
						(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
						(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
						evt.stopPropagation();
						evt.preventDefault();
						typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
					}else if(typematic._obj == keyObject){
						typematic.stop();
					}
				})),
				on(node, "keyup", lang.hitch(this, function(){
					if(typematic._obj == keyObject){
						typematic.stop();
					}
				}))
			];
			return { remove: function(){
				array.forEach(handles, function(h){
					h.remove();
				});
			} };
		},

		addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
			// summary:
			//		Start listening for a typematic mouse click.
			//		See the trigger method for other parameters.
			// returns:
			//		a connection handle
			var handles = [
				on(node, "mousedown", lang.hitch(this, function(evt){
					evt.preventDefault();
					typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
				})),
				on(node, "mouseup", lang.hitch(this, function(evt){
					if(this._obj){
						evt.preventDefault();
					}
					typematic.stop();
				})),
				on(node, "mouseout", lang.hitch(this, function(evt){
					if(this._obj){
						evt.preventDefault();
					}
					typematic.stop();
				})),
				on(node, "dblclick", lang.hitch(this, function(evt){
					evt.preventDefault();
					if(has("ie") < 9){
						typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
						setTimeout(lang.hitch(this, typematic.stop), 50);
					}
				}))
			];
			return { remove: function(){
				array.forEach(handles, function(h){
					h.remove();
				});
			} };
		},

		addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
			// summary:
			//		Start listening for a specific typematic key and mouseclick.
			//		This is a thin wrapper to addKeyListener and addMouseListener.
			//		See the addMouseListener and addKeyListener methods for other parameters.
			// mouseNode:
			//		the DOM node object to listen on for mouse events.
			// keyNode:
			//		the DOM node object to listen on for key events.
			// returns:
			//		a connection handle
			var handles = [
				this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay),
				this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay)
			];
			return { remove: function(){
				array.forEach(handles, function(h){
					h.remove();
				});
			} };
		}
	});

	return typematic;
});

},
'dijit/_base/wai':function(){
define([
	"dojo/dom-attr", // domAttr.attr
	"dojo/_base/lang", // lang.mixin
	"../main",	// export symbols to dijit
	"../hccss"			// not using this module directly, but loading it sets CSS flag on <html>
], function(domAttr, lang, dijit){

	// module:
	//		dijit/_base/wai

	var exports = {
		// summary:
		//		Deprecated methods for setting/getting wai roles and states.
		//		New code should call setAttribute()/getAttribute() directly.
		//
		//		Also loads hccss to apply dj_a11y class to root node if machine is in high-contrast mode.

		hasWaiRole: function(/*Element*/ elem, /*String?*/ role){
			// summary:
			//		Determines if an element has a particular role.
			// returns:
			//		True if elem has the specific role attribute and false if not.
			//		For backwards compatibility if role parameter not provided,
			//		returns true if has a role
			var waiRole = this.getWaiRole(elem);
			return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
		},

		getWaiRole: function(/*Element*/ elem){
			// summary:
			//		Gets the role for an element (which should be a wai role).
			// returns:
			//		The role of elem or an empty string if elem
			//		does not have a role.
			 return lang.trim((domAttr.get(elem, "role") || "").replace("wairole:",""));
		},

		setWaiRole: function(/*Element*/ elem, /*String*/ role){
			// summary:
			//		Sets the role on an element.
			// description:
			//		Replace existing role attribute with new role.

			domAttr.set(elem, "role", role);
		},

		removeWaiRole: function(/*Element*/ elem, /*String*/ role){
			// summary:
			//		Removes the specified role from an element.
			//		Removes role attribute if no specific role provided (for backwards compat.)

			var roleValue = domAttr.get(elem, "role");
			if(!roleValue){ return; }
			if(role){
				var t = lang.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
				domAttr.set(elem, "role", t);
			}else{
				elem.removeAttribute("role");
			}
		},

		hasWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Determines if an element has a given state.
			// description:
			//		Checks for an attribute called "aria-"+state.
			// returns:
			//		true if elem has a value for the given state and
			//		false if it does not.

			return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
		},

		getWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Gets the value of a state on an element.
			// description:
			//		Checks for an attribute called "aria-"+state.
			// returns:
			//		The value of the requested state on elem
			//		or an empty string if elem has no value for state.

			return elem.getAttribute("aria-"+state) || "";
		},

		setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
			// summary:
			//		Sets a state on an element.
			// description:
			//		Sets an attribute called "aria-"+state.

			elem.setAttribute("aria-"+state, value);
		},

		removeWaiState: function(/*Element*/ elem, /*String*/ state){
			// summary:
			//		Removes a state from an element.
			// description:
			//		Sets an attribute called "aria-"+state.

			elem.removeAttribute("aria-"+state);
		}
	};

	lang.mixin(dijit, exports);

	/*===== return exports; =====*/
	return dijit;	// for back compat :-(
});

},
'dijit/_base/window':function(){
define([
	"dojo/window", // windowUtils.get
	"../main"	// export symbol to dijit
], function(windowUtils, dijit){
	// module:
	//		dijit/_base/window

	/*=====
	return {
		// summary:
		//		Back compatibility module, new code should use windowUtils directly instead of using this module.
	};
	=====*/

	dijit.getDocumentWindow = function(doc){
		return windowUtils.get(doc);
	};
});

},
'dijit/layout/_LayoutWidget':function(){
define([
	"dojo/_base/lang", // lang.mixin
	"../_Widget",
	"../_Container",
	"../_Contained",
	"../Viewport",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style" // domStyle.getComputedStyle
], function(lang, _Widget, _Container, _Contained, Viewport,
	declare, domClass, domGeometry, domStyle){

	// module:
	//		dijit/layout/_LayoutWidget


	return declare("dijit.layout._LayoutWidget", [_Widget, _Container, _Contained], {
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under <body>.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent();
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when viewport size changes so that I can re-layout.
				this.own(Viewport.on("resize", lang.hitch(this, "resize")));
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		####Change size mode:
			//
			//		When changeSize is specified, changes the marginBox of this widget
			//		and forces it to re-layout its contents accordingly.
			//		changeSize may specify height, width, or both.
			//
			//		If resultSize is specified it indicates the size the widget will
			//		become after changeSize has been applied.
			//
			//		####Notification mode:
			//
			//		When changeSize is null, indicates that the caller has already changed
			//		the size of the widget, or perhaps it changed because the browser
			//		window was resized.  Tells widget to re-layout its contents accordingly.
			//
			//		If resultSize is also specified it indicates the size the widget has
			//		become.
			//
			//		In either mode, this method also:
			//
			//		1. Sets this._borderBox and this._contentBox to the new size of
			//			the widget.  Queries the current domNode size if necessary.
			//		2. Calls layout() to resize contents (and maybe adjust child widgets).
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				domGeometry.setMarginBox(node, changeSize);
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			lang.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = lang.mixin(domGeometry.getMarginBox(node), mb);	// just use domGeometry.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling domGeometry.getContentBox() since that may fail if size was recently set)
			var cs = domStyle.getComputedStyle(node);
			var me = domGeometry.getMarginExtents(node, cs);
			var be = domGeometry.getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = domGeometry.getPadExtents(node, cs);
			this._contentBox = {
				l: domStyle.toPixelValue(node, cs.paddingLeft),
				t: domStyle.toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit/_WidgetBase*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			var cls = this.baseClass + "-child "
				+ (child.baseClass ? this.baseClass + "-" + child.baseClass : "");
			domClass.add(child.domNode, cls);
		},

		addChild: function(/*dijit/_WidgetBase*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit/_WidgetBase*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			var cls = this.baseClass + "-child"
					+ (child.baseClass ?
						" " + this.baseClass + "-" + child.baseClass : "");
			domClass.remove(child.domNode, cls);

			this.inherited(arguments);
		}
	});
});

},
'dijit/Dialog':function(){
define([
	"require",
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/aspect",
	"dojo/_base/declare", // declare
	"dojo/Deferred", // Deferred
	"dojo/dom", // dom.isDescendant
	"dojo/dom-class", // domClass.add domClass.contains
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/_base/fx", // fx.fadeIn fx.fadeOut
	"dojo/i18n", // i18n.getLocalization
	"dojo/keys",
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on",
	"dojo/ready",
	"dojo/sniff", // has("ie") has("opera") has("dijit-legacy-requires")
	"dojo/window", // winUtils.getBox, winUtils.get
	"dojo/dnd/Moveable", // Moveable
	"dojo/dnd/TimedMoveable", // TimedMoveable
	"./focus",
	"./_base/manager", // manager.defaultDuration
	"./_Widget",
	"./_TemplatedMixin",
	"./_CssStateMixin",
	"./form/_FormMixin",
	"./_DialogMixin",
	"./DialogUnderlay",
	"./layout/ContentPane",
	"dojo/text!./templates/Dialog.html",
	"dojo/i18n!./nls/common"
], function(require, array, aspect, declare, Deferred,
			dom, domClass, domGeometry, domStyle, fx, i18n, keys, lang, on, ready, has, winUtils,
			Moveable, TimedMoveable, focus, manager, _Widget, _TemplatedMixin, _CssStateMixin, _FormMixin, _DialogMixin,
			DialogUnderlay, ContentPane, template){

	// module:
	//		dijit/Dialog

	/*=====
	dijit._underlay = function(kwArgs){
		// summary:
		//		A shared instance of a `dijit.DialogUnderlay`
		//
		// description:
		//		A shared instance of a `dijit.DialogUnderlay` created and
		//		used by `dijit.Dialog`, though never created until some Dialog
		//		or subclass thereof is shown.
	};
	=====*/

	var _DialogBase = declare("dijit._DialogBase" + (has("dojo-bidi") ? "_NoBidi" : ""), [_TemplatedMixin, _FormMixin, _DialogMixin, _CssStateMixin], {
		templateString: template,

		baseClass: "dijitDialog",

		cssStateNodes: {
			closeButtonNode: "dijitDialogCloseIcon"
		},

		// Map widget attributes to DOMNode attributes.
		_setTitleAttr: { node: "titleNode", type: "innerHTML" },

		// open: [readonly] Boolean
		//		True if Dialog is currently displayed on screen.
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: manager.defaultDuration,

		// refocus: Boolean
		//		A Toggle to modify the default focus behavior of a Dialog, which
		//		is to re-focus the element which had focus before being opened.
		//		False will disable refocusing. Default: true
		refocus: true,

		// autofocus: Boolean
		//		A Toggle to modify the default focus behavior of a Dialog, which
		//		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// _firstFocusItem: [private readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit/_DialogMixin._getFocusItems()`.
		_firstFocusItem: null,

		// _lastFocusItem: [private readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit/_DialogMixin._getFocusItems()`.
		_lastFocusItem: null,

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for Dialog, since Dialog
		//		is never a child of a layout container, nor can you specify the size of
		//		Dialog in order to control the size of an inner widget.
		doLayout: false,

		// draggable: Boolean
		//		Toggles the movable aspect of the Dialog. If true, Dialog
		//		can be dragged by it's title. If false it will remain centered
		//		in the viewport.
		draggable: true,
		_setDraggableAttr: function(/*Boolean*/ val){
			// Avoid _WidgetBase behavior of copying draggable attribute to this.domNode,
			// as that prevents text select on modern browsers (#14452)
			this._set("draggable", val);
		},

		// maxRatio: Number
		//		Maximum size to allow the dialog to expand to, relative to viewport size
		maxRatio: 0.9,

		// closable: Boolean
		//		Dialog show [x] icon to close itself, and ESC key will close the dialog.
		closable: true,
		_setClosableAttr: function(val){
			this.closeButtonNode.style.display = val ? "" : "none";
			this._set("closable", val);
		},

		postMixInProperties: function(){
			var _nlsResources = i18n.getLocalization("dijit", "common");
			lang.mixin(this, _nlsResources);
			this.inherited(arguments);
		},

		postCreate: function(){
			domStyle.set(this.domNode, {
				display: "none",
				position: "absolute"
			});
			this.ownerDocumentBody.appendChild(this.domNode);

			this.inherited(arguments);

			aspect.after(this, "onExecute", lang.hitch(this, "hide"), true);
			aspect.after(this, "onCancel", lang.hitch(this, "hide"), true);

			this._modalconnects = [];
		},

		onLoad: function(){
			// summary:
			//		Called when data has been loaded from an href.
			//		Unlike most other callbacks, this function can be connected to (via `dojo.connect`)
			//		but should *not* be overridden.
			// tags:
			//		callback

			// when href is specified we need to reposition the dialog after the data is loaded
			// and find the focusable elements
			this._size();
			this._position();

			if(this.autofocus && DialogLevelManager.isTop(this)){
				this._getFocusItems(this.domNode);
				focus.focus(this._firstFocusItem);
			}

			this.inherited(arguments);
		},

		focus: function(){
			this._getFocusItems(this.domNode);
			focus.focus(this._firstFocusItem);
		},

		_endDrag: function(){
			// summary:
			//		Called after dragging the Dialog. Saves the position of the dialog in the viewport,
			//		and also adjust position to be fully within the viewport, so user doesn't lose access to handle
			var nodePosition = domGeometry.position(this.domNode),
				viewport = winUtils.getBox(this.ownerDocument);
			nodePosition.y = Math.min(Math.max(nodePosition.y, 0), (viewport.h - nodePosition.h));
			nodePosition.x = Math.min(Math.max(nodePosition.x, 0), (viewport.w - nodePosition.w));
			this._relativePosition = nodePosition;
			this._position();
		},

		_setup: function(){
			// summary:
			//		Stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons).
			// tags:
			//		private

			var node = this.domNode;

			if(this.titleBar && this.draggable){
				this._moveable = new ((has("ie") == 6) ? TimedMoveable // prevent overload, see #5285
					: Moveable)(node, { handle: this.titleBar });
				aspect.after(this._moveable, "onMoveStop", lang.hitch(this, "_endDrag"), true);
			}else{
				domClass.add(node, "dijitDialogFixed");
			}

			this.underlayAttrs = {
				dialogId: this.id,
				"class": array.map(this["class"].split(/\s/),function(s){
					return s + "_underlay";
				}).join(" "),
				_onKeyDown: lang.hitch(this, "_onKey"),
				ownerDocument: this.ownerDocument
			};
		},

		_size: function(){
			// summary:
			//		If necessary, shrink dialog contents so dialog fits in viewport.
			// tags:
			//		private

			this._checkIfSingleChild();

			// If we resized the dialog contents earlier, reset them back to original size, so
			// that if the user later increases the viewport size, the dialog can display w/out a scrollbar.
			// Need to do this before the domGeometry.position(this.domNode) call below.
			if(this._singleChild){
				if(typeof this._singleChildOriginalStyle != "undefined"){
					this._singleChild.domNode.style.cssText = this._singleChildOriginalStyle;
					delete this._singleChildOriginalStyle;
				}
			}else{
				domStyle.set(this.containerNode, {
					width: "auto",
					height: "auto"
				});
			}

			var bb = domGeometry.position(this.domNode);

			// Get viewport size but then reduce it by a bit; Dialog should always have some space around it
			// to indicate that it's a popup.  This will also compensate for possible scrollbars on viewport.
			var viewport = winUtils.getBox(this.ownerDocument);
			viewport.w *= this.maxRatio;
			viewport.h *= this.maxRatio;

			if(bb.w >= viewport.w || bb.h >= viewport.h){
				// Reduce size of dialog contents so that dialog fits in viewport

				var containerSize = domGeometry.position(this.containerNode),
					w = Math.min(bb.w, viewport.w) - (bb.w - containerSize.w),
					h = Math.min(bb.h, viewport.h) - (bb.h - containerSize.h);

				if(this._singleChild && this._singleChild.resize){
					if(typeof this._singleChildOriginalStyle == "undefined"){
						this._singleChildOriginalStyle = this._singleChild.domNode.style.cssText;
					}
					this._singleChild.resize({w: w, h: h});
				}else{
					domStyle.set(this.containerNode, {
						width: w + "px",
						height: h + "px",
						overflow: "auto",
						position: "relative"    // workaround IE bug moving scrollbar or dragging dialog
					});
				}
			}else{
				if(this._singleChild && this._singleChild.resize){
					this._singleChild.resize();
				}
			}
		},

		_position: function(){
			// summary:
			//		Position the dialog in the viewport.  If no relative offset
			//		in the viewport has been determined (by dragging, for instance),
			//		center the dialog.  Otherwise, use the Dialog's stored relative offset,
			//		adjusted by the viewport's scroll.
			if(!domClass.contains(this.ownerDocumentBody, "dojoMove")){    // don't do anything if called during auto-scroll
				var node = this.domNode,
					viewport = winUtils.getBox(this.ownerDocument),
					p = this._relativePosition,
					bb = p ? null : domGeometry.position(node),
					l = Math.floor(viewport.l + (p ? p.x : (viewport.w - bb.w) / 2)),
					t = Math.floor(viewport.t + (p ? p.y : (viewport.h - bb.h) / 2))
					;
				domStyle.set(node, {
					left: l + "px",
					top: t + "px"
				});
			}
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles the keyboard events for accessibility reasons
			// tags:
			//		private

			if(evt.keyCode == keys.TAB){
				this._getFocusItems(this.domNode);
				var node = evt.target;
				if(this._firstFocusItem == this._lastFocusItem){
					// don't move focus anywhere, but don't allow browser to move focus off of dialog either
					evt.stopPropagation();
					evt.preventDefault();
				}else if(node == this._firstFocusItem && evt.shiftKey){
					// if we are shift-tabbing from first focusable item in dialog, send focus to last item
					focus.focus(this._lastFocusItem);
					evt.stopPropagation();
					evt.preventDefault();
				}else if(node == this._lastFocusItem && !evt.shiftKey){
					// if we are tabbing from last focusable item in dialog, send focus to first item
					focus.focus(this._firstFocusItem);
					evt.stopPropagation();
					evt.preventDefault();
				}
			}else if(this.closable && evt.keyCode == keys.ESCAPE){
				this.onCancel();
				evt.stopPropagation();
				evt.preventDefault();
			}
		},

		show: function(){
			// summary:
			//		Display the dialog
			// returns: dojo/promise/Promise
			//		Promise object that resolves when the display animation is complete

			if(this.open){
				return;
			}

			if(!this._started){
				this.startup();
			}

			// first time we show the dialog, there's some initialization stuff to do
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized = true;
			}

			if(this._fadeOutDeferred){
				// There's a hide() operation in progress, so cancel it, but still call DialogLevelManager.hide()
				// as though the hide() completed, in preparation for the DialogLevelManager.show() call below.
				this._fadeOutDeferred.cancel();
				DialogLevelManager.hide(this);
			}

			// Recenter Dialog if user scrolls browser.  Connecting to document doesn't work on IE, need to use window.
			var win = winUtils.get(this.ownerDocument);
			this._modalconnects.push(on(win, "scroll", lang.hitch(this, "resize")));

			this._modalconnects.push(on(this.domNode, "keydown", lang.hitch(this, "_onKey")));

			domStyle.set(this.domNode, {
				opacity: 0,
				display: ""
			});

			this._set("open", true);
			this._onShow(); // lazy load trigger

			this._size();
			this._position();

			// fade-in Animation object, setup below
			var fadeIn;

			this._fadeInDeferred = new Deferred(lang.hitch(this, function(){
				fadeIn.stop();
				delete this._fadeInDeferred;
			}));

			// If delay is 0, code below will delete this._fadeInDeferred instantly, so grab promise while we can.
			var promise = this._fadeInDeferred.promise;

			fadeIn = fx.fadeIn({
				node: this.domNode,
				duration: this.duration,
				beforeBegin: lang.hitch(this, function(){
					DialogLevelManager.show(this, this.underlayAttrs);
				}),
				onEnd: lang.hitch(this, function(){
					if(this.autofocus && DialogLevelManager.isTop(this)){
						// find focusable items each time dialog is shown since if dialog contains a widget the
						// first focusable items can change
						this._getFocusItems(this.domNode);
						focus.focus(this._firstFocusItem);
					}
					this._fadeInDeferred.resolve(true);
					delete this._fadeInDeferred;
				})
			}).play();

			return promise;
		},

		hide: function(){
			// summary:
			//		Hide the dialog
			// returns: dojo/promise/Promise
			//		Promise object that resolves when the display animation is complete

			// If we haven't been initialized yet then we aren't showing and we can just return.
			// Likewise if we are already hidden, or are currently fading out.
			if(!this._alreadyInitialized || !this.open){
				return;
			}
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}

			// fade-in Animation object, setup below
			var fadeOut;

			this._fadeOutDeferred = new Deferred(lang.hitch(this, function(){
				fadeOut.stop();
				delete this._fadeOutDeferred;
			}));

			// fire onHide when the promise resolves.
			this._fadeOutDeferred.then(lang.hitch(this, 'onHide'));

			// If delay is 0, code below will delete this._fadeOutDeferred instantly, so grab promise while we can.
			var promise = this._fadeOutDeferred.promise;

			fadeOut = fx.fadeOut({
				node: this.domNode,
				duration: this.duration,
				onEnd: lang.hitch(this, function(){
					this.domNode.style.display = "none";
					DialogLevelManager.hide(this);
					this._fadeOutDeferred.resolve(true);
					delete this._fadeOutDeferred;
				})
			}).play();

			if(this._scrollConnected){
				this._scrollConnected = false;
			}
			var h;
			while(h = this._modalconnects.pop()){
				h.remove();
			}

			if(this._relativePosition){
				delete this._relativePosition;
			}
			this._set("open", false);

			return promise;
		},

		resize: function(){
			// summary:
			//		Called when viewport scrolled or size changed.  Adjust Dialog as necessary to keep it visible.
			// tags:
			//		private
			if(this.domNode.style.display != "none"){
				this._size();
				if(!has("touch")){
					// If the user has scrolled the display then reposition the Dialog.  But don't do it for touch
					// devices, because it will counteract when a keyboard pops up and then the browser auto-scrolls
					// the focused node into view.
					this._position();
				}
			}
		},

		destroy: function(){
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}
			if(this._fadeOutDeferred){
				this._fadeOutDeferred.cancel();
			}
			if(this._moveable){
				this._moveable.destroy();
			}
			var h;
			while(h = this._modalconnects.pop()){
				h.remove();
			}

			DialogLevelManager.hide(this);

			this.inherited(arguments);
		}
	});

	if(has("dojo-bidi")){
		_DialogBase = declare("dijit._DialogBase", _DialogBase, {
			_setTitleAttr: function(/*String*/ title){
				this._set("title", title);
				this.titleNode.innerHTML = title;
				this.applyTextDir(this.titleNode);
			},

			_setTextDirAttr: function(textDir){
				if(this._created && this.textDir != textDir){
					this._set("textDir", textDir);
					this.set("title", this.title);
				}
			}
		});
	}

	var Dialog = declare("dijit.Dialog", [ContentPane, _DialogBase], {
		// summary:
		//		A modal dialog Widget.
		// description:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.).
		// example:
		// |	<div data-dojo-type="dijit/Dialog" data-dojo-props="href: 'test.html'"></div>
		// example:
		// |	var foo = new Dialog({ title: "test dialog", content: "test content" });
		// |	foo.placeAt(win.body());
		// |	foo.startup();
	});
	Dialog._DialogBase = _DialogBase;	// for monkey patching and dojox/widget/DialogSimple

	var DialogLevelManager = Dialog._DialogLevelManager = {
		// summary:
		//		Controls the various active "levels" on the page, starting with the
		//		stuff initially visible on the page (at z-index 0), and then having an entry for
		//		each Dialog shown.

		_beginZIndex: 950,

		show: function(/*dijit/_WidgetBase*/ dialog, /*Object*/ underlayAttrs){
			// summary:
			//		Call right before fade-in animation for new dialog.
			//		Saves current focus, displays/adjusts underlay for new dialog,
			//		and sets the z-index of the dialog itself.
			//
			//		New dialog will be displayed on top of all currently displayed dialogs.
			//
			//		Caller is responsible for setting focus in new dialog after the fade-in
			//		animation completes.

			// Save current focus
			ds[ds.length - 1].focus = focus.curNode;

			// Set z-index a bit above previous dialog
			var zIndex = ds[ds.length - 1].dialog ? ds[ds.length - 1].zIndex + 2 : Dialog._DialogLevelManager._beginZIndex;
			domStyle.set(dialog.domNode, 'zIndex', zIndex);

			// Display the underlay, or if already displayed then adjust for this new dialog
			DialogUnderlay.show(underlayAttrs, zIndex - 1);

			ds.push({dialog: dialog, underlayAttrs: underlayAttrs, zIndex: zIndex});
		},

		hide: function(/*dijit/_WidgetBase*/ dialog){
			// summary:
			//		Called when the specified dialog is hidden/destroyed, after the fade-out
			//		animation ends, in order to reset page focus, fix the underlay, etc.
			//		If the specified dialog isn't open then does nothing.
			//
			//		Caller is responsible for either setting display:none on the dialog domNode,
			//		or calling dijit/popup.hide(), or removing it from the page DOM.

			if(ds[ds.length - 1].dialog == dialog){
				// Removing the top (or only) dialog in the stack, return focus
				// to previous dialog

				ds.pop();

				var pd = ds[ds.length - 1];	// the new active dialog (or the base page itself)

				// Adjust underlay
				if(ds.length == 1){
					// Returning to original page.  Hide the underlay.
					DialogUnderlay.hide();
				}else{
					// Popping back to previous dialog, adjust underlay.
					DialogUnderlay.show(pd.underlayAttrs, pd.zIndex - 1);
				}

				// Adjust focus.
				// TODO: regardless of setting of dialog.refocus, if the exeucte() method set focus somewhere,
				// don't shift focus back to button.  Note that execute() runs at the start of the fade-out but
				// this code runs later, at the end of the fade-out.  Menu has code like this.
				if(dialog.refocus){
					// If we are returning control to a previous dialog but for some reason
					// that dialog didn't have a focused field, set focus to first focusable item.
					// This situation could happen if two dialogs appeared at nearly the same time,
					// since a dialog doesn't set it's focus until the fade-in is finished.
					var focus = pd.focus;
					if(pd.dialog && (!focus || !dom.isDescendant(focus, pd.dialog.domNode))){
						pd.dialog._getFocusItems(pd.dialog.domNode);
						focus = pd.dialog._firstFocusItem;
					}

					if(focus){
						// Refocus the button that spawned the Dialog.   This will fail in corner cases including
						// page unload on IE, because the dijit/form/Button that launched the Dialog may get destroyed
						// before this code runs.  (#15058)
						try{
							focus.focus();
						}catch(e){
						}
					}
				}
			}else{
				// Removing a dialog out of order (#9944, #10705).
				// Don't need to mess with underlay or z-index or anything.
				var idx = array.indexOf(array.map(ds, function(elem){
					return elem.dialog;
				}), dialog);
				if(idx != -1){
					ds.splice(idx, 1);
				}
			}
		},

		isTop: function(/*dijit/_WidgetBase*/ dialog){
			// summary:
			//		Returns true if specified Dialog is the top in the task
			return ds[ds.length - 1].dialog == dialog;
		}
	};

	// Stack representing the various active "levels" on the page, starting with the
	// stuff initially visible on the page (at z-index 0), and then having an entry for
	// each Dialog shown.
	// Each element in stack has form {
	//		dialog: dialogWidget,
	//		focus: returnFromGetFocus(),
	//		underlayAttrs: attributes to set on underlay (when this widget is active)
	// }
	var ds = Dialog._dialogStack = [
		{dialog: null, focus: null, underlayAttrs: null}    // entry for stuff at z-index: 0
	];

	// If focus was accidentally removed from the dialog, such as if the user clicked a blank
	// area of the screen, or clicked the browser's address bar and then tabbed into the page,
	// then refocus.   Won't do anything if focus was removed because the Dialog was closed, or
	// because a new Dialog popped up on top of the old one, or when focus moves to popups
	focus.watch("curNode", function(attr, oldNode, node){
 		// Note: if no dialogs, ds.length==1 but ds[ds.length-1].dialog is null
		var topDialog = ds[ds.length - 1].dialog;

		// If a node was focused, and there's a Dialog currently showing, and not in the process of fading out...
		// Ignore focus events on other document though because it's likely an Editor inside of the Dialog.
		if(node && topDialog && !topDialog._fadeOutDeferred && node.ownerDocument == topDialog.ownerDocument){
			// If the node that was focused is inside the dialog or in a popup, even a context menu that isn't
			// technically a descendant of the the dialog, don't do anything.
			do{
				if(node == topDialog.domNode || domClass.contains(node, "dijitPopup")){ return; }
			}while(node = node.parentNode);

			// Otherwise, return focus to the dialog.  Use a delay to avoid confusing dijit/focus code's
			// own tracking of focus.
			topDialog.focus();
		}
	});

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/TooltipDialog"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return Dialog;
});

},
'dojo/dnd/Moveable':function(){
define([
	"../_base/array", "../_base/declare", "../_base/lang",
	"../dom", "../dom-class", "../Evented", "../on", "../topic", "../touch", "./common", "./Mover", "../_base/window"
], function(array, declare, lang, dom, domClass, Evented, on, topic, touch, dnd, Mover, win){

// module:
//		dojo/dnd/Moveable


var Moveable = declare("dojo.dnd.Moveable", [Evented], {
	// summary:
	//		an object, which makes a node movable

	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,

	constructor: function(node, params){
		// node: Node
		//		a node (or node's id) to be moved
		// params: Moveable.__MoveableArgs?
		//		optional parameters
		this.node = dom.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dom.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : Mover;
		this.events = [
			on(this.handle, touch.press, lang.hitch(this, "onMouseDown")),
			// cancel text selection and text dragging
			on(this.handle, "dragstart",   lang.hitch(this, "onSelectStart")),
			on(this.handle, "selectstart",   lang.hitch(this, "onSelectStart"))
		];
	},

	// markup methods
	markupFactory: function(params, node, Ctor){
		return new Ctor(node, params);
	},

	// methods
	destroy: function(){
		// summary:
		//		stops watching for possible move, deletes all references, so the object can be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		this.events = this.node = this.handle = null;
	},

	// mouse event processors
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown/ontouchstart, creates a Mover for the node
		// e: Event
		//		mouse/touch event
		if(this.skip && dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(
				on(this.handle, touch.move, lang.hitch(this, "onMouseMove")),
				on(this.handle, touch.release, lang.hitch(this, "onMouseUp"))
			);
			this._lastX = e.pageX;
			this._lastY = e.pageY;
		}else{
			this.onDragDetected(e);
		}
		e.stopPropagation();
		e.preventDefault();
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove, used only for delayed drags
		// e: Event
		//		mouse/touch event
		if(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			this.onDragDetected(e);
		}
		e.stopPropagation();
		e.preventDefault();
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup, used only for delayed drags
		// e: Event
		//		mouse event
		for(var i = 0; i < 2; ++i){
			this.events.pop().remove();
		}
		e.stopPropagation();
		e.preventDefault();
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skip || !dnd.isFormElement(e)){
			e.stopPropagation();
			e.preventDefault();
		}
	},

	// local events
	onDragDetected: function(/*Event*/ e){
		// summary:
		//		called when the drag is detected;
		//		responsible for creation of the mover
		new this.mover(this.node, e, this);
	},
	onMoveStart: function(/*Mover*/ mover){
		// summary:
		//		called before every move operation
		topic.publish("/dnd/move/start", mover);
		domClass.add(win.body(), "dojoMove");
		domClass.add(this.node, "dojoMoveItem");
	},
	onMoveStop: function(/*Mover*/ mover){
		// summary:
		//		called after every move operation
		topic.publish("/dnd/move/stop", mover);
		domClass.remove(win.body(), "dojoMove");
		domClass.remove(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/*===== mover, e =====*/){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		// mover: Mover
		// e: Event

		// default implementation does nothing
	},
	onMove: function(mover, leftTop /*=====, e =====*/){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		// mover: Mover
		// leftTop: Object
		// e: Event
		this.onMoving(mover, leftTop);
		var s = mover.node.style;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/*===== mover, leftTop =====*/){
		// summary:
		//		called before every incremental move; can be overwritten.
		// mover: Mover
		// leftTop: Object

		// default implementation does nothing
	},
	onMoved: function(/*===== mover, leftTop =====*/){
		// summary:
		//		called after every incremental move; can be overwritten.
		// mover: Mover
		// leftTop: Object

		// default implementation does nothing
	}
});

/*=====
Moveable.__MoveableArgs = declare([], {
	// handle: Node||String
	//		A node (or node's id), which is used as a mouse handle.
	//		If omitted, the node itself is used as a handle.
	handle: null,

	// delay: Number
	//		delay move by this number of pixels
	delay: 0,

	// skip: Boolean
	//		skip move of form elements
	skip: false,

	// mover: Object
	//		a constructor of custom Mover
	mover: dnd.Mover
});
=====*/

return Moveable;
});

},
'dojo/dnd/Mover':function(){
define([
	"../_base/array", "../_base/declare", "../_base/lang", "../sniff", "../_base/window",
	"../dom", "../dom-geometry", "../dom-style", "../Evented", "../on", "../touch", "./common", "./autoscroll"
], function(array, declare, lang, has, win, dom, domGeom, domStyle, Evented, on, touch, dnd, autoscroll){

// module:
//		dojo/dnd/Mover

return declare("dojo.dnd.Mover", [Evented], {
	// summary:
	//		an object which makes a node follow the mouse, or touch-drag on touch devices.
	//		Used as a default mover, and as a base class for custom movers.

	constructor: function(node, e, host){
		// node: Node
		//		a node (or node's id) to be moved
		// e: Event
		//		a mouse event, which started the move;
		//		only pageX and pageY properties are used
		// host: Object?
		//		object which implements the functionality of the move,
		//	 	and defines proper events (onMoveStart and onMoveStop)
		this.node = dom.byId(node);
		this.marginBox = {l: e.pageX, t: e.pageY};
		this.mouseButton = e.button;
		var h = (this.host = host), d = node.ownerDocument;

		function stopEvent(e){
			e.preventDefault();
			e.stopPropagation();
		}

		this.events = [
			// At the start of a drag, onFirstMove is called, and then the following
			// listener is disconnected.
			on(d, touch.move, lang.hitch(this, "onFirstMove")),

			// These are called continually during the drag
			on(d, touch.move, lang.hitch(this, "onMouseMove")),

			// And these are called at the end of the drag
			on(d, touch.release,  lang.hitch(this, "onMouseUp")),

			// cancel text selection and text dragging
			on(d, "dragstart",   stopEvent),
			on(d.body, "selectstart", stopEvent)
		];

		// Tell autoscroll that a drag is starting
		autoscroll.autoScrollStart(d);

		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove
		// e: Event
		//		mouse/touch event
		autoscroll.autoScroll(e);
		var m = this.marginBox;
		this.host.onMove(this, {l: m.l + e.pageX, t: m.t + e.pageY}, e);
		e.preventDefault();
		e.stopPropagation();
	},
	onMouseUp: function(e){
		if(has("webkit") && has("mac") && this.mouseButton == 2 ?
				e.button == 0 : this.mouseButton == e.button){ // TODO Should condition be met for touch devices, too?
			this.destroy();
		}
		e.preventDefault();
		e.stopPropagation();
	},
	// utilities
	onFirstMove: function(e){
		// summary:
		//		makes the node absolute; it is meant to be called only once.
		//		relative and absolutely positioned nodes are assumed to use pixel units
		var s = this.node.style, l, t, h = this.host;
		switch(s.position){
			case "relative":
			case "absolute":
				// assume that left and top values are in pixels already
				l = Math.round(parseFloat(s.left)) || 0;
				t = Math.round(parseFloat(s.top)) || 0;
				break;
			default:
				s.position = "absolute";	// enforcing the absolute mode
				var m = domGeom.getMarginBox(this.node);
				// event.pageX/pageY (which we used to generate the initial
				// margin box) includes padding and margin set on the body.
				// However, setting the node's position to absolute and then
				// doing domGeom.marginBox on it *doesn't* take that additional
				// space into account - so we need to subtract the combined
				// padding and margin.  We use getComputedStyle and
				// _getMarginBox/_getContentBox to avoid the extra lookup of
				// the computed style.
				var b = win.doc.body;
				var bs = domStyle.getComputedStyle(b);
				var bm = domGeom.getMarginBox(b, bs);
				var bc = domGeom.getContentBox(b, bs);
				l = m.l - (bc.l - bm.l);
				t = m.t - (bc.t - bm.t);
				break;
		}
		this.marginBox.l = l - this.marginBox.l;
		this.marginBox.t = t - this.marginBox.t;
		if(h && h.onFirstMove){
			h.onFirstMove(this, e);
		}

		// Disconnect touch.move that call this function
		this.events.shift().remove();
	},
	destroy: function(){
		// summary:
		//		stops the move, deletes all references, so the object can be garbage-collected
		array.forEach(this.events, function(handle){ handle.remove(); });
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = this.host = null;
	}
});

});

},
'dojo/dnd/TimedMoveable':function(){
define(["../_base/declare", "./Moveable" /*=====, "./Mover" =====*/], function(declare, Moveable /*=====, Mover =====*/){
	// module:
	//		dojo/dnd/TimedMoveable

	/*=====
	var __TimedMoveableArgs = declare([Moveable.__MoveableArgs], {
		// timeout: Number
		//		delay move by this number of ms,
		//		accumulating position changes during the timeout
		timeout: 0
	});
	=====*/

	// precalculate long expressions
	var oldOnMove = Moveable.prototype.onMove;

	return declare("dojo.dnd.TimedMoveable", Moveable, {
		// summary:
		//		A specialized version of Moveable to support an FPS throttling.
		//		This class puts an upper restriction on FPS, which may reduce
		//		the CPU load. The additional parameter "timeout" regulates
		//		the delay before actually moving the moveable object.

		// object attributes (for markup)
		timeout: 40,	// in ms, 40ms corresponds to 25 fps

		constructor: function(node, params){
			// summary:
			//		an object that makes a node moveable with a timer
			// node: Node||String
			//		a node (or node's id) to be moved
			// params: __TimedMoveableArgs
			//		object with additional parameters.

			// sanitize parameters
			if(!params){ params = {}; }
			if(params.timeout && typeof params.timeout == "number" && params.timeout >= 0){
				this.timeout = params.timeout;
			}
		},

		onMoveStop: function(/*Mover*/ mover){
			if(mover._timer){
				// stop timer
				clearTimeout(mover._timer);
				// reflect the last received position
				oldOnMove.call(this, mover, mover._leftTop);
			}
			Moveable.prototype.onMoveStop.apply(this, arguments);
		},
		onMove: function(/*Mover*/ mover, /*Object*/ leftTop){
			mover._leftTop = leftTop;
			if(!mover._timer){
				var _t = this;	// to avoid using dojo.hitch()
				mover._timer = setTimeout(function(){
					// we don't have any pending requests
					mover._timer = null;
					// reflect the last received position
					oldOnMove.call(_t, mover, mover._leftTop);
				}, this.timeout);
			}
		}
	});
});

},
'dijit/DialogUnderlay':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch
	"dojo/aspect", // aspect.after
	"dojo/dom-attr", // domAttr.set
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/on",
	"dojo/window", // winUtils.getBox, winUtils.get
	"./_Widget",
	"./_TemplatedMixin",
	"./BackgroundIframe",
	"./Viewport",
	"./main" // for back-compat, exporting dijit._underlay (remove in 2.0)
], function(declare, lang, aspect, domAttr, domStyle, on,
			winUtils, _Widget, _TemplatedMixin, BackgroundIframe, Viewport, dijit){

	// module:
	//		dijit/DialogUnderlay

	var DialogUnderlay = declare("dijit.DialogUnderlay", [_Widget, _TemplatedMixin], {
		// summary:
		//		A component used to block input behind a `dijit/Dialog`.
		//
		//		Normally this class should not be instantiated directly, but rather shown and hidden via
		//		DialogUnderlay.show() and DialogUnderlay.hide().  And usually the module is not accessed directly
		//		at all, since the underlay is shown and hidden by Dialog.DialogLevelManager.
		//
		//		The underlay itself can be styled based on and id:
		//	|	#myDialog_underlay { background-color:red; }
		//
		//		In the case of `dijit.Dialog`, this id is based on the id of the Dialog,
		//		suffixed with _underlay.

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class='dijitDialogUnderlayWrapper'><div class='dijitDialogUnderlay' tabIndex='-1' data-dojo-attach-point='node'></div></div>",

		// Parameters on creation or updatable later

		// dialogId: String
		//		Id of the dialog.... DialogUnderlay's id is based on this id
		dialogId: "",

		// class: String
		//		This class name is used on the DialogUnderlay node, in addition to dijitDialogUnderlay
		"class": "",

		// This will get overwritten as soon as show() is call, but leave an empty array in case hide() or destroy()
		// is called first.   The array is shared between instances but that's OK because we never write into it.
		_modalConnects: [],

		_setDialogIdAttr: function(id){
			domAttr.set(this.node, "id", id + "_underlay");
			this._set("dialogId", id);
		},

		_setClassAttr: function(clazz){
			this.node.className = "dijitDialogUnderlay " + clazz;
			this._set("class", clazz);
		},

		postCreate: function(){
			// Append the underlay to the body
			this.ownerDocumentBody.appendChild(this.domNode);

			this.own(on(this.domNode, "keydown", lang.hitch(this, "_onKeyDown")));

			this.inherited(arguments);
		},

		layout: function(){
			// summary:
			//		Sets the background to the size of the viewport
			//
			// description:
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.
			// tags:
			//		private

			var is = this.node.style,
				os = this.domNode.style;

			// hide the background temporarily, so that the background itself isn't
			// causing scrollbars to appear (might happen when user shrinks browser
			// window and then we are called to resize)
			os.display = "none";

			// then resize and show
			var viewport = winUtils.getBox(this.ownerDocument);
			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";
			os.display = "block";
		},

		show: function(){
			// summary:
			//		Show the dialog underlay
			this.domNode.style.display = "block";
			this.open = true;
			this.layout();
			this.bgIframe = new BackgroundIframe(this.domNode);

			var win = winUtils.get(this.ownerDocument);
			this._modalConnects = [
				Viewport.on("resize", lang.hitch(this, "layout")),
				on(win, "scroll", lang.hitch(this, "layout"))
			];

		},

		hide: function(){
			// summary:
			//		Hides the dialog underlay

			this.bgIframe.destroy();
			delete this.bgIframe;
			this.domNode.style.display = "none";
			while(this._modalConnects.length){ (this._modalConnects.pop()).remove(); }
			this.open = false;
		},

		destroy: function(){
			while(this._modalConnects.length){ (this._modalConnects.pop()).remove(); }
			this.inherited(arguments);
		},

		_onKeyDown: function(){
			// summary:
			//		Extension point so Dialog can monitor keyboard events on the underlay.
		}
	});

	DialogUnderlay.show = function(/*Object*/ attrs, /*Number*/ zIndex){
		// summary:
		//		Display the underlay with the given attributes set.  If the underlay is already displayed,
		//		then adjust it's attributes as specified.
		// attrs:
		//		The parameters to create DialogUnderlay with.
		// zIndex:
		//		zIndex of the underlay

		var underlay = DialogUnderlay._singleton;
		if(!underlay || underlay._destroyed){
			underlay = dijit._underlay = DialogUnderlay._singleton = new DialogUnderlay(attrs);
		}else{
			if(attrs){ underlay.set(attrs); }
		}
		domStyle.set(underlay.domNode, 'zIndex', zIndex);
		if(!underlay.open){
			underlay.show();
		}
	};

	DialogUnderlay.hide = function(){
		// summary:
		//		Hide the underlay.

		// Guard code in case the underlay widget has already been destroyed
		// because we are being called during page unload (when all widgets are destroyed)
		var underlay = DialogUnderlay._singleton;
		if(underlay && !underlay._destroyed){
			underlay.hide();
		}
	};

	return DialogUnderlay;
});

},
'dijit/form/CheckBox':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/has",		// has("dijit-legacy-requires")
	"dojo/query", // query
	"dojo/ready",
	"./ToggleButton",
	"./_CheckBoxMixin",
	"dojo/text!./templates/CheckBox.html",
	"dojo/NodeList-dom", // NodeList.addClass/removeClass
	"../a11yclick"	// template uses ondijitclick
], function(require, declare, domAttr, has, query, ready, ToggleButton, _CheckBoxMixin, template){

	// module:
	//		dijit/form/CheckBox

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/form/RadioButton"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.form.CheckBox", [ToggleButton, _CheckBoxMixin], {
		// summary:
		//		Same as an HTML checkbox, but with fancy styling.
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//
		//		There are two modes:
		//
		//		1. High contrast mode
		//		2. Normal mode
		//
		//		In case 1, the regular html inputs are shown and used by the user.
		//		In case 2, the regular html inputs are invisible but still used by
		//		the user. They are turned quasi-invisible and overlay the background-image.

		templateString: template,

		baseClass: "dijitCheckBox",

		_setValueAttr: function(/*String|Boolean*/ newValue, /*Boolean*/ priorityChange){
			// summary:
			//		Handler for value= attribute to constructor, and also calls to
			//		set('value', val).
			// description:
			//		During initialization, just saves as attribute to the `<input type=checkbox>`.
			//
			//		After initialization,
			//		when passed a boolean, controls whether or not the CheckBox is checked.
			//		If passed a string, changes the value attribute of the CheckBox (the one
			//		specified as "value" when the CheckBox was constructed
			//		(ex: `<input data-dojo-type="dijit/CheckBox" value="chicken">`).
			//
			//		`widget.set('value', string)` will check the checkbox and change the value to the
			//		specified string.
			//
			//		`widget.set('value', boolean)` will change the checked state.

			if(typeof newValue == "string"){
				this.inherited(arguments);
				newValue = true;
			}
			if(this._created){
				this.set('checked', newValue, priorityChange);
			}
		},
		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			// description:
			//		If the CheckBox is checked, returns the value attribute.
			//		Otherwise returns false.
			return this.checked && this._get("value");
		},

		// Override behavior from Button, since we don't have an iconNode or valueNode
		_setIconClassAttr: null,
		_setNameAttr: "focusNode",

		postMixInProperties: function(){
			this.inherited(arguments);

			// Need to set initial checked state via node.setAttribute so that form submit works
			// and IE8 radio button tab order is preserved.
			// domAttr.set(node, "checked", bool) doesn't work on IE until node has been attached
			// to <body>, see #8666
			this.checkedAttrSetting = "";
		},

		 _fillContent: function(){
			// Override Button::_fillContent() since it doesn't make sense for CheckBox,
			// since CheckBox doesn't even have a container
		},

		_onFocus: function(){
			if(this.id){
				query("label[for='"+this.id+"']").addClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		},

		_onBlur: function(){
			if(this.id){
				query("label[for='"+this.id+"']").removeClass("dijitFocusedLabel");
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/ToggleButton':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // kernel.deprecated
	"./Button",
	"./_ToggleButtonMixin"
], function(declare, kernel, Button, _ToggleButtonMixin){

	// module:
	//		dijit/form/ToggleButton


	return declare("dijit.form.ToggleButton", [Button, _ToggleButtonMixin], {
		// summary:
		//		A templated button widget that can be in two states (checked or not).
		//		Can be base class for things like tabs or checkbox or radio buttons.

		baseClass: "dijitToggleButton",

		setChecked: function(/*Boolean*/ checked){
			// summary:
			//		Deprecated.  Use set('checked', true/false) instead.
			kernel.deprecated("setChecked("+checked+") is deprecated. Use set('checked',"+checked+") instead.", "", "2.0");
			this.set('checked', checked);
		}
	});
});

},
'dijit/form/_ToggleButtonMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr" // domAttr.set
], function(declare, domAttr){

	// module:
	//		dijit/form/_ToggleButtonMixin

	return declare("dijit.form._ToggleButtonMixin", null, {
		// summary:
		//		A mixin to provide functionality to allow a button that can be in two states (checked or not).

		// checked: Boolean
		//		Corresponds to the native HTML `<input>` element's attribute.
		//		In markup, specified as "checked='checked'" or just "checked".
		//		True if the button is depressed, or the checkbox is checked,
		//		or the radio button is selected, etc.
		checked: false,

		// aria-pressed for toggle buttons, and aria-checked for checkboxes
		_aria_attr: "aria-pressed",

		_onClick: function(/*Event*/ evt){
			var original = this.checked;
			this._set('checked', !original); // partially set the toggled value, assuming the toggle will work, so it can be overridden in the onclick handler
			var ret = this.inherited(arguments); // the user could reset the value here
			this.set('checked', ret ? this.checked : original); // officially set the toggled or user value, or reset it back
			return ret;
		},

		_setCheckedAttr: function(/*Boolean*/ value, /*Boolean?*/ priorityChange){
			this._set("checked", value);
			var node = this.focusNode || this.domNode;
			if(this._created){ // IE is not ready to handle checked attribute (affects tab order)
				// needlessly setting "checked" upsets IE's tab order
				if(domAttr.get(node, "checked") != !!value){
					domAttr.set(node, "checked", !!value); // "mixed" -> true
				}
			}
			node.setAttribute(this._aria_attr, String(value)); // aria values should be strings
			this._handleOnChange(value, priorityChange);
		},

		postCreate: function(){ // use postCreate instead of startup so users forgetting to call startup are OK
			this.inherited(arguments);
			var node = this.focusNode || this.domNode;
			if(this.checked){
				// need this here instead of on the template so IE8 tab order works
				node.setAttribute('checked', 'checked');
			}
		},

		reset: function(){
			// summary:
			//		Reset the widget's value to what it was at initialization time

			this._hasBeenBlurred = false;

			// set checked state to original setting
			this.set('checked', this.params.checked || false);
		}
	});
});

},
'dijit/form/_CheckBoxMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr" // domAttr.set
], function(declare, domAttr){

	// module:
	//		dijit/form/_CheckBoxMixin

	return declare("dijit.form._CheckBoxMixin", null, {
		// summary:
		//		Mixin to provide widget functionality corresponding to an HTML checkbox
		//
		// description:
		//		User interacts with real html inputs.
		//		On onclick (which occurs by mouse click, space-bar, or
		//		using the arrow keys to switch the selected radio button),
		//		we update the state of the checkbox/radio.
		//

		// type: [private] String
		//		type attribute on `<input>` node.
		//		Overrides `dijit/form/Button.type`.  Users should not change this value.
		type: "checkbox",

		// value: String
		//		As an initialization parameter, equivalent to value field on normal checkbox
		//		(if checked, the value is passed as the value when form is submitted).
		value: "on",

		// readOnly: Boolean
		//		Should this widget respond to user input?
		//		In markup, this is specified as "readOnly".
		//		Similar to disabled except readOnly form values are submitted.
		readOnly: false,

		// aria-pressed for toggle buttons, and aria-checked for checkboxes
		_aria_attr: "aria-checked",

		_setReadOnlyAttr: function(/*Boolean*/ value){
			this._set("readOnly", value);
			domAttr.set(this.focusNode, 'readOnly', value);
		},

		// Override dijit/form/Button._setLabelAttr() since we don't even have a containerNode.
		// Normally users won't try to set label, except when CheckBox or RadioButton is the child of a dojox/layout/TabContainer
		_setLabelAttr: undefined,

		_getSubmitValue: function(/*String*/ value){
			return (value == null || value === "") ? "on" : value;
		},

		_setValueAttr: function(newValue){
			newValue = this._getSubmitValue(newValue);	// "on" to match browser native behavior when value unspecified
			this._set("value", newValue);
			domAttr.set(this.focusNode, "value", newValue);
		},

		reset: function(){
			this.inherited(arguments);
			// Handle unlikely event that the <input type=checkbox> value attribute has changed
			this._set("value", this._getSubmitValue(this.params.value));
			domAttr.set(this.focusNode, 'value', this.value);
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Internal function to handle click actions - need to check
			//		readOnly, since button no longer does that check.
			if(this.readOnly){
				e.stopPropagation();
				e.preventDefault();
				return false;
			}
			return this.inherited(arguments);
		}
	});
});

},
'dijit/Tree':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.map
	"dojo/aspect",
	"dojo/_base/connect", // connect.isCopyKey()
	"dojo/cookie", // cookie
	"dojo/_base/declare", // declare
	"dojo/Deferred", // Deferred
	"dojo/promise/all",
	"dojo/dom", // dom.isDescendant
	"dojo/dom-class", // domClass.add domClass.remove domClass.replace domClass.toggle
	"dojo/dom-geometry", // domGeometry.setMarginBox domGeometry.position
	"dojo/dom-style", // domStyle.set
	"dojo/errors/create", // createError
	"dojo/fx", // fxUtils.wipeIn fxUtils.wipeOut
	"dojo/has",
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/keys", // arrows etc.
	"dojo/_base/lang", // lang.getObject lang.mixin lang.hitch
	"dojo/on", // on(), on.selector()
	"dojo/topic",
	"dojo/touch",
	"dojo/when",
	"./a11yclick",
	"./focus",
	"./registry", // registry.byNode(), registry.getEnclosingWidget()
	"./_base/manager", // manager.defaultDuration
	"./_Widget",
	"./_TemplatedMixin",
	"./_Container",
	"./_Contained",
	"./_CssStateMixin",
	"./_KeyNavMixin",
	"dojo/text!./templates/TreeNode.html",
	"dojo/text!./templates/Tree.html",
	"./tree/TreeStoreModel",
	"./tree/ForestStoreModel",
	"./tree/_dndSelector",
	"dojo/query!css2"	// needed when on.selector() used with a string for the selector
], function(array, aspect, connect, cookie, declare, Deferred, all,
			dom, domClass, domGeometry, domStyle, createError, fxUtils, has, kernel, keys, lang, on, topic, touch, when,
			a11yclick, focus, registry, manager, _Widget, _TemplatedMixin, _Container, _Contained, _CssStateMixin, _KeyNavMixin,
			treeNodeTemplate, treeTemplate, TreeStoreModel, ForestStoreModel, _dndSelector){

	// module:
	//		dijit/Tree

	function shimmedPromise(/*Deferred|Promise*/ d){
		// summary:
		//		Return a Promise based on given Deferred or Promise, with back-compat addCallback() and addErrback() shims
		//		added (TODO: remove those back-compat shims, and this method, for 2.0)

		return lang.delegate(d.promise || d, {
			addCallback: function(callback){
				this.then(callback);
			},
			addErrback: function(errback){
				this.otherwise(errback);
			}
		});
	}

	var TreeNode = declare("dijit._TreeNode", [_Widget, _TemplatedMixin, _Container, _Contained, _CssStateMixin], {
		// summary:
		//		Single node within a tree.   This class is used internally
		//		by Tree and should not be accessed directly.
		// tags:
		//		private

		// item: [const] Item
		//		the dojo.data entry this tree represents
		item: null,

		// isTreeNode: [protected] Boolean
		//		Indicates that this is a TreeNode.   Used by `dijit.Tree` only,
		//		should not be accessed directly.
		isTreeNode: true,

		// label: String
		//		Text of this tree node
		label: "",
		_setLabelAttr: function(val){
			this.labelNode[this.labelType == "html" ? "innerHTML" : "innerText" in this.labelNode ?
				"innerText" : "textContent"] = val;
			this._set("label", val);
		},

		// labelType: [const] String
		//		Specifies how to interpret the label.  Can be "html" or "text".
		labelType: "text",

		// isExpandable: [private] Boolean
		//		This node has children, so show the expando node (+ sign)
		isExpandable: null,

		// isExpanded: [readonly] Boolean
		//		This node is currently expanded (ie, opened)
		isExpanded: false,

		// state: [private] String
		//		Dynamic loading-related stuff.
		//		When an empty folder node appears, it is "NotLoaded" first,
		//		then after dojo.data query it becomes "Loading" and, finally "Loaded"
		state: "NotLoaded",

		templateString: treeNodeTemplate,

		baseClass: "dijitTreeNode",

		// For hover effect for tree node, and focus effect for label
		cssStateNodes: {
			rowNode: "dijitTreeRow"
		},

		// Tooltip is defined in _WidgetBase but we need to handle the mapping to DOM here
		_setTooltipAttr: {node: "rowNode", type: "attribute", attribute: "title"},

		buildRendering: function(){
			this.inherited(arguments);

			// set expand icon for leaf
			this._setExpando();

			// set icon and label class based on item
			this._updateItemClasses(this.item);

			if(this.isExpandable){
				this.labelNode.setAttribute("aria-expanded", this.isExpanded);
			}

			//aria-selected should be false on all selectable elements.
			this.setSelected(false);
		},

		_setIndentAttr: function(indent){
			// summary:
			//		Tell this node how many levels it should be indented
			// description:
			//		0 for top level nodes, 1 for their children, 2 for their
			//		grandchildren, etc.

			// Math.max() is to prevent negative padding on hidden root node (when indent == -1)
			var pixels = (Math.max(indent, 0) * this.tree._nodePixelIndent) + "px";

			domStyle.set(this.domNode, "backgroundPosition", pixels + " 0px");	// TODOC: what is this for???
			domStyle.set(this.rowNode, this.isLeftToRight() ? "paddingLeft" : "paddingRight", pixels);

			array.forEach(this.getChildren(), function(child){
				child.set("indent", indent + 1);
			});

			this._set("indent", indent);
		},

		markProcessing: function(){
			// summary:
			//		Visually denote that tree is loading data, etc.
			// tags:
			//		private
			this.state = "Loading";
			this._setExpando(true);
		},

		unmarkProcessing: function(){
			// summary:
			//		Clear markup from markProcessing() call
			// tags:
			//		private
			this._setExpando(false);
		},

		_updateItemClasses: function(item){
			// summary:
			//		Set appropriate CSS classes for icon and label dom node
			//		(used to allow for item updates to change respective CSS)
			// tags:
			//		private
			var tree = this.tree, model = tree.model;
			if(tree._v10Compat && item === model.root){
				// For back-compat with 1.0, need to use null to specify root item (TODO: remove in 2.0)
				item = null;
			}
			this._applyClassAndStyle(item, "icon", "Icon");
			this._applyClassAndStyle(item, "label", "Label");
			this._applyClassAndStyle(item, "row", "Row");

			this.tree._startPaint(true);		// signifies paint started and finished (synchronously)
		},

		_applyClassAndStyle: function(item, lower, upper){
			// summary:
			//		Set the appropriate CSS classes and styles for labels, icons and rows.
			//
			// item:
			//		The data item.
			//
			// lower:
			//		The lower case attribute to use, e.g. 'icon', 'label' or 'row'.
			//
			// upper:
			//		The upper case attribute to use, e.g. 'Icon', 'Label' or 'Row'.
			//
			// tags:
			//		private

			var clsName = "_" + lower + "Class";
			var nodeName = lower + "Node";
			var oldCls = this[clsName];

			this[clsName] = this.tree["get" + upper + "Class"](item, this.isExpanded);
			domClass.replace(this[nodeName], this[clsName] || "", oldCls || "");

			domStyle.set(this[nodeName], this.tree["get" + upper + "Style"](item, this.isExpanded) || {});
		},

		_updateLayout: function(){
			// summary:
			//		Set appropriate CSS classes for this.domNode
			// tags:
			//		private
			var parent = this.getParent();
			if(!parent || !parent.rowNode || parent.rowNode.style.display == "none"){
				/* if we are hiding the root node then make every first level child look like a root node */
				domClass.add(this.domNode, "dijitTreeIsRoot");
			}else{
				domClass.toggle(this.domNode, "dijitTreeIsLast", !this.getNextSibling());
			}
		},

		_setExpando: function(/*Boolean*/ processing){
			// summary:
			//		Set the right image for the expando node
			// tags:
			//		private

			var styles = ["dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
					"dijitTreeExpandoClosed", "dijitTreeExpandoLeaf"],
				_a11yStates = ["*", "-", "+", "*"],
				idx = processing ? 0 : (this.isExpandable ? (this.isExpanded ? 1 : 2) : 3);

			// apply the appropriate class to the expando node
			domClass.replace(this.expandoNode, styles[idx], styles);

			// provide a non-image based indicator for images-off mode
			this.expandoNodeText.innerHTML = _a11yStates[idx];

		},

		expand: function(){
			// summary:
			//		Show my children
			// returns:
			//		Promise that resolves when expansion is complete

			// If there's already an expand in progress or we are already expanded, just return
			if(this._expandDeferred){
				return shimmedPromise(this._expandDeferred);		// dojo/promise/Promise
			}

			// cancel in progress collapse operation
			if(this._collapseDeferred){
				this._collapseDeferred.cancel();
				delete this._collapseDeferred;
			}

			// All the state information for when a node is expanded, maybe this should be
			// set when the animation completes instead
			this.isExpanded = true;
			this.labelNode.setAttribute("aria-expanded", "true");
			if(this.tree.showRoot || this !== this.tree.rootNode){
				this.containerNode.setAttribute("role", "group");
			}
			domClass.add(this.contentNode, 'dijitTreeContentExpanded');
			this._setExpando();
			this._updateItemClasses(this.item);

			if(this == this.tree.rootNode && this.tree.showRoot){
				this.tree.domNode.setAttribute("aria-expanded", "true");
			}

			var wipeIn = fxUtils.wipeIn({
				node: this.containerNode,
				duration: manager.defaultDuration
			});

			// Deferred that fires when expand is complete
			var def = (this._expandDeferred = new Deferred(function(){
				// Canceller
				wipeIn.stop();
			}));

			aspect.after(wipeIn, "onEnd", function(){
				def.resolve(true);
			}, true);

			wipeIn.play();

			return shimmedPromise(def);		// dojo/promise/Promise
		},

		collapse: function(){
			// summary:
			//		Collapse this node (if it's expanded)
			// returns:
			//		Promise that resolves when collapse is complete

			if(this._collapseDeferred){
				// Node is already collapsed, or there's a collapse in progress, just return that Deferred
				return shimmedPromise(this._collapseDeferred);
			}

			// cancel in progress expand operation
			if(this._expandDeferred){
				this._expandDeferred.cancel();
				delete this._expandDeferred;
			}

			this.isExpanded = false;
			this.labelNode.setAttribute("aria-expanded", "false");
			if(this == this.tree.rootNode && this.tree.showRoot){
				this.tree.domNode.setAttribute("aria-expanded", "false");
			}
			domClass.remove(this.contentNode, 'dijitTreeContentExpanded');
			this._setExpando();
			this._updateItemClasses(this.item);

			var wipeOut = fxUtils.wipeOut({
				node: this.containerNode,
				duration: manager.defaultDuration
			});

			// Deferred that fires when expand is complete
			var def = (this._collapseDeferred = new Deferred(function(){
				// Canceller
				wipeOut.stop();
			}));

			aspect.after(wipeOut, "onEnd", function(){
				def.resolve(true);
			}, true);

			wipeOut.play();

			return shimmedPromise(def);		// dojo/promise/Promise
		},

		// indent: Integer
		//		Levels from this node to the root node
		indent: 0,

		setChildItems: function(/* Object[] */ items){
			// summary:
			//		Sets the child items of this node, removing/adding nodes
			//		from current children to match specified items[] array.
			//		Also, if this.persist == true, expands any children that were previously
			//		opened.
			// returns:
			//		Promise that resolves after all previously opened children
			//		have been expanded again (or fires instantly if there are no such children).

			var tree = this.tree,
				model = tree.model,
				defs = [];	// list of deferreds that need to fire before I am complete


			// Orphan all my existing children.
			// If items contains some of the same items as before then we will reattach them.
			// Don't call this.removeChild() because that will collapse the tree etc.
			var oldChildren = this.getChildren();
			array.forEach(oldChildren, function(child){
				_Container.prototype.removeChild.call(this, child);
			}, this);

			// All the old children of this TreeNode are subject for destruction if
			//		1) they aren't listed in the new children array (items)
			//		2) they aren't immediately adopted by another node (DnD)
			this.defer(function(){
				array.forEach(oldChildren, function(node){
					if(!node._destroyed && !node.getParent()){
						// If node is in selection then remove it.
						tree.dndController.removeTreeNode(node);

						// Deregister mapping from item id --> this node and its descendants
						function remove(node){
							var id = model.getIdentity(node.item),
								ary = tree._itemNodesMap[id];
							if(ary.length == 1){
								delete tree._itemNodesMap[id];
							}else{
								var index = array.indexOf(ary, node);
								if(index != -1){
									ary.splice(index, 1);
								}
							}
							array.forEach(node.getChildren(), remove);
						}

						remove(node);

						// Remove any entries involving this node from cookie tracking expanded nodes
						if(tree.persist){
							var destroyedPath = array.map(node.getTreePath(),function(item){
								return tree.model.getIdentity(item);
							}).join("/");
							for(var path in tree._openedNodes){
								if(path.substr(0, destroyedPath.length) == destroyedPath){
									delete tree._openedNodes[path];
								}
							}
							tree._saveExpandedNodes();
						}

						// And finally we can destroy the node
						node.destroyRecursive();
					}
				});
			});

			this.state = "Loaded";

			if(items && items.length > 0){
				this.isExpandable = true;

				// Create _TreeNode widget for each specified tree node, unless one already
				// exists and isn't being used (presumably it's from a DnD move and was recently
				// released
				array.forEach(items, function(item){    // MARKER: REUSE NODE
					var id = model.getIdentity(item),
						existingNodes = tree._itemNodesMap[id],
						node;
					if(existingNodes){
						for(var i = 0; i < existingNodes.length; i++){
							if(existingNodes[i] && !existingNodes[i].getParent()){
								node = existingNodes[i];
								node.set('indent', this.indent + 1);
								break;
							}
						}
					}
					if(!node){
						node = this.tree._createTreeNode({
							item: item,
							tree: tree,
							isExpandable: model.mayHaveChildren(item),
							label: tree.getLabel(item),
							labelType: (tree.model && tree.model.labelType) || "text",
							tooltip: tree.getTooltip(item),
							ownerDocument: tree.ownerDocument,
							dir: tree.dir,
							lang: tree.lang,
							textDir: tree.textDir,
							indent: this.indent + 1
						});
						if(existingNodes){
							existingNodes.push(node);
						}else{
							tree._itemNodesMap[id] = [node];
						}
					}
					this.addChild(node);

					// If node was previously opened then open it again now (this may trigger
					// more data store accesses, recursively)
					if(this.tree.autoExpand || this.tree._state(node)){
						defs.push(tree._expandNode(node));
					}
				}, this);

				// note that updateLayout() needs to be called on each child after
				// _all_ the children exist
				array.forEach(this.getChildren(), function(child){
					child._updateLayout();
				});
			}else{
				this.isExpandable = false;
			}

			if(this._setExpando){
				// change expando to/from dot or + icon, as appropriate
				this._setExpando(false);
			}

			// Set leaf icon or folder icon, as appropriate
			this._updateItemClasses(this.item);

			var def = all(defs);
			this.tree._startPaint(def);		// to reset TreeNode widths after an item is added/removed from the Tree
			return shimmedPromise(def);		// dojo/promise/Promise
		},

		getTreePath: function(){
			var node = this;
			var path = [];
			while(node && node !== this.tree.rootNode){
				path.unshift(node.item);
				node = node.getParent();
			}
			path.unshift(this.tree.rootNode.item);

			return path;
		},

		getIdentity: function(){
			return this.tree.model.getIdentity(this.item);
		},

		removeChild: function(/* treeNode */ node){
			this.inherited(arguments);

			var children = this.getChildren();
			if(children.length == 0){
				this.isExpandable = false;
				this.collapse();
			}

			array.forEach(children, function(child){
				child._updateLayout();
			});
		},

		makeExpandable: function(){
			// summary:
			//		if this node wasn't already showing the expando node,
			//		turn it into one and call _setExpando()

			// TODO: hmm this isn't called from anywhere, maybe should remove it for 2.0

			this.isExpandable = true;
			this._setExpando(false);
		},

		setSelected: function(/*Boolean*/ selected){
			// summary:
			//		A Tree has a (single) currently selected node.
			//		Mark that this node is/isn't that currently selected node.
			// description:
			//		In particular, setting a node as selected involves setting tabIndex
			//		so that when user tabs to the tree, focus will go to that node (only).
			this.labelNode.setAttribute("aria-selected", selected ? "true" : "false");
			domClass.toggle(this.rowNode, "dijitTreeRowSelected", selected);
		},

		focus: function(){
			focus.focus(this.focusNode);
		}
	});

	if(has("dojo-bidi")){
		TreeNode.extend({
			_setTextDirAttr: function(textDir){
				if(textDir && ((this.textDir != textDir) || !this._created)){
					this._set("textDir", textDir);
					this.applyTextDir(this.labelNode);
					array.forEach(this.getChildren(), function(childNode){
						childNode.set("textDir", textDir);
					}, this);
				}
			}
		});
	}

	var Tree = declare("dijit.Tree", [_Widget, _KeyNavMixin, _TemplatedMixin, _CssStateMixin], {
		// summary:
		//		This widget displays hierarchical data from a store.

		baseClass: "dijitTree",

		// store: [deprecated] String|dojo/data/Store
		//		Deprecated.  Use "model" parameter instead.
		//		The store to get data to display in the tree.
		store: null,

		// model: [const] dijit/tree/model
		//		Interface to read tree data, get notifications of changes to tree data,
		//		and for handling drop operations (i.e drag and drop onto the tree)
		model: null,

		// query: [deprecated] anything
		//		Deprecated.  User should specify query to the model directly instead.
		//		Specifies datastore query to return the root item or top items for the tree.
		query: null,

		// label: [deprecated] String
		//		Deprecated.  Use dijit/tree/ForestStoreModel directly instead.
		//		Used in conjunction with query parameter.
		//		If a query is specified (rather than a root node id), and a label is also specified,
		//		then a fake root node is created and displayed, with this label.
		label: "",

		// showRoot: [const] Boolean
		//		Should the root node be displayed, or hidden?
		showRoot: true,

		// childrenAttr: [deprecated] String[]
		//		Deprecated.   This information should be specified in the model.
		//		One ore more attributes that holds children of a tree node
		childrenAttr: ["children"],

		// paths: String[][] or Item[][]
		//		Full paths from rootNode to selected nodes expressed as array of items or array of ids.
		//		Since setting the paths may be asynchronous (because of waiting on dojo.data), set("paths", ...)
		//		returns a Promise to indicate when the set is complete.
		paths: [],

		// path: String[] or Item[]
		//		Backward compatible singular variant of paths.
		path: [],

		// selectedItems: [readonly] Item[]
		//		The currently selected items in this tree.
		//		This property can only be set (via set('selectedItems', ...)) when that item is already
		//		visible in the tree.   (I.e. the tree has already been expanded to show that node.)
		//		Should generally use `paths` attribute to set the selected items instead.
		selectedItems: null,

		// selectedItem: [readonly] Item
		//		Backward compatible singular variant of selectedItems.
		selectedItem: null,

		// openOnClick: Boolean
		//		If true, clicking a folder node's label will open it, rather than calling onClick()
		openOnClick: false,

		// openOnDblClick: Boolean
		//		If true, double-clicking a folder node's label will open it, rather than calling onDblClick()
		openOnDblClick: false,

		templateString: treeTemplate,

		// persist: Boolean
		//		Enables/disables use of cookies for state saving.
		persist: false,

		// autoExpand: Boolean
		//		Fully expand the tree on load.   Overrides `persist`.
		autoExpand: false,

		// dndController: [protected] Function|String
		//		Class to use as as the dnd controller.  Specifying this class enables DnD.
		//		Generally you should specify this as dijit/tree/dndSource.
		//		Setting of dijit/tree/_dndSelector handles selection only (no actual DnD).
		dndController: _dndSelector,

		// parameters to pull off of the tree and pass on to the dndController as its params
		dndParams: ["onDndDrop", "itemCreator", "onDndCancel", "checkAcceptance", "checkItemAcceptance", "dragThreshold", "betweenThreshold"],

		//declare the above items so they can be pulled from the tree's markup

		// onDndDrop: [protected] Function
		//		Parameter to dndController, see `dijit/tree/dndSource.onDndDrop()`.
		//		Generally this doesn't need to be set.
		onDndDrop: null,

		itemCreator: null,
		/*=====
		itemCreator: function(nodes, target, source){
			// summary:
			//		Returns objects passed to `Tree.model.newItem()` based on DnD nodes
			//		dropped onto the tree.   Developer must override this method to enable
			//		dropping from external sources onto this Tree, unless the Tree.model's items
			//		happen to look like {id: 123, name: "Apple" } with no other attributes.
			//
			//		For each node in nodes[], which came from source, create a hash of name/value
			//		pairs to be passed to Tree.model.newItem().  Returns array of those hashes.
			// nodes: DomNode[]
			//		The DOMNodes dragged from the source container
			// target: DomNode
			//		The target TreeNode.rowNode
			// source: dojo/dnd/Source
			//		The source container the nodes were dragged from, perhaps another Tree or a plain dojo/dnd/Source
			// returns: Object[]
			//		Array of name/value hashes for each new item to be added to the Tree, like:
			// |	[
			// |		{ id: 123, label: "apple", foo: "bar" },
			// |		{ id: 456, label: "pear", zaz: "bam" }
			// |	]
			// tags:
			//		extension
			return [{}];
		},
		=====*/

		// onDndCancel: [protected] Function
		//		Parameter to dndController, see `dijit/tree/dndSource.onDndCancel()`.
		//		Generally this doesn't need to be set.
		onDndCancel: null,

		/*=====
		checkAcceptance: function(source, nodes){
			// summary:
			//		Checks if the Tree itself can accept nodes from this source
			// source: dijit/tree/dndSource
			//		The source which provides items
			// nodes: DOMNode[]
			//		Array of DOM nodes corresponding to nodes being dropped, dijitTreeRow nodes if
			//		source is a dijit/Tree.
			// tags:
			//		extension
			return true;	// Boolean
		},
		=====*/
		checkAcceptance: null,

		/*=====
		checkItemAcceptance: function(target, source, position){
			// summary:
			//		Stub function to be overridden if one wants to check for the ability to drop at the node/item level
			// description:
			//		In the base case, this is called to check if target can become a child of source.
			//		When betweenThreshold is set, position="before" or "after" means that we
			//		are asking if the source node can be dropped before/after the target node.
			// target: DOMNode
			//		The dijitTreeRoot DOM node inside of the TreeNode that we are dropping on to
			//		Use registry.getEnclosingWidget(target) to get the TreeNode.
			// source: dijit/tree/dndSource
			//		The (set of) nodes we are dropping
			// position: String
			//		"over", "before", or "after"
			// tags:
			//		extension
			return true;	// Boolean
		},
		=====*/
		checkItemAcceptance: null,

		// dragThreshold: Integer
		//		Number of pixels mouse moves before it's considered the start of a drag operation
		dragThreshold: 5,

		// betweenThreshold: Integer
		//		Set to a positive value to allow drag and drop "between" nodes.
		//
		//		If during DnD mouse is over a (target) node but less than betweenThreshold
		//		pixels from the bottom edge, dropping the the dragged node will make it
		//		the next sibling of the target node, rather than the child.
		//
		//		Similarly, if mouse is over a target node but less that betweenThreshold
		//		pixels from the top edge, dropping the dragged node will make it
		//		the target node's previous sibling rather than the target node's child.
		betweenThreshold: 0,

		// _nodePixelIndent: Integer
		//		Number of pixels to indent tree nodes (relative to parent node).
		//		Default is 19 but can be overridden by setting CSS class dijitTreeIndent
		//		and calling resize() or startup() on tree after it's in the DOM.
		_nodePixelIndent: 19,

		_publish: function(/*String*/ topicName, /*Object*/ message){
			// summary:
			//		Publish a message for this widget/topic
			topic.publish(this.id, lang.mixin({tree: this, event: topicName}, message || {}));	// publish
		},

		postMixInProperties: function(){
			this.tree = this;

			if(this.autoExpand){
				// There's little point in saving opened/closed state of nodes for a Tree
				// that initially opens all it's nodes.
				this.persist = false;
			}

			this._itemNodesMap = {};

			if(!this.cookieName && this.id){
				this.cookieName = this.id + "SaveStateCookie";
			}

			// Deferred that resolves when all the children have loaded.
			this.expandChildrenDeferred = new Deferred();

			// Promise that resolves when all pending operations complete.
			this.pendingCommandsPromise = this.expandChildrenDeferred.promise;

			this.inherited(arguments);
		},

		postCreate: function(){
			this._initState();

			// Catch events on TreeNodes
			var self = this;
			this.own(
				on(this.containerNode, on.selector(".dijitTreeNode", touch.enter), function(evt){
					self._onNodeMouseEnter(registry.byNode(this), evt);
				}),
				on(this.containerNode, on.selector(".dijitTreeNode", touch.leave), function(evt){
					self._onNodeMouseLeave(registry.byNode(this), evt);
				}),
				on(this.containerNode, a11yclick, function(evt){
					var node = registry.getEnclosingWidget(evt.target);
					if(node.isInstanceOf(TreeNode)){
						self._onClick(node, evt);
					}
				}),
				on(this.containerNode, on.selector(".dijitTreeNode", "dblclick"), function(evt){
					self._onDblClick(registry.byNode(this), evt);
				})
			);

			// Create glue between store and Tree, if not specified directly by user
			if(!this.model){
				this._store2model();
			}

			// monitor changes to items
			this.own(
				aspect.after(this.model, "onChange", lang.hitch(this, "_onItemChange"), true),
				aspect.after(this.model, "onChildrenChange", lang.hitch(this, "_onItemChildrenChange"), true),
				aspect.after(this.model, "onDelete", lang.hitch(this, "_onItemDelete"), true)
			);

			this.inherited(arguments);

			if(this.dndController){
				// TODO: remove string support in 2.0.
				if(lang.isString(this.dndController)){
					this.dndController = lang.getObject(this.dndController);
				}
				var params = {};
				for(var i = 0; i < this.dndParams.length; i++){
					if(this[this.dndParams[i]]){
						params[this.dndParams[i]] = this[this.dndParams[i]];
					}
				}
				this.dndController = new this.dndController(this, params);
			}

			this._load();

			// onLoadDeferred should fire when all commands that are part of initialization have completed.
			// It will include all the set("paths", ...) commands that happen during initialization.
			this.onLoadDeferred = shimmedPromise(this.pendingCommandsPromise);

			this.onLoadDeferred.then(lang.hitch(this, "onLoad"));
		},

		_store2model: function(){
			// summary:
			//		User specified a store&query rather than model, so create model from store/query
			this._v10Compat = true;
			kernel.deprecated("Tree: from version 2.0, should specify a model object rather than a store/query");

			var modelParams = {
				id: this.id + "_ForestStoreModel",
				store: this.store,
				query: this.query,
				childrenAttrs: this.childrenAttr
			};

			// Only override the model's mayHaveChildren() method if the user has specified an override
			if(this.params.mayHaveChildren){
				modelParams.mayHaveChildren = lang.hitch(this, "mayHaveChildren");
			}

			if(this.params.getItemChildren){
				modelParams.getChildren = lang.hitch(this, function(item, onComplete, onError){
					this.getItemChildren((this._v10Compat && item === this.model.root) ? null : item, onComplete, onError);
				});
			}
			this.model = new ForestStoreModel(modelParams);

			// For backwards compatibility, the visibility of the root node is controlled by
			// whether or not the user has specified a label
			this.showRoot = Boolean(this.label);
		},

		onLoad: function(){
			// summary:
			//		Called when tree finishes loading and expanding.
			// description:
			//		If persist == true the loading may encompass many levels of fetches
			//		from the data store, each asynchronous.   Waits for all to finish.
			// tags:
			//		callback
		},

		_load: function(){
			// summary:
			//		Initial load of the tree.
			//		Load root node (possibly hidden) and it's children.
			this.model.getRoot(
				lang.hitch(this, function(item){
					var rn = (this.rootNode = this.tree._createTreeNode({
						item: item,
						tree: this,
						isExpandable: true,
						label: this.label || this.getLabel(item),
						labelType: this.model.labelType || "text",
						textDir: this.textDir,
						indent: this.showRoot ? 0 : -1
					}));

					if(!this.showRoot){
						rn.rowNode.style.display = "none";
						// if root is not visible, move tree role to the invisible
						// root node's containerNode, see #12135
						this.domNode.setAttribute("role", "presentation");
						this.domNode.removeAttribute("aria-expanded");
						this.domNode.removeAttribute("aria-multiselectable");

						// move the aria-label or aria-labelledby to the element with the role
						if(this["aria-label"]){
							rn.containerNode.setAttribute("aria-label", this["aria-label"]);
							this.domNode.removeAttribute("aria-label");
						}else if(this["aria-labelledby"]){
							rn.containerNode.setAttribute("aria-labelledby", this["aria-labelledby"]);
							this.domNode.removeAttribute("aria-labelledby");
						}
						rn.labelNode.setAttribute("role", "presentation");
						rn.containerNode.setAttribute("role", "tree");
						rn.containerNode.setAttribute("aria-expanded", "true");
						rn.containerNode.setAttribute("aria-multiselectable", !this.dndController.singular);
					}else{
						this.domNode.setAttribute("aria-multiselectable", !this.dndController.singular);
						this.rootLoadingIndicator.style.display = "none";
					}

					this.containerNode.appendChild(rn.domNode);
					var identity = this.model.getIdentity(item);
					if(this._itemNodesMap[identity]){
						this._itemNodesMap[identity].push(rn);
					}else{
						this._itemNodesMap[identity] = [rn];
					}

					rn._updateLayout();		// sets "dijitTreeIsRoot" CSS classname

					// Load top level children, and if persist==true, all nodes that were previously opened
					this._expandNode(rn).then(lang.hitch(this, function(){
						// Then, select the nodes specified by params.paths[].

						this.rootLoadingIndicator.style.display = "none";
						this.expandChildrenDeferred.resolve(true);
					}));
				}),
				lang.hitch(this, function(err){
					console.error(this, ": error loading root: ", err);
				})
			);
		},

		getNodesByItem: function(/*Item or id*/ item){
			// summary:
			//		Returns all tree nodes that refer to an item
			// returns:
			//		Array of tree nodes that refer to passed item

			if(!item){
				return [];
			}
			var identity = lang.isString(item) ? item : this.model.getIdentity(item);
			// return a copy so widget don't get messed up by changes to returned array
			return [].concat(this._itemNodesMap[identity]);
		},

		_setSelectedItemAttr: function(/*Item or id*/ item){
			this.set('selectedItems', [item]);
		},

		_setSelectedItemsAttr: function(/*Items or ids*/ items){
			// summary:
			//		Select tree nodes related to passed items.
			//		WARNING: if model use multi-parented items or desired tree node isn't already loaded
			//		behavior is undefined. Use set('paths', ...) instead.
			var tree = this;
			return this.pendingCommandsPromise = this.pendingCommandsPromise.always(lang.hitch(this, function(){
				var identities = array.map(items, function(item){
					return (!item || lang.isString(item)) ? item : tree.model.getIdentity(item);
				});
				var nodes = [];
				array.forEach(identities, function(id){
					nodes = nodes.concat(tree._itemNodesMap[id] || []);
				});
				this.set('selectedNodes', nodes);
			}));
		},

		_setPathAttr: function(/*Item[]|String[]*/ path){
			// summary:
			//		Singular variant of _setPathsAttr
			if(path.length){
				return shimmedPromise(this.set("paths", [path]).then(function(paths){ return paths[0]; }));
			}else{
				// Empty list is interpreted as "select nothing"
				return shimmedPromise(this.set("paths", []).then(function(paths){ return paths[0]; }));
			}
		},

		_setPathsAttr: function(/*Item[][]|String[][]*/ paths){
			// summary:
			//		Select the tree nodes identified by passed paths.
			// paths:
			//		Array of arrays of items or item id's
			// returns:
			//		Promise to indicate when the set is complete

			var tree = this;

			function selectPath(path, nodes){
				// Traverse path, returning Promise for node at the end of the path.
				// The next path component should be among "nodes".
				var nextPath = path.shift();
				var nextNode = array.filter(nodes, function(node){
					return node.getIdentity() == nextPath;
				})[0];
				if(!!nextNode){
					if(path.length){
						return tree._expandNode(nextNode).then(function(){
							return selectPath(path, nextNode.getChildren());
						});
					}else{
						// Successfully reached the end of this path
						return nextNode;
					}
				}else{
					throw new Tree.PathError("Could not expand path at " + nextPath);
				}
			}

			// Let any previous set("path", ...) commands complete before this one starts.
			// TODO for 2.0: make the user do this wait themselves?
			return shimmedPromise(this.pendingCommandsPromise = this.pendingCommandsPromise.always(function(){
				// We may need to wait for some nodes to expand, so setting
				// each path will involve a Deferred. We bring those deferreds
				// together with a dojo/promise/all.
				return all(array.map(paths, function(path){
					// normalize path to use identity
					path = array.map(path, function(item){
						return lang.isString(item) ? item : tree.model.getIdentity(item);
					});

					if(path.length){
						return selectPath(path, [tree.rootNode]);
					}else{
						throw new Tree.PathError("Empty path");
					}
				}));
			}).then(function setNodes(newNodes){
				// After all expansion is finished, set the selection to last element from each path
				tree.set("selectedNodes", newNodes);
				return tree.paths;
			}));
		},

		_setSelectedNodeAttr: function(node){
			this.set('selectedNodes', [node]);
		},
		_setSelectedNodesAttr: function(nodes){
			// summary:
			//		Marks the specified TreeNodes as selected.
			// nodes: TreeNode[]
			//		TreeNodes to mark.
			this.dndController.setSelection(nodes);
		},


		expandAll: function(){
			// summary:
			//		Expand all nodes in the tree
			// returns:
			//		Promise that resolves when all nodes have expanded

			var _this = this;

			function expand(node){
				// Expand the node
				return _this._expandNode(node).then(function(){
					// When node has expanded, call expand() recursively on each non-leaf child
					var childBranches = array.filter(node.getChildren() || [], function(node){
						return node.isExpandable;
					});

					// And when all those recursive calls finish, signal that I'm finished
					return all(array.map(childBranches, expand));
				});
			}

			return shimmedPromise(expand(this.rootNode));
		},

		collapseAll: function(){
			// summary:
			//		Collapse all nodes in the tree
			// returns:
			//		Promise that resolves when all nodes have collapsed

			var _this = this;

			function collapse(node){
				// Collapse children first
				var childBranches = array.filter(node.getChildren() || [], function(node){
						return node.isExpandable;
					}),
					defs = all(array.map(childBranches, collapse));

				// And when all those recursive calls finish, collapse myself, unless I'm the invisible root node,
				// in which case collapseAll() is finished
				if(!node.isExpanded || (node == _this.rootNode && !_this.showRoot)){
					return defs;
				}else{
					// When node has collapsed, signal that call is finished
					return defs.then(function(){
						return _this._collapseNode(node);
					});
				}
			}

			return shimmedPromise(collapse(this.rootNode));
		},

		////////////// Data store related functions //////////////////////
		// These just get passed to the model; they are here for back-compat

		mayHaveChildren: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Deprecated.   This should be specified on the model itself.
			//
			//		Overridable function to tell if an item has or may have children.
			//		Controls whether or not +/- expando icon is shown.
			//		(For efficiency reasons we may not want to check if an element actually
			//		has children until user clicks the expando node)
			// tags:
			//		deprecated
		},

		getItemChildren: function(/*===== parentItem, onComplete =====*/){
			// summary:
			//		Deprecated.   This should be specified on the model itself.
			//
			//		Overridable function that return array of child items of given parent item,
			//		or if parentItem==null then return top items in tree
			// tags:
			//		deprecated
		},

		///////////////////////////////////////////////////////
		// Functions for converting an item to a TreeNode
		getLabel: function(/*dojo/data/Item*/ item){
			// summary:
			//		Overridable function to get the label for a tree node (given the item)
			// tags:
			//		extension
			return this.model.getLabel(item);	// String
		},

		getIconClass: function(/*dojo/data/Item*/ item, /*Boolean*/ opened){
			// summary:
			//		Overridable function to return CSS class name to display icon
			// tags:
			//		extension
			return (!item || this.model.mayHaveChildren(item)) ? (opened ? "dijitFolderOpened" : "dijitFolderClosed") : "dijitLeaf"
		},

		getLabelClass: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS class name to display label
			// item: dojo/data/Item
			// opened: Boolean
			// returns: String
			//		CSS class name
			// tags:
			//		extension
		},

		getRowClass: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS class name to display row
			// item: dojo/data/Item
			// opened: Boolean
			// returns: String
			//		CSS class name
			// tags:
			//		extension
		},

		getIconStyle: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS styles to display icon
			// item: dojo/data/Item
			// opened: Boolean
			// returns: Object
			//		Object suitable for input to dojo.style() like {backgroundImage: "url(...)"}
			// tags:
			//		extension
		},

		getLabelStyle: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS styles to display label
			// item: dojo/data/Item
			// opened: Boolean
			// returns:
			//		Object suitable for input to dojo.style() like {color: "red", background: "green"}
			// tags:
			//		extension
		},

		getRowStyle: function(/*===== item, opened =====*/){
			// summary:
			//		Overridable function to return CSS styles to display row
			// item: dojo/data/Item
			// opened: Boolean
			// returns:
			//		Object suitable for input to dojo.style() like {background-color: "#bbb"}
			// tags:
			//		extension
		},

		getTooltip: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Overridable function to get the tooltip for a tree node (given the item)
			// tags:
			//		extension
			return "";	// String
		},

		/////////// Keyboard and Mouse handlers ////////////////////


		_onDownArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		down arrow pressed; get next visible node, set focus there

			var nextNode = this._getNext(node);
			if(nextNode && nextNode.isTreeNode){
				this.focusNode(nextNode);
			}
		},

		_onUpArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Up arrow pressed; move to previous visible node

			// if younger siblings
			var previousSibling = node.getPreviousSibling();
			if(previousSibling){
				node = previousSibling;
				// if the previous node is expanded, dive in deep
				while(node.isExpandable && node.isExpanded && node.hasChildren()){
					// move to the last child
					var children = node.getChildren();
					node = children[children.length - 1];
				}
			}else{
				// if this is the first child, return the parent
				// unless the parent is the root of a tree with a hidden root
				var parent = node.getParent();
				if(!(!this.showRoot && parent === this.rootNode)){
					node = parent;
				}
			}

			if(node && node.isTreeNode){
				this.focusNode(node);
			}
		},

		_onRightArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Right arrow pressed; go to child node

			// if not expanded, expand, else move to 1st child
			if(node.isExpandable && !node.isExpanded){
				this._expandNode(node);
			}else if(node.hasChildren()){
				node = node.getChildren()[0];
				if(node && node.isTreeNode){
					this.focusNode(node);
				}
			}
		},

		_onLeftArrow: function(/*Event*/ evt, /*TreeNode*/ node){
			// summary:
			//		Left arrow pressed.
			//		If not collapsed, collapse, else move to parent.

			if(node.isExpandable && node.isExpanded){
				this._collapseNode(node);
			}else{
				var parent = node.getParent();
				if(parent && parent.isTreeNode && !(!this.showRoot && parent === this.rootNode)){
					this.focusNode(parent);
				}
			}
		},

		focusLastChild: function(){
			// summary:
			//		End key pressed; go to last visible node.

			var node = this._getLast();
			if(node && node.isTreeNode){
				this.focusNode(node);
			}
		},

		_getFirst: function(){
			// summary:
			//		Returns the first child.
			// tags:
			//		abstract extension
			return this.showRoot ? this.rootNode : this.rootNode.getChildren()[0];
		},

		_getLast: function(){
			// summary:
			//		Returns the last descendant.
			// tags:
			//		abstract extension
			var node = this.rootNode;
			while(node.isExpanded){
				var c = node.getChildren();
				if(!c.length){
					break;
				}
				node = c[c.length - 1];
			}
			return node;
		},

		// Tree only searches forward so dir parameter is unused
		_getNext: function(node){
			// summary:
			//		Returns the next descendant, compared to "child".
			// node: Widget
			//		The current widget
			// tags:
			//		abstract extension

			if(node.isExpandable && node.isExpanded && node.hasChildren()){
				// if this is an expanded node, get the first child
				return node.getChildren()[0];		// TreeNode
			}else{
				// find a parent node with a sibling
				while(node && node.isTreeNode){
					var returnNode = node.getNextSibling();
					if(returnNode){
						return returnNode;		// TreeNode
					}
					node = node.getParent();
				}
				return null;
			}
		},

		// Implement _KeyNavContainer.childSelector, to identify which nodes are navigable
		childSelector: ".dijitTreeRow",

		isExpandoNode: function(node, widget){
			// summary:
			//		check whether a dom node is the expandoNode for a particular TreeNode widget
			return dom.isDescendant(node, widget.expandoNode) || dom.isDescendant(node, widget.expandoNodeText);
		},

		__click: function(/*TreeNode*/ nodeWidget, /*Event*/ e, /*Boolean*/doOpen, /*String*/func){
			var domElement = e.target,
				isExpandoClick = this.isExpandoNode(domElement, nodeWidget);

			if(nodeWidget.isExpandable && (doOpen || isExpandoClick)){
				// expando node was clicked, or label of a folder node was clicked; open it
				this._onExpandoClick({node: nodeWidget});
			}else{
				this._publish("execute", { item: nodeWidget.item, node: nodeWidget, evt: e });
				this[func](nodeWidget.item, nodeWidget, e);
				this.focusNode(nodeWidget);
			}
			e.stopPropagation();
			e.preventDefault();
		},
		_onClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
			// summary:
			//		Translates click events into commands for the controller to process
			this.__click(nodeWidget, e, this.openOnClick, 'onClick');
		},
		_onDblClick: function(/*TreeNode*/ nodeWidget, /*Event*/ e){
			// summary:
			//		Translates double-click events into commands for the controller to process
			this.__click(nodeWidget, e, this.openOnDblClick, 'onDblClick');
		},

		_onExpandoClick: function(/*Object*/ message){
			// summary:
			//		User clicked the +/- icon; expand or collapse my children.
			var node = message.node;

			// If we are collapsing, we might be hiding the currently focused node.
			// Also, clicking the expando node might have erased focus from the current node.
			// For simplicity's sake just focus on the node with the expando.
			this.focusNode(node);

			if(node.isExpanded){
				this._collapseNode(node);
			}else{
				this._expandNode(node);
			}
		},

		onClick: function(/*===== item, node, evt =====*/){
			// summary:
			//		Callback when a tree node is clicked
			// item: Object
			//		Object from the dojo/store corresponding to this TreeNode
			// node: TreeNode
			//		The TreeNode itself
			// evt: Event
			//		The event
			// tags:
			//		callback
		},
		onDblClick: function(/*===== item, node, evt =====*/){
			// summary:
			//		Callback when a tree node is double-clicked
			// item: Object
			//		Object from the dojo/store corresponding to this TreeNode
			// node: TreeNode
			//		The TreeNode itself
			// evt: Event
			//		The event
			// tags:
			//		callback
		},
		onOpen: function(/*===== item, node =====*/){
			// summary:
			//		Callback when a node is opened
			// item: dojo/data/Item
			// node: TreeNode
			// tags:
			//		callback
		},
		onClose: function(/*===== item, node =====*/){
			// summary:
			//		Callback when a node is closed
			// item: Object
			//		Object from the dojo/store corresponding to this TreeNode
			// node: TreeNode
			//		The TreeNode itself
			// tags:
			//		callback
		},

		_getNextNode: function(node){
			// summary:
			//		Get next visible node

			kernel.deprecated(this.declaredClass + "::_getNextNode(node) is deprecated. Use _getNext(node) instead.", "", "2.0");
			return this._getNext(node);
		},

		_getRootOrFirstNode: function(){
			// summary:
			//		Get first visible node
			kernel.deprecated(this.declaredClass + "::_getRootOrFirstNode() is deprecated. Use _getFirst() instead.", "", "2.0");
			return this._getFirst();
		},

		_collapseNode: function(/*TreeNode*/ node){
			// summary:
			//		Called when the user has requested to collapse the node
			// returns:
			//		Promise that resolves when the node has finished closing

			if(node._expandNodeDeferred){
				delete node._expandNodeDeferred;
			}

			if(node.state == "Loading"){
				// ignore clicks while we are in the process of loading data
				return;
			}

			if(node.isExpanded){
				var ret = node.collapse();

				this.onClose(node.item, node);
				this._state(node, false);

				this._startPaint(ret);	// after this finishes, need to reset widths of TreeNodes

				return ret;
			}
		},

		_expandNode: function(/*TreeNode*/ node){
			// summary:
			//		Called when the user has requested to expand the node
			// returns:
			//		Promise that resolves when the node is loaded and opened and (if persist=true) all it's descendants
			//		that were previously opened too

			if(node._expandNodeDeferred){
				// there's already an expand in progress, or completed, so just return
				return node._expandNodeDeferred;	// dojo/Deferred
			}

			var model = this.model,
				item = node.item,
				_this = this;

			// Load data if it's not already loaded
			if(!node._loadDeferred){
				// need to load all the children before expanding
				node.markProcessing();

				// Setup deferred to signal when the load and expand are finished.
				// Save that deferred in this._expandDeferred as a flag that operation is in progress.
				node._loadDeferred = new Deferred();

				// Get the children
				model.getChildren(
					item,
					function(items){
						node.unmarkProcessing();

						// Display the children and also start expanding any children that were previously expanded
						// (if this.persist == true).   The returned Deferred will fire when those expansions finish.
						node.setChildItems(items).then(function(){
							node._loadDeferred.resolve(items);
						});
					},
					function(err){
						console.error(_this, ": error loading " + node.label + " children: ", err);
						node._loadDeferred.reject(err);
					}
				);
			}

			// Expand the node after data has loaded
			var def = node._loadDeferred.then(lang.hitch(this, function(){
				var def2 = node.expand();

				// seems like these should delayed until node.expand() completes, but left here for back-compat about
				// when this.isOpen flag gets set (ie, at the beginning of the animation)
				this.onOpen(node.item, node);
				this._state(node, true);

				return def2;
			}));

			this._startPaint(def);	// after this finishes, need to reset widths of TreeNodes

			return def;	// dojo/promise/Promise
		},

		////////////////// Miscellaneous functions ////////////////

		focusNode: function(/* _tree.Node */ node){
			// summary:
			//		Focus on the specified node (which must be visible)
			// tags:
			//		protected

			this.focusChild(node);
		},

		_onNodeMouseEnter: function(/*dijit/_WidgetBase*/ /*===== node =====*/){
			// summary:
			//		Called when mouse is over a node (onmouseenter event),
			//		this is monitored by the DND code
		},

		_onNodeMouseLeave: function(/*dijit/_WidgetBase*/ /*===== node =====*/){
			// summary:
			//		Called when mouse leaves a node (onmouseleave event),
			//		this is monitored by the DND code
		},

		//////////////// Events from the model //////////////////////////

		_onItemChange: function(/*Item*/ item){
			// summary:
			//		Processes notification of a change to an item's scalar values like label
			var model = this.model,
				identity = model.getIdentity(item),
				nodes = this._itemNodesMap[identity];

			if(nodes){
				var label = this.getLabel(item),
					tooltip = this.getTooltip(item);
				array.forEach(nodes, function(node){
					node.set({
						item: item, // theoretically could be new JS Object representing same item
						label: label,
						tooltip: tooltip
					});
					node._updateItemClasses(item);
				});
			}
		},

		_onItemChildrenChange: function(/*dojo/data/Item*/ parent, /*dojo/data/Item[]*/ newChildrenList){
			// summary:
			//		Processes notification of a change to an item's children
			var model = this.model,
				identity = model.getIdentity(parent),
				parentNodes = this._itemNodesMap[identity];

			if(parentNodes){
				array.forEach(parentNodes, function(parentNode){
					parentNode.setChildItems(newChildrenList);
				});
			}
		},

		_onItemDelete: function(/*Item*/ item){
			// summary:
			//		Processes notification of a deletion of an item.
			//		Not called from new dojo.store interface but there's cleanup code in setChildItems() instead.

			var model = this.model,
				identity = model.getIdentity(item),
				nodes = this._itemNodesMap[identity];

			if(nodes){
				array.forEach(nodes, function(node){
					// Remove node from set of selected nodes (if it's selected)
					this.dndController.removeTreeNode(node);

					var parent = node.getParent();
					if(parent){
						// if node has not already been orphaned from a _onSetItem(parent, "children", ..) call...
						parent.removeChild(node);
					}
					node.destroyRecursive();
				}, this);
				delete this._itemNodesMap[identity];
			}
		},

		/////////////// Miscellaneous funcs

		_initState: function(){
			// summary:
			//		Load in which nodes should be opened automatically
			this._openedNodes = {};
			if(this.persist && this.cookieName){
				var oreo = cookie(this.cookieName);
				if(oreo){
					array.forEach(oreo.split(','), function(item){
						this._openedNodes[item] = true;
					}, this);
				}
			}
		},

		_state: function(node, expanded){
			// summary:
			//		Query or set expanded state for an node
			if(!this.persist){
				return false;
			}
			var path = array.map(node.getTreePath(),function(item){
				return this.model.getIdentity(item);
			}, this).join("/");
			if(arguments.length === 1){
				return this._openedNodes[path];
			}else{
				if(expanded){
					this._openedNodes[path] = true;
				}else{
					delete this._openedNodes[path];
				}
				this._saveExpandedNodes();
			}
		},

		_saveExpandedNodes: function(){
			if(this.persist && this.cookieName){
				var ary = [];
				for(var id in this._openedNodes){
					ary.push(id);
				}
				cookie(this.cookieName, ary.join(","), {expires: 365});
			}
		},

		destroy: function(){
			if(this._curSearch){
				this._curSearch.timer.remove();
				delete this._curSearch;
			}
			if(this.rootNode){
				this.rootNode.destroyRecursive();
			}
			if(this.dndController && !lang.isString(this.dndController)){
				this.dndController.destroy();
			}
			this.rootNode = null;
			this.inherited(arguments);
		},

		destroyRecursive: function(){
			// A tree is treated as a leaf, not as a node with children (like a grid),
			// but defining destroyRecursive for back-compat.
			this.destroy();
		},

		resize: function(changeSize){
			if(changeSize){
				domGeometry.setMarginBox(this.domNode, changeSize);
			}

			// The main JS sizing involved w/tree is the indentation, which is specified
			// in CSS and read in through this dummy indentDetector node (tree must be
			// visible and attached to the DOM to read this).
			// If the Tree is hidden domGeometry.position(this.tree.indentDetector).w will return 0, in which case just
			// keep the default value.
			this._nodePixelIndent = domGeometry.position(this.tree.indentDetector).w || this._nodePixelIndent;

			// resize() may be called before this.rootNode is created, so wait until it's available
			this.expandChildrenDeferred.then(lang.hitch(this, function(){
				// If tree has already loaded, then reset indent for all the nodes
				this.rootNode.set('indent', this.showRoot ? 0 : -1);

				// Also, adjust widths of all rows to match width of Tree
				this._adjustWidths();
			}));
		},

		_outstandingPaintOperations: 0,
		_startPaint: function(/*Promise|Boolean*/ p){
			// summary:
			//		Called at the start of an operation that will change what's displayed.
			// p:
			//		Promise that tells when the operation will complete.  Alternately, if it's just a Boolean, it signifies
			//		that the operation was synchronous, and already completed.

			this._outstandingPaintOperations++;
			if(this._adjustWidthsTimer){
				this._adjustWidthsTimer.remove();
				delete this._adjustWidthsTimer;
			}

			var oc = lang.hitch(this, function(){
				this._outstandingPaintOperations--;

				if(this._outstandingPaintOperations <= 0 && !this._adjustWidthsTimer && this._started){
					// Use defer() to avoid a width adjustment when another operation will immediately follow,
					// such as a sequence of opening a node, then it's children, then it's grandchildren, etc.
					this._adjustWidthsTimer = this.defer("_adjustWidths");
				}
			});
			when(p, oc, oc);
		},

		_adjustWidths: function(){
			// summary:
			//		Size container to match widest TreeNode, so that highlighting with scrolling works (#13141, #16132)

			if(this._adjustWidthsTimer){
				this._adjustWidthsTimer.remove();
				delete this._adjustWidthsTimer;
			}

			this.containerNode.style.width = "auto";
			this.containerNode.style.width = this.domNode.scrollWidth > this.domNode.offsetWidth ? "auto" : "100%";
		},

		_createTreeNode: function(/*Object*/ args){
			// summary:
			//		creates a TreeNode
			// description:
			//		Developers can override this method to define their own TreeNode class;
			//		However it will probably be removed in a future release in favor of a way
			//		of just specifying a widget for the label, rather than one that contains
			//		the children too.
			return new TreeNode(args);
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the previously focused child, or first child.
			//		Some applications may want to change this method to focus the [first] selected child.

			if(this.lastFocusedChild){
				this.focusNode(this.lastFocusedChild);
			}else{
				this.focusFirstChild();
			}
		}
	});

	if(has("dojo-bidi")){
		Tree.extend({
			_setTextDirAttr: function(textDir){
				if(textDir && this.textDir != textDir){
					this._set("textDir", textDir);
					this.rootNode.set("textDir", textDir);
				}
			}
		});
	}

	Tree.PathError = createError("TreePathError");
	Tree._TreeNode = TreeNode;	// for monkey patching or creating subclasses of TreeNode

	return Tree;
});

},
'dojo/cookie':function(){
define(["./_base/kernel", "./regexp"], function(dojo, regexp){

// module:
//		dojo/cookie

/*=====
var __cookieProps = {
	// expires: Date|String|Number?
	//		If a number, the number of days from today at which the cookie
	//		will expire. If a date, the date past which the cookie will expire.
	//		If expires is in the past, the cookie will be deleted.
	//		If expires is omitted or is 0, the cookie will expire when the browser closes.
	// path: String?
	//		The path to use for the cookie.
	// domain: String?
	//		The domain to use for the cookie.
	// secure: Boolean?
	//		Whether to only send the cookie on secure connections
};
=====*/


dojo.cookie = function(/*String*/name, /*String?*/ value, /*__cookieProps?*/ props){
	// summary:
	//		Get or set a cookie.
	// description:
	//		If one argument is passed, returns the value of the cookie
	//		For two or more arguments, acts as a setter.
	// name:
	//		Name of the cookie
	// value:
	//		Value for the cookie
	// props:
	//		Properties for the cookie
	// example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	require(["dojo/cookie", "dojo/json"], function(cookie, json){
	//	|		cookie("configObj", json.stringify(config, {expires: 5 }));
	//	|	});
	//
	// example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	require(["dojo/cookie", "dojo/json"], function(cookie, json){
	//	|		config = json.parse(cookie("configObj"));
	//	|	});
	//
	// example:
	//		delete a cookie:
	//	|	require(["dojo/cookie"], function(cookie){
	//	|		cookie("configObj", null, {expires: -1});
	//	|	});
	var c = document.cookie, ret;
	if(arguments.length == 1){
		var matches = c.match(new RegExp("(?:^|; )" + regexp.escapeString(name) + "=([^;]*)"));
		ret = matches ? decodeURIComponent(matches[1]) : undefined; 
	}else{
		props = props || {};
// FIXME: expires=0 seems to disappear right away, not on close? (FF3)  Change docs?
		var exp = props.expires;
		if(typeof exp == "number"){
			var d = new Date();
			d.setTime(d.getTime() + exp*24*60*60*1000);
			exp = props.expires = d;
		}
		if(exp && exp.toUTCString){ props.expires = exp.toUTCString(); }

		value = encodeURIComponent(value);
		var updatedCookie = name + "=" + value, propName;
		for(propName in props){
			updatedCookie += "; " + propName;
			var propValue = props[propName];
			if(propValue !== true){ updatedCookie += "=" + propValue; }
		}
		document.cookie = updatedCookie;
	}
	return ret; // String|undefined
};

dojo.cookie.isSupported = function(){
	// summary:
	//		Use to determine if the current browser supports cookies or not.
	//
	//		Returns true if user allows cookies.
	//		Returns false if user doesn't allow cookies.

	if(!("cookieEnabled" in navigator)){
		this("__djCookieTest__", "CookiesAllowed");
		navigator.cookieEnabled = this("__djCookieTest__") == "CookiesAllowed";
		if(navigator.cookieEnabled){
			this("__djCookieTest__", "", {expires: -1});
		}
	}
	return navigator.cookieEnabled;
};

return dojo.cookie;
});

},
'dijit/tree/TreeStoreModel':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.indexOf array.some
	"dojo/aspect", // aspect.after
	"dojo/_base/declare", // declare
	"dojo/_base/lang" // lang.hitch
], function(array, aspect, declare, lang){

	// module:
	//		dijit/tree/TreeStoreModel

	return declare("dijit.tree.TreeStoreModel", null, {
		// summary:
		//		Implements dijit/Tree/model connecting to a dojo.data store with a single
		//		root item.  Any methods passed into the constructor will override
		//		the ones defined here.

		// store: dojo/data/api/Read
		//		Underlying store
		store: null,

		// childrenAttrs: String[]
		//		One or more attribute names (attributes in the dojo.data item) that specify that item's children
		childrenAttrs: ["children"],

		// newItemIdAttr: String
		//		Name of attribute in the Object passed to newItem() that specifies the id.
		//
		//		If newItemIdAttr is set then it's used when newItem() is called to see if an
		//		item with the same id already exists, and if so just links to the old item
		//		(so that the old item ends up with two parents).
		//
		//		Setting this to null or "" will make every drop create a new item.
		newItemIdAttr: "id",

		// labelAttr: String
		//		If specified, get label for tree node from this attribute, rather
		//		than by calling store.getLabel()
		labelAttr: "",

		// root: [readonly] dojo/data/Item
		//		Pointer to the root item (read only, not a parameter)
		root: null,

		// query: anything
		//		Specifies datastore query to return the root item for the tree.
		//		Must only return a single item.   Alternately can just pass in pointer
		//		to root item.
		// example:
		//	|	{id:'ROOT'}
		query: null,

		// deferItemLoadingUntilExpand: Boolean
		//		Setting this to true will cause the TreeStoreModel to defer calling loadItem on nodes
		//		until they are expanded. This allows for lazying loading where only one
		//		loadItem (and generally one network call, consequently) per expansion
		//		(rather than one for each child).
		//		This relies on partial loading of the children items; each children item of a
		//		fully loaded item should contain the label and info about having children.
		deferItemLoadingUntilExpand: false,

		constructor: function(/* Object */ args){
			// summary:
			//		Passed the arguments listed above (store, etc)
			// tags:
			//		private

			lang.mixin(this, args);

			this.connects = [];

			var store = this.store;
			if(!store.getFeatures()['dojo.data.api.Identity']){
				throw new Error("dijit.tree.TreeStoreModel: store must support dojo.data.Identity");
			}

			// if the store supports Notification, subscribe to the notification events
			if(store.getFeatures()['dojo.data.api.Notification']){
				this.connects = this.connects.concat([
					aspect.after(store, "onNew", lang.hitch(this, "onNewItem"), true),
					aspect.after(store, "onDelete", lang.hitch(this, "onDeleteItem"), true),
					aspect.after(store, "onSet", lang.hitch(this, "onSetItem"), true)
				]);
			}
		},

		destroy: function(){
			var h;
			while(h = this.connects.pop()){ h.remove(); }
			// TODO: should cancel any in-progress processing of getRoot(), getChildren()
		},

		// =======================================================================
		// Methods for traversing hierarchy

		getRoot: function(onItem, onError){
			// summary:
			//		Calls onItem with the root item for the tree, possibly a fabricated item.
			//		Calls onError on error.
			if(this.root){
				onItem(this.root);
			}else{
				this.store.fetch({
					query: this.query,
					onComplete: lang.hitch(this, function(items){
						if(items.length != 1){
							throw new Error("dijit.tree.TreeStoreModel: root query returned " + items.length +
								" items, but must return exactly one");
						}
						this.root = items[0];
						onItem(this.root);
					}),
					onError: onError
				});
			}
		},

		mayHaveChildren: function(/*dojo/data/Item*/ item){
			// summary:
			//		Tells if an item has or may have children.  Implementing logic here
			//		avoids showing +/- expando icon for nodes that we know don't have children.
			//		(For efficiency reasons we may not want to check if an element actually
			//		has children until user clicks the expando node)
			return array.some(this.childrenAttrs, function(attr){
				return this.store.hasAttribute(item, attr);
			}, this);
		},

		getChildren: function(/*dojo/data/Item*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
			// summary:
			//		Calls onComplete() with array of child items of given parent item, all loaded.

			var store = this.store;
			if(!store.isItemLoaded(parentItem)){
				// The parent is not loaded yet, we must be in deferItemLoadingUntilExpand
				// mode, so we will load it and just return the children (without loading each
				// child item)
				var getChildren = lang.hitch(this, arguments.callee);
				store.loadItem({
					item: parentItem,
					onItem: function(parentItem){
						getChildren(parentItem, onComplete, onError);
					},
					onError: onError
				});
				return;
			}
			// get children of specified item
			var childItems = [];
			for(var i=0; i<this.childrenAttrs.length; i++){
				var vals = store.getValues(parentItem, this.childrenAttrs[i]);
				childItems = childItems.concat(vals);
			}

			// count how many items need to be loaded
			var _waitCount = 0;
			if(!this.deferItemLoadingUntilExpand){
				array.forEach(childItems, function(item){ if(!store.isItemLoaded(item)){ _waitCount++; } });
			}

			if(_waitCount == 0){
				// all items are already loaded (or we aren't loading them).  proceed...
				onComplete(childItems);
			}else{
				// still waiting for some or all of the items to load
				array.forEach(childItems, function(item, idx){
					if(!store.isItemLoaded(item)){
						store.loadItem({
							item: item,
							onItem: function(item){
								childItems[idx] = item;
								if(--_waitCount == 0){
									// all nodes have been loaded, send them to the tree
									onComplete(childItems);
								}
							},
							onError: onError
						});
					}
				});
			}
		},

		// =======================================================================
		// Inspecting items

		isItem: function(/* anything */ something){
			return this.store.isItem(something);	// Boolean
		},

		fetchItemByIdentity: function(/* object */ keywordArgs){
			// summary:
			//		Given the identity of an item, this method returns the item that has
			//		that identity through the onItem callback.  Conforming implementations
			//		should return null if there is no item with the given identity.
			//		Implementations of fetchItemByIdentity() may sometimes return an item
			//		from a local cache and may sometimes fetch an item from a remote server.
			// tags:
			//		extension
			this.store.fetchItemByIdentity(keywordArgs);
		},

		getIdentity: function(/* item */ item){
			return this.store.getIdentity(item);	// Object
		},

		getLabel: function(/*dojo/data/Item*/ item){
			// summary:
			//		Get the label for an item
			if(this.labelAttr){
				return this.store.getValue(item,this.labelAttr);	// String
			}else{
				return this.store.getLabel(item);	// String
			}
		},

		// =======================================================================
		// Write interface

		newItem: function(/* dijit/tree/dndSource.__Item */ args, /*dojo/data/api/Item*/ parent, /*int?*/ insertIndex){
			// summary:
			//		Creates a new item.   See `dojo/data/api/Write` for details on args.
			//		Used in drag & drop when item from external source dropped onto tree.
			// description:
			//		Developers will need to override this method if new items get added
			//		to parents with multiple children attributes, in order to define which
			//		children attribute points to the new item.

			var pInfo = {parent: parent, attribute: this.childrenAttrs[0]}, LnewItem;

			if(this.newItemIdAttr && args[this.newItemIdAttr]){
				// Maybe there's already a corresponding item in the store; if so, reuse it.
				this.fetchItemByIdentity({identity: args[this.newItemIdAttr], scope: this, onItem: function(item){
					if(item){
						// There's already a matching item in store, use it
						this.pasteItem(item, null, parent, true, insertIndex);
					}else{
						// Create new item in the tree, based on the drag source.
						LnewItem=this.store.newItem(args, pInfo);
						if(LnewItem && (insertIndex!=undefined)){
							// Move new item to desired position
							this.pasteItem(LnewItem, parent, parent, false, insertIndex);
						}
					}
				}});
			}else{
				// [as far as we know] there is no id so we must assume this is a new item
				LnewItem=this.store.newItem(args, pInfo);
				if(LnewItem && (insertIndex!=undefined)){
					// Move new item to desired position
					this.pasteItem(LnewItem, parent, parent, false, insertIndex);
				}
			}
		},

		pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int?*/ insertIndex){
			// summary:
			//		Move or copy an item from one parent item to another.
			//		Used in drag & drop
			var store = this.store,
				parentAttr = this.childrenAttrs[0];	// name of "children" attr in parent item

			// remove child from source item, and record the attribute that child occurred in
			if(oldParentItem){
				array.forEach(this.childrenAttrs, function(attr){
					if(store.containsValue(oldParentItem, attr, childItem)){
						if(!bCopy){
							var values = array.filter(store.getValues(oldParentItem, attr), function(x){
								return x != childItem;
							});
							store.setValues(oldParentItem, attr, values);
						}
						parentAttr = attr;
					}
				});
			}

			// modify target item's children attribute to include this item
			if(newParentItem){
				if(typeof insertIndex == "number"){
					// call slice() to avoid modifying the original array, confusing the data store
					var childItems = store.getValues(newParentItem, parentAttr).slice();
					childItems.splice(insertIndex, 0, childItem);
					store.setValues(newParentItem, parentAttr, childItems);
				}else{
					store.setValues(newParentItem, parentAttr,
						store.getValues(newParentItem, parentAttr).concat(childItem));
				}
			}
		},

		// =======================================================================
		// Callbacks

		onChange: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback whenever an item has changed, so that Tree
			//		can update the label, icon, etc.   Note that changes
			//		to an item's children or parent(s) will trigger an
			//		onChildrenChange() so you can ignore those changes here.
			// tags:
			//		callback
		},

		onChildrenChange: function(/*===== parent, newChildrenList =====*/){
			// summary:
			//		Callback to do notifications about new, updated, or deleted items.
			// parent: dojo/data/Item
			// newChildrenList: dojo/data/Item[]
			// tags:
			//		callback
		},

		onDelete: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback when an item has been deleted.
			// description:
			//		Note that there will also be an onChildrenChange() callback for the parent
			//		of this item.
			// tags:
			//		callback
		},

		// =======================================================================
		// Events from data store

		onNewItem: function(/* dojo/data/Item */ item, /* Object */ parentInfo){
			// summary:
			//		Handler for when new items appear in the store, either from a drop operation
			//		or some other way.   Updates the tree view (if necessary).
			// description:
			//		If the new item is a child of an existing item,
			//		calls onChildrenChange() with the new list of children
			//		for that existing item.
			//
			// tags:
			//		extension

			// We only care about the new item if it has a parent that corresponds to a TreeNode
			// we are currently displaying
			if(!parentInfo){
				return;
			}

			// Call onChildrenChange() on parent (ie, existing) item with new list of children
			// In the common case, the new list of children is simply parentInfo.newValue or
			// [ parentInfo.newValue ], although if items in the store has multiple
			// child attributes (see `childrenAttr`), then it's a superset of parentInfo.newValue,
			// so call getChildren() to be sure to get right answer.
			this.getChildren(parentInfo.item, lang.hitch(this, function(children){
				this.onChildrenChange(parentInfo.item, children);
			}));
		},

		onDeleteItem: function(/*Object*/ item){
			// summary:
			//		Handler for delete notifications from underlying store
			this.onDelete(item);
		},

		onSetItem: function(item, attribute /*===== , oldValue, newValue =====*/){
			// summary:
			//		Updates the tree view according to changes in the data store.
			// description:
			//		Handles updates to an item's children by calling onChildrenChange(), and
			//		other updates to an item by calling onChange().
			//
			//		See `onNewItem` for more details on handling updates to an item's children.
			// item: Item
			// attribute: attribute-name-string
			// oldValue: Object|Array
			// newValue: Object|Array
			// tags:
			//		extension

			if(array.indexOf(this.childrenAttrs, attribute) != -1){
				// item's children list changed
				this.getChildren(item, lang.hitch(this, function(children){
					// See comments in onNewItem() about calling getChildren()
					this.onChildrenChange(item, children);
				}));
			}else{
				// item's label/icon/etc. changed.
				this.onChange(item);
			}
		}
	});
});

},
'dijit/tree/ForestStoreModel':function(){
define([
	"dojo/_base/array", // array.indexOf array.some
	"dojo/_base/declare", // declare
	"dojo/_base/kernel", // global
	"dojo/_base/lang", // lang.hitch
	"./TreeStoreModel"
], function(array, declare, kernel, lang, TreeStoreModel){

// module:
//		dijit/tree/ForestStoreModel

return declare("dijit.tree.ForestStoreModel", TreeStoreModel, {
	// summary:
	//		Interface between a dijit.Tree and a dojo.data store that doesn't have a root item,
	//		a.k.a. a store that has multiple "top level" items.
	//
	// description:
	//		Use this class to wrap a dojo.data store, making all the items matching the specified query
	//		appear as children of a fabricated "root item".  If no query is specified then all the
	//		items returned by fetch() on the underlying store become children of the root item.
	//		This class allows dijit.Tree to assume a single root item, even if the store doesn't have one.
	//
	//		When using this class the developer must override a number of methods according to their app and
	//		data, including:
	//
	//		- onNewRootItem
	//		- onAddToRoot
	//		- onLeaveRoot
	//		- onNewItem
	//		- onSetItem

	// Parameters to constructor

	// rootId: String
	//		ID of fabricated root item
	rootId: "$root$",

	// rootLabel: String
	//		Label of fabricated root item
	rootLabel: "ROOT",

	// query: String
	//		Specifies the set of children of the root item.
	// example:
	//	|	{type:'continent'}
	query: null,

	// End of parameters to constructor

	constructor: function(params){
		// summary:
		//		Sets up variables, etc.
		// tags:
		//		private

		// Make dummy root item
		this.root = {
			store: this,
			root: true,
			id: params.rootId,
			label: params.rootLabel,
			children: params.rootChildren	// optional param
		};
	},

	// =======================================================================
	// Methods for traversing hierarchy

	mayHaveChildren: function(/*dojo/data/Item*/ item){
		// summary:
		//		Tells if an item has or may have children.  Implementing logic here
		//		avoids showing +/- expando icon for nodes that we know don't have children.
		//		(For efficiency reasons we may not want to check if an element actually
		//		has children until user clicks the expando node)
		// tags:
		//		extension
		return item === this.root || this.inherited(arguments);
	},

	getChildren: function(/*dojo/data/Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError){
		// summary:
		//		Calls onComplete() with array of child items of given parent item, all loaded.
		if(parentItem === this.root){
			if(this.root.children){
				// already loaded, just return
				callback(this.root.children);
			}else{
				this.store.fetch({
					query: this.query,
					onComplete: lang.hitch(this, function(items){
						this.root.children = items;
						callback(items);
					}),
					onError: onError
				});
			}
		}else{
			this.inherited(arguments);
		}
	},

	// =======================================================================
	// Inspecting items

	isItem: function(/* anything */ something){
		return (something === this.root) ? true : this.inherited(arguments);
	},

	fetchItemByIdentity: function(/* object */ keywordArgs){
		if(keywordArgs.identity == this.root.id){
			var scope = keywordArgs.scope || kernel.global;
			if(keywordArgs.onItem){
				keywordArgs.onItem.call(scope, this.root);
			}
		}else{
			this.inherited(arguments);
		}
	},

	getIdentity: function(/* item */ item){
		return (item === this.root) ? this.root.id : this.inherited(arguments);
	},

	getLabel: function(/* item */ item){
		return	(item === this.root) ? this.root.label : this.inherited(arguments);
	},

	// =======================================================================
	// Write interface

	newItem: function(/* dijit/tree/dndSource.__Item */ args, /*Item*/ parent, /*int?*/ insertIndex){
		// summary:
		//		Creates a new item.   See dojo/data/api/Write for details on args.
		//		Used in drag & drop when item from external source dropped onto tree.
		if(parent === this.root){
			this.onNewRootItem(args);
			return this.store.newItem(args);
		}else{
			return this.inherited(arguments);
		}
	},

	onNewRootItem: function(/* dijit/tree/dndSource.__Item */ /*===== args =====*/){
		// summary:
		//		User can override this method to modify a new element that's being
		//		added to the root of the tree, for example to add a flag like root=true
	},

	pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ bCopy, /*int?*/ insertIndex){
		// summary:
		//		Move or copy an item from one parent item to another.
		//		Used in drag & drop
		if(oldParentItem === this.root){
			if(!bCopy){
				// It's onLeaveRoot()'s responsibility to modify the item so it no longer matches
				// this.query... thus triggering an onChildrenChange() event to notify the Tree
				// that this element is no longer a child of the root node
				this.onLeaveRoot(childItem);
			}
		}
		this.inherited(arguments, [childItem,
			oldParentItem === this.root ? null : oldParentItem,
			newParentItem === this.root ? null : newParentItem,
			bCopy,
			insertIndex
		]);
		if(newParentItem === this.root){
			// It's onAddToRoot()'s responsibility to modify the item so it matches
			// this.query... thus triggering an onChildrenChange() event to notify the Tree
			// that this element is now a child of the root node
			this.onAddToRoot(childItem);
		}
	},

	// =======================================================================
	// Handling for top level children

	onAddToRoot: function(/* item */ item){
		// summary:
		//		Called when item added to root of tree; user must override this method
		//		to modify the item so that it matches the query for top level items
		// example:
		//	|	store.setValue(item, "root", true);
		// tags:
		//		extension
		console.log(this, ": item ", item, " added to root");
	},

	onLeaveRoot: function(/* item */ item){
		// summary:
		//		Called when item removed from root of tree; user must override this method
		//		to modify the item so it doesn't match the query for top level items
		// example:
		//	|	store.unsetAttribute(item, "root");
		// tags:
		//		extension
		console.log(this, ": item ", item, " removed from root");
	},

	// =======================================================================
	// Events from data store

	_requeryTop: function(){
		// reruns the query for the children of the root node,
		// sending out an onSet notification if those children have changed
		var oldChildren = this.root.children || [];
		this.store.fetch({
			query: this.query,
			onComplete: lang.hitch(this, function(newChildren){
				this.root.children = newChildren;

				// If the list of children or the order of children has changed...
				if(oldChildren.length != newChildren.length ||
					array.some(oldChildren, function(item, idx){ return newChildren[idx] != item;})){
					this.onChildrenChange(this.root, newChildren);
				}
			})
		});
	},

	onNewItem: function(/* dojo/data/api/Item */ item, /* Object */ parentInfo){
		// summary:
		//		Handler for when new items appear in the store.  Developers should override this
		//		method to be more efficient based on their app/data.
		// description:
		//		Note that the default implementation requeries the top level items every time
		//		a new item is created, since any new item could be a top level item (even in
		//		addition to being a child of another item, since items can have multiple parents).
		//
		//		If developers can detect which items are possible top level items (based on the item and the
		//		parentInfo parameters), they should override this method to only call _requeryTop() for top
		//		level items.  Often all top level items have parentInfo==null, but
		//		that will depend on which store you use and what your data is like.
		// tags:
		//		extension
		this._requeryTop();

		this.inherited(arguments);
	},

	onDeleteItem: function(/*Object*/ item){
		// summary:
		//		Handler for delete notifications from underlying store

		// check if this was a child of root, and if so send notification that root's children
		// have changed
		if(array.indexOf(this.root.children, item) != -1){
			this._requeryTop();
		}

		this.inherited(arguments);
	},

	onSetItem: function(/* item */ item,
					/* attribute-name-string */ attribute,
					/* Object|Array */ oldValue,
					/* Object|Array */ newValue){
		// summary:
		//		Updates the tree view according to changes to an item in the data store.
		//		Developers should override this method to be more efficient based on their app/data.
		// description:
		//		Handles updates to an item's children by calling onChildrenChange(), and
		//		other updates to an item by calling onChange().
		//
		//		Also, any change to any item re-executes the query for the tree's top-level items,
		//		since this modified item may have started/stopped matching the query for top level items.
		//
		//		If possible, developers should override this function to only call _requeryTop() when
		//		the change to the item has caused it to stop/start being a top level item in the tree.
		// tags:
		//		extension

		this._requeryTop();
		this.inherited(arguments);
	}

});

});

},
'dijit/tree/_dndSelector':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.map
	"dojo/_base/connect", // connect.isCopyKey
	"dojo/_base/declare", // declare
	"dojo/_base/kernel",	// global
	"dojo/_base/lang", // lang.hitch
	"dojo/dom", // isDescendant
	"dojo/mouse", // mouse.isLeft
	"dojo/on",
	"dojo/touch",
	"../a11yclick",
	"./_dndContainer"
], function(array, connect, declare, kernel, lang, dom, mouse, on, touch, a11yclick, _dndContainer){

	// module:
	//		dijit/tree/_dndSelector

	return declare("dijit.tree._dndSelector", _dndContainer, {
		// summary:
		//		This is a base class for `dijit/tree/dndSource`, and isn't meant to be used directly.
		//		It's based on `dojo/dnd/Selector`.
		// tags:
		//		protected

		/*=====
		// selection: Object
		//		(id to DomNode) map for every TreeNode that's currently selected.
		//		The DOMNode is the TreeNode.rowNode.
		selection: {},
		=====*/

		constructor: function(){
			// summary:
			//		Initialization
			// tags:
			//		private

			this.selection={};
			this.anchor = null;

			this.events.push(
				// listeners setup here but no longer used (left for backwards compatibility
				on(this.tree.domNode, touch.press, lang.hitch(this,"onMouseDown")),
				on(this.tree.domNode, touch.release, lang.hitch(this,"onMouseUp")),

				// listeners used in this module
				on(this.tree.domNode, touch.move, lang.hitch(this,"onMouseMove")),
				on(this.tree.domNode, a11yclick.press, lang.hitch(this,"onClickPress")),
				on(this.tree.domNode, a11yclick.release, lang.hitch(this,"onClickRelease"))
			);
		},

		// singular: Boolean
		//		Allows selection of only one element, if true.
		//		Tree hasn't been tested in singular=true mode, unclear if it works.
		singular: false,

		// methods
		getSelectedTreeNodes: function(){
			// summary:
			//		Returns a list of selected node(s).
			//		Used by dndSource on the start of a drag.
			// tags:
			//		protected
			var nodes=[], sel = this.selection;
			for(var i in sel){
				nodes.push(sel[i]);
			}
			return nodes;
		},

		selectNone: function(){
			// summary:
			//		Unselects all items
			// tags:
			//		private

			this.setSelection([]);
			return this;	// self
		},

		destroy: function(){
			// summary:
			//		Prepares the object to be garbage-collected
			this.inherited(arguments);
			this.selection = this.anchor = null;
		},
		addTreeNode: function(/*dijit/Tree._TreeNode*/ node, /*Boolean?*/isAnchor){
			// summary:
			//		add node to current selection
			// node: Node
			//		node to add
			// isAnchor: Boolean
			//		Whether the node should become anchor.

			this.setSelection(this.getSelectedTreeNodes().concat( [node] ));
			if(isAnchor){ this.anchor = node; }
			return node;
		},
		removeTreeNode: function(/*dijit/Tree._TreeNode*/ node){
			// summary:
			//		remove node and it's descendants from current selection
			// node: Node
			//		node to remove
			var newSelection = array.filter(this.getSelectedTreeNodes(), function(selectedNode){
				return !dom.isDescendant(selectedNode.domNode, node.domNode); // also matches when selectedNode == node
			});
			this.setSelection(newSelection);
			return node;
		},
		isTreeNodeSelected: function(/*dijit/Tree._TreeNode*/ node){
			// summary:
			//		return true if node is currently selected
			// node: Node
			//		the node to check whether it's in the current selection

			return node.id && !!this.selection[node.id];
		},
		setSelection: function(/*dijit/Tree._TreeNode[]*/ newSelection){
			// summary:
			//		set the list of selected nodes to be exactly newSelection. All changes to the
			//		selection should be passed through this function, which ensures that derived
			//		attributes are kept up to date. Anchor will be deleted if it has been removed
			//		from the selection, but no new anchor will be added by this function.
			// newSelection: Node[]
			//		list of tree nodes to make selected
			var oldSelection = this.getSelectedTreeNodes();
			array.forEach(this._setDifference(oldSelection, newSelection), lang.hitch(this, function(node){
				node.setSelected(false);
				if(this.anchor == node){
					delete this.anchor;
				}
				delete this.selection[node.id];
			}));
			array.forEach(this._setDifference(newSelection, oldSelection), lang.hitch(this, function(node){
				node.setSelected(true);
				this.selection[node.id] = node;
			}));
			this._updateSelectionProperties();
		},
		_setDifference: function(xs,ys){
			// summary:
			//		Returns a copy of xs which lacks any objects
			//		occurring in ys. Checks for membership by
			//		modifying and then reading the object, so it will
			//		not properly handle sets of numbers or strings.

			array.forEach(ys, function(y){ y.__exclude__ = true; });
			var ret = array.filter(xs, function(x){ return !x.__exclude__; });

			// clean up after ourselves.
			array.forEach(ys, function(y){ delete y['__exclude__'] });
			return ret;
		},
		_updateSelectionProperties: function(){
			// summary:
			//		Update the following tree properties from the current selection:
			//		path[s], selectedItem[s], selectedNode[s]

			var selected = this.getSelectedTreeNodes();
			var paths = [], nodes = [], selects = [];
			array.forEach(selected, function(node){
				var ary = node.getTreePath(), model = this.tree.model;
				nodes.push(node);
				paths.push(ary);
				ary = array.map(ary, function(item){
					return model.getIdentity(item);
				}, this);
				selects.push(ary.join("/"))
			}, this);
			var items = array.map(nodes,function(node){ return node.item; });
			this.tree._set("paths", paths);
			this.tree._set("path", paths[0] || []);
			this.tree._set("selectedNodes", nodes);
			this.tree._set("selectedNode", nodes[0] || null);
			this.tree._set("selectedItems", items);
			this.tree._set("selectedItem", items[0] || null);
		},

		// selection related events
		onClickPress: function(e){
			// summary:
			//		Event processor for onmousedown/ontouchstart/onkeydown corresponding to a click event
			// e: Event
			//		onmousedown/ontouchstart/onkeydown event
			// tags:
			//		protected

			// ignore mouse or touch on expando node
			if(this.current && this.current.isExpandable && this.tree.isExpandoNode(e.target, this.current)){ return; }

			if(mouse.isLeft(e)){
				// Prevent text selection while dragging on desktop, see #16328.   But don't call preventDefault()
				// for mobile because it will break things completely, see #15838.
				e.preventDefault();
			}

			var treeNode = e.type == "keydown" ? this.tree.focusedChild : this.current;

			if(!treeNode){
				// Click must be on the Tree but not on a TreeNode, happens especially when Tree is stretched to fill
				// a pane of a BorderContainer, etc.
				return;
			}

			var copy = connect.isCopyKey(e), id = treeNode.id;

			// if shift key is not pressed, and the node is already in the selection,
			// delay deselection until onmouseup so in the case of DND, deselection
			// will be canceled by onmousemove.
			if(!this.singular && !e.shiftKey && this.selection[id]){
				this._doDeselect = true;
				return;
			}else{
				this._doDeselect = false;
			}
			this.userSelect(treeNode, copy, e.shiftKey);
		},

		onClickRelease: function(e){
			// summary:
			//		Event processor for onmouseup/ontouchend/onkeyup corresponding to a click event
			// e: Event
			//		onmouseup/ontouchend/onkeyup event
			// tags:
			//		protected

			// _doDeselect is the flag to indicate that the user wants to either ctrl+click on
			// an already selected item (to deselect the item), or click on a not-yet selected item
			// (which should remove all current selection, and add the clicked item). This can not
			// be done in onMouseDown, because the user may start a drag after mousedown. By moving
			// the deselection logic here, the user can drag an already selected item.
			if(!this._doDeselect){ return; }
			this._doDeselect = false;
			this.userSelect(e.type == "keyup" ? this.tree.focusedChild : this.current, connect.isCopyKey(e), e.shiftKey);
		},
		onMouseMove: function(/*===== e =====*/){
			// summary:
			//		event processor for onmousemove/ontouchmove
			// e: Event
			//		onmousemove/ontouchmove event
			this._doDeselect = false;
		},

		// mouse/touch events that are no longer used
		onMouseDown: function(){
			// summary:
			//		Event processor for onmousedown/ontouchstart
			// e: Event
			//		onmousedown/ontouchstart event
			// tags:
			//		protected
		},
		onMouseUp: function(){
			// summary:
			//		Event processor for onmouseup/ontouchend
			// e: Event
			//		onmouseup/ontouchend event
			// tags:
			//		protected
		},

		_compareNodes: function(n1, n2){
			if(n1 === n2){
				return 0;
			}

			if('sourceIndex' in document.documentElement){ //IE
				//TODO: does not yet work if n1 and/or n2 is a text node
				return n1.sourceIndex - n2.sourceIndex;
			}else if('compareDocumentPosition' in document.documentElement){ //FF, Opera
				return n1.compareDocumentPosition(n2) & 2 ? 1: -1;
			}else if(document.createRange){ //Webkit
				var r1 = doc.createRange();
				r1.setStartBefore(n1);

				var r2 = doc.createRange();
				r2.setStartBefore(n2);

				return r1.compareBoundaryPoints(r1.END_TO_END, r2);
			}else{
				throw Error("dijit.tree._compareNodes don't know how to compare two different nodes in this browser");
			}
		},

		userSelect: function(node, multi, range){
			// summary:
			//		Add or remove the given node from selection, responding
			//		to a user action such as a click or keypress.
			// multi: Boolean
			//		Indicates whether this is meant to be a multi-select action (e.g. ctrl-click)
			// range: Boolean
			//		Indicates whether this is meant to be a ranged action (e.g. shift-click)
			// tags:
			//		protected

			if(this.singular){
				if(this.anchor == node && multi){
					this.selectNone();
				}else{
					this.setSelection([node]);
					this.anchor = node;
				}
			}else{
				if(range && this.anchor){
					var cr = this._compareNodes(this.anchor.rowNode, node.rowNode),
					begin, end, anchor = this.anchor;

					if(cr < 0){ //current is after anchor
						begin = anchor;
						end = node;
					}else{ //current is before anchor
						begin = node;
						end = anchor;
					}
					var nodes = [];
					//add everything betweeen begin and end inclusively
					while(begin != end){
						nodes.push(begin);
						begin = this.tree._getNext(begin);
					}
					nodes.push(end);

					this.setSelection(nodes);
				}else{
					if( this.selection[ node.id ] && multi ){
						this.removeTreeNode( node );
					}else if(multi){
						this.addTreeNode(node, true);
					}else{
						this.setSelection([node]);
						this.anchor = node;
					}
				}
			}
		},

		getItem: function(/*String*/ key){
			// summary:
			//		Returns the dojo/dnd/Container._Item (representing a dragged node) by it's key (id).
			//		Called by dojo/dnd/Source.checkAcceptance().
			// tags:
			//		protected

			var widget = this.selection[key];
			return {
				data: widget,
				type: ["treeNode"]
			}; // dojo/dnd/Container._Item
		},

		forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
			// summary:
			//		Iterates over selected items;
			//		see `dojo/dnd/Container.forInItems()` for details
			o = o || kernel.global;
			for(var id in this.selection){
				// console.log("selected item id: " + id);
				f.call(o, this.getItem(id), id, this);
			}
		}
	});
});

},
'dijit/tree/_dndContainer':function(){
define([
	"dojo/aspect", // aspect.after
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove domClass.replace
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/on",
	"dojo/touch"
], function(aspect, declare, domClass, lang, on, touch){

	// module:
	//		dijit/tree/_dndContainer

	/*=====
	 var __Args = {
	 // summary:
	 //		A dict of parameters for Tree source configuration.
	 // isSource: Boolean?
	 //		Can be used as a DnD source. Defaults to true.
	 // accept: String[]
	 //		List of accepted types (text strings) for a target; defaults to
	 //		["text", "treeNode"]
	 // copyOnly: Boolean?
	 //		Copy items, if true, use a state of Ctrl key otherwise,
	 // dragThreshold: Number
	 //		The move delay in pixels before detecting a drag; 0 by default
	 // betweenThreshold: Integer
	 //		Distance from upper/lower edge of node to allow drop to reorder nodes
	 };
	 =====*/

	return declare("dijit.tree._dndContainer", null, {

		// summary:
		//		This is a base class for `dijit/tree/_dndSelector`, and isn't meant to be used directly.
		//		It's modeled after `dojo/dnd/Container`.
		// tags:
		//		protected

		/*=====
		 // current: TreeNode
		 //		The currently hovered TreeNode.  Not set to anything for keyboard operation.  (TODO: change?)
		 current: null,
		 =====*/

		constructor: function(tree, params){
			// summary:
			//		A constructor of the Container
			// tree: Node
			//		Node or node's id to build the container on
			// params: __Args
			//		A dict of parameters, which gets mixed into the object
			// tags:
			//		private
			this.tree = tree;
			this.node = tree.domNode;	// TODO: rename; it's not a TreeNode but the whole Tree
			lang.mixin(this, params);

			// states
			this.containerState = "";
			domClass.add(this.node, "dojoDndContainer");

			// set up events
			this.events = [
				// Mouse (or touch) enter/leave on Tree itself
				on(this.node, touch.enter, lang.hitch(this, "onOverEvent")),
				on(this.node, touch.leave, lang.hitch(this, "onOutEvent")),

				// switching between TreeNodes
				aspect.after(this.tree, "_onNodeMouseEnter", lang.hitch(this, "onMouseOver"), true),
				aspect.after(this.tree, "_onNodeMouseLeave", lang.hitch(this, "onMouseOut"), true),

				// cancel text selection and text dragging
				on(this.node, "dragstart, selectstart", function(evt){
					evt.preventDefault();
				})
			];
		},

		destroy: function(){
			// summary:
			//		Prepares this object to be garbage-collected

			var h;
			while(h = this.events.pop()){
				h.remove();
			}

			// this.clearItems();
			this.node = this.parent = null;
		},

		// mouse events
		onMouseOver: function(widget /*===== , evt =====*/){
			// summary:
			//		Called when mouse is moved over a TreeNode
			// widget: TreeNode
			// evt: Event
			// tags:
			//		protected
			this.current = widget;
		},

		onMouseOut: function(/*===== widget, evt =====*/){
			// summary:
			//		Called when mouse is moved away from a TreeNode
			// widget: TreeNode
			// evt: Event
			// tags:
			//		protected
			this.current = null;
		},

		_changeState: function(type, newState){
			// summary:
			//		Changes a named state to new state value
			// type: String
			//		A name of the state to change
			// newState: String
			//		new state
			var prefix = "dojoDnd" + type;
			var state = type.toLowerCase() + "State";
			//domClass.replace(this.node, prefix + newState, prefix + this[state]);
			domClass.replace(this.node, prefix + newState, prefix + this[state]);
			this[state] = newState;
		},

		_addItemClass: function(node, type){
			// summary:
			//		Adds a class with prefix "dojoDndItem"
			// node: Node
			//		A node
			// type: String
			//		A variable suffix for a class name
			domClass.add(node, "dojoDndItem" + type);
		},

		_removeItemClass: function(node, type){
			// summary:
			//		Removes a class with prefix "dojoDndItem"
			// node: Node
			//		A node
			// type: String
			//		A variable suffix for a class name
			domClass.remove(node, "dojoDndItem" + type);
		},

		onOverEvent: function(){
			// summary:
			//		This function is called once, when mouse is over our container
			// tags:
			//		protected
			this._changeState("Container", "Over");
		},

		onOutEvent: function(){
			// summary:
			//		This function is called once, when mouse is out of our container
			// tags:
			//		protected
			this._changeState("Container", "");
		}
	});
});

},
'dijit/tree/ObjectStoreModel':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.indexOf array.some
	"dojo/aspect", // aspect.before, aspect.after
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch
	"dojo/when"
], function(array, aspect, declare, lang, when){

	// module:
	//		dijit/tree/ObjectStoreModel

	return declare("dijit.tree.ObjectStoreModel", null, {
		// summary:
		//		Implements dijit/tree/model connecting dijit/Tree to a dojo/store/api/Store that implements
		//		getChildren().
		//
		//		If getChildren() returns an array with an observe() method, then it will be leveraged to reflect
		//		store updates to the tree.   So, this class will work best when:
		//
		//			1. the store implements dojo/store/Observable
		//			2. getChildren() is implemented as a query to the server (i.e. it calls store.query())
		//
		//		Drag and Drop: To support drag and drop, besides implementing getChildren()
		//		and dojo/store/Observable, the store must support the parent option to put().
		//		And in order to have child elements ordered according to how the user dropped them,
		//		put() must support the before option.

		// store: dojo/store/api/Store
		//		Underlying store
		store: null,

		// labelAttr: String
		//		Get label for tree node from this attribute
		labelAttr: "name",

		// labelType: [const] String
		//		Specifies how to interpret the labelAttr in the data store items.
		//		Can be "html" or "text".
		labelType: "text",

		// root: [readonly] Object
		//		Pointer to the root item from the dojo/store/api/Store (read only, not a parameter)
		root: null,

		// query: anything
		//		Specifies datastore query to return the root item for the tree.
		//		Must only return a single item.   Alternately can just pass in pointer
		//		to root item.
		// example:
		//	|	{id:'ROOT'}
		query: null,

		constructor: function(/* Object */ args){
			// summary:
			//		Passed the arguments listed above (store, etc)
			// tags:
			//		private

			lang.mixin(this, args);

			this.childrenCache = {};	// map from id to array of children
		},

		destroy: function(){
			// TODO: should cancel any in-progress processing of getRoot(), getChildren()
			for(var id in this.childrenCache){
				this.childrenCache[id].close && this.childrenCache[id].close();
			}
		},

		// =======================================================================
		// Methods for traversing hierarchy

		getRoot: function(onItem, onError){
			// summary:
			//		Calls onItem with the root item for the tree, possibly a fabricated item.
			//		Calls onError on error.
			if(this.root){
				onItem(this.root);
			}else{
				var res;
				when(res = this.store.query(this.query),
					lang.hitch(this, function(items){
						//console.log("queried root: ", res);
						if(items.length != 1){
							throw new Error("dijit.tree.ObjectStoreModel: root query returned " + items.length +
								" items, but must return exactly one");
						}
						this.root = items[0];
						onItem(this.root);

						// Setup listener to detect if root item changes
						if(res.observe){
							res.observe(lang.hitch(this, function(obj){
								// Presumably removedFrom == insertedInto == 1, and this call indicates item has changed.
								//console.log("root changed: ", obj);
								this.onChange(obj);
							}), true);	// true to listen for updates to obj
						}
					}),
					onError
				);
			}
		},

		mayHaveChildren: function(/*===== item =====*/){
			// summary:
			//		Tells if an item has or might have children.  Implementing logic here
			//		avoids showing +/- expando icon for nodes that we know won't have children.
			//		(For efficiency reasons we may not want to check if an element actually
			//		has children until user clicks the expando node).
			//
			//		Application code should override this method based on the data, for example
			//		it could be `return item.leaf == true;`.
			//
			//		Note that mayHaveChildren() must return true for an item if it could possibly
			//		have children in the future, due to drag-an-drop or some other data store update.
			//		Also note that it may return true if it's just too expensive to check during tree
			//		creation whether or not the item has children.
			// item: Object
			//		Item from the dojo/store
			return true;
		},

		getChildren: function(/*Object*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
			// summary:
			//		Calls onComplete() with array of child items of given parent item.
			// parentItem:
			//		Item from the dojo/store

			var id = this.store.getIdentity(parentItem);
			if(this.childrenCache[id]){
				when(this.childrenCache[id], onComplete, onError);
				return;
			}

			var res = this.childrenCache[id] = this.store.getChildren(parentItem);

			// User callback
			when(res, onComplete, onError);

			// Setup listener in case children list changes, or the item(s) in the children list are
			// updated in some way.
			if(res.observe){
				res.observe(lang.hitch(this, function(obj, removedFrom, insertedInto){
					//console.log("observe on children of ", id, ": ", obj, removedFrom, insertedInto);

					// If removedFrom == insertedInto, this call indicates that the item has changed.
					// Even if removedFrom != insertedInto, the item may have changed.
					this.onChange(obj);

					if(removedFrom != insertedInto){
						// Indicates an item was added, removed, or re-parented.  The children[] array (returned from
						// res.then(...)) has already been updated (like a live collection), so just use it.
						when(res, lang.hitch(this, "onChildrenChange", parentItem));
					}
				}), true);	// true means to notify on item changes
			}
		},

		// =======================================================================
		// Inspecting items

		isItem: function(/*===== something =====*/){
			return true;	// Boolean
		},

		getIdentity: function(/* item */ item){
			return this.store.getIdentity(item);	// Object
		},

		getLabel: function(/*dojo/data/Item*/ item){
			// summary:
			//		Get the label for an item
			return item[this.labelAttr];	// String
		},

		// =======================================================================
		// Write interface, for DnD

		newItem: function(/* dijit/tree/dndSource.__Item */ args, /*Item*/ parent, /*int?*/ insertIndex, /*Item*/ before){
			// summary:
			//		Creates a new item.   See `dojo/data/api/Write` for details on args.
			//		Used in drag & drop when item from external source dropped onto tree.

			return this.store.put(args, {
				parent: parent,
				before: before
			});
		},

		pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem,
					/*Boolean*/ bCopy, /*int?*/ insertIndex, /*Item*/ before){
			// summary:
			//		Move or copy an item from one parent item to another.
			//		Used in drag & drop

			if(!bCopy){
				// In order for DnD moves to work correctly, childItem needs to be orphaned from oldParentItem
				// before being adopted by newParentItem.   That way, the TreeNode is moved rather than
				// an additional TreeNode being created, and the old TreeNode subsequently being deleted.
				// The latter loses information such as selection and opened/closed children TreeNodes.
				// Unfortunately simply calling this.store.put() will send notifications in a random order, based
				// on when the TreeNodes in question originally appeared, and not based on the drag-from
				// TreeNode vs. the drop-onto TreeNode.

				var oldParentChildren = [].concat(this.childrenCache[this.getIdentity(oldParentItem)]), // concat to make copy
					index = array.indexOf(oldParentChildren, childItem);
				oldParentChildren.splice(index, 1);
				this.onChildrenChange(oldParentItem, oldParentChildren);
			}

			return this.store.put(childItem, {
				overwrite: true,
				parent: newParentItem,
				before: before
			});
		},

		// =======================================================================
		// Callbacks

		onChange: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback whenever an item has changed, so that Tree
			//		can update the label, icon, etc.   Note that changes
			//		to an item's children or parent(s) will trigger an
			//		onChildrenChange() so you can ignore those changes here.
			// tags:
			//		callback
		},

		onChildrenChange: function(/*===== parent, newChildrenList =====*/){
			// summary:
			//		Callback to do notifications about new, updated, or deleted items.
			// parent: dojo/data/Item
			// newChildrenList: Object[]
			//		Items from the store
			// tags:
			//		callback
		},

		onDelete: function(/*dojo/data/Item*/ /*===== item =====*/){
			// summary:
			//		Callback when an item has been deleted.
			//		Actually we have no way of knowing this with the new dojo.store API,
			//		so this method is never called (but it's left here since Tree connects
			//		to it).
			// tags:
			//		callback
		}
	});
});

},
'dijit/form/Form':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/kernel", // kernel.deprecated
	"dojo/sniff", // has("ie")
	"../_Widget",
	"../_TemplatedMixin",
	"./_FormMixin",
	"../layout/_ContentPaneResizeMixin"
], function(declare, domAttr, kernel, has, _Widget, _TemplatedMixin, _FormMixin, _ContentPaneResizeMixin){

	// module:
	//		dijit/form/Form


	return declare("dijit.form.Form", [_Widget, _TemplatedMixin, _FormMixin, _ContentPaneResizeMixin], {
		// summary:
		//		Widget corresponding to HTML form tag, for validation and serialization
		//
		// example:
		//	|	<form data-dojo-type="dijit/form/Form" id="myForm">
		//	|		Name: <input type="text" name="name" />
		//	|	</form>
		//	|	myObj = {name: "John Doe"};
		//	|	dijit.byId('myForm').set('value', myObj);
		//	|
		//	|	myObj=dijit.byId('myForm').get('value');

		// HTML <FORM> attributes

		// name: String?
		//		Name of form for scripting.
		name: "",

		// action: String?
		//		Server-side form handler.
		action: "",

		// method: String?
		//		HTTP method used to submit the form, either "GET" or "POST".
		method: "",

		// encType: String?
		//		Encoding type for the form, ex: application/x-www-form-urlencoded.
		encType: "",

		// accept-charset: String?
		//		List of supported charsets.
		"accept-charset": "",

		// accept: String?
		//		List of MIME types for file upload.
		accept: "",

		// target: String?
		//		Target frame for the document to be opened in.
		target: "",

		templateString: "<form data-dojo-attach-point='containerNode' data-dojo-attach-event='onreset:_onReset,onsubmit:_onSubmit' ${!nameAttrSetting}></form>",

		postMixInProperties: function(){
			// Setup name=foo string to be referenced from the template (but only if a name has been specified)
			// Unfortunately we can't use _setNameAttr to set the name due to IE limitations, see #8660
			this.nameAttrSetting = this.name ? ("name='" + this.name + "'") : "";
			this.inherited(arguments);
		},

		execute: function(/*Object*/ /*===== formContents =====*/){
			// summary:
			//		Deprecated: use submit()
			// tags:
			//		deprecated
		},

		onExecute: function(){
			// summary:
			//		Deprecated: use onSubmit()
			// tags:
			//		deprecated
		},

		_setEncTypeAttr: function(/*String*/ value){
			domAttr.set(this.domNode, "encType", value);
			if(has("ie")){
				this.domNode.encoding = value;
			}
			this._set("encType", value);
		},

		reset: function(/*Event?*/ e){
			// summary:
			//		restores all widget values back to their init values,
			//		calls onReset() which can cancel the reset by returning false

			// create fake event so we can know if preventDefault() is called
			var faux = {
				returnValue: true, // the IE way
				preventDefault: function(){ // not IE
					this.returnValue = false;
				},
				stopPropagation: function(){
				},
				currentTarget: e ? e.target : this.domNode,
				target: e ? e.target : this.domNode
			};
			// if return value is not exactly false, and haven't called preventDefault(), then reset
			if(!(this.onReset(faux) === false) && faux.returnValue){
				this.inherited(arguments, []);
			}
		},

		onReset: function(/*Event?*/ /*===== e =====*/){
			// summary:
			//		Callback when user resets the form. This method is intended
			//		to be over-ridden. When the `reset` method is called
			//		programmatically, the return value from `onReset` is used
			//		to compute whether or not resetting should proceed
			// tags:
			//		callback
			return true; // Boolean
		},

		_onReset: function(e){
			this.reset(e);
			e.stopPropagation();
			e.preventDefault();
			return false;
		},

		_onSubmit: function(e){
			var fp = this.constructor.prototype;
			// TODO: remove this if statement beginning with 2.0
			if(this.execute != fp.execute || this.onExecute != fp.onExecute){
				kernel.deprecated("dijit.form.Form:execute()/onExecute() are deprecated. Use onSubmit() instead.", "", "2.0");
				this.onExecute();
				this.execute(this.getValues());
			}
			if(this.onSubmit(e) === false){ // only exactly false stops submit
				e.stopPropagation();
				e.preventDefault();
			}
		},

		onSubmit: function(/*Event?*/ /*===== e =====*/){
			// summary:
			//		Callback when user submits the form.
			// description:
			//		This method is intended to be over-ridden, but by default it checks and
			//		returns the validity of form elements. When the `submit`
			//		method is called programmatically, the return value from
			//		`onSubmit` is used to compute whether or not submission
			//		should proceed
			// tags:
			//		extension

			return this.isValid(); // Boolean
		},

		submit: function(){
			// summary:
			//		programmatically submit form if and only if the `onSubmit` returns true
			if(!(this.onSubmit() === false)){
				this.containerNode.submit();
			}
		}
	});
});

},
'dijit/form/NumberSpinner':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/keys", // keys.END keys.HOME
	"./_Spinner",
	"./NumberTextBox"
], function(declare, keys, _Spinner, NumberTextBox){

	// module:
	//		dijit/form/NumberSpinner

	return declare("dijit.form.NumberSpinner", [_Spinner, NumberTextBox.Mixin], {
		// summary:
		//		Extends NumberTextBox to add up/down arrows and pageup/pagedown for incremental change to the value
		//
		// description:
		//		A `dijit/form/NumberTextBox` extension to provide keyboard accessible value selection
		//		as well as icons for spinning direction. When using the keyboard, the typematic rules
		//		apply, meaning holding the key will gradually increase or decrease the value and
		//		accelerate.
		//
		// example:
		//	| new NumberSpinner({ constraints:{ max:300, min:100 }}, "someInput");

		baseClass: "dijitTextBox dijitSpinner dijitNumberTextBox",

		adjust: function(/*Object*/ val, /*Number*/ delta){
			// summary:
			//		Change Number val by the given amount
			// tags:
			//		protected

			var tc = this.constraints,
				v = isNaN(val),
				gotMax = !isNaN(tc.max),
				gotMin = !isNaN(tc.min)
				;
			if(v && delta != 0){ // blank or invalid value and they want to spin, so create defaults
				val = (delta > 0) ?
					gotMin ? tc.min : gotMax ? tc.max : 0 :
					gotMax ? this.constraints.max : gotMin ? tc.min : 0
				;
			}
			var newval = val + delta;
			if(v || isNaN(newval)){
				return val;
			}
			if(gotMax && (newval > tc.max)){
				newval = tc.max;
			}
			if(gotMin && (newval < tc.min)){
				newval = tc.min;
			}
			return newval;
		},

		_onKeyDown: function(e){
			if(this.disabled || this.readOnly){
				return;
			}
			if((e.keyCode == keys.HOME || e.keyCode == keys.END) && !(e.ctrlKey || e.altKey || e.metaKey)
				&& typeof this.get('value') != 'undefined' /* gibberish, so HOME and END are default editing keys*/){
				var value = this.constraints[(e.keyCode == keys.HOME ? "min" : "max")];
				if(typeof value == "number"){
					this._setValueAttr(value, false);
				}
				// eat home or end key whether we change the value or not
				e.stopPropagation();
				e.preventDefault();
			}
		}
	});
});

},
'dijit/form/_Spinner':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/keys", // keys keys.DOWN_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.UP_ARROW
	"dojo/_base/lang", // lang.hitch
	"dojo/sniff", // has("mozilla")
	"dojo/mouse", // mouse.wheel
	"dojo/on",
	"../typematic",
	"./RangeBoundTextBox",
	"dojo/text!./templates/Spinner.html",
	"./_TextBoxMixin"    // selectInputText
], function(declare, keys, lang, has, mouse, on, typematic, RangeBoundTextBox, template, _TextBoxMixin){

	// module:
	//		dijit/form/_Spinner

	return declare("dijit.form._Spinner", RangeBoundTextBox, {
		// summary:
		//		Mixin for validation widgets with a spinner.
		// description:
		//		This class basically (conceptually) extends `dijit/form/ValidationTextBox`.
		//		It modifies the template to have up/down arrows, and provides related handling code.

		// defaultTimeout: Number
		//		Number of milliseconds before a held arrow key or up/down button becomes typematic
		defaultTimeout: 500,

		// minimumTimeout: Number
		//		minimum number of milliseconds that typematic event fires when held key or button is held
		minimumTimeout: 10,

		// timeoutChangeRate: Number
		//		Fraction of time used to change the typematic timer between events.
		//		1.0 means that each typematic event fires at defaultTimeout intervals.
		//		Less than 1.0 means that each typematic event fires at an increasing faster rate.
		timeoutChangeRate: 0.90,

		// smallDelta: Number
		//		Adjust the value by this much when spinning using the arrow keys/buttons
		smallDelta: 1,

		// largeDelta: Number
		//		Adjust the value by this much when spinning using the PgUp/Dn keys
		largeDelta: 10,

		templateString: template,

		baseClass: "dijitTextBox dijitSpinner",

		// Set classes like dijitUpArrowButtonHover or dijitDownArrowButtonActive depending on
		// mouse action over specified node
		cssStateNodes: {
			"upArrowNode": "dijitUpArrowButton",
			"downArrowNode": "dijitDownArrowButton"
		},

		adjust: function(val /*=====, delta =====*/){
			// summary:
			//		Overridable function used to adjust a primitive value(Number/Date/...) by the delta amount specified.
			//		The val is adjusted in a way that makes sense to the object type.
			// val: Object
			// delta: Number
			// tags:
			//		protected extension
			return val;
		},

		_arrowPressed: function(/*Node*/ nodePressed, /*Number*/ direction, /*Number*/ increment){
			// summary:
			//		Handler for arrow button or arrow key being pressed
			if(this.disabled || this.readOnly){
				return;
			}
			this._setValueAttr(this.adjust(this.get('value'), direction * increment), false);
			_TextBoxMixin.selectInputText(this.textbox, this.textbox.value.length);
		},

		_arrowReleased: function(/*Node*/ /*===== node =====*/){
			// summary:
			//		Handler for arrow button or arrow key being released
			this._wheelTimer = null;
		},

		_typematicCallback: function(/*Number*/ count, /*DOMNode*/ node, /*Event*/ evt){
			var inc = this.smallDelta;
			if(node == this.textbox){
				var key = evt.keyCode;
				inc = (key == keys.PAGE_UP || key == keys.PAGE_DOWN) ? this.largeDelta : this.smallDelta;
				node = (key == keys.UP_ARROW || key == keys.PAGE_UP) ? this.upArrowNode : this.downArrowNode;
			}
			if(count == -1){
				this._arrowReleased(node);
			}
			else{
				this._arrowPressed(node, (node == this.upArrowNode) ? 1 : -1, inc);
			}
		},

		_wheelTimer: null,
		_mouseWheeled: function(/*Event*/ evt){
			// summary:
			//		Mouse wheel listener where supported

			evt.stopPropagation();
			evt.preventDefault();
			// FIXME: Safari bubbles

			// be nice to DOH and scroll as much as the event says to
			var wheelDelta = evt.wheelDelta / 120;
			if(Math.floor(wheelDelta) != wheelDelta){
				// If not an int multiple of 120, then its touchpad scrolling.
				// This can change very fast so just assume 1 wheel click to make it more manageable.
				wheelDelta = evt.wheelDelta > 0 ? 1 : -1;
			}
			var scrollAmount = evt.detail ? (evt.detail * -1) : wheelDelta;
			if(scrollAmount !== 0){
				var node = this[(scrollAmount > 0 ? "upArrowNode" : "downArrowNode" )];

				this._arrowPressed(node, scrollAmount, this.smallDelta);

				if(this._wheelTimer){
					this._wheelTimer.remove();
				}
				this._wheelTimer = this.defer(function(){
					this._arrowReleased(node);
				}, 50);
			}
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			this.inherited(arguments);
			if(this.focusNode){ // not set when called from postMixInProperties
				if(this.constraints.min !== undefined){
					this.focusNode.setAttribute("aria-valuemin", this.constraints.min);
				}else{
					this.focusNode.removeAttribute("aria-valuemin");
				}
				if(this.constraints.max !== undefined){
					this.focusNode.setAttribute("aria-valuemax", this.constraints.max);
				}else{
					this.focusNode.removeAttribute("aria-valuemax");
				}
			}
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', ...) works.

			this.focusNode.setAttribute("aria-valuenow", value);
			this.inherited(arguments);
		},

		postCreate: function(){
			this.inherited(arguments);

			// extra listeners
			this.own(
				on(this.domNode, mouse.wheel, lang.hitch(this, "_mouseWheeled")),
				typematic.addListener(this.upArrowNode, this.textbox, {keyCode: keys.UP_ARROW, ctrlKey: false, altKey: false, shiftKey: false, metaKey: false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout),
				typematic.addListener(this.downArrowNode, this.textbox, {keyCode: keys.DOWN_ARROW, ctrlKey: false, altKey: false, shiftKey: false, metaKey: false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout),
				typematic.addListener(this.upArrowNode, this.textbox, {keyCode: keys.PAGE_UP, ctrlKey: false, altKey: false, shiftKey: false, metaKey: false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout),
				typematic.addListener(this.downArrowNode, this.textbox, {keyCode: keys.PAGE_DOWN, ctrlKey: false, altKey: false, shiftKey: false, metaKey: false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout, this.minimumTimeout)
			);
		}
	});
});

},
'dijit/form/NumberTextBox':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch lang.mixin
	"dojo/number", // number._realNumberRegexp number.format number.parse number.regexp
	"./RangeBoundTextBox"
], function(declare, lang, number, RangeBoundTextBox){

	// module:
	//		dijit/form/NumberTextBox


	var NumberTextBoxMixin = declare("dijit.form.NumberTextBoxMixin", null, {
		// summary:
		//		A mixin for all number textboxes
		// tags:
		//		protected

		// Override ValidationTextBox.pattern.... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale (plus formatting options too?)
		pattern: function(constraints){
			// if focused, accept either currency data or NumberTextBox format
			return '(' + (this.focused && this.editOptions ? this._regExpGenerator(lang.delegate(constraints, this.editOptions)) + '|' : '')
				+ this._regExpGenerator(constraints) + ')';
		},

		/*=====
		// constraints: NumberTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including minimum/maximum allowed values) as well as
		//		formatting options like places (the number of digits to display after
		//		the decimal point).
		constraints: {},
		======*/

		// value: Number
		//		The value of this NumberTextBox as a Javascript Number (i.e., not a String).
		//		If the displayed value is blank, the value is NaN, and if the user types in
		//		an gibberish value (like "hello world"), the value is undefined
		//		(i.e. get('value') returns undefined).
		//
		//		Symmetrically, set('value', NaN) will clear the displayed value,
		//		whereas set('value', undefined) will have no effect.
		value: NaN,

		// editOptions: [protected] Object
		//		Properties to mix into constraints when the value is being edited.
		//		This is here because we edit the number in the format "12345", which is
		//		different than the display value (ex: "12,345")
		editOptions: { pattern: '#.######' },

		/*=====
		_formatter: function(value, options){
			// summary:
			//		_formatter() is called by format().  It's the base routine for formatting a number,
			//		as a string, for example converting 12345 into "12,345".
			// value: Number
			//		The number to be converted into a string.
			// options: number.__FormatOptions?
			//		Formatting options
			// tags:
			//		protected extension

			return "12345";		// String
		},
		 =====*/
		_formatter: number.format,

		/*=====
		_regExpGenerator: function(constraints){
			// summary:
			//		Generate a localized regular expression as a string, according to constraints.
			// constraints: number.__ParseOptions
			//		Formatting options
			// tags:
			//		protected

			return "(\d*).(\d*)";	// string
		},
		=====*/
		_regExpGenerator: number.regexp,

		postMixInProperties: function(){
			this.inherited(arguments);
			this._set("type", "text"); // in case type="number" was specified which messes up parse/format
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			var places = typeof constraints.places == "number"? constraints.places : 0;
			if(places){ places++; } // decimal rounding errors take away another digit of precision
			if(typeof constraints.max != "number"){
				constraints.max = 9 * Math.pow(10, 15-places);
			}
			if(typeof constraints.min != "number"){
				constraints.min = -9 * Math.pow(10, 15-places);
			}
			this.inherited(arguments, [ constraints ]);
			if(this.focusNode && this.focusNode.value && !isNaN(this.value)){
				this.set('value', this.value);
			}
		},

		_onFocus: function(){
			if(this.disabled){ return; }
			var val = this.get('value');
			if(typeof val == "number" && !isNaN(val)){
				var formattedValue = this.format(val, this.constraints);
				if(formattedValue !== undefined){
					this.textbox.value = formattedValue;
				}
			}
			this.inherited(arguments);
		},

		format: function(/*Number*/ value, /*number.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Number, according to constraints.
			// tags:
			//		protected

			var formattedValue = String(value);
			if(typeof value != "number"){ return formattedValue; }
			if(isNaN(value)){ return ""; }
			// check for exponential notation that dojo/number.format() chokes on
			if(!("rangeCheck" in this && this.rangeCheck(value, constraints)) && constraints.exponent !== false && /\de[-+]?\d/i.test(formattedValue)){
				return formattedValue;
			}
			if(this.editOptions && this.focused){
				constraints = lang.mixin({}, constraints, this.editOptions);
			}
			return this._formatter(value, constraints);
		},

		/*=====
		_parser: function(value, constraints){
			// summary:
			//		Parses the string value as a Number, according to constraints.
			// value: String
			//		String representing a number
			// constraints: number.__ParseOptions
			//		Formatting options
			// tags:
			//		protected

			return 123.45;		// Number
		},
		=====*/
		_parser: number.parse,

		parse: function(/*String*/ value, /*number.__FormatOptions*/ constraints){
			// summary:
			//		Replaceable function to convert a formatted string to a number value
			// tags:
			//		protected extension

			var v = this._parser(value, lang.mixin({}, constraints, (this.editOptions && this.focused) ? this.editOptions : {}));
			if(this.editOptions && this.focused && isNaN(v)){
				v = this._parser(value, constraints); // parse w/o editOptions: not technically needed but is nice for the user
			}
			return v;
		},

		_getDisplayedValueAttr: function(){
			var v = this.inherited(arguments);
			return isNaN(v) ? this.textbox.value : v;
		},

		filter: function(/*Number*/ value){
			// summary:
			//		This is called with both the display value (string), and the actual value (a number).
			//		When called with the actual value it does corrections so that '' etc. are represented as NaN.
			//		Otherwise it dispatches to the superclass's filter() method.
			//
			//		See `dijit/form/TextBox.filter()` for more details.
			return (value == null /* or undefined */ || value === '') ? NaN : this.inherited(arguments); // set('value', null||''||undefined) should fire onChange(NaN)
		},

		serialize: function(/*Number*/ value, /*Object?*/ options){
			// summary:
			//		Convert value (a Number) into a canonical string (ie, how the number literal is written in javascript/java/C/etc.)
			// tags:
			//		protected
			return (typeof value != "number" || isNaN(value)) ? '' : this.inherited(arguments);
		},

		_setBlurValue: function(){
			var val = lang.hitch(lang.delegate(this, { focused: true }), "get")('value'); // parse with editOptions
			this._setValueAttr(val, true);
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Hook so set('value', ...) works.
			if(value !== undefined && formattedValue === undefined){
				formattedValue = String(value);
				if(typeof value == "number"){
					if(isNaN(value)){ formattedValue = '' }
					// check for exponential notation that number.format chokes on
					else if(("rangeCheck" in this && this.rangeCheck(value, this.constraints)) || this.constraints.exponent === false || !/\de[-+]?\d/i.test(formattedValue)){
						formattedValue = undefined; // lets format compute a real string value
					}
				}else if(!value){ // 0 processed in if branch above, ''|null|undefined flows through here
					formattedValue = '';
					value = NaN;
				}else{ // non-numeric values
					value = undefined;
				}
			}
			this.inherited(arguments, [value, priorityChange, formattedValue]);
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works.
			//		Returns Number, NaN for '', or undefined for unparseable text
			var v = this.inherited(arguments); // returns Number for all values accepted by parse() or NaN for all other displayed values

			// If the displayed value of the textbox is gibberish (ex: "hello world"), this.inherited() above
			// returns NaN; this if() branch converts the return value to undefined.
			// Returning undefined prevents user text from being overwritten when doing _setValueAttr(_getValueAttr()).
			// A blank displayed value is still returned as NaN.
			if(isNaN(v) && this.textbox.value !== ''){
				if(this.constraints.exponent !== false && /\de[-+]?\d/i.test(this.textbox.value) && (new RegExp("^"+number._realNumberRegexp(lang.delegate(this.constraints))+"$").test(this.textbox.value))){	// check for exponential notation that parse() rejected (erroneously?)
					var n = Number(this.textbox.value);
					return isNaN(n) ? undefined : n; // return exponential Number or undefined for random text (may not be possible to do with the above RegExp check)
				}else{
					return undefined; // gibberish
				}
			}else{
				return v; // Number or NaN for ''
			}
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit/form/RangeBoundTextBox.isValid() to check that the editing-mode value is valid since
			// it may not be formatted according to the regExp validation rules
			if(!this.focused || this._isEmpty(this.textbox.value)){
				return this.inherited(arguments);
			}else{
				var v = this.get('value');
				if(!isNaN(v) && this.rangeCheck(v, this.constraints)){
					if(this.constraints.exponent !== false && /\de[-+]?\d/i.test(this.textbox.value)){ // exponential, parse doesn't like it
						return true; // valid exponential number in range
					}else{
						return this.inherited(arguments);
					}
				}else{
					return false;
				}
			}
		}
	});

	var NumberTextBox = declare("dijit.form.NumberTextBox", [RangeBoundTextBox, NumberTextBoxMixin], {
		// summary:
		//		A TextBox for entering numbers, with formatting and range checking
		// description:
		//		NumberTextBox is a textbox for entering and displaying numbers, supporting
		//		the following main features:
		//
		//		1. Enforce minimum/maximum allowed values (as well as enforcing that the user types
		//			a number rather than a random string)
		//		2. NLS support (altering roles of comma and dot as "thousands-separator" and "decimal-point"
		//			depending on locale).
		//		3. Separate modes for editing the value and displaying it, specifically that
		//			the thousands separator character (typically comma) disappears when editing
		//			but reappears after the field is blurred.
		//		4. Formatting and constraints regarding the number of places (digits after the decimal point)
		//			allowed on input, and number of places displayed when blurred (see `constraints` parameter).

		baseClass: "dijitTextBox dijitNumberTextBox"
	});

	NumberTextBox.Mixin = NumberTextBoxMixin;	// for monkey patching

	/*=====
	 NumberTextBox.__Constraints = declare([RangeBoundTextBox.__Constraints, number.__FormatOptions, number.__ParseOptions], {
		 // summary:
		 //		Specifies both the rules on valid/invalid values (minimum, maximum,
		 //		number of required decimal places), and also formatting options for
		 //		displaying the value when the field is not focused.
		 // example:
		 //		Minimum/maximum:
		 //		To specify a field between 0 and 120:
		 //	|		{min:0,max:120}
		 //		To specify a field that must be an integer:
		 //	|		{fractional:false}
		 //		To specify a field where 0 to 3 decimal places are allowed on input:
		 //	|		{places:'0,3'}
	 });
	 =====*/

	return NumberTextBox;
});

},
'dijit/form/SimpleTextarea':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add
	"dojo/sniff", // has("ie") has("opera")
	"./TextBox"
], function(declare, domClass, has, TextBox){

	// module:
	//		dijit/form/SimpleTextarea

	return declare("dijit.form.SimpleTextarea", TextBox, {
		// summary:
		//		A simple textarea that degrades, and responds to
		//		minimal LayoutContainer usage, and works with dijit/form/Form.
		//		Doesn't automatically size according to input, like Textarea.
		//
		// example:
		//	|	<textarea data-dojo-type="dijit/form/SimpleTextarea" name="foo" value="bar" rows=30 cols=40></textarea>
		//
		// example:
		//	|	new SimpleTextarea({ rows:20, cols:30 }, "foo");

		baseClass: "dijitTextBox dijitTextArea",

		// rows: Number
		//		The number of rows of text.
		rows: "3",

		// rows: Number
		//		The number of characters per line.
		cols: "20",

		templateString: "<textarea ${!nameAttrSetting} data-dojo-attach-point='focusNode,containerNode,textbox' autocomplete='off'></textarea>",

		postMixInProperties: function(){
			// Copy value from srcNodeRef, unless user specified a value explicitly (or there is no srcNodeRef)
			// TODO: parser will handle this in 2.0
			if(!this.value && this.srcNodeRef){
				this.value = this.srcNodeRef.value;
			}
			this.inherited(arguments);
		},

		buildRendering: function(){
			this.inherited(arguments);
			if(has("ie") && this.cols){ // attribute selectors is not supported in IE6
				domClass.add(this.textbox, "dijitTextAreaCols");
			}
		},

		filter: function(/*String*/ value){
			// Override TextBox.filter to deal with newlines... specifically (IIRC) this is for IE which writes newlines
			// as \r\n instead of just \n
			if(value){
				value = value.replace(/\r/g, "");
			}
			return this.inherited(arguments);
		},

		_onInput: function(/*Event?*/ e){
			// Override TextBox._onInput() to enforce maxLength restriction
			if(this.maxLength){
				var maxLength = parseInt(this.maxLength);
				var value = this.textbox.value.replace(/\r/g, '');
				var overflow = value.length - maxLength;
				if(overflow > 0){
					var textarea = this.textbox;
					if(textarea.selectionStart){
						var pos = textarea.selectionStart;
						var cr = 0;
						if(has("opera")){
							cr = (this.textbox.value.substring(0, pos).match(/\r/g) || []).length;
						}
						this.textbox.value = value.substring(0, pos - overflow - cr) + value.substring(pos - cr);
						textarea.setSelectionRange(pos - overflow, pos - overflow);
					}else if(this.ownerDocument.selection){ //IE
						textarea.focus();
						var range = this.ownerDocument.selection.createRange();
						// delete overflow characters
						range.moveStart("character", -overflow);
						range.text = '';
						// show cursor
						range.select();
					}
				}
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/form/Textarea':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-style", // domStyle.set
	"./_ExpandingTextAreaMixin",
	"./SimpleTextarea"
], function(declare, domStyle, _ExpandingTextAreaMixin, SimpleTextarea){

	// module:
	//		dijit/form/Textarea

	return declare("dijit.form.Textarea", [SimpleTextarea, _ExpandingTextAreaMixin], {
		// summary:
		//		A textarea widget that adjusts it's height according to the amount of data.
		//
		// description:
		//		A textarea that dynamically expands/contracts (changing it's height) as
		//		the user types, to display all the text without requiring a scroll bar.
		//
		//		Takes nearly all the parameters (name, value, etc.) that a vanilla textarea takes.
		//		Rows is not supported since this widget adjusts the height.


		// TODO: for 2.0, rename this to ExpandingTextArea, and rename SimpleTextarea to TextArea

		baseClass: "dijitTextBox dijitTextArea dijitExpandingTextArea",

		// Override SimpleTextArea.cols to default to width:100%, for backward compatibility
		cols: "",

		buildRendering: function(){
			this.inherited(arguments);

			// tweak textarea style to reduce browser differences
			domStyle.set(this.textbox, { overflowY: 'hidden', overflowX: 'auto', boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' });
		}
	});
});

},
'dijit/form/_ExpandingTextAreaMixin':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom-construct", // domConstruct.create
	"dojo/has",
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/_base/window", // win.body
	"../Viewport"
], function(declare, domConstruct, has, lang, on, win, Viewport){

	// module:
	//		dijit/form/_ExpandingTextAreaMixin

	// feature detection, true for mozilla and webkit
	has.add("textarea-needs-help-shrinking", function(){
		var body = win.body(),	// note: if multiple documents exist, doesn't matter which one we use
			te = domConstruct.create('textarea', {
			rows:"5",
			cols:"20",
			value: ' ',
			style: {zoom:1, fontSize:"12px", height:"96px", overflow:'hidden', visibility:'hidden', position:'absolute', border:"5px solid white", margin:"0", padding:"0", boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' }
		}, body, "last");
		var needsHelpShrinking = te.scrollHeight >= te.clientHeight;
		body.removeChild(te);
		return needsHelpShrinking;
	});

	return declare("dijit.form._ExpandingTextAreaMixin", null, {
		// summary:
		//		Mixin for textarea widgets to add auto-expanding capability

		_setValueAttr: function(){
			this.inherited(arguments);
			this.resize();
		},

		postCreate: function(){
			this.inherited(arguments);
			var textarea = this.textbox;
			textarea.style.overflowY = "hidden";
			this.own(on(textarea, "focus, resize", lang.hitch(this, "_resizeLater")));
		},

		startup: function(){ 
			this.inherited(arguments);
			this.own(Viewport.on("resize", lang.hitch(this, "_resizeLater")));
			this._resizeLater();
		},

		_onInput: function(e){
			this.inherited(arguments);
			this.resize();
		},

		_estimateHeight: function(){
			// summary:
			//		Approximate the height when the textarea is invisible with the number of lines in the text.
			//		Fails when someone calls setValue with a long wrapping line, but the layout fixes itself when the user clicks inside so . . .
			//		In IE, the resize event is supposed to fire when the textarea becomes visible again and that will correct the size automatically.
			//
			var textarea = this.textbox;
			// #rows = #newlines+1
			textarea.rows = (textarea.value.match(/\n/g) || []).length + 1;
		},

		_resizeLater: function(){
			this.defer("resize");
		},

		resize: function(){
			// summary:
			//		Resizes the textarea vertically (should be called after a style/value change)

			var textarea = this.textbox;

			function textareaScrollHeight(){
				var empty = false;
				if(textarea.value === ''){
					textarea.value = ' ';
					empty = true;
				}
				var sh = textarea.scrollHeight;
				if(empty){ textarea.value = ''; }
				return sh;
			}

			if(textarea.style.overflowY == "hidden"){ textarea.scrollTop = 0; }
			if(this.busyResizing){ return; }
			this.busyResizing = true;
			if(textareaScrollHeight() || textarea.offsetHeight){
				var newH = textareaScrollHeight() + Math.max(textarea.offsetHeight - textarea.clientHeight, 0);
				var newHpx = newH + "px";
				if(newHpx != textarea.style.height){
					textarea.style.height = newHpx;
					textarea.rows = 1; // rows can act like a minHeight if not cleared
				}
				if(has("textarea-needs-help-shrinking")){
					var	origScrollHeight = textareaScrollHeight(),
						newScrollHeight = origScrollHeight,
						origMinHeight = textarea.style.minHeight,
						decrement = 4, // not too fast, not too slow
						thisScrollHeight,
						origScrollTop = textarea.scrollTop;
					textarea.style.minHeight = newHpx; // maintain current height
					textarea.style.height = "auto"; // allow scrollHeight to change
					while(newH > 0){
						textarea.style.minHeight = Math.max(newH - decrement, 4) + "px";
						thisScrollHeight = textareaScrollHeight();
						var change = newScrollHeight - thisScrollHeight;
						newH -= change;
						if(change < decrement){
							break; // scrollHeight didn't shrink
						}
						newScrollHeight = thisScrollHeight;
						decrement <<= 1;
					}
					textarea.style.height = newH + "px";
					textarea.style.minHeight = origMinHeight;
					textarea.scrollTop = origScrollTop;
				}
				textarea.style.overflowY = textareaScrollHeight() > textarea.clientHeight ? "auto" : "hidden";
				if(textarea.style.overflowY == "hidden"){ textarea.scrollTop = 0; }
			}else{
				// hidden content of unknown size
				this._estimateHeight();
			}
			this.busyResizing = false;
		}
	});
});

},
'dijit/form/CurrencyTextBox':function(){
define([
	"dojo/currency", // currency._mixInDefaults currency.format currency.parse currency.regexp
	"dojo/_base/declare", // declare
	"dojo/_base/lang", // lang.hitch
	"./NumberTextBox"
], function(currency, declare, lang, NumberTextBox){

	// module:
	//		dijit/form/CurrencyTextBox

	/*=====
	var __Constraints = declare([NumberTextBox.__Constraints, currency.__FormatOptions, currency.__ParseOptions], {
		// summary:
		//		Specifies both the rules on valid/invalid values (minimum, maximum,
		//		number of required decimal places), and also formatting options for
		//		displaying the value when the field is not focused (currency symbol,
		//		etc.)
		// description:
		//		Follows the pattern of `dijit/form/NumberTextBox.__Constraints`.
		//		In general developers won't need to set this parameter
		// example:
		//		To ensure that the user types in the cents (for example, 1.00 instead of just 1):
		//	|		{fractional:true}
	});
	=====*/

	return declare("dijit.form.CurrencyTextBox", NumberTextBox, {
		// summary:
		//		A validating currency textbox
		// description:
		//		CurrencyTextBox is similar to `dijit/form/NumberTextBox` but has a few
		//		extra features related to currency:
		//
		//		1. After specifying the currency type (american dollars, euros, etc.) it automatically
		//			sets parse/format options such as how many decimal places to show.
		//		2. The currency mark (dollar sign, euro mark, etc.) is displayed when the field is blurred
		//			but erased during editing, so that the user can just enter a plain number.

		// currency: [const] String
		//		the [ISO4217](http://en.wikipedia.org/wiki/ISO_4217) currency code, a three letter sequence like "USD"
		currency: "",

		/*=====
		// constraints: __Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including minimum/maximum allowed values) as well as
		//		formatting options.
		constraints: {},
		======*/

		baseClass: "dijitTextBox dijitCurrencyTextBox",

		// Override NumberTextBox._formatter to deal with currencies, ex: converts "123.45" to "$123.45"
		_formatter: currency.format,

		_parser: currency.parse,

		_regExpGenerator: currency.regexp,

		parse: function(/*String*/ value, /*Object*/ constraints){
			// summary:
			//		Parses string value as a Currency, according to the constraints object
			// tags:
			//		protected extension
			var v = this.inherited(arguments);
			if(isNaN(v) && /\d+/.test(value)){ // currency parse failed, but it could be because they are using NumberTextBox format so try its parse
				v = lang.hitch(lang.delegate(this, { _parser: NumberTextBox.prototype._parser }), "inherited")(arguments);
			}
			return v;
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			if(!constraints.currency && this.currency){
				constraints.currency = this.currency;
			}
			this.inherited(arguments, [ currency._mixInDefaults(lang.mixin(constraints, { exponent: false })) ]); // get places
		}
	});
});

},
'dijit/form/HorizontalSlider':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dnd/move",
	"dojo/_base/fx", // fx.animateProperty
	"dojo/dom-geometry", // domGeometry.position
	"dojo/dom-style", // domStyle.getComputedStyle
	"dojo/keys", // keys.DOWN_ARROW keys.END keys.HOME keys.LEFT_ARROW keys.PAGE_DOWN keys.PAGE_UP keys.RIGHT_ARROW keys.UP_ARROW
	"dojo/_base/lang", // lang.hitch
	"dojo/sniff", // has("ie") has("mozilla")
	"dojo/dnd/Moveable", // Moveable
	"dojo/dnd/Mover", // Mover Mover.prototype.destroy.apply
	"dojo/query", // query
	"dojo/mouse", // mouse.wheel
	"dojo/on",
	"../_base/manager", // defaultDuration
	"../focus", // focus.focus()
	"../typematic",
	"./Button",
	"./_FormValueWidget",
	"../_Container",
	"dojo/text!./templates/HorizontalSlider.html"
], function(array, declare, move, fx, domGeometry, domStyle, keys, lang, has, Moveable, Mover, query, mouse, on,
			manager, focus, typematic, Button, _FormValueWidget, _Container, template){

	// module:
	//		dijit/form/HorizontalSlider

	var _SliderMover = declare("dijit.form._SliderMover", Mover, {
		onMouseMove: function(e){
			var widget = this.widget;
			var abspos = widget._abspos;
			if(!abspos){
				abspos = widget._abspos = domGeometry.position(widget.sliderBarContainer, true);
				widget._setPixelValue_ = lang.hitch(widget, "_setPixelValue");
				widget._isReversed_ = widget._isReversed();
			}
			var pixelValue = e[widget._mousePixelCoord] - abspos[widget._startingPixelCoord];
			widget._setPixelValue_(widget._isReversed_ ? (abspos[widget._pixelCount] - pixelValue) : pixelValue, abspos[widget._pixelCount], false);
		},

		destroy: function(e){
			Mover.prototype.destroy.apply(this, arguments);
			var widget = this.widget;
			widget._abspos = null;
			widget._setValueAttr(widget.value, true);
		}
	});

	var HorizontalSlider = declare("dijit.form.HorizontalSlider", [_FormValueWidget, _Container], {
		// summary:
		//		A form widget that allows one to select a value with a horizontally draggable handle

		templateString: template,

		// Overrides FormValueWidget.value to indicate numeric value
		value: 0,

		// showButtons: [const] Boolean
		//		Show increment/decrement buttons at the ends of the slider?
		showButtons: true,

		// minimum: [const] Integer
		//		The minimum value the slider can be set to.
		minimum: 0,

		// maximum: [const] Integer
		//		The maximum value the slider can be set to.
		maximum: 100,

		// discreteValues: Integer
		//		If specified, indicates that the slider handle has only 'discreteValues' possible positions,
		//		and that after dragging the handle, it will snap to the nearest possible position.
		//		Thus, the slider has only 'discreteValues' possible values.
		//
		//		For example, if minimum=10, maxiumum=30, and discreteValues=3, then the slider handle has
		//		three possible positions, representing values 10, 20, or 30.
		//
		//		If discreteValues is not specified or if it's value is higher than the number of pixels
		//		in the slider bar, then the slider handle can be moved freely, and the slider's value will be
		//		computed/reported based on pixel position (in this case it will likely be fractional,
		//		such as 123.456789).
		discreteValues: Infinity,

		// pageIncrement: Integer
		//		If discreteValues is also specified, this indicates the amount of clicks (ie, snap positions)
		//		that the slider handle is moved via pageup/pagedown keys.
		//		If discreteValues is not specified, it indicates the number of pixels.
		pageIncrement: 2,

		// clickSelect: Boolean
		//		If clicking the slider bar changes the value or not
		clickSelect: true,

		// slideDuration: Number
		//		The time in ms to take to animate the slider handle from 0% to 100%,
		//		when clicking the slider bar to make the handle move.
		slideDuration: manager.defaultDuration,

		// Map widget attributes to DOMNode attributes.
		_setIdAttr: "", // Override _FormWidget which sends id to focusNode
		_setNameAttr: "valueNode", // Override default behavior to send to focusNode

		baseClass: "dijitSlider",

		// Apply CSS classes to up/down arrows and handle per mouse state
		cssStateNodes: {
			incrementButton: "dijitSliderIncrementButton",
			decrementButton: "dijitSliderDecrementButton",
			focusNode: "dijitSliderThumb"
		},

		_mousePixelCoord: "pageX",
		_pixelCount: "w",
		_startingPixelCoord: "x",
		_handleOffsetCoord: "left",
		_progressPixelSize: "width",

		_onKeyUp: function(/*Event*/ e){
			if(this.disabled || this.readOnly || e.altKey || e.ctrlKey || e.metaKey){
				return;
			}
			this._setValueAttr(this.value, true);
		},

		_onKeyDown: function(/*Event*/ e){
			if(this.disabled || this.readOnly || e.altKey || e.ctrlKey || e.metaKey){
				return;
			}
			switch(e.keyCode){
				case keys.HOME:
					this._setValueAttr(this.minimum, false);
					break;
				case keys.END:
					this._setValueAttr(this.maximum, false);
					break;
				// this._descending === false: if ascending vertical (min on top)
				// (this._descending || this.isLeftToRight()): if left-to-right horizontal or descending vertical
				case ((this._descending || this.isLeftToRight()) ? keys.RIGHT_ARROW : keys.LEFT_ARROW):
				case (this._descending === false ? keys.DOWN_ARROW : keys.UP_ARROW):
				case (this._descending === false ? keys.PAGE_DOWN : keys.PAGE_UP):
					this.increment(e);
					break;
				case ((this._descending || this.isLeftToRight()) ? keys.LEFT_ARROW : keys.RIGHT_ARROW):
				case (this._descending === false ? keys.UP_ARROW : keys.DOWN_ARROW):
				case (this._descending === false ? keys.PAGE_UP : keys.PAGE_DOWN):
					this.decrement(e);
					break;
				default:
					return;
			}
			e.stopPropagation();
			e.preventDefault();
		},

		_onHandleClick: function(e){
			if(this.disabled || this.readOnly){
				return;
			}
			if(!has("ie")){
				// make sure you get focus when dragging the handle
				// (but don't do on IE because it causes a flicker on mouse up (due to blur then focus)
				focus.focus(this.sliderHandle);
			}
			e.stopPropagation();
			e.preventDefault();
		},

		_isReversed: function(){
			// summary:
			//		Returns true if direction is from right to left
			// tags:
			//		protected extension
			return !this.isLeftToRight();
		},

		_onBarClick: function(e){
			if(this.disabled || this.readOnly || !this.clickSelect){
				return;
			}
			focus.focus(this.sliderHandle);
			e.stopPropagation();
			e.preventDefault();
			var abspos = domGeometry.position(this.sliderBarContainer, true);
			var pixelValue = e[this._mousePixelCoord] - abspos[this._startingPixelCoord];
			this._setPixelValue(this._isReversed() ? (abspos[this._pixelCount] - pixelValue) : pixelValue, abspos[this._pixelCount], true);
			this._movable.onMouseDown(e);
		},

		_setPixelValue: function(/*Number*/ pixelValue, /*Number*/ maxPixels, /*Boolean?*/ priorityChange){
			if(this.disabled || this.readOnly){
				return;
			}
			var count = this.discreteValues;
			if(count <= 1 || count == Infinity){
				count = maxPixels;
			}
			count--;
			var pixelsPerValue = maxPixels / count;
			var wholeIncrements = Math.round(pixelValue / pixelsPerValue);
			this._setValueAttr(Math.max(Math.min((this.maximum - this.minimum) * wholeIncrements / count + this.minimum, this.maximum), this.minimum), priorityChange);
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', value) works.
			this._set("value", value);
			this.valueNode.value = value;
			this.focusNode.setAttribute("aria-valuenow", value);
			this.inherited(arguments);
			var percent = (value - this.minimum) / (this.maximum - this.minimum);
			var progressBar = (this._descending === false) ? this.remainingBar : this.progressBar;
			var remainingBar = (this._descending === false) ? this.progressBar : this.remainingBar;
			if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
				this._inProgressAnim.stop(true);
			}
			if(priorityChange && this.slideDuration > 0 && progressBar.style[this._progressPixelSize]){
				// animate the slider
				var _this = this;
				var props = {};
				var start = parseFloat(progressBar.style[this._progressPixelSize]);
				var duration = this.slideDuration * (percent - start / 100);
				if(duration == 0){
					return;
				}
				if(duration < 0){
					duration = 0 - duration;
				}
				props[this._progressPixelSize] = { start: start, end: percent * 100, units: "%" };
				this._inProgressAnim = fx.animateProperty({ node: progressBar, duration: duration,
					onAnimate: function(v){
						remainingBar.style[_this._progressPixelSize] = (100 - parseFloat(v[_this._progressPixelSize])) + "%";
					},
					onEnd: function(){
						delete _this._inProgressAnim;
					},
					properties: props
				});
				this._inProgressAnim.play();
			}else{
				progressBar.style[this._progressPixelSize] = (percent * 100) + "%";
				remainingBar.style[this._progressPixelSize] = ((1 - percent) * 100) + "%";
			}
		},

		_bumpValue: function(signedChange, /*Boolean?*/ priorityChange){
			if(this.disabled || this.readOnly){
				return;
			}
			var s = domStyle.getComputedStyle(this.sliderBarContainer);
			var c = domGeometry.getContentBox(this.sliderBarContainer, s);
			var count = this.discreteValues;
			if(count <= 1 || count == Infinity){
				count = c[this._pixelCount];
			}
			count--;
			var value = (this.value - this.minimum) * count / (this.maximum - this.minimum) + signedChange;
			if(value < 0){
				value = 0;
			}
			if(value > count){
				value = count;
			}
			value = value * (this.maximum - this.minimum) / count + this.minimum;
			this._setValueAttr(value, priorityChange);
		},

		_onClkBumper: function(val){
			if(this.disabled || this.readOnly || !this.clickSelect){
				return;
			}
			this._setValueAttr(val, true);
		},

		_onClkIncBumper: function(){
			this._onClkBumper(this._descending === false ? this.minimum : this.maximum);
		},

		_onClkDecBumper: function(){
			this._onClkBumper(this._descending === false ? this.maximum : this.minimum);
		},

		decrement: function(/*Event*/ e){
			// summary:
			//		Decrement slider
			// tags:
			//		private
			this._bumpValue(e.keyCode == keys.PAGE_DOWN ? -this.pageIncrement : -1);
		},

		increment: function(/*Event*/ e){
			// summary:
			//		Increment slider
			// tags:
			//		private
			this._bumpValue(e.keyCode == keys.PAGE_UP ? this.pageIncrement : 1);
		},

		_mouseWheeled: function(/*Event*/ evt){
			// summary:
			//		Event handler for mousewheel where supported
			evt.stopPropagation();
			evt.preventDefault();
			this._bumpValue(evt.wheelDelta < 0 ? -1 : 1, true); // negative scroll acts like a decrement
		},

		startup: function(){
			if(this._started){
				return;
			}

			array.forEach(this.getChildren(), function(child){
				if(this[child.container] != this.containerNode){
					this[child.container].appendChild(child.domNode);
				}
			}, this);

			this.inherited(arguments);
		},

		_typematicCallback: function(/*Number*/ count, /*Object*/ button, /*Event*/ e){
			if(count == -1){
				this._setValueAttr(this.value, true);
			}else{
				this[(button == (this._descending ? this.incrementButton : this.decrementButton)) ? "decrement" : "increment"](e);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			if(this.showButtons){
				this.incrementButton.style.display = "";
				this.decrementButton.style.display = "";
			}

			// find any associated label element and add to slider focusnode.
			var label = query('label[for="' + this.id + '"]');
			if(label.length){
				if(!label[0].id){
					label[0].id = this.id + "_label";
				}
				this.focusNode.setAttribute("aria-labelledby", label[0].id);
			}

			this.focusNode.setAttribute("aria-valuemin", this.minimum);
			this.focusNode.setAttribute("aria-valuemax", this.maximum);
		},

		postCreate: function(){
			this.inherited(arguments);

			if(this.showButtons){
				this.own(
					typematic.addMouseListener(this.decrementButton, this, "_typematicCallback", 25, 500),
					typematic.addMouseListener(this.incrementButton, this, "_typematicCallback", 25, 500)
				);
			}
			this.own(
				on(this.domNode, mouse.wheel, lang.hitch(this, "_mouseWheeled"))
			);

			// define a custom constructor for a SliderMover that points back to me
			var mover = declare(_SliderMover, {
				widget: this
			});
			this._movable = new Moveable(this.sliderHandle, {mover: mover});

			this._layoutHackIE7();
		},

		destroy: function(){
			this._movable.destroy();
			if(this._inProgressAnim && this._inProgressAnim.status != "stopped"){
				this._inProgressAnim.stop(true);
			}
			this.inherited(arguments);
		}
	});

	HorizontalSlider._Mover = _SliderMover;	// for monkey patching

	return HorizontalSlider;
});

},
'dojo/dnd/move':function(){
define([
	"../_base/declare",
	"../dom-geometry", "../dom-style",
	"./common", "./Mover", "./Moveable"
], function(declare, domGeom, domStyle, dnd, Mover, Moveable){

// module:
//		dojo/dnd/move

/*=====
var __constrainedMoveableArgs = declare([Moveable.__MoveableArgs], {
	// constraints: Function
	//		Calculates a constraint box.
	//		It is called in a context of the moveable object.
	constraints: function(){},

	// within: Boolean
	//		restrict move within boundaries.
	within: false
});
=====*/

var constrainedMoveable = declare("dojo.dnd.move.constrainedMoveable", Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,

	constructor: function(node, params){
		// summary:
		//		an object that makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __constrainedMoveableArgs?
		//		an optional object with additional parameters;
		//		the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/*Mover*/ mover){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover);
		c.r = c.l + c.w;
		c.b = c.t + c.h;
		if(this.within){
			var mb = domGeom.getMarginSize(mover.node);
			c.r -= mb.w;
			c.b -= mb.h;
		}
	},
	onMove: function(/*Mover*/ mover, /*Object*/ leftTop){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		var c = this.constraintBox, s = mover.node.style;
		this.onMoving(mover, leftTop);
		leftTop.l = leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l;
		leftTop.t = leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	}
});

/*=====
var __boxConstrainedMoveableArgs = declare([__constrainedMoveableArgs], {
	// box: Object
	//		a constraint box
	box: {}
});
=====*/

var boxConstrainedMoveable = declare("dojo.dnd.move.boxConstrainedMoveable", constrainedMoveable, {
	// box:
	//		object attributes (for markup)
	box: {},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __boxConstrainedMoveableArgs?
		//		an optional object with parameters
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

/*=====
var __parentConstrainedMoveableArgs = declare( [__constrainedMoveableArgs], {
	// area: String
	//		A parent's area to restrict the move.
	//		Can be "margin", "border", "padding", or "content".
	area: ""
});
=====*/

var parentConstrainedMoveable = declare("dojo.dnd.move.parentConstrainedMoveable", constrainedMoveable, {
	// area:
	//		object attributes (for markup)
	area: "content",

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: __parentConstrainedMoveableArgs?
		//		an optional object with parameters
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode,
				s = domStyle.getComputedStyle(n),
				mb = domGeom.getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = domGeom.getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = domGeom.getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = domGeom.getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});


return {
	// summary:
	//		TODOC
	constrainedMoveable: constrainedMoveable,
	boxConstrainedMoveable: boxConstrainedMoveable,
	parentConstrainedMoveable: parentConstrainedMoveable
};

});

},
'dijit/layout/BorderContainer':function(){
define([
	"dojo/_base/array", // array.filter array.forEach array.map
	"dojo/cookie", // cookie
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.remove domClass.toggle
	"dojo/dom-construct", // domConstruct.destroy domConstruct.place
	"dojo/dom-geometry", // domGeometry.marginBox
	"dojo/dom-style", // domStyle.style
	"dojo/keys",
	"dojo/_base/lang", // getObject() hitch() delegate()
	"dojo/on",
	"dojo/touch",
	"../_WidgetBase",
	"../_Widget",
	"../_TemplatedMixin",
	"./LayoutContainer",
	"./utils"        // layoutUtils.layoutChildren
], function(array, cookie, declare, domClass, domConstruct, domGeometry, domStyle, keys, lang, on, touch,
			_WidgetBase, _Widget, _TemplatedMixin, LayoutContainer, layoutUtils){

	// module:
	//		dijit/layout/BorderContainer

	var _Splitter = declare("dijit.layout._Splitter", [_Widget, _TemplatedMixin ], {
		// summary:
		//		A draggable spacer between two items in a `dijit/layout/BorderContainer`.
		// description:
		//		This is instantiated by `dijit/layout/BorderContainer`.  Users should not
		//		create it directly.
		// tags:
		//		private

		/*=====
		 // container: [const] dijit/layout/BorderContainer
		 //		Pointer to the parent BorderContainer
		 container: null,

		 // child: [const] dijit/layout/_LayoutWidget
		 //		Pointer to the pane associated with this splitter
		 child: null,

		 // region: [const] String
		 //		Region of pane associated with this splitter.
		 //		"top", "bottom", "left", "right".
		 region: null,
		 =====*/

		// live: [const] Boolean
		//		If true, the child's size changes and the child widget is redrawn as you drag the splitter;
		//		otherwise, the size doesn't change until you drop the splitter (by mouse-up)
		live: true,

		templateString: '<div class="dijitSplitter" data-dojo-attach-event="onkeydown:_onKeyDown,press:_startDrag,onmouseenter:_onMouse,onmouseleave:_onMouse" tabIndex="0" role="separator"><div class="dijitSplitterThumb"></div></div>',

		constructor: function(){
			this._handlers = [];
		},

		postMixInProperties: function(){
			this.inherited(arguments);

			this.horizontal = /top|bottom/.test(this.region);
			this._factor = /top|left/.test(this.region) ? 1 : -1;
			this._cookieName = this.container.id + "_" + this.region;
		},

		buildRendering: function(){
			this.inherited(arguments);

			domClass.add(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V"));

			if(this.container.persist){
				// restore old size
				var persistSize = cookie(this._cookieName);
				if(persistSize){
					this.child.domNode.style[this.horizontal ? "height" : "width"] = persistSize;
				}
			}
		},

		_computeMaxSize: function(){
			// summary:
			//		Return the maximum size that my corresponding pane can be set to

			var dim = this.horizontal ? 'h' : 'w',
				childSize = domGeometry.getMarginBox(this.child.domNode)[dim],
				center = array.filter(this.container.getChildren(), function(child){
					return child.region == "center";
				})[0];

			// Can expand until center is crushed.  But always leave room for center's padding + border,
			//  otherwise on the next call domGeometry methods start to lie about size.
			var spaceAvailable = domGeometry.getContentBox(center.domNode)[dim] - 10;

			return Math.min(this.child.maxSize, childSize + spaceAvailable);
		},

		_startDrag: function(e){
			if(!this.cover){
				this.cover = domConstruct.place("<div class=dijitSplitterCover></div>", this.child.domNode, "after");
			}
			domClass.add(this.cover, "dijitSplitterCoverActive");

			// Safeguard in case the stop event was missed.  Shouldn't be necessary if we always get the mouse up.
			if(this.fake){
				domConstruct.destroy(this.fake);
			}
			if(!(this._resize = this.live)){ //TODO: disable live for IE6?
				// create fake splitter to display at old position while we drag
				(this.fake = this.domNode.cloneNode(true)).removeAttribute("id");
				domClass.add(this.domNode, "dijitSplitterShadow");
				domConstruct.place(this.fake, this.domNode, "after");
			}
			domClass.add(this.domNode, "dijitSplitterActive dijitSplitter" + (this.horizontal ? "H" : "V") + "Active");
			if(this.fake){
				domClass.remove(this.fake, "dijitSplitterHover dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover");
			}

			//Performance: load data info local vars for onmousevent function closure
			var factor = this._factor,
				isHorizontal = this.horizontal,
				axis = isHorizontal ? "pageY" : "pageX",
				pageStart = e[axis],
				splitterStyle = this.domNode.style,
				dim = isHorizontal ? 'h' : 'w',
				childCS = domStyle.getComputedStyle(this.child.domNode),
				childStart = domGeometry.getMarginBox(this.child.domNode, childCS)[dim],
				max = this._computeMaxSize(),
				min = Math.max(this.child.minSize, domGeometry.getPadBorderExtents(this.child.domNode, childCS)[dim] + 10),
				region = this.region,
				splitterAttr = region == "top" || region == "bottom" ? "top" : "left", // style attribute of splitter to adjust
				splitterStart = parseInt(splitterStyle[splitterAttr], 10),
				resize = this._resize,
				layoutFunc = lang.hitch(this.container, "_layoutChildren", this.child.id),
				de = this.ownerDocument;

			this._handlers = this._handlers.concat([
				on(de, touch.move, this._drag = function(e, forceResize){
					var delta = e[axis] - pageStart,
						childSize = factor * delta + childStart,
						boundChildSize = Math.max(Math.min(childSize, max), min);

					if(resize || forceResize){
						layoutFunc(boundChildSize);
					}
					// TODO: setting style directly (usually) sets content box size, need to set margin box size
					splitterStyle[splitterAttr] = delta + splitterStart + factor * (boundChildSize - childSize) + "px";
				}),
				on(de, "dragstart", function(e){
					e.stopPropagation();
					e.preventDefault();
				}),
				on(this.ownerDocumentBody, "selectstart", function(e){
					e.stopPropagation();
					e.preventDefault();
				}),
				on(de, touch.release, lang.hitch(this, "_stopDrag"))
			]);
			e.stopPropagation();
			e.preventDefault();
		},

		_onMouse: function(e){
			// summary:
			//		Handler for onmouseenter / onmouseleave events
			var o = (e.type == "mouseover" || e.type == "mouseenter");
			domClass.toggle(this.domNode, "dijitSplitterHover", o);
			domClass.toggle(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover", o);
		},

		_stopDrag: function(e){
			try{
				if(this.cover){
					domClass.remove(this.cover, "dijitSplitterCoverActive");
				}
				if(this.fake){
					domConstruct.destroy(this.fake);
				}
				domClass.remove(this.domNode, "dijitSplitterActive dijitSplitter"
					+ (this.horizontal ? "H" : "V") + "Active dijitSplitterShadow");
				this._drag(e); //TODO: redundant with onmousemove?
				this._drag(e, true);
			}finally{
				this._cleanupHandlers();
				delete this._drag;
			}

			if(this.container.persist){
				cookie(this._cookieName, this.child.domNode.style[this.horizontal ? "height" : "width"], {expires: 365});
			}
		},

		_cleanupHandlers: function(){
			var h;
			while(h = this._handlers.pop()){
				h.remove();
			}
		},

		_onKeyDown: function(/*Event*/ e){
			// should we apply typematic to this?
			this._resize = true;
			var horizontal = this.horizontal;
			var tick = 1;
			switch(e.keyCode){
				case horizontal ? keys.UP_ARROW : keys.LEFT_ARROW:
					tick *= -1;
//				break;
				case horizontal ? keys.DOWN_ARROW : keys.RIGHT_ARROW:
					break;
				default:
//				this.inherited(arguments);
					return;
			}
			var childSize = domGeometry.getMarginSize(this.child.domNode)[ horizontal ? 'h' : 'w' ] + this._factor * tick;
			this.container._layoutChildren(this.child.id, Math.max(Math.min(childSize, this._computeMaxSize()), this.child.minSize));
			e.stopPropagation();
			e.preventDefault();
		},

		destroy: function(){
			this._cleanupHandlers();
			delete this.child;
			delete this.container;
			delete this.cover;
			delete this.fake;
			this.inherited(arguments);
		}
	});

	var _Gutter = declare("dijit.layout._Gutter", [_Widget, _TemplatedMixin], {
		// summary:
		//		Just a spacer div to separate side pane from center pane.
		//		Basically a trick to lookup the gutter/splitter width from the theme.
		// description:
		//		Instantiated by `dijit/layout/BorderContainer`.  Users should not
		//		create directly.
		// tags:
		//		private

		templateString: '<div class="dijitGutter" role="presentation"></div>',

		postMixInProperties: function(){
			this.inherited(arguments);
			this.horizontal = /top|bottom/.test(this.region);
		},

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode, "dijitGutter" + (this.horizontal ? "H" : "V"));
		}
	});

	var BorderContainer = declare("dijit.layout.BorderContainer", LayoutContainer, {
		// summary:
		//		A BorderContainer is a `dijit/LayoutContainer` that can have draggable splitters between the children,
		//		in order to adjust their sizes.
		//
		//		In addition, it automatically adds some space between the children even
		//		if they don't have a draggable splitter between them, and space between the edge of the BorderContainer
		//		and the children that are adjacent to the edge.  Note that the intended style is that all the children
		//		have borders, but (despite the name) the BorderContainer itself does not.
		//
		//		See `BorderContainer.ChildWidgetProperties` for details on the properties that can be set on
		//		children of a `BorderContainer`.

		// gutters: [const] Boolean
		//		Give each pane a border and margin.
		//		Margin determined by domNode.paddingLeft.
		//		When false, only resizable panes have a gutter (i.e. draggable splitter) for resizing.
		gutters: true,

		// liveSplitters: [const] Boolean
		//		Specifies whether splitters resize as you drag (true) or only upon mouseup (false)
		liveSplitters: true,

		// persist: Boolean
		//		Save splitter positions in a cookie.
		persist: false,

		baseClass: "dijitBorderContainer",

		// _splitterClass: Function||String
		//		Optional hook to override the default Splitter widget used by BorderContainer
		_splitterClass: _Splitter,

		postMixInProperties: function(){
			// change class name to indicate that BorderContainer is being used purely for
			// layout (like LayoutContainer) rather than for pretty formatting.
			if(!this.gutters){
				this.baseClass += "NoGutter";
			}
			this.inherited(arguments);
		},

		_setupChild: function(/*dijit/_WidgetBase*/ child){
			// Override LayoutContainer._setupChild().

			this.inherited(arguments);

			var region = child.region, ltr = child.isLeftToRight();
			if(region == "leading"){
				region = ltr ? "left" : "right";
			}
			if(region == "trailing"){
				region = ltr ? "right" : "left";
			}

			if(region){
				// Create draggable splitter for resizing pane,
				// or alternately if splitter=false but BorderContainer.gutters=true then
				// insert dummy div just for spacing
				if(region != "center" && (child.splitter || this.gutters) && !child._splitterWidget){
					var _Splitter = child.splitter ? this._splitterClass : _Gutter;
					if(lang.isString(_Splitter)){
						_Splitter = lang.getObject(_Splitter);	// for back-compat, remove in 2.0
					}
					var splitter = new _Splitter({
						id: child.id + "_splitter",
						container: this,
						child: child,
						region: region,
						live: this.liveSplitters
					});
					splitter.isSplitter = true;
					child._splitterWidget = splitter;

					// Make the tab order match the visual layout by placing the splitter before or after the pane,
					// depending on where the splitter is visually compared to the pane.
					var before = region == "bottom" || region == (this.isLeftToRight() ? "right" : "left");
					domConstruct.place(splitter.domNode, child.domNode, before ? "before" : "after");

					// Splitters aren't added as Contained children, so we need to call startup explicitly
					splitter.startup();
				}
			}
		},

		layout: function(){
			// Implement _LayoutWidget.layout() virtual method.
			this._layoutChildren();
		},

		removeChild: function(/*dijit/_WidgetBase*/ child){
			// Override _LayoutWidget.removeChild().

			var splitter = child._splitterWidget;
			if(splitter){
				splitter.destroy();
				delete child._splitterWidget;
			}

			this.inherited(arguments);
		},

		getChildren: function(){
			// Override _LayoutWidget.getChildren() to only return real children, not the splitters.
			return array.filter(this.inherited(arguments), function(widget){
				return !widget.isSplitter;
			});
		},

		// TODO: remove in 2.0
		getSplitter: function(/*String*/region){
			// summary:
			//		Returns the widget responsible for rendering the splitter associated with region
			// tags:
			//		deprecated
			return array.filter(this.getChildren(), function(child){
				return child.region == region;
			})[0]._splitterWidget;
		},

		resize: function(newSize, currentSize){
			// Overrides _LayoutWidget.resize().

			// resetting potential padding to 0px to provide support for 100% width/height + padding
			// TODO: this hack doesn't respect the box model and is a temporary fix
			if(!this.cs || !this.pe){
				var node = this.domNode;
				this.cs = domStyle.getComputedStyle(node);
				this.pe = domGeometry.getPadExtents(node, this.cs);
				this.pe.r = domStyle.toPixelValue(node, this.cs.paddingRight);
				this.pe.b = domStyle.toPixelValue(node, this.cs.paddingBottom);

				domStyle.set(node, "padding", "0px");
			}

			this.inherited(arguments);
		},

		_layoutChildren: function(/*String?*/ changedChildId, /*Number?*/ changedChildSize){
			// summary:
			//		This is the main routine for setting size/position of each child.
			// description:
			//		With no arguments, measures the height of top/bottom panes, the width
			//		of left/right panes, and then sizes all panes accordingly.
			//
			//		With changedRegion specified (as "left", "top", "bottom", or "right"),
			//		it changes that region's width/height to changedRegionSize and
			//		then resizes other regions that were affected.
			// changedChildId:
			//		Id of the child which should be resized because splitter was dragged.
			// changedChildSize:
			//		The new width/height (in pixels) to make specified child

			if(!this._borderBox || !this._borderBox.h){
				// We are currently hidden, or we haven't been sized by our parent yet.
				// Abort.   Someone will resize us later.
				return;
			}

			// Combining the externally specified children with splitters and gutters
			var childrenAndSplitters = [];
			array.forEach(this._getOrderedChildren(), function(pane){
				childrenAndSplitters.push(pane);
				if(pane._splitterWidget){
					childrenAndSplitters.push(pane._splitterWidget);
				}
			});

			// Compute the box in which to lay out my children
			var dim = {
				l: this.pe.l,
				t: this.pe.t,
				w: this._borderBox.w - this.pe.w,
				h: this._borderBox.h - this.pe.h
			};

			// Layout the children, possibly changing size due to a splitter drag
			layoutUtils.layoutChildren(this.domNode, dim, childrenAndSplitters,
				changedChildId, changedChildSize);
		},

		destroyRecursive: function(){
			// Destroy splitters first, while getChildren() still works
			array.forEach(this.getChildren(), function(child){
				var splitter = child._splitterWidget;
				if(splitter){
					splitter.destroy();
				}
				delete child._splitterWidget;
			});

			// Then destroy the real children, and myself
			this.inherited(arguments);
		}
	});

	BorderContainer.ChildWidgetProperties = {
		// summary:
		//		These properties can be specified for the children of a BorderContainer.

		// splitter: [const] Boolean
		//		Parameter for children where region != "center".
		//		If true, enables user to resize the widget by putting a draggable splitter between
		//		this widget and the region=center widget.
		splitter: false,

		// minSize: [const] Number
		//		Specifies a minimum size (in pixels) for this widget when resized by a splitter.
		minSize: 0,

		// maxSize: [const] Number
		//		Specifies a maximum size (in pixels) for this widget when resized by a splitter.
		maxSize: Infinity
	};
	lang.mixin(BorderContainer.ChildWidgetProperties, LayoutContainer.ChildWidgetProperties);

	// Since any widget can be specified as a BorderContainer child, mix it
	// into the base widget class.  (This is a hack, but it's effective.)
	// This is for the benefit of the parser.   Remove for 2.0.  Also, hide from doc viewer.
	lang.extend(_WidgetBase, /*===== {} || =====*/ BorderContainer.ChildWidgetProperties);

	// For monkey patching
	BorderContainer._Splitter = _Splitter;
	BorderContainer._Gutter = _Gutter;

	return BorderContainer;
});

},
'dijit/layout/LayoutContainer':function(){
define([
	"dojo/_base/array",
	"dojo/_base/declare", // declare
	"dojo/dom-class",
	"dojo/dom-style",
	"dojo/_base/lang",
	"../_WidgetBase",
	"./_LayoutWidget",
	"./utils" // layoutUtils.layoutChildren
], function(array, declare, domClass, domStyle, lang, _WidgetBase, _LayoutWidget, layoutUtils){

	// module:
	//		dijit/layout/LayoutContainer

	var LayoutContainer = declare("dijit.layout.LayoutContainer", _LayoutWidget, {
		// summary:
		//		A LayoutContainer is a box with a specified size, such as style="width: 500px; height: 500px;",
		//		that contains a child widget marked region="center" and optionally children widgets marked
		//		region equal to "top", "bottom", "leading", "trailing", "left" or "right".
		//		Children along the edges will be laid out according to width or height dimensions. The remaining
		//		space is designated for the center region.
		//
		//		The outer size must be specified on the LayoutContainer node.  Width must be specified for the sides
		//		and height for the top and bottom, respectively.  No dimensions should be specified on the center;
		//		it will fill the remaining space.  Regions named "leading" and "trailing" may be used just like
		//		"left" and "right" except that they will be reversed in right-to-left environments.
		//
		//		For complex layouts, multiple children can be specified for a single region.   In this case, the
		//		layoutPriority flag on the children determines which child is closer to the edge (low layoutPriority)
		//		and which child is closer to the center (high layoutPriority).   layoutPriority can also be used
		//		instead of the design attribute to control layout precedence of horizontal vs. vertical panes.
		//
		//		See `LayoutContainer.ChildWidgetProperties` for details on the properties that can be set on
		//		children of a `LayoutContainer`.
		//
		//		If layoutPriority is not set, lays out each child in the natural order the children occur in.
		//		Basically each child is laid out into the "remaining space", where "remaining space" is initially
		//		the content area of this widget, but is reduced to a smaller rectangle each time a child is added.

		// design: String
		//		Which design is used for the layout:
		//
		//		- "headline" (default) where the top and bottom extend the full width of the container
		//		- "sidebar" where the left and right sides extend from top to bottom.
		design: "headline",

		baseClass: "dijitLayoutContainer",

		startup: function(){
			if(this._started){
				return;
			}
			array.forEach(this.getChildren(), this._setupChild, this);
			this.inherited(arguments);
		},

		_setupChild: function(/*dijit/_WidgetBase*/ child){
			// Override _LayoutWidget._setupChild().

			this.inherited(arguments);

			var region = child.region;
			if(region){
				domClass.add(child.domNode, this.baseClass + "Pane");
			}
		},

		_getOrderedChildren: function(){
			// summary:
			//		Return list of my children in the order that I want layoutChildren()
			//		to process them (i.e. from the outside to the inside)

			var wrappers = array.map(this.getChildren(), function(child, idx){
				return {
					pane: child,
					weight: [
						child.region == "center" ? Infinity : 0,
						child.layoutPriority,
						(this.design == "sidebar" ? 1 : -1) * (/top|bottom/.test(child.region) ? 1 : -1),
						idx
					]
				};
			}, this);
			wrappers.sort(function(a, b){
				var aw = a.weight, bw = b.weight;
				for(var i = 0; i < aw.length; i++){
					if(aw[i] != bw[i]){
						return aw[i] - bw[i];
					}
				}
				return 0;
			});

			return array.map(wrappers, function(w){ return w.pane; });
		},

		layout: function(){
			layoutUtils.layoutChildren(this.domNode, this._contentBox, this._getOrderedChildren());
		},

		addChild: function(/*dijit/_WidgetBase*/ child, /*Integer?*/ insertIndex){
			this.inherited(arguments);
			if(this._started){
				this.layout();
			}
		},

		removeChild: function(/*dijit/_WidgetBase*/ child){
			this.inherited(arguments);
			if(this._started){
				this.layout();
			}

			// Clean up whatever style changes we made to the child pane.
			// Unclear how height and width should be handled.
			domClass.remove(child.domNode, this.baseClass + "Pane");
			domStyle.set(child.domNode, {
				top: "auto",
				bottom: "auto",
				left: "auto",
				right: "auto",
				position: "static"
			});
			domStyle.set(child.domNode, /top|bottom/.test(child.region) ? "width" : "height", "auto");
		}
	});

	LayoutContainer.ChildWidgetProperties = {
		// summary:
		//		These properties can be specified for the children of a LayoutContainer.

		// region: [const] String
		//		Values: "top", "bottom", "leading", "trailing", "left", "right", "center".
		//		See the `dijit/layout/LayoutContainer` description for details.
		region: '',

		// layoutAlign: [const deprecated] String
		//		Synonym for region, except using "client" instead of "center".  Deprecated; use region instead.
		layoutAlign: '',

		// layoutPriority: [const] Number
		//		Children with a higher layoutPriority will be placed closer to the LayoutContainer center,
		//		between children with a lower layoutPriority.
		layoutPriority: 0
	};

	// Since any widget can be specified as a LayoutContainer child, mix it
	// into the base widget class.  (This is a hack, but it's effective.)
	// This is for the benefit of the parser.   Remove for 2.0.  Also, hide from doc viewer.
	lang.extend(_WidgetBase, /*===== {} || =====*/ LayoutContainer.ChildWidgetProperties);

	return LayoutContainer;
});

},
'dijit/layout/TabContainer':function(){
define([
	"dojo/_base/lang", // lang.getObject
	"dojo/_base/declare", // declare
	"./_TabContainerBase",
	"./TabController",
	"./ScrollingTabController"
], function(lang, declare, _TabContainerBase, TabController, ScrollingTabController){

	// module:
	//		dijit/layout/TabContainer


	return declare("dijit.layout.TabContainer", _TabContainerBase, {
		// summary:
		//		A Container with tabs to select each child (only one of which is displayed at a time).
		// description:
		//		A TabContainer is a container that has multiple panes, but shows only
		//		one pane at a time.  There are a set of tabs corresponding to each pane,
		//		where each tab has the name (aka title) of the pane, and optionally a close button.
		//
		//		See `StackContainer.ChildWidgetProperties` for details on the properties that can be set on
		//		children of a `TabContainer`.

		// useMenu: [const] Boolean
		//		True if a menu should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useMenu: true,

		// useSlider: [const] Boolean
		//		True if a slider should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useSlider: true,

		// controllerWidget: Class
		//		An optional parameter to override the widget used to display the tab labels
		controllerWidget: "",

		_makeController: function(/*DomNode*/ srcNode){
			// summary:
			//		Instantiate tablist controller widget and return reference to it.
			//		Callback from _TabContainerBase.postCreate().
			// tags:
			//		protected extension

			// "string" branch for back-compat, remove for 2.0
			var cls = this.baseClass + "-tabs" + (this.doLayout ? "" : " dijitTabNoLayout"),
				TabController = typeof this.controllerWidget == "string" ? lang.getObject(this.controllerWidget) :
						this.controllerWidget;

			return new TabController({
				id: this.id + "_tablist",
				ownerDocument: this.ownerDocument,
				dir: this.dir,
				lang: this.lang,
				textDir: this.textDir,
				tabPosition: this.tabPosition,
				doLayout: this.doLayout,
				containerId: this.id,
				"class": cls,
				nested: this.nested,
				useMenu: this.useMenu,
				useSlider: this.useSlider,
				tabStripClass: this.tabStrip ? this.baseClass + (this.tabStrip ? "":"No") + "Strip": null
			}, srcNode);
		},

		postMixInProperties: function(){
			this.inherited(arguments);

			// Scrolling controller only works for horizontal non-nested tabs
			if(!this.controllerWidget){
				this.controllerWidget = (this.tabPosition == "top" || this.tabPosition == "bottom") && !this.nested ?
							ScrollingTabController : TabController;
			}
		}
	});
});

},
'dijit/layout/_TabContainerBase':function(){
define([
	"dojo/text!./templates/TabContainer.html",
	"./StackContainer",
	"./utils", // marginBox2contextBox, layoutChildren
	"../_TemplatedMixin",
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add
	"dojo/dom-geometry", // domGeometry.contentBox
	"dojo/dom-style" // domStyle.style
], function(template, StackContainer, layoutUtils, _TemplatedMixin, declare, domClass, domGeometry, domStyle){

	// module:
	//		dijit/layout/_TabContainerBase

	return declare("dijit.layout._TabContainerBase", [StackContainer, _TemplatedMixin], {
		// summary:
		//		Abstract base class for TabContainer.   Must define _makeController() to instantiate
		//		and return the widget that displays the tab labels
		// description:
		//		A TabContainer is a container that has multiple panes, but shows only
		//		one pane at a time.  There are a set of tabs corresponding to each pane,
		//		where each tab has the name (aka title) of the pane, and optionally a close button.

		// tabPosition: String
		//		Defines where tabs go relative to tab content.
		//		"top", "bottom", "left-h", "right-h"
		tabPosition: "top",

		baseClass: "dijitTabContainer",

		// tabStrip: [const] Boolean
		//		Defines whether the tablist gets an extra class for layouting, putting a border/shading
		//		around the set of tabs.   Not supported by claro theme.
		tabStrip: false,

		// nested: [const] Boolean
		//		If true, use styling for a TabContainer nested inside another TabContainer.
		//		For tundra etc., makes tabs look like links, and hides the outer
		//		border since the outer TabContainer already has a border.
		nested: false,

		templateString: template,

		postMixInProperties: function(){
			// set class name according to tab position, ex: dijitTabContainerTop
			this.baseClass += this.tabPosition.charAt(0).toUpperCase() + this.tabPosition.substr(1).replace(/-.*/, "");

			this.srcNodeRef && domStyle.set(this.srcNodeRef, "visibility", "hidden");

			this.inherited(arguments);
		},

		buildRendering: function(){
			this.inherited(arguments);

			// Create the tab list that will have a tab (a.k.a. tab button) for each tab panel
			this.tablist = this._makeController(this.tablistNode);

			if(!this.doLayout){
				domClass.add(this.domNode, "dijitTabContainerNoLayout");
			}

			if(this.nested){
				/* workaround IE's lack of support for "a > b" selectors by
				 * tagging each node in the template.
				 */
				domClass.add(this.domNode, "dijitTabContainerNested");
				domClass.add(this.tablist.containerNode, "dijitTabContainerTabListNested");
				domClass.add(this.tablistSpacer, "dijitTabContainerSpacerNested");
				domClass.add(this.containerNode, "dijitTabPaneWrapperNested");
			}else{
				domClass.add(this.domNode, "tabStrip-" + (this.tabStrip ? "enabled" : "disabled"));
			}
		},

		_setupChild: function(/*dijit/_WidgetBase*/ tab){
			// Overrides StackContainer._setupChild().
			domClass.add(tab.domNode, "dijitTabPane");
			this.inherited(arguments);
		},

		startup: function(){
			if(this._started){
				return;
			}

			// wire up the tablist and its tabs
			this.tablist.startup();

			this.inherited(arguments);
		},

		layout: function(){
			// Overrides StackContainer.layout().
			// Configure the content pane to take up all the space except for where the tabs are

			if(!this._contentBox || typeof(this._contentBox.l) == "undefined"){
				return;
			}

			var sc = this.selectedChildWidget;

			if(this.doLayout){
				// position and size the titles and the container node
				var titleAlign = this.tabPosition.replace(/-h/, "");
				this.tablist.region = titleAlign;
				var children = [this.tablist, {
					domNode: this.tablistSpacer,
					region: titleAlign
				}, {
					domNode: this.containerNode,
					region: "center"
				}];
				layoutUtils.layoutChildren(this.domNode, this._contentBox, children);

				// Compute size to make each of my children.
				// children[2] is the margin-box size of this.containerNode, set by layoutChildren() call above
				this._containerContentBox = layoutUtils.marginBox2contentBox(this.containerNode, children[2]);

				if(sc && sc.resize){
					sc.resize(this._containerContentBox);
				}
			}else{
				// just layout the tab controller, so it can position left/right buttons etc.
				if(this.tablist.resize){
					//make the tabs zero width so that they don't interfere with width calc, then reset
					var s = this.tablist.domNode.style;
					s.width = "0";
					var width = domGeometry.getContentBox(this.domNode).w;
					s.width = "";
					this.tablist.resize({w: width});
				}

				// and call resize() on the selected pane just to tell it that it's been made visible
				if(sc && sc.resize){
					sc.resize();
				}
			}
		},

		destroy: function(preserveDom){
			if(this.tablist){
				this.tablist.destroy(preserveDom);
			}
			this.inherited(arguments);
		}
	});
});

},
'dijit/layout/StackContainer':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf array.some
	"dojo/cookie", // cookie
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.replace
	"dojo/dom-construct",
	"dojo/has", // has("dijit-legacy-requires")
	"dojo/_base/lang", // lang.extend
	"dojo/on",
	"dojo/ready",
	"dojo/topic", // publish
	"dojo/when",
	"../registry", // registry.byId
	"../_WidgetBase",
	"./_LayoutWidget",
	"dojo/i18n!../nls/common"
], function(array, cookie, declare, domClass, domConstruct, has, lang, on, ready, topic, when, registry, _WidgetBase, _LayoutWidget){

	// module:
	//		dijit/layout/StackContainer

	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/layout/StackController"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	var StackContainer = declare("dijit.layout.StackContainer", _LayoutWidget, {
		// summary:
		//		A container that has multiple children, but shows only
		//		one child at a time
		//
		// description:
		//		A container for widgets (ContentPanes, for example) That displays
		//		only one Widget at a time.
		//
		//		Publishes topics [widgetId]-addChild, [widgetId]-removeChild, and [widgetId]-selectChild
		//
		//		Can be base class for container, Wizard, Show, etc.
		//
		//		See `StackContainer.ChildWidgetProperties` for details on the properties that can be set on
		//		children of a `StackContainer`.

		// doLayout: Boolean
		//		If true, change the size of my currently displayed child to match my size
		doLayout: true,

		// persist: Boolean
		//		Remembers the selected child across sessions
		persist: false,

		baseClass: "dijitStackContainer",

		/*=====
		// selectedChildWidget: [readonly] dijit._Widget
		//		References the currently selected child widget, if any.
		//		Adjust selected child with selectChild() method.
		selectedChildWidget: null,
		=====*/

		buildRendering: function(){
			this.inherited(arguments);
			domClass.add(this.domNode, "dijitLayoutContainer");
		},

		postCreate: function(){
			this.inherited(arguments);
			this.own(
				on(this.domNode, "keydown", lang.hitch(this, "_onKeyDown"))
			);
		},

		startup: function(){
			if(this._started){
				return;
			}

			var children = this.getChildren();

			// Setup each page panel to be initially hidden
			array.forEach(children, this._setupChild, this);

			// Figure out which child to initially display, defaulting to first one
			if(this.persist){
				this.selectedChildWidget = registry.byId(cookie(this.id + "_selectedChild"));
			}else{
				array.some(children, function(child){
					if(child.selected){
						this.selectedChildWidget = child;
					}
					return child.selected;
				}, this);
			}
			var selected = this.selectedChildWidget;
			if(!selected && children[0]){
				selected = this.selectedChildWidget = children[0];
				selected.selected = true;
			}

			// Publish information about myself so any StackControllers can initialize.
			// This needs to happen before this.inherited(arguments) so that for
			// TabContainer, this._contentBox doesn't include the space for the tab labels.
			topic.publish(this.id + "-startup", {children: children, selected: selected, textDir: this.textDir});

			// Startup each child widget, and do initial layout like setting this._contentBox,
			// then calls this.resize() which does the initial sizing on the selected child.
			this.inherited(arguments);
		},

		resize: function(){
			// Overrides _LayoutWidget.resize()
			// Resize is called when we are first made visible (it's called from startup()
			// if we are initially visible). If this is the first time we've been made
			// visible then show our first child.
			if(!this._hasBeenShown){
				this._hasBeenShown = true;
				var selected = this.selectedChildWidget;
				if(selected){
					this._showChild(selected);
				}
			}
			this.inherited(arguments);
		},

		_setupChild: function(/*dijit/_WidgetBase*/ child){
			// Overrides _LayoutWidget._setupChild()

			// For aria support, wrap child widget in a <div role="tabpanel">
			var childNode = child.domNode,
				wrapper = domConstruct.place(
					"<div role='tabpanel' class='" + this.baseClass + "ChildWrapper dijitHidden'>",
					child.domNode,
					"replace"),
				label = child["aria-label"] || child.title || child.label;
			if(label){
				// setAttribute() escapes special chars, and if() statement avoids setting aria-label="undefined"
				wrapper.setAttribute("aria-label", label);
			}
			domConstruct.place(childNode, wrapper);
			child._wrapper = wrapper;	// to set the aria-labelledby in StackController

			this.inherited(arguments);

			// child may have style="display: none" (at least our test cases do), so remove that
			if(childNode.style.display == "none"){
				childNode.style.display = "block";
			}

			// remove the title attribute so it doesn't show up when i hover over a node
			child.domNode.title = "";
		},

		addChild: function(/*dijit/_WidgetBase*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to do layout and publish events

			this.inherited(arguments);

			if(this._started){
				topic.publish(this.id + "-addChild", child, insertIndex);	// publish

				// in case the tab titles have overflowed from one line to two lines
				// (or, if this if first child, from zero lines to one line)
				// TODO: w/ScrollingTabController this is no longer necessary, although
				// ScrollTabController.resize() does need to get called to show/hide
				// the navigation buttons as appropriate, but that's handled in ScrollingTabController.onAddChild().
				// If this is updated to not layout [except for initial child added / last child removed], update
				// "childless startup" test in StackContainer.html to check for no resize event after second addChild()
				this.layout();

				// if this is the first child, then select it
				if(!this.selectedChildWidget){
					this.selectChild(child);
				}
			}
		},

		removeChild: function(/*dijit/_WidgetBase*/ page){
			// Overrides _Container.removeChild() to do layout and publish events

			var idx = array.indexOf(this.getChildren(), page);

			this.inherited(arguments);

			// Remove the child widget wrapper we use to set aria roles.  This won't affect the page itself since it's
			// already been detached from page._wrapper via the this.inherited(arguments) call above.
			domConstruct.destroy(page._wrapper);
			delete page._wrapper;

			if(this._started){
				// This will notify any tablists to remove a button; do this first because it may affect sizing.
				topic.publish(this.id + "-removeChild", page);
			}

			// If all our children are being destroyed than don't run the code below (to select another page),
			// because we are deleting every page one by one
			if(this._descendantsBeingDestroyed){
				return;
			}

			// Select new page to display, also updating TabController to show the respective tab.
			// Do this before layout call because it can affect the height of the TabController.
			if(this.selectedChildWidget === page){
				this.selectedChildWidget = undefined;
				if(this._started){
					var children = this.getChildren();
					if(children.length){
						this.selectChild(children[Math.max(idx - 1, 0)]);
					}
				}
			}

			if(this._started){
				// In case the tab titles now take up one line instead of two lines
				// (note though that ScrollingTabController never overflows to multiple lines),
				// or the height has changed slightly because of addition/removal of tab which close icon
				this.layout();
			}
		},

		selectChild: function(/*dijit/_WidgetBase|String*/ page, /*Boolean*/ animate){
			// summary:
			//		Show the given widget (which must be one of my children)
			// page:
			//		Reference to child widget or id of child widget

			var d;

			page = registry.byId(page);

			if(this.selectedChildWidget != page){
				// Deselect old page and select new one
				d = this._transition(page, this.selectedChildWidget, animate);
				this._set("selectedChildWidget", page);
				topic.publish(this.id + "-selectChild", page);	// publish

				if(this.persist){
					cookie(this.id + "_selectedChild", this.selectedChildWidget.id);
				}
			}

			// d may be null, or a scalar like true.  Return a promise in all cases
			return when(d || true);		// Promise
		},

		_transition: function(newWidget, oldWidget /*===== ,  animate =====*/){
			// summary:
			//		Hide the old widget and display the new widget.
			//		Subclasses should override this.
			// newWidget: dijit/_WidgetBase
			//		The newly selected widget.
			// oldWidget: dijit/_WidgetBase
			//		The previously selected widget.
			// animate: Boolean
			//		Used by AccordionContainer to turn on/off slide effect.
			// tags:
			//		protected extension
			if(oldWidget){
				this._hideChild(oldWidget);
			}
			var d = this._showChild(newWidget);

			// Size the new widget, in case this is the first time it's being shown,
			// or I have been resized since the last time it was shown.
			// Note that page must be visible for resizing to work.
			if(newWidget.resize){
				if(this.doLayout){
					newWidget.resize(this._containerContentBox || this._contentBox);
				}else{
					// the child should pick it's own size but we still need to call resize()
					// (with no arguments) to let the widget lay itself out
					newWidget.resize();
				}
			}

			return d;	// If child has an href, promise that fires when the child's href finishes loading
		},

		_adjacent: function(/*Boolean*/ forward){
			// summary:
			//		Gets the next/previous child widget in this container from the current selection.

			// TODO: remove for 2.0 if this isn't being used.   Otherwise, fix to skip disabled tabs.

			var children = this.getChildren();
			var index = array.indexOf(children, this.selectedChildWidget);
			index += forward ? 1 : children.length - 1;
			return children[ index % children.length ]; // dijit/_WidgetBase
		},

		forward: function(){
			// summary:
			//		Advance to next page.
			return this.selectChild(this._adjacent(true), true);
		},

		back: function(){
			// summary:
			//		Go back to previous page.
			return this.selectChild(this._adjacent(false), true);
		},

		_onKeyDown: function(e){
			topic.publish(this.id + "-containerKeyDown", { e: e, page: this});	// publish
		},

		layout: function(){
			// Implement _LayoutWidget.layout() virtual method.
			var child = this.selectedChildWidget;
			if(child && child.resize){
				if(this.doLayout){
					child.resize(this._containerContentBox || this._contentBox);
				}else{
					child.resize();
				}
			}
		},

		_showChild: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Show the specified child by changing it's CSS, and call _onShow()/onShow() so
			//		it can do any updates it needs regarding loading href's etc.
			// returns:
			//		Promise that fires when page has finished showing, or true if there's no href
			var children = this.getChildren();
			page.isFirstChild = (page == children[0]);
			page.isLastChild = (page == children[children.length - 1]);
			page._set("selected", true);

			if(page._wrapper){	// false if not started yet
				domClass.replace(page._wrapper, "dijitVisible", "dijitHidden");
			}

			return (page._onShow && page._onShow()) || true;
		},

		_hideChild: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Hide the specified child by changing it's CSS, and call _onHide() so
			//		it's notified.
			page._set("selected", false);

			if(page._wrapper){	// false if not started yet
				domClass.replace(page._wrapper, "dijitHidden", "dijitVisible");
			}

			page.onHide && page.onHide();
		},

		closeChild: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Callback when user clicks the [X] to remove a page.
			//		If onClose() returns true then remove and destroy the child.
			// tags:
			//		private
			var remove = page.onClose && page.onClose(this, page);
			if(remove){
				this.removeChild(page);
				// makes sure we can clean up executeScripts in ContentPane onUnLoad
				page.destroyRecursive();
			}
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			this._descendantsBeingDestroyed = true;
			this.selectedChildWidget = undefined;
			array.forEach(this.getChildren(), function(child){
				if(!preserveDom){
					this.removeChild(child);
				}
				child.destroyRecursive(preserveDom);
			}, this);
			this._descendantsBeingDestroyed = false;
		}
	});

	StackContainer.ChildWidgetProperties = {
		// summary:
		//		These properties can be specified for the children of a StackContainer.

		// selected: Boolean
		//		Specifies that this widget should be the initially displayed pane.
		//		Note: to change the selected child use `dijit/layout/StackContainer.selectChild`
		selected: false,

		// disabled: Boolean
		//		Specifies that the button to select this pane should be disabled.
		//		Doesn't affect programmatic selection of the pane, nor does it deselect the pane if it is currently selected.
		disabled: false,

		// closable: Boolean
		//		True if user can close (destroy) this child, such as (for example) clicking the X on the tab.
		closable: false,

		// iconClass: String
		//		CSS Class specifying icon to use in label associated with this pane.
		iconClass: "dijitNoIcon",

		// showTitle: Boolean
		//		When true, display title of this widget as tab label etc., rather than just using
		//		icon specified in iconClass
		showTitle: true
	};

	// Since any widget can be specified as a StackContainer child, mix them
	// into the base widget class.  (This is a hack, but it's effective.)
	// This is for the benefit of the parser.   Remove for 2.0.  Also, hide from doc viewer.
	lang.extend(_WidgetBase, /*===== {} || =====*/ StackContainer.ChildWidgetProperties);

	return StackContainer;
});

},
'dijit/layout/TabController':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"dojo/dom-attr", // domAttr.attr
	"dojo/dom-class", // domClass.toggle
	"dojo/has",
	"dojo/i18n", // i18n.getLocalization
	"dojo/_base/lang", // lang.hitch lang.trim
	"./StackController",
	"../registry",
	"../Menu",
	"../MenuItem",
	"dojo/text!./templates/_TabButton.html",
	"dojo/i18n!../nls/common"
], function(declare, dom, domAttr, domClass, has, i18n, lang, StackController, registry, Menu, MenuItem, template){

	// module:
	//		dijit/layout/TabController

	var TabButton = declare("dijit.layout._TabButton" + (has("dojo-bidi") ? "_NoBidi" : ""), StackController.StackButton, {
		// summary:
		//		A tab (the thing you click to select a pane).
		// description:
		//		Contains the title of the pane, and optionally a close-button to destroy the pane.
		//		This is an internal widget and should not be instantiated directly.
		// tags:
		//		private

		// baseClass: String
		//		The CSS class applied to the domNode.
		baseClass: "dijitTab",

		// Apply dijitTabCloseButtonHover when close button is hovered
		cssStateNodes: {
			closeNode: "dijitTabCloseButton"
		},

		templateString: template,

		// Button superclass maps name to a this.valueNode, but we don't have a this.valueNode attach point
		_setNameAttr: "focusNode",

		// Override _FormWidget.scrollOnFocus.
		// Don't scroll the whole tab container into view when the button is focused.
		scrollOnFocus: false,

		buildRendering: function(){
			this.inherited(arguments);

			dom.setSelectable(this.containerNode, false);
		},

		startup: function(){
			this.inherited(arguments);
			var n = this.domNode;

			// Required to give IE6 a kick, as it initially hides the
			// tabs until they are focused on.
			this.defer(function(){
				n.className = n.className;
			}, 1);
		},

		_setCloseButtonAttr: function(/*Boolean*/ disp){
			// summary:
			//		Hide/show close button
			this._set("closeButton", disp);
			domClass.toggle(this.domNode, "dijitClosable", disp);
			this.closeNode.style.display = disp ? "" : "none";
			if(disp){
				var _nlsResources = i18n.getLocalization("dijit", "common");
				if(this.closeNode){
					domAttr.set(this.closeNode, "title", _nlsResources.itemClose);
				}
			}
		},

		_setDisabledAttr: function(/*Boolean*/ disabled){
			// summary:
			//		Make tab selected/unselectable

			this.inherited(arguments);

			// Don't show tooltip for close button when tab is disabled
			if(this.closeNode){
				if(disabled){
					domAttr.remove(this.closeNode, "title");
				}else{
					var _nlsResources = i18n.getLocalization("dijit", "common");
					domAttr.set(this.closeNode, "title", _nlsResources.itemClose);
				}
			}
		},

		_setLabelAttr: function(/*String*/ content){
			// summary:
			//		Hook for set('label', ...) to work.
			// description:
			//		takes an HTML string.
			//		Inherited ToggleButton implementation will Set the label (text) of the button;
			//		Need to set the alt attribute of icon on tab buttons if no label displayed
			this.inherited(arguments);
			if(!this.showLabel && !this.params.title){
				this.iconNode.alt = lang.trim(this.containerNode.innerText || this.containerNode.textContent || '');
			}
		}
	});

	if(has("dojo-bidi")){
		TabButton = declare("dijit.layout._TabButton", TabButton, {
			_setLabelAttr: function(/*String*/ content){
				this.inherited(arguments);
				this.applyTextDir(this.iconNode, this.iconNode.alt);
			}
		});
	}

	var TabController = declare("dijit.layout.TabController", StackController, {
		// summary:
		//		Set of tabs (the things with titles and a close button, that you click to show a tab panel).
		//		Used internally by `dijit/layout/TabContainer`.
		// description:
		//		Lets the user select the currently shown pane in a TabContainer or StackContainer.
		//		TabController also monitors the TabContainer, and whenever a pane is
		//		added or deleted updates itself accordingly.
		// tags:
		//		private

		baseClass: "dijitTabController",

		templateString: "<div role='tablist' data-dojo-attach-event='onkeydown:onkeydown'></div>",

		// tabPosition: String
		//		Defines where tabs go relative to the content.
		//		"top", "bottom", "left-h", "right-h"
		tabPosition: "top",

		// buttonWidget: Constructor
		//		The tab widget to create to correspond to each page
		buttonWidget: TabButton,

		// buttonWidgetCloseClass: String
		//		Class of [x] close icon, used by event delegation code to tell when close button was clicked
		buttonWidgetCloseClass: "dijitTabCloseButton",

		postCreate: function(){
			this.inherited(arguments);

			// Setup a close menu to be shared between all the closable tabs (excluding disabled tabs)
			var closeMenu = new Menu({
				id: this.id + "_Menu",
				ownerDocument: this.ownerDocument,
				dir: this.dir,
				lang: this.lang,
				textDir: this.textDir,
				targetNodeIds: [this.domNode],
				selector: function(node){
					return domClass.contains(node, "dijitClosable") && !domClass.contains(node, "dijitTabDisabled");
				}
			});
			this.own(closeMenu);

			var _nlsResources = i18n.getLocalization("dijit", "common"),
				controller = this;
			closeMenu.addChild(new MenuItem({
				label: _nlsResources.itemClose,
				ownerDocument: this.ownerDocument,
				dir: this.dir,
				lang: this.lang,
				textDir: this.textDir,
				onClick: function(evt){
					var button = registry.byNode(this.getParent().currentTarget);
					controller.onCloseButtonClick(button.page);
				}
			}));
		}
	});

	TabController.TabButton = TabButton;	// for monkey patching

	return TabController;
});

},
'dijit/layout/StackController':function(){
define([
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/dom-class",
	"dojo/dom-construct",
	"dojo/keys", // keys
	"dojo/_base/lang", // lang.getObject
	"dojo/on",
	"dojo/topic",
	"../focus", // focus.focus()
	"../registry", // registry.byId
	"../_Widget",
	"../_TemplatedMixin",
	"../_Container",
	"../form/ToggleButton",
	"dojo/touch",	// for normalized click handling, see dojoClick property setting in postCreate()
	"dojo/i18n!../nls/common"
], function(array, declare, domClass, domConstruct, keys, lang, on, topic, focus, registry, _Widget, _TemplatedMixin, _Container, ToggleButton){

	// module:
	//		dijit/layout/StackController

	var StackButton = declare("dijit.layout._StackButton", ToggleButton, {
		// summary:
		//		Internal widget used by StackContainer.
		// description:
		//		The button-like or tab-like object you click to select or delete a page
		// tags:
		//		private

		// Override _FormWidget.tabIndex.
		// StackContainer buttons are not in the tab order by default.
		// Probably we should be calling this.startupKeyNavChildren() instead.
		tabIndex: "-1",

		// closeButton: Boolean
		//		When true, display close button for this tab
		closeButton: false,

		_aria_attr: "aria-selected",

		buildRendering: function(/*Event*/ evt){
			this.inherited(arguments);
			(this.focusNode || this.domNode).setAttribute("role", "tab");
		}
	});


	var StackController = declare("dijit.layout.StackController", [_Widget, _TemplatedMixin, _Container], {
		// summary:
		//		Set of buttons to select a page in a `dijit/layout/StackContainer`
		// description:
		//		Monitors the specified StackContainer, and whenever a page is
		//		added, deleted, or selected, updates itself accordingly.

		baseClass: "dijitStackController",

		templateString: "<span role='tablist' data-dojo-attach-event='onkeydown'></span>",

		// containerId: [const] String
		//		The id of the page container that I point to
		containerId: "",

		// buttonWidget: [const] Constructor
		//		The button widget to create to correspond to each page
		buttonWidget: StackButton,

		// buttonWidgetCloseClass: String
		//		CSS class of [x] close icon, used by event delegation code to tell when close button was clicked
		buttonWidgetCloseClass: "dijitStackCloseButton",

		pane2button: function(/*String*/ id){
			// summary:
			//		Returns the button corresponding to the pane w/the given id.
			// tags:
			//		protected
			return registry.byId(this.id + "_" + id);
		},

		postCreate: function(){
			this.inherited(arguments);

			// Listen to notifications from StackContainer.  This is tricky because the StackContainer may not have
			// been created yet, so abstracting it through topics.
			// Note: for TabContainer we can do this through bubbled events instead of topics; maybe that's
			// all we support for 2.0?
			this.own(
				topic.subscribe(this.containerId + "-startup", lang.hitch(this, "onStartup")),
				topic.subscribe(this.containerId + "-addChild", lang.hitch(this, "onAddChild")),
				topic.subscribe(this.containerId + "-removeChild", lang.hitch(this, "onRemoveChild")),
				topic.subscribe(this.containerId + "-selectChild", lang.hitch(this, "onSelectChild")),
				topic.subscribe(this.containerId + "-containerKeyDown", lang.hitch(this, "onContainerKeyDown"))
			);

			// Listen for click events to select or close tabs.
			// No need to worry about ENTER/SPACE key handling: tabs are selected via left/right arrow keys,
			// and closed via shift-F10 (to show the close menu).
			// Also, add flag to use normalized click handling from dojo/touch
			this.containerNode.dojoClick = true;
			this.own(on(this.containerNode, 'click', lang.hitch(this, function(evt){
				var button = registry.getEnclosingWidget(evt.target);
				if(button != this.containerNode && !button.disabled && button.page){
					for(var target = evt.target; target !== this.containerNode; target = target.parentNode){
						if(domClass.contains(target, this.buttonWidgetCloseClass)){
							this.onCloseButtonClick(button.page);
							break;
						}else if(target == button.domNode){
							this.onButtonClick(button.page);
							break;
						}
					}
				}
			})));
		},

		onStartup: function(/*Object*/ info){
			// summary:
			//		Called after StackContainer has finished initializing
			// tags:
			//		private
			this.textDir = info.textDir;
			array.forEach(info.children, this.onAddChild, this);
			if(info.selected){
				// Show button corresponding to selected pane (unless selected
				// is null because there are no panes)
				this.onSelectChild(info.selected);
			}

			// Reflect events like page title changes to tab buttons
			var containerNode = registry.byId(this.containerId).containerNode,
				pane2button = lang.hitch(this, "pane2button"),
				paneToButtonAttr = {
					"title": "label",
					"showtitle": "showLabel",
					"iconclass": "iconClass",
					"closable": "closeButton",
					"tooltip": "title",
					"disabled": "disabled",
					"textdir": "textdir"
				},
				connectFunc = function(attr, buttonAttr){
					return on(containerNode, "attrmodified-" + attr, function(evt){
						var button = pane2button(evt.detail && evt.detail.widget && evt.detail.widget.id);
						if(button){
							button.set(buttonAttr, evt.detail.newValue);
						}
					});
				};
			for(var attr in paneToButtonAttr){
				this.own(connectFunc(attr, paneToButtonAttr[attr]));
			}
		},

		destroy: function(preserveDom){
			// Since the buttons are internal to the StackController widget, destroy() should remove them.
			// When #5796 is fixed for 2.0 can get rid of this function completely.
			this.destroyDescendants(preserveDom);
			this.inherited(arguments);
		},

		onAddChild: function(/*dijit/_WidgetBase*/ page, /*Integer?*/ insertIndex){
			// summary:
			//		Called whenever a page is added to the container.
			//		Create button corresponding to the page.
			// tags:
			//		private

			// create an instance of the button widget
			// (remove typeof buttonWidget == string support in 2.0)
			var Cls = lang.isString(this.buttonWidget) ? lang.getObject(this.buttonWidget) : this.buttonWidget;
			var button = new Cls({
				id: this.id + "_" + page.id,
				name: this.id + "_" + page.id, // note: must match id used in pane2button()
				label: page.title,
				disabled: page.disabled,
				ownerDocument: this.ownerDocument,
				dir: page.dir,
				lang: page.lang,
				textDir: page.textDir || this.textDir,
				showLabel: page.showTitle,
				iconClass: page.iconClass,
				closeButton: page.closable,
				title: page.tooltip,
				page: page
			});

			this.addChild(button, insertIndex);
			page.controlButton = button;	// this value might be overwritten if two tabs point to same container
			if(!this._currentChild){
				// If this is the first child then StackContainer will soon publish that it's selected,
				// but before that StackContainer calls layout(), and before layout() is called the
				// StackController needs to have the proper height... which means that the button needs
				// to be marked as selected now.   See test_TabContainer_CSS.html for test.
				this.onSelectChild(page);
			}

			// Add this StackController button to the list of things that labels that StackContainer pane.
			// Also, if there's an aria-labelledby parameter for the pane, then the aria-label parameter is unneeded.
			var labelledby = page._wrapper.getAttribute("aria-labelledby") ?
				page._wrapper.getAttribute("aria-labelledby") + " " + button.id : button.id;
			page._wrapper.removeAttribute("aria-label");
			page._wrapper.setAttribute("aria-labelledby", labelledby);
		},

		onRemoveChild: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Called whenever a page is removed from the container.
			//		Remove the button corresponding to the page.
			// tags:
			//		private

			if(this._currentChild === page){
				this._currentChild = null;
			}

			var button = this.pane2button(page.id);
			if(button){
				this.removeChild(button);
				button.destroy();
			}
			delete page.controlButton;
		},

		onSelectChild: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Called when a page has been selected in the StackContainer, either by me or by another StackController
			// tags:
			//		private

			if(!page){
				return;
			}

			if(this._currentChild){
				var oldButton = this.pane2button(this._currentChild.id);
				oldButton.set('checked', false);
				oldButton.focusNode.setAttribute("tabIndex", "-1");
			}

			var newButton = this.pane2button(page.id);
			newButton.set('checked', true);
			this._currentChild = page;
			newButton.focusNode.setAttribute("tabIndex", "0");
			var container = registry.byId(this.containerId);
		},

		onButtonClick: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Called whenever one of my child buttons is pressed in an attempt to select a page
			// tags:
			//		private

			var button = this.pane2button(page.id);

			// For TabContainer where the tabs are <span>, need to set focus explicitly when left/right arrow
			focus.focus(button.focusNode);

			if(this._currentChild && this._currentChild.id === page.id){
				//In case the user clicked the checked button, keep it in the checked state because it remains to be the selected stack page.
				button.set('checked', true);
			}
			var container = registry.byId(this.containerId);
			container.selectChild(page);
		},

		onCloseButtonClick: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Called whenever one of my child buttons [X] is pressed in an attempt to close a page
			// tags:
			//		private

			var container = registry.byId(this.containerId);
			container.closeChild(page);
			if(this._currentChild){
				var b = this.pane2button(this._currentChild.id);
				if(b){
					focus.focus(b.focusNode || b.domNode);
				}
			}
		},

		// TODO: this is a bit redundant with forward, back api in StackContainer
		adjacent: function(/*Boolean*/ forward){
			// summary:
			//		Helper for onkeydown to find next/previous button
			// tags:
			//		private

			if(!this.isLeftToRight() && (!this.tabPosition || /top|bottom/.test(this.tabPosition))){
				forward = !forward;
			}
			// find currently focused button in children array
			var children = this.getChildren();
			var idx = array.indexOf(children, this.pane2button(this._currentChild.id)),
				current = children[idx];

			// Pick next/previous non-disabled button to focus on.   If we get back to the original button it means
			// that all buttons must be disabled, so return current child to avoid an infinite loop.
			var child;
			do{
				idx = (idx + (forward ? 1 : children.length - 1)) % children.length;
				child = children[idx];
			}while(child.disabled && child != current);

			return child; // dijit/_WidgetBase
		},

		onkeydown: function(/*Event*/ e, /*Boolean?*/ fromContainer){
			// summary:
			//		Handle keystrokes on the page list, for advancing to next/previous button
			//		and closing the current page if the page is closable.
			// tags:
			//		private

			if(this.disabled || e.altKey){
				return;
			}
			var forward = null;
			if(e.ctrlKey || !e._djpage){
				switch(e.keyCode){
					case keys.LEFT_ARROW:
					case keys.UP_ARROW:
						if(!e._djpage){
							forward = false;
						}
						break;
					case keys.PAGE_UP:
						if(e.ctrlKey){
							forward = false;
						}
						break;
					case keys.RIGHT_ARROW:
					case keys.DOWN_ARROW:
						if(!e._djpage){
							forward = true;
						}
						break;
					case keys.PAGE_DOWN:
						if(e.ctrlKey){
							forward = true;
						}
						break;
					case keys.HOME:
						// Navigate to first non-disabled child
						var children = this.getChildren();
						for(var idx = 0; idx < children.length; idx++){
							var child = children[idx];
							if(!child.disabled){
								this.onButtonClick(child.page);
								break;
							}
						}
						e.stopPropagation();
						e.preventDefault();
						break;
					case keys.END:
						// Navigate to last non-disabled child
						var children = this.getChildren();
						for(var idx = children.length - 1; idx >= 0; idx--){
							var child = children[idx];
							if(!child.disabled){
								this.onButtonClick(child.page);
								break;
							}
						}
						e.stopPropagation();
						e.preventDefault();
						break;
					case keys.DELETE:
					case "W".charCodeAt(0):    // ctrl-W
						if(this._currentChild.closable &&
							(e.keyCode == keys.DELETE || e.ctrlKey)){
							this.onCloseButtonClick(this._currentChild);
						}
						// avoid browser tab closing
						e.stopPropagation();
						e.preventDefault();
						break;
					case keys.TAB:
						if(e.ctrlKey){
							this.onButtonClick(this.adjacent(!e.shiftKey).page);
							e.stopPropagation();
							e.preventDefault();
						}
						break;
				}
				// handle next/previous page navigation (left/right arrow, etc.)
				if(forward !== null){
					this.onButtonClick(this.adjacent(forward).page);
					e.stopPropagation();
					e.preventDefault();
				}
			}
		},

		onContainerKeyDown: function(/*Object*/ info){
			// summary:
			//		Called when there was a keydown on the container
			// tags:
			//		private
			info.e._djpage = info.page;
			this.onkeydown(info.e);
		}
	});

	StackController.StackButton = StackButton;	// for monkey patching

	return StackController;
});

},
'dijit/layout/ScrollingTabController':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-class", // domClass.add domClass.contains
	"dojo/dom-geometry", // domGeometry.contentBox
	"dojo/dom-style", // domStyle.style
	"dojo/_base/fx", // Animation
	"dojo/_base/lang", // lang.hitch
	"dojo/on",
	"dojo/query", // query
	"dojo/sniff", // has("ie"), has("webkit"), has("quirks")
	"../registry", // registry.byId()
	"dojo/text!./templates/ScrollingTabController.html",
	"dojo/text!./templates/_ScrollingTabControllerButton.html",
	"./TabController",
	"./utils", // marginBox2contextBox, layoutChildren
	"../_WidgetsInTemplateMixin",
	"../Menu",
	"../MenuItem",
	"../form/Button",
	"../_HasDropDown",
	"dojo/NodeList-dom", // NodeList.style
	"../a11yclick"	// template uses ondijitclick (not for keyboard support, but for responsive touch support)
], function(array, declare, domClass, domGeometry, domStyle, fx, lang, on, query, has,
	registry, tabControllerTemplate, buttonTemplate, TabController, layoutUtils, _WidgetsInTemplateMixin,
	Menu, MenuItem, Button, _HasDropDown){

	// module:
	//		dijit/layout/ScrollingTabController

	var ScrollingTabController = declare("dijit.layout.ScrollingTabController", [TabController, _WidgetsInTemplateMixin], {
		// summary:
		//		Set of tabs with left/right arrow keys and a menu to switch between tabs not
		//		all fitting on a single row.
		//		Works only for horizontal tabs (either above or below the content, not to the left
		//		or right).
		// tags:
		//		private

		baseClass: "dijitTabController dijitScrollingTabController",

		templateString: tabControllerTemplate,

		// useMenu: [const] Boolean
		//		True if a menu should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useMenu: true,

		// useSlider: [const] Boolean
		//		True if a slider should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useSlider: true,

		// tabStripClass: [const] String
		//		The css class to apply to the tab strip, if it is visible.
		tabStripClass: "",

		// _minScroll: Number
		//		The distance in pixels from the edge of the tab strip which,
		//		if a scroll animation is less than, forces the scroll to
		//		go all the way to the left/right.
		_minScroll: 5,

		// Override default behavior mapping class to DOMNode
		_setClassAttr: { node: "containerNode", type: "class" },

		buildRendering: function(){
			this.inherited(arguments);
			var n = this.domNode;

			this.scrollNode = this.tablistWrapper;
			this._initButtons();

			if(!this.tabStripClass){
				this.tabStripClass = "dijitTabContainer" +
					this.tabPosition.charAt(0).toUpperCase() +
					this.tabPosition.substr(1).replace(/-.*/, "") +
					"None";
				domClass.add(n, "tabStrip-disabled")
			}

			domClass.add(this.tablistWrapper, this.tabStripClass);
		},

		onStartup: function(){
			this.inherited(arguments);

			// TabController is hidden until it finishes drawing, to give
			// a less visually jumpy instantiation.   When it's finished, set visibility to ""
			// to that the tabs are hidden/shown depending on the container's visibility setting.
			domStyle.set(this.domNode, "visibility", "");
			this._postStartup = true;

			// changes to the tab button label or iconClass will have changed the width of the
			// buttons, so do a resize
			this.own(on(this.containerNode, "attrmodified-label, attrmodified-iconclass", lang.hitch(this, function(evt){
				if(this._dim){
					this.resize(this._dim);
				}
			})));
		},

		onAddChild: function(page, insertIndex){
			this.inherited(arguments);

			// Increment the width of the wrapper when a tab is added
			// This makes sure that the buttons never wrap.
			// The value 200 is chosen as it should be bigger than most
			// Tab button widths.
			domStyle.set(this.containerNode, "width",
				(domStyle.get(this.containerNode, "width") + 200) + "px");
		},

		onRemoveChild: function(page, insertIndex){
			// null out _selectedTab because we are about to delete that dom node
			var button = this.pane2button(page.id);
			if(this._selectedTab === button.domNode){
				this._selectedTab = null;
			}

			this.inherited(arguments);
		},

		_initButtons: function(){
			// summary:
			//		Creates the buttons used to scroll to view tabs that
			//		may not be visible if the TabContainer is too narrow.

			// Make a list of the buttons to display when the tab labels become
			// wider than the TabContainer, and hide the other buttons.
			// Also gets the total width of the displayed buttons.
			this._btnWidth = 0;
			this._buttons = query("> .tabStripButton", this.domNode).filter(function(btn){
				if((this.useMenu && btn == this._menuBtn.domNode) ||
					(this.useSlider && (btn == this._rightBtn.domNode || btn == this._leftBtn.domNode))){
					this._btnWidth += domGeometry.getMarginSize(btn).w;
					return true;
				}else{
					domStyle.set(btn, "display", "none");
					return false;
				}
			}, this);
		},

		_getTabsWidth: function(){
			var children = this.getChildren();
			if(children.length){
				var leftTab = children[this.isLeftToRight() ? 0 : children.length - 1].domNode,
					rightTab = children[this.isLeftToRight() ? children.length - 1 : 0].domNode;
				return rightTab.offsetLeft + rightTab.offsetWidth - leftTab.offsetLeft;
			}else{
				return 0;
			}
		},

		_enableBtn: function(width){
			// summary:
			//		Determines if the tabs are wider than the width of the TabContainer, and
			//		thus that we need to display left/right/menu navigation buttons.
			var tabsWidth = this._getTabsWidth();
			width = width || domStyle.get(this.scrollNode, "width");
			return tabsWidth > 0 && width < tabsWidth;
		},

		resize: function(dim){
			// summary:
			//		Hides or displays the buttons used to scroll the tab list and launch the menu
			//		that selects tabs.

			// Save the dimensions to be used when a child is renamed.
			this._dim = dim;

			// Set my height to be my natural height (tall enough for one row of tab labels),
			// and my content-box width based on margin-box width specified in dim parameter.
			// But first reset scrollNode.height in case it was set by layoutChildren() call
			// in a previous run of this method.
			this.scrollNode.style.height = "auto";
			var cb = this._contentBox = layoutUtils.marginBox2contentBox(this.domNode, {h: 0, w: dim.w});
			cb.h = this.scrollNode.offsetHeight;
			domGeometry.setContentSize(this.domNode, cb);

			// Show/hide the left/right/menu navigation buttons depending on whether or not they
			// are needed.
			var enable = this._enableBtn(this._contentBox.w);
			this._buttons.style("display", enable ? "" : "none");

			// Position and size the navigation buttons and the tablist
			this._leftBtn.region = "left";
			this._rightBtn.region = "right";
			this._menuBtn.region = this.isLeftToRight() ? "right" : "left";
			layoutUtils.layoutChildren(this.domNode, this._contentBox,
				[this._menuBtn, this._leftBtn, this._rightBtn, {domNode: this.scrollNode, region: "center"}]);

			// set proper scroll so that selected tab is visible
			if(this._selectedTab){
				if(this._anim && this._anim.status() == "playing"){
					this._anim.stop();
				}
				this.scrollNode.scrollLeft = this._convertToScrollLeft(this._getScrollForSelectedTab());
			}

			// Enable/disabled left right buttons depending on whether or not user can scroll to left or right
			this._setButtonClass(this._getScroll());

			this._postResize = true;

			// Return my size so layoutChildren() can use it.
			// Also avoids IE9 layout glitch on browser resize when scroll buttons present
			return {h: this._contentBox.h, w: dim.w};
		},

		_getScroll: function(){
			// summary:
			//		Returns the current scroll of the tabs where 0 means
			//		"scrolled all the way to the left" and some positive number, based on #
			//		of pixels of possible scroll (ex: 1000) means "scrolled all the way to the right"
			return (this.isLeftToRight() || has("ie") < 8 || (has("ie") && has("quirks")) || has("webkit")) ? this.scrollNode.scrollLeft :
				domStyle.get(this.containerNode, "width") - domStyle.get(this.scrollNode, "width")
					+ (has("ie") >= 8 ? -1 : 1) * this.scrollNode.scrollLeft;
		},

		_convertToScrollLeft: function(val){
			// summary:
			//		Given a scroll value where 0 means "scrolled all the way to the left"
			//		and some positive number, based on # of pixels of possible scroll (ex: 1000)
			//		means "scrolled all the way to the right", return value to set this.scrollNode.scrollLeft
			//		to achieve that scroll.
			//
			//		This method is to adjust for RTL funniness in various browsers and versions.
			if(this.isLeftToRight() || has("ie") < 8 || (has("ie") && has("quirks")) || has("webkit")){
				return val;
			}else{
				var maxScroll = domStyle.get(this.containerNode, "width") - domStyle.get(this.scrollNode, "width");
				return (has("ie") >= 8 ? -1 : 1) * (val - maxScroll);
			}
		},

		onSelectChild: function(/*dijit/_WidgetBase*/ page){
			// summary:
			//		Smoothly scrolls to a tab when it is selected.

			var tab = this.pane2button(page.id);
			if(!tab){
				return;
			}

			var node = tab.domNode;

			// Save the selection
			if(node != this._selectedTab){
				this._selectedTab = node;

				// Scroll to the selected tab, except on startup, when scrolling is handled in resize()
				if(this._postResize){
					var sl = this._getScroll();

					if(sl > node.offsetLeft ||
						sl + domStyle.get(this.scrollNode, "width") <
							node.offsetLeft + domStyle.get(node, "width")){
						this.createSmoothScroll().play();
					}
				}
			}

			this.inherited(arguments);
		},

		_getScrollBounds: function(){
			// summary:
			//		Returns the minimum and maximum scroll setting to show the leftmost and rightmost
			//		tabs (respectively)
			var children = this.getChildren(),
				scrollNodeWidth = domStyle.get(this.scrollNode, "width"), // about 500px
				containerWidth = domStyle.get(this.containerNode, "width"), // 50,000px
				maxPossibleScroll = containerWidth - scrollNodeWidth, // scrolling until right edge of containerNode visible
				tabsWidth = this._getTabsWidth();

			if(children.length && tabsWidth > scrollNodeWidth){
				// Scrolling should happen
				return {
					min: this.isLeftToRight() ? 0 : children[children.length - 1].domNode.offsetLeft,
					max: this.isLeftToRight() ?
						(children[children.length - 1].domNode.offsetLeft + children[children.length - 1].domNode.offsetWidth) - scrollNodeWidth :
						maxPossibleScroll
				};
			}else{
				// No scrolling needed, all tabs visible, we stay either scrolled to far left or far right (depending on dir)
				var onlyScrollPosition = this.isLeftToRight() ? 0 : maxPossibleScroll;
				return {
					min: onlyScrollPosition,
					max: onlyScrollPosition
				};
			}
		},

		_getScrollForSelectedTab: function(){
			// summary:
			//		Returns the scroll value setting so that the selected tab
			//		will appear in the center
			var w = this.scrollNode,
				n = this._selectedTab,
				scrollNodeWidth = domStyle.get(this.scrollNode, "width"),
				scrollBounds = this._getScrollBounds();

			// TODO: scroll minimal amount (to either right or left) so that
			// selected tab is fully visible, and just return if it's already visible?
			var pos = (n.offsetLeft + domStyle.get(n, "width") / 2) - scrollNodeWidth / 2;
			pos = Math.min(Math.max(pos, scrollBounds.min), scrollBounds.max);

			// TODO:
			// If scrolling close to the left side or right side, scroll
			// all the way to the left or right.  See this._minScroll.
			// (But need to make sure that doesn't scroll the tab out of view...)
			return pos;
		},

		createSmoothScroll: function(x){
			// summary:
			//		Creates a dojo._Animation object that smoothly scrolls the tab list
			//		either to a fixed horizontal pixel value, or to the selected tab.
			// description:
			//		If an number argument is passed to the function, that horizontal
			//		pixel position is scrolled to.  Otherwise the currently selected
			//		tab is scrolled to.
			// x: Integer?
			//		An optional pixel value to scroll to, indicating distance from left.

			// Calculate position to scroll to
			if(arguments.length > 0){
				// position specified by caller, just make sure it's within bounds
				var scrollBounds = this._getScrollBounds();
				x = Math.min(Math.max(x, scrollBounds.min), scrollBounds.max);
			}else{
				// scroll to center the current tab
				x = this._getScrollForSelectedTab();
			}

			if(this._anim && this._anim.status() == "playing"){
				this._anim.stop();
			}

			var self = this,
				w = this.scrollNode,
				anim = new fx.Animation({
					beforeBegin: function(){
						if(this.curve){
							delete this.curve;
						}
						var oldS = w.scrollLeft,
							newS = self._convertToScrollLeft(x);
						anim.curve = new fx._Line(oldS, newS);
					},
					onAnimate: function(val){
						w.scrollLeft = val;
					}
				});
			this._anim = anim;

			// Disable/enable left/right buttons according to new scroll position
			this._setButtonClass(x);

			return anim; // dojo/_base/fx/Animation
		},

		_getBtnNode: function(/*Event*/ e){
			// summary:
			//		Gets a button DOM node from a mouse click event.
			// e:
			//		The mouse click event.
			var n = e.target;
			while(n && !domClass.contains(n, "tabStripButton")){
				n = n.parentNode;
			}
			return n;
		},

		doSlideRight: function(/*Event*/ e){
			// summary:
			//		Scrolls the menu to the right.
			// e:
			//		The mouse click event.
			this.doSlide(1, this._getBtnNode(e));
		},

		doSlideLeft: function(/*Event*/ e){
			// summary:
			//		Scrolls the menu to the left.
			// e:
			//		The mouse click event.
			this.doSlide(-1, this._getBtnNode(e));
		},

		doSlide: function(/*Number*/ direction, /*DomNode*/ node){
			// summary:
			//		Scrolls the tab list to the left or right by 75% of the widget width.
			// direction:
			//		If the direction is 1, the widget scrolls to the right, if it is -1,
			//		it scrolls to the left.

			if(node && domClass.contains(node, "dijitTabDisabled")){
				return;
			}

			var sWidth = domStyle.get(this.scrollNode, "width");
			var d = (sWidth * 0.75) * direction;

			var to = this._getScroll() + d;

			this._setButtonClass(to);

			this.createSmoothScroll(to).play();
		},

		_setButtonClass: function(/*Number*/ scroll){
			// summary:
			//		Disables the left scroll button if the tabs are scrolled all the way to the left,
			//		or the right scroll button in the opposite case.
			// scroll: Integer
			//		amount of horizontal scroll

			var scrollBounds = this._getScrollBounds();
			this._leftBtn.set("disabled", scroll <= scrollBounds.min);
			this._rightBtn.set("disabled", scroll >= scrollBounds.max);
		}
	});


	var ScrollingTabControllerButtonMixin = declare("dijit.layout._ScrollingTabControllerButtonMixin", null, {
		baseClass: "dijitTab tabStripButton",

		templateString: buttonTemplate,

		// Override inherited tabIndex: 0 from dijit/form/Button, because user shouldn't be
		// able to tab to the left/right/menu buttons
		tabIndex: "",

		// Similarly, override FormWidget.isFocusable() because clicking a button shouldn't focus it
		// either (this override avoids focus() call in FormWidget.js)
		isFocusable: function(){
			return false;
		}
	});

	// Class used in template
	declare("dijit.layout._ScrollingTabControllerButton", [Button, ScrollingTabControllerButtonMixin]);

	// Class used in template
	declare("dijit.layout._ScrollingTabControllerMenuButton", [Button, _HasDropDown, ScrollingTabControllerButtonMixin], {
		// id of the TabContainer itself
		containerId: "",

		// -1 so user can't tab into the button, but so that button can still be focused programatically.
		// Because need to move focus to the button (or somewhere) before the menu is hidden or IE6 will crash.
		tabIndex: "-1",

		isLoaded: function(){
			// recreate menu every time, in case the TabContainer's list of children (or their icons/labels) have changed
			return false;
		},

		loadDropDown: function(callback){
			this.dropDown = new Menu({
				id: this.containerId + "_menu",
				ownerDocument: this.ownerDocument,
				dir: this.dir,
				lang: this.lang,
				textDir: this.textDir
			});
			var container = registry.byId(this.containerId);
			array.forEach(container.getChildren(), function(page){
				var menuItem = new MenuItem({
					id: page.id + "_stcMi",
					label: page.title,
					iconClass: page.iconClass,
					disabled: page.disabled,
					ownerDocument: this.ownerDocument,
					dir: page.dir,
					lang: page.lang,
					textDir: page.textDir || container.textDir,
					onClick: function(){
						container.selectChild(page);
					}
				});
				this.dropDown.addChild(menuItem);
			}, this);
			callback();
		},

		closeDropDown: function(/*Boolean*/ focus){
			this.inherited(arguments);
			if(this.dropDown){
				this._popupStateNode.removeAttribute("aria-owns");	// remove ref to node that we are about to delete
				this.dropDown.destroyRecursive();
				delete this.dropDown;
			}
		}
	});

	return ScrollingTabController;
});

},
'dijit/Toolbar':function(){
define([
	"require",
	"dojo/_base/declare", // declare
	"dojo/has",
	"dojo/keys", // keys.LEFT_ARROW keys.RIGHT_ARROW
	"dojo/ready",
	"./_Widget",
	"./_KeyNavContainer",
	"./_TemplatedMixin"
], function(require, declare, has, keys, ready, _Widget, _KeyNavContainer, _TemplatedMixin){

	// module:
	//		dijit/Toolbar


	// Back compat w/1.6, remove for 2.0
	if(has("dijit-legacy-requires")){
		ready(0, function(){
			var requires = ["dijit/ToolbarSeparator"];
			require(requires);	// use indirection so modules not rolled into a build
		});
	}

	return declare("dijit.Toolbar", [_Widget, _TemplatedMixin, _KeyNavContainer], {
		// summary:
		//		A Toolbar widget, used to hold things like `dijit/Editor` buttons

		templateString:
			'<div class="dijit" role="toolbar" tabIndex="${tabIndex}" data-dojo-attach-point="containerNode">' +
			'</div>',

		baseClass: "dijitToolbar",

		_onLeftArrow: function(){
			this.focusPrev();
		},

		_onRightArrow: function(){
			this.focusNext();
		}
	});
});

},
'dijit/ToolbarSeparator':function(){
define([
	"dojo/_base/declare", // declare
	"dojo/dom", // dom.setSelectable
	"./_Widget",
	"./_TemplatedMixin"
], function(declare, dom, _Widget, _TemplatedMixin){

	// module:
	//		dijit/ToolbarSeparator


	return declare("dijit.ToolbarSeparator", [_Widget, _TemplatedMixin], {
		// summary:
		//		A spacer between two `dijit.Toolbar` items

		templateString: '<div class="dijitToolbarSeparator dijitInline" role="presentation"></div>',

		buildRendering: function(){
			this.inherited(arguments);
			dom.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		This widget isn't focusable, so pass along that fact.
			// tags:
			//		protected
			return false;
		}
	});
});

},
'dijit/form/RadioButton':function(){
define([
	"dojo/_base/declare", // declare
	"./CheckBox",
	"./_RadioButtonMixin"
], function(declare, CheckBox, _RadioButtonMixin){

	// module:
	//		dijit/form/RadioButton

	return declare("dijit.form.RadioButton", [CheckBox, _RadioButtonMixin], {
		// summary:
		//		Same as an HTML radio, but with fancy styling.

		baseClass: "dijitRadio"
	});
});

},
'dijit/form/_RadioButtonMixin':function(){
define([
	"dojo/_base/array", // array.forEach
	"dojo/_base/declare", // declare
	"dojo/dom-attr", // domAttr.set
	"dojo/_base/lang", // lang.hitch
	"dojo/query", // query
	"../registry"    // registry.getEnclosingWidget
], function(array, declare, domAttr, lang, query, registry){

	// module:
	//		dijit/form/_RadioButtonMixin

	return declare("dijit.form._RadioButtonMixin", null, {
		// summary:
		//		Mixin to provide widget functionality for an HTML radio button

		// type: [private] String
		//		type attribute on `<input>` node.
		//		Users should not change this value.
		type: "radio",

		_getRelatedWidgets: function(){
			// Private function needed to help iterate over all radio buttons in a group.
			var ary = [];
			query("input[type=radio]", this.focusNode.form || this.ownerDocument).forEach(// can't use name= since query doesn't support [] in the name
				lang.hitch(this, function(inputNode){
					if(inputNode.name == this.name && inputNode.form == this.focusNode.form){
						var widget = registry.getEnclosingWidget(inputNode);
						if(widget){
							ary.push(widget);
						}
					}
				})
			);
			return ary;
		},

		_setCheckedAttr: function(/*Boolean*/ value){
			// If I am being checked then have to deselect currently checked radio button
			this.inherited(arguments);
			if(!this._created){
				return;
			}
			if(value){
				array.forEach(this._getRelatedWidgets(), lang.hitch(this, function(widget){
					if(widget != this && widget.checked){
						widget.set('checked', false);
					}
				}));
			}
		},

		_getSubmitValue: function(/*String*/ value){
			return value == null ? "on" : value;
		},

		_onClick: function(/*Event*/ e){
			if(this.checked || this.disabled){ // nothing to do
				e.stopPropagation();
				e.preventDefault();
				return false;
			}
			if(this.readOnly){ // ignored by some browsers so we have to resync the DOM elements with widget values
				e.stopPropagation();
				e.preventDefault();
				array.forEach(this._getRelatedWidgets(), lang.hitch(this, function(widget){
					domAttr.set(this.focusNode || this.domNode, 'checked', widget.checked);
				}));
				return false;
			}
			return this.inherited(arguments);
		}
	});
});

},
'vsd/common/vsmCommonConst':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
vsmCommonConst = {
    CANCEL_SECONDARY_EDITOR: 'CANCEL_SECONDARY_EDITOR',
    CLASSIFICATION_BINARY: 'BINARY',
    CLASSIFICATION_CATEGORICAL: 'CATEGORICAL',
    CLASSIFICATION_CONTINUOUS: 'CONTINUOUS',
    CONDITION_BUILDER_CHANGE: 'CONDITION_BUILDER_CHANGE',
    CONDITION_BUILDER_DELETE: 'CONDITION_BUILDER_DELETE',
    CONDITION_BUILDER_ERROR: 'CONDITION_BUILDER_ERROR',
    CONDITION_BUILDER_MODE_FULL_EDIT: 'CONDITION_BUILDER_MODE_FULL_EDIT',
    CONDITION_BUILDER_MODE_RHS_EDIT: 'CONDITION_BUILDER_MODE_RHS_EDIT',
    CONDITION_BUILDER_OPER_CHANGE: 'CONDITION_BUILDER_OPER_CHANGE',
    DATA_TYPE_CHAR: 'CHARACTER',
    DATA_TYPE_NUMERIC: "NUMERIC",
    DECISION_TREE_MAX_NOMINAL_LEVELS: 64,
    DECISION_TREE_MAX_TARGET_LEVELS: 5,
    DIAGRAM_DEPLOYMENT_COLOR: '#ffe7a6',
    DIAGRAM_SCENARIO_COLOR: '#cfde9e',
    DIAGRAM_TABLE_COLOR: '#f8c68c',
    DIAGRAM_WINDOW_COLOR: '#a6b5da',
    DOJO_NUMBER_FORMAT_PATTERN: "#,##0.00",
    EDIT_CONTROL_TYPE_BOOLEAN: 'BOOLEAN',
    EXPR_VAL_REASON_DBL_QUOTES: 'EXPR_VAL_REASON_DBL_QUOTES',
    EXPR_VAL_REASON_INVALID_BOOLEAN_CONST: 'EXPR_VAL_REASON_INVALID_BOOLEAN_CONST',
    EXPR_VAL_REASON_INVALID_DATE_CONST: 'EXPR_VAL_REASON_INVALID_DATE_CONST',
    EXPR_VAL_REASON_INVALID_DATETIME_CONST: 'EXPR_VAL_REASON_INVALID_DATETIME_CONST',
    EXPR_VAL_REASON_NON_NUMERIC: 'EXPR_VAL_REASON_NON_NUMERIC',
    EXPR_VAL_REASON_NULL: 'EXPR_VAL_REASON_NULL',
    EXPR_VAL_REASON_DBL_QUOTES: 'EXPR_VAL_REASON_DBL_QUOTES',
    EXPR_VAL_REASON_LITERAL_MAXLEN: 'EXPR_VAL_REASON_LITERAL_MAXLEN',
    EXPR_VAL_REASON_TOTAL_MAXLEN: 'EXPR_VAL_REASON_TOTAL_MAXLEN',
    GRAPH_OTHER_CATEGORY: '__SASVSD_OTHER',
    GRID_SK_VAR_NAME: '.__vsd_gui_row_id__',
    IMPORT_DIALOG_COMPLETE: 'IMPORT_DIALOG_COMPLETE',
    IMPORT_RETURN_STATUS_200: '200',
    IMPORT_RETURN_STATUS_100: '100',
    IMPORT_RETURN_STATUS_101: '101',
    IMPORT_RETURN_STATUS_102: '102',
    IMPORT_ENTITY_TYPE_DEPLOYMENT: 'DEPLOYMENT',
    IMPORT_ENTITY_TYPE_WINDOW: 'WINDOW',
    INVALID_NAME_REASON_DUPLICATE: 'INVALID_NAME_REASON_DUPLICATE',
    INVALID_NAME_REASON_SYNTAX: 'INVALID_NAME_REASON_SYNTAX',
    LOWER_BOUND_OPER_NAME: '>=',
    OBJ_STATUS_INVALID: "invalid",
    OBJ_STATUS_VALID: "valid",
    OBJ_STATUS_WARNING: "warning",
    OBJ_TYPE_DEPLOYMENT: 'DEPLOYMENT',
    OBJ_TYPE_SCENARIO: 'SCENARIO',
    OBJ_TYPE_TABLE: 'TABLE',
    OBJ_TYPE_WINDOW: 'WINDOW',
    pageSize: 200,
    PCHART_MAX_CHAR_LEVELS: 20,
    RULE_OPERAND2_CONSTANT_MAXLEN: 1000,
    RULE_OPERAND2_CONSTANT_MAXLEN_ERROR: 'RULE_OPERAND2_CONSTANT_MAXLEN_ERROR',
    SAS_MAX_CHAR_LENGTH: 32767,
    LASR_MAX_LITERAL_LENGTH: 256,
    SCENARIO_MODE_CHANGE: 'SCENARIO_MODE_CHANGE',
    SCENARIO_SAVE_COMPLETE: 'SCENARIO_SAVE_COMPLETE',
    UPPER_BOUND_OPER_NAME: '<',
    VALID_NAME_REGEX: /^[a-zA-Z_\u0080-\u024F\u4E00-\u9FFF][\u0080-\u024F\u4E00-\u9FFF\w]*$/,
    WINDOW_NAME_LENGTH: 25,
    WINDOW_OUTPUTLEVEL_GROUP: 0,
    WINDOW_OUTPUTLEVEL_GROUPDATE: 1,
    WINDOW_OUTPUTLEVEL_TRANSACTION: 2,
    WINDOW_TYPE_AGGREGATION: 'AGGREGATION',
    WINDOW_TYPE_SLIDING: 'SLIDING',
    WINDOW_TYPE_TRANSFORMATION: 'TRANSFORMATION',
    WINDOW_TYPE_ROLLING: 'ROLLING',
    CONFIG_OPTION_WINDOW_OUTPUT_LEVEL_DISABLE_GROUP_AND_DATE: 'window_output_level.disable_group_and_date',
    CONFIG_OPTION_USE_UTC_TIME: 'use_utc'
};
});

},
'dojox/main':function(){
define(["dojo/_base/kernel"], function(dojo) {
	// module:
	//		dojox/main

	/*=====
	return {
		// summary:
		//		The dojox package main module; dojox package is somewhat unusual in that the main module currently just provides an empty object.
		//		Apps should require modules from the dojox packages directly, rather than loading this module.
	};
	=====*/

	return dojo.dojox;
});
},
'vsd/common/charts/d3DistribChart':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('d3DistribChart', function (d3UtilsSvc,dojoModules,vsdUtil, SvgTooltip) {
        return {
              restrict: 'E',
              template: '<div class="distchart"></div>',
              replace: true,
              scope: {
                'sngModel': '=',
                'parentLoading': '='
              },
              link: function(scope, element,attrs) {
                  //Create variable on scope for svg
                  var parentElement = element[0],
                      resources = dojoModules.resources,
                      d3Wrapper,
	                   format = (scope.sngModel||{}).columnFormat;

                  //Create wrapper module to assist in directive execution
                  d3Wrapper = d3UtilsSvc.d3DirectiveWrapper({
                      scope: scope,
                      elSelector: element,
                      elName:  "parentElement",
                      dataVarName: "sngModel.bins",
	                   watchType: d3UtilsSvc.WATCH_TYPE_ALL,
	                   pLoading: "parentLoading",
                      renderFunc: renderChart,
                      clearBeforeRender: true,
	                   parentScope: scope,
                      margins:  {top: 10, left: 70, bottom: 40, right: 10}
                  });

	              new SvgTooltip({
		              connectId: attrs.id,
		              selector: "text, .bar",
		              getContent: function(matchedNode){
			              return matchedNode.textContent.replace('|','-');
		              }
	              });

                  //Function to render d3 histogram from provided data
                  function renderChart(svg,chartData) {
                      var colIsNumeric=true,
                          formattedData=[],
                          formatComma = d3UtilsSvc.commaFmtFunc,
                          formatInteger = d3UtilsSvc.integerFmtFunc,  // doesn't add decimal places when they aren't needed
                          colIsLowCardinalityNumeric;

	                  format = (scope.sngModel||{}).columnFormat;

                      //Determine if column is numeric
                      colIsNumeric = angular.isDefined(chartData[0].name) ? false: true;
                      colIsLowCardinalityNumeric = angular.isDefined(chartData[0].midValue) &&
                                                   chartData[0].minValue === chartData[0].midValue &&
                                                   chartData[0].midValue === chartData[0].maxValue;
                      //Prepare binning data output for chart
                      var binNbr = 0,
                          numBins = chartData.length;
                      angular.forEach(chartData,function(d) {
                          var row={};
                          row.cnt= d.rowCount||0; // converts to int
                          if(angular.isDefined(d.name) || colIsLowCardinalityNumeric)  {
                              colIsNumeric = false;
                              //Replace other placeholder text if required
                              if(d.name === vsmCommonConst.GRAPH_OTHER_CATEGORY) {
                                  row.binMid = resources.chartsD3_DistributionChart_otherLbl_txt;
                              }
                              else if(colIsLowCardinalityNumeric) {
                                  row.binMid = vsdUtil.applySASNumericFormat(d.midValue,format);
                              }
                              else {
                                  row.binMid = d.name;
                              }
                          }
                          else {
                              //Set min value
                              if (isNaN(d.minValue))  {
                                  row.binMin = d.minValue ;
                                  colIsNumeric = false;
                              }
                              else{
                                  var minValue = d.minValue;
                                  row.binMin = vsdUtil.applySASNumericFormat(minValue,format);
                              }
                              //Set max value
                              if (isNaN(d.maxValue)) {
                                  row.binMax =d.maxValue ;
                              }
                              else {
                                  row.binMax = vsdUtil.applySASNumericFormat(d.maxValue,format);
                              }
                              //Set mid value
                              /*if(row.binMin === row.binMax) {
                                  row.binMid = row.binMin;

                              }
                              else {
                                  row.binMid = row.binMin +  ' | ' + row.binMax;
                              }*/
                              row.binMid = createRangeString(row.binMin,row.binMax,binNbr,numBins);

                              binNbr++;
                          }
                          formattedData.push(row);
                      });

                      //Create y-scale
                      var y = d3.scale.linear()
                          .domain([0, d3.max(formattedData, function(d) { return d.cnt; }) *1.2 ])
                          //.rangeRound([d3Wrapper.getHeight(), 0]);
                          .range([d3Wrapper.getHeight(), 0]);

                      //Create x-scale
                      var x = d3.scale.ordinal()
                          .domain(formattedData.map(function(d) {
                              return d.binMid;
                          }))
                          .rangeRoundBands([0, d3Wrapper.getWidth()], 0.1);

                      //Chart frame
                      svg.append("rect")
                          .attr("class", "chartFrame")
                          .attr("width", d3Wrapper.getWidth())
                          .attr("height", d3Wrapper.getHeight());

                      //Create x-axis
                      var xAxis = d3.svg.axis()
                          .scale(x)
                          .orient("bottom")
                          .tickPadding(6);
                      //Create y-axis
                      var numTicks = (d3Wrapper.getHeight() -10)/15;
                      var yAxis = d3.svg.axis()
                          .scale(y)
                          .orient("left")
                          .ticks(numTicks)
                          .tickSize(-d3Wrapper.getWidth(), 0)  //adds grid lines on chart
                          .tickPadding(6);

                      var xAxisG = svg.append("g")
                          .attr("class", "x axis")
                          .attr("transform", "translate(0," + d3Wrapper.getHeight() + ")")
                          .call(xAxis);

                      //Split the x-axis labels between lines based on dash
                      if(colIsNumeric) {
		                  d3UtilsSvc.splitXAxisLabels(svg,'|',true,dojoModules.resources.app_rangeConjunctionLbl_txt);
	                  }

                      //Append tooltip to each bar label element
                      var bLabels = xAxisG.selectAll("text")[0];
                      for (var ii=0; ii < bLabels.length; ii++){
                          bLabels[ii].id= 'bLabel' + ii;
                          var barLbl = bLabels[ii].__data__.replace('|','-');
                          if  (barLbl.length > 10 && bLabels.length > 5) {
                              bLabels[ii].textContent = barLbl.substring(0,10) + "...";
                          }
                      }

                      //Create y-axis
                      var yAxisG = svg.append("g")
                          .attr("class", "y axis")
                          .call(yAxis);

                      yAxisG.selectAll("line").style("stroke-dasharray", ("3, 9"));

                      //Create distribution chart bars
                      svg.selectAll(".distchart")
                          .data(formattedData)
                          .enter().append("rect")
                          .attr("class", "bar")
                          .attr("x", function(d) { return x(d.binMid); })
                          .attr("width", x.rangeBand())
                          .attr("y", function(d) { return y(d.cnt); })
                          .attr("height", function(d) { return d3Wrapper.getHeight() - y(d.cnt); })
                          .attr("id",function(d,i) {
                              return "bar" + i;
                          })
                          .text(function(d) {
                              return  resources.chartsD3_DistributionChart_valueLbl_txt + ":  " + d.binMid + "<br> " + resources.chartsD3_DistributionChart_countLbl_txt + ": " + formatInteger(d.cnt);
                          })
                          .style("text-align","left");

                      //Add svg text to display the bar count, horizontally centered above each bar
                      svg.selectAll(".distchart")
                          .data(formattedData)
                          .enter().append("text")
                          .text(function (d) {
                              return formatInteger(d.cnt);
                          })
                          .attr("x", function (d) {
                              return x(d.binMid) + x.rangeBand()/2 - (this.getBBox().width/2);
                          })
                          .attr("y", function (d) {
                              return y(d.cnt) - 3;
                          });
                  }

                  function createRangeString(min,max,binNbr,numBins) {
                      if(binNbr === 0) {
                        return "< " + max;
                      }
                      else if(binNbr === numBins - 1) {
                          return ">= " + min;
                      }

                      else {
                          return ">= " + min + "|" + "< " + max;
                      }
                  }


             }
        };
  });

});

},
'vsd/common/charts/d3DualSlider':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('d3DualSlider', function () {
        return {
              restrict: 'E',
              template: '<span></span>',
              replace: true,
              scope: {
                'absoluteMin':    '=',
                'absoluteMax':    '=',
                'currMin':        '=',
                'currMax':        '=',
                'changeHandler':  '&'
              },
              link: function(scope, element, attrs,ctrl) {
                  var svg,
                      margins = {
                          left: 0,
                          right: 0,
                          top: 0,
                          bottom: 0
                      },
                      sliderRadius = 5,
                      edgeTextWidth = 20,
                      barHeight = 2;

                  //Do initial gauge render
                  createGauge();

                  function createGauge() {
                      var width = parseInt(attrs.width) - margins.left - margins.right,
                          height = parseInt(attrs.height) - margins.top - margins.bottom,
                          ctrlGroup,
                          barWidth = width - edgeTextWidth* 2 - sliderRadius* 2,
                          barStartXPos = edgeTextWidth + sliderRadius,
                          barEndXPos = barWidth + barStartXPos,
                          barScale = d3.scale.linear()
                              .domain([scope.absoluteMin,scope.absoluteMax])
                              .range([barStartXPos,barEndXPos]);

                      svg = d3.select(element[0]).append("svg")
                          .attr("width",width)
                          .attr("height",height)
                          .append("g")
                          //.attr("transform", "translate(" + margins.left + "," + margins.top + ")"),

                      //Draw group vertically centered to simplyfy positioning
                      ctrlGroup = svg.append("g")
                          .attr("transform","translate(0, " + (height/2 - barHeight/ 2) + ")");

                      //Draw bar for gauge - leave space at edges for sliders and edge numbers
                      ctrlGroup.append("rect")
                          .attr("width",barWidth)
                          .attr("height",barHeight)
                          .attr("y",height/2)
                          .attr("x",barStartXPos);

                      //Draw circles for sliders
                      ctrlGroup.selectAll('.d3DualSliderKnobs')
                          .data([scope.currMin,scope.currMax])
                          .enter()
                          .append("circle")
                          .attr("class", "d3DualSliderKnobs")
                          .attr("cx",function(d){
                              return barScale(d)
                          })
                          .attr("cy",height/2)
                          .attr("r",sliderRadius);


                      //Draw absolute min text
                      ctrlGroup.append("text")
                          .attr("x",0)
                          .attr("y",height/2)
                          .text(scope.absoluteMin);

                      //Draw absolute max text
                      ctrlGroup.append("text")
                          .attr("x",barEndXPos)
                          .attr("y",height/2)
                          .text(scope.absoluteMax);

                  }
              }
        };
  });

});

},
'vsd/common/charts/d3ParallelCoordChart':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .directive('d3ParallelCoordChart', ['$rootScope',
        'd3UtilsSvc',
        'dojoModules',
        'vsdUtil',
        function ($rootScope, d3UtilsSvc, dojoModules, vsdUtil) {

            return {
                restrict: 'E',
                template: '<div class="d3Chart parallelCoord"></div>',
                replace: true,
                scope: {
                    sngData: '=',    // array of data collections   [{"var1":"a", "var2":"b", "var3":1}, {"var1":"x", "var2":"y", "var3":2}...]
                    selectedItem: '=',    // selected object from sngData
                    axisVars: '=',    // array containing the variable keys to plot, ["var1", "var2", "var3"]
                    axisLabels: '=',    // array containing the axis variable display labels
                    axisFormats: '=',    // array containing the axis formats
                    brushHandler: '&',    // handler for brushing (selecting ranges on axes)
                    highlightAxes: '=',    // array containing the axis variables to render in red
                    parentLoading: '='     // parent loading flag
                },
                link: function (scope, element, attrs, ctrl) {

                    var dataColl = scope.sngData ? scope.sngData : [],
                        varKeys = attrs.axisVars ? scope.axisVars : (dataColl.length > 0 ? dataColl[0] : []),
                        varLabels = attrs.axisLabels ? _.map(varKeys, function (key) {
                            return scope.axisLabels[key];
                        }) : varKeys,
                        varFmts = attrs.axisFormats ? scope.axisFormats : {},
                        hlKeys = attrs.highlightAxes ? scope.highlightAxes : {},
                        xRange = null,
                        elementWidth = null,
                        getFormatter = function (format) {
                            if (format === vsdUtil.defaultFormats.date) {
                                return (function (value) {
                                    if (vsdUtil.isSasDateConstant(value)) {
                                        return value;
                                    }
                                    return dojoModules.locale.format(value, {selector: 'date'});
                                });
                            }
                            else if (format === vsdUtil.defaultFormats.datetime) {
                                return (function (value) {
                                    if (vsdUtil.isSasDateTimeConstant(value)) {
                                        return value;
                                    }
                                    return dojoModules.locale.format(value, {formatLength: 'medium'});
                                });
                            }
                            return (function (val) {
                                var numVal = (typeof val === 'number') ? val : parseFloat(val);
                                return vsdUtil.applySASNumericFormat(numVal, format);
                            });
                        };


                    //Create variable on scope for svg
                    var parentElement = element[0],
                        resources = dojoModules.resources,
                        d3Wrapper;

                    //Create wrapper module to assist in directive execution
                    var params = {
                        scope: scope,
                        elSelector: element,
                        elName: "parentElement",
                        dataVarName: "sngData",
                        pLoading: "parentLoading",
                        renderFunc: renderChart,
                        clearBeforeRender: true,
                        parentScope: scope,
                        margins: {top: 50, bottom: 30, left: 15, right: 5},
                        watchType: "WATCH_TYPE_ALL"   // watch both data array object changes and array item changes
                    };

                    createD3Wrapper();

                    scope.$watch("sngData", function (newValue) {
                        d3Wrapper.destroy();
                        createD3Wrapper();
                    }, true);


                    function createD3Wrapper(resized) {

                        elementWidth = element[0].clientWidth;

                        //Calculate minimum width needed for axis labels and adjust x-axis ranges if needed to ensure everything fits
                        xRange = null;
                        var axisMinRange = [],
                            numExceeds = 0,
                            axisPadding = 35,  // axis bar and ticks plus extra padding
                            totalWidth = 0,
                            numVars = varKeys.length,
                            lastVarLbl = (varLabels[numVars - 1] ? varLabels[numVars - 1] : varKeys[numVars - 1]) || "";

                        var availWidth = elementWidth - params.margins.left - params.margins.right,
                            defaultAxisWidth = varKeys.length > 0 ? (availWidth / varKeys.length) : availWidth;  // d3 uses equal widths to separate axes by default

                        for (var i = 0; i < varKeys.length; i++) {
                            var key = varKeys[i],
                                varLabel = varLabels[i] ? varLabels[i] : varKeys[i],
                                varOnLeftLbl = "";

                            if (i > 0) {
                                varOnLeftLbl = varLabels[i - 1] ? varLabels[i - 1] : varKeys[i - 1];
                            }

                            // Get width of title text that falls between the previous axis bar and the current axis bar
                            // Axis titles are staggered when the # axes > 2 to minimize horizontal space requirements
                            var axisTitleWidth = (varKeys.length == 2) ? (getTextWidth(varOnLeftLbl) + getTextWidth(varLabel)) / 2
                                : Math.max(getTextWidth(varOnLeftLbl), getTextWidth(varLabel)) / 2;

                            // Get width of widest data string for the variable
                            var formatter = (varFmts && varFmts[key]) ? getFormatter(varFmts[key]) : function (val) {
                                return val.toString();
                            };
                            var dataMaxWidth = _.max(_.map(_.pluck(dataColl, key), function (val) {
                                var formattedVal = formatter(val);
                                var width = getTextWidth(formattedVal);
                                return width;
                            })) + axisPadding;   // include extra space for readability and axis bar itself

                            var axisWidth = Math.max(dataMaxWidth, axisTitleWidth);
                            totalWidth += axisWidth;
                            axisMinRange.push(totalWidth);
                            if (totalWidth > defaultAxisWidth) {
                                numExceeds++;
                            }
                        }

                        totalWidth += (getTextWidth(lastVarLbl) / 2);    // last axis title extends past axis by half its width

                        // If total width needed is greater than available width, use calculated ranges
                        if (totalWidth > availWidth) {
                            params.staticWidth = totalWidth + params.margins.left + params.margins.right;
                            xRange = axisMinRange;
                        }
                        // If some axes exceed default width, use axisMinRange but add enough padding to fill available horizontal space so it doesn't look funny
                        else if (numExceeds > 0) {
                            var newRange = [],
                                fillPadding = Math.floor((availWidth - totalWidth) / axisMinRange.length),
                                factor = 0;
                            angular.forEach(axisMinRange, function (val) {
                                factor++;
                                newRange.push(val + fillPadding * factor);
                            });
                            xRange = newRange;
                        }
                        else if (params.hasOwnProperty('staticWidth')) {
                            delete params.staticWidth;
                        }
                        d3Wrapper = d3UtilsSvc.d3DirectiveWrapper(params);
                    }


                    function renderChart(chartSvg, chartData) {

                        // If element was resized since last render, need to create new wrapper using recalculated width
                        if (elementWidth !== element[0].clientWidth) {
                            createD3Wrapper(true);
                        }

                        var ypos = 0,
                            axis_y_margin = 15;

                        if (attrs.selectedItem) {
                            scope.$watch('selectedItem', function () {
                                highlight(scope.selectedItem);
                            });
                        }

                        var dimensions = {},
                            dragging = {},
                            highlighted = null;

                        //Chart frame
                        chartSvg.append("rect")
                            .attr("class", "chartFrame")
                            .attr("width", d3Wrapper.getWidth())
                            .attr("height", d3Wrapper.getHeight());

                        var line = d3.svg.line().interpolate('cardinal').tension(0.9),
                            axis = d3.svg.axis().orient("left"),
                            background,
                            foreground,
                            h = d3Wrapper.getHeight(),
                            w = d3Wrapper.getWidth(),

                            x = (xRange == null) ? d3.scale.ordinal().rangePoints([40, w], 1)
                                : d3.scale.ordinal().range(xRange, 1),

                        //x = d3.scale.ordinal().rangePoints([50, w],1),
                            y = {};

                        // Extract the list of dimensions and create a scale for each.
                        x.domain(dimensions = varKeys.filter(function (d) {
                            var value = Number(chartData[0][d]);
                            if (!isNaN(value)) {
                                var fmt = (varFmts && varFmts[d] ? varFmts[d] : null);
                                if (fmt === vsdUtil.defaultFormats.date) {
                                    // Create time scale
                                    return  (y[d] = d3.time.scale()
                                        .domain(d3.extent(chartData, function (p) {
                                            return p[d];  // note: data must be a JavaScript Date
                                        }))
                                        .range([h - axis_y_margin, axis_y_margin], 1));
                                }
                                else if (fmt === vsdUtil.defaultFormats.datetime) {
                                    // Create time scale
                                    return  (y[d] = d3.time.scale()
                                        .domain(d3.extent(chartData, function (p) {
                                            return p[d];  // note: data must be a JavaScript Date
                                        }))
                                        .range([h - axis_y_margin, axis_y_margin], 1));
                                }
                                else {
                                    // Create numeric linear scale
                                    return  (y[d] = d3.scale.linear()
                                        .domain(d3.extent(chartData, function (p) {
                                            return +p[d];
                                        }))
                                        .range([h - axis_y_margin, axis_y_margin], 1));
                                }
                            }
                            else {  // Create categorical scale
                                return (y[d] = d3.scale.ordinal()
                                    .domain(_.pluck(chartData, d))
                                    .rangePoints([axis_y_margin, h - axis_y_margin], 1));
                            }
                        }));

                        // Add background lines for context.
                        background = chartSvg.append("svg:g")
                            .attr("class", "background")
                            .selectAll("path")
                            .data(chartData)
                            .enter().append("svg:path")
                            .attr("d", path);


                        // Add foreground lines for focus.
                        foreground = chartSvg.append("svg:g")
                            .attr("class", "foreground")
                            .selectAll("path")
                            .data(chartData)
                            .enter().append("svg:path")
                            .attr("d", path);


                        // Define dimensions
                        var g = chartSvg.selectAll(".dimension")
                            .data(dimensions)
                            .enter().append("svg:g")
                            .attr("class", "dimension")
                            .attr("transform", function (d) {
                                return "translate(" + x(d) + ")";
                            })
                            .call(d3.behavior.drag()
                                .on("dragstart", function (d) {
                                    dragging[d] = this.__origin__ = x(d);
                                    background.attr("visibility", "hidden");
                                })
                                /*
                                 // This allows axes to be dragged around
                                 .on("drag", function(d) {
                                 dragging[d] = Math.min(w, Math.max(0, this.__origin__ += d3.event.dx));
                                 foreground.attr("d", path);
                                 dimensions.sort(function(a, b) { return position(a) - position(b); });
                                 x.domain(dimensions);
                                 g.attr("transform", function(d) { return "translate(" + position(d) + ")"; })
                                 })
                                 */
                                .on("dragend", function (d) {
                                    delete this.__origin__;
                                    delete dragging[d];
                                    transition(d3.select(this)).attr("transform", "translate(" + x(d) + ")");
                                    transition(foreground)
                                        .attr("d", path);
                                    transition(background)
                                        .attr("d", path)
                                        .transition()
                                        .delay(500)
                                        .duration(0)
                                        .attr("visibility", null);
                                }));

                        // Add an axis and title.
                        g.append("svg:g")
                            .attr("class", function (d) {
                                var cls = hlKeys.indexOf(d) >= 0 ? "axis axisHighlighted:" : "axis";
                                return cls;
                            })
                            .each(function (d) {
                                // create and format axis scale
                                if (varFmts && varFmts[d]) {
                                    var formatter = getFormatter(varFmts[d]);
                                    d3.select(this).call(axis.scale(y[d]).tickFormat(formatter));
                                }
                                else {
                                    d3.select(this).call(axis.scale(y[d]).tickFormat(null));  // resets to default
                                }
                            })

                            .append("svg:text")
                            .attr("text-anchor", "middle")

                            // Vertical axis title

                            .attr("y", function () {
                                if (varKeys.length > 2) {
                                    // stagger alternating title heights so than adjacent labels don't overlap
                                    return (ypos++ % 2 == 1) ? -12 : -30;
                                }
                                return -19;

                            })
                            .text(function (String) {   // Use axis label
                                var pos = varKeys.indexOf(String);
                                return (pos < varLabels.length ? varLabels[pos] : String);
                            })
                            .attr("style", "font-weight: bold;");

                        /*
                         .append("svg:foreignObject")
                         .attr("text-anchor", "middle")
                         .attr("width",  w/(varKeys.length) )
                         .attr("height", 50)
                         .attr("y", -30)
                         .text(String)
                         .attr("text-anchor", "middle")
                         .attr("style", "font-weight: bold;text-align:center");
                         */

                        // Add and store a brush for each axis.
                        g.append("svg:g")
                            .attr("class", "brush")
                            .each(function (d) {
                                d3.select(this).call(y[d].brush = d3.svg.brush().y(y[d]).on("brush", handleBrush));
                            })
                            .selectAll("rect")
                            .attr("x", -12)
                            .attr("width", 24);

                        function position(d) {
                            var v = dragging[d];
                            return v == null ? x(d) : v;
                        }

                        // Returns the path for a given data point.
                        function path(d) {
                            if (d) {
                                return line(dimensions.map(function (p) {
                                    return [position(p), y[p](d[p])];
                                }));
                            }
                        }

                        // Handles a brush event, toggling the display of foreground lines.

                        function handleBrush() {
                            var actives = dimensions.filter(function (p) {
                                return !y[p].brush.empty();
                            });

                            var extents = actives.map(function (p) {
                                return y[p].brush.extent();
                            });

                            var filter = {};
                            _(actives).each(function (key, i) {
                                filter[key] = {
                                    min: extents[i][0],
                                    max: extents[i][1]
                                }
                            });

                            var matchedData = (actives.length > 0) ? [] : null;

                            foreground.style("display", function (d) {
                                return actives.every(function (p, i) {
                                    var numVal = Number(d[p]);
                                    var value = (!isNaN(numVal)) ? d[p] : y[p](d[p]);
                                    if (extents[i][0] <= value && value <= extents[i][1]) {
                                        matchedData.push(d);
                                        return true;
                                    }
                                    return false;
                                }) ? null : "none";
                            });

                            if (attrs.brushHandler) {
                                scope.brushHandler({selection: matchedData});
                                safeApply();
                            }
                        }

                        function transition(g) {
                            return g.transition().duration(500);
                        }

                        function highlight(i) {
                            if (typeof i == "undefined") {
                                d3.select(".foreground").style("opacity", function (d, j) {
                                    return "1";
                                });
                                if (highlighted) {
                                    highlighted.remove();
                                }
                            } else {
                                d3.select(".foreground").style("opacity", function (d, j) {
                                    return ".35";
                                });
                                if (highlighted != null) {
                                    highlighted.remove();
                                }
                                highlighted = chartSvg.append("svg:g")
                                    .attr("class", "highlight")
                                    .selectAll("path")
                                    .data([i])
                                    .enter().append("svg:path")
                                    .attr("d", path)
                            }
                        }
                    }

                    function safeApply() {
                        if (!$rootScope.$$phase) {
                            scope.$apply();
                        }
                    }

                    function getTextWidth(text) {
                        // Get width of rendered text in pixels
                        var cellFont = getTextWidth.cellFont || getFont();
                        var canvas = getTextWidth.canvas || (getTextWidth.canvas = document.createElement("canvas"));
                        var context = canvas.getContext("2d");
                        context.font = cellFont || "14px Lato, sans-serif";
                        var metrics = context.measureText(text);
                        return metrics.width;

                        function getFont() {
                            var cell = $(".tick");
                            if (cell && cell.length > 0) {
                                return (cell.css("font-style") + " " +
                                    cell.css("font-weight") + " " +
                                    cell.css("font-size") + " " +
                                    cell.css("font-family"));
                            }
                            return "14px Lato, sans-serif";
                        }
                    }

                }
            };
        }]);

});

},
'vsd/common/charts/d3PctGauge':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('d3PctGauge', function (d3UtilsSvc) {
        return {
              restrict: 'E',
              template: '<div></div>',
              replace: true,
              scope: {
                'sngModel':         '=',
                'pctWidthField':    '@',
                'numQtyField':      '@',
                'numTotalQtyField': '@',
                'parentLoading':    '=',
                'staticWidth':      '@',
                'staticHeight':     '@'
              },
              link: function(scope, element, attrs,ctrl) {
                  //Create variable on scope for svg
                  var parentElement = element[0],
                      d3Wrapper,
                      //pctFmtFunc = d3.format("0.2%"),
                      //commaFmtFunc = d3.format(",.0f");
                      pctFmtFunc = d3UtilsSvc.pctFmtFunc,
                      commaFmtFunc = d3UtilsSvc.commaFmtFunc,
                      integerFmtFunc = d3UtilsSvc.integerFmtFunc;    // doesn't add decimal places when not needed

                  scope.$on('$destroy', function () {
                      d3Wrapper.destroy();
                  });

                  //Create wrapper module to assist in directive execution
                  d3Wrapper = d3UtilsSvc.d3DirectiveWrapper({
                      scope: scope,
                      elSelector: element,
                      elName:  "parentElement",
                      dataVarName: "sngModel",
	                  pLoading: "parentLoading",
                      renderFunc: createGauge,
                      updateRenderFunc: updateGauge,
                      clearBeforeRender: true,
	                  parentScope: scope,
                      margins:  {top: 1, left: 1, bottom: 1, right: 1} ,
                      staticWidth: parseInt(attrs.staticWidth),
                      staticHeight: parseInt(attrs.staticHeight)
                  });

                  function updateGauge(svg,chartData) {
                      var w = d3Wrapper.getWidth(),
                          pct = chartData[attrs.numQtyField]/chartData[attrs.numTotalQtyField] || 0,
                          newRectWidth = w * pct;

                     svg.select(".gaugeBar")
                          .transition().duration(750)
                          .attr("width",newRectWidth);

                      var prevContentToks = svg.select("text").node().textContent.split(" ");
                      var prevCount = parseInt(prevContentToks[0].replace(",",""));
                      var prevPct = parseFloat(prevContentToks[1].substr(1,prevContentToks.length-1));
                      svg.select("text")
                          .transition().duration(750)
                          .ease('linear')
                          .tween("text", function() {
                              var i = d3.interpolate(prevCount, chartData[attrs.numQtyField]);
                              return function(t) {
                                  var newQty =  i(t),
                                  newPct = (newQty / chartData[attrs.numTotalQtyField]) || 0;
                                  this.textContent = integerFmtFunc(newQty) + " (" + pctFmtFunc(newPct) + ")";
                              };
                          });
                  }
                  function createGauge(svg,chartData) {
                      var barHRatio = 1,
                          barH =  0,
                          barHDiff = 0,
                          pct = chartData[attrs.numQtyField]/chartData[attrs.numTotalQtyField] || 0;

                      var w = d3Wrapper.getWidth();
                      var h = d3Wrapper.getHeight();
                      barH =  h * barHRatio;
                      barHDiff = h - barH;


                      //Append a rectangle with a background to make text visible
                      svg.append("rect")
                          .attr("width",w)
                          .attr("height",h)
                          .attr("class","gaugeBack")
                          .attr("transform","translate(0,0)");
                      //Append another rectangle for gauge bar
                      svg.append("rect")
                          .attr("height",barH)
                          .attr("class","gaugeBar")
                          .attr("transform",function(){return "translate(0," + barHDiff/2 + ")";})
                          .attr("width",(w * pct));
                      //Append text for percentage
                      svg.append("text")
                          .attr("class", "gaugePctTxt")
                          .attr("dy",".35em")
                          .attr("text-anchor","middle")
                          .attr("x",(w)/2)
                          .attr("y",h/2)
                          .text(function() {
                              return integerFmtFunc(chartData[attrs.numQtyField]) + " (" + pctFmtFunc( pct ) + ")";
                          });

                      return svg;
                  }
             }
        };
  });

});

},
'vsd/common/charts/d3StackedBarChart':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('d3StackedBarChart', ['d3UtilsSvc',
                                   'dojoModules',
                                   'vsdUtil',
     function (d3UtilsSvc,
	     dojoModules,
	     vsdUtil) {

	     return {
              restrict: 'E',
              template: '<div class="d3Chart stackedBar"></div>',
              replace: true,
              scope: {
                'sngData':       '=',  // array of objects containing variable key:value pairs
                'stackVars':     '=',  // variables to stack (array of keys)
	             'groupVar':      '@',  // key identifying the group variable
	             'labelVar':      '@',  // key identifying the group label
                'sortVar':       '@',  // optional, variable to order the grouped bars on, or "BARSIZE" to order by bar size
                'descending':    '@',  // "true" or "false", used with sortVar
	             'showLegend':    '@',  // "true" or "false"
	             'sngTitle':      '@',  // chart title
	             'barMouseoverHandler':   '&',
	             'barMouseoutHandler':    '&',
	             'barRmbclickHandler':    '&',
	             'parentLoading': '='   // parent loading flag
              },

		     link: function(scope, element, attrs,ctrl) {
			     //Create variable on scope for svg
			     var parentElement = element[0],
				     resources = dojoModules.resources,
				     margins = {top: 10, left: 70, bottom: 40, right: 5},
				     d3Wrapper,
				     intFormatter = function(value) {
                         return vsdUtil.applySASNumericFormat(value, vsdUtil.defaultFormats.integer);
                     };

			     //Create wrapper module to assist in directive execution
			     d3Wrapper = d3UtilsSvc.d3DirectiveWrapper({
				     scope: scope,
				     elSelector: element,
				     elName:  "parentElement",
				     dataVarName: "sngData",
				     pLoading: "parentLoading",
				     renderFunc: renderChart,
				     clearBeforeRender: true,
				     parentScope: scope,
				     margins:  margins,
				     watchType: "WATCH_TYPE_ALL"   // watch both data array object changes and array item changes
			     });

			     scope.$watch("sngData", function() {
				     d3Wrapper.renderChart();
			     });

			     function renderChart(chartSvg, chartData) {

				     var width = d3Wrapper.getWidth(),
					     height =  d3Wrapper.getHeight(),
					     formatComma = d3UtilsSvc.commaFmtFunc,
				        data= angular.copy(chartData),
					     varKeys = attrs.stackVars ? scope.stackVars : chartData[0].keys(),
					     groupKey = attrs.groupVar ? scope.groupVar : varKeys[0],  // default to first variable if not groupVar not assigned
					     labelKey = attrs.labelVar ? scope.labelVar : groupKey,
					     sortKey =  attrs.sortVar ? (scope.sortVar.toLowerCase() === "barsize" ? "barsize" : scope.sortVar) : null,
					     title = attrs.sngTitle ? scope.sngTitle.trim() : "",
					     showLegend = attrs.showLegend ? scope.showLegend : true;

				     function sortFunc(a, b) {
					     var desc = b[sortKey] - a[sortKey];
					     return (attrs.descending && scope.descending.toLowerCase() == "true" ) ? desc : -1*desc;
				     };



				     //Chart frame
				     chartSvg.append("rect")
					     .attr("class", "chartFrame")
					     .attr("width", d3Wrapper.getWidth())
					     .attr("height", d3Wrapper.getHeight())
					     .attr("pointer-events", "none"); // Pointer events will be captured by Overlay layer and Bars defined below


				     // define x and y axes
				     var x = d3.scale.ordinal()
					     .rangeRoundBands([0, width],.1,.4);

				     var y = d3.scale.linear()
					     .rangeRound([height, showLegend == true ? 20 : 0]); // make room for legend at top

				     var xAxis = d3.svg.axis()
					     .scale(x)
					     .orient("bottom")
					     .tickPadding(6);

				     var yAxis = d3.svg.axis()
					     .scale(y)
					     .orient("left")
					     .ticks(10)
					     .tickPadding(6)
					     .tickSize(-d3Wrapper.getWidth(), 0)  //adds grid lines on chart
					     .tickFormat(intFormatter)
					     .tickSubdivide(0)


				     // Create array containing fill colors to map to each stack variable
				     // Note there are 8 fill colors available, so if there are more than 8 variables
				     // they have to be cycled.
				     var fillColors = [],
					     colors = d3UtilsSvc.getCorporateColorList();

				     for (var i=0; i<varKeys.length; i++) {
					     fillColors.push(colors[i%8]);
				     }

				     // Map variables to fill colors
				     var z = d3.scale.ordinal()
					     .domain(varKeys)
					     .range(fillColors);

				     if (title.length > 0) {
					     chartSvg.append("text")
						     .attr("x", (width / 2))
						     .attr("y", 0)
						     .attr("text-anchor", "middle")
						     .attr("class","title")
						     .text(title);
				     }

				     // Render legend
				     if (showLegend == true)  {
					     var legend = chartSvg.selectAll(".legend")
						     .data(z.domain().slice().reverse())
						     .enter().append("svg:g")
						     .attr("class", "legend")
						     .attr("transform", function(d, i) {
							     return "translate(-" + (i * 100) + ",0)";
						     });

					     legend.append("rect")
						     .attr("x", width - 20)
						     .attr("y", -12)
						     .attr("width", 16)
						     .attr("height", 16)
						     .attr("class",z);

					     legend.append("text")
						     .attr("x", width - 24)
						     .attr("y", 0)
						     .style("text-anchor", "end")
						     .text(function(d) { return d; });
				     }

				     // sort data if sorted on a data variable rather than bar size
				     if (sortKey && sortKey != "barsize") {
					     data.sort(sortFunc);
				     }

				     // Define stacks for each row
				     var stacks = [];
				     _.each(data, function(d) {
					     var y0 = 0;
					     var bars = z.domain().map(function(name) {
						     var obj= {
							     name: name,
							     group: d[groupKey],
							     label: d[labelKey],
							     y0: y0,
							     y1: y0 += +d[name]
						     };
						     return obj;
					     });
					     var stack = { bars: bars, group: d[groupKey], label: d[labelKey], barsize: bars[bars.length-1].y1};
					     stacks.push(stack);
				     });

				     // sort stacks if sorting on total bar size
				     if (sortKey == "barsize") {
				         stacks.sort(sortFunc);
				     }

				     x.domain(stacks.map(function(d) {
					     return d.label;
				     }));

				     y.domain([0, d3.max(stacks, function(d) {
					     return d.barsize * 1.1;      /// leave a little space above tallest bar
				     })]);

				     // append x-axis
				     chartSvg.append("svg:g")
					     .attr("class", "x axis")
					     .attr("transform", "translate(0," + height + ")")
					     .call(xAxis);

				     d3UtilsSvc.splitXAxisLabels(chartSvg,';', false);

				     // append y-axis
				     chartSvg.append("svg:g")
					     .attr("class", "y axis")
					     .call(yAxis);
					     /*
					     .append("text")
					     .attr("transform", "rotate(-90)")
					     .attr("y", 6)
					     .attr("x", -20)
					     .attr("dy", ".71em")
					     .style("text-anchor", "end")
					     .text(attrs.sngLabel ? scope.sngLabel : "");
					     */

				     // Create Overlay layer to consume pointer events, make it cover entire chart area
				     chartSvg.append("rect")
					     .attr("fill", "none")
					     .attr("stroke-width",0)
					     .attr("x", -margins.left)
					     .attr("y", -margins.top)
					     .attr("width", element[0].clientWidth )
					     .attr("height", element[0].clientHeight )
					     .attr("pointer-events", "all")
					     // If mouse is over a bar or tooltip, this will get triggered when mouse leaves either item
					     .on('mouseover', onBarMouseOut)
					     // Closes tooltip if mouse leaves chart area
					     .on('mouseout', onBarMouseOut);


				     var group = chartSvg.selectAll(".groups")
					     .data(stacks)
					     .enter().append("svg:g")
					     .attr("class", "stack")
					     .attr("transform", function(d) {
						     return "translate(" + x(d.label) + ",0)";
					     });

				     var barWidth = Math.min.apply(null, [x.rangeBand(), 100]);

				     // Draw Bars
				     group.selectAll("rect")
					     .data(function(d) {
						     return d.bars;
					     })
					     .enter().append("rect")
					     .attr("width", barWidth)
					     .attr("x", (x.rangeBand()-barWidth)/2 )
					     .attr("y", function(d) {
						     return y(d.y1);
					     })
					     .attr("height", function(d) {
						     return y(d.y0) - y(d.y1);
					     })
					     .style("fill", function(d) {
						     return z(d.name);
					     })
					     .attr("pointer-events", "all")
					     .on('mouseover', onBarMouseOver)
				     /* .on('mouseout', onBarMouseOut) */
					     .on('contextmenu', onBarRmbclick);

                     // Add svg text to display the bar group's maximum y value, represented by the data's "barsize" property
                     // Center text horizontally above each bar group
                     group.append("text")
                         .text(function(d) {
                            return intFormatter(d.barsize);
                         })
                         .attr("x", function(d) {
                             return (x.rangeBand() - this.getBBox().width) / 2;
                         })
                         .attr("y", function (d) {
                             return y(d.barsize) - 3;
                         });
			     }

			     function onBarMouseOver(barData) {
				     if (scope.barMouseoverHandler && d3.event) {
					     d3.event.preventDefault();
					     scope.barMouseoverHandler({barData: barData, domNode: d3.event.currentTarget});
					     scope.$apply();
				     }
			     }

			     function onBarMouseOut(barData) {
				     if (scope.barMouseoutHandler) {
					     if (d3.event) {
					        d3.event.preventDefault();
					     }
					     scope.barMouseoutHandler();
					     scope.$apply();
				     }
			     }

			     function onBarRmbclick(barData) {
				     if(scope.barRmbclickHandler && d3.event) {
					     d3.event.preventDefault();
				        scope.barRmbclickHandler({barData: barData, domNode: d3.event.currentTarget});
					     scope.$apply();
				     }
			     }
		     }
	     };
     }]);

});

},
'vsd/common/charts/d3UtilsSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('d3UtilsSvc', ['$window', 'dojoModules', 'SvgTooltip','$timeout', 'vsdUtil',
    function ($window, dojoModules, SvgTooltip,$timeout,vsdUtil) {

        function handleParentWidgetResize(parentEl,handler) {
            var parentWidget = dojoModules.registry.getEnclosingWidget(parentEl);
            if (parentWidget && parentWidget.resize) {
                var debouncedResize = _.debounce(handler, 50);
                dojoModules.aspect.after(parentWidget, "resize", debouncedResize);
            }
        }
		return {
			d3DirectiveWrapper: function (params) {
				var WATCH_TYPE_OBJ = 'WATCH_TYPE_OBJ',
					WATCH_TYPE_ARRAY = 'WATCH_TYPE_ARRAY',
					WATCH_TYPE_ALL = 'WATCH_TYPE_ALL';

				var svg,
					height,
					width,
					elScope = params.scope,
					elSelector = params.elSelector,
					el = elSelector[0],
					elName = params.elName,
					dataVarName = params.dataVarName,
					renderFunc = params.renderFunc,
					updateRenderFunc = params.updateRenderFunc,
					firstRendering = true,
//                    chartData = elScope[dataVarName],
					chartData = null,
					watchType = angular.isDefined(params.watchType) ? params.watchType : WATCH_TYPE_OBJ,
					pLoading = params.parentLoading,
					clearBeforeRender = angular.isDefined(params.clearBeforeRender) ? params.clearBeforeRender : true,
					truncateSvg = angular.isDefined(params.truncateSvg) ? params.truncateSvg : false,
					margins = angular.isDefined(params.margins) ?
						params.margins :
					{
						left:   0,
						right:  0,
						top:    0,
						bottom: 0
                    },
                    staticWidth =  params.staticWidth,
                    staticHeight =  params.staticHeight,
                    resizeWatcherInitialized = false;

				createSVG();
				//Create watch on data
				createDataWatch();
				//Return public properties/methods
				return {
					createSvg:        createSVG,
					getSvg:           getSvg,
					getWidth:         getWidth,
					getHeight:        getHeight,
					renderChart:      renderChart,
					WATCH_TYPE_OBJ:   WATCH_TYPE_OBJ,
					WATCH_TYPE_ARRAY: WATCH_TYPE_ARRAY,
					WATCH_TYPE_ALL:   WATCH_TYPE_ALL,
					destroy:          destroy
				}

				function getWidth() {
					return width;
				}

				function getHeight() {
					return height;
				}

				function getSvg() {
					return svg;
				}

				function createSVG() {
					var truncateFactor = truncateSvg ? .99 : 1,
						svgWidth = el.clientWidth * truncateFactor,
						svgHeight = el.clientHeight * truncateFactor;
					width = svgWidth - margins.left - margins.right,
						height = svgHeight - margins.top - margins.bottom;

                    if(staticWidth) {
                        width = staticWidth - margins.left - margins.right;
                        svgWidth = staticWidth ;
                    }
						  if (staticHeight) {
                        height = staticHeight - margins.top - margins.bottom;
                        svgHeight = staticHeight ;
                    }


					if (width > 0 && height > 0) {
						svg = d3.select(el).append("svg")
							.attr("width", svgWidth)
							.attr("height", svgHeight)
							.append("g")
							.attr("transform", "translate(" + margins.left + "," + margins.top + ")");
					}

				}

				function createResizeWatches() {
                    if(!resizeWatcherInitialized) {
                        handleParentWidgetResize(el,function() {
                            renderChart(true,elScope.parentLoading);
                        });
                        resizeWatcherInitialized = true;
                    }

				}

				function destroy() {
					angular.element($window).off('resize', renderChart);
				}

				function createDataWatch() {
					if (watchType === WATCH_TYPE_ARRAY || watchType === WATCH_TYPE_ALL) {
						// watch when chartData array items change
						elScope.$watchCollection(dataVarName, function (newVal, oldVal) {
							if (newVal !== oldVal) {
								renderChart(false,elScope.parentLoading);
							}
						});
					}
					if (watchType === WATCH_TYPE_OBJ) {
						elScope.$watch(dataVarName, function (newVal, oldVal) {
							//TJS:  Commented out check of old against new since it broke the eventing for the decision tree tooltip - might have something to do with ng-repeat
							if (newVal) {
								chartData = newVal;
								renderChart(false,elScope.parentLoading);
							}
						});
					}
					else {
						if (watchType === WATCH_TYPE_ALL) {
							elScope.$watch(dataVarName, function (newVal, oldVal) {
								// In the case where dataVarName points to an array object that doesn't change but only
								// gets updated, newval !== oldVal test always returns false. We need to compare with
								// chartData instead.
								if (newVal && newVal !== chartData) {
									chartData = newVal;
									renderChart(false,elScope.parentLoading);
								}
							});
						}
					}
				}

				function renderChart(resizing,parentLoading) {
					if (!updateRenderFunc || firstRendering || resizing) {
						//Remove SVG
						elSelector.empty();
						//Create SVG
						createSVG();
						if (!svg) {
							return;
						}
                        //Create watches on window and element
                        createResizeWatches();
					}

					//Perform custom rendering through handler function
					/*if (chartData && angular.isArray(chartData) && chartData.length >0)
					 renderFunc(svg,chartData);*/
					if (chartData && height > 0 && width > 0 && (!angular.isArray(chartData) || (angular.isArray(chartData) && chartData.length > 0))) {
						if (firstRendering || !updateRenderFunc || resizing) {
							renderFunc(svg, chartData);
							firstRendering = false;
						}
						else {
							updateRenderFunc(svg, chartData);
						}
					}
					else {
						if (height > 0 && width > 0 && chartData && !parentLoading) {
// FIXME
//	                    $scope.showLoadingMask('Loading distribution chart...');
//		                 loadingMaskSvc.showLoadingMask('Loading distribution chart...');
//	                    var div = dojoModules.put(el, 'div');
//	                    dojoModules.put(div, 'div', 'Click Run to populate Distribution chart');
							//show loading mask
//	                    var mask = dojoModules.put(el,'div.loadingMask div.ajax-loader');
//		                // var loader = dojoModules.put(mask,'');
//			              dojoModules.put(mask,'h4','loading data...');
//	                    dojoModules.put(mask,'img[src=themes/corporate/images/loadingAnimation.gif]');
						}
					}
				}

			},

            handleParentWidgetResize: handleParentWidgetResize,

			splitXAxisLabels: function (svg, splitChar, showSplitChar,formatChar) {
				var insertLinebreaks = function (d) {
					var el = d3.select(this);
					var words = d.split(splitChar);
					el.text('');

					for (var i = 0; i < words.length; i++) {
						var appendChar = '';
						if (showSplitChar) {
							if (i < words.length - 1) {
								appendChar = formatChar ? ' ' + formatChar : ' ' + splitChar;
							}
						}
						var tspan = el.append('tspan').text(words[i].trim() + appendChar);
						if (i > 0) {
							tspan.attr('x', 0).attr('dy', '15');
						}
					}
				};

				svg.selectAll('g.x.axis g text').each(insertLinebreaks);
			},

			wrapTextByFixedWidth: function (svg, el, textParams, splitChar, lineSpacing, ulTooltip) {
				var origText,
					yOffset = textParams.initLineVPad,
					elSelector = d3.select(el),
					workTextSel,
					workTextEl,
					didTruncation = false;
				//If already short enough, do nothing
				if (el.getBBox().width <= textParams.maxWidth) {
					return false;
				}

				//Otherwise, wrap
				origText = el.textContent;
				el.textContent = "";
				var words = origText.split(splitChar),
					wordsCopy = angular.copy(words);

				workTextSel = svg.append("text")
					.attr("x", -100)
					.attr("y", -100)
					.attr("class", "hiddenMeasureText");

				workTextEl = workTextSel.node();

				var line = [];
				var lineCount = 0;
				do {
					var appendChar = '';
					var word = words.shift();
					var textBeforeAdd = workTextEl.textContent;
					line.push(word.trim());
                    workTextEl.textContent = line.join(splitChar);

                    if (workTextEl.getBBox().width > textParams.maxWidth && textBeforeAdd.trim() !== "") {
                        elSelector.append('tspan')
                            .text(textBeforeAdd.trim() + splitChar)
                            .attr('x', textParams.x)
                            .attr('y', yOffset)
                            .attr('text-anchor', textParams.anchor);

                        //Determine if we've exceeded max height - if so, remove added element, tooltip and break out of loop
                        if (el.getBBox().height > textParams.maxHeight) {
                            truncateForHeight();
                            didTruncation = true;
                            break;
                        }

                        //Increment line tracking info
                        line = [];
                        line.push(word.trim());
                        lineCount++;
                        yOffset += lineSpacing;
                        workTextEl.textContent = word.trim();
                    }

                    if (words.length === 0 && !didTruncation) {   // add tspan containing last word
                        elSelector.append('tspan')
                            .text(line.join(splitChar))
                            .attr('x', textParams.x)
                            .attr('y', yOffset)
                            .attr('text-anchor', textParams.anchor);
                        if (el.getBBox().height > textParams.maxHeight) {
                            truncateForHeight();
                        }
                    }

				} while (words.length);

				//Blank out the original text data that isn't in a tspan
				svg.selectAll('.hiddenMeasureText').remove();

				return didTruncation;

				function truncateForHeight() {
					//Get rid of added child
					//el.lastChild.remove();
					//Get new last child and truncate the last entry with dots if it is not the last item in the entire list
                    var lastItem = _.last(origText.split(splitChar)),
                        lastWorkItem = _.last(workTextEl.textContent.split(splitChar));
                    if (lastItem.trim() !== lastWorkItem.trim()) {
                        workTextEl.textContent = el.lastChild.textContent + " ...";
                    }
					if (workTextEl.getBBox().width > textParams.maxWidth) {
						el.lastChild.textContent = _.initial(el.lastChild.textContent.split(splitChar),
							2).join(splitChar) + "...";
					}
					else {
						el.lastChild.textContent = workTextEl.textContent;
					}

					//Make tooltip with original text
					/* var tooltipTxt;
					 if(ulTooltip) {
					 tooltipTxt = '<div class="' + textParams.tooltipClass + '"><ul>';
					 angular.forEach(wordsCopy, function (word) {
					 tooltipTxt += "<li>" + word + "</li>"
					 });
					 tooltipTxt += "</ul></div>";
					 }
					 else {
					 tooltipTxt = '<div class="' + textParams.tooltipClass +  '">' + origText + "</div>";
					 }
					 new SvgTooltip({
					 connectId: el,
					 label: tooltipTxt
					 });*/
				}
			},

			truncateTextWithDots: function (textEl, width, tokenizeFunc, joinChar) {
				var headerPadding = 5,
					paddedWidth = width - headerPadding;
				//If label already fits in area, return
				if (textEl.getBBox().width < paddedWidth) {
					return;
				}
				//Tokenize text
				var origText = textEl.textContent;

				if (!angular.isDefined(joinChar)) {
					joinChar = "_";
				}
				var words = tokenizeFunc ? tokenizeFunc(textEl.textContent) : textEl.textContent.split(/[ _]/);
				var line = [];
				var text = "";
				var word;
				//Loop through text tokens, adding a token at a time and checking length
				do {
					word = words.shift();
					line.push(word);
					if (words.length) {
						textEl.textContent = line.join(joinChar) + joinChar + words[0] + "...";
					}
					else {
						textEl.textContent = line.join(' ') + "...";
					}
					//Check width, see if it exceeds available
					if (textEl.getBBox().width >= paddedWidth) {
						//Set text content to tokens already used plus truncation indicator
						text = line.join(' ');
						textEl.textContent = text + "...";
						//Add tooltip to node so user can get full column name
						new SvgTooltip({
							connectId: textEl.id,
							label:     origText
						});
						return;
					}
				} while (words.length);
			},

			truncateTextWithDots2: function (textEl, maxWidth, tokenizeFunc, joinChar) {

				//If label already fits in area, return
				if (textEl.getBoundingClientRect().width < maxWidth) {
					return;
				}
				//Tokenize text
				var origText = textEl.textContent;

				if (!angular.isDefined(joinChar)) {
					joinChar = "_";
				}
				var words = tokenizeFunc ? tokenizeFunc(textEl.textContent) : textEl.textContent.split(/[ _]/);
				var line = [];
				var text = "";
				var word;
				//Loop through text tokens, adding a token at a time and checking length
				do {
					word = words.shift();
					line.push(word);
					if (words.length) {
						textEl.textContent = line.join(joinChar) + joinChar + words[0] + "...";
					}
					else {
						textEl.textContent = line.join(' ') + "...";
					}
					//Check width, see if it exceeds available
					if (textEl.getBoundingClientRect().width >= maxWidth) {
						//Set text content to tokens already used plus truncation indicator
						text = line.join(joinChar);
						textEl.textContent = text + "...";
						//Add tooltip to node so user can get full column name
						var toolTip = new SvgTooltip({
							connectId: textEl,
							label:     origText
						});
						return;
					}
				} while (words.length);
			},

			getTextDimensions: function (svg, textStr, fontSize) {
				var textEl = svg.append("text")
					.attr("x", 0)
					.attr("y", 0)
					.attr("dy", fontSize + "pt")
					.text(textStr);

				var dims = {
					width:  textEl[0][0].getBBox().width,
					height: textEl[0][0].getBBox().height

				};

				textEl.remove();
				return dims;
			},

			getCorporateColorList:   function () {
				return [
					'#3D5AAE',
					'#90B328',
					'#9C2910',
					'#FFCA39',
					'#00929F',
					'#736519',
					'#F08000',
				   '#A6427C'
				];
			},

			//Needed to display Dojo tooltip for SVG node
			addOffSetWidthAndHeight: function (node) {
				var offset = angular.element(node).offset();

                if($(node).closest('.dontTransposeWidthAndHeight').length == 0){
                    $(node).css('width', offset.left);
                    $(node).css('height', offset.top);
                }
			},

			pctFmtFunc: function (val) {
				return vsdUtil.applySASNumericFormat(val, vsdUtil.defaultFormats.percentage);
			},


			commaFmtFunc: function (val) {
				return vsdUtil.applySASNumericFormat(val, vsdUtil.defaultFormats.number);
			},

            integerFmtFunc: function (val) {
                return vsdUtil.applySASNumericFormat(val, vsdUtil.defaultFormats.integer);
            },

			currencyFmtFunc: function (val) {
				return vsdUtil.applySASNumericFormat(val, vsdUtil.defaultFormats.currency);
			},

			generalFmtFunc: function (val) {
				return vsdUtil.applySASNumericFormat(val, vsdUtil.defaultFormats.none);
			}
		}
	}]);


});

},
'vsd/common/charts/SvgTooltip':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

/**
 * Dojo's Tooltip object does not display in IE or Firefox when attaching SVG elements.
 * This service will add an .offsetWidth and .offsetHeight to the SVG element using jquery.
 * By doing this, dojo will use these values directly in dijit/place.js.
 */
angular.module('vsd')
	.factory('SvgTooltip', [ 'dojoModules', function (dojoModules) {

		return function (opts) {

			var options = opts || {},
				label = opts.label,
				getContent = opts.getContent;

			// This calculates the offset height and width for the Svg element,
			// then calls getContent() or resorts to the default behavior
			function getContentOverride(node) {
				var offset = angular.element(node).offset();

				if (getContent) {
					return getContent(node);
				}
				return label || node.innerHTML;
			}


			options.getContent = getContentOverride;

			new dojoModules.Tooltip(options);
		};
	}]);

});

},
'vsd/common/charts/vsdBoxplotChart':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('vsdBoxplotChart', function (d3UtilsSvc,dojoDepends,vsdUtil) {
        return {
              restrict: 'E',
              template: '<div class="sumStats"></div>',
              replace: true,
              scope: {
                'sngModel': '=',
	              'parentLoading': '='
              },
              link: function(scope, element, attrs,ctrl) {
                  //Create variable on scope for svg
                  var parentElement = element[0],
                      resources = dojoDepends.resources,
                      d3Wrapper;

                  //Create wrapper module to assist in directive execution
                  d3Wrapper = d3UtilsSvc.d3DirectiveWrapper({
                      scope: scope,
	                  elSelector: element,
	                  elName:  "parentElement",
                      dataVarName: "sngModel.stats",
	                  pLoading: "parentLoading",
                      renderFunc: renderChart,
                      clearBeforeRender: true,
	                     parentScope: scope,
                      margins:  {top: 5, left: 100, bottom: 5, right: 60}
                  });
                  //Function to render d3 histogram from provided data
                  function renderChart(svg,chartData) {
                      var me = this,
                          formatComma = function(val) {
                              return vsdUtil.applySASNumericFormat(val,scope.sngModel.format)
                          },
                          width = d3Wrapper.getWidth(),
                          height =  d3Wrapper.getHeight();

                      svg.append("rect")
                          .attr("class", "chartFrame")
                          .attr("width",width )
                          .attr("height",height);

                      var boxWidth = d3Wrapper.getWidth();

                      //Compute quartiles. Must return exactly 3 elements.
                      var quartileData = [chartData.q1, chartData.q2, chartData.q3];
                      //Compute whiskers. Must return exactly 2 elements, or null.
                      var iqr = (chartData.q3 - chartData.q1)  ;
                      var whiskerData = [chartData.q1 - iqr, chartData.q3 + iqr];
                      var yaxisScaleData = [chartData.q1 - (iqr + (iqr *1.5)), chartData.q3 + (iqr +(iqr*1.5))];
                      //Compute the new y-scale.
                      var x1 = d3.scale.linear()
                          .domain(yaxisScaleData)
                          .range([height, 0]);

                      //y-axis
                      var numTicks = (height -10)/15;
                      var yAxis = d3.svg.axis()
                          .scale(x1)
                          .orient("left")
                          .ticks(numTicks)
                          .tickSize(-boxWidth, 0)  //adds grid lines on chart
                          .tickPadding(10)
                          .tickFormat(formatComma);

                      var yAxisG = svg.append("g")
                          .attr("class", "y axis")
                          .call(yAxis);

                      yAxisG.append("text")
                          .attr("transform", "rotate(-90)")
                          .attr("y", chartData.length);

                      var yAxisLines = yAxisG.selectAll("line").style("stroke-dasharray", ("3, 9"));

                      //Whisker Line
                      var center = svg.selectAll("whiskerLine").data( [whiskerData]);

                      center.enter().append("line")
                          .attr("class", "center")
                          .attr("x1", (boxWidth/2 ) )
                          .attr("y1", function(d) {
                              return x1(d[0]); })
                          .attr("x2", (boxWidth/2) )
                          .attr("y2", function(d) {
                              return x1(d[1]); })
                          .style("opacity",0.5);

                      //Update innerquartile box.
                      var box = svg.selectAll("rect.box")
                          .data([quartileData]);

                      box.enter().append("rect")
                          .attr("class", "box")
                          .attr("x", 15)
                          .attr("y", function(d) {
                              return x1(d[2]); })
                          .attr("width",boxWidth-30)
                          .attr("height", function(d) {
                              return x1(d[0]) - x1(d[2]); } );

                      //Update median line.
                      var medianLine = svg.selectAll("line.median").data([quartileData[1]]);
                      medianLine.enter().append("line")
                          .attr("class", "median")
                          .attr("x1", 0)
                          .attr("y1", x1)
                          .attr("x2", boxWidth)
                          .attr("y2", x1) ;

                      //Add Mean circle.
                      var meanDot = svg.selectAll("circle.mean")
                          .data([chartData.mean]);
                      meanDot.enter().append("circle")
                          .attr("class", "mean")
                          .attr("r", 5)
                          .attr("cx", boxWidth/2 )
                          .attr("cy", function(d) {
                              return x1(d);
                          });

                      // Update whiskers with base lines.
                      var whisker = svg.selectAll("line.whisker").data(whiskerData);
                      whisker.enter().append("line")
                          .attr("class", "whisker")
                          .attr("x1", boxWidth/4 + 15)
                          .attr("y1", x1)
                          .attr("x2", boxWidth - boxWidth/4 - 15)
                          .attr("y2", x1);
                  }


             }
        };
  });

});

},
'vsd/common/charts/vsdD3VariableImportanceChart':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('vsdD3VariableImportanceChart', ['d3UtilsSvc', 'vsdUtil',function (d3UtilsSvc,vsdUtil) {
        return {
              restrict: 'E',
              template: '<div class="fullScreen"></div>',
              replace: true,
              scope: {
                'maxBars':             '@',
                'chartData':           '=',
                'varColName':          '@',
                'worthColName':        '@',
                'inputColumnList':     '=',
                'usedColumnList':      '=',
                'removeColumnHandler': '&',
                'resources':           '='
              },
              link: function(scope, element, attrs,ctrl) {
                  //Create variable on scope for svg
                  var parentElement = element[0],
                      d3Wrapper,
                      transDuration = 1000,
                      transEasing = "in-out",
                      margins =  {top: 30, left: 225, bottom: 30, right: 30},
                      nonTextMarginWidth = 25,
                      legendWidth = 130,
                      usageColorScale = d3.scale.ordinal().domain([true,false]).range(_.first(d3UtilsSvc.getCorporateColorList(),2));

                  //Create wrapper module to assist in directive execution
                  d3Wrapper = d3UtilsSvc.d3DirectiveWrapper({
                      scope: scope,
                      elSelector: element,
                      elName:  "parentElement",
                      dataVarName: "chartData",
	                  pLoading: "parentLoading",
                      renderFunc: createChart,
                      updateRenderFunc: updateChart,
                      clearBeforeRender: false,
	                  parentScope: scope,
                      margins:  margins
                  });

                  //Add extra watch to redraw when usedColumnList changes
                  scope.$watch('usedColumnList', function (newVal, oldVal) {
                      if(newVal && scope.inputColumnList && d3Wrapper.getSvg()) {
                          updateChart(d3Wrapper.getSvg(),scope.chartData);
                      }
                  });

                  function updateChart(svg,chartData) {
                      var nonBlankRows,
                          yScale,
                          xScale,
                          join,
                          enterSel,
                          newBars;


                      //Filter out blank rows
                      nonBlankRows = prepareData(chartData);
                      //Create y-scale
                      yScale = createYScale(nonBlankRows);
                      //Create y-axis
                      createYAxis(svg,yScale,nonBlankRows);
                      //Create x-scale
                      xScale = createXScale(nonBlankRows,d3Wrapper.getWidth());
                      //Create x-axis
                      createXAxis(svg,xScale);
                      //Join data with existing bars with id to create enter/update/remove selections
                      join = svg.selectAll(".bar")
                          .data(nonBlankRows,function(d){return d[attrs.varColName];});

                      //Create new bars
                      enterSel = join.enter();
                      newBars = createNewBars(enterSel,nonBlankRows,xScale,yScale);
                      //Adjust attributes on new bars and add transition
                      newBars.attr("y",-yScale.rangeBand())
                          .transition()
                              .duration(transDuration)
                              .ease(transEasing)
                              .attr("y", function(d) { return yScale(d.label); });

                      //Update position and size of existing bars
                      join.attr("width", function(d) { return xScale(d[attrs.worthColName]);})
                          .attr("height", yScale.rangeBand())
                          .transition()
                          .duration(transDuration)
                          .ease(transEasing)
                              .attr("y", function(d) { return yScale(d.label); })
                              .attr("height", yScale.rangeBand());

                      //Animate bars off screen and then remove
                      join.exit().transition()
                          .duration(transDuration)
                          .ease(transEasing)
                            .attr("y",d3Wrapper.getHeight()+yScale.rangeBand())
                            .remove();
                  }

                  function createChart(svg,chartData) {
                      var xScale,
                          yScale,
                          nonBlankRows,
                          enterSel,
                          chartBodyWidth = d3Wrapper.getWidth() - legendWidth,
                          legendX = chartBodyWidth + 15,
                          legendY = d3Wrapper.getHeight()/ 2,
                          legendParams,
                          legendEntries = [
                              scope.resources.variableImportanceChart_includedLbl_txt,
                              scope.resources.variableImportanceChart_excludedLbl_txt
                          ];

                      nonBlankRows = prepareData(chartData);

                      //Chart frame
                      svg.append("rect")
                          .attr("class", "chartFrame")
                          .attr("width", d3Wrapper.getWidth())
                          .attr("height", d3Wrapper.getHeight());

                      //Create y-scale
                      yScale = createYScale(nonBlankRows);
                      //Create y-axis
                      createYAxis(svg,yScale,nonBlankRows);
                      //Create x-scale
                      xScale = createXScale(nonBlankRows,d3Wrapper.getWidth());
                      //Create x-axis
                      createXAxis(svg,xScale);
                      //Create distribution chart bars
                      enterSel = svg.selectAll(".bar")
                          .data(nonBlankRows)
                          .enter();
                      createNewBars(enterSel,nonBlankRows,xScale,yScale);
                  }

                  function createNewBars(enterSel,nonBlankRows,xScale,yScale) {
                      return enterSel.append("rect")
                          .attr("class", "bar")
                          .attr("x", 0)
                          .attr("width", function(d) { return xScale(d[attrs.worthColName]);})
                          .attr("y", function(d) { return yScale(d.label); })
                          .attr("height", yScale.rangeBand())
                          .attr("fill",function(d){
                              return usageColorScale(d.used);
                          })
                          .each(function(d) {
                              var menu = createRMBMenu(d);
                              menu.bindDomNode(this);
                          });
                  }

                  function createXScale(nonBlankRows,width) {
                    return d3.scale.linear()
                        .domain([0, d3.max(nonBlankRows, function(d) { return d[attrs.worthColName]; })])
                        .range([0,width]);

                  }

                  function createXAxis(svg,xScale) {
                      //Remove y-axis if it already exists
                      svg.select("g.x.axis").remove();
                      //Create axis
                      var xAxis = d3.svg.axis()
                        .scale(xScale)
                        .orient("bottom")
                        .ticks(5)
                        .tickSize(-d3Wrapper.getHeight(),0)  //adds grid lines on chart
                        .tickPadding(6)
                        .tickFormat("");  //Specify blank string for format so no axis labels added (these aren't relative to biz users)

                      svg.append("g")
                          .attr("class", "x axis")
                          .attr("transform","translate(0," + d3Wrapper.getHeight() + ")")
                          .call(xAxis);

                  }

                  function createYScale(nonBlankRows) {
                    return d3.scale.ordinal()
                        .domain(nonBlankRows.map(function(d) {
                            return d.label;
                        }))
                        .rangeRoundBands([0, d3Wrapper.getHeight()], 0.1);
                  }

                  function createYAxis(svg,yScale,nonBlankRows) {
                      //Remove y-axis if it already exists
                      svg.select("g.y.axis").remove();
                      //Create axis
                      var yAxis = d3.svg.axis()
                          .scale(yScale)
                          .orient("left")
                          .tickPadding(6);

                      var axisGroup = svg.append("g")
                          .attr("class", "y axis")
                          .call(yAxis);

                      //Select all text in y axis and truncate text that won't fit in the margin
                      axisGroup.selectAll("text")
                          .each(function() {
                              var matchingEntry = _.findWhere(nonBlankRows,{label: this.textContent});
                              var menu = createRMBMenu(matchingEntry);
                              menu.bindDomNode(this);
                              d3UtilsSvc.truncateTextWithDots2(this,margins.left - nonTextMarginWidth,vsdUtil.splitAllChars, "");
                          });
                  }

                  function prepareData(chartData) {
                      //Reject rows with now worth value
                      var filteredRows =  _.reject(chartData,function(row) {
                          return !row[attrs.worthColName];
                      });
                      //For each remaining row, add a label property and a used property
                      angular.forEach(filteredRows, function (row) {
                          var matchCol = _.find(scope.inputColumnList, function(curr) {
                              return curr.name === row[attrs.varColName];
                          });
                          row.label = matchCol ? matchCol.label : row[attrs.varColName];
                          row.used = _.contains(scope.usedColumnList,row[attrs.varColName]);
                      });


                      filteredRows.sort(function(a,b) {
                          var fieldA = a[attrs.worthColName],
                              fieldB = b[attrs.worthColName];
                          if(fieldA > fieldB) {
                              return -1;
                          }
                          else if(fieldA < fieldB) {
                              return 1;
                          }
                          else {
                              return 0;
                          }
                      });
                      if(attrs.maxBars) {
                          return _.first(filteredRows,parseInt(scope.maxBars));
                      }
                      else {
                          return filteredRows;
                      }
                  }

                  function createRMBMenu(objData) {
                      var items = [];

                      items.push({
                          label: scope.resources.variableImportanceChart_rmbActionRemove,
                          onClick: function() {
                              scope.removeColumnHandler({colName: objData[attrs.varColName]});
                              scope.$apply();
                          }
                      });
                      return vsdUtil.menuFromItemsArray(items);
                  }

                  function createVerticalLegend(contEl,entries,params) {
                      //Default entry member of params if it's not there to avoid NPEs
                      params.entry = params.entry || {};

                      var legendHPad = params.hpad || 10,
                          legendVPad = params.vPad || 10,
                          barWidth = params.entry.barWidth || 40,
                          barHeight = params.entry.barWidth || 20,
                          textMargin = params.entry.textMargin || 5,
                          entryVMargin = params.entry.vMargin || 10,
                          entryTextWidth = params.entry.textMargin || 60,
                          entryWidth =  barWidth + entryTextWidth + textMargin,
                          entryHeight = params.entry.h || 30,
                          entryHMargin = params.entry.hMargin || 15,
                          height = entries.length * entryHeight  + legendVPad,
                          width = entryWidth + legendHPad*2;


                      var legendBox = contEl.append("g")
                          .attr("height",height)
                          .attr("width", width)
                          .attr("transform", "translate(" + params.x + "," + (params.y - height/2) + ")");

                      //Append box to g for legend background
                      legendBox.append("rect")
                          .attr("height",height)
                          .attr("width", width)
                          .attr("class","legendBox");

                      //Append key entries
                      var entryNbr = 0,
                          rowNbr = 0,
                          colNbr = 0;
                      angular.forEach(entries,function(entry) {
                          var boxXPos = legendHPad + colNbr*entryWidth + entryHMargin*colNbr,
                              boxYPos = (entryVMargin + entryHeight) * rowNbr;

                          if(rowNbr === 0) {
                              boxYPos += legendVPad;
                          }
                          //Create colored rectangle for level
                          legendBox.append("rect")
                              .attr("class","legendComponent")
                              .attr("x",boxXPos)
                              .attr("y",boxYPos)
                              .attr("width", barWidth)
                              .attr("height",barHeight)
                              .style("fill",function() {
                                  return params.colorScale(entry);
                              });
                          //Create label for level
                          legendBox.append("text")
                              .attr("class","legendComponent")
                              .attr("x",function() {
                                  return boxXPos + barWidth + textMargin;
                              })
                              .attr("y",function() {
                                  return boxYPos + barHeight/2 - 5;
                              })
                              .attr("dy", "10px")
                              .text(entry);


                          entryNbr++;
                          colNbr++;
                          //Increment rowNbr and reset column number each iteration for now since only doing vertical
                          rowNbr++;
                          colNbr = 0;
                      });
                  }

             }
        };
  }]);

});

},
'vsd/common/dialogs/ConfirmDialogCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('ConfirmDialogCtrl', ['$scope',
       function ($scope) {
          //Private variables
          var confirmCallback,
             cancelCallback,
	          closeCallback;

          //region *** Scope Data/Functions ***
          $scope.data = {
             title:        null,
             msgTxt:       null,
             confirmLabel: null,
             cancelLabel:  null,
	          showConfirmationDialog: false
          };

          $scope.onConfirm = function () {
             if (confirmCallback) {
                confirmCallback();
             }
             hideDialog();
          };

          $scope.onCancel = function () {
             if (cancelCallback) {
                cancelCallback();
             }
             hideDialog();
          };

          function hideDialog() {
             $scope.data.showConfirmationDialog = false;
          }

	       // Handles the escape key and clicking the x icon
	       $scope.onClose = function(){
		       if(closeCallback){
			       closeCallback();
		       }
		       hideDialog();
	       };

          $scope.$on('ConfirmDialogCtrl.showDialog', function (evt, options) {
             $scope.data.title = options.title;
             $scope.data.msgTxt = options.msg;
             $scope.data.confirmLabel = options.confirmLabel;
             $scope.data.cancelLabel = options.cancelLabel;
             confirmCallback = options.confirmCallback;
             cancelCallback = options.cancelCallback;
	          closeCallback = options.closeCallback;
	          $scope.data.showConfirmationDialog = true;
          });
	                                     //endregion

}])
;


});

},
'vsd/common/dialogs/confirmDialogSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('confirmDialogSvc', ['$rootScope', function ($rootScope) {
		var options = {};
		function reset(){
			options = {
				confirmLabel:    null,
				confirmCallback: null,
				cancelLabel:     null,
				cancelCallback:  null,
				closeCallback:   null,
				title:           null,
				msg:             null
			};
		}
		reset();

		return {
			setConfirmLabel:    function (l) {
				options.confirmLabel = l;
				return this;
			},
			getConfirmLabel:    function () {
				return options.confirmLabel;
			},
			setConfirmCallback: function (c) {
				options.confirmCallback = c;
				return this;
			},
			getConfirmCallback: function () {
				return options.confirmCallback;
			},

			setCancelLabel:    function (l) {
				options.cancelLabel = l;
				return this;
			},
			getCancelLabel:    function () {
				return options.cancelLabel;
			},
			setCancelCallback: function (c) {
				options.cancelCallback = c;
				return this;
			},
			getCancelCallback: function () {
				return options.cancelCallback;
			},
			setCloseCallback: function (c) {
				options.closeCallback = c;
				return this;
			},
			getCloseCallback: function () {
				return options.closeCallback;
			},
			setTitle:          function (t) {
				options.title = t;
				return this;
			},
			getTitle:          function () {
				return options.title;
			},
			setMsg:            function (m) {
				options.msg = m;
				return this;
			},
			getMsg:            function () {
				return options.msg;
			},
			showDialog:        function () {
				$rootScope.$broadcast('ConfirmDialogCtrl.showDialog', options);
				reset();

			},
			reset: reset
		};
	}]);

});

},
'vsd/common/dialogs/OpenDialogCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
	controller('OpenDialogCtrl', ['$scope','dojoDepends','objectBrowseSvc', '$cookies',
        function ($scope, dojoDepends,objectBrowseSvc, $cookies) {

            var showAllCookieName = 'vsdShowAllOpenItems';

            function loadObjectList() {
                $scope.showLoadingMask($scope.resources.main_loadingAvailFiles_msg_txt);

               // using jQuery here is not ideal but there is a timing issue using ngModel
               $cookies[showAllCookieName] = "" + $('#showAllOpenItems').is(':checked');
               if ($('#showAllOpenItems').is(':checked')) {
                  objectBrowseSvc.getBrowseAll()
                     .then(function (data) {
                        $scope.data.gridCols = objectBrowseSvc.getHierarchicalColDefs();
                        $scope.data.objectList = data;
                        $scope.hideLoadingMask();
                     });
               }
               else {
                  objectBrowseSvc.getBrowseUser()
                     .then(function (data) {
                        $scope.data.gridCols = objectBrowseSvc.getHierarchicalColDefs();
                        $scope.data.objectList = data;
                        $scope.hideLoadingMask();
                     });
               }
            }

            function resetForm() {
                $scope.data.objectList =  [];
                $scope.data.objectIdProperty =  'id';
                $scope.data.gridCols =  null;
                $scope.data.objectDeletable =  false;
                $scope.data.searchTxt =  "";
                $scope.data.selectedItem =  null;
            }

            $scope.data = {
               showAll: ($cookies[showAllCookieName] === 'true')
            };
            $scope.objectListGetChildren = objectBrowseSvc.objectListGetChildren;
            $scope.objectListMayHaveChildren = objectBrowseSvc.objectListMayHaveChildren;
            $scope.objectListFilterQuery = function(item, index, items) {
                return objectBrowseSvc.objectListFilterQuery(item, index, items,$scope.data.searchTxt);
            };
            $scope.onRowSelection = function(row) {
                $scope.data.selectedItem = row;
                $scope.data.objectDeletable = objectBrowseSvc.objectDeletable(row);
            };

            $scope.deleteItem = function(item) {
                objectBrowseSvc.deleteObject($scope.data.selectedItem)
                    .then(function() {
                        $scope.refreshView();
                        $scope.data.selectedItem = null;
                    });
            };

            $scope.refreshView = function() {
                loadObjectList();
            };

            $scope.openObject = function () {
                objectBrowseSvc.openObject($scope.data.selectedItem);
                $scope.hideDialog();
            };

            $scope.hideDialog = function () {
                $scope.dialogFlags.showOpenDialog = false;
            };

            $scope.$watch('dialogFlags.showOpenDialog',
                function () {
                    if ($scope.dialogFlags.showOpenDialog) {
                        $scope.data.showAll = ($cookies[showAllCookieName] === 'true');
                        resetForm();
                        loadObjectList();
                    }
                });
            //Set initial values for scope variables

            resetForm();
	}])
;


});

},
'vsd/common/loadingMask/loadingMaskSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').factory('loadingMaskSvc',
	['$rootScope', function ($rootScope) {
//		var loadingMask = {show: false, message: ''};
		return {
			showLoadingMask: function (message) {
				// FIXME
//				var dl = $('.dijitDialog:visible').length;
				$('.loadingMask h4').text(message);
				$('.loadingMask').show();
//				$rootScope.loadingMask.message = message;
//				$rootScope.loadingMask.show = true;
			},
			hideLoadingMask: function () {
				$('.loadingMask h4').text('');
				$('.loadingMask').hide();

//				$rootScope.loadingMask.show = false;
			}
		};
	}
	]);


});

},
'vsd/common/message/messageSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

// TODO: should this service be named notificationSvc?
angular.module('vsd')
	.factory('messageSvc',
		['dojoModules',
		 function (dojoModules) {
			 var Messages = new dojoModules.VSDMessages();
			 return {
				 showError:   function (message) {
					 Messages.showError(message);
				 },
				 showInfo:    function (message) {
					 Messages.showInfo(message);
				 },
				 showStatus:  function (message) {
					 Messages.showStatus(message);
				 },
//				 showWarning: function (message) {
//					 Messages.showWarning(message);
//				 },
                 closeMessage: function() {
                     Messages.closeMessage();
                 }
			 };
		 }
		]);

});

},
'vsd/common/ng/directives/conditionBuilder/conditionBuilder':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .directive('conditionBuilder', [ '$compile', 'dojoModules', 'vsdUtil', 'operatorSvc' ,
        function ($compile, dojoModules, vsdUtil, operatorSvc) {
            return {
                //template: '<div ><span ng-show="isCondition">{{ condition.operand1ColumnInfo.name  }}</span> <span ng-show="isCondition">{{ condition.operator.name  }}</span> <span ng-show="isCondition">{{ condition.operand2Constant  }}</span></div>',
                templateUrl: 'vsd/common/ng/directives/conditionBuilder/conditionBuilder.html',
                replace: true,
                restrict: 'E',
                scope: {
                    'condition': '=',
                    'conditionPos': '=',
                    'lhsColumns': '=',
                    'lhsLabel': '@',
                    'operLabel': '@',
                    'operators': '=',
                    'fullEdit': '@',
                    'showRowButtons': '@',
                    'immediateUpdate': '@',
                    'addButtonClick': '=',
                    'deleteButtonClick': '=',
                    'nestButtonClick': '=',
                    'skipValidation': '@',
                    'readOnly': '@',
                    'addMissingSpace': '=',
                    'broadcastChange': '=',
                    'broadcastId': '@'
                },
                controller: ['$rootScope', '$scope', '$element', '$attrs', function ($rootScope, $scope, $element, $attrs) {
//          var validateRHSMissing = $attrs.validateRhsMissing === 'false' ? false : true;

                    $scope.$watch('condition.operand2Constant', function (value) {
                        $scope.condition.operand2ConstantView = value || '';
                        var condDetails = getConditionDetails();
                        if ($scope.condition.operand1ColumnInfo.dataType === 'NUMERIC' && !condDetails.missingOperator) {
                            if (condDetails.isDate) {
                                $scope.condition.operand2ConstantView = vsdUtil.getFormattedDateString(value);
                            }
                            else if (condDetails.isDateTime) {
                                $scope.condition.operand2ConstantView = vsdUtil.getFormattedDateTimeString(value);
                            }
                            else {
                                $scope.condition.operand2ConstantView = vsdUtil.getFormattedNumberString(value);
                            }
                        }
                    });

                    var me = this;

                    this.getOperators = function (cond) {
                        return setOperatorList(cond);
                    };

                    function getRHSValidationErrMsg(reason) {
                        var msg;
                        switch (reason) {
                            case  vsmCommonConst.EXPR_VAL_REASON_INVALID_DATE_CONST:
                                msg = $scope.resources.scenario_conditionEditor_invalidDateFormatErrMsg_txt;
                                break;
                            case  vsmCommonConst.EXPR_VAL_REASON_INVALID_DATETIME_CONST:
                                msg = $scope.resources.scenario_conditionEditor_invalidDateTimeFormatErrMsg_txt;
                                break;
                            case vsmCommonConst.EXPR_VAL_REASON_NON_NUMERIC:
                                msg = $scope.resources.scenario_conditionEditor_nonNumericContentErrMsg_txt;
                                break;
                            case vsmCommonConst.EXPR_VAL_REASON_NULL:
                                msg = $scope.resources.scenario_conditionEditor_nullErrMsg_text;
                                break;
                            case vsmCommonConst.EXPR_VAL_REASON_DBL_QUOTES:
                                msg = $scope.resources.scenario_conditionEditor_dblQuotes_text;
                                break;
                            case vsmCommonConst.EXPR_VAL_REASON_LITERAL_MAXLEN:
                                msg = $scope.resources.scenario_conditionEditor_exceedsLiteralMaxLen_text;
                                break;
                            case vsmCommonConst.EXPR_VAL_REASON_TOTAL_MAXLEN:
                                msg = $scope.resources.scenario_conditionEditor_exceedsOperand2MaxLen_text;
                                break;
                        }
                        return msg;
                    }


                    function setOperatorList(condCol) {
                        var columnType = condCol.dataType == vsmCommonConst.DATA_TYPE_CHAR ? "CHAR" : "NUM";
                        return operatorSvc.getLogicalOperators(columnType).then(function (data) {
                            if (data.length > 0) {
                                //$scope.operatorMap = data;
                                $scope.boundPrimitives.operList = data;
                                $scope.boundPrimitives.defaultOperValue = _.findWhere(data, {name: '='});
                            }
                        });
                    }

                    function getConditionDetails() {
                        var details = {
                            hasLhsColumn: $scope.condition.operand1ColumnInfo && $scope.condition.operand1ColumnInfo.name,
                            hasRhsValue: !!$scope.condition.operand2Constant,
                            hasOperator: !!$scope.condition.operator,
                            missingOperator: ($scope.condition.operator.name.indexOf("MISSING") > -1),
                            isDateTime: ($scope.condition.operand1ColumnInfo.sasDateTime ||
                                vsdUtil.startsWith($scope.condition.operand1ColumnInfo.columnFormat, ["DATETIME"]) )
                        };
                        details.isDate = ($scope.condition.operand1ColumnInfo.sasDate ||
                            (!details.isDateTime && vsdUtil.startsWith($scope.condition.operand1ColumnInfo.columnFormat, ["DATE"])) );
                        return details;
                    }

                    function broadcastChangeEvt() {
                        if ($scope.broadcastId) {
                            $scope.$emit(vsmCommonConst.CONDITION_BUILDER_CHANGE, $scope.broadcastId);
                        }
                        else {
                            $scope.$emit(vsmCommonConst.CONDITION_BUILDER_CHANGE);
                        }
                    }

                    $scope.resources = dojoModules.resources;
                    $scope.COND_PART_LHS = 'LHS';
                    $scope.COND_PART_OPER = 'OPER';
                    $scope.COND_PART_RHS = 'RHS';
                    $scope.COND_TEXT_BOX = 'CB_TEXT_BOX';
                    $scope.boundPrimitives = {};

                    $scope.boundPrimitives.lhsLabel = ($attrs.lhsLabel) ? $attrs.lhsLabel : false;
                    $scope.boundPrimitives.operLabel = ($attrs.operLabel) ? $attrs.operLabel : false;
                    $scope.boundPrimitives.immediateChanges = ($attrs.immediateUpdate) ? $attrs.immediateUpdate : false;
                    $scope.boundPrimitives.readOnly = ($attrs.readOnly) ? $attrs.readOnly : false;

                    if ($attrs.skipValidation == 'true') {
                        $scope.boundPrimitives.doValidation = false;
                    }
                    else {
                        $scope.boundPrimitives.doValidation = true;
                    }


                    $scope.rhsValidator = function (value) {
                        var condDetails = getConditionDetails();
                        if (condDetails.hasLhsColumn && condDetails.hasOperator && !condDetails.missingOperator) {
                            var valRet = vsdUtil.valExprConstByDataType(value, $scope.condition.operand1ColumnInfo, $scope.condition.operator, false, true);
                            var invalidMessage = "";
                            if (!valRet.isValid) {
                                if (valRet.reason === vsmCommonConst.EXPR_VAL_REASON_NULL && !$scope.boundPrimitives.doValidation) {
                                    $scope.condition.operand2Constant = value;
                                    return {isValid: true};
                                }
                                else {
                                    invalidMessage = getRHSValidationErrMsg(valRet.reason);
                                }
                            }
                            else {  // store the valid value
                                $scope.condition.operand2Constant = '' + valRet.parsedValue;
                            }
                            return { isValid: valRet.isValid, invalidMessage: invalidMessage };
                        }
                        else {
                            return {isValid: true};
                        }
                    };

                    $scope.onOperChange = function () {
                        var condDetails = getConditionDetails(),
                            operand1ColumnInfo = $scope.condition.operand1ColumnInfo,
                            operand2Constant = $scope.condition.operand2Constant,
                            operator = $scope.condition.operator,
                            valRes = true,
                            conditionReady = false;

                        if (condDetails.missingOperator) {
                            $scope.condition.operand2Constant = '';
                            $scope.missingFlag = true;
                            if ($scope.boundPrimitives.broadcastChange && condDetails.hasLhsColumn) {
                                broadcastChangeEvt();
                            }
                        }
                        else {
                            $scope.missingFlag = false;
                            $scope.condition.operand2Constant = $scope.condition.operand2Constant || "";

                            if (condDetails.hasLhsColumn) {
                                valRes = vsdUtil.valExprConstByDataType(operand2Constant, operand1ColumnInfo, operator, false, true).isValid;
                            }
                            conditionReady = condDetails.hasLhsColumn && condDetails.hasRhsValue && valRes;

                            if ($scope.boundPrimitives.broadcastChange) {
                                if (($attrs.lhsLabel && $attrs.operLabel) || (conditionReady)) {
                                    broadcastChangeEvt();
                                }
                            }
                        }
                    };

                    $scope.onConditionLhsChange = function () {

                        me.getOperators($scope.condition.operand1ColumnInfo).then(function () {
                            $scope.condition.operator = _.findWhere($scope.boundPrimitives.operList,
                                {name: $scope.condition.operator.name});
                            if (!$scope.condition.operator) {
                                $scope.condition.operator = $scope.boundPrimitives.defaultOperValue;
                            }

                            var condDetails = getConditionDetails(),
                                operand1ColumnInfo = $scope.condition.operand1ColumnInfo,
                                operand2Constant = $scope.condition.operand2Constant,
                                operator = $scope.condition.operator,
                                valRes = true,
                                conditionReady = false;

                            if (condDetails.hasLhsColumn && !condDetails.missingOperator && condDetails.hasRhsValue) {
                                valRes = vsdUtil.valExprConstByDataType(operand2Constant, operand1ColumnInfo, operator, false, true).isValid;
                            }

                            conditionReady = condDetails.hasLhsColumn && (condDetails.missingOperator || condDetails.hasRhsValue) && valRes;

                            $scope.condition.isValid = valRes; // TODO: remove?

                            if (!valRes) {
                                $scope.condition.operand2Constant = '';
                                $scope.$emit(vsmCommonConst.CONDITION_BUILDER_ERROR);
                                return;
                            }

                            //If haven't exited with error or because rhs edit only mode, see if rest of condition populated and then do change event
                            if ($scope.boundPrimitives.broadcastChange) {
                                if (conditionReady) {
                                    broadcastChangeEvt();
                                }
                            }
                        });
                    };

                    $scope.showWidget = function(widgetId){
                        if($scope.$root.showWidgetMap) {
                            $scope.$root.showWidgetMap[widgetId] = true;
                        }
                    }

                    $scope.getShowWidgetClass = function(){

                        if(!($scope.$root.showWidgetMap)){
                            return false;
                        }

                        return !($scope.boundPrimitives.readOnly || $scope.boundPrimitives.isReadOnly) ;
                    }

                    $scope.shouldShowWidget = function(){

                        if($scope.$root.showWidgetMap) {

                            if ($scope.boundPrimitives.isReadOnly || $scope.boundPrimitives.readOnly) {
                                return false;
                            }

                            var val = false;

                            try {
                                val = $scope.$root.showWidgetMap['condition_id_' + $scope.condition.id];
                            } catch (err) {
                                console.log(err);
                            }
                            var boxId = '#condition_id_' + $scope.condition.id + '_text_box';

                            val = val ? val : false;
                            return val;
                        }
                        else{
                            return !($scope.boundPrimitives.readOnly);
                        }
                    }

                    $scope.onConditionRhsChange = function () {

                        var condDetails = getConditionDetails(),
                            operand1ColumnInfo = $scope.condition.operand1ColumnInfo,
                            operand2Constant = $scope.condition.operand2Constant,
                            operator = $scope.condition.operator,
                            valRes = true,
                            conditionReady = false;


                        if (condDetails.hasLhsColumn) {
                            valRes = vsdUtil.valExprConstByDataType(operand2Constant, operand1ColumnInfo, operator, false, false).isValid;
                        }

                        conditionReady = condDetails.hasLhsColumn && condDetails.hasRhsValue && valRes;

                        $scope.condition.isValid = valRes; // TODO: remove?

                        if (!valRes) {
                            $scope.$emit(vsmCommonConst.CONDITION_BUILDER_ERROR);
                            return;
                        }

                        if ($scope.boundPrimitives.broadcastChange) {
                            if (($attrs.lhsLabel && $attrs.operLabel) || (conditionReady)) {
                                broadcastChangeEvt();
                            }
                        }
                    };

                }],

                link: function (scope, element, attrs, controller) {

                    scope.boundPrimitives.broadcastChange = (scope.broadcastChange == false) ? false : true;

                    scope.missingFlag = false;

                    if (scope.condition.operator.name.indexOf('MISSING') > -1) {
                        scope.missingFlag = true;
                    }

                    controller.getOperators(scope.condition.operand1ColumnInfo).then(function () {
                        scope.condition.operator = _.findWhere(scope.boundPrimitives.operList,
                            {name: scope.condition.operator.name});
                        if (!scope.condition.operator) {
                            scope.condition.operator = scope.boundPrimitives.defaultOperValue;
                        }

                    });

                    scope.$watch('condition', function (newVal, oldVal) {
                        if (newVal && newVal.operator.name.indexOf('MISSING') == -1) {
                            scope.missingFlag = false;
                        }
                    });

                }
            };
        }]);

});

},
'vsd/common/ng/directives/conditionBuilder/conditionBuilderHierarchy':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('conditionBuilderHierarchy', ['dojoModules', 'operatorSvc', '$compile','messageSvc',
      function (dojoModules, operatorSvc, $compile, messageSvc) {

		function createTemplate(resources) {
			var buttonProps = function(iconCls, bnLabel){
				var lbl = bnLabel.replace(/'/g,"&apos;");
				var str =  "iconClass: &quot;" + iconCls + "&quot;, label: &quot;" + lbl + "&quot;, showLabel: false" ;
				return str;

				//return  "iconClass: &quot;" + iconCls + "&quot;, label: &quot;" + bnLabel + "&quot;, showLabel: false" ;
			}
			//Cant use templateUrl with recursive directives - there is a bug
			var template= "<li ng-switch on='isOperator(condition)'> " +
				"<div ng-switch-when='true' class='oper' >" +
				'  <div ng-if="boundPrimitives.editRhsOnly || boundPrimitives.readOnly" class="conditionNest" >  ' +
				"     <div class='nestConnector'>{{condition.operator.label}} </div>  " +
				"  </div> " +
				'  <div ng-if="!(boundPrimitives.editRhsOnly || boundPrimitives.readOnly)">' +
				"     <sng-list-button  selected-item='condition.operator' "+
				"                          list='boundPrimitives.operatorList' " +
				"                          list-type='object'" +
				"                          sng-disabled='isNotAggregated' " +
				"                          label-field='label'" +
				'                          change-handler="onOperChange()">' +
				"     </sng-list-button>  " +
				"  </div> " +
				"</div> " +

				"<div ng-switch-when='false' class='condition'>  " +
				'  <div ng-if="boundPrimitives.editRhsOnly" class="conditionNest" >  ' +
				"     <condition-builder condition='condition'  oper-label='true' lhs-label='true'  broadcast-change='broadcastChange' add-missing-space='true'> " +
				"     </condition-builder> " +
				"  </div> " +
				'  <div ng-if="boundPrimitives.readOnly" class="conditionNest" >  ' +
				"     <condition-builder condition='condition'  oper-label='true' lhs-label='true' read-only='true'  broadcast-change='broadcastChange'" +
				"        add-missing-space='true' > </condition-builder> " +
				"  </div> " +
				'  <div ng-if="!(boundPrimitives.editRhsOnly || boundPrimitives.readOnly)" class="conditionNest" >  ' +
				"     <condition-builder condition='condition' lhs-columns='lhsColumns'  broadcast-change='broadcastChange' add-missing-space='true'> </condition-builder> " +
				"     <div style='display: inline-block'>  <span sng-dijit='dijit/form/Button' sng-click='addCond(conditionPos)' data-dojo-props='" +
								buttonProps('sasAddIcon',resources.scWindow_AggBuilder_addFilterRowBtn_txt )  + "' ></span> " +
				"        <span sng-dijit='dijit/form/Button' sng-click='deleteCond(conditionPos)' data-dojo-props='" +
								buttonProps('sasDeleteIcon',resources.scWindow_AggBuilder_deleteFilterBtn_txt )  + "' ></span> " +
				"        <span sng-dijit='dijit/form/Button' sng-click='addNestedCond(condition)' data-dojo-props='" +
								buttonProps('sasNestRowsIcon',resources.scWindow_AggBuilder_addNestFilterBtn_txt )  + "' ></span> " +
				"     </div>" +
				"  </div>" +
				"</div> </li> ";

			return template;

		}

    return {

	    restrict: 'E',
	    replace: true,
	    require: '^conditionPackage',
	    scope: {
				    "condition":        "=",
				    "lhsColumns":    '=',
				    "rootCondition":    '=',
				    "conditionPos":  '=',
				    "parentPos": '=',
				    "rootHier":   "@",
				    "editRhsOnly": '=',
				    "readOnly": '=',
		          "broadcastChange": "="
        },
	     controller: ['$scope','$element','$attrs',function($scope, $element, $attrs ) {
		     $scope.resources = dojoModules.resources;

		     operatorSvc.getOperators().then(function(data) {
			     $scope.operatorMap = data;
			     $scope.defaultNestOper =  _.findWhere($scope.operatorMap, {name: 'OR'});
              popupulateConjunctionList();

			     $scope.condition.operator = _.findWhere($scope.operatorMap,{name: $scope.condition.operator.name});
			     if (!$scope.condition.operator) {
				     $scope.condition.operator = $scope.defaultNestOper;
			     }
		     });

		     $scope.boundPrimitives={
			     editRhsOnly: false,
			     readOnly: false,
			     operatorList: []
		     };
		     $scope.boundPrimitives.readOnly = ($scope.readOnly) ? $scope.readOnly : $scope.boundPrimitives.readOnly ;
		     if ($scope.boundPrimitives.readOnly) {  // readOnly needs to override editRhsOnly
			     $scope.boundPrimitives.editRhsOnly = false;
		     }
		     else {
			     $scope.boundPrimitives.editRhsOnly = ($scope.editRhsOnly) ? $scope.editRhsOnly : $scope.boundPrimitives.editRhsOnly ;
		     }

		     $scope.isOperator = function(condition) {
			      return (condition && condition.childConditions && condition.childConditions.length > 0);
		      };

            //Watch childConditions so that if children > 1, NOT is removed from conjunction operator list
            $scope.$watchCollection('condition.childConditions',function() {
	            var condition = $scope.condition || {},
		             operator = condition.operator || {},
		             childConditions = condition.childConditions || [];

                //If user chooses not and there is more than one child condition
                if($scope.condition && operator.name === "NOT" && childConditions.length > 1) {
                    $scope.condition.operator = _.findWhere($scope.operatorMap,{name: "AND"});
                    $scope.$apply();
                    //Show message informing user of issue and noting that operator defaulted to 'AND'
                    var message = dojoModules.string.substitute($scope.resources.scenario_conditionEditor_invalidNotUsageMsg_txt,
                        [$scope.resources.common_operators_not_txt,
                         $scope.resources.common_operators_and_txt,
                         $scope.resources.common_operators_or_txt,
                         $scope.resources.common_operators_and_txt]);
                    messageSvc.showError(message);
                }

                popupulateConjunctionList();
            });

            $scope.onOperChange = function() {
                 $scope.$emit(vsmCommonConst.CONDITION_BUILDER_OPER_CHANGE);
            }

            function popupulateConjunctionList() {
                 var condition = $scope.condition || {},
                     childConditions = condition.childConditions || [];

                if(!$scope.operatorMap || $scope.operatorMap.length == 0) {
                    return;
                }

                 $scope.boundPrimitives.operatorList = [];
                 angular.forEach(['AND', 'OR', 'NOT'], function(key) {
                     if(key !== 'NOT' || (childConditions.length === 1)) { //Only allow NOT If there  is a single child
                         var currOper = angular.copy(_.findWhere($scope.operatorMap,{name: key}));
                         $scope.boundPrimitives.operatorList.push(currOper);
                     }
                 });
            }
	    }],

	    link: function (scope, element, attrs) {
		    var me = scope.condition;
		    scope.condition.displayOrder = scope.$parent.$index;
		    if (scope.rootCondition) {
			    for (var ii = 0; ii < scope.rootCondition.childConditions.length; ii++) {
				    scope.rootCondition.childConditions[ii].displayOrder = ii;
			    }
		    }
		    scope.resources = dojoModules.resources;

		    scope.rootHier = (attrs.rootHier) ? true : false;
		    scope.broadcastChange = (scope.broadcastChange == false) ? false: true;
		    element.append(createTemplate(scope.resources));
		    $compile(element.contents())(scope);

		    //have to verify condition exits since user may have deleted an existing one
		    if (scope.condition && scope.condition.childConditions && scope.condition.childConditions.length > 0) {

			    scope.parentCondition= scope.condition;

			    var template=  '<ul class="conditionHeir">' +
				         '<condition-builder-hierarchy ng-repeat="condition in condition.childConditions" condition="condition" ' +
				            'broadcast-change="broadcastChange" add-missing-space="true"  ' ;

			    if ( scope.boundPrimitives.editRhsOnly)  {
				    template +=  'edit-rhs-only="true" root-condition="parentCondition" condition-pos="$index" parent-pos="conditionPos">' +
				            '</condition-builder-hierarchy></ul>';
			    }
			    else if (scope.boundPrimitives.readOnly) {
				    template +=  '  read-only="true" root-condition="parentCondition" condition-pos="$index" parent-pos="conditionPos">' +
					         '</condition-builder-hierarchy></ul>';
			    }
			    else {
				    template +=  '  lhs-columns="lhsColumns" root-condition="parentCondition" condition-pos="$index" parent-pos="conditionPos">' +
				            '</condition-builder-hierarchy></ul>';
			    }

			    var childChoice = $compile(template)(scope);
			    element.append(childChoice);
		    }

		    scope.$watch('condition', function(value) {
			    if (value == null)
			      scope.$parent.$parent.condition = null;
		    });

		    scope.addCond = function(idx) {
			    var defaultOper = _.findWhere(scope.operatorMap, {name: '='});
			    var newCondition= { operator: defaultOper,
				    operand1ColumnInfo: {name: '', label: ''},
				    operand2Constant: "", childConditions: []};
			    scope.$parent.$parent.condition.childConditions.splice(idx+1,0,newCondition);
		    };

		    scope.deleteCond = function(idx) {
			    // is the condition the last condition delete condition hierarchy all together
			    if ( scope.$parent.$parent.rootHier && scope.rootCondition.childConditions.length == 1)  {
				    scope.rootCondition = null;
				    scope.$parent.$parent.condition = null;
				    scope.$apply();
			    }
			    else if (scope.rootCondition.childConditions.length == 2 &&
				      (scope.rootCondition.childConditions[idx ? 0 : 1].operator.name === 'OR' ||
					      scope.rootCondition.childConditions[idx ? 0 : 1].operator.name === 'AND' ||
					      scope.rootCondition.childConditions[idx ? 0 : 1].operator.name === 'NOT')){
				    var hier = scope.rootCondition.childConditions[idx ? 0 : 1];
				    scope.rootCondition.operator = hier.operator;
				    scope.rootCondition.childConditions.length = 0;
				    angular.forEach(hier.childConditions, function(child){
					    scope.rootCondition.childConditions.push(child);
				    });
			    }
			    else if (scope.rootCondition.childConditions.length == 1)
				    scope.$parent.$parent.rootCondition.childConditions.splice(scope.parentPos,1); //nested level
			    else
				    scope.rootCondition.childConditions.splice(scope.conditionPos,1);


				  /*if (scope.conditionPos == 0 && scope.rootCondition) {
					  if ( scope.parentPos > 0 )
					      scope.$parent.$parent.rootCondition.childConditions.splice(scope.parentPos,1); //nested level
					  else  {
						  scope.rootCondition = null;
						  scope.$parent.$parent.condition = null;
						  scope.$apply();
					  }
				  }
				  else
					   scope.rootCondition.childConditions.splice(scope.conditionPos,1);*/

              //Dispatch event to let clients know condition deleted
              scope.$emit(vsmCommonConst.CONDITION_BUILDER_DELETE,[idx]);

		    };

		    scope.addNestedCond = function(cond) {
			    var defaultCondOper = _.findWhere(scope.operatorMap, {name: '='});
			    var newCondition= { operator: scope.defaultNestOper,
				    operand1ColumnInfo: {},  operand2Constant: null,
				    childConditions: [ { operator: defaultCondOper,
					                      operand1ColumnInfo: {name: '', label: ''},
					                      operand2Constant: "", childConditions: []} ]
			    };

			    scope.$parent.$parent.condition.childConditions.splice(scope.$parent.$index+1,0,newCondition);

		    };

	    }
    };
  }]);

});

},
'vsd/common/ng/directives/conditionBuilder/conditionPackage':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('conditionPackage',[ '$compile', function ($compile) {
    return {

	    restrict: 'E',
	    template: '<div><ul class="conditionHeir" style="padding: 0px 0 0 0px;" ><condition-builder-hierarchy ng-repeat="condition in boundPrimitives.conditionPkg" ' +
		              'condition="condition" edit-rhs-only="boundPrimitives.editRhsOnly" read-only="boundPrimitives.readOnly"' +
		              'lhs-columns="lhsColumns" root-condition="conditionPkg" broadcast-change="boundPrimitives.broadcastChange" root-hier="true">' +
		               '</condition-builder-hierarchy></ul></div> ',
	    replace: true,
	    scope: {
		    'condition':        '=',
		    'lhsColumns':       '=',
		    'editRhsOnly':      '=',
		    'readOnly':         '=',
		    "broadcastChange":  '@'
	    },
		 controller: ['$scope','$element','$attrs',function($scope, $element, $attrs ) {
			 $scope.boundPrimitives={
				 editRhsOnly: $scope.editRhsOnly === "true" ? true : false,
				 readOnly: $scope.readOnly === "true" ? true : false
			 };


		 }],
	    link: function (scope, element, attrs) {
		    scope.boundPrimitives.readOnly = (scope.readOnly) ? scope.readOnly : scope.boundPrimitives.readOnly ;
		    scope.boundPrimitives.broadcastChange = (attrs.broadcastChange == 'false') ? false : true;

		    if (scope.boundPrimitives.readOnly) {  // readOnly needs to override editRhsOnly
			    scope.boundPrimitives.editRhsOnly = false;
		    }
		    else {
			    scope.boundPrimitives.editRhsOnly = (scope.editRhsOnly) ? scope.editRhsOnly : scope.boundPrimitives.editRhsOnly ;
		    }
		    scope.boundPrimitives.conditionPkg = [scope.condition];

		    // if condition changes to a null value then go back to showing "addConditions" button
            //FIXME:  Determine if this watch still needed - since this doesn't fire after save - and remove if not
		    scope.$watch('boundPrimitives.conditionPkg[0]', function(value) {
			    if  (value == null)
				    scope.condition = null;
		    });

            scope.$watch('condition',function(value) {
	            scope.boundPrimitives.conditionPkg  = [scope.condition];
            });

		    scope.deleteConditionRoot = function() {
			    scope.boundPrimitives.conditionPkg[0] = null;
			    scope.condition= null;
		    };
       }

  }
	}

	]);

});

},
'vsd/common/ng/directives/dualSelector/sngDualSelector':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .directive('sngDualSelector', [ 'dojoModules', function (dojoModules) {
      return {
        templateUrl: 'vsd/common/ng/directives/dualSelector/dualSelector.html',
        restrict: 'E',
        replace: true,
        scope: {
          'lhsElementId': '@',
          'rhsElementId': '@',
          'idProperty': '@',
          'selectedCandidateItemIds': '=',
          'candidateItems': '=',  //Populates the LHS
          'lhsLabel': '@',
          'selectedItemIds': '=',
          'items': '=',           //Populates the RHS
          'rhsLabel': '@'
        },
        controller: ['$scope', '$element', '$attrs', function ($scope, $element, $attrs) {
          $scope.resources = dojoModules.resources;

          var itemIdProperty = $attrs.idProperty;

          function selected(ItemCol) {
            return _.contains($scope.selectedItemIds, ItemCol[itemIdProperty].toString());
          }

          function swap(arr, i, j) {
            var tmp = arr[j];
            arr[j] = arr[i];
            arr[i] = tmp;
          }

          function sortCandidateItems() {
            $scope.candidateItems = _.sortBy($scope.candidateItems, itemIdProperty);
          }

          $scope.addAllItems = function () {
            if ($scope.candidateItems.length == 0) {
              return;  //nothing to add
            }

            //Add all the candidate items, selected or not, to the items
            _.each($scope.candidateItems,
                function (candidateItem) {
                  $scope.items.push(candidateItem);
                });
            $scope.candidateItems = [];

            //Unselect all the selected candidate partition keys
            $scope.selectedCandidateItemIds = [];
          }

          $scope.addSelectedItems = function () {
            if ($scope.selectedCandidateItemIds.length == 0) {
              return;  //nothing to add
            }

            //Add the selected candidates to the selected items and the items
            $scope.selectedItemIds = $scope.selectedCandidateItemIds.slice();
            _.each($scope.selectedItemIds,
                function (id) {
                  $scope.items.push(_.find($scope.candidateItems,
                      function (item) {
                        return item[itemIdProperty] == id;
                      }));
                });

            //Remove from the candidate collections
            $scope.candidateItems = _.reject($scope.candidateItems,
                function (candidateItem) {
                  return _.contains($scope.selectedItemIds, candidateItem[itemIdProperty].toString());
                });
            $scope.selectedCandidateItemIds = [];
          }

          $scope.removeAllItems = function () {
            if ($scope.items.length == 0) {
              return;  //nothing to remove
            }

            //Remove all the keys, selected or not, and add them back to the candidate list
            _.each($scope.items,
                function (item) {
                  $scope.candidateItems.push(item);
                });
            $scope.selectedItemIds = [];
            $scope.items = [];

            sortCandidateItems();
          }

          $scope.moveSelectedOptionsUp = function () {
            if ($scope.selectedItemIds.length == 0) {
              return;
            }
            var Items = $scope.items;
            for (var i = 0; i < Items.length; i++) {
              if (selected(Items[i])) {
                if (i != 0 && !selected(Items[i - 1])) {
                  swap(Items, i, i - 1);
                }
              }
            }
          }

          $scope.moveSelectedOptionsDown = function (obj) {
            var selectedItemIds = $scope.selectedItemIds;
            if (selectedItemIds.length == 0) {
              return;
            }

            var Items = $scope.items;
            for (var i = Items.length - 1; i >= 0; i--) {
              if (selected(Items[i])) {
                if (i != (Items.length - 1) && !selected(Items[i + 1])) {
                  swap(Items, i, i + 1);
                }
              }
            }
          }

          $scope.removeSelectedItems = function () {
            if ($scope.selectedItemIds.length == 0) {
              return;  //nothing to remove
            }
            //Add to the candidate collections
            $scope.selectedCandidateItemIds = $scope.selectedItemIds.slice();
            _.each($scope.selectedCandidateItemIds,
                function (id) {
                  $scope.candidateItems.push(_.find($scope.items,
                      function (item) {
                        return item[itemIdProperty].toString() == id;
                      }));
                });

            //Remove from the item collections
            $scope.items = _.reject($scope.items,
                function (item) {
                  return _.contains($scope.selectedCandidateItemIds, item[itemIdProperty].toString());
                });
            $scope.selectedItemIds = [];
            sortCandidateItems();
          }
        }]};
    }]);

});

},
'vsd/common/ng/directives/filereader':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

/* This directive is the workaround for the angular onchange problem with the <input type=file> tag  */
/* usage: <filereader files="data.importFilename"></filereader> */
/* where the "files=" value is the object that will contain the file list returned by the native filedialog */

angular.module('vsd')
    .directive('filereader', function () {
        return {
            scope: {
                files: '=',
                uploadFile: '=',
                data: '='
            },
            restrict: 'E',
            replace: true,
            template: "<input type='file' />",
            link: function (scope, element, attrs) {
                element.bind('change', function() {
                   var reader = new FileReader();
                   var uploadbtn = dijit.registry.byId("uploadBtn");

                   // Read the file name selected by the user from the fileinput <input type=file> dialog
                   scope.files = element[0].files;

                   // test for "cancel" button selection in OS "Open" dialog. If 'cancel', files.length = 0.
                   if (scope.files.length > 0) {
                       reader.onload = function (evt) {
                           scope.uploadFile = evt.target.result;
                           scope.$apply();
                       }
                       // read the xml file contents into the reader object.
                       reader.readAsText(scope.files[0]);

                    // enable the upload button
                       uploadbtn.set("disabled", false);
                   }
                   else {  /* user selected "cancel" button in OS Open dialog */
                       // disable the upload button
                       uploadbtn.set("disabled", true);
                       scope.uploadFile = " ";
                       scope.$apply();
                   }
                })
            }
        };
    });

});

},
'vsd/common/ng/directives/sngCheckboxGrid':function(){
'use strict';

angular.module('vsd').
	directive('sngCheckboxGrid', ['Util', 'dojoDepends', function (Util, dojoDepends) {

		return {
			restrict:   "E",
			replace:    true,
			template:   "<div></div>",
			transclude: false,
			require:    [
				'?gridData',
				'?idProperty',
				'?queryText'
			],
			scope:      {
				'gridData':           '=',
				'idProperty':         '@',
				'queryText':          '=',
				'columns':            '=',
				'dojoProps':          '@',
				'allowTextSelection': '=',
				'sortOrder':          '=',
				'changeHandler':      '&'

			},
			link:       function (scope, element, attrs, ctrl) {
				require(["dojo/ready",
				         'dojo/_base/declare',
				         "dgrid/OnDemandGrid",
				         "dgrid/Selection",
				         "dgrid/Keyboard",
				         'dgrid/extensions/ColumnResizer',
				         "dgrid/editor",
				         "dgrid/extensions/DijitRegistry",
				         'dojo/store/Memory',
				         "dojo/on",
				         "dijit/form/CheckBox" ],
					function (ready,
						declare,
						OnDemandGrid,
						Selection,
						KeyBoard,
						ColumnResizer,
						editor,
						DijitRegistry,
						Memory,
						on,
						CheckBox) {

						var
							checkboxColumn = [],
							gridColumns = [],
							filterSubset = [],
							filterAllChecked = false;

						scope.resources = dojoDepends.resources;

						// Define checkbox column, adding the header checkbox
						scope.hdrCheckBox = new CheckBox({
							tabIndex:  0,
							className: 'dgrid-input', /* To make it look like the dgrid editor checkboxes  */
							//	style:     'margin-left:9px; background:none;',
							title:     scope.resources.sngCheckboxGrid_checkBoxLbl_txt
						});
						checkboxColumn = [
							editor({
								field:            'checked',
								label:            ' ',
								renderHeaderCell: function (contentNode) {
									return scope.hdrCheckBox.domNode;
								},
								autoSave:         true,
								sortable:         true
							}, "checkbox")
						];

						// Add additional columns passed in
						if (scope.columns) {
							gridColumns = checkboxColumn.concat(scope.columns);
						}
						else {
							gridColumns = angular.copy(checkboxColumn);
						}


						//Configure grid properties
						scope.gridProperties = {};
						setNonStoreGridProperties();

						//Create the store for the grid
						scope.gridProperties.store = createStore();

						//Parse out plugins and add all requested ones to array
						var dgridMixins = [OnDemandGrid, Selection, KeyBoard, DijitRegistry, ColumnResizer];

						//Create dgrid mixin with all requested plugins
						var MixInGrid = declare(dgridMixins);
						scope.grid = new MixInGrid(scope.gridProperties, element[0]);

						//Apply styles to widget
						Util.applyStyles(scope.grid, attrs.style);

						if (scope.gridData && scope.gridData.length > 0) {
							refreshGridData();
						}

						//Set initial sort order, if passed
						if (scope.sortOrder) {
							scope.grid.set('sort', scope.sortOrder.field, scope.sortOrder.desc);
						}

						//Watch the grid data to see when the data property changes
						scope.$watch('gridData', function () {
							refreshGridData();
						});

						//If column definitions change, refresh grid
						scope.$watch('columns', function () {
							gridColumns = checkboxColumn.concat(scope.columns);
							scope.grid.set('columns', gridColumns);
							refreshGridData();
						});

						scope.hdrCheckBox.on('change', function (newVal) {
							angular.forEach(filterSubset, function (id) {
								var obj = scope.grid.store.get(id);
								obj.checked = newVal;
							});
							scope.grid.save();
							scope.grid.refresh();   // updates checkboxes
							updateHdrCheckBoxTitle();
							if (attrs.changeHandler) {
								scope.changeHandler();
								scope.$evalAsync();
							}
						});


						scope.grid.on('dgrid-refresh-complete', function (evt) {
							refreshHdrCheckBox();
							$(".field-checked").find("input").attr('tabindex',0);
						});

						// dgrid editor/Checkbox uses this handler, triggered when user clicks on a checkbox in the dgrid
						scope.grid.on('dgrid-datachange', function (evt) {
							//console.log("data changed: ", evt.oldValue, " -> ", evt.value);
							//console.log("cell: ", evt.cell, evt.cell.row.data.checked);
							if (evt.value === false) {
								filterAllChecked = false;
								refreshHdrCheckBox();
							}
							if (attrs.changeHandler) {
								evt.cell.row.data.checked = evt.value;
								scope.changeHandler();
								scope.$evalAsync();
							}
						});



						scope.$watch('queryText', function (newVal) {
							if (scope.grid) {
								scope.grid.refresh();
							}
						});

						function getCheckedItems() {
							scope.grid.save();
							return scope.grid.store.query({"checked": true});
						}

						function updateHdrCheckBoxTitle() {
							// update hover title based on current checked state
							if (scope.hdrCheckBox.checked) {
								scope.hdrCheckBox.set('title', scope.resources.sngCheckboxGrid_deselectAll_txt);
							}
							else {
								scope.hdrCheckBox.set('title', scope.resources.sngCheckboxGrid_selectAll_txt);
							}
						}

						function refreshHdrCheckBox() {
							// update checkbox state without triggering change handler
							if (scope.hdrCheckBox.domNode)
								scope.hdrCheckBox.set("checked", filterAllChecked, false);
							updateHdrCheckBoxTitle();
						}

						function refreshGridData() {
							scope.grid.set('columns', gridColumns);
							scope.grid.set('store', createStore());
							scope.grid.refresh();
							scope.grid.resize();
						}

						function setNonStoreGridProperties(properties) {
							if (attrs.dojoProps) {
								scope.gridProperties = Util.parseProps(scope.dojoProps, scope);
							}
							//Configure named grid properties
							scope.gridProperties.allowTextSelection =
								scope.allowTextSelection ? scope.allowTextSelection : null;

							scope.gridProperties.value = "";

							// Dgrid query function
							if (attrs.queryText) {
								scope.gridProperties.query = function (item, index, items) {

									// Update the filterSubset and filterAllChecked items to hold the latest filter matched ids and checked state

									if (index == 0) {
										filterSubset = [];
										filterAllChecked = true;
									}

									var filterString = scope.queryText ? scope.queryText.toLowerCase() : "";
									var name = item ? item.name.toLowerCase() : "";

									if (filterString.length < 1 || name.indexOf(filterString) >= 0) {
										filterSubset.push(item.id);
										if (!item.checked) {
											filterAllChecked = false;
										}
										return true;
									}
									else {
										return false;
									}
								};
							}
						}

						function createStore() {
							var store = new Memory({
								data:       scope.gridData,
								idProperty: attrs.idProperty || 'id'
							});
							return store;
						}

					});
			}
		};
	}]);

},
'vsd/common/ng/directives/sngDateTextBox':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .directive('sngDateTextBox', ['$rootScope', 'dojoModules', 'Util', function ($rootScope, dojoModules, Util) {
        return {
            template: "<div></div>",
            replace:  true,
            restrict: 'E',
            scope:    {
                'sngModel':    '=',
                'sngMinDate':  '=',
                'validateOnEnter':  '&',
                'sngMaxDate':  '=',
                'sngChange':   '&',
                'sngDisabled': '=',
                'sngRequired': '='
            },

            link: function (scope, element, attrs) {

                var properties = {};
                if (attrs.dojoProps) {
                    properties = Util.parseProps(attrs.dojoProps, scope);
                }

                var dateTextBox = new dojoModules.DateTextBox(properties, element[0]);

                Util.applyStyles(dateTextBox, attrs.style);

                if (attrs.sngModel) {
                    // dateTextBox strips off milliseconds
                    clearTime(scope.sngModel);
                    dateTextBox.set('value', scope.sngModel);
                    scope.$watch('sngModel', function (newVal, oldVal) {
                        clearTime(newVal);
                        clearTime(oldVal);
                        if (newVal !== oldVal && (newVal - oldVal) !== 0) {
                            dateTextBox.set('value', newVal);
                            if (attrs.sngChange) {
                                scope.sngChange();
                            }
                        }
                    });

                    if(attrs.validateOnEnter) {
                        dateTextBox.on("keyup", function (event) {
                            if (event.keyCode === 13) {
                                this.set('value', new Date(this.displayedValue));
                            }
                        });
                    }

                    dateTextBox.on('change', function (newVal) {
                        scope.sngModel = newVal;
                        //set the activeWidget - this is to stop angular bindings from updating elements
                        //that the user is currently updating
                        $rootScope.activeWidget = this.id;
                        safeApply();
                    });
                }

                if (attrs.sngMinDate) {
                    scope.$watch('sngMinDate', function (newVal, oldVal) {
                        if (newVal !== oldVal) {
                            dateTextBox.constraints.min = newVal;
                        }
                    });
                }


                if (attrs.sngMaxDate) {
                    scope.$watch('sngMaxDate', function (newVal, oldVal) {
                        if (newVal !== oldVal) {
                            dateTextBox.constraints.max = newVal;
                        }
                    });
                }

                if (attrs.sngDisabled) {
                    scope.$watch('sngDisabled', function (newVal) {
                        dateTextBox.set('disabled', newVal);
                    });
                }

                if (attrs.sngRequired) {
                    scope.$watch('sngRequired', function (newVal) {
                        dateTextBox.set('required', newVal);
                    });
                }

                function clearTime(date){
                    if(date && date instanceof Date){
                        date.setHours(0);
                        date.setMinutes(0);
                        date.setSeconds(0);
                        date.setMilliseconds(0);
                    }
                }
                function safeApply() {
                    if (!$rootScope.$$phase) {
                        scope.$apply();
                    }
                }
            }
        };
    }]);

});

},
'vsd/common/ng/directives/sngDiagram':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';
//var template;
angular.module('vsd')
    .directive('sngDiagram', ['$rootScope', function ($rootScope) {
        return {
            restrict: 'E',
            template: function (element, attrs) {
                return '<div id="' + attrs.sngId + '" style="width: 100%; height: 100%"></div>';
            },
            replace: true,
            transclude: false,
            scope: {
                'sngModel': '=',
                'sngLegend': '=',
                'sngResize': '='
            },

            link: function (scope, element, attrs) {

                function init() {

                            var $ = go.GraphObject.make;  // for conciseness in defining templates

                            var myDiagram =
                                $(go.Diagram, attrs.sngId,  // must be the ID or reference to div
                                    {
                                        hasVerticalScrollbar: false,
                                        hasHorizontalScrollbar: false,
                                        allowCopy: false,
                                        initialContentAlignment: go.Spot.Center,
                                        "toolManager.hoverDelay": 500,
                                        layout:  // create a TreeLayout for the family tree
                                            $(go.TreeLayout,
                                                { angle: 180, nodeSpacing: 10 })
                                    });


                            function tooltipNameConverter(node) {
                                return node.nodeType + ': ' + node.name;
                            }

                            /*
                             get tooltip text from the object's data
                             function tooltipTextConverter(node) {
                             var str = "";
                             if (node.toolTip !== undefined) str = node.toolTip;
                             return str;

                             }
                             */

                            function textStyle() {
                                return{

                                    font: "bold 8pt Helvetica, bold Arial, sans-serif",
                                    alignment: go.Spot.Left,
                                    margin: 16,
                                    wrap: go.TextBlock.None,
                                    overflow: go.TextBlock.OverflowEllipsis,
                                    width: 145
                                };
                            }

                            function tooltipStyle() {
                                return{

                                    font: "bold 8pt Helvetica, bold Arial, sans-serif",
                                    alignment: go.Spot.Left,
                                    margin: 16,
                                    width: 175
                                };
                            }

                            // define tooltips for nodes
                            var tooltiptemplate =
                                $(go.Adornment, "Auto",
                                    $(go.Shape, "Rectangle",
                                        { fill: "#E8EBEE", stroke: "#99A2A7" }),
                                    $(go.TextBlock, tooltipStyle(),
                                        /* new go.Binding("text", "", tooltipTextConverter)*/
                                        new go.Binding("text", "", tooltipNameConverter))
                                );


                            // define Converters to be used for Bindings
                            function nodeTypeBrushConverter(nodeType) {
                                var returnColor = '';
                                angular.forEach(scope.sngLegend, function (item) {
                                    if (item.nodeType === nodeType) {
                                        returnColor = "#E8EBEE";
                                    }
                                });
                                return returnColor;
                            }


                            // replace the default Node template in the nodeTemplateMap
                            myDiagram.nodeTemplate =
                                $(go.Node, "Auto",
                                    { deletable: false, toolTip: tooltiptemplate },
                                    new go.Binding("text", "name"),
                                    $(go.Shape, "RoundedRectangle",
                                        { fill: "#E8EBEE",
                                            stroke: "#99A2A7",
                                            width: 175,
                                            height: 80
                                            /*alignment: go.Spot.Center*/ },
                                        new go.Binding("fill", "nodeType", nodeTypeBrushConverter)),

                                    $(go.Picture,

                                        new go.Binding("source", "", function (data) {
                                            var path = "vsd/images/";
                                            var url = path + data.nodeType + ".png";
                                            return url;
                                        }),
                                        {
                                            alignment: go.Spot.TopRight,
                                            width: 24,
                                            height: 24
                                        }),

                                    $(go.TextBlock, textStyle(),
                                        new go.Binding("text", "name")),
                                    $(go.Panel,  // this is underneath the "BODY"
                                        { height: 15,
                                            alignment: go.Spot.Left,
                                            alignmentFocus: go.Spot.Center},  // always this height, even if the TreeExpanderButton is not visible
                                        $("TreeExpanderButton")
                                    )
                                );

                            // define the Link template
                            myDiagram.linkTemplate =
                                $(go.Link,  // the whole link panel
                                    /*{ routing: go.Link.Orthogonal, corner: 5, selectable: false },*/
                                    $(go.Shape,  // the link path shape
                                        { isPanelMain: true, stroke: "#99A2A7", strokeWidth: 1 }),
                                    $(go.Shape,  // the arrowhead
                                        { fromArrow: "backward", stroke: null, fill: "#99A2A7"}));

                            if (attrs.sngModel) {
                                // create the model for the family tree
                                myDiagram.model = new go.TreeModel(scope.sngModel);
                            }

                            scope.$watch('sngModel', function (newVal, oldVal) {
                                myDiagram.model = new go.TreeModel(scope.sngModel);
                            });

                            scope.$watch('sngResize', function (newVal, oldVal) {
                                if (newVal === oldVal) {
                                    return;
                                }
                                if (newVal) {
                                    myDiagram.requestUpdate();
                                }
                            });

                            /*
                             scope.$watch('sngLegend', function (newVal, oldVal) {

                             if (scope.sngLegend) {

                             //Create Legend display on diagram based on sngLegend attribute sent into directive
                             var table = $(go.Part, "Table", { position: new go.Point(0, 175), selectable: false });

                             var indx = 0;
                             angular.forEach(scope.sngLegend, function (item) {
                             table.add( $(go.Panel, "Horizontal",
                             { row: 0, column: indx++, alignment: go.Spot.Left },
                             $(go.Shape, "Rectangle",
                             { desiredSize: new go.Size(30, 15), fill: item.color, margin: 5 }),
                             $(go.TextBlock, item.nodeType,
                             { font: "bold 8pt Helvetica, bold Arial, sans-serif" })
                             ));
                             });
                             // Set up a Part as a legend, and place it directly on the diagram
                             myDiagram.add(table);
                             }
                             });
                             */

                        }

                init();

            }
        };
    }]);


});

},
'vsd/common/ng/directives/sngDgrid':function(){
'use strict';

angular.module('vsd').
    directive('sngDgrid',
    ['$rootScope', 'Util', 'dojoModules', '$timeout', function ($rootScope, Util, dojoModules, $timeout) {

        return {
            restrict: "E",
            replace: true,
            template: "<span></span>",
            transclude: false,
            scope: {
                'gridData': '=',
                'reloadIndicator': '=',
                'columns': '=',
                'dojoProps': '@',
                'idProperty': '@',
                'showHeader': '&',
                'showFooter': '&',
                'footerText': '=',
                'selectionMode': '@',
                'minRowsPerPage': '&',
                'maxRowsPerPage': '&',
                'allowDuplicates': '&',
                'collapseOnRefresh': '&',
                'indentWidth': '&',
                'allowTextSelection': '&',
                'dblclickHandler': '&',
                'rmbclickHandler': '&',
                'headerRmbclickHandler': '&',
                'queryHandler': '&',
                'queryText': '=',
                'rowSelectHandler': '&',
                'rowDeselectHandler': '&',
                'rowMouseoverHandler': '&',
                'rowMouseoutHandler': '&',
                'cellSelectHandler': '&',
                'getChildrenHandler': '&',
                'mayHaveChildrenHandler': '&',
                'dropHandler': '&',
                'sortOrder': '&',
                'sortIndicator': '&',
                'protectedColumnClass': '@',
                'columnHeaderTitle': '@',
                'scrollToRowAfterInsert': '=',
                'onEnterKeyHandler': '&',
                'onDeleteKeyHandler': '&',
                'focusOnColumn': '@',
                'cellNavigation': '&'
            },
            link: function (scope, element, attrs, ctrl) {
                require(['dojo/_base/declare',
                        'dojo/aspect',
                        "dgrid/OnDemandGrid",
                        "dgrid/Selection",
                        "dgrid/extensions/DnD",
                        "dgrid/Keyboard",
                        'dgrid/extensions/ColumnResizer',
                        'dgrid/extensions/ColumnReorder',
                        "dgrid/extensions/CompoundColumns",
                        'dgrid/ColumnSet',
                        "dgrid/editor",
                        "dgrid/CellSelection",
                        "dojo/_base/NodeList",
                        "dgrid/extensions/DijitRegistry",
                        'dojo/store/Memory'],
					function (declare, aspect, OnDemandGrid, Selection, DnD, KeyBoard, ColumnResizer, ColumnReorder,
						CompoundColumns, ColumnSet, editor, CellSelection, NodeList, DijitRegistry, Memory) {
                        var dropFlag = false,
							calledCellHandler = false,
                            currColPos = null,
                            showDropColumn = angular.isDefined(attrs.showDropColumn),
                            highlightColumnSelect = angular.isDefined(attrs.highlightColumnSelect),
                            currSelectCol = null,
                            prevSelectCells = null,
                            prevSelectCol = null,
                            protectedColumnClass = null,
                            lastHighlightedPosition = null,
                            gridProperties = null;

                        var dndGridSource = declare(DnD.GridSource, {
                            onDrop: function (sourceSource, nodes, copy) {

                                var newColList = [];
                                if (showDropColumn) {
                                    dropFlag = true;
                                }
                                angular.forEach(nodes, function (node) {
                                    if (attrs.dropCustomDataField) {
                                        newColList.push(node[attrs.dropCustomDataField]);
                                    }
                                    else {
                                        newColList.push(node);
                                    }
                                });
                                if (scope.dropHandler) {
                                    scope.dropHandler({cols: newColList});
//										safeApply();
                                    scope.$evalAsync();
                                }

                            },
                            copyState: function () {
                                return true; // never copy, only swap
                            },

                            getSelectedNodes: function () {
                                var nodes = new NodeList(),
                                    orderedProperties = [],
                                    dataItem;

                                angular.forEach(this.grid.selection, function (value, id) {
                                    orderedProperties.push({sortKey: id.toUpperCase(), id: id});
                                });
                                orderedProperties.sort(function (a, b) {
                                    if (a.sortKey > b.sortKey) {
                                        return 1;
                                    }
                                    if (a.sortKey < b.sortKey) {
                                        return -1;
                                    }
                                    return 0;
                                });

                                angular.forEach(orderedProperties, function (prop) {
                                    var id = prop.id,
                                        currNode = this._selectedNodes[id];

                                    if (currNode) {
                                        if (attrs.dragCustomDataField) {
                                            currNode[attrs.dragCustomDataField] = {};
                                            dataItem = _.find(this.grid.store.data, function (item) {
                                                return String(item[attrs.idProperty]) === id;
                                            });
                                            angular.copy(dataItem, currNode[attrs.dragCustomDataField]);
                                        }
                                        nodes.push(currNode);
                                    }
                                }, this);
                                this.grid.clearSelection();
                                return nodes;
                            }
                        });

                        //Configure grid properties
                        gridProperties = getGridProperties();

                        //Create dgrid mixin with all requested mixins
                        var MixInGrid = declare(getDgridMixins());
                        var grid = new MixInGrid(gridProperties, element[0]);

                        //Apply styles to widget
                        Util.applyStyles(grid, attrs.style);

                        if (attrs.protectedColumnClass) {
                            protectedColumnClass = attrs.protectedColumnClass;
                        }


                        scope.$watch('footerText', function () {
                            if (attrs.showFooter) {
                                grid.footerNode.innerHTML = '<span style="text-align: left; margin-right: 5px;">' + scope.footerText + '</span>';
                            }
                        });

                        scope.$watch('reloadIndicator', function (newStuff, oldStuff) {
                            if (newStuff !== oldStuff) {
                                grid.refresh();
                                grid.resize();
                            }
                        });

                        //Watch the grid data to see when the data property changes
                        scope.$watch('gridData', function (newStuff, oldStuff) {

                            if (newStuff === oldStuff) {
                                grid.set('store', createStore(newStuff));
                                //Watch the grid preview data total

                                // Indicate sort order from server
                                if (attrs.sortIndicator) {
                                    var si = scope.sortIndicator;
                                    grid.updateSortArrow([
                                        {attribute: si().field, descending: si().desc}
                                    ], true);
                                }
                                // else set initial sort order, if passed
                                else if (attrs.sortOrder) {
                                    grid.set('sort', scope.sortOrder());
                                }

                            }
                            else if (angular.isArray(scope.columns) && scope.columns.length) {
                                grid.store.setData(newStuff);
                                grid.refresh();
                                grid.resize();
                                scrollAndHighlight();

                            }

                            if (attrs.focusOnColumn && grid.store && grid.store.data.length > 0) {
                                var rowId = grid.store.data[0][attrs.idProperty];
                                setTimeout(function () {
                                    var row = grid.row(rowId);
                                    var cell = grid.cell(row, attrs.focusOnColumn);
                                    /*if (!cell.column)
                                     cell = grid.cell(row,"0");*/
                                    if (cell) {
                                        grid.focus(cell);
                                    }
                                    cell.element.focus();
                                }, 2000);
                            }

                        });

                        //Refresh grid if data array items change
                        scope.$watchCollection('gridData', function (newStuff, oldStuff) {
                            if (newStuff !== oldStuff) {
                                grid.refresh();
                                grid.resize();
                                scrollAndHighlight();
                            }
                        });

                        function updateColumnHeaderTitles() {
                            // Adds classic hover titles to dgrid column header cells.
                            // These behave better than dojo tooltips for simple text strings
                            // Uses the specified property on the column object for the title
                            if (attrs.columnHeaderTitle) {
                                var titleProperty = scope.columnHeaderTitle || "title";
                                var cols = _.keys(grid.columns);
                                cols.forEach(function (key) {
                                    dojoModules.query(".dgrid-column-" + key).forEach(function (node) {
                                        if ((node.nodeName == "TH" || node.tagName == "TH") && grid.columns[key][titleProperty]) {
                                            node.title = grid.columns[key][titleProperty];
                                        }
                                    });
                                });
                            }
                        }

                        function scrollAndHighlight() {
                            var firstCol = {},
                                colInfo = null;
                            if (!showDropColumn) {
                                return;
                            }
                            if (protectedColumnClass) {
                                angular.forEach(scope.columns, function (column) {
                                    if (column.headerNode && column.headerNode.className.indexOf(protectedColumnClass) === -1 && !firstCol.name && scope.gridData.length) {
                                        firstCol.name = column.name;
                                        firstCol.classList = column.headerNode.className.split(" ");
                                        colInfo = findClassList(firstCol.classList);
                                        if (showDropColumn && dropFlag) {
                                            //scroll to dragged columns
                                            var row = grid.row(grid.store.data[0][scope.idProperty]);
                                            grid.scrollTo({x: row.element.offsetWidth});
                                            dropFlag = false;
                                        }
                                        else {
                                            if (!lastHighlightedPosition) {
                                                // if column was dropped on don't highlight column - only pre-highlight if opening a new/existing window
                                                highlightColumn(colInfo);
                                                scope.cellSelectHandler({column: firstCol.name, currColPos: colInfo.currColPos});
                                                scope.$evalAsync();
                                            }
                                        }
                                        if (lastHighlightedPosition) {
                                            highlightPreviousColumn();
                                        }
                                    }

                                });
                            }
                        }


                        //If column definitions change, refresh grid
                        scope.$watchCollection('columns', function (newStuff, oldStuff, equalVals) {
                            grid.set('columns', newStuff);
                            if (highlightColumnSelect && lastHighlightedPosition) {
                                // chk to see if last highlighted column was deleted
                                var selCol = _.filter(scope.columns, function (cols) {
                                    return cols.headerNode.className.indexOf('dgrid-column-' + lastHighlightedPosition + ' ') > -1;
                                });
                                if (selCol.length === 0) {
                                    // it was deleted so go back to highlighting the first column
                                    lastHighlightedPosition = null;
                                }
                                scrollAndHighlight();
                            }
                            updateColumnHeaderTitles();
                            grid.resize();
                        });


                        // If this attribute is defined and has value of true, then the dgrid scrolls to newly
                        // inserted row
                        if (attrs.scrollToRowAfterInsert) {
                            grid.on('dgrid-refresh-complete', function (evt) {
                                aspect.after(grid, 'insertRow', function (row) {
                                    if (scope.scrollToRowAfterInsert == true) {
                                        grid.bodyNode.scrollTop = this.bodyNode.scrollHeight;
                                        return row;
                                    }
                                });
                            });
                        }

                        //If query configured watch for query text changes to refresh grid
                        if (attrs.queryHandler) {
                            scope.$watch('queryText', function (newVal, oldVal) {
                                if (newVal !== oldVal) {
                                    if (grid) {
                                        grid.refresh();
                                    }
                                }
                            });
                        }


                        //Hook up listener for selection of cell
                        if (attrs.cellSelectHandler) {
                            var fieldName = null;
                            grid.on('.dgrid-cell:click', function (evt) {
                                var row = grid.cell(evt.target).row;
                                var colIndex = null;
                                if (typeof evt.target.cellIndex !== 'undefined' && evt.target.cellIndex >= 0) {
                                    colIndex = evt.target.cellIndex;
                                }
                                else if (typeof evt.target.parentElement.cellIndex !== 'undefined' && evt.target.parentElement.cellIndex >= 0) {
                                    colIndex = evt.target.parentElement.cellIndex;
                                }
                                if (colIndex === null) {
                                    return;
                                }
                                var colModel = scope.columns[colIndex],
                                    classList = this.className.split(" "),
                                    colInfo;
                                // clicking on a protected cell?
                                if (_.indexOf(classList, protectedColumnClass) === -1) {
                                    if (highlightColumnSelect) {
                                        classList = (evt.target.parentElement.className.length > 0) ?
                                            evt.target.parentElement.className.split(" ") : evt.target.className.split(" ");
                                        colInfo = findClassList(classList);
                                        if (colInfo) {
                                            fieldName = highlightColumn(colInfo);
                                            scope.cellSelectHandler({column: colModel.name, currColPos: colInfo.currColPos });
                                            scope.$evalAsync();
                                        }
                                    }
                                    else {
                                        fieldName = colModel.name;
                                    }
                                }
                            });

                        }
                        else {
                            grid.on('dgrid-select', function (evt) {
                                scope.selObj = evt.rows[0].data;
                                if (attrs.rowSelectHandler) {
                                    scope.rowSelectHandler({row: scope.selObj});

//										safeApply();
                                    scope.$evalAsync();
                                }
                            });
                            grid.on('dgrid-deselect', function (evt) {
                                if (attrs.rowDeselectHandler) {
                                    scope.rowDeselectHandler();
                                    scope.$evalAsync();
                                }
                            });
                        }


                        //Hook up double click event
                        if (attrs.dblclickHandler) {
                            grid.on('dblclick', function (evt) {
                                scope.dblclickHandler({selRow: grid.row(evt)});
                                scope.$evalAsync();
//								scope.$apply();
                            });
                        }

                        //Hook up mouseover event
                        if (attrs.rowMouseoverHandler) {
                            grid.on('.dgrid-row:mouseover', function (evt) {
                                evt.preventDefault();
                                var rowObj = grid.row(evt).data;
                                scope.rowMouseoverHandler({row: rowObj});
                                scope.$evalAsync();
                            });
                        }

                        //Hook up mouseout event
                        if (attrs.rowMouseoutHandler) {
                            grid.on('.dgrid-row:mouseout', function (evt) {
                                evt.preventDefault();
                                var rowObj = grid.row(evt).data;
                                scope.rowMouseoutHandler({row: rowObj});
                                scope.$evalAsync();
// FIXME: replace scope.$evalAsync
                            });
                        }

                        if (attrs.rmbclickHandler) {
                            scope.rowRMBMenu = new dojoModules.Menu({
                                targetNodeIds: [grid.domNode],
                                selector: "div.dgrid-row"
                            });
                            //Hook up right click event
                            grid.on(".dgrid-row:contextmenu", function (evt) {
                                //prevent rmb default menu from appearing
                                evt.preventDefault();
                                //Remove all existing menu items
                                scope.rowRMBMenu.getChildren().forEach(function (child) {
                                    scope.rowRMBMenu.removeChild(child);
                                    //TODO:  Do I also need to destroy the child here?
                                });
                                var selItems = [];
                                for (var key in grid.selection) {
                                    selItems.push(grid.row(key));

                                }
                                //Dynamically create menu items
                                var menuItems = scope.rmbclickHandler({rows: selItems});
                                menuItems.forEach(function (item) {
                                    scope.rowRMBMenu.addChild(new dojoModules.MenuItem(item));
                                });

                            });
                        }

                        if (attrs.headerRmbclickHandler) {
                            scope.headerRMBMenu = new dojoModules.Menu({
                                targetNodeIds: [grid.domNode],
                                selector: "div.dgrid-header .dgrid-cell"
                            });
                            grid.on('.dgrid-header .dgrid-cell:contextmenu', function (evt) {
                                //prevent rmb default menu from appearing
                                evt.preventDefault();
                                //Remove all existing menu items
                                scope.headerRMBMenu.getChildren().forEach(function (child) {
                                    scope.headerRMBMenu.removeChild(child);
                                    //TODO:  Do I also need to destroy the child here?
                                });
                                //Get cell related to event
                                var cell = grid.cell(evt);
                                //Invoke handler for event
                                var menuItems = scope.headerRmbclickHandler({column: cell.column});
                                menuItems.forEach(function (item) {
                                    var subMenu;
                                    if (item.isSubMenu) {
                                        subMenu = new dojoModules.Menu({label: item.label})
                                        item.menuItems.forEach(function (subMenuItem) {
                                            var menuItem = new dojoModules.MenuItem({
                                                label: subMenuItem.label,
                                                iconClass: subMenuItem.iconClass,
                                                onClick: subMenuItem.onClick
                                            });
                                            subMenu.addChild(menuItem);
                                        });
                                        scope.headerRMBMenu.addChild(new dojoModules.PopupMenuItem({label: item.label, popup: subMenu}));
                                    }
                                    else {
                                        scope.headerRMBMenu.addChild(new dojoModules.MenuItem(item));

                                    }

                                });
                            });
                        }

                        if (attrs.onEnterKeyHandler) {
                            // user press ENTER on object in Grid – open object
                            grid.addKeyHandler(13, function (evt) {
                                var row = grid.cell(evt.target).row;
                                scope.onEnterKeyHandler({row: row.data});
                                scope.$evalAsync();
                            });
                        }

                        if (attrs.onDeleteKeyHandler) {
                            // user press DELETE on object in Grid – open object
                            grid.addKeyHandler(46, function (evt) {
                                var row = grid.cell(evt.target).row;
                                scope.onDeleteKeyHandler({row: row.data});
                                scope.$evalAsync();
                            });
                        }

                        function getGridProperties() {
                            var p = {};
                            if (attrs.dojoProps) {
                                p = Util.parseProps(scope.dojoProps, scope);
                            }
                            //Configure named grid properties
                            p.showHeader = attrs.showHeader ? scope.showHeader() : true;
                            p.cellNavigation = attrs.cellNavigation ? scope.cellNavigation() : true;
                            p.showFooter = attrs.showFooter ? scope.showFooter() : true;
                            p.minRowsPerPage = (attrs.minRowsPerPage) ? scope.minRowsPerPage() : null;
                            p.maxRowsPerPage = (attrs.maxRowsPerPage) ? scope.maxRowsPerPage() : null;
                            p.selectionMode =
                                attrs.selectionMode ? scope.selectionMode : 'single';
                            p.allowDuplicates =
                                attrs.allowDuplicates ? scope.allowDuplicates() : false;
                            p.collapseOnRefresh =
                                attrs.collapseOnRefresh ? scope.collapseOnRefresh() : false;
                            p.indentWidth = attrs.indentWidth ? scope.indentWidth() : null;
                            p.allowTextSelection =
                                attrs.allowTextSelection ? scope.allowTextSelection() : true;
                            p.query = attrs.queryHandler ? function (item, index, items) {
                                return scope.queryHandler({item: item, index: index, items: items});
                            } : null;
                            if (attrs.dropHandler || attrs.dragCustomDataField) {
                                p.dndConstructor = dndGridSource;
                            }

                            p.keepScrollPosition = true;

                            p.value = "";

                            return p;

                        }

                        function createStore(data) {
                            var store = new Memory({
                                data: data,
                                idProperty: attrs.idProperty,
                                getChildren: attrs.getChildrenHandler ? function (parent, options) {
                                    return scope.getChildrenHandler({store: store, parent: parent, options: options});
                                } : null,
                                mayHaveChildren: attrs.mayHaveChildrenHandler ? function (parent) {
                                    return scope.mayHaveChildrenHandler({store: store, parent: parent});
                                } : null
                            });


                            return store;
                        }

                        function getDgridMixins() {
                            var dgridMixins = [OnDemandGrid];
                            if (attrs.mixins) {
                                var supportedMixins = {
                                    'Selection': Selection,
                                    'CellSelection': CellSelection,
                                    'DnD': DnD,
                                    'KeyBoard': KeyBoard,
                                    'ColumnResizer': ColumnResizer,
                                    'ColumnReorder': ColumnReorder,
                                    'ColumnSet': ColumnSet,
                                    'CompoundColumns': CompoundColumns,
                                    'editor': editor,
                                    'DijitRegistry': DijitRegistry};

                                var mixinStrArray = attrs.mixins.split(",");
                                mixinStrArray.forEach(function (mixinStr) {
                                    var mixin = supportedMixins[mixinStr];
                                    if (mixin) {
                                        dgridMixins.push(mixin);
                                    }
                                    else {
                                        throw new Error("Unsupported mixin '" + mixinStr + "'");
                                    }
                                });
                            }
                            return dgridMixins;
                        }


                        function highlightColumn(colInfo) {

                            // if clicked on protected cell or protected header
                            var isProtected = _.indexOf(colInfo.classList, protectedColumnClass);
                            if (isProtected > -1) {
                                return;
                            }

                            // find col info like label and var type;
                            //currSelectCol = _.findWhere(grid.columns, { name: fieldName })[0];
                            currSelectCol = grid.columns[colInfo.currColPos];

                            // retrieve all the dom cells in the selected column
                            //var colSelectCells = dojo.query("td.dgrid-column-" + currColPos, evt.currentTarget);
                            var colSelectCells = dojo.query("td.dgrid-column-" + currColPos, grid.contentNode);

                            if (currSelectCol && currSelectCol.field) {
                                angular.forEach(colSelectCells, function (cell) {
                                    dojo.toggleClass(cell, "columnHighlight");
                                });
                            }
                            //un-select prev selected column
                            if (prevSelectCells) {
                                angular.forEach(prevSelectCells, function (cell) {
                                    dojo.toggleClass(cell, "columnHighlight");
                                });
                            }

                            prevSelectCells = ( prevSelectCells == colSelectCells ) ? null : colSelectCells;
                            //TODO: Need to disable delete toolbar when prevSelCells = null
                            prevSelectCol = currSelectCol;


                            //TODO: Is it in the temp table? if so then we can view col details
                            lastHighlightedPosition = currColPos;

                            return fieldName;
                        }

                        function highlightPreviousColumn() {
                            // retrieve all the dom cells in the selected column
                            var colSelectCells = dojo.query("td.dgrid-column-" + lastHighlightedPosition,
                                grid.contentNode);

                            //un-select prev selected column
                            if (prevSelectCells) {
                                angular.forEach(prevSelectCells, function (cell) {
                                    dojo.toggleClass(cell, "columnHighlight");
                                });
                            }

                            if (currSelectCol && currSelectCol.field) {
                                angular.forEach(colSelectCells, function (cell) {
                                    dojo.toggleClass(cell, "columnHighlight");
                                });
                            }

                            prevSelectCells = ( prevSelectCells === colSelectCells ) ? null : colSelectCells;

                        }

                        function findClassList(classList) {

                            classList.forEach(function (classname) {
                                if (classname.split('-')[0] === 'field') {
                                    fieldName = classname.split('-')[1];
                                }
                                if (classname.split('-')[1] === 'column') {
                                    currColPos = classname.split('-')[2];
                                }
                            });

                            return {classList: classList, fieldName: fieldName, currColPos: currColPos};
                        }

//							function safeApply(){
//								if (!$rootScope.$$phase) {
//									scope.$apply();
//								}
//							}

                    });
            }
        };
    }]);

},
'vsd/common/ng/directives/sngDialog':function(){
'use strict';

angular.module('vsd').
	directive('sngDialog', function ($timeout, Util, dojoModules) {
		return {
			restrict:   "A",
			replace:    false,
			transclude: true,
			template:   "<div ng-transclude></div>",
			scope:      {
				dojoProps:   '@',
				sngTitle:    '=',
				show:        '=',
				onShow:      '&',
				resetOnShow: '@',
				cancel:      '&',
				enter:       '&'
			},
			link:       function (scope, element, attrs, ctrl) {
				if (element.closest(document.documentElement).length === 0) {
					return;
				}

				// allow Angular to interpolate resources before Dojo creates widgets
				$timeout(function () {
					require(['dijit/Dialog', 'dojo/on', 'dojo/keys'],
						function (Dialog, on, keys) {

							var properties = {},
								dialog = null,
								resetOnShow = (attrs.resetOnShow !== "false"),
								hidePromise = null;

							if (attrs['class']) {
								properties['class'] = attrs['class'];
							}

							if (attrs.style) {
								properties.style = attrs.style;
							}

							if (attrs.dojoProps) {
								angular.extend(properties, Util.parseProps(scope.dojoProps, scope));
							}

							dialog = new Dialog(properties, element[0]);

							// Rewrite destroy method to handle screen transitions without errors
							updateDestroyFunction(dialog);

							if (attrs.sngTitle) {
								scope.$watch('sngTitle', function (newVal) {
									dialog.set('title', newVal);
								});
							}

							if (attrs.enter) {
								dialog.on('keyup', function (event) {
									if (event.keyCode === keys.ENTER) {
										scope.enter();
										scope.$apply();
									}
								});
							}

							dialog.on('cancel', function () {
								dialog.reset();
								scope.cancel();
								scope.$apply();
							});

							scope.$watch('show', function (newVal, oldVal) {
								if (newVal !== oldVal) {
									if (newVal) {
										hidePromise = null;
										if (resetOnShow) {
											dialog.reset();
										}
										dialog.show();
										if (attrs.onShow) {
											scope.onShow();
										}
									}
									else { // capture hide promise for future use
										hidePromise = dialog.hide();
									}
								}
							});

							function updateDestroyFunction(dialog) {
								var defaultDestroy = dialog.destroy;
								dialog.destroy = function (preserveDom) {
									if (hidePromise) {
										hidePromise.then(function () {
											defaultDestroy.call(dialog, preserveDom);
										});
									}
									else if (dialog.open) {
										dialog.hide().then(function () {
											defaultDestroy.call(dialog, preserveDom);
										});
									}
									else {
										defaultDestroy.call(dialog, preserveDom);
									}
								};
							}

						});
				}, 0); // $timeout
			}
		};
	})
;

},
'vsd/common/ng/directives/sngDijit':function(){
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
    directive('sngDijit', function ($rootScope, Util, vsdUtil) {
        return {
            restrict: "A",
            replace: false,
            transclude: false,
            scope: {
                'sngModel': '=',
                'sngClick': '&',
                'sngChange': '&',
                'sngDisabled': '=',
                'sngRequired': '=',
                'sngEntries': '=',
                'sngEntryReturnId': '@',
                'sngIdProperty': '@',
                'sngLabel': '=',
                'sngIconClass': '=',
                'sngValidator': '&',
                'sngErrorMsgProp': '=',
                'sngShow': '=',
                'sngConstraints': '=',
                'sngResetOnFalsy': '=',
                'dojoProps': '@'

            },
            link: function (scope, element, attrs, ctrl) {
                require([attrs.sngDijit,
                        "dojo/store/Memory"
                    ],
                    function (DojoWidget, Memory) {
                        var properties = {},
                            sngIdProperty = null,
                            isModelEntryObject = !angular.isDefined(attrs.sngEntryReturnId),
                            selectFirstEntry = angular.isDefined(attrs.sngSelectFirstEntry),
                            dojoWidget;

                        if (attrs['class']) {
                            properties['class'] = attrs['class'];
                        }

                        if (attrs.dojoProps) {
                            properties = Util.parseProps(scope.dojoProps, scope);
                        }
                        if (attrs.sngIdProperty) {
                            sngIdProperty = attrs.sngIdProperty;
                        }
                        if (attrs.sngValidator) {
                            if (attrs.sngErrorMsgProp) {
                                scope.$watch('sngErrorMsgProp', function (newValue) {
                                    if (newValue) {
                                        dojoWidget.focus();
                                        dojoWidget._set('state', 'Error');
                            dojoWidget.displayMessage(newValue);
                        }
                    });
            }
            if (attrs.sngDijit === 'dijit/form/FilteringSelect') {
            properties.isValid = function () {
                if (!!this.item || (!this.required && this.get('displayedValue') === "")) {
                                        return this.validator(this.item, this.get('constraints'));
                }
                return false;
            };
        }
                            properties.validator = function (value) {
                                var result = scope.sngValidator({value: value});
                                if (result) {
                                    if (!result.isValid && result.invalidMessage) {
                                        dojoWidget.set('invalidMessage', result.invalidMessage);
                                    }
                                    return result.isValid;
                                } else {
                                    return false;
                                }
                            };
                        }

                        dojoWidget = new DojoWidget(properties, element[0]);

                        //Apply styles to widget
                        Util.applyStyles(dojoWidget, attrs.style);

                        //Handle click event
                        if (attrs.sngClick) {
                            dojoWidget.on('click', function () {
                                scope.sngClick();
                                safeApply();

                            });
                        }

                        //Handle disabled setting
                        if (attrs.sngDisabled) {
                            scope.$watch('sngDisabled', function (newVal) {
                                dojoWidget.set('disabled', newVal);
                            });
                        }

                        //Handle disabled setting
                        if (attrs.sngRequired) {
                            scope.$watch('sngRequired', function (newVal) {
                                dojoWidget.set('required', newVal);
                            });
                        }

                        if (attrs.sngLabel) {
                            scope.$watch('sngLabel', function (newVal) {
                                dojoWidget.set('label', newVal);
                            });
                        }

                        if (attrs.sngIconClass) {
                            scope.$watch('sngIconClass', function (newVal) {
                                dojoWidget.set('iconClass', newVal);
                            });
                        }

                        if (attrs.sngShow) {
                            scope.$watch('sngShow', function (newVal) {
                                if (newVal) {
                                    vsdUtil.toggleDojoWidgetDisplay(dojoWidget, '');
                                }
                                else {
                                    vsdUtil.toggleDojoWidgetDisplay(dojoWidget, 'none');
                                }
                            });
                        }

                        if (attrs.sngConstraints) {
                            scope.$watch('sngConstraints', function (newVal, oldVal) {
                                if (newVal) {
                                    dojoWidget.set('constraints', scope.sngConstraints);
                                }
                            });
                        }

                        function getWidgetValue(model) {
                            var value = model;
                            if (model && sngIdProperty && isModelEntryObject) {
                                value = model[sngIdProperty];
                            }
                            return value === null ? "" : value;
                        }

                        if (attrs.sngModel) {

                            if (attrs.sngEntries) {
                                // Create a memory store for the entries
                                dojoWidget.set('store', new Memory({data: [], idProperty: sngIdProperty}));
                                scope.$watchCollection('sngEntries', function (newVal) {
                                    updateStoreWidget(newVal);
                                });
                            }

                            scope.$watch('sngModel', function (newValue, oldValue) {
                                var widgetValue = getWidgetValue(newValue);

                                if (angular.isDefined(widgetValue)) {
                                    if (widgetValue !== dojoWidget.get('value')) {
                                        dojoWidget.set('value', widgetValue);
                                    }
                                    if (attrs.sngChange && getWidgetValue(oldValue) !== widgetValue) {
//										console.log("sngDijit."+attrs.sngModel+" changing from " + getWidgetValue(oldValue) + " to "+ widgetValue);
                                        scope.sngChange();
                                    }
                                }
                                /* TJS:  Commented out since cleared validation messages on sngSpinner that were required to show the user what is wrong
                                 *       Code was originally added due to perceived issues on the new window dialog.  Can't replace those issues with this code commented out
                                 */
                                else if (angular.isDefined(attrs.sngResetOnFalsy)) {
                                    dojoWidget.reset();
                                }
                            });

                            dojoWidget.on("change", function (newValue) {
                                //set the activeWidget - this is to stop angular bindings from updating elements
                                //that the user is currently updating
                                $rootScope.activeWidget = this.id;
                                if (sngIdProperty && isModelEntryObject) {
                                    if (!scope.sngModel || scope.sngModel[sngIdProperty] !== newValue) {
                                        scope.sngModel = dojoWidget.item;
                                    }
                                }
                                else {
                                    //TJS:  Added since it seems if sngModel was set to NaN, broke binding
                                    //IMPORTANT:  Regular isNaN function doesn't work since it is true for other data types (String, Boolean, etc)
                                    if (_.isNaN(newValue)) {
                                        newValue = null;
                                    }
                                    scope.sngModel = newValue;
                                }
                                safeApply();
                            });

                            dojoWidget.on("keyup", function (event) {
                                if (event.keyCode === 13) {
                                    dojoWidget.set('value', sngIdProperty ? this.value : this.displayedValue);
                                }
                            });
                        }

                        function updateStoreWidget(sngEntries) {

                            var entries = sngEntries || [],
                                selValue = scope.sngModel ? scope.sngModel[sngIdProperty] : null,
                                query = {},
                                result;

                            dojoWidget.store.setData(entries);

                            query[sngIdProperty] = selValue;
                            result = dojoWidget.store.query(query);
                            // if current value is not in new list, reset the value and select the first one if needed
                            if (!result.length) {
                                dojoWidget.reset();
                                if (selectFirstEntry && entries.length) {
                                    selValue = entries[0][sngIdProperty];
                                }
                            }

                            if (angular.isDefined(selValue)) {
                                dojoWidget.set('value', selValue);
                            }
                        }

                        function safeApply() {
                            if (!$rootScope.$$phase) {
                                scope.$apply();
                            }
                        }


                    });
            }
        };
    });

});

},
'vsd/common/ng/directives/sngDojoDndTargetArea':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
	directive('sngDojoDndTargetArea', ['dojoModules', function (dojoModules) {
		return {
			restrict:   'A',
			replace:    false,
		 	transclude: true,
			template: "<div ng-transclude style='width: 100%;height: 100%'></div>",
			scope: {
				'sngDropHandler':          '&',
				'sngCheckAcceptHandler':   '&',
			   'acceptNodeType':          '@',
			   'acceptNodeClasses':       '@'
			},

			link:  function (scope, element, attrs, ctrl) {

				var properties = {
					skipForm: true, // Allows selection of text in IE
					accept: (attrs.acceptNodeType) ? [attrs.acceptNodeType] :  ["dgrid-row"]
				},
					dropDomNode=null;

			    var dndWidget = new dojoModules.DojoDNDTarget(element[0],properties);
                //Hook up handlers
                dndWidget.onDrop = function(source, nodes, copy) {
                    scope.sngDropHandler({source:source, cols:nodes, copy:copy} );
	                scope.$apply();
               };

              if(attrs.sngCheckAcceptHandler) {
	                 // if (dropDomNode)
	                     dndWidget.checkAcceptance = function(source,nodes) {
	                        return scope.sngCheckAcceptHandler({source:source, cols:nodes, target: dropDomNode});
	                     };
              }

				 /* dndWidget.onMouseOver  = function(evt) {
					  console.log(evt.currentTarget.className);
					  if ( evt.currentTarget.className.indexOf(attrs.acceptNodeClasses) > -1)
					      dropDomNode = evt.currentTarget;
				  };
*/
              scope.$on('$destroy',function() {
	             // if ($(dndWidget.node).closest(document.documentElement).length)
                    dndWidget.destroy();
              });


			}
		} ;
	}]);

});

},
'vsd/common/ng/directives/sngDojoParse':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
	directive('sngDojoParse', function (dojoDepends, $timeout) {
		return {
			restrict:   'A',
			replace:    false,
			transclude: false,
			scope:      true,
			priority:   -1,
			link:       function (scope, element, attrs, ctrl) {
				function dojoParse() {
					if (!scope.$root.$$phase) {
						// make sure element is still attached to document; can be detached while transitioning screens
						if (element.closest(document.documentElement).length) {
						dojoDepends.parser.parse(element[0]).then(function () {
							if (scope.dojoWidget && scope.dojoWidget.resize) {
								scope.dojoWidget.resize();
                                //Watch resize on all content panes and notify scope when resize occurs
                                configContentPaneResizeWatch() ;
                                //Watch tab switch and resize child panel and any embedded grids when switch occurs
                                configTabSwitchWatchResize();
							}
						});
						}
					}
					else {
						$timeout(dojoParse, 0);
					}
				}

                function configContentPaneResizeWatch() {
                    var contentPaneEls = element.find('.dijitContentPane');
                    angular.forEach(contentPaneEls,function(contentPanelEl) {
                        var currWidget = dojoDepends.registry.byId(contentPanelEl.id);
                        dojoDepends.aspect.after(currWidget,'resize',function() {
	                        if (!scope.$root.$$phase)  {
                              scope.$apply();
                              scope.$broadcast("CONTENT_PANE_RESIZE")
                            }
                        });
                    });
                }
                function configTabSwitchWatchResize() {
                    var tabConts = element.find('.dijitTabContainer');
                    var tabWidgets = [];
                    angular.forEach(tabConts,function(tabContEl) {
                        var tabCont = dojoDepends.registry.byId(tabContEl.id);
                        tabCont.watch('selectedChildWidget',function(prop,prevSelChild,newSelChild) {
                            if(newSelChild && newSelChild.resize) {
                                if(!newSelChild.hasBeenResized) {
                                    newSelChild.hasBeenResized = true;
                                    newSelChild.resize();
                                    var gridEls = angular.element(newSelChild.domNode).find('.dgrid-grid');
                                    if(gridEls.length > 0) {
                                        angular.forEach(gridEls,function(gridEl) {
                                            var grid = dojoDepends.registry.byId(gridEl.id);
                                            if(grid) {
                                                grid.resize();
                                            }

                                        });
                                    }
                                }

                            }
                        });
                    });
                }

				dojoParse();
			}
		};
	});

});

},
'vsd/common/ng/directives/sngDragSort':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
	directive('sngDragSort', ['$rootScope', function ($rootScope) {
		return {
			restrict: 'A',
			replace:  false,
			scope:    {
				'sngDisabled':  '&',
				'sngStartDrag': '&',
				'sngEndDrag':   '&'
			},

			link: function (scope, element, attrs, ctrl) {

				if (!scope.sngDisabled()) {
					element.sortable({
							axis:    "y",
							opacity: 0.7,
							revert:  500,
							start:   function (e, ui) {
								scope.sngStartDrag({e: e, ui: ui});
							},
							update:  function (e, ui) {
								scope.sngEndDrag({e: e, ui: ui});
								safeApply();
							}
						}
					);
				}

				function safeApply() {
					if (!$rootScope.$$phase) {
						scope.$apply();
					}
				}

			}
		};
	}])
;

});

},
'vsd/common/ng/directives/sngDropDownButton':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.directive('sngDropDownButton', ['dojoModules', 'Util', function (dojoModules, Util) {
		return {
			template: "<div></div>",
			replace:  true,
			restrict: 'E',
			priority: 2,
			scope:    {
				'buttonMenu':  '=',
				'iconClass':   '=',
				'label':       '=',
				'showLabel':   '@',
				'sngDisabled': '='
			},

			link: function (scope, element, attrs) {
				//Configure grid properties
				var buttonProperties = {},
					button;

				setButtonProperties();
				//Parse out plugins and add all requested ones to array
				button = new dojoModules.DropDownButton(buttonProperties, element[0]);

				//Apply styles to widget
				Util.applyStyles(button, attrs.style);

				//Update menu items when array changes
				scope.$watchCollection('buttonMenu', function (newVal, oldVal) {
					if (newVal != oldVal) {
						createMenu();
					}
				});

				//Enabled/disable button when expression changes
				scope.$watch('sngDisabled', function (newVal, oldVal) {
					button.set('disabled', newVal);
				});


				function createMenu() {
					var ddMenu = new dojoModules.DropDownMenu({ style: "display: none;"});
					angular.forEach(scope.buttonMenu, function (item) {
						if (item.separator) {
							ddMenu.addChild(new dojoModules.MenuSeparator());
						}
						else {
							ddMenu.addChild(new dojoModules.MenuItem({
								label:     item.label,
								iconClass: item.iconClass,
								onClick:   function () {
									item.onClick(item.value);
									scope.$apply();
								}
							}));
						}
					});
					button.set('dropDown', ddMenu);
				}


				function setButtonProperties() {
					if (attrs.dojoProps) {
						buttonProperties = Util.parseProps(scope.dojoProps, scope);
					}
					//Configure named grid properties
					buttonProperties.style = angular.isDefined(scope.style) ? scope.style : "";
					buttonProperties.iconClass = attrs.iconClass;
					buttonProperties.showLabel = attrs.showLabel && attrs.showLabel === "false" ? false : true;
					if (attrs.label) {
						buttonProperties.label = scope.label;
						scope.$watch('label', function (newVal, oldVal) {
							if (newVal !== oldVal) {
								button.set('label', newVal);
							}
						});
					}
				}

			}
		};
	}]);

});

},
'vsd/common/ng/directives/sngIndentedDropDown':function(){
'use strict';

angular.module('vsd')
    .directive('sngIndentedDropDown', ['dojoDepends','Util', function (dojoDepends, Util) {
        return {
            template: "<div></div>",
            replace: true,
            restrict: 'E',
            scope:   {
                sngModel:            '=',
                sngChange:           '=',
                sngEntries:          '=',
                sngEntryReturnId:    '&',
                sngSelectFirstEntry: '&',
                idProperty:          '&',
                childProperty:       '&',
                indentedClass:       '&',
                parentLabelPrefix:   '@',
                childLabelPrefix:    '@',
	             dojoProps :          '@'
            },

            link: function(scope, element, attrs) {
                require(["dojo/ready",
                         "dijit/form/FilteringSelect",
                         "dojo/on",
                         "dojo/store/Memory"
                        ],
                    function (ready,
                              FilteringSelect,
                              on,
                              Memory)
                    {
                        ready(function () {
                            var properties = {},
                                sngIdProperty = "",
                                isModelEntryObject = !angular.isDefined(attrs.sngEntryReturnId),
                                selectFirstEntry = angular.isDefined(attrs.sngSelectFirstEntry),
                                dojoWidget,
                                dropDownData;

                            if (attrs.dojoProps) {
                                properties = Util.parseProps(scope.dojoProps, scope);
                            }
                            //Add labelAttr attribute to use custom label created in this directive
                            properties.labelAttr = "label";
                            properties.labelType = 'html';

                            if (attrs.sngIdProperty) {
                                sngIdProperty = attrs.sngIdProperty;
                            }
                            //Create drop down
                            dojoWidget= new FilteringSelect(properties, element[0]);

                            //Create a memory store for the entries
                            dojoWidget.set('store', new Memory({data: [], idProperty: sngIdProperty}));
                            scope.$watchCollection('sngEntries', function (newVal) {
                                //Prepare hierarchical data
                                dropDownData = createHierarchicalItemList(scope.sngEntries);
                                updateStoreWidget(dropDownData);
                            });

                            //Watch model property
                            scope.$watch('sngModel', function (newValue, oldValue) {
                                var widgetValue = getWidgetValue(newValue);
                                if (angular.isDefined(widgetValue)) {
                                    dojoWidget.set('value', widgetValue);
                                    if (attrs.sngChange && newValue !== oldValue) {
                                        scope.sngChange();
                                    }
                                }
                                else {
                                    dojoWidget.reset();
                                }
                            });

                            //Handle selections
                            on(dojoWidget, "change", function (newValue) {
                                //If the user has selected a parent entry, don't accept selection and reopen box
                                if(dojoWidget.item && !dojoWidget.item.parentId) {
                                    dojoWidget.set('displayedValue','');
                                    dojoWidget.openDropDown();
                                }
                                else {
                                    if (sngIdProperty && isModelEntryObject) {
                                        scope.sngModel = dojoWidget.item;
                                    }
                                    else {
                                        scope.sngModel = newValue;
                                    }
                                    scope.$apply();
                                }
                            });

                            function updateStoreWidget(data) {
                                var entries = data || [],
                                    selValue = dojoWidget.get('value'),
                                    query = {},
                                    result;

                                dojoWidget.store.setData(entries);

                                query[sngIdProperty] = selValue;
                                result = dojoWidget.store.query(query);
                                // if current value is not in new list, reset the value and select the first one if needed
                                if (!result.length) {
                                    dojoWidget.reset();
                                    if (selectFirstEntry && entries.length) {
                                        selValue = entries[0][sngIdProperty];
                                        if (angular.isDefined(selValue)) {
                                            dojoWidget.set('value', selValue);
                                        }
                                    }
                                }
                            }

                            function getWidgetValue(model) {
                                var value = model;
                                if (model && sngIdProperty && isModelEntryObject) {
                                    value = model[sngIdProperty];
                                }
                                return value === null ? "" : value;
                            }

                            function createHierarchicalItemList(inData) {
                                var outData = [];
                                angular.forEach(inData, function(parent) {
                                    var parentLabel = attrs.parentLabelPrefix ? attrs.parentLabelPrefix + ":  " + parent.name : parent.name;
                                    parent.label = parentLabel;
                                    outData.push(parent);
                                    angular.forEach(parent[attrs.childProperty], function(child) {
                                        var childNameAndPrefix = attrs.childLabelPrefix ? attrs.childLabelPrefix + ":  " + child.name : child.name;
                                        child.label = '<span class="' + attrs.indentedClass + '">' + childNameAndPrefix + "</span>";
                                        child.parentId = parent[attrs.sngIdProperty];
                                        outData.push(child);
                                    });
                                });
                                return outData;
                            }

                        });
                    });
            }
        };
    }]);

},
'vsd/common/ng/directives/sngLayoutContainer':function(){
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
	directive('sngLayoutContainer', function (Util, dojoModules) {
		return {
			restrict:   "A",
			replace:    false,
			transclude: false,
			scope:      true,
			link:       function (scope, element, attrs, ctrl) {
				require([attrs.sngLayoutContainer,
				         "dojo/on"],
					function (DojoWidget, on) {

						var properties = {},
							resizeHandle,
							aspectHandle;

						scope.$on("$destroy", function () {

							resizeHandle.remove();
							aspectHandle.remove();
							scope.dojoWidget.destroyRecursive();

						});

						if (attrs.dojoProps) {
							properties = Util.parseProps(attrs.dojoProps, scope);
						}

						scope.dojoWidget = new DojoWidget(properties, element[0]);

						resizeHandle = on(window, 'resize', function () {
							if (scope.dojoWidget.domNode) {
								scope.dojoWidget.resize();
							}
						});

						aspectHandle = dojoModules.aspect.after(scope.dojoWidget, "resize", function(value){
							scope.$broadcast("WINDOW_RESIZE");
						});

					});
			}
		};
	});
});

},
'vsd/common/ng/directives/sngListButton':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.directive('sngListButton', ['dojoModules', 'Util', function (dojoModules, Util) {
		return {
			template: "<div></div>",
			replace:  true,
			restrict: 'E',
			scope:    {
				'list':             '=',
				'selectedItem':     '=',
				'listType':         '@',
				'labelField':       '@',
				'stringValueField': '@',
				'changeHandler':    '&',
				'sngDisabled':      '=',
				'dojoProps': '@'
			},
			link:     function (scope, element, attrs) {
				var properties = {},
					button;

				//Add any configured dojo properties to starting prop set
				if (attrs.dojoProps) {
					angular.extend(properties, Util.parseProps(scope.dojoProps, scope));

				}
				//Instantiate widget
				button = new dojoModules.DropDownButton(properties, element[0]);
				//Apply styles to widget
				Util.applyStyles(scope.button, attrs.style);

				//Watch list for change and populate drop down menu on change
				scope.$watch('list', function (newVal, oldVal) {
					if (newVal) {
						createMenu();
						if (!scope.selectedItem) {
							//see if value=null is in the menu list
							var newVal = _.findWhere(scope.list, { value: null});
							if (!newVal) {
								newVal = scope.list[0];
							}
							setButtonLabel(newVal[attrs.labelField]);
						}

					}
				});

				//Watch selected item for change and set button label when changed
				scope.$watch('selectedItem', function (newVal, oldVal) {
					if (newVal) {
						if (attrs.listType === "object") {
							setButtonLabel(newVal[attrs.labelField]);
						}
						else if (scope.list) {
							var entry = _.find(scope.list, function (currEntry) {
								return currEntry[attrs.stringValueField] === newVal;
							});
							setButtonLabel(entry[attrs.labelField]);
						}
						//TODO: probably need to move changeHandler here instead of in CreateMenu
						// this would allow the changeHandler to run when the value is changed not just on a onClick()
//						if (attrs.changeHandler) {
//							scope.changeHandler();
//						}
					}
				});
				//Enabled/disable button when expression changes
				scope.$watch('sngDisabled', function (newVal, oldVal) {
					button.set('disabled', newVal);
				});

				function setButtonLabel(newLabel) {
					button.set("label", newLabel);
				}

				function createMenu() {
					var ddMenu = new dojoModules.DropDownMenu({ style: "display: none;"});
					angular.forEach(scope.list, function (item) {
						ddMenu.addChild(new dojoModules.MenuItem({
							label:   item[attrs.labelField],
							onClick: function () {
								if (attrs.listType === "object") {
									scope.selectedItem = item;
								}
								else {
									scope.selectedItem = item[attrs.stringValueField]
								}
								setButtonLabel(item.label);
								scope.$apply();   //This apply is so change handler gets updated value
								if (attrs.changeHandler) {
									scope.changeHandler();
									scope.$apply();
								}
							}
						}));
					});
					ddMenu.startup();
					button.set('dropDown', ddMenu);
					button.startup();
				}
			}
		};
	}]);

});

},
'vsd/common/ng/directives/sngMultiSelect':function(){
'use strict';

angular.module('vsd').
    directive('sngMultiSelect', function ($rootScope, Util, vsdUtil) {
      return {
        restrict: "E",
        replace: true,
        template: "<span></span>",
        transclude: false,
        scope: {
          'sngEntries': '=',      // Array of objects
          'sngElementId': '@',    // The ID to be assigned to the select element
          'sngIdProperty': '@',   // Object id property
          'sngModel': '=',        // Selected id values (corresponds to dojo widget "value" property)
          'sngClick': '&',
          'sngChange': '&',
          'sngShowTitle': '@',
          'sngDisabled': '=',
          'dojoProps': '@'

        },
        link: function (scope, element, attrs) {
          require(["dijit/form/MultiSelect"],
              function (MultiSelect) {
                var properties = {},
                    dojoWidget;

                // Parse dojoProps string into a javascript object
                if (attrs.dojoProps) {
                  properties = Util.parseProps(scope.dojoProps, scope);
                }

                properties.id = scope.sngElementId;

                // Assign style class
                if (attrs['class']) {
                  properties['class'] = attrs['class'];
                }

                // Create the dojo widget using the assigned properties
                dojoWidget = new MultiSelect(properties, element[0]);

                //Apply styles to widget
                Util.applyStyles(dojoWidget, attrs.style);

                //Handle click event
                if (attrs.sngClick) {
                  dojoWidget.on('click', function (evt) {
                    //console.log("click newValue=",evt);
                    scope.sngClick(evt);
                    safeApply();
                  });
                }

                //Handle disabled setting
                if (attrs.sngDisabled) {
                  scope.$watch('sngDisabled', function (newVal) {
                    dojoWidget.set('disabled', newVal);
                  });
                }

                //Handle sngEntries changes
                scope.$watchCollection('sngEntries', function (newVal) {
                  updateOptions(newVal);
                });

                //Handle sngModel changes
                scope.$watch('sngModel', function (newValue) {
                  var widgetValue = dojoWidget.get('value');
                  //console.log("watch sngModel newValue=",newValue, "\n  oldValue=",oldValue, "\n  widgetValue=",widgetValue );

                  if (widgetValue !== newValue) {
                    dojoWidget.set('value', newValue);
                  }

                  if (attrs.sngChange) {
                    //scope.sngChange({values: {newValue: newValue, oldValue: oldValue}});
                    scope.sngChange();
                  }
                });

                // Handle dojo widget change
                // newValue contains an array of the selected option id values
                dojoWidget.on("change", function (newValue) {
                  //console.log("dojoWidget change newValue=",newValue, "\n  oldValue=", oldValue);
                  if (attrs.sngModel && scope.sngModel !== newValue) {
                    scope.sngModel = newValue;
                    safeApply();
                  }
                });


                // Update MultiSelect options
                function updateOptions(entries) {

                  var labelAttr = properties.labelAttr ? properties.labelAttr : (attrs.sngLabelProperty ? $scope.sngLabelProperty : 'label'),
                      valueAttr = properties.valueAttr ? properties.valueAttr : (attrs.sngIdProperty ? scope.sngIdProperty : 'value');

                  // Remove existing options
                  dojo.query('option', dojoWidget.domNode).forEach(function (opt) {
                    dojo.destroy(opt);
                  });

                  // Add new options
                  angular.forEach(entries, function (opt) {
                    var selected = false,
                        option = null;
                    if (scope.sngModel && opt && opt[valueAttr]) {
                      // Selected ids are converted into strings when options are rendered,
                      //  therefore sngModel entries will always be strings.
                      selected = _.contains(scope.sngModel, opt[valueAttr].toString());
                      option = dojo.create('option', {
                        innerHTML: opt[labelAttr] || opt[valueAttr],    // use value if label is not set
                        value: opt[valueAttr],
                        selected: selected,
                        title: (attrs.sngShowTitle) ? opt[labelAttr] || opt[valueAttr] : null
                      });
                      if (option !== null) {
                        dojoWidget.domNode.add(option);
                      }
                    }
                  });
                }

                function safeApply() {
                  if (!$rootScope.$$phase) {
                    scope.$apply();
                  }
                }

              });
        }
      };
    });



},
'vsd/common/ng/directives/sngStringNumberSpinner':function(){
'use strict';

angular.module('vsd').
	directive('sngStringNumberSpinner', function (Util, dojoDepends) {
		return {
			restrict:   "A",
			replace:    false,
			transclude: false,
			scope:      {
				dojoProps:   '@',
                sngModel:    '=',
                sngChange:   '&',
                sngDisabled: '=',
                numberType:  '@'
			},
			link:       function (scope, element, attrs, ctrl) {
				require(['dijit/form/NumberSpinner', 'dojo/on'], function (NumberSpinner, on) {

                    //Set widget properties
					var properties = {};
					if (attrs.dojoProps) {
						properties = Util.parseProps(scope.dojoProps, scope);
					}
                    //Create widget
					var w = new NumberSpinner(properties, element[0]);

					//Apply styles to widget
					Util.applyStyles(w, attrs.style);

                    //Watch model for changes and update widget with numeric value
                    scope.$watch('sngModel',function(newVal,oldVal) {
                        if(angular.isDefined(newVal)) {
                            var numericModelVal = numberStringConverter(newVal,true,attrs.numberType);
                            w.set('value',numericModelVal);
                            if (attrs.sngChange && newVal !== oldVal) {
                                scope.sngChange();
                            }
                        }
                    });
                    //Watch widget for changes and update model with string value
					on(w, 'change', function (newVal) {
                        if(angular.isDefined(newVal)) {
                            scope.sngModel = numberStringConverter(newVal,false);
                            scope.$apply();
                        }
					});

                    //Handle disabled setting
                    if (attrs.sngDisabled) {
                        scope.$watch('sngDisabled', function (newVal) {
                            w.set('disabled', newVal);
                        });
                    }


                    function numberStringConverter(input,incoming,type) {
                        if(incoming) {
                            if(type === "INTEGER") {
                                return parseInt(input);
                            }
                            else {
                                return parseFloat(input);
                            }
                        }
                        else {
                            return String(input);
                        }


                    }
				});
			}
		};
	});

},
'vsd/common/ng/directives/sngTooltipDialog':function(){
'use strict';

angular.module('vsd').
	directive('sngTooltipDialog', ['Util','$rootScope','d3UtilsSvc','$timeout', 'dojoModules',
                                  function (Util, $rootScope,d3UtilsSvc,$timeout, dojoModules) {
		return {
			restrict:   "A",
			replace:    false,
			transclude: true,
			template:   "<div ng-transclude></div>",
			scope:      {
				dojoProps:       '@',
				show:            '=',
				cancel:          '&',
				popupAroundNode: '=',
                absXPos:         '&',
                absYPos:         '&',
			    orient:          '@'
			},
			link:       function (scope, element, attrs, ctrl) {

				require(['dijit/TooltipDialog','dijit/popup'],
					function (TooltipDialog,popup) {

						var properties = {},
							dialog = null;

                        scope.$on("$destroy",function() {
                            if(dialog) {
                                popup.close(dialog);
                            }
                        });

						if (attrs['class']) {
							properties['class'] = attrs['class'];
						}

						if (attrs.dojoProps) {
							angular.extend(properties, Util.parseProps(scope.dojoProps, scope));
						}

						/*
						properties.onMouseLeave = function () {
							closeTooltipPopup();
						};
                  */

						scope.$watch('show', function (newVal, oldVal) {
							if (newVal !== oldVal) {
								if (newVal) {
                                    openTooltipPopup();
								}
								else {
									closeTooltipPopup();
								}
							}
						});

						function openTooltipPopup() {
                            if(scope.show) {
                                if (!dialog) {
                                    dialog = new TooltipDialog(properties, element[0]);
                                }

                                // If you set onMouseLeave property of the dialog, then the dialog will close when
                                // user mouses over the enclosing popup div's scrollbar.
                                // Need to set mouseleave of the popup div to be able to scroll its content.
                                dojoModules.query(".dijitTooltipDialogPopup",document.body).forEach(function(node) {
                                   if (!node.onmouseleave) {
                                      node.onmouseleave = closeTooltipPopup;
                                   }
                                });

                                var openProps = {
                                    popup: dialog
                                };

                                if(attrs.popupAroundNode) {
                                    openProps.around =  scope.popupAroundNode;
                                    d3UtilsSvc.addOffSetWidthAndHeight(scope.popupAroundNode);
                                }
                                else {
                                    openProps.x = parseFloat(attrs.x);
                                    openProps.y = parseFloat(attrs.y);
                                }


                                if (attrs.orient) {
                                    openProps.orient = scope.orient.split(",");
//                                    openProps.orient = ['above-centered'];
                                }
                                //First render off screen
                                //scope.$apply();
	                             popup.moveOffScreen(dialog);

                                $timeout(function(){
                                    popup.open(openProps);
                                });
                            }
						}

						function closeTooltipPopup() {
							popup.close(dialog);
							scope.cancel();
							safeApply();
                     angular.element(scope.popupAroundNode).blur();
						}

						function safeApply() {
							if (!$rootScope.$$phase) {
								scope.$apply();
							}
						}



					});
			}
		};
	}]);

},
'vsd/common/ng/directives/sngTree':function(){
'use strict';

angular.module('vsd').
	directive('sngTree', ['$rootScope',
                         'Util',
                         'dojoDepends',
   function (
	   $rootScope,
	   Util,
	   dojoDepends) {

		return {
			restrict:   "E",
			replace:    true,
			template:   "<span></span>",
			transclude: false,
			scope:      {
				'dojoProps':              '@',  // optional dijit/Tree properties
				'treeData':               '=',  // data to populate tree (required)
				'labelProperty':          '@',  // node label property (optional, will default to 'label')
				'idProperty':             '@',  // node id property (optional, will default to 'id')
				'parentProperty':         '@',  // node parent property (optional, will default to 'parentId')
				'paths':                  '=',  // initially selected tree paths (optional)
				'selectedItem':           '=',  // selected item in tree (optional)
				'getChildren':            '=',  // getChildren handler for tree  (optional)
				'mayHaveChildren':        '=',  // mayHaveChildren handler for tree (optional)
				'sngClick':               '&',  // node click handler
				'sngDblClick':            '&',  // node double click handler
				'queryHandler':           '=',  // query handler  (optional)
				'queryText':              '='   // query text     (optional)
			},
			link:       function (scope, element, attrs, ctrl) {
				require([
					"dijit/Tree",
					"dojo/ready",
					"dojo/store/Memory",
					'dijit/tree/ObjectStoreModel',
				   "dojo/on"],
					function (
						Tree,
						ready,
						Memory,
						ObjectStoreModel,
						on) {

						ready(function () {    //FIXME:  Possible race condition between columns being read (since they required dojo resource bunde) and this function running.  IF it runs before colunns ready, there are no columns in teh grid

							var properties = {};

							scope.labelProperty = attrs.labelProperty ? scope.labelProperty : 'label';
							scope.idProperty = attrs.idProperty ? scope.idProperty : 'id';
							scope.parentProperty = attrs.parentProperty ? scope.parentProperty : 'parentId';

							if (attrs.dojoProps) {
								properties = Util.parseProps(scope.dojoProps, scope);
							}

							if (attrs.treeData && scope.treeData.length > 0) {
								createTree();
							}

							//Watch the tree data to see when the data property changes
							scope.$watch('treeData', function (newData) {
								if (newData.length > 0) {
									if (!scope.tree) {
										createTree();
									}
									else {
										updateModel(newData);
									}
								}
							});

							//If query configured watch for query text changes to refresh grid
							/*
							if (attrs.queryHandler) {
								scope.$watch('queryText', function (newVal) {
									if (scope.tree && scope.tree.model) {
										// Todo: implement query handler
									}
								});
							}
							*/


							function createTree() {
								properties.model = createModel();
								scope.tree = new Tree(properties, element[0]);

								//Apply styles to widget
								Util.applyStyles(scope.tree, attrs.style);

								if (attrs.paths) {
									scope.tree.set('paths', scope.paths);
								}

								//Hook up listener for selection of cell
								on(scope.tree, 'click', function(item) {
									if (attrs.selectedItem) {
											scope.selectedItem = scope.tree.selectedItem;
									}
									if (attrs.sngClick) {
										scope.sngClick();
									}
									scope.$apply();
								});


								//Hook up double click event
								on(scope.tree, 'dblClick', function (item) {
									if (attrs.sngDblClick) {
										scope.sngDblClick(item);
									}
									scope.$apply();
								});


								// sets to single select, might want to make this an attribute
								scope.tree.dndController.singular = true;

								if (attrs.selectedItem) {
									// This isn't working yet, need to figure out how to set tree path or not use this attribute to set the selected item,
									// only to bind in the other direction
									var path = [0, scope.selectedItem[scope.parentProperty], scope.selectedItem[scope.idProperty]];
									scope.tree.set('path', path).then(scope.tree.focusNode(scope.tree._itemNodesMap[scope.selectedItem.id]));
								}
							}

							function createModel() {
								// Create the model
								var model = new ObjectStoreModel({
									store: createStore(),
									query: {id: scope.treeData[0].id},
									labelAttr: scope.labelProperty ? scope.labelProperty : "label"
								});
								return model;
							}

							function updateModel(treeData) {
								scope.tree.set('model', createModel());
							}

							function createStore() {
								var store = new Memory({
									data:            scope.treeData,
									idProperty:      scope.idProperty,
									getChildren:     attrs.getChildren ? scope.getChildren : function(item, onComplete, onError) {
										 var queryProps = {};
										 queryProps[scope.parentProperty] = item.id;
										 return this.query(queryProps);
									},
									mayHaveChildren: attrs.mayHaveChildren ? scope.mayHaveChildren : function(item) {
										 return item.hasChildren;
									}
								});
								return store;
							}

						});
					});
			}
		};
	}]);

},
'vsd/common/ng/services/arrayReorderSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .factory('arrayReorderSvc', ['dojoModules', function(dojoModules) {
        return {
            createReorderMenuItems: function(params) {
                var a = params.a,
                    getMinIdx = angular.isDefined(params.getMinIdx) ? params.getMinIdx : function(){return 0;},
                    getMaxIdx = angular.isDefined(params.getMaxIdx) ? params.getMaxIdx : function(){return a.length -1;},
                    defaultCallback =  angular.isDefined(params.defaultCallback) ? params.defaultCallback : this.reorderArrayItems,
                    moveFirstCallback =  angular.isDefined(params.moveFirstCallback) ? params.moveFirstCallback : null,
                    moveLastCallback =  angular.isDefined(params.moveLastCallback) ? params.moveLastCallback : null;


                return function (idx) {
                    var menuItem,
                        reorderMenuItem = {
                            isSubMenu: true,
                            label: dojoModules.resources.scenario_ScenarioPartChart_moveColumnLbl_txt,
                            menuItems: []
                        };
                    if(idx > getMinIdx()) {
                        menuItem = {label: dojoModules.resources.scenario_ScenarioPartChart_moveFirstLbl_txt, onClick: function(evt) {
                            moveItem({startIdx: idx, targetIdx: getMinIdx()},moveFirstCallback);
                        }};
                        reorderMenuItem.menuItems.push(menuItem);
                        if(idx > getMinIdx() + 1) {
                            menuItem = {label: dojoModules.resources.scenario_ScenarioPartChart_moveLeftLbl_txt, onClick: function(evt){
                                moveItem({startIdx: idx, targetIdx: idx - 1});
                            }};
                            reorderMenuItem.menuItems.push(menuItem);
                        }
                    }
                    if(idx < (getMaxIdx())) {
                        if(idx <  getMaxIdx() - 1) {
                            menuItem = {label: dojoModules.resources.scenario_ScenarioPartChart_moveRightLbl_txt, onClick: function(evt){
                                moveItem({startIdx: idx, targetIdx: idx + 1});
                            }};
                            reorderMenuItem.menuItems.push(menuItem);
                        }
                        menuItem = {label: dojoModules.resources.scenario_ScenarioPartChart_moveLastLbl_txt, onClick: function(evt) {
                            moveItem({startIdx: idx, targetIdx: getMaxIdx()},moveLastCallback);
                        }};
                        reorderMenuItem.menuItems.push(menuItem);
                    }

                    return reorderMenuItem;

                    function moveItem(moveParams,priorityCallback) {
                        if(priorityCallback) {
                            priorityCallback(moveParams, a);
                        }
                        else if(defaultCallback) {
                            defaultCallback(moveParams, a);
                        }
                        else {
                            this.reorderArrayItems(moveParams, a);
                        }
                    }

                };
            },

            createReorderMenuDojoPopupMenuItem: function(generator,idx) {
                var item = generator(idx);
                var subMenu;
                subMenu = new dojoModules.Menu({label: item.label})
                item.menuItems.forEach(function(subMenuItem) {
                    var menuItem = new dojoModules.MenuItem({
                        label: subMenuItem.label,
                        iconClass: subMenuItem.iconClass,
                        onClick: subMenuItem.onClick
                    });
                    subMenu.addChild(menuItem);
                });
                return new dojoModules.PopupMenuItem({label: item.label, popup: subMenu});
            },

            reorderArrayItems: function (locations, a) {
                //Remove the item from the array
                var tgtCol = a.splice(locations.startIdx,1)[0];
                //Insert column in new position
                a.splice(locations.targetIdx,0,tgtCol);
            }
        };
    }]);


});

},
'vsd/common/ng/services/columnNameValidationSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .factory('columnNameValidationSvc', [ 'reservedWords', function (reservedWords) {

        var myColumnNameValidationSvc = {};
        var reservedColNames = ['vsd_deployment_id', 'vsd_window_name', 'vsd_scenario_id', 'vsd_scenario_name', 'vsd_risk_flg', 'vsd_entity_key', 'vsd_entity_type', 'vsd_autogen_key', 'vsd_auto_rec_count', 'vsd_alert'];

        function notAllowed(name, reservedNames) {
            return _.some(reservedNames, function (reservedName) {
                return name.toLowerCase() === reservedName;
            });
        }

        myColumnNameValidationSvc.isReserved = function (name) {
            if(name === ""){
                return false;
            }
            var isReservedColName = notAllowed(name, reservedColNames);
            var isReservedWord = notAllowed(name, reservedWords.esp) || notAllowed(name, reservedWords.lasr);

            return isReservedColName || isReservedWord;
        };

        return myColumnNameValidationSvc;
    }])
;


});

},
'vsd/common/ng/services/encoderSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('encoderSvc', [ function () {

		var hex = [];
		for (var c = 0; c < 0xFF; c++) {
			if (c >= 0x30 && c <= 0x39 || c >= 0x41 && c <= 0x5A || c >= 0x61 && c <= 0x7A) {
				hex[c] = null;
			}
			else {
				hex[c] = c.toString(16);
			}
		}

		var entityToCharacterMap = [];
		entityToCharacterMap["&quot"] = "34";
		/* 34 : quotation mark */
		entityToCharacterMap["&amp"] = "38";
		/* 38 : ampersand */
		entityToCharacterMap["&lt"] = "60";
		/* 60 : less-than sign */
		entityToCharacterMap["&gt"] = "62";
		/* 62 : greater-than sign */
		entityToCharacterMap["&nbsp"] = "160";
		/* 160 : no-break space */
		entityToCharacterMap["&iexcl"] = "161";
		/* 161 : inverted exclamation mark */
		entityToCharacterMap["&cent"] = "162";
		/* 162  : cent sign */
		entityToCharacterMap["&pound"] = "163";
		/* 163  : pound sign */
		entityToCharacterMap["&curren"] = "164";
		/* 164  : currency sign */
		entityToCharacterMap["&yen"] = "165";
		/* 165  : yen sign */
		entityToCharacterMap["&brvbar"] = "166";
		/* 166  : broken bar */
		entityToCharacterMap["&sect"] = "167";
		/* 167  : section sign */
		entityToCharacterMap["&uml"] = "168";
		/* 168  : diaeresis */
		entityToCharacterMap["&copy"] = "169";
		/* 169  : copyright sign */
		entityToCharacterMap["&ordf"] = "170";
		/* 170  : feminine ordinal indicator */
		entityToCharacterMap["&laquo"] = "171";
		/* 171 : left-pointing double angle quotation mark */
		entityToCharacterMap["&not"] = "172";
		/* 172  : not sign */
		entityToCharacterMap["&shy"] = "173";
		/* 173  : soft hyphen */
		entityToCharacterMap["&reg"] = "174";
		/* 174  : registered sign */
		entityToCharacterMap["&macr"] = "175";
		/* 175  : macron */
		entityToCharacterMap["&deg"] = "176";
		/* 176  : degree sign */
		entityToCharacterMap["&plusmn"] = "177";
		/* 177 : plus-minus sign */
		entityToCharacterMap["&sup2"] = "178";
		/* 178  : superscript two */
		entityToCharacterMap["&sup3"] = "179";
		/* 179  : superscript three */
		entityToCharacterMap["&acute"] = "180";
		/* 180  : acute accent */
		entityToCharacterMap["&micro"] = "181";
		/* 181  : micro sign */
		entityToCharacterMap["&para"] = "182";
		/* 182  : pilcrow sign */
		entityToCharacterMap["&middot"] = "183";
		/* 183  : middle dot */
		entityToCharacterMap["&cedil"] = "184";
		/* 184  : cedilla */
		entityToCharacterMap["&sup1"] = "185";
		/* 185  : superscript one */
		entityToCharacterMap["&ordm"] = "186";
		/* 186  : masculine ordinal indicator */
		entityToCharacterMap["&raquo"] = "187";
		/* 187 : right-pointing double angle quotation mark */
		entityToCharacterMap["&frac14"] = "188";
		/* 188  : vulgar fraction one quarter */
		entityToCharacterMap["&frac12"] = "189";
		/* 189  : vulgar fraction one half */
		entityToCharacterMap["&frac34"] = "190";
		/* 190  : vulgar fraction three quarters */
		entityToCharacterMap["&iquest"] = "191";
		/* 191  : inverted question mark */
		entityToCharacterMap["&Agrave"] = "192";
		/* 192  : Latin capital letter a with grave */
		entityToCharacterMap["&Aacute"] = "193";
		/* 193  : Latin capital letter a with acute */
		entityToCharacterMap["&Acirc"] = "194";
		/* 194  : Latin capital letter a with circumflex */
		entityToCharacterMap["&Atilde"] = "195";
		/* 195  : Latin capital letter a with tilde */
		entityToCharacterMap["&Auml"] = "196";
		/* 196  : Latin capital letter a with diaeresis */
		entityToCharacterMap["&Aring"] = "197";
		/* 197  : Latin capital letter a with ring above */
		entityToCharacterMap["&AElig"] = "198";
		/* 198  : Latin capital letter ae */
		entityToCharacterMap["&Ccedil"] = "199";
		/* 199  : Latin capital letter c with cedilla */
		entityToCharacterMap["&Egrave"] = "200";
		/* 200  : Latin capital letter e with grave */
		entityToCharacterMap["&Eacute"] = "201";
		/* 201  : Latin capital letter e with acute */
		entityToCharacterMap["&Ecirc"] = "202";
		/* 202  : Latin capital letter e with circumflex */
		entityToCharacterMap["&Euml"] = "203";
		/* 203  : Latin capital letter e with diaeresis */
		entityToCharacterMap["&Igrave"] = "204";
		/* 204  : Latin capital letter i with grave */
		entityToCharacterMap["&Iacute"] = "205";
		/* 205  : Latin capital letter i with acute */
		entityToCharacterMap["&Icirc"] = "206";
		/* 206  : Latin capital letter i with circumflex */
		entityToCharacterMap["&Iuml"] = "207";
		/* 207  : Latin capital letter i with diaeresis */
		entityToCharacterMap["&ETH"] = "208";
		/* 208  : Latin capital letter eth */
		entityToCharacterMap["&Ntilde"] = "209";
		/* 209  : Latin capital letter n with tilde */
		entityToCharacterMap["&Ograve"] = "210";
		/* 210  : Latin capital letter o with grave */
		entityToCharacterMap["&Oacute"] = "211";
		/* 211  : Latin capital letter o with acute */
		entityToCharacterMap["&Ocirc"] = "212";
		/* 212 : Latin capital letter o with circumflex */
		entityToCharacterMap["&Otilde"] = "213";
		/* 213 : Latin capital letter o with tilde */
		entityToCharacterMap["&Ouml"] = "214";
		/* 214 : Latin capital letter o with diaeresis */
		entityToCharacterMap["&times"] = "215";
		/* 215 : multiplication sign */
		entityToCharacterMap["&Oslash"] = "216";
		/* 216 : Latin capital letter o with stroke */
		entityToCharacterMap["&Ugrave"] = "217";
		/* 217 : Latin capital letter u with grave */
		entityToCharacterMap["&Uacute"] = "218";
		/* 218 : Latin capital letter u with acute */
		entityToCharacterMap["&Ucirc"] = "219";
		/* 219 : Latin capital letter u with circumflex */
		entityToCharacterMap["&Uuml"] = "220";
		/* 220 : Latin capital letter u with diaeresis */
		entityToCharacterMap["&Yacute"] = "221";
		/* 221 : Latin capital letter y with acute */
		entityToCharacterMap["&THORN"] = "222";
		/* 222 : Latin capital letter thorn */
		entityToCharacterMap["&szlig"] = "223";
		/* 223 : Latin small letter sharp s, German Eszett */
		entityToCharacterMap["&agrave"] = "224";
		/* 224 : Latin small letter a with grave */
		entityToCharacterMap["&aacute"] = "225";
		/* 225 : Latin small letter a with acute */
		entityToCharacterMap["&acirc"] = "226";
		/* 226 : Latin small letter a with circumflex */
		entityToCharacterMap["&atilde"] = "227";
		/* 227 : Latin small letter a with tilde */
		entityToCharacterMap["&auml"] = "228";
		/* 228 : Latin small letter a with diaeresis */
		entityToCharacterMap["&aring"] = "229";
		/* 229 : Latin small letter a with ring above */
		entityToCharacterMap["&aelig"] = "230";
		/* 230 : Latin lowercase ligature ae */
		entityToCharacterMap["&ccedil"] = "231";
		/* 231 : Latin small letter c with cedilla */
		entityToCharacterMap["&egrave"] = "232";
		/* 232 : Latin small letter e with grave */
		entityToCharacterMap["&eacute"] = "233";
		/* 233 : Latin small letter e with acute */
		entityToCharacterMap["&ecirc"] = "234";
		/* 234 : Latin small letter e with circumflex */
		entityToCharacterMap["&euml"] = "235";
		/* 235 : Latin small letter e with diaeresis */
		entityToCharacterMap["&igrave"] = "236";
		/* 236 : Latin small letter i with grave */
		entityToCharacterMap["&iacute"] = "237";
		/* 237 : Latin small letter i with acute */
		entityToCharacterMap["&icirc"] = "238";
		/* 238 : Latin small letter i with circumflex */
		entityToCharacterMap["&iuml"] = "239";
		/* 239 : Latin small letter i with diaeresis */
		entityToCharacterMap["&eth"] = "240";
		/* 240 : Latin small letter eth */
		entityToCharacterMap["&ntilde"] = "241";
		/* 241 : Latin small letter n with tilde */
		entityToCharacterMap["&ograve"] = "242";
		/* 242 : Latin small letter o with grave */
		entityToCharacterMap["&oacute"] = "243";
		/* 243 : Latin small letter o with acute */
		entityToCharacterMap["&ocirc"] = "244";
		/* 244 : Latin small letter o with circumflex */
		entityToCharacterMap["&otilde"] = "245";
		/* 245 : Latin small letter o with tilde */
		entityToCharacterMap["&ouml"] = "246";
		/* 246 : Latin small letter o with diaeresis */
		entityToCharacterMap["&divide"] = "247";
		/* 247 : division sign */
		entityToCharacterMap["&oslash"] = "248";
		/* 248 : Latin small letter o with stroke */
		entityToCharacterMap["&ugrave"] = "249";
		/* 249 : Latin small letter u with grave */
		entityToCharacterMap["&uacute"] = "250";
		/* 250 : Latin small letter u with acute */
		entityToCharacterMap["&ucirc"] = "251";
		/* 251 : Latin small letter u with circumflex */
		entityToCharacterMap["&uuml"] = "252";
		/* 252 : Latin small letter u with diaeresis */
		entityToCharacterMap["&yacute"] = "253";
		/* 253 : Latin small letter y with acute */
		entityToCharacterMap["&thorn"] = "254";
		/* 254 : Latin small letter thorn */
		entityToCharacterMap["&yuml"] = "255";
		/* 255 : Latin small letter y with diaeresis */
		entityToCharacterMap["&OElig"] = "338";
		/* 338 : Latin capital ligature oe */
		entityToCharacterMap["&oelig"] = "339";
		/* 339 : Latin small ligature oe */
		entityToCharacterMap["&Scaron"] = "352";
		/* 352 : Latin capital letter s with caron */
		entityToCharacterMap["&scaron"] = "353";
		/* 353 : Latin small letter s with caron */
		entityToCharacterMap["&Yuml"] = "376";
		/* 376 : Latin capital letter y with diaeresis */
		entityToCharacterMap["&fnof"] = "402";
		/* 402 : Latin small letter f with hook */
		entityToCharacterMap["&circ"] = "710";
		/* 710 : modifier letter circumflex accent */
		entityToCharacterMap["&tilde"] = "732";
		/* 732 : small tilde */
		entityToCharacterMap["&Alpha"] = "913";
		/* 913 : Greek capital letter alpha */
		entityToCharacterMap["&Beta"] = "914";
		/* 914 : Greek capital letter beta */
		entityToCharacterMap["&Gamma"] = "915";
		/* 915 : Greek capital letter gamma */
		entityToCharacterMap["&Delta"] = "916";
		/* 916 : Greek capital letter delta */
		entityToCharacterMap["&Epsilon"] = "917";
		/* 917 : Greek capital letter epsilon */
		entityToCharacterMap["&Zeta"] = "918";
		/* 918 : Greek capital letter zeta */
		entityToCharacterMap["&Eta"] = "919";
		/* 919 : Greek capital letter eta */
		entityToCharacterMap["&Theta"] = "920";
		/* 920 : Greek capital letter theta */
		entityToCharacterMap["&Iota"] = "921";
		/* 921 : Greek capital letter iota */
		entityToCharacterMap["&Kappa"] = "922";
		/* 922 : Greek capital letter kappa */
		entityToCharacterMap["&Lambda"] = "923";
		/* 923 : Greek capital letter lambda */
		entityToCharacterMap["&Mu"] = "924";
		/* 924 : Greek capital letter mu */
		entityToCharacterMap["&Nu"] = "925";
		/* 925 : Greek capital letter nu */
		entityToCharacterMap["&Xi"] = "926";
		/* 926 : Greek capital letter xi */
		entityToCharacterMap["&Omicron"] = "927";
		/* 927 : Greek capital letter omicron */
		entityToCharacterMap["&Pi"] = "928";
		/* 928 : Greek capital letter pi */
		entityToCharacterMap["&Rho"] = "929";
		/* 929 : Greek capital letter rho */
		entityToCharacterMap["&Sigma"] = "931";
		/* 931 : Greek capital letter sigma */
		entityToCharacterMap["&Tau"] = "932";
		/* 932 : Greek capital letter tau */
		entityToCharacterMap["&Upsilon"] = "933";
		/* 933 : Greek capital letter upsilon */
		entityToCharacterMap["&Phi"] = "934";
		/* 934 : Greek capital letter phi */
		entityToCharacterMap["&Chi"] = "935";
		/* 935 : Greek capital letter chi */
		entityToCharacterMap["&Psi"] = "936";
		/* 936 : Greek capital letter psi */
		entityToCharacterMap["&Omega"] = "937";
		/* 937 : Greek capital letter omega */
		entityToCharacterMap["&alpha"] = "945";
		/* 945 : Greek small letter alpha */
		entityToCharacterMap["&beta"] = "946";
		/* 946 : Greek small letter beta */
		entityToCharacterMap["&gamma"] = "947";
		/* 947 : Greek small letter gamma */
		entityToCharacterMap["&delta"] = "948";
		/* 948 : Greek small letter delta */
		entityToCharacterMap["&epsilon"] = "949";
		/* 949 : Greek small letter epsilon */
		entityToCharacterMap["&zeta"] = "950";
		/* 950 : Greek small letter zeta */
		entityToCharacterMap["&eta"] = "951";
		/* 951 : Greek small letter eta */
		entityToCharacterMap["&theta"] = "952";
		/* 952 : Greek small letter theta */
		entityToCharacterMap["&iota"] = "953";
		/* 953 : Greek small letter iota */
		entityToCharacterMap["&kappa"] = "954";
		/* 954 : Greek small letter kappa */
		entityToCharacterMap["&lambda"] = "955";
		/* 955 : Greek small letter lambda */
		entityToCharacterMap["&mu"] = "956";
		/* 956 : Greek small letter mu */
		entityToCharacterMap["&nu"] = "957";
		/* 957 : Greek small letter nu */
		entityToCharacterMap["&xi"] = "958";
		/* 958 : Greek small letter xi */
		entityToCharacterMap["&omicron"] = "959";
		/* 959 : Greek small letter omicron */
		entityToCharacterMap["&pi"] = "960";
		/* 960 : Greek small letter pi */
		entityToCharacterMap["&rho"] = "961";
		/* 961 : Greek small letter rho */
		entityToCharacterMap["&sigmaf"] = "962";
		/* 962 : Greek small letter final sigma */
		entityToCharacterMap["&sigma"] = "963";
		/* 963 : Greek small letter sigma */
		entityToCharacterMap["&tau"] = "964";
		/* 964 : Greek small letter tau */
		entityToCharacterMap["&upsilon"] = "965";
		/* 965 : Greek small letter upsilon */
		entityToCharacterMap["&phi"] = "966";
		/* 966 : Greek small letter phi */
		entityToCharacterMap["&chi"] = "967";
		/* 967 : Greek small letter chi */
		entityToCharacterMap["&psi"] = "968";
		/* 968 : Greek small letter psi */
		entityToCharacterMap["&omega"] = "969";
		/* 969 : Greek small letter omega */
		entityToCharacterMap["&thetasym"] = "977";
		/* 977 : Greek theta symbol */
		entityToCharacterMap["&upsih"] = "978";
		/* 978 : Greek upsilon with hook symbol */
		entityToCharacterMap["&piv"] = "982";
		/* 982 : Greek pi symbol */
		entityToCharacterMap["&ensp"] = "8194";
		/* 8194 : en space */
		entityToCharacterMap["&emsp"] = "8195";
		/* 8195 : em space */
		entityToCharacterMap["&thinsp"] = "8201";
		/* 8201 : thin space */
		entityToCharacterMap["&zwnj"] = "8204";
		/* 8204 : zero width non-joiner */
		entityToCharacterMap["&zwj"] = "8205";
		/* 8205 : zero width joiner */
		entityToCharacterMap["&lrm"] = "8206";
		/* 8206 : left-to-right mark */
		entityToCharacterMap["&rlm"] = "8207";
		/* 8207 : right-to-left mark */
		entityToCharacterMap["&ndash"] = "8211";
		/* 8211 : en dash */
		entityToCharacterMap["&mdash"] = "8212";
		/* 8212 : em dash */
		entityToCharacterMap["&lsquo"] = "8216";
		/* 8216 : left single quotation mark */
		entityToCharacterMap["&rsquo"] = "8217";
		/* 8217 : right single quotation mark */
		entityToCharacterMap["&sbquo"] = "8218";
		/* 8218 : single low-9 quotation mark */
		entityToCharacterMap["&ldquo"] = "8220";
		/* 8220 : left double quotation mark */
		entityToCharacterMap["&rdquo"] = "8221";
		/* 8221 : right double quotation mark */
		entityToCharacterMap["&bdquo"] = "8222";
		/* 8222 : double low-9 quotation mark */
		entityToCharacterMap["&dagger"] = "8224";
		/* 8224 : dagger */
		entityToCharacterMap["&Dagger"] = "8225";
		/* 8225 : double dagger */
		entityToCharacterMap["&bull"] = "8226";
		/* 8226 : bullet */
		entityToCharacterMap["&hellip"] = "8230";
		/* 8230 : horizontal ellipsis */
		entityToCharacterMap["&permil"] = "8240";
		/* 8240 : per mille sign */
		entityToCharacterMap["&prime"] = "8242";
		/* 8242 : prime */
		entityToCharacterMap["&Prime"] = "8243";
		/* 8243 : double prime */
		entityToCharacterMap["&lsaquo"] = "8249";
		/* 8249 : single left-pointing angle quotation mark */
		entityToCharacterMap["&rsaquo"] = "8250";
		/* 8250 : single right-pointing angle quotation mark */
		entityToCharacterMap["&oline"] = "8254";
		/* 8254 : overline */
		entityToCharacterMap["&frasl"] = "8260";
		/* 8260 : fraction slash */
		entityToCharacterMap["&euro"] = "8364";
		/* 8364 : euro sign */
		entityToCharacterMap["&image"] = "8365";
		/* 8465 : black-letter capital i */
		entityToCharacterMap["&weierp"] = "8472";
		/* 8472 : script capital p, Weierstrass p */
		entityToCharacterMap["&real"] = "8476";
		/* 8476 : black-letter capital r */
		entityToCharacterMap["&trade"] = "8482";
		/* 8482 : trademark sign */
		entityToCharacterMap["&alefsym"] = "8501";
		/* 8501 : alef symbol */
		entityToCharacterMap["&larr"] = "8592";
		/* 8592 : leftwards arrow */
		entityToCharacterMap["&uarr"] = "8593";
		/* 8593 : upwards arrow */
		entityToCharacterMap["&rarr"] = "8594";
		/* 8594 : rightwards arrow */
		entityToCharacterMap["&darr"] = "8595";
		/* 8595 : downwards arrow */
		entityToCharacterMap["&harr"] = "8596";
		/* 8596 : left right arrow */
		entityToCharacterMap["&crarr"] = "8629";
		/* 8629 : downwards arrow with corner leftwards */
		entityToCharacterMap["&lArr"] = "8656";
		/* 8656 : leftwards double arrow */
		entityToCharacterMap["&uArr"] = "8657";
		/* 8657 : upwards double arrow */
		entityToCharacterMap["&rArr"] = "8658";
		/* 8658 : rightwards double arrow */
		entityToCharacterMap["&dArr"] = "8659";
		/* 8659 : downwards double arrow */
		entityToCharacterMap["&hArr"] = "8660";
		/* 8660 : left right double arrow */
		entityToCharacterMap["&forall"] = "8704";
		/* 8704 : for all */
		entityToCharacterMap["&part"] = "8706";
		/* 8706 : partial differential */
		entityToCharacterMap["&exist"] = "8707";
		/* 8707 : there exists */
		entityToCharacterMap["&empty"] = "8709";
		/* 8709 : empty set */
		entityToCharacterMap["&nabla"] = "8711";
		/* 8711 : nabla */
		entityToCharacterMap["&isin"] = "8712";
		/* 8712 : element of */
		entityToCharacterMap["&notin"] = "8713";
		/* 8713 : not an element of */
		entityToCharacterMap["&ni"] = "8715";
		/* 8715 : contains as member */
		entityToCharacterMap["&prod"] = "8719";
		/* 8719 : n-ary product */
		entityToCharacterMap["&sum"] = "8721";
		/* 8721 : n-ary summation */
		entityToCharacterMap["&minus"] = "8722";
		/* 8722 : minus sign */
		entityToCharacterMap["&lowast"] = "8727";
		/* 8727 : asterisk operator */
		entityToCharacterMap["&radic"] = "8730";
		/* 8730 : square root */
		entityToCharacterMap["&prop"] = "8733";
		/* 8733 : proportional to */
		entityToCharacterMap["&infin"] = "8734";
		/* 8734 : infinity */
		entityToCharacterMap["&ang"] = "8736";
		/* 8736 : angle */
		entityToCharacterMap["&and"] = "8743";
		/* 8743 : logical and */
		entityToCharacterMap["&or"] = "8744";
		/* 8744 : logical or */
		entityToCharacterMap["&cap"] = "8745";
		/* 8745 : intersection */
		entityToCharacterMap["&cup"] = "8746";
		/* 8746 : union */
		entityToCharacterMap["&int"] = "8747";
		/* 8747 : integral */
		entityToCharacterMap["&there4"] = "8756";
		/* 8756 : therefore */
		entityToCharacterMap["&sim"] = "8764";
		/* 8764 : tilde operator */
		entityToCharacterMap["&cong"] = "8773";
		/* 8773 : congruent to */
		entityToCharacterMap["&asymp"] = "8776";
		/* 8776 : almost equal to */
		entityToCharacterMap["&ne"] = "8800";
		/* 8800 : not equal to */
		entityToCharacterMap["&equiv"] = "8801";
		/* 8801 : identical to, equivalent to */
		entityToCharacterMap["&le"] = "8804";
		/* 8804 : less-than or equal to */
		entityToCharacterMap["&ge"] = "8805";
		/* 8805 : greater-than or equal to */
		entityToCharacterMap["&sub"] = "8834";
		/* 8834 : subset of */
		entityToCharacterMap["&sup"] = "8835";
		/* 8835 : superset of */
		entityToCharacterMap["&nsub"] = "8836";
		/* 8836 : not a subset of */
		entityToCharacterMap["&sube"] = "8838";
		/* 8838 : subset of or equal to */
		entityToCharacterMap["&supe"] = "8839";
		/* 8839 : superset of or equal to */
		entityToCharacterMap["&oplus"] = "8853";
		/* 8853 : circled plus */
		entityToCharacterMap["&otimes"] = "8855";
		/* 8855 : circled times */
		entityToCharacterMap["&perp"] = "8869";
		/* 8869 : up tack */
		entityToCharacterMap["&sdot"] = "8901";
		/* 8901 : dot operator */
		entityToCharacterMap["&lceil"] = "8968";
		/* 8968 : left ceiling */
		entityToCharacterMap["&rceil"] = "8969";
		/* 8969 : right ceiling */
		entityToCharacterMap["&lfloor"] = "8970";
		/* 8970 : left floor */
		entityToCharacterMap["&rfloor"] = "8971";
		/* 8971 : right floor */
		entityToCharacterMap["&lang"] = "9001";
		/* 9001 : left-pointing angle bracket */
		entityToCharacterMap["&rang"] = "9002";
		/* 9002 : right-pointing angle bracket */
		entityToCharacterMap["&loz"] = "9674";
		/* 9674 : lozenge */
		entityToCharacterMap["&spades"] = "9824";
		/* 9824 : black spade suit */
		entityToCharacterMap["&clubs"] = "9827";
		/* 9827 : black club suit */
		entityToCharacterMap["&hearts"] = "9829";
		/* 9829 : black heart suit */
		entityToCharacterMap["&diams"] = "9830";
		/* 9830 : black diamond suit */

		var characterToEntityMap = [];

		for (var entity in entityToCharacterMap) {
			characterToEntityMap[entityToCharacterMap[entity]] = entity;
		}


		function encodeHtmlCharacter(aImmune, c) {
			if (aImmune.indexOf(c) > -1) {
				return c;
			}

			var hex = getHexForNonAlphanumeric(c);
			if (!hex) {
				return c;
			}

			var cc = c.charCodeAt(0);
			if (( cc <= 0x1f && c !== '\t' && c !== '\n' && c !== '\r' ) || ( cc >= 0x7f && cc <= 0x9f ) || c === ' ') {
				return " ";
			}

			var entityName = characterToEntityMap[cc];
			if (entityName) {
				return entityName + ";";
			}

			return "&#x" + hex + ";";
		}

		function getHexForNonAlphanumeric(c) {
			if (c.charCodeAt(0) < 256) {
				return hex[c.charCodeAt(0)];
			}
			return c.charCodeAt(0).toString(16);
		}


		return {

			encodeForDojo: function (sInput) {
				return this.encodeForJavascript(this.encodeForHtml(sInput));
			},

			encodeForHtml: function (sInput) {
				var out = '',
					aImmune = [',', '.', '-', '_', ' '];
				for (var i = 0; i < sInput.length; i++) {
					var c = sInput.charAt(i);
					out += encodeHtmlCharacter(aImmune, c);
				}
				return out;
			},

			encodeForJavascript: function (sInput) {
				var out = '',
					aImmune = [',', '.', '_'];

				for (var idx = 0; idx < sInput.length; idx++) {
					var ch = sInput.charAt(idx);
					if (aImmune.indexOf(ch) > -1) {
						out += ch;
					}
					else {
						var hex = getHexForNonAlphanumeric(ch);
						if (!hex) {
							out += ch;
						}
						else {
							var tmp = ch.charCodeAt(0).toString(16);
							if (ch.charCodeAt(0) < 256) {
								var pad = "00".substr(tmp.length);
								out += "\\x" + pad + tmp.toUpperCase();
							}
							else {
								pad = "0000".substr(tmp.length);
								out += "\\u" + pad + tmp.toUpperCase();
							}
						}
					}
				}

				return out;
			}
		};
	}
	])
;



});

},
'vsd/common/ng/services/fieldParserSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .factory('fieldParserSvc',  [ 'vsdUtil','dojoModules', function(vsdUtil,dojoModules) {
        return {
            fieldParser: function(inCol, inFieldList, outFieldList, inChopChars) {
                var chopChars = "+-/*%()",
                    identifier = /^[_a-z\u0080-\u024F\u4E00-\u9FFF][_a-z0-9\u0080-\u024F\u4E00-\u9FFF]{0,31}$/i,
                    funcPat = /[_a-z\u0080-\u024F\u4E00-\u9FFF][_a-z0-9\u0080-\u024F\u4E00-\u9FFF]{0,31}\([^)]*\)/i,
                    funcPatGrp = /([_a-z\u0080-\u024F\u4E00-\u9FFF][_a-z0-9\u0080-\u024F\u4E00-\u9FFF]{0,31})\((.*)\)/i,
                    number = /^\-*[0-9.]+$/,
                /*validParamStr2 = /^(([a-z][A-Z](\w)*),)+([a-z][A-Z](\w)*)$/,*/
                    validParamStr = /^(([a-zA-Z\u0080-\u024F\u4E00-\u9FFF]([\w\u0080-\u024F\u4E00-\u9FFF])*),)+([a-zA-Z\u0080-\u024F\u4E00-\u9FFF]([\w\u0080-\u024F\u4E00-\u9FFF])*)$/,
                    compOps = ['=',
                               '<>',
                               '>',
                               '>=',
                               '<',
                               '<='],
                    parseErrors = null,
                    inputFields = inFieldList,
                    outputFields = outFieldList,
                    col = null,
                    UNTERMINATED_QUOTED_STR = 'UNTERMINATED_QUOTED_STR',
                    FIELDS_MULTI_SOURCES = 'FIELDS_MULTI_SOURCES',
                    supportedFunctions = [
                        {
                            name: 'sum',
                            minNumParams: 2,
                            variableParams: true,
                            variableParamsType: "NUMERIC"
                        },
                        {
                            name: 'min',
                            minNumParams: 2,
                            variableParams: true,
                            variableParamsType: "NUMERIC"
                        },
                        {
                            name: 'max',
                            minNumParams: 2,
                            variableParams: true,
                            variableParamsType: "NUMERIC"
                        }
                    ];

                setCol(inCol);

                if(inChopChars) {
                    setChopChars(inChopChars);
                }

                function extractFields() {
                    //Clear parseErrors
                    parseErrors = [];
                    //Parse expression and return fields
                    var fields = extractFieldsHelper(col.expression);
                    fields = _.uniq(fields,false,function(item){ return item.name; });
                    //Ensure all fields from same source
                    if(fields.length > 1) {
                        var firstSrc = fields[0].dataSource;
                        if(!_.every(fields,function(field){ return field.dataSource === firstSrc;})) {
                            parseErrors.push({text: dojoModules.resources.fieldParser_multipleSourcesMsg_txt});
                            return [];
                        }
                    }


                    if(parseErrors.length === 0) {
                        //Put each field into parentColumns member of col
                        fields.forEach(function(parentCol){
                            col.parentColumns.push(parentCol);
                        });
                    }

                    return fields;
                }

                function paramDataTypeValid(paramCol,funcReqs) {
                    if(funcReqs.variableParams && funcReqs.variableParamsType) {
                        return paramCol.dataType === funcReqs.variableParamsType;
                    }
                    return true;
                }

                function splitBooleanExpression(boolExpr) {
                    var operFound = false,
                        opsFoundCount = 0,
                        toks = [],
                        operPos,
                        foundOperPos;
                    for(var i= 0; i < compOps.length;i++) {
                        operPos = boolExpr.indexOf(compOps[i]);
                        if(operPos !== -1) {
                            operFound = true;
                            opsFoundCount++;
                            foundOperPos = operPos;
                        }
                    }
                    if(opsFoundCount === 1) {
                        toks.push(boolExpr.substring(0,foundOperPos).trim());
                        toks.push(boolExpr.substring(foundOperPos+1).trim());
                    }

                    return toks;
                }

                function extractFieldsHelper(expr,withinFunction) {
                    var outFields = [],
                        fieldTokens = [],
                        splitOut,
                        parseError = null,
                        supportedFunc;

                    //Parse fields using regex
                    //splitOut = splitIgnoreQuoted(expr.replace(' ',''),chopChars);
                    fieldTokens = getTokens(expr.replace(' ',''));
                    //Iterate through tokens and capture fields
                    fieldTokens.forEach(function(fieldTok) {
                        fieldTok = fieldTok.trim();
                        //If it's a numeric constant, skip the token
                        if(!isNumericConstant(fieldTok) && !isStringConstant(fieldTok) && fieldTok !== "") {
                            var funcTxt = getFunctionText(fieldTok),
                                funcName,
                                funcParts,
                                errorMsg;
                            //If it is a function, report as unsupported
                            if(fieldTok.match(funcPat)){
                                funcParts = fieldTok.match(funcPatGrp);
                                funcName = funcParts[1]
                                supportedFunc = _.findWhere(supportedFunctions,{name: funcName});
                                if(!supportedFunc) {
                                    errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_unsupportedFunctionMsg_txt,[funcName])
                                    parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                    parseErrors.push(parseError);
                                    return;
                                }
                                //Validate parameters
                                else {
                                    var paramStr = funcParts.length >= 3 ? funcParts[2] : "";

                                    //Ensure parameter string has valid syntax
                                   /* if(!validParamStr.test(paramStr)) {
                                        errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_functionInvalidParameterStrMsg_txt,[funcName]);
                                        parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                        parseErrors.push(parseError);
                                        return;
                                    }*/

                                    if(_.contains(["ifn","ifc"],funcName)) {
                                        var params = paramStr.split(",");
                                        if(params.length !== 3) {
                                            errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_functionXParametersRequiredMsg_txt,[funcName,3]);
                                            parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                            parseErrors.push(parseError);
                                            return;
                                        }
                                        //Validate boolean expression
                                        var boolParts = splitBooleanExpression(params[0]);
                                        if(boolParts.length !== 2) {
                                            errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_invalidBooleanParameterForFunctionMsg_txt,[1,funcName]);
                                            parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                            parseErrors.push(parseError);
                                            return;
                                        }
                                        outFields = _.union(outFields,extractFieldsHelper(boolParts[0],true));
                                        outFields = _.union(outFields,extractFieldsHelper(boolParts[1],true));
                                        outFields = _.union(outFields,extractFieldsHelper(params[1],true));
                                        outFields = _.union(outFields,extractFieldsHelper(params[2],true));
                                    }
                                    else {
                                        var paramStrCols = paramStr.split(/\s*,\s*/);
                                        //Ensure there are at least two parameters
                                        if(paramStrCols.length < 2) {
                                            errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_functionMinParametersRequiredMsg_txt,[2]);
                                            parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                            parseErrors.push(parseError);
                                            return;
                                        }
                                        var error = false;
                                        var paramCnt = 1;
                                        _.each(paramStrCols,function(tok) {
                                            if(tok) {
                                                var isId = identifier.test(tok),
                                                    isNumber = number.test(tok);
                                                if(!isId && !isNumber) {
                                                    errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_functionParameterNotColOrNbr_txt,[paramCnt,funcName]);
                                                    parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                                    parseErrors.push(parseError);
                                                    error = true;
                                                }
                                                else if(isId){
                                                    var col;
                                                    if(inputFields[tok.toLowerCase()]){
                                                        col = inputFields[tok.toLowerCase()];
                                                    }
                                                    else if(outputFields[tok.toLowerCase()]){
                                                        col = outputFields[tok.toLowerCase()];
                                                    }

                                                    if(!col) {
                                                        errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_unknownField_txt,[tok]);
                                                        parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                                        parseErrors.push(parseError);
                                                    }
                                                    else if(!paramDataTypeValid(col,supportedFunc)) {
                                                        errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_functionParameterInvalidParameter_txt,[col.name,funcName]);
                                                        parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                                        parseErrors.push(parseError);
                                                        error = true;
                                                    }
                                                    else if(!error) {
                                                        outFields.push(col);
                                                    }
                                                }
                                                paramCnt++;
                                            }
                                            else {
                                                errorMsg = dojoModules.string.substitute(dojoModules.resources.fieldParser_functionExtraComma_txt,[funcName]);
                                                parseError = {text: errorMsg, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                                parseErrors.push(parseError);
                                                error = true;
                                            }
                                        });

                                        var dataTypeError = false;

                                    }



                                }

                                /*parseError = {text: "Possible function invocation (unsupported): '" + funcName, colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                 parseErrors.push(parseError);*/
                            }

                            else {
                                //Since we provide there are no functions, replace all open and close parens with nothing
                                fieldTok = fieldTok.replace(/\(/,"");
                                fieldTok = fieldTok.replace(/\)/,"");
                                //If it is a legal field, add to list
                                if(inputFields[fieldTok.toLowerCase()]){
                                    outFields.push( inputFields[fieldTok.toLowerCase()]);
                                }
                                else if(outputFields[fieldTok.toLowerCase()]){
                                    outFields.push( outputFields[fieldTok.toLowerCase()]);
                                }
                                //Not a legal field, log error
                                else {
                                    parseError = {text: dojoModules.string.substitute(dojoModules.resources.fieldParser_unknownField_txt,[fieldTok]), colNbr: getErrorColNbr(fieldTok), lineNbr: 1};
                                    parseErrors.push(parseError);
//                                console.log("ERROR:  " + parseError.text + " at (line:column) " + parseError.lineNbr + ":" + parseError.colNbr);
                                }
                            }
                        }
                    });
                    return outFields;
                }

                function getTokens(expr) {
                    var toks = [],
                        nonFuncParts,
                        funcParts;
                    //Split expression into function and non-function components
                    nonFuncParts = expr.split(funcPat);
                    _.each(nonFuncParts,function(part) {
                        var partRes;
                        //If token not empty, split it into additional tokens by operator
                        if(part) {
                            partRes = splitIgnoreQuoted(part,chopChars);
                            if(partRes.error === UNTERMINATED_QUOTED_STR) {
                                parseErrors.push({text: dojoModules.resources.fieldParser_unterminatedString, colNbr: -1, lineNbr: 1});
                            }
                            else {
                                _.each(partRes.toks,function(partTok) {
                                    if(partTok) {
                                        toks.push(partTok);
                                    }
                                });
                            }
                        }
                    });
                    //Find each function invocation and store as token
                    funcParts = expr.match(funcPat);
                    if(funcParts) {
                        toks = _.flatten([toks,funcParts]);
                    }
                    return toks;

                }


                function splitIgnoreQuoted(expr,chopChars) {
                    var exprLen = expr.length,
                        isSingleQuotedStr = false,
                        isDoubleQuotedStr = false,
                        currTok = "",
                        toks = [];
                    for(var i = 0; i < exprLen; i++) {
                        var currChar = expr.charAt(i);
                        //If the current character is NOT a string delimiter
                        if(currChar !== '"' && currChar !== "'") {
                            if(isSingleQuotedStr || isDoubleQuotedStr || chopChars.indexOf(currChar) === -1) {
                                currTok += currChar;
                            }
                            else if(currTok.length > 0){
                                toks.push(currTok);
                                currTok = "";
                            }
                        }
                        //If it is a string delimiter
                        else {
                            if(currChar === "'") {
                                if(isSingleQuotedStr) {
                                    isSingleQuotedStr = false;
                                }
                                else {
                                    isSingleQuotedStr = true;
                                }
                            }
                            else {
                                if(isDoubleQuotedStr) {
                                    isDoubleQuotedStr = false;
                                }
                                else {
                                    isDoubleQuotedStr = true;
                                }
                            }

                            currTok += currChar;
                        }

                    }
                    //See if there is an unterminated quoted string
                    if(isSingleQuotedStr || isDoubleQuotedStr) {
                        return {toks: null, error: UNTERMINATED_QUOTED_STR};
                    }

                    //Account for last token
                    toks.push(currTok);

                    return {toks: toks, error: null};
                }

                function getErrorColNbr(fieldTok) {
                    return col.expression.indexOf(fieldTok) + 1;
                }

                function setChopChars(inChopChars) {
                    chopChars = inChopChars;
                }

                function setCol(inCol) {
                    if(!inCol) {
                        throw new Error(dojoModules.resources.fieldParser_firstParamColRequired);
                    }
                    col = inCol;
                    col.parentColumns = [];
                }

                function isNumericConstant(tok) {
                    var numericRegEx = /^([-+]?[0-9]*\.?[0-9]*)$/;
                    return tok.match(numericRegEx);
                }

                function isStringConstant(tok) {
                    var firstChar = vsdUtil.getLeft(tok,1),
                        lastChar = vsdUtil.getRight(tok,1);
                    if((firstChar === "'" || firstChar === '"') && (lastChar === "'" || lastChar === '"')) {
                        return true;
                    }
                    else {
                        return false;
                    }
                }

                function getFunctionText(tok) {
                    //If it somewhere after teh beginning of the token
                    if(tok.indexOf("(") > 0 ) {
                        return tok;
                    }
                    else {
                        return null;
                    }
                }

                function assignExecutionOrder() {
                    //If not parent columns, assume it is a numeric expression?
                    if(!col.parentColumns || col.parentColumns.length === 0) {
                        col.executionOrder = 1;
                        return 0;
                    }
                    else {
                        var maxExecutionOrder = -1,
                            minExecutionOrder = 9999,
                            inputFields = [],
                            outputFields = [];
                        col.parentColumns.forEach(function(parentCol) {
                            //Store zero and non zero level fields
                            if(parentCol.executionOrder === 0) {
                                inputFields.push(parentCol.name);
                            }
                            else {
                                outputFields.push(parentCol.name);
                            }
                            //Track min/max execution order encountered
                            if(parentCol.executionOrder > maxExecutionOrder) {
                                maxExecutionOrder = parentCol.executionOrder;
                            }
                            if(parentCol.executionOrder < minExecutionOrder) {
                                minExecutionOrder = parentCol.executionOrder;
                            }
                        });
                        //Check to make sure there is no mix of level zero and other levels
                        if(minExecutionOrder === 0 && maxExecutionOrder !== 0) {
//                            var msg =  "Mix of original input and output fields not supported.  ";
//                            msg += "Input Fields:  " + inputFields;
//                            msg += ", Output Fields:  " + outputFields;
                            return dojoModules.resources.fieldParser_mixOutputInputFields_txt;
                        }
                        col.executionOrder = maxExecutionOrder + 1;
                        return col.executionOrder;
                    }
                }

                function parseAndAssignExecutionOrder() {
                    var valid =  true,
                        dataType = col.dataType,
                        execOrder,
                        execOrderMsg,
                    //Extract fields and push them into parent array
                        fields = extractFields();

                    valid = parseErrors.length === 0;
                    if(valid) {
                        //Assign executionOrder
                        execOrder = assignExecutionOrder();
                        if(_.isString(execOrder)) {
                            parseErrors.push({text: execOrder});
                            valid = false;
                            execOrderMsg = execOrder;
                            execOrder = -1;
                        }
                    }

                    return {valid: valid, parseErrors: parseErrors,executionOrder: execOrder, executionOrderMsg: execOrderMsg};
                }

                //Return object with only publicly exposed methods
                return {
                    UNTERMINATED_QUOTED_STR: UNTERMINATED_QUOTED_STR,
                    FIELDS_MULTI_SOURCES: FIELDS_MULTI_SOURCES,
                    parseAndAssignExecutionOrder: parseAndAssignExecutionOrder,
                    extractFields: extractFields,
                    assignExecutionOrder: assignExecutionOrder,
                    getChopRegEx: function() {
                        return chopChars;
                    },
                    setChopRegEx: function(inChopRegEx) {
                        setChopRegEx(inChopRegEx);
                        return this;
                    },
                    getParseErrors : function() {
                        return parseErrors;
                    },
                    getInputFields: function() {
                        return inputFields;
                    },
                    setInputFields: function(infieldList) {
                        inputFields = infieldList;
                        return this;
                    },
                    getOutputFields: function() {
                        return outputFields;
                    },
                    setOutputFields: function(outfieldList) {
                        outputFields = outfieldList;
                        return this;
                    },
                    getCol: function() {
                        return col;
                    }
                };

            }
        }
    }]);

});

},
'vsd/common/ng/services/localeSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';
angular.module('vsd')
    .factory('localeSvc', function ($window,currencyCodes) {
        var locale = ($window.serverLocale || '').toLowerCase(),
            lang = locale.substr(0, 2),
            langAndCtry = locale.substr(0, 5),
            serverCurrencyCode = currencyCodes[langAndCtry] || currencyCodes[lang] || "USD";

        return {
            getServerCurrencyCode: function () {
                return serverCurrencyCode;
            }
        };
    });

});

},
'vsd/common/ng/services/objectBrowseSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
    factory('objectBrowseSvc', ['$http',
                                '$location',
                                'dojoModules',
                                'windowSvc',
                                'scenarioSvc',
        function ($http,
                  $location,
                  dojoModules,
                  windowSvc,
                  scenarioSvc) {
            return {
                getHierarchicalColDefs: function() {
	                var me = this;
                    return [
                        dojoModules.tree({
                            field:             "name",
                            label:             dojoModules.resources.common_OpenDialog_name_txt,
                            sortable:          true,
                            collapseOnRefresh: true
                        }),

                        { field: "objType", label: dojoModules.resources.common_OpenDialog_type_txt, sortable: true,
                           formatter: me.getObjectTypeLabel},
                        { field: "createUser", label: dojoModules.resources.common_OpenDialog_createdBy_txt, sortable: true},
                        { field: "lstupdtUser", label: dojoModules.resources.common_OpenDialog_updatedBy_txt, sortable: true},
                        { field:      "lstupdtTimestamp",
                            label:     dojoModules.resources.common_OpenDialog_lastModified_txt,
                            sortable:  true,
                            formatter: function (value) {
	                            return (value ? dojoModules.locale.format(new Date(value)) : "");
                            }
                        }
                    ];

                },


                openObject:  function (selObject) {
                    var locStr = "/";

                    switch ( selObject.objType) {
                        case "WINDOW":
                            locStr += "window/";
                            locStr +=  selObject.id;
                            break;
                        case "SCENARIO":
                            locStr += "scenario/";
                            locStr +=  selObject.id.substring(1);
                            break;
                        case "DEPLOYMENT":
                            locStr += "deployment/";
                            locStr += selObject.id.substring(1);
                            break;
                    }
                    $location.path(locStr);
                },

                objectDeletable:  function (selObject) {
                    var rowSelected = !!selObject;
                    return rowSelected && selObject && selObject.objType !== 'DEPLOYMENT';
                },

                objectListGetChildren: function (store, parent, options) {
                    return store.query({parent: parent.id}, options);
                },

                objectListMayHaveChildren:  function (store,parent) {
                    return parent.scenarios && parent.scenarios.length > 0 && store.getChildren(parent).length > 0;
                },

                objectListFilterQuery:  function (item, index, items,filterString) {
                    filterString = filterString.toLowerCase();
                    if (filterString.length < 1) {
                        return !!!item.parent;
                    }
                    else {
                        if (!item.parent) {
                            var name = item.name.toLowerCase();
                            var matched = name.indexOf(filterString.toLowerCase()) > -1;
                            if (matched) {
                                return true;
                            }
                            //check scenarios, if any match, show parent
                            if (item.scenarios) {
                                var childrenMatched = false;
                                item.scenarios.forEach(function (s) {
                                    name = s.name.toLowerCase();
                                    childrenMatched = childrenMatched || (name.indexOf(filterString.toLowerCase()) > -1);
                                });
                                return childrenMatched;
                            }
                        }
                    }

                    return false;
                },

                buildTreeData: function(data) {
                    var treedata = [];
                    data.forEach(function (row) {
                        if (row.objType == vsmCommonConst.OBJ_TYPE_WINDOW) {
                            row.id = row.id.toString();
                            row.parent = null;
                            treedata.push(row);

                            row.scenarios.forEach(function (scen) {
                                scen.id = 's' + scen.id;
                                scen.objType = vsmCommonConst.OBJ_TYPE_SCENARIO;
                                scen.windowName = row.name;
                                scen.parent = row.id;
                                treedata.push(scen);
                            });
                        }
                        else {
                            if (row.objType == vsmCommonConst.OBJ_TYPE_DEPLOYMENT) {
                                // Add 'd' prefix so that memory store 'get' function will return correct object.
                                // Windows and deployments can possibly have the same id value
                                row.id = 'd' + row.id.toString();
                                row.parent = null;
                                treedata.push(row);
                            }
                            else {
                                row.parent = null;
                                treedata.push(row);
                            }
                        }
                    });

                    return treedata;
                },

                getBrowseAll: function() {
                  var me = this;
                  var url = "action/browse/all";
                  return $http.get(url).then(function(response) {
                     return me.buildTreeData(response.data);
                  });
                },

                getBrowseUser: function() {
                  var me = this;
                  var url = "action/browse/user";
                  return $http.get(url).then(function(response) {
                     return me.buildTreeData(response.data);
                  });
                },

                deleteObject: function(selObject) {
                    var objId = selObject.id,
                        objName = selObject.name;
                    if (selObject.objType === 'WINDOW') {
                        return windowSvc.deleteWindow(objId, selObject.lstupdtNbr);
                    }
                    else {
                        if (selObject.objType === 'SCENARIO') {
                            objId = objId.substr(1);
                            return (function() {
                                scenarioSvc.deleteScenario(objId, selObject.lstupdtNbr).then(function (result) {
                                    if (!result.success) {
                                        scenarioSvc.showFailedDeleteMsg(result, objName);
                                    }
                                });
                            });
                        }
                    }
                    return null;
                },

                getObjectTypeLabel: function(value) {
                  switch(value) {
                     case "WINDOW":
                        return dojoModules.resources.objTypeLabel_window_txt;
                     case "SCENARIO":
                        return dojoModules.resources.objTypeLabel_scenario_txt;
                     case "DEPLOYMENT":
                        return dojoModules.resources.objTypeLabel_deployment_txt;
                     case "SIMULATION":
                        return dojoModules.resources.objTypeLabel_simulation_txt;
                     default:
                        return value;
                  }
                }

            };
}]);

});

},
'vsd/common/ng/services/operatorSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('operatorSvc', [ '$http', '$q', 'dojoModules', function ($http, $q, dojoModules) {

		var url = 'action/operators';
		var operators = null;
		var resources = dojoModules.resources;
		var operatorLabels = {
			EQ:             resources.common_operators_eq_txt,
			NE:             resources.common_operators_ne_txt,
			GT:             resources.common_operators_gt_txt,
			GE:             resources.common_operators_ge_txt,
			LT:             resources.common_operators_lt_txt,
			LE:             resources.common_operators_le_txt,
			'=':            resources.common_operators_eq_txt,
			'<>':           resources.common_operators_ne_txt,
			'>':            resources.common_operators_gt_txt,
			'>=':           resources.common_operators_ge_txt,
			'<':            resources.common_operators_lt_txt,
			'<=':           resources.common_operators_le_txt,
			AND:            resources.common_operators_and_txt,
			OR:             resources.common_operators_or_txt,
			NOT:            resources.common_operators_not_txt,
			BETWEEN:        resources.common_operators_between_txt,
			IN:             resources.common_operators_in_txt,
            "NOT IN":       resources.common_operator_not_in_txt,
			"IS MISSING":   resources.common_operators_missing_txt,
			"NOT MISSING":  resources.common_operators_notMissing_txt
		};

		var getOperators = function () {
			var charOperatorRegEx = /^(=|<>|IN|NOT IN|IS MISSING|NOT MISSING)/,
				//numOperatorRegEx = /^(EQ|NE|GT|LT|GE|LE|IS MISSING|NOT MISSING)/;
				numOperatorRegEx = /^(=|<>|>|<|>=|<=|IS MISSING|NOT MISSING)/;
			var deferred = $q.defer();
			if (operators) {
				deferred.resolve(operators);
			}
			else {
				$http.get(url, {cache: true}).success(function (results) {
					operators = results;
					angular.forEach(operators, function (op) {
						op.label = operatorLabels[op.name] || op.name;
						op.logicalType = {
							"CHAR": charOperatorRegEx.test(op.name),
							"NUM": numOperatorRegEx.test(op.name)
						};
					});

					deferred.resolve(operators);
				}).error(function () {
					deferred.reject();
				});
			}

			return deferred.promise;
		};

		var getLogicalOperators = function (type) {
			var logicalOperators = [];
			var deferred = $q.defer();

			getOperators().then(function (data) {
				angular.forEach(data, function (oper) {
					oper.value = oper.name;
					if(oper.logicalType[type]){
						logicalOperators.push(oper);
					}
				});

				deferred.resolve(logicalOperators);
			});
			return deferred.promise;
		};

		return {
			getOperators:        getOperators,
			getLogicalOperators: getLogicalOperators
		};
	}])
;

});

},
'vsd/common/ng/services/configSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .factory('configSvc', [ '$http', function ($http) {

        var configSvc = {};
        var urlBase = 'action/config';
        var configOptions = [];
        var initialized = false;

        var getConfigProperties = function () {
            //configOptions = [{ key: 'window_output_level.disable_group_and_date', value: 'Y'}];
            $http.get(urlBase).then(function (resp) {
                configOptions = resp.data;
           });
        }();

        configSvc.getBoolOption = function (key) {
            var rc = false;
            if (configOptions !== null) {
                var thisOption = _.filter(configOptions, function (option) {
                    return option.name === key;
                });
                if (thisOption !== null && thisOption.length > 0) {
                    if (thisOption[0].value !== null && thisOption[0].value.toLowerCase() === 'y') {
                        rc = true;
                    }
                }
            }
            return rc;
        };

        configSvc.getList = function (key) {
            var sortedList = [];
            if (configOptions !== null) {
                var filteredList = _.filter(configOptions, function (option) {
                    return option.name === key;
                });
                if (filteredList!== null && filteredList.length > 0) {
                    sortedList = _.sortBy(filteredList, sortOrder);
                }
            }
            return sortedList;
        };

        return configSvc;
    }])
;


});

},
'vsd/common/ng/services/tableSelectionSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('tableSelectionSvc', function () {
		return {
			tableSelector:    function (inSelectMode) {
                var selectedIdxs = [],
                    lastSelection = null,
                    SELECT_MODE_SINGLE = 'single',
                    SELECT_MODE_MULTIPLE = 'multiple',
                    selectMode = inSelectMode ? inSelectMode : SELECT_MODE_MULTIPLE;

                function createRange(start,end) {
                    var range = [],
                        n = Math.abs(Math.abs(end) - Math.abs(start)) + 1,
                        by = end > start ? 1 : -1;

                    for(var i = 0; i < n; i++) {
                        range.push(start + (by * i));
                    }
                    return range;
                }

                function selectWithFilteredArray(rowId,a) {
                    var inSelection = false;
                    for(var i = 0; i < a.length; i++) {
                        if(a[i] === lastSelection) {
                            inSelection = true;
                        }
                        if(inSelection) {
                            selectedIdxs.push(a[i]);
                        }
                        if(a[i] === rowId) {
                            break;
                        }
                    }
                }

                function select(rowIdx,$event,idArray) {
                    if(selectMode === SELECT_MODE_SINGLE || $event.shiftKey|| !$event.ctrlKey) {
                        selectedIdxs.length = 0;
                    }

                    //If control key selected, extend range
                    if(lastSelection !== null && $event.shiftKey && !$event.ctrlKey) {
                        if(idArray) {
                            selectWithFilteredArray(rowIdx,idArray);
                        }
                        else {
                            var range = createRange(lastSelection,rowIdx);
                            selectedIdxs = _.union(selectedIdxs,range);
                        }
                    }

                    else {
                        //If row already in selection
                        if(_.contains(selectedIdxs,rowIdx)) {
                            selectedIdxs = _.without(selectedIdxs,rowIdx);
                        }
                        //Otherwise, append to selection
                        else {
                            selectedIdxs.push(rowIdx);
                        }
                        //Reset last selection
                        lastSelection = rowIdx;
                    }

                    return selectedIdxs;
                }

                function isSelected(rowIdx) {
                    return _.contains(selectedIdxs,rowIdx);
                }

                function clearSelection() {
                    selectedIdxs = [];
                    lastSelection = null;
                }

                function getSelectedIdxs() {
                    return selectedIdxs;
                }

                function getLastSelection() {
                    return lastSelection;
                }


                return {
                    SELECT_MODE_SINGLE: SELECT_MODE_SINGLE,
                    SELECT_MODE_MULTIPLE: SELECT_MODE_MULTIPLE,
                    select: select,
                    isSelected: isSelected,
                    clearSelection: clearSelection,
                    getSelectedIdxs: getSelectedIdxs,
                    getLastSelection:  getLastSelection
                }
			}
		};
	});

});

},
'vsd/common/ng/services/treeUtilSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .factory('treeUtilSvc', ['dojoModules', function(dojoModules) {
        return {
            collectFromRootToLeaves: function(node,collectFunction, childrenProperty, collection) {
                //Create collection as array if not exists
                if(!collection) {
                    collection = [];
                }
                if(!childrenProperty) {
                    childrenProperty = "children";
                }

                //Call collection function for current node
                var entry = collectFunction(node);
                if(entry) {
                    collection.push(entry);
                }

                if(node[childrenProperty]) {
                    //Collect from child nodes
                    var me = this;
                    angular.forEach(node[childrenProperty],function(child) {
                        me.collectFromRootToLeaves(child,collectFunction,childrenProperty, collection);
                    });

                }

                return collection;
            },

            collectFromLeafToParent: function(node,collectFunction, parentProperty, collection) {
                //Create collection as array if not exists
                if(!collection) {
                    collection = [];
                }
                if(!parentProperty) {
                    parentProperty = "parent";
                }

                //If at root node, return collection
                if(!node[parentProperty]) {
                    return collection;
                }
                //Call collection function for current node
                collection.push(collectFunction(node));
                //Collect from parent node
                this.collectFromLeafToParent(node[parentProperty],collectFunction,parentProperty, collection);

                return collection;
            }
        }
    }]);


});

},
'vsd/common/ng/services/Util':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
    factory('Util', ['dojoDepends', function (dojoDepends) {

        function clearWidgetError(dojoWidgetId) {
            var dojoWidget = dijit.byId(dojoWidgetId);
            dojoWidget.set("state", "");
            dojoWidget.set("invalidMessage", "");
            dojoWidget.set("displayedValue", "");

        }

        function trim(str) {
            return (str || '').replace(/^\s\s*/, '').replace(/\s\s*$/, '');
        }

        function escapeQuotes(propsStr) {

            // This function is needed to avoid parse errors in the case where a properties string references a resource bundle value
            // that contains a single quote, e.g. "label: '{{resources.labelContainingSingleQuoteChar}}'"
            // This is important when testing i18n using pseudo bundles because the pseudo translated strings contain single quotes

            var searchStr = propsStr,
                parsedStr = "",
                hasMatch = false,
                done = false;

            while (!done) {
                // Find the next property that encloses its value with single quotes, and escape any single quote found in the value string.
                // This will have the pattern "varname:'value'", which may contain whitespace anywhere and is delimited by commas
                // or the beginning and end of propsStr.
                var nextProp = searchStr.match(/([^|,]\s*\w+\s*:\s*')(.*?)('\s*($|,))/);
                if (nextProp) {
                    hasMatch = true;
                    parsedStr += searchStr.substring(0, nextProp.index) + nextProp[1] + nextProp[2].replace(/'/g, "\\'") + nextProp[3];
                    searchStr = searchStr.substring(nextProp.index + nextProp[0].length);
                }
                else {
                    done = true;
                }
            }   // while (!done)

            if (hasMatch) {   // concatenate parsed portion of string with the remaining un-parsed portion
                parsedStr += searchStr;
            }
            //console.log("parsedStr="+searchStr);
            return hasMatch ? parsedStr : propsStr;
        }

        return {
            parseProps: function (propsStr, scope) {
                return scope.$eval("{" + escapeQuotes(propsStr) + "}");
                //return scope.$eval("{" + propsStr + "}");
            },

            applyStyles: function (widget, stylesProp) {
                if (stylesProp) {
                    angular.forEach(stylesProp.split(";"), function (style) {
                        var nameValueArray = style.split(":");
                        dojoDepends.domStyle.set(widget.domNode, nameValueArray[0], nameValueArray[1]);
                    });
                }
            },

            clearError: function(dojoWidgetId) {
                clearWidgetError(dojoWidgetId);
            }
        };
    }]);

});

},
'vsd/common/ng/services/vsdUtil':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('vsdUtil', ['localeSvc', 'dojoModules', '$timeout', 'configSvc', function (localeSvc, dojoModules, $timeout, configSvc) {

		return {
			dropLeft: function (inStr, numDrop) {
				if (!inStr || !numDrop) {
					throw new Error("Must pass arguments inStr:String and numDrop:int");
				}
				return inStr.substring(numDrop, inStr.length);
			},

			dropRight: function (inStr, numDrop) {
				return inStr.substr(0, inStr.length - numDrop);
			},

			getLeft: function (inStr, numGet) {
				return inStr.substr(0, numGet);
			},

			getRight: function (inStr, numGet) {
				return inStr.substr(numGet * -1, inStr.length);
			},

			clearArray: function (inArray) {
				inArray.length = 0;
			},

			toggleDojoWidgetDisplay: function (widget, displayType) {
				if (widget) {
					dojoModules.domStyle.set(widget.domNode, "display", displayType);
				}
			},

			classToJson: function (classObj) {
				var jsonStr = dojoModules.json.stringify(classObj, function (key, value) {
					if (key !== "_inherited") {
						if (value) {
							var tf = value.__json__ || value.json;
							if (typeof tf === "function") {
								return tf.call(value);
							}
						}
						return value;
					}
				});

				return  jsonStr;
			},

			clearIds: function (obj, /* Optional */ ignoreProps) {

				var ignorePropMap = {};
				angular.forEach(ignoreProps, function (item) {
					ignorePropMap[item] = true;
				});

				angular.forEach(obj, function (value, prop) {
					var type = typeof value;
					if (ignorePropMap[prop]) {
						return;
					}
					else if (type === 'number' && prop === 'id') {
						obj[prop] = 0;
					}
					else if (type === 'object') {
						if (!angular.isArray(value)) {
							this.clearIds(value, ignoreProps);
						}
						else { //loop through array
							angular.forEach(value, function (item) {
								this.clearIds(item, ignoreProps);
							}, this);
						}
					}
				}, this);
			},

			clearUnusedConditions: function (inCondition) {

				var clearUnUsed = function (conditions) {
					var cleanConditions = [];
					if (conditions) {
						var len = conditions.length;
						for (var ii = 0; ii < len; ii++) {
							var cond = conditions[ii];
							if (cond) {
								cond.childConditions = clearUnUsed(cond.childConditions || []);
								if (cond.childConditions.length) {
									cleanConditions.push(cond);
								}
								else if (cond.operand2Constant && dojoModules.lang.trim(cond.operand2Constant).length !== 0) {
									var col = cond.operand1ColumnInfo || {};
									if (col.name && col.name !== '---') {
										cleanConditions.push(cond);
									}
								}
							}
						}
					}
					return cleanConditions;
				};

				if (inCondition) {

					inCondition.childConditions = clearUnUsed(inCondition.childConditions || []);

					if (inCondition.childConditions.length) {
						return inCondition;
					}

				}

				return null;
			},

			createServerError: function (type, header, err) {
				var errors = [];
				if (type) {
					errors.push(type);
				}
				if (header) {
					errors.push(header);
				}

				var error = err;
				if (typeof err === "string") { // treat string as json
					error = dojo.fromJson(err);
				}

				if (error) {
					if (error.exceptionType) {
						errors.push(error.exceptionType);
					}
					if (error.errorType) {
						errors.push(error.errorType);
					}
					if (error.message) {
						errors.push(error.message);
					}
					if (error.stackTrace) {
						var trace = error.stackTrace.substr(0, 2000) + " ....";
						errors.push(trace);
					}
				}
				return errors;
			},

			strOperandToNbr: function (operand) {
				return parseFloat(operand.replace(/,/g, ""));
			},

			startsWith: function(str,values){
				if(!str || !values) {
                    return false;
                }

				for(var ii = 0; ii < values.length; ii++){
					if(str.slice(0,values[ii].length) === values[ii]){
						return true;
					}
				}
				return false;
			},

			defaultFormats: {
				none:       'BEST32.',
				number:     'COMMA32.2',
				percentage: 'PERCENT32.2',
				currency:   'NLMNY32.2',
				/*dollar:     'NLMNLUSD32.2',
				yen:        'NLMNLJPY32.2',
				euro:       'NLMNLEUR32.2',*/
				integer:    'COMMA32.0',
				date:       'DATE9.',
				datetime:   'DATETIME20.'
			},

			getSASNumericFormat: function (format) {
				var idx,
					fmts = this.defaultFormats,
					serverCurrencyCode = localeSvc.getServerCurrencyCode();
				format = format || '';

				if (this.startsWith(format, ["NLMNY"])) {
					return fmts.currency;
				}
				else if (this.startsWith(format, ["DOLLAR"])) {
					return serverCurrencyCode === 'USD' ? fmts.currency : fmts.number;
				}
				else if (this.startsWith(format, ["YEN"])) {
					return serverCurrencyCode === 'JPY' ? fmts.currency : fmts.number;
				}
				else if (this.startsWith(format, ["EURO"])) {
					return serverCurrencyCode === 'EUR' ? fmts.currency : fmts.number;
				}
				else if (this.startsWith(format, ["NLMNI","NLMNL"])) {
					return format.substr(5,3) === serverCurrencyCode ? fmts.currency : fmts.number;
				}
				else if (this.startsWith(format, ["PERCENT"])) {
					return fmts.percentage;
				}
				else if (this.startsWith(format, ["BEST"])) {
					return fmts.none;
				}
				else if (this.startsWith(format, ["COMMA"])) {
					idx = format.indexOf('.');
					if (idx < 0 || idx === (format.length - 1) || format.substr(idx + 1) === '0') {
						return fmts.integer;
					}
					return fmts.number;
				}
				else if (this.startsWith(format, ["DATETIME"])) {
					return fmts.datetime;
				}
				else if (this.startsWith(format, ["DATE"])) {
					return fmts.date;
				}
				// default to no formatting
				return fmts.none;
			},

			applySASNumericFormat: function (value, format) {
				if (typeof value === 'number') {
					format = format || '';
					if (this.startsWith(format, ["NLMNY"])) {
						return dojoModules.currency.format(value, {currency: localeSvc.getServerCurrencyCode()});
					}
					else if (this.startsWith(format, ["PERCENT"])) {
						return dojoModules.number.format(value, {type: 'percent', places: 2});
					}
					else if (format === this.defaultFormats.integer) { //integer
						return dojoModules.number.format(value, {type: 'decimal', places: 0});
					}
					else if (this.startsWith(format, ["DATETIME"])) {
						return this.getFormattedDateTimeString(value);
					}
					else if (this.startsWith(format, ["DATE"])) {
						return this.getFormattedDateString(value);
					}
                    else {  // For BEST or unrecognized SAS format, display number without formatting

                        // Use a number pattern with no thousands separator. We need to pass the number of decimal places,
                        // otherwise dojo will round to the number of decimal places in the pattern.
                        var numString = '' + value,
                            decimalPos = numString.indexOf('.'),
                            places = (decimalPos === -1) ? 0 : (numString.length - 1 - decimalPos);
                        //dojo only supports 0-20 decimal places
                        if (places > 20) {
                            places = 20;
                        }
                        if (places < 0) {
                            places = 0;
                        }
                        return dojoModules.number.format(value, {type: 'decimal', places: places, pattern: '###0.#'});
					}
				}
				else if (this.isSasDateConstant(value) || this.isSasDateTimeConstant(value)) {
                    return value;
				}
				return '';
			},

            isDateFormat: function (format) {
                if (!format) {
                    return false;
                }
                if (this.startsWith(format, ["DATETIME"])) {
                    return false;
                }
                if (this.startsWith(format, ["DATE"])) {
                    return true;
                }
                return false;
            },

            isDateTimeFormat: function (format) {
                if (!format) {
                    return false;
                }
                return this.startsWith(format, "DATETIME");
            },

            isDateOrDateTimeFormat: function (format) {
            return this.isDateFormat(format) || this.isDateTimeFormat(format);
            },

			removeFormat: function (fmtValue) {
				var numValue,
					strippedVal;
				if (_.isNumber(fmtValue)) {
					return fmtValue;
				}
				if (fmtValue) {
                    // If we have a SAS date or datetime constant, leave it alone
                    if (this.isSasDateConstant(fmtValue) || this.isSasDateTimeConstant(fmtValue)) {
                        return fmtValue;
                    }

					// Check if it holds a formatted date string
					var dateValue = this.parseFormattedDate(fmtValue);
					if (dateValue !== null) {
						return dateValue;
					}

					// Check if this is it holds a formatted datetime string
					var dateTimeValue = this.parseFormattedDateTime(fmtValue);
					if (dateTimeValue !== null) {
						return dateTimeValue;
					}

                    //remove any character except a number, decimal point, or comma
                    strippedVal = fmtValue.replace(/[^\d\.,]/g, '');

                    // parse number using user's locale
					numValue = dojoModules.number.parse(strippedVal, {
						locale: dojoConfig.locale
					});

					//if had parens make number negative
					if (fmtValue.indexOf("(") > -1) {
						numValue *= -1;
					}

					//if percent format, convert to decimal
					if (fmtValue.indexOf("%") > -1) {
						numValue *= 0.01;
					}

					if (isNaN(numValue)) {
						numValue = null;
					}
				}
				else {
					numValue = fmtValue;
				}
				return  numValue;
			},

			isSasDateConstant: function (value) {
                return /^(['"])[a-zA-Z0-9-\/]+\1[dD]$/.test(value);
			},

            isSasDateTimeConstant: function (value) {
                return /^(['"])[a-zA-Z0-9-\/:]+\1[dD][tT]$/.test(value);
            },

			parseNumeric: function (value) {
				if (!value) {
					return null;
				}
				//remove carriage returns
				var simpleString = value.replace(/\n|<br\s*\/?>/gi, "\r");
				//split out operators
				var toks = simpleString.split(/[<>=]/);
				var finalParse = _.filter(toks, function (val) {
					return val !== "" && val !== " ";
				});
				var numOnly = this.removeFormat(finalParse[0]);
				if (angular.isNumber(numOnly) || this.isSasDateConstant(numOnly) || this.isSasDateTimeConstant(numOnly) ) {
					return numOnly;
				}
				else {
					return null;
				}
			},


			getFormattedNumberString: function(value) {
				// Given an unformatted value, returns a formatted number string appropriate for the current
				// dojo locale, preserving the decimal places.
				// E.g., a numString value of "1234.5678" would return "1,234.5678" in 'en-us' locale, and "1 234,5678" in "fr" locale.
				var numString = (typeof value === 'number') ? String(value) : value;
				if (numString && numString.trim().length && isFinite(numString)) { // make sure the string contains only a number
					var decimalPos = numString.indexOf('.'),
						places = (decimalPos === -1) ? 0 : (numString.length - 1 - decimalPos);
					if (typeof value === 'number') {
						return dojoModules.number.format(value, {type: 'decimal', places: places});
					}
					else {
						return dojoModules.number.format(parseFloat(value), {type: 'decimal', places: places});
					}
				}
				else {
					return '' + value;
				}
			},

			parseFormattedDate: function (fmtValue) {
                var dateVal = null;

                //Check to see if this is a date, if so convert to sas date value
                if (typeof fmtValue === 'number') {
                    dateVal = fmtValue;
                }
                else {
                    dateVal = dojoModules.locale.parse(fmtValue + '', {
                        locale: dojoConfig.locale,
                        selector: "date"
                    });
                }
				if (dateVal !== null && typeof dateVal.getTime !== 'undefined' && !isNaN(dateVal.getTime()) ) {
                    // Convert to sas date value, which is number of day since Jan 1, 1960
                    return this.convertJavaScriptDateToSASDays(dateVal);
				}
				return null;
			},

			parseFormattedDateTime: function (fmtValue) {
                var dateTimeVal = null;

				//Check to see if this is a date-time, if so convert to sas datetime value
                if (typeof fmtValue === 'number') {
                    dateTimeVal = fmtValue;
                }
                else {
                    //Try default format
                    var dateTimeVal = dojoModules.locale.parse(fmtValue + '', {
                        locale: dojoConfig.locale
                    });
                    if (dateTimeVal === null || isNaN(dateTimeVal.getTime())) {
                        //Try 'medium' format
                        dateTimeVal = dojoModules.locale.parse(fmtValue + '', {
                            locale: dojoConfig.locale,
                            formatLength: 'medium'
                        });
                    }
                }
				if (dateTimeVal !== null && typeof dateTimeVal.getTime !== 'undefined' && !isNaN(dateTimeVal.getTime()) ) {
                    //convert to sas time value

					return this.convertJavaScriptDateToSASSeconds(dateTimeVal);
				}
				return null;
			},

			getFormattedDateString: function(value) {
				// If we have a SAS date constant, don't try to format it
				if (this.isSasDateConstant(value)) {
					return value;
				}
				// If it is already formatted, we don't need to do anything
				if (this.parseFormattedDate(value)) {
					return value;
				}
                var numVal = (typeof value === 'number') ? value : parseFloat(value);
                if (!isNaN(numVal)) { // make sure the string contains only a number
                    return dojoModules.locale.format(this.convertSASToJavaScriptDate(numVal), {selector: 'date'});
                }
                else {
                    return value;
                }
			},

			getFormattedDateTimeString: function (value) {
				// If we have a SAS datetime constant, don't try to format it
				if (this.isSasDateTimeConstant(value)) {
					return value;
				}
				// If it is already formatted, we don't need to do anything
				if (this.parseFormattedDateTime(value)) {
					return value;
				}
                var numVal = (typeof value === 'number') ? value : parseFloat(value);
                if (!isNaN(numVal)) { // make sure the string contains only a number
                    return dojoModules.locale.format(this.convertSASToJavaScriptDateTime(numVal), {formatLength: 'medium'});
                }
                else {
                    return value;
                }
			},
			/**
			 * Converts a JavaScript Date to number of SAS Days.
             *
             * This is tricky.  We aren't really taking into account timezone, but the unit tests seem to be ok.
             *
             * Again they shouldn't use SAS Date if they are trying to use UTC.  They should use Datetime.
			 *
			 * @param jsDate
			 * @returns {*}
			 */
            convertJavaScriptDateToSASDays: function(jsDate){
                // Convert to sas date value, which is number of days since Jan 1, 1960
                var mDateVal, returnValue,
					m01Jan1960 = moment.utc('1960-01-01');

                mDateVal = moment.utc([jsDate.getFullYear(), jsDate.getMonth(), jsDate.getDate()]);

				returnValue = mDateVal.diff(m01Jan1960, 'days');
				//console.debug("Local Time " + jsDate.toString() + " UTC Time " + jsDate.toUTCString() + " to SAS Days " + returnValue);
				return returnValue;
            },

			/**
			 * Convert the JavaScript Date to number of SAS Seconds.  We will add the users
			 * time zone offset to the result.  This way we normailize the values
			 * to what is in the LASR tables.
			 *
			 * If configSvc.getBoolOption(vsmCommonConst.CONFIG_OPTION_USE_UTC_TIME) is false then we will
             * not add the offset. If they are using this value then their whole infrustructure and data should
             * configured to UTC.  This way timezones work well.
			 *
			 * @param jsDate
			 * @returns {number}
			 */
            convertJavaScriptDateToSASSeconds: function(jsDate){
                var secsIn1960s = 315619200;
                var momentTime, timeZone, returnValue;

                if (configSvc.getBoolOption(vsmCommonConst.CONFIG_OPTION_USE_UTC_TIME) === false){
                    momentTime = moment(jsDate);
                    timeZone = momentTime.toDate().getTimezoneOffset();
                    momentTime.subtract(timeZone, 'minutes');
                }else{
                    momentTime = moment.utc(jsDate)
                }

                momentTime = momentTime.add(secsIn1960s, 'seconds');
				returnValue =  (momentTime.toDate().getTime())/1000;
				//console.debug("Local Time " + jsDate.toString() + " UTC Time " + jsDate.toUTCString() + " to SAS Seconds " + returnValue);
				return returnValue;
            },

			/**
			 * convert a SAS Date to a JavaScript Date (which is date and time).
			 * We have to add the users timezone offset in order for the date to
			 * show as it is in the LASR Tables.
             *
             * This will not play well with UTC time.  Customers should not use the SAS Date
             * datatype if they care about utc.
			 *
			 * @param sasDateValue
			 * @returns {*}
			 */
			convertSASToJavaScriptDate: function (sasDateValue) {
                // Converts a SAS date value into a JavaScript date object
                // SAS date value contains number of days since Jan 1, 1960
                // JavaScript date value contains number of milliseconds since Jan 1, 1970
                var m01Jan1960 = moment.utc('1960-01-01');
                var mDateVal = m01Jan1960.add(sasDateValue, 'days');
                var timeZone, returnValue;

				timeZone = mDateVal.toDate().getTimezoneOffset();
				mDateVal.add(timeZone, 'minutes');

				returnValue = mDateVal.toDate();
				//console.debug("Local Time " + returnValue.toString() + " UTC Time " + returnValue.toUTCString());
				return returnValue;
			},

			/**
			 *
			 * Convert the number of SAS Seconds to a JavaScript Date.  We will add the users
			 * time zone offset to the result.  This way we sort of normailize the values
			 * to what is in the LASR tables.
			 *
			 * If configSvc.getBoolOption(vsmCommonConst.CONFIG_OPTION_USE_UTC_TIME) is false then we will
             * not add the offset. If they are using this value then their whole infrustructure and data should
             * configured to UTC.  This way timezones work well.
             *
			 * @param sasDateTimeValue
			 * @param skipTimeZoneOffset
			 * @returns {*}
			 */
			convertSASToJavaScriptDateTime: function (sasDateTimeValue) {
				// converts a SAS datetime value into a JavaScript date object
                var m01Jan1960 = moment.utc('1960-01-01');
                var mDateVal = m01Jan1960.add(sasDateTimeValue, 'seconds');
                var timeZone, returnValue;

				if (configSvc.getBoolOption(vsmCommonConst.CONFIG_OPTION_USE_UTC_TIME) === false){
                    timeZone = mDateVal.toDate().getTimezoneOffset();
					mDateVal.add(timeZone, 'minutes');
				}

				returnValue = mDateVal.toDate();

				//console.debug("Local Time " + returnValue.toString() + " UTC Time " + returnValue.toUTCString());
				return returnValue;
			},


            // This function is used for validating the operand constant value in a condition
            // useDojoLocale should be set to true only when the input value contains a FORMATTED number string
			valExprConstByDataType: function (/* String */value, lhsCol, operator, save, useDojoLocale) {
				var isValid = true,
					reason,
					parsedValue = value,
					parseAsNumber = function() {
                        var pVal = null;
                        if (useDojoLocale) {
                            // Parse formatted number string
                            pVal = dojoModules.number.parse(value, {
                                locale: dojoConfig.locale
                            });
                        }
                        else {
                            // Parse unformatted number string
                            pVal = parseFloat(value);
                        }
                        if (isNaN(pVal)) {
                            return false;
                        }
						parsedValue = pVal;
						return true;
					};

				if(typeof value === 'number') {
                    value = value.toString();
                }
                //Check for null value
				if (!angular.isDefined(value) || !value || !value.trim().length) {
					isValid = false;
					reason = vsmCommonConst.EXPR_VAL_REASON_NULL;
				}
				else if (lhsCol && lhsCol.dataType) {
					// Validate numeric value
					if (lhsCol.dataType === vsmCommonConst.DATA_TYPE_NUMERIC) {
                        // Parse datetime value
						if (lhsCol.sasDateTime || (lhsCol.columnFormat && this.startsWith(lhsCol.columnFormat, ["DATETIME"]))) {
							if ( !parseAsNumber() && !this.isSasDateTimeConstant(value) ) {
								var pVal = this.parseFormattedDateTime(value);
								if (pVal !== null) {
									parsedValue = pVal;
								}
								else {
									isValid = false;
									reason = vsmCommonConst.EXPR_VAL_REASON_INVALID_DATETIME_CONST;
								}
							}
						}
						// Parse date value
                        else if (lhsCol.sasDate || (lhsCol.columnFormat && this.startsWith(lhsCol.columnFormat, ["DATE"])) ) {
                            // Try parsing the value using the possible formats of date values
                            if (!parseAsNumber() && !this.isSasDateConstant(value)) {
                                var pVal = this.parseFormattedDate(value);
                                if (pVal !== null) {
                                    parsedValue = pVal;
                                }
                                else {
                                    isValid = false;
                                    reason = vsmCommonConst.EXPR_VAL_REASON_INVALID_DATE_CONST;
                                }
                            }
                        }
                        // Parse "boolean" value
						else if (!save && lhsCol.editControlType && lhsCol.editControlType.name === vsmCommonConst.EDIT_CONTROL_TYPE_BOOLEAN) {
							var testValue = String(value);
							if (!_.contains(['False', 'True'], testValue)) {
								isValid = false;
								reason = vsmCommonConst.EXPR_VAL_REASON_INVALID_BOOLEAN_CONST;
							}
						}
						else if (!parseAsNumber()) {
							isValid = false;
							reason = vsmCommonConst.EXPR_VAL_REASON_NON_NUMERIC;
						}
					}
                    // Validate character value
					else if (lhsCol.dataType === vsmCommonConst.DATA_TYPE_CHAR){
						// Strings cannot contain double quotes
						if(value.indexOf('"') > -1){
							isValid = false;
							reason = vsmCommonConst.EXPR_VAL_REASON_DBL_QUOTES;
						}
						// Check if string exceeds maximum allowed total length
						else if (value.length > vsmCommonConst.RULE_OPERAND2_CONSTANT_MAXLEN) {
                            isValid = false;
                            reason = vsmCommonConst.EXPR_VAL_REASON_TOTAL_MAXLEN;
                        }
                        // Check if any items in IN list exceed LASR string literal maximum length
                        else if (operator && operator.name && operator.name.toUpperCase().indexOf("IN") >= 0) {
                            var strings = value.split(',');
                            if (_.any(strings, function (val) {
                                return (val && val.trim().length > vsmCommonConst.LASR_MAX_LITERAL_LENGTH);
                            })) {
                                isValid = false;
                                reason = vsmCommonConst.EXPR_VAL_REASON_LITERAL_MAXLEN;
                            }
                        }
                        // Check if value exceeds LASR string literal maximum length
                        else if (value.length > vsmCommonConst.LASR_MAX_LITERAL_LENGTH) {
                            isValid = false;
                            reason = vsmCommonConst.EXPR_VAL_REASON_LITERAL_MAXLEN;
                        }
                    }
				}
				//console.log("isValid=",isValid, ", value=",value, ", parsedValue=",parsedValue);
				return {isValid: isValid, reason: reason, parsedValue: parsedValue};
			},

			gridFilterByAttribute: function (filterString, item, attr) {
				if (filterString.length < 1) {
					return true;
				}
				else if (item[attr].toLowerCase().indexOf(filterString.toLowerCase()) > -1) {
					return true;
				}
				return false;
			},

			buildWhereClause: function (filterCond) {
				if (!filterCond.isValid) {
					return '';
				}

				var name = filterCond.operand1ColumnInfo.name,
					dataType = filterCond.operand1ColumnInfo.dataType,
					format = filterCond.operand1ColumnInfo.columnFormat,
					operator = filterCond.operator.name,
					filterConstant = filterCond.operand2Constant,
					numDecimals = 0,
					splits,
					decimalPos;

				if (operator === 'NOT MISSING') {
					return name + ' IS NOT MISSING';
				}

				if (operator === 'IS MISSING') {
					return name + ' ' + operator;
				}

				if (!filterConstant || filterConstant.length === 0) {
					return '';
				}

				decimalPos = filterConstant.indexOf('.');

				if (dataType === 'CHARACTER') {
					name = 'STRIP(' + name + ')';
					if (operator !== 'IN' && operator !== 'NOT IN') {
						filterConstant = "'" + filterConstant.trim() + "'";
					}
					else {
						splits = filterConstant.split(',');
						filterConstant = '(';
						angular.forEach(splits, function (value, idx) {
							if (idx !== 0) {
								filterConstant += ',';
							}
							filterConstant += '"' + value.trim() + '"';
						});
						filterConstant += ')';
					}
				}
				else if (format !== this.defaultFormats.none && !(this.startsWith(format, ["DATE","DATETIME"])) ) { // NUMERIC and has format
					if (operator === '=' || operator === '^=' || operator === '<>') {
						if (decimalPos > -1) {
							numDecimals = filterConstant.length - decimalPos - 1;
						}
						name = 'STRIP(PUT(' + name + ', 22.' + numDecimals + '))';
						filterConstant = '"' + filterConstant + '"';
					}
				}
				return name + ' ' + operator + ' ' + filterConstant;
			},

			getUniqueName: function (existingNames, name, upcase) {
				var matched,
					nbr = 1,
					newName = name;

				do {
					matched = _.find(existingNames, function (existingName) {
						return existingName.toUpperCase() === newName.toUpperCase();
					});
					if (matched) {
						newName = name + nbr;
						nbr++;
					}
				} while (matched);

				if (upcase) {
					newName = newName.toUpperCase();
				}

				return newName;
			},

			guranteedApply: function (scope, callback) {
				var me = this;
				if (!scope.$root.$$phase) {
					if (callback) {
						scope.$apply(function () {
							callback();
						});
					}
					else {
						scope.$apply();
					}

				}
				else {
					$timeout(function () {
						me.guranteedApply(scope, callback)
					}, 0);
				}
			},

			validateDomainObjName: function (name, existingNames) {
				//Validate name syntax
				var isValid = true,
					reason;
				var regEx = vsmCommonConst.VALID_NAME_REGEX;
				if (!regEx.test(name)) {
					isValid = false;
					reason = vsmCommonConst.INVALID_NAME_REASON_SYNTAX;
				}
				//Validate that name is unique
				if (!_.every(existingNames, function (existName) {
					return existName.toUpperCase() !== name.toUpperCase();
				})) {
					isValid = false;
					reason = vsmCommonConst.INVALID_NAME_REASON_DUPLICATE;
				}
				return {isValid: isValid, reason: reason};
			},

			sortByFields: function (a, sortCriteria, sortFieldExtractor) {
				a.sort(function (a, b) {
					for (var i = 0; i < sortCriteria.length; i++) {
						var sc = sortCriteria[i],
							lhs = sc.order === 'A' ? a : b,
							rhs = sc.order === 'A' ? b : a,
							lhsValue,
							rhsValue;

						if (sortFieldExtractor) {
							lhsValue = sortFieldExtractor(lhs[sc.field]);
							rhsValue = sortFieldExtractor(rhs[sc.field]);
						}
						else {
							lhsValue = lhs[sc.field];
							rhsValue = rhs[sc.field];
						}

						if (lhsValue > rhsValue) {
							return 1;
						}
						else if (lhsValue < rhsValue) {
							return -1;
						}
					}
					return 0;
				});
			},

			singleFieldNaturalSortComparator: function () {
				return function (a, b) {
					if (a > b) {
						return 1;
					}
					else if (a < b) {
						return -1;
					}
					else {
						return 0;
					}
				};
			},

			menuFromItemsArray:    function (items, beforeMenuDisplayFunc, afterItemClickFunc) {
				var menu = new dojoModules.Menu();
				angular.forEach(items, function (itemConfig) {
					if (itemConfig.hasOwnProperty("menuItems")) {
						var subMenu = new dojoModules.Menu();
						angular.forEach(itemConfig.menuItems, function (subItemConfig) {
							subMenu.addChild(menuItemFromConfigObj(subItemConfig));
						});
						menu.addChild(new dojoModules.PopupMenuItem({label: itemConfig.label, popup: subMenu}));
					}
					else {
						menu.addChild(menuItemFromConfigObj(itemConfig));
					}

				});

				return menu;

				function menuItemFromConfigObj(config) {
					return new dojoModules.MenuItem({
						label:     config.label,
						iconClass: config.iconClass,
						onClick:   function () {
							if (beforeMenuDisplayFunc) {
								beforeMenuDisplayFunc();
							}
							config.onClick();

							if (afterItemClickFunc) {
								afterItemClickFunc();
							}
						}
					});
				}
			},
			broadcastEventToScope: function (scope, eventName, eventParams) {
				scope.$broadcast(eventName, eventParams);
			},

			isValidIdentifier: function (identifier, required, maxLength) {
				var validName = vsmCommonConst.VALID_NAME_REGEX;
				if (required && (!identifier || identifier.length === 0)) {
					return {valid: false, reason: "NULL_OR_ZEROLENGTH"};
				}
				else if (identifier.length > maxLength) {
					return {valid: false, reason: "MAX_LENGTH"};
				}
				else if (!validName.test(identifier)) {
					return {valid: false, reason: "SYNTAX"};
				}
				else {
					return {valid: true};
				}
			},
			isUniqueEntry:     function (list, compEntry, compareFunc) {
				var appearanceCnt = 0;
				angular.forEach(list, function (entry) {
					if (compareFunc) {
						if (compareFunc(entry, compEntry)) {
							appearanceCnt++;
						}
					}
					else if (compEntry.toUpperCase() === entry.toUpperCase()) {
						appearanceCnt++;
					}
				});
				return appearanceCnt === 0;
			},
			pluck:             function (list, propertyName) {
				return _.pluck(list, propertyName);
			},

			getScrollbarWidth: function () {
				var div, width;
				if (width === undefined) {
					div = document.createElement('div');
					div.innerHTML = '<div style="width:50px;height:50px;position:absolute;left:-50px;top:-50px;overflow:auto;"><div style="width:1px;height:100px;"></div></div>';
					div = div.firstChild;
					document.body.appendChild(div);
					width = div.offsetWidth - div.clientWidth;
					document.body.removeChild(div);
				}
				return width;
			},

			splitAllChars: function (text) {
				return text.split("");
			},

			markDroppedColumn: function (col, colData) {
				colData.dropNodeId = col.id;
				dojoModules.domClass.add(col.id, 'disabledItem');

			},

			unmarkRemovedColumn: function (nodeId) {
				if (angular.element(nodeId)) {
					dojoModules.domClass.remove(nodeId, 'disabledItem');
					return true;
				}
				else {
					return false;
				}
			},

			realRound: function (number, places) {
				if (!places) {
					places = 2;
				}
				var multiplier = Math.pow(10, places);
				return Math.round(number * multiplier) / multiplier;
			},

			conditionToFilterString: function (cond, caseSensitive, suppressNullCheck, useLabel) {
				var filtStr = "",
					nextCond = "",
					conj = null,
					operand1ColName = "",
					operand2Constant = "",
					rhs,
					minMax,
					nullCheck,
					me = this;


				//For logical AND/OR operators, recurse down tree
				if (cond.operator.name === "AND" || cond.operator.name === "OR") {
					conj = cond.operator.name;
					filtStr += "(";
					cond.childConditions.forEach(function (childCond) {
						nextCond = me.conditionToFilterString(childCond, caseSensitive, suppressNullCheck,
							useLabel);
						if (nextCond.trim() !== "") {
							// Add conjunctor only if there is already a condition
							if (filtStr.trim().substr(filtStr.trim().length - 1, 1) !== "(") {
								filtStr += " " + conj + " ";
							}
							filtStr += nextCond;
						}
					});
					//Add parentheses
					filtStr += ")";
				}
				//For logical NOT operator, recurse to single child condition (which could be complex condition
				else if (cond.operator.name === "NOT") {
					filtStr += " NOT (";
					filtStr += me.conditionToFilterString(cond.childConditions[0], caseSensitive, suppressNullCheck,
						useLabel);
					filtStr += ")";
				}
				//Otherwise, process single level condition
				else {
					if (!cond || !cond.operand1ColumnInfo || !cond.operand1ColumnInfo.name || !cond.operand1ColumnInfo.name.trim().length || !cond.operator || !cond.operator.name || !cond.operator.name.trim().length ||
						(!_.contains(['IS MISSING', 'NOT MISSING'],
							cond.operator.name.trim()) && !(cond.operand2Constant != null && cond.operand2Constant.toString().trim().length))) {
						return '';
					}

					var operatorMissingVariant = cond.operator.name.indexOf("MISSING") != -1;

					if (cond.operand1ColumnInfo.dataType === "CHARACTER" && !operatorMissingVariant) {
						// IN condition operand cannot contain expressions, only comma-delimited list of constants
						// Add quotes around the character strings.
						if (cond.operator.name === 'IN' || cond.operator.name === 'NOT IN') {
							operand2Constant = "";
							var values = cond.operand2Constant.split(",");
							if (values && values.length > 0) {
								angular.forEach(values, function(val) {
									if (operand2Constant !== "") {
										operand2Constant += ",";
									}
									// remove leading and trailing whitespace and add quotes
									operand2Constant += "'" + val.replace(/^\s+|\s+$/g, '') + "'";
								});
							}
							if (!caseSensitive) {
								operand2Constant = operand2Constant.toUpperCase();
							}
						}
						else {
							if (!caseSensitive) {
								operand2Constant = "upcase(strip(";
							}
							operand2Constant += "'" + cond.operand2Constant + "'";
							if (!caseSensitive) {
								operand2Constant += "))";
							}
						}
						if (useLabel) {  //This is for condition display in GUI rather than code gen
							operand1ColName = cond.operand1ColumnInfo.label;
						}
						else {
							if (!caseSensitive) {
								operand1ColName = "upcase(";
							}
							operand1ColName += "strip(" + cond.operand1ColumnInfo.name + ")";
							if (!caseSensitive) {
								operand1ColName += ")";
							}
						}

					}
					else {
						operand2Constant = cond.operand2Constant;
						if (useLabel) {  //This is for condition display in GUI rather than code gen
							operand1ColName = cond.operand1ColumnInfo.label;
						}
						else {
							operand1ColName = cond.operand1ColumnInfo.name;
						}
					}

					if (cond.operator.name === "IS MISSING") {
						filtStr += operand1ColName + " IS MISSING";
					}
					else if (cond.operator.name === "NOT MISSING") {
						filtStr += operand1ColName + " NOT IS MISSING";
					}
					else if (cond.operator.name === "<" || cond.operator.name === "<=") {
						var missingInd = cond.operand1ColumnInfo.dataType === vsmCommonConst.DATA_TYPE_NUMERIC ? "." : "''";
						nullCheck = suppressNullCheck ? "" : operand1ColName + " NE " + missingInd + " AND ";
						filtStr += nullCheck + operand1ColName + " " + cond.operator.name + " " + operand2Constant;
					}
					else if (cond.operator.name === "IN" || cond.operator.name === "NOT IN" ) {
						if (operand2Constant) {
							filtStr += operand1ColName + " " + cond.operator.name + " (" + operand2Constant + ")";
						}
					}
					else {
						operand2Constant = operand2Constant.toString().replace(/,/g, "");
						if (cond.operator.name === "BETWEEN") {
							rhs = operand2Constant;
							minMax = rhs.split("...");
							filtStr += "(" + minMax[0] + " < " + operand1ColName + " <= " + minMax[1] + ")";
						}
						else {
							filtStr += operand1ColName + " " + cond.operator.name + " " + operand2Constant;
						}
					}
				}

				return filtStr;
			}

		};

	}]);

});

},
'vsd/common/ng/currencyCodes':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
/**
 * Created by stboyd on 3/27/2014.
 */
angular.module('vsd')
    .constant("currencyCodes",
    {
        "en":    "USD",
        "en_gb": "GBP",
        "en_ca": "CAD",
        "ja":    "JPY",
        "fr":    "EUR",
        "fr_ca": "CAD",
        "ko":    "KRW",
        "zh":    "CNY",
        "zh_hk": "HKD",
        "zh_tw": "TWD"
    }
);
});

},
'vsd/common/ng/reservedWords':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
/**
 * These are reserved words in ESP and/or LASR that should *not* be used in expressions or as column names (ref.
 * defect S1124902).
 *
 * @author doherk
 */
angular.module('vsd')
    .constant("reservedWords",
    {
        "esp": ["and",
            "array",
            "begin",
            "boolean",
            "bytes",
            "call",
            "date",
            "else",
            "end",
            "for",
            "global",
            "goto",
            "hidden",
            "if",
            "integer",
            "not",
            "null",
            "or",
            "pointer",
            "private",
            "public",
            "real",
            "return",
            "static",
            "step",
            "string",
            "then",
            "to",
            "visible",
            "while"],
        "lasr": [
            "__first_in_partition",
            "__last_in_partition",
            "do",
            "else",
            "end",
            "format",
            "if",
            "label",
            "length",
            "output",
            "retain",
            "then",
            "where"
        ]
    }
)
;
});

},
'vsd/common/ng/errorInterceptor':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').config(function ($httpProvider) {
    $httpProvider.interceptors.push(['$rootScope', '$q', 'messageSvc', 'dojoModules', function ($rootScope, $q, messageSvc, dojoModules) {
        return {
            "responseError": function (response) {
                var messages = [];
                var msg1 = response.data ? response.data.message : response.message;
                if (!msg1) {
                    //If there is nothing to be extracted from the response, show a general error message
                    msg1 = dojoModules.resources.app_genericException_txt;
                }
                $rootScope.$broadcast('hideLoadingMask');
                if (msg1) {
                    messages.push(msg1);
                    if (response.data && response.data.stackTrace && response.data.showStackTrace) {
                        messages.push(response.data.stackTrace.substr(0, 2000));
                    }
                    if ((typeof response.data.showMessage === undefined) ? true : response.data.showMessage) {
                        messageSvc.showError(messages);
                    }
                }

                return $q.reject(response);
            }
        };
    }
    ]);
});

});

},
'vsd/import/ImportDialogCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ImportDialogCtrl',['$scope', '$location', '$http', 'messageSvc', 'windowSvc', 'importRenameDialogSvc', 'dojoModules',
        function ($scope, $location, $http, messageSvc, windowSvc, importRenameDialogSvc, dojoModules) {

            $scope.data = {importFilename:null,
                           fileContents: null,
                           uploadFile: null,
                           windowName: null
            };

            $scope.expanded = false;

            $scope.duplicateWindows = [];

            /*
             * uploadFile function
             * CREATE a New Window
             * -OR-
             * Display the Import Rename Dialog if the XML import contains ANY windows with a name and guid
             * that are a duplicate of window(s) in the target database.
             *
             * This function sends the XML import file contents to ExportImportController.java where it processed to update the database.
             * 1) If ALL windows in the XML import file are identical (same window name and same guid) to windows already in the database,
             *    the java will return a message to the user without creating any windows.  (status = 101)
             *    - OR -
             * 2) If NO window name and guid in the XML import file is found in the database, the window(s) will be automatically imported
             *    using the names in the XML import file. (status = 100)  Also used for Deployments.
             *    -OR-
             * 3) If ANY (but not all) window in the XML import file has a duplicate (window name and guid) in the database,
             *    the importRenameDialog screen will be invoked with new, unique names for each window in the XML import file.
             *    The user may then accept those names, change any or all names to another unique name, or cancel the import. (status=200)
             *    - OR -
             * 4) Look up values are missing in the target system (102)
             *
             */
            $scope.uploadFile = function() {
                // this is the action path for a New Window
                var uploadUrl = null;
                var winInfo = null;
                var msg = null;

                var xmlContentType = null;
                var entityType = null;

                //Determine whether the selected XML file contains a window(s) or a deployment
                xmlContentType = $scope.inspectSelectedXml(xmlContentType);

                // select action based on selected content type
                if (xmlContentType === vsmCommonConst.IMPORT_ENTITY_TYPE_DEPLOYMENT) {
                    /* /import/deployment/{fileName} */
                    uploadUrl = 'action/import/deployment/';
                    entityType=xmlContentType;
                    $scope.showLoadingMask($scope.resources.importDlg_importing_deployment_txt);
                }
                else if (xmlContentType === vsmCommonConst.IMPORT_ENTITY_TYPE_WINDOW) {
                    uploadUrl = 'action/import/window';
                    entityType=xmlContentType;
                    $scope.showLoadingMask($scope.resources.importDlg_importing_txt);
                }
                else { // this will be implemented later for Scenario imports
                    //ERROR
                }

                // call ExportImportController.importScenarioWindow() to process the selected XML file
                $http.post(uploadUrl, $scope.data.uploadFile,{headers: {'Content-Type': 'text/plain;charset=utf-8'}})
                    .success(function(winInfo){
                     // if any (but not all) of the window names in the XML data is a duplicate of one in the target database
                        if (winInfo.status === vsmCommonConst.IMPORT_RETURN_STATUS_200) {

                            $scope.hideLoadingMask();

                            // reset the Import dialog buttons to the default before closing the dialog
                            // if this is not done, and the window was in preview mode, the button label
                            // will be wrong on the next invocation.
                            $scope.cancelReset();

                            // close the import dialog
                            $scope.dialogFlags.showImportDialog = false;

                            // store the primary window name for retrieval in ImportRenameDialogCtrl.js
                            importRenameDialogSvc.addWindowName(winInfo.windowName);

                            // if the object's newName property is missing, assign a default newName value
                            var idx = 0;
                            angular.forEach(winInfo.allWindows, function(oneWin) {
                                if (oneWin.newName === "") {
                                    winInfo.allWindows[idx].newName = oneWin.name + "X";
                                }
                                idx++;
                            });

                            // store the list of allXmlWindows for retrieval in ImportRenameDialogCtrl.js
                            importRenameDialogSvc.addAllXmlWindows(winInfo.allWindows.reverse());  // Reverse the order of the list so the primary window will be at the top

                            // display the import Rename dialog
                            $scope.dialogFlags.showImportRenameDialog = true;
                        }
                        else if (winInfo.status === vsmCommonConst.IMPORT_RETURN_STATUS_100) { // used by deployments; OR windows if the import file contains ALL NEW window names
                            // close the import dialog
                            $scope.dialogFlags.showImportDialog = false;

                            /* status = 100 is valid for both windows and deployments */
                            if (entityType === vsmCommonConst.IMPORT_ENTITY_TYPE_DEPLOYMENT) {
                                // Deployment Successfully Imported
                                $scope.showStatusMessage(winInfo.deploymentName + $scope.resources.importDlg_importCompleteMsg_txt);
                            }
                            else {
                                // Window Successfully Imported
                                $scope.showStatusMessage(winInfo.windowName + $scope.resources.importDlg_importCompleteMsg_txt);
                            }

                            $scope.hideLoadingMask();

                            // notify the listeners (in MainCtrl.js and Overview.js) that an import has been successfully completed
                            // this will trigger a screen refresh of the list of windows in those programs.
                            $scope.$emit(vsmCommonConst.IMPORT_DIALOG_COMPLETE);

                            // reset the Import dialog buttons to the default before closing the dialog
                            // if this is not done, and the window was in preview mode, the button label
                            // will be wrong on the next invocation.
                            $scope.cancelReset();

                            $scope.hideLoadingMask();
                        }
                        else if (winInfo.status === vsmCommonConst.IMPORT_RETURN_STATUS_101) { // status=101 - the window(s) in the XML import file are identical to existing window(s) in the database
                            // close the import dialog
                            $scope.dialogFlags.showImportDialog = false;
                            $scope.cancelReset();
                            /* importDlg_import_status101_txt=All windows in the selected XML file already exist in the target database. XYZ was not imported */
                            $scope.showInfoMessage(dojoModules.string.substitute($scope.resources.importDlg_import_status101_txt, [winInfo.windowName]));
                        }
                        else if (winInfo.status === vsmCommonConst.IMPORT_RETURN_STATUS_102) { // status=102 - look up values are missing in the target system
                            // close the import dialog
                            $scope.dialogFlags.showImportDialog = false;
                            $scope.cancelReset();

                            /* importDlg_import_status102_txt= XYZ successfully imported. Note that one or more custom site-specific reference data values are missing in the target system. */
                            /* status = 102 is valid for both windows and deployments */
                            if (entityType === vsmCommonConst.IMPORT_ENTITY_TYPE_DEPLOYMENT) {
                                // Deployment Successfully Imported
                                //$scope.showStatusMessage(winInfo.deploymentName + $scope.resources.importDlg_importCompleteMsg_txt);
                                $scope.showInfoMessage(dojoModules.string.substitute($scope.resources.importDlg_import_status102_txt, [winInfo.deploymentName]));
                            }
                            else {
                                // Window Successfully Imported
                                //$scope.showStatusMessage(winInfo.windowName + $scope.resources.importDlg_importCompleteMsg_txt);
                                $scope.showInfoMessage(dojoModules.string.substitute($scope.resources.importDlg_import_status102_txt, [winInfo.windowName]));
                            }

                            $scope.$emit(vsmCommonConst.IMPORT_DIALOG_COMPLETE);
                        }
                        else {
                            // none of the above
                        }
                    })
                    .error(function(e) {
                        // for all errors besides a DuplicateNameException, show the error message
                    	$scope.showErrorMessage(e.message);
                        // reset the Import dialog buttons to the default before closing the dialog
                        // if this is not done, and the window was in preview mode, the button label
                        // will be wrong on the next invocation.
                        $scope.cancelReset();

                        // set the flag used to close the import dialog
                        $scope.dialogFlags.showImportDialog = false;
                    });
            };

            /*
             * cancelReset function
             * Executes when the 'Cancel' button is clicked or the Import dialog is closed with X button
             * Resets Import dialog form, button labels, and button states to the default settings
             */
            $scope.cancelReset = function () {
                // clear the file input field (jQuery syntax)
                // CAUTION: DO NOT MODIFY OR REPLACE THIS FUNCTION CALL
                // See complete explanation in reset_fileInput_field block below.
                reset_fileInput_field($('#fileInput'));

                // disable upload button for next time import is selected.
                var uploadbtn = dijit.registry.byId("uploadBtn");
                uploadbtn.set("disabled", true);

                // set the flag used to close the import dialog
                $scope.dialogFlags.showImportDialog = false;
            };

            /*
             * reset_fileInput_field function
             * This function:
             * For Firefox or Chrome:
             *   1) Wraps a form around the <filereader> control (<input type=file>) in importDialog.haml
             *   2) clears the form
             *   3) removes the form (unwraps the <filereader>)
             *
             * For IE:
             *   1) Clones a copy of the input field and Replaces the existing input field with the cloned copy
             *
             * CAUTION: DO NOT MODIFY OR REPLACE THIS FUNCTION
             * This function is necessary because the file input field was designed (for security reasons)
             * so its value cannot be set programmatically -- even to an empty string.
             * However, since all fields in a form may be reset, the <filereader> may be reset by
             * wrapping the input field in a form, resetting the form, then unwrapping the input field.
             */
            function reset_fileInput_field (inputField) {
                // if navigator.userAgent is either firefox or Chrome, wrap the input field with a form, reset it and unwrap it
                if (/Firefox/i.test(navigator.userAgent) || /Chrome/i.test(navigator.userAgent)) {
                    inputField.wrap('<form>').parent('form').trigger('reset');
                    inputField.unwrap();
                }
                // if the browser is IE 10 or 11, use the clone method
                else if (/MSIE 10/i.test(navigator.userAgent) || /rv:11.0/i.test(navigator.userAgent)) {
                    $(inputField).replaceWith($(inputField).clone(true));
                }
                else {
                    $scope.showErrorMessage($scope.resources.importDlg_browser_notsupported_txt);
                }
            };

            // when the importDialog is first called, initialize the following fields
            $scope.$watch('dialogFlags.showImportDialog',
                function () {
                        if ($scope.dialogFlags.showImportDialog) {
                            $scope.data = {importFilename: null,
                                fileContents: null,
                                uploadFile: null};
                        }
                }
            );

            // determine the content type of the xml file, by searching the file for "WINDOW" or "DEPLOYOMENT" tags.
            $scope.inspectSelectedXml = function(xmlContentType) {

                var xmlFileAsString = $scope.data.uploadFile;

                if (xmlFileAsString.indexOf("<scenarioWindow>") > 0) {
                    xmlContentType = vsmCommonConst.IMPORT_ENTITY_TYPE_WINDOW;
                }
                else if (xmlFileAsString.indexOf("<deployment>")) {
                    xmlContentType = vsmCommonConst.IMPORT_ENTITY_TYPE_DEPLOYMENT;
                }
                else {
                    // xmlContentType = "NONE";
                }
                return xmlContentType;
            }

        }]);

});

},
'vsd/import/ImportRenameDialogCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ImportRenameDialogCtrl', ['$scope','$http', 'dojoDepends','importRenameDialogSvc', 'deploymentSvc', 'dojoModules', 'messageSvc', '$q', 'vsdUtil',
        function ($scope, $http, dojoDepends, importRenameDialogSvc, deploymentSvc, dojoModules, messageSvc, $q, vsdUtil) {

            $scope.data = {};

            function resetForm() {
                // list of all window names in the target database
                $scope.data.windowList =  [];

                // column definitions for the dgrid table in importRenameDialog.html
                $scope.data.gridCols =  null;

                // flag that enables/disables "Rename Content" button on UI
                $scope.data.entriesValidated = true;

                // list of all window names in the XML import file
                $scope.data.allXmlWindows = [];

                $scope.data.allEntriesToValidate = {};

                // name of primary window in XML import file
                $scope.data.windowName = null;

            }  /* end resetForm() */

            function loadObjectList() {

                // primary window in XML import file, needed as a parameter in queries below
                $scope.data.windowName = importRenameDialogSvc.getWindowName();

                // list of all window names in the XML file
                $scope.data.allXmlWindows = importRenameDialogSvc.getAllXmlWindows();

                for (var i=0; i < $scope.data.allXmlWindows.length; i++) {
                    $scope.data.allEntriesToValidate[$scope.data.allXmlWindows[i].name] = ({name:$scope.data.allXmlWindows[i].name, key:$scope.data.allXmlWindows[i].name, flag:true});
                }

                $scope.windowLabel = importRenameDialogSvc.getWindowLabel();

                $scope.data.comboBoxWidgets = {};

                $scope.data.tempVal = false;

                // gridCols contains two columns:
                //   1) Window name(s) read from XML import file ("name")
                //   2) Unique window name(s) supplied by system ("newName")
                // A changeRequired flag is returned by the java query for each window name
                // if changeRequired is true, the newName cell in the dgrid is prepopulated with an incremented value of the name cell
                // if changeRequired is false, the newName cell in the dgrid is prepopulated with the same value as that in the name cell
                $scope.data.gridCols = [
                    {
                        field: "name",
                        label: $scope.resources.renameDlg_SelectionGrid_windowName_txt,
                        sortable: false},
                    {
                        field: "newName",
                        label: $scope.resources.renameDlg_SelectionGrid_newWindowName_txt,
                        sortable: false,
                        renderCell: function(object, value, node, options) {
                            var widget = new dojoModules.ComboBox({
                                required: false,
                                store:   new dojoModules.Memory({data: $scope.data.windowList, idProperty: "name"}),
                                value: object.newName,
                                hasDownArrow: false,
                                maxLength:'25',
                                isValid: function () {
                                    // Check that value does not match any names in window list and syntax is good
                                    var valResult = vsdUtil.validateDomainObjName(this.value, _.pluck($scope.data.windowList, "name"));
                                    //var valResult = vsdUtil.validateDomainObjName(this.displayedValue, _.pluck($scope.data.windowList, "name"));
                                    if (valResult.reason === vsmCommonConst.INVALID_NAME_REASON_SYNTAX){
                                        this.set('invalidMessage', $scope.resources.app_object_name_specCharValidationMsg_txt);
                                    }
                                    else if (valResult.reason === vsmCommonConst.INVALID_NAME_REASON_DUPLICATE) {
                                        this.set('invalidMessage', $scope.resources.renameDlg_invalidName_txt);
                                    }

                                    $scope.data.allEntriesToValidate[object.name].flag = valResult.isValid;
                                    $scope.data.entriesValidated = valResult.isValid;

                                    return valResult.isValid;
                                },
                                onChange: function() {
                                    // change the object value to the displayed value
                                    object.newName = this.displayedValue;
                                    var valResult = vsdUtil.validateDomainObjName(this.displayedValue, _.pluck($scope.data.windowList, "name"));
                                    if (valResult.reason === vsmCommonConst.INVALID_NAME_REASON_SYNTAX){
                                        this.set('invalidMessage', $scope.resources.app_object_name_specCharValidationMsg_txt);
                                        $scope.data.allEntriesToValidate[object.name].flag = false;
                                        $scope.data.entriesValidated = false;
                                    }
                                    else if (valResult.reason === vsmCommonConst.INVALID_NAME_REASON_DUPLICATE) {
                                        this.set('invalidMessage', $scope.resources.renameDlg_invalidName_txt);
                                        $scope.data.allEntriesToValidate[object.name].flag = false;
                                        $scope.data.entriesValidated = false;
                                    }
                                    if (valResult.isValid === true) {
                                        $scope.data.allEntriesToValidate[object.name].flag = true;
                                        $scope.data.entriesValidated = true;
                                    }
                                   $scope.$apply();
                                }
                            });
                            // Store widget objects so that we can retrieve later
                            $scope.data.comboBoxWidgets[object.name] = widget;

                            return widget.domNode;
                        } /* end renderCell */
                    }/* end field: "newName" */
                ]; /* end $scope.data.gridCols */

                // get a list of all windows in the database for use as the comboBox validation list
                importRenameDialogSvc.getListOfAllWindows()
                    .then(function(data) {
                        $scope.data.windowList = data;
                        // Update comboBox widgets to use new data array
                        angular.forEach($scope.data.comboBoxWidgets, function(widget) {
                            widget.store.setData($scope.data.windowList);
                        });
                        $scope.hideLoadingMask();
                    });

                $scope.showLoadingMask($scope.resources.app_loading_msg_txt);

            }  /* end function loadObjectList() */

            // if the user selects the cancel button or the dialog X button,
            // we have to clear the window name and window name map from the session
            $scope.hideDialog = function () {
                 return $http['delete']('action/import/window/clearSession/'+ $scope.data.windowName)
                        .then(function (response) {
                            $scope.dialogFlags.showImportRenameDialog = false;
                            return response.data;
                        })
             }; // end hideDialog() function

            $scope.$watch('dialogFlags.showImportRenameDialog',
                function () {
                    if ($scope.dialogFlags.showImportRenameDialog) {
                        resetForm();
                        loadObjectList();
                    }
                    //$scope.dialogFlags.showImportRenameDialog = false;
                }); /* end $watch */

            // Called when values have been changed in the "New Window Name" column,
            // and the Rename Content button has been clicked
            $scope.importNewNames = function() {
                // submit the list of user modified New Window Names
                // this is the action path for a New Window
                var uploadUrl = 'action/import/window/'+ $scope.data.windowName +'/newNames';
                var winInfo = null;
                var msg = null;
                $scope.showLoadingMask($scope.resources.importDlg_importing_txt);
                // call ExportImportController.importScenarioWindow() to process the selected XML file
                $http.post(uploadUrl, $scope.data.allXmlWindows)
                    .success(function(winInfo){
                        // if any (but not all) of the window names in the XML data is a duplicate of one in the target database
                        if (winInfo.status === "200") {

                            $scope.hideLoadingMask();

                            // store the list of allXmlWindows for retrieval in the next pass of ImportRenameDialogCtrl.js
                            importRenameDialogSvc.addAllXmlWindows(winInfo.allWindows.reverse());  // Reverse the order of the list so the primary window will be at the top

                            // display the import Rename dialog
                            $scope.dialogFlags.showImportRenameDialog = true;
                        }
                        else if (winInfo.status === "100") { // this import contains all new window names

                            // close the import rename dialog
                            $scope.dialogFlags.showImportRenameDialog = false;

                            // display message that window import has been successful
                            $scope.showStatusMessage(winInfo.windowName + $scope.resources.importDlg_importCompleteMsg_txt);

                            $scope.hideLoadingMask();

                            // notify the listeners (in MainCtrl.js and Overview.js) that an import has been successfully completed
                            // this will trigger a screen refresh of the list of windows in those programs.
                            $scope.$emit(vsmCommonConst.IMPORT_DIALOG_COMPLETE);

                        }
                        else if (winInfo.status === "101") {// status=1 - the window(s) in the XML import file are identical to existing window(s) in the database
                            $scope.hideLoadingMask();
                            // close the import rename dialog
                            $scope.dialogFlags.showImportRenameDialog = false;

                            // display message that import has been successful
                            $scope.showInfoMessage(dojoModules.string.substitute($scope.resources.importDlg_import_status101_txt, [$scope.data.windowName]));
                        }
                        else if (winInfo.status === "102") { // status=102 - look up values are missing in the target system
                            $scope.hideLoadingMask();
                            // close the import dialog
                            $scope.dialogFlags.showImportRenameDialog = false;
                            $scope.showInfoMessage(dojoModules.string.substitute($scope.resources.importDlg_import_status102_txt, [$scope.data.windowName]));

                            $scope.$emit(vsmCommonConst.IMPORT_DIALOG_COMPLETE);
                        }
                        else {
                            // none of the above
                            $scope.hideLoadingMask();
                            $scope.$emit(vsmCommonConst.IMPORT_DIALOG_COMPLETE);
                        }
                    }) /* end .success() function */
                    .error(function(e) {
                        // for all errors besides a DuplicateNameException, show the error message
                        $scope.showErrorMessage(e.message);

                        // set the flag used to close the Import Rename Dialog
                        $scope.dialogFlags.showImportRenameDialog = false;
                    });  /* end error() function */

            }; /* end importNewName() function */

        } /* end ImportRenameDialogCtrl function */

    ]); /* end .controller */

});

},
'vsd/import/importRenameDialogSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
    factory('importRenameDialogSvc', ['$http', '$location', 'dojoModules', 'windowSvc', 'scenarioSvc',
        function ($http, $location, dojoModules, windowSvc, scenarioSvc) {

            // ImportDialogCtrl pushes the list of duplicate windows to this object
            // ImportRenameDialogCtrl retrieves the list of duplicate windows and adds it to its $scope.data object
            var windowName = null;
            var allXmlWindows = [];
            var windowLabel = null;

            return {
                /**
                 * getListOfAllWindows function
                 * retrieve a list of all window names currently stored in the database
                 * this will be compared to the list of windows in the import xml file
                 * to determine if any xml window names are duplicates of existing window names
                 * @returns {*} List of all existing windows
                 */
                getListOfAllWindows: function() {
                    var me = this;
                    var url = "action/browse/windows";
                    return $http.get(url).then(function(response) {
                        return me.buildListOfAllWindowsData(response.data);
                    })
                },

                /**
                 * buildListOfAllWindowsData function
                 * builds a list of all windows currently in the database.
                 * @param data - the response data created by the action/browse/windows action
                 * @returns {Array} - list of ScenarioWindow objects
                 */
                buildListOfAllWindowsData: function(data) {
                    var windowList = [];
                    data.forEach(function (row) {
                        if (row.objType == vsmCommonConst.OBJ_TYPE_WINDOW) {
                            row.id = row.id.toString();
                            row.parent = null;
                            windowList.push(row);
                        }
                        else {
                            if (row.objType == vsmCommonConst.OBJ_TYPE_DEPLOYMENT) {
                                // Add 'd' prefix so that memory store 'get' function will return correct object.
                                // Windows and deployments can possibly have the same id value
                                row.id = 'd' + row.id.toString();
                                row.parent = null;
                                windowList.push(row);
                            }
                            else {
                                row.parent = null;
                                windowList.push(row);
                            }
                        }
                    });
                    return windowList;
                },
                /**
                 * addWindowName function
                 * add windowName
                 * @param windowName - primary window name passed by ImportDialogCtrl.js
                 */
                addWindowName: function(windowname) {
                    windowName = windowname;
                },

                /**
                 * getWindowName function
                 * returns the name of the primary window (previously stored via addWindowName above) to ImportRenameDialogCtrl.js
                 * @returns windowName
                 */
                getWindowName: function() {
                    return windowName;
                },

                /**
                 * addAllXmlWindows function
                 * add all windows contained in XML import file to list
                 * @param allxmlwindows - list of all xml windows passed by ImportDialogCtrl.js
                 */
                addAllXmlWindows: function(allxmlwindows) {
                    allXmlWindows = allxmlwindows;
                },

                /**
                 * getAllXmlWindows function
                 * returns the list of all windows in xml import file (previously stored via addAllXmlWindows above) to ImportRenameDialogCtrl.js
                 * @returns {Array} allXmlWindows
                 */
                getAllXmlWindows: function() {
                    return allXmlWindows;
                },

                /**
                 * getWindowLabel function
                 * returns the window Label containing primary window name contained in XML import file
                 * @returns windowLabel
                 */
                getWindowLabel: function() {
                    return windowLabel;
                }

            };
        }]);


});

},
'vsd/main/AppCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('AppCtrl',
	['$scope', '$location', '$window', 'dojoModules', 'confirmDialogSvc', 'messageSvc', 'loadingMaskSvc', 'windowSvc',
	 'encoderSvc', 'vsdUtil',
	 function ($scope, $location, $window, dojoModules, confirmDialogSvc, messageSvc, loadingMaskSvc, windowSvc,
		 encoderSvc, vsdUtil) {

		 var resources = dojoModules.resources,
			 fmts = vsdUtil.defaultFormats;

		 function setConfirmNavDialogOpts(validInd, confirmCallback, cancelCallback, objTypeLbl, objName) {
			 //Set title and callbacks
			 confirmDialogSvc.setConfirmCallback(confirmCallback)
				 .setCancelCallback(cancelCallback);
			 //Set message and button labels depending on validity
			 if (validInd) {
				 confirmDialogSvc.setConfirmLabel($scope.resources.common_banner_toolbarSave_txt)
					 .setCancelLabel($scope.resources.confirmDlg_unsavedChanges_doNotSaveDialogLbl_txt)
					 .setMsg(dojoModules.string.substitute($scope.resources.confirmDlg_unsavedChanges_msgLbl_txt,
						 [objTypeLbl, objName]));
			 }
			 else {
				 confirmDialogSvc.setConfirmLabel($scope.resources.confirmDlg_unsavedChanges_discardChangesLbl_txt)
					 .setCancelLabel($scope.resources.confirmDlg_unsavedChanges_cancelDialogLbl_txt)
					 .setMsg(dojoModules.string.substitute($scope.resources.confirmDlg_unsavedChangesInvalid_msgLbl_txt,
						 [objTypeLbl, objName]));
			 }
			 confirmDialogSvc.showDialog();
		 }

		 function navigateWithConfirm(validInd, saveCallback, navigateCallback, objTypeLbl, objName) {
			 var confirmCallback,
				 cancelCallback;
			 //Set callbacks based on validity of current view
			 if (validInd) {
				 confirmCallback = onConfirmForValid;
				 cancelCallback = navigateCallback;
			 }
			 else {
				 confirmCallback = navigateCallback;
				 cancelCallback = null;
			 }
			 //Set confirmation dialog properties
			 setConfirmNavDialogOpts(validInd, confirmCallback, cancelCallback, objTypeLbl, objName);

			 function onConfirmForValid() {
				 if (saveCallback) {
					 saveCallback()
						 .then(function () {
							 navigateCallback();
						 });
				 }
			 }
		 }

		 function createBlankPreviewFilter() {
			 return {
				 operator:           {name: '=', label: ''},
				 operand1ColumnInfo: {name: '', label: ''},
				 operand2Constant:   ""
			 };
		 }

		 $scope.loadingMask = {show: false, message: ''};

		 $scope.showLoadingMask = function (message) {
			 loadingMaskSvc.showLoadingMask(message);
		 };

		 $scope.hideLoadingMask = function () {
			 loadingMaskSvc.hideLoadingMask();
		 };


		 $scope.showErrorMessage = function (message) {
			 $scope.hideLoadingMask();
			 messageSvc.showError(message);
		 };

		 $scope.showInfoMessage = function (message) {
			 $scope.hideLoadingMask();
			 messageSvc.showInfo(message);
		 };

		 $scope.showStatusMessage = function (message) {
			 $scope.hideLoadingMask();
			 messageSvc.showStatus(message);
		 };

		 $scope.createBlankPreviewFilter = createBlankPreviewFilter;


		 $scope.$on('hideLoadingMask', function (evt) {
			 $scope.hideLoadingMask();
		 });

		 $scope.resources = resources;

		 $scope.username = encoderSvc.encodeForDojo($window.username);

		 $scope.vaeAuthorized = $window.vaeAuthorized;

		 $scope.paths = {
			 scenario: '/scenario'
		 };

		 $scope.commonLists = {
			 trueFalse:      [
				 {id: "1", name: resources.scenario_ActionBuilder_true_txt},
				 {id: "0", name: resources.scenario_ActionBuilder_false_txt}
			 ],
			 dataTypes:      [
				 {id: "CHARACTER", name: resources.scenario_ActionColumnEditor_charType_txt},
				 {id: "NUMERIC", name: resources.scenario_ActionColumnEditor_numType_txt}
			 ],
			 entityTypes:    [],
			 numericFormats: [
				 {name: fmts.none, label: resources.scWindow_formats_none},
				 {name: fmts.number, label: resources.scWindow_formats_number},
				 {name: fmts.percentage, label: resources.scWindow_formats_percentage},
				 {name: fmts.currency, label: resources.scWindow_formats_currency},
				 {name: fmts.integer, label: resources.scWindow_formats_integer},
				 {name: fmts.date, label: resources.scWindow_formats_date},
				 {name: fmts.datetime, label: resources.scWindow_formats_datetime}
			 ]
		 };

		 windowSvc.getEntityTypes().then(function (value) {
			 $scope.commonLists.entityTypes = value;
		 });

         windowSvc.getOutputTypes().then(function (value) {
             $scope.commonLists.outputTypes = value;
         });

		 $scope.dialogFlags = {
			 showAboutDialog:         false,
			 showConfirmationDialog:  false,
			 showNewDeploymentDialog: false,
			 showNewScenarioDialog:   false,
			 showNewWindowDialog:     false,
			 showOpenDialog:          false,
			 showImportDialog:        false,
			 showImportRenameDialog:  false,
			 showSaveAsDialog:        false,
             showHelpDialog:          false
		 };

		 $scope.tree = dojoModules.tree;

		 $scope.newWindow = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName) {
			 if (angular.isDefined(dirtyInd) && dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, launchNewWindowDialog, objTypeLbl, objName);
			 }
			 //Otherwise, just show new scenario dialog
			 else {
				 launchNewWindowDialog();
			 }

			 function launchNewWindowDialog() {
				 $scope.dialogFlags.showNewWindowDialog = true;
			 }
		 };

		 $scope.newScenario = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName) {
			 //If current view dirty, make user confirm navigation
			 if (angular.isDefined(dirtyInd) && dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, launchNewScenarioDialog, objTypeLbl, objName);
			 }
			 //Otherwise, just show new scenario dialog
			 else {
				 launchNewScenarioDialog();
			 }
			 function launchNewScenarioDialog() {
				 $scope.dialogFlags.showNewScenarioDialog = true;
			 }
		 };

		 $scope.newDeployment = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName) {
			 //If current view dirty, make user confirm navigation
			 if (angular.isDefined(dirtyInd) && dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, launchNewDeploymentDialog, objTypeLbl, objName);
			 }
			 //Otherwise, just show new scenario dialog
			 else {
				 launchNewDeploymentDialog();
			 }
			 function launchNewDeploymentDialog() {
				 $scope.dialogFlags.showNewDeploymentDialog = true;
			 }
		 };

		 $scope.showOpen = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName) {
			 //If current view dirty, make user confirm navigation
			 if (angular.isDefined(dirtyInd) && dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, launchOpenDialog, objTypeLbl, objName);
			 }
			 //Otherwise, just show new scenario dialog
			 else {
				 launchOpenDialog();
			 }
			 function launchOpenDialog() {
				 $scope.dialogFlags.showOpenDialog = true;
			 }
		 };

		 $scope.goToHub = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName) {
			 //If current view dirty, make user confirm navigation
			 if (dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, goToHub, objTypeLbl, objName);
			 }
			 //Otherwise, just navigate
			 else {
				 goToHub();
			 }

			 function goToHub() {
				 $scope.showLoadingMask(resources.main_loadingVaHub_msg_txt);
				 angular.element($window).off("beforeunload");
				 $window.location = "transfer.jsp";
			 }
		 };

		 $scope.showHome = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName) {
			 //If current view dirty, make user confirm navigation
			 if (dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, navigateToHome, objTypeLbl, objName);
			 }
			 //Otherwise, just navigate
			 else {
				 navigateToHome();
			 }

			 function navigateToHome() {
				 $location.path("/");
			 }
		 };

		 $scope.showWindow = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName, windowId) {
			 //If current view dirty, make user confirm navigation
			 if (dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, navigateToWindow, objTypeLbl, objName);
			 }
			 //Otherwise, just navigate
			 else {
				 navigateToWindow();
			 }

			 function navigateToWindow() {
				 $location.path("/window/" + windowId);
			 }
		 };

		 $scope.showScenario = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName, scenarioId) {
			 //If current view dirty, make user confirm navigation
			 if (dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, navigateToScenario, objTypeLbl, objName);
			 }
			 //Otherwise, just navigate
			 else {
				 navigateToScenario();
			 }

			 function navigateToScenario() {
				 var locStr = "/scenario/" + scenarioId;
				 $location.path(locStr);
			 }
		 };

		 $scope.showDeployment = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName, deploymentId) {
			 //If current view dirty, make user confirm navigation
			 if (dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, navigateToDeployment, objTypeLbl, objName);
			 }
			 //Otherwise, just navigate
			 else {
				 navigateToDeployment();
			 }

			 function navigateToDeployment() {
				 $location.path("/deployment/" + deploymentId);
			 }
		 };

		 $scope.showSimulation = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName, deploymentId) {
			 //If current view dirty, make user confirm navigation
			 if (dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, navigateToSimulation, objTypeLbl, objName);
			 }
			 //Otherwise, just navigate
			 else {
				 navigateToSimulation();
			 }

			 function navigateToSimulation() {
				 $location.path("/deployment/" + deploymentId + "/simulation");
			 }
		 };

		 $scope.showSaveAs = function (dirtyInd, validInd, saveCallback, objTypeLbl, objName) {
			 //If current view dirty, make user confirm navigation
			 if (dirtyInd) {
				 navigateWithConfirm(validInd, saveCallback, showSaveAsDialog, objTypeLbl, objName);
			 }
			 //Otherwise, just show dialog
			 else {
				 showSaveAsDialog();
			 }

			 function showSaveAsDialog() {
				 $scope.dialogFlags.showSaveAsDialog = true;
			 }
		 };


		 $scope.showAboutDialog = function () {
			 $scope.dialogFlags.showAboutDialog = true;
		 };

		 $scope.closeAboutDialog = function () {
			 $scope.dialogFlags.showAboutDialog = false;
		 };

		 $scope.openHelp = function openHelp(path) {
			 $window.open(path);
		 };

         $scope.showHelpDialog = function () {
             $scope.dialogFlags.showHelpDialog = true;
         };

         $scope.closeHelpDialog = function () {
             $scope.dialogFlags.showHelpDialog = false;
         };

		 $scope.signOut = function () {
			 $scope.showLoadingMask(resources.app_signingOut_txt);
			 angular.element($window).off("beforeunload");
			 $window.location = 'Logoff';
		 };

		 $scope.showImportDialog = function () {
			 $scope.dialogFlags.showImportDialog = true;
		 };

		 $scope.$on('$routeChangeStart', function (evt, next, current) {
				 if (current) {
					 dojoModules.registry.findWidgets(document.body).forEach(function (w) {
							 if (!w._destroyed && w.domNode.className.indexOf('globalDijit') < 0 &&
								 w.domNode.className.indexOf('dijitTooltip') < 0) {
								 if (w.destroyRecursive) {
									 w.destroyRecursive();
								 }
								 else {
									 w.destroy();
								 }
							 }
						 }
					 )
					 ;
				 }
			 }
		 )
		 ;
	 }
	])
;

});

},
'vsd/main/MainCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
	controller('MainCtrl',
		['$scope', '$http', '$location', '$route', '$cookies', 'windowSvc', 'scenarioSvc', 'confirmDialogSvc', 'dojoModules', 'objectBrowseSvc',
		 function ($scope, $http, $location, $route, $cookies, windowSvc, scenarioSvc, confirmDialogSvc, dojoModules, objectBrowseSvc) {

			 var resources = $scope.resources,
				 loadingItemsMsg = resources.main_loadingAvailFiles_msg_txt;

			 //region *** Private Functions ***
			 function refreshGrid(data) {
				 var leftGridData = [];
				 var rightGridDdata = [];
				 data.forEach(function (row) {
					 if (row.objType === 'WINDOW') {
						 row.id = row.id.toString();
						 row.parent = null;
						 leftGridData.push(row);

						 row.scenarios.forEach(function (scen) {
							 scen.id = 's' + scen.id;
							 scen.objType = 'SCENARIO';
							 scen.windowName = row.name;
							 scen.parent = row.id;
							 leftGridData.push(scen);
						 });
					 }
					 else {
						 if (row.objType === 'DEPLOYMENT') {
							 // Add 'd' prefix so that memory store 'get' function will return correct object.
							 // Windows and deployments can possibly have the same id value
							 row.id = 'd' + row.id.toString();
							 row.parent = null;
							 rightGridDdata.push(row);
						 }
						 else {
							 row.parent = null;
							 leftGridData.push(row);
						 }
					 }
				 });

				 return { left: leftGridData, right: rightGridDdata};
			 }

			 function removeItem() {
				 if ($scope.boundPrimitives.selObject) {
					 $scope.showLoadingMask(resources.app_deleting_msg_txt);
					 var objId = $scope.boundPrimitives.selObject.id,
                         objName =  $scope.boundPrimitives.selObject.name;

					 if ($scope.boundPrimitives.selObject.objType === 'WINDOW') {
						 windowSvc.deleteWindow(objId, $scope.boundPrimitives.selObject.lstupdtNbr).then(
							 $scope.items.refresh
						 );
					 }
					 else {
						 if ($scope.boundPrimitives.selObject.objType === 'SCENARIO') {
							 objId = objId.substr(1);
							 scenarioSvc.deleteScenario(objId, $scope.boundPrimitives.selObject.lstupdtNbr).then(function(result) {
                                 if(!result.success) {
                                     scenarioSvc.showFailedDeleteMsg(result,objName);
                                     $scope.hideLoadingMask();
                                 }
                                 else {
                                     $scope.items.refresh();
                                 }

                             });
						 }
					 }
					 $scope.boundPrimitives.rowSelected = false;
					 $scope.boundPrimitives.selObject = null;
				 }
			 }

			 //endregion

			 //region *** ViewModel properties ***
			 $scope.boundPrimitives = {
				 showNewWindowDialog:     false,
				 showNewDeploymentDialog: false,
				 showRunSimulationDialog: false,
				 showOverviewDialog:      false,
				 objectListFilterStr:     "",
				 runSimulation:           false,
				 selObject:               null,
				 rowSelected:             false,
				 objectDeletable:         false
			 };

			 $scope.showLoadingMask(loadingItemsMsg);
			 // $scope.selObject = null;
			 // $scope.objectList = [];
			 $scope.objectIdProperty = "id";

			 $scope.leftTreeCols = [
				 $scope.tree({
					 field:             "name",
					 label:             resources.common_OpenDialog_name_txt,
					 sortable:          false,
					 /* shouldExpand:       function(){ return true; }*/
					 collapseOnRefresh: true
				 }),

				 { field: "objType",
					 label: resources.common_OpenDialog_type_txt,
					 sortable: false,
					 formatter: objectBrowseSvc.getObjectTypeLabel
		       },
				 /* { field: "createUser", label: resources.common_OpenDialog_createdBy_txt, sortable: true},*/
				 { field:      "lstupdtTimestamp",
					 label:     resources.common_OpenDialog_lastModified_txt,
					 sortable:  false,
					 formatter: function (value) {
						 return (value ? dojoModules.locale.format(new Date(value)) : "");
					 }
				 }
			 ];

			 $scope.rightGridCols = [
				 {
					 field:             "name",
					 label:             resources.common_OpenDialog_name_txt,
					 sortable:          false,
					 collapseOnRefresh: true
				 },
				 /* { field: "createUser", label: resources.common_OpenDialog_createdBy_txt, sortable: true},*/
				 { field:      "lstupdtTimestamp",
					 label:     resources.common_OpenDialog_lastModified_txt,
					 sortable:  false,
					 formatter: function (value) {
						 return (value ? dojoModules.locale.format(new Date(value)) : "");
					 }
				 }
			 ];
			 //endregion

			 //region *** ViewModel Methods ***
			 $scope.openObject = function () {
				 var locStr = "/",
					 selObject = $scope.boundPrimitives.selObject;

				 switch (selObject.objType) {
					 case "WINDOW":
						 locStr += "window/";
						 locStr += selObject.id;
						 break;
					 case "SCENARIO":
						 locStr += "scenario/";
						 locStr += selObject.id.substring(1);
						 break;
					 case "DEPLOYMENT":
						 locStr += "deployment/";
						 locStr += selObject.id.substring(1);
						 if ($scope.boundPrimitives.runSimulation) {
							 locStr += "/simulation";
						 }
						 break;
				 }
				 //console.log(" opening: " + selObject.name);
				 $location.path(locStr);
			 };

			 $scope.onRowSelection = function (selObject) {
				 $scope.boundPrimitives.selObject = selObject;
				 $scope.boundPrimitives.rowSelected = !!selObject;
				 $scope.boundPrimitives.objectDeletable = $scope.boundPrimitives.rowSelected && selObject && selObject.objType !== 'DEPLOYMENT';
			 };

			 $scope.onEnterKeyHandler = function (selObject) {
				 $scope.boundPrimitives.selObject = selObject;
				 $scope.openObject();

			 }

             $scope.onDeleteKeyHandler = function (selObject) {
                 $scope.boundPrimitives.selObject = selObject;
                 $scope.items.deleteItem();
             }

			 $scope.objectListGetChildren = function (store, parent, options) {
				 return store.query({parent: parent.id}, options);
			 };

			 $scope.objectListMayHaveChildren = function (store, parent) {
				 return parent.scenarios && parent.scenarios.length > 0 && store.getChildren(parent).length > 0;
			 };

			 $scope.objectListFilterQuery = function (item, index, items) {
				 var filterString = $scope.boundPrimitives.objectListFilterStr.toLowerCase();
				 if (filterString.length < 1) {
					 return !!!item.parent;
				 }
				 else {
					 if (!item.parent) {
						 var name = item.name.toLowerCase();
						 var matched = name.indexOf(filterString.toLowerCase()) > -1;
						 if (matched) {
							 return true;
						 }
						 //check scenarios, if any match, show parent
						 if (item.scenarios) {
							 var childrenMatched = false;
							 item.scenarios.forEach(function (s) {
								 name = s.name.toLowerCase();
								 childrenMatched = childrenMatched || (name.indexOf(filterString.toLowerCase()) > -1);
							 });
							 return childrenMatched;
						 }
					 }
				 }

				 return false;
			 };

			 $scope.openOverview = function () {
				 $scope.boundPrimitives.showOverviewDialog = true;
				 // var locStr = "/home/";
				 // $location.path(locStr);
			 };

			 $scope.goToHubScreen = function () {
				 $scope.goToHub();
			 };


			 $scope.runSimulation = function () {
				 $scope.boundPrimitives.showRunSimulationDialog = true;
			 };

			 $scope.runExport = function () {
				 // did not work with .then or return statements
				 windowSvc.exportIframe($scope.boundPrimitives.selObject.id, $scope.boundPrimitives.selObject.objType);
			 };

			 $scope.items = {
				 refresh:    function () {
					 $scope.showLoadingMask(loadingItemsMsg);
					 var url = "action/browse/user/recent";
					 $http.get(url)
						 .success(function (data) {
							 var dataGrids = refreshGrid(data);
							 $scope.leftGridData = dataGrids.left || [];
							 $scope.rightGridData = dataGrids.right || [];
							 $scope.hideLoadingMask();
                         });
				 },
				 deleteItem: function () {
					 var selObject = $scope.boundPrimitives.selObject;
					 confirmDialogSvc.setTitle($scope.resources.scWindow_HomePanel_deleteConfirmDlgTitle_txt + ' ' + (
							 selObject.objType === 'WINDOW' ? $scope.resources.obj_type_window_txt :
								 $scope.resources.obj_type_scenario_txt))
						 .setConfirmCallback(function () {
							 removeItem();
						 });
//				      .setCancelCallback(angular.noop);
					 //Set message and button labels depending on validity
					 confirmDialogSvc.setConfirmLabel($scope.resources.common_OpenDialog_deleteBtn_txt)
						 .setCancelLabel($scope.resources.confirmDlg_unsavedChanges_cancelDialogLbl_txt)
						 .setMsg(dojoModules.string.substitute($scope.resources.scWindow_HomePanel_deleteConfirmMsg_txt,
							 [selObject.name]));
					 confirmDialogSvc.showDialog();

				 }
			 };

			 $scope.$watch('boundPrimitives.showRunSimulationDialog', function () {
				 if (!$scope.boundPrimitives.showRunSimulationDialog && $scope.boundPrimitives.selObject && $scope.boundPrimitives.runSimulation) {
					 $scope.openObject();
				 }
			 });


			 // refresh the screen if an import has been successfully completed
			 $scope.$on(vsmCommonConst.IMPORT_DIALOG_COMPLETE, function () {
				 $scope.items.refresh();
                 $route.reload();  // force a page refresh as a workaround for Chrome bug
			 });

			 // Add Export popmenu for right mouse button click on a window row
			 $scope.getRMBMenuItems = function (rows) {
				 //var row = row[0].data;
				 var menuItems = [];
				 $scope.boundPrimitives.selObject = rows[0].data;
				 if ($scope.boundPrimitives.selObject.objType === 'WINDOW' || $scope.boundPrimitives.selObject.objType === 'DEPLOYMENT') { // allow deployment export
					 menuItems.push({label: $scope.resources.common_banner_toolbarExport_txt,
						 onClick:            function (evt) {
							 $scope.runExport();
							 $scope.$apply();
						 }});
				 }

				 menuItems.push({label: $scope.resources.common_banner_toolbarOpen_txt,
					 onClick:            function (evt) {
						 $scope.openObject();
						 $scope.$apply();
					 }});

				 if ($scope.boundPrimitives.selObject.objType !== 'DEPLOYMENT') {
					 menuItems.push({label: $scope.resources.common_banner_toolbarDelete_txt,
						 onClick:            function (evt) {
							 $scope.items.deleteItem();
							 $scope.$apply();
						 } });
				 }
				 return menuItems;
			 };


//endregion
			 $scope.items.refresh();
		 }]);


});

},
'vsd/main/OverviewCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
	controller('OverviewCtrl',
		['$scope', '$http', '$location', '$cookies', 'windowSvc', 'scenarioSvc', 'confirmDialogSvc', 'dojoModules', 'objectBrowseSvc',
		 function ($scope, $http, $location, $cookies, windowSvc, scenarioSvc, confirmDialogSvc, dojoModules, objectBrowseSvc) {

			 /* dojoModules.registry.forEach(function (w) {
			  if (w.domNode.className.indexOf('globalDijit') < 0 &&
			  w.domNode.className.indexOf('dijitTooltip') < 0) {
			  w.destroyRecursive();
			  }
			  });
			  */
			 var resources = $scope.resources,
				 loadingItemsMsg = resources.main_loadingAvailFiles_msg_txt,
				 showAllCookieName = 'vsdHomeShowAllItems';

			 //region *** Private Functions ***
			 function refreshGrid(data) {
				 var treedata = [];
				 data.forEach(function (row) {
					 if (row.objType === 'WINDOW') {
						 row.id = row.id.toString();
						 row.parent = null;
						 treedata.push(row);

						 row.scenarios.forEach(function (scen) {
							 scen.id = 's' + scen.id;
							 scen.objType = 'SCENARIO';
							 scen.windowName = row.name;
							 scen.parent = row.id;
							 treedata.push(scen);
						 });
					 }
					 else {
						 if (row.objType === 'DEPLOYMENT') {
							 // Add 'd' prefix so that memory store 'get' function will return correct object.
							 // Windows and deployments can possibly have the same id value
							 row.id = 'd' + row.id.toString();
							 row.parent = null;
							 treedata.push(row);
						 }
						 else {
							 row.parent = null;
							 treedata.push(row);
						 }
					 }
				 });

				 return treedata;
			 }

			 //endregion


			 //region *** ViewModel properties ***
			 /* $scope.boundPrimitives = {
			  showNewWindowDialog:     false,
			  showNewDeploymentDialog: false,
			  showImportDialog:        false,
			  objectListFilterStr:     "",
			  objectDeletable:         false
			  };
			  */
			 $scope.boundPrimitives.listFilterStr = "";
			 $scope.boundPrimitives.objectDeletable = false;
			 $scope.boundPrimitives.objectExportable = false;

			 $scope.selObject = null;
			 $scope.objectList = [];
			 $scope.objectIdProperty = "id";

			 $scope.gridCols = [
				 $scope.tree({
					 field:             "name",
					 label:             resources.common_OpenDialog_name_txt,
					 sortable:          true,
					 collapseOnRefresh: true
				 }),

				 { field: "objType",
					 label: resources.common_OpenDialog_type_txt,
					 sortable: true,
			       formatter: objectBrowseSvc.getObjectTypeLabel
				 },
				 { field: "createUser", label: resources.common_OpenDialog_createdBy_txt, sortable: true},
				 { field: "lstupdtUser", label: resources.common_OpenDialog_updatedBy_txt, sortable: true },
				 { field:      "lstupdtTimestamp",
					 label:     resources.common_OpenDialog_lastModified_txt,
					 sortable:  true,
					 formatter: function (value) {
						 return (value ? dojoModules.locale.format(new Date(value)) : "");
					 }
				 }
			 ];
			 //endregion


			 //region *** ViewModel Methods ***
			 $scope.openObject = function () {
				 var locStr = "/",
					 selObject = $scope.selObject;

				 switch (selObject.objType) {
					 case "WINDOW":
						 locStr += "window/";
						 locStr += selObject.id;
						 break;
					 case "SCENARIO":
						 locStr += "scenario/";
						 locStr += selObject.id.substring(1);
						 break;
					 case "DEPLOYMENT":
						 locStr += "deployment/";
						 locStr += selObject.id.substring(1);
						 break;
				 }
				 $location.path(locStr);
			 };

			 $scope.showMain = function () {
				 $location.path("/");
			 };

			 $scope.onRowSelection = function (selObject) {
				 $scope.selObject = selObject;
				 $scope.boundPrimitives.objectDeletable = selObject && selObject.objType !== 'DEPLOYMENT';
                 // enable export on all window types AND deployments
				 $scope.boundPrimitives.objectExportable = selObject && selObject.objType === 'WINDOW' || selObject && selObject.objType === 'DEPLOYMENT' ;
			 };

			 $scope.overviewListGetChildren = function (store, parent, options) {
				 return store.query({parent: parent.id}, options);
			 };

			 $scope.overviewListMayHaveChildren = function (store, parent) {
				 return parent.scenarios && parent.scenarios.length > 0 && store.getChildren(parent).length > 0;
			 };

			 $scope.overviewListFilterQuery = function (item, index, items) {
				 var filterString = $scope.boundPrimitives.listFilterStr.toLowerCase();
				 if (filterString.length < 1) {
					 return !!!item.parent;
				 }
				 else {
					 if (!item.parent) {
						 var name = item.name.toLowerCase();
						 var matched = name.indexOf(filterString.toLowerCase()) > -1;
						 if (matched) {
							 return true;
						 }
						 //check scenarios, if any match, show parent
						 if (item.scenarios) {
							 var childrenMatched = false;
							 item.scenarios.forEach(function (s) {
								 name = s.name.toLowerCase();
								 childrenMatched = childrenMatched || (name.indexOf(filterString.toLowerCase()) > -1);
							 });
							 return childrenMatched;
						 }
					 }
				 }

				 return false;
			 };

			 $scope.cancelDialog = function () {
				 $scope.boundPrimitives.showOverviewDialog = false;
			 };

			 $scope.items = {
				 refresh:    function () {
					 $scope.showLoadingMask(loadingItemsMsg);
					 var url = "action/browse/user";
					 // using jQuery here is not ideal but there is a timing issue using ngModel
					 $cookies[showAllCookieName] = "" + $('#showAllItems').is(':checked');
//					 if ($scope.items.showAll) {
					 if ($('#showAllItems').is(':checked')) {
						 url = "action/browse/all";
					 }
					 $http.get(url)
						 .success(function (data) {
							 $scope.objectList = refreshGrid(data);
							 $scope.hideLoadingMask();
						 });
				 },
				 deleteItem: function () {

					 var selWindow = $scope.selObject;
					 confirmDialogSvc.setTitle($scope.resources.scWindow_HomePanel_deleteConfirmDlgTitle_txt + ' ' + (
							 $scope.selObject.objType === 'WINDOW' ? $scope.resources.obj_type_window_txt :
								 $scope.resources.obj_type_scenario_txt))
						 .setConfirmCallback(function () {
							 removeItem();
						 });
//				      .setCancelCallback(angular.noop);
					 //Set message and button labels depending on validity
					 confirmDialogSvc.setConfirmLabel($scope.resources.common_OpenDialog_deleteBtn_txt)
						 .setCancelLabel($scope.resources.confirmDlg_unsavedChanges_cancelDialogLbl_txt)
						 .setMsg(dojoModules.string.substitute($scope.resources.scWindow_HomePanel_deleteConfirmMsg_txt,
							 [$scope.selObject.name]));
					 confirmDialogSvc.showDialog();

				 },
				 showAll:    ($cookies[showAllCookieName] === 'true'),
				 onShow:     function () {
					 this.showAll = ($cookies[showAllCookieName] === 'true');
					 this.refresh();
				 }
			 };

			 function removeItem() {
				 if ($scope.selObject) {
					 $scope.showLoadingMask(resources.app_deleting_msg_txt);
					 var objId = $scope.selObject.id,
                        objName = $scope.selObject.name;
					 if ($scope.selObject.objType === 'WINDOW') {
						 windowSvc.deleteWindow(objId, $scope.selObject.lstupdtNbr)
							 .then(
								 $scope.items.refresh)
							 .then(
								 // FIXME: accessing $parent is not a good idea; but this is getting refactored anyway
								 $scope.$parent.items.refresh
							 );
					 }
					 else {
						 if ($scope.selObject.objType === 'SCENARIO') {
							 objId = objId.substr(1);
							 scenarioSvc.deleteScenario(objId, $scope.selObject.lstupdtNbr).then(function(result) {
                                 if(!result.success) {
                                     scenarioSvc.showFailedDeleteMsg(result,objName);
                                     $scope.hideLoadingMask();
                                 }
                                 else {
                                     $scope.items.refresh();
                                 }
                             });
						 }
					 }
					 $scope.selObject = null;
				 }
			 }

			 // Add Export popmenu for right mouse button click on a window row
			 $scope.getRMBMenuItems = function (rows) {
				 //var row = row[0].data;
				 var menuItems = [];
				 $scope.selObject = rows[0].data;
                 if ($scope.selObject.objType === 'WINDOW' || $scope.selObject.objType === 'DEPLOYMENT') {
					 menuItems.push({label: $scope.resources.common_banner_toolbarExport_txt,
						 onClick:            function (evt) {
							 $scope.runExport();
							 $scope.$apply();
						 }});
				 }

				 menuItems.push({label: $scope.resources.common_banner_toolbarOpen_txt,
					 onClick:            function (evt) {
						 $scope.openObject();
						 $scope.$apply();
					 }});

				 if ($scope.selObject.objType !== 'DEPLOYMENT') {
					 menuItems.push({label: $scope.resources.common_banner_toolbarDelete_txt,
						 onClick:            function (evt) {
							 $scope.items.deleteItem();
							 $scope.$apply();
						 } });
				 }
				 return menuItems;
			 };

             $scope.onEnterKeyHandler = function (selObject) {
                 $scope.boundPrimitives.selObject = selObject;
                 $scope.openObject();
             }

             $scope.onDeleteKeyHandler = function (selObject) {
                 $scope.boundPrimitives.selObject = selObject;
                 if ($scope.selObject.objType !== 'DEPLOYMENT') {   // Do not allow Deployments to be deleted
                     $scope.items.deleteItem();
                 }
             }

             $scope.runExport = function () {
				 // did not work with .then or return statements
				 windowSvc.exportIframe($scope.selObject.id, $scope.selObject.objType);
			 };

			 // refresh the screen if an import has been successfully completed
			 // this is triggered by an $emit in ImportDialogCtrl.js
			 $scope.$on(vsmCommonConst.IMPORT_DIALOG_COMPLETE, function () {
				 $scope.items.refresh();
			 });
//endregion

		 }]);


});

},
'vsd/deployment/DeploymentCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('DeploymentCtrl', ['$scope',
        'deploymentSvc',
        'scenarioSvc',
        'windowSvc',
        '$routeParams',
        'dojoModules',
        '$location',
        'newDeploymentSvc',
        'simulationSvc',
        'confirmDialogSvc',
        'vsdUtil',
        '$timeout',
        '$q',
        function ($scope, deploymentSvc, scenarioSvc, windowSvc, $routeParams, dojoModules, $location, newDeploymentSvc, simulationSvc, confirmDialogSvc, vsdUtil, $timeout, $q) {

            //region *** Private Variables ***

            var resources = dojoModules.resources,
                selVersionWatch = null,
                nodeIndex = 0;

            //end region *** Private Variables ***


            //region *** Private Functions ***

            function initializeDeployment() {

                $scope.initializing = true;

                simulationSvc.setParameters({
                    deployment: null
                });

                // If id specified on url, open that deployment
                if ($routeParams.id || $routeParams.name) {
                    openDeployment($routeParams);
                }

                //Otherwise, create a new deployment
                else {
                    var newDeployment = newDeploymentSvc.getDeployment();
                    if (newDeployment) {
                        loadNewDeployment(newDeployment);
                    }
                    else {
                        //$scope.showErrorMessage(resources.deployment_openError_txt);
                        $scope.showHome();
                    }
                }
            }

            function openDeployment(depParams) {
                $scope.showLoadingMask(resources.main_loadingDeployment_msg_txt);
                if (depParams.id) {
                    deploymentSvc.getDeployment(depParams.id)
                        .then(function (results) {
                            $scope.deployment = results;
                            getEspCompatibility();
                            loadSelectionLists();
                            //$scope.hideLoadingMask();
                            displayUnshownScenariosMsg();
                        });
                }
            }

            function loadNewDeployment(newDeployment) {
                $scope.showLoadingMask(resources.main_loadingDeployment_msg_txt);

                angular.forEach(newDeployment.windows, function (win) {
                    adjustNullConditions(win);
                });
                $scope.deployment = newDeployment;
                loadSelectionLists();
                //$scope.hideLoadingMask();
                displayUnshownScenariosMsg();
            }

            function getEspCompatibility() {
                $scope.boundPrimitives.espIncompatibleReasons = null;
                deploymentSvc.checkEspCompatibility($scope.deployment.id)
                    .then(function (messages) {
                        if (messages.length > 0) {
                            $scope.boundPrimitives.espIncompatibleReasons = messages.join("\n&#8226;&nbsp;");
                        }
                    });
            }

            function loadSelectedVersion(versionNumber) {
                $scope.initializing = true;
                $scope.showLoadingMask(resources.main_loadingDeployment_msg_txt);
                deploymentSvc.getDeploymentByName($scope.deployment.name, versionNumber)
                    .then(function (results) {
                        //console.log('getDeployment: ', results);
                        $scope.deployment = results;
                        getEspCompatibility();
                        loadVersions(true);   // load versions list and initialize deployment
                        //$scope.hideLoadingMask();
                        displayUnshownScenariosMsg();
                    });
            }


            function addWindowToNodeHierarchy(win, parentIndex) {
                var windowNodeIndex = nodeIndex;
                $scope.diagram.nodeData.push({ key: windowNodeIndex, parent: parentIndex, name: win.name, nodeType: resources.objTypeLabel_window_txt, toolTip: resources.objTypeLabel_window_txt});
                nodeIndex++;

                angular.forEach(win.scenarios, function (scen) {
                    $scope.diagram.nodeData.push({ key: nodeIndex, parent: windowNodeIndex, name: scen.name, nodeType: resources.objTypeLabel_scenario_txt, toolTip: resources.objTypeLabel_scenario_txt});
                    nodeIndex++;
                });

                var firstWinInput = win.windowInputs[0];

                if (firstWinInput) {
                    if (firstWinInput.leftHandTable) {
                        $scope.diagram.nodeData.push({ key: nodeIndex, parent: windowNodeIndex, name: firstWinInput.leftHandTable.name, nodeType: resources.objTypeLabel_table_txt, toolTip: resources.objTypeLabel_table_txt});
                        nodeIndex++;
                    }
                    else {
                        if (firstWinInput.leftHandWindow) {
                            addWindowToNodeHierarchy(firstWinInput.leftHandWindow, windowNodeIndex);
                        }
                    }
                }

                angular.forEach(win.windowInputs, function (winInput) {

                    if (winInput.rightHandTable) {
                        $scope.diagram.nodeData.push({ key: nodeIndex, parent: windowNodeIndex, name: winInput.rightHandTable.name, nodeType: resources.objTypeLabel_table_txt, toolTip: resources.objTypeLabel_table_txt});
                        nodeIndex++;
                    }
                    else if (winInput.rightHandWindow) {
                        addWindowToNodeHierarchy(winInput.rightHandWindow, windowNodeIndex);
                    }
                });
            }

            function prepareHierarchicalDisplay() {

                // register the width & height of initial window for sng-resize
                // sngDiagram watches for stop and calls gojs requestUpdate

                var handle = dojoModules.registry.byId("hierarchicalViewResizeHandle");
                dojo.subscribe("/dojo/resize/stop", function (n) {
                    var targetDomNode = dojo.marginBox(handle.targetDomNode);
                    $scope.diagram.size = {h: targetDomNode.h, w: targetDomNode.w};
                    $scope.$apply();
                });

                $scope.diagram.legendData = [
                    {nodeType: resources.objTypeLabel_table_txt, color: vsmCommonConst.DIAGRAM_TABLE_COLOR},
                    {nodeType: resources.objTypeLabel_scenario_txt, color: vsmCommonConst.DIAGRAM_SCENARIO_COLOR},
                    {nodeType: resources.objTypeLabel_window_txt, color: vsmCommonConst.DIAGRAM_WINDOW_COLOR},
                    {nodeType: resources.objTypeLabel_deployment_txt, color: vsmCommonConst.DIAGRAM_DEPLOYMENT_COLOR}
                ];

                $scope.diagram.nodeData = [];

                $scope.diagram.nodeData.push({ key: 0, name: $scope.deployment.name, nodeType: resources.objTypeLabel_deployment_txt, toolTip: resources.objTypeLabel_deployment_txt});
                nodeIndex++;
                angular.forEach($scope.deployment.windows, function (win) {
                    if (!win.predecessor)
                        addWindowToNodeHierarchy(win, 0);
                });
            }


            function prepareDeployment() {

                $scope.initializing = true;
                $scope.boundPrimitives.isValid = true;
                $scope.boundPrimitives.isDirty = false;
                $scope.boundPrimitives.selWindow = null;
                $scope.boundPrimitives.filterSelWindow = null;

                if ($scope.deployment.majorVersion === null || $scope.deployment.majorVersion === 0) {
                    $scope.boundPrimitives.isDirty = true;  // new deployment
                }

                if (angular.isArray($scope.deployment.windows) && $scope.deployment.windows[0]) {
                    $scope.boundPrimitives.selWindow = $scope.deployment.windows[0];
                }
                else {
                    $scope.boundPrimitives.selWindow = {id: 0, sourceId: 0};
                }
                $scope.boundPrimitives.filterSelWindow = $scope.boundPrimitives.selWindow;

                prepareWindowView();

                $scope.$root.showWidgetMap = {};

                // Initialize expansion, disabled and other view states for the windows
                $scope.viewStates = {};
                angular.forEach($scope.deployment.windows, function (win) {
                    initWindowViewState(win);
                });

                $scope.$watch('deployment.name', function (newValue, oldValue) {
                    if (newValue !== oldValue) {
                        $scope.setDirty(true);
                    }
                });

                $scope.$watch('deployment.desc', function (newValue, oldValue) {
                    if (newValue !== oldValue) {
                        $scope.setDirty(true);
                    }
                });

                $scope.$watch('deployment.libraryName', function (newValue, oldValue) {
                    if (newValue !== oldValue) {
                        $scope.setDirty(true);
                    }
                });

                $scope.$on(vsmCommonConst.CONDITION_BUILDER_CHANGE, function (evt) {
                    evt.preventDefault();
                    $scope.setDirty(true);
                });

                $scope.$on(vsmCommonConst.CONDITION_BUILDER_ERROR, function (evt) {
                    evt.preventDefault();
                    $scope.setDirty(true);
                });


                // Need to validate current window form before allowing user to change window.
                // Because of the way conditions are being rendered and validated, user validation errors would be lost
                // if user switched to another window, so we need to prevent user from switching windows
                // until there are no errors.
                $scope.$watch('boundPrimitives.filterSelWindow', function (newValue, oldValue) {
                    if ($scope.boundPrimitives.filterSelWindow && newValue !== oldValue) {

                        var isValid = validateWindowForm();
                        if (!isValid && newValue !== $scope.boundPrimitives.selWindow && !$scope.boundPrimitives.isReadOnly) {
                            $scope.boundPrimitives.filterSelWindow = $scope.boundPrimitives.selWindow;
                            $scope.showErrorMessage(resources.deployment_invalidInputMsg_txt);
                        }
                        else {
                            $scope.boundPrimitives.selWindow = newValue;
                            prepareWindowView();
                        }
                    }
                });

                // Update FilteringSelect selected item
                $scope.$watch('boundPrimitives.selWindow', function (newValue, oldValue) {
                    if ($scope.boundPrimitives.selWindow && newValue !== $scope.boundPrimitives.filterSelWindow) {
                        $scope.boundPrimitives.filterSelWindow = newValue;
                    }
                });

                $scope.$watch('boundPrimitives.isReadOnly', function (newValue, oldValue) {
                    if (newValue !== oldValue) {
                        prepareWindowView();
                    }
                });

                $scope.initializing = false;
            }


            function prepareWindowView() {
                // Updates window widget states, e.g. disabled, validation, etc.

                updateConditionDisabled();

                if (!$scope.boundPrimitives.isReadOnly) {
                    // Highlights any errors in form
                    validateWindowForm();
                }
            }


            // Save current deployment
            function saveDeployment() {

                $scope.showLoadingMask(resources.deployment_savingDeployment_txt);

                return deploymentSvc.saveDeployment($scope.deployment)
                    .then(function (results) {
                        $scope.deployment = results;
                        updateDeploymentViewStates();
                        getEspCompatibility();
                        loadVersions(false); // update version info without re-initializing deployment
                        updateSelectedWindow();
                        $scope.setDirty(false);
                        $scope.hideLoadingMask();
                        $scope.showStatusMessage(resources.deployment_saveCompleteMsg_txt);
                    });
            }

            // Saves deployment in background without loading newly created object
            function saveInBackground() {
                $scope.showLoadingMask(resources.deployment_savingDeployment_txt);

                return deploymentSvc.saveDeployment($scope.deployment)
                    .then(function (results) {
                        // Update this in case they are navigating to simulation
                        simulationSvc.setParameters({deployment: results});
                        $scope.hideLoadingMask();
                        $scope.showStatusMessage(resources.deployment_saveCompleteMsg_txt);
                    });
            }

            // Removes windows from the deployment based on sourceId property
            function removeWindows(sourceIds) {

                var selSourceId = $scope.boundPrimitives.selWindow.sourceId,
                    resetSelWindow = false;

                var depWindows = $scope.deployment.windows;
                angular.forEach(sourceIds, function (srcId) {
                    for (var i = 0; i < depWindows.length; i++) {
                        var win = depWindows[i];
                        if (win && win.sourceId === srcId) {
                            if (selSourceId === srcId) {
                                if (sourceIds.length === 1) {  // change selWindow to the previous or first window in the deployment
                                    $scope.boundPrimitives.selWindow = (i === 0) ? depWindows[i + 1] : depWindows[i - 1];
                                    selSourceId = $scope.boundPrimitives.selWindow.sourceId;
                                }
                                else {
                                    resetSelWindow = true;
                                }
                            }
                            depWindows.splice(i, 1);  // remove item
                            delete $scope.viewStates[srcId];
                            break;
                        }
                    }
                });
                if (resetSelWindow) { // set selWindow to first window in deployment
                    $scope.boundPrimitives.selWindow = $scope.deployment.windows[0] || {};
                }
                $scope.$apply();
                $scope.setDirty(true);
            }

            // Retrieves a refreshed copy of the specified shadow window objects and prompts user for confirmation
            // Input 'selWins' is an array of shadow window objects
            function getRefreshedWindows(selWins) {

                $scope.showLoadingMask(resources.deployment_checkingForWindowUpdates_txt);

                // Prepare array of windows
                var windows = [];
                if (angular.isArray(selWins)) {
                    windows = selWins;
                }
                else {
                    if (selWins) {
                        windows = [selWins];
                    }
                    else {
                        return [];
                    }
                }

                // Create array of shadow ids
                var shadowIds = [];
                angular.forEach(windows, function (win) {
                    shadowIds.push(win.id);
                });

                deploymentSvc.refreshWindows($scope.deployment.id, shadowIds)
                    .then(function (results) {
                        $scope.refreshedDeployment = results;
                        $scope.hideLoadingMask();
                        $scope.boundPrimitives.showConfirmRefreshDialog = true;
                    });
            }

            // Retrieves generated code for the specified window
            function generateWindowCode(windowId) {

                $scope.showLoadingMask(resources.deployment_CodeViewer_generatingCode_txt);
                $scope.boundPrimitives.generatedCode = "";

                deploymentSvc.getGeneratedCode(windowId)
                    .then(function (results) {
                        $scope.boundPrimitives.generatedCode = results;
                        $scope.hideLoadingMask();
                        $scope.boundPrimitives.showCodeDialog = true;
                    });
            }

            function loadSelectionLists() {

                $q.all([
                    scenarioSvc.getScenarioTypes(),
                    deploymentSvc.getLibraries()
                ]).then(function (results) {

                    // add clear all option to front of list
                    //results.splice(0, 0, {id: 0, name: resources.deployment_WindowDiv_scTypeFilter_clear_txt});
                    var scTypes = results[0];
                    scTypes.splice(0, 0, {id: 0, name: resources.deployment_WindowDiv_scTypeFilter_clear_txt});
                    $scope.selectionData.scenarioTypes = scTypes;

                    $scope.selectionData.libraries = results[1];
                    $scope.boundPrimitives.selLibrary = _.findWhere($scope.selectionData.libraries, {path: $scope.deployment.libraryName});

                    loadVersions(true);
                });
            }


            function loadVersions(doInit) {

                // Remove existing watch on selVersion so it doesn't get triggered when we update the list
                if (selVersionWatch) {
                    selVersionWatch();
                }
                // Populate versions array and sets current and selected version, and isReadOnly flag
                $scope.selectionData.versions = [];
                $scope.boundPrimitives.selVersion = null;
                $scope.currentVersion = null;

                deploymentSvc.getCurrentVersion($scope.deployment.name)
                    .then(function (results) {
                        var value = parseInt(results);
                        $scope.currentVersion = value;
                        var versions = [];
                        if (value > 0) {
                            versions.push({version: $scope.currentVersion,
                                label: $scope.currentVersion + "  (" + resources.deployment_properties_currentLbl_txt + ")"});
                            for (var i = $scope.currentVersion - 1; i > 0; i--) {
                                versions.push({version: i, label: i});
                            }
                        }
                        if ($scope.deployment.majorVersion) {
                            $scope.boundPrimitives.selVersion = _.findWhere(versions, {version: $scope.deployment.majorVersion});
                        }
                        updateReadOnlyFlag();
                        $scope.selectionData.versions = versions;

                        // Can't use the FilteringSelect change handler for this because it causes the deployment to be loaded twice
                        selVersionWatch = $scope.$watch('boundPrimitives.selVersion', function (newValue, oldValue) {
                            if (newValue !== oldValue) {
                                selVersionChange(newValue, oldValue);
                            }
                        });

                        if (doInit === true) {
                            prepareDeployment();
                        }
                    });
            }

            function selVersionChange(newValue, oldValue) {

                if (newValue === oldValue || newValue.version === $scope.deployment.majorVersion) {
                    return;
                }

                depNavWithConfirm(loadSelectedVersion, cancelChange, newValue.version);

                function cancelChange() {
                    if ($scope.deployment.majorVersion && $scope.selectionData.versions) {
                        $scope.boundPrimitives.selVersion = _.findWhere($scope.selectionData.versions, {version: $scope.deployment.majorVersion});
                    }
                }
            }

            function updateReadOnlyFlag() {
                if (!$scope.deployment.majorVersion || $scope.deployment.majorVersion <= 0) {
                    $scope.boundPrimitives.isReadOnly = false;
                }
                else {
                    $scope.boundPrimitives.isReadOnly = $scope.deployment.majorVersion !== $scope.currentVersion;
                }
            }

            function displayUnshownScenariosMsg() {
                var messages = [resources.deployment_unshownScenarioMsg_txt];
                angular.forEach($scope.deployment.windows, function (win) {
                    angular.forEach(win.scenarios, function (scen) {
                        if (!$scope.hasConditions(scen.events) || !(scen.valid && scen.valid === true)) {
                            messages.push(win.name + ": " + scen.name);
                        }
                    });
                });
                $timeout(function () {  // Added timeout to try and avoid jerky screen rendering
                    $scope.hideLoadingMask();
                    if (messages.length > 1) {
                        $scope.showInfoMessage(messages);
                    }
                }, 1000);
            }

            // Updates selected window, used in cases where deployment window object references may have changed
            function updateSelectedWindow() {
                angular.forEach($scope.deployment.windows, function (win) {
                    if (win.sourceId === $scope.boundPrimitives.selWindow.sourceId) {
                        $scope.boundPrimitives.selWindow = win;
                    }
                });
            }

            // The ConditionBuilder doesn't yet handle null childConditions
            // Make sure there aren't any null-value window and scenario childConditions
            function adjustNullConditions(win) {
                adjustChildConditions(win.outputColumns);
                angular.forEach(win.scenarios, function (scen) {
                    adjustChildConditions(scen.events);
                });
            }

            function adjustChildConditions(objArray) {
                angular.forEach(objArray, function (obj) {
                    replaceNull(obj.condition);
                });
                function replaceNull(cond) {
                    if (cond && cond.hasOwnProperty('childConditions')) {
                        if (!cond.childConditions) {
                            cond.childConditions = [];
                        }
                        angular.forEach(cond.childConditions, function (child) {
                            replaceNull(child);
                        });
                    }
                }
            }

            // Create model to hold expansion states of the window and scenario divs, and other view states
            // Use sourceId as key instead of id so that when deployments are saved (and new ids are generated) the expansion states can be remembered.
            // Also, newly added windows have id values of null
            // Initialize all expansion states to false
            function initWindowViewState(window) {
                $scope.viewStates [window.sourceId] = {
                    expanded: false,
                    expandAllScenarios: false,
                    disableAllScenarios: false,
                    scenarios: {},
                    selScType: {},
                    hasWindowConditions: false,
                    hasScenarioConditions: false,
                    lookbackUnitLabel: "",
                    dependentSuccessors: [],
                    successorNames: ""
                };
                var viewState = $scope.viewStates[window.sourceId];
                angular.forEach(window.scenarios, function (sc) {
                    viewState.scenarios[sc.sourceId] = {
                        expanded: false
                    };
                });
                viewState.disableAllScenarios = (!_.findWhere(window.scenarios, {activeInDeployment: true})) ? true : false;
                viewState.hasWindowConditions = $scope.hasWindowConditions(window);
                viewState.hasScenarioConditions = $scope.hasScenarioConditions(window);
                viewState.isValid = true;
                if (!!window.anchorSourceColumn && !!window.entityTimeUnit && window.entityTimeUnit.granularityUnit) {
                    viewState.lookbackUnitLabel = resources['scWindow_AggBuilder_' + window.entityTimeUnit.granularityUnit + 'Label_txt'] || "";
                }
                viewState.dependentSuccessors = getDependentSuccessors(window) || [];
                if (viewState.dependentSuccessors.length > 0) {
                    // create bulleted list of names
                    viewState.successorNames = _.map(viewState.dependentSuccessors, function (succ) {
                        return "\n   " + succ.name;
                    })
                        .join(' ');
                }
            }

            // Returns array of all immediate and indirect successors of given window
            function getDependentSuccessors(win) {
                if (!(win.successorSourceIds && win.successorSourceIds.length > 0)) {
                    return [];
                }
                var successorList = [];
                addSuccessors(win);
                return _.uniq(successorList);

                function addSuccessors(win) {
                    angular.forEach(win.successorSourceIds, function (srcId) {
                        var succ = _.findWhere($scope.deployment.windows, {sourceId: srcId});
                        if (succ) {
                            successorList.push(succ);
                            addSuccessors(succ);
                        }
                    });
                }
            }

            // Call after a deployment has been updated to restore expansion states on changed objects
            function updateDeploymentViewStates() {
                angular.forEach($scope.deployment.windows, function (window) {
                    var prevViewState = $scope.viewStates[window.sourceId];
                    initWindowViewState(window);
                    var newViewState = $scope.viewStates[window.sourceId];
                    if (prevViewState) {
                        newViewState.expanded = prevViewState.expanded;
                        newViewState.expandAllScenarios = prevViewState.expandAllScenarios;
                        var wasExpanded = false;
                        angular.forEach(window.scenarios, function (sc) {
                            wasExpanded = (prevViewState.scenarios && prevViewState.scenarios[sc.sourceId]) ?
                                prevViewState.scenarios[sc.sourceId].expanded : false;
                            newViewState.scenarios[sc.sourceId].expanded = wasExpanded || prevViewState.expandAllScenarios;
                        });

                    }
                });
            }

            // Updates condition TextBox disabled state
            // Use this method until we add a disabled attribute on conditionPackage
            // For some reason if a dojo widget is not visible at the time you set its disabled property,
            // it doesn't get rendered as disabled. Need to call this every time a div gets exposed.
            function updateConditionDisabled() {

                if ($scope.boundPrimitives.isReadOnly) {
                    // Disable items within divs having deploymentDisabled class or if in read-only mode
                    dojoModules.query(".deploymentConditionsDivContent").forEach(function (node) {
                        dojoModules.query(".dijitTextBox", node).forEach(function (tbNode) {
                            var tb = dojoModules.registry.byNode(tbNode);
                            tb.set("readOnly", true);
                        });
                    });
                }
                else {
                    // Clear current settings in window div
                    dojoModules.query(".deploymentWindowDiv").forEach(function (node) {
                        dojoModules.query(".dijitTextBox", node).forEach(function (tbNode) {
                            var tb = dojoModules.registry.byNode(tbNode);
                            tb.set("disabled", false);
                        });
                    });
                    // Disable items within divs having deploymentDisabled class or if in read-only mode
                    dojoModules.query(".deploymentDisabled").forEach(function (node) {
                        dojoModules.query(".dijitTextBox", node).forEach(function (tbNode) {
                            var tb = dojoModules.registry.byNode(tbNode);
                            tb.set("disabled", true);
                        });
                    });
                }
            }

            // Validates window and returns valid status
            function isWindowValid(win) {
                if (hasInvalidConditions(win.outputColumns, !!win.anchorSourceColumn)) {
                    return false;
                }
                var isValid = true;
                angular.forEach(win.scenarios, function (scen) {
                    if (isValid) {  // no need to evaluate further if we already know obj is is invalid
                        isValid = !hasInvalidConditions(scen.events);
                    }
                });
                return isValid;
            }

            function hasInvalidConditions(objArray, validateLookback) {
                var isInvalid = false;
                angular.forEach(objArray, function (obj) {
                    if (!isInvalid) {     // no need to evaluate further if we already know obj is is invalid
                        if (validateLookback && !!obj.aggrFunction) {
                            if (obj.lookbackPeriod < 0) {
                                isInvalid = true;
                            }
                            else {
                                var valResults = vsdUtil.valExprConstByDataType(obj.lookbackPeriod, {dataType: vsmCommonConst.DATA_TYPE_NUMERIC});
                                isInvalid = !valResults.isValid;
                            }
                        }
                        if (!isInvalid && obj.condition) {
                            validateCondition(obj.condition);
                        }
                        if (!isInvalid && obj.actions) {
                            validateActions(obj.actions);
                        }
                    }
                });
                function validateCondition(cond) {
                    if (cond.operator && (cond.operator.name === "IS MISSING" || cond.operator.name === "NOT MISSING")) {
                        return;   // nothing to validate
                    }
                    if (cond.operand1ColumnInfo && _.size(cond.operand1ColumnInfo) > 0 &&
                        cond.hasOwnProperty('operand2Constant') && cond.operand2Constant !== null) {

                        // ConditionBuilder doesn't change operand2Constant when there's a validation error, so
                        // you won't know if the user entered an invalid value unless you check operand2ConstantView
                        var valResults = {isValid: true};
                        valResults = vsdUtil.valExprConstByDataType(cond.operand2Constant, cond.operand1ColumnInfo, cond.operator);
                        if (!valResults.isValid) {
                            isInvalid = true;
                            return;
                        }
                    }
                    if (cond.hasOwnProperty('childConditions') && cond.childConditions !== null) {
                        angular.forEach(cond.childConditions, function (child) {
                            validateCondition(child);
                        });
                    }
                }

                function validateActions(actions) {
                    angular.forEach(actions, function (act) {
                        if (!isInvalid && act.hasOwnProperty('actionText') && act.lhsColumn && _.size(act.lhsColumn.editControlType) > 0) {
                            var dataType = act.lhsColumn.editControlType.name;
                            if (dataType === "BOOLEAN") {  // We aren't surfacing boolean actions in deployment
                                return;
                            }
                            dataType = dataType === "NUMBER" ? vsmCommonConst.DATA_TYPE_NUMERIC : dataType;
                            var valResults = vsdUtil.valExprConstByDataType(act.actionText, {dataType: dataType});
                            if (!valResults.isValid) {
                                isInvalid = true;
                                return;
                            }
                        }
                    });
                }
                return isInvalid;
            }

            function validateProperties() {
                var valid = true;
                dojoModules.query(".deploymentPropertiesForm").forEach(function (node) {
                    var form = dojoModules.registry.byNode(node);
                    valid = form.validate();
                });
                return valid;
            }

            function validateWindowForm() {
                var valid = true;
                dojoModules.query(".deploymentWindowForm").forEach(function (node) {
                    var form = dojoModules.registry.byNode(node);
                    valid = valid && form.validate();
                });
                return valid;
            }

            function validateDeployment() {

                // For read only views validation doesn't apply because user can't change anything.
                if ($scope.boundPrimitives.isReadOnly) {
                    return true;
                }

                // validate properties
                var isValid = validateProperties();

                //validate currently selected window first, which is most likely to have errors
                isValid = isValid && validateWindowForm();

                if (isValid) {   // find first invalid window
                    var firstInvalidWindow = null;
                    angular.forEach($scope.deployment.windows, function (win) {
                        if (!firstInvalidWindow && !isWindowValid(win)) {
                            firstInvalidWindow = win;
                            isValid = false;
                        }
                    });
                    if (firstInvalidWindow) {
                        if (firstInvalidWindow.sourceId !== $scope.boundPrimitives.selWindow.sourceId) {
                            $scope.boundPrimitives.selWindow = firstInvalidWindow;
                        }
                    }
                }
                if (!isValid) {
                    validateWindowForm(); // highlights errors in selected window
                }
                $scope.boundPrimitives.isValid = isValid;
                return isValid;
            }

            function getDepId() {
                return $scope.deployment.id;
            }

            // Use this for external navigating
            function navWithConfirm(navFunc) {
                var isValid = false;
                if ($scope.boundPrimitives.isReadOnly) {
                    isValid = true;
                }
                else {
                    isValid = validateDeployment();
                    if (!isValid) {
                        $scope.boundPrimitives.isDirty = true;
                    }
                }
                navFunc($scope.boundPrimitives.isDirty,
                    isValid,
                    saveDeployment,
                    resources.obj_type_deployment_txt,
                    $scope.deployment.name,
                    $scope.deployment.id);
            }

            // Use this for deployment internal navigating
            function depNavWithConfirm(navigateCallback, closeCallback, params) {

                if (!$scope.boundPrimitives.isDirty) {
                    navigateCallback(params);
                    return;
                }
                var confirmCallback,
                    cancelCallback,
                    depLabel = $scope.deployment.name + " " + resources.deployment_properties_versionLbl_txt + " " + $scope.deployment.majorVersion;

                var isValid = validateDeployment();

                if (isValid) {
                    confirmCallback = onConfirmForValid;
                    cancelCallback = onNavigateCallback;
                    confirmDialogSvc.setConfirmLabel($scope.resources.common_banner_toolbarSave_txt)
                        .setCancelLabel($scope.resources.confirmDlg_unsavedChanges_doNotSaveDialogLbl_txt)
                        .setMsg(dojoModules.string.substitute($scope.resources.confirmDlg_unsavedChanges_msgLbl_txt,
                            [resources.obj_type_deployment_txt, depLabel]));
                }
                else {
                    confirmCallback = onNavigateCallback;
                    cancelCallback = closeCallback;
                    confirmDialogSvc.setConfirmLabel($scope.resources.confirmDlg_unsavedChanges_discardChangesLbl_txt)
                        .setCancelLabel($scope.resources.confirmDlg_unsavedChanges_cancelDialogLbl_txt)
                        .setMsg(dojoModules.string.substitute($scope.resources.confirmDlg_unsavedChangesInvalid_msgLbl_txt,
                            [resources.obj_type_deployment_txt, depLabel]));
                }
                confirmDialogSvc.setConfirmCallback(confirmCallback)
                    .setCancelCallback(cancelCallback).setCloseCallback(closeCallback);

                confirmDialogSvc.showDialog();

                function onConfirmForValid() {
                    saveInBackground()
                        .then(function () {
                            navigateCallback(params);
                        });
                }

                function onNavigateCallback() {
                    navigateCallback(params);
                }
            }

            function setActiveInDeployment(win, active) {
                win.activeInDeployment = active;
                angular.forEach(win.scenarios, function (sc) {
                    sc.activeInDeployment = active;
                });
                $scope.viewStates[win.sourceId].disableAllScenarios = !active;
                if (!active) { // collapse all
                    $scope.viewStates[win.sourceId].expanded = false;
                    $scope.setExpandAll(false);
                }
                $scope.setDirty(true);
            }

            function enableWindow(selWindow) {
                // If you enable a window you have to make sure its predecessor windows are also enabled

                var inactivePredecessors = [];
                addInactivePredecessors(selWindow);
                var uniqueList = _.uniq(inactivePredecessors);
                if (uniqueList.length > 0) {
                    var predNames = _.pluck(uniqueList, 'name').join(', ') || "";
                    confirmDialogSvc.setTitle(resources.deployment_ConfirmEnableWindow_title_txt)
                        .setConfirmCallback(applyEnable)
                        .setCancelCallback(function () {
                        });
                    //Set message and button labels
                    confirmDialogSvc.setConfirmLabel(resources.deployment_ConfirmEnableWindow_enableBtn_txt)
                        .setCancelLabel(resources.deployment_ConfirmEnableWindow_cancelBtn_txt)
                        .setMsg(dojoModules.string.substitute(resources.deployment_ConfirmEnableWindow_msg_txt, [selWindow.name, predNames]));
                    confirmDialogSvc.showDialog();
                }
                else {
                    applyEnable();
                }

                function addInactivePredecessors(win) {
                    var predecessors = _.filter($scope.deployment.windows, function (depWin) {
                        return !depWin.activeInDeployment && _.contains(depWin.successorSourceIds, win.sourceId);
                    });
                    angular.forEach(predecessors, function (pred) {
                        addInactivePredecessors(pred);
                    });
                    inactivePredecessors.push.apply(inactivePredecessors, predecessors);
                }

                function applyEnable() {
                    setActiveInDeployment(selWindow, true);
                    angular.forEach(inactivePredecessors, function (win) {
                            setActiveInDeployment(win, true);
                        }
                    );
                }
            }

            function disableWindow(selWindow) {
                // If you disable a window you have to make sure its successor windows are also disabled

                var activeSuccessors = [];
                if (selWindow.successorSourceIds && selWindow.successorSourceIds.length > 0) {
                    addActiveSuccessors(selWindow);
                    if (activeSuccessors.length > 0) {
                        var succNames = _.pluck(activeSuccessors, 'name').join(', ') || "";
                        confirmDialogSvc.setTitle(resources.deployment_ConfirmDisableWindow_title_txt)
                            .setConfirmCallback(applyDisable)
                            .setCancelCallback(function () {
                            });
                        //Set message and button labels
                        confirmDialogSvc.setConfirmLabel(resources.deployment_ConfirmDisableWindow_disableBtn_txt)
                            .setCancelLabel(resources.deployment_ConfirmDisableWindow_cancelBtn_txt)
                            .setMsg(dojoModules.string.substitute(resources.deployment_ConfirmDisableWindow_msg_txt, [selWindow.name, succNames]));
                        confirmDialogSvc.showDialog();
                    }
                    else {
                        applyDisable();
                    }
                }
                else {
                    applyDisable();
                }

                function addActiveSuccessors(win) {
                    angular.forEach(win.successorSourceIds, function (succSourceId) {
                        var succ = _.findWhere($scope.deployment.windows, {sourceId: succSourceId});
                        if (succ && succ.activeInDeployment) {
                            activeSuccessors.push(succ);
                            addActiveSuccessors(succ);
                        }
                    });
                }

                function applyDisable() {
                    setActiveInDeployment(selWindow, false);
                    angular.forEach(activeSuccessors, function (win) {
                            setActiveInDeployment(win, false);
                        }
                    );
                }
            }

            //endregion


            //region *** ViewModel properties ***

            $scope.resources = dojoModules.resources;
            $scope.initializing = true;
            $scope.deployment = {};
            $scope.currentVersion = null;
            $scope.viewStates = {};
            $scope.selectionData = {
                versions: [],
                libraries: [],
                scenarioTypes: [],
                refreshOptions: [
                    {label: resources.deployment_refreshCurrentWindow_txt, onClick: function () {
                        $scope.refreshWindows(false);
                    }},
                    {label: resources.deployment_refreshAllWindows_txt, onClick: function () {
                        $scope.refreshWindows(true);
                    }}
                ]
            };

            $scope.diagram = {};
            $scope.diagram.size = null;

            $scope.boundPrimitives = {
                isValid: true,
                isReadOnly: true,
                isDirty: false,
                showEditSelectionDialog: false,
                showCodeDialog: false,
                showConfirmRefreshDialog: false,
                showEspLoadDialog: false,
                showEspModelDialog: false,
                espIncompatibleReasons: null,
                selLibrary: null,
                selVersion: null,
                selWindow: null,  // selected window
                generatedCode: ""
            };

            //endregion


            //region *** ViewModel Methods ***

            $scope.isSlidingWindow = function(window) {
                if (window.type === vsmCommonConst.WINDOW_TYPE_SLIDING ||
                    window.type === vsmCommonConst.WINDOW_TYPE_ROLLING) {
                    return true;
                } else {
                    return false;
                }
            };

            $scope.isRollingWindow = function(window) {
                if (window.type === vsmCommonConst.WINDOW_TYPE_ROLLING) {
                    return true;
                } else {
                    return false;
                }
            };

            $scope.setDirty = function (newVal) {
                if (!$scope.initializing && !$scope.boundPrimitives.isReadOnly) {
                    $scope.boundPrimitives.isDirty = newVal;
                }
            };

            $scope.displayDate = function (value) {
                return (value ? dojoModules.locale.format(new Date(value)) : "");
            };

            $scope.actionNumberValidator = function (value) {
                var valResults = vsdUtil.valExprConstByDataType(value, {dataType: vsmCommonConst.DATA_TYPE_NUMERIC});
                var msg;
                switch (valResults.reason) {
                    case  vsmCommonConst.EXPR_VAL_REASON_INVALID_DATE_CONST:
                        msg = resources.scenario_conditionEditor_invalidDateFormatErrMsg_txt;
                        break;
                    case vsmCommonConst.EXPR_VAL_REASON_NON_NUMERIC:
                        msg = resources.scenario_conditionEditor_nonNumericContentErrMsg_txt;
                        break;
                    case vsmCommonConst.EXPR_VAL_REASON_NULL:
                        msg = resources.scenario_conditionEditor_nullErrMsg_text;
                        break;
                }
                return { isValid: valResults.isValid, invalidMessage: msg };
            };

            $scope.save = function () {
                if (!validateDeployment()) {
                    $scope.showErrorMessage(resources.deployment_saveInvalidInputMsg_txt);
                }
                else {
                    if ($scope.deployment.id) {
                        saveDeployment();
                    }
                    else {  // validate name before saving
                        var name = $scope.deployment.name;
                        deploymentSvc.isValidNewDeploymentName(name)
                            .then(function (results) {
                                if (results === 'true') {
                                    saveDeployment();
                                }
                                else {
                                    $scope.showErrorMessage(dojoModules.string.substitute(resources.deployment_invalidNameErrorMsg_txt, [name]));
                                }
                            });

                    }
                }
            };

            $scope.refreshWindows = function (allFlag) {
                $scope.refreshAll = false;
                if (allFlag) {
                    $scope.refreshAll = true;
                    getRefreshedWindows($scope.deployment.windows);
                }
                else {
                    getRefreshedWindows($scope.boundPrimitives.selWindow);
                }
            };

            $scope.viewGeneratedCode = function () {
                generateWindowCode($scope.boundPrimitives.selWindow.id);
            };

            $scope.removeWindow = function (sourceId) {
                removeWindows([sourceId]);
            };

            $scope.removeSelWindow = function () {

                var selWindow = $scope.boundPrimitives.selWindow;
                confirmDialogSvc.setTitle(resources.deployment_RemoveWindowTitle_txt)
                    .setConfirmCallback(function () {
                        removeWindows([selWindow.sourceId]);
                    })
                    .setCancelCallback(function () {
                    });
                //Set message and button labels depending on validity
                confirmDialogSvc.setConfirmLabel(resources.deployment_dialogs_yesBtn_txt)
                    .setCancelLabel(resources.deployment_dialogs_noBtn_txt)
                    .setMsg(dojoModules.string.substitute(resources.deployment_confirmRemoveWindowMsg_txt, [selWindow.name]));
                confirmDialogSvc.showDialog();

            };

            $scope.editWindows = function () {
                $scope.boundPrimitives.showEditSelectionDialog = true;
            };


            $scope.applyEditDialogResults = function (addedItems, deletedItems) {

                if (addedItems.length > 0) {
                    $scope.showLoadingMask(resources.deployment_updatingDeployment_txt);

                    // First make service call to add the newly selected windows
                    deploymentSvc.addWindows($scope.deployment.id, addedItems)
                        .then(function (results) {
                            var updatedDeployment = results;
                            angular.forEach(updatedDeployment.windows, function (win) {
                                adjustNullConditions(win);
                            });
                            $scope.deployment = updatedDeployment;
                            updateDeploymentViewStates();
                            if (deletedItems.length > 0) {
                                removeWindows(deletedItems);
                            }
                            $scope.setDirty(true);
                            displayUnshownScenariosMsg();
                        });
                }
                else if (deletedItems.length > 0) {
                    removeWindows(deletedItems);
                }
            };


            // Replaces old versions of the window objects in the deployment with refreshed versions
            $scope.applyRefreshedWindows = function () {

                if (!$scope.refreshedDeployment) {
                    return;
                } // nothing to do

                // Prepare windows in refreshed deployment
                angular.forEach($scope.refreshedDeployment.windows, function (w) {
                    adjustNullConditions(w);
                });

                $scope.deployment = $scope.refreshedDeployment;
                updateDeploymentViewStates();

                $scope.$apply();
                updateSelectedWindow();
                $scope.setDirty(true);

                $scope.showStatusMessage(resources.deployment_refreshCompleteMsg_txt);
                displayUnshownScenariosMsg();
            };

            // Name widget validation function, dojo validates with every keystroke
            $scope.deploymentNameValidator = function () {
                var valResults = vsdUtil.validateDomainObjName($scope.deployment.name, []);
                var msg = valResults.isValid ? "" : resources.app_object_name_specCharValidationMsg_txt;
                return {isValid: valResults.isValid, invalidMessage: msg };
            };

            $scope.selLibraryChange = function () {
                if ($scope.boundPrimitives.selLibrary) {
                    $scope.deployment.libraryName = $scope.boundPrimitives.selLibrary.path;
                }
            };

            $scope.scTypeChange = function (sourceId) {
                var windowStates = $scope.viewStates[sourceId];
                if (!windowStates.selScType || windowStates.selScType.id === 0) {
                    windowStates.selScType = {};  // reset to nothing selected
                }
            };

            $scope.hasWindowConditions = function (window) {
                var hasLookback = false;
                if (!!window.anchorSourceColumn) {
                    angular.forEach(window.outputColumns, function (col) {
                        hasLookback = hasLookback || !!col.aggrFunction;
                    });
                }
                return window.filterCondition || hasLookback || $scope.hasConditions(window.outputColumns);
            };

            $scope.hasScenarioConditions = function (window) {
                var hasCond = false;
                angular.forEach(window.scenarios, function (scen) {
                    hasCond = hasCond || ( $scope.hasConditions(scen.events) && scen.valid && scen.valid === true );
                });
                return hasCond;
            };

            $scope.scenarioFilter = function () {
                var win = $scope.boundPrimitives.selWindow;
                return function (scen) {
                    var selectedScType = $scope.viewStates[win.sourceId].selScType;
                    return $scope.hasConditions(scen.events) &&
                        (scen.valid && scen.valid === true) &&
                        ( !selectedScType.id || selectedScType.id === 0 || scen.type.id === selectedScType.id);
                };
            };

            $scope.outputColumnFilter = function (col) {
                var win = $scope.boundPrimitives.selWindow;
                return (!!win.anchorSourceColumn && !!col.aggrFunction) || col.condition;
            };

            $scope.showActions = function (event) {
                var retValue = false;
                angular.forEach(event.actions, function (action) {
                    retValue = retValue ||
                    (action.lhsColumn && action.lhsColumn.editControlType && action.lhsColumn.editControlType.name.trim() !== "BOOLEAN");
                });
                return retValue;
            };

            // Check to see if items contains at least one condition
            $scope.hasConditions = function (items) {
                if (!angular.isDefined(items)) {
                    return false;
                }
                var hasCond = false;
                angular.forEach(items, function (itm) {
                    var cond = (itm && itm.hasOwnProperty('condition')) ? itm.condition : itm;
                    if (cond) {
                        hasCond = hasCond ||
                        cond.operand2Constant !== null ||
                        cond.operator && cond.operator.name === "IS MISSING" ||
                        $scope.hasConditions(cond.childConditions);
                    }
                });
                return hasCond;
            };

            $scope.canSimulate = function () {
                if (!$scope.deployment && !$scope.deployment.windows) {
                    return false;
                }
                var canSimulate = false;
                angular.forEach($scope.deployment.windows, function (win) {
                    var srcId = win.sourceId;
                    canSimulate = canSimulate ||
                    (win.activeInDeployment && $scope.viewStates[srcId] && $scope.viewStates[srcId].hasScenarioConditions);
                });
                return canSimulate;
            };

            $scope.setExpandAll = function (expandVal) {
                var win = $scope.boundPrimitives.selWindow;
                var expanded = expandVal === true ? true : false;
                $scope.viewStates[win.sourceId].expandAllScenarios = expanded;

                var scenarios = $scope.boundPrimitives.selWindow.scenarios

                for(var j = 0; j < scenarios.length ; j++){
                    var scen = scenarios[j];
                    var scenario =     $scope.viewStates[win.sourceId].scenarios[scen.sourceId];
                    if(j >= 10){
                        break;
                    }
                    scenario.expanded = expanded;
                    if(expanded){
                        scenario.hasExpandedOnce = true;
                    }
                };

                $scope.$apply();
                updateConditionDisabled();
            };


            $scope.isExpanded = function (scen) {
                var win = $scope.boundPrimitives.selWindow;
                var expandObj = (scen) ? $scope.viewStates[win.sourceId].scenarios[scen.sourceId] :
                    $scope.viewStates[win.sourceId];
                if(expandObj.expanded){
                    expandObj.hasExpandedOnce = true;
                }
                return expandObj.hasExpandedOnce;
            };

            $scope.toggleExpand = function (scen) {
                var win = $scope.boundPrimitives.selWindow;
                var expandObj = (scen) ? $scope.viewStates[win.sourceId].scenarios[scen.sourceId] :
                    $scope.viewStates[win.sourceId];
                expandObj.expanded = !expandObj.expanded;
                var allExpanded = true;

                angular.forEach($scope.viewStates[win.sourceId].scenarios, function (sc) {
                    if (!sc.expanded) {
                        allExpanded = false;
                    }
                });
                $scope.viewStates[win.sourceId].expandAllScenarios = allExpanded;
                //$scope.$apply();
                if(expandObj.expanded){
                    expandObj.hasExpandedOnce = true;
                }
                if (expandObj.expanded) {
                    updateConditionDisabled();
                }
            };

            $scope.toggleExpandAll = function () {
                var win = $scope.boundPrimitives.selWindow;
                var newVal = !$scope.viewStates[win.sourceId].expandAllScenarios;
                $scope.setExpandAll(newVal);
            };

            $scope.toggleDisableWindow = function () {
                var selWindow = $scope.boundPrimitives.selWindow,
                    active = !selWindow.activeInDeployment;
                if (active) { // Enabling window
                    enableWindow(selWindow);
                }
                else {
                    disableWindow(selWindow);
                }
            };

            $scope.toggleDisableAllScenarios = function () {
                var win = $scope.boundPrimitives.selWindow;
                var newValue = !$scope.viewStates[win.sourceId].disableAllScenarios;
                $scope.viewStates[win.sourceId].disableAllScenarios = newValue;
                angular.forEach(win.scenarios, function (sc) {
                    sc.activeInDeployment = !newValue;
                });
                $scope.setExpandAll(false);
                $scope.setDirty(true);
            };

            $scope.toggleDisableScenario = function (scen) {
                var win = $scope.boundPrimitives.selWindow;
                var active = !scen.activeInDeployment;
                scen.activeInDeployment = active;
                var allDisabled = true;
                angular.forEach(win.scenarios, function (sc) {
                    if (sc.activeInDeployment) {
                        allDisabled = false;
                    }
                    else {  // collapse disabled scenario
                        $scope.viewStates[win.sourceId].scenarios[scen.sourceId].expanded = false;
                    }
                    $scope.viewStates[win.sourceId].disableAllScenarios = allDisabled;
                });
                $scope.$apply();
                updateConditionDisabled();
                $scope.setDirty(true);
            };

            $scope.simulate = function () {

                // Set simulation service parameters so that these items don't have to be retrieved again from server
                simulationSvc.setParameters({deployment: $scope.deployment});
                depNavWithConfirm(navToSimulation, function () {
                });

                function navToSimulation() {
                    var params = simulationSvc.getParameters();  // this gets updated during save
                    $location.path("/deployment/" + (params && params.deployment ? params.deployment.id : $scope.deployment.id) + "/simulation");
                }

            };

            $scope.showNewWindowDialog = function () {
                navWithConfirm($scope.newWindow);
            };

            $scope.showNewScenarioDialog = function () {
                navWithConfirm($scope.newScenario);
            };

            $scope.showNewDeploymentDialog = function () {
                navWithConfirm($scope.newDeployment);
            };

            $scope.deployToEsp = function () {
                depNavWithConfirm(loadEsp, function () {
                });
                function loadEsp() {
                    $scope.boundPrimitives.showEspLoadDialog = true;
                }
            };

            $scope.previewXml = function () {
                depNavWithConfirm(preview, function () {
                });
                function preview() {
                    $scope.showLoadingMask($scope.resources.deployment_espDialog_retrievingModelMsg_txt);
                    deploymentSvc.checkEspCompatibility($scope.deployment.id)
                        .then(function (messages) {
                            if (messages.length === 0) {
                                deploymentSvc.getEspModel($scope.deployment.id)
                                    .then(function (results) {
                                        $scope.boundPrimitives.generatedCode = results;
                                        $scope.boundPrimitives.showCodeDialog = true;
                                        $scope.hideLoadingMask();
                                    });
                            }
                            else {
                                $scope.showInfoMessage(messages);
                            }
                        });
                }
            };

            $scope.showHomeScreen = function () {
                navWithConfirm($scope.showHome);
            };

            $scope.goToHubScreen = function () {
                navWithConfirm($scope.goToHub);
            };

            $scope.showOpenScreen = function () {
                navWithConfirm($scope.showOpen);
            };

            $scope.displayHierarchicalView = function () {
                prepareHierarchicalDisplay();
                $scope.boundPrimitives.showHierarchicalView = true;
            };

            $scope.exportDeployment = function () {
                // did not work with .then or return statements
                deploymentSvc.exportDeploymentIframe($scope.deployment.id);
            };

            $scope.$on("$destroy", function() {
                $scope.$root.showWidgetMap = undefined;
            });

            //endregion

            initializeDeployment();

        }]);

});

},
'vsd/deployment/DeploymentSvcs':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('deploymentSvc', [ '$http',
                               'dojoModules',
                               'vsdUtil',
                               '$timeout',
   function (
	   $http,
	   dojoModules,
	   vsdUtil,
	   $timeout) {

	   var urlBase = 'action/',
		   resources = dojoModules.resources,
		   deploymentSvc = {};

		var espDefaults = {
		   pubSubPort: 6000,
		   dateFormat: "%Y-%m-%dT%H:%M:%S",
		   noOfThreads: 1
	   };

	   // Returns an array of localized messages
	   // Input is array of message information arrays in form of [resourceBundleKey, var0, var2, ....]
	   function getLocalizedMessages(msgArray) {
		   if (!msgArray) {
			   return [];
		   }
		   var msgs = [];
		   angular.forEach(msgArray, function(msgInfo) {
			   var resourceKey = msgInfo[0] || "deployment_svc_missingMessageMsg_txt",
				   params = msgInfo.length > 1 ? msgInfo.slice(1) : [];
				if (resources[resourceKey])	{
					msgs.push(dojoModules.string.substitute(resources[resourceKey], params));
				}
			   else {
					msgs.push(resourceKey + params.toString());
				}
		   });
		   return msgs;
	   }

	   deploymentSvc.getDeployment = function (id) {
		   return $http.get(urlBase + 'deployment/' + id).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.getCurrentVersion = function (name) {
		   return $http.get(urlBase + 'deployment/currentVersion/' + name).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.getDeploymentByName = function (name, version) {
		   return $http.get(urlBase + 'deployment/byName/' + name + (version ? "/" + version : "")).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.isValidNewDeploymentName = function (name) {
		   return $http.get(urlBase + 'isValidNewDeploymentName/' + name).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.createDeployment = function (segmentId, windowIds) {
		   var payload = {
			   segmentId: segmentId,
			   windowIds: windowIds
		   };
		   return $http.post(urlBase + 'createDeployment', payload).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.saveDeployment = function (deployment) {
		   return $http.put(urlBase + 'deployment', deployment).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.addWindows = function (deploymentId, windowShadowIds) {
		   return $http.post(urlBase + 'deployment/' + deploymentId + '/addWindows',
				   windowShadowIds).then(function (response) {
				   return response.data;
			   });
	   };

	   deploymentSvc.refreshWindows = function (deploymentId, windowShadowIds) {
		   return $http.post(urlBase + 'deployment/' + deploymentId + '/refreshWindows',
				   windowShadowIds).then(function (response) {
				   return response.data;
			   });
	   };

	   deploymentSvc.getGeneratedCode = function (windowId) {
		   return $http.get(urlBase + 'code/' + windowId).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.getSegments = function () {
		   return $http.get(urlBase + 'segments').then(function (response) {
               return _.reject(response.data, function (segment) {
                   return segment.id.trim() === '-1';
               });
		   });
	   };

	   deploymentSvc.getWindows = function () {
		   return $http.get(urlBase + 'win').then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.getEspDefaults = function() {
		   return angular.copy(espDefaults);
	   }

	   deploymentSvc.checkEspCompatibility = function (deploymentId) {
		   return $http.get(urlBase + 'deployment/' + deploymentId  + '/checkEspCompatibility').then(function (response) {
			   //console.log("checkEspCompatibility response=",response);
			   // Returns empty array if deployment is compatible
			   // If deployment is not compatible, returns array of problems in form of [resourceBundleKey, var0, var2, ....]
			   var messages = getLocalizedMessages(response.data);
			   if (messages.length > 0) {
				   messages.unshift(resources.esp_deploymentNotCompatible_txt);  // add header message to front
			   }
			   return getLocalizedMessages(messages);
		   });
	   };

	   deploymentSvc.loadOnEsp = function (deploymentId, properties) {
		   var payload = {
			   serverHost: properties.serverHost,
			   xmlServerPort: properties.xmlServerPort,
			   noOfThreads: properties.noOfThreads
		   };
		   return $http.post(urlBase + 'deployment/' + deploymentId  + '/loadOnEspServer', payload).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.getEspModel = function (deploymentId, properties) {
		   var payload = {
			   pubSubPort: properties &&  properties.pubSubPort ? properties.pubSubPort : espDefaults.pubSubPort,
			   dateFormat: properties && properties.dateFormat ? properties.dateFormat : espDefaults.dateFormat,
			   noOfThreads: properties && properties.noOfThreads ? properties.noOfThreads : espDefaults.noOfThreads
		   };
		   return $http.post(urlBase + 'deployment/' + deploymentId  + '/espModel', payload).then(function (response) {
			   return response.data;
		   });
	   };

	   deploymentSvc.getLibraries = function () {
		   return $http.get(urlBase + 'deployment/libraries').then(function (response) {
			   return response.data;
		   });
	   };

       deploymentSvc.exportDeploymentIframe = function (objId) {
		   // this is a jQuery call. The native saveas dialog does not work with XHR due to an AngularJS bug.
		   $('#exportDeploymentIframe').attr('src', urlBase + 'export/deployment/' + objId);
       };

	   return deploymentSvc;
}]);

});

},
'vsd/deployment/confirmRefresh/DeploymentRefreshCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('DeploymentRefreshCtrl', ['$scope', 'dojoModules',
	function ($scope, dojoModules) {

   	function resetForm() {
			$scope.refreshDialogProps.gridData=[];
		   $scope.refreshDialogProps.hasDependencyUpdates = false;
		}

		function initialize() {
			$scope.showLoadingMask($scope.resources.deployment_checkingForWindowUpdates_txt);
			resetForm();
			getData();
		}

		// Prepares data for confirmation dialog that will allow user to see which windows
		// have updates available so they can decide to accept or cancel the refresh
		function getData() {

			var windowUpdateArray = [];

			// Create a summary description conveying whether the selected windows have updates available
			var depWindows = $scope.deployment.windows,
				refWindows = $scope.refreshedDeployment.windows,
				hasUpdates = false,
				chosenWindow = $scope.refreshAll == false ? $scope.boundPrimitives.selWindow : null;

			if (angular.isArray(depWindows) && angular.isArray(refWindows)) {

				for (var idx = 0; idx < depWindows.length; idx++) {
					var depWin = depWindows[idx];
					angular.forEach(refWindows, function (refWin) {
						if (refWin.sourceId === depWin.sourceId) {

							// determine if user chose to refresh this window only
							var isChosenWindow = (chosenWindow == null) ? false : (refWin.sourceId === chosenWindow.sourceId),
								winName = depWin.name;

							if (depWin.logicalDelInd ) {
								// Core object was deleted, no updates available. Show info only if user chose to refresh this window.
								if (isChosenWindow || $scope.refreshAll == true) {
									windowUpdateArray.push({
										name: winName,
										winLastUpdated: depWin.sourceLstupdtTimestamp,
										refLastUpdated: $scope.resources.deployment_confirmRefreshNoUpdateMsg_txt
									});
								}
							}
							else {
							   // If no changes to core window.  Show info only if user chose to refresh this window.
								if (depWin.sourceLstupdtTimestamp === refWin.sourceLstupdtTimestamp ) {
									if (isChosenWindow || $scope.refreshAll == true) {
										windowUpdateArray.push({
											name: winName,
											winLastUpdated: depWin.sourceLstupdtTimestamp,
											refLastUpdated: $scope.resources.deployment_confirmRefreshUpToDateMsg_txt
										});
									}
								}
								else {
									// Window was changed.
									windowUpdateArray.push({
										name: winName,
										winLastUpdated: depWin.sourceLstupdtTimestamp,
										refLastUpdated: refWin.sourceLstupdtTimestamp
									});
									if (!isChosenWindow && $scope.refreshAll === false) {  // window was updated due to dependency
										$scope.refreshDialogProps.hasDependencyUpdates = true;
									}
									hasUpdates = true;
								}
							}
						}
					});
				}
			}
			$scope.hideLoadingMask();
			if (hasUpdates) {
				$scope.refreshDialogProps.gridData = windowUpdateArray;
			}
			else {
				$scope.showInfoMessage($scope.resources.deployment_refreshWindowsNoUpdatesMsg_txt);
				$scope.boundPrimitives.showConfirmRefreshDialog = false;
			}
		}

		//endregion

		//region *** Scope variables ***

		$scope.refreshDialogProps = {
			gridData: [],
			dependencyGridData: [],
			hasDependencyUpdates: false,
			columns:  [
				{ field: "name", label: $scope.resources.deployment_ConfirmRefreshDialog_windowHdr_txt, sortable: true },
				{ field:      "winLastUpdated",
					label:     $scope.resources.deployment_ConfirmRefreshDialog_updatedHdr_txt,
					sortable:  true,
					formatter: function (value) {
						return (value ? dojoModules.locale.format(new Date(value)) : "");
					}
				},
				{ field:      "refLastUpdated",
					label:     $scope.resources.deployment_ConfirmRefreshDialog_refreshUpdated_txt,
					sortable:  true,
					formatter: function (value) {
						if (angular.isString(value)) {
							return value;
						}
						return (value ? dojoModules.locale.format(new Date(value)) : "");
					}
				}
			]
		};

		//endregion

		//region *** Scope functions ***

		$scope.acceptRefreshWindows = function () {
			$scope.boundPrimitives.showConfirmRefreshDialog = false;
			$scope.applyRefreshedWindows();
		}

		$scope.cancelDialog = function () {
			resetForm();
			$scope.boundPrimitives.showConfirmRefreshDialog = false;
		};

		$scope.$watch('boundPrimitives.showConfirmRefreshDialog',
			function () {
				if ($scope.boundPrimitives.showConfirmRefreshDialog) {
					initialize();
				}
			});

		//endregion

	}]);
});

},
'vsd/deployment/editSelection/DeploymentEditSelectionCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('DeploymentEditSelectionCtrl', ['$scope',
                                     'deploymentSvc',
                                     'messageSvc',
                                     'dojoModules',
	function ($scope,
		deploymentSvc,
		messageSvc,
		dojoModules)
	{

		function resetForm() {
			$scope.editDialogPropswinSearchText = "";
			$scope.editDialogPropswindows = [];
		}

		function initialize() {
			$scope.showLoadingMask($scope.resources.deployment_EditSelectionDialog_loadingMsg_txt);
			resetForm();
			getData();
		}

		function getData() {
			deploymentSvc.getWindows()
				.then(function (results) {
                    // For imported deployments we should allow user to only remove existing windows,
                    // but not let them add any new windows to the deployment
                    var filteredWindows = [];
                    if ($scope.deployment.imported) {
                        angular.forEach(results, function (win) {
                            if (_.findWhere($scope.deployment.windows, {sourceId: win.id})) {
                                filteredWindows.push(win);
                            }
                        });
                    } else {
                        filteredWindows = results;
                    }

					$scope.editDialogProps.windows = filteredWindows;
					var deletedWindows = [];
					angular.forEach($scope.deployment.windows, function (depWin) {
						var winObj = _.findWhere($scope.editDialogProps.windows, {id: depWin.sourceId});
						if (winObj) {
							winObj.checked = true;
						}
						else {  // window was deleted outside of deployment, create an edit object for the window
							deletedWindows.push({
								id: depWin.sourceId,
								name: depWin.name,
								createUser: depWin.createUser,
								lstupdtTimestamp: depWin.lstupdtTimestamp,
								checked: true
							});
						}
					});
					if (deletedWindows.length > 0) {
						$scope.editDialogProps.windows.push.apply($scope.editDialogProps.windows, deletedWindows);
					}
					$scope.editDialogProps.allowOk = true;  // nothing has been removed yet
					$scope.hideLoadingMask();
				});
		}

		function adjustPredecessorDependencies() {
			var nonRemovedWindows = [];
			angular.forEach($scope.deployment.windows, function(depWin) {
				// get matching window object from list and set checked state to true if it has selected successors
				var winObj =  _.findWhere($scope.editDialogProps.windows, {id: depWin.sourceId});
				if (winObj && !winObj.checked && hasSelectedSuccessor(depWin)){
					winObj.checked = true;
					nonRemovedWindows.push(winObj.name);
				}
			});
			if (nonRemovedWindows.length > 0) {
				$scope.showInfoMessage(dojoModules.string.substitute($scope.resources.deployment_EditSelectionDialog_cannotRemoveWindow_txt,
					[nonRemovedWindows.join(', ')]));
			}
		}

		function hasSelectedSuccessor(w) {
			if (!w || !w.successorSourceIds) {
				return false;
			}
			var successorFound = successorFound || false;
			angular.forEach(w.successorSourceIds, function(srcId) {
				if (!successorFound) { // only need to keep checking if still false

					var depWin =_.findWhere($scope.deployment.windows, {sourceId: srcId}),
						editWin = _.findWhere($scope.editDialogProps.windows, {id: srcId});

					// handle case where successor window was in the edit list
					if (editWin) {
						if (editWin.checked) {
							successorFound = true;
						}
						else {  // check successor's successor windows
							successorFound = hasSelectedSuccessor(depWin);
						}
					}
					// handle case where successor window was NOT in edit list but is in deployment (window no longer exists outside of the deployment)
					else if (depWin) {
						successorFound = true;
					}
				}
			});
			return successorFound;
		}

		//endregion

		//region *** Scope variables ***

		$scope.editDialogProps = {
			winSearchText: "",
			windows:       [],
			allowOk:       false,
			sortOrder: {field: 'lstupdtTimestamp', desc: true},
			winCols:       [
				{field: "name", label: $scope.resources.deployment_SelectionGrid_nameHdr_txt, sortable: true},
				{field: "createUser", label: $scope.resources.deployment_SelectionGrid_createdByHdr_txt, sortable: true},
				{
					field:     "lstupdtTimestamp",
					label:     $scope.resources.deployment_SelectionGrid_lastupdtHdr_txt,
					sortable:  true,
					formatter: function (value) {
						return (value ? dojoModules.locale.format(new Date(value)) : "");
					}
				}
			]
		};

		//endregion

		//region *** Scope functions ***

		$scope.acceptEditSelection = function () {

			var checkedWindows = _.where($scope.editDialogProps.windows, {checked: true});

			if (checkedWindows.length === 0) {
				messageSvc.showError($scope.resources.deployment_EditSelectionDialog_missingItemError_txt);
			}
			else {
				adjustPredecessorDependencies();

				var addedItems = [],   // added core window ids
					deletedItems = [];  // deleted core window ids

				// Populate addedItems and deletedItems based on checked/unchecked items
				// NOTE: windows in the deployment may not exist anymore outside the
				// deployment, so only delete if a match is found that was unchecked

				addedItems =  _.pluck(_.filter(checkedWindows, function(editWin) {
					return (!_.findWhere($scope.deployment.windows, {sourceId: editWin.id}));
				}), 'id');

				deletedItems = _.pluck( _.filter($scope.deployment.windows, function(depWin) {
					return (_.findWhere($scope.editDialogProps.windows, {id: depWin.sourceId, checked: false}));
				}), 'sourceId');

				$scope.applyEditDialogResults(addedItems, deletedItems);
				$scope.boundPrimitives.showEditSelectionDialog = false;
			}
		};

		$scope.updateAllowOk = function() {
			if (_.findWhere($scope.editDialogProps.windows, {checked: true})) {
				$scope.editDialogProps.allowOk = true;
			}
			else {
				$scope.editDialogProps.allowOk = false;
			}
		};

		$scope.cancelDialog = function () {
			resetForm();
			$scope.boundPrimitives.showEditSelectionDialog = false;
			messageSvc.closeMessage();
		};

		$scope.$watch('boundPrimitives.showEditSelectionDialog',
			function () {
				if ($scope.boundPrimitives.showEditSelectionDialog) {
					initialize();
				}
			});

		//endregion

	}]);
});

},
'vsd/deployment/esp/DeploymentESPCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('DeploymentESPCtrl', ['$scope',
                                     'deploymentSvc',
                                     'dojoModules',
	function ($scope,
		deploymentSvc,
		dojoModules) {

		var initialDialogProperties = {
			showDialog: false,
			showLoadDialog: false,
			showModelDialog: false,
			title: $scope.resources.deployment_espModelDialog_title_txt,
			action: "getModel"
			};

		var initialInputValues = deploymentSvc.getEspDefaults();
		initialInputValues.serverHost = "";
		initialInputValues.xmlServerPort = "";

		resetForm();

		function resetForm() {
			$scope.espDialogProperties = angular.copy(initialDialogProperties);
			$scope.espInputValues = angular.copy(initialInputValues);
			$scope.lastInputValues = angular.copy(initialInputValues);
		}

		function saveLastInputValues() {
			$scope.lastInputValues = angular.copy($scope.espInputValues);
		}

		function restoreLastInputValues() {
			$scope.espInputValues = angular.copy($scope.lastInputValues);
		}

		function validate() {
			var isValid = true;
			// This highlights the validation widgets containing errors
			dojoModules.query(".deployESPForm").forEach(function(node){
				var form = dojoModules.registry.byNode(node);
				isValid = form.validate();
			});
			return isValid;
		}

		//endregion

		//region *** Scope variables ***

		//endregion

		//region *** Scope functions ***

		$scope.getModel = function() {
			var isValid = validate();
			if (isValid) {
				saveLastInputValues();
				$scope.cancelDialog();
				$scope.previewXmlCode($scope.espInputValues);
			}
		}

		$scope.previewXmlCode = function(inputs) {
			$scope.showLoadingMask($scope.resources.deployment_espDialog_retrievingModelMsg_txt);
			deploymentSvc.getEspModel($scope.deployment.id, inputs)
				.then(function (results) {

					$scope.boundPrimitives.generatedCode = results;
					$scope.boundPrimitives.showCodeDialog = true;
					$scope.hideLoadingMask();
					/*
					 var xmlDoc;
					 if (window.DOMParser)
					 {
					 var parser=new DOMParser();
					 xmlDoc=parser.parseFromString(results,"text/xml");
					 }
					 else // Internet Explorer
					 {
					 xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
					 xmlDoc.async=false;
					 xmlDoc.loadXML(results);
					 }

					 // this approach doesn't work for IE
					 var uriContent="data:application/xml," + encodeURIComponent(results);
					 var newWin= window.open(uriContent,'_blank', 'menubar=1', true);
					 newWin.focus();
					 */
				});
		}

		$scope.loadOnEsp = function() {
			var isValid = validate();
			if (isValid) {
				saveLastInputValues();
				$scope.cancelDialog();
				$scope.showLoadingMask($scope.resources.deployment_espDialog_loadingOnEspMsg_txt);
				deploymentSvc.loadOnEsp($scope.deployment.id, $scope.espInputValues)
					.then(function (results) {
						// results contains a status message
						$scope.showInfoMessage(dojoModules.string.substitute($scope.resources.deployment_espDialog_loadingCompleteMsg_txt, [results]));
						console.log("loadOnEsp status:",results);
					});
			}
		}

		$scope.cancelDialog = function () {
			$scope.espDialogProperties.showDialog = false;
			$scope.boundPrimitives.showEspModelDialog = false;
			$scope.boundPrimitives.showEspLoadDialog = false;
		};

		$scope.$watch('boundPrimitives.showEspLoadDialog',
			function () {
				if ($scope.boundPrimitives.showEspLoadDialog === true) {
					$scope.showLoadingMask($scope.resources.esp_loadingMsg_txt);
					deploymentSvc.checkEspCompatibility($scope.deployment.id)
						.then(function(messages) {
							$scope.hideLoadingMask();
							if (messages.length == 0) {
								restoreLastInputValues();
								$scope.espDialogProperties.action = "loadOnEsp";
								$scope.espDialogProperties.title = $scope.resources.deployment_espDialog_load_title_txt;
								$scope.espDialogProperties.showDialog = true;
							}
							else {
								$scope.showInfoMessage(messages);
								$scope.boundPrimitives.showEspLoadDialog = false;
							}
						});
				}
			});

		$scope.$watch('boundPrimitives.showEspModelDialog',
			function () {
				if ($scope.boundPrimitives.showEspModelDialog === true) {
					$scope.showLoadingMask($scope.resources.esp_loadingMsg_txt);
					deploymentSvc.checkEspCompatibility($scope.deployment.id)
						.then(function(messages) {
							$scope.hideLoadingMask();
							if (messages.length == 0) {
								restoreLastInputValues();
								$scope.espDialogProperties.action = "getModel";
								$scope.espDialogProperties.title = $scope.resources.deployment_espDialog_model_title_txt;
								$scope.espDialogProperties.showDialog = true;
							}
							else {
								$scope.showInfoMessage(messages);
								$scope.boundPrimitives.showEspModelDialog = false;
							}
						});
				}
			});

		//endregion

	}]);
});

},
'vsd/deployment/new/NewDeploymentCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('NewDeploymentCtrl', ['$scope',
                                     'deploymentSvc',
                                     'dojoModules',
                                     '$location',
                                     'newDeploymentSvc',
                                     'messageSvc',
                                     '$q',
                                     'vsdUtil',
                                     '$route',
	function ($scope,
		deploymentSvc,
		dojoModules,
		$location,
		newDeploymentSvc,
		messageSvc,
		$q,
		vsdUtil,
		$route) {

		var initialDialogProperties = {
			name: "",
			desc: "",
			library: null,
			libraries: [],
			segment: null,
			orgs: [],
			windows: [],
			selectedWindowIds: [],
			winSearchText: "",
			allowCreate: false
		};

		resetForm();

		function resetForm() {
			$scope.dialogProperties = angular.copy(initialDialogProperties);
		}

		function initialize() {
			$scope.showLoadingMask($scope.resources.deployment_NewDeploymentDlg_loadingMsg_txt);
			resetForm();
			getData();
		}

		function getData() {
			$q.all([
					deploymentSvc.getLibraries(),
					deploymentSvc.getSegments(),
					deploymentSvc.getWindows()
				]).then(function(results) {

					$scope.dialogProperties.libraries = results[0];
					$scope.dialogProperties.orgs = results[1];
					$scope.dialogProperties.windows = results[2];

					if (!$scope.dialogProperties.orgs) {
						$scope.dialogProperties.orgs=[];
						messageSvc.showError($scope.resources.deployment_getAvailOrgsError_txt);
					}

					if (!$scope.dialogProperties.libraries) {
						$scope.dialogProperties.libraries=[];
						messageSvc.showError($scope.resources.deployment_getAvailLibrariesError_txt);
					}

					if (!$scope.dialogProperties.windows)  {
						$scope.dialogProperties.windows = [];
						messageSvc.showError($scope.resources.deployment_getAvailWindowsError_txt);
					}
					else {
						var i = 0;
						angular.forEach($scope.dialogProperties.windows, function(win) {
							win.checked = false;
							var initWindowIds = newDeploymentSvc.getInitWindowIds() || [];
							angular.forEach(initWindowIds, function(id)  {
								if (win.id === id) {
									win.checked = true;
								}
							});
						});
					}
					$scope.hideLoadingMask();
				});
		}

		// Create a new deployment from New Deployment dialog selections
		function createNewDeployment() {

			$scope.showLoadingMask($scope.resources.main_loadingDeployment_msg_txt);

			deploymentSvc.createDeployment($scope.dialogProperties.segment.id, $scope.dialogProperties.selectedWindowIds)
				.then(function (results) {
					var deployment = results;
					deployment.name = $scope.dialogProperties.name;
					deployment.desc = "";
					deployment.libraryName = $scope.dialogProperties.library.path;
					newDeploymentSvc.setDeployment(deployment);
					$scope.hideLoadingMask();
					$scope.dialogFlags.showNewDeploymentDialog = false;
					if ($location.path() === '/deployment/') {
						$route.reload();
					}
					else {
						$location.path('/deployment/');
					}
				});
		}

		function validate() {

			var props=$scope.dialogProperties;
			var isValid = true;

			// This highlights the validation widgets containing errors
			dojoModules.query(".newDeploymentForm").forEach(function(node){
				var widget = dojoModules.registry.byNode(node);
				isValid = isValid && widget.validate();
			});

			if (isValid) {
				updateSelectedWindowIds();
				if (props.selectedWindowIds.length <= 0) {
					isValid = false;
					props.allowCreate = false;
					messageSvc.showError($scope.resources.deployment_NewDeploymentDlg_missingWindowError_txt);
				}
			}
			return isValid;
		}

		function updateSelectedWindowIds() {
			$scope.dialogProperties.selectedWindowIds = _.pluck(_.where($scope.dialogProperties.windows, {checked: true}), 'id');
		}

		//endregion


		//region *** Scope variables ***

		$scope.selGridProps = {
			sortOrder: {field: 'lstupdtTimestamp', desc: true},
			winCols: [
				{field: "name", label: $scope.resources.deployment_SelectionGrid_nameHdr_txt, sortable: true},
				{field: "createUser", label: $scope.resources.deployment_SelectionGrid_createdByHdr_txt ,  sortable: true},
				{
					field: "lstupdtTimestamp",
					label: $scope.resources.deployment_SelectionGrid_lastupdtHdr_txt,
					sortable: true,
					formatter: function (value) {
						return (value ? dojoModules.locale.format(new Date(value)) : "");
					}
				}
			]
		};

		//endregion

		//region *** Scope functions ***

		// Name widget validation function, dojo validates with every keystroke
		$scope.deploymentNameValidator = function(value) {
		//	var valResults = vsdUtil.validateDomainObjName($scope.dialogProperties.name,[]);
			var valResults = vsdUtil.validateDomainObjName(value,[]);
			var msg = valResults.isValid ? "" : $scope.resources.app_object_name_specCharValidationMsg_txt;
			if (!valResults.isValid) {
				$scope.dialogProperties.allowCreate = false;
			}
			return {isValid: valResults.isValid, invalidMessage: msg };
		};

		$scope.createDeployment = function() {
			var isValid = validate();
			if (isValid) {
				var name = $scope.dialogProperties.name;
				deploymentSvc.isValidNewDeploymentName(name)
					.then(function (results) {
						if (results === 'true') {
							createNewDeployment();
							$scope.dialogFlags.showNewDeploymentDialog = false;
							newDeploymentSvc.setInitWindowIds([]);
						}
						else {
							messageSvc.showError(dojoModules.string.substitute($scope.resources.deployment_invalidNameErrorMsg_txt, [name]));
						}
					});
			}
		};

		$scope.updateAllowCreate = function() {
			updateSelectedWindowIds();
			if ($scope.dialogProperties.selectedWindowIds && $scope.dialogProperties.selectedWindowIds.length > 0 &&
				$scope.dialogProperties.name &&
				$scope.dialogProperties.library &&
				$scope.dialogProperties.segment) {

				$scope.dialogProperties.allowCreate = true;
			}
			else {
				$scope.dialogProperties.allowCreate = false;
			}
		};

		$scope.cancelDialog = function () {
			$scope.dialogFlags.showNewDeploymentDialog = false;
			resetForm();
			newDeploymentSvc.setInitWindowIds([]);
			messageSvc.closeMessage();
		};

		$scope.$watch('dialogFlags.showNewDeploymentDialog',
			function () {
				if ($scope.dialogFlags.showNewDeploymentDialog) {
					initialize();
				}
			});

		//endregion

	}]);
});

},
'vsd/deployment/new/newDeploymentSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('newDeploymentSvc', function () {
		var deployment = null,
			initWindowIds = [];

		return {
			setDeployment:    function (d) {
				deployment = d;
			},
			getDeployment:    function (d) {
				return deployment;
			},
			setInitWindowIds: function (wIds) {
				initWindowIds = wIds;
			},
			getInitWindowIds: function () {
				return initWindowIds;
			}

		};
	});

});

},
'vsd/deployment/simulation/RunSimulationDialogCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('RunSimulationDialogCtrl', ['$scope',
                                     'dojoModules',
                                     '$cookies',
                                     '$http',
	function ($scope,
		dojoModules,
		$cookies,
		$http) {

			var resources = $scope.resources,
				showAllCookieName = 'vsdShowAllDeployments';

			$scope.dialogProperties = {
				depSearchText: ""
			};

			//region *** Scope variables ***

			$scope.selGridProps = {
				sortOrder: {field: 'lstupdtTimestamp', desc: true},
				depCols: [
					{field: "name", label: resources.deployment_SelectionGrid_nameHdr_txt, sortable: true},
					{field: "createUser", label: resources.deployment_SelectionGrid_createdByHdr_txt ,  sortable: true},
					{
						field: "lstupdtTimestamp",
						label: resources.deployment_SelectionGrid_lastupdtHdr_txt,
						sortable: true,
						formatter: function (value) {
							return (value ? dojoModules.locale.format(new Date(value)) : "");
						}
					}
				]
			};

			//endregion

			//region *** Scope functions ***
			$scope.listFilterQuery = function (item, index, items) {
				var filterString = $scope.dialogProperties.depSearchText.toLowerCase();
				if (filterString.length < 1) {
					return item;
				}
				else {
					var name = item.name.toLowerCase();
					var matched = name.indexOf(filterString.toLowerCase()) > -1;
					if (matched) {
						return true;
					}
				}
				return false;
			};

			$scope.depItems = {
				refresh:    function () {
					$scope.showLoadingMask(resources.runSimulation_loadingDeployments_msg_txt);
					var url = "action/browse/user";
					// using jQuery here is not ideal but there is a timing issue using ngModel
					$cookies[showAllCookieName] = "" + $('#showAllDeployments').is(':checked');
					if ($('#showAllDeployments').is(':checked')) {
						url = "action/browse/all";
					}
					$http.get(url)
						.success(function (data) {
							$scope.deploymentData = _.where(data, {objType: 'DEPLOYMENT'});
							$scope.deploymentData.forEach(function(dep) {
								dep.id = 'd' + dep.id.toString();  // Main Controller needs deployment id to be prefixed by 'd'
							});
							$scope.hideLoadingMask();
						});
				},
				showAll:    ($cookies[showAllCookieName] === 'true'),
				onShow:     function () {
					this.showAll = ($cookies[showAllCookieName] === 'true');
					this.refresh();
				}
			};

			$scope.onRowSelection = function (selObject) {
				$scope.boundPrimitives.selObject = selObject;
			};

			$scope.runSimulation = function() {
				$scope.boundPrimitives.runSimulation = true;
				$scope.boundPrimitives.showRunSimulationDialog = false;
			};

			$scope.cancelDialog = function () {
				$scope.boundPrimitives.selObject = null;
				$scope.boundPrimitives.runSimulation = false;
				$scope.boundPrimitives.showRunSimulationDialog = false;
			};

			//endregion

	}]);
});

},
'vsd/deployment/simulation/simParameterTree':function(){
'use strict';

angular.module('vsd').
    directive('simParameterTree', ['$rootScope',
        '$timeout',
        '$compile',
        'Util',
        'dojoModules',
        function ($rootScope, $timeout, $compile, Util, dojoModules) {

            return {
                restrict: "E",
                replace: true,
                templateUrl: 'vsd/deployment/simulation/simParameterTree.html',
                scope: false,

                link: function (scope, element, attrs, ctrl) {
                    require(['dojo/_base/declare',
                        'dojo/aspect',
                        'dgrid/tree',
                        "dgrid/OnDemandGrid",
                        "dgrid/Selection",
                        "dgrid/Keyboard",
                        'dgrid/extensions/ColumnResizer',
                        "dgrid/CellSelection",
                        "dgrid/extensions/DijitRegistry",
                        'dojo/store/Memory',
                        "dojo/on",
                        "dojo/dom-class",
                        "dijit/form/CheckBox"],

                    function (declare, aspect, tree, OnDemandGrid, Selection, KeyBoard, ColumnResizer, CellSelection, DijitRegistry, Memory, on, domClass, cb) {

                        if (!scope.data.boundThresholds) {
                            scope.data.boundThresholds = {};
                        }

                        var gridCols = [
                            /*
                             {
                             label : " ",
                             field : 'include',
                             sortable: false,
                             renderCell: renderIncludeColumn
                             },
                             */
                            tree({
                                label: scope.resources.simulation_parameterTree_nameColumnLbl_txt,
                                field: "name",
                                sortable: false,
                                indentWidth: 20,
                                collapseOnRefresh: false,
                                shouldExpand: function (obj) {
                                    try {
                                        return obj.data.treeLevel <= 3;
                                    }catch(err){
                                            return false;
                                    }
                                },
                                renderCell: renderName
                            }),
                            {
                                label: scope.resources.simulation_parameterTree_conditionColumnLbl_txt,
                                field: 'condition',
                                sortable: false,
                                renderCell: renderCondition
                            }
                        ];

                        var expandableNodes = [],
                            codeOperators = {};

                        codeOperators['AND'] = scope.resources.common_operators_and_txt;
                        codeOperators['OR'] = scope.resources.common_operators_or_txt;
                        codeOperators['NOT'] = scope.resources.common_operators_not_txt;

                        //Create dgrid mixin
                        var MixInGrid = declare([OnDemandGrid, KeyBoard, DijitRegistry, ColumnResizer]);
                        var containerDiv = $(".simParameterTree");
                        var grid = new MixInGrid({
                            columns: gridCols,
                            className: "simParameterTree",
                            showHeader: true, // needed for column resizer to work
                            store: createStore(scope.data.parameterTree),
                            collapseOnRefresh: false
                        }, containerDiv[0]);

                        //Apply styles to widget
                        Util.applyStyles(grid, attrs.style);

                        //Watch the grid data to see when the data property changes
                        scope.$watch('data.parameterTree', function (newTree, oldTree) {
                            var rootLevel = newTree && newTree.windows ? newTree.windows : [],
                                leafWindows = _.where(rootLevel, {hasSuccessorInTree: false});
                            if (newTree !== oldTree || !grid.store) {
                                grid.set('store', createStore(leafWindows));
                                scope.data.baseThresholds = {};  // reset base conditions for new tree objects
                                scope.data.boundThresholds = {}; // reset bound conditions for new tree objects
                                expandableNodes = []; // reset expandable nodes
                            }
                            else {
                                grid.store.setData([leafWindows]);
                            }
                            grid.refresh();
                            grid.resize();
                        });

                        on(grid, 'dgrid-refresh-complete', function (evt) {
                            setReadOnlyProperties();
                        });

                        scope.$on("SIMULATION_THRESHOLD_ERROR", function (evt) {
                            evt.preventDefault();
                            if (scope.boundPrimitives.initialized) {
                                scope.boundPrimitives.isDirty = true;
                                scope.boundPrimitives.isValid = false;
                            }
                        });

                        scope.$watch('data.scrollToThreshold', function (newValue, oldValue) {
                            if (newValue && newValue !== oldValue) {
                                var rowId = null,
                                    threshold = newValue.threshold;
                                _.each(grid._rowIdToObject, function (v, k) {
                                    if (v === threshold) {
                                        rowId = k;
                                    }
                                });
                                if (rowId) {
                                    var row = document.getElementById(rowId);
                                    grid.bodyNode.scrollTop = row.offsetTop;
                                }
                            }
                        });

                        scope.expandAllTreeNodes = function (expanded) {
                            dojoModules.query(".simParameterTree").forEach(function (node) {
                                var grid = dojoModules.registry.byNode(node);
                                if (grid) {
                                    angular.forEach(expandableNodes, function (nodeId) {
                                        grid.expand(nodeId, expanded);
                                    });
                                }
                            });
                        };

                        function comparatorByExecutionOrder(a,b) {
                            if (a == null && b == null) {
                                return 0;
                            }
                            if (a == null) {
                                return -1;
                            }
                            if (b == null) {
                                return 1;
                            }
                            var execOrderA = (a.executionOrder == null) ? 0 : a.executionOrder;
                            var execOrderB = (b.executionOrder == null) ? 0 : b.executionOrder;
                            return (execOrderA > execOrderB) ? 1 : ((execOrderB > execOrderA) ? -1 : 0);
                        }

                        function createStore(data) {
                            var storeData = data ? data : [];
                            var store = new Memory({
                                data: storeData,
                                idProperty: "nodeId",
                                // dgrid tree handlers
                                getChildren: getNodeChildren,
                                mayHaveChildren: function (object) {
                                    return store.getChildren(object).length > 0;
                                }
                            });
                            return store;
                        }

                        function setReadOnlyProperties() {
                            dojoModules.query(".dijitTextBox", element[0]).forEach(function (child) {
                                var widget = dojoModules.registry.byNode(child);
                                if (widget) {
                                    widget.set("readOnly", scope.boundPrimitives.isReadOnly);
                                }
                            });
                            dojoModules.query(".dijitCheckBox", element[0]).forEach(function (child) {
                                var widget = dojoModules.registry.byNode(child);
                                if (widget) {
                                    widget.set("readOnly", scope.boundPrimitives.isReadOnly);
                                }
                            });
                        }

                        // This is the getChildren function used by the memory store
                        // It retrieves the child nodes when a node in the tree is expanded
                        function getNodeChildren(object, options) {
                            var children = [],
                                windowId = object.objectType === "Window" ? object.id : object.windowId;  // root node is always a window

                            // Note: each tree nodes should contain a unique nodeId for the dgrid tree to work right

                            if (object.condition) {   // Create Condition root node
                                object.condition.windowId = windowId;
                                var nodeName = object.objectType === "Window" ? scope.resources.simulation_parameterTree_windowFilterLbl_txt :
                                    scope.resources.simulation_parameterTree_conditionLbl_txt;
                                if (object.condition.childConditions && object.condition.childConditions.length == 1) {
                                    // Don't need logical operator node if there is only one child condition
                                    populateWindowId(object.condition.childConditions, windowId);
                                    children.push({
                                        name: nodeName,
                                        childConditions: object.condition.childConditions,
                                        windowId: windowId,
                                        nodeId: "C" + object.nodeId});
                                }
                                else {  // Create node for CONJUNCTION  (AND, OR, NOT)
                                    children.push({
                                        name: nodeName,
                                        childConditions: [object.condition],
                                        windowId: windowId,
                                        nodeId: "C" + object.nodeId});
                                }
                            }
                            if (object.childConditions && object.childConditions.length > 0) {
                                populateWindowId(object.childConditions, windowId);
                                if (object.childConditions.length == 1) {
                                    children.push(object.childConditions[0]); // don't need extra logical operator nesting for just one condition
                                }
                                else {
                                    children.push.apply(children, object.childConditions);
                                }
                            }
                            if (object.actions) {
                                var actionNodes = [];
                                angular.forEach(object.actions, function (act) {
                                    if (act.editControlTypeName != "BOOLEAN") {
                                        actionNodes.push(act);
                                    }
                                });
                                if (actionNodes.length > 0) {
                                    populateWindowId(actionNodes, windowId);
                                    children.push({
                                        name: scope.resources.simulation_parameterTree_actionsLbl_txt,
                                        actionNodes: actionNodes,
                                        windowId: windowId,
                                        nodeId: "A" + object.nodeId});
                                }
                            }
                            if (object.actionNodes) {
                                children.push.apply(children, object.actionNodes);
                            }

                            if (object.thresholds && object.thresholds.length > 0) {
                                populateWindowId(object.thresholds, windowId);
                                children.push({
                                    name: scope.resources.simulation_parameterTree_thresholdsLbl_txt,
                                    thresholdNodes: object.thresholds,
                                    windowId: windowId,
                                    nodeId: "T_" + object.nodeId});
                            }
                            if (object.thresholdNodes) {
                                var showNodes = [];
                                angular.forEach(object.thresholdNodes, function (node) {
                                    if ((node.hasOwnProperty('lookbackPeriod') && scope.data.lookbackUnitLabels[windowId]) || node.condition || node.actions) {
                                        showNodes.push(node);
                                    }
                                });

				                showNodes.sort(comparatorByExecutionOrder);
                                children.push.apply(children, showNodes);
                            }
                            if (object.scenarios && object.scenarios.length > 0) {
                                populateWindowId(object.scenarios, windowId);
                                children.push({
                                    name: scope.resources.simulation_parameterTree_scenariosLbl_txt,
                                    windowId: windowId,
                                    nodeId: "S_" + windowId,
                                    scenarioNodes: object.scenarios});
                            }
                            if (object.scenarioNodes) {
                                children.push.apply(children, object.scenarioNodes);
                            }
                            // In this release we will only show source windows, not source tables
                            if (object.sources && _.where(object.sources, {nodeType: "WINDOW"}).length > 0) {
                                populateWindowId(object.sources, windowId);
                                children.push({
                                    name: scope.resources.simulation_parameterTree_sourcesLbl_txt,
                                    sourceNodes: object.sources,
                                    windowId: windowId,
                                    nodeId: "P" + object.nodeId});
                            }
                            if (object.sourceNodes) {
                                children.push.apply(children, object.sourceNodes);
                            }
                            if (object.nodeType == "OUTPUT_COLUMN" && object.hasOwnProperty('lookbackPeriod') && scope.data.lookbackUnitLabels[windowId]) {
                                // Create a new node for lookback period, with pseudo nodetype to distinquish it from the node that
                                // contains the output column label
                                children.push({
                                        name: scope.resources.scWindow_AggBuilder_lookBackLabel_txt + " (" + scope.data.lookbackUnitLabels[windowId] + ")",
                                        lookbackPeriod: object.lookbackPeriod,
                                        objectType: object.objectType,
                                        id: object.id,
                                        windowId: windowId,
                                        nodeType: 'LOOKBACK',
                                        windowType: scope.data.windowTypes[windowId],
                                        nodeId: 'LB' + object.nodeId}
                                );
                            }

                            // Keep track of tree position, will be used for styling the grid row
                            // Add treeLevel property
                            var position = 0,
                                objectPosition = !angular.isDefined(object.treePosition) ? null : object.treePosition;

                            if (object.treeLevel == null) {
                                object.treeLevel = 1;
                            }

                            angular.forEach(children, function (child) {
                                if (!angular.isDefined(child.treePosition) || child.treePosition == null) {
                                    if (objectPosition != null) {
                                        child.treePosition = object.treePosition;
                                    }
                                    else {
                                        child.treePosition = position;
                                    }
                                    position++;
                                }
                                child.treeLevel = object.treeLevel + 1;
                            });

                            if (children.length > 0) {
                                var nodeType = object.nodeType;
                                var level = object.treeLevel;
                                if(level <= 3){
                                    expandableNodes.push(object.nodeId);
                                }
                            }
                            return children;
                        }

                        // Renderer for name column
                        function renderName(object, value, node, options) {

                            // style row for odd or even based on tree position
                            styleNode(node, object.treePosition);

                            var div = document.createElement('div');
                            var name = ' ';

                            if (object.nodeType == "WINDOW") {
                                name = dojoModules.string.substitute(scope.resources.simulation_parameterTree_windowNameLbl_txt, [object.name]);
                            }
                            else if (object.nodeType == "SCENARIO") {
                                name = dojoModules.string.substitute(scope.resources.simulation_parameterTree_scenarioNameLbl_txt, [object.name]);
                            }
                            else if (angular.isDefined(object.label)) {
                                name = object.label;
                            }
                            else if (angular.isDefined(object.name)) {
                                name = object.name;
                            }
                            else if (angular.isDefined(object.columnLabel)) {
                                name = object.columnLabel;
                            }
                            else if (angular.isDefined(object.executionOrder)) {      // numeric
                                name = dojoModules.string.substitute(scope.resources.simulation_parameterTree_execOrderLbl_txt,
                                    [ eval(parseInt(object.executionOrder) + 1) ]);
                            }
                            else if (object.operator && object.nodeType == "CONJUNCTION") {
                                name = codeOperators[object.operator];
                                //node.className = " "; // remove expand icon within conditions
                            }
                            if (object.nodeType == "ACTION" || object.nodeType == "CONDITION" || object.nodeType == "LOOKBACK") {
                                div.style.fontWeight = 'bold';
                            }

                            div.innerHTML = name;
                            div.title = name;
                            div.style.whiteSpace = 'nowrap';

                            // Render include checkbox for non-leaf windows (predecessor windows)
                            if (object.nodeType == "WINDOW" && object.treeLevel > 1) {
                                renderIncludeCheckbox(object, value, node, options);
                            }

                            else if (!(object.nodeType == "ACTION" && object.editControlTypeName == "BOOLEAN")) {
                                node.appendChild(div);
                            }

                        }

                        // Renderer for condition column
                        function renderCondition(object, value, node, options) {

                            // style row for odd or even based on tree position assignment
                            styleNode(node, object.treePosition);

                            // Only render conditions and actions
                            if (!object) {
                                return;
                            }
                            if (!( object.nodeType == 'LOOKBACK' || object.nodeType == "CONDITION" || object.nodeType == "ACTION")
                                || ( object.nodeType == "ACTION" && object.editControlTypeName == "BOOLEAN")) {
                                return;
                            }

                            var valueProp = null;

                            switch (object.nodeType) {
                                case "LOOKBACK":
                                    object.valueProperty = "lookbackPeriod";
                                    break;
                                case "ACTION":
                                    object.valueProperty = "actionText";
                                    break;
                                default:  // CONDITION
                                    object.valueProperty = "operand2Constant";
                            }


                            // Save original value in baseThresholds hash map.
                            scope.data.baseThresholds[object.nodeId] = {
                                id: object.id,
                                windowId: object.windowId,
                                nodeId: object.nodeId,
                                objectType: object.objectType,
                                value: object[object.valueProperty],
                                valueProperty: object.valueProperty
                            };

                            // Place object in boundThresholds hash map, these objects are bound to the UI
                            scope.data.boundThresholds[object.nodeId] = object;

                            // The sng-dijit onChange handler doesn't get triggered for some dijits,
                            // e.g. NumberSpinner, when value is set to an invalid value, so use watches instead.
                            scope.$watch("data.boundThresholds." + object.nodeId, function (newValue, oldValue) {
                                if (newValue !== oldValue) {
                                    onConditionChange(object.nodeId);
                                }
                            }, true);    // use deep watch to detect object property changes

                            var template = '<span style="display:inline-block;">'
                                + '<simulation-condition condition="data.boundThresholds.' + object.nodeId + '"'
                                + ' is-read-only="boundPrimitives.isReadOnly"'
                                + '></simulation-condition></span>';

                            var cond = $compile(template)(scope);
                            node.appendChild(cond[0]);

                            safeApply();
                        }

                        // Renderer for inline includeInAlertCount checkboxes on window nodes
                        function renderIncludeCheckbox(object, value, node, options) {

                            var name = dojoModules.string.substitute(scope.resources.simulation_parameterTree_windowNameLbl_txt, [object.name]);

                            // Todo: Decide if we want to disable checkboxes under certain circumstances
                            var disabled = "false";

                            // Place object in boundThresholds hash map, these objects are bound to the UI
                            object.valueProperty = "includeInAlertCount";
                            scope.data.boundThresholds[object.nodeId] = object;

                            // Save original value in baseThresholds hash map.
                            scope.data.baseThresholds[object.nodeId] = {
                                id: object.id,
                                nodeId: object.nodeId,
                                value: object.includeInAlertCount,
                                valueProperty: object.valueProperty
                            };

                            var template = '<span style="display:inline-block;">' +
                                '<span style="padding-left:3px;padding-right:2px;" sng-dijit="dijit/form/CheckBox" ' +
                                ' data-dojo-props="' +
                                " disabled: " + disabled +
                                ", title: '" + scope.resources.simulation_parameterTree_includeInAlertsLbl_txt + "'" +
                                '"' +
                                ' sng-model="data.boundThresholds.' + object.nodeId + '.includeInAlertCount"' +
                                '></span>' + name + '</span>';

                            var elements = $compile(template)(scope);
                            node.appendChild(elements[0]);
                            safeApply();


                            scope.$watch('data.boundThresholds.' + object.nodeId + '.includeInAlertCount', function (newValue, oldValue) {
                                if (newValue !== oldValue) {
                                    updateScenarios(object);
                                    // Need to keep all nodes for the same predecessor window in sync
                                    angular.forEach(scope.data.boundThresholds, function (boundObj) {
                                        if (boundObj.nodeId !== object.nodeId
                                            && boundObj.id === object.id
                                            && boundObj.nodeType === object.nodeType
                                            && boundObj.includeInAlertCount !== newValue) {
                                            boundObj.includeInAlertCount = newValue;
                                            updateScenarios(boundObj);
                                        }
                                    });
                                }
                                function updateScenarios(win) {
                                    angular.forEach(win.scenarios, function (scen) {
                                        scen.includeInAlertCount = newValue;
                                    });
                                }
                            });
                        }

                        function populateWindowId(objArray, windowId) {
                            angular.forEach(objArray, function (obj) {
                                obj.windowId = windowId;
                            });
                        }

                        function styleNode(node, position) {
                            // style row for odd or even based on tree position
                            var pos = !angular.isDefined(position) ? 1 : position;  // assign odd value if undefined
                            var addClass = !(pos % 2) ? 'dgrid-row-odd' : 'dgrid-row-even';
                            var removeClass = (addClass === 'dgrid-row-odd') ? 'dgrid-row-even' : 'dgrid-row-odd';
                            domClass.add(node, addClass);
                            domClass.remove(node, removeClass);
                        }

                        function onConditionChange(nodeId) {
                            if (scope.boundPrimitives.initialized) {
                                scope.boundPrimitives.isDirty = true;
                                if (!scope.boundPrimitives.isReadOnly) {
                                    // Need to keep thresholds belonging to same predecessor windows in sync
                                    var changedObj = scope.data.boundThresholds[nodeId];
                                    angular.forEach(scope.data.boundThresholds, function (boundObj) {
                                        var valProp = boundObj.valueProperty;
                                        if (boundObj.nodeId !== changedObj.nodeId
                                            && boundObj.windowId === changedObj.windowId
                                            && boundObj.id === changedObj.id
                                            && boundObj.nodeType === changedObj.nodeType
                                            && boundObj[valProp] !== changedObj[valProp]) {

                                            boundObj[valProp] = changedObj[valProp];
                                        }
                                    });
                                }
                            }
                        }

                        function safeApply() {
                            if (!$rootScope.$$phase) {
                                scope.$apply();
                            }
                        }

                    });
                }
            };
        }]);

},
'vsd/deployment/simulation/simulationCondition':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .directive('simulationCondition', [ '$compile',
                                         'dojoModules',
                                         'vsdUtil',
                                         '$rootScope',
                                         'operatorSvc',
    function ($compile,
      dojoModules,
      vsdUtil,
      $rootScope,
      operatorSvc) {

        return {
            templateUrl: 'vsd/deployment/simulation/simulationCondition.html',
            replace: true,
            restrict: 'E',
            scope: {
                'condition': '=',
                'isReadOnly': '='
            },
            controller: ['$scope', '$element', '$attrs', function ($scope, $element, $attrs) {

                var valueProp = null,
                    viewProp = null,
                    thresholdType = null,
                    dataType = null,
                    isDate = false,
                    isDateTime = false,
                    operatorObj = null;

                $scope.resources = dojoModules.resources;

                $scope.boundPrimitives = {
                    isReadOnly: $scope.isReadOnly === true ? true : false,
                    operatorDisplayValue: $scope.condition.operator,
                    isMissingOperator: $scope.condition.operator && $scope.condition.operator.indexOf("MISSING") >= 0
                };

                $scope.isSlidingWindow = false;
                if ($scope.condition.hasOwnProperty("windowType")) {
                    $scope.isSlidingWindow = ($scope.condition.windowType === vsmCommonConst.WINDOW_TYPE_SLIDING || $scope.condition.windowType ===  vsmCommonConst.WINDOW_TYPE_ROLLING);
                }

                $scope.isRollingWindow = false;
                if ($scope.condition.hasOwnProperty("windowType")) {
                    $scope.isRollingWindow = ($scope.condition.windowType === vsmCommonConst.WINDOW_TYPE_ROLLING);
                }

                switch ($scope.condition.nodeType) {
                    case "LOOKBACK":
                        // Localization formatting/parsing is built into NumberSpinner widget, so
                        // we don't need a "View" version for lookbackPeriod.
                        dataType = 'NUMERIC';
                        valueProp = "lookbackPeriod";
                        break;
                    case "ACTION":
                        dataType = $scope.condition.editControlTypeName;
                        valueProp = "actionText";
                        viewProp = "actionTextView";
                        break;
                    default:  // CONDITION
                        dataType = $scope.condition.columnDataType;
                        valueProp = "operand2Constant";
                        viewProp = "operand2ConstantView";
                }

                if (dataType === "NUMBER") {
                    dataType = vsmCommonConst.DATA_TYPE_NUMERIC;
                }
                isDate = ($scope.condition.columnFormat === vsdUtil.defaultFormats.date);
                isDateTime = ($scope.condition.columnFormat === vsdUtil.defaultFormats.datetime);

                if ($scope.condition.nodeType === "CONDITION") {
                    var columnType = (dataType === vsmCommonConst.DATA_TYPE_CHAR) ? "CHAR" : "NUM";
                    operatorSvc.getLogicalOperators(columnType).then(function (data) {
                        if (data.length > 0) {
                            var operatorList = data,
                                displayOperator = _.findWhere(operatorList, {name: $scope.condition.operator});
                            if (displayOperator) {
                                operatorObj = displayOperator;
                                $scope.boundPrimitives.operatorDisplayValue = displayOperator.label;
                            }
                        }
                    });
                }

                $scope.$watch("condition." + valueProp, function (newVal, oldVal) {
                    if ($scope.condition) {
                        setViewValue(newVal);
                    }
                });

                function getValidationErrMsg(reason) {
                    var msg;
                    switch (reason) {
                        case  vsmCommonConst.EXPR_VAL_REASON_INVALID_DATE_CONST:
                            msg = $scope.resources.scenario_conditionEditor_invalidDateFormatErrMsg_txt;
                            break;
                        case  vsmCommonConst.EXPR_VAL_REASON_INVALID_DATETIME_CONST:
                            msg = $scope.resources.scenario_conditionEditor_invalidDateTimeFormatErrMsg_txt;
                            break;
                        case vsmCommonConst.EXPR_VAL_REASON_NON_NUMERIC:
                            msg = $scope.resources.scenario_conditionEditor_nonNumericContentErrMsg_txt;
                            break;
                        case vsmCommonConst.EXPR_VAL_REASON_NULL:
                            msg = $scope.resources.scenario_conditionEditor_nullErrMsg_text;
                            break;
                        case vsmCommonConst.EXPR_VAL_REASON_DBL_QUOTES:
                            msg = $scope.resources.scenario_conditionEditor_dblQuotes_text;
                            break;
                        case vsmCommonConst.EXPR_VAL_REASON_LITERAL_MAXLEN:
                            msg = $scope.resources.scenario_conditionEditor_exceedsLiteralMaxLen_text;
                            break;
                        case vsmCommonConst.EXPR_VAL_REASON_TOTAL_MAXLEN:
                            msg = $scope.resources.scenario_conditionEditor_exceedsOperand2MaxLen_text;
                            break;
                    }
                    return msg;
                }

                function setViewValue(newVal) {
                    if (!viewProp || $scope.boundPrimitives.isMissingOperator) {
                        return;
                    }
                    if (dataType === 'NUMERIC') {
                        //Get localized formatted number string and display to user
                        if (isDate) {
                            $scope.condition[viewProp] = vsdUtil.getFormattedDateString(newVal);
                        }
                        else if (isDateTime) {
                            $scope.condition[viewProp] = vsdUtil.getFormattedDateTimeString(newVal);
                        }
                        else {
                            $scope.condition[viewProp] = vsdUtil.getFormattedNumberString(newVal);
                        }
                    }
                    else {
                        //Just display the text string
                        $scope.condition[viewProp] = newVal || '';
                    }
                    setHoverTitle();
                }

                // Sets hover title on condition node
                function setHoverTitle() {
                    var title = $scope.condition.operator ? $scope.boundPrimitives.operatorDisplayValue : "";

                    if (!$scope.boundPrimitives.isMissingOperator) {
                        if ($scope.condition[viewProp]) {
                            title += " " + $scope.condition[viewProp];
                        }
                        else if ($scope.condition[valueProp]) {
                            title += " " + $scope.condition[valueProp];
                        }
                    }
                    $element[0].title = title;
                }

                $scope.validator = function (value) {

                    if ($scope.boundPrimitives.isMissingOperator) {
                        return {isValid: true};
                    }
                    // Validate what the user entered using the user's locale
                    var valRet = vsdUtil.valExprConstByDataType(value,
                        {dataType: dataType.trim(), sasDate: isDate, sasDateTime: isDateTime}, operatorObj, false, true);
                    var invalidMessage = "";
                    if (valRet.isValid === true) {
                        // Extract the numeric value from the formatted string and update the condition's value property
                        $scope.condition[valueProp] = '' + valRet.parsedValue;
                    }
                    else {
                        invalidMessage = getValidationErrMsg(valRet.reason);
                        $scope.$emit("SIMULATION_THRESHOLD_ERROR");
                    }
                    return { isValid: valRet.isValid, invalidMessage: invalidMessage };
                };
            }],


            link: function (scope, element, attrs) {
                //console.log("condition=",scope.condition);
            }
        };
    }]);

});

},
'vsd/deployment/simulation/SimulationCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('SimulationCtrl',['$scope',
                                '$routeParams',
                                'simulationSvc',
                                'confirmDialogSvc',
                                'deploymentSvc',
                                'windowSvc',
                                'dojoModules',
                                '$location',
                                '$q',
                                'vsdUtil',
	function ($scope,
		$routeParams,
		simulationSvc,
		confirmDialogSvc,
		deploymentSvc,
        windowSvc,
		dojoModules,
		$location,
		$q,
		vsdUtil
		) {

		var resources = dojoModules.resources,
			selVersionWatch = null;

		var initData = {
			minAnchorDate: null,
			maxAnchorDate: null,
			anchorEndDate: null,
			anchorStartDate: null,
			parameterTree: null,            // parameter tree data for the simulation
			baseThresholds:  {},            // has map containing base CONDITION and ACTION object threshold values
			boundThresholds: {},            // hash map containing bound CONDITION and ACTION objects, where key = nodeId
													  // used by the Parameter Tree UI component
			lookbackUnitLabels: {},         // labels for lookback period time unit, key = window id
            windowTypes: {},                // window types, key = window id
			scrollToThreshold: null,          // threshold id of first item changed, used by loadValuesIntoTree
			flatParams:  [],                // flattened parameters
			predecessorWindows: [],          // array of the predecessor windows in the simulation
			runObjects: {},                 // maps trial id to its simulation run object
			currTrialNbr: 0,                // current trial number
			baseTrialNbr: 0,                // baseline trial number
			trialHistory: [],               // trial history data for grid
			filteredHistory: [],            // filtered trial history data
			trialGridColumns: [],           // trial grid columns
			trialGridScroll: false,         // flag - scroll to bottom of trial grid after row inserts
			selectedTrial: null,            // selected trial in trialHistory
			hoveredTrial: null,             // hovered trial in trialHistory
			selectedVars: [],               // of the modified thresholds variables shown in grid and charts
			pcAxisLabels: {},               // parallel coord chart axis labels
			pcAxisFormats: {},              // parallel coord chart axis formats
			metricVars: [],                 //  metric variable property keys
			chartType: null,                 // 'STACKEDBAR' or 'PARALLEL', set to null initially so chart doesn't render yet
			trialGridSort: {field: "trialNumber", descending: false}   // trial grid sortOrder object
		};

		var metrics = {
			totalAlertCount: resources.simulation_grid_totalAlertsLbl_txt
		};

  	   //region *** Private Methods

		function initializeSimulation() {

			$scope.showLoadingMask(resources.simulation_loadingSimulation_msg_txt);

			var params = simulationSvc.getParameters();
			if ($routeParams.id) {
				if (params && params.deployment && $routeParams.id === params.deployment.id) {  // use the existing deployment object, user most likely did a reload
					$scope.deployment = params.deployment;
					initSimulation();
				}
				else {  // retrieve the deployment object
					loadDeployment($routeParams);
				}
			}
			else if (params && params.deployment) {  // use the existing deployment object
				$scope.deployment = params.deployment;
				initSimulation();
			}
			else {
				$scope.hideLoadingMask();
				$scope.showErrorMessage(resources.simulation_loadingError_msg_txt);
				$scope.showHome();
			}
		}

		function loadDeployment(params) {
			deploymentSvc.getDeployment(params.id)
				.then(function (results) {
					$scope.deployment = results;
					initSimulation();
				});
		}

		function initSimulation() {
			$scope.boundPrimitives.useAnchorStartDate = false;
			$scope.boundPrimitives.useAnchorEndDate = false;
			$scope.boundPrimitives.anchorDateIsDateTime = false;
			$scope.boundPrimitives.showAnchorDateColumns = false;
			$scope.boundPrimitives.initialized = false;
			$scope.boundPrimitives.isReadOnly = true;
			$scope.boundPrimitives.canSave = false;
			$scope.boundPrimitives.isValid = true;
			$scope.data = angular.copy(initData);
			$scope.alertDetails.data = [];
			$scope.alertDetails.detailData = [];
			getLookbackUnitLabels();
			loadVersions();
			initializeTrialData();
			loadParameterTreeData();
		}

		function loadVersions() {

			deploymentSvc.getCurrentVersion($scope.deployment.name)
				.then(function (results) {
					var value = parseInt(results);
					$scope.currentVersion = value;
					populateVersions();
				});

			function populateVersions() {
				// Remove existing watch on selVersion so it doesn't get triggered when we update the list
				if (selVersionWatch) {
					selVersionWatch();
				}
				// Create selection list of available versions, currentVersion being the latest
				if (!$scope.versions || $scope.versions.length <=0) {
					var versions = [];
					if ($scope.currentVersion > 0) {
						versions.push({version: $scope.currentVersion,
							label: $scope.currentVersion + "  (" + resources.deployment_properties_currentLbl_txt + ")"});
						for (var i = $scope.currentVersion-1; i > 0; i--) {
							versions.push({version: i, label: i});
						}
					}
					updateReadOnlyFlag();
					$scope.selectionData.versions = versions;
				}

				// Set current and selected version
				$scope.boundPrimitives.selVersion = null;
				if ($scope.deployment.majorVersion) {
					$scope.boundPrimitives.selVersion = _.findWhere(versions, {version: $scope.deployment.majorVersion});
				}

				// Can't use the FilteringSelect change handler for this because it causes the deployment to be loaded twice
				selVersionWatch = $scope.$watch('boundPrimitives.selVersion', function (newValue, oldValue) {
					if (newValue !== oldValue) {
						selVersionChange(newValue, oldValue);
					}
				});
			}
		}

		function updateReadOnlyFlag() {
			// No version exists yet, this is a new deployment
			if (!$scope.deployment.majorVersion || $scope.deployment.majorVersion <= 0) {
				$scope.boundPrimitives.isReadOnly = false;
			}
			else {  // Only current version can be modified
				$scope.boundPrimitives.isReadOnly = $scope.deployment.majorVersion != $scope.currentVersion;
			}
		}

		function selVersionChange(newValue, oldValue) {
			// Don't proceed if value hasn't changed or version matches current deployment
			// This covers case where user cancelled out of a prior change navigation and selVersion was reverted back
			if (newValue == oldValue || newValue.version == $scope.deployment.majorVersion) {
				return;
			}
			$scope.showLoadingMask(resources.simulation_loadingSimulation_msg_txt);
			deploymentSvc.getDeploymentByName($scope.deployment.name, $scope.boundPrimitives.selVersion.version)
				.then(function (results) {
					$scope.deployment = results;
					initSimulation();
				});
		}

		function getLookbackUnitLabels() {
			angular.forEach($scope.deployment.windows, function(win) {
				if (!!win.anchorSourceColumn && !!win.entityTimeUnit && win.entityTimeUnit.granularityUnit) {
					$scope.data.lookbackUnitLabels[win.id] = resources['scWindow_AggBuilder_' + win.entityTimeUnit.granularityUnit + 'Label_txt'] || "";
					$scope.data.windowTypes[win.id] =  win.type;
				}
			});
		}

		function initializeTrialData() {
			// Update the scope variables used by trial grid and charts
			// Initially we will start out with no threshold variables in grid.
			$scope.data.metricVars = _.keys(metrics);
			$scope.data.selectedVars = _.keys(metrics);
			$scope.data.trialGridColumns = initTrialGridColumns();
			for (var k in metrics) {
				if (metrics.hasOwnProperty(k)) {
					$scope.data.pcAxisLabels[k] = metrics[k];
					$scope.data.pcAxisFormats[k] = vsdUtil.defaultFormats.integer;
		}
			}
		}

		// Calls service to retrieve the simulation parameter tree and builds structures needed by the UI
		function loadParameterTreeData() {

			$scope.data.parameterTree = {};
			$scope.data.baseThresholds = {};
			$scope.data.boundThresholds = {};
			$scope.data.predecessorWindows = [];

			simulationSvc.getParameterTree($scope.deployment.id)
				.then(function (results) {
					var tree = results;
					tree.windows = _.compact(tree.windows);  // remove falsy values from windows array, e.g. null
					addSuccessorInfo(tree);
					$scope.data.parameterTree = tree;
					//console.log("parameterTree=",results);
					$scope.data.flatParams = flattenParameterTree($scope.data.parameterTree,"",[]);
					//console.log("flatParams=",$scope.data.flatParams);
					loadRuns();
				});

			// Determines if a window has a successor in a parameter tree
			function addSuccessorInfo(tree) {
				angular.forEach(tree.windows, function(window) {
					window.hasSuccessorInTree = false;
					findSuccessor(_.filter(tree.windows, function(win) {return win.id !== window.id;} ), window);
				});
				function findSuccessor(windowList, windowToFind) {
					if (windowToFind.hasSuccessorInTree) {
						return; // no need ot keep looking
					}
					if (_.findWhere(windowList, {id: windowToFind.id})){
						windowToFind.hasSuccessorInTree = true;
					}
					else {
						angular.forEach(windowList, function(win) {
							findSuccessor(_.where(win.sources, {objectType: "Window"}), windowToFind);
						});
					}
				}
			}
		}

		//adds fact table of win to inputTables
        function addInputTable(win, inputTables) {
            if (win.windowInputs[0].leftHandTable !== null) {
                inputTables.push(win.windowInputs[0].leftHandTable);
                return true;
            }
            else if (win.windowInputs[0].leftHandWindow !== null) {
                //addInputTable(win.windowInputs[0].leftHandWindow, inputTables);
                return false;
            }
        }

        function loadRuns() {
			$scope.showLoadingMask(resources.simulation_runningSimulation_msg_txt);

			// Populate inputTables and anchor date flags
			var inputTables = [],
				anchorColumns = [],
				useStartDate = false,
				useEndDate = false,
				usesDateTime = false;

			angular.forEach($scope.deployment.windows, function (win) {
											
				if (win.anchorSourceColumn) {				
					//S1123239
					//Populate the input tables for simulation
					//<WINDOW>_IN may not exist in the following use cases
					//	a) core windows deleted after creating deployments
					//	b) imported deployments which do not have core windows and the associated _IN table for
					//     windows using table or window joins
					//Hence, we have to make use fact tables.
                    var windowAdded  = addInputTable(win, inputTables);
                    if (windowAdded) {
                        anchorColumns.push(win.anchorSourceColumn);
                    }

                    anchorColumns.push(win.anchorSourceColumn);
					if (win.type === vsmCommonConst.WINDOW_TYPE_SLIDING || win.type === vsmCommonConst.WINDOW_TYPE_ROLLING) {
						useStartDate = true;
						useEndDate = true;
					}
					else if (win.type === vsmCommonConst.WINDOW_TYPE_AGGREGATION) {
						useEndDate = true;
					}
					if (win.anchorSourceColumn.sasDateTime) {
						usesDateTime = true;
					}
				}			
			});
						
			$scope.boundPrimitives.useAnchorStartDate = useStartDate;
			$scope.boundPrimitives.useAnchorEndDate = useEndDate;
			$scope.boundPrimitives.anchorDateIsDateTime = usesDateTime;

			$q.all([
				windowSvc.getDateRangeForTables(inputTables, anchorColumns),
				simulationSvc.getAllRuns($scope.deployment.id)

			]).then(function (results) {

            // Initialize anchor date ranges
            var dateRanges = results[0];
            var minDate = new Date(dateRanges.startDate),
               maxDate = new Date(dateRanges.endDate);
				// Need to clear time portion of dates to avoid problems with sngDateTextBox
				// Todo: remove this if we ever switch to a Date time picker
				clearTime(minDate);
				clearTime(maxDate);
				$scope.data.minAnchorDate = minDate;
				$scope.data.maxAnchorDate = maxDate;

            if ($scope.boundPrimitives.useAnchorStartDate) {
               $scope.data.anchorStartDate = angular.copy($scope.data.minAnchorDate);
            }
            if ($scope.boundPrimitives.useAnchorEndDate) {
               $scope.data.anchorEndDate = angular.copy($scope.data.maxAnchorDate);
            }

            var allRuns = results[1];
            if (allRuns && allRuns.length > 0) {  // There are existing trial runs
               // Initialize Anchor dates to match baseline run
               var baseline = _.findWhere(allRuns, {baseline: true});
               if (baseline) {
                  if (baseline.anchorStartDate) {
                     var startDate = new Date(baseline.anchorStartDate);
                     clearTime(startDate);  // clear date portion to avoid problems with sngDateTextBox
                     $scope.data.anchorStartDate = startDate;
                  }
                  if (baseline.anchorEndDate) {
                     var endDate = new Date(baseline.anchorEndDate);
                     clearTime(endDate);   // clear date portion to avoid problems with sngDateTextBox
                     $scope.data.anchorEndDate = endDate;
                  }
               }
               loadTrialHistory(allRuns);
               finishInitializing();
            }
            else if ($scope.deployment.majorVersion === $scope.currentVersion) { // There is no trial history, run baseline if current version
               var endDate = getAdjustedAnchorEndDatetime($scope.data.anchorEndDate);
               simulationSvc.runSimulation($scope.deployment.id, [], [], $scope.data.anchorStartDate, endDate)

               .then(function (simResults) {
                  results.modifiedParams = [];
                  results.excludedWindows = [];
                  loadNewTrial(simResults);
                  finishInitializing();
               });
            }
            else {
               finishInitializing();
            }
			});
			
			function finishInitializing() {
				$scope.data.filteredHistory = $scope.data.trialHistory;
				if ($scope.data.trialHistory.length > 0) {
					$scope.data.chartType = "STACKEDBAR";
					$scope.data.trialGridScroll = true;
				}

				// For stack bar tooltip, if user mouses out of chart too quickly, tooltip mouseout event doesn't get captured
				// and tooltip doesn't close. Add mouseover events to the adjacent content panes so tooltip gets closed.
				$(".simContentPane").mouseenter(function(){
					$scope.alertDetails.closeDetails();
					$scope.$apply();
				});


				$scope.boundPrimitives.initialized = true;
				$scope.hideLoadingMask();
				if ($scope.data.trialHistory.length == 0) {
					$scope.showInfoMessage(resources.simulation_noTrialHistoryMsg_txt);
				}
				//console.log("baseThresholds=",$scope.data.baseThresholds);
				//console.log("boundThresholds=",$scope.data.boundThresholds);
			}
		}

		function clearTime(date) {
			if (date && date instanceof Date) {
				date.setHours(0);
				date.setMinutes(0);
				date.setSeconds(0);
				date.setMilliseconds(0);
			}
		}

		function getAdjustedAnchorEndDatetime(date) {
			// Adjust End date if it is a datetime value, so that we include the full day's worth of time.
			// The sngDateTextBox directive strips off time portion of bound value, so we have to do this
			// on a copy of the object, otherwise it will end up getting stripped off again.
			// Todo: If we get a DateTime picker built, the adjustement won't bee needed anymore
			var newDate = angular.copy(date);
			if (date && date instanceof Date && $scope.boundPrimitives.anchorDateIsDateTime) {
				newDate.setHours(23);
				newDate.setMinutes(59);
				newDate.setSeconds(59);
				newDate.setMilliseconds(999);
				return newDate;
			}
			return newDate;
		}

		function loadTrialHistory(allRuns) {

			angular.forEach(allRuns, function(run) {

				// Create new trial object using flatParams as a template
				// Todo: make sure flatParams always matches baseline, because values that aren't in modifiedParams will default to the flatParams value.
                // Todo: Do we need to add window includeInAlertCount values to flatParmas/trialHistory?

				var newParams = angular.copy($scope.data.flatParams);
				angular.forEach(newParams, function(param) {
					angular.forEach(run.modifiedParams, function(threshold) {
						if (param && param.id == threshold.id && threshold.hasOwnProperty(param.valueProperty)) {
							param.value = threshold[param.valueProperty];
							addToSelectedVars(param);
						}
					});
				});
				addToTrialHistory(run, newParams, false);
			});
			//console.log("trialHistory=",$scope.data.trialHistory);
			updateTrialGridColumns();
		}

		function loadNewTrial(run) {

			if (!run.trialNumber) {
				run.trialNumber =  $scope.data.currTrialNbr++;
			}
			$scope.data.currTrialNbr = run.trialNumber;

			// Create new trial object using FlatParams as a template
			// FlatParams contains all the threshold variables that exist in the deployment.
			var newParams = angular.copy($scope.data.flatParams);

			if (!run.baseline) {
				angular.forEach(newParams, function(param) {
					// loop over bound parameter tree thresholds and update values that are different than base
					angular.forEach($scope.data.baseThresholds, function(obj) {
						var threshold = $scope.data.boundThresholds[obj.nodeId];
						if (threshold) {
						   // update new trial object and selectedVars array
							if (param && param.id === obj.id && param.objectType === obj.objectType && param.windowId === obj.windowId) {
								param.value = threshold[obj.valueProperty];
								if (param.value !== obj.value) {
									addToSelectedVars(param);
								}
							}
						}
					});
				});
			}

			// Create trial object for the current trial run
			addToTrialHistory(run, newParams, true);
			updateTrialGridColumns();
		}


		// Add the parameter label to selected variable array if it's not already there, maintaining correct order
		function addToSelectedVars(param) {
			var srcParam = _.findWhere($scope.data.flatParams, {label: param.label});
			if (srcParam && $scope.data.selectedVars.indexOf(param.label) == -1) {
				var newPosition = $scope.data.selectedVars.length == 0 ? 0 : findPosition(srcParam);
				$scope.data.selectedVars.splice(newPosition, 0, param.label);
				$scope.data.pcAxisLabels[param.label] = param.label;
				$scope.data.pcAxisFormats[param.label] = param.dataType === vsmCommonConst.DATA_TYPE_NUMERIC ? param.dataFormat : null;
			}
            function findPosition(item) {
                var position = $scope.data.flatParams.indexOf(item),
                    neighbor = null,
                    neighborPos = -1;
                position++;    // find position of nearest neighbor to the right
                while (position < $scope.data.flatParams.length && neighborPos == -1) {
                    neighbor = $scope.data.flatParams[position++];
                    neighborPos = $scope.data.selectedVars.indexOf(neighbor.label);
                }
                if (neighborPos == 0) {  // needs to go in front of neighbor
                    return 0;
                }
                else if (neighborPos > 0) {
                    return neighborPos;     // take neighbor's slot, moving neighbor to the right
                }
                return $scope.data.selectedVars.length;
            }
    }

		function addToTrialHistory(run, newParameters, setSelected) {

			// Save run details in hash map for easy access later
			$scope.data.runObjects[run.trialNumber]=run;
			// The first baseline run should be used for change comparisons
			if (run.baseline && $scope.data.baseTrialNbr === 0) {
				$scope.data.baseTrialNbr = run.trialNumber;
			}
			else { // Set flag to show Anchor dates in trial grid if any runs differ from baseline
				var baseTrial = $scope.data.runObjects[$scope.data.baseTrialNbr];
				if (baseTrial && run.anchorStartDate !== baseTrial.anchorStartDate || run.anchorEndDate !== baseTrial.anchorEndDate ) {
					$scope.boundPrimitives.showAnchorDateColumns = true;
				}
			}

            // Todo: Do we need to add window includeInAlertCount values to flatParams/trialHistory?

            var trial = {};
			trial.trialNumber = run.trialNumber;
			trial.baseline = run.baseline;
			trial.createUser = run.createUser.trim();
			trial.createTimestamp = run.createTimestamp;
			trial.anchorStartDate = run.anchorStartDate;
			trial.anchorEndDate = run.anchorEndDate;
			trial.trialDescription = trial.trialNumber + " | " + dojoModules.locale.format(new Date(run.createTimestamp)) + " | " + trial.createUser;
			trial.barLabel = dojoModules.locale.format(new Date(run.createTimestamp), {selector: 'date'}) + ";(" + trial.trialNumber + ")";

			angular.forEach(newParameters, function(x) {
				if (x) {
					// Convert SAS date and datetime value to JavaScript date, because it makes formatting easier in the dgrid and charts,
					// and we can use date axis scales for parallel coord chart.
					if (x.dataFormat === vsdUtil.defaultFormats.date && x.value != null) {
                        if (vsdUtil.isSasDateConstant(x.value)) {
                    trial[x.label] = x.value;
				}
                        else {
                            trial[x.label] = vsdUtil.convertSASToJavaScriptDate(x.value);
                            trial[x.label+":sasDateValue"] = x.value;
                        }
					}
					else if (x.dataFormat === vsdUtil.defaultFormats.datetime && x.value != null) {
                        if (vsdUtil.isSasDateTimeConstant(x.value)) {
                            trial[x.label] = x.value;
                        }
                        else {
                            trial[x.label] = vsdUtil.convertSASToJavaScriptDateTime(x.value);
                            trial[x.label + ":sasDateTimeValue"] = x.value;
                        }
					}
					else {
						trial[x.label] = x.value;
					}
				}
			});
			angular.forEach($scope.data.metricVars, function(metric) {
				if (metric) {
					trial[metric] = run[metric];
				}
			});

			var winDetails = null;

			// Add an alertDetails data entry for each scenario in the trial, needed for charts
			angular.forEach($scope.deployment.windows, function(win) {
				winDetails = run.detailedResults[win.id];
				angular.forEach(win.scenarios, function(scen) {
					angular.forEach(winDetails, function(scenDetails) {
						if (scen.id == scenDetails.scenarioShadowId && !scenDetails.exludedFromTotalAlertCount) {
							$scope.alertDetails.data.push({
								trialNumber: run.trialNumber,
								totalAlertCount: dojoModules.number.format(run.totalAlertCount, {type: 'decimal', places: 0}),
								barLabel: trial.barLabel,
								label: win.name + ": " + scen.name,
								windowName: win.name,
								scenarioName: scen.name,
								alertCount: dojoModules.number.format(scenDetails.alertCount, {type: 'decimal', places: 0})
							});
						}
					});
				});
			});

			// Add to trial history array and update the selected grid row
			$scope.data.trialHistory.push(trial);
			if (setSelected) {
				$scope.data.selectedItem = trial;
			}
		}

		function initTrialGridColumns() {
			var initCols = [{
					label: resources.simulation_grid_runColumnLbl_txt,
					field: "trialDescription",
					// These properties are used by simulationTrialGrid to control the column widths
					baseClass: "simTrialDesc",
					className: "simTrialDesc",
					sizeStyle: ""
				}
			];
			var metricCols = [];
			angular.forEach($scope.data.metricVars, function(varkey) {
				var col = {field: varkey, label: metrics[varkey]};
				metricCols.push(col);
			});
			initCols.push({ label: resources.simulation_grid_metricsGroupLbl_txt, children: metricCols});
			return initCols;
		}

		function updateTrialGridColumns()  {
			// Define trial grid columns based on selected variables
			// Place metrics and thresholds in separate compound column groupings
			// Allow for future possibility that metrics may be added or removed from selection
			var metricCols = [],
				threshCols = [],
				metricIdx = 0,
				threshIdx = 0;
			angular.forEach($scope.data.selectedVars, function(varkey) {
				var col = {field: varkey};
				if ($scope.data.metricVars.indexOf(varkey) >= 0) {
					col.label =  metrics[varkey] || varkey;
					// We currently only have alert count, which is an integer
					// Todo: When using formatter, dojo dgrid sorts by formatted string rather than column value, so this needs to be looked into
					col.formatter = function(value) {
						return dojoModules.number.format(value, {type: 'decimal', places: 0});
					};
					// These three properties are used by simulationTrialGrid to adjust column widths
					col.baseClass = "simMetric_"+metricIdx;
					col.className = col.baseClass;
					col.sizeStyle = "";

					metricCols.push(col);
					metricIdx++;
				}
				else {
					col.label = varkey;
					var param = _.findWhere($scope.data.flatParams, {label: col.label});
					if (param) {  // path is used on header title attribute
						col.path = param.path;
						// Todo: When using formatter, dojo dgrid sorts by formatted string rather than column value, so this needs to be looked into
                        col.formatter = vsdUtil.getFormattedNumberString;
                        if (param.dataFormat === vsdUtil.defaultFormats.date) {
							col.formatter = function(value) {
                                if (vsdUtil.isSasDateConstant(value)) {
                                    return value;
                                }
                                // value is a javascript Date
                                return dojoModules.locale.format(value, {selector: 'date'});
							};
						}
						else if (param.dataFormat === vsdUtil.defaultFormats.datetime) {
							col.formatter = function(value) {
                                if (vsdUtil.isSasDateTimeConstant(value)) {
                                    return value;
                                }
                                // value is a javascript Date
                                return dojoModules.locale.format(value, {formatLength: 'medium'});
							};
						}
						else if (param.dataType === vsmCommonConst.DATA_TYPE_NUMERIC) {
                            col.formatter = function (value) {
                                return vsdUtil.applySASNumericFormat(parseFloat(value), param.dataFormat);
                            };
						}
					}
					// These three properties are used by simulationTrialGrid to adjust column widths
					col.baseClass = "simVar_"+threshIdx;
					col.className = col.baseClass;
					col.sizeStyle = "";

					threshCols.push(col);
					threshIdx++;
				}
			});

			var initCols = initTrialGridColumns(),
			newCols = _.where(initCols, {label: resources.simulation_grid_runColumnLbl_txt});

			if ($scope.boundPrimitives.showAnchorDateColumns) {
            var anchorDateCols = [];
				if ($scope.boundPrimitives.useAnchorStartDate) {
					anchorDateCols.push({
						field:     'anchorStartDate',
						label:     resources.simulation_grid_startDateLbl_txt,
						formatter: anchorDateFormatter,
						baseClass: "simAnchorDate",
						className: "simAnchorDate"
					});
				}
				if ($scope.boundPrimitives.useAnchorEndDate) {
					anchorDateCols.push({
						field:     'anchorEndDate',
						label:     resources.simulation_grid_endDateLbl_txt,
						formatter: anchorDateFormatter,
						baseClass: "simAnchorDate",
						className: "simAnchorDate"
					});
				}
				newCols.push.apply(newCols, anchorDateCols);

				// Add to front of selectedVars, which correspond to the parallel coordinate chart axes
				// Todo: check to see if we want to plot these
				/*
				for (var i=0; i<anchorDateCols.length; i++) {
					$scope.data.selectedVars.splice(i, 0, anchorDateCols[i].field);
					$scope.data.pcAxisLabels.splice(i, 0, anchorDateCols[i].label);
				};
				*/
			}

			//$scope.data.trialGridColumns.push({ label: resources.simulation_grid_runColumnLbl_txt, children: trialInfo});
			newCols.push({ label: resources.simulation_grid_metricsGroupLbl_txt, children: metricCols});
			if (threshCols.length > 0) {
				newCols.push({ label: resources.simulation_grid_thresholdsGroupLbl_txt, children: threshCols} );
			}
			$scope.data.trialGridColumns = newCols;

			$scope.data.trialGridSort = {field: "trialNumber", descending: false};
		}

		function executePromotion() {

			if (!$scope.boundPrimitives.canSave) {
				$scope.showErrorMessage(resources.simulation_saveSelectTrialMsg_txt);
				return;
			}

			// Get run object for selected trial
			var run = $scope.data.runObjects[$scope.data.selectedTrial.trialNumber];
			if (!(run && run.simulationRunId)) {
				$scope.showErrorMessage(resources.simulation_grid_runNotFoundError_txt);
				return;
			}
			$scope.showLoadingMask(resources.simulation_savingToDeployment_msg_txt);
			simulationSvc.saveToDeployment($scope.deployment.id,  run.simulationRunId)
				.then(function (results) {
					$scope.showStatusMessage(resources.simulation_saveCompleted_msg_txt);
					$location.path('/deployment/' + results.id);
				});
		}

		function confirmPromotion(row) {
			confirmDialogSvc.setTitle(resources.simulation_grid_promoteConfirmTitle_txt)
				.setConfirmCallback(function() {
					$scope.data.selectedTrial = row;
					executePromotion();
				})
				.setCancelCallback(
				function(){
				}
			);
			//Set message and button labels depending on validity
			confirmDialogSvc.setConfirmLabel(resources.deployment_dialogs_yesBtn_txt)
				.setCancelLabel(resources.deployment_dialogs_noBtn_txt)
				.setMsg(dojoModules.string.substitute(resources.simulation_grid_promoteConfirmMsg_txt,[row.trialNumber]));
			confirmDialogSvc.showDialog();
		}

		function flattenParameterTree(level,currPath,threshVars, winId) {

			if (!level ) {
				return [];
			}
			//Allocate parameter array
			if(!threshVars) {
				threshVars = [];
			}
			// Need to add windowId to parameter objects because you might have two windows in the deployment
			// having the same predecessor window, and the predecessor window child nodes need to use the
			// same threshold parameters.
			var windowId = level.objectType === "Window" ? level.id : winId;

			//Get parameters from current level
			angular.forEach(level.thresholds,function(t) {
				var tParams,
					path = currPath + "/" + level.nodeType + ": " + (level.label || level.name);
				switch(level.nodeType) {
					case "SCENARIO":
						getRuleParams(t,path,threshVars, windowId);
						break;
					case "WINDOW":
						getConditionParams(level.condition, path+"/"+resources.simulation_path_conditionLbl_txt+": ", threshVars, windowId);
						getWindowColParams(t,path,threshVars, windowId);
						if (level.predecessor && !_.findWhere($scope.data.predecessorWindows, {id: windowId})) {
							$scope.data.predecessorWindows.push(level);
						}
						break;
				}
			});

			var winParams, scenParams, sourceParams;

			//Get parameters from leaf windows
			angular.forEach(level.windows,function(w) {
				if (!w.predecessor || !w.hasSuccessorInTree) {
					winParams = flattenParameterTree(w,currPath,threshVars, windowId);
				}
			});

			//Get parameters from any level scenarios
			angular.forEach(level.scenarios,function(sc) {
				scenParams = flattenParameterTree(sc,currPath,threshVars, windowId);
			});

			//Get parameters from any level sources
			angular.forEach(level.sources,function(s) {
				sourceParams = flattenParameterTree(s,currPath,threshVars, windowId);
			});

			//console.log("threshVars=",threshVars);
			return threshVars;


			function getRuleParams(rule,currPath,threshVars, windowId) {
				var path = currPath + "/" + resources.simulation_path_ruleLbl_txt + ": " + (rule.executionOrder + 1);

				//Process condition
				getConditionParams(rule.condition,path + "/" + resources.simulation_path_conditionLbl_txt + ":  ",threshVars, windowId);
				//Process actions
				angular.forEach(rule.actions,function(a) {
					if (a.editControlTypeName !== "BOOLEAN") {
						var actPath = path + "/" + resources.simulation_path_actionLbl_txt + ": " + a.label;
						createThresholdVariable(threshVars, windowId, a, a.label, actPath, "actionText");
					}
				});
			}

			function getWindowColParams(winCol,currPath,threshVars, windowId) {
				var colPath = currPath + "/" + resources.simulation_path_outputColumnLbl_txt + ": " + winCol.label;
				if ($scope.data.lookbackUnitLabels[windowId] && !!winCol.aggrFunction) {
					getColumnLookbackParam(winCol, colPath, threshVars, windowId);
				}
				colPath = currPath + "/" + resources.simulation_path_outputColumnLbl_txt + ": "
					+ winCol.label + "/" + resources.simulation_path_conditionLbl_txt + ": ";
				getConditionParams(winCol.condition,colPath,threshVars, windowId);
			}

			function getConditionParams(cond,currPath,threshVars, windowId) {
				if (!cond) {
					return;
				}
				if(_.contains(["AND","OR","NOT"],cond.operator)) {
					angular.forEach(cond.childConditions,function(cc) {
						var path = currPath + "/" + cond.operator;
						getConditionParams(cc,currPath,threshVars, windowId);
					});
				}
				else {
					var path = currPath +  (cond.columnLabel || cond.columnName) + " " + cond.operator,
						varLabel = (cond.columnLabel || cond.columnName) + " " + cond.operator;
					createThresholdVariable(threshVars, windowId, cond, varLabel, path, "operand2Constant");
				}
			}

			function getColumnLookbackParam(col, currPath, threshVars, windowId) {
				var path = currPath + "/" + resources.scWindow_AggBuilder_lookBackLabel_txt,
				   varLabel = dojoModules.string.substitute(resources.simulation_lookbackPeriodLbl_txt, [col.label || col.name]);
				createThresholdVariable(threshVars, windowId, col, varLabel, path, "lookbackPeriod");
			}

			function createThresholdVariable(threshVars, windowId, object, varLabel, path, valueProp) {
				if (!_.findWhere(threshVars, {id: object.id, objectType: object.objectType, windowId: windowId})) {

					//console.log("object=",object);
					var dataType = vsmCommonConst.DATA_TYPE_CHAR,
						dataFormat = null;

					switch (object.nodeType) {
						case "OUTPUT_COLUMN":
							if (object.lookbackPeriod) {
								dataType = vsmCommonConst.DATA_TYPE_NUMERIC;
								dataFormat = vsdUtil.defaultFormats.integer;
							}
							break;
						case "CONDITION":
							dataType = object.columnDataType;
							dataFormat = object.columnFormat;
							break;
						case "ACTION":
							dataType = object.editControlTypeName;
							if (dataType === "NUMBER") {
								dataType = vsmCommonConst.DATA_TYPE_NUMERIC;
								dataFormat = vsdUtil.defaultFormats.number;  // we don't currently have the action's format on the object
							}
							break;
					}

					var existLabels = _.pluck(threshVars, "label");
					var newLabel = createUniqueLabel(existLabels, varLabel);
					threshVars.push({
						dataType: dataType,
						dataFormat: dataFormat,
						windowId: windowId,
						id: object.id,
						objectType: object.objectType,
						label: newLabel,
						path: path,
						value: object[valueProp],
						valueProperty: valueProp
					});
				}
			}
			function createUniqueLabel(existing,currValue) {
				//Derive unique label
				var retLabel = currValue;
				if(_.contains(existing,retLabel)) {
					var nbr = 1;
					do {
						retLabel = currValue + '(' + nbr++ + ')';
					}while(_.contains(existing,retLabel));
				}
				return retLabel;
			}
		}

		function runNewSimulation() {

			var modifiedParams = [],
				excludedWindows = [];

			// Get threshold parameters that differ from baseline
			angular.forEach($scope.data.baseThresholds, function(obj) {
				var threshold = $scope.data.boundThresholds[obj.nodeId];
				if (threshold && threshold.nodeType !== "WINDOW") {
					if (threshold[obj.valueProperty] !== obj.value) {
						//Todo: update to only include params that are needed
						if (!_.findWhere(modifiedParams, {id: obj.id, objectType: obj.objectType})) {
							modifiedParams.push(threshold);
						}
					}
				}
			});

			// Get ids of the windows to be excluded from alert count
			angular.forEach($scope.data.predecessorWindows, function(w) {
				if (!w.includeInAlertCount) {
					excludedWindows.push(w.id);
				}
			});

			var endDate = getAdjustedAnchorEndDatetime($scope.data.anchorEndDate);

            // Check if anchor dates match baseline
            var baseline = $scope.data.runObjects[$scope.data.baseTrialNbr],
                baselineStartDate = baseline.anchorStartDate ? new Date(baseline.anchorStartDate) : null,
                baselineEndDate = baseline.anchorEndDate ? new Date(baseline.anchorEndDate) : null,
                anchorDatesMatchBaseline = true;

            if (($scope.boundPrimitives.useAnchorStartDate && baselineStartDate && $scope.data.anchorStartDate && baselineStartDate.valueOf() !== $scope.data.anchorStartDate.valueOf()) ||
                ($scope.boundPrimitives.useAnchorEndDate && baselineEndDate && endDate && baselineEndDate.valueOf() !== endDate.valueOf())) {
                anchorDatesMatchBaseline = false;
                // Set flag to show anchor date columns in trial grid
                $scope.boundPrimitives.showAnchorDateColumns = true;
            }

            // Show message if there are no changes from baseline and stop processing.
			if ( modifiedParams.length === 0 && excludedWindows.length===0 && anchorDatesMatchBaseline) {
				$scope.showInfoMessage(resources.simulation_noThresholdChangesMsg_txt);
				return;
			}

			$scope.showLoadingMask(resources.simulation_runningSimulation_msg_txt);
			simulationSvc.runSimulation($scope.deployment.id, modifiedParams, excludedWindows, $scope.data.anchorStartDate, endDate)
				.then(function (results) {
					results.modifiedParams = modifiedParams;
					results.excludedWindows = excludedWindows;
					loadNewTrial(results);
					$scope.hideLoadingMask();
				});
		}

		function validateSimulation() {
			if (!$scope.boundPrimitives.isValid) {
				$scope.showErrorMessage(resources.simulation_saveSelectTrialMsg_txt);
			}
			return $scope.boundPrimitives.isValid;
		}

		function anchorDateFormatter(value) {
			if (value) {
                if ($scope.boundPrimitives.anchorDateIsDateTime) {
                    return dojoModules.locale.format(new Date(value));
                }
                else {
                    return dojoModules.locale.format(new Date(value), {selector: 'date'});
                }
			}
            else {
                return "";
            }
		}
		//endregion


		//region *** ViewModel Data **

		$scope.resources = dojoModules.resources;

		// Bound primitives used to control view, e.g whether items are disabled, etc.
		$scope.boundPrimitives = {
			initialized: false,
			isDirty: false,
			isReadOnly: true,
			canSave: false,
			isValid: true,
			selVersion: null,     // selected deployment version
			useAnchorStartDate:    false,
			useAnchorEndDate:      false,
			showAnchorDateColumns: false,
			anchorDateIsDateTime:  false
		};

		$scope.deployment = {};
		$scope.currentVersion = null;
		$scope.selectionData = {
			versions: []
		}
		$scope.data = {};

		$scope.paramDetails = {
			data: null,
			selectedDOM: null,
			showTooltip: false,
			openDetails: function(colData, domNode) {
				if (colData && colData.path && domNode) {
					$scope.paramDetails.data = colData;
					$scope.paramDetails.selectedDOM = domNode;
					$scope.paramDetails.showTooltip = true;
				}
			},
			closeDetails: function() {
				$scope.paramDetails.showTooltip = false;
				$scope.alertDetails.selectedDOM = null;
				$scope.paramDetails.data = null;
			}
		};

		// Define attributes for bar chart hover tooltip
		$scope.alertDetails = {
			title1: resources.simulation_alertsTooltip_title1_txt,
			title2: "",
			data: [],           // all trial detail data
			detailData: [],     // selected trial detail data
			stackVars: ["alertCount"],   // detail metric
			groupVar: "label",           // detail category var
			sortVar: "label",
			selectedBarDOM: null,
			showTooltip: false,
			openDetails: function(barData, domNode) {
				if (!(barData && domNode)) {
					return;
				}
				$scope.alertDetails.type = 'LIST';
				$scope.alertDetails.title2 = dojoModules.string.substitute(resources.simulation_alertsTooltip_title2_txt,
					[barData.label.replace(";"," ")]);
				$scope.alertDetails.detailData = _.where($scope.alertDetails.data, {trialNumber: barData.group});
				//Assign dom node
				$scope.alertDetails.selectedBarDOM = domNode;
				//Trigger dialog
				$scope.alertDetails.showTooltip = true;
			},
			closeDetails: function() {
				$scope.alertDetails.showTooltip = false;
				$scope.alertDetails.title2 = "";
				$scope.alertDetails.title3 = "";
				$scope.alertDetails.selectedBarDOM = null;
				$scope.alertDetails.detailData = [];
			}
		};

		//endregion

		//region *** ViewModel Methods ***

		// Breadcrumb link
		$scope.goToParentDeployment = function() {
			$location.path('/deployment/' + $scope.deployment.id);
		};

		$scope.showHomeScreen = function() {
			$scope.showHome();
		};

		$scope.showNewWindowDialog = function() {
			$scope.newWindow();
		};

		$scope.showNewScenarioDialog = function() {
			$scope.newScenario();
		};

		$scope.showNewDeploymentDialog = function() {
			$scope.newDeployment();
		};

		$scope.goToHubScreen = function() {
			$scope.goToHub();
		};

		$scope.showOpenScreen = function() {
			$scope.showOpen();
		};

		$scope.runSimulation = function() {
			$scope.validateParameterTree();
			if (!$scope.boundPrimitives.isValid) {
				$scope.showErrorMessage(resources.simulation_validationErrorsMsg_txt);
				return;
			}
			runNewSimulation();
		};

		$scope.save = function() {
			if (validateSimulation) {
				confirmPromotion($scope.data.selectedTrial);
			}
		};

		$scope.saveTrialNumber = function(trialNbr) {
			if ($scope.data.selectedTrial && $scope.data.selectedTrial.trialNumber !== trialNbr) {
				$scope.data.selectedTrial = _.findWhere($scope.data.trialHistory,{trialNumber: trialNbr});
			}
			$scope.save();
		};

		$scope.switchChart = function(type) {
			$scope.data.chartType=type;
			// Restore trial grid's full history when they switch chart type.
			$scope.data.filteredHistory =  $scope.data.trialHistory;
			$scope.$apply();
		};

		$scope.swapTooltipDetails = function() {
			$scope.alertDetails.type = ($scope.alertDetails.type == 'LIST') ? 'CHART' : 'LIST';
			$scope.$apply();
		};

		$scope.onRowSelection = function (selObject) {
			$scope.data.selectedTrial = selObject;
			$scope.boundPrimitives.canSave = !!selObject && !selObject.baseline;
		};

		$scope.loadValuesIntoTree = function (row) {
			if (!row) {
				return;
			}
            // Load selected trial's values into parameter tree's bound thresholds
			var firstChangedThreshold = null;
			angular.forEach($scope.data.flatParams, function(param) {  // use flatParams to map column names in row to the id
                var rowProp = param.label;

                // Need sas date/time values for bound thresholds instead of the javascript date values that are used by the trial grid
                if (row.hasOwnProperty(param.label + ":sasDateValue")) {
                    rowProp += ":sasDateValue";
                }
                else if (row.hasOwnProperty(param.label + ":sasDateTimeValue")) {
                    rowProp += ":sasDateTimeValue";
                }
                var rowValue = row[rowProp];
				angular.forEach($scope.data.boundThresholds, function(threshold) {
					if (threshold.id == param.id && threshold.objectType == param.objectType && threshold.windowId == param.windowId) {
						var property = param.valueProperty;
						if (rowValue !== threshold[property]) {
							threshold[property] = rowValue;
							$scope.$apply();
							if (firstChangedThreshold == null) {
								firstChangedThreshold = threshold;
							}
						}
					}
				});
			});
			if (firstChangedThreshold) {
				$scope.data.scrollToThreshold = {threshold: firstChangedThreshold};
				$scope.$apply();
			}
		};

		// This would have been better defined within the parameter tree directive, but it needs to be callable from
		// this controller at time user tries to run simulation.
		$scope.validateParameterTree = function () {
			var valid = true;
			dojoModules.query(".simParameterTree .dijitValidationTextBox").forEach(function (child) {
				var widget = dojoModules.registry.byNode(child);
				if (widget && widget.validate) {
					var widgetValid = widget.isValid();
					valid = valid && widgetValid;
					if (!widgetValid) {
						//console.log("invalid widget: ", widget);
					}
				}
			});
			$scope.boundPrimitives.isValid = valid;
		};

		$scope.onRowRMB = function(rows) {      // note: RMB event also triggers the row-select event
            var row = rows[0].data
			var inputRowMenuItems = [];
			var loadValuesIntoTree = {
				label: dojoModules.string.substitute(resources.simulation_grid_loadValuesIntoTree_txt, [row.trialNumber]),
				onClick: function(evt) {
					if(row) {
						$scope.$apply($scope.loadValuesIntoTree(row));
					}}
			};
			inputRowMenuItems.push(loadValuesIntoTree);
			var promoteToDeployment = {
				label: dojoModules.string.substitute(resources.simulation_grid_promoteToDeployment_txt, [row.trialNumber]),
				onClick: function(evt) {
					if(row) {
						confirmPromotion(row);
					}}
			};
			if (!row.baseline && !$scope.boundPrimitives.isReadOnly) {
				inputRowMenuItems.push(promoteToDeployment);
			}
			return inputRowMenuItems;
		};

		$scope.onRowHover = function (row) {
			$scope.data.hoveredTrial = row;
		};

		$scope.onParCoordBrush = function(selection) {
			$scope.data.filteredHistory = selection ? selection : $scope.data.trialHistory;
		};

		$scope.$on(vsmCommonConst.CONDITION_BUILDER_CHANGE,function(evt) {
			evt.preventDefault();
			//$scope.setDirty(true);
		});

		//endregion

		initializeSimulation();

	}]);
});

},
'vsd/deployment/simulation/simulationSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('simulationSvc', ['$http',
	function ($http) {

		var urlBase = 'action/',
			initParameters = {
				deployment: null
			};

		return {
			setParameters:    function (p) {
				initParameters = p;
			},
			getParameters:    function () {
				return initParameters;
			},

			// returns {parameter tree <object>}
			getParameterTree: function (deploymentId) {
				return $http.get(urlBase + 'simulation/paramTree/' + deploymentId).then(function (response) {
					return response.data;
				});
			},

			// returns {simulationId: <string>, parameter: <object>}
			runSimulation: function (deploymentId, modifiedParams, excludedWindows, anchorStartDate, anchorEndDate) {
				var payload = {
					modifiedParams: modifiedParams,
					excludedWindows: excludedWindows,
					anchorStartDate: anchorStartDate,
					anchorEndDate: anchorEndDate
				};
				return $http.post(urlBase + 'simulation/run/' + deploymentId, payload).then(function (response) {
					return response.data;
				});
			},

			// returns {simulationId: <string>, parameter: <object>}
			getAllRuns: function (deploymentId, modifiedParams, excludedWindows) {
				return $http.get(urlBase + 'simulation/allRunsForDeployment/' + deploymentId).then(function (response) {
					return response.data;
				});
			},

			// returns {simulationId: <string>, parameter: <object>}
			saveToDeployment: function (deploymentId, simRunId) {
				return $http.post(urlBase + 'deployment/' + deploymentId + '/updateParamsAsInSimRun/' + simRunId).then(function (response) {
					//console.log("saveToDeployment:",response.data);
					return response.data;
				});
			}

		};
	}]);

});

},
'vsd/deployment/simulation/simulationTrialGrid':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
	directive('simulationTrialGrid', ['$rootScope',
                                     '$compile',
                                     'Util',
                                     'dojoModules',
      function (
         $rootScope,
         $compile,
         Util,
         dojoModules) {

         return {
            templateUrl:   'vsd/deployment/simulation/simulationTrialGrid.html',
            replace:    true,
            restrict:   "E",
            scope: false,

            link:       function (scope, element, attrs, ctrl) {

               var grid = null,
                  minColumnWidth = 100,
                  saveColumn = {
                     field: "trialNumber",
                     label: "",
                     className: "simSaveCol",
                     renderCell: renderSaveColumn
                  };

               // Add/update column containing save icon
               scope.$watch('data.trialGridColumns', function(newVal, oldVal) {

                  if (grid === null) {
                     var node = $(".simTrialGrid");
                     if (node && node.length > 0) {
                        grid = dojoModules.registry.byNode(node[0]);
                        grid.on("resize", function () {
                          //  checkColumnSizes();
                        });
                     }
                  }

                  if (newVal) {
                     var saveColIndex = scope.data.trialGridColumns.indexOf(saveColumn);
                     if (saveColIndex === -1 && !scope.boundPrimitives.isReadOnly) {
                        // add save column
                        scope.data.trialGridColumns.push(saveColumn);
                     }
                     else if (saveColIndex >= 0 && scope.boundPrimitives.isReadOnly) {
                        // remove save column
                        scope.data.trialGridColumns.splice(saveColIndex,1);
                     }
                     checkColumnSizes();
                  }
               });

               function renderSaveColumn(object, value, node, options) {

                  if (object.baseline) {
                     node.title = dojoModules.string.substitute(scope.resources.simulation_grid_saveNATitleMsg_txt, [object.trialNumber]);
                     return;  // save is not applicable to base trial run
                  }
                  var template = '<span style="display:inline-block;align:align:center;">'  +
                     '<span sng-dijit="dijit/form/Button" sng-click="saveTrialNumber(' + object.trialNumber + ')" ' +
                     'data-dojo-props="label: ' + "'" +
                     dojoModules.string.substitute(scope.resources.simulation_grid_promoteToDeployment_txt, [object.trialNumber]) + "'" +
                     ", showLabel:false, iconClass:'sasSaveIcon'" + '"></span>';

                  var btn = $compile(template)(scope);
                  node.appendChild(btn[0]);

                  safeApply();
               }

               function checkColumnSizes() {
                  // Adjust column widths in case new threshold values don't fit.
                  // Resizer plugin does not work with CompoundColumns, and you can only control
                  // width by assigning style class containing width property or manipulate dom objects.

                  if (scope.data.trialGridColumns && scope.data.trialGridColumns.length >= 3 &&
                     scope.data.trialHistory && scope.data.trialHistory.length > 0) {

                     var varCnt = 0,
                        idx = 0,
                        numCols = scope.data.trialGridColumns.length;

                     // Count number of columns in the grid
                     for (idx = 0; idx < numCols; idx ++) {
                        if (scope.data.trialGridColumns[idx].children) {  // this is a column group
                           varCnt += scope.data.trialGridColumns[idx].children.length;
                        }
                        else {  // this is a regular column
                           varCnt ++;
                        }
                     }

                     if (varCnt > 5) {
                        for (idx = 1; idx < numCols; idx++) {
                           if (scope.data.trialGridColumns[idx].children) {  // adjust columns inside the column group
                              adjustColumnWidths(scope.data.trialGridColumns[idx].children);
                           }
                           else {
                              adjustColumnWidths(scope.data.trialGridColumns[idx]);  // adjust the column
                           }
                        }
                     }
                  }

                  function adjustColumnWidths(columns) {
                     var cols = angular.isArray(columns) ? columns : [columns];
                     angular.forEach(cols, function (col) {
                        if (col.baseClass) {
                           var calcWidth = getColWidth(col);
                           col.sizeStyle = getColSizeStyle(calcWidth);
                           col.className = col.baseClass + "." + col.sizeStyle;
                        }
                     });
                  }
               }

               function getColWidth(col) {
                  // Get width of largest item in the column
                  var varkey = col.field,
                     maxLength = _.max(_.map(_.pluck(scope.data.trialHistory, varkey), function (val) {
                     //return (val ? val.toString().length : 0); }
                     var displayedVal = (col.formatter && val != null) ? col.formatter(val) : val.toString();
                     var length = getTextWidth(displayedVal);
                     //console.log("val=", val, ", length=", length);
                     return length;
                  }));
                  return Math.max(maxLength, minColumnWidth);
               }

               function getTextWidth(text) {
                  // Get width of rendered text in pixels
                  var cellFont = getTextWidth.cellFont || getFont();
                  var canvas = getTextWidth.canvas || (getTextWidth.canvas = document.createElement("canvas"));
                  var context = canvas.getContext("2d");
                  context.font = cellFont || "14px Lato, sans-serif";
                  var metrics = context.measureText(text);
                  return metrics.width;

                  function getFont() {
                     var cell = $(".simTrialGrid td[role='gridcell']");
                     if (cell && cell.length > 0) {
                        return (cell.css("font-style") + " " +
                           cell.css("font-weight") + " " +
                           cell.css("font-size") + " " +
                           cell.css("font-family"));
                     }
                     return "14px Lato, sans-serif";
                  }
               }

               function getColSizeStyle(width) {
                  // return column style with closest width
                  if (width <= 100) {
                     return "colSize100";
                  }
                  else if (width >= 400) {
                     return "colSize400";
                  }
                  else {
                     // Find closest matching colSize width. They are in increments of 25.
                     var floor = Math.floor(width/25),
                        mod = width % 25,
                        styleWidth = (mod > 0) ? (floor+1)*25 : floor*25;
                     return "colSize" + styleWidth;
                  }
               }

               function safeApply(){
                  if (!$rootScope.$$phase) {
                     scope.$apply();
                  }
               }
				}
         };
      }]);

});

},
'vsd/scenario/ScenarioCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioCtrl', ['$scope',
        '$routeParams',
        '$q',
        '$filter',
        'scenarioSvc',
        'newScenarioSvc',
        'newDeploymentSvc',
        'windowSvc',
        'operatorSvc',
        'vsdUtil',
        'dojoModules',
        'messageSvc',
        'actionEditorSvc',
        '$timeout',
        'saveAsSvc',
        function ($scope,
                  $routeParams,
                  $q,
                  $filter,
                  scenarioSvc,
                  newScenarioSvc,
                  newDeploymentSvc,
                  windowSvc,
                  operatorSvc,
                  vsdUtil,
                  dojoModules,
                  messageSvc,
                  actionEditorSvc,
                  $timeout,
                  saveAsSvc) {


            var scenarioNameValid,
                $orderBy = $filter('orderBy');

            $scope.EDIT_MODE_MANUAL = 'M';
            $scope.EDIT_MODE_EXPLORE = 'E';
            $scope.EDIT_MODE_DECISONTABLE = 'B';
            $scope.EDIT_MODE_DECISIONTREE = 'R';
            $scope.WINDOW_OUTPUT = 'WO';
            $scope.SCENARIO_OUTPUT = 'SO';

            $scope.EVT_DECISIONTREE_RUN = "EVT_DECISIONTREE_RUN";


            //region *** Private Methods
            function setDirty(dirty) {
                $scope.data.isDirty = dirty;
                $scope.data.disableSave = !dirty;
            }

            function getMaxConditionNbr(cond, currMax) {
                if (!cond) {
                    return currMax;
                }
                if (cond.executionOrder > currMax) {
                    currMax = cond.executionOrder;
                }
                if (cond.childConditions.length > 0) {
                    cond.childConditions.forEach(function (child) {
                        var childMax = getMaxConditionNbr(child, currMax);
                        if (childMax > currMax) {
                            currMax = childMax;
                        }
                    });
                }
                return currMax;
            }

            function initFromScenario(scenarioParams) {
                //Create scenario object with defaults
                var scenario = {
                    id: 0,
                    name: null,
                    desc: null,
                    scenarioWindowId: 0,
                    scenarioWindowPartitionKeyColumns: scenarioParams.scenarioWindowPartitionKeyColumns,
                    createTimestamp: new Date(),
                    createUser: null,
                    active: true,
                    type: null,
                    entityType: null,
                    entityKeyColumns: [],
                    editMode: null, /* $scope.EDIT_MODE_EXPLORE,*/
                    executionFrequency: {id: 0},
                    executionOrder: scenarioParams.executionOrder,
                    riskFactor: false,
                    state: vsmCommonConst.OBJ_STATUS_VALID,
                    events: [],
                    scenarioColumns: []
                };
                //Copy over scenarioParams passed in, if any
                angular.extend(scenario, scenarioParams);

                //Create blank array in data for scenario columns
                $scope.data.usedConditionColumns = [];
                $scope.data.usedActionColumns = [];

                return scenario;
            }

            function createBlankRule(currNumRules) {
                return {
                    active: true,
                    name: "Rule 1",
                    createTimestamp: new Date(),
                    createUser: null,
                    executionOrder: currNumRules,
                    condition: createBlankConjCondition("AND"),
                    actions: []
                };
            }

            function createBlankConjCondition(conj) {
                var operator = $scope.data.operators ? _.findWhere($scope.data.operators, {name: conj}) : {
                    name: conj,
                    type: "LOGICAL"
                };
                return {operand1ColumnInfo: {}, operator: operator, childConditions: [], executionOrder: 0};
            }

            function createBlankCondition(rule) {
                return {
                    operand1ColumnInfo: $scope.data.windowTableOutColsCond[0],
                    operator: _.findWhere($scope.data.operators, {name: '='}),
                    operand2Constant: null,
                    childConditions: [],
                    executionOrder: getMaxConditionNbr(rule.condition, $scope.maxCondNbr)
                };
            }

            function createSimpleCond(lhsCol, operStr, operandStr) {
                return {
                    operand1ColumnInfo: lhsCol,
                    operator: _.findWhere($scope.data.operators, {name: operStr}),
                    operand2Constant: operandStr,
                    childConditions: []
                };
            }

            function processOpenOrNew() {
                scenarioNameValid = true;
                // If no id and parameters not set, refreshed before getting an id - show the home screen
                if (!$routeParams.id && !newScenarioSvc.getScenarioParams().name) {
                    if ($scope.scenario) {
                        $scope.scenario.editMode = null;
                    }
                    $scope.showHome();
                    return;
                }
                /*else
                 $scope.scenario.editMode = newScenarioSvc.getScenarioParams().editMode;
                 */
                $scope.showLoadingMask($scope.resources.main_loadingScenario_msg_txt);
                $q.all([
                    operatorSvc.getOperators(),
                    scenarioSvc.getScenarioTypes(),
                    scenarioSvc.getExecutionFrequencies(),
                    scenarioSvc.getEntityTypes()
                ]).then(
                    function (results) {
                        $scope.data.operators = results[0];
                        $scope.data.scenarioTypes = results[1];
                        $scope.data.executionFrequencies = results[2];
                        $scope.data.entityTypes = results[3];
                        execFrequencyAddLocalizedLabels();


                        if ($routeParams.id) {
                            $scope.scenario = scenarioSvc.getScenario($routeParams.id)
                                .then(function (data) {
                                    $scope.scenario = data;
                                    $scope.data.activeRule = $scope.scenario.events[0];
                                    //Initialize simplified scenario columns array
                                    populateCondActColumns();
                                    //Grab scenario dependencies
                                    getScenarioDependencies();
                                });
                        }
                        else {
                            //Initialize scenario from new scenario service
                            $scope.scenario = initFromScenario(newScenarioSvc.getScenarioParams());
                            $scope.data.activeRule = $scope.scenario.events[0];
                            //Grab scenario dependencies (using blank scenario initialized on scope)
                            getScenarioDependencies();
                        }
                        //Clear out service variables for next use
                        newScenarioSvc.clearSettings();
                    }
                );
            }

            function execFrequencyAddLocalizedLabels() {
                _.each($scope.data.executionFrequencies, function (item) {
                    switch (item.name) {
                        case "DAILY":
                            item.label = $scope.resources.scenario_ScenarioPropertiesDlg_freq_daily_txt;
                            break;
                        case "WEEKLY":
                            item.label = $scope.resources.scenario_ScenarioPropertiesDlg_freq_weekly_txt;
                            break;
                        case "MONTHLY":
                            item.label = $scope.resources.scenario_ScenarioPropertiesDlg_freq_monthly_txt;
                            break;
                        case "INTRADAY":
                            item.label = $scope.resources.scenario_ScenarioPropertiesDlg_freq_intraday_txt;
                            break;
                    }
                });
            }

            function populateCondActColumns() {
                //Initialize simplified scenario columns array
                var condCols = [],
                    actCols = [];
                angular.forEach($scope.scenario.scenarioColumns, function (sc) {
                    if (sc.levelCode === 'C') {
                        condCols.push(sc.column);
                    }
                    else {
                        actCols.push(sc.column);
                    }
                });
                $scope.data.usedConditionColumns = condCols;
                $scope.data.usedActionColumns = actCols;
            }

            function tempCondActToDB(src, tgt, typeStr) {
                angular.forEach(src, function (col) {
                    var dbObj = {
                        column: col,
                        levelCode: typeStr,
                        order: tgt.length
                    };
                    tgt.push(dbObj);
                });
            }

            function getAvailableActionCols() {
                var availCols = [];
                angular.forEach($scope.actionInfo.allVars, function (actCol) {
                    var actionColInUse = _.find($scope.data.activeRule.actions, function (action) {
                        return action.lhsColumn.id === actCol.id;
                    });
                    if (!actionColInUse) {
                        availCols.push(actCol);
                    }
                });
                return availCols;
            }

            function createActionMenuItems() {
                var items = [];
                angular.forEach(getAvailableActionCols(), function (col) {
                    items.push({label: col.label, value: col, onClick: addAction});
                });
                return items;
            }

            function initActions(replaceLhsVars) {
                //Get the list of action columns for the window
                windowSvc.getActionColumns($scope.scenario.scenarioWindowId)
                    .success(function (data) {
                        $scope.actionInfo.allVars = data.columns;
                        $scope.actionInfo.menuItems = createActionMenuItems();
                        if (replaceLhsVars) {
                            updateActionsWithColumnChanges();
                        }
                    });
            }

            function updateActionsWithColumnChanges() {
                angular.forEach($scope.data.activeRule.actions, function (act) {
                    act.lhsColumn = _.findWhere($scope.actionInfo.allVars, {id: act.lhsColumn.id});
                });
            }

            function addAction(lhsCol) {
                var actionText;
                //Determine how to default action text
                switch (lhsCol.editControlType.name) {
                    case "BOOLEAN":    //Default to "false"
                        actionText = "0";
                        break;
                    case "NUMBER":  //Default to zero
                        actionText = "0";
                        break;
                }
                //Create new action
                var act = {
                    actionText: actionText,
                    executionOrder: $scope.data.activeRule.actions.length,
                    columnActions: [{column: lhsCol, leftHandSide: true}],
                    lhsColumn: lhsCol,
                    lhsColumnName: lhsCol.name
                };
                //Add action to active rule
                $scope.data.activeRule.actions.push(act);
                //Add to list of used action columns
                $scope.data.usedActionColumns.push(lhsCol);
                //Recreate menu items
                $scope.actionInfo.menuItems = createActionMenuItems();
                //Mark scenario dirty
                setDirty(true);
            }

            function deleteAction(idx) {
                var actColName = $scope.data.activeRule.actions[idx].lhsColumn.name;
                $scope.data.usedActionColumns = _.reject($scope.data.usedActionColumns, function (col) {
                    return col.name === actColName;
                });
                $scope.data.activeRule.actions.splice(idx, 1);
                //Recreate menu items
                $scope.actionInfo.menuItems = createActionMenuItems();
                //Set dirty flag
                setDirty(true);
            }

            function getScenarioDependencies() {
                $scope.showLoadingMask($scope.resources.main_runningScenarios_msg_txt);


                scenarioSvc.getScenarioDependencies($scope.scenario.scenarioWindowId, $scope.scenario.executionOrder).then(function (results) {


                    //Process window results
                    var totalRecCnt = parseInt(results.data.runWindowResult);
                    $scope.setCounters(0, 0, totalRecCnt);
                    //Process output column results
                    $scope.data.windowOutCols = results.data.outputColumnsResult;
                    //Cache sibling scenario names to validate name change
                    $scope.data.siblingScenarioNames = _.reject(results.data.listScenarioNamesResult, function (name) {
                        return $scope.scenario.name === name;
                    });
                    //$scope.data.siblingScenarioNames = [];
                    //Get window otuput columns (must do after run window since calculating column distinct values
                    windowSvc.getOutputTableColumnsWithClassification($scope.scenario.scenarioWindowId, $scope.scenario.executionOrder).then(function (results) {
                        $scope.data.windowTableOutCols = results;
                        $scope.data.windowTableOutColsCond = _.filter(results, function (col) {
                            return col.columnType === "WO";
                        });
                        $scope.data.windowTableOutColsAct = _.filter(results, function (col) {
                            return col.columnType === "SO";
                        });
                        //Append distinct count to window output colunms to allow restriction by count
                        addDistinctCountToWindowOutCols();
                        //Get record preview from window
                        preparePreviewInfo();
                        //Let all scope listeners know scenario ready
                        $scope.data.scenarioReady = true;
                        //Remove load mask
                        $scope.boundPrimitives.loading = false;
                        //getPreviewData();
                        $scope.hideLoadingMask();
                    });

                });
            }

            function addDistinctCountToWindowOutCols() {
                var distinctCountMap = {};
                angular.forEach($scope.data.windowTableOutColsCond, function (col) {
                    distinctCountMap[col.name] = col.distinctCount;
                });
                angular.forEach($scope.data.windowOutCols, function (col) {
                    col.distinctCount = distinctCountMap[col.name];
                });
            }

            function preparePreviewInfo() {

                var filterCols = angular.copy($scope.data.windowTableOutColsCond),
                    tempCols = angular.copy($scope.data.windowOutCols),
                    structure = [];

                tempCols.sort(function (a, b) {
                    return a.displayOrder - b.displayOrder;
                });

                // Add scenario output columns (actions) that are used upstream
                if ($scope.scenario.editMode === $scope.EDIT_MODE_DECISONTABLE || $scope.scenario.editMode === $scope.EDIT_MODE_DECISIONTREE) {
                    var actCols = _.filter($scope.data.windowTableOutColsAct, function (col) {
                        return ( col.earliestAssignment !== null &&
                        col.earliestAssignment >= 0 &&
                        col.earliestAssignment < $scope.scenario.executionOrder );
                    });
                    actCols.sort(sortByDataTypeAndLabel);
                    tempCols.push.apply(tempCols, actCols);
                    filterCols.push.apply(filterCols, actCols);
                }
                tempCols.forEach(function (col) {
                    var className = null;
                    var formatter = null;
                    if (col.dataType === 'NUMERIC') {
                        className = 'numericCol';
                        formatter = function (value) {
                            return vsdUtil.applySASNumericFormat(value, col.columnFormat);
                        };
                    }
                    var gridCol = {field: col.name, label: col.label, className: className, formatter: formatter};

                    structure.push(gridCol);
                });
                $scope.previewInfo.filterColumns = filterCols;
                $scope.previewInfo.gridStructure = structure;
            }

            function formatScenarioInputCols() {
                var rowNbr = 1;
                //Reformat data for use in left panel
                var tmpArray = [];
                $scope.data.windowTableOutColsCond.forEach(function (col) {
                    if (!isScenarioAutoCol(col)) {
                        var outRec = {
                            id: rowNbr,
                            label: col.label,
                            dataType: col.dataType ? col.dataType.trim() : null,
                            distinctVals: null,
                            outCol: col
                        };
                        tmpArray.push(outRec);
                        //Save input column in a map for lookup by id
                        rowNbr++;
                    }
                });

                //Sort by data type and label
                tmpArray.sort(sortByDataTypeAndLabel);
                //Set scope variable to store data and trigger UI rendering
                $scope.inputColsProps.inputColsData = tmpArray;
            }

            function sortByDataTypeAndLabel(a, b) {
                //Sort by data type and label
                if (a.dataType > b.dataType) {
                    return 1;
                }
                else if (a.dataType < b.dataType) {
                    return -1;
                }
                else {
                    if (a.label > b.label) {
                        return 1;
                    }
                    else if (a.label < b.label) {
                        return -1;
                    }
                    else {
                        return 0;
                    }
                }
            }

            function isScenarioAutoCol(col) {
                return scenarioSvc.isScenarioAutoCol(col, $scope.scenario.scenarioWindowPartitionKeyColumns);
            }

            function findLHSColumnForCondition(cond) {
                var lhsCol,
                    res;
                if (cond.operator.type === "LOGICAL") {
                    cond.childConditions.forEach(function (childCond) {
                        res = findLHSColumnForCondition(childCond);
                        if (res) {
                            lhsCol = res;
                        }
                    });
                }
                else {
                    lhsCol = cond.operand1ColumnInfo;
                }
                return lhsCol;
            }

            function getPreviewData() {

                var filter = "",
                    condFilterStr,
                    previewFilterStr,
                    missingFilterStr,
                    targetColName,
                    columns = $scope.data.windowOutCols;

                if ($scope.scenario.editMode === $scope.EDIT_MODE_DECISONTABLE || $scope.scenario.editMode === $scope.EDIT_MODE_DECISIONTREE) {
                    // Add scenario output columns that are assigned upstream
                    columns = columns.concat(_.filter($scope.data.windowTableOutColsAct, function (col) {
                        return ( col.earliestAssignment !== null &&
                        col.earliestAssignment >= 0 &&
                        col.earliestAssignment < $scope.scenario.executionOrder );
                    }));
                }

                //Exit if conditions invalid
                if (!$scope.validateScenario().detail.conditionsValid) {
                    return;
                }

                //Build filter string based on preview filter parameters
                if (!$scope.previewInfo.filterCond || !$scope.previewInfo.filterCond.operand1ColumnInfo || !$scope.previewInfo.filterCond.operand1ColumnInfo.name || !$scope.previewInfo.filterCond.operator ||
                    (!$scope.previewInfo.filterCond.operand2Constant && $scope.previewInfo.filterCond.operator.name.indexOf("MISSING") === -1)) {
                    previewFilterStr = "";
                }
                else {
                    previewFilterStr = vsdUtil.conditionToFilterString($scope.previewInfo.filterCond);
                }
                //Build filter as a combination of current condition and any preview filter
                if ($scope.data.activeRule && getNonGroupingCondCount($scope.data.activeRule.condition) > 0) {
                    condFilterStr = vsdUtil.conditionToFilterString($scope.data.activeRule.condition);
                }
                //Build combined filter string
                if (condFilterStr && previewFilterStr) {
                    filter = condFilterStr + " AND " + previewFilterStr;
                }
                else if (condFilterStr) {
                    filter = condFilterStr;
                }
                else {
                    filter = previewFilterStr;
                }
                // execute for decision tree only
                // remove missing values from target and variables in path of  decisiontree

                if ($scope.data.activeRule.viewMode === 'treemode') {
                    var treeInputs = $scope.data.activeRule.treeInputs;
                    if (!(treeInputs.lasrTreeOpts.useMissingVar)) {

                        if (treeInputs.targetColumn.dataType === "CHARACTER") {
                            filter = filter + " AND (" + treeInputs.targetColumn.name + " NE  \"\" )";
                        } else {
                            if (treeInputs.targetColumn.dataType === "NUMERIC") {
                                filter = filter + " AND (" + treeInputs.targetColumn.name + " NE  . )";
                            }
                        }
                        var condCol = treeInputs.usedConditionColumns;
                        for (var i = 0; i < condCol.length; i++) {
                            if (condCol[i].dataType === "CHARACTER") {
                                filter += " AND (" + condCol[i].name + " NE  \"\" )";
                            } else {
                                if (condCol[i].dataType === "NUMERIC")
                                    filter += " AND (" + condCol[i].name + " NE  . )";
                            }
                        }
                    }
                }


                //Invoke preview function
                windowSvc.previewData(columns, [], filter, $scope.previewInfo.start, $scope.previewInfo.start + $scope.previewInfo.pageSize)
                    .then(function (data) {
                        $scope.previewInfo.rows = data;
                        //$scope.hideLoadingMask();
                    });
            }

            function clearPreviewData() {
                $scope.previewInfo.rows = [];
                // $scope.$apply();
            }

            function addConditionColumns(newCols) {
                var colsAdded = [];
                newCols.forEach(function (newCol) {
                    var matchedCol = _.find($scope.data.usedConditionColumns, function (existCol) {
                        return existCol.name === newCol.name;
                    });
                    if (!matchedCol) {
                        $scope.data.usedConditionColumns.push(newCol);
                        colsAdded.push(newCol);
                    }
                });
                //Call necessary downstream functions and topic publishes as result of add
                if (colsAdded.length > 0) {
                    //Set dirty flag
                    setDirty(true);
                }
                return colsAdded;
            }

            function deleteConditionsByColName(colName) {
                //Cache unaffected conditions in temp array
                var keepConds = [];
                if ($scope.data.activeRule.condition.childConditions.length > 0) {
                    $scope.data.activeRule.condition.childConditions.forEach(function (cond) {
                        var delChildren = [];
                        if (cond.operator.type === "LOGICAL" && cond.childConditions.length > 0) {
                            //Mark any matching children for deletion
                            for (var i = 0; i < cond.childConditions.length; i++) {
                                var matchingLHSCol = findLHSColumnForCondition(cond.childConditions[i]);
                                if (matchingLHSCol && matchingLHSCol.name === colName) {
                                    delChildren.push(cond.childConditions[i]);
                                }
                            }
                            //Delete any marked children
                            delChildren.forEach(function (delChild) {
                                var idx = cond.childConditions.indexOf(delChild);
                                cond.childConditions.splice(idx, 1);
                            });
                            //If parent had any children left, push it into keep columns
                            if (cond.childConditions.length > 0) {
                                keepConds.push(cond);
                            }
                        }
                        else if (cond.operand1ColumnInfo.name !== colName) {
                            keepConds.push(cond);
                        }

                    });
                    //Empty conditions array
                    vsdUtil.clearArray($scope.data.activeRule.condition.childConditions);
                    //Add all unaffected conditions back - done this way so array ref views might be using aren't changed
                    keepConds.forEach(function (cond) {
                        $scope.data.activeRule.condition.childConditions.push(cond);
                    });
                    //Set dirty flag
                    setDirty(true);

                }
            }

            function deleteConditionCol(col) {
                var colIdx = null,
                    i = 0;

                //Set dirty flag
                setDirty(true);

                $scope.data.usedConditionColumns.forEach(function (condVar) {
                    if (condVar.name === col.name) {
                        colIdx = i;
                    }
                    i++;
                });
                //Remove condition from condVar array and map
                var colName = $scope.data.usedConditionColumns[colIdx].name;
                $scope.data.usedConditionColumns.splice(colIdx, 1);
                //Remove conditions using the deleted column
                deleteConditionsByColName(colName);
                //If there are any condition columns, get preview data (since filter has changed)
                if ($scope.data.usedConditionColumns.length > 0) {
                    getPreviewData();
                }
                //Remove used marking for column (if present)
                if (col.dropNodeId) {
                    try {
                        vsdUtil.unmarkRemovedColumn(col.dropNodeId);
                    }
                    catch (err) {
                        return false;
                    }

                }
                else {
                    return false;
                }


                return true;
            }

            function reorderConditionCol(locations) {
                //Remove the column from the condVars array
                var tgtCol = $scope.data.usedConditionColumns.splice(locations.startIdx, 1)[0];
                //Insert column in new position
                $scope.data.usedConditionColumns.splice(locations.targetIdx, 0, tgtCol);
                //Set dirty flag
                setDirty(true);
            }


            function deleteConditions(inConds) {
                var delIdx = -1;
                inConds.forEach(function (delCond) {
                    for (var i = 0; i < $scope.data.activeRule.condition.childConditions.length; i++) {
                        var cond = $scope.data.activeRule.condition.childConditions[i];
                        if (cond === delCond) {
                            delIdx = i;
                        }
                    }
                    if (delIdx !== -1) {
                        $scope.data.activeRule.condition.childConditions.splice(delIdx, 1);
                    }
                });
                //Set dirty flag
                setDirty(true);

                //Get unique combos
                getPreviewData();
            }

            function countConditions(cond, countGrouping) {
                var count = 0;
                if (cond.operator.type === "LOGICAL") {
                    if (countGrouping || cond.operator.name === "IS MISSING") {
                        count++;
                    }
                    if (cond.childConditions !== null && cond.childConditions.length > 0) {
                        cond.childConditions.forEach(function (childCond) {
                            count += countConditions(childCond);
                        });
                    }
                }
                else {
                    return 1;
                }
                return count;
            }


            function conditionsIdentical(cond1, cond2) {
                if (cond1.operator.type !== cond2.operator.type) {
                    return false;
                }
                else if (cond1.operator.type === "COMPARISON") {
                    if (cond1.operator.name === cond2.operator.name &&
                        cond1.operand1ColumnInfo.name === cond2.operand1ColumnInfo.name &&
                        cond1.operand2Constant === cond2.operand2Constant) {
                        return true;
                    }
                }
                else if (cond1.operator.name === "AND" && cond2.operator.name === "AND") {
                    var childCond1 = cond1.childConditions[0],
                        childCond2 = cond2.childConditions[0];

                    if (childCond1.operator.name === childCond2.operator.name &&
                        childCond1.operand1ColumnInfo.name === childCond2.operand1ColumnInfo.name &&
                        childCond1.operand2Constant === childCond2.operand2Constant) {
                        return true;
                    }

                }
                else if (cond1.operator.name === "NOT" && cond2.operator.name === "NOT") {
                    return conditionsIdentical(cond1.childConditions[0], cond2.childConditions[0]);
                }
                return false;
            }

            function removeIncompleteConditions(parent, conds, level) {
                var delCondsIdxs = [];

                for (var i = 0; i < conds.length; i++) {
                    var cond = conds[i];
                    if (cond.operator.name === "AND" || cond.operator.name === "OR" || cond.operator.name === "NOT") {
                        removeIncompleteConditions(cond, cond.childConditions, level + 1);
                        if (cond.childConditions.length === 0) {
                            delCondsIdxs.push(i);
                        }
                    }
                    else if (!cond.operand1ColumnInfo || !cond.operator || (!cond.operand2Constant && cond.operator && cond.operator.name !== "IS MISSING" && cond.operator.name !== "NOT MISSING")) {
                        delCondsIdxs.push(i);
                    }
                }
                //Delete marked conditions
                var delCount = 0;
                delCondsIdxs.forEach(function (idx) {
                    parent.childConditions.splice(idx - delCount, 1);
                    delCount++;
                });
            }

            function conditionVarsFromConditions(cond) {
                if (cond) {
                    if (cond.operator.type === "LOGICAL" && cond.operator.name !== "IS MISSING") {
                        cond.childConditions.forEach(function (childCond) {
                            conditionVarsFromConditions(childCond);
                        });
                    }
                    else {
                        var matched = _.find($scope.data.usedConditionColumns, function (sc) {
                            return sc.name === cond.operand1ColumnInfo.name;
                        });
                        if (cond.operand1ColumnInfo && !matched) {
                            $scope.data.usedConditionColumns.push(cond.operand1ColumnInfo);
                        }
                    }
                }

            }

            function getNonGroupingCondCount(cond) {
                if (!cond) {
                    return 0;
                }
                return countConditions(cond, false);
            }

            function newScenarioForWindowId() {
                newScenarioSvc.setSelectedWindowId($scope.scenario.scenarioWindowId);
                newScenarioSvc.setSelectedWindowName($scope.scenario.scenarioWindowName);
                navWithConfirm($scope.newScenario);
            }

            function newDeploymentForWindowId() {
                newDeploymentSvc.setInitWindowIds([$scope.scenario.scenarioWindowId]);
                navWithConfirm($scope.newDeployment);
            }

            function savePrepConditions(scenario) {
                if (scenario.editMode === 'E' || scenario.editMode === 'M') {
                    //Remove incomplete conditions under top level parent
                    if (scenario.events[0].condition && scenario.events[0].condition.childConditions && scenario.events[0].condition.childConditions.length > 0) {
                        removeIncompleteConditions(scenario.events[0].condition, scenario.events[0].condition.childConditions, 0);
                    }
                    //If top level condition blank, remove it
                    if (scenario.events[0].condition && (!scenario.events[0].condition.childConditions || scenario.events[0].condition.childConditions.length === 0)) {
                        scenario.events[0].condition = null;
                    }
                }
            }

            function savePrepUsedCondsAndActs(scenario) {
                scenario.scenarioColumns = [];
                tempCondActToDB($scope.data.usedConditionColumns, scenario.scenarioColumns, "C");
                tempCondActToDB($scope.data.usedActionColumns, scenario.scenarioColumns, "A");
            }

            function savePrep(scenario) {
                $scope.showLoadingMask($scope.resources.app_savingChangesMsg_txt);
                //broadcast to child controllers in case there needs to be a prep for save
                $scope.$broadcast("PREP_SAVE_SCENARIO");

                //Prepare conditions for save
                savePrepConditions(scenario);

                //Determine if scenario is valid - if not, popup message and exit
                var valRes = validateScenario(true);
                if (!valRes.isValid) {
                    messageSvc.showError(valRes.msg);
                    //Let children know about save completion so they can restore default conditions
                    $scope.$broadcast(vsmCommonConst.SCENARIO_SAVE_COMPLETE);
                    //Hide loading mask
                    $scope.hideLoadingMask();
                    return false;
                }

                //Convert temporary condition and action columns arrays to final format
                savePrepUsedCondsAndActs(scenario);

                return true;
            }


            function saveScenario() {
                var doSave = savePrep($scope.scenario);
                if (!doSave) {
                    return;
                }

                return scenarioSvc.saveScenario({scenario: $scope.scenario, binColumns: $scope.data.binUpdateColumns})
                    .then(function (savedScenario) {
                        $scope.scenario = savedScenario;
                        $scope.data.activeRule = $scope.scenario.events[0];
                        //Reset bin updates to empty
                        $scope.data.binUpdateColumns = [];
                        //Convert scenario colunm assoc records to simpler format
                        populateCondActColumns();
                        //Let children know about save completion so they can restore default conditions
                        $scope.$broadcast(vsmCommonConst.SCENARIO_SAVE_COMPLETE);
                        //Hide loading mask
                        $scope.hideLoadingMask();
                        //Show save confirmation message
                        messageSvc.showStatus([$scope.resources.scenario_saveCompleteMsg_txt]);
                        $timeout(function () {
                            setDirty(false);
                        });

                    });
            }

            function saveScenarioAs() {
                //Make copy of scenario
                var scenarioCopy = angular.copy($scope.scenario);
                //Prepare scenario metadata for save
                var doSave = savePrep(scenarioCopy);
                if (!doSave) {
                    return;
                }
                //Set properties in saveAs service
                saveAsSvc.setSourceObject(scenarioCopy)
                    .setObjectTypeDisplay($scope.resources.obj_type_scenario_txt)
                    .setMaxNameLength(255)
                    .setPathWithoutId($scope.paths.scenario)
                    .setRemoveObjectIdsFunc(clearScenarioIds)
                    .setUsedNameListFunc(function () {
                        return windowSvc.listScenarioNames($scope.scenario.scenarioWindowId);
                    })
                    .setSaveFunc(function (scenario) {
                        var deferred = $q.defer();
                        scenarioSvc.saveScenario({
                            scenario: scenario,
                            binColumns: $scope.data.binUpdateColumns
                        }).then(function (results) {
                            deferred.resolve(results);
                        });
                        return deferred.promise;
                    });

                $scope.hideLoadingMask();
                //Show dialog after confirm
                navWithConfirm($scope.showSaveAs);

                function clearScenarioIds(scenario) {
                    scenario.name = null;
                    scenario.id = 0;
                    scenario.executionOrder = $scope.data.siblingScenarioNames.length;
                    angular.forEach(scenario.events, function (rule) {
                        //Clear rule id
                        rule.id = 0;
                        //Clear condition ids
                        vsdUtil.clearIds(rule.condition, ['operator', 'operand1ColumnInfo', 'operand2ColumnInfo']);
                        //Clear action ids
                        vsdUtil.clearIds(rule.actions, ['column']);
                    });

                }
            }

            function validateScenario(save) {

                var conditionsValid = true,
                    actionsValid = true,
                    scenarioValid,
                    msg,
                    areas = [];

                if (!scenarioNameValid) {
                    areas.push($scope.resources.scenario_areaNameLbl_txt);
                }

                //Validate manual editing for partition chart conditions table
                var ruleRes = validateRules(save),
                    areaStr;
                if (!ruleRes.valid) {
                    conditionsValid = ruleRes.condsValid;
                    if (!conditionsValid) {
                        areaStr = $scope.resources.scenario_areaConditionsLbl_txt;
                        if (ruleRes.condReason) {
                            switch (ruleRes.condReason.errorType) {
                                case vsmCommonConst.RULE_OPERAND2_CONSTANT_MAXLEN_ERROR:
                                    areaStr += "("
                                    + dojoModules.string.substitute($scope.resources.decisionTable_cellWidthErrrMsg_txt,
                                        [ruleRes.condReason.colName, ruleRes.ruleErrorIdx, vsmCommonConst.RULE_OPERAND2_CONSTANT_MAXLEN, ruleRes.condReason.actualLength])
                                    + ")";
                                    break;
                            }
                            areas.push(areaStr);
                        }
                    }
                    actionsValid = ruleRes.actsValid;
                    if (!actionsValid) {
                        areas.push($scope.resources.scenario_areaActionsLbl_txt);
                    }
                }

                scenarioValid = conditionsValid && actionsValid && scenarioNameValid;
                if (!scenarioValid) {
                    msg = dojoModules.string.substitute($scope.resources.scenario_noSaveInvalidMsg_txt, [areas]);
                }

                return {
                    isValid: scenarioValid,
                    detail: {
                        nameValid: scenarioNameValid,
                        conditionsValid: conditionsValid,
                        actionsValid: actionsValid
                    },
                    msg: msg
                };
            }


            function validateRules(save) {
                var condsValid = true,
                    actsValid = true,
                    condReason,
                    ruleIdx = 1,
                    ruleErrorIdx;
                angular.forEach($scope.scenario.events, function (rule) {
                    var condRes = validateConditions(rule.condition, save);
                    if (!condRes.isValid) {
                        condsValid = false;
                        condReason = condRes.reason;
                        ruleErrorIdx = ruleIdx;
                    }
                    if (!validateActions(rule.actions, save)) {
                        actsValid = false;
                    }
                    ruleIdx++;
                });
                return {
                    valid: condsValid && actsValid,
                    condsValid: condsValid,
                    condReason: condReason,
                    ruleErrorIdx: ruleErrorIdx,
                    actsValid: actsValid
                };
            }

            function validateConditions(condition, save) {

                var result = {isValid: true};
                doValidation(condition);
                return result;

                function doValidation(cond) {
                    if (cond) {
                        if (cond.operator.type === "LOGICAL" && !_.contains(["IS MISSING", "NOT MISSING"], cond.operator.name) && cond.childConditions.length > 0) {
                            cond.childConditions.forEach(function (cond) {
                                // Continue checking if we haven't found an error yet
                                if (result.isValid) {
                                    var opName = cond.operator.name;
                                    if (!_.contains(["IS MISSING", "NOT MISSING"], opName)) {
                                        if (opName === "AND") {
                                            //Send individual conditions in for validation
                                            doValidation(cond);
                                        }
                                        else if (opName === "NOT") {
                                            doValidation(cond.childConditions[0]);
                                        }
                                        else if (opName === "OR") {
                                            doValidation(cond);
                                        }
                                        else {
                                            if (!cond.operand1ColumnInfo || !cond.operand1ColumnInfo.name || !cond.operator || !vsdUtil.valExprConstByDataType(cond.operand2Constant, cond.operand1ColumnInfo, cond.operator, save).isValid) {
                                                result.isValid = false;
                                            }
                                            else if (cond.operand2Constant.length > vsmCommonConst.RULE_OPERAND2_CONSTANT_MAXLEN) {
                                                result.isValid = false;
                                                result.reason = {
                                                    errorType: vsmCommonConst.RULE_OPERAND2_CONSTANT_MAXLEN_ERROR,
                                                    colName: result.reasonColumnName = cond.operand1ColumnInfo.name,
                                                    actualLength: result.reasonActualLength = cond.operand2Constant.length
                                                };
                                            }
                                        }
                                    }
                                }
                            });
                        }
                        else if (cond.operator.type !== "LOGICAL" && !_.contains(["IS MISSING", "NOT MISSING"], cond.operator.name)) {
                            if (!vsdUtil.valExprConstByDataType(cond.operand2Constant, cond.operand1ColumnInfo, cond.operator, save).isValid) {
                                result.isValid = false;
                            }
                            else if (cond.operand2Constant.length > vsmCommonConst.RULE_OPERAND2_CONSTANT_MAXLEN) {
                                result.isValid = false;
                                result.reason = {
                                    errorType: vsmCommonConst.RULE_OPERAND2_CONSTANT_MAXLEN_ERROR,
                                    colName: result.reasonColumnName = cond.operand1ColumnInfo.name,
                                    actualLength: result.reasonActualLength = cond.operand2Constant.length
                                };
                            }
                        }
                    }
                }
            }

            //Note:
            function validateActions(actions, save) {
                var actionsValid = true;
                angular.forEach(actions, function (act) {
                    var valRes = vsdUtil.valExprConstByDataType(act.actionText, act.lhsColumn, null, save);
                    if (!valRes.isValid) {
                        actionsValid = false;
                    }
                });
                return actionsValid;
            }

            $scope.validateScenarioName = function (value) {
                var valRes = vsdUtil.validateDomainObjName(value, $scope.data.siblingScenarioNames),
                    invalidMessage;
                scenarioNameValid = valRes.isValid;
                //If name invalid, set invalid message on text box
                if (!valRes.isValid) {
                    switch (valRes.reason) {
                        case vsmCommonConst.INVALID_NAME_REASON_SYNTAX:
                            invalidMessage = $scope.resources.app_object_name_specCharValidationMsg_txt;
                            break;
                        case vsmCommonConst.INVALID_NAME_REASON_DUPLICATE:
                            invalidMessage = dojoModules.string.substitute($scope.resources.scenario_Controller_invalidNameErrorMsg_txt, [value]);
                            break;
                    }
                }
                return {isValid: valRes.isValid, invalidMessage: invalidMessage};
            };


            function navWithConfirm(navFunc, locationId) {
                navFunc($scope.data.isDirty, validateScenario(true).isValid, saveScenario, $scope.resources.obj_type_scenario_txt, $scope.scenario.name, locationId);
            }


            //endregion

            //region *** ViewModel Data **
            $scope.LOWER_BOUND_OPERATOR_NAME = '<';
            $scope.UPPER_BOUND_OPERATOR_NAME = ">=";
            $scope.resources = dojoModules.resources;
            $scope.boundPrimitives = {
                loading: false,
                loadMsg: $scope.resources.scenario_Controller_calculatingWaitLbl_txt
            };

            $scope.data = {
                activeRule: null,
                scenarioReady: false,
                scenarioTypes: null,
                executionFrequencies: null,
                operators: null,
                windowOutCols: null,
                windowTableOutCols: null,
                windowTableOutColsCond: null,
                windowTableOutColsAct: null,
                scenarioColumns: null,
                usedConditionColumns: null,
                usedActionColumns: null,
                binUpdateColumns: [], //Used to allow explorer view to save bins
                showSave: true,
                disableSave: false,
                disableSaveAs: false,
                isDirty: false,
                secondaryEditorShowing: false,
                siblingScenarioNames: null
            };

            $scope.counters = {
                totalRecCnt: 0,
                filteredRecCnt: 0,
                popCoveragePct: 1
            };

            $scope.inputColsProps = {
                inputColsDefs: [
                    {
                        id: "label",
                        field: "label",
                        label: "",
                        renderCell: function (object, value, node, options) {
                            var iconClass = object.dataType === 'NUMERIC' ? 'NUMERICIcon' : 'CHARACTERIcon';
                            var nodeStr = '<span class="hasContextMenu"><span class="treeGridIconCont ' + iconClass + '" >' +
                                '&nbsp;</span><span  title="' + object.label + '" >' + object.label + '</span></span>';

                            var usedCol = _.findWhere($scope.data.usedConditionColumns, {id: object.outCol.id});
                            if (usedCol) {
                                dojo.toggleClass(node, "disabledItem");
                            }
                            return dojoModules.domConstruct.toDom(nodeStr);
                        }
                    }
                ],
                inputColsData: [],
                columnNodes: [],
                inputIdProperty: "id",
                inputColsFilter: ''
            };

            $scope.inputTree = {
                inputGridColDef: [
                    $scope.tree({
                        field: "label",
                        label: dojoModules.resources.common_OpenDialog_name_txt,
                        sortable: true,
                        singular: false,
                        collapseOnRefresh: true,
                        //renderCell: $scope.inputTree.inputTreeRenderer,
                        renderCell: function (object, value, node, options) {
                            //Top level grouping node
                            if (!object.parent) {
                                return dojoModules.domConstruct.toDom('<span class="treeGridGroupingLabel">' + object.label + '</span>');
                            }
                            //Field - show icon before label depending on data type
                            else {
                                var iconClass = object.dataType === 'NUMERIC' ? 'NUMERICIcon' : 'CHARACTERIcon';
                                var nodeStr = '<span class="hasContextMenu"><span class="treeGridIconCont ' + iconClass + '" >' +
                                    '&nbsp;</span><span  title="' + object.label + '" >' + object.label + '</span></span>';
                                var usedCol = _.findWhere($scope.data.usedConditionColumns, {name: object.name});
                                if (!usedCol) {
                                    usedCol = _.findWhere($scope.data.usedActionColumns, {name: object.name});
                                }
                                if ($scope.scenario.targetColumn && object.name === $scope.scenario.targetColumn.name) {
                                    usedCol = true;
                                    $scope.inputTree.targetColNode = node;
                                }

                                if (usedCol) {
                                    dojo.toggleClass(node, "disabledItem");
                                }

                                //$scope.inputColsProps.columnNodes[object.name] = node;
                                return dojoModules.domConstruct.toDom(nodeStr);
                            }
                        },
                        shouldExpand: function (row, level, previouslyExpanded) {
                            if (level === 0) {
                                return true;
                            }
                            else {
                                return false;
                            }
                        }

                    })
                ],
                inputColumnsId: "treeId",
                inputColumns: [],
                filterInputColumns: function (item, index, items) {
                    var filterString = $scope.inputColsProps.inputColsFilter.toLowerCase();
                    if (filterString.length < 1) {
                        return !!!item.parent;
                    }
                    else {
                        //if (!item.parent) {
                        var name = item.label.toLowerCase();
                        var matched = name.indexOf(filterString.toLowerCase()) > -1;
                        if (matched) {
                            return true;
                        }
                    }

                    return false;
                },
                inputColsGetChildren: function (store, parent, options) {
                    return store.query({parent: parent.treeId}, options);
                },
                inputColsMayHaveChildren: function (store, parent) {
                    return parent.columns && parent.columns.length > 0 && store.getChildren(parent).length > 0;
                },
                generateInputColumns: function () {
                    this.inputColumns = [];
                    var allVars = [];

                    var windowColumns = {
                        label: dojoModules.resources.decisionTable_windowOutputCols_txt,
                        treeId: '1', parent: null, columnType: $scope.WINDOW_OUTPUT, columns: []
                    };
                    allVars.push(windowColumns);
                    var scenarioColumns = {
                        label: dojoModules.resources.decisionTable_scenarioOutputCols_txt,
                        treeId: '2', parent: null, columnType: $scope.SCENARIO_OUTPUT, columns: []
                    };
                    allVars.push(scenarioColumns);

                    angular.forEach($scope.data.windowTableOutCols, function (inputCol) {

                        var column = angular.copy(inputCol);

                        if (column.columnType === "WO") {
                            column.treeId = windowColumns.columnType + column.id;
                            column.parent = windowColumns.treeId;
                            if (_.findWhere($scope.data.usedConditionColumns, {name: column.name}) !== null) {
                                column.usedInDt = true;
                            }
                            else if ($scope.scenario.targetColumn && column.name === $scope.scenario.targetColumn.name) {
                                column.usedInDt = true;
                            }
                            if (!_.findWhere($scope.scenario.scenarioWindowPartitionKeyColumns, {name: column.name})) {
                                windowColumns.columns.push(column);
                            }
                        }
                        else {
                            column.treeId = scenarioColumns.columnType + column.id;
                            column.parent = scenarioColumns.treeId;
                            scenarioColumns.columns.push(column);
                        }

                        if (!_.findWhere($scope.scenario.scenarioWindowPartitionKeyColumns, {name: column.name})) {
                            allVars.push(column);
                        }
                    });

                    this.inputColumns = $orderBy(allVars, ['-columnType', '-parent', 'dataType', 'label']);
                }
            };

            $scope.previewInfo = {
                startRow: 0,
                pageSize: 200,
                filterCond: $scope.createBlankPreviewFilter(),
                filterColumns: [],
                gridStructure: [],
                rows: []
            };
            $scope.actionInfo = {
                allVars: [],
                menuItems: [],
                trueFalseList: [
                    "True",
                    "False"
                ],
                showActionColumnEditor: false
            };
            $scope.maxCondNbr = 0;
            $scope.explBinData = {
                selectedColumnIdx: null,
                showRebinDialog: false
            };

            //region *** Decision Table dialogs properties ***


            //endregion

            //endregion

            //region *** ViewModel Methods ***

            $scope.switchEditMode = function (newMode) {

                //Don't allow switch to Explorer if using non-and condition
                if (newMode === $scope.EDIT_MODE_EXPLORE && $scope.data.activeRule && $scope.data.activeRule.condition && $scope.data.activeRule.condition.operator.name !== "AND") {
                    messageSvc.showError($scope.resources.scenario_ManualConditionEditor_nonANDWarningMsg_txt);
                    return;
                }

                //Drop incomplete conditions from active rule
                if ($scope.data.activeRule.condition && $scope.data.activeRule.condition.childConditions) {
                    removeIncompleteConditions($scope.data.activeRule.condition, $scope.data.activeRule.condition.childConditions, 0);
                }
                //Derive new condition columns
                conditionVarsFromConditions($scope.data.activeRule.condition);
                //Let current view know the mode is about to change so it can do cleanup
                $scope.$broadcast(vsmCommonConst.SCENARIO_MODE_CHANGE, [newMode]);
                //Change mode
                $scope.scenario.editMode = newMode;
            };

            $scope.filterInputColumns = function (item, index, items) {
                var filterString = $scope.inputColsProps.inputColsFilter.toLowerCase();
                return vsdUtil.gridFilterByAttribute(filterString, item, "label");
            };

            $scope.clearPreviewFilter = function () {
                if ($scope.previewInfo.filterCond) {
                    $scope.previewInfo.filterCond = $scope.createBlankPreviewFilter();
                    getPreviewData();
                }
            };

            $scope.setCounters = function (filtered, popPct, total) {
                $scope.counters = {
                    totalRecCnt: total ? total : $scope.counters.totalRecCnt,
                    filteredRecCnt: filtered,
                    popCoveragePct: popPct ? popPct : filtered / $scope.counters.totalRecCnt
                };
            };

            $scope.showActionColumnEditorDialog = function () {
                actionEditorSvc.setUsedActionColumns($scope.data.usedActionColumns);
                $scope.actionInfo.showActionColumnEditor = true;
            };

            $scope.showOpenScreen = function () {
                navWithConfirm($scope.showOpen);
            };

            $scope.showHomeScreen = function () {
                navWithConfirm($scope.showHome);
            };

            $scope.goToHubScreen = function () {
                navWithConfirm($scope.goToHub);
            };

            $scope.gotoParentWindow = function () {
                navWithConfirm($scope.showWindow, $scope.scenario.scenarioWindowId);
            };

            $scope.showNewWindowDialog = function () {
                navWithConfirm($scope.newWindow);
            };

            $scope.onActionColsUpdate = function () {
                if ($scope.scenario.editMode === $scope.EDIT_MODE_DECISONTABLE) {
                    windowSvc.getOutputTableColumnsWithClassification($scope.scenario.scenarioWindowId, $scope.scenario.executionOrder)
                        .then(function (result) {
                            $scope.data.windowTableOutCols = result;
                        });
                }
                else {
                    $scope.initActions(true);
                }
            };

            $scope.addSavedBinColumn = function (binColIdx, wbc) {
                var existIdx = -1;
                //Replace column in window table output columns so new bins are used
                $scope.data.windowOutCols.splice(binColIdx, 1, wbc);

                //See if column already present
                for (var i = 0; i < $scope.data.binUpdateColumns.length; i++) {
                    var col = $scope.data.binUpdateColumns[i];
                    if (col.id === wbc.id) {
                        existIdx = i;
                        break;
                    }

                }
                //Create a bin save object
                var saveBin = {
                    outputColumnId: wbc.id,
                    bins: wbc.bins
                };

                //If already present, replace it
                if (existIdx >= 0) {
                    $scope.data.binUpdateColumns.splice(existIdx, 1, saveBin);
                }
                //Otherwise, append
                else {
                    $scope.data.binUpdateColumns.push(saveBin);
                }
                setDirty(true);
            };

            $scope.setActiveRule = function (rule) {
                $scope.data.activeRule = rule;
            };

            $scope.cancelSecondaryEditor = function () {
                $scope.data.secondaryEditorShowing = false;
                $scope.data.disableSaveAs = false;
                $scope.$broadcast(vsmCommonConst.CANCEL_SECONDARY_EDITOR);
            };


            //Listen for condition builder change event, which at this scope should be for the preview pane
            $scope.$on(vsmCommonConst.CONDITION_BUILDER_CHANGE, function (evt, id) {
                evt.preventDefault();
                if ($scope.data.scenarioReady && id !== 'dtFilter') {
                    getPreviewData();
                }
            });


            $scope.$on('mousedown', function () {
                return false;
            });

            $scope.setDirty = setDirty;
            $scope.createBlankRule = createBlankRule;
            $scope.countConditions = countConditions;
            $scope.getNonGroupingCondCount = getNonGroupingCondCount;
            $scope.addConditionColumns = addConditionColumns;
            $scope.deleteConditionCol = deleteConditionCol;
            $scope.reorderConditionCol = reorderConditionCol;
            $scope.deleteConditions = deleteConditions;
            $scope.conditionsIdentical = conditionsIdentical;
            $scope.removeIncompleteConditions = removeIncompleteConditions;
            $scope.createBlankRule = createBlankRule;
            $scope.createBlankCondition = createBlankCondition;
            $scope.createBlankConjCondition = createBlankConjCondition;
            $scope.createSimpleCond = createSimpleCond;
            $scope.deleteAction = deleteAction;
            $scope.initActions = initActions;
            $scope.saveScenario = saveScenario;
            $scope.saveScenarioAs = saveScenarioAs;
            $scope.validateScenario = validateScenario;
            $scope.getPreviewData = getPreviewData;
            $scope.clearPreviewData = clearPreviewData;
            $scope.newScenarioForWindowId = newScenarioForWindowId;
            $scope.newDeploymentForWindowId = newDeploymentForWindowId;
            $scope.formatScenarioInputCols = formatScenarioInputCols;
            $scope.isScenarioAutoCol = isScenarioAutoCol;
            //endregion

            //Perform initialization
            processOpenOrNew();
        }]);



});

},
'vsd/scenario/ScenarioSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

//Scenario CRUD Services
angular.module('vsd')
	.factory('scenarioSvc', ['$http','dojoDepends', 'messageSvc',function ($http, dojoDepends, messageSvc) {

        var editControlTypesLabelMap = {
            "BOOLEAN": dojoDepends.resources.scenario_ActionColumnEditor_trueFalseLbl_txt,
            "STRING": dojoDepends.resources.scenario_ActionColumnEditor_expressionLbl_txt,
            "NUMBER": dojoDepends.resources.scenario_ActionColumnEditor_numLbl_txt
        }

		var urlBase = 'action';
		var scenarioSvc = {};

		scenarioSvc.getScenario = function (id) {
			return $http.get(urlBase + '/scenario/' + id).then(function(response) {
                return response.data;
            });
		};

		scenarioSvc.deleteScenario = function (id, version) {
			return $http['delete'](urlBase + '/scenario/' + id + ' / ' + version).then(function(response) {
                return response.data;
            });
		};

		scenarioSvc.saveScenario = function (saveObj) {
			return $http.put(urlBase + '/scenario/', saveObj).then(function(response) {
                return response.data;
            });
		};

		scenarioSvc.getScenarioTypes = function () {
			return $http.get(urlBase + '/scenarioTypes').then(function(response) {
                return _.reject(response.data, function(scenarioType) {
                    return scenarioType.id === -1;
                });
            });
		};

        scenarioSvc.getEntityTypes = function () {
            return $http.get(urlBase + '/entityTypes').then(function(response) {
                return _.reject(response.data, function(entityType) {
                    return entityType.id === -1;
                });
            });
        };

		scenarioSvc.getExecutionFrequencies = function () {
			return $http.get(urlBase + '/executionFrequencies').then(function(response) {
                return response.data;
            });
		};

        scenarioSvc.getCountForCondFilter = function() {

        }

        scenarioSvc.getUniqueCombos = function(requestData,scenarioName) {
            var uri = "./action/getWindowColumnValueCombos/" + scenarioName;
            return $http.post(uri,requestData).then(function(response) {
                return response.data;
            });
        }

        scenarioSvc.getRecordCountForConditionFilter = function(cond) {
            var url = urlBase + '/getRecordCountForConditionFilter';
            return $http.post(url,cond).then(function(response) {
                return response.data;
            });
        }

        scenarioSvc.getEditControlTypes = function() {
            var url = urlBase + "/editControlTypes";
            return $http.get(url, {cache:true}).then(function(response) {
                var retVals = [];
                //Attach label to each entry based on lookup table
                angular.forEach(response.data,function(editControType) {
                    if(editControlTypesLabelMap[editControType.name.trim()]){
                        editControType.label = editControlTypesLabelMap[editControType.name.trim()];
                        retVals.push(editControType);
                    }
                });
                return retVals;
            })
        }

		scenarioSvc.getUniqueValues = function (column,whereClause) {
			var uri = "./action/getUniqueValuesSingleColumn/" + column + '/0';
			return $http.post(uri,{whereClause: whereClause}).then(function(response) {
				return response.data;
			});
		};

        scenarioSvc.getStatsForScenario = function(rules) {
            var url = urlBase + "/getStatsForScenario";
            return $http.post(url,{rules: rules}).then(function(response) {
                return response.data;
            });
        }

        scenarioSvc.getSummaryStatsForInputColumn = function(colName) {
            var url = urlBase + "/summaryStats/" + colName;
            return $http.get(url).then(function(response){
                return response.data.summaryStats;
            });
        },

        scenarioSvc.createDecisionTreeAuto = function(targetColumn,targetValue,condCols,lasrOpts) {
            var url = urlBase +  '/getDecisionTree',
                reqBody,
                allInputColumns = [],
                nominalColumns = [];

            // Including missing columns will result in an empty tree unless useMissingVar is true
            angular.forEach(condCols, function (col) {
                if (col.distinctCount > 0 || lasrOpts.useMissingVar) {
                    allInputColumns.push(col);
                }
            });

            angular.forEach(allInputColumns, function (col) {
                if(col.dataType === vsmCommonConst.DATA_TYPE_NUMERIC && col.classification == "BINARY") {
                    nominalColumns.push(col);
                }
            });

            reqBody = {
                targetColumn: targetColumn,
                targetValue: targetValue,
                allInputColumns: allInputColumns,
                nominalColumns: nominalColumns,
                lasrDTOptions: lasrOpts
            };

            return $http.post(url,reqBody).then(function(response) {
                return response.data;
            });
        },

        scenarioSvc.getScenarioDependencies =  function (windowId, scenarioCount) {
            var uri = "action/getScenarioDependencies/" + windowId + "/" + scenarioCount;
            return $http.post(uri);
        };


        scenarioSvc.formatScenarioInputCols = function(columns, partitionKeyColumns) {
            var rowNbr = 1,
                me = this;
            //Reformat data for use in left panel
            var tmpArray = [];
            columns.forEach(function (col, partitionKeyColumns) {
                if (!me.isScenarioAutoCol(col,partitionKeyColumns)) {
                    var outRec = {
                        id:           rowNbr,
                        label:        col.label,
                        dataType:     col.dataType ? col.dataType.trim() : null,
                        distinctVals: null,
                        outCol:       col
                    };
                    tmpArray.push(outRec);
                    //Save input column in a map for lookup by id
                    rowNbr++;
                }
            });
            //Sort by data type and label
            tmpArray.sort(function(a,b) {
                if(a.dataType > b.dataType) {
                    return 1;
                }
                else if(a.dataType < b.dataType){
                    return -1;
                }
                else {
                    if(a.label > b.label) {
                        return 1;
                    }
                    else if(a.label < b.label) {
                        return -1;
                    }
                    else{
                        return 0;
                    }
                }
            });

            return tmpArray
        },

        scenarioSvc.isScenarioAutoCol = function(col, partitionKeyColumns) {
            var partKeyCols = partitionKeyColumns || [];
            if ( _.findWhere(partKeyCols, {name: col.name})) {
                return true;
            }
            else {
                return false;
            }
        }

        scenarioSvc.validateAction = function(value) {

        }

        scenarioSvc.allowScenarioOutputColumnDrop = function(col,executionOrder,resources,suppressMsg) {
            if ( col.columnType === "SO"  &&
                (col.earliestAssignment === null ||
                 col.earliestAssignment === -1   ||
                 executionOrder <= col.earliestAssignment )) {
                if(!suppressMsg) {
                    var scenOutputNoGo = dojoDepends.string.substitute(resources.decisionTable_scenOuputDropNoInit_remove_txt,[col.name]);
                    messageSvc.showInfo(scenOutputNoGo);
                }
                return false;
            }
            else {
                return true;
            }
        }

        scenarioSvc.populateUniqueCharMergeList = function(inList,isMissingStr) {
            var idx = 1,
                outList = [];
            angular.forEach(inList, function (val) {
                outList.push({
                    rowId: idx++,
                    rowValue:  val
                });
            });
            outList.splice(0,0,{
                rowId: 0,
                rowValue: isMissingStr
            });
            return outList;
        }

        scenarioSvc.showFailedDeleteMsg = function(result,scenarioName) {
            var msgParams;
            var msgTxt;
            if(result.scenariosUsingActionColumn) {
                msgParams = [scenarioName,result.actColName,result.scenariosUsingActionColumn.join(", ")];
                msgTxt = dojoDepends.string.substitute(dojoDepends.resources.scenario_failedScenarioDeleteMsg_txt,msgParams);
            }
            else {
                msgParams = [scenarioName,result.actColName,result.windowsUsingActionColumn.join(", ")];
                msgTxt = dojoDepends.string.substitute(dojoDepends.resources.scenario_failedWindowDeleteMsg_txt,msgParams);
            }
            messageSvc.showError(msgTxt);
        }

		return scenarioSvc;

	}]);




});

},
'vsd/scenario/actionEditor/ScenarioActionColumnEditorCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioActionColumnEditorCtrl', ['$scope',
                                                   '$q',
                                                   'scenarioSvc',
                                                   'windowSvc',
                                                   'dojoModules',
                                                   'vsdUtil',
                                                   'messageSvc',
                                                   'actionEditorSvc',
                                                   'columnNameValidationSvc',
        function ($scope,
                  $q,
                  scenarioSvc,
                  windowSvc,
                  dojoModules,
                  vsdUtil,
                  messageSvc,
                  actionEditorSvc,
                  columnNameValidationSvc) {
            //Private variables
            var allEditControlTypes,
                deleteCols = [],
                addEditCols = [],
                origActionCols,
                title = $scope.resources.scenario_ActionColumnEditor_titleLbl_txt,
                nameValid = true,
                labelValid = true;

            function selectedColCompleteObj() {
                if(!$scope.data.selectedActCol) {
                    return false;
                }
                else if(!$scope.data.selectedActCol.hasOwnProperty("name") ||
                        !$scope.data.selectedActCol.hasOwnProperty("label") ||
                        !$scope.data.selectedActCol.hasOwnProperty("dataType") ||
                        !$scope.data.selectedActCol.hasOwnProperty("editControlType") ||
                        !$scope.data.selectedActCol.hasOwnProperty("columnFormat")) {
                    return false;
                }
                else {
                    return true;
                }
            }

            function processActionColumnResults(actColResults) {
                //Clone action columns for later comparison
                origActionCols = angular.copy(actColResults.columns);
                var scenarioUsageMap = actColResults.scenarioUsageMap;
                angular.forEach(actColResults.columns,function(actCol) {
                    var scenarioColUsage = scenarioUsageMap[actCol.name] || [],
                        windowColUsage = actColResults.windowUsageMap[actCol.name] || [] ;

                    //Remove server generated entries for current scenario
                    scenarioColUsage = _.reject(scenarioColUsage,function(scName) {
                        return scName === $scope.scenario.name;
                    });
                    //Append usage for current scenario only if it's in in memory copy of scenario
                    if(_.findWhere($scope.data.usedActionColumns,{id: actCol.id})) {
                        scenarioColUsage.push($scope.scenario.name);
                    }

                    actCol.scenarioRefs = scenarioColUsage;
                    actCol.windowRefs = windowColUsage;
                });


                //Set action columns scope variables
                $scope.data.actCols = actColResults.columns;
            }

            function getDependencies() {
                $scope.showLoadingMask($scope.resources.app_loading_msg_txt);

                $q.all([
                    windowSvc.getActionColumns($scope.scenario.scenarioWindowId,true),
                    scenarioSvc.getEditControlTypes()

                ]).then(function(results) {
                        //////////////////////////////////////////////
                        //Process action columns results
                        //////////////////////////////////////////////
                        processActionColumnResults(results[0].data);
                        if($scope.data.actCols.length > 0) {
                            $scope.setSelectedColumn(0,true);
                        }
                        //Process control type results
                        allEditControlTypes = _.reject(results[1],function(entry){
			                  return entry.name === "STRING";
		                  });
                        setSelectedEditControlTypes();
                        $scope.hideLoadingMask();
                });
            }

            function setSelectedEditControlTypes() {
                if($scope.data.selectedActCol && $scope.data.selectedActCol.dataType === "CHARACTER") {
                    $scope.data.editControlTypes = _.filter(allEditControlTypes,function(entry){return entry.name.trim() === 'STRING';});
                    $scope.data.selectedActCol.columnFormat = null;
                }
                else {
                    $scope.data.editControlTypes =  allEditControlTypes;
                }
            }

            function actionColChanged() {
                if(!origActionCols || !$scope.data.selectedActCol) {
                    return false;
                }
                else if($scope.data.selectedActColIdx >= origActionCols.length){
                    return true;
                }
                else if(origActionCols[$scope.data.selectedActColIdx].label !== $scope.data.selectedActCol.label           ||
                   origActionCols[$scope.data.selectedActColIdx].name !== $scope.data.selectedActCol.name             ||
                   origActionCols[$scope.data.selectedActColIdx].dataType !== $scope.data.selectedActCol.dataType     ||
                   origActionCols[$scope.data.selectedActColIdx].dataLength !== $scope.data.selectedActCol.dataLength ||
                   origActionCols[$scope.data.selectedActColIdx].columnFormat !== $scope.data.selectedActCol.columnFormat   ||
                   origActionCols[$scope.data.selectedActColIdx].editControlType.name !== $scope.data.selectedActCol.editControlType.name) {
                    return true;
                }
                else {
                    return false;
                }

            }

            function markDirty() {
                $scope.data.changesPending = true;
                $scope.data.title = title + " *";
            }

            function createActionColName(name,existing) {
                for(var i = 0; i < existing.length; i++) {
                    var actVar = existing[i];
                    if(actVar.name === name) {
                        var currNbr = name.match(/\d+$/);
                        if(currNbr && currNbr.length > 0) {
                            currNbr = parseInt(currNbr[0]) + 1;
                            name = "a" + currNbr;
                            createActionColName(name,existing);
                        }
                    }
                }
                return name;
            }

            function processFailedDeletes(delMsgs) {

                    var msg = $scope.resources.scenario_ActionColumnEditor_failedDeleteMsg_txt;
                    msg += "\n";
                    delMsgs.forEach(function(delMsg) {
                        msg += $scope.resources.scenario_ActionColumnEditor_failedDeleteMsg_columnLbl_txt + "  " + delMsg.columnName;
                        if(delMsg.usingWindows.length  > 0) {
                            msg += ", " + $scope.resources.scenario_ActionColumnEditor_failedDeleteMsg_windowsLbl_txt + "  ";
                            delMsg.usingWindows.forEach(function(usingWindowName) {
                                msg += usingWindowName + ",";
                            });
                            msg = vsdUtil.dropRight(msg,1);
                        }

                        if(delMsg.usingScenarios.length  > 0) {
                            msg += ", " + $scope.resources.scenario_ActionColumnEditor_failedDeleteMsg_scenariosLbl_txt + "  ";
                            delMsg.usingScenarios.forEach(function(usingScenarioName) {
                                msg += usingScenarioName + ",";
                            });
                            msg = vsdUtil.dropRight(msg,1);
                        }
                    });

                    //Re-add the columns that failed delete back to grid
                    delMsgs.forEach(function(delMsg) {
                        var currCol = _.findWhere(deleteCols, {name: delMsg.columnName});
                        $scope.data.actCols.push(currCol);
                    });
                    //Remove loading mask
                    $scope.hideLoadingMask();
                    //Show error message
                    messageSvc.showError([msg]);

            }

            function applyAddEditChanges() {
                windowSvc.addUpdateActionColumns($scope.scenario.scenarioWindowId,addEditCols).then(
                    function(results) {
                        var newFocusIdx = -1;
                        var saveFailed = _.chain(results).pluck("success").contains(false).value();
                        if(saveFailed) {
                            var errorCols = [];
                            var msg = $scope.resources.scenario_ActionColumnEditor_serverDupes_txt + ":  ";
                            angular.forEach(results,function(result) {
                                if(!result.success) {
                                    msg += "'" + result.column.label + "',";
                                    errorCols.push(result.column);
                                }
                            });
                            msg = vsdUtil.dropRight(msg,1);
                            //Get the new list of action columns so we can tell the user when the problem is fixed
                            windowSvc.getActionColumns($scope.scenario.scenarioWindowId,true)
                                .then(function(results) {
                                    //Do default processing or action columns
                                    processActionColumnResults(results.data);
                                    //Overlay columns in list that have errors
                                    var colIdx = 0;
                                    angular.forEach($scope.data.actCols,function(actCol) {
                                        var matchCol = _.findWhere(errorCols,{id: actCol.id});
                                        if(matchCol) {
                                            matchCol.scenarioRefs = actCol.scenarioRefs;
                                            newFocusIdx = colIdx;
                                        }
                                        colIdx++;
                                    });
                                    //If there were no update columns, will focus on first new column
                                    if(newFocusIdx === -1) {
                                        newFocusIdx =   $scope.data.actCols.length;
                                    }
                                    //Add new error columns  to list
                                    angular.forEach(errorCols,function(errorCol) {
                                        if(!errorCol.id) {
                                            $scope.data.actCols.push(errorCol);
                                        }
                                    });
                                    //Set focus on the first error column
                                    $scope.setSelectedColumn(newFocusIdx,true);
                                    //Display failure message
                                    messageSvc.showError(msg);
                                    //Remove loading mask
                                    $scope.hideLoadingMask();
                                });

                        }
                        else {
	                        //Remove loading mask if not in Decision Table - DT will remove it once it has updated its changes
	                         if ( $scope.scenario.editMode !==  "B") {
                                $scope.hideLoadingMask();
                             }

                            //Hide dialog
                            $scope.hideDialog();
                            //Invoke scenario controller function to reinitialize actions menu
                            $scope.onActionColsUpdate();
                        }
                    }
                );
            }

            function columnIsValid() {
                if(!selectedColCompleteObj()) {
                    return true;
                }
                else {
                    return nameValid && labelValid;
                }
            }

            //region *** Scope Data/Functions ***
            $scope.data = {
                changesPending: false,
                actCols: [],
                selectedActCol: null,
                selectedActColIdx: 0,
                unselectedColumns: [],
                title: title,
                editControlTypes: [],
                actNumericFormats: [],
                nameFieldTooltip: null
            };

            $scope.onDataTypeChange = function() {
                if($scope.data.selectedActCol && $scope.data.editControlTypes.length > 0) {
                    setSelectedEditControlTypes();
                    if(!$scope.data.selectedActCol.editControlType ||
                       !_.findWhere($scope.data.editControlTypes, {id: $scope.data.selectedActCol.editControlType.id})) {
                        $scope.data.selectedActCol.editControlType =  $scope.data.editControlTypes[0];
                    }

                    $scope.appendToAddEditCols();
                }
            };

            $scope.setSelectedColumn = function(idx,override) {
                if($scope.data.actCols.length > 0 && (override ||columnIsValid())) {
                    //Set selected column and index
                    $scope.data.selectedActCol = $scope.data.actCols[idx];
                    $scope.data.selectedActColIdx = idx;
                    setSelectedEditControlTypes();
                    $scope.data.unselectedColumns = _.reject($scope.data.actCols,function(col) {return col.name === $scope.data.selectedActCol.name;});
                }
                else if($scope.data.actCols.length === 0) {
                    $scope.data.selectedActCol = null;
                }


                if($scope.data.selectedActCol && !columnIsValid()) {
                    messageSvc.showError([$scope.resources.scenario_ActionColumnEditor_invalidColumnMsg_txt]);
                }
            };

            $scope.appendToAddEditCols = function() {
                if(actionColChanged()) {
                    if($scope.data.selectedActCol.editControlType.name === "BOOLEAN") {
                        $scope.data.selectedActCol.columnFormat = vsdUtil.defaultFormats.none;
                    }
                    if(!_.findWhere(addEditCols,{name: $scope.data.selectedActCol.name})){
                        addEditCols.push($scope.data.selectedActCol);
                        markDirty();
                    }
                }
            };

            $scope.newActionColumn = function(colIdx) {
                //Create column name
                var name = createActionColName("a1",$scope.data.actCols);
                //Create column
                var actCol = {
                    name:  name,
                    label: name,
                    dataType: "NUMERIC",
                    dataLength: 8,
                    earliestAssignment: Number.MAX_VALUE,
                    columnFormat: _.findWhere($scope.commonLists.numericFormats,{name: vsdUtil.defaultFormats.number}).name,
                    editControlType: _.find(allEditControlTypes,function(entry){return entry.name.trim() === 'NUMBER';})
                };
                //Add to array and select variable
                var idx = $scope.data.actCols.length;
                $scope.data.actCols.push(actCol);
                $scope.setSelectedColumn(idx,true);
               //Add column to change list for add
                $scope.appendToAddEditCols();
            };

            $scope.deleteActionColumn = function(colIdx) {
                var delCol = $scope.data.actCols[colIdx];
                //Delete from actCols array
                $scope.data.actCols.splice(colIdx,1);
                //Delete from origActionCols
                if(colIdx < origActionCols.length) {
                    origActionCols.splice(colIdx,1);
                }
                //Add to list to delete from server
                if(delCol.id && delCol.id !== 0) {
                    deleteCols.push(delCol);
                    if(!_.isUndefined(delCol.id)) {
                        markDirty();
                    }
                }
                //Select next column
               $scope.setSelectedColumn(0);

            };

            $scope.validateCurrColLabel = function(value) {
                labelValid = true;
                //If action column not set yet, to early to do validation
                if(!selectedColCompleteObj()) {                    
                    $scope.data.selectedActColValid = true;
                    return {isValid: true};
                }

                if(!value) {
                    msg = $scope.resources.scenario_ActionColumnEditor_invalidNameNull_txt;
                    labelValid = false;
                    return {isValid: false};
                }                
                else if(!vsdUtil.isUniqueEntry(vsdUtil.pluck($scope.data.unselectedColumns,"label"),value)) {
                    var msg = dojoModules.string.substitute($scope.resources.scenario_ActionColumnEditor_dupeNameMsg_txt,
                        [$scope.resources.scenario_ActionColumnEditor_label_txt]);
                    labelValid = false;
                    return {isValid: false, invalidMessage: msg};
                }            
                else {
                    return {isValid: true};
                }
            };

            $scope.validateCurrColName = function(value) {
                nameValid = true;
                var valRes,
                    msg,
                    retObj;

                //If action column not set yet, to early to do validation
                if(!selectedColCompleteObj()) {                    
                    $scope.data.selectedActColValid = true;
                    return {isValid: true};
                }

                valRes = vsdUtil.isValidIdentifier(value,true,32);
                if(columnNameValidationSvc.isReserved(value)){
                    msg = dojoModules.string.substitute($scope.resources.scWindow_columnProperties_reservedName_fmt_txt,[value]);
                    nameValid = false;
                    return {isValid: false, invalidMessage: msg};
                }
                if(!valRes.valid) {
                    switch(valRes.reason) {
                        case "NULL_OR_ZEROLENGTH":
                            msg = $scope.resources.scenario_ActionColumnEditor_invalidNameNull_txt;
                            break;
                        case "MAX_LENGTH":
                            msg = $scope.resources.scenario_ActionColumnEditor_invalidNameLength_txt;
                            break;
                        case "SYNTAX":
                            msg = $scope.resources.scenario_ActionColumnEditor_invalidNameSyntax_txt;
                            break;
                    }

                    nameValid = false;
                    return {isValid: false, invalidMessage: msg};
                }
                else {
                    if(!vsdUtil.isUniqueEntry(vsdUtil.pluck($scope.data.unselectedColumns,"name"),value)) {
                        msg = dojoModules.string.substitute($scope.resources.scenario_ActionColumnEditor_dupeNameMsg_txt,
                                                            [$scope.resources.scenario_ActionColumnEditor_name_txt]);
                        nameValid = false;
                        return {isValid: false, invalidMessage: msg};
                    }
                    else {
                        $scope.data.selectedActColValid = true;
                        return {isValid: true};
                    }
                }
            };

            $scope.applyChanges = function() {
                $scope.showLoadingMask($scope.resources.scenario_savingChangesMsg_text);

                //If deletes exist, apply them first
                if(deleteCols.length > 0) {
                    var delIds = _.pluck(deleteCols,'id');
                    windowSvc.deleteActionColumns($scope.scenario.scenarioWindowId,delIds).then(function(delMsgs) {
                        //Special case - column was used and saved in scenario before delete was applied
                        var failed = _.chain(delMsgs).pluck("status").contains("FAILURE").value();
                        if(failed) {
                            //If there are failed deletes, notify user and add deleted columns back to displayed list
                            processFailedDeletes(delMsgs);
                        }
                        else {
                            applyAddEditChanges();
                        }
                    });
                }
                //If no deletes, just apply add/edit changes
                else {
                    applyAddEditChanges();
                }
            };

            $scope.hideDialog = function () {
                $scope.actionInfo.showActionColumnEditor = false;
                messageSvc.closeMessage();
            };

            $scope.$watch('actionInfo.showActionColumnEditor',
                function () {
                    if ($scope.actionInfo.showActionColumnEditor) {
                        origActionCols = null;
                        deleteCols = [];
                        addEditCols = [];
                        $scope.data.changesPending =  false;
                        $scope.data.actCols =  [];
                        $scope.data.selectedActCol = null;
                        nameValid = true;
                        labelValid = true;
                        $scope.data.selectedActColIdx =  0;
                        $scope.data.title =  title;
                        $scope.data.usedActionColumns = actionEditorSvc.getusedActionColumns() || [];
                        // Exclude Date and Datetime action formats
                        $scope.data.actNumericFormats = _.reject($scope.commonLists.numericFormats, function(fmt) {
                            return (fmt.name.trim() === vsdUtil.defaultFormats.date || fmt.name.trim() === vsdUtil.defaultFormats.datetime);
                        });
                        getDependencies();

                    }
                });

            $scope.columnIsValid = columnIsValid;
            //endregion



        }]);


});

},
'vsd/scenario/actionEditor/actionEditorSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('actionEditorSvc', function () {
		var usedActionColumns = [];

		return {
			setUsedActionColumns:    function (c) {
                usedActionColumns = c;
                return this;
			},
            getusedActionColumns:    function () {
				return usedActionColumns;
			},
            clearSettings: function() {
                usedActionColumns = [];

            }
		};
	});

});

},
'vsd/scenario/actionEditor/actionInput':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';
/**
 * @description
 *
 *  This directive creates a widget that allows the user to edit Scenario "Action" column values.
 *  The type of widget and constraints used are based on the action type and format
 *
 *  Scope variables:
 *      action: the action column object
 *      editControlLists: collection containing action entry value lists (e.g True/False)
 *      changeHandler: action input change handler
 *      readOnly: action value renders as non-editable text
 */
angular.module('vsd')
    .directive('actionInput', ['dojoModules', '$timeout', 'vsdUtil', 'localeSvc',
        function (dojoModules, $timeout, vsdUtil, localeSvc) {
            return {
                restrict: 'E',
                replace: true,
                templateUrl: 'vsd/scenario/actionEditor/actionInput.html',
                scope: {
                    "action": "=",
                    "editControlLists": "=",
                    "changeHandler": "&",
                    "readOnly": "="
                },
                controller: ['$scope', '$element', '$attrs', function ($scope, $element, $attrs) {
                    $scope.resources = dojoModules.resources;
                    $scope.currencyCode = localeSvc.getServerCurrencyCode();
                    $scope.editControlType = null;
                    $scope.numberType = "decimal";
                    $scope.invalidMsg = null;
                    $scope.numberConstraints = {};
                    $scope.view = {
                        numberValue: 0,
                        formattedString: ""
                    };
                    $scope.isReadOnly = ($attrs.readOnly && $scope.readOnly === true) ? true : false;


                    $scope.$watch('action', function (value) {
                        updateActionInfo();
                    }, true);   // use deep watch to detect action property changes

                    // Change handler for input widgets
                    $scope.actionValueChg = function () {
                        if ($scope.editControlType === "NUMBER") {
                            // Update actionText with new widget value converted into a String
                            $scope.action.actionText = $scope.view.numberValue || '';
                        }
                        if ($attrs.changeHandler) {
                            $scope.changeHandler();
                        }
                    };

                    $scope.getShowWidgetClass = function(){

                        if(!($scope.$root.showWidgetMap)){
                            return false;
                        }

                        return !$scope.isReadOnly ;
                    }


                    $scope.shouldShowActionWidget = function(action){
                        if($scope.$root.showWidgetMap) {
                            if($scope.isReadOnly){
                                return false;
                            }

                            var val = false;

                            try{
                                val = $scope.$root.showWidgetMap['action_id_'+ action.id] ;
                            }catch(err){
                                console.log(err);
                            }

                            val =  val ?  val : false;
                            return    val;
                        }
                        else{
                            return !($scope.isReadOnly);
                        }
                    }

                    $scope.showWidget = function(widgetId){

                        $scope.$root.showWidgetMap[widgetId] = true;
                    }

                    // Update the scope objects needed by the UI based on the action object's properties
                    function updateActionInfo() {
                        // Assign numeric constraints based on number type and format
                        // Convert our actionText string into a javascript Number as required for use with NumberTextBox/CurrencyTextBox
                        if ($scope.action) {
                            $scope.editControlType = $scope.action.lhsColumn.editControlType.name.trim();
                            if ($scope.editControlType === "BOOLEAN") {
                                if ($scope.action.actionText !== "0" && $scope.action.actionText !== "1") {
                                    $scope.action.actionText = "0";
                                }
                                var boolObj = _.findWhere($scope.editControlLists.trueFalse, {id: $scope.action.actionText});
                                $scope.view.formattedString = (boolObj) ? boolObj.name : $scope.action.actionText;
                            }
                            else if ($scope.editControlType === "NUMBER") {
                                $scope.invalidMsg = $scope.resources.scenario_conditionEditor_nonNumericContentErrMsg_txt;
                                switch ($scope.action.lhsColumn.columnFormat) {
                                    case vsdUtil.defaultFormats.percentage:
                                        $scope.numberConstraints.type = "percent";
                                        // the percent default is 2 decimal places, expand to allow fractional percents
                                        $scope.numberConstraints.places = "0,4";
                                        $scope.invalidMsg = $scope.resources.scenario_ActionColumnEditor_invalidPercentValue_txt;
                                        break;
                                    case vsdUtil.defaultFormats.currency:
                                        $scope.numberConstraints.type = "currency";
                                        // the currency default requires exactly 2 decimal places, we want to allow 0-2
                                        $scope.numberConstraints.places = "0,2";
                                        $scope.invalidMsg = $scope.resources.scenario_ActionColumnEditor_invalidCurrencyValue_txt;
                                        break;
                                    case vsdUtil.defaultFormats.integer:
                                        $scope.numberConstraints.type = "decimal";
                                        $scope.numberConstraints.places = 0;
                                        $scope.invalidMsg = $scope.resources.scenario_ActionColumnEditor_invalidIntegerValue_txt;
                                        break;
                                    default:
                                        $scope.numberConstraints.type = "decimal";
                                        // the decimal default is 3 decimal places, expand to allow maximum range of 0-20
                                        $scope.numberConstraints.places = "0,20";
                                        $scope.invalidMsg = $scope.resources.scenario_ActionColumnEditor_invalidNumericValue_txt;
                                }
                                $scope.view.numberValue = Number($scope.action.actionText);
                                $scope.view.formattedString = vsdUtil.applySASNumericFormat($scope.view.numberValue, $scope.action.lhsColumn.columnFormat );
                            }
                            else {
                                $scope.view.formattedString = $scope.action.actionText;
                            }
                        }
                    }
                }],

                link: function (scope, element, attrs, controller) {
                }
            };
	}]);

});

},
'vsd/scenario/decisionTable/directives/actionCell':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.directive('actionCell', ['dojoModules', 'operatorSvc', '$compile','vsdUtil',
                             function (dojoModules, operatorSvc, $compile, vsdUtil) {

    return {

	    restrict: 'E',
	    replace: true,
	    templateUrl: 'vsd/scenario/decisionTable/directives/actionCell.html',
	    require: '^decisionTable',
	    scope: {
		    "row": "=",
		    "edit": "=",
		    "actionCol": "="
	    },
	    link: function (scope, element, attrs, controller) {
		    scope.resources = dojoModules.resources;
 		     
		     if (scope.row === "" && scope.actionCol.entriesModel)
			     scope.row  = scope.actionCol.entriesModel[0];

		    // watch row value so that a "formattedRow" can be created every time it changes.
		    scope.$watch('row', function(newVal, oldVal){
			    if ( newVal === null ) return;
			    if  (!scope.actionCol.entriesModel) {
				    var numInput = _.isNumber(newVal) ? newVal : vsdUtil.removeFormat(newVal);
				    scope.formattedRow=  vsdUtil.applySASNumericFormat(numInput,scope.actionCol.columnFormat);
			    }
			    else
				    scope.formattedRow= newVal.name;
		    });

		    scope.actionValueChg = function(){
			    scope.row = (this.row == null) ? " " : this.row;
		    };

		    scope.cellChange= function(event,action,actionCol) {
			    if ( event.keyCode == '13') {
				    if(attrs.changeHandler)
					    scope.changeHandler({action:action,col:actionCol});
			    }
		    };

	    }
    }

	}]);

});

},
'vsd/scenario/decisionTable/directives/conditionCell':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.directive('conditionCell', ['dojoModules', 'operatorSvc', '$compile', '$http', '$templateCache', 'vsdUtil',
        function (dojoModules, operatorSvc, $compile, $http, $templateCache ,vsdUtil ) {

        return {
            restrict: 'E',
            replace: true,
	         require: '^decisionTable',
             scope: {
                  "minMax": "@",
                  "row": "=",
                  "edit": "=",
                  "condCol": "=",
                  "rowNum": "="
            },
            link: function (scope, element, attrs, controller) {

	            scope.operCharMenu=null;
	            scope.defaultOperValue=null;
	            scope.resources = dojoModules.resources;


	            scope.operMissing = (scope.row && scope.row.oper && scope.row.oper.name.indexOf('MISSING') > -1 ) ? true: false;
	            var templateUrl= "vsd/scenario/decisionTable/directives/";



	            if ( scope.condCol.classification !== vsmCommonConst.CLASSIFICATION_CONTINUOUS) {

		            operatorSvc.getLogicalOperators('CHAR').then(function (data) {
			            templateUrl += "conditionCharCell.html";

			            if ( scope.row ) {
				            if (!scope.condCol.entriesModel )  {
					            scope.row.formatValue = scope.row.value;
					            if ( scope.condCol.columnFormat  && scope.condCol.dataType == vsmCommonConst.DATA_TYPE_NUMERIC)
						            scope.row.formatValue = vsdUtil.applySASNumericFormat(scope.row.value,scope.condCol.columnFormat) ;
					            }
				            else
					            scope.row.boolFormat = scope.row.name;
			            }
			            if (data.length > 0) {
				            scope.operCharMenu = data;
				            scope.defaultOperValue = scope.operCharMenu[0];
			            }

			            var charLoader =  $http.get(templateUrl, {cache: $templateCache});
			            var promise = charLoader.success(function(html) {
				            element.html(html);
			            }).then(function (response) {
					            element.append($compile(element.contents())(scope));
				            });
		            });

	            }
	            else {
		            operatorSvc.getLogicalOperators('NUM').then(function (data) {
			            templateUrl += "conditionNumCell.html";

			            if (data.length > 0) {
				            scope.operMenu = data;
				            scope.defaultOperValue = scope.operMenu[0];
			            }

			            if  (scope.condCol.entriesModel)
				            scope.row.boolFormat = scope.row.name;
			            else if (scope.condCol.classification !== vsmCommonConst.CLASSIFICATION_CONTINUOUS &&
				            scope.condCol.columnFormat  &&  scope.condCol.dataType == vsmCommonConst.DATA_TYPE_NUMERIC)
				            scope.row.formatValue = vsdUtil.applySASNumericFormat(scope.row.value, scope.condCol.columnFormat) ;
			            else
				            scope.row.formatValue =  scope.row.value;

			            var loader =  $http.get(templateUrl, {cache: $templateCache});
			            var promise = loader.success(function(html) {
				            element.html(html);
			            }).then(function (response) {
					            element.append($compile(element.contents())(scope));
				            });
		            });

	            }

	            scope.$watch("row.oper", function(newVal, oldVal){
		            scope.operMissing=false;
		            if ( scope.condCol.classification === vsmCommonConst.CLASSIFICATION_CONTINUOUS ) {
			            if (  newVal && ( newVal.name.indexOf('MISSING') > -1) )  {
				            scope.operMissing=true;
				            scope.row.value.min= (scope.row.value.min !== null) ? " " : null;
				            scope.row.value.max= (scope.row.value.max !== null) ? " " : null;
			            }
		            }
		            else {
			            if (  newVal &&  newVal.name.indexOf('MISSING') > -1) {
				            scope.operMissing=true;
				            scope.row.value=" ";
		               }
		            }
	            });

	           /* scope.$watch("row", function(newVal, oldVal){
		            if (!newVal) return;
		            if  (scope.condCol.entriesModel)
			            scope.boolFormat = scope.row.name;
		            else if (scope.condCol.classification !== vsmCommonConst.CLASSIFICATION_CONTINUOUS &&
			            scope.condCol.columnFormat  &&
			            scope.condCol.dataType == vsmCommonConst.DATA_TYPE_NUMERIC)
			            scope.row.formatValue = vsdUtil.applySASNumericFormat(scope.row.value,scope.condCol.columnFormat) ;
		            else
			            scope.row.formatValue =  scope.row.value;
	            });*/

	            scope.condSelectChg = function(){
		            scope.row = this.row;
	            };


            }
        }

	}]);

});

},
'vsd/scenario/decisionTable/directives/contextMenu':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
   .directive('contextHeaderMenu',[   'dojoModules',
                                    function (  dojoModules) {

		return {
			restrict: 'A',
			require: '^decisionTable',
			link: function (scope, element, attrs, controller) {
                var menu;
                //Listen for notification to regenerate menu
                scope.$on("generateHeaderMenu",generateMenu);

                //Generate first menu
                generateMenu();

                function generateMenu() {
                    if(menu) {
                        //dojo.destroy(menu);
                    }
                    // header is not in scope when it is NOT a dragged over column - like pop cnt/%
                    if ( scope.header) {
                        menu =  controller.getHeaderMenu(scope.header);
                    }
                    else {
                        menu =  controller.getHeaderMenu({name:attrs.header});
                    }

	                if(menu)
                     menu.bindDomNode(element[0]);
                }
			}
		}
	}])
	.directive('contextRowMenu',[  'dojoModules',
                                 function (  dojoModules) {

			return {
				restrict: 'A',
				require: '^decisionTable',
				link: function (scope, element, attrs, controller) {
					var menu=  controller.getRowMenu(scope.$index);
					if (menu)
						menu.bindDomNode(element[0]);
				}
			}
	}])

	.directive('contextCellMenu',[ 'dojoModules',
                              function ( dojoModules) {

         return {
            restrict: 'A',
            require: '^decisionTable',
            link: function (scope, element, attrs, controller) {
	            var cell = scope.$parent.row[scope.condition.name];
               var menu= controller.getCellMenu( scope.$parent.$index, scope.condition,cell);
	            if (menu)
                  menu.bindDomNode(element[0]);
            }
         }
   }])
;
});

},
'vsd/scenario/decisionTable/directives/decisionTable':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .directive('decisionTable', [ '$compile', '$filter', '$timeout', 'dojoModules', 'scenarioSvc', 'windowSvc', 'vsdUtil' , 'tableSelectionSvc',
        'arrayReorderSvc', 'messageSvc',
        function ($compile, $filter, $timeout, dojoModules, scenarioSvc, windowSvc, vsdUtil, tableSelectionSvc, arrayReorderSvc, messageSvc) {

            var $filter = $filter('filter');

            return {
                restrict: 'E',
                templateUrl: 'vsd/scenario/decisionTable/directives/decisionTable.html',
                replace: true,
                scope: true,
                controller: ['$scope', '$element', '$attrs', function ($scope, $element, $attrs) {

                    //service to manage selection of multiple rows
                    var rowSelectionMgr = tableSelectionSvc.tableSelector();

                    //functions used to determine which menu options are appropriate for the column in regards for moving the column
                    var reorderCondMenuGenerator = arrayReorderSvc.createReorderMenuItems({
                        getMaxIdx: function () {
                            return $scope.dtPrimitives.conditions.length - 1;
                        },
                        getMinIdx: function () {
                            return 0;
                        },
                        defaultCallback: moveCondition
                    });
                    var reorderActionMenuGenerator = arrayReorderSvc.createReorderMenuItems({
                        getMaxIdx: function () {
                            return $scope.dtPrimitives.actions.length - 1;
                        },
                        getMinIdx: function () {
                            return 0;
                        },
                        defaultCallback: moveAction
                    });

                    var editMode = false,
                        editCell = null,
                        allowPreview = true;

                    //$scope.resources = dojoModules.resources;
                    $scope.currentPage = 1;
                    $scope.pageSize = 100;
                    $scope.rowSpanByNumericPresent = 1;

                    $scope.currentRows = [];
                    $scope.newRows = [];

                    function isConditionColumn(col) {
                        return _.findWhere($scope.dtPrimitives.conditions, {id: col.id});
                    }

                    function moveCondition(locations) {
                        // need to do a re-load of all the columns into the scoped array so that it refreshes teh header menus for each of them
                        var tempCols = [];
                        angular.copy($scope.dtPrimitives.conditions, tempCols);
                        $scope.dtPrimitives.conditions.length = 0;
                        //$scope.$apply();
                        var col = angular.copy(tempCols[locations.startIdx]);
                        tempCols.splice(locations.startIdx, 1);
                        tempCols.splice(locations.targetIdx, 0, col);

                        $scope.dtPrimitives.minMaxHdr.length = 0;
                        angular.forEach(tempCols, function (col) {
                            $scope.dtPrimitives.conditions.push(col);
                            if (col.classification === vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                                $scope.dtPrimitives.minMaxHdr.push(dojoModules.resources.decisionTable_minTableSubHeaderLbl_txt + "(>=)");
                                $scope.dtPrimitives.minMaxHdr.push(dojoModules.resources.decisionTable_maxTableSubHeaderLbl_txt + "(<)");
                            }

                        });
                        refreshModel();
                    }

                    function moveAction(locations) {
                        // need to do a re-load of all the columns into the scoped array so that it refreshes teh header menus for each of them
                        var tempCols = [];
                        angular.copy($scope.dtPrimitives.actions, tempCols);
                        $scope.dtPrimitives.actions.length = 0;
                        //$scope.$apply();
                        var col = angular.copy(tempCols[locations.startIdx]);
                        tempCols.splice(locations.startIdx, 1);
                        tempCols.splice(locations.targetIdx, 0, col);

                        angular.forEach(tempCols, function (col) {
                            $scope.dtPrimitives.actions.push(col);
                        });
                        refreshModel();
                    }

                    function reGroupColumn(column) {
                        $scope.showLoadingMask(dojoModules.resources.decisionTable_getCharValues_txt);
                        //Set dialog title
                        $scope.charDropAttrs.title = dojoModules.string.substitute($scope.resources.decisionTable_regroupValuesDlg_title_txt, [column.label]);

                        // since values for each condition exist in a an object that has a property "value" need to do 2 plucks - then only the unique values.
                        var uniqueVals = [];
                        $scope.dtPrimitives.selectedColumn = column;
                        $scope.charDropAttrs.selectedCharValues.length = 0;
                        angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                            if ($filter(uniqueVals, row[column.name].value).length === 0) {
                                var value;
                                if (row[column.name].oper.name.indexOf('MISSING') > -1) {
                                    uniqueVals.push(dojoModules.resources.decisionTable_missingVal_txt);
                                }
                                else {
                                    uniqueVals.push(row[column.name].value);
                                }
                                $scope.charDropAttrs.selectedCharValues.push(row[column.name]);
                            }
                        });
                        // var uniqVals = _.uniq($filter(_.pluck($scope.dtPrimitives.dtModel, column.name ), "value"))
                        scenarioSvc.getUniqueValues(column.name)
                            .then(function (results) {
                                $scope.charDropAttrs.uniqueChars = scenarioSvc.populateUniqueCharMergeList(results.uniqueValues, dojoModules.resources.decisionTable_isMissingVal_txt);
                                $scope.charDropAttrs.totalValueCount = results.totalValueCount;
                                $scope.charDropAttrs.reGroupFlag = true;
                                $scope.charDropAttrs.showCharacterDialog = true;
                                $scope.hideLoadingMask();
                            });
                    }

                    function reBinColumn(column) {
                        $scope.showLoadingMask(dojoModules.resources.decisionTable_getCharValues_txt);
                        //Set dialog title
                        $scope.numericDropAttrs.title = dojoModules.string.substitute($scope.resources.decisionTable_rangeValuesDlg_title_txt, [column.label]);
                        $scope.numericDropAttrs.columnFormat = column.columnFormat;
                        var uniqueVals = [],
                            uniqueValsMap = {},
                            uniqueRowsWithoutCol = {};
                        $scope.dtPrimitives.selectedColumn = column;

                        //Get unique values for all cells in column
                        var copy = angular.copy($scope.dtPrimitives.dtModel);
                        angular.forEach(copy, function (row) {
                            var cell = row[column.name],
                                outCell = {},
                                cellConcatVal = getCellConcatValue(cell);

                            var match = uniqueValsMap[cellConcatVal];
                            if (!match) {
                                // empty cell?
                                if (cell.value.min === " " && cell.value.max === " ") {
                                    return;
                                }
                                else if (cell.value.min === null || cell.value.max === null) {
                                    // this is not a split cell
                                    var parsed = parseNumericCell(cell, column);
                                    cell.singleOperand = parsed.operandStr + "";
                                    if (cell.oper.name.indexOf('MISSING') === -1) {
                                        if (cell.value.min) {
                                            cell.value.min = cell.singleOperand;
                                        }
                                        else {
                                            cell.value.max = cell.singleOperand;
                                        }
                                    }

                                }
                                else { // this is a split cell
                                    cell.value.max = vsdUtil.parseNumeric(cell.value.max) + "";
                                    cell.value.min = vsdUtil.parseNumeric(cell.value.min) + "";
                                }

                                uniqueValsMap[cellConcatVal] = cellConcatVal;
                                uniqueVals.push(cell);
                            }
                            //Keep track of how many unique rows there are without this column
                            trackUniqueRow(row, uniqueRowsWithoutCol, column.name);
                        });
                        //Sort unique values
                        uniqueVals.sort(function (a, b) {
                            var compA = $scope.numericGetMaxSortValue(a.value, a.oper),
                                compB = $scope.numericGetMaxSortValue(b.value, b.oper);

                            return compA - compB;

                        });

                        //Set rule counts
                        $scope.numericDropAttrs.currRuleCountDisp = vsdUtil.applySASNumericFormat($scope.dtPrimitives.dtModel.length, vsdUtil.defaultFormats.integer);
                        $scope.numericDropAttrs.newRuleCountDisp = $scope.numericDropAttrs.currRuleCountDisp;
                        var baseCount = 0;
                        for (var key in uniqueRowsWithoutCol) {
                            baseCount++;
                        }
                        $scope.numericDropAttrs.baseRuleCount = baseCount;
                        //Retrieve summary statistics for column
                        scenarioSvc.getSummaryStatsForInputColumn(column.name)
                            .then(function (result) {
                                function applyFormat(num) {
                                    if (vsdUtil.isDateFormat(column.columnFormat)) {
                                        return vsdUtil.getFormattedDateString(num);
                                    }
                                    if (vsdUtil.isDateTimeFormat(column.columnFormat)) {
                                        return vsdUtil.getFormattedDateTimeString(num);
                                    }
                                    return vsdUtil.applySASNumericFormat(num);
                                }
                                //Set formatted stats
                                $scope.numericDropAttrs.sumStats = {
                                    min: applyFormat(result.min),
                                    max: applyFormat(result.max),
                                    mean:applyFormat(result.mean),
                                    q2: applyFormat(result.q2)
                                };
                                //Show dialog
                                $scope.numericDropAttrs.showNumericDialog = true;
                                //Assign to scope variable
                                $scope.numericDropAttrs.outNumericRanges = [];
                                $scope.numericDropAttrs.inNumericRanges = uniqueVals;
                                $scope.hideLoadingMask();
                            });


                        function getCellConcatValue(cell) {
                            if (cell.value.min && cell.value.max) {
                                return vsdUtil.parseNumeric(cell.value.min) + "-" + vsdUtil.parseNumeric(cell.value.max);
                            }
                            else if (cell.value.min) {
                                return cell.oper.name + " " + cell.value.min;
                            }
                            else if (cell.oper) {
                                return cell.oper.name + " " + cell.value.max;
                            }
                            else {
                                return " " + cell.value.max;
                            }
                        }

                        function trackUniqueRow(row, uniqueRows, excludeColName) {
                            var rowKey = "";
                            angular.forEach($scope.dtPrimitives.conditions, function (condCol) {
                                if (condCol.name !== excludeColName) {
                                    var cell = row[condCol.name];
                                    if (condCol.classification === vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                                        rowKey += getCellConcatValue(cell);
                                    }
                                    else {
                                        rowKey += cell.value;
                                    }
                                }
                            });
                            uniqueRows[rowKey] = rowKey;
                        }
                    }


                    function refreshModel() {
                        var tempRows = angular.copy($scope.dtPrimitives.dtModel);
                        $scope.dtPrimitives.dtModel.length = 0;
                        $scope.safeApply();
                        for (var i = 0; i < tempRows.length; i++) {
                            tempRows.rowNum = i;
                            //$scope.dtPrimitives.dtModel[i].__execOrder = i+1;
                            // tempRows.push($scope.dtPrimitives.dtModel[i]);
                        }
                        $scope.setDirty(true);  //ScenarioCtrl function
                        $timeout(function () {
                            $scope.dtPrimitives.dtModel = tempRows;
                        });

                        //	 $scope.safeApply();

                    }


                    function hasConditions() {
                        if ($scope.dtPrimitives.conditions.length > 0 && $scope.dtPrimitives.conditions[0].__temp !== "") {
                            return true;
                        }
                        else {
                            return false;
                        }
                    }

                    function parseNumericCell(cell, columnInfo) {
                        var location = "both",
                            value;
                        //Determine if min and max both populated (split cell)
                        if (cell.value.min && cell.value.min !== null && cell.value.max && cell.value.max !== null) {

                            if (cell.value.min.trim() === "" && cell.value.max.trim() === "") {
                                return {result: true, location: 'empty', min: cell.value.min, max: cell.value.max};
                            }

                            // Remove carriage returns
                            cell.value.min = cell.value.min.replace(/\n|<br\s*\/?>/gi, "");
                            cell.value.max = cell.value.max.replace(/\n|<br\s*\/?>/gi, "");

                            // Validate values
                            var parsedMin = cell.value.min,
                                parsedMax = cell.value.max,
                                minValid = false,
                                maxValid = false;

                            // Get SAS date or datetime numeric value from the formatted date string
                            if (columnInfo.sasDate || columnInfo.sasDateTime) {
                                var minValResults = vsdUtil.valExprConstByDataType(cell.value.min, columnInfo, cell.oper, false, true),
                                    maxValResults = vsdUtil.valExprConstByDataType(cell.value.max, columnInfo, cell.oper, false, true);
                                parsedMin = minValResults.parsedValue;
                                parsedMax = maxValResults.parsedValue;
                            }
                            // Remove format characters to get parsed number (e.g. %, $, etc)
                            else {
                                parsedMin = vsdUtil.removeFormat(cell.value.min);
                                parsedMax = vsdUtil.removeFormat(cell.value.max);
                            }

                            // SAS date and date constants will have string, not numeric representation
                            minValid = (angular.isNumber(parsedMin) || vsdUtil.isSasDateConstant(parsedMin) || vsdUtil.isSasDateTimeConstant(parsedMin));
                            maxValid = (angular.isNumber(parsedMax) || vsdUtil.isSasDateConstant(parsedMax) || vsdUtil.isSasDateTimeConstant(parsedMax));

                            location = (!minValid) ? "min" : location;
                            location = (!maxValid) ? "max" : location;

                            return {result: minValid && maxValid, location: location, min: parsedMin, max: parsedMax};
                        }
                        else {  // This is not a split cell

                            location = cell.value.min !== null ? "min" : "max";
                            value = cell.value.min !== null ? cell.value.min.trim() : cell.value.max.trim();

                            if (value === "" || value === null) {
                                return {result: true, location: 'empty', operandStr: ""};
                            }

                            //remove carriage returns
                            value = value.replace(/\n|<br\s*\/?>/gi, "");

                            var parsedVal = value,
                                valid = false;

                            // Get SAS date or datetime numeric value from the formatted date string
                            if (columnInfo.sasDate || columnInfo.sasDateTime) {
                                var valResults = vsdUtil.valExprConstByDataType(value, columnInfo, cell.oper, false, true);
                                valid = valResults.isValid;
                                parsedVal = valResults.parsedValue;
                            }
                            // Remove format characters to get parsed number (e.g. %, $, etc)
                            else {
                                parsedVal = vsdUtil.removeFormat(value);
                            }

                            // SAS date and date constants will have string, not numeric representation
                            valid = (angular.isNumber(parsedVal) || vsdUtil.isSasDateConstant(parsedVal) || vsdUtil.isSasDateTimeConstant(parsedVal));

                            // Assign the unsplit cell the parsed value
                            if (valid && parsedVal !== "" && parsedVal !== null) {
                                if (cell.value.min !== null && cell.value.min !== undefined) {
                                    cell.value.min = parsedVal;
                                }
                                else {
                                    cell.value.max = parsedVal;
                                }
                                return {result: true, location: location, operandStr: parsedVal};
                            }
                            else {
                                return {result: false};
                            }
                        }
                    }

                    function deleteSelectedRows() {
                        var selectedRows = rowSelectionMgr.getSelectedIdxs();
                        var delRowNums = [];
                        if (selectedRows.length === 0) {
                            $scope.showInfoMessage(dojoModules.resources.decisionTable_noSelectedRows_txt);
                            return;
                        }
                        //Collect all row ids to delete
                        angular.forEach(selectedRows, function (idx) {
                            delRowNums.push($scope.dtPrimitives.dtModel[idx].rowNum);
                        });

                        $scope.dtPrimitives.dtModel = _.reject($scope.dtPrimitives.dtModel, function (row) {
                            return _.contains(delRowNums, row.rowNum);
                        });

                        // if user has a filter applied to the table remove them from the completeModel as well
                        if ($scope.dtPrimitives.filterFlag) {
                            $scope.dtPrimitives.completeModel = _.reject($scope.dtPrimitives.completeModel, function (row) {
                                return _.contains(delRowNums, row.rowNum);
                            });
                        }
                        rowSelectionMgr.clearSelection();
                        $scope.$emit('DECISION_TABLE_SELECT', []);
                        var tempRows = [];
                        for (var i = 0; i < $scope.dtPrimitives.dtModel.length; i++) {
                            $scope.dtPrimitives.dtModel[i].rowNum = i;
                            $scope.dtPrimitives.dtModel[i].__execOrder = i + 1;
                            tempRows.push($scope.dtPrimitives.dtModel[i]);
                        }
                        // $scope.dtPrimitives.completeModel = tempRows;
                        $scope.getTableMetrics(tempRows, null);
                    }

                    function getOperator(operStr) {
                        return $scope.operMap[operStr];
                    }


                    this.getHeaderMenu = function (column) {
                        var menuItem,
                            reorderItem,
                            i;
                        //Based on location in Decision table add appropriate menu items
                        var menu = new dojoModules.Menu({});

                        // NOT decision Table Metrics Header - like Population cnt/%
                        if (column.name != 'dtMetric') {
                            var colLocation = -1,
                                numCond = $scope.dtPrimitives.conditions.length;
                            //Get location of Column
                            for (i = 0; i < $scope.dtPrimitives.conditions.length; i++) {
                                if ($scope.dtPrimitives.conditions[i].name === column.name) {
                                    colLocation = i;
                                }

                            }
                            // if not is condition array chk actions array
                            if (colLocation == -1) {
                                for (i = 0; i < $scope.dtPrimitives.actions.length; i++) {
                                    if ($scope.dtPrimitives.actions[i].name === column.name)
                                        colLocation = i;
                                }
                            }

                            if (isConditionColumn(column)) {
                                if (!column.hasOwnProperty("__temp")) {
                                    //Delete menu item
                                    menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_colRmbMenuItemDeleteColumnLbl_txt, onClick: function (evt) {
                                        $scope.deleteCondition(column);
                                    }});
                                    menu.addChild(menuItem);
                                    //Regroup/re-define range menu item
                                    if (column.columnType === $scope.WINDOW_OUTPUT) {
                                        if (column.classification == vsmCommonConst.CLASSIFICATION_CONTINUOUS)
                                            menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_colRmbMenuItemRedefineRangesLbl_txt,
                                                onClick: function (evt) {
                                                    reBinColumn(column);
                                                }});
                                        else
                                            menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_colRmbMenuItemRegroupValuesLbl_txt,
                                                onClick: function (evt) {
                                                    reGroupColumn(column);
                                                }});

                                        menu.addChild(menuItem);
                                        //Reorder menu items
                                        if ($scope.dtPrimitives.conditions.length > 1) {
                                            reorderItem = arrayReorderSvc.createReorderMenuDojoPopupMenuItem(reorderCondMenuGenerator, colLocation);
                                            menu.addChild(reorderItem);
                                        }
                                    }
                                }
                            }
                            else {
                                if (!column.hasOwnProperty("__temp")) {
                                    //Delete menu item
                                    menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_colRmbMenuItemDeleteColumnLbl_txt,
                                        onClick: function (evt) {
                                            $scope.deleteAction(column);
                                        }});
                                    menu.addChild(menuItem);
                                    if ($scope.dtPrimitives.actions.length > 1) {
                                        //Reorder menu items
                                        reorderItem = arrayReorderSvc.createReorderMenuDojoPopupMenuItem(reorderActionMenuGenerator, colLocation);
                                        menu.addChild(reorderItem);
                                    }
                                }
                            }
                        }

                        menu.startup();

                        return menu;
                    };

                    this.getRowMenu = function (idx) {

                        // if table is empty dont provide content menus
                        if ($scope.dtPrimitives.emptyTableClass != "")
                            return;
                        //Based on location in Decision table add appropriate menu items
                        var menu = new dojoModules.Menu();
                        //Duplicate Row Menu item
                        var menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_rowRmbMenuItemDuplicateRowLbl_txt,
                            onClick: function (evt) {
                                if ($scope.isTableMaxSize())
                                    return;
                                var newRow = {};
                                angular.copy($scope.dtPrimitives.dtModel[idx], newRow);
                                $scope.dtPrimitives.dtModel.splice(idx, 0, newRow);
                                refreshModel();
                            }});
                        menu.addChild(menuItem);

                        menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_rowRmbMenuItemDeleteSelectedRowLbl_txt,
                            onClick: function () {
                                deleteSelectedRows();
                            }
                        });
                        menu.addChild(menuItem);
                        menu.startup();

                        return menu;
                    }

                    this.getCellMenu = function (idx, cond, cell) {

                        // if table is empty dont provide content menus
                        if ($scope.dtPrimitives.emptyTableClass != "")
                            return;
                        //Based on location in Decision table add appropriate menu items
                        var menu = new dojoModules.Menu(),
                            menuItem,
                            splits;
                        //Add Split Option
                        if (cond && cond.classification === vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                            if (cell.value.max == null || cell.value.min == null) {
                                var menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_cellRmbMenuItemSplitCellLbl_txt,
                                    onClick: function (evt) {
                                        var newRow = {};
                                        if (cell.value.max != null) {
                                            // Not sure why this was being done, but it breaks functionality when formatted values contain spaces
                                            //splits = cell.value.max.split(" ");
                                            //cell.value.max = splits[splits.length - 1];
                                            cell.value.max = cell.value.max.trim();
                                            cell.value.min = '0';
                                        }
                                        else {
                                            // Not sure why this was being done, but it breaks functionality when formatted values contain spaces
                                            //splits = cell.value.min.split(" ");
                                            //cell.value.min = splits[splits.length - 1];
                                            cell.value.min = cell.value.min.trim();
                                            cell.value.max = '0';
                                        }
                                        cell.colSpan = 1;
                                        cell.oper = null;
                                        refreshModel();
                                    }});
                                menu.addChild(menuItem);

                            }
                            else {
                                var menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_cellRmbMenuItemMergeCellsLbl_txt,
                                    onClick: function (evt) {
                                        var newRow = {};
                                        if (cell.value.max && cell.value.max.length > 0) {
                                            cell.oper = getOperator("<");
                                            cell.value.min = null;
                                        }
                                        else {
                                            cell.oper = getOperator(">=");
                                            cell.value.max = null;
                                        }
                                        cell.colSpan = 2;
                                        refreshModel();
                                    }});
                                menu.addChild(menuItem);

                            }

                        }

                        //Duplicate row
                        var menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_rowRmbMenuItemDuplicateRowLbl_txt,
                            onClick: function (evt) {
                                if ($scope.isTableMaxSize())
                                    return;

                                var newRow = {};
                                angular.copy($scope.dtPrimitives.dtModel[idx], newRow);
                                $scope.dtPrimitives.dtModel.splice(idx, 0, newRow);
                                refreshModel();
                            }});
                        menu.addChild(menuItem);
                        //Delete Row Menu item
                        /* menuItem = new dojoModules.MenuItem({label: "Delete Row", onClick: function(evt) {
                         $scope.dtPrimitives.dtModel.splice(idx,1);
                         var tempRows = [];
                         for(var i=0; i < $scope.dtPrimitives.dtModel.length; i++) {
                         $scope.dtPrimitives.dtModel[i].rowNum=i;
                         tempRows.push($scope.dtPrimitives.dtModel[i]);
                         }
                         $scope.getTableMetrics(tempRows, null);
                         }});
                         menu.addChild(menuItem);*/

                        menuItem = new dojoModules.MenuItem({label: dojoModules.resources.decisionTable_rowRmbMenuItemDeleteSelectedRowLbl_txt,
                            onClick: function () {
                                deleteSelectedRows();
                            }
                        });
                        menu.addChild(menuItem);
                        menu.startup();

                        return menu;
                    }


                    //Check to see if dropped column is valid for an CONDITION column
                    $scope.acceptCondDrop = function (source, cols) {
                        var goodNodes = true;
                        // Make sure we are in the Decision Table View and we are viewing the Table
                        if (!$scope.dtPrimitives || $scope.dtPrimitives.view !== 'D')
                            goodNodes = false;
                        else {
                            angular.forEach(cols, function (node) {
                                if (node.custNodeData.columnType !== $scope.SCENARIO_OUTPUT && node.custNodeData.columnType !== $scope.WINDOW_OUTPUT) {
                                    goodNodes = false;
                                }
                            });
                        }
                        return goodNodes;
                    };

                    //Check to see if dropped column is valid for an ACTION column
                    $scope.acceptActionDrop = function (source, cols) {
                        var goodNodes = true;
                        angular.forEach(cols, function (node) {
                            if (node.custNodeData.columnType !== $scope.SCENARIO_OUTPUT) {
                                goodNodes = false;
                            }
                        });
                        return goodNodes;
                    };

                    //Column dropped on ACTION DND space
                    $scope.onActionDrop = function (source, cols, copy) {

                        angular.forEach(cols, function (node) {
                            dojoModules.domClass.add(node, 'disabledItem');
                            var col = node.custNodeData;
                            // has the column already been added to the Actions?
                            if (_.findWhere($scope.dtPrimitives.actions, {name: col.name}) !== undefined) {
                                $scope.showInfoMessage(col.label + dojoModules.resources.decisionTable_colPreviouslyAdded_txt);
                                return;
                            }
                            var newCol = {};
                            angular.copy($scope.newAction, newCol);
                            newCol = node.custNodeData;
                            newCol.dropNodeId = node.id;
                            $scope.addAction(newCol, node);
                        });

                    };

                    //Column dropped on CONDITION DND space
                    $scope.onConditionDrop = function (source, cols, copy) {

                        angular.forEach(cols, function (node) {
                            var col;
                            col = node.custNodeData.outCol;
                            // scenario output col dropped?
                            if (!col)
                                col = node.custNodeData;

                            //Check to see if the scenario output col they are trying to drop as a condition is valid.
                            if (!scenarioSvc.allowScenarioOutputColumnDrop(col, $scope.scenario.executionOrder, dojoModules.resources)) {
                                return;
                            }


                            dojoModules.domClass.add(node, 'disabledItem');

                            // has the column already been added to the Conditions or Actions?
                            if ((_.findWhere($scope.dtPrimitives.conditions, {name: col.name}) != null ) ||
                                (_.findWhere($scope.dtPrimitives.actions, {name: col.name}) != null )) {
                                $scope.showInfoMessage(col.label + dojoModules.resources.decisionTable_colPreviouslyAdded_txt);
                                return;
                            }
                            // column defined as a target column - can't use in Decision table
                            if ($scope.scenario.targetColumn && $scope.scenario.targetColumn.name === col.name) {
                                $scope.showInfoMessage(col.label + dojoModules.resources.decisionTable_colIsTargetCol_txt);
                                return;
                            }

                            var newCol = {};
                            angular.copy(col, newCol);
                            // capture node id to gray/ungray it out in the input list on the left
                            newCol.dropNodeId = node.id;
                            $scope.addCondition(newCol);
                        });
                    };


                    $scope.rowClick = function ($index, $event) {

                        if (!editMode) {
                            editCell = $event.target;
                            rowSelectionMgr.select($index, $event);
                        }
                        editMode = false;
                        if (allowPreview) {
                            $scope.$emit('DECISION_TABLE_SELECT', rowSelectionMgr.getSelectedIdxs());
                        }
                    };

                    $scope.inRowSelect = function ($index) {

                        return rowSelectionMgr.isSelected($index);
                    };


                    $scope.applyColumnSort = function (columnName, colType) {
                        if (editMode) return;
                        $scope.sortByColumn(columnName, colType);
                        $scope.setDirty(true);

                    };

                    // Handler for Edit Row button in decision table row
                    $scope.editRow = function (row) {
                        var goodData = true,
                            emptyCount = 0,
                            editMode = true;

                        allowPreview = false;  // Don't allow data preview calls while in edit mode
                        $scope.data.disableSave = false;
                        $scope.data.disableSaveAs = false;
                        rowSelectionMgr.clearSelection();
                        row.edit = ( row.edit ) ? false : true;
                        // leaving EDIT mode
                        if (!row.edit) {
                            row.empty = false;
                            $scope.setDirty(true);
                            emptyCount = 0;
                            //if there are no conditions then don't process the population count
                            if (!hasConditions()){
                                return;
                            }
                            // Validate each condition on row
                            angular.forEach($scope.dtPrimitives.conditions, function (cond) {
                                var cell = row[cond.name];
                                // if is is a Window output Col and the oper is NOT a MISSING parse the info.
                                if (cond.classification === vsmCommonConst.CLASSIFICATION_CONTINUOUS &&
                                    (  (!cell.oper) || ( cell.oper && cell.oper.name.indexOf("MISSING") == -1 ))) {
                                    //Numeric Cell?
                                    if (cell.value && cell.value.hasOwnProperty('min')) {
                                        var parsedInfo = parseNumericCell(cell, cond);
                                        if (!parsedInfo.result) {
                                            cell.error = true;
                                            goodData = false;
                                            //Show error message
                                            messageSvc.showError([dojoModules.resources.decisionTable_invalidTableContentsMsg_txt]);
                                        }
                                        else {
                                            cell.error = false;
                                            if (parsedInfo.location === 'both') {
                                                //return formatted info for MIN AND MAX.
                                                cell.value = {
                                                    min: $scope.formatNumberString(parsedInfo.min, cond.columnFormat),
                                                    max: $scope.formatNumberString(parsedInfo.max, cond.columnFormat)
                                                };
                                            }
                                            else if (parsedInfo.location === 'empty') {
                                                cell.colSpan = 1;
                                                cell.value.min = (cell.value.min == null) ? cell.value.min : " ";
                                                cell.value.max = (cell.value.max == null) ? cell.value.max : " ";
                                                cell.oper = null;
                                                emptyCount++;
                                            }
                                            else if (parsedInfo.operandStr === null || parsedInfo.operandStr === undefined) {
                                                row[cond.name].value[parsedInfo.location] = " ";
                                                //set empty Row to true;
                                                emptyCount++;
                                            }
                                            else {
                                                //return formatted info for MIN OR MAX.
                                                cell.value[parsedInfo.location] = $scope.formatNumberString(parsedInfo.operandStr, cond.columnFormat);
                                            }
                                        }
                                    }
                                    //Char Cell
                                    else if (cell.value == null || cell.value.trim() === "") {
                                        emptyCount++;
                                    }
                                }
                                // Categorical column cell
                                // Note: blanks are allowed in Decision Table edit mode
                                else if (cond.classification === vsmCommonConst.CLASSIFICATION_CATEGORICAL &&
                                    cell.value.trim() !== "" &&
                                    (  (!cell.oper) || ( cell.oper && cell.oper.name.indexOf("MISSING") === -1 ))) {
                                    var valResults = vsdUtil.valExprConstByDataType(cell.value, {dataType: vsmCommonConst.DATA_TYPE_CHAR}, cell.oper);
                                    if (valResults.isValid) {
                                        cell.error = false;
                                    }
                                    else {
                                        cell.error=true;
                                        goodData = false;
                                        //Show error message
                                        var msg = dojoModules.resources.decisionTable_invalidTableContentsMsg_txt;
                                        switch (valResults.reason) {
                                            case vsmCommonConst.EXPR_VAL_REASON_DBL_QUOTES:
                                                msg = $scope.resources.scenario_conditionEditor_dblQuotes_text;
                                                break;
                                            case vsmCommonConst.EXPR_VAL_REASON_LITERAL_MAXLEN:
                                                msg = $scope.resources.scenario_conditionEditor_exceedsLiteralMaxLen_text;
                                                break;
                                            case vsmCommonConst.EXPR_VAL_REASON_TOTAL_MAXLEN:
                                                msg = $scope.resources.scenario_conditionEditor_exceedsOperand2MaxLen_text;
                                                break;
                                        }
                                        // show error message
                                        messageSvc.showError([msg]);
                                    }
                                }
                                // Action cell
                                else if (cond.editControlType) {
                                    if (cond.entriesModel) {
                                        cell.boolFormat = cell.name;
                                    }
                                    else {
                                        cell.formattedValue = $scope.formatNumberString(cell.value, cond.columnFormat);
                                    }
                                }
                            });

                            //if all cells are empy in the row then don't process the row as a rule
                            if (emptyCount == $scope.dtPrimitives.conditions.length) {
                                row.empty = true;
                            }

                            //get Metrics for the good rows
                            if (goodData) {
                                var tempRows = angular.copy($scope.dtPrimitives.dtModel);
                                $scope.getTableMetrics(tempRows);
                                allowPreview = true;
                            }
                            else {
                                row.edit = true;
                                $scope.data.disableSave = true;
                                $scope.data.disableSaveAs = true;
                            }
                            //clear prev selected rows
                            rowSelectionMgr.clearSelection();
                        }
                        // Entering EDIT mode
                        else {
                            $scope.data.disableSave = true;
                            $scope.data.disableSaveAs = true;
                            messageSvc.closeMessage();
                            $scope.safeApply();

                            //after the page has changed to edit mode for the row  place focus on the closest input field in relation to where they double clicked
                            $timeout(function () {
                                if (editCell) {

                                    var inputField = (editCell.className.indexOf('condCell') > -1 ) ?
                                        angular.element(editCell).find('textarea') : angular.element(editCell).find('input');
                                    inputField.focus();
                                    editCell = null;
                                }

                            })
                        }

                    };

                }],
                link: function (scope, element, attrs, controller) {
                    var currentTableLength = 0,
                        dtElement = element,
                        handlingResize = false;

                    var dtTable = $('.decisionTableCond');

                    //jQuery Code to allow htm TABLE tag to have a scrolling TBODY
                    var wrapper = $('.wrapper');
                    var contentPane = dtTable.closest(".dijitContentPane");

                    dtTable.floatThead({
                        scrollingTop: 'pageTop',
                        scrollContainer: function ($table) {
                            return $table.closest(".wrapper");
                        }
                    });

                    // After enclosing ContentPane is re-sized, we need to resize the wrapper div that holds the decision table
                    var contentPaneWidget = dojoModules.registry.byId(contentPane.attr('id'));
                    if (!handlingResize && contentPaneWidget && contentPaneWidget.resize) {
                        dojoModules.aspect.after(contentPaneWidget, "resize", resizeWrapper);
                        handlingResize = true;
                    }

                    resizeWrapper();

                    function resizeWrapper() {
                        wrapper.height(contentPane.height() - 60);
                        wrapper.width(contentPane.width() - 10);
                        dtTable.floatThead('reflow');  // realign header to re-sized table
                    }

                    function dtRowToString(row) {
                        var rowStr = "";
                        for (var cell in row) {
                            if (row[cell].value) {
                                if (row[cell].value.hasOwnProperty("min"))
                                    rowStr += "|" + row[cell].value.min + "-" + row[cell].value.max;
                                else
                                    rowStr += "|" + row[cell].value;
                            }
                        }
                        return rowStr;
                    }


                    scope.$on('WINDOW_RESIZE', function (newVal, oldVal) {

                        var contentPane = dtTable.closest(".dijitContentPane");
                        wrapper.height(contentPane.height() - 60);
                        wrapper.width(contentPane.width() - 10);
                        //adjust floated header to be properly positioned again:
                        dtTable.trigger('reflow');
                        scope.safeApply();
                    });

                    scope.$watchCollection('dtPrimitives.dtModel', function (newVal, oldVal) {
                        //change the location of the table in the dom:
                        dtTable.css('marginTop', -2);
                        //adjust floated header to be properly positioned again:
                        dtTable.trigger('reflow');
                        scope.safeApply();

                        // console.log('newVal: ' + newVal.length +  '   existing rows: ' + scope.dtPrimitives.existingRowStrng.length + "  # new rows:" + scope.newRows.length );

                    });

                    scope.$watchCollection('dtPrimitives.conditions', function (newVal, oldVal) {
                        if (oldVal == newVal) return;
                        scope.$broadcast("generateHeaderMenu");
                        scope.safeApply();
                    });

                    scope.$watchCollection('dtPrimitives.actions', function (newVal, oldVal) {
                        if (oldVal == newVal) return;

                        scope.$broadcast("generateHeaderMenu");
                        scope.safeApply();
                    });


                    scope.safeApply = function () {
                        if (!scope.$root.$$phase) {
                            scope.$apply();
                        }
                    };

                }

            };
        }

    ]);

});

},
'vsd/scenario/decisionTable/decisionTableFilters':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
/**
 * Created with IntelliJ IDEA.
 * User: sasdcj
 * Date: 2/17/14
 * Time: 7:45 PM
 * To change this template use File | Settings | File Templates.
 */
angular.module('filters', [])
	.filter("boolFormat", function(){
		return function(input) {

			return input.name;
		};
	})
	/*.service('vsdUtil').filter("actionFmt", function(vsdUtil){
		return function(input, fmt) {
			var numInput = _.isNumber(input) ? input : vsdUtil.removeFormat(input);
			console.log( 'formatting: ' + input + " to: " + vsdUtil.applySASNumericFormat(numInput,fmt));
			return vsdUtil.applySASNumericFormat(numInput,fmt);
		};
	})*/;
});

},
'vsd/scenario/decisionTable/ScenarioCharDropCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioCharDropCtrl', ['$scope', 'scenarioSvc', 'windowSvc','dojoModules','tableSelectionSvc',
                                         'vsdUtil', 'messageSvc',
        function ($scope, scenarioSvc, windowSvc, dojoModules,tableSelectionSvc, vsdUtil, messageSvc) {
            var availValsSelMgr = tableSelectionSvc.tableSelector(),
                groupsSelMgr = tableSelectionSvc.tableSelector(),
                fmtValueCnt,
                fmtTotalValueCnt,
                SRC_TYPE_BEGIN = 1,
                SRC_TYPE_END = 2,
                SRC_TYPE_CONTAINS = 3;


            function selMgrFromLabel(label) {
                if (label == 'LEFT') {
                    return availValsSelMgr;
                }
                else {
                    return groupsSelMgr;
                }
            }

            function formatValueCounts() {
                fmtValueCnt = vsdUtil.applySASNumericFormat($scope.charDropAttrs.uniqueChars.length,vsdUtil.defaultFormats.integer);
                fmtTotalValueCnt = vsdUtil.applySASNumericFormat($scope.charDropAttrs.totalValueCount+1,vsdUtil.defaultFormats.integer);
                $scope.uniqueCharsData.uniqueValsCntMsg =   dojoModules.string.substitute($scope.resources.decisionTable_regroupValuesDlg_valuesShowing_txt,[fmtValueCnt,fmtTotalValueCnt]) ;
            }


            $scope.uniqueCharsData = {
                searchTypes: [
                    {name: $scope.resources.decisionTable_regroupValuesDlg_searchTypeBeginsWith_txt,id: 1},
                    {name: $scope.resources.decisionTable_regroupValuesDlg_searchTypeEndsWith_txt, id: 2},
                    {name: $scope.resources.decisionTable_regroupValuesDlg_searchTypeContainsWith_txt, id: 3}
                ],
                valueFilterText: null,
                valueSearchText: null
            }


            $scope.searchUniqueValues = function() {
                var whereClause = "",
                    searchLikeVal,
                    upStripBegin = "upcase(strip(",
                    upStripEnd = "))";

                //Build like value and where clause
                var
                searchLikeVal = upStripBegin;
                switch($scope.uniqueCharsData.selSearchType.id) {
                    case SRC_TYPE_BEGIN:
                        searchLikeVal += "'" + $scope.uniqueCharsData.valueSearchText + "%'";
                        break;
                    case SRC_TYPE_END:
                        searchLikeVal += "'%" + $scope.uniqueCharsData.valueSearchText + "'";
                        break;
                    case SRC_TYPE_CONTAINS:
                        searchLikeVal += "'%" + $scope.uniqueCharsData.valueSearchText + "%'";
                        break;
                }
                searchLikeVal += upStripEnd;
                whereClause = upStripBegin + $scope.dtPrimitives.selectedColumn.name + upStripEnd + ' LIKE ' + searchLikeVal;
                //Get new list matching filter
                scenarioSvc.getUniqueValues($scope.dtPrimitives.selectedColumn.name,whereClause)
                    .then(function(results) {
                        $scope.charDropAttrs.uniqueChars = scenarioSvc.populateUniqueCharMergeList(results.uniqueValues,dojoModules.resources.decisionTable_isMissingVal_txt);
                        $scope.charDropAttrs.totalValueCount = results.totalValueCount;
                        formatValueCounts();
                        $scope.selectedRow = null;
                        availValsSelMgr.clearSelection();
                        groupsSelMgr.clearSelection();
                    });

            }


	        $scope.addGroupValues = function () {
		        var group={value:"", oper: $scope.operMap['IN'], colSpan: 1};
		        angular.forEach(availValsSelMgr.getSelectedIdxs(), function(rowId){
                    var value = _.findWhere($scope.charDropAttrs.uniqueChars,{rowId: rowId});
			        if (value.rowValue == '-- ' + $scope.operMap['IS MISSING'].name + ' --' )
				        messageSvc.showInfo(dojoModules.resources.decisionTable_regroupValuesDlg_noGroupMissing_txt);
			        else
			         group.value += value.rowValue.trim() + ", "
		        });
		        group.value = group.value.substr(0,group.value.length-2);
		        $scope.selectedChars.push(group);
                availValsSelMgr.clearSelection();
	        };

	        $scope.removeSelectedValues = function() {

                var itr = 0;
		        angular.forEach(groupsSelMgr.getSelectedIdxs(), function(idx){
			        $scope.selectedChars.splice(idx-itr,1);
                    itr++;
		        });
                groupsSelMgr.clearSelection();

	        };

            $scope.removeAllValues = function() {
                $scope.selectedChars = [];
            };

	        $scope.addSelectedValues = function() {
		        var group;
		        angular.forEach(availValsSelMgr.getSelectedIdxs(), function(rowId){
                    var value = _.findWhere($scope.charDropAttrs.uniqueChars,{rowId: rowId});
			        if (value.rowValue == '-- ' + $scope.operMap['IS MISSING'].name + ' --')
				         group={value:" ", oper: $scope.operMap['IS MISSING'], colSpan: 1};
			        else
				        group={value:value.rowValue, oper: $scope.operMap['='], colSpan: 1};
			        $scope.selectedChars.push( group  );
		        });
                availValsSelMgr.clearSelection();
	        };

	        $scope.rowSelect= function(tableLabel,rowId,$event) {
                //Point to correct selection manager based on table source
                var selMgr = selMgrFromLabel(tableLabel);
                //Update selected row array
                var filteredRows = _.filter($scope.charDropAttrs.uniqueChars,function(entry) {
                    return !$scope.uniqueCharsData.valueFilterText || entry.rowValue.toUpperCase().indexOf($scope.uniqueCharsData.valueFilterText.toUpperCase()) !== -1;
                });
                selMgr.select(rowId,$event, _.pluck(filteredRows,"rowId"));
		        $scope.selectedRow = rowId;
	        };

	        $scope.inRowSelect= function(tableLabel,rowId){
                //Point to correct selection manager based on table source
                var selMgr = selMgrFromLabel(tableLabel);
                return selMgr.isSelected(rowId);
	        };

	        $scope.addAllValues = function(){
		        $scope.charDropAttrs.selectedCharValues.length=0;
		        $scope.selectedChars.length=0;

		        angular.forEach($scope.charDropAttrs.uniqueChars, function(value) {
			        var oper = (value == '-- ' + $scope.operMap['IS MISSING'].name + ' --') ? $scope.operMap['IS MISSING'] : $scope.operMap['='] ;
			        var valueObj={value:value.rowValue, oper: oper, colSpan: 1};
			        $scope.selectedChars.push(valueObj);
		        });
		        angular.copy($scope.selectedChars, $scope.charDropAttrs.selectedCharValues);
		      //  $scope.charDropAttrs.showCharacterDialog = false;
	        };

            $scope.addCharValues = function () {
	           $scope.charDropAttrs.selectedCharValues.length=0;
	           angular.copy($scope.selectedChars, $scope.charDropAttrs.selectedCharValues );
	           $scope.charDropAttrs.showCharacterDialog = false;
           };

            $scope.cancelDialog = function () {
	            $scope.selectedChars.length=0;
	            $scope.charDropAttrs.selectedCharValues.length=0;
                if(!$scope.charDropAttrs.reGroupFlag) {
                    dojoModules.domClass.remove($scope.dtPrimitives.selectedColumn.dropNodeId,'disabledItem');
                }
                $scope.charDropAttrs.showCharacterDialog = false;
           };

            $scope.$watch('charDropAttrs.showCharacterDialog',  function (value) {
	           if ( value == true) {
		           $scope.selectedChars= [];
                   var idx = 0;
		           angular.forEach($scope.charDropAttrs.selectedCharValues, function(selVal){
			           if (selVal && selVal.value !== " ")
			               $scope.selectedChars.push(selVal);
		           });
		          // angular.copy($scope.charDropAttrs.selectedCharValues, $scope.selectedChars);
                   formatValueCounts();
		           $scope.selectedRow = null;
		           $scope.selectedGroupRow = null;
                   $scope.uniqueCharsData.selSearchType = $scope.uniqueCharsData.searchTypes[0];
                   $scope.uniqueCharsData.valueSearchText = null;
                   $scope.filteredRows = [];

                   availValsSelMgr.clearSelection();
                   groupsSelMgr.clearSelection();
	           }

           });
        }]);


});

},
'vsd/scenario/decisionTable/ScenarioDecisionTableCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioDecisionTableCtrl',
    ['$scope',
        '$window',
        '$filter',
        'scenarioSvc',
        'windowSvc',
        '$timeout',
        'vsdUtil',
        'Util',
        'dojoModules',
        'arrayReorderSvc',
        'decisionTreeSvc',
        'messageSvc',
        function ($scope, $window, $filter, scenarioSvc, windowSvc, $timeout, vsdUtil, Util, dojoModules, arrayReorderSvc, decisionTreeSvc, messageSvc) {
            var $$window = angular.element($window),
                minHeader = $scope.resources.decisionTable_minTableSubHeaderLbl_txt + "(>=)",
                maxHeader = $scope.resources.decisionTable_maxTableSubHeaderLbl_txt + "(<)";

            var $filter = $filter('filter');

            $scope.filterNotPresent = true;

            $$window.resize(resizeDecisionTable);
            //Clear out decision tree service so we don't retain settings between scenarios
            decisionTreeSvc.clear();

            function resizeDecisionTable() {
                var dtTable = $('.decisionTableCond');
                var contentPane = dtTable.closest(".dijitContentPane");
                var wrapper = dtTable.closest(".wrapper");
                wrapper.height(contentPane.height() - 60);
                wrapper.width(contentPane.width() - 10);
            }


            //region *** FUNCTIONS TO ADD new Columns to Decision Table
            function newCharacterColumn(newCol) {

                newCol.colSpan = 1;
                newCol.rowSpan = 2;

                $scope.dtPrimitives.selectedColumn = newCol;
                if ($scope.dtPrimitives.assistOn) {

                    $scope.showLoadingMask(dojoModules.resources.decisionTable_getCharValues_txt);
                    scenarioSvc.getUniqueValues(newCol.name)
                        .then(function (results) {
                            // columns with classification of CATEGORICAL AND BINARY are treated as CHAR.
                            // IF BINARY need to convert values from NUM to CHAR for correct display
                            if (newCol.dataType === vsmCommonConst.DATA_TYPE_NUMERIC) {
                                if (newCol.editControlType && newCol.editControlType.name.trim() === 'BOOLEAN') {
                                    newCol.entriesModel = $scope.commonLists.trueFalse;
                                }
                                if (!results.uniqueValues) {
                                    // all rows have missing values for this column
                                    results.uniqueValues = ["__MISSING__"];
                                }
                                else if (newCol.columnType == $scope.WINDOW_OUTPUT || newCol.entriesModel) {
                                    for (var i = 0; i < results.uniqueValues.length; i++) {
                                        results.uniqueValues[i] = String(results.uniqueValues[i]);
                                    }

                                }
                            }

                            if (results.uniqueValues && results.uniqueValues.length > 10) {
                                $scope.charDropAttrs.reGroupFlag = false;
                                $scope.charDropAttrs.title = dojoModules.string.substitute(
                                    dojoModules.resources.decisionTable_regroupValuesDlg_title_txt, [newCol.label]);
                                $scope.charDropAttrs.uniqueChars = scenarioSvc.populateUniqueCharMergeList(results.uniqueValues, dojoModules.resources.decisionTable_isMissingVal_txt);
                                $scope.charDropAttrs.totalValueCount = results.totalValueCount;
                                $scope.charDropAttrs.showCharacterDialog = true;

                                $scope.hideLoadingMask();
                            }
                            else {
                                chkForDndDummy('CONDITION');
                                var firstDrop = addPossibleDummyColumn("ACTION");
                                $scope.dtPrimitives.topConditionSpan++;
                                $scope.dtPrimitives.conditions.push(newCol);
                                $scope.charDropAttrs.selectedCharValues.length = 0;
                                if (newCol.columnType === $scope.WINDOW_OUTPUT) {
                                    // all rows have missing values for this column?
                                    if (!results.uniqueValues)
                                        results.uniqueValues = ["__MISSING__"];
                                    else
                                        results.uniqueValues.splice(0, 0, "__MISSING__");
                                }
                                addRows(newCol, results.uniqueValues, false);
                            }
                        });
                }
                else {
                    // Assistant is OFF so just add one row *OR*
                    // User dragged over a scenario output column
                    $timeout(function () {
                        chkForDndDummy('CONDITION');
                        addPossibleDummyColumn("ACTION");
                        $scope.dtPrimitives.conditions.push(newCol);
                        $scope.dtPrimitives.topConditionSpan++;
                        addRows(newCol, [" "], false);
                    });
                }
            }

            function newNumericColumn(newCol) {

                $scope.dtPrimitives.selectedColumn = newCol;
                var ranges = angular.copy($scope.dtNumericAttrs.rangeRows);
                $scope.dtPrimitives.addNumericColFlag = true;
                $scope.dtPrimitives.headerClass =
                    ($scope.dtPrimitives.addNumericColFlag) ? "wrapNum" : "wrapNoNum";
                $scope.dtPrimitives.topConditionSpan = $scope.dtPrimitives.topConditionSpan + 2;
                newCol.colSpan = 2;
                newCol.rowSpan = 1;

                if ($scope.dtPrimitives.assistOn) {
                    $scope.showLoadingMask(dojoModules.resources.decisionTable_getNumStats_txt);
                    windowSvc.getSummaryStats(newCol)
                        .then(function (results) {
                            var stats = results.summaryStats;

                            chkForDndDummy('CONDITION');
                            addPossibleDummyColumn("ACTION");
                            $scope.dtPrimitives.conditions.push(newCol);
                            $scope.dtPrimitives.minMaxHdr.push(minHeader);
                            $scope.dtPrimitives.minMaxHdr.push(maxHeader);
                            if ($scope.dtPrimitives.dtModel.length == 1 && $scope.dtPrimitives.dtModel[0].newCondition === "")
                                $scope.dtPrimitives.emptyTableClass = "";

                            if (stats.mean === 'NaN') {
                                addRows(newCol, [
                                    {min: null, max: null}
                                ], true);
                            }
                            else {
                                var mean = vsdUtil.applySASNumericFormat(Math.ceil(stats.mean), newCol.columnFormat);
                                ranges[0].min = mean;
                                ranges[0].max = null;
                                ranges[1].min = null;
                                ranges[1].max = mean;

                                var newRows = [];
                                addRows(newCol, ranges, true);
                            }
                        });
                }
                else {

                    $timeout(function () {
                        chkForDndDummy('CONDITION');
                        addPossibleDummyColumn("ACTION");
                        // Assistant is OFF so just add one row/column to the existing table *OR*
                        // User dragged over a scenario output column

                        $scope.dtPrimitives.conditions.push(newCol);
                        $scope.dtPrimitives.minMaxHdr.push(minHeader);
                        $scope.dtPrimitives.minMaxHdr.push(maxHeader);

                        // by default we get two entries for ranges - if assist is off then we only need one
                        ranges.pop();
                        ranges.pop();
                        ranges[0] = {min: " ", max: " "};

                        addRows(newCol, ranges, true);
                    });

                }

            }

            function addRows(newCol, results, isNumeric) {
                var newDtModel = [];
                var newRows = [];

                function loadCell(value, col) {
                    var newValue = {value: {}};

                    // if value came from a re-bin or re-define Range dialog box so value already contains correct info to load the cell
                    if (value && value.hasOwnProperty('oper') && value.hasOwnProperty('colSpan'))
                        return value;


                    if (col.classification === vsmCommonConst.CLASSIFICATION_CONTINUOUS)
                        newValue.value = angular.copy(value, newValue.value);
                    else {

                        // column is a Scenario output col
                        if (col.columnType === $scope.SCENARIO_OUTPUT && col.entriesModel)
                            return _.findWhere(col.entriesModel, {id: value});
                        //	 else
                        //		 return value;
                        // }
                        else
                            newValue.value = (value === '__MISSING__') ? "" : value;
                    }


                    newValue.colSpan = 1;
                    if (value && value.hasOwnProperty('min')) {
                        // if both values are null then set oper to MISSING and reset to min=' '
                        if (value.min == null && value.max == null) {
                            newValue.oper = getOperator('IS MISSING');
                            newValue.value.min = " ";
                            newValue.colSpan = 2;
                            return newValue;
                        }
                        if (value.min === " " && value.max === " ") {
                            //Assistant was turned OFF
                            newValue.oper = null;
                            return newValue;
                        }
                        else {
                            newValue.oper = (value.min && value.max) ? null : getOperator('>=');
                            newValue.colSpan = (value.min && value.max) ? 1 : 2;
                            newValue.oper = (value.min) ? getOperator(">=") : getOperator("<");
                        }
                    }
                    else
                        newValue.oper = (value === '__MISSING__') ? getOperator('IS MISSING') : getOperator('=');

                    return newValue;
                }

                if ($scope.dtPrimitives.dtModel.length == 0) {
                    var execOrder = 1;
                    angular.forEach(results, function (value) {
                        var loadedCell = loadCell(value, newCol);
                        addRow(null, newCol, loadedCell, newDtModel, 0, execOrder++);
                    });
                }
                else {
                    newRows = angular.copy($scope.dtPrimitives.dtModel);
                    $scope.dtPrimitives.dtModel.length = 0;
                    var rowNum = 1,
                        execOrder = 1;
                    angular.forEach(newRows, function (row) {
                        angular.forEach(results, function (value) {
                            var loadedCell = loadCell(value, newCol);
                            addRow(row, newCol, loadedCell, newDtModel, rowNum++, execOrder++);
                        });
                    });
                }

                if ($scope.dtPrimitives.assistOn || newDtModel.length > 1)
                    $scope.getTableMetrics(newDtModel, false, null, false);
                else
                    $timeout(function () {
                        $scope.dtPrimitives.dtModel = newDtModel;
                    });

            }

            function addRow(row, col, value, destArray, rowNum, execOrder) {
                var newRow = {};
                if (row) {
                    angular.copy(row, newRow);
                }
                newRow[col.name] = value;
                newRow.rowNum = rowNum;
                newRow.empty = false;
                newRow.edit = false;
                newRow.__execOrder = execOrder;
                if ($scope.dtPrimitives.actions.length == 1 && $scope.dtPrimitives.actions[0].__new === "") {
                    newRow.__new = "";
                }
                if ($scope.dtPrimitives.conditions.length == 1 && !$scope.dtPrimitives.conditions[0].__new === "") {
                    newRow.__new = "";
                }

                if (hasTempColumns("ACTION") || hasTempColumns("CONDITION"))
                    newRow.__temp = " ";
                destArray.push(newRow);
                $scope.setDirty(true);
            }

            function getOperator(operStr) {
                return $scope.operMap[operStr];
            }

            //endregion

            //Function used to Sort Table
            function conditionsComparator(a, b) {

                var compA,
                    compB,
                    cond;
                for (var i = 0; i < $scope.dtPrimitives.conditions.length; i++) {
                    cond = $scope.dtPrimitives.conditions[i];
                    if (cond.classification !== vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                        compA = a[cond.name].value;
                        compB = b[cond.name].value;
                    }
                    else {
                        compA = $scope.numericGetMaxSortValue(a[cond.name].value, a[cond.name].oper);
                        compB = $scope.numericGetMaxSortValue(b[cond.name].value, b[cond.name].oper);
                    }

                    if (compA > compB) {
                        return 1;
                    }
                    else {
                        if (compA < compB) {
                            return -1;
                        }
                    }
                }
                return 0;
            }


            // transforms flat Decision Table model to domain Rule/Event model
            function dtModelToRules(dtModel) {
                var rules = [];
                //inner function to test for an actual value before creating a rule condition
                function chkforValidValue(condCol, val, oper) {
                    var valid = true,
                        chkNum = false;

                    switch (condCol.classification) {

                        case vsmCommonConst.CLASSIFICATION_CONTINUOUS :
                            /* switch(condCol.columnType){
                             //FIXME: once we can figure out if a scenario output col has been previously used we won't need this
                             case $scope.SCENARIO_OUTPUT:
                             chkNum= true;
                             break;
                             case $scope.WINDOW_OUTPUT:*/
                            if (!oper && (!val.min || !val.max)) {
                                valid = false;
                                chkNum = false;
                            }
                            break;
                        /*  }
                         break;*/
                        default:
                            if (!val || val.length == 0) {
                                if (!oper || oper.name.indexOf('MISSING') == -1)
                                    valid = false;
                            }

                            break;
                    }

                    if (chkNum) {
                        val.error = null;
                        if ((val.min === "" || val.min == null) && (val.max === "" || val.max == null)) {
                            valid = false;
                        }
                        else {
                            if (val.min != null && vsdUtil.parseNumeric(val.min) == null) {
                                valid = false;
                                val.error = 'min';
                            }
                            else {
                                if (val.max != null && vsdUtil.parseNumeric(val.max) == null) {
                                    valid = false;
                                    val.error = 'max';
                                }
                            }
                        }
                    }

                    return valid;
                }

                // create a hierarchical rule for each row in the model
                dtModel.forEach(function (row) {
                    var rule = $scope.createBlankRule(rules.length),
                        condCol,
                        colVal,
                        strColVal,
                        cond,
                        subCond,
                        numVal;

                    if (!hasTempColumns("CONDITION")) {
                        rule.condition = $scope.createBlankConjCondition('AND');
                        angular.forEach($scope.dtPrimitives.conditions, function (condCol) {
                            var colVal = null,
                                oper = null;
                            if (condCol.columnType === $scope.SCENARIO_OUTPUT && condCol.classification !== 'CONTINUOUS')
                                colVal = (condCol.entriesModel) ? row[condCol.name].id : String(row[condCol.name].value);
                            /*if ( condCol.columnType === $scope.SCENARIO_OUTPUT && condCol.entriesModel)
                             colVal =  row[condCol.name].id  ;*/
                            else {
                                colVal = row[condCol.name].value;

                            }
                            oper = row[condCol.name].oper;

                            if (chkforValidValue(condCol, colVal, oper)) {

                                //If we're dealing with a numeric column
                                if (colVal.hasOwnProperty("min")) {
                                    var goodVals = 0;
                                    if (colVal.columnType === $scope.SCENARIO_OUTPUT)
                                        colVal = {min: String(colVal.min), max: String(colVal.max)};
                                    if (colVal.min && colVal.max) {
                                        //Create grouping condition for between
                                        cond = $scope.createBlankConjCondition('AND');
                                        //Create sub-condition for lower end of range
                                        if (colVal.min && colVal.min.trim().length > 0) {
                                            numVal = vsdUtil.removeFormat(colVal.min);
                                            subCond = $scope.createSimpleCond(condCol, ">=", numVal);
                                            cond.childConditions.push(subCond);
                                            goodVals++;
                                        }
                                        //Create sub-condition for upper end of range
                                        if (colVal.max && colVal.max.trim().length > 0) {
                                            numVal = vsdUtil.removeFormat(colVal.max);
                                            subCond = $scope.createSimpleCond(condCol, "<", numVal);
                                            cond.childConditions.push(subCond);
                                            goodVals++;
                                        }

                                    }
                                    else {
                                        strColVal = colVal.min ? colVal.min : colVal.max;
                                        numVal = vsdUtil.removeFormat(strColVal);
                                        cond = $scope.createSimpleCond(condCol, oper.name, numVal);
                                        goodVals++;
                                    }
                                    if (goodVals == 0) {
                                        cond = null;
                                    }
                                }

                                else {
                                    if (condCol.columnType === $scope.SCENARIO_OUTPUT && colVal != null)
                                        cond = $scope.createSimpleCond(condCol, "=", String(colVal));
                                    else if (colVal.trim().length > 0)
                                    //Character column
                                        cond = $scope.createSimpleCond(condCol, oper.name, colVal);
                                    else if (oper && oper.name.indexOf('MISSING') > -1)
                                        cond = $scope.createSimpleCond(condCol, oper.name, null);
                                }

                                if (cond) {
                                    if (rule.condition && rule.condition.childConditions) {
                                        rule.condition.childConditions.push(cond);
                                    }
                                    else
                                        rule.condition = cond;
                                }
                            }
                            cond = null;
                        });
                    }

                    if (rule.condition.childConditions.length == 0) {
                        rule.condition = null;
                    }

                    rule.actions = [];
                    var actionCnt = 0;
                    if (!hasTempColumns("ACTION")) {
                        angular.forEach($scope.dtPrimitives.actions,
                            function (action) {

                                var newAction = {};
                                var column = angular.copy(action);
                                newAction.lhsColumn = column;
                                column.entriesModel = null;
                                newAction.executionOrder = actionCnt++;
                                newAction.columnActions = [
                                    {column: column, leftHandSide: true}
                                ];
                                newAction.lhsColumnName = action.name;
                                if (typeof row[action.name] === 'object') {
                                    newAction.actionText = row[action.name].id;
                                }
                                else {
                                    var noFormatText = vsdUtil.removeFormat(row[action.name]);
                                    newAction.actionText = (noFormatText !== null) ? noFormatText.toString() : null;
                                    // newAction.actionText = (noFormatText) ? noFormatText.toString() : null ;
                                }
                                if (newAction.actionText)
                                    rule.actions.push(newAction);
                            });
                    }

                    if (rule.condition) {
                        rules.push(rule);
                    }
                });

                return rules;
            }

            // transforms domain Rule/Event model to flat Decision Table model
            function rulesToDtModel(events) {

                var eventList = [];

                function addConditionToEvent(cond) {
                    var condVal = {value: null},
                        minMax;
                    condVal.colSpan = 1;

                    if (cond.operator && cond.operator.name === 'AND') {
                        minMax = {min: null, max: null};
                        condVal.colSpan = 1;
                        condVal.oper = null;
                        var minCond = getChildConditionByOperatorName(cond.childConditions, vsmCommonConst.LOWER_BOUND_OPER_NAME),
                            maxCond = getChildConditionByOperatorName(cond.childConditions, vsmCommonConst.UPPER_BOUND_OPER_NAME);

                        minMax.min = formatNumberString(minCond.operand2Constant, minCond.operand1ColumnInfo.columnFormat);
                        minMax.max = formatNumberString(maxCond.operand2Constant, maxCond.operand1ColumnInfo.columnFormat);

                        condVal.format = cond.childConditions[0].operand1ColumnInfo.columnFormat;
                        condVal.value = minMax;
                    }

                    else {
                        // find column is condition list as there are addl DecTbl attributes in that object
                        var conditionCol = _.findWhere($scope.dtPrimitives.conditions, {name: cond.operand1ColumnInfo.name});

                        if (conditionCol.classification === vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                            minMax = {min: null, max: null};
                            condVal.colSpan = 2;
                            condVal.oper = (cond.operator) ? $scope.operMap[cond.operator.name] : null;
                            condVal.format = cond.operand1ColumnInfo.columnFormat;
                            // condition may contain a dummy placeholder event
                            if (!condVal.oper && cond.operand2Constant === " ") {
                                condVal.colSpan = 1;
                                condVal.value = {min: " ", max: " "};
                            }
                            else {
                                if (condVal.oper.name.indexOf('<') > -1)
                                    minMax.max = formatNumberString(cond.operand2Constant, cond.operand1ColumnInfo.columnFormat);
                                else
                                    minMax.min = formatNumberString(cond.operand2Constant, cond.operand1ColumnInfo.columnFormat);
                                condVal.value = minMax;
                                if (condVal.oper && condVal.oper.name.indexOf('MISSING') > -1)
                                    condVal.value.min = " ";
                            }
                        }
                        else if (conditionCol.columnType === $scope.SCENARIO_OUTPUT) {
                            if (conditionCol.entriesModel)
                                condVal = _.findWhere(conditionCol.entriesModel, {id: cond.operand2Constant});
                            else
                                condVal.value = (cond.operand2Constant) ? parseFloat(cond.operand2Constant) : null;
                        }

                        else {
                            condVal.value = (cond.operator.name.indexOf('MISSING') > -1) ? " " : cond.operand2Constant;
                            condVal.oper = (cond.operator) ? $scope.operMap[cond.operator.name] : null;
                        }
                    }

                    return condVal;

                    function getChildConditionByOperatorName(childConds, operName) {
                        return _.find(childConds, function (childCond) {
                            return childCond.operator.name === operName;
                        });
                    }
                }

                var eventNum = 1;

                // user dropped action cols or conditions cols but created no events.
                // so create a empty row for them to start with
                if (events.length == 0 &&
                    ( $scope.dtPrimitives.actions.length > 0 || $scope.dtPrimitives.conditions.length > 0)) {
                    var newEvent = {empty: false};

                    angular.forEach($scope.dtPrimitives.conditions, function (condCol) {
                        var oper = (condCol.dataType === vsmCommonConst.DATA_TYPE_CHAR) ? getOperator('=') : null;
                        var cond = {
                            operand1ColumnInfo: condCol,
                            operator: oper,
                            operand2Constant: " "
                        };
                        newEvent[condCol.name] = addConditionToEvent(cond);
                    });
                    angular.forEach($scope.dtPrimitives.actions, function (actCol) {
                        if (actCol.entriesModel)
                            newEvent[actCol.name] = actCol.entriesModel[0];
                        else
                            newEvent[actCol.name] = " ";
                    });

                    eventList.push(newEvent);
                    return eventList;
                }

                angular.forEach(events, function (event) {
                    var newEvent = {empty: false};
                    var conditions = event.condition.childConditions;
                    var actions = event.actions;
                    if (actions.length == 0) {
                        // no actions exist
                        newEvent.edit = false;
                        newEvent.rowNum = 0;

                        newEvent.__temp = "";
                        addPossibleDummyColumn("ACTION");

                    }
                    angular.forEach(actions, function (action) {
                        var actionCol = _.findWhere($scope.dtPrimitives.actions, {name: action.lhsColumn.name});
                        if (actionCol.entriesModel)
                            newEvent[action.lhsColumn.name] = _.findWhere(actionCol.entriesModel, {id: action.actionText});
                        else {
                            if (action.actionText)
                                newEvent[action.lhsColumn.name] = parseFloat(action.actionText);
                            else
                                newEvent[action.lhsColumn.name] = " ";
                        }

                    });

                    var dtActionMap = {};

                    for(var i = 0; i < actions.length; i++){
                        var act = actions[i];
                        dtActionMap[act.lhsColumn.name] = act;
                    }

                    //fill in the rows that don't have the action defined
                    if ($scope.dtPrimitives.actions.length > 0 && actions.length < $scope.dtPrimitives.actions.length) {
                        angular.forEach($scope.dtPrimitives.actions, function (action) {
                            // Check that this is an actual action column first
                            if (!action.hasOwnProperty('name') || action.name === null) {
                                return;
                            }
                            // Assign action a blank value
                            if (actions.length === 0)
                                newEvent[action.name] = "";
                            else {
                                var act = dtActionMap[action.name];
                                if(!act){
                                    newEvent[action.name] = "";
                                }
                            }
                        });
                    }

                    if (conditions.length == 0) {
                        // a single condition exists
                        if (event.condition.operand1ColumnInfo.name) {
                            var cond = event.condition;
                            newEvent[cond.operand1ColumnInfo.name] = addConditionToEvent(cond);
                        }
                        else {
                            // no conditions exist
                            newEvent.edit = false;
                            newEvent.rowNum = 0;
                            newEvent.__temp = "";
                            addPossibleDummyColumn('CONDITION');
                        }
                    }
                    else {
                        angular.forEach(conditions, function (cond) {
                            var colName = cond.operand1ColumnInfo.name;
                            //Chk for Min.Max values if there are then cond will have oper=AND and childConditions
                            if (cond.operator.name === 'AND') {
                                colName = cond.childConditions[0].operand1ColumnInfo.name;
                            }
                            newEvent[colName] = addConditionToEvent(cond);
                        });

                        newEvent = fillTableSparsity(newEvent, $scope.dtPrimitives.conditions);
                    }

                    newEvent.__execOrder = (event.executionOrder) ? event.executionOrder : $scope.dtPrimitives.dtModel.length;
                    eventList.push(newEvent);
                });

                return eventList;
            }

            // Check for Dummy DND placeholder cell
            function chkForDndDummy(type) {
                // if dummy first first there for DND prompt then delete first.
                if ($scope.dtPrimitives.dtModel.length == 1) {
                    if ($scope.dtPrimitives.dtModel[0].newCondition === "") {
                        $scope.dtPrimitives.dtModel.length = 0;
                        if (type === 'CONDITION') {
                            $scope.dtPrimitives.conditions.length = 0;
                        }
                        if (type === 'ACTION') {
                            $scope.dtPrimitives.actions.length = 0;
                        }
                    }
                    else {
                        if (type === 'CONDITION' && $scope.dtPrimitives.conditions.length > 0 && $scope.dtPrimitives.conditions[0].__temp == "") {
                            $scope.dtPrimitives.conditions.length = 0;
                        }
                        else {
                            if (type === 'ACTION' && $scope.dtPrimitives.actions.length > 0 && $scope.dtPrimitives.actions[0].__temp == "") {
                                $scope.dtPrimitives.actions.length = 0;
                            }
                        }
                    }
                }
            }

            //IF condition is dragged over but there is no action column yet have to
            // create a dummy column holder and visa versa for adding action column with no condition
            function addPossibleDummyColumn(type) {
                // No column has been added to conditions or Actions yet - create a dummy column
                var testArray = (type === 'CONDITION') ?
                    $scope.dtPrimitives.conditions : $scope.dtPrimitives.actions;
                if (testArray.length === 1 && testArray[0].__temp === "")
                    return false;
                if (testArray.length === 1 && testArray[0].__new === "") {
                    testArray.length = 0;
                    $scope.dtPrimitives.emptyTableClass = "";
                    testArray.push({
                        __temp: "",
                        dataType: 'CHARACTER',
                        classification: vsmCommonConst.CLASSIFICATION_CATEGORICAL
                    });
                    return true;
                }
                return false;

            }

            // if opening an existing DT then add all saved conditions to DT
            function addConditionColumns(conditions) {
                var newConditions = [];

                // determine condition already exists in array if so discard
                angular.forEach(conditions, function (cond) {
                    var foundCond = _.findWhere($scope.dtPrimitives.conditions,
                        {name: cond.name});
                    if (!foundCond) {
                        //Add classification to each used condition column
                        cond.classification = _.findWhere($scope.data.windowTableOutCols, {id: cond.id}).classification;
                        newConditions.push(cond);
                    }
                });

                // set up default sort indicator on table
                if (newConditions.length > 0) {
                    $scope.dtPrimitives.sortInfo = {};
                    $scope.dtPrimitives.sortInfo[newConditions[0].name] = 'D';
                }

                angular.forEach(newConditions, function (cond) {

                    // determine the column Type
                    var foundInputCol = _.findWhere($scope.inputTree.inputColumns,
                        {name: cond.name});
                    cond.columnType = foundInputCol.columnType;

                    if (cond.editControlType && cond.editControlType.name.trim() === 'BOOLEAN') {
                        cond.entriesModel = $scope.commonLists.trueFalse;
                    }

                    if (cond.classification !== vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                        cond.rowSpan = 2;
                        cond.colSpan = 1;
                        $scope.dtPrimitives.topConditionSpan++;
                    }
                    else {
                        $scope.dtPrimitives.minMaxHdr.push(minHeader);
                        $scope.dtPrimitives.minMaxHdr.push(maxHeader);
                        $scope.dtPrimitives.addNumericColFlag = true;
                        $scope.dtPrimitives.topConditionSpan = $scope.dtPrimitives.topConditionSpan + 2;
                        cond.colSpan = 2;
                        cond.rowSpan = 1;
                    }
                });

                $scope.dtPrimitives.headerClass =
                    ($scope.dtPrimitives.addNumericColFlag) ? "wrapNum" : "wrapNoNum";


                return newConditions;
            }

            // if user modifies the the DT need to rebuild it
            function buildTable() {

                // sub-function of build table to add action cells
                function addActionCells(row) {
                    if ($scope.dtPrimitives.actions.length > 0) {
                        angular.forEach($scope.dtPrimitives.actions,
                            function (action) {
                                row[action.name] = "";
                            });
                    }
                    return row;
                }

                // sub-function of build table to add conditions cells
                function addConditionToTable(condValues, condCol, tempModel) {
                    if (tempModel.length == 0) {
                        angular.forEach(condValues, function (newVal) {
                            var row = {};
                            row[condCol.name] = newVal;
                            row = addActionCells(row);
                            tempModel.push(row);
                        });
                    }
                    else {
                        var newRows = angular.copy(tempModel);
                        tempModel.length = 0;
                        angular.forEach(newRows, function (row) {
                            angular.forEach(condValues, function (newVal) {
                                var newObjVal = angular.copy(newVal);
                                var newRow = angular.copy(row);
                                newRow[condCol.name] = newObjVal;
                                newRow = addActionCells(newRow);
                                tempModel.push(newRow);
                            });
                        });
                    }
                    return tempModel;
                }

                if (($scope.dtPrimitives.conditions.length == 0 && $scope.dtPrimitives.actions.length == 1 && $scope.dtPrimitives.actions[0].__temp == "" ) ||
                    ($scope.dtPrimitives.actions.length == 0 && $scope.dtPrimitives.conditions.length == 1 && $scope.dtPrimitives.conditions[0].__temp == "" )) {
                    $scope.clearDecisionTable();
                    return;
                }

                $scope.dtPrimitives.minMaxHdr.length = 0;
                $scope.dtPrimitives.uniqueMatrix = {};
                //store all the unique values for each condition
                angular.forEach($scope.dtPrimitives.conditions, function (col) {
                    // if user has re-defined NUM ranges or re-grouped CHAR values get the new values for the column
                    if ($scope.dtPrimitives.selectedColumn && $scope.dtPrimitives.selectedColumn.name == col.name) {
                        $scope.dtPrimitives.uniqueMatrix[col.name] = [];
                        switch ($scope.dtPrimitives.selectedColumn.classification) {

                            case  vsmCommonConst.CLASSIFICATION_CONTINUOUS:
                                angular.forEach($scope.numericDropAttrs.outNumericRanges,
                                    function (uniqVal) {
                                        $scope.dtPrimitives.uniqueMatrix[col.name].push(uniqVal);
                                    });
                                break;
                            default:
                                angular.forEach($scope.charDropAttrs.selectedCharValues,
                                    function (uniqVal) {
                                        $scope.dtPrimitives.uniqueMatrix[col.name].push(uniqVal);
                                    });
                                break;
                        }
                    }
                    else {
                        var uniqueVals = [];
                        $scope.dtPrimitives.uniqueMatrix[col.name] = [];
                        angular.forEach($scope.dtPrimitives.dtModel, function (row) {

                            switch (col.classification) {
                                case  vsmCommonConst.CLASSIFICATION_CONTINUOUS:
                                    var rowVal = row[col.name].value.min + '-' + row[col.name].value.max;
                                    if ($filter(uniqueVals, rowVal).length == 0) {
                                        uniqueVals.push(rowVal);
                                        $scope.dtPrimitives.uniqueMatrix[col.name].push(row[col.name]);
                                    }
                                    break;
                                default:
                                    var rowVal = row[col.name].value;
                                    if ($filter(uniqueVals,
                                            rowVal).length == 0) {
                                        uniqueVals.push(rowVal);
                                        $scope.dtPrimitives.uniqueMatrix[col.name].push(row[col.name]);
                                    }
                                    break;
                            }
                        });
                    }
                });

                //re-build table
                $scope.charDropAttrs.selectedCharValues.length = 0;

                var tempModel = [];

                // This modification was added to resolve defect S1132432
                // If user has re-grouped or redefined ranges for a column then:
                // Remove that column from the existing conditions.
                // Add column back with new values.
                if ($scope.dtPrimitives.selectedColumn && $scope.dtPrimitives.selectedColumn.name) {
                    angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                        tempModel.push(_.omit(row, $scope.dtPrimitives.selectedColumn.name));
                    });
                }

                $scope.dtPrimitives.dtModel.length = 0;

                for (var i = 0; i < $scope.dtPrimitives.conditions.length; i++) {
                    var name = $scope.dtPrimitives.conditions[i].name;
                    if ($scope.dtPrimitives.conditions[i].classification == vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                        $scope.dtPrimitives.minMaxHdr.push("Min(>=)");
                        $scope.dtPrimitives.minMaxHdr.push("Max(<)");
                    }
                    // defect S1132432: if we are regrouping or redefining ranges for a column, we only need to add the conditions back for that column
                    if ($scope.dtPrimitives.selectedColumn && $scope.dtPrimitives.selectedColumn.name !== name) {
                        continue;  // skip to next iteration of loop
                    }
                    tempModel = addConditionToTable($scope.dtPrimitives.uniqueMatrix[name],
                        $scope.dtPrimitives.conditions[i], tempModel);
                }
                $scope.dtPrimitives.completeModel = tempModel;
                $scope.getTableMetrics(tempModel, false, null, false);
            }

            function fillTableSparsity(event, conditions) {
                //Fill in any sparsity  in table in case user wants to fill in values
                angular.forEach(conditions, function (cond) {
                    if (!event[cond.name]) {
                        event[cond.name] = {};
                        if (cond.classification != vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                            event[cond.name].value = (cond.columnType === $scope.WINDOW_OUTPUT || cond.entriesModel) ? " " : null;
                            event[cond.name].oper = getOperator("=");
                        }
                        else {
                            event[cond.name].value = {min: " ", max: " "};
                            event[cond.name].oper = null;
                        }
                    }
                });
                return event;
            }


            function hasTempColumns(type) {
                var testArray = (type == 'CONDITION') ?
                    $scope.dtPrimitives.conditions : $scope.dtPrimitives.actions;
                if (testArray.length > 0 && (testArray[0].__new == "" || testArray[0].__temp == "")) {
                    return true;
                }
                else {
                    return false;
                }
            }

            function removeCondition(column) {
                var uniqueRows = [];

                // if last condition in table then just clear entire table
                if ($scope.dtPrimitives.conditions.length == 1 &&
                    $scope.dtPrimitives.actions.length == 1 &&
                    $scope.dtPrimitives.actions[0].__temp == "") {
                    $scope.clearDecisionTable();
                    return;
                }

                $scope.setDirty(true);
                // remove the column from the conditions Array
                $scope.dtPrimitives.conditions = _.reject($scope.dtPrimitives.conditions,
                    function (cond) {
                        return cond.name == column.name;
                    });


                //Reset Conditions area
                if ($scope.dtPrimitives.conditions.length == 0) {
                    $scope.dtPrimitives.conditions.push({
                        __new: "",
                        dataType: 'CHARACTER',
                        classification: vsmCommonConst.CLASSIFICATION_CATEGORICAL
                    });
                    addPossibleDummyColumn('CONDITION');
                }

                //remove class that grays out input column
                if (column.dropNodeId) {
                    dojoModules.domClass.remove(column.dropNodeId, 'disabledItem');
                }
                // look to see if used cols came from initial load of saved DT
                // cant determine nodeID's when opening a saved DT s o have to search tree
                else {
                    var nodes = dojoModules.query('.columnList td.disabledItem');
                    angular.forEach(nodes, function (node) {
                        if (node.children.length > 0 && node.children[1].innerText.trim() == column.label) {
                            dojoModules.domClass.remove(node, 'disabledItem');
                        }
                    });
                }

                // top Header colspan - needs to expand by 2 for numerics due to min/max cols
                if (column.classification == vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                    $scope.dtPrimitives.topConditionSpan = $scope.dtPrimitives.topConditionSpan - 2;
                    $scope.dtPrimitives.minMaxHdr.pop();
                    $scope.dtPrimitives.minMaxHdr.pop();
                }
                else {
                    $scope.dtPrimitives.topConditionSpan = $scope.dtPrimitives.topConditionSpan - 1;
                }

                $scope.dtPrimitives.selectedColumn = null;

            }

            function removeDuplicateRows(inRows) {
                var outRows = [],
                    execOrder = 1,
                    compareRows = [];

                angular.forEach(inRows, function (row) {
                    var rowString = $scope.dtRowToString(row);
                    if (_.indexOf(compareRows, rowString, false) == -1) {
                        compareRows.push(rowString);
                        row.__execOrder = execOrder++;
                        outRows.push(row);
                    }
                });

                return outRows;

            }


            //doherk - is applyFormat really needed?
            function applyFormat(num, format) {
                if (vsdUtil.isDateFormat(format)) {
                    return vsdUtil.getFormattedDateString(num);
                }
                if (vsdUtil.isDateTimeFormat(format)) {
                    return vsdUtil.getFormattedDateTimeString(num);
                }
                return vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(num), format);
            }

            function formatCell(cell, format) {
                if (cell && cell.value && cell.value.hasOwnProperty('min')) {
                    if (cell.oper == null) {
                        cell.value.min = applyFormat(cell.value.min, format);
                        cell.value.max = applyFormat(cell.value.max, format);
                    }
                    else if (cell.oper.name.indexOf('MISSING') == -1) {
                        if (cell.value.min != null)
                            cell.value.min = applyFormat(cell.value.min, format);
                        if (cell.value.max != null)
                            cell.value.max = applyFormat(cell.value.max, format);
                    }
                }
                return cell;
            }

            function getEmptyTable() {
                $scope.dtPrimitives.dtModel.push({newCondition: "", newAction: ""});
                $scope.dtPrimitives.conditions.push({
                    __new: "",
                    dataType: 'CHARACTER',
                    classification: vsmCommonConst.CLASSIFICATION_CATEGORICAL
                });
                $scope.dtPrimitives.actions.push({__new: ""});

                $scope.dtPrimitives.completeModel = angular.copy($scope.dtPrimitives.dtModel);
                if (!$scope.$$phase) {
                    $scope.$apply();
                }

            }

            function getFilterTableColumns() {
                var actionCols = _.filter($scope.dtPrimitives.actions, function (actCol) {
                    return !actCol.hasOwnProperty("__temp") && !actCol.hasOwnProperty("__new")
                });
                var conditionCols = _.filter($scope.dtPrimitives.conditions, function (condCol) {
                    return !condCol.hasOwnProperty("__temp") && !condCol.hasOwnProperty("__new")
                });
                var newCols = _.flatten([conditionCols, actionCols]);
                return newCols;
            }


            function clearDropNodeIds() {
                //copy existing conditions into usedConditions array and remove any dropNode Id's cause dgrid is redrawn so node ids are no longer valid
                angular.forEach($scope.dtPrimitives.conditions, function (cond) {
                    cond.dropNodeId = null;
                    //$scope.data.usedConditionColumns.push(cond);
                });
                //remove any dropNode Id's cause dgrid is redrawn so node ids are no longer valid
                angular.forEach($scope.dtPrimitives.actions, function (action) {
                    action.dropNodeId = null;
                });
            }

            function returnFromDecisionTreeView() {
                var ruleAddMsg;

                $scope.cancelSecondaryEditor();
                $scope.dtPrimitives.addingTreeRules = true;
                chkForDndDummy('CONDITION');

                clearDropNodeIds();


                //If rules passed back
                var rules = decisionTreeSvc.getSelectedRules();
                if (rules) {

                    //Grab decisionTreeSvc.getUsedConditionColumns() and determine if condition columns must be added
                    var treeConditions = addConditionColumns(decisionTreeSvc.getUsedConditionColumns());
                    $scope.data.usedConditionColumns = angular.copy($scope.dtPrimitives.conditions);

                    angular.forEach(treeConditions, function (cond) {
                        $scope.dtPrimitives.conditions.push(cond);
                        $scope.data.usedConditionColumns.push(cond);
                    });


                    //Give user message about added rules
                    ruleAddMsg = dojoModules.string.substitute(dojoModules.resources.decisionTable_ruleAddMsg_txt,
                        [decisionTreeSvc.getSelectedRules().length]);
                    messageSvc.showInfo(ruleAddMsg);
                }

                //Reset view to decision table
                $scope.dtPrimitives.view = $scope.VIEW_DEFAULT;
                $scope.data.showSave = true;
                $scope.data.disableSaveAs = false;


                var newRows = rulesToDtModel(rules);
                var existingRows = [];

                angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                    row = fillTableSparsity(row, treeConditions);
                    row.treeRow = false;
                    existingRows.push(row);
                    $scope.dtPrimitives.existingRowStrng.push($scope.dtRowToString(row));
                });

                angular.forEach(newRows, function (row) {
                    row.treeRow = true;
                    row = fillTableSparsity(row, $scope.dtPrimitives.conditions);
                    existingRows.push(row);

                });


                $scope.dtPrimitives.dtModel = $scope.dtPrimitives.dtModel.concat(newRows);

                $scope.getTableMetrics(existingRows, true, null, false);

                // remove the highlighting of any new row add by the Tree
                $timeout(function () {
                    angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                        row.treeRow = false;
                    });
                    $scope.dtPrimitives.addingTreeRules = false;
                }, 5000);


            }

            function checkForScenarioOutColUpdates(acCol, updatedVars) {
                var acColChanged = false,
                    oldName = acCol.name;

                angular.forEach(updatedVars, function (col) {
                    if (col.columnType == $scope.WINDOW_OUTPUT)
                        return;
                    //found the scenario output Column - see if it has changed in anyway
                    if (col.id == acCol.id) {
                        // has the name, format, or editType changed?
                        if (oldName != col.name) {
                            acCol.oldName = oldName;
                            acColChanged = true;
                        }
                        if (!acColChanged && ( acCol.columnFormat != col.columnFormat ))
                            acColChanged = true;

                        acCol.name = col.name;
                        acCol.label = col.label;
                        acCol.columnFormat = col.columnFormat;

                        if (acCol.editControlType.name.trim() != col.editControlType.name.trim()) {
                            acCol.editControlType = col.editControlType;
                            acColChanged = true;
                            if (col.editControlType.name.trim() == 'BOOLEAN' && !acCol.entriesModel)
                                acCol.entriesModel = $scope.commonLists.trueFalse;
                            else
                                acCol.entriesModel = null;
                        }

                    }
                });
                return {acColChanged: acColChanged, acCol: acCol};
            }

            function updateUsedScenOutputCol(acCol, row) {
                // name has changed so rename value on row in Table model
                if (acCol.oldName) {
                    row[acCol.name] = angular.copy(row[acCol.oldName]);
                    delete row[acCol.oldName];
                }
                // if column has an entries model then change row value to the first value in the list
                if (acCol.entriesModel) {
                    if (!_.isObject(row[acCol.name]))
                        row[acCol.name] = acCol.entriesModel[0];
                }
                else if (row[acCol.name].id)
                //format the ID of the previous value object
                    row[acCol.name] = vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(row[acCol.name].id), acCol.columnFormat);
                else
                    row[acCol.name] = formatCell(row[acCol.name], acCol.columnFormat);
                // row[acCol.name] =vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(row[acCol.name].value),acCol.columnFormat);

            }

            function getRelatedRows(selRuleIdxs) {

                if (!selRuleIdxs || _.size(selRuleIdxs) <= 0) {
                    $scope.clearPreviewData();
                }
                else {
                    // Populate activeRule with current decision table conditions, which getPreviewData uses to get the detail data
                    var events = dtModelToRules($scope.dtPrimitives.dtModel);
                    var orCond = {
                        operator: _.findWhere($scope.data.operators, {name: "OR"}),
                        childConditions: []
                    };
                    /*
                     angular.forEach(events, function(evt) {
                     if (evt.condition) {
                     orCond.childConditions.push(evt.condition);
                     }
                     });
                     */
                    angular.forEach(selRuleIdxs, function (idx) {
                        if (events[idx] && events[idx].condition) {
                            orCond.childConditions.push(events[idx].condition);
                        }
                    });
                    $scope.data.activeRule = {condition: orCond};
                    $scope.getPreviewData();
                }
            }

            function formatNumberString(value, format) {
                if (vsdUtil.isSasDateConstant(value) || vsdUtil.isSasDateTimeConstant(value)) {
                    return value;
                }
                return vsdUtil.applySASNumericFormat(parseFloat(value), format);
            }


            //region *** Scoped Vars and Objects ****
            $scope.WINDOW_OUTPUT = 'WO';
            $scope.SCENARIO_OUTPUT = 'SO';
            $scope.VIEW_DEFAULT = "D";
            $scope.VIEW_TREE = "T";


            $scope.dtPrimitives = {
                showFilters: false,
                assistOn: true,
                selectedColumn: null,
                addNumericColFlag: false,
                showCharacterDialog: false,
                showDTree: false,
                actions: [],
                conditions: [],
                filterColumns: [],
                minMaxHdr: [],
                uniqueMatrix: {},
                dtModel: [],
                completeModel: [], //non-filtered DT model
                uniqueChars: [],
                selectedCharValues: [],
                topConditionSpan: 0,
                topActionSpan: 0,
                maxCells: 800,  // not used right now
                maxRows: 50,
                currentCellCnt: 0,
                emptyTableClass: "newDrop",
                headerClass: "wrapEmpty",
                inputGridColDef: [],
                sortInfo: {},
                view: "D",
                addingTreeRules: false,
                filterFlag: false,
                existingRowStrng: []
            };

            $scope.inputTree.selectionMode = "single";

            $scope.numericDropAttrs = {
                columnFormat: null,
                selectedColumnIdx: null,
                inNumericRanges: [],
                outNumericRanges: [],
                operators: [],
                reBinFlag: false,
                showNumericDialog: false,
                title: null,
                sumStats: null,
                currRuleCountDisp: "0",
                baseRuleCount: 0,
                newRuleCountDisp: "0"
            };

            $scope.charDropAttrs = {
                selectedColumn: null,
                selectedCharValues: [],
                totalValueCount: 5,
                uniqueChars: [],
                reGroupFlag: false,
                showCharacterDialog: false,
                title: null
            };

            $scope.dtNumericAttrs = {
                rangeRows: [
                    {min: null, max: null},
                    {min: ' ', max: ''},
                    {min: null, max: null}
                ]
            };

            $scope.formatNumberString = formatNumberString;

            //endregion


            //region *** WATCH and EVENT "ON" Listeners
            $scope.$watch('scenario.editMode', function (newVal, oldVal) {
                if (newVal == 'B') {
                    $scope.clearDecisionTable();
                    $scope.setDirty(false);
                    $scope.operMap = {};

                    angular.forEach($scope.data.operators, function (oper) {
                        $scope.operMap[oper.name] = oper;
                    });

                    $scope.dtPrimitives.filterCondition = {
                        operator: $scope.operMap['='],
                        operand1ColumnInfo: {name: '', label: ''},
                        operand2Constant: "",
                        childConditions: []
                    };
                }
            });


            // Disable Save button is showing Decision Tree
            $scope.$watch('view', function (newVal, oldVal) {
                $scope.data.disableSave = (newVal == 'D') ? false : true;
            });


            $scope.$on('PREP_SAVE_SCENARIO', function () {
                $scope.clearFilter();
                $scope.data.usedConditionColumns =
                    (hasTempColumns("CONDITION")) ? null :
                        $scope.dtPrimitives.conditions;
                $scope.data.usedActionColumns =
                    (hasTempColumns("ACTION")) ? null : $scope.dtPrimitives.actions;

                $scope.scenario.events = dtModelToRules($scope.dtPrimitives.dtModel);
            });

            $scope.$on('$destroy', function () {
                $$window.off('resize', resizeDecisionTable);
            });


            //Watch when Re-Group Char Dialog closes - add values/Groups to table
            $scope.$watch('charDropAttrs.showCharacterDialog', function (value) {
                if (value == false && $scope.charDropAttrs.selectedCharValues.length > 0) {
                    if ($scope.isTableMaxSize($scope.charDropAttrs.selectedCharValues.length)) {
                        if ($scope.dtPrimitives.selectedColumn.dropNodeId) {
                            dojoModules.domClass.remove($scope.dtPrimitives.selectedColumn.dropNodeId, 'disabledItem');
                        }
                        return;
                    }
                    if ($scope.charDropAttrs.reGroupFlag) {
                        buildTable();
                    }
                    else {
                        chkForDndDummy('CONDITION');
                        var firstDrop = addPossibleDummyColumn("ACTION");
                        $scope.dtPrimitives.conditions.push($scope.dtPrimitives.selectedColumn);
                        $scope.dtPrimitives.topConditionSpan++;
                        addRows($scope.dtPrimitives.selectedColumn, $scope.charDropAttrs.selectedCharValues, false);

                    }

                    $scope.charDropAttrs.selectedCharValues.length = 0;
                }
            });

            //Watch when Re-define Ranges Dialog closes - add  ranges to table
            $scope.$watch('numericDropAttrs.showNumericDialog', function (value) {
                if (value == false && $scope.numericDropAttrs.outNumericRanges.length > 0) {
                    if ($scope.isTableMaxSize($scope.numericDropAttrs.outNumericRanges.length)) {
                        if ($scope.dtPrimitives.selectedColumn.dropNodeId) {
                            dojoModules.domClass.remove($scope.dtPrimitives.selectedColumn.dropNodeId, 'disabledItem');
                        }
                        return;
                    }
                    angular.forEach($scope.numericDropAttrs.outNumericRanges, function (row) {
                        // split values - min and max
                        if (row.oper == null) {
                            row.value.min = vsdUtil.applySASNumericFormat(row.value.min, $scope.dtPrimitives.selectedColumn.columnFormat);
                            row.value.max = vsdUtil.applySASNumericFormat(row.value.max, $scope.dtPrimitives.selectedColumn.columnFormat);
                        }
                        else if (row.oper.name.indexOf('MISSING') > -1) {
                            row.value.min = " ";
                            row.value.max = null;
                        }
                        else {
                            if (row.value.min != null)
                                row.value.min = vsdUtil.applySASNumericFormat(row.value.min, $scope.dtPrimitives.selectedColumn.columnFormat);
                            if (row.value.max != null)
                                row.value.max = vsdUtil.applySASNumericFormat(row.value.max, $scope.dtPrimitives.selectedColumn.columnFormat);
                        }
                    });
                    buildTable();
                    $scope.charDropAttrs.selectedCharValues.length = 0;
                }
            });


            $scope.$watch('data.windowTableOutCols', function (newVars, oldVars) {
                if (newVars == oldVars) {
                    return;
                }

                var firstRun = $scope.inputTree.inputColumns.length == 0;
                $scope.inputTree.generateInputColumns();

                if (firstRun) {
                    var conditionCols = $scope.data.usedConditionColumns,
                        actionCols = $scope.data.usedActionColumns,
                        hasConditions = conditionCols.length > 0,
                        hasActions = actionCols.length > 0;

                    $scope.clearDecisionTable();

                    $scope.dtPrimitives.filterColumns = _.flatten([conditionCols, actionCols]);

                    if (( conditionCols && hasConditions) || (actionCols && hasActions)) {
                        // load condition Columns
                        if (conditionCols && hasConditions) {
                            $scope.dtPrimitives.conditions.length = 0;
                            $scope.dtPrimitives.conditions = addConditionColumns(conditionCols);
                            $scope.dtPrimitives.emptyTableClass = "";
                        }
                        else
                            addPossibleDummyColumn("CONDITION");
                        // load action Columns
                        if (actionCols && hasActions) {
                            $scope.dtPrimitives.actions = [];
                            angular.forEach(actionCols, function (action) {
                                //IF column is in ACTIONS array then columnType is always SCENARIO_OUTPUT
                                action.columnType = $scope.SCENARIO_OUTPUT;
                                if (!scenarioSvc.allowScenarioOutputColumnDrop(action, $scope.scenario.executionOrder, dojoModules.resources))
                                    return;
                                if (action.editControlType.name.trim() == 'BOOLEAN')
                                    action.entriesModel = $scope.commonLists.trueFalse;

                                $scope.dtPrimitives.actions.push(action);
                            });
                            $scope.dtPrimitives.emptyTableClass = "";
                        }
                        else
                            addPossibleDummyColumn("ACTION");

                        $scope.dtPrimitives.dtModel.length = 0;

                        //convert domainModel to flat table model
                        var model = rulesToDtModel($scope.scenario.events);


                        if ($scope.scenario.events.length == 0 && (hasConditions || hasActions)) {
                            // user added conditions and actions but no events were created
                            $timeout(function () {
                                $scope.dtPrimitives.dtModel = model;
                            });
                        }

                        $scope.setDirty(false);

                        //get table Metrics if Conditions exist
                        if (!hasTempColumns('CONDITION')) {
                            $scope.getTableMetrics(model, false, $scope.scenario.events, firstRun);
                            if (!hasTempColumns('ACTION')) {
                                angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                                    row.__temp = " ";
                                });
                            }
                        }
                        else {
                            $timeout(function () {
                                $scope.dtPrimitives.dtModel = model;
                            });
                        }

                    }
                }
                else if ($scope.dtPrimitives.emptyTableClass == "") {
                    //Check to see if user changed the properties of action Column that is used in the decision table
                    var condChk = {acColChanged: false},
                        actionChk = {acColChanged: false};

                    //are any scen output cols used as a condition?
                    angular.forEach($scope.dtPrimitives.conditions, function (condCol) {
                        if (condCol.columnType === $scope.WINDOW_OUTPUT) return;
                        if (!_.findWhere(newVars, {id: condCol.id})) {
                            if (!condCol.hasOwnProperty("__temp"))
                                removeCondition(condCol);
                        }
                        condChk = checkForScenarioOutColUpdates(condCol, newVars);
                        if (condChk.acColChanged) {
                            condCol = condChk.acCol;
                            angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                                updateUsedScenOutputCol(condCol, row);
                            });
                        }

                    });

                    //are any scen output cols used as an action?
                    angular.forEach($scope.dtPrimitives.actions, function (actCol) {
                        if (!_.findWhere(newVars, {columnType: $scope.SCENARIO_OUTPUT, id: actCol.id})) {
                            if (!actCol.hasOwnProperty("__temp"))
                                $scope.deleteAction(actCol);
                        }
                        actionChk = checkForScenarioOutColUpdates(actCol, newVars);
                        if (actionChk.acColChanged) {
                            actCol = actionChk.acCol;
                            angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                                updateUsedScenOutputCol(actCol, row);
                            });
                        }


                    });

                    if (actionChk.acColChanged || condChk.acColChanged) {

                        /*// if action Col has an entries Model but not value has been set initially then set it to the first value in the list
                         angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                         angular.forEach($scope.dtPrimitives.actions, function(actCol) {
                         // name has changed so rename value on row in Table model
                         if ( actCol.oldName) {
                         row[actCol.name] = angular.copy(row[actCol.oldName]);
                         delete row[actCol.oldName];
                         }
                         // if column has an entries model then change row value to the first value in the list
                         if ( actCol.entriesModel) {
                         if (!_.isObject(row[actCol.name]))
                         row[actCol.name] = actCol.entriesModel[0];
                         }
                         else if (_.isObject(row[actCol.name]) )
                         //format the ID of the previous value object
                         row[actCol.name] =vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(row[actCol.name].id),actCol.columnFormat);
                         */
                        /*else
                         row[actCol.name] =vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(row[actCol.name]),actCol.columnFormat);*/
                        /*
                         });
                         });
                         */

                        var model = angular.copy($scope.dtPrimitives.dtModel);
                        $scope.dtPrimitives.dtModel.length = 0;

                        $timeout(function () {
                            $scope.getTableMetrics(model, false, null);
                        });
                    }
                }
                //Remove loading mask
                $scope.hideLoadingMask();

            });

            $scope.$on(vsmCommonConst.CONDITION_BUILDER_CHANGE, function (evt, id) {
                $scope.filterNotPresent = false;
                evt.preventDefault();
                if (id === "dtFilter") {
                    $scope.filterTable();
                }
            });

            $scope.$on(vsmCommonConst.CONDITION_BUILDER_ERROR, function (evt, id) {
                $scope.filterNotPresent = false;
                evt.preventDefault();
            });

            $scope.$on("DECISION_TABLE_SELECT", function (evt, selIdxs) {
                evt.preventDefault();
                getRelatedRows(selIdxs);
            });

            $scope.$on(vsmCommonConst.CANCEL_SECONDARY_EDITOR, function () {
                clearDropNodeIds();
                $scope.dtPrimitives.view = $scope.VIEW_DEFAULT;
            });

            //endregion


            $scope.isTableMaxSize = function (newRowCnt) {
                var newCount = $scope.dtPrimitives.dtModel.length * newRowCnt;
                if (newRowCnt == 0) return false;

                // if ( $scope.dtPrimitives.currentCellCnt >  $scope.dtPrimitives.maxCells) {
                if (newCount > $scope.dtPrimitives.maxRows) {
                    messageSvc.showInfo(dojoModules.string.substitute(dojoModules.resources.decisionTable_maxTableSize_txt,
                        [newCount, $scope.dtPrimitives.maxRows]));
                    return true;
                }
                else return false;
            }

            //region *** Add/Delete CONDITION / ACTION Functions
            $scope.addCondition = function (col) {


                //Add classification to each used condition column
                col.classification = _.findWhere($scope.data.windowTableOutCols, {id: col.id}).classification;
                if (col.classification == vsmCommonConst.CLASSIFICATION_CONTINUOUS) {

                    if ($scope.isTableMaxSize(3)) {
                        if (col.dropNodeId) {
                            dojoModules.domClass.remove(col.dropNodeId, 'disabledItem');
                        }
                        return false;
                    }
                    newNumericColumn(col);
                }
                else {
                    var cnt = (col.distinctCount <= 10) ? col.distinctCount : 0;
                    if ($scope.isTableMaxSize(cnt + 1)) {
                        if (col.dropNodeId) {
                            dojoModules.domClass.remove(col.dropNodeId, 'disabledItem');
                        }
                        return false;
                    }
                    newCharacterColumn(col);
                }

                if ($scope.dtPrimitives.conditions.length > 1) {
                    $scope.dtPrimitives.sortInfo[$scope.dtPrimitives.conditions[0].name] = 'D';
                }

                $scope.dtPrimitives.filterColumns.push(col);
                return true;
            };

            $scope.addAction = function (col, selectedNode) {
                var defaultValue = "";

                if (col.editControlType && col.editControlType.name.trim() == 'BOOLEAN') {
                    col.entriesModel = $scope.commonLists.trueFalse;
                    defaultValue = col.entriesModel[0];
                }
                var firstDrop = addPossibleDummyColumn("CONDITION");

                var tempRows = [];
                if ($scope.dtPrimitives.dtModel.length == 1 && firstDrop) {
                    var newRow = {edit: false, rowNum: 0};
                    newRow[col.name] = defaultValue;
                    newRow.__temp = "";
                    tempRows.push(newRow);
                }
                else {
                    angular.forEach($scope.dtPrimitives.dtModel, function (row) {
                        row[col.name] = defaultValue;
                        row.edit = false;
                        tempRows.push(row);
                    });
                }

                // $scope.dtPrimitives.completeModel = angular.copy($scope.dtPrimitives.dtModel);
                $scope.dtPrimitives.actions.push(col);
                $scope.setDirty(true);


                // needed to put this in a nested $timeout so that the dojo DND processing of the dom nodes would finish first,
                // then clearing the table and finally populating the model again
                $timeout(function () {
                    // We have to wait until after the drop to remove the temporary node, otherwise its stopDrag won't get called and the
                    // avatar won't go away.
                    if ($scope.dtPrimitives.actions.length > 1 && ( $scope.dtPrimitives.actions[0].__temp == "" || $scope.dtPrimitives.actions[0].__new == "")) {
                        $scope.dtPrimitives.actions.splice(0, 1);
                    }
                    $scope.dtPrimitives.dtModel.length = 0;
                    $timeout(function () {
                        $scope.dtPrimitives.dtModel = tempRows;
                        $scope.dtPrimitives.completeModel = angular.copy(tempRows);
                    });
                });

                $scope.dtPrimitives.filterColumns.push(col);

            };


            $scope.deleteCondition = function (column) {
                var tempRows,
                    uniqueRows = [];

                removeCondition(column);
                tempRows = angular.copy($scope.dtPrimitives.dtModel);
                angular.forEach(tempRows, function (row) {
                    delete row[column.name];

                    var rowString = $scope.dtRowToString(row);
                    if (_.indexOf(uniqueRows, rowString, false) == -1)
                        uniqueRows.push(rowString);
                });

                $timeout(function () {
                    $scope.dtPrimitives.dtModel.length = 0;
                });

                var model = removeDuplicateRows(tempRows);
                //$scope.dtPrimitives.completeModel = angular.copy(model);
                if (!hasTempColumns('CONDITION'))
                    $scope.getTableMetrics(model, false, null);
                else
                    $timeout(function () {
                        $scope.dtPrimitives.dtModel = tempRows;
                    });

                if (column.name === $scope.dtPrimitives.filterCondition.operand1ColumnInfo.name) {
                    $scope.clearFilter();
                }
            };

            $scope.deleteAction = function (column) {

                // if last Action in table then just clear entire table
                if ($scope.dtPrimitives.actions.length == 1 &&
                    $scope.dtPrimitives.conditions.length == 1 &&
                    $scope.dtPrimitives.conditions[0].__temp == "") {
                    $scope.clearDecisionTable();
                    return;
                }

                // remove the column from the conditions Array
                $scope.dtPrimitives.actions = _.reject($scope.dtPrimitives.actions,
                    function (action) {
                        return action.name == column.name;
                    });
                $scope.setDirty(true);

                if ($scope.dtPrimitives.actions.length == 0) {
                    $scope.dtPrimitives.actions.push({__new: ""});
                    addPossibleDummyColumn("ACTION");
                }


                //var selectedNode = angular.element("#" + column.dropNodeId);
                if (column.dropNodeId) {
                    dojoModules.domClass.remove(column.dropNodeId, 'disabledItem');
                }
                // look to see if used cols came from initial load of saved DT
                // cant determine nodeID's when opening a saved DT s o have to search tree
                else {
                    var nodes = dojoModules.query('.columnList td.disabledItem');
                    angular.forEach(nodes, function (node) {
                        if (node.children.length > 0 && node.children[1].innerText.trim() == column.label) {
                            dojoModules.domClass.remove(node, 'disabledItem');
                        }
                    });
                }

                $scope.dtPrimitives.topActionSpan = $scope.dtPrimitives.topActionSpan - 1;
                $scope.dtPrimitives.selectedColumn = null;

                var tempModel = angular.copy($scope.dtPrimitives.dtModel);
                $scope.dtPrimitives.dtModel.length = 0;
                angular.forEach(tempModel, function (row) {
                    delete row[column.name];
                });

                $timeout(function () {
                    $scope.dtPrimitives.dtModel = tempModel;
                });


            };
            //endregion

            $scope.getTableMetrics = function (model, animate, rules, firstRun) {
                var rules = rules,
                    filterOn = false;
                if (!firstRun)
                    $scope.setDirty(true);

                //Blank out existing sort order
                $scope.dtPrimitives.sortInfo = {};

                //Sort new dtModel
                //model.sort(conditionsComparator);

                // does user have a filter when they when into edit mode on the table?
                // if so splice back in the filtered out rows
                if ($scope.dtPrimitives.filterFlag) {
                    angular.forEach($scope.dtPrimitives.completeModel, function (row) {
                        if (!_.findWhere(model, {__execOrder: row.__execOrder}))
                            model.push(row);
                    });
                    filterOn = true;
                }

                // sort model by execution order
                model.sort(function (a, b) {
                    return a.__execOrder - b.__execOrder;
                });

                //Generate rules from dtModel
                if (!rules) {
                    rules = dtModelToRules(model);
                }

                $scope.dtPrimitives.dtModel.length = 0;
                //Make service call to get counts for the scenario and each rule
                $scope.showLoadingMask(dojoModules.resources.decisionTable_getPopPercent_txt);
                scenarioSvc.getStatsForScenario(rules)
                    .then(function (stats) {
                        //Set global population count/percent
                        $scope.setCounters(stats.scenarioCount);
                        //Set rule population count/percent
                        var statCnt = 0;
                        for (var i = 0; i < model.length; i++) {
                            var row = model[i],
                                rowStats = {},
                                fmtCount,
                                fmtPct;

                            if (!row.empty) {
                                fmtCount = vsdUtil.applySASNumericFormat(stats.ruleCounts[statCnt], vsdUtil.defaultFormats.integer);
                                fmtPct = vsdUtil.applySASNumericFormat(stats.ruleCounts[statCnt] / $scope.counters.totalRecCnt,
                                    vsdUtil.defaultFormats.percentage);
                                rowStats.populationCnt = stats.ruleCounts[statCnt];
                                statCnt++;
                                rowStats.populationDisp = (fmtPct && fmtCount) ?
                                fmtCount + ' (' + fmtPct + ')' : " ";
                                row.__rowStats = rowStats;
                            }
                            else {
                                row.__rowStats.populationDisp = "";
                            }

                            row.rowNum = i;
                            row.edit = false;

                            // format all the NUM condition Columns - needed if user Edited multiple rows
                            angular.forEach($scope.dtPrimitives.conditions, function (condCol) {
                                $scope.dtPrimitives.currentCellCnt++;
                                if (condCol.columnType == $scope.SCENARIO_OUTPUT && condCol.entriesModel)
                                    return;
                                // var cell = row[condCol.name];
                                row[condCol.name] = formatCell(row[condCol.name], condCol.columnFormat);
                                /*if ( cell.value && cell.value.hasOwnProperty('min') ) {
                                 if (cell.oper == null) {
                                 cell.value.min = vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(cell.value.min),condCol.columnFormat);
                                 cell.value.max = vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(cell.value.max),condCol.columnFormat);
                                 }
                                 else if ( cell.oper.name.indexOf('MISSING') == -1){
                                 if ( cell.value.min != null)
                                 cell.value.min = vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(cell.value.min),condCol.columnFormat);
                                 if ( cell.value.max != null)
                                 cell.value.max = vsdUtil.applySASNumericFormat(vsdUtil.removeFormat(cell.value.max),condCol.columnFormat);
                                 }
                                 }*/
                            });

                            // count cells for actionCols
                            $scope.dtPrimitives.currentCellCnt += $scope.dtPrimitives.actions.length;

                        }

                        // set up sort indicator on table
                        if ($scope.dtPrimitives.conditions.length > 0) {
                            $scope.dtPrimitives.sortInfo = {};
                            $scope.dtPrimitives.sortInfo[$scope.dtPrimitives.conditions[0].name] = 'D';
                        }

                        $scope.hideLoadingMask();

                        $scope.dtPrimitives.dtModel = model;

                        $scope.dtPrimitives.completeModel = angular.copy($scope.dtPrimitives.dtModel);

                        if (filterOn) {
                            $scope.filterTable();
                        }
                        else {
                            $scope.clearFilter();
                        }
                        // No rows are selected yet
                        $scope.clearPreviewData();

                    });
            };


            $scope.toggleLoadingMask = function (onOff) {


            }

            // used for sorting min/max values
            $scope.numericGetMaxSortValue = function (valueObj, oper) {
                if (valueObj.min && valueObj.max) {
                    return parseFloat(vsdUtil.removeFormat(valueObj.max));
                }
                else {
                    var sortNumber = valueObj.min != null ? valueObj.min : valueObj.max;

                    if (oper && oper.name === ">=") {
                        return sortNumber + 0.00001;
                    }
                    else {
                        return sortNumber;
                    }
                }
            }

            //region *** Table Sorting Functions ***
            $scope.sortTable = function (model) {
                model.sort(conditionsComparator);
            }

            $scope.sortByColumn = function (column, colType) {
                if (column.hasOwnProperty("__temp")) {  // this is an empty column
                    return;
                }
                var columnName = (column.name) ? column.name : column,
                    sortOrder = 'A',
                    colClassif = column.classification;
                $scope.showLoadingMask();

                if ($scope.dtPrimitives.sortInfo[columnName]) {
                    sortOrder = $scope.dtPrimitives.sortInfo[columnName] === 'A' ? 'D' : 'A';
                }

                //
                /* //Set callback for value extraction
                 var extractValueCallback;
                 switch (colType) {
                 case 'COND':
                 extractValueCallback = getMaxDtModelValue;
                 break;
                 case 'ACT':
                 col = _.findWhere($scope.dtPrimitives.actions,{name: columnName})
                 extractValueCallback = function(field){
                 if(col.editControlType.name === "BOOLEAN") {
                 return field.name
                 }
                 else {
                 return parseFloat(vsdUtil.removeFormat(field));
                 }
                 };
                 break;
                 case 'POP_METRIC':
                 extractValueCallback = getPopCnt;
                 break;
                 case 'EXEC_ORDER':
                 extractValueCallback = getSimpleFloat;
                 break;

                 }

                 vsdUtil.sortByFields(tempArray, [
                 {field: columnName, order: sortOrder}
                 ], extractValueCallback);
                 */
                // this ensures that we have digested clearing out the dtModel first then we re-populate.
                $timeout(function () {
                    //Apply sort
                    var tempArray = angular.copy($scope.dtPrimitives.dtModel);
                    $scope.dtPrimitives.dtModel.length = 0;
                    angular.forEach(tempArray, function (row) {
                        var cell = row[columnName];
                        switch (colType) {
                            case 'COND':
                                if (colClassif === vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                                    cell.sortValue = (vsdUtil.removeFormat(cell.value.min) !== null) ?
                                    parseFloat(vsdUtil.removeFormat(cell.value.min)) + 0.00001 : parseFloat(vsdUtil.removeFormat(cell.value.max));
                                    //chk for empty string so returns NaN with a  parseFloat
                                    cell.sortValue = (_.isNaN(cell.sortValue)) ? null : cell.sortValue;
                                    // chk for MISSING operator - if so make greater than null;
                                    cell.sortValue = (cell.oper && cell.oper.name.indexOf('MISSING') > -1) ? Number.MIN_VALUE : cell.sortValue;
                                }
                                else if (colClassif === vsmCommonConst.CLASSIFICATION_BINARY)
                                    cell.sortValue = cell.name;
                                else
                                    cell.sortValue = cell.value;
                                break;
                            case 'ACT':
                                if (colClassif === vsmCommonConst.CLASSIFICATION_CONTINUOUS)
                                    cell.sortValue = (vsdUtil.removeFormat(cell.min) !== null) ?
                                        parseFloat(vsdUtil.removeFormat(cell.min)) : parseFloat(vsdUtil.removeFormat(cell.max));
                                else
                                    cell.sortValue = cell.name;
                                break;
                            case 'POP_METRIC':
                                cell.sortValue = row.__rowStats.populationDisp;
                                break;
                            case 'EXEC_ORDER':
                                cell.sortValue = row.__execOrder;
                                break;
                        }
                        // tempArray.push(row);
                    });

                    tempArray = _.sortBy(tempArray, function (row) {
                        return row[columnName].sortValue;
                    });
                    // tempArray=  _.sortBy(tempArray, row[columnName].sortValue);
                    if ($scope.dtPrimitives.sortInfo[columnName] === 'D')
                        tempArray.reverse();
                    $scope.dtPrimitives.dtModel = tempArray;
                    $timeout(function () {
                        $scope.hideLoadingMask();
                    });
                });
                //scope.updateModel();
                //Recreate sort info object with new information
                $scope.dtPrimitives.sortInfo = {};
                $scope.dtPrimitives.sortInfo[columnName] = sortOrder;

                /*function getSimpleFloat(field) {
                 return parseFloat(field);
                 }

                 function getPopCnt(field) {
                 return field.populationCnt;
                 }

                 function getMaxDtModelValue(field) {
                 var value = field.value;

                 if ( value.hasOwnProperty("min"))  {
                 if (value.min && value.max) {
                 return parseFloat(vsdUtil.removeFormat(value.max));
                 }
                 else {
                 if ((value.min && value.min.trim())|| (value.max && value.max.trim())) {
                 var valueStr = value.min ? value.min : value.max;
                 var floatVal = parseFloat(vsdUtil.removeFormat(valueStr));
                 if (value.min) {
                 return floatVal + 0.00001;
                 }
                 else {
                 return floatVal;
                 }
                 }
                 else {  //This is missing - return the smallest value
                 return Number.MIN_VALUE;
                 }
                 }
                 }
                 else
                 return value;


                 }*/


            };
            //endregion


            $scope.inputColsGetChildren = function (store, parent, options) {
                return store.query({parent: parent.treeId}, options);
            };

            $scope.inputColsMayHaveChildren = function (store, parent) {
                return parent.columns && parent.columns.length > 0 && store.getChildren(parent).length > 0;
            };
            //endregion

            //region *** TOOLBAR FUNCTIONS ***
            $scope.clearDecisionTable = function () {
                var cnt = 0;
                //remove class from input col Tree that grays out the column
                // node id is stored on dropped column when it is dropped
                angular.forEach($scope.dtPrimitives.conditions, function (cond) {
                    if (cond.dropNodeId) {
                        dojoModules.domClass.remove(cond.dropNodeId, 'disabledItem');
                        cnt++;
                    }

                });
                // see if a dropped column was in the actions Array
                angular.forEach($scope.dtPrimitives.actions, function (cond) {
                    if (cond.dropNodeId) {
                        dojoModules.domClass.remove(cond.dropNodeId, 'disabledItem');
                        cnt++;
                    }
                });
                // look to see if used cols came from initial load of saved DT
                // cant determine nodeID's when opening a saved DT
                if (cnt < ($scope.dtPrimitives.conditions.length + $scope.dtPrimitives.actions.length)) {
                    dojoModules.query('.columnList td.disabledItem').removeClass('disabledItem');
                }

                //Set global population count/percent
                $scope.setCounters(0);

                //needed to clear tabel completely so it re-builds correctly
                $scope.dtPrimitives.dtModel.length = 0;
                ;
                $scope.dtPrimitives.completeModel.length = 0;
                if (!$scope.$$phase) {
                    $scope.$apply();
                }

                $scope.dtPrimitives.conditions = [];
                $scope.dtPrimitives.actions = [];
                $scope.dtPrimitives.minMaxHdr = [];
                $scope.dtPrimitives.addNumericColFlag = false;
                $scope.dtPrimitives.topConditionSpan = 0;
                $scope.dtPrimitives.topActionSpan = 0;
                $scope.dtPrimitives.emptyTableClass = "newDrop";
                $scope.dtPrimitives.headerClass = 'wrapEmpty';
                $scope.dtPrimitives.filterColumns = getFilterTableColumns();
                // get dummy row with DND images
                getEmptyTable();

                if ($scope.inputTree.targetColNode)
                    dojoModules.domClass.add($scope.inputTree.targetColNode, 'disabledItem');

                $scope.setDirty(true);
            };


            $scope.updateTable = function () {
                var tempRows = [];
                angular.copy($scope.dtPrimitives.dtModel, tempRows);
                $scope.getTableMetrics(tempRows, false, null, false);
            }

            // toolbar checkbox for "Assistant ON"
            $scope.assist = function () {
                $scope.dtPrimitives.assistOn = (
                    $scope.dtPrimitives.assistOn ? false : true);
            };

            $scope.filterTable = function () {
                $scope.dtPrimitives.filterFlag = true;
                $scope.dtPrimitives.dtModel = _.filter($scope.dtPrimitives.completeModel,
                    function (row) {
                        var filter = "",
                            whereCol = $scope.dtPrimitives.filterCondition.operand1ColumnInfo,
                            oper = $scope.dtPrimitives.filterCondition.operator.name,
                            compValue = $scope.dtPrimitives.filterCondition.operand2Constant,
                            passFilter = false,
                            isCondFilter = _.findWhere($scope.dtPrimitives.conditions, {name: whereCol.name});

                        function comparison(operator, lhs, rhs) {
                            var compareGood = false,
                                lhsCompare = lhs;
                            if (_.isNumber(lhs)) {
                                lhsCompare = vsdUtil.parseNumeric(lhs);
                            }
                            // Remove numeric formatting so that we have numeric value to use for the comparison
                            else if (whereCol.dataType === vsmCommonConst.DATA_TYPE_NUMERIC) {
                                lhsCompare = vsdUtil.removeFormat(lhs);
                            }
                            switch (operator) {
                                case "=":
                                    compareGood = (lhsCompare == rhs);
                                    break;
                                case ">":
                                    compareGood = (lhsCompare > rhs);
                                    break;
                                case ">=":
                                    compareGood = (lhsCompare >= rhs);
                                    break;
                                case "<=":
                                    compareGood = (lhsCompare <= rhs);
                                    break;
                                case "<":
                                    compareGood = (lhsCompare < rhs);
                                    break;
                                case "IN":
                                    compareGood = (lhsCompare.indexOf(rhs) != -1);
                                    break;
                                case "<>":
                                    compareGood = lhsCompare != rhs;
                                    break;
                                /*case "IS MISSING":
                                 compareGood = lhs == null;
                                 break;
                                 case "NOT MISSING":
                                 compareGood = lhs != null;
                                 break;*/

                            }
                            return compareGood;
                        }

                        function notMissing() {
                            return oper === 'NOT MISSING' && row[whereCol.name].oper.name != 'IS MISSING';
                        }

                        if (isCondFilter) {
                            if (whereCol.classification == vsmCommonConst.CLASSIFICATION_CONTINUOUS) {
                                var minPass = false,
                                    maxPass = false;

                                if (oper.indexOf('MISSING') > -1) {
                                    if (row[whereCol.name].oper && oper === row[whereCol.name].oper.name) {
                                        passFilter = true;
                                    }
                                    else if (notMissing()) {
                                        passFilter = true;
                                    }
                                }
                                else {
                                    if (row[whereCol.name].value.min) {
                                        minPass = comparison(oper, row[whereCol.name].value.min, compValue);
                                    }
                                    if (row[whereCol.name].value.max) {
                                        maxPass = comparison(oper, row[whereCol.name].value.max, compValue);
                                    }
                                    if (minPass || maxPass) {
                                        passFilter = true;
                                    }
                                }
                            }
                            else {
                                if (oper.indexOf('MISSING') > -1) {
                                    if (oper == row[whereCol.name].oper.name) {
                                        passFilter = true;
                                    }
                                    else if (notMissing()) {
                                        passFilter = true;
                                    }
                                }
                                else {
                                    var rowValue = whereCol.entriesModel ? row[whereCol.name].name.toUpperCase() :
                                        row[whereCol.name].value.trim().toUpperCase();
                                    if (oper === 'IN') {  // compare  with each value in the list
                                        passFilter = _.contains(compValue.trim().toUpperCase().split(","), rowValue);
                                    }
                                    if (oper === 'NOT IN') {  // compare  with each value in the list
                                        passFilter = !_.contains(compValue.trim().toUpperCase().split(","), rowValue);
                                    }
                                    if (oper === '<>' && rowValue != compValue.trim().toUpperCase()) {
                                        passFilter = true;
                                    }
                                    else if (oper === '=' && rowValue == compValue.trim().toUpperCase()) {
                                        passFilter = true;
                                    }
                                }
                            }
                        }
                        else {
                            var cellValue = (whereCol.entriesModel) ? row[whereCol.name].name : String(row[whereCol.name]);
                            passFilter = comparison(oper, cellValue, compValue);
                        }


                        return passFilter;
                    });

                if ($scope.dtPrimitives.dtModel.length == 0)
                    messageSvc.showInfo(dojoModules.resources.decisionTable_filterTableNoRows_txt);
            };


            $scope.clearFilter = function () {
                $scope.dtPrimitives.filterFlag = false;
                $scope.dtPrimitives.filterColumns = getFilterTableColumns();
                $scope.dtPrimitives.filterCondition = null;
                $scope.dtPrimitives.filterCondition = {
                    operator: $scope.operMap['='],
                    operand1ColumnInfo: {name: '', label: ''},
                    operand2Constant: "", childConditions: []
                };

                $scope.dtPrimitives.dtModel.length = 0;
                $scope.dtPrimitives.dtModel = angular.copy($scope.dtPrimitives.completeModel);
                $scope.filterNotPresent = true;

                // Clear error state of the ValidationTextBoxes nested under the 'dtFilterCondition' node
                dojoModules.query(".dtFilterCondition").forEach(function (node) {
                    dojoModules.query(".dijitValidationTextBox", node).forEach(function (tbNode) {
                        var widget = dojoModules.registry.byNode(tbNode);
                        Util.clearError(widget.id);
                    });
                });
            };

            $scope.onTargetColumnSelect = function () {
                $scope.inputTree.inputColumns = [];
                $scope.inputTree.generateInputColumns();
            }

            $scope.switchToDecisionTreeView = function () {
                //Initialize required properties in decisionTreeSvc
                var partitionKeyColumns = $scope.scenario.scenarioWindowPartitionKeyColumns || [];
                decisionTreeSvc.setInputColumns($scope.data.windowTableOutCols)
                    .setInputRecordsCount($scope.counters.totalRecCnt)
                    .setReturnToParentScreenHandler(returnFromDecisionTreeView)
                    .setPartitionKeyColumns(partitionKeyColumns)
                    .setScenarioExecutionOrder($scope.scenario.executionOrder);
                $scope.data.disableSave = true;
                $scope.data.disableSaveAs = true;
                $scope.clearPreviewData();
                $scope.dtPrimitives.view = $scope.VIEW_TREE;
                $scope.data.secondaryEditorShowing = true;
                $scope.$apply();
            }

            $scope.dtRowToString = function (row) {
                var rowStr = "";
                for (var cell in row) {
                    if (row[cell].value) {
                        if (row[cell].value.hasOwnProperty("min"))
                            rowStr += "|" + row[cell].value.min + "-" + row[cell].value.max;
                        else
                            rowStr += "|" + row[cell].value;
                    }
                }
                return rowStr;
            }

            $scope.getInputGridRMBMenuItems = function (rows) {
                //Only allow one selected row at a time, so only one entry can come in
                var me = this,
                    row = rows[0],
                    isCondCol = false,
                    isActCol = false,
                    allowScenarioColumnDrop = scenarioSvc.allowScenarioOutputColumnDrop(row.data, $scope.scenario.executionOrder, $scope.resources, true);

                //Determine how many condition and actions columns in selection
                if (_.findWhere($scope.dtPrimitives.conditions, {id: row.data.id})) {
                    isCondCol = true;
                }
                else if (_.findWhere($scope.dtPrimitives.actions, {id: row.data.id})) {
                    isActCol = true
                }
                var inputRowMenuItems = [];
                //Use as target column menu item
                inputRowMenuItems.push({
                    label: dojoModules.resources.decisionTable_inputColsRMBMenu_addToConditions_txt,
                    disabled: isCondCol || isActCol || !allowScenarioColumnDrop,
                    onClick: function (evt) {
                        if ($scope.addCondition(row.data)) {
                            vsdUtil.markDroppedColumn(row.element, row.data);
                        }
                        $scope.$apply(); //Must $apply here since event originates in Dojo
                    }
                });
                //Add as condition column menu item
                inputRowMenuItems.push({
                    label: dojoModules.resources.decisionTable_inputColsRMBMenu_addToActions_txt,
                    disabled: isCondCol || isActCol || row.data.columnType === "WO",
                    onClick: function (evt) {
                        $scope.addAction(row.data);
                        vsdUtil.markDroppedColumn(row.element, row.data);
                        $scope.$apply(); //Must $apply here since event originates in Dojo
                    }
                });
                inputRowMenuItems.push({
                    label: dojoModules.resources.decisionTable_inputColRMBMenu_remove_txt,
                    disabled: !isCondCol && !isActCol,
                    onClick: function (evt) {
                        if (isCondCol) {
                            $scope.deleteCondition(row.data);
                        }
                        else {
                            $scope.deleteAction(row.data);
                        }
                        vsdUtil.unmarkRemovedColumn(row.element.id);
                        $scope.$apply(); //Must $apply here since event originates in Dojo
                    }
                });

                return inputRowMenuItems;
            }


            //endregion

        }]);

});

},
'vsd/scenario/decisionTable/ScenarioNumericDropCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioNumericDropCtrl', ['$rootScope', '$scope', 'vsdUtil', 'operatorSvc',
        function ($rootScope, $scope, vsdUtil, operatorSvc) {
            var rowValidity = [],
                numberPattern;

            function calcNewRuleCountDisp() {
                var rangeCount = $scope.numericDropAttrs.inNumericRanges.length > 0 ? $scope.numericDropAttrs.inNumericRanges.length : 1;
                var newCount = $scope.numericDropAttrs.baseRuleCount * rangeCount;
                $scope.numericDropAttrs.newRuleCountDisp = vsdUtil.applySASNumericFormat(newCount, vsdUtil.defaultFormats.integer);
            }

            function setAllowApply(row) {
                //var allowApply = true;
                if ($scope.numericDropAttrs.inNumericRanges.length === 0) {
                    $scope.numericDropCtrlData.allowApply = false;
                    return;
                }
                $scope.numericDropCtrlData.allowApply = row.rowValid;

            }

            function getNumber(row) {
                var format = $scope.numericDropAttrs.columnFormat;
                if (vsdUtil.isDateFormat(format)) {
                    return vsdUtil.parseFormattedDate(row.singleOperand);
                }
                if (vsdUtil.isDateTimeFormat(format)) {
                    return vsdUtil.parseFormattedDateTime(row.singleOperand);
                }
                return row.singleOperand;
            }

            function applyFormatIfNecessary(num, format) {
                if (vsdUtil.isDateFormat(format) || vsdUtil.isDateFormat($scope.numericDropAttrs.columnFormat)) {
                    return vsdUtil.getFormattedDateString(num);
                }
                if (vsdUtil.isDateTimeFormat(format) || vsdUtil.isDateTimeFormat($scope.numericDropAttrs.columnFormat)) {
                    return vsdUtil.getFormattedDateTimeString(num);
                }
                return num;
            }


            function convertFromDateIfNecessary(val, format) {
                if (null == val) {
                    return null;
                }
                if ((typeof val === 'string')) {
                    if (vsdUtil.isDateFormat(format)) {
                        return vsdUtil.parseFormattedDate(val);
                    } else if (vsdUtil.isDateTimeFormat(format)) {
                        return vsdUtil.parseFormattedDateTime(val);
                    }
                }
                return val;
            }

            //region *** Scope Data/Functions ***

            $scope.selColumn = null;
            $scope.numericDropCtrlData = {
                defaultOper: "",
                operators: [],
                allowAccept: true
            };

            numberPattern = /^-?[1-9][\.\d]*(,\d+)?$/;


            $scope.addRow = function () {
                $scope.numericDropAttrs.inNumericRanges.push({colSpan: 1, value: {min: null, max: null}, oper: null});
                rowValidity.push(false);
                $scope.numericDropCtrlData.allowApply = false;
                calcNewRuleCountDisp();
            };

            $scope.removeRow = function (rowIdx) {
                $scope.numericDropAttrs.inNumericRanges.splice(rowIdx, 1);
                rowValidity.splice(rowIdx, 1);
                calcNewRuleCountDisp();
                setAllowApply({rowValid: true});
            };

            $scope.removeAllRows = function (rowIdx) {
                $scope.numericDropAttrs.inNumericRanges = [];
                rowValidity = [];
                calcNewRuleCountDisp();
                setAllowApply({rowValid: true});
            };

            $scope.duplicateRow = function (rowIdx) {
                var newRow = angular.copy($scope.numericDropAttrs.inNumericRanges[rowIdx]);
                $scope.numericDropAttrs.inNumericRanges.splice(rowIdx, 0, newRow);
                rowValidity.splice(rowIdx, 0, rowValidity[rowIdx]);
                calcNewRuleCountDisp();
            };

            $scope.splitMergeCell = function (row) {
                row.value.max = applyFormatIfNecessary(row.value.max ? row.value.max : row.value.min, row.format);
                row.value.min = applyFormatIfNecessary(row.value.min ? row.value.min : row.value.max, row.format);
                if (row.colSpan === 2) {
                    row.colSpan = 1;
                    row.oper = null;
                    row.singleOperand = row.value.max;
                }
                else {
                    row.colSpan = 2;
                    row.singleOperand = row.value.max;
                    row.value.min = applyFormatIfNecessary(row.value.max ? null : row.value.min, row.format);
                    if (row.singleOperand)
                        $scope.numericDropCtrlData.allowApply = true;
                    row.oper = (row.value.max) ? $scope.operMap["<"] : $scope.operMap[">="];
                }

            };


            function parseFormattedDateOrDateTime(colFormat, minOrMax) {
                if (vsdUtil.isDateFormat(colFormat)) {
                    return vsdUtil.parseFormattedDate(minOrMax);
                }
                return vsdUtil.parseFormattedDateTime(minOrMax);
            }

            function validateRow(row) {
                var colFormat = $scope.numericDropAttrs.columnFormat;
                if (vsdUtil.isDateOrDateTimeFormat(colFormat)) {
                    if (_.isEmpty(row.value.min) || _.isEmpty(row.value.min)) {
                        return false;
                    }
                    var min = parseFormattedDateOrDateTime(colFormat, row.value.min);
                    var max = parseFormattedDateOrDateTime(colFormat, row.value.max);
                    return numberPattern.test(min) && numberPattern.test(max);
                }

                return numberPattern.test(row.value.min) && numberPattern.test(row.value.max);
            }

            $scope.validateRow = function (row) {

                switch (row.colSpan) {
                    case 2:
                        var number = getNumber(row);
                        row.rowValid = numberPattern.test(number);
                        if (row.rowValid) {
                            if (!row.value.min && !row.value.max)
                                row.value.min = row.singleOperand;
                            else {
                                row.value.min = (row.value.min) ? number : row.value.min;
                                row.value.max = (row.value.max) ? number : row.value.max;
                            }
                        }
                        break;
                    case 1:
                        row.rowValid = validateRow(row);
                        break;
                }

                setAllowApply(row);
            }

            $scope.updateRanges = function () {
                var outRanges = [];
                angular.forEach($scope.numericDropAttrs.inNumericRanges, function (range) {
                    var newRange = angular.copy(range);
                    var colFormat = $scope.numericDropAttrs.columnFormat;
                    switch (newRange.colSpan) {
                        case 2:
                            newRange.value.min = (range.value.min) ? range.value.min : null;
                            newRange.value.max = (range.value.max) ? range.value.max : null;
                            break;
                        case 1:
                            newRange.value.min = range.value.min;
                            newRange.value.max = range.value.max;
                            break;
                    }

                    outRanges.push({
                        colSpan: newRange.colSpan,
                        oper: newRange.oper,
                        value: {
                            min: convertFromDateIfNecessary(newRange.value.min, colFormat),
                            max: convertFromDateIfNecessary(newRange.value.max, colFormat)
                        }
                    });
                });
                $scope.numericDropAttrs.outNumericRanges = outRanges;
                $scope.numericDropAttrs.showNumericDialog = false;
            };

            $scope.updateOperValue = function (row) {
                var me = row.oper;
                if (row.oper.name.indexOf('MISSING') > -1) {
                    row.value.min = " ";
                    row.value.max = null;
                    // row.colSpan=5;
                }
                // else
                //   row.colSpan=2;
            };

            $scope.cancelDialog = function () {
                $scope.numericDropAttrs.outNumericRanges = [];
                $scope.numericDropAttrs.showNumericDialog = false;
            };

            $scope.$watch('numericDropAttrs.showNumericDialog', function () {
                function useFormatValueIfNecessary(row, maxOrMinProperty, maxOrMin) {
                    if (vsdUtil.isDateOrDateTimeFormat(row.format) || vsdUtil.isDateOrDateTimeFormat($scope.numericDropAttrs.columnFormat)) {
                        return row.formatValue[maxOrMinProperty];
                    }
                    return maxOrMin;
                }

                function cleanValOrDateStr(rowFormat, minOrMax) {
                    if (vsdUtil.isDateOrDateTimeFormat(rowFormat)) {
                        if (vsdUtil.isDateFormat(rowFormat)) {
                            return vsdUtil.getFormattedDateString(minOrMax);
                        }
                        if (vsdUtil.isDateTimeFormat(rowFormat)) {
                            return vsdUtil.getFormattedDateString(minOrMax);
                        }
                    }
                    var cleanMinOrMax = parseFloat(minOrMax);
                    return (cleanMinOrMax) ? cleanMinOrMax : null;
                }

                if ($scope.numericDropAttrs.showNumericDialog) {

                    operatorSvc.getLogicalOperators('NUM').then(function (data) {
                        if (data.length > 0) {
                            $scope.numericDropCtrlData.operators = data;
                            $scope.numericDropCtrlData.defaultOper = _.findWhere(data, {name: ">="})
                        }
                    });

                    var tempRows = angular.copy($scope.numericDropAttrs.inNumericRanges);
                    $scope.numericDropAttrs.inNumericRanges = [];

                    for (var i = 0; i < tempRows.length; i++) {
                        var row = tempRows[i];

                        if (!row.oper) {
                            row.value.min = cleanValOrDateStr(row.format, row.value.min);
                            row.value.max = cleanValOrDateStr(row.format, row.value.max);
                        }
                        else if (row.oper.name.indexOf('MISSING') > -1) {
                            // row is testing for MISSING - null out min/max
                            row.value.min = null;
                            row.value.max = null;
                        }
                        else {
                            // min or max has a value
                            if (row.value.min != null) {
                                row.value.min = vsdUtil.removeFormat(row.value.min);
                                row.value.min = parseFloat(row.value.min);
                                row.singleOperand = useFormatValueIfNecessary(row, "min", row.value.min);
                            }

                            if (row.value.max != null) {
                                row.value.max = vsdUtil.removeFormat(row.value.max);
                                row.value.max = parseFloat(row.value.max);
                                row.singleOperand = useFormatValueIfNecessary(row, "max", row.value.max);
                            }
                        }
                        rowValidity[i] = true;
                    }

                    // if we have more than one row remove any empty rows.
                    if (tempRows.length > 1)
                        tempRows = _.reject(tempRows, function (row) {
                            return row.oper == null && row.value && row.value.min == null && row.value.max == null;
                        });

                    $scope.numericDropAttrs.inNumericRanges = tempRows;
                    //Reset allowApply
                    $scope.numericDropCtrlData.allowApply = true;
                    //Calculate new rule number
                    calcNewRuleCountDisp();
                }
            });
            //endregion
        }]);


});

},
'vsd/scenario/decisionTree/decisionTreeSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('decisionTreeSvc',['d3UtilsSvc', function (d3UtilsSvc) {
		var returnToParentScreenHandler = null,
            treeSettings = null,
            inputColumns = null,
            inputRecordsCount = 0,
            selectedRules =  null,
            targetColumn = null,
            usedConditionColumns = null,
            entityKeyColumnName = null,
            partitionKeyColumns = [],
            distinctTargetValues = null,
            scenarioExecutionOrder = -1;

        return {
			setReturnToParentScreenHandler:    function (f) {
                returnToParentScreenHandler = f;
                return this;
			},
            getReturnToParentScreenHandler:    function () {
				return returnToParentScreenHandler;
			},
            setTreeSettings:    function (s) {
                treeSettings = s;
                return this;
			},
            getTreeSettings:    function () {
                if(treeSettings) {
				    return treeSettings;
                }
                else {
                    return this.getDefaultTreeSettings();
                }
			},
            setInputColumns:    function (c) {
                inputColumns = c;
                return this;
			},
            getInputRecordsCount:    function () {
                return inputRecordsCount;
			},
            setInputRecordsCount:    function (c) {
                inputRecordsCount = c;
                return this;
			},
            getInputColumns:    function () {
                return inputColumns;
			},
            setEntityKeyColumnName: function (c) {
                entityKeyColumnName = c;
                return this;
			},
            getEntityKeyColumnName:    function () {
                return entityKeyColumnName;
			},
            setPartitionKeyColumns: function (p) {
                partitionKeyColumns = p;
                return this;
			},
				getPartitionKeyColumns:    function () {
                return partitionKeyColumns;
			},
            setSelectedRules:    function (r) {
                selectedRules = r;
                return this;
			},
            getSelectedRules:    function () {
				return selectedRules;
			},
            setTargetColumn:    function (c) {
                targetColumn = c;
                return this;
			},
            getTargetColumn:    function () {
				return targetColumn;
			},
            setScenarioExecutionOrder:    function (e) {
                scenarioExecutionOrder = e;
                return this;
            },
            getScenarioExecutionOrder:    function () {
                return scenarioExecutionOrder;
            },
            setDistinctTargetValues:    function (v) {
                distinctTargetValues = v;
                return this;
			},
            getDistinctTargetValues:    function () {
				return distinctTargetValues;
			},
            setUsedConditionColumns:    function (u) {
                usedConditionColumns = u;
                return this;
			},
            getUsedConditionColumns:    function () {
				return usedConditionColumns;
			},
            getDefaultTreeSettings: function() {
                return {
                    binCount: 20,
                    leafSize: 1,
                    maxBranches: 2,
                    maxLevel: 6,
                    multiVar: false,       // Todo: set to true once we can build equivalent rules correctly for decision table
                    optLevel: false,
                    pruneTree: false,
                    useMissingVar: false,  // Todo: do not allow until we can incorporate missing rules for decision table
                    aggressiveSearch: false
                };
            },
            clear: function() {
                returnToParentScreenHandler = null;
                treeSettings = null;
                inputRecordsCount = 0;
                entityKeyColumnName = null;
                selectedRules =  null;
                targetColumn = null;
                distinctTargetValues = null;
                usedConditionColumns = null;
                scenarioExecutionOrder = -1;
            },
            createStatDisplayArray: function(scope,nodeData) {
                var displayArray = [];
               // console.log("nodeData=",nodeData);
                angular.forEach([
                    scope.resources.decisionTree_leafNodeStats_purityLbl_txt,
                    scope.resources.decisionTree_leafNodeStats_countLbl_txt
                ],function(statLbl) {
                    var statTxt =  "";
                    switch(statLbl) {
                        case scope.resources.decisionTree_leafNodeStats_purityLbl_txt:
                            statTxt += d3UtilsSvc.pctFmtFunc(d3.max(nodeData.classificationRatios));
                            break;
                        case scope.resources.decisionTree_leafNodeStats_countLbl_txt:
                            statTxt += d3UtilsSvc.integerFmtFunc(nodeData.dataRecs) +
                                " (" + d3UtilsSvc.pctFmtFunc(nodeData.dataRecs / scope.totalRecCnt) + ")"
                                ;
                            break;
                    }
                    displayArray.push({
                        statLbl: statLbl,
                        statTxt: statTxt
                    });
                });
                angular.forEach(nodeData.targetInfo, function (obj) {
                    if(obj.id !== '_ALL_') {
                        displayArray.push({
                            statLbl: obj.name,
                            statTxt: obj.formattedStats
                        });
                    }
                });
                return displayArray;
            }
		};
	}]);

});

},
'vsd/scenario/decisionTree/ScenarioDecisionTreeCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioDecisionTreeCtrl',
        ['$scope',
         'scenarioSvc',
         'vsdUtil',
         'dojoModules',
         'treeUtilSvc',
         'decisionTreeSvc',
         'scenarioInputTreeSvc',
         'messageSvc',
         '$timeout',
        function ($scope,
                  scenarioSvc,
                  vsdUtil,
                  dojoModules,
                  treeUtilSvc,
                  decisionTreeSvc,
                  scenarioInputTreeSvc,
                  messageSvc,
                  $timeout) {

            var FILTER_TYPE_ACCURACY = "A",
                FILTER_TYPE_POPULATION_PCT = "PP",
                FILTER_TYPE_POPULATION_CNT = "PC",
                treeData,
                varImportanceData,
                optionResourceMap,
                colUsedChange = false;

            $scope.DROP_MODE_TARGET_DROP = "targetDrop";
            $scope.DROP_MODE_CONDITION_DROP = "conditionDrop";
            $scope.DROP_MODE_SHOW_TREE = "showTree";
            $scope.CHART_VIEW_TREE = "tree";
            $scope.CHART_VIEW_IMPORTANCE = "importance";



            function fillOptionResourceMap() {
                optionResourceMap = {};
                optionResourceMap["maxBranches"] = $scope.resources.decisionTree_maxBranchesLbl_txt;
                optionResourceMap["maxLevel"] = $scope.resources.decisionTree_maxDepthLbl_txt;
                optionResourceMap["leafSize"] = $scope.resources.decisionTree_minRecordsPerRuleLbl_txt;
            }


            function createDecisionTreeAutoWithConfirm() {
                if($scope.treeOutputs.userModified) {
                    //TODO:  Add code to show confirmation box
                }
                else {
                    createDecisionTreeAutoWithRuleClear();
                }
            }

            function createDecisionTreeAutoWithRuleClear() {
                //Clear out selected nodes array since new leaf nodes will be generated
                $scope.treeOutputs.selRuleLeafNodes = [];
                //Run decision table generation
                createDecisionTreeAuto();
            }

            function createDecisionTreeAuto() {
                if($scope.treeInputs.targetColumn && $scope.treeInputs.usedConditionColumns.length) {
                    $scope.treeInputs.dropMode = $scope.DROP_MODE_SHOW_TREE;

                    //Ensure no settings are undefined before executing (this can happen when user deletes number to retype
                    if(treeOptionBlank()) {
                        return;
                    }

                    var targetVal = $scope.treeInputs.targetValue ? $scope.treeInputs.targetValue.value : null;

                    $scope.showLoadingMask($scope.resources.scenario_Controller_calculatingWaitLbl_txt);
                    //Call service to create decision tree structure

                    scenarioSvc.createDecisionTreeAuto($scope.treeInputs.targetColumn, targetVal, $scope.treeInputs.usedConditionColumns,$scope.treeInputs.lasrTreeOpts)
                        .then(function(results) {
                            treeData = results.rootNode;
                            varImportanceData = results.variableImportance;

                            if(treeData) {

                                $scope.counters.totalRecCnt = treeData.dataRecs;

                                //Clear any existing filters
                                $scope.clearAllFilters();

                                //Get target value stats
                                populateNodeTargetInfo(treeData);

                                //Get list of dominant target and purity values from treeData
                                var tgtAndPurityList = treeUtilSvc.collectFromRootToLeaves(treeData,getTargetAndPurityVals);
                                //Select distinct target value
                                var dominantTargetValStrs = _.uniq(_.pluck(tgtAndPurityList,'dominantTargetValue'));
                                var dominantTargetEntries = [];
                                angular.forEach(dominantTargetValStrs,function(dStr) {
                                    var idVal = dStr === $scope.resources.decisionTree_targetValsAllValues_txt ? "_ALL_" : dStr,
                                        nameVal;

                                    if ($scope.treeInputs.targetColumn.dataType === "CHARACTER" || idVal === "_ALL_") {
                                        nameVal = dStr;
                                    }
                                    else {
                                        nameVal = $scope.formatTargetValue(dStr);
                                    }

                                    dominantTargetEntries.push({name: nameVal, id: idVal});
                                });
                                // sort entries by id
                                sortTargetEntries(dominantTargetEntries, 'id');

                                //Add an "All Values" entry
                                dominantTargetEntries = _.union([
                                    {id: '_ALL_', name: $scope.resources.decisionTree_targetValsAllValues_txt}
                                ], dominantTargetEntries);

                                $scope.treeOutputs.dominantTargetVals = dominantTargetEntries;
                                $scope.treeOutputs.filterTargetVal = dominantTargetEntries[0];

                                //Get the min and max purity
                                var min = 100,
                                    max = 0;
                                angular.forEach(tgtAndPurityList, function (entry) {
                                    var currPurity = entry.purity * 100;
                                    if (currPurity < min) {
                                        min = currPurity;
                                    }
                                    else if (currPurity > max) {
                                        max = currPurity;
                                    }
                                });
                                $scope.treeOutputs.purityRange = [min, max];

                                //Store list of columns tree selected in scope
                                $scope.treeOutputs.treeSelectedColNames = getTreeSelectedColumnNames();

                                //Initialize counters
                                setCounters(0);

                                //Set data for current graph view
                                if ($scope.treeOutputs.chartView === $scope.CHART_VIEW_TREE) {
                                    $scope.treeOutputs.treeData = treeData;
                                    $scope.treeOutputs.varImportanceData = [];
                                    $scope.treeOutputs.varImportanceUsedList = [];
                                    updateTargetCounters();
                                    $scope.updateFiltersChanged();

                                }
                                else {
                                    $scope.treeOutputs.treeData = null;
                                    $scope.treeOutputs.varImportanceData = varImportanceData;
                                    $scope.treeOutputs.varImportanceUsedList = $scope.treeOutputs.treeSelectedColNames;
                                }

                                var importantCount = _.countBy($scope.treeOutputs.varImportanceData, function (entry) {
                                    return entry.Importance > 0 ? 'important' : 'irrelevant';
                                });

                                var varImpTitlNbrLimit = importantCount > $scope.treeOutputs.maxVarImportanceDisplayVars
                                    ? dojoModules.string.substitute($scope.resources.variableImportanceChart_titleTopLimitLbl_txt, [$scope.treeOutputs.maxVarImportanceDisplayVars])
                                    : "";
                                $scope.treeOutputs.varImportanceChartTitle = dojoModules.string.substitute($scope.resources.variableImportanceChart_titleLbl_txt,
                                    [$scope.treeInputs.targetColumn.label, varImpTitlNbrLimit]);

                                $scope.hideLoadingMask();
                            }
                            //Empty tree - show placeholder text
                            else {
                                $scope.treeOutputs.treeData = null;
                                $scope.treeOutputs.varImportanceData = null;
                                $scope.clearConditionCols();
                                messageSvc.showInfo($scope.resources.decisionTree_emptyTreeMessage_txt);
                                $scope.hideLoadingMask();
                            }

                            function populateNodeTargetInfo(rootNode) {
                                //Todo: move this to the service side
                                // Populate targetInfo stats for each node in the tree in format consumable by decision tree
                                // Makes sure values are listed in same order for each node

                                var rootTargetInfo = getTargetInfo(rootNode),
                                totalCnt = rootNode.dataRecs;
                                var nodes = treeUtilSvc.collectFromRootToLeaves(rootNode, getTargetInfo);

                                function getTargetInfo(node) {
                                    var targetInfo = [{
                                        id: "_ALL_",
                                        //name: $scope.resources.decisionTree_targetValsAllValues_txt,
                                        name: $scope.resources.decisionTree_leafNodeStats_countLbl_txt,
                                        count: node.dataRecs,
                                        ratio: 1
                                    }];

                                    var addVarInfo = function(index) {
                                        // There should be a one-to-one correspondence between targetLevelNames and classificationRatios entries
                                        var id = node.targetLevelNames[index],
                                            ratio = node.classificationRatios[index];
                                        if (ratio == null) {
                                            ratio = 0;
                                        }
                                        targetInfo.push({
                                            id: id,
                                            name: $scope.formatTargetValue(id),
                                            ratio: ratio,
                                            rawCount: ratio * node.dataRecs,
                                            count: Math.round(ratio * node.dataRecs)
                                        });
                                    };

                                    if (node.targetLevelNames && node.classificationRatios) {
                                        if (rootTargetInfo && rootTargetInfo.length > 0) {
                                            // Add items in same order as on root node
                                            angular.forEach(rootTargetInfo, function(val) {
                                                var pos = node.targetLevelNames.indexOf(val.id);
                                                if (pos >= 0) {
                                                    addVarInfo(pos);
                                                }
                                            });
                                        }
                                        else {
                                            // Either this is the root node or the root node doesn't have stats
                                            for (var i = 0; i < node.targetLevelNames.length && i < node.classificationRatios.length; i++) {
                                                addVarInfo(i);
                                            }
                                        }
                                    }
                                    angular.forEach(targetInfo, function(info) {
                                        info.formattedStats = vsdUtil.applySASNumericFormat(info.count, vsdUtil.defaultFormats.integer) +
                                            "  (" + vsdUtil.applySASNumericFormat(info.ratio, vsdUtil.defaultFormats.percentage) + ")";
                                        info.formattedStatsTotal = vsdUtil.applySASNumericFormat(info.count, vsdUtil.defaultFormats.integer) +
                                            "  (" + vsdUtil.applySASNumericFormat((info.count / totalCnt), vsdUtil.defaultFormats.percentage) + ")";
                                    });
                                    node.targetInfo = targetInfo;
                                    return targetInfo;
                                }
                            }
                        });
                }
            }

            function sortTargetEntries(list, field) {
                //Sort entries
                vsdUtil.sortByFields(list, [
                    {order: 'A', field: field}
                ], getTargetAsNumber);

                function getTargetAsNumber(targetListVal) {
                    if ($scope.treeInputs.targetColumn.dataType === vsmCommonConst.DATA_TYPE_NUMERIC && typeof targetListVal === 'string') {
                        return parseFloat(targetListVal);
                    }
                    else {
                        return targetListVal;
                    }
                }
            }


            function getTreeSelectedColumnNames() {
                return  _.uniq(treeUtilSvc.collectFromRootToLeaves(treeData,collectColumnName));

                function collectColumnName(node) {
                    if(node.name !== $scope.treeInputs.targetColumn.name) {
                        return node.name;
                    }
                    else {
                        return null;
                    }


                }
            }


            function getRelatedRows(node) {

                if (!node) {
                    $scope.clearPreviewData();
                }
                else {
                    // Populate activeRule used by ScenarioCtrl with selected decision tree rule, which getPreviewData uses to get the detail data
                    $scope.data.activeRule = node.rule;
                    $scope.data.activeRule.viewMode = 'treemode';
                    $scope.data.activeRule.treeInputs=$scope.treeInputs;
                    $scope.getPreviewData();
                }
            }


            function treeOptionBlank() {
                for(var propName in $scope.treeInputs.lasrTreeOpts) {
                    if($scope.treeInputs.lasrTreeOpts.hasOwnProperty(propName)) {
                        var opt = $scope.treeInputs.lasrTreeOpts[propName];
                        if(!opt && opt !== 0 && opt !== false) {
                            messageSvc.showError(dojoModules.string.substitute($scope.resources.decisionTree_invalidOptionMsg_txt,[optionResourceMap[propName]]));
                            return true;
                        }
                    }
                }
                return false;
            }

            function getTargetAndPurityVals(node) {
                if(node.type === "LEAF") {
                    return {
                        dominantTargetValue: node.dominantTargetValue,
                        purity: _.max(node.classificationRatios)
                    };
                }
                else {
                    return null;
                }

            }

            function removeUsedCondColByName(name) {
                //Remove column from used condition columns
                $scope.treeInputs.usedConditionColumns =  _.reject($scope.treeInputs.usedConditionColumns,function(entry) {
                    return entry.name === name;
                });
                //Get reference to available column and set it to unused
                var availCol = _.findWhere($scope.treeInputs.availCondCols,{name: name});
                if(availCol)  {
                    if(availCol.dropNodeId) {
                        vsdUtil.unmarkRemovedColumn(availCol.dropNodeId);
                    }
                    availCol.selected = false;
                }
                $scope.treeOutputs.treeSelectedColNames = _.reject($scope.treeOutputs.treeSelectedColNames, function(entry) {
                    return entry === name;
                });
                //Recalculate available columns
                setAvailCondCols();
            }

            function setAvailCondCols() {
                if($scope.treeInputs.targetColumn) {
                    $scope.treeInputs.availCondCols = _.reject($scope.treeInputs.allInputCondCols,function(currCol) {
                        return currCol.name === $scope.treeInputs.targetColumn.name;
                    });
                }
                else {
                    $scope.treeInputs.availCondCols = $scope.treeInputs.allInputCondCols;
                }

            }

            function addRemoveConditionColumn(col) {
                //Add to usedConditionColumns if it isn't already there
                if(col.selected) {
                    var colPresent = _.find($scope.treeInputs.usedConditionColumns,function(currCol) {
                        return currCol.name === col.name;
                    });
                    if(!colPresent) {
                        $scope.treeInputs.usedConditionColumns.push(col);
                    }
                }
                else {
                    removeUsedCondColByName(col.name);
                }
            }

            function removeColumnFromTree(colName) {
                removeUsedCondColByName(colName);
                createDecisionTreeAutoWithConfirm();
            }

            function leafNodeSelected(compNode) {
                return _.any($scope.treeOutputs.selRuleLeafNodes,function(node) {
                    return compNode.id === node.id;
                });
            }

            function sortTopLevelConditionsByCol(cond) {
                cond.childConditions.sort(function(a,b) {
                    var condAColName = getCondColName(a),
                        condBColName = getCondColName(b);

                    if(condAColName < condBColName) {
                        return -1;
                    }
                    else if(condAColName > condBColName) {
                        return 1;
                    }
                    else {
                        return 0;
                    }

                    function getCondColName(cond) {
                        if(cond.childConditions.length === 0) {
                            return cond.operand1ColumnInfo.name;
                        }
                        else {
                            return cond.childConditions[0].operand1ColumnInfo.name;
                        }
                    }
                });
            }

            function formatInputColumns(inputColumns,partitionKeyColumns) {
                //Prepare data for input column list on columns tab
                var formattedCols = scenarioSvc.formatScenarioInputCols(inputColumns, partitionKeyColumns);
                //Prepare data for available/used columns list
                return _.reject(formattedCols,function(col) {
                    return  $scope.treeInputs.targetColumn && $scope.treeInputs.targetColumn.name === col.name;
                });
            }

            function setCounters(filtered) {
                $scope.counters =  {
                    totalRecCnt: $scope.counters.totalRecCnt,
                    filteredRecCnt: filtered,
                    popCoveragePct: filtered/$scope.counters.totalRecCnt
                };
            }

            function updateTargetCounters() {
                // Update cumulative target value stats
                var targetCounts = [];
                if ($scope.treeOutputs.treeData) {
                    angular.forEach($scope.treeOutputs.treeData.targetInfo, function (obj) {
                        // Get number of selected rules having selected target value
                        var cnt = _.reduce($scope.treeOutputs.selRuleLeafNodes, function (count, currEntry) {
                            var leafInfo = _.findWhere(currEntry.targetInfo, {id: obj.id});
                            return count + leafInfo.count;
                        }, 0);
                        targetCounts.push({
                            id: obj.id,
                            name: obj.name,
                            filteredRecCnt: cnt,
                            totalRecCnt: obj.count,
                            formattedStats: vsdUtil.applySASNumericFormat(cnt, vsdUtil.defaultFormats.integer) +
                                "  (" + vsdUtil.applySASNumericFormat((cnt / obj.count), vsdUtil.defaultFormats.percentage) + ")"
                        });
                    });
                }
                $scope.targetCounters = targetCounts;
            }

            function collectCondColsForRules(rules) {
                var condCols = [];
                //Get condition columns from each rule
                angular.forEach(rules,function(rule) {
                    treeUtilSvc.collectFromRootToLeaves(rule.condition,collectCondCols,"childConditions",condCols);
                });
                //De-dupe and return them
                return _.uniq(condCols,false,function(col) {
                    return col.name;
                });

                function collectCondCols(cond) {
                    if(cond.operand1ColumnInfo.name) {
                        condCols.push(cond.operand1ColumnInfo);
                    }
                }
            }

            function isUsedColumn(col) {
                return  ($scope.treeInputs.targetColumn && col.name === $scope.treeInputs.targetColumn.name) ||
                       _.findWhere($scope.treeInputs.usedConditionColumns,{name: col.name});
            }

            function addConditionColumn(colElement,colData) {
                colData.selected = true;
                vsdUtil.markDroppedColumn(colElement,colData);
                    $scope.treeInputs.usedConditionColumns.push(colData);
                }


            $scope.lists = {
                statsFilterTypes: [
                    {id:  FILTER_TYPE_ACCURACY, name: $scope.resources.decisionTree_filterTypeAccuracyPctLbl_txt},
                    {id:  FILTER_TYPE_POPULATION_PCT, name: $scope.resources.decisionTree_filterTypePopulationPctLbl_txt},
                    {id:  FILTER_TYPE_POPULATION_CNT, name: $scope.resources.decisionTree_filterTypePopulationCntLbl_txt}
                ]
            };

            $scope.counters = {
                totalRecCnt: 0,
                filteredRecCnt: 0
            };

            $scope.targetCounters = [];

            $scope.treeInputs = {
                dropMode: $scope.DROP_MODE_TARGET_DROP,
                targetColumn: null,
                targetValue: null,
                allTargetValues: [],
                prevTargetColumn: null,
                allInputCondCols: null,
                inputListCols: null,
                inputColsFilter: "",
                inputColumnsId: "treeId",
                inputColsGetChildren: scenarioInputTreeSvc.inputColsGetChildren,
                inputColsMayHaveChildren: scenarioInputTreeSvc.inputColsMayHaveChildren,
                availCondCols:  null,
                usedConditionColumns: [],
                showOnlySelected: false,
                lasrTreeOpts: null,
                lasrTreeOptsLimits: {
                    binsMin: 2,
                    binsMax: 10,
                    confLevelMin: 0,
                    confLevelMax: 100,
                    maxBranchesMin: 2,
                    maxBranchesMax: 10,
                    maxLevelMin: 3,
                    maxLevelMax: 10,
                    leafSizeMin: 1
                }
            };
            $scope.treeInputs.filterInputColumns = scenarioInputTreeSvc.filterInputColumns($scope, "treeInputs.inputColsFilter");
            $scope.treeInputs.inputGridColDef =  scenarioInputTreeSvc.getInputTreeDef(isUsedColumn);

            $scope.treeOutputs = {
                treeData: null,
                varImportanceData: [],
                varImportanceUsedList: [],
                chartView: $scope.CHART_VIEW_TREE,
                showNodeDetail: false,
                selectedNode: null,
                selectedNodeDOM: null,
                selectedNodeCoords: null,
                selectedLeafNode: {
                    show: false,
                    nodeStats: [],
                    displaySplitConds: [],
                    targetValue: null
                },
                selSplitCondition: {
                    show: false,
                    domNode: null,
                    splitValues: [],
                    varLabel: null
                },
                dominantTargetVals: [],
                filterTargetVal: null,
                purityRange: [0,100],
                filtersChanged: false,
                statsFilter: {
                    constraints: {
                        min: 0,
                        max: 100
                    },
                    min: 0,
                    max: 100,
                    type: $scope.lists.statsFilterTypes[0]
                },
                selRuleLeafNodes: [],
                allRulesSelected: null,
                userModified: false,
                addRulesButtonLabel: $scope.resources.decisionTree_addZeroRulesToParentObject_txt,
                treeSelectedColNames: [],
                maxVarImportanceDisplayVars: 20,
                varImportanceChartTitle: null
            };

            $scope.clearConditionCols = function() {
                //Unmark all used condition columns
                angular.forEach($scope.treeInputs.usedConditionColumns, function (col) {
                    if(col.dropNodeId) {
                        vsdUtil.unmarkRemovedColumn(col.dropNodeId);
                    }
                });
                $scope.treeInputs.usedConditionColumns = [];
                $scope.treeInputs.dropMode = $scope.DROP_MODE_CONDITION_DROP;
                $scope.treeOutputs.selRuleLeafNodes = [];
                $scope.treeOutputs.addRulesButtonLabel = $scope.resources.decisionTree_addZeroRulesToParentObject_txt;
                $scope.clearPreviewData();
                $scope.clearAllFilters();
            };

            $scope.formatSplitCondition = function(splitCond,format) {
                var colName = splitCond.parentVarName ? splitCond.parentVarName : splitCond.colName;
                var col = _.findWhere($scope.treeInputs.usedConditionColumns,{name: colName});
                //If dealing with leaf node
                if(!col) {
                    col = $scope.treeInputs.targetColumn;
                }
                if(!splitCond.nominalSplitValues) {
                    var min, max;
                    min = vsdUtil.applySASNumericFormat(splitCond.lowerBounds,format);
                    max = vsdUtil.applySASNumericFormat(splitCond.upperBounds,format);
                    switch(splitCond.rangeType) {
                        case "L":
                            return "< " + max;
                        case "M":
                            return ">= " + min  + " " + $scope.resources.app_rangeConjunctionLbl_txt + " " + "< " + max;
                        case "U":
                            return ">= " + min;
                    }
                }
                else {
                    var nomRetVal = "";
                    angular.forEach(splitCond.nominalSplitValues,function(nomVal){
                        if(nomVal.trim().length === 0) {
                            nomRetVal += $scope.resources.decisionTree_missingLbl_txt + ", ";
                        }
                        else if(col.editControlType && col.editControlType.name === "BOOLEAN") {
                            switch(nomVal.trim()) {
                                case '0':
                                    nomRetVal += $scope.resources.app_falseLbl_txt;
                                    break;
                                case '1':
                                    nomRetVal += $scope.resources.app_trueLbl_txt;
                                    break;
                            }
                            nomRetVal += ", ";
                        }
                        else {
                            nomRetVal += nomVal + ", ";
                        }

                    });
                    nomRetVal = vsdUtil.dropRight(nomRetVal,2);

                    return nomRetVal;
                }
            };

            $scope.decisionTreeOnDrop = function(source,cols,copy) {
                if($scope.treeInputs.dropMode === $scope.DROP_MODE_TARGET_DROP) {
                    $scope.treeInputs.targetColumn = cols[0].custNodeData;
                    vsdUtil.markDroppedColumn(cols[0],cols[0].custNodeData);
                    $scope.onTargetColumnSelect();
                }
                else {
                    angular.forEach(cols,function(col) {
                        //Get matching column and mark as in use
                        if(col.custNodeData.id !== $scope.treeInputs.targetColumn.id) {
                            var match = _.findWhere($scope.treeInputs.availCondCols,{name: col.custNodeData.name});
                            addConditionColumn(col,match);
                        }
                    });

                    //Build decision tree
                    createDecisionTreeAutoWithConfirm();
                }
            };

            $scope.decisionTreeAcceptDrop = function(source, cols) {
                var goodNodes = true;
	             // Make sure we are in the Decision Table View and we are viewing the Tree
	             if ( !$scope.treeInputs) {
		             goodNodes = false;
                 }
	             else {
		             //Make sure the custNodeData property on all nodes
		             angular.forEach(cols,function(col) {
			             if(!col.custNodeData.name) {
				             goodNodes = false;
			             }
		             });
		             if(goodNodes) {
			             //Reject if uninitialized scenario output columns in drop
                         for(var i = 0; i < cols.length; i++) {
                             var col  = cols[i].custNodeData;
                             if(!scenarioSvc.allowScenarioOutputColumnDrop(col,decisionTreeSvc.getScenarioExecutionOrder(),$scope.resources)) {
                                 return false;
                             }
                         }
                         //Reject if waiting for target
			             if($scope.treeInputs.dropMode === $scope.DROP_MODE_TARGET_DROP) {
                             //and there is more than on column in drop
				             if(cols.length > 1) {
                                 messageSvc.showInfo($scope.resources.decisionTree_targetColumnSingleOnlyMsg_txt);
				                 return false;
                             }
                             //And there are too many levels in the target variable
                             if(cols[0].custNodeData.distinctCount > vsmCommonConst.DECISION_TREE_MAX_TARGET_LEVELS) {
                                 messageSvc.showInfo(dojoModules.string.substitute($scope.resources.decisionTree_tooManyTargetLevelsMsg_txt,[cols[0].custNodeData.label]));
                                 return false;
                             }
                             //And column has only one level
                             if(cols[0].custNodeData.distinctCount === 1) {
                                messageSvc.showInfo($scope.resources.decisionTree_cannotUseOneLevelTargetColumnMsg_txt);
                                 return false;
                             }
                             //And column is completely null
                             if(cols[0].custNodeData.distinctCount === 0) {
                                 messageSvc.showInfo($scope.resources.decisionTree_cannotUseBlankColumnMsg_txt);
                                 return false;
                             }
			             }
                         else {
                             //Check for too many distinct levels
                             var tooManyNominalLevels = _.some(cols,function(col) {
                                  return col.custNodeData.dataType === vsmCommonConst.DATA_TYPE_CHAR &&
                                         col.custNodeData.distinctCount > vsmCommonConst.DECISION_TREE_MAX_NOMINAL_LEVELS;
                              });
                             if(tooManyNominalLevels) {
                                 messageSvc.showInfo($scope.resources.decisionTree_tooManyNominalLevelsMsg_txt);
                                 return false;
                             }
                             var enoughLevels = _.some(cols,function(col) {
                                return col.custNodeData.distinctCount > 1;
                             });
                             if(!enoughLevels) {
                                messageSvc.showInfo($scope.resources.decisionTree_notEnoughLevelsMsg_txt);
                                return false;
                             }
                         }
		             }
	             }
                return goodNodes;
            };

            $scope.filterAvailCondColCBList = function(col) {
                var zeroLenFilterStr =   $scope.inputColsProps.inputColsFilter.length === 0,
                    filterStr;
                if(zeroLenFilterStr && !$scope.treeInputs.showOnlySelected) {
                    return true;
                }
                else {
                    filterStr =  $scope.inputColsProps.inputColsFilter.toLowerCase();
                    return  (!$scope.treeInputs.showOnlySelected || col.selected) &&
                            (zeroLenFilterStr || col.label.toLowerCase().indexOf(filterStr) !== -1);
                }
            };

            $scope.addRemoveConditionColumn = function(col) {
                //Add to usedConditionColumns if it isn't already there
                if(col.selected) {
                    var colPresent = _.find($scope.treeInputs.usedConditionColumns,function(currCol) {
                        return currCol.name === col.name;
                    });
                    if(!colPresent) {
                        $scope.treeInputs.usedConditionColumns.push(col);
                    }
                }
                else {
                    removeUsedCondColByName(col.name);
                }
            };

            $scope.onTargetColumnSelect = function(refreshInputGrid) {
                var isChangedTargetCol =  $scope.treeInputs.prevTargetColumn &&  $scope.treeInputs.targetColumn !==  $scope.treeInputs.prevTargetColumn;
                if($scope.treeInputs.targetColumn && (!$scope.treeInputs.prevTargetColumn || isChangedTargetCol)) {

                    $scope.showLoadingMask($scope.resources.scenario_Controller_calculatingWaitLbl_txt);

                    //Check to make sure column doesn't have too many levels
                    if($scope.treeInputs.targetColumn.distinctCount > vsmCommonConst.DECISION_TREE_MAX_TARGET_LEVELS) {
                        //Show message
                        messageSvc.showInfo(dojoModules.string.substitute($scope.resources.decisionTree_tooManyTargetLevelsMsg_txt,[$scope.treeInputs.targetColumn.label]));
                        //Reset target column and mode, as appropriate
                        $scope.treeInputs.targetColumn = $scope.treeInputs.prevTargetColumn;
                        if(!$scope.treeInputs.targetColumn) {
                            $scope.treeInputs.dropMode = $scope.DROP_MODE_TARGET_DROP;
                        }
                        return;
                    }

                    if($scope.treeInputs.dropMode === $scope.DROP_MODE_TARGET_DROP) {
                        $scope.treeInputs.dropMode = $scope.DROP_MODE_CONDITION_DROP;
                    }
                    //Unmark the old target column, if it exists
                    if($scope.treeInputs.prevTargetColumn && $scope.treeInputs.prevTargetColumn.dropNodeId) {
                        vsdUtil.unmarkRemovedColumn($scope.treeInputs.prevTargetColumn.dropNodeId);
                    }

                    /*$scope.dtPrimitives.inputColumns=[];
                    $scope.inputTree.generateInputColumns();*/

                    //Save current column as prev column for later changes
                    $scope.treeInputs.prevTargetColumn =  $scope.treeInputs.targetColumn;

                    //Clear selected target value
                    $scope.treeInputs.targetValue = null;

                    //Remove from list of available and used condition columns, if present
                    removeUsedCondColByName($scope.treeInputs.targetColumn.name);
                    setAvailCondCols();

                    // Populate target value list
                    var colValues = [];
                    scenarioSvc.getUniqueValues($scope.treeInputs.targetColumn.name, "")
                        .then(function (values) {
                            angular.forEach(values.uniqueValues, function (val) {
                                colValues.push({
                                    value: val,
                                    label: ($scope.treeInputs.targetColumn.dataType === vsmCommonConst.DATA_TYPE_CHAR) ?
                                        val : vsdUtil.applySASNumericFormat(val, $scope.treeInputs.targetColumn.columnFormat)
                                });
                            });
                            //Sort entries by value
                            sortTargetEntries(colValues, 'value');
                            $scope.treeInputs.allTargetValues = colValues;
                            $scope.hideLoadingMask();
                        });

                    if ($scope.treeInputs.dropMode === $scope.DROP_MODE_SHOW_TREE) {
                        //Build decision tree
                        createDecisionTreeAutoWithConfirm();
                    }
                }

                if(!$scope.treeInputs.inputListCols || (refreshInputGrid && isChangedTargetCol)) {
                    $scope.treeInputs.inputListCols = [];
                    $scope.treeInputs.inputListCols = scenarioInputTreeSvc.generateInputColumns(decisionTreeSvc.getInputColumns(),
                        $scope.treeInputs.usedConditionColumns,
                        decisionTreeSvc.getPartitionKeyColumns(),
                        $scope.resources);
                }
            };


            $scope.onTargetValueSelect = function () {
                if ($scope.treeInputs.dropMode === $scope.DROP_MODE_SHOW_TREE) {
                    //Build decision tree
                    createDecisionTreeAutoWithConfirm();
                }
            };

            //Any time rules array changes, retrieve new coverage stats
            $scope.$watchCollection('treeOutputs.selRuleLeafNodes',function() {
                var popCount = _.reduce($scope.treeOutputs.selRuleLeafNodes,function(total, currEntry) { return total + currEntry.dataRecs;},0);
                setCounters(popCount);
                updateTargetCounters();
            });

            $scope.getInputGridRMBMenuItems = function(rows) {
                var me = this,
                    condsContained = 0,
                    matchesTargetColumn = false,
                    containsAnyCondCols = false,
                    containsAllCondCols = false,
                    uninitializedSCCols = false,
                    blankColumn = false,
                    tooManyNominalLevels = false,
                    enoughLevels =  true,
                    oneLevelTarget = rows[0].data.distinctCount === 1,
                    tooManyLevelsTarget = rows[0].data.distinctCount > vsmCommonConst.DECISION_TREE_MAX_TARGET_LEVELS,
                    allowScenarioColumnDrop = scenarioSvc.allowScenarioOutputColumnDrop(rows[0].data,decisionTreeSvc.getScenarioExecutionOrder(),$scope.resources,true);

                //Determine how many condition columns in selection
                angular.forEach(rows,function(row) {
                    if(_.findWhere($scope.treeInputs.usedConditionColumns,{id: row.data.id})) {
                        condsContained++;
                    }
                    else if($scope.treeInputs.targetColumn && $scope.treeInputs.targetColumn.id === row.data.id) {
                        matchesTargetColumn = true;
                    }
                    if(!tooManyNominalLevels) {
                        tooManyNominalLevels = row.data.dataType === vsmCommonConst.DATA_TYPE_CHAR &&
                                               row.data.distinctCount > vsmCommonConst.DECISION_TREE_MAX_NOMINAL_LEVELS;
                    }
                    if(!blankColumn) {
                        if(row.data.distinctCount === 0) {
                            blankColumn = true;
                        }
                    }

                });
                containsAnyCondCols = condsContained > 0;
                containsAllCondCols = condsContained === rows.length;
                uninitializedSCCols = _.any(rows,function(row) {
                   return !scenarioSvc.allowScenarioOutputColumnDrop(row.data,decisionTreeSvc.getScenarioExecutionOrder(),$scope.resources,true);
                });
                enoughLevels = _.some(rows,function(row) {
                    return row.data.distinctCount > 1;
                });

                var inputRowMenuItems = [];
                //Use as target column menu item
                inputRowMenuItems.push({
                        label: $scope.resources.decisionTree_rmbMenuUseAsTargetColumnLbl_txt,
                        disabled: rows.length > 1 || matchesTargetColumn || !allowScenarioColumnDrop || tooManyLevelsTarget || oneLevelTarget || blankColumn,
                        onClick: function(evt) {
                            if(rows[0].data.columnType === "WO" || scenarioSvc.allowScenarioOutputColumnDrop(rows[0].data,decisionTreeSvc.getScenarioExecutionOrder(),$scope.resources)) {
                                $scope.treeInputs.targetColumn = rows[0].data;
                                $scope.onTargetColumnSelect();
                                vsdUtil.markDroppedColumn(rows[0].element,$scope.treeInputs.targetColumn);
                                $scope.$apply(); //Must $apply here since event originates in Dojo
                            }
                        }
                });
                //Add as condition column menu item
                inputRowMenuItems.push({
                        label: $scope.resources.decisionTree_rmbMenuAddAsConditionColumnLbl_txt,
                        disabled: !$scope.treeInputs.targetColumn || containsAllCondCols || uninitializedSCCols || tooManyNominalLevels || matchesTargetColumn || !enoughLevels,
                        onClick: function(evt) {
                            var allowAdd = true;
                            //For each column in list, add it
                            angular.forEach(rows, function (row) {
                                var match = _.findWhere($scope.treeInputs.availCondCols,{name: row.data.name});
                                if(match) {
                                    if(match.columnType === "SO" && !scenarioSvc.allowScenarioOutputColumnDrop(match,decisionTreeSvc.getScenarioExecutionOrder(),$scope.resources)) {
                                        allowAdd = false;
                                    }
                                    if(allowAdd) {
                                        addConditionColumn(row.element,match);
                                    }
                                }
                            });
                            if(allowAdd) {
                                //Run decision tree
                                createDecisionTreeAutoWithConfirm();
                                $scope.$apply(); //Must $apply here since event originates in Dojo
                            }
                        }
                });
                inputRowMenuItems.push({
                        label: $scope.resources.decisionTree_rmbMenuRemoveFRomConditionColumnsLbl_txt,
                        disabled: !containsAnyCondCols,
                        onClick: function(evt) {
                            //For any in used condition columns, delete
                            angular.forEach(rows, function (row) {
                                removeUsedCondColByName(row.data.name);
                            });
                            //Run decision tree
                            createDecisionTreeAutoWithConfirm();
                            $scope.$apply(); //Must $apply here since event originates in Dojo
                        }
                });

                return inputRowMenuItems;
            };

            $scope.showNodeData = function(nodeData, domNode) {
                if (!nodeData) {
                    return;
                }
                var col = _.findWhere($scope.treeInputs.usedConditionColumns, {name: nodeData.name});
                //Create node level properties
                var displayNodeData = {
                    columnLabel: col.label,
                    showStats: $scope.treeInputs.targetColumn.dataType === "NUMERIC",
                    children: [],
                    purity: vsdUtil.applySASNumericFormat(_.max(nodeData.classificationRatios), vsdUtil.defaultFormats.percentage),
                    mean: vsdUtil.applySASNumericFormat(nodeData.mean,vsdUtil.defaultFormats.number),
                    std:  vsdUtil.applySASNumericFormat(nodeData.std,vsdUtil.defaultFormats.number),
                    missingPct: vsdUtil.applySASNumericFormat(nodeData.missing/$scope.counters.totalRecCnt, vsdUtil.defaultFormats.percentage),
                    x: nodeData.absX,
                    y: nodeData.absY,
                    targetInfo: nodeData.targetInfo
                };
                //Set split conditions display values
                angular.forEach(nodeData.children,function(child) {
                    displayNodeData.children.push({
                        id: child.id,
                        splitCond: $scope.formatSplitCondition(child,col.columnFormat),
                        splitCondList: child.nominalSplitValues || [],
                        totalRecCnt: $scope.counters.totalRecCnt,
                        dataRecs: child.dataRecs
                    });

                });
                //Trigger dialog
                $scope.treeOutputs.showNodeDetail = true;
                //Set selected node
                $scope.treeOutputs.selectedNode =  displayNodeData;
                //Assign dom node
                $scope.treeOutputs.selectedNodeDOM = domNode;



            };

            $scope.closeNodeData = function() {
                $scope.treeOutputs.showNodeDetail = false;
                $scope.treeOutputs.selectedNode = null;
                $scope.treeOutputs.selectedNodeDOM = null;
            };

            $scope.showLeafNodeData = function(nodeData,splitConds,domNode) {
                var nodeStats = decisionTreeSvc.createStatDisplayArray($scope,nodeData);
                //If target data type non-character, drop numeric statistics
                if($scope.treeInputs.targetColumn.dataType === "CHARACTER") {
                    nodeStats = _.reject(nodeStats,function(entry) {
                        _.contains([$scope.resources.decisionTree_leafNodeTargetMeanLbl_txt,$scope.resources.decisionTree_leafNodeTargetStddevLbl_txt],entry.statLbl);
                    });
                }
                //Create split condition display list
                var displaySplitConds = [];
                for(var idx in splitConds) {
                    var splitCond = splitConds[idx];
                    displaySplitConds.push({
                        condLbl: _.findWhere($scope.treeInputs.usedConditionColumns, {name: splitCond.colName}).label,
                        condTxt: $scope.formatSplitCondition(splitCond)
                    });
                }

                $scope.treeOutputs.selectedLeafNode.nodeStats = nodeStats;
                $scope.treeOutputs.selectedLeafNode.displaySplitConds =  displaySplitConds;
                $scope.treeOutputs.selectedLeafNode.targetValue = $scope.formatTargetValue(nodeData.dominantTargetValue);
                $scope.treeOutputs.selectedLeafNode.domNode = domNode;
                $scope.$apply();
                $timeout(function(){
                    $scope.treeOutputs.selectedLeafNode.show = true;
                });


            };

            $scope.closeLeafNodeData = function() {
                $scope.treeOutputs.selectedLeafNode.show = false;
            };

            $scope.showSplitCondTooltip = function(levelData,domNode) {
                $scope.treeOutputs.selSplitCondition.domNode = domNode;
                $scope.treeOutputs.selSplitCondition.varLabel = levelData.levelVarTxt;
                $scope.treeOutputs.selSplitCondition.splitValues = levelData.splitCondTxt.split(",");
                $scope.treeOutputs.selSplitCondition.show = true;
            };

            $scope.closeSplitCondTooltip = function() {
                $scope.treeOutputs.selSplitCondition.show = false;
                $scope.treeOutputs.selSplitCondition.domNode = null;
            };

            $scope.processHighlightRule = function(node) {
               getRelatedRows(node);
            };

            $scope.processLeafNodeSelection = function(node,selected) {
                //Insert rule
                if(selected) {
                    $scope.treeOutputs.selRuleLeafNodes.push(node);
                }
                //Process rule delete
                else {
                    $scope.treeOutputs.selRuleLeafNodes = _.reject($scope.treeOutputs.selRuleLeafNodes,function(currNode) {
                        return currNode.id === node.id;
                    });
                }
                //Update button label
                if($scope.treeOutputs.selRuleLeafNodes.length) {
                    $scope.treeOutputs.addRulesButtonLabel = dojoModules.string.substitute($scope.resources.decisionTree_addXRulesToParentObject_txt,[$scope.treeOutputs.selRuleLeafNodes.length]);
                }
                else {
                    $scope.treeOutputs.addRulesButtonLabel = $scope.resources.decisionTree_addZeroRulesToParentObject_txt;
                }
            };

            $scope.setAllRulesSelected = function (selected) {
                $scope.treeOutputs.allRulesSelected = (selected === true) ? true : false;
            };

            $scope.processLeafNodeSelectAll = function (selectedNodes) {
                var rules = _.pluck(selectedNodes, "rule");
                decisionTreeSvc.setSelectedRules(rules);
                $scope.treeOutputs.selRuleLeafNodes = selectedNodes;

                //Update button label
                if ($scope.treeOutputs.selRuleLeafNodes.length) {
                    $scope.treeOutputs.addRulesButtonLabel = dojoModules.string.substitute($scope.resources.decisionTree_addXRulesToParentObject_txt, [$scope.treeOutputs.selRuleLeafNodes.length]);
                }
                else {
                    $scope.treeOutputs.addRulesButtonLabel = $scope.resources.decisionTree_addZeroRulesToParentObject_txt;
                }
                $scope.treeOutputs.allRulesSelected = null; // reset
            };

            $scope.filterByTargetValue = function() {
                $scope.updateFiltersChanged();
            };

            $scope.getCondColListElId = function(col) {
                return "condColListEl_" + col.name;
            };

            $scope.formatTargetValue = function(value) {
                //console.log("formatTargetValue:", value);
                if (value == null) {
                    return $scope.resources.decisionTree_missingLbl_txt;
                }
                if($scope.treeInputs.targetColumn.dataType === "CHARACTER") {
                    if(value.trim().length === 0) {
                        return $scope.resources.decisionTree_missingLbl_txt;
                    }
                    else {
                        return value;
                    }
                }
                else {
                    if(value.trim() === ".") {
                        return $scope.resources.decisionTree_missingLbl_txt;
                    }
                    else if ($scope.treeInputs.targetColumn.columnFormat.indexOf("DATE") === 0) {
                        return value;   // date values are already formatted
                    }
                    else {
                        return  vsdUtil.applySASNumericFormat(parseFloat(value),$scope.treeInputs.targetColumn.columnFormat);
                    }
                }
            };

            $scope.returnToParentScreen = function() {
                var returnHandler = decisionTreeSvc.getReturnToParentScreenHandler(),
                    rules = [],
                    condCols;
                //Clear out old settings in service
                decisionTreeSvc.clear();
                //Set rules and other output properties via service
                if($scope.treeOutputs.selRuleLeafNodes.length > 0) {
                    rules = _.pluck($scope.treeOutputs.selRuleLeafNodes,"rule");
                    decisionTreeSvc.setSelectedRules(rules);
                    decisionTreeSvc.setTargetColumn($scope.treeInputs.targetColumn);
                    //Get distinct condition columns used in selected rules
                    condCols = collectCondColsForRules(rules);
                    decisionTreeSvc.setUsedConditionColumns(condCols);
                }

                //Invoke method to return to parent screen
                returnHandler();
            };

            $scope.updateFiltersChanged = function() {
                var changed = false;
                if ($scope.treeOutputs.filterTargetVal !== $scope.treeOutputs.dominantTargetVals[0]) {
                    changed = true;
                }
                else if ($scope.treeOutputs.statsFilter.type.id === FILTER_TYPE_POPULATION_CNT) {
                    if ($scope.treeOutputs.statsFilter.min !== 0 || $scope.treeOutputs.statsFilter.max !== $scope.counters.totalRecCnt) {
                        changed = true;
                    }
                }
                else if ($scope.treeOutputs.statsFilter.min !== 0 || $scope.treeOutputs.statsFilter.max !== 100) {
                    changed = true;
                }
                $scope.treeOutputs.filtersChanged = changed;
            };

            $scope.resetStatFilterBounds = function() {
                if(_.contains([FILTER_TYPE_ACCURACY,FILTER_TYPE_POPULATION_PCT],$scope.treeOutputs.statsFilter.type.id)) {
                    $scope.treeOutputs.statsFilter.constraints = {
                        min: 0,
                        max: 100
                    };
                    $scope.treeOutputs.statsFilter.min = 0;
                    $scope.treeOutputs.statsFilter.max = 100;
                }
                //Population count
                else if($scope.treeOutputs.statsFilter.type.id === FILTER_TYPE_POPULATION_CNT) {
                    $scope.treeOutputs.statsFilter.constraints = {
                        min: 0,
                        max: $scope.counters.totalRecCnt
                    };
                    $scope.treeOutputs.statsFilter.min = 0;
                    $scope.treeOutputs.statsFilter.max = $scope.counters.totalRecCnt;
                }
                $scope.updateFiltersChanged();
            };

            $scope.clearAllFilters = function() {
                $scope.treeOutputs.filterTargetVal =  $scope.treeOutputs.dominantTargetVals[0];
                $scope.treeOutputs.statsFilter.type = $scope.lists.statsFilterTypes[0];
                $scope.resetStatFilterBounds();
                $scope.updateFiltersChanged();
            };

            $scope.swapChartView = function(newView) {
                $scope.treeOutputs.chartView = newView;
                if($scope.treeOutputs.chartView === $scope.CHART_VIEW_TREE) {
                    $scope.treeOutputs.treeData = treeData;
                }
                else if($scope.treeOutputs.chartView === $scope.CHART_VIEW_IMPORTANCE) {
                    $scope.treeOutputs.varImportanceData = varImportanceData;
                    $scope.treeOutputs.varImportanceUsedList = $scope.treeOutputs.treeSelectedColNames;
                }
            };

            $scope.deleteColumnsFromPrune = function(prunedNodeNames) {
                var treeCondColNames = treeUtilSvc.collectFromRootToLeaves($scope.treeOutputs.treeData,collectTreeCondColNames);
                angular.forEach(prunedNodeNames, function (prunedNodeName) {
                    if(!_.contains(treeCondColNames,prunedNodeName)) {
                        varImportanceData = _.reject(varImportanceData,function(entry) {
                            return entry.VarName === prunedNodeName;
                        });
                    }
                });

                function collectTreeCondColNames(node) {
                    return node.name;
                }

            };

            //Helper functions published to scope
            $scope.addRemoveConditionColumn = addRemoveConditionColumn;
            $scope.leafNodeSelected = leafNodeSelected;
            $scope.removeColumnFromTree = removeColumnFromTree;
            $scope.createDecisionTreeAutoWithConfirm = createDecisionTreeAutoWithConfirm;

            //One-time initialization as part of construction
           /* $scope.treeInputs.targetColumn = decisionTreeSvc.getTargetColumn(); */
            $scope.onTargetColumnSelect(true);
            $scope.treeInputs.allInputCondCols =  decisionTreeSvc.getInputColumns();
            $scope.treeInputs.validTargetCols = _.reject($scope.treeInputs.allInputCondCols,function(col) {
                return col.distinctCount < 2 ||
                       col.sasDate ||
                       col.sasDateTime ||
                       col.distinctCount > vsmCommonConst.DECISION_TREE_MAX_TARGET_LEVELS ||
                      !scenarioSvc.allowScenarioOutputColumnDrop(col,decisionTreeSvc.getScenarioExecutionOrder(),$scope.resources,true);
            });
            setAvailCondCols();
            $scope.counters.totalRecCnt = decisionTreeSvc.getInputRecordsCount();
            $scope.treeInputs.lasrTreeOpts =  decisionTreeSvc.getTreeSettings();

            //Setup map between display tree options and resource strings in case error messages must be displayed
            fillOptionResourceMap();
	    }
    ]);

});

},
'vsd/scenario/decisionTree/scenarioInputTreeSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
    factory('scenarioInputTreeSvc', ['dojoModules','vsdUtil',function (dojoModules,vsdUtil) {
        var WINDOW_OUTPUT = 'WO',
            SCENARIO_OUTPUT = 'SO';

        function isScenarioAutoCol(col,partitionKeyColumns) {
            var partKeyCols = partitionKeyColumns || [];
            //Do not allow sas date/datetime columns with decision tree per product mgmt
            if (col.sasDate || col.sasDateTime || _.findWhere(partKeyCols,{name: col.name})) {
                return true;
            }
            else {
                return false;
            }
        }

        function inputVarsComparator(a,b) {
            if(a.dataType > b.dataType) {
                return 1;
            }
            else if(a.dataType < b.dataType){
                return -1;
            }
            else {
                if(a.label > b.label) {
                    return 1;
                }
                else if(a.label < b.label) {
                    return -1;
                }
                else{
                    return 0;
                }
            }
        }

        return {
            getInputTreeDef: function(isUsedColFunc) {
                return [
                    dojoModules.tree({
                        field:             "label",
                        label:             dojoModules.resources.common_OpenDialog_name_txt,
                        sortable:          true,
                        singular:          false,
                        collapseOnRefresh: true,
                        renderCell: function(object, value, node, options) {
                            //Top level grouping node
                            if(!object.parent) {
                                return dojoModules.domConstruct.toDom('<span class="treeGridGroupingLabel">' + object.label + '</span>');
                            }
                            //Field - show icon before label depending on data type
                            else {
                                var iconClass = object.dataType === 'NUMERIC' ? 'NUMERICIcon' : 'CHARACTERIcon';
                                var nodeStr = '<span class="hasContextMenu" title="' + object.label + '"><span  class="treeGridIconCont ' + iconClass + '">&nbsp;</span><span>' + object.label + '</span></span>';
                                var usedCol = isUsedColFunc(object);
                                if (usedCol) {
                                    dojo.toggleClass(node, "disabledItem");
                                }
                                return dojoModules.domConstruct.toDom(nodeStr);
                            }
                        },
                        shouldExpand: function(row, level, previouslyExpanded) {
                            if(level === 0) {
                                return true;
                            }
                            else {
                                return false;
                            }
                        }

                    }),
                    {
                        field: 'distinctCount',
                        renderCell: function(object, value, node, options) {
                            var exceedsMaxCount =  object.distinctCount > vsmCommonConst.DECISION_TREE_MAX_NOMINAL_LEVELS;
                            var adjustCount = exceedsMaxCount ? object.distinctCount -1 : object.distinctCount;
                            var countStr =  vsdUtil.applySASNumericFormat(adjustCount,vsdUtil.defaultFormats.integer);

                            if(exceedsMaxCount) {
                                countStr += "+";
                            }


                            var nodeStr = '<span>' + countStr + '</span>';
                            return dojoModules.domConstruct.toDom(nodeStr);
                        }
                    }
                ]
            },

            filterInputColumns: function(scope,propertyPath){
                return function (item, index, items) {
                    var filterString = getFilterValue().toLowerCase();
                    if (filterString.length < 1) {
                        return !!!item.parent;
                    }
                    else {
                        var label = item.label.toLowerCase();
                        var matched = label.indexOf(filterString) > -1;
                        if (matched) {
                            return true;
                        }
                    }

                    return false;
                }

                function getFilterValue() {
                    var propToks = propertyPath.split("."),
                        currVal;
                    for(var i = 0; i < propToks.length; i++) {
                        var currPropName = propToks[i];
                        if(i == 0) {
                            currVal = scope[currPropName]
                        }
                        else {
                            currVal = currVal[currPropName];
                        }
                    }
                    return currVal;
                }
            },
            inputColsGetChildren: function (store, parent, options) {
                var cols = store.query({parent: parent.treeId}, options) ;
                return cols.sort(inputVarsComparator);
            },
            inputColsMayHaveChildren: function (store, parent) {
                return  parent.columns && parent.columns.length > 0 && store.getChildren(parent).length > 0;
            },
            generateInputColumns: function (windowTableOutCols,usedConditionColumns, partitionKeyColumns,resources) {
                var inputColumns = [];
                var allVars=[];
                var windowColumns={ label: dojoModules.resources.decisionTable_windowOutputCols_txt,treeId: '1', parent: null, columnType: WINDOW_OUTPUT, columns:[]};
                allVars.push(windowColumns);
                var scenarioColumns={ label: dojoModules.resources.decisionTable_scenarioOutputCols_txt, treeId: '2',  parent: null, columnType: SCENARIO_OUTPUT, columns:[] };
                allVars.push(scenarioColumns);
                angular.forEach(windowTableOutCols, function(inputCol) {

                    var column = angular.copy(inputCol);
                    if(column.columnType === WINDOW_OUTPUT && !isScenarioAutoCol(column, partitionKeyColumns)) {
                        column.treeId =windowColumns.columnType  + column.id;
                        column.parent = windowColumns.treeId;
                        if(_.findWhere(usedConditionColumns, {name: column.name}) != null)
                            column.usedInDt = true;
                        windowColumns.columns.push(column);
                    }
                    else if(!isScenarioAutoCol(column, partitionKeyColumns)) {
                        column.treeId = scenarioColumns.columnType  + column.id;
                        column.parent = scenarioColumns.treeId;
                        scenarioColumns.columns.push(column);


                    }
                    if(!isScenarioAutoCol(column, partitionKeyColumns))  {
                        allVars.push(column);
                    }
                });

                return allVars;
            }

        };
}]);

});

},
'vsd/scenario/decisionTree/vsdD3DecisionTree':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('vsdD3DecisionTree', ['d3UtilsSvc','dojoModules', 'vsdUtil','treeUtilSvc', 'decisionTreeSvc','$timeout',
    function (d3UtilsSvc,dojoModules,vsdUtil,treeUtilSvc,decisionTreeSvc,$timeout) {
        return {
              restrict: 'E',
              template: '<div class="decisionTree">' +
                             '<div class="background" style="height: 100%;min-width: 500px;float: left;overflow-x: auto;overflow-y: hidden"></div>' +
                             '<div class="background" style="height: 100%;width: 300px;float: right;overflow: hidden"></div>' +
                        '</div>',
                  /*'<div class="decisionTree" style="width: 100%;height: 100%">' +
                      '<div class="background" style="position: absolute; top: 0px; bottom: 0px;left: 0px;right: 300px;overflow-x: auto;overflow-y: hidden"></div>' +
                      '<div class="background" style="position: absolute; top: 0px; bottom: 0px;right: 0px;width: 300px"></div>' +
                      '</div>',*/

              replace: true,
              scope: {
                'treeData':                         '=',
                'targetColumn':                     '=',
                'conditionColumns':                 '=',
                'totalRecCnt':                      '=',
                'parentLoading':                    '=',
                'resources':                        '=',
                'nodeMouseoverHandler':             '&',
                'nodeMouseoutHandler':              '&',
                'leafNodeMouseoverHandler':         '&',
                'leafNodeMouseoutHandler':          '&',
                'highlightRuleHandler':             '&',
                'sidebarSplitcondMouseoverHandler': '&',
                'sidebarSplitcondMouseoutHandler':  '&',
                'pruneTreeHandler':                 '&',
                'removeTreeColumnHandler':          '&',
                'splitCondFormatter':               '&',
                'targetValueFormatter':             '&',
                'leafNodeSelectionHandler':         '&',
                'leafNodeSelectedComparator':       '&',
                'leafNodeSelectAllHandler':         '&',
                'allRulesSelected':                 '=',
                'targetValueFilter':                '=',
                'statsFilter':                      '='
              },
              link: function(scope, element, attrs,ctrl) {
                  //Create variable on scope for svg
                  var parentElement = element[0],
                      d3Wrapper,
                      allNodes,//Variable declared outside main renderer so separate filtering functions can acccess nodes,
                      allRulesSelectedWatch = null,
                      currNodes,
                      currData,
                      levelVals,
                      tree,
                      treeCont,
                      treeWidth,
                      treeHeight,
                      treeBandCont,
                      sidebarBandCont,
                      treeLevels = 0,
                      nodeVOffset = 5,
                      leafNodeWidth =  25,
                      leafNodeHeight =  55,
                      bandVBounds,
                      targetValueFilterState,
                      classificationScale,
                      minLinkWidth = 5,
                      maxLinkWidth = 20,
                      linkStrokeScale = d3.scale.linear()
                        .domain([0, 1])
                        .range([minLinkWidth, maxLinkWidth]),
                      diagonal = d3.svg.diagonal()
                        .projection(function(d) { return [d.x, d.y+15]; }),
                      selectedNodeData,
                      redrawDuration = 750,
                      defaultEasingFunction = "in-out",
                      treeSvg,
                      sidebarSvg,
                      treeSvgWidth,
                      sidebarSvgWidth,
                      defaultSvgHeight,
                      margins =  {top: 0, left: 0, bottom: 0, right: 0},
                      scrollBarHeight = 15,
                      scrollBarWidth = vsdUtil.getScrollbarWidth(),
                      truncEllipseWidth = 0,
                      toolTipDelay = 250;


                  var treeSvgDiv = element.children()[0],
                      rightSvgDiv = element.children()[1],
                      treeSvgDivWidth;

                  //Watch for data to change and render chart when it does
                  scope.$watch('treeData', function (newVal, oldVal) {
                      if(newVal !== null && newVal !== oldVal && element.width() > 0 && element.height() > 0) {

                          // Make sure there's enough room for coverage gauges, header stats and margin
                          var statRowHeight = 17,
                              headerHeight = Math.max(60, newVal.targetInfo.length * statRowHeight) + 5;
                          $('#decisionTreeBody').css({ "top": headerHeight + 'px'});

                          //if(!defaultSvgHeight) {
                          defaultSvgHeight = element[0].clientHeight;
                          //}
                          renderChart(newVal);

                          //Register to get redrawn when parent container resized
                          d3UtilsSvc.handleParentWidgetResize(element[0], function () {
                              defaultSvgHeight = element[0].clientHeight;
                              renderChart(currData, 0, false, true);
                          });
                      }
                  });

                  //Watch leaf filter and apply filter when change occurs
                  scope.$watch('targetValueFilter',function(newVal,oldVal) {
                      if(oldVal !== newVal && scope.treeData) {
                          if(targetValueFilterState !== newVal.id) {
                              filterByLeafValue(combinedExclude);
                              targetValueFilterState = newVal.id;
                          }
                      }
                  });

                  scope.$watchCollection('statsFilter',function(newVal,oldVal) {
                      // If either value is undefined stop execution. There is a bug with the NumberSpinner where if the user puts the mouse cursor
                      // at the end of an existing number and hits <enter> it triggers the NumberSpinner's change handler with a new value of undefined
                      if (newVal && (typeof newVal.min === 'undefined' || typeof newVal.max === 'undefined')) {
                          return;
                      }
                      if(oldVal !== newVal && scope.treeData) {
                          filterByLeafValue(combinedExclude);
                      }
                  });

                  function combinedExclude(clickedNodeData) {
                      return excludeByDominantTargetVal(clickedNodeData) || excludeByStat(clickedNodeData);
                  }

                  function excludeByDominantTargetVal(clickedNodeData) {
                      if(scope.targetValueFilter.id === '_ALL_') {
                          return false;
                      }
                      else {
                          return clickedNodeData.dominantTargetValue !== (scope.targetValueFilter.id);
                      }
                  }

                  function excludeByStat(clickedNodeData) {
                      var min = parseFloat(scope.statsFilter.min),
                          max = parseFloat(scope.statsFilter.max),
                          stat = getStat(clickedNodeData);

                      if((min || min === 0) && (max  || max === 0) && scope.statsFilter.type.id) {
                          return stat < min || stat > max;
                      }
                      else {
                          return false;
                      }


                      function getStat(clickedNodeData) {
                          switch(scope.statsFilter.type.id) {
                              case "A":
                                  return vsdUtil.realRound(d3.max(clickedNodeData.classificationRatios)*100) ; //Need to round since display numbers are rounded
                              case 'PP':
                                  return vsdUtil.realRound(clickedNodeData.dataRecs/scope.totalRecCnt * 100) ;   //Need to round since display numbers are rounded
                              case 'PC':
                                  return clickedNodeData.dataRecs;
                          }
                      }
                  }

                  function pruneTreeBelowNode(clickedNodeData) {
                      //Collect list of nodes to remove
                      var delNodes,
                          delCondColNames = [],
                          delRules = [],
                          clickedNodeName = clickedNodeData.name;

                      //Collected nodes to delete and rules
                      delNodes = treeUtilSvc.collectFromRootToLeaves(clickedNodeData,collectChildNode);
                      //For each rule, delete it from the domain object
                      angular.forEach(delNodes,function(node) {
                          scope.leafNodeSelectionHandler({node: node,selected: false});
                      });
                      //Modify node to remove children make it a leaf
                      var leafNode = _.findWhere(delNodes,{type: "LEAF"});
                      clickedNodeData.name = leafNode.name;
                      clickedNodeData.children = [];
                      clickedNodeData.type = "LEAF";
                      //Add rule to leaf node by truncating conditions containing remove condition columns
                      clickedNodeData.rule = leafNode.rule;
                      clickedNodeData.rule.condition.childConditions = _.reject(clickedNodeData.rule.condition.childConditions,function(cond) {
                          var condColName = cond.operator.name  === 'AND' ?  cond.childConditions[0].operand1ColumnInfo.name : cond.operand1ColumnInfo.name;
                          return _.contains(delCondColNames,condColName);
                      });
                      //Set the x/y position to allow animation
                      clickedNodeData.oldX = clickedNodeData.x;
                      clickedNodeData.oldY = clickedNodeData.y;
                      //Update allNodes so child nodes doesn't reappear during filtering
                      overlayNodeContents(allNodes,clickedNodeData);
                      //Remove parent (since can't pass in tree with parent refs) and remove from tree data so child nodes doesn't reappear during filtering
                      delete clickedNodeData.parent;
                      overlayNodeContents(scope.treeData,clickedNodeData);
                      //Create copy of original tree output (since can't pass in tree with parent refs)
                      var treeDataCopy = angular.copy(scope.treeData);
                      //Cache current x/y position for remaining nodes to support animation
                      copyXAndY(treeDataCopy,currNodes);
                      //Re-render chart with transitions
                      renderChart(treeDataCopy,redrawDuration,true);

                      //Call prune tree handler for additional cleanup
                      var prunedNodeNames = _.pluck(_.reject(delNodes,function(node){return node.type === 'LEAF';}),"name");
                      prunedNodeNames.push(clickedNodeName);
                      scope.pruneTreeHandler({prunedNodeNames: prunedNodeNames});
                      scope.$apply();

                      function overlayNodeContents(currNode,repNode,parentNode) {
                          if(currNode.id === repNode.id) {
                              parentNode.children = _.reject(parentNode.children,function(child) {
                                  return child.id === repNode.id;
                              });
                              parentNode.children.push(repNode);
                          }
                          else {
                             angular.forEach(currNode.children,function(child) {
                                 overlayNodeContents(child,repNode,currNode);
                             });
                          }
                      }

                      function collectChildNode(node) {
                          if(node.id !== clickedNodeData.id) {
                              if(node.type === "LEAF") {
                                  delRules.push(node.rule);
                              }
                              delCondColNames.push(node.parentVarName);
                              return node;
                          }
                          else {
                              return null;
                          }
                      }
                  }

                  function filterByLeafValue(excludeFunc) {
                      var levelNbrs = d3.range(treeLevels -1,0,-1),
                          removeNodeIds = [],
                          removeLinks = [];

                      if(!scope.treeData) {
                          return;
                      }

                      //Restore children properties of allNodes
                      angular.forEach(allNodes,function(node) {
                          if(node.origChildren) {
                              node.children = node.origChildren;
                          }
                      });

                      //Build list of nodes to remove from tree starting from bottom.  Do so level by level so can determine at each level if any children are left
                      angular.forEach(levelNbrs,function(levelNbr) {
                          angular.forEach(allNodes, function(d) {
                              if(d.depth === levelNbr) {
                                  var removeNode = false;

                                  if(d.type === "LEAF" && excludeFunc(d)) {
                                      removeNodeIds.push(d.id);
                                      removeNode = true;
                                  }
                                  else if(d.type !== "LEAF" && d.children.length === 0) {
                                      removeNodeIds.push(d.id);
                                      removeNode = true;
                                  }
                                  if(removeNode){
                                      if(!d.parent.origChildren) {
                                          d.parent.origChildren = d.parent.children;
                                      }
                                      d.parent.children = _.reject(d.parent.children,function(child) {
                                          return child.id === d.id;
                                      });
                                  }
                              }
                          });
                      });
                      //Create copy of tree data without removed leaf nodes and now empty parents
                      var treeDataCopy = angular.copy(scope.treeData);
                      filterTreeDataByLeafNodes(treeDataCopy,removeNodeIds);
                      //Cache current x/y position for remaining nodes to support animation
                      copyXAndY(treeDataCopy,currNodes);
                      //Re-render chart with transitions
                      renderChart(treeDataCopy,redrawDuration);
                  }

                  function copyXAndY(node,nodeList) {
                      var matchNode = _.findWhere(nodeList,{id: node.id});
                      if(matchNode) {
                          node.oldX = matchNode.x;
                          node.oldY = matchNode.y;
                      }
                      if(node.children.length) {
                          angular.forEach(node.children,function(child) {
                              copyXAndY(child,nodeList);
                          });
                      }
                  }

                  function filterTreeDataByLeafNodes(currNode,removeLeafNodes) {
                      if(currNode.children.length > 0) {
                          currNode.children = _.reject(currNode.children,function(child) {
                              return _.contains(removeLeafNodes,child.id);
                          });
                          //Navigate through remaining children to do removals
                          angular.forEach(currNode.children,function(child) {
                              filterTreeDataByLeafNodes(child,removeLeafNodes);
                          });
                      }
                  }

                  //Function to render d3 histogram from provided data
                  function renderChart(chartData,duration,prune,resizing) {
                      var me = this,
                          treeMargins = {top: 10, left: 20, bottom: 60, right: 20},
                          legendHPad = 20,
                          legendSepMargin = 15,
                          legendVPadO = 20,
                          legendVPadI = 5,
                          legendEntryHeight = 30,
                          legendEntryBarWidth = 40,
                          legendEntryBarHeight = 20,
                          maxLegendTextWidth = 150,
                          legendTextLeftMargin = 5,
                          legendEntryWidth,
                          legendTitleWidth,
                          legendTitleHeight = 30,
                          legendTitle,
                          legendEntriesPerLine,
                          legendWidth,
                          legendLines,
                          legendHeight,
                          legendHeightWithPadding ,
                          legendBottomMargin = 5,
                          sidebarWidth = 300,
                          textWidth = 50,
                          minNonLeafLevelHeight = 55,
                          minLeafLevelHeight = 175,
                          splitCondColLineHeight = 20,
                          splitCondTextMargin = 15,
                          splitCondLineVGap = 20,
                          splitCondDetailLineHeight = 15,
                          defs,
                          ruleSelectCBDims = {w: 15, h: 15},
                          minLeafNodeGap = 3,
                          minNonLeafNodeGap = 10,
                          leafNodes,
                          nonLeafNodes,
                          svgHeight;

                      duration = duration ? duration : 0;


                      //If we got back and empty tree
                      if(!chartData) {
                          duration = redrawDuration;
                          drawEmptyTreeDisplay(true);
                          return;
                      }

                      //Create SVGs
                      if(!duration) {
                          treeLevels = getTreeLevelCount(chartData);
                          var svgMinHeight = getMinTreeHeight(treeLevels);
                          treeSvgDivWidth = element.width() - sidebarWidth;
                          if(svgMinHeight > defaultSvgHeight) {
                              svgHeight = svgMinHeight;
                              treeSvgDivWidth -= scrollBarWidth;
                              element.css("overflow-y","auto");
                          }
                          else {
                              svgHeight = defaultSvgHeight;
                              element.css("overflow-y","hidden");
                          }
                          treeSvgWidth = treeSvgDivWidth;
                          //Remove svgs to present screen flashing
                          angular.element(treeSvgDiv).empty();
                          angular.element(rightSvgDiv).empty();
                          //Resize divs
                          angular.element(treeSvgDiv).css("width",treeSvgDivWidth);
                          angular.element(treeSvgDiv).css("height",svgHeight);
                          angular.element(rightSvgDiv).css("height",svgHeight);
                          //Create SVGs
                          treeSvg = createSVG(treeSvgDiv,treeSvgWidth,svgHeight,margins);
                          sidebarSvg = createSVG(rightSvgDiv,sidebarWidth,svgHeight,margins);
                      }

                      if(treeSvgWidth !== 0 && sidebarSvgWidth !== 0 && defaultSvgHeight !== 0) {
                          //Do first attempt to render tree and see if the leaf nodes fit
                          var leafNodeOverlap = renderTree();
                          //If we're doing an initial rendering and there is leaf node overlap, resize SVG until leaf nodes fit
                          if(!duration && leafNodeOverlap) {
                              var newOverlap = 0;
                              do {
                                  treeSvgWidth += leafNodeOverlap *3;  //Just adding overlap often isn't enough to prevent overlap - more space needed for layout, thus the *3
                                  treeSvg = createSVG(treeSvgDiv,treeSvgWidth,svgHeight,margins);
                                  sidebarSvg = createSVG(rightSvgDiv,sidebarWidth,svgHeight,margins);
                                  newOverlap = renderTree();
                              } while(newOverlap > 0);
                          }
                      }

                      function renderTree() {
                          //If not transitioning, doing a redraw, so do all first draw initialization
                          if(!duration) {
                              initRedraw();
                          }

                          //Bind data to tree layout and calculate node an link positions
                          currData = angular.copy(chartData);
                          currNodes =  tree.nodes(angular.copy(chartData));
                          

                         if(chartData.children.length > 0) {

                              nonLeafNodes = currNodes.filter(function(d) {
                                  return d.type !== 'LEAF';
                              });
                              leafNodes =  currNodes.filter(function(d) {
                                  return d.type === 'LEAF';
                              });
                              removeNoFilterMatchesDisplay();
                          }
                          else {
                              nonLeafNodes = [];
                              leafNodes = [];
                              drawEmptyTreeDisplay();
                          }

                          //Mark initial rule selections
                          initLeafNodeSelection(leafNodes);
                          //*** NOTE ** - Order of functions below critical since draw order determines visibility
                          //Draw bands for levels
                          drawBandsForLevels();
                          //Render regular links, with width based on population flow
                          drawLinks();
                          //Draw non-leaf nodes
                          drawNonLeafNodes();
                          //Draw leaf nodes
                          drawLeafNodes();
                          //Draw legend (must redrawn each time since bands redrawn every time and legend must be on top of bands)
                          if(!duration) {
                              drawLegend();
                          }
                          //Remove legend
                          else {
                              var legendComponents =  treeSvg.selectAll('.legendComponent'),
                                  opacity;
                              if(leafNodes.length > 0) {
                                  opacity = 1;
                              }
                              else {
                                  opacity = 0;
                              }
                              legendComponents.style("opacity",opacity);
                          }

                          if(resizing) {
                              treeCont.selectAll('.nonLeafNode,.levelBandEven, .levelBand')
                                  .each(function(d) {
                                    var x = 0;
                                  });
                          }

                          if (allRulesSelectedWatch !== null) {
                              allRulesSelectedWatch(); // clears old watch
                          }
                          allRulesSelectedWatch = scope.$watch('allRulesSelected', function (newVal, oldVal) {
                              if (newVal !== null && newVal !== oldVal) {
                                  var selNodes = [];
                                  treeCont.selectAll('.ruleSelectCheck')
                                      .style('opacity', function (d) {
                                          d.ruleSelected = newVal;
                                          if (newVal === true) {
                                              selNodes.push(d);
                                          }
                                          return d.ruleSelected ? 1 : 0;
                                      });
                                  scope.leafNodeSelectAllHandler({selectedNodes: selNodes});
                              }
                          });

                          return calcLeafNodeOverlaps();
                      }

                      function drawEmptyTreeDisplay(noTreeData) {
                          var drawText = noTreeData ? scope.resources.decisionTree_emptyTreeReturnedLbl_txt :   scope.resources.decisionTree_noFilterMatchesLbl_txt;
                          //Draw a rectangle the size of the SVG for background
                          treeSvg.append("rect")
                              .attr("class","emptyTreeBackground")
                              .attr("width",treeSvgWidth)
                              .attr("height",defaultSvgHeight);
                          sidebarSvg.append("rect")
                              .attr("class","emptyTreeBackground")
                              .attr("width",sidebarWidth)
                              .attr("height",defaultSvgHeight);
                          //Draw text indicating empty tree
                          treeSvg.append("text")
                              .attr("class","emptyTreeText")
                              .attr("x",treeSvgWidth/2)
                              .attr("y",defaultSvgHeight/2)
                              .style("text-anchor","middle")
                              .style("font-size","0pt")
                              .text(drawText)
                              .transition().duration(duration)
                              .style("font-size","24pt");

                      }

                      function removeNoFilterMatchesDisplay() {
                          treeSvg.selectAll(".emptyTreeBackground, .emptyTreeText").remove();
                          sidebarSvg.selectAll(".emptyTreeBackground").remove();
                      }

                      function drawLeafNodes() {
                          var leafNodeSel = treeCont.selectAll(".leafNodeOuterGroup")
                              .data(leafNodes,function(d){return d.id;});
                          var leafNodeEnter = leafNodeSel.enter();
                          //Construct leaf node groups and contents for new leaf nodes
                          var leafNodeGroupAdds = leafNodeEnter.append("g")
                              .attr("class","leafNodeOuterGroup")
                              .attr("transform",function(d){
                                  var x,
                                      y;
                                  if(prune) {
                                      x = d.oldX;
                                      y = d.oldY;
                                  }
                                  else {
                                      x = d.x - leafNodeWidth/2 - 50;
                                      y = d.y + nodeVOffset;
                                  }

                                  return "translate(" + x + "," + y + ")";
                              });

                          //Render leaf nodes as rectangles
                          leafNodeGroupAdds.append('rect')
                              .attr("class","leafNodeOuter")
                              .attr("width",leafNodeWidth)
                              .attr("height",leafNodeHeight)
                              .on('click',highlightSelectedRule)
                              .on('mouseover',onLeafNodeMouseOver);

                          //Add internal rect for graph
                          leafNodeGroupAdds.append('rect')
                              .attr("class","leafNodeInner")
                              .attr("y",function(d){
                                  var pct = d3.max(d.classificationRatios);
                                  return leafNodeHeight * (1-pct);
                              })
                              .style("fill",function(d) {
                                  return classificationScale(scope.targetValueFormatter({value: d.dominantTargetValue}));
                              })
                              .attr("height",function(d){
                                  var pct = d3.max(d.classificationRatios);
                                  return leafNodeHeight * pct;
                              })
                              .attr("width",leafNodeWidth)
                              .on('click',highlightSelectedRule)
                              .on('mouseover',onLeafNodeMouseOver);

                          //Rect for node outline leaf nodes as rectangles
                          leafNodeGroupAdds.append('rect')
                              .attr("class","leafNodeStroke")
                              .attr("width",leafNodeWidth)
                              .attr("height",leafNodeHeight)
                              .on('click',highlightSelectedRule)
                              .on('mouseover',onLeafNodeMouseOver);

                          leafNodeGroupAdds.transition()
                              .duration(duration)
                              .ease(defaultEasingFunction)
                              .attr("transform",function(d){return "translate(" + (d.x - leafNodeWidth/2) + "," + (d.y + nodeVOffset) + ")";});

                          //Draw checkboxes for rule selectors
                          drawRuleSelectors();
                          //Assign selected node and update highlighted rule path
                          if(!selectedNodeData || !_.findWhere(leafNodes,{id: selectedNodeData.id})){
                              //Select first leaf node
                              selectedNodeData = _.min(leafNodes,function(node) {
                                  return node.x;
                              });
                          }
                          else {
                              selectedNodeData = _.findWhere(leafNodes, {id: selectedNodeData.id});  //Must re-point to data since node objects recreated through tree.nodes() each call
                          }
                          highlightRulePath(selectedNodeData);

                          //Animate existing leaf nodes into position
                          var numTrans = leafNodeSel.transition().size();
                          var transCompleted = 0;
                          leafNodeSel.transition().duration(duration)
                              .ease(defaultEasingFunction)
                              .attr("transform",function(d){return "translate(" + (d.x - leafNodeWidth/2) + "," + (d.y + nodeVOffset) + ")";})
                              .each('end',function() {
//                              if(!initRuleSelectDone) {
                                  transCompleted++;
                                  if(transCompleted === numTrans){
                                      highlightSelectedRule(selectedNodeData);
                                      truncateOutOfBoundsNonLeafNodes();
                                      truncateOverlappingNonLeafNodes();
                                  }
                              });

                          //Remove deleted leaf nodes
                          leafNodeSel.exit().remove();

                          function drawRuleSelectors() {
                              var cbPathStr,
                                  cbMargin = 3,
                                  checkbox = leafNodeGroupAdds.append("g")
                                      .attr('transform',function(d){
                                          return 'translate(' +
                                              (leafNodeWidth/2 - ruleSelectCBDims.w/2) +
                                              "," +
                                              (leafNodeHeight + nodeVOffset*2) +
                                              ")";
                                      })
                                      .attr("height",ruleSelectCBDims.h)
                                      .attr("width",ruleSelectCBDims.w);

                              checkbox.append('rect')
                                  .attr("height",ruleSelectCBDims.h)
                                  .attr("width",ruleSelectCBDims.w)
                                  .attr("class","ruleSelectCheckbox")
                                  .on('click',processClick);

                              cbPathStr = "M" + cbMargin + " " + ruleSelectCBDims.h/2
                                  + " L" + ruleSelectCBDims.h/2 + " " + (ruleSelectCBDims.h - cbMargin)
                                  + " L" + (ruleSelectCBDims.h - cbMargin) + " " + cbMargin;
                              checkbox.append("path")
                                  .attr("d",cbPathStr)
                                  .attr("class","ruleSelectCheck")
                                  .style('opacity',function(d){
                                      return d.ruleSelected ? 1 : 0;
                                  })
                                  .on('click',processClick);


                              function processClick(clickedNodeData) {  // checkbox clicked
                                  var opacity;

                                  //If rule was previously
                                  if(clickedNodeData.ruleSelected) {
                                      clickedNodeData.ruleSelected = false;
                                      opacity = 0;
                                  }
                                  else {
                                      clickedNodeData.ruleSelected = true;
                                      opacity = 1;
                                  }
                                  leafNodeGroupAdds.selectAll('.ruleSelectCheck')
                                      .filter(function(d){return d.id === clickedNodeData.id;})
                                      .style("opacity",opacity);

                                  //Pass selection change to user
                                  scope.leafNodeSelectionHandler({node: clickedNodeData, selected: clickedNodeData.ruleSelected});
                                  //Show selection path for rule
                                  highlightSelectedRule(clickedNodeData);
                                  //Notify scope of change
                                  scope.$apply();
                              }
                          }
                      }

                      function drawNonLeafNodes() {
                          //Create selectors for enter/update/exit
                          var nonLeafNodeSel = treeCont.selectAll(".nonLeafNode")
                              .data(nonLeafNodes,function(d){return d.id;});
                          var nonLeafNodeEnter = nonLeafNodeSel.enter();

                          //Render new non-leaf nodes
                          nonLeafNodeEnter.append("text")
                              .attr("class","nonLeafNode")
                              .attr("x",function(d){ return d.x + 50;})
                              .attr("y",function(d){return d.y;})
                              .attr("dy", "14px")
                              .attr("text-anchor", "middle")
                              .text(function(d){
                                  return _.findWhere(scope.conditionColumns, {name:d.name}).label;
                              })
                              .on('mouseover', _.debounce(onNodeMouseOver,toolTipDelay,{leading: false}))
                              .each(function(d) {
                                  var menu = createRMBMenu(d);
                                  menu.bindDomNode(this);
                              })
                              .transition()
                              .duration()
                              .attr("x",function(d){return d.x;});


                          //Reposition existing non-leaf nodes
                          nonLeafNodeSel.transition().duration(duration)
                              .ease(defaultEasingFunction)
                              .attr("x",function(d){return d.x;})
                              .attr("y",function(d){return d.y + nodeVOffset;});

                          //Remove deleted non-leaf nodes
                          nonLeafNodeSel.exit().remove();
                      }

                      function drawLinks() {
                          var linkSel = treeCont.selectAll(".link")
                              .data(tree.links(currNodes),function(d){return d.target.id;});

                          //Add new links
                          linkSel.enter()
                              .append("path")
                              .attr("class", "link")
                              .attr("d",function(d) {
                                  var xPos = (d.source.x ? d.source.x -50 : 0);
                                  var oldPos = {x: d.source.x -50, y: d.source.y};
                                  return diagonal({source: oldPos, target:d.target});
                              })
                              .style("stroke-width", function(d) {
                                  return linkStrokeScale(d.target.dataRecs/scope.totalRecCnt);
                              })
                              .transition().duration(duration)
                              .ease(defaultEasingFunction)
                              .attr("d", diagonal);

                          //Update existing links
                          linkSel.transition()
                              .duration(duration)
                              .ease(defaultEasingFunction)
                              .attr("d", diagonal)
                              .style("opacity", 1);
                          //Remove deleted links
                          linkSel.exit().style("opacity",0);


                          //Render highlight links (invisible) for later use in highlighting rules
                          var highlightLinkSel = treeCont.selectAll(".highlightLink")
                              .data(tree.links(currNodes),function(d){return d.target.id;});

                          //Render new links
                          highlightLinkSel.enter()
                              .append("path")
                              .attr("class", "highlightLink")
                              .style('stroke', 'url(#diagonalLinePattern)')
                              .style("opacity",0)
                              .attr("d",function(d) {
                                  var oldPos = {x: d.source.x -50, y: d.source.y};
                                  return diagonal({source: oldPos, target:d.target});
                              })
                              .transition().duration(duration)
                              .ease(defaultEasingFunction)
                              .attr("d", diagonal);
                          //Update existing links
                          highlightLinkSel.transition()
                              .duration(duration)
                              .ease(defaultEasingFunction)
                              .attr("d",diagonal);
                          //Remove deleted links
                          highlightLinkSel.exit().remove();
                      }

                      function drawBandsForLevels() {
                          //Remove old bands
                          treeBandCont.selectAll(".levelBandEven, .levelBandOdd").remove();
                          sidebarBandCont.selectAll(".levelBandEven, .levelBandOdd").remove();

                          if(leafNodes.length > 0) {
                              //Init band location array
                              bandVBounds = [];
                              //Draw banded rectangles for each level in the tree
                              var bandDims = calculateRowBandingDims(nonLeafNodes,leafNodes,treeSvgWidth);
                              //Draw bands for treeSvg
                              drawBands(treeBandCont,treeSvgWidth);
                              //Draw bands for sidebar
                              drawBands(sidebarBandCont,sidebarWidth);

                              //Get the absolute position of all band rects and store them for later use
                              var depth = 0;
                              treeSvg.selectAll('.levelBandEven, .levelBandOdd')
                                  .selectAll('rect')
                                  .each(function(d) {
                                      bandVBounds.push({
                                          depth: depth++,
                                          y: d.y
                                      });
                                      if(!duration){
                                          treeLevels++;
                                      }

                                  });
                          }

                          function drawBands(group,width) {
                              var newGrp = group.selectAll(".levelBandEven, .levelBandOdd")
                                  .data(bandDims).enter()
                                  .append("g")
                                  .attr("transform", function(d){return "translate(" + d.x + "," + d.y + ")";})
                                  .attr('class',function(d,i) {
                                      if((i+1) % 2 === 0) {
                                          return "levelBandEven";
                                      }
                                      else {
                                          return "levelBandOdd";
                                      }
                                  })
                                  .append('rect')
                                  .attr('width',function(d){return width;})
                                  .attr('height',function(d){return Math.max(d.height,0);});

                              newGrp.on("mouseover",function() {
                                  closeAllToolTips();
                              });


                          }

                      }

                      function drawLegend() {
                          var legendBox = treeCont.append("g")
                              .attr("class","legendComponent")
                              .attr("height",legendHeight)
                              .attr("width", legendWidth)
                              .attr("transform", "translate(" + (treeWidth/2 - legendWidth/2) + "," + (treeHeight+leafNodeHeight + legendVPadO + nodeVOffset*2 + ruleSelectCBDims.h - legendBottomMargin) + ")");

                          //Append box to g for legend background
                          legendBox.append("rect")
                              .attr("class","legendComponent")
                              .attr("height",legendHeight)
                              .attr("width", legendWidth)
                              .attr("class","legendBox");

                          //Append legend title
                          legendBox.append("text")
                              .attr("class","legendTitle")
                              .attr("y",5)
                              .attr("x",legendWidth/2)
                              .attr("text-anchor","middle")
                              .attr("dy", "12px")
                              .text(legendTitle);

                          //Append key entries
                          var entryNbr = 0,
                              rowNbr = 0,
                              colNbr = 0;
                          angular.forEach(levelVals,function(val) {
                              var boxXPos = legendHPad + colNbr*legendEntryWidth + legendSepMargin*colNbr,
                                  boxYPos = legendTitleHeight + ((legendVPadI + legendEntryHeight + legendVPadI) * rowNbr);

                              //Create colored rectangle for level
                              legendBox.append("rect")
                                  .attr("class","legendComponent leafNodeStroke")
                                  .attr("x",function() {
                                      return boxXPos;
                                  })
                                  .attr("y",function(d,i) {
                                      return boxYPos;
                                  })
                                  .attr("width", legendEntryBarWidth)
                                  .attr("height",legendEntryBarHeight)
                                  .style("fill",function() {
                                      return classificationScale(val);
                                  });
                              //Create label for level
                              legendBox.append("text")
                                  .attr("class","legendComponent")
                                  .attr("x",function() {
                                      return boxXPos + legendEntryBarWidth + legendTextLeftMargin;
                                  })
                                  .attr("y",function() {
                                      return boxYPos + legendEntryBarHeight/2 - 5;
                                  })
                                  .attr("dy", "10px")
                                  .text(val);


                              entryNbr++;
                              colNbr++;
                              //Increment rowNbr depending on entry and number of entries per line
                              if(entryNbr !== 1 && entryNbr % legendEntriesPerLine === 0) {
                                  rowNbr++;
                                  colNbr = 0;
                              }
                          });
                      }

                      function initRedraw() {
                          selectedNodeData = null;

                          if(!resizing) {
                              targetValueFilterState = "_ALL_";
                              treeLevels = 0;
                              //Get widths of all target variable levels
                              levelVals = getDistinctLevelValues(chartData).sort(vsdUtil.singleFieldNaturalSortComparator());
                          }

                          //Create a pattern for path highlighting
                          defs = treeSvg.append("defs");
                          //Create pattern for link highlighting
                          createDiagonalLinePatternStroke();
                          //Create effect for non-leaf node text
                          createTextContrast(defs);

                          legendEntriesPerLine = d3.min([levelVals.length,5]);
                          var legendTextWidth = 0;
                          for(var i = 0; i < levelVals.length; i++) {
                              var currWidth =  d3UtilsSvc.getTextDimensions(treeSvg, levelVals[i] ,10).width;
                              if(currWidth > maxLegendTextWidth) {
                                  //Truncate text with dots

                                  //Reset currWidth to default max
                                  currWidth = maxLegendTextWidth;
                              }
                              if(currWidth > legendTextWidth) {
                                  legendTextWidth = currWidth;
                              }
                          }

                          //Set dimensions for downstream elements based on legend dimensions
                          legendTitle = dojoModules.resources.decisionTree_keyTargetLbl_txt +  ":  '" + scope.targetColumn.label  + "'";
                          legendTitleWidth = d3UtilsSvc.getTextDimensions(treeSvg,legendTitle,12).width + legendHPad*2;
                          legendEntryWidth = legendEntryBarWidth + legendTextWidth + legendTextLeftMargin;
                          legendWidth = (legendEntriesPerLine * legendEntryWidth) +
                              ((legendEntriesPerLine -1)*legendSepMargin) +
                              legendHPad*2;
                          legendWidth = legendWidth < legendTitleWidth ? legendTitleWidth : legendWidth; //Ensure enough space for title in case legend entries few and narrow
                          legendLines = levelVals.length <= 5 ? 1 : 2;
                          legendHeight = legendEntryHeight*legendLines + legendTitleHeight + legendVPadI;
                          legendHeightWithPadding = legendHeight + legendVPadO * 2;
                          treeWidth = treeSvgWidth - treeMargins.left - treeMargins.right;

                          treeHeight = svgHeight - treeMargins.top - treeMargins.bottom - legendHeightWithPadding - legendBottomMargin;




                          treeBandCont = treeSvg.append('g')
                              .attr('width',treeSvgWidth)
                              .attr('height',defaultSvgHeight);

                          sidebarBandCont = sidebarSvg.append('g')
                              .attr('width',sidebarSvgWidth)
                              .attr('height',defaultSvgHeight);


                          //Declare tree layout container
                          treeCont = treeSvg.append('g')
                              .attr('width',0)
                              .attr('height',0)
                              .attr("transform", "translate(" + treeMargins.left + "," + treeMargins.top + ")");


                          //treeCont.call(zoomListener);

                          //Declare tree layout
                          tree = d3.layout.cluster()
                              .size([treeWidth,treeHeight])
                              .sort(function(a,b) {
                                  if(a.lowerBounds > b.lowerBounds) {
                                      return 1;
                                  }
                                  else if(a.lowerBounds < b.lowerBounds) {
                                      return -1;
                                  }
                                  else {
                                      return 0;
                                  }
                              });
                          //Bind all tree data to directive level variable for use in later re-draws
                          //Work with copy of chart data so properties not added to original
                          if(!resizing) {
                              allNodes = tree.nodes(angular.copy(chartData));
                          }


                          //Set up color scale for  leaf node graph
                          classificationScale = d3.scale.ordinal()
                              .domain(levelVals)
                              .range(_.first(d3UtilsSvc.getCorporateColorList(),levelVals.length));

                          //Set length of ellispe text used for truncation
                          if(!truncEllipseWidth) {
                              truncEllipseWidth  = measureTruncElipseWidth(treeCont,14);
                          }


                      }
                      
                      function truncateOutOfBoundsNonLeafNodes() {
                          //Get non-leaf nodes grouped by level
                          var nodesByLevel = getNonLeafNodesByLevel();
                          //Get start and end nodes of each level
                          var boundaryNodes = [];
                          angular.forEach(nodesByLevel, function (nodesLevel) {
                              //Include only the first and last nodes of level
                              boundaryNodes.push(nodesLevel.values[0]);
                              boundaryNodes.push(nodesLevel.values[nodesLevel.values.length-1]);
                          });
                          //For each boundary node, determine if it's out of bounds, and if so, truncate it
                          var treeSvgBounds = treeSvg.node().getBoundingClientRect();
                          angular.forEach(boundaryNodes, function (node) {
                              if(isOutOfBounds(node,treeSvgBounds)) {  //Can only tell if it's up against the boundary - not how far outside
                                  truncateTextUntilInBounds(node,treeSvgBounds,splitAllChars,"");
                              }
                          });


                      }

                      function truncateTextUntilInBounds(textEl,containerBounds,splitFunc,joinChar) {
                          var origText = textEl.textContent;
                          var chars = splitFunc(textEl.textContent);
                          var accumStrToks = [];
                          do {
                              var currChar = chars.shift();
                              accumStrToks.push(currChar);
                              if (chars.length) {
                                  textEl.textContent = accumStrToks.join(joinChar) + joinChar + chars[0] + "...";
                              }
                              else {
                                  textEl.textContent = accumStrToks.join(joinChar) + "...";
                              }
                              var textElBounds = textEl.getBoundingClientRect();
                              //If latest addition exceeded container boundaries, revert to character before and break
                              if(isOutOfBounds(textEl,containerBounds)) {
                                  textEl.textContent = accumStrToks.join(joinChar) + "...";
                                  return;
                              }
                          }while(chars.length);
                      }

                      function truncateOverlappingNonLeafNodes() {
                          //Get non-leaf nodes grouped by level
                          var nodesByLevel = getNonLeafNodesByLevel();

                          angular.forEach(nodesByLevel, function (nodesLevel) {
                              processAdjacentNodes(nodesLevel.values,truncateNodeWidth);
                          });

                          function calcNewNodeWidth(textEl,overlap) {
                              return textEl.getBoundingClientRect().width - overlap - minNonLeafNodeGap - truncEllipseWidth;
                          }

                          function truncateNodeWidth(l,r) {
                              //Find overlap between adjacent nodes
                              var overlap = calculateHOverlap(l,r),
                                            textEls = [l,r],
                                            origLeftText = l.textContent,
                                            origRightText = r.textContent,
                                            newLWidth = calcNewNodeWidth(l,overlap),
                                            newRWidth = calcNewNodeWidth(r,overlap);

                              if(overlap) {
                                  //Remove ellipses from text elements if already present
                                  angular.forEach(textEls, function (el) {
                                      el.textContent = vsdUtil.dropRight(el.textContent,3);
                                  });

                                  //Do initial truncation
                                  if(origLeftText.indexOf("...") !== -1 &&
                                      origRightText.indexOf("...") === -1) {
                                      l.textContent = origLeftText;
                                      r.textContent = origRightText;
                                      d3UtilsSvc.truncateTextWithDots2(r,newRWidth,splitAllChars, "");
                                  }
                                  else {
                                      d3UtilsSvc.truncateTextWithDots2(l,newLWidth,splitAllChars, "");
                                      d3UtilsSvc.truncateTextWithDots2(r,newRWidth,splitAllChars, "");
                                  }

                                  //If still overlap, perform addition truncation on the right element
                                  overlap = calculateHOverlap(l,r);
                                  if(overlap) {
                                      var truncEl = l.textContent.length > r.textContent.length ? l : r;
                                      var newWidth = calcNewNodeWidth(truncEl,overlap);
                                      d3UtilsSvc.truncateTextWithDots2(truncEl,newWidth,splitAllChars, "");
                                  }
                              }
                          }
                      }

                      function getNonLeafNodesByLevel() {
                          //Get all non leaf nodes but root
                          var nonLeafNodes = treeCont.selectAll('.nonLeafNode')
                              .filter(function(d){ return d.depth > 0;})[0];
                          //Group nodes by level
                          return d3.nest()
                              .key(function(d){ return d.__data__.y; })
                              .sortValues(function(a,b){ return a.__data__.x - b.__data__.x; })
                              .entries(nonLeafNodes);
                      }

                      function getNodeBandDepth(nodeData) {
                          return _.findWhere(bandVBounds, {y: nodeData.y}).depth;
                      }

                      function calcLeafNodeOverlaps() {
                          var totalOverlap = 0;
                          processAdjacentNodes(d3.selectAll(".leafNodeOuter")[0],incrOverlap);
                          return totalOverlap;

                          function incrOverlap(l,r) {
                              totalOverlap += calculateHOverlap(l,r);
                          }
                      }

                      function processAdjacentNodes(nodes,pairActionFunc) {
                          nodes.sort(function(a,b) {
                              var ax = a.__data__.x,
                                  bx = b.__data__.x;

                              return ax - bx;
                          });

                          angular.forEach(d3.pairs(nodes), function (pair) {
                              pairActionFunc(pair[0],pair[1]);
                          });
                      }

                      function isOutOfBounds(node,containerBounds) {
                          var nodeBounds = node.getBoundingClientRect();
                          if(nodeBounds.left - nodeBounds.width/2  <= containerBounds.left) {
                              return true;
                          }
                          else if(nodeBounds.right >= containerBounds.right) {
                              return true;
                          }
                          else {
                              return false;
                          }
                      }

                      function calculateHOverlap(left,right) {
                          var leftBounds = left.getBoundingClientRect(),
                              rightBounds = right.getBoundingClientRect();

                          if(leftBounds.right >= rightBounds.left) {
                              return leftBounds.right - rightBounds.left + minLeafNodeGap;
                          }
                          else {
                              return 0;
                          }
                      }

                      function calculateHOutOfBoundsAmt(node) {
                          var nodeBounds = node.getBoundingClientRect(),
                              outOfBoundsAmt = 0;

                      }

                      function splitAllChars(text) {
                          return text.split("");
                      }

                      function highlightSelectedRule(clickedNodeData) {
                          selectedNodeData = clickedNodeData;
                          //Highlight the selected leaf node
                          highlightSelectedLeafNode(clickedNodeData);
                          //Get split conditions by level for each parent condition
                          var splitCondMap = splitCondsFromNodes(clickedNodeData);
                          //Render split conditions in sidebar
                          splitConditionSideBarRender(splitCondMap, clickedNodeData);
                          //Highlight the path for the selected rule in the tree
                          highlightRulePath(clickedNodeData);
                          scope.highlightRuleHandler({node: clickedNodeData});

                          function highlightSelectedLeafNode(clickedNodeData) {
                              var leafNodeOuterSel = d3.selectAll('.leafNodeOuterGroup')
                                      .filter(function(d){ return d.id === clickedNodeData.id;});

                              //Remove outline from preview node, if it exists
                              treeCont.select(".leafNodeOuterSelector").remove();

                              //Draw outline rectangle around node
                              leafNodeOuterSel.append("rect")
                                  .attr("class","leafNodeOuterSelector")
                                  .attr("x",leafNodeOuterSel.attr("x"))
                                  .attr("y",leafNodeOuterSel.attr("y"))
                                  .attr("width",leafNodeWidth)
                                  .attr("height",leafNodeHeight);
                          }

                          function splitConditionSideBarRender(splitCondMap,clickedNodeData) {
                              //Clear out any existing text
                              sidebarBandCont.selectAll("text.splitColumnName, text.splitCondition")
                                  .transition()
                                  .attr('opacity',0)
                                  .remove();

                              var lastBandNbr = _.max(bandVBounds,function(band) {
                                  return band.depth;
                              }).depth;
                              //Create text for split conditions at correct level
                              var bands = sidebarSvg.selectAll("g.levelBandEven, g.levelBandOdd");
                              //Append Text for column position
                              bands.append("text")
                                  .attr("class","splitColumnName")
                                  .attr("text-anchor","end")
                                  .attr("x",sidebarWidth - splitCondTextMargin)
                                  .attr("y",12)
                                  .attr("dy",".35em")
                                  .text(function(d,i) {
                                      var splitCond = splitCondMap[i];
                                      if(splitCond) {
                                          var col = _.findWhere(scope.conditionColumns, {name: splitCondMap[i].colName});
                                          return col ? col.label : "";
                                      }
                                      else if(i === lastBandNbr) {
                                          return scope.targetValueFormatter({value: clickedNodeData.dominantTargetValue});
                                      }
                                      else {
                                          return null;
                                      }
                                  })
                                  .each(function(d) {
                                      d.origLevelVarText = this.textContent;
                                  });
                              //Append text for split conditions
                              bands.append("text")
                                  .attr("class", "splitCondition")
                                  .attr("x",sidebarWidth - splitCondTextMargin)
                                  .attr("y",12+ splitCondColLineHeight)
                                  .attr("text-anchor","end")
                                  .attr("dy",".35em")
                                  .text(function(d,i) {
                                      var splitCond = splitCondMap[i];
                                      if(splitCond) {
                                          var col = _.findWhere(scope.conditionColumns, {name: splitCond.colName});
                                          var format = col ? col.columnFormat : "";
                                          return scope.splitCondFormatter({splitCond: splitCond, format: format});
                                      }
                                      else {
                                          return null;
                                      }
                                  });

                              //Split text between lines and truncate/add tooltips if necessary to avoid sidebar overflowing into tree
                              bands.selectAll(".splitCondition")
                                  .each(function(d) {
                                      d.origSplitCondText = this.textContent;
                                      var textParams =  {
                                          // Decrease maxWidth and maxHeight by 5 to avoid truncation for defect S1129993
                                          maxWidth: sidebarWidth - splitCondTextMargin - 10,
                                          maxHeight: d.height - splitCondColLineHeight - splitCondTextMargin - 10,
                                          x: sidebarWidth - splitCondTextMargin,
                                          initLineVPad: 10 + splitCondColLineHeight,   // y offset of first line with 10pt margin on top
                                          anchor: "end",
                                          tooltipClass: "decisionTreeSplitCondTooltip"
                                      };
                                      var truncated = d3UtilsSvc.wrapTextByFixedWidth(sidebarBandCont,this,textParams,", ",splitCondLineVGap,true);
                                      if(truncated) {
                                          //Add mouse over and mouse out handlers for tooltip
                                          d3.select(this).on('mouseover',onSidebarSplitCondMouseOver);
                                      }

                                  });

                              //Append statistics for leaf node
                              drawLeafNodeStats(clickedNodeData,lastBandNbr);
                          }

                          function drawLeafNodeStats(nodeData,lastBandNbr) {
                              var band = sidebarBandCont.selectAll("g.levelBandEven, g.levelBandOdd")
                                  .filter(function(d) {
                                      return  d.bandNbr === lastBandNbr;
                                  });

                              var statLine = 1;
                              var displayArray = decisionTreeSvc.createStatDisplayArray(scope,nodeData);
                              var maxStatTxt = d3.max(_.map(displayArray, function (entry) {
                                  return d3UtilsSvc.getTextDimensions(sidebarSvg, entry.statTxt, 12).width;
                              }));
                              var statsWidth = splitCondTextMargin + maxStatTxt + 7,
                                availLblWidth = sidebarWidth - statsWidth;
                              angular.forEach(displayArray,function(entry) {
                                 //Append stat label text node
                                  band.append("text")
                                      .attr("class", "splitCondition sideBarStatLbl")
                                      .attr("x", sidebarWidth - statsWidth)
                                      .attr("y",splitCondDetailLineHeight * statLine + splitCondColLineHeight)
                                      .attr("text-anchor","end")
                                      .attr("dy",".35em")
                                      .text(entry.statLbl + ':');

                                  //Append stat value text node
                                  band.append("text")
                                      .attr("class", "splitCondition")
                                      .attr("x",sidebarWidth - splitCondTextMargin)
                                      .attr("y",splitCondDetailLineHeight * statLine + splitCondColLineHeight)
                                      .attr("text-anchor","end")
                                      .attr("dy",".35em")
                                      .text(entry.statTxt);

                                  statLine++;
                              });
                              // truncate labels that are too wide to fit in sidebar
                              d3.selectAll(".sideBarStatLbl")[0].forEach(function (element) {
                                  d3UtilsSvc.truncateTextWithDots2(element, availLblWidth, splitAllChars, "");
                              });
                          }
                      }

                      function splitCondsFromNodes(node,splitCondMap) {
                          var depth;
                          //If map doesn't exist, create it
                          if(!splitCondMap) {
                              splitCondMap = {};
                          }
                          if(!node.parent) {
                              return splitCondMap;
                          }

                          depth = _.findWhere(bandVBounds, {y: node.parent.y}).depth;
                          //Add entry to map for level
                          splitCondMap[depth] = {
                              colName: node.parentVarName,
                              lowerBounds: node.lowerBounds,
                              upperBounds: node.upperBounds,
                              nominalSplitValues: node.nominalSplitValues,
                              rangeType: node.rangeType

                          };
                          //If parent exist, recurse
                          if(node.parent) {
                              splitCondsFromNodes(node.parent,splitCondMap);
                          }

                          return splitCondMap;
                      }

                      function highlightRulePath(node) {
                          //Collect link ids from rule path
                          var linkIds = treeUtilSvc.collectFromLeafToParent(node,getLinkInfo);
                          //Turn off any previous highlight
                          treeCont.selectAll(".highlightLink")
                              .style('opacity',0);
                          //Highlight selected rule links
                          treeCont.selectAll(".highlightLink")
                              .filter(function(d) {
                                  return _.find(linkIds,function(linkId) {
                                      return linkId.from === d.source.id && linkId.to === d.target.id;
                                  });
                              })
                              .style('opacity',1);
                      }

                      function getLinkInfo(node) {
                          return{
                              to: node.id,
                              from: node.parent.id
                          };
                      }

                      function collectFromLeafToParent(node,parentProperty,collectFunction, collection) {
                          //Create collection as array if not exists
                          if(!collection) {
                              collection = [];
                          }
                          //If at root node, return collection
                          if(!node[parentProperty]) {
                              return collection;
                          }
                          //Call collection function for current node
                          collection.push(collectFunction(node));
                          //Collect from parent node
                          collectFromLeafToParent(node.parent,parentProperty,collectFunction,collection);

                          return collection;
                      }

                      function calculateRowBandingDims(nonLeafNodes,leafNodes,width) {
                          var levelYs = [];
                          var bandDims = [];
                          var accumHeight = 0;
                          angular.forEach(nonLeafNodes, function(node) {
                              if(levelYs.indexOf(node.y) === -1) {
                                  levelYs.push(node.y);
                              }
                          });
                          levelYs.push(leafNodes[0].y);
                          levelYs.sort(vsdUtil.singleFieldNaturalSortComparator());
                          for(var i = 0; i < levelYs.length; i++) {
                                var currY = levelYs[i],
                                    currHeight;
                                if(i < levelYs.length -1) {
                                    currHeight = levelYs[i+1] - currY;
                                }
                                else {
                                    currHeight = defaultSvgHeight - accumHeight;
                                }
                                bandDims.push({
                                    x: 0,
                                    y: currY,
                                    width: width,
                                    height: currHeight ,
                                    bandNbr: i
                                });
                                accumHeight += currHeight;
                          }
                          return bandDims;
                      }

                      function getDistinctLevelValues(node,distinctVals) {
                            var currVal = scope.targetValueFormatter({value: node.dominantTargetValue});
                            if(!distinctVals) {
                                distinctVals = [];
                            }
                            if(node.type === "LEAF") {
                                var existVal = distinctVals.indexOf(currVal) !== -1;
                                if(!existVal) {
                                    distinctVals.push(currVal);
                                }
                            }
                            else {
                                angular.forEach(node.children,function(child) {
                                    getDistinctLevelValues(child,distinctVals);
                                });
                            }
                            return distinctVals;
                      }

                      function getTreeLevelCount(node) {
                          if(node.type === "LEAF") {
                              return 1;
                          }
                          else {
                              var childLevelCounts = [];
                              for(var i = 0; i < node.children.length; i++) {
                                  childLevelCounts[i] = getTreeLevelCount(node.children[i]);
                              }
                              return _.max(childLevelCounts) + 1;
                          }
                      }

                      function getMinTreeHeight(numLevels) {
                          return (numLevels -1) *  minNonLeafLevelHeight + minLeafLevelHeight;
                      }

                      function createDiagonalLinePatternStroke() {
                          var g = defs.append("pattern")
                              .attr('id', 'diagonalLinePattern')
                              .attr('patternUnits', 'userSpaceOnUse')
                              .attr('width', '4')
                              .attr('height', '4')
                              .style("stroke-width",1);
                          g.append("rect")
                              .style("fill", "#8A8A8A")
                              .attr("width",8)
                              .attr("height",8);
                          g.append("path").attr("d", "M -1,2 l 6,0")
                              .style("stroke", "#000");
                      }

                      function onNodeMouseOver(nodeData) {
                          closeAllToolTips();
                          var eventNode = treeCont.selectAll(".nonLeafNode")
                              .filter(function(d){
                                  return nodeData.id === d.id;
                              })
                              .node();
                          scope.nodeMouseoverHandler({nodeData: nodeData, domNode: eventNode});
                          scope.$apply();
                      }

                      function onNodeMouseOut() {
                          scope.nodeMouseoutHandler();
                          scope.$apply();
                      }

                      function onLeafNodeMouseOver(nodeData) {
                          //Close any open tooltips
                          closeAllToolTips();
                          //Call handler
                          scope.leafNodeMouseoverHandler({nodeData: nodeData, splitConds: splitCondsFromNodes(nodeData), domNode: d3.event.currentTarget});
                          scope.$apply();
                      }

                      function onLeafNodeMouseOut() {
                          scope.leafNodeMouseoutHandler();
                          scope.$apply();
                      }

                      function onSidebarSplitCondMouseOver(bandData) {
                          //Close any open tooltips
                          closeAllToolTips();
                          //Get band level title and split condition
                          scope.sidebarSplitcondMouseoverHandler({levelData: {
                                                                              levelVarTxt: bandData.origLevelVarText,
                                                                              splitCondTxt: bandData.origSplitCondText
                                                                 },
                                                                domNode: this});
                          scope.$apply();
                      }

                      function onSidebarSplitCondMouseOut() {
                          scope.sidebarSplitcondMouseoutHandler();
                          scope.$apply();
                      }

                      function toggleNonLeafNodeTooltipReg(register) {
                          var handler = register ? onNodeMouseOver  : angular.noop;
                          treeCont.selectAll('.nonLeafNode')
                              .on('mouseover',handler);
                      }

                      function closeAllToolTips() {
                          onNodeMouseOut();
                          onSidebarSplitCondMouseOut();
                          onLeafNodeMouseOut();
                      }

                      function initLeafNodeSelection(leafNodes) {
                          angular.forEach(leafNodes,function(node) {
                              node.ruleSelected = scope.leafNodeSelectedComparator({node: node});
                          });
                      }

                      function createRMBMenu(clickedNodeData) {
                         var items = [],
                             menu;
                          if(clickedNodeData.type !== "LEAF") {
                              //Add item to remove column from tree
                              items.push({
                                  label: scope.resources.decisionTree_nodeRmbRemoveColLbl_txt,
                                  onClick: function() {
                                      scope.removeTreeColumnHandler({colName: clickedNodeData.name});
                                      closeAllToolTips();
                                      scope.$apply();
                                  }
                              });
                              //Add item to truncate tree below node
                              items.push({
                                  label: scope.resources.decisionTree_nodeRmbRemoveDescendantsLbl_txt,
                                  onClick: function(){
                                      pruneTreeBelowNode(clickedNodeData);
                                      closeAllToolTips();
                                      scope.$apply();

                                  }
                              });
                          }

                          //Get list of menu items and create menu
                          menu = vsdUtil.menuFromItemsArray(items,beforeMenuDisplay,afterMenuProcessing);

                          return menu;

                          function beforeMenuDisplay() {
                              //Hide mouse over popup
                              closeAllToolTips();
//                              toggleNonLeafNodeTooltipReg(false);
                          }

                          function afterMenuProcessing() {
//                              toggleNonLeafNodeTooltipReg(true);
                          }
                      }

                      function createTextContrast(defs) {
                          var filter = defs.append("filter")
                              .attr("id", "textContrast");

                          filter.append("feGaussianBlur")
                              .attr("in", "SourceAlpha")
                              .attr("stdDeviation", 1)
                              .attr("result", "blur");
                          filter.append("feOffset")
                              .attr("in", "blur")
                              .attr("dx", 1)
                              .attr("dy", 1)
                              .attr("result", "offsetBlurredAlpha");
                          filter.append("feGaussianBlur")
                              .attr("in", "SourceAlpha")
                              .attr("stdDeviation",0.8)
                              .attr("result", "glow");

                          var feMerge = filter.append("feMerge");
                          feMerge.append("feMergeNode")
                              .attr("in", "glow");
                          feMerge.append("feMergeNode")
                              .attr("in", "offsetBlurredAlpha");
                          feMerge.append("feMergeNode")
                              .attr("in", "SourceGraphic");
                      }

                      function measureTruncElipseWidth(svg,fontSize) {
                          var retWidth;
                          var measureText = svg.append("text")
                              .text("...")
                              .attr("dy", "14px")
                              .attr("text-anchor", "middle")
                              .attr("filter",'url("#textContrast")')
                              .style("opacity",0);

                          retWidth = measureText.node().getBBox().width;
                          measureText.remove();
                          return retWidth;
                      }

                      function getNodeStartX(d) {
                          //See if old x coord available
                          if(d.oldX) {
                              return d.oldX;
                          }
                          //If not,place it to the left or right boundary (which ever is closer)
                          else {
                              var retX =  treeWidth - d.x <  d.x ? treeWidth : 0;
                              return retX;
                          }
                      }

                      function getNodeStartY(d) {
                          return d.oldY ? d.oldY : d.y;
                      }

                      function zoom() {
                          var newCoords = [d3.event.translate[0], 0];
                          treeCont.attr("transform", "translate(" + newCoords + ")");
                      }

                  }

                  function createSVG(el,width,height,margins) {
                    var finalWidth = width - margins.left - margins.right,
                        finalHeight = height - margins.top - margins.bottom;
                    //Remove existing svg
                    angular.element(el).empty();
                    return d3.select(el).append("svg")
                        .attr("width",finalWidth)
                        .attr("height",finalHeight)
                        .append("g")
                        .attr("transform", "translate(" + margins.left + "," + margins.top + ")");
                  }
             }
        };
  }]);

});

},
'vsd/scenario/exploreDirect/pchartHoverMenuSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .factory('pchartHoverMenuSvc', ['dojoDepends','vsdUtil', function(dojoDepends,vsdUtil) {
        return {
            hoverMenu: function(params) {
                var menu,
                    selectPart,
                    isVisible = true,
                    resources = dojoDepends.resources;

                function init() {
                    menu = d3.select("body").append("div")
                        .attr("class","hoverMenu")
                        .style("opacity", 0)
                        .html(
                            '<span>' +
                                '<button class="hoverMenuButton sasAddIcon" title="' + resources.scenario_PartitionChartHoverMenu_toggleIncFilter_txt + '"></button>' +
                                '<button class="hoverMenuButton sasClearIcon" title="' +resources.scenario_PartitionChartHoverMenu_toggleExcFilter_txt + '"></button>' +
                                '</span>'
                        )
                        .on("mouseover",function(){menu.style("opacity","1")});

                    menu.select(".sasAddIcon").on("click", function() {
                        params.clickHandler(false);
                    });
                    menu.select(".sasClearIcon").on("click", function() {
                        params.clickHandler(true);
                    });

                }

                function deletePopupDiv() {

                }

                function hide() {
                    hideShow(0);
                }

                function show(params){
                    hideShow(params.alpha);
                }

                function hideShow(setting) {
                    menu.style("opacity",setting);
                }

                function toggleInclusionFilter() {

                    var selectRect = d3.select( selectPart);
                    toggleFilter(selectRect,"parentSelectInclude")
                }

                function toggleExclusionFilter() {

                    var selectRect = d3.select( selectPart);
                    toggleFilter(selectRect,"parentSelectExclude")
                }

                function toggleFilter(selectRect,selectClass) {

                    if(selectRect.attr("class") === selectClass) {
                        selectRect.attr("class","parent");
                    }
                    else {
                        selectRect.attr("class",selectClass)
                    }

                }

                function runEventFilter() {
                    hide();
                   /* //Get all selected include nodes
                    var includeSel = d3.selectAll(".parentSelectInclude"),
                        excludeSel = d3.selectAll(".parentSelectExclude"),
                        retConds = null;

                    retConds = addConditionsFromNodes(includeSel[0],false);
                    retConds = retConds.concat(addConditionsFromNodes(excludeSel[0],true));
                    //TODO:  Invoke the proper controller function

                    function addConditionsFromNodes(nodes,negate) {
                        var lastVar,
                            d,
                            currCond,
                            newConds = [],
                            currOperator,
                            parentCond,
                            j;
                        for(var i = 0; i < nodes.length; i++) {
                            //Assign current node to variable
                            d = nodes[i];

                            currOperator = d.__data__.operator;
                            //Create new filter if on first node, reached, reached a new variable or using between operator
                            if(!lastVar || currOperator.name != "IN" || lastVar !=  d.__data__.varName) {
                                currCond =  {
                                    operator: currOperator,
                                    operand2Constant: "",
                                    childConditions: []
                                };
                                //Deal with NOT
                                if(negate) {
                                    parentCond = {
                                        operator: _.findWhere(operators,{name: "NOT"}),
                                        childConditions: []
                                    }
                                    parentCond.childConditions.push(currCond);
                                    newConds.push(parentCond)
                                }
                                else {
                                    newConds.push(currCond);
                                }
                            }


                            //Assign filter value based on operator
                            if( currCond.operator.name === "AND") {
                                //Assign child conditions to current condition for range
                                for(j = 0; j < 2; j++) {
                                    //Update the operand 1 column in each child condition to use the parent operand 1 column
                                    d.__data__.childConditions[j].operand1ColumnInfo =    scenarioMetadata.condColMap[d.__data__.varName];
                                    currCond.childConditions.push(d.__data__.childConditions[j]);
                                }
                            }
                            else {
                                //Assign filtered column
                                currCond.operand1ColumnInfo =  scenarioMetadata.condColMap[d.__data__.varName];
                                if(currCond.operator.name === "IN") {

                                    currCond.operand2Constant +=   d.__data__.varValue + ", "
                                }
                                else {
                                    currCond.operand2Constant =  d.__data__.operand2Constant;
                                }
                            }


                            lastVar =  d.__data__.varName;
                        }
                        //Trim the trialing comma from the RHS if IN operator
                        newConds.forEach(function(f){
                            if((f.operator.name=== "IN" || f.operator.name === "NOT IN") &&
                                vsdUtil.getRight(f.operand2Constant,2) === ", ") {
                                f.operand2Constant = vsdUtil.dropRight(f.operand2Constant,2);
                            }
                        })

                        return newConds;
                    }*/
                }

                function setPosition(coords) {
                    menu.style("left", (coords.left) + "px");
                    menu.style("top", (coords.top) + "px");
                    return this;
                }

                function getMenuWidth() {
                    return menu.style("width");
                }

                function setVisibility(visiblity) {
                    var visibility;
                    isVisible = visiblity;
                    if(!isVisible) {
                        visibility = 'hidden';
                        isVisible = false;
                    }
                    else {
                        visibility = 'visible';
                        isVisible = true;
                    }
                    menu.style("visibility",visibility);

                }

                function destroy() {
                   d3.selectAll(".hoverMenu").remove();
                }


                //Do initialization of local variables
                init();

                //Return interface to this module
                return {
                    hide: hide,
                    show: show,
                    setSelectPart: function(inSelectPart) {
                        selectPart = inSelectPart;
                        return this;
                    },
                    setPosition: setPosition,
                    getMenuWidth: getMenuWidth,
                    setVisiblity: setVisibility,
                    destroy: destroy
                }

            }


        }
    }]);


});

},
'vsd/scenario/exploreDirect/ScenarioDirectCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioDirectCtrl', ['$scope',
                                       'scenarioSvc',
                                       'messageSvc',
                                       'dojoModules',
    function ($scope,
              scenarioSvc,
              messageSvc,
              dojoModules) {
        var scenarioReady = false;

        //region *** Private Methods ***
        function getRecordCountForConditionFilter() {
            if(!scenarioReady || !$scope.data.activeRule.condition || $scope.data.activeRule.condition.childConditions.length === 0) {
                $scope.setCounters($scope.counters.totalRecCnt);
                return;
            }

            //Validate scenario
            if($scope.validateScenario().detail.conditionsValid) {
                //Turn on load mask
	            $scope.showLoadingMask($scope.resources.scenario_Controller_calculatingWaitLbl_txt);
	            //Invoke service to retrieve count
                scenarioSvc.getRecordCountForConditionFilter($scope.data.activeRule.condition)
                    .then(function(results) {
                        //Remove load mask
		                  $scope.hideLoadingMask();
                        //Set counters for gauges
                        var filteredRecCnt = results[0].count;
                        $scope.setCounters(filteredRecCnt);
                    });
            }
        }

        function createDefaultConditions() {
            if(!$scope.data.activeRule.condition) {
                $scope.data.activeRule.condition = $scope.createBlankConjCondition("AND");
            }

            if($scope.data.activeRule.condition.childConditions.length === 0) {
                $scope.data.activeRule.condition.childConditions.push($scope.createBlankCondition($scope.data.activeRule));
            }
        }


        function onScenarioReady() {
            scenarioReady = true;
            if($scope.scenario.events.length === 0) {
                //Create blank rule and set active
                $scope.scenario.events.push($scope.createBlankRule(0));
                $scope.setActiveRule($scope.scenario.events[0]);
            }

            //Initialize action variables
            $scope.initActions();
            getRecordCountForConditionFilter();
            createDefaultConditions();
            checkTopLevelCondition();
            $scope.getPreviewData();
        }

        function conditionChange(evt) {
            if(evt) {
                evt.preventDefault();
            }
            getRecordCountForConditionFilter();
            $scope.getPreviewData();
            $scope.setDirty(true);
        }

        function checkTopLevelCondition() {
            if($scope.data.activeRule && $scope.data.activeRule.condition && $scope.data.activeRule.condition.operator.name !== "AND") {
                messageSvc.showInfo($scope.resources.scenario_ManualConditionEditor_nonANDWarningMsg_txt);
            }
        }
        //endregion

        $scope.createDefaultConditions = createDefaultConditions;

        //region *** Event Subscriptions ***
        $scope.$watch('data.scenarioReady',function(ready,oldReady) {
            if(ready && ready !== oldReady) {
                onScenarioReady();
            }
        });

        $scope.$on(vsmCommonConst.SCENARIO_SAVE_COMPLETE,function() {
            createDefaultConditions();
        });
        $scope.$on(vsmCommonConst.CONDITION_BUILDER_DELETE,conditionChange);
        $scope.$on(vsmCommonConst.CONDITION_BUILDER_CHANGE,conditionChange);
        $scope.$on(vsmCommonConst.CONDITION_BUILDER_OPER_CHANGE,checkTopLevelCondition);

        $scope.$watch("data.activeRule.condition",function(newVal,oldVal) {
            if(scenarioReady) {
                if(newVal !== oldVal) {
                    if(newVal === null) {
                        conditionChange();
                    }

                }
            }
        });

        //endregion

        //When controller initialized, retrieve count
        if($scope.data.scenarioReady) {
            onScenarioReady();
        }
    }]);

});

},
'vsd/scenario/exploreDirect/ScenarioExplorerCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioExplorerCtrl', ['$scope',
                                         '$timeout',
                                         'scenarioSvc',
                                         'vsdUtil',
                                         'messageSvc',
                                         'dojoModules',
        function ($scope,
                  $timeout,
                  scenarioSvc,
                  vsdUtil,
                  messageSvc,
                  dojoModules) {
            var scenarioReady = false;

            //region *** Private Functions ***
            function condVarsToComboRequestCols() {
                var outCols = [];

                $scope.data.usedConditionColumns.forEach(function(sc) {
                    var woc = _.find($scope.data.windowOutCols,function(oc) {
                        return oc.name === sc.name;
                    }) || {};

                    var reqVar = {
                        physColumnName: sc.name,
                        dataType: sc.dataType,
                        columnFormat: sc.columnFormat,
                        binnedPhysColumnName: null,
                        binBoundaries: woc.bins
                    };
                    outCols.push(reqVar);
                });
                return outCols;
            }

            function getUniqueCombos() {
                var filterStr,
                    reqPayload;

                //Exit if not ready yet or no condition columns
                if(!scenarioReady || $scope.data.usedConditionColumns.length === 0) {
                    return;
                }

                //Exit if conditions invalid
                if(!$scope.validateScenario().detail.conditionsValid){
                    return;
                }
                //Turn on load mask
                $scope.showLoadingMask($scope.resources.scenario_Controller_calculatingWaitLbl_txt);

                if($scope.getNonGroupingCondCount($scope.data.activeRule.condition) > 0) {
                    filterStr = vsdUtil.conditionToFilterString($scope.data.activeRule.condition,true);
                }
                reqPayload = {comboRequestColumns: condVarsToComboRequestCols(),whereClause: filterStr};
                scenarioSvc.getUniqueCombos(reqPayload,$scope.scenario.name)
                    .then(function(results) {
                        //Replace counters variable with new values - replaced so watchers will see the changes
                        $scope.setCounters(results.count);
                        //Update pchart data in a #timeout so that pchart div has height - if do this without timeout, angular hasn't applied the ng-show change yet and pchart svg gets zero height
                        $timeout(function() {
                            //Set pchart data
                            results.varName = $scope.resources.scenario_explorerView_allCell_txt; //Field comes NULL from server since it must be localized
                            results.operatorRHSStr  = $scope.resources.scenario_explorerView_allCell_txt; //Field comes NULL from server since it must be localized
                            $scope.explData.pchartData = results;
                            //Remove load mask
                            $scope.hideLoadingMask();
                        });

                    });
            }

            function updateSortOption(newSortVal) {
                //Copy sort options array
                var newSortOpts = angular.copy($scope.explData.sortOptions);
                //Reset sort icon
                var selected = _.find(newSortOpts,function(item){return item.iconClass !== null;});
                var newSel = _.findWhere(newSortOpts,{sortType: newSortVal});
                newSel.iconClass = selected.iconClass;
                selected.iconClass = null;
                //Reset sortOptions array
                $scope.explData.sortOptions = newSortOpts;
                //Perform sort
                $scope.setPChartSort(newSortVal);
            }

            function mergeConditions(cond) {
                var keepConds = {},
                    orMap = {},
                    deleteList = [],
                    mergeList = [],
                    existMin,
                    currMax,
                    existMaxNbr,
                    currMinNbr,
                    currKeepOp,
                    mergeOp,
                    currParent,
                    currParentChildList,
                    mergeDataType;

                //Build set of records for sort
                buildMergeList(cond.childConditions,0,cond);

                //Sort merge list by key so mergeable conditions are adjacent
                mergeList.sort(function(condA,condB){
                    if(condA.level < condB.level) {
                        return -1;
                    }
                    else if(condA.level > condB.level) {
                        return 1;
                    }
                    else {
                        if(condA.operand1ColumnName < condB.operand1ColumnName) {
                            return -1;
                        }
                        else if(condA.operand1ColumnName > condB.operand1ColumnName) {
                            return 1;
                        }
                        else {
                            if(condA.sortOperator < condB.sortOperator) {
                                return -1;
                            }
                            else if(condA.sortOperator > condB.sortOperator) {
                                return 1;
                            }
                            else {
                                if(condA.sortOperand < condB.sortOperand) {
                                    return -1;
                                }
                                else if(condA.sortOperand < condB.sortOperand) {
                                    return 1;
                                }
                                else {
                                    return 0;
                                }
                            }
                        }
                    }

                });
                //Iterate through sorted conditions, doing combining
                mergeList.forEach(function(mergeRec) {
                    var currKeepRec = keepConds[createMergeLookupKey(mergeRec)];
                    if(!currKeepRec) {
                        currKeepRec = mergeRec;
                        keepConds[createMergeLookupKey(mergeRec)] = mergeRec;

                    }
                    else {
                        if($scope.conditionsIdentical(currKeepRec.cond,mergeRec.cond)) {
                            addToDeleteList(mergeRec);
                        }
                        else {
                            currKeepOp = currKeepRec.cond.operator.name;
                            mergeOp = mergeRec.cond.operator.name;
                            mergeDataType = mergeOp === "AND" ? mergeRec.cond.childConditions[0].operand1ColumnInfo.dataType :   mergeRec.cond.operand1ColumnInfo.dataType;

                            if(mergeDataType !== vsmCommonConst.DATA_TYPE_CHAR) {
                                if(currKeepOp !== "IS MISSING") {
                                    existMaxNbr = currKeepOp === "AND" ? parseFloat(currKeepRec.cond.childConditions[1].operand2Constant.replace(",","")) : parseFloat(currKeepRec.cond.operand2Constant.replace(",",""));
                                }
                                currMinNbr = mergeOp === "AND" ? parseFloat(mergeRec.cond.childConditions[0].operand2Constant.replace(",","")) : parseFloat(mergeRec.cond.operand2Constant.replace(",",""));
                            }
                            //Contiguous ranges - combine range
                            if(mergeDataType !== vsmCommonConst.DATA_TYPE_CHAR &&
                               !(currKeepOp === $scope.LOWER_BOUND_OPERATOR_NAME && mergeOp === $scope.UPPER_BOUND_OPERATOR_NAME) &&
                               existMaxNbr === currMinNbr &&
                               currKeepRec.cond.operator.name !== "IS MISSING") {
                                existMin =   currKeepOp === "AND" ? currKeepRec.cond.childConditions[0].operand2Constant : parseFloat(currKeepRec.cond.operand2Constant.replace(",","")) ;
                                currMax = mergeOp === "AND" ? mergeRec.cond.childConditions[1].operand2Constant : parseFloat(mergeRec.cond.operand2Constant.replace(",",""));
                                //////////////////////////////////////////////////////////////
                                //Determine how to combine based on operands
                                /////////////////////////////////////////////////////////////
                                //For <= and BETWEEN, make <= max of between
                                if(currKeepOp === $scope.LOWER_BOUND_OPERATOR_NAME && mergeOp === "AND") {
                                    currKeepRec.cond.operator = _.findWhere($scope.data.operators,{name: $scope.LOWER_BOUND_OPERATOR_NAME});
                                    currKeepRec.cond.operand2Constant = currMax;
                                    addToDeleteList(mergeRec);

                                }
                                //Keep lower from currKeep and upper from merge
                                else if(currKeepOp === "AND" && mergeOp === "AND") {
                                    currKeepRec.cond.childConditions[0].operand2Constant = existMin;
                                    currKeepRec.cond.childConditions[1].operand2Constant = currMax;
                                    addToDeleteList(mergeRec);
                                }
                                //If between and >=, switch sign to >= and keep larger value
                                else if(currKeepOp === "AND" && mergeOp === $scope.UPPER_BOUND_OPERATOR_NAME) {
                                    currKeepRec.cond.operator = mergeRec.cond.operator;
                                    currKeepRec.cond.operand1ColumnInfo = mergeRec.cond.operand1ColumnInfo;
                                    currKeepRec.cond.operand2Constant = existMin;
                                    currKeepRec.cond.childConditions = [];
                                    addToDeleteList(mergeRec);

                                }
                            }
                            //Non-contiguous - OR together
                            else {
                                //Save variables so they can be OR'd together later
                                var orArray,
                                    varName;
                                if(currKeepRec.cond.operator.name === "AND") {
                                    varName = currKeepRec.cond.childConditions[0].operand1ColumnInfo.name;
                                }
                                else {
                                    varName = currKeepRec.cond.operand1ColumnInfo.name;

                                }
                                orArray = orMap[varName];

                                if(!orArray) {
                                    orArray = [];
                                    orMap[varName] = orArray;
                                }

                                if(!recInOrList(orArray,currKeepRec)) {
                                    orArray.push(currKeepRec);
                                }
                                if(!recInOrList(orArray,mergeRec)) {
                                    orArray.push(mergeRec);
                                }

                                //Put both recs on delete list since they will be replaced by an OR condition at their level
                                deleteList.push(currKeepRec);
                                addToDeleteList(mergeRec);   //Only account for NOT for second record, since first one should keep the NOT parent
                                //Make merge rec new keep rec so it can keep combining with others
                                keepConds[createMergeLookupKey(mergeRec)] = mergeRec;
                            }
                        }
                    }
                });
                //Process deletes
                deleteList.forEach(function(delRec) {
                    var childIdx;
                    currParentChildList = delRec.parent.childConditions;
                    //Regular condition rather than conjunction
                    if(delRec.cond)  {
                        childIdx = currParentChildList.indexOf(delRec.cond);
                    }
                    else {
                        childIdx = currParentChildList.indexOf(delRec);
                    }

                    if(childIdx !== -1) {
                        currParentChildList.splice(childIdx,1);
                    }
                });
                //Process ORs
                for(var key in orMap) {
                    //Build or condition and add it to current parent
                    var orArray = orMap[key],
                        orCond;
                    currParent = orArray[0].parent;
                    //If already have OR as parent, just clear out children
                    if(currParent.operator.name === "OR") {
                        orCond = currParent;
                    }
                    //Otherwise, create new OR
                    else {
                        orCond = {
                            operator: _.findWhere($scope.data.operators,{name:"OR"}),
                            childConditions: []
                        };
                        currParent.childConditions.push(orCond);
                    }

                    //Add all conditions in orArray to OR cond (can't just add the array since these aren't only condition objects
                    orArray.forEach(function(childObj) {orCond.childConditions.push(childObj.cond);});
                }

                //Remove parent references
                removeParentRefs(cond);

                //Remove any parents with no children
                $scope.removeIncompleteConditions(cond,cond.childConditions,0);

                function recInOrList(list,rec) {
                    return _.find(list,function(currRec) {
                        return currRec.sortKey === rec.sortKey;
                    });
                }

                function removeParentRefs(cond) {
                    if(cond.hasOwnProperty("parent")) {
                        delete cond["parent"];
                    }
                    if(cond.childConditions.length > 0) {
                        cond.childConditions.forEach(function(child) {
                            removeParentRefs(child);
                        });
                    }
                }

                function addToDeleteList(mergeRec) {
                    if(mergeRec.parent.operator.name === "NOT") {
                        deleteList.push(mergeRec.parent);
                    }
                    else {
                        deleteList.push(mergeRec);
                    }
                }

                function createMergeLookupKey(mergeRec) {
                    if(mergeRec.cond.operator.name === "AND") {
                        return mergeRec.level+mergeRec.cond.childConditions[0].operand1ColumnInfo.name;
                    }
                    else {
                        return mergeRec.level+mergeRec.cond.operand1ColumnInfo.name;
                    }


                }

                function buildMergeList(condList,inLevel,parent) {
                    var level = inLevel;

                    condList.forEach(function(cond) {
                        var sortOperator,
                            sortOperand,
                            operand1ColumnName;

                        if(cond.operator.type !== "LOGICAL" || cond.operator.name === "AND" || cond.operator.name === "IS MISSING") {
                            sortOperand = cond.operand2Constant;
                            if(cond.operator.name === "IN") {
                                sortOperator = 1;
                                operand1ColumnName = cond.operand1ColumnInfo.name;
                            }
                            else {
                                if(cond.operator.name === "IS MISSING") {
                                    operand1ColumnName = cond.operand1ColumnInfo.name;
                                    sortOperator = 2;
                                }
                                else if(cond.operator.name === $scope.LOWER_BOUND_OPERATOR_NAME) {
                                    operand1ColumnName = cond.operand1ColumnInfo.name;
                                    sortOperator = 3;
                                }
                                else if(cond.operator.name === "AND") {
                                    operand1ColumnName = cond.childConditions[0].operand1ColumnInfo.name;
                                    sortOperand = cond.childConditions[0].operand2Constant;
                                    sortOperator = 4;
                                }
                                else {
                                    operand1ColumnName = cond.operand1ColumnInfo.name;
                                    sortOperator = 5;
                                }
                            }
                            var sortRec = {
                                cond: cond,
                                parent: parent,
                                level: level,
                                operand1ColumnName: operand1ColumnName,
                                sortOperator: sortOperator,
                                sortOperand: sortOperand,
                                sortKey: level+operand1ColumnName+sortOperator+sortOperand

                            };
                            mergeList.push(sortRec);
                        }
                        else {
                            //Add parent property to allow for deletion later
                            cond.parent = parent;
                        }

                        if(cond.operator.name !== "AND" && cond.childConditions && cond.childConditions.length > 0) {
                            var newLevel = inLevel;
                            if(cond.operator.name !== "OR") {
                                newLevel++;
                            }
                            buildMergeList(cond.childConditions,newLevel,cond);
                        }
                    });
                }
            }
            function explAddConditions(inConds) {
                var condCopy = angular.copy($scope.data.activeRule.condition);
                $scope.data.activeRule.condition.childConditions = [];
                //Determine if any incoming conditions already exist in duplicate - if so, warn user
                condCopy.childConditions.forEach(function(existCond) {
                    inConds.forEach(function(newCond) {
                        //If conditions identical, publish a message to this effect
                        if($scope.conditionsIdentical(existCond,newCond)) {
                            //TODO:  Signal warning with shared component
                            /*topic.publish(appTopics.SHOW_WARNING_MESSAGE,{
                                warnings: [string.substitute(resources.scenario_Controller_duplicateConditionMsg_txt,[conditionToFilterString(newCond,true,true)])]

                            });*/
                        }
                    });
                });

                inConds.forEach(function(newCond) {
                    newCond.executionOrder = ++$scope.maxCondNbr;
                    condCopy.childConditions.push(newCond);
                });
                //Merge conditions, if possible
                if($scope.getNonGroupingCondCount(condCopy) > 1) {
                    mergeConditions(condCopy);
                }

                $scope.data.activeRule.condition.childConditions = condCopy.childConditions;

                //Set dirty flag
                $scope.setDirty(true);

            }

            function createDefaultParentCond() {
                if(!$scope.data.activeRule.condition) {
                    $scope.data.activeRule.condition = $scope.createBlankConjCondition("AND");
                }
            }

            function onScenarioReady() {
                scenarioReady = true;
                if($scope.scenario.events.length === 0) {
                    //Create blank rule and set active
                    $scope.scenario.events.push($scope.createBlankRule(0));
                    $scope.setActiveRule($scope.scenario.events[0]);
                }
                //Initialize action variables
                $scope.initActions();
                createDefaultParentCond();
                getUniqueCombos();
                $scope.getPreviewData();
            }

            function addConditionColumns(rows) {
                //Get column information
                var addCols = _.pluck(_.pluck(rows,"data"),"outCol");
                //Add columns
                var addedCols = $scope.addConditionColumns(addCols);
                //Mark columns on tree as used
                for(var i = 0; i < addedCols.length; i++) {
                    var matchRow = _.find(rows,function(row) {
                        return row.data.outCol.id === addedCols[i].id;
                    });
                    vsdUtil.markDroppedColumn(matchRow.element,addedCols[i]);
                }
            }

            function deleteAllConditions() {
                //Empty conditions array for current Event
                if($scope.data.activeRule) {
                    vsdUtil.clearArray($scope.data.activeRule.condition.childConditions);
                }
                //Set dirty flag
                $scope.setDirty(true);
            }

            function deleteAllConditionCols() {
                var doInputFieldRefresh = false;
                //Remote all conditions
                deleteAllConditions();
                //Remove used styling on variable list
                $scope.formatScenarioInputCols();
                //Remove all event condition variables
                $scope.data.usedConditionColumns = [];
                //Set dirty flag
                $scope.setDirty(true);
            }

            function columnIsHighCardinalityChar(col) {
                return col.dataType === vsmCommonConst.DATA_TYPE_CHAR && col.distinctCount > vsmCommonConst.PCHART_MAX_CHAR_LEVELS;
            }


            function showTooManyCharLevelsMsg() {
                var msg = dojoModules.string.substitute($scope.resources.pchart_cannotHighCardinalityCharColumnMsg_txt,[vsmCommonConst.PCHART_MAX_CHAR_LEVELS]);
                messageSvc.showInfo(msg);
            }

            //endregion

            //region *** ViewModel Data ***
            $scope.SORT_FREQ_ASC = 'SORT_FREQ_ASC';
            $scope.SORT_FREQ_DESC = 'SORT_FREQ_DESC';
            $scope.SORT_VAL_ASC  = 'SORT_VAL_ASC';
            $scope.SORT_VAL_DESC  = 'SORT_VAL_DESC';

            $scope.PCT_DISPLAY = 'PCT_DISPLAY';
            $scope.NBR_DISPLAY = 'NBR_DISPLAY';
            $scope.explOpts = {
                chartSort:          $scope.SORT_VAL_ASC,
                chartLabelDisplay:  $scope.PCT_DISPLAY
            };

            $scope.explData = {
                pchartData: {},
                breadCrumbLevels: null,
                sortOptions: [
                    {sortType:  $scope.SORT_VAL_ASC, label: $scope.resources.scenario_ScenarioPartChart_sortValAsc_txt, iconClass: 'sasRadioSelectedIcon',onClick: function(){updateSortOption($scope.SORT_VAL_ASC);}},
                    {sortType:  $scope.SORT_VAL_DESC, label: $scope.resources.scenario_ScenarioPartChart_sortValDesc_txt, iconClass: null,onClick: function(){updateSortOption($scope.SORT_VAL_DESC);}},
                    {sortType:  $scope.SORT_FREQ_ASC, label: $scope.resources.scenario_ScenarioPartChart_sortFreqAsc_txt, iconClass: null,onClick: function(){updateSortOption($scope.SORT_FREQ_ASC);}},
                    {sortType:  $scope.SORT_FREQ_DESC, label: $scope.resources.scenario_ScenarioPartChart_sortFreqDesc_txt, iconClass: null,onClick: function(){updateSortOption($scope.SORT_FREQ_DESC);}}
                ]
            };


            //endregion

            //region *** ViewModel Functions ****
            $scope.$watch('data.scenarioReady',function(ready,oldReady) {
                if(ready && ready !== oldReady && $scope.scenario.editMode === $scope.EDIT_MODE_EXPLORE) {
                    onScenarioReady();
                }
            });

            //Watch for changes to condition columns.  Handles refresh after add, delete, delete all and reorder
            $scope.$watchCollection('data.usedConditionColumns',function(newStuff,oldStuff) {
                if($scope.data.usedConditionColumns.length > 0) {
                    getUniqueCombos();
                }
                else {
                    //Reset the partition chart data
                    $scope.explData.pchartData = {};
                }
            });

            //Watch for changes to conditions.  Handles refreshs after add, delete, delete all (will this work for change)?
            $scope.$watchCollection('data.activeRule.condition.childConditions',function(newStuff,oldStuff) {
                getUniqueCombos();
                if(scenarioReady) {
                    $scope.getPreviewData();
                }
            });

            //Watch for input columns to be available
            $scope.$watch('data.windowTableOutColsCond',function(newVal) {
                if(newVal && newVal.length > 0) {
                    $scope.formatScenarioInputCols();
                }
            });


            $scope.$on(vsmCommonConst.SCENARIO_SAVE_COMPLETE,function() {
                createDefaultParentCond();
            });

            //Listen for change events in conditions list
            $scope.$on(vsmCommonConst.CONDITION_BUILDER_CHANGE,function(evt) {
                evt.preventDefault();
                getUniqueCombos();
                if(scenarioReady) {
                    $scope.getPreviewData();
                }
                $scope.setDirty(true);
            });



            //Listen for changes to bin columns from dialog
            $scope.$watchCollection('data.binUpdateColumns', function() {
                if($scope.data.binUpdateColumns.length > 0) {
                    //Get new unique combos data
                    getUniqueCombos();
                }
            });

            $scope.removeCrumbsTo = function(idx) {
                $scope.explData.breadCrumbLevels = _.reject($scope.explData.breadCrumbLevels,function(crumb){
                    return crumb.depth >= idx;
                });
            };

            $scope.setPChartLabelDisplay = function(newDisplay) {
                $scope.explOpts.chartLabelDisplay = newDisplay;
            };

            $scope.setPChartSort = function(newSort) {
                $scope.explOpts.chartSort = newSort;
            };

            $scope.pchartAcceptDrop = function(source,nodes) {

	            if ( $scope.scenario.editMode !== "E") {
                    return true;
                }

                var goodNodes = true,
                    i,
                    node;

                //Check to see that drops are allowed (conditions must be valid)
                if(!$scope.validateScenario().detail.conditionsValid){
                    messageSvc.showError([$scope.resources.scenario_ScenarioPartChart_dropColumnDisabledMsg_txt]);
                    return false;
                }
                else {
                    messageSvc.closeMessage();
                    for(i = 0; i < nodes.length; i++) {
                        node = nodes[i];
                        if(node.custNodeData.outCol.executionOrder === 0) {
                            goodNodes = false;
                            break;
                        }
                        else if(node.custNodeData.outCol.binCount === 0) {     //TODO:  Test this once window is fixed to produce this condition
                            goodNodes = false;
                            messageSvc.showError(dojoModules.string.substitute($scope.resources.scenario_explorerNoBinsMsg_txt,[node.custNodeData.outCol.label]));
                            break;
                        }
                    }
                    var highCardCharVar = _.some(nodes,function(node) {
                        return columnIsHighCardinalityChar(node.custNodeData.outCol);
                    });
                    if(highCardCharVar) {
                        showTooManyCharLevelsMsg();
                        return false;
                    }

                    return goodNodes;
                }
            };

            $scope.pchartOnColumnDrop = function(source, nodes, copy) {
                var newCols = [];
                angular.forEach(nodes,function(node) {
                    newCols.push({
                        element: node,
                        data: node.custNodeData
                    });
                });
                //Invoke the parent controller function to add the new columns
                //var addedCnt = $scope.addConditionColumns(newCols);
                addConditionColumns(newCols);
            };

            $scope.onInputFieldsDblClick = function(selRow) {
                if(columnIsHighCardinalityChar(selRow.data.outCol)) {
                    showTooManyCharLevelsMsg();
                    return;
                }
                addConditionColumns([selRow]);
            };

            $scope.getInputGridRMBMenuItems = function(rows) {
                var me = this,
                    condsContained = 0,
                    containsAnyCondCols = false,
                    containsAllCondCols = false,
                    containsHighCardinalityChar = false;

                //Determine how many condition columns in selection
                angular.forEach(rows,function(row) {
                    if(_.findWhere($scope.data.usedConditionColumns,{id: row.data.outCol.id})) {
                        condsContained++;
                    }
                    if(!containsHighCardinalityChar) {
                        containsHighCardinalityChar = columnIsHighCardinalityChar(row.data.outCol);
                    }
                });
                containsAnyCondCols = condsContained > 0;
                containsAllCondCols = condsContained === rows.length;

                var inputRowMenuItems = [];
                var inputGridAddColumnItem = {
                    label: $scope.resources.scenario_ScenarioPartChart_inputGrid_addColMenuItem_txt,
                    disabled: containsAllCondCols || containsHighCardinalityChar,
                    onClick: function(evt) {
                        addConditionColumns(rows);
                        $scope.$apply(); //Must $apply here since event originates in Dojo
                    }
                };
                inputRowMenuItems.push(inputGridAddColumnItem);
                var inputGridRemoveColumnItem = {
                    label: $scope.resources.scenario_ScenarioPartChart_inputGrid_removeColMenuItem_txt,
                    disabled: !containsAnyCondCols,
                    onClick: function(evt) {
                        angular.forEach(rows, function (row) {
                            $scope.deleteConditionCol(row.data.outCol);
                        });
                        $scope.$apply(); //Must $apply here since event originates in Dojo
                    }
                };
                inputRowMenuItems.push(inputGridRemoveColumnItem);

                return inputRowMenuItems;
            };

            $scope.deleteExplCondition = function(idx) {
                var cond = $scope.data.activeRule.condition.childConditions[idx];
                $scope.deleteConditions([cond]);
            };

            $scope.condsFromExplChartSelections = function(nodes,negate) {
                //Get all selected include nodes
                var lastVar,
                    d,
                    currCond,
                    newConds = [],
                    currOperator,
                    parentCond,
                    j;

                for(var i = 0; i < nodes.length; i++) {
                    //Assign current node to variable
                    d = nodes[i];

                    currOperator = d.__data__.operator;
                    //Create new filter if on first node, reached, reached a new variable or using between operator
                    if(!lastVar || currOperator.name !== "IN" || lastVar !==  d.__data__.varName) {
                        currCond =  {
                            operator: currOperator,
                            operand2Constant: "",
                            childConditions: []
                        };
                        //Deal with NOT
                        if(negate) {
                            parentCond = {
                                operator: _.findWhere($scope.data.operators,{name: "NOT"}),
                                childConditions: []
                            };
                            parentCond.childConditions.push(currCond);
                            newConds.push(parentCond);
                        }
                        else {
                            newConds.push(currCond);
                        }
                    }


                    //Assign filter value based on operator
                    if( currCond.operator.name === "AND") {
                        //Assign child conditions to current condition for range
                        for(j = 0; j < 2; j++) {
                            //Update the operand 1 column in each child condition to use the parent operand 1 column
                            d.__data__.childConditions[j].operand1ColumnInfo = _.findWhere($scope.data.usedConditionColumns, {name: d.__data__.varName});
                            d.__data__.childConditions[j].operand2Constant = formatOperand2Constant(d.__data__.childConditions[j].operand1ColumnInfo, d.__data__.childConditions[j].operand2Constant);
                            currCond.childConditions.push(d.__data__.childConditions[j]);
                        }
                    }
                    else {
                        //Assign filtered column
                        currCond.operand1ColumnInfo =  _.findWhere($scope.data.usedConditionColumns, {name: d.__data__.varName});
                        if(currCond.operator.name === "IN") {
                            currCond.operand2Constant +=   formatOperand2Constant(currCond.operand1ColumnInfo,d.__data__.varValue);
                        }
                        else {
                            currCond.operand2Constant =  formatOperand2Constant(currCond.operand1ColumnInfo, d.__data__.operand2Constant);
                        }
                    }

                    lastVar =  d.__data__.varName;
                }
                //Trim the trialing comma from the RHS if IN operator
                newConds.forEach(function(f){
                    if((f.operator.name=== "IN" || f.operator.name === "NOT IN") &&
                        vsdUtil.getRight(f.operand2Constant,2) === ", ") {
                        f.operand2Constant = vsdUtil.dropRight(f.operand2Constant,2);
                    }
                });

                //Add conditions to current rule
                explAddConditions(newConds);


                function formatOperand2Constant(col,val) {
                    if(val && col.dataType === vsmCommonConst.DATA_TYPE_NUMERIC) {
                        if(col.columnFormat === vsdUtil.defaultFormats.integer) {
                            var parts = val.split(".");
                            return parts[0];
                        }
                        else {
                            return String(vsdUtil.realRound(parseFloat(val), 2));
                        }
                    }
                    else {
                        return val;
                    }
                }
            };

            $scope.showRebinDialog = function(idx) {
                $scope.explBinData.selectedColumnIdx = idx;
                $scope.explBinData.showRebinDialog = true;
            };


            $scope.$on('$routeChangeStart',function() {
                $scope.pchartAcceptDrop = angular.noop;
                $scope.pchartOnColumnDrop = angular.noop;
            });

             $scope.deleteAllConditions = deleteAllConditions;
             $scope.deleteAllConditionCols = deleteAllConditionCols;

            //endregion

            //Do initial call to get unique combos if scenario is ready (will be the case if switched from another view)
            if($scope.data.scenarioReady && $scope.scenario.editMode === $scope.EDIT_MODE_EXPLORE) {
                onScenarioReady();
            }
    }]);

});

},
'vsd/scenario/exploreDirect/scenarioPChart':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
  .directive('scenarioPchart',
      ['d3UtilsSvc',
       'dojoDepends',
       'vsdUtil',
       'pchartHoverMenuSvc',
       function (d3UtilsSvc,
                dojoDepends,
                vsdUtil,
                pchartHoverMenuSvc) {
            return {
                  restrict: 'A',
                  template: '<div class="fullScreen">' +
                                '<div id="pchartHeader" style="width=100%;height: 30px"></div>'  +
                                '<div id="pchartBody" style="position: absolute;top: 30px;bottom: 5px;left: 0px;right: 0px"></div>' +
                            '</div>',
                  replace: true,
                  scope: false,
                  link: function(scope, element, attrs,ctrl) {
                      //Create variable on scope for svg
                      var childEls = element.children(),
                          chartHdrEl = childEls[0],
                          chartBodyEl = childEls[1],
                          resources = dojoDepends.resources,
                          chartBodyD3Wrapper,
                          hoverMenu,
                          selectPart,
                          pChartXScale,
                          pChartYScale,
                          pChartG,
                          headerSvg,
                          headerHeight = 30,
                          chartDepth,
                          currDrillCell;

                      //Listen to routeChangeStart to destroy the hover menu if it exists to prevent it from showing up on other screens
                      scope.$on("$routeChangeStart", function () {
                          destroyHoverMenu();
                      });


                      scope.$on(vsmCommonConst.SCENARIO_MODE_CHANGE,function(event,args) {
                        if(args[0] !== scope.EDIT_MODE_EXPLORE) {
                            destroyHoverMenu();
                        }
                      });

                      //Create hover menu
                      hoverMenu = pchartHoverMenuSvc.hoverMenu({clickHandler: onHoverMenuClick});

                      scope.$watchCollection('counters.filteredRecCnt',function(newVal) {
                          if(newVal === 0) {
                              destroyHoverMenu();
                          }
                      });

                      //Create wrapper module to assist in directive execution
                      chartBodyD3Wrapper = d3UtilsSvc.d3DirectiveWrapper({
                          scope: scope,
                          elSelector: angular.element(chartBodyEl),
                          elName:  "chartBodyEl",
                          dataVarName: "explData.pchartData",
                          renderFunc: renderPChartBody,
                          clearBeforeRender: true,
                          margins:  {top: 0, left: 0, bottom: 0, right: 0}
                      });

                      //Watch label display option and re-render chart
                      scope.$watch('explOpts.chartLabelDisplay',function(newLbl) {
                          chartBodyD3Wrapper.renderChart();
                      });
                      //Watch sort option and re-render chart
                      scope.$watch('explOpts.chartSort',function(newLbl) {
                          chartBodyD3Wrapper.renderChart();
                      });

                      //Watch breadcrumbs collection to navigate up when crumbs clicked
                      scope.$watchCollection('explData.breadCrumbLevels',function(newArray,oldArray) {
                          if(newArray !== oldArray) {
                              if(chartDepth !== scope.explData.breadCrumbLevels.length) {
                                  var currCell = currDrillCell.parent;
                                  //Navigate up hierarchy until get to current drill level
                                  while(currCell.depth !== scope.explData.breadCrumbLevels.length) {
                                    currCell = currCell.parent;
                                  }
                                  //Zoom chart to current level
                                  pChartCellDblClick(currCell);
                              }
                          }
                      });


                      function onHoverMenuClick(negate) {
                        scope.$apply(function() {
                            hoverMenu.hide();
                            var  nodes = d3.selectAll(".parentSelectedOutline");
                            if(nodes.length > 0) {
                                scope.condsFromExplChartSelections(nodes[0],negate);
                            }

                        });
                      }

                      //Function to render d3 histogram from provided data
                      function renderPChartHeader(headerLabels,depth,newWidth) {
                          var numSects = headerLabels.length,
                              newSectWidth,
                              svg;

                          //Delete header svg, if it exists
                          if(headerSvg) {
                              headerSvg.remove();
                          }
                          //Create header svg
                          headerSvg = d3.select(chartHdrEl).append("svg")
                              .attr("class", "chartHdr")
                              .attr("width", chartBodyD3Wrapper.getWidth())
                              .attr("height", headerHeight);

                          //Set the width
                          if(newWidth) {
                              newSectWidth = newWidth;
                          }
                          else {
                              var tempRects = chartBodyD3Wrapper.getSvg().selectAll("rect")
                                  .filter(function(d){return d.depth === depth;});

                              newSectWidth    = parseInt(tempRects[0][0].attributes["width"].value);
                          }
                          //Set up horizontal scale
                          var hx = d3.scale.linear()
                              .domain([0,numSects]);
                          //Setup vertical scale
                          if(depth !== 0) {
                              hx.range([40, chartBodyD3Wrapper.getWidth()]);
                          }
                          else {
                              hx.range([newSectWidth,chartBodyD3Wrapper.getWidth()]);
                          }
                          //Select elements to remove/recreate
                          var hdrRects = headerSvg.selectAll("g rect").data(headerLabels);
                          var hdrText = headerSvg.selectAll("g text").data(headerLabels);
                          //Remove header elements
                          hdrRects.exit().remove();
                          hdrText.exit().remove();

                          hdrRects.transition().duration(800).ease("linear")
                              .attr('x', function(d,i){
                                  return  hx(i) ;})
                              .attr("width", newSectWidth )
                              .attr("height", 30);

                          hdrRects.enter().append("g")
                              .attr("transform", function(d,i) {
                                  return "translate(" +  (hx(i)) + ", 10 )";
                              })
                              .on("mouseover", function(d,i) {
                                  //Hide hover menu
                                  hoverMenu.hide();
                              })
                              .call(d3.behavior.drag()
                                  .on("dragstart", function(d) {
                                      //this.__origin__ = d.slice();
                                  })
                                  .on("drag", function(d) {
                                      d[0] += d3.event.dx;
                                      d[1] += d3.event.dy;
                                  })
                                  .on("dragend", function() {
                                      //delete this.__origin__;
                                  })
                              )
                              .append("rect")
                              .attr("width", newSectWidth )
                              .attr("height", 30)
                              .attr("class", "headerPartition");
                          //Recreate header rects
                          hdrText.enter()
                              .append("text")
                              .attr("width",newSectWidth)
                              .attr("y", 20)
                              .attr('x', function(d,i){
                                  return    hx(i) + 10;} )
                              .attr("dy", ".35em")
                              .attr ("class","headerText")
                              .text( function(d,i){
                                  return d.label;
                              });
                          //Add menus to headers
                          var headers = hdrRects[0];
                          var labels = hdrText[0];
                          for (var i=0; i < headers.length ; i++) {
                              var hdrMenu1 = createHeaderMenu(i);
                              hdrMenu1.bindDomNode(headers[i]);
                              var hdrMenu2 = createHeaderMenu(i);
                              hdrMenu2.bindDomNode(labels[i]);
                          }
                          //Truncate headers that are too long
                          var textIdIdx = 1;
                          labels.forEach(function(textEl) {
                              textEl.id = "__vsd_pchart_header" + i++;
                              d3UtilsSvc.truncateTextWithDots(textEl,newSectWidth);
                          });
                      }

                      function renderPChartBody(svg,chartData) {
                          var me = this,
                              level,
                              headerHeight = 30,
                              ALL_COLUMN_WIDTH = 150,
                              pctFmtFunc = d3.format("0.2%"),
                              commaFmtFunc = d3.format(","),
                              pChartW = chartBodyD3Wrapper.getWidth(),
                              pChartH = chartBodyD3Wrapper.getHeight(),
                              pChartKY = (pChartH)/ 1;

                          if(!chartData.children || !chartData.children.length) {
                              return;
                          }

                          if(!hoverMenu) {
                              //Create hover menu
                              hoverMenu = pchartHoverMenuSvc.hoverMenu({clickHandler: onHoverMenuClick});
                          }

                          //Set up scales
                          pChartXScale = d3.scale.linear().range([0, pChartW]);
                          pChartYScale = d3.scale.linear().range([0, pChartH]);

                          //Create partition layout and setup method to return frequency for each record to a partition
                          var partition = d3.layout.partition()
                              .value(function(d) {return d.shareOfParentCount;})
                              .sort(pChartValueComparator);
                          //Create groups for all chart nodes from data
                          var restColWidth = (pChartW - ALL_COLUMN_WIDTH)/(scope.data.usedConditionColumns.length),
                              rectColGap = restColWidth - ALL_COLUMN_WIDTH;

                          pChartG = svg.selectAll("g")
                              .data(partition.nodes(chartData))
                              .enter().append("g")
                              .attr("transform", function(d) {
                                  if(d.level === -1) {
                                      return "translate(" + 0 + "," + (pChartYScale(d.x)) + ")";
                                  }
                                  else {
                                      return "translate(" + pChartXScale(d.y) + "," + (pChartYScale(d.x)) + ")";
                                  }
                              })
                              .attr("class", "level")
                              .on("dblclick", function(d){scope.$apply(pChartCellDblClick.apply(me,[d]));});


                          //Create chart rectangles for each data point
                          pChartG.append("rect")
                              .attr("width", function(d) {
                                  if(d.depth === -1) {
                                      return 0;
                                  }
                                  else {
                                      return pChartXScale(d.dy);
                                  }
                              })
                              // .style("fill","url(#gradient)" )
                              .attr("height", function(d) {
                                  if(d.depth === -1) {
                                      return 0;
                                  }
                                  else {
                                      return pChartYScale(d.dx);
                                  }
                              })
                              .attr("class", function(d) { return d.children ? "parent" : "child"; })
                              .on("click", function(){
                                  applyCellSelection(this,d3.event);
                              })
                              //Mouse over handler sets up hover menu for nodes
                              .on("mouseover", function(d) {
                                  //If already captured hovered over cell, reset class to normal
                                  if(selectPart) {
                                      selectPart.nextSibling.className.baseVal = "parentText";
                                  }
                                  //Either on parent node or too small to see text - don't show menu
                                  if(d.depth < 1 || d.dx * pChartKY <= 12) {
                                      hoverMenu.hide();
                                  }
                                  else {
                                      //Capture current node
                                      selectPart = this;
                                      hoverMenu.setSelectPart(selectPart);

                                      var boundingRect = this.getBoundingClientRect();
                                      var divLeft = boundingRect.left + boundingRect.width - parseInt(hoverMenu.getMenuWidth());
                                      var divTop = boundingRect.top;
                                      //Setup hover menu
                                      hoverMenu.show({alpha: 0.5});
                                      hoverMenu.setPosition({left: divLeft, top: divTop});
                                      //Make text bold so under understand which item is selected
                                      this.nextSibling.className.baseVal = "parentTextHighlighted";

                                  }
                              })
                              //Mouse out menu clears hover menu from node
                              .on("mouseout", function(d) {
                                  hoverMenu.hide();
                              });

                          //Append node text (variable value + frequency) to all nodes
                          pChartG.append("svg:text")
                              .attr("transform", function(d){ return "translate(8," + d.dx * pChartKY / 2 + ")";})
                              .attr("class", function(d) { return d.children ? "parentText" : "childText"; })
                              .attr("dy",".35em")
                              .style("opacity", function(d) { return d.dx * pChartKY > 12 ? 1 : 0; })
                              .text(function(d) {
                                  var size,
                                      valueLbl;
                                  if(scope.explOpts.chartLabelDisplay === scope.PCT_DISPLAY) {
                                      size =  pctFmtFunc(d.count/scope.counters.totalRecCnt);
                                  }
                                  else {
                                      size = vsdUtil.getFormattedNumberString(d.count);
                                  }
                                  if(d.operator && d.operator.name === "IS MISSING") {
                                      valueLbl =  scope.resources.scenario_ScenarioPartChart_missingLbl_text;
                                  }
                                  else {
                                      var fmt;
                                      var col = _.findWhere(scope.data.usedConditionColumns,{name: d.varName});
                                      if(col && col.dataType === vsmCommonConst.DATA_TYPE_NUMERIC) {
                                          if(d.operator.type !== "LOGICAL") {
                                              var operStr = d.operator.name === "=" ? "" : d.operator.name;
                                              valueLbl = operStr + " " + vsdUtil.applySASNumericFormat(parseFloat(d.operand2Constant),col.columnFormat);
                                          }
                                          else {
                                              if(d.operatorRHSStr.indexOf("...") !== -1) {
                                                  var parts = d.operatorRHSStr.split("...");
                                                  var oper1 = d.childConditions[0].operator.name,
                                                      oper2 = d.childConditions[1].operator.name;

                                                  valueLbl = oper1+ " " + vsdUtil.applySASNumericFormat(parseFloat(parts[0].replace(",","")),col.columnFormat);
                                                  valueLbl += " " + dojoDepends.resources.app_rangeConjunctionLbl_txt + " ";
                                                  valueLbl += oper2 + " " + vsdUtil.applySASNumericFormat(parseFloat(parts[1].replace(",","")),col.columnFormat);
                                              }
                                              else {
                                                  valueLbl = vsdUtil.applySASNumericFormat(parseFloat(d.operatorRHSStr.replace(",","")),col.columnFormat);
                                              }

                                          }

                                      }
                                      else {
                                          valueLbl = d.operatorRHSStr;
                                      }

                                  }
                                  return valueLbl + " (" + size + ")";

                              })
                              .on("click", cellTextClick);

                          //Do initial rendering of headers
                          renderPChartHeader(scope.data.usedConditionColumns,0);

                          //When clicking on a level in graph, drill up or down
                          d3.select("#level1").on("click", changeHeir);


                          function pChartValueComparator(nodeA, nodeB) {
                              var compA,
                                  compB;

                              if(_.contains([scope.SORT_FREQ_ASC, scope.SORT_FREQ_DESC],scope.explOpts.chartSort)) {
                                  compA = nodeA.count;
                                  compB = nodeB.count;
                              }
                              else {
                                  if(nodeA.operator.name === "AND" || nodeB.operator.name === "AND" ||
                                      nodeA.operator.name === "IS MISSING" || nodeB.operator.name === "IS MISSING") {

                                      if(nodeA.operator.name === scope.LOWER_BOUND_OPERATOR_NAME) {
                                          compA = vsdUtil.strOperandToNbr(nodeA.operand2Constant);
                                          if(nodeB.operator.name !== "IS MISSING" && compA === vsdUtil.strOperandToNbr(nodeB.childConditions[0].operand2Constant)) {
                                              compB = vsdUtil.strOperandToNbr(nodeB.childConditions[1].operand2Constant);
                                          }
                                          else {
                                              if(nodeB.operator.name === "AND") {
                                                  compB = vsdUtil.strOperandToNbr(nodeB.childConditions[0].operand2Constant);
                                              }
                                              //Operator is MISSING
                                              else {
                                                  compB = Number.MIN_VALUE;
                                              }

                                          }
                                      }
                                      else if(nodeB.operator.name === scope.LOWER_BOUND_OPERATOR_NAME) {
                                          compB = vsdUtil.strOperandToNbr(nodeB.operand2Constant);
                                          if(nodeA.operator.name !== "IS MISSING" && compB === vsdUtil.strOperandToNbr(nodeA.childConditions[0].operand2Constant)) {
                                              compA = vsdUtil.strOperandToNbr(nodeA.childConditions[1].operand2Constant);
                                          }
                                          else {
                                              if(nodeA.operator.name === "AND") {
                                                  compA = vsdUtil.strOperandToNbr(nodeA.childConditions[0].operand2Constant);
                                              }//Operator is MISSING
                                              else {
                                                  compA = Number.MIN_VALUE;
                                              }

                                          }
                                      }
                                      else {
                                          if(nodeA.operator.name === "AND") {
                                              compA = vsdUtil.strOperandToNbr(nodeA.childConditions[0].operand2Constant);
                                          }
                                          else if(nodeA.operator.name === "IS MISSING") {
                                              compA = Number.MIN_VALUE;
                                          }
                                          else {
                                              compA =  vsdUtil.strOperandToNbr(nodeA.operand2Constant);
                                          }

                                          if(nodeB.operator.name === "AND") {
                                              compB = vsdUtil.strOperandToNbr(nodeB.childConditions[0].operand2Constant);
                                          }
                                          else if(nodeB.operator.name === "IS MISSING") {
                                              compB = Number.MIN_VALUE;
                                          }
                                          else {
                                              compB =  vsdUtil.strOperandToNbr(nodeB.operand2Constant);
                                          }
                                      }
                                  }
                                  else {
                                      compA = vsdUtil.strOperandToNbr(nodeA.operand2Constant);
                                      compB = vsdUtil.strOperandToNbr(nodeB.operand2Constant);
                                  }
                              }

                              if(compA < compB) {
                                  if(_.contains([scope.SORT_FREQ_ASC,scope.SORT_VAL_ASC],scope.explOpts.chartSort)) {
                                      return -1;
                                  }
                                  else {
                                      return 1;
                                  }

                              }
                              else if(compA > compB) {
                                  if(_.contains([scope.SORT_FREQ_ASC, scope.SORT_VAL_ASC],scope.explOpts.chartSort)) {
                                      return 1;
                                  }
                                  else {
                                      return -1;
                                  }
                              }
                              else {
                                  return 0;
                              }
                          }
                      }

                      function createHeaderMenu(idx) {
                          var me = this,
                              menu = new dojoDepends.Menu(),
                              condCol = scope.data.usedConditionColumns[idx],
                              numCondCols = scope.data.usedConditionColumns.length;
                          //Add delete column menu
                          var menuItem = new dojoDepends.MenuItem({label: resources.scenario_ScenarioPartChart_deleteColLbl_txt, onClick: function(evt){
                              var res = scope.deleteConditionCol(condCol);
                              //If delete had problem, trigger regeneration of input fields list so
                              if(!res) {
                                  scope.formatScenarioInputCols();
                              }
                              scope.$apply();

                          }});
                          menu.addChild(menuItem);
                          //Add move menu
                          var subMenu = new dojoDepends.Menu();
                          if(idx > 0) {
                              menuItem = new dojoDepends.MenuItem({label: resources.scenario_ScenarioPartChart_moveFirstLbl_txt, onClick: function(evt) {
                                  scope.$apply(scope.reorderConditionCol({startIdx: idx, targetIdx: 0}));
                              }});
                              subMenu.addChild(menuItem);
                              if(idx > 1) {
                                  menuItem = new dojoDepends.MenuItem({label: resources.scenario_ScenarioPartChart_moveLeftLbl_txt, onClick: function(evt){
                                      scope.$apply(scope.reorderConditionCol({startIdx: idx, targetIdx: idx - 1}));
                                  }});
                                  subMenu.addChild(menuItem);
                              }
                          }
                          if(idx < numCondCols - 1) {
                              if(idx < numCondCols - 2) {
                                  menuItem = new dojoDepends.MenuItem({label: resources.scenario_ScenarioPartChart_moveRightLbl_txt, onClick: function(evt){
                                      scope.$apply(scope.reorderConditionCol({startIdx: idx, targetIdx: idx + 1}));
                                  }});
                                  subMenu.addChild(menuItem);
                              }
                              menuItem = new dojoDepends.MenuItem({label: resources.scenario_ScenarioPartChart_moveLastLbl_txt, onClick: function(evt) {
                                  scope.$apply(scope.reorderConditionCol({startIdx: idx, targetIdx: scope.data.usedConditionColumns.length - 1}));
                              }});
                              subMenu.addChild(menuItem);
                          }
                          menu.addChild(new dojoDepends.PopupMenuItem({label: resources.scenario_ScenarioPartChart_moveColumnLbl_txt, popup: subMenu}));
                          //Add menu item for adjusting bins
                          if(condCol.dataType === vsmCommonConst.DATA_TYPE_NUMERIC) {
                              var woc = _.find(scope.data.windowOutCols,function(oc) {
                                                return oc.name === condCol.name;
                                        }) || {},
                                  lowCardBins =  woc.bins.length && woc.bins[0].minValue === woc.bins[0].midValue;

                              menuItem = new dojoDepends.MenuItem({
                                  label: resources.scenario_ScenarioPartChart_editBinsLbl_txt,
                                  disabled: lowCardBins || !woc.bins.length ,
                                  onClick: function(evt){
                                    scope.$apply(function(){scope.showRebinDialog(idx);});
                                  }
                              });
                              menu.addChild(menuItem);
                          }

                          return menu;

                      }

                      function changeHeir() {
                          d3.selectAll(".background").style("visibility", "hidden");
                      }

                      function cellTextClick() {
                          var svg = chartBodyD3Wrapper.getSvg();
                          var clickedData = d3.event.currentTarget.__data__;
                           var cell = svg.selectAll('rect')
                              .filter(function(d) {
                                  return d.varName === clickedData.varName &&
                                         d.operatorRHSStr === clickedData.operatorRHSStr;
                              })
                              .node();

                            applyCellSelection(cell,d3.event);
                      }

                      function applyCellSelection(cell,evt) {
                          var startClass,
                              newClass;

                          startClass = d3.select(cell).attr("class");
                          //If control key not held, clear current selection
                          if(!evt.ctrlKey) {
                              d3.selectAll(".parentSelectedOutline")
                                  .attr("class","parent");
                          }
                          if(startClass === "parent") {
                              newClass = "parentSelectedOutline";
                          }
                          else {
                              newClass = "parent";
                          }
                          //Apply select style to new cell
                          d3.select(cell).attr("class",newClass);
                      }

                      function pChartCellDblClick(d) {
                          var me = this,
                              i,
                              newWidth,
                              newHeaders;

                          //Save drill cell for later navigating
                          currDrillCell = d;
                          chartDepth = d.depth;
                          hoverMenu.hide();
                          //If there are no children, or we're on teh second to last level, don't allow drill down
                          if (!d.children || d.chartDepth === scope.data.usedConditionColumns.length) {
                              return;
                          }

                          //Create breadcrumbs for navigation
                          var breadCrumbLevels = [];
                          for(i = 0; i < chartDepth; i++) {
                              //No header for the first element - just use all
                              if(i === 0) {
                                  breadCrumbLevels.push({label: "All",depth: i});
                              }
                              else {
                                  breadCrumbLevels.push({label: scope.data.usedConditionColumns[i-1].label,depth: i});
                              }
                          }
                          scope.explData.breadCrumbLevels  = breadCrumbLevels;

                          var pChartKX =  (d.y ? chartBodyD3Wrapper.getWidth() - 40 : chartBodyD3Wrapper.getWidth()) / (1 - d.y),
                              pChartKY = chartBodyD3Wrapper.getHeight() / d.dx;

                          //Reset scales
                          pChartXScale.domain([d.y, 1]).range([d.y ? 40 : 0, chartBodyD3Wrapper.getWidth()]);
                          pChartYScale.domain([d.x, d.x + d.dx]);

                          var t = pChartG.transition()
                              .duration(750)
                              .attr("transform", function(d) {
                                  return "translate(" + pChartXScale(d.y) + "," + pChartYScale(d.x) + ")"; });

                          t.select("rect")
                              .attr("width", function(d) {
                                  newWidth =  d.dy * pChartKX;
                                  newHeaders = _.rest(scope.data.usedConditionColumns,  (chartDepth === 0 ) ? 0 : (chartDepth-1));
                                  return d.dy * pChartKX ;
                              })
                              .attr("height", function(d) {
                                  return d.dx * pChartKY; });

                          t.select("text")
                              .attr("transform", function(d){ return "translate(8," + d.dx * pChartKY / 2 + ")";})
                              .style("opacity", function(d) {
                                  return d.dx * pChartKY > 12 ? 1 : 0; });

                          renderPChartHeader(newHeaders,chartDepth,newWidth);

                          if(d3.event) {
                              d3.event.stopPropagation();
                          }
                      }

                      function destroyHoverMenu() {
                          if(hoverMenu) {
                              hoverMenu.destroy();
                              hoverMenu = null;
                          }
                      }
                 }
            };
      }
  ]);

});

},
'vsd/scenario/exploreDirect/ScenarioRebinCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('ScenarioRebinCtrl', ['$scope', 'scenarioSvc', 'windowSvc','dojoDepends',
        function ($scope, scenarioSvc, windowSvc, dojoDepends) {

            //region *** Scope Data/Functions ***
            $scope.binData = {
                allowApply: false,
                endOnBoundaryList: [
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_auto_txt, id: 0},
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_5_txt, id: 5},
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_10_txt, id: 10},
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_25_txt, id:25},
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_50_txt, id: 50},
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_100_txt, id: 100},
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_1000_txt, id: 1000},
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_10000_txt, id: 10000},
                    {name: $scope.resources.scenario_AdjustColumnBinsDlg_wholeNbrLbl_10000000_txt, id: 100000}
                ],
                selectedColumn: {bins: [], binCount: 0},
                title: ""
            };
            $scope.binData.endOnBoundary = $scope.binData.endOnBoundaryList[0];

            $scope.recalculateBins = function() {
                $scope.showLoadingMask($scope.resources.scenario_Controller_calculatingWaitLbl_txt);
                windowSvc.getDistributionData([$scope.binData.selectedColumn], $scope.binData.endOnBoundary.id)
                    .then(function(newCol) {
                        $scope.binData.selectedColumn.bins = newCol[$scope.binData.selectedColumn.name];
                        $scope.binData.allowApply = true;
                        $scope.hideLoadingMask();
                    });
            };

            $scope.saveBins = function () {
                $scope.explBinData.showRebinDialog = false;
                $scope.addSavedBinColumn($scope.explBinData.selectedColumnIdx, $scope.binData.selectedColumn);
            };

            $scope.cancelDialog = function () {
                $scope.explBinData.showRebinDialog = false;
            };

            $scope.$watch('explBinData.showRebinDialog',
                function () {
                    if ($scope.explBinData.showRebinDialog) {
                        var windowOutCol = _.find($scope.data.windowOutCols,function(woc) {
                            var selCol = $scope.data.usedConditionColumns[$scope.explBinData.selectedColumnIdx];
                            return selCol.name === woc.name;
                        });
                        $scope.binData.selectedColumn = angular.copy(windowOutCol);
                        $scope.binData.endOnBoundary = $scope.binData.endOnBoundaryList[0];
                        $scope.binData.allowApply = false;
                        $scope.binData.title = dojoDepends.string.substitute($scope.resources.scenario_AdjustColumnBinsDlg_title_txt,
                                                                             [$scope.binData.selectedColumn.label]);
                    }
                });
            //endregion
        }]);


});

},
'vsd/scenario/new/NewScenarioCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('NewScenarioCtrl', ['$scope',
                                    '$location',
                                    '$route',
                                    '$q',
                                    'scenarioSvc',
                                    'newScenarioSvc',
                                    'windowSvc',
                                    'dojoDepends',
                                    'vsdUtil',
        function ($scope,
                  $location,
                  $route,
                  $q,
                  scenarioSvc,
                  newScenarioSvc,
                  windowSvc,
                  dojoDepends,
                  vsdUtil) {
            //Private variables
            var scenarioNamesForWindow = [],
                nameValid = false,
                newScenarioParams = {
                    name: null,
                    desc: null,
                    scenarioWindow: null,
                    scenarioWindowName: null,
                    scenarioWindowPartitionKeyColumns: [],
                    editMode: null,
                    executionFrequency: {id: 0, name: 'DAILY'},
                    executionOrder: 0,
                    type: null,
                    entityType: null
                },
                defaultScopeData = {
                    availableWindows: null,
                    initWindowId: 0,
                    selectedWindow: {id: null, name: null},
                    allowCreate: false
                };

            //region *** Private Methods ***
            function getNonWindowDependencies() {
                if($scope.dialogFlags.showNewScenarioDialog) {
                    $scope.showLoadingMask($scope.resources.app_loading_msg_txt);
                }
                $q.all([
                    scenarioSvc.getScenarioTypes(),
                    scenarioSvc.getExecutionFrequencies()
                ]).then(function(results) {


                    $scope.data.scenarioTypes = results[0];
                    $scope.data.executionFrequencies = results[1];
                    if($scope.dialogFlags.showNewScenarioDialog) {
                        $scope.hideLoadingMask();
                    }
                });
            }

            function getWindowList() {
                $scope.showLoadingMask($scope.resources.app_loading_msg_txt);
                if(!newScenarioSvc.getSelectedWindowId()) {
                    windowSvc.getWindowList()
                        .success(function(wl) {
                            $scope.data.availableWindows = wl;
                            $scope.hideLoadingMask();
                        });
                }
            }

            function getAssociatedWindow() {
                $scope.showLoadingMask($scope.resources.app_loading_msg_txt);
                windowSvc.getWindow($scope.data.initWindowId)
                    .then(function(results) {
                        $scope.data.selectedWindow = results;
                        scenarioNamesForWindow = _.pluck(results.scenarios,"name");
                        $scope.scenarioParams.executionOrder = scenarioNamesForWindow.length;
                        $scope.hideLoadingMask();
                    });
            }

            function resetForm() {
                $scope.data.allowCreate = false;
                $scope.data.selectedWindow = null;
                $scope.scenarioParams = angular.copy(newScenarioParams);
                $scope.scenarioParams.scenarioWindowPartitionKeyColumns = [];
                $scope.scenarioParams.executionFrequency = $scope.data.executionFrequencies[0];
                $scope.data.editModeExp = true;
                $scope.data.editModeTbl = false;
                $scope.data.editModeTree = false;
                newScenarioSvc.setScenarioParams(angular.copy(newScenarioParams));
                angular.extend($scope.data,defaultScopeData);

            }


            //endregion

            //region *** Scope Data/Functions ***
            $scope.data = {
                availableWindows: null,
                initWindowId: null,
                scenarioTypes: null,
                entityTypes: null,
                executionFrequencies: null,
                selectedWindow: null,
                allowCreate: false,
                editModeExp: true,
                editModeTbl: false,
                editModeTree: false
            };


            $scope.onEnter = function() {
                if($scope.data.allowCreate) {
                    $scope.createScenario();
                }
            };

            $scope.setExistingScenarioNames = function() {
                if($scope.data.selectedWindow) {
                    scenarioNamesForWindow = _.pluck($scope.data.selectedWindow.scenarios,"name");
                    //Set execution order for new scenario to be one greater than the current max
                    $scope.scenarioParams.executionOrder = scenarioNamesForWindow.length;
                    //Determine if existing name is valid, if popuplated
                    //$scope.data.allowCreate = false;
                    if(newScenarioParams.name) {
                        var valRes = vsdUtil.validateDomainObjName(newScenarioParams.name,scenarioNamesForWindow);
                        if(valRes.isValid) {
                            nameValid = true;
                        }
                    }
                }
                $scope.updateAllowCreate();
            };

            $scope.validateScenarioName = function(name) {
                var result = {isValid: false, invalidMessage: ""},
	                 valRes = vsdUtil.validateDomainObjName(name,scenarioNamesForWindow);
                //If name invalid, set invalid message on text box
                if(!valRes.isValid) {
                    switch(valRes.reason) {
                        case vsmCommonConst.INVALID_NAME_REASON_SYNTAX:
                            result.invalidMessage = $scope.resources.app_object_name_specCharValidationMsg_txt;
                            break;
                        case vsmCommonConst.INVALID_NAME_REASON_DUPLICATE:
	                        result.invalidMessage = dojoDepends.string.substitute($scope.resources.scenario_Controller_invalidNameErrorMsg_txt,[name]);
                            break;
                    }
                    nameValid = false;
                }
                else {
                    nameValid = true;
                }

                $scope.updateAllowCreate();

                result.isValid =  valRes.isValid;
	            return result;
            };

            $scope.updateAllowCreate = function() {
                if(nameValid &&
                    $scope.data.selectedWindow && $scope.data.selectedWindow.id ) {
                    $scope.data.allowCreate = true;
                }
                else {
                    $scope.data.allowCreate = false;
                }
            };

            $scope.createScenario = function () {
                $scope.dialogFlags.showNewScenarioDialog = false;
                $scope.scenarioParams.scenarioWindowId = $scope.data.selectedWindow.id;
                $scope.scenarioParams.scenarioWindowName = $scope.data.selectedWindow.name;
                $scope.scenarioParams.scenarioWindowPartitionKeyColumns = $scope.data.selectedWindow.partitionKeyColumns;
                //Set edit mode from binary indicators
                if($scope.data.editModeTree) {
                    $scope.scenarioParams.editMode = 'R';
                }
                else if($scope.data.editModeTbl) {
                    $scope.scenarioParams.editMode = 'B';
                }
                else {
                    $scope.scenarioParams.editMode = 'E';
                }
                newScenarioSvc.setScenarioParams($scope.scenarioParams);
                //Navigate to scenario
                if ($location.path() === '/scenario/') {
                    $route.reload(); //If already on scenario, will need to reload
                }
                else {
                    $location.path('/scenario/');
                }
            };

            $scope.cancelDialog = function () {
                newScenarioSvc.clearSettings();  //Clear out settings so previously selected window not still selected when dialog re-entered
                $scope.dialogFlags.showNewScenarioDialog = false;
            };

            $scope.$watch('dialogFlags.showNewScenarioDialog',
                function () {
                    if ($scope.dialogFlags.showNewScenarioDialog) {
                        //Reset variables
                        resetForm();
                        //Get window metadata
                        $scope.data.initWindowId = newScenarioSvc.getSelectedWindowId();
                        if($scope.data.initWindowId) {
                            $scope.data.selectedWindow.id = newScenarioSvc.getSelectedWindowId();
                            $scope.data.selectedWindowName = newScenarioSvc.getSelectedWindowName();
                            getAssociatedWindow();
                        }
                        else {
                            $scope.data.initWindowId = 0;
                            getWindowList();
                        }
                        //Disable creation initially
                        $scope.data.allowCreate = false;
                    }
                }
            );
            //endregion

            //Call initialization functions
            getNonWindowDependencies();

        }])
;


});

},
'vsd/scenario/new/newScenarioSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('newScenarioSvc', function () {
		var scenarioParams = {};
        var selectedWindowId = null;
        var selectedWindowName = null;

		return {
			setScenarioParams:    function (s) {
                scenarioParams = s;
			},
            getScenarioParams:    function () {
				return scenarioParams;
			},
			deleteScenarioParams: function () {
                scenarioParams = {};
			},
            setSelectedWindowId:  function (id) {
                selectedWindowId = id;
                this.deleteScenarioParams();
            },
            getSelectedWindowId:    function () {
                return selectedWindowId;
            },
            setSelectedWindowName:  function (name) {
                selectedWindowName = name;
                this.deleteScenarioParams();
            },
            getSelectedWindowName:    function () {
                return selectedWindowName;
            },
            deleteSelectedWindowId: function () {
                selectedWindowId = null;
            },
            clearSettings: function() {
                scenarioParams = {};
                selectedWindowId = null;
                selectedWindowName = null;

            }
		};
	});

});

},
'vsd/scenario/saveAs/SaveAsCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('SaveAsCtrl', ['$scope',
                               '$location',
                               'saveAsSvc',
                               'vsdUtil',
                               'dojoModules',
                               'messageSvc',
        function ($scope,
                  $location,
                  saveAsSvc,
                  vsdUtil,
                  dojoModules,
                  messageSvc) {

            var saveFunc,
                nameListFunc,
                removeObjectIdsFunc,
                usedNames,
                objTypeDisplay;

            function getUsedNameList() {
                //Get used object names and save them to list
                nameListFunc().then(function(results){
                    usedNames = results;
                });
            }

            $scope.data = {
                title: null,
                domainObj: null,
                allowCreate: false,
                maxNameLength: 255
            };

            $scope.validateName = function(value) {
                var result = {isValid: false, invalidMessage: ""},
                    valRes = vsdUtil.validateDomainObjName(value,usedNames);
                //If name invalid, set invalid message on text box
                if(!valRes.isValid) {
                    switch(valRes.reason) {
                        case vsmCommonConst.INVALID_NAME_REASON_SYNTAX:
                            result.invalidMessage = $scope.resources.app_object_name_specCharValidationMsg_txt;
                            break;
                        case vsmCommonConst.INVALID_NAME_REASON_DUPLICATE:
                            result.invalidMessage = dojoModules.string.substitute($scope.resources.app_object_name_duplicate_txt,
                                                                                  [objTypeDisplay,value]);
                            break;
                    }
                    $scope.data.allowCreate = false;
                }
                else {
                    $scope.data.allowCreate = true;
                }

                result.isValid =  valRes.isValid;
                return result;
            };

            $scope.createObject = function() {
                //Save the object
               saveFunc($scope.data.domainObj).then(function(result) {
                   //Hide dialog
                   $scope.dialogFlags.showSaveAsDialog  = false;
                   //Go to newly created object
                   var id = result.id;
                   var path = saveAsSvc.getPathWithoutId() + "/" + id;
                   $location.path(path);
                   messageSvc.showInfo(dojoModules.string.substitute($scope.resources.saveAsDlg_successMsgLbl_txt,
                                                                     [objTypeDisplay,$scope.data.domainObj.name]));
               });
            };

            $scope.onEnter = function() {
                if($scope.data.allowCreate) {
                    $scope.createObject();
                }
            };

            $scope.cancelDialog = function() {
                $scope.dialogFlags.showSaveAsDialog  = false;
            };

            $scope.$watch('dialogFlags.showSaveAsDialog',
                function () {
                    if ($scope.dialogFlags.showSaveAsDialog) {
                        //Reset variables
                        $scope.data.allowCreate = false;
                        usedNames = [];
                        $scope.data.maxNameLength = saveAsSvc.getMaxNameLength();
                        objTypeDisplay = saveAsSvc.getObjectTypeDisplay();
                        $scope.data.title = $scope.resources.saveAsDlg_title_txt;
                        //Set function pointers for cleaner syntax
                        saveFunc = saveAsSvc.getSaveFunc();
                        nameListFunc = saveAsSvc.getUsedNameListFunc();
                        removeObjectIdsFunc = saveAsSvc.getRemoveObjectIdsFunc();
                        //Get copy of domain object without ids
                        $scope.data.domainObj = removeObjectIdsFunc();
                        //Invoke function to get name list
                        getUsedNameList();
                    }
                }
            );
            //endregion
        }])
;


});

},
'vsd/scenario/saveAs/saveAsSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('saveAsSvc',['$q',function ($q) {
		var sourceObject = null,
            objectTypeDisplay = null,
            removeObjectIdsFunc = null,
            usedNameListFunc = null,
            saveFunc = null,
            pathWithoutId,
            maxNameLength;

        return {
			setSourceObject:    function (o) {
                sourceObject = o;
                return this;
			},
            getSourceObject:    function () {
				return sourceObject;
			},
            setRemoveObjectIdsFunc: function(f) {
                removeObjectIdsFunc = f;
                return this;
            },
            getRemoveObjectIdsFunc: function() {
                return function() {
                    removeObjectIdsFunc(sourceObject);
                    return sourceObject;
                }
            },
            setUsedNameListFunc:    function (f) {
                usedNameListFunc = f;
                return this;
            },
            getUsedNameListFunc:    function () {
                return usedNameListFunc;
            },
            setSaveFunc:    function (f) {
                saveFunc = f;
                return this;
            },
            getSaveFunc:    function () {
                return saveFunc;
            },
            setPathWithoutId:    function (p) {
                pathWithoutId = p;
                return this;
            },
            getPathWithoutId:    function () {
                return pathWithoutId;
            },
            setObjectTypeDisplay:    function (d) {
                objectTypeDisplay = d;
                return this;
            },
            getObjectTypeDisplay:    function () {
                return objectTypeDisplay;
            },
            setMaxNameLength:    function (l) {
                maxNameLength = l;
                return this;
            },
            getMaxNameLength:    function () {
                return maxNameLength;
            }
		};
	}]);

});

},
'vsd/window/WindowCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
    .controller('WindowCtrl', ['$scope',
        '$routeParams',
        '$filter',
        '$location',
        '$q',
        '$timeout',
        'windowSvc',
        'messageSvc',
        'dojoModules',
        'vsdUtil',
        'newScenarioSvc',
        'newDeploymentSvc',
        'arrayReorderSvc',
        'operators',
        'window',
        function ($scope, $routeParams, $filter, $location, $q, $timeout, windowSvc, messageSvc, dojoModules, vsdUtil, newScenarioSvc, newDeploymentSvc, arrayReorderSvc, operators, window) {

            //region *** Private Variables ****
            var
                newCols = [],
                $orderBy = $filter('orderBy'),
                $filterBy = $filter('filter'),
                defaultBinCount = 5,
                defaultLookBackPeriod = 0,
                defaultDateAggregation = 'MAX',
                tempTableColumns = {},
                inputToTempColumnMappings = {},
                cachedSummaryStats = {},
                cachedDistributions = {},
                scenarioDependency = {},
                numOutputRecords = 0,
                useCachedOutput = true,
                nodeIndex = 0;
            //endregion

            var resources = dojoModules.resources,
                selVersionWatch = null,
                nodeIndex = 0;
            $scope.diagram = {};
            $scope.diagram.size = null;
            $scope.firstLoad = true;


            if (!window) {
                $scope.showHome();
                return;
            }

            $scope.reloadInputData = false;

            $scope.data = {
                windowState: 'PREVIEW',
                inputOutputRecCntStr: "",
                disableSave: false,
                isDirty: false,
                inputColumnMappings: {},
                minAnchorDate: null,
                maxAnchorDate: null,
                filterCondition: null,
                filterNotPresent: true,
                showHierarchicalView: false,
                outputTypes: []
            };

            // This structure is a "side-model" to the sngDateTextBoxes used in the window screen.  This means that
            // this structure is kept "off to the side" of the model used by the directive due to its truncation
            // of time granularity.  The need for this workaround will be moot when we use another component/directive
            // that does not do this truncation.
            //
            // The *real* model for this controller is $scope.window.anchorStart and
            // $scope.window.anchorEnd.  Those properties are kept updated via the sngDateTextBox's sng-change
            // invocation of anchorStartDateTimeChanged() and anchorEndDateTimeChanged().
            $scope.anchorDateTimeViews = {
                viewAnchorStartDateTime: null,  //The sngDateTextBox model
                oldAnchorStartDateTime: null,   //The old value of the window's start date/time
                viewAnchorEndDateTime: null,    //The sngDateTextBox model
                oldAnchorEndDateTime: null      //The old value of the window's end date/time
            };

            $scope.filterNotPresent = true;

            $scope.invalidTransactionKeyColumnMsg = null;
            $scope.validTransactionKeys = {};
            $scope.windowRunning = false;

            //region *** Private Functions ***

            function setDirty(dirty) {
                $scope.data.isDirty = dirty;
            }

            function enableSave() {
                $scope.data.disableSave = false;
            }

            function disableSave() {
                $scope.data.disableSave = true;
            }

            function getLHSColumnNames(condition) {
                var names = {};
                if (condition.childConditions) {
                    angular.forEach(condition.childConditions, function (child) {
                        angular.extend(names, getLHSColumnNames(child));
                    });

                    if (condition.operand1ColumnInfo) {
                        names[condition.operand1ColumnInfo.name] = '1';
                    }
                }
            }

            function addScenarioDependency(colName, scenarioName) {

                var index = (scenarioDependency[colName] || '').indexOf(scenarioName);

                if (index < 0) {
                    scenarioDependency[colName] = (scenarioDependency[colName] || '') + ' ' + scenarioName;
                }
            }

            function updateScenarioDependencies() {
                var scenarios = window.scenarios || [];
                scenarioDependency = {};
                angular.forEach(scenarios, function (scenario) {
                    var name = scenario.name;
                    if (scenario.targetColumn) {
                        addScenarioDependency(scenario.targetColumn.name, name);
                    }

                    angular.forEach(scenario.scenarioColumns, function (sCol) {
                        addScenarioDependency(sCol.column.name, name);
                    });

                    angular.forEach(scenarios.events, function (event) {
                        var names = getLHSColumnNames(event.condition);
                        angular.forEach(names, function (value, colName) {
                            addScenarioDependency(colName, name);
                        });
                    });
                });
            }

            function setupWindow() {

                updateScenarioDependencies();

                createWindowInputProperties();
                $scope.windowProps.disableEditName = !!$scope.window.id;
                $scope.windowProps.transactionKeyColumn = $scope.window.transactionKeyColumn;
                $scope.windowProps.partitionKeyColumns = $scope.window.partitionKeyColumns;
                $scope.windowProps.outputLevel = determineOutputLevel();

                $scope.hideLoadingMask();
                // FIXME: is this necessary?
                $scope.data.inputOutputRecCntStr = "";
                inputToTempColumnMappings = {};
                $scope.previewProps = {
                    previewCols: [],
                    previewData: [],
                    filterCond: {
                        operator: {name: '='},
                        operand1ColumnInfo: {name: ' ', label: ' '},
                        operand2Constant: ""
                    }
                };
                $scope.colDetailsProps.currentDistribution = {};
                $scope.colDetailsProps.currentStats = {};

                $scope.colDetailsProps.statsList = {};
                $scope.colDetailsProps.columnDetail = null;

                setupInputColumns();
                createWindow();
                loadScenarios();
                setupOutputColumns();
                getOutputTypes();
                getInputDateRange();

                $scope.options.defaultNumericAggregationTypes = $scope.options.nonDateBasedNumericAggregationTypes.slice(0);
                if (window.anchorSourceColumn) {
                    $scope.options.defaultNumericAggregationTypes.push.apply($scope.options.defaultNumericAggregationTypes, $scope.options.dateBasedNumericAggregationTypes);
                }

                $scope.options.defaultNumericAggregation = _.findWhere($scope.options.defaultNumericAggregationTypes, {value: window.defaultNumericAggregation});

                if ($scope.window.type !== vsmCommonConst.WINDOW_TYPE_TRANSFORMATION && $scope.window.anchorEnd !== null) {
                    $scope.anchorDateTimeViews.viewAnchorEndDateTime = new Date($scope.window.anchorEnd);
                    $scope.anchorDateTimeViews.oldAnchorEndDateTime = new Date($scope.window.anchorEnd);
                }

                if ($scope.window.type === vsmCommonConst.WINDOW_TYPE_SLIDING || $scope.window.type === vsmCommonConst.WINDOW_TYPE_ROLLING) {
                    $scope.anchorDateTimeViews.viewAnchorStartDateTime = new Date($scope.window.anchorStart);
                    $scope.anchorDateTimeViews.oldAnchorStartDateTime = new Date($scope.window.anchorStart);
                }
            }

            function determineOutputLevel() {
                var outputLevel;
                switch ($scope.window.type) {
                    case vsmCommonConst.WINDOW_TYPE_TRANSFORMATION:
                        outputLevel = $scope.resources.windowTypeTransformation_window_txt;
                        break;
                    case vsmCommonConst.WINDOW_TYPE_AGGREGATION:
                        outputLevel = $scope.resources.windowOutputLevelGroup_window_txt;
                        break;
                    case vsmCommonConst.WINDOW_TYPE_SLIDING:
                        outputLevel = $scope.resources.windowOutputLevelGroupDate_window_txt;
                        break;
                    case vsmCommonConst.WINDOW_TYPE_ROLLING:
                        outputLevel = $scope.resources.windowOutputLevelTransaction_window_txt;
                        break;
                }
                return outputLevel;
            }


            function createWindowInputProperties() {
                var props = {primaryInput: null, dimensionInputs: []},
                    winInputs = $scope.window.windowInputs,
                    primaryInput = winInputs[0],
                    lhWin = primaryInput.leftHandWindow,
                    lhTable = primaryInput.leftHandTable;

                if (primaryInput) {
                    props.primaryInput = lhWin ? lhWin.name : lhTable.name;

                    angular.forEach(winInputs, function (input) {
                        if (input.joinColumns.length) {
                            var joinColumn = input.joinColumns[0],
                                rhWin = input.rightHandWindow,
                                rhTable = input.rightHandTable;

                            props.dimensionInputs.push({
                                name: rhWin ? rhWin.name : rhTable.name,
                                leftJoinColumn: joinColumn.leftColumn.name,
                                rightJoinColumn: joinColumn.rightColumn.name,
                                alias: input.rightHandAlias
                            });
                        }
                    });
                }

                angular.extend($scope.windowProps, props);
            }

            function loadTableWithDistinctCounts(originalTable) {
                var tableId = originalTable.fqid;
                var tableLoaded = windowSvc.getTableWithDistinctCounts(tableId).then(function (table) {
                    var newCols = table.columns;

                    for (var i = 0; i < newCols.length; i++) {

                        try {
                            var col = table.columns[i];
                            var winTableIndex = originalTable.columns.map(function (e) {
                                return e.name;
                            }).indexOf(col.name);

                            if (originalTable.columns[winTableIndex]) {
                                originalTable.columns[winTableIndex].cnt = col.cnt;
                            }

                            var foundIndex = -1;

                            var arr = $.grep($scope.inputColsProps.inputData, function( n, i ) {
                                var tableFqid = n.tableFqid;
                                var originalColName = n.originalColName;
                                var alias = n.alias;
                                if(originalColName == col.name && tableFqid == originalTable.fqid){
                                    foundIndex = i;
                                }
                                return true;
                            });

                            if( foundIndex >= 0 && $scope.inputColsProps.inputData[foundIndex]){
                                $scope.inputColsProps.inputData[foundIndex].cnt = col.cnt;
                            }

                        } catch (err) {
                            console.error(err);
                        }
                    }
                    $scope.reloadInputData = {reload: true};
                });
            }

            function setupInputColumns() {

                var window = $scope.window,
                    newInputCols = [],
                    sourceCols = [],
                    dataSourceColMap,
                    firstInput = window.windowInputs[0],
                    primaryDataSource = firstInput.leftHandTable ? firstInput.leftHandTable.name : firstInput.leftHandWindow.name,
                    factTable,
                    dataSourceName,
                    dimTable;

                var tableRefs = [];

                if (window.inputTable.role === 'JOIN_OUTPUT') {
                    factTable = firstInput.leftHandTable || firstInput.leftHandWindow.outputTable;
                    tableRefs.push(factTable);
                    angular.forEach(factTable.columns, function (col) {
                        col.tableFqid = factTable.fqid;
                        sourceCols.push({
                            dataSourceName: primaryDataSource,
                            tableFqid:factTable.fqid,
                            name: col.name
                        });
                    });
                    angular.forEach(window.windowInputs, function (windowInput) {
                        dimTable = windowInput.rightHandTable || windowInput.rightHandWindow.outputTable;
                        tableRefs.push(dimTable);
                        dataSourceName = windowInput.rightHandTable ? windowInput.rightHandTable.name : windowInput.rightHandWindow.name;
                        angular.forEach(dimTable.columns, function (col) {
                            col.tableFqid = dimTable.fqid;
                            sourceCols.push({
                                dataSourceName: dataSourceName,
                                tableFqid:dimTable.fqid,
                                name: windowInput.rightHandAlias + '_' + col.name
                            });
                        });
                    });
                    windowSvc.handleNameCollisions(sourceCols);
                    dataSourceColMap = {};
                    angular.forEach(sourceCols, function (item) {
                        dataSourceColMap[item.name] = item.dataSourceName;
                    });
                }

                // Populate input column grid data
                angular.forEach(window.inputTable.columns, function (column) {
                    column.executionOrder = 0;
                    column.dataSource = dataSourceColMap ? dataSourceColMap[column.name] : primaryDataSource;
                    // don't include key columns
                    if (/*column.name !== $scope.windowProps.dateKeyColumn.name && */!_.findWhere($scope.windowProps.partitionKeyColumns, {name: column.name})) {


                    if (tableRefs.length > 0 && !(column.tableFqid)) {
                        tableRefs = [];
                    }

                    //Need all input columns for windowFilter LHS dropdown
                    if (!column.tableFqid) {
                        column.tableFqid = window.inputTable.fqid;
                        column.originalColName = column.name;
                    }
                    newInputCols.push(column);
                    }
                    //add default executionOrder of 0 to input Columns
                    $scope.data.inputColumnMappings[column.name.toLowerCase()] = {
                        name: column.name,
                        executionOrder: 0,
                        dataType: column.dataType,
                        dataLength: column.dataLength,
                        columnFormat: column.columnFormat
                    };
                });

                $scope.inputColsProps.inputData = $orderBy(newInputCols, ['dataType', 'label']);

                if(tableRefs.length == 0){
                    tableRefs.push($scope.window.inputTable);
                }

                if(!windowSvc.gotDistinctCounts || $scope.hasNewJoins){
                    for(var i = 0; i < tableRefs.length; i++){
                        loadTableWithDistinctCounts(tableRefs[i]);
                    }
                }
                $scope.hasNewJoins = false;
                windowSvc.gotDistinctCounts = false;
            }

            function setupOutputColumns() {

                angular.forEach($scope.window.outputColumns, function (outputColumn) {

                    if (outputColumn.dataType === 'CHARACTER') {
                        outputColumn.bins = cachedDistributions[outputColumn.name] || [];
                    }
                    if ($scope.windowProps.partitionKeyColumns && _.findWhere($scope.windowProps.partitionKeyColumns, {name: outputColumn.name})) {
                        outputColumn.vartype = 'key';
                    }
                    else if ($scope.windowProps.dateKeyColumn && $scope.windowProps.dateKeyColumn.name === outputColumn.name) {
                        outputColumn.vartype = 'key';
                    }
                    else if ($scope.window.type === vsmCommonConst.WINDOW_TYPE_ROLLING && $scope.windowProps.transactionKeyColumn && $scope.windowProps.transactionKeyColumn.name === outputColumn.name) {
                        outputColumn.vartype = 'key';
                    }
                    else {
                        outputColumn.vartype = '';
                    }

                    if (typeof outputColumn.lookbackPeriod !== 'number') {
                        outputColumn.lookbackPeriod = $scope.window.defaultLookbackPeriod ? $scope.window.defaultLookbackPeriod : 0;
                    }
                    if (typeof outputColumn.executionOrder !== 'number') {
                        outputColumn.executionOrder = 0;
                    }
                    if (typeof outputColumn.binCount !== 'number') {
                        outputColumn.binCount = defaultBinCount;
                    }

                    outputColumn.usedInScenario = false;
                    if (scenarioDependency[outputColumn.name]) {
                        outputColumn.usedInScenario = true;
                    }
                });
            }

            function addWindowToNodeHierarchy(win, parentIndex) {
                var windowNodeIndex = nodeIndex;
                $scope.diagram.nodeData.push({
                    key: windowNodeIndex,
                    parent: parentIndex,
                    name: win.name,
                    nodeType: $scope.resources.objTypeLabel_window_txt,
                    toolTip: $scope.resources.objTypeLabel_window_txt
                });
                nodeIndex++;

                angular.forEach(win.scenarios, function (scen) {
                    $scope.diagram.nodeData.push({
                        key: nodeIndex,
                        parent: windowNodeIndex,
                        name: scen.name,
                        nodeType: $scope.resources.objTypeLabel_scenario_txt,
                        toolTip: $scope.resources.objTypeLabel_scenario_txt
                    });
                    nodeIndex++;
                });

                var firstWinInput = win.windowInputs[0];

                if (firstWinInput) {
                    if (firstWinInput.leftHandTable) {
                        $scope.diagram.nodeData.push({
                            key: nodeIndex,
                            parent: windowNodeIndex,
                            name: firstWinInput.leftHandTable.name,
                            nodeType: $scope.resources.objTypeLabel_table_txt,
                            toolTip: $scope.resources.objTypeLabel_table_txt
                        });
                        nodeIndex++;
                    }
                    else {
                        if (firstWinInput.leftHandWindow) {
                            addWindowToNodeHierarchy(firstWinInput.leftHandWindow, windowNodeIndex);
                        }
                    }
                }

                angular.forEach(win.windowInputs, function (winInput) {

                    if (winInput.rightHandTable) {
                        $scope.diagram.nodeData.push({
                            key: nodeIndex,
                            parent: windowNodeIndex,
                            name: winInput.rightHandTable.name,
                            nodeType: $scope.resources.objTypeLabel_table_txt,
                            toolTip: $scope.resources.objTypeLabel_table_txt
                        });
                        nodeIndex++;
                    }
                    else if (winInput.rightHandWindow) {
                        addWindowToNodeHierarchy(winInput.rightHandWindow, windowNodeIndex);
                    }
                });
            }

            function prepareHierarchicalDisplay() {

                var handle = dojoModules.registry.byId("hierarchicalViewResizeHandle");

                dojo.subscribe("/dojo/resize/stop", function () {
                    var targetDomNode = dojo.marginBox(handle.targetDomNode);
                    $scope.diagram.size = {h: targetDomNode.h, w: targetDomNode.w};
                    $scope.$apply();
                });

                $scope.diagram.legendData = [
                    {nodeType: $scope.resources.objTypeLabel_table_txt, color: vsmCommonConst.DIAGRAM_TABLE_COLOR},
                    {
                        nodeType: $scope.resources.objTypeLabel_scenario_txt,
                        color: vsmCommonConst.DIAGRAM_SCENARIO_COLOR
                    },
                    {nodeType: $scope.resources.objTypeLabel_window_txt, color: vsmCommonConst.DIAGRAM_WINDOW_COLOR}
                ];

                $scope.diagram.nodeData = [];
                addWindowToNodeHierarchy($scope.window, 0);
            }

            function getOutputTypes() {
                return windowSvc.getOutputTypes()
                    .then(function (results) {
                        $scope.data.outputTypes = results;
                    });
            }

            function createWindow() {
                $scope.showLoadingMask($scope.resources.main_loadingWindow_msg_txt);
                tempTableColumns = {};
                cachedSummaryStats = {};
                cachedDistributions = {};
                return windowSvc.createWindow($scope.window, useCachedOutput)
                    .then(function (results) {
                        useCachedOutput = false;
                        numOutputRecords = results.rowCount;

                        angular.forEach(results.columnMappings, function (value, key) {

                            // This check is needed to handle a differences in how transformation window temp table columns are named
                            var useExpression = $scope.isTransformation() && value.expression.toLowerCase() === key.toLowerCase();
                            if (useExpression) {
                                tempTableColumns[value.expression] = 1;
                            }
                            else {
                                tempTableColumns[key] = 1;
                            }
                            if (value.aggregation ||
                                $scope.isTransformation() ||
                                (key.toLowerCase() === value.expression.toLowerCase())) {
                                if (useExpression) {
                                    inputToTempColumnMappings[value.expression] = value.expression;
                                }
                                else {
                                    inputToTempColumnMappings[value.expression] = key;
                                }
                            }
                        });

                        newCols = [];

                        if ($scope.selectedColumn) {
                            var selectedColumnName = $scope.selectedColumn.name;
                            $scope.selectedColumn = _.findWhere($scope.window.outputColumns, {name: selectedColumnName});
                        }

                        // get grid column structure
                        $scope.previewProps.previewCols = getPreviewColumns();

                        // get grid data
                        getPreviewData();

                        //Send message to previewGrid footer with record count for window
                        $scope.data.inputOutputRecCntStr = $scope.resources.scWindow_numInputRecsMsg_txt + " " +
                        dojo.number.format(results.inputTableRowCount, {places: 0}) + " / " +
                        $scope.resources.scWindow_numOutputRecsMsg_txt + " " +
                        dojo.number.format(results.rowCount, {places: 0});

                    }).then(function () {
                        getSummaryStatsAndDistributions($scope.window.outputColumns);
                    });
            }


            function getPreviewData() {
                var whereCl = null,
                    previewCols = [],
                    orderByCols = [];
                if (!$scope.window.outputColumns) {
                    return;
                }

                //
                angular.forEach($scope.previewProps.previewCols, function (column) {
                    var pColumn = {};
                    pColumn.name = column.name;
                    pColumn.dataType = column.dataType;
                    pColumn.columnFormat = column.columnFormat;
                    pColumn.expression = column.expression;
                    pColumn.aggrFunction = column.aggrFunction;
                    pColumn.sasDate = column.sasDate;
                    pColumn.sasDateTime = column.sasDateTime;
                    if (!tempTableColumns[pColumn.name]) {
                        newCols.push(pColumn);
                    }
                    else {
                        previewCols.push(pColumn);
                    }
                });

                $scope.showLoadingMask($scope.resources.main_loadingPreview_msg_txt);

                whereCl = vsdUtil.buildWhereClause($scope.previewProps.filterCond);

                orderByCols = $scope.window.partitionKeyColumns.slice();

                if ($scope.window.type && ($scope.window.type === vsmCommonConst.WINDOW_TYPE_SLIDING || $scope.window.type === vsmCommonConst.WINDOW_TYPE_ROLLING)) {
                    orderByCols.push($scope.window.anchorSourceColumn);
                }

                windowSvc.previewData(previewCols, orderByCols, whereCl).then(function (results) {
                    // Note: previewData only uses data that is currently in the temp table, it does NOT run the window first.
                    // Therefore, newly dragged columns will not be put into the temp table until the user runs the window.
                    var prevCols = $scope.previewProps.previewCols;
                    angular.forEach(results, function (row, i) {
                        angular.forEach(prevCols, function (col) {
                            if (row[col.name] && col.dataType === 'NUMERIC') {
                                if (col.sasDate) {
                                    col.columnFormat = vsdUtil.defaultFormats.date;
                                }
                                else if (col.sasDateTime) {
                                    col.columnFormat = vsdUtil.defaultFormats.datetime;
                                }
                                row[col.name] = vsdUtil.applySASNumericFormat(row[col.name], col.columnFormat);
                            }
                        });
                        row._id_ = i;

                        if (newCols.length > 0) {
                            angular.forEach(newCols, function (newCol) {
                                // check if there is an existing preview column that uses the same expression and aggrFunction, if so copy its value
                                var colsWithSameVal = _.where(previewCols,
                                        {expression: newCol.expression, aggrFunction: newCol.aggrFunction}),
                                    value = null;
                                angular.forEach(colsWithSameVal, function (col) {
                                    // copy value if column exists in the results row
                                    if (!value && row.hasOwnProperty(col.name)) {
                                        value = angular.copy(row[col.name]);
                                    }
                                });
                                if (value != null) {
                                    row[newCol.name] = value;
                                }
                                else {  // values aren't available yet for the column, display asterisks
                                    //	row[newCol.name] = "*********";
                                    //	row[newCol.name] = "•••••••••";
                                    row[newCol.name] = "▪▪▪▪▪▪▪▪▪";
                                    //	row[newCol.name] = "■■■■■■■■";
                                    //	row[newCol.name] = "□□□□□□□□";
                                    //	row[newCol.name] = "◆◆◆◆◆◆◆◆";
                                    //	row[newCol.name] = "●●●●●●●●●●";

                                }
                            });
                        }
                    });
                    $scope.previewProps.previewData = results;
                    $scope.hideLoadingMask();
                    if($scope.firstLoad ){
                        $scope.firstLoad = false;
                        $scope.reloadInputData = !$scope.reloadInputData;
                    }
                    $scope.colDetailsProps.columnDetail = 'dist';
                });

            }


            function getPreviewColumns() {
                var cols = [];
                angular.forEach($scope.window.outputColumns, function (column, key) {
                    var newCol = prepColumnForPreview(column, false);
                    cols.push(newCol);
                });

                return cols;
            }

            function prepColumnForPreview(column, newCol) {
                var prevCol = {};
                // Have make a copy of the outputColumn because dgrid adds a lot of extra stuff to the col list when applied to the grid
//			if (newCol) {
//				if(inputToTempColumnMappings[column.name.toUpperCase()])
//					column.name = inputToTempColumnMappings[column.name.toUpperCase()];
//
//				column.name = windowSvc.getUniqueColumnName($scope.window.outputColumns,column.name);
//				column.label = windowSvc.getUniqueColumnLabel($scope.window.outputColumns,column.name);
//			}

                angular.copy(column, prevCol);
                prevCol.field = prevCol.name;
                prevCol.sortable = true;
                prevCol.resizable = true;
                if (prevCol.vartype === 'key') {
                    prevCol.renderHeaderCell = dgridRenderHeaderCell;
                }

                prevCol.renderCell = dgridRenderCell;

                return prevCol;
            }

            function dgridRenderHeaderCell(cell) {
                dojo.toggleClass(cell, "protectCol");
                return dojoModules.put('span', this.label);
            }

            function dgridRenderCell(rowData, cellData, cell) {

                // Get column name from cell class "field-<columnName>"  string
                // There seems to be no other way of identifying the field associated with the cell
                var matches = cell.className.match(/field-\w*\b/);
                if (angular.isArray(matches) && matches[0]) {
                    var fieldClass = matches[0],
                        field = fieldClass.substring(fieldClass.indexOf('-') + 1);
                    if ($scope.windowProps.partitionKeyColumns) {
                        _.forEach($scope.windowProps.partitionKeyColumns, function (col) {
                            if (col.name === field) {
                                dojo.toggleClass(cell, "protectCol");
                            }
                        });
                    }
                    if ($scope.windowProps.dateKeyColumn && $scope.windowProps.dateKeyColumn.name === field) {
                        dojo.toggleClass(cell, "protectCol");
                    }
                }
                if (this.dataType == vsmCommonConst.DATA_TYPE_NUMERIC) {
                    dojo.toggleClass(cell, "numericCol");
                    //FIXME: need format
                    //dataWrap.innerHTML=  vsdUtil.applySASNumericFormat(data, this.columnFormat);
                    return dojoModules.put("span", cellData);
                }
                else {
                    return dojoModules.put("span", cellData);
                }
            }

            $scope.scenarioStartDrag = function (e, ui) {
                ui.item.data('start', ui.item.index());
            };

            $scope.scenarioEndDrag = function (e, ui) {
                var start = ui.item.data('start'),
                    end = ui.item.index();
                $scope.window.scenarios.splice(end, 0,
                    $scope.window.scenarios.splice(start, 1)[0]);
                if (start !== end) {
                    $scope.dirty();
                }
            };

            function loadScenarios() {
                $scope.window.scenarios.forEach(function (scenario) {
                    // add windowInfo so it can be used for the breadcrumb
                    scenario.windowName = $scope.window.name;
                    scenario.scenarioWindowId = $scope.window.id;
                });
            }

            // Creates structure for view
            // if stats are not passed in, it uses the cachedSummaryStats
            function createCurrentStats(column, /* Optional */stats) {
                return {
                    format: column.columnFormat,
                    stats: stats ? stats : cachedSummaryStats[column.name]
                };
            }

            $scope.$watch('windowRunning', function (newVal, oldVal) {

                var runButtonIds = ['windowRunBtn', 'editColRunBtn', 'filterRunBtn', 'windowPropsRunBtn'];

                if(newVal){

                    $('.windowRunButton .dijitButtonText').addClass('showLoadingLabel');
                    $('.windowsStatsAndDist').addClass('windowsStatsAndDistStale');
                    $('.statsOverlay').removeClass('statsOverlayHidden');

                    for(var i = 0; i < runButtonIds.length; i++){
                        dijit.byId(runButtonIds[i]).set('label', $scope.resources.scenario_Controller_calculatingWaitLbl_txt);
                        dijit.byId(runButtonIds[i]).set('iconClass', 'loadingButtonGif');
                    }
                }else{
                    $('.windowRunButton .dijitButtonText').removeClass('showLoadingLabel');
                    $('.windowsStatsAndDist.windowsStatsAndDistStale').removeClass('windowsStatsAndDistStale');
                    $('.statsOverlay').addClass('statsOverlayHidden');

                    for(var i = 0; i < runButtonIds.length; i++){
                        dijit.byId(runButtonIds[i]).set('label', $scope.resources.scWindow_PreviewToolbar_Rerun_txt);
                        dijit.byId(runButtonIds[i]).set('iconClass', 'sasGoButtonIcon');
                    }
                }
            });

            function getSummaryStatsAndDistributions(outputCols, newCol) {
                $scope.showLoadingMask($scope.resources.main_loadingPreview_msg_txt);

                disableSave();
                // create list that only contains a rows that show distributions
                var distCols = [],
                    distResponse;

                $scope.colDetailsProps.currentDistribution = {};

                angular.forEach(outputCols, function (outputRow) {
                    var distCol = {};
                    angular.copy(outputRow, distCol);
                    distCol.dist = null;
                    if (distCol.name !== $scope.windowProps.dateKeyColumn.name && !_.findWhere($scope.windowProps.partitionKeyColumns, {name: distCol.name})) {

                        distCol.bins = cachedDistributions[distCol.name] || [];
                        if (tempTableColumns[distCol.name] && !distCol.bins.length) {
                            distCols.push(distCol);
                        }
                    }
                });


                if (distCols.length > 0) {
                    $scope.windowRunning = true;

                    distResponse = windowSvc.getSummaryStatsAndDistributions(distCols)
                        .then(function (results) {
                            angular.forEach(distCols, function (distRow) {
                                var colname = distRow.name;
                                var column;
                                //if origAddedName then column was added by user via dblClick, RMB or DND
                                if (distRow.newlyAdded) {
                                    column = _.findWhere($scope.window.outputColumns, {name: colname});
                                }
                                else {
                                    column = _.findWhere($scope.window.outputColumns, {name: colname});
                                }
                                cachedDistributions[column.name] = column.bins = results[colname].bins;
                                if (results[colname].summaryStats) {
                                    cachedSummaryStats[column.name] = results[colname].summaryStats;
                                    if ($scope.selectedColumn) {
                                        $scope.colDetailsProps.statsList[column.name] = tranposeStatsDataToRows(column.columnFormat,
                                            results[colname].summaryStats);
                                        $scope.colDetailsProps.currentStats = createCurrentStats(column, results[colname].summaryStats);
                                    }
                                }
                            });
                            $scope.windowRunning = false;
                            $scope.hideLoadingMask();
                        }, function errorCallback(response) {
                            $scope.windowRunning = false;
                        });
                }

                $q.when(distResponse, function () {
                    if ($scope.data.windowState === "PREVIEW") {
                        // Check state first because if user clicks on Edit button before request completes, the Save button
                        // will be enabled in Edit mode, which we don't want.
                        if (!$scope.window.needsRerun || $scope.window.needsRerun === false) {
                            enableSave();
                        }
                    }
                    // user has selected a column before all the dist data has been processed - need to show data as soon as it it avail
                    if ($scope.selectedColumn) {
                        $scope.colDetailsProps.currentDistribution = $scope.selectedColumn;
                    }
                });
            }

            function tranposeStatsDataToRows(format, stats) {
                var missingStat,
                    formattedMissing = vsdUtil.applySASNumericFormat(stats.nMiss, vsdUtil.defaultFormats.integer);

                if (numOutputRecords > 0) {
                    missingStat = formattedMissing + " (" + vsdUtil.applySASNumericFormat(stats.nMiss / numOutputRecords, vsdUtil.defaultFormats.percentage) + ")";
                }
                else {
                    missingStat = formattedMissing;
                }
                var rows = [
                    {stat: $scope.resources.chartsD3_SummaryStats_missingLbl_txt, value: missingStat},
                    {
                        stat: $scope.resources.chartsD3_SummaryStats_minLbl_txt,
                        value: vsdUtil.applySASNumericFormat(stats.min,
                            format)
                    },
                    {
                        stat: $scope.resources.chartsD3_SummaryStats_maxLbl_txt,
                        value: vsdUtil.applySASNumericFormat(stats.max,
                            format)
                    },
                    {
                        stat: $scope.resources.chartsD3_SummaryStats_Q1Lbl_txt,
                        value: vsdUtil.applySASNumericFormat(stats.q1,
                            format)
                    },
                    {
                        stat: $scope.resources.chartsD3_SummaryStats_medianLbl_txt,
                        value: vsdUtil.applySASNumericFormat(stats.q2,
                            format)
                    },
                    {
                        stat: $scope.resources.chartsD3_SummaryStats_Q3Lbl_txt,
                        value: vsdUtil.applySASNumericFormat(stats.q3,
                            format)
                    },
                    {
                        stat: $scope.resources.chartsD3_SummaryStats_meanLbl_txt,
                        value: vsdUtil.applySASNumericFormat(stats.mean,
                            format)
                    }
                    /*	{stat:  $scope.resources.chartsD3_SummaryStats_stdLbl_txt, value: vsdUtil.applySASNumericFormat(stats.std,format)},
                     {stat:  $scope.resources.chartsD3_SummaryStats_stdErrLbl_txt, value: vsdUtil.applySASNumericFormat(stats.stdErr,format)},*/
                    /*{stat: $scope.resources.chartsD3_SummaryStats_varLbl_txt, value: vsdUtil.applySASNumericFormat(stats["var"],
                     format)}*/
                    /*{stat:  $scope.resources.chartsD3_SummaryStats_ussLbl_txt, value: vsdUtil.applySASNumericFormat(stats.uss,format)}*/
                ];

                return rows;
            }

            function navWithConfirm(navFunc, id) {
                //TODO:  Need a function to call for the second parameter below to validate the whole window
                navFunc($scope.data.isDirty, true, $scope.saveWindow, $scope.resources.obj_type_window_txt, $scope.window.name, id);
            }

            function reorderColumns(locations) {
                //Apply reordering to preview and outputColumns arrays
                angular.forEach([$scope.previewProps.previewCols, $scope.window.outputColumns], function (a) {
                    arrayReorderSvc.reorderArrayItems(locations, a);
                });
                $scope.dirty();
                $scope.$apply();
            }

            function getInputTableDateRange() {

                if ($scope.window.anchorSourceColumn) {
                    windowSvc.getDateRange(window.inputTable, window.anchorSourceColumn).then(function (results) {

                        //initialize the anchorEnd date to the last date\time in the input table
                        if (results.startDate) {
                            $scope.data.minAnchorDate = new Date(results.startDate);
                            $scope.data.maxAnchorDate = new Date(results.endDate);
                        }
                    });
                }
            }

            function getInputDateRange() {
                if ($scope.window.anchorSourceColumn) {
                    if ($scope.window.windowInputs[0].leftHandTable) {
                        windowSvc.getDateRange($scope.window.windowInputs[0].leftHandTable, window.anchorSourceColumn).then(function (results) {

                            //initialize the anchorEnd date to the last date\time in the input table
                            if (results.startDate) {
                                $scope.data.minAnchorDate = new Date(results.startDate);
                                $scope.data.maxAnchorDate = new Date(results.endDate);
                            }
                        });
                    } else {
                        windowSvc.getDateRangeForWindow($scope.window.windowInputs[0].leftHandWindow.id, $scope.window.windowInputs[0].leftHandWindow, window.anchorSourceColumn).then(function (results) {

                            //initialize the anchorEnd date to the last date\time in the input table
                            if (results.startDate) {
                                $scope.data.minAnchorDate = new Date(results.startDate);
                                $scope.data.maxAnchorDate = new Date(results.endDate);
                            }
                        });
                    }
                }
            }

            function greaterThan(newVal, minDateTime) {
                var newValDatePart = moment(newVal).clone().startOf('day');
                var minDatePart = moment(minDateTime).clone().startOf('day');
                return newValDatePart >= minDatePart;
            }

            /**
             * Invoked when the user selects or types a value for the window's start date.
             *
             * The "side-model" used to work around the sndDateTextBox's truncation of time is updated properly
             * with time granularity if needed.
             */
            $scope.anchorStartDateTimeChanged = function () {
                var new_val = $scope.anchorDateTimeViews.viewAnchorStartDateTime;  //This is the sngDateTextBox's model, *without* a time component
                if ((new_val <= $scope.window.anchorEnd) &&
                    greaterThan(new_val, $scope.data.minAnchorDate)) {
                    //Then this is an acceptable date/time value
                    if (moment(new_val).startOf('day').isSame(moment($scope.data.minAnchorDate).startOf('day'))) {
                        //Insure that the window's anchor start time component is reset to include the time component (if any)
                        //of the original min anchor date
                        $scope.window.anchorStart = new Date($scope.data.minAnchorDate.getTime());
                    } else {
                        $scope.window.anchorStart = new Date(new_val.getTime());
                    }
                    $scope.anchorDateTimeViews.oldAnchorStartDateTime = $scope.window.anchorStart;
                    disableSave();
                } else {
                    //The user may have typed in a value, and circumvented the popup's legal selections
                    //Reset the anchor start date to the old value
                    $scope.window.anchorStart = new Date($scope.anchorDateTimeViews.oldAnchorStartDateTime.getTime());
                    $scope.anchorDateTimeViews.viewAnchorStartDateTime = new Date($scope.anchorDateTimeViews.oldAnchorStartDateTime.getTime());
                }
            };

            /**
             * Invoked when the user selects or types a value for the window's end date.
             *
             * The "side-model" used to work around the sndDateTextBox's truncation of time is updated properly
             * with time granularity if needed.
             */
            $scope.anchorEndDateTimeChanged = function () {
                var new_val = $scope.anchorDateTimeViews.viewAnchorEndDateTime;  //This is the sngDateTextBox's model, *without* a time component
                if (new_val <= $scope.data.maxAnchorDate &&
                    greaterThan(new_val, $scope.data.minAnchorDate)) {
                    if (moment(new_val).clone().startOf('day').isSame(moment($scope.data.maxAnchorDate).clone().startOf('day'))) {
                        $scope.window.anchorEnd = new Date($scope.data.maxAnchorDate.getTime());
                    } else {
                        $scope.window.anchorEnd = moment(new_val).endOf('day').toDate();
                    }
                    $scope.anchorDateTimeViews.oldAnchorEndDateTime = $scope.window.anchorEnd;
                    disableSave();
                } else {
                    //The user may have typed in a value, and circumvented the popup's legal selections
                    //Reset the anchor end date to the old value
                    $scope.window.anchorEnd = new Date($scope.anchorDateTimeViews.oldAnchorEndDateTime.getTime());
                    $scope.anchorDateTimeViews.viewAnchorEndDateTime = new Date($scope.anchorDateTimeViews.oldAnchorEndDateTime.getTime());
                }
            };

            function clearDateRange() {
                $scope.data.minAnchorDate = null;
                $scope.data.maxAnchorDate = null;
                window.anchorEnd = new Date($scope.data.maxAnchorDate.getTime());
                if ($scope.window.type === vsmCommonConst.WINDOW_TYPE_SLIDING || $scope.window.type === vsmCommonConst.WINDOW_TYPE_ROLLING) {
                    window.anchorStart = new Date($scope.data.minAnchorDate.getTime());
                }
            }

            function getPreviewColIndexByName(colName) {
                var colPos = -1,
                    cntr = 0;
                angular.forEach($scope.previewProps.previewCols, function (col) {
                    if (col.name === colName) {
                        colPos = cntr;
                    }
                    cntr++;
                });
                return colPos;
            }


            function setMaxAnchorEndDateTime() {
                if ($scope.window.anchorSourceColumn) {
                    if ($scope.window.anchorSourceColumn.sasDateTime) {
                        if ($scope.window.anchorEnd) {
                            //we are using datetime as opposed to date
                            //we have changed the date to be something other than the max date of the dataset
                            //so we have to be sure to set ourselves to the very end of the day (hr, min, sec, ms)
                            $scope.window.anchorEnd.setHours(23);
                            $scope.window.anchorEnd.setMinutes(59);
                            $scope.window.anchorEnd.setSeconds(59);
                            $scope.window.anchorEnd.setMilliseconds(999);
                        }
                    }
                }
            }


            //endregion

            //region *** Scoped Variables ***


            $scope.showLoadingMask($scope.resources.main_openingWindow_msg_txt);
            $scope.window = {};
            $scope.status = "";

            $scope.selectedColumn = null;

            $scope.previewProps = {
                previewCols: [],
                previewData: [],
                startRow: 0,
                pageSize: 200,
                filterCond: $scope.createBlankPreviewFilter()
            };

            var previewReorderMenuGenerator = arrayReorderSvc.createReorderMenuItems({
                getMinIdx: function () {
                    return _.where($scope.previewProps.previewCols, {vartype: "key"}).length;
                },
                getMaxIdx: function () {
                    return $scope.previewProps.previewCols.length - 1;
                },
                defaultCallback: reorderColumns,
                moveLastCallback: function (moveParams) {
                    var previewParams = angular.copy(moveParams),
                        numKeyCols = _.where($scope.previewProps.previewCols, {vartype: "key"}).length;
                    moveParams.targetIdx = $scope.previewProps.previewCols.length - numKeyCols;
                    arrayReorderSvc.reorderArrayItems(previewParams, $scope.previewProps.previewCols);

                    var ocParams = angular.copy(moveParams);
                    ocParams.targetIdx = $scope.window.outputColumns.length;
                    arrayReorderSvc.reorderArrayItems(ocParams, $scope.window.outputColumns);

                    $scope.dirty();
                    $scope.$apply();
                }
            });


            $scope.inputColsProps = {
                inputCols: [
                    {
                        id: "dataType", field: "dataType", label: "",
                        sortable: true,
                        formatter: function (icon) {

                            if (icon) {
                                var newIcon = icon ? icon.trim() : "";
                                var iconClass = "";
                                if (newIcon === "NUMERIC") {
                                    iconClass = "NUMERICIcon";
                                }
                                else {
                                    if (newIcon === "CHARACTER") {
                                        iconClass = "CHARACTERIcon";
                                    }
                                }

                                return '<div class="' + iconClass + '">&nbsp;</div>';
                            }
                            else {
                                return "";
                            }
                        }
                    },
                    {
                        id: "label",
                        field: "label",
                        formatter: function (obj) {
                            return '<span title="' + obj.dataSource + '">' + obj.label + '</span>';
                        },
                        get: function (object) {
                            return object;
                        }
                    },
                    {
                        id: "distinctVals", field: "cnt", label: "",
                        get: function (object) {
                            if (object.cnt) {
                                var fmt = "";
                                var plus = "";

                                if (object.cnt >= 5000) {
                                    object.cnt = 5000;
                                    plus = "+";
                                }
                                return object.cnt + plus;
                                // return number.format( object.cnt,{ places: 0 } ) + plus;

                            }
                            else {
                                return "";
                            }
                        }
                    }
                ],
                inputData: [],
                inputColsFilter: '',
                inputIdProperty: "name"
            };

            $scope.options = {};

            $scope.colDetailsProps = {
                currentDistribution: {},
                currentStats: {},
                statsList: {},
                statListColumns: [
                    {field: 'stat'},
                    {field: 'value'}
                ],
                columnDetailOptions: [
                    {name: 'dist', label: $scope.resources.scWindow_bottomPanel_bigDist_title_txt},
                    {name: 'stats', label: $scope.resources.scWindow_bottomPanel_summStats_title_txt}
                ],
                columnDetail: null
            };

            $scope.windowProps = {
                lasrTables: [],
                timeUnits: [],
                disableEditName: true,
                partitionKeyColumns: [],
                dateKeyColumn: null,
                transactionKeyColumn: {label: 'None'}
            };

            //endregion

            //region *** ViewModel Methods ***

            // FIXME:  duplicate code in NewWindowCtrl.js
            $scope.validateName = function (name) {
//			var resp =  windowSvc.isUniqueName(0,name).then(function(result){ return result; });
//			return resp;
                var result = vsdUtil.validateDomainObjName(name, []);
                if (!result.isValid) {
                    result.invalidMessage = $scope.resources.app_object_name_specCharValidationMsg_txt;
                }
                if (result.isValid && name.length > vsmCommonConst.WINDOW_NAME_LENGTH) {
                    result.isValid = false;
                    result.invalidMessage = dojoModules.string.substitute($scope.resources.app_object_property_LengthValidationMsg_txt,
                        [$scope.resources.common_EntryDialog_name_txt,
                            vsmCommonConst.WINDOW_NAME_LENGTH]);
                }
                return result;
            };

            $scope.assignNewColumnDefaults = function (column) {
                var newcol = angular.copy(column);
                newcol.expression = column.name;
                newcol.origAddedName = column.name;
                newcol.newlyAdded = true;
                newcol.binCount = defaultBinCount;
                newcol.id = null;
                newcol.executionOrder = 1;
                newcol.usedInScenario = false;
                newcol.sasDate = column.sasDate;
                newcol.sasDateTime = column.sasDateTime;
                newcol.lookbackPeriod = ($scope.window.defaultLookBackPeriod ? $scope.window.defaultLookBackPeriod : defaultLookBackPeriod);
                newcol.derived = (newcol.dataType === 'NUMERIC' );

                // set its derived column format if numeric
                if (newcol.dataType === "NUMERIC") {
                    if (newcol.sasDate) {
                        newcol.columnFormat = vsdUtil.defaultFormats.date;
                    }
                    else if (newcol.sasDateTime) {
                        newcol.columnFormat = vsdUtil.defaultFormats.datetime;
                    }
                    else {
                        newcol.columnFormat = vsdUtil.getSASNumericFormat(newcol.columnFormat);
                    }
                }

                var initAggregation = (newcol.sasDate || newcol.sasDateTime) ? defaultDateAggregation : $scope.window.defaultNumericAggregation;

                newcol.aggrFunction = (newcol.dataType === 'NUMERIC' && !$scope.isTransformation()) ? initAggregation : null;
                newcol.lookbackPeriod = ($scope.window.defaultLookBackPeriod ? $scope.window.defaultLookBackPeriod : defaultLookBackPeriod);

                var nameSuffix = "",
                    labelPrefix = "";
                switch (newcol.aggrFunction) {
                    case "SUM" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_sum_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_SUM_txt;
                        break;
                    case "MAX" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_max_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_MAX_txt;
                        break;
                    case "MIN" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_min_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_MIN_txt;
                        break;
                    case "MEAN" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_mean_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_MEAN_txt;
                        break;
                    case "N" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_count_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_COUNT_txt;
                        break;
                    case "MODE" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_mode_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_MODE_txt;
                        break;
                    case "NDISTINCT" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_ndistinct_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_NDISTINCT_txt;
                        break;
                    case "NMISS" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_nmiss_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_NMISS_txt;
                        break;
                    case "FIRST" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_first_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_FIRST_txt;
                        break;
                    case "LAST" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_last_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_LAST_txt;
                        break;
                    case "STD" :
                        nameSuffix = $scope.resources.scWindow_aggrFunction_std_suffix;
                        labelPrefix = $scope.resources.scWindow_AggBuilder_functions_STD_txt;
                        break;
                }

                //is new column already in the temp table?
                if (inputToTempColumnMappings[newcol.name]) {
                    newcol.name = inputToTempColumnMappings[newcol.name];
                }
                // add aggregation suffix to name if it isn't already there
                else if (nameSuffix.length > 0 && newcol.name.indexOf(nameSuffix) !== (newcol.name.length - nameSuffix.length)) {
                    newcol.name = newcol.name.substr(0, 28) + '_' + nameSuffix;
                }

                // add aggregation prefix to label if it isn't already there
                if (labelPrefix.length > 0 && newcol.label.indexOf(labelPrefix) !== 0) {
                    newcol.label = labelPrefix + ' ' + newcol.label;
                }

                // used cached distributions if available
                newcol.bins = cachedDistributions[newcol.name] || [];

                return newcol;
            };


            $scope.addColumns = function (cols) {
                var newColList = [];

                cols.forEach(function (column) {
                    var newcol = $scope.assignNewColumnDefaults(column);

                    newcol.name = windowSvc.getUniqueColumnName($scope.window.outputColumns, newcol.name);
                    newcol.label = windowSvc.getUniqueColumnLabel($scope.window.outputColumns, newcol.label);

                    //Determine if column is a duplicate - if not, don't add it
                    var preppedCol = prepColumnForPreview(newcol, true);
                    $scope.previewProps.previewCols.push(preppedCol);
                    $scope.window.outputColumns.push(newcol);
                    newColList.push(newcol);

                });

                getPreviewData();
                getSummaryStatsAndDistributions(newColList, true);

                $scope.dirty();
                $scope.windowNeedsRerun();
            };

            $scope.deleteColumn = function (deletedColumn) {

                if (!deletedColumn) {
                    return;
                }

                var colName = deletedColumn.name;
                if (scenarioDependency[deletedColumn.name]) {
                    messageSvc.showError(dojoModules.string.substitute($scope.resources.scWindow_outputColumnDeleteError_fmt_txt, [scenarioDependency[deletedColumn.name]]));
                    return;
                }
                $scope.window.outputColumns = _.filter($scope.window.outputColumns, function (column) {
                    return column.name !== colName;
                });

                if ($scope.window.transactionKeyColumn && $scope.window.transactionKeyColumn.name.toLowerCase() === deletedColumn.name.toLowerCase()) {
                    $scope.window.transactionKeyColumn = null;
                }

                var colPos = getPreviewColIndexByName(colName);
                if (colPos !== -1) {
                    $scope.previewProps.previewCols.splice(colPos, 1);
                }

                $scope.dirty();
            };

            $scope.validateTransactionKeyColumn = function (new_val) {
                //TODO: Make validation a property that can be set on config table
                if(new_val) {
                    $scope.validTransactionKeys[new_val.name] = true;
                    return {isValid: true, invalidMessage: null};
                }

                if(!new_val) {
                    return {isValid: true, invalidMessage: null};
                }
                /*
                $scope.invalidTransactionKeyColumnMsg = null;

                function clearValidatorState() {
                    $scope.invalidTransactionKeyColumnMsg = null;
                    var transactionKeyValidator = dojoModules.registry.byId("windowPropTransactionKeyTextBox");
                    transactionKeyValidator.set('invalidMessage', '');
                    transactionKeyValidator._set('state', '');
                }

                if (new_val && $scope.validTransactionKeys.hasOwnProperty(new_val.name)) {
                    return {isValid: true, invalidMessage: null};
                }
                $scope.showLoadingMask($scope.resources.scWindow_ValidateKey_txt);
                if (new_val && $scope.window.inputTable) {
                    var windowId;
                    if ($scope.window.inputTable.role === "WINDOW_OUTPUT" || $scope.window.inputTable.role === "JOIN_OUTPUT") {
                        if ($scope.window.windowInputs[0].leftHandWindow) {
                            windowId = $scope.window.windowInputs[0].leftHandWindow.id;
                        } else {
                            windowId = undefined;
                        }
                    }

                    if(windowSvc.validateTransactionKeyColSync(windowId, $scope.window.windowInputs[0].leftHandTable, new_val)){
                        $scope.hideLoadingMask();
                        clearValidatorState();
                        $scope.validTransactionKeys[new_val.name] = true;
                        return {isValid: true, invalidMessage: null};
                    } else{
                        $scope.hideLoadingMask();
                        return {isValid: false, invalidMessage: $scope.resources.scWindow_InvalidKey_txt};
                    }
                } else {
                    $scope.hideLoadingMask();
                    clearValidatorState();
                    return {isValid: true, invalidMessage: null};
                }
                */
            };


            $scope.$watch('window.transactionKeyColumn', function (new_val, old_val) {
                if (new_val && old_val && (new_val.name.toLowerCase() === old_val.name.toLowerCase())) {
                    return;
                }

                if (new_val && !$scope.validTransactionKeys.hasOwnProperty(new_val.name)) {
                    return;
                }

                if (new_val && $scope.window.type === vsmCommonConst.WINDOW_TYPE_TRANSFORMATION) {

                    //Reset parameters on old window.transactionKeyColumn
                    if (old_val && (old_val.name !== new_val.name)) {
                        var oldTransactionKeyCol = _.find($scope.window.outputColumns, function (column) {
                            return column.name.toLowerCase() === old_val.name.toLowerCase();
                        });

                        if (oldTransactionKeyCol) {
                            oldTransactionKeyCol.executionOrder = 1;
                            oldTransactionKeyCol.derived = true;
                        }
                    }

                    //Add transaction key column to outputTable
                    var onOutputTable;
                    onOutputTable = _.find($scope.window.outputColumns, function (column) {
                        return column.name.toLowerCase() === $scope.window.transactionKeyColumn.name.toLowerCase();
                    });
                    if (!onOutputTable) {
                        var newCols = [];
                        newCols.push($scope.window.transactionKeyColumn);
                        $scope.addColumns(newCols);
                        onOutputTable = _.find($scope.window.outputColumns, function (column) {
                            return column.name.toLowerCase() === $scope.window.transactionKeyColumn.name.toLowerCase();
                        });
                    }

                    if ($scope.window.type === vsmCommonConst.WINDOW_TYPE_ROLLING) {
                        onOutputTable.executionOrder = 0;
                    } else {
                        onOutputTable.executionOrder = 1;
                    }
                    onOutputTable.derived = false;
                    onOutputTable.expression = onOutputTable.name;
                    $scope.windowNeedsRerun();
                }
            });

            //function to filter input columns
            $scope.filterInputColumns = function (item, index, items) {
                var filterString = $scope.inputColsProps.inputColsFilter.toLowerCase();
                return vsdUtil.gridFilterByAttribute(filterString, item, "label");
            };

            $scope.$watchCollection('previewProps.previewCols', function (newColumns, oldColumns) {
                $scope.previewProps.filterCols = _.filter(newColumns, function (col) {
                    return tempTableColumns[col.name];
                });
            });

            //watch filter in preview tool bar for changes
            $scope.$on(vsmCommonConst.CONDITION_BUILDER_CHANGE, function (evt) {
                $scope.filterNotPresent = false;
                evt.preventDefault();
                var filterCond = $scope.previewProps.filterCond;
                if (filterCond.operand1ColumnInfo && filterCond.operand1ColumnInfo.name && (filterCond.operand2Constant || filterCond.operator.name.indexOf('MISSING') > -1)) {
                    getPreviewData();
                }
            });

            $scope.clearPreviewFilter = function () {
                if ($scope.previewProps.filterCond) {
                    $scope.previewProps.filterCond = $scope.createBlankPreviewFilter();
                    getPreviewData();
                }
                $scope.filterNotPresent = true;
            };

            //cellHandler for previewGrid - this brings in the selected column
            $scope.columnSelect = function (column, currColPos) {
                if (!column) {
                    $scope.selectedColumn = null;
                    return;
                }
                else {
                    //if (preSelect ) {
                    $scope.selectedColumn = _.findWhere($scope.window.outputColumns, {name: column});
                    if (!$scope.selectedColumn) {
                        var aggrColName = _.findWhere($scope.previewProps.previewCols, {name: column});
                        $scope.selectedColumn = _.findWhere($scope.window.outputColumns, {name: aggrColName.origAddedName});

                    }
                    $scope.currColPos = currColPos;
                    //}


                    if ($scope.selectedColumn.bins) {
                        $scope.colDetailsProps.currentDistribution = $scope.selectedColumn;
                    }
                    else {
                        $scope.colDetailsProps.currentDistribution = null;
                    }

                    //if selected Column is CHAR then summary stats not avail
                    if ($scope.colDetailsProps.columnDetail == 'statsChar' || $scope.colDetailsProps.columnDetail == 'stats') {
                        switch ($scope.selectedColumn.dataType) {
                            case  'CHARACTER':
                                $scope.colDetailsProps.columnDetail = 'statsChar';
                                break;
                            case  'NUMERIC':
                                $scope.colDetailsProps.columnDetail = 'stats';
                                $scope.colDetailsProps.currentStats = createCurrentStats($scope.selectedColumn);
                                break;
                        }
                    }
                }
            };

            $scope.isWindowDisabled = function(){
                if($scope.windowRunning == false){
                    return false;
                }
                return true;
            }

            $scope.detailSelect = function (index) {

                $scope.colDetailsProps.columnDetail = this.option.name;
                //if selected Column is CHAR then summary stats not avail
                if ($scope.colDetailsProps.columnDetail == 'statsChar' || $scope.colDetailsProps.columnDetail == 'stats') {
                    switch ($scope.selectedColumn.dataType) {
                        case  'CHARACTER':
                            $scope.colDetailsProps.columnDetail = 'statsChar';
                            break;
                        case  'NUMERIC':
                            $scope.colDetailsProps.columnDetail = 'stats';
                            break;
                    }
                }
            };

            $scope.toggleWindowState = function () {
                if ($scope.data.windowState === "PREVIEW") {
                    $scope.data.windowState = 'EDIT';
                    disableSave();
                }
                else {
                    $scope.data.windowState = "PREVIEW";
                    if (!$scope.window.needsRerun || $scope.window.needsRerun === false) {
                        enableSave();
                    }
                }
            };

            $scope.isTransformation = function () {
                return ($scope.window.type === vsmCommonConst.WINDOW_TYPE_TRANSFORMATION);
            };

            $scope.isSliding = function () {
                return ($scope.window.type === vsmCommonConst.WINDOW_TYPE_SLIDING) || ($scope.window.type === vsmCommonConst.WINDOW_TYPE_ROLLING);
            };

            $scope.isRolling = function () {
                return ($scope.window.type === vsmCommonConst.WINDOW_TYPE_ROLLING);
            };

            $scope.isAggregation = function () {
                return ($scope.window.type === vsmCommonConst.WINDOW_TYPE_AGGREGATION);
            };

            $scope.editWindowFilter = function () {
                $scope.data.filterCondition = angular.copy(window.filterCondition);
                $scope.data.windowState = "WINDOW_FILTER";
                disableSave();
            };

            $scope.cancelWindowFilterChanges = function () {
                $scope.data.filterCondition = null;
                $scope.toggleWindowState();
            };

            $scope.clearWindowFilter = function () {
                $scope.data.filterCondition = null;
            };

            $scope.addWindowFilter = function () {
                var defaultOper = _.findWhere($scope.data.operatorMap, {name: 'AND'});
                var defaultCondOper = _.findWhere($scope.data.operatorMap, {name: '='});
                // replace null entry in first position with an actual nest operator and empty condition
                $scope.data.filterCondition = window.filterCondition ? angular.copy(window.filterCondition) :
                {
                    operator: defaultOper,
                    displayOrder: 0,
                    operand1ColumnInfo: {},
                    operand2Constant: null,
                    childConditions: [
                        {
                            operator: defaultCondOper,
                            operand1ColumnInfo: {name: '', label: ''},
                            operand2Constant: "", childConditions: []
                        }
                    ]
                };
            };

            $scope.applyWindowFilter = function () {
                if (dojoModules.registry.byId('windowFilterForm').validate()) {
                    window.filterCondition = $scope.data.filterCondition;
                    $scope.dirty();
                    $scope.toggleWindowState();
                    $scope.runWindow();
                }
            };

            $scope.runWindow = function () {
                $scope.dirty();
                $scope.window.needsRerun = false;
//                setMaxAnchorEndDateTime();
                createWindow();
            };

            $scope.refreshPreview = function () {
                $scope.dirty();
                $scope.previewProps.previewCols = getPreviewColumns();
            };


            $scope.saveWindow = function () {
                //FIXME:  we shouldn't do it this way, should create a directive that talks back to Ctrl.
                var valid = dojoModules.registry.byId("windowPropertiesForm").validate();
                if (!valid) {
                    messageSvc.showError($scope.resources.scWindow_properties_error_txt);
                    return;
                }
                $scope.showLoadingMask($scope.resources.scWindow_savingMsg_txt);

                angular.forEach($scope.window.outputColumns, function (o, idx) {
                    o.displayOrder = idx;
                });

//                setMaxAnchorEndDateTime();

                return windowSvc.save($scope.window)
                    .then(function (results) {
                        window = $scope.window = results;
                        $scope.hideLoadingMask();
                        setupOutputColumns();
                        $scope.showStatusMessage($scope.resources.scWindow_saveCompleteMsg_txt);
                        $scope.windowProps.disableEditName = true;
                        if ($scope.window.anchorSourceColumn) {  //???doherk - is this needed?  Isn;t the
                            if ($scope.window.anchorEnd) {
                                $scope.window.anchorEnd = new Date($scope.window.anchorEnd); //have to get from days or seconds since 1970 to Date object
                            }
                            if ($scope.window.anchorStart) {
                                $scope.window.anchorStart = new Date($scope.window.anchorStart); //have to get from days or seconds since 1970 to Date object
                            }

                        }

//					$timeout(function(){setDirty(false);}, 0);
                        setDirty(false);
                    });
            };

            // DND handler for a new window before users have dragged over any columns - so preview grid is not showing yet.
            $scope.onNewWinDrop = function (source, nodes, copy, target) {
                var newCols = [];
                angular.forEach(nodes, function (node) {
                    newCols.push(node.custNodeData);
                });
                $scope.addColumns(newCols);

            };

            $scope.userHasDroppedColumn = function () {
                var rc = false;
                var colCount = $scope.window.outputColumns.length - $scope.window.partitionKeyColumns.length;
                if ($scope.window.type === vsmCommonConst.WINDOW_TYPE_SLIDING || $scope.window.type === vsmCommonConst.WINDOW_TYPE_ROLLING) {
                    colCount -= 1;
                }
                if (colCount > 0) {
                    rc = true;
                }
                return rc;
            };

            $scope.goToHubScreen = function () {
                navWithConfirm($scope.goToHub);
            };

            $scope.showNewWindowDialog = function () {
                navWithConfirm($scope.newWindow);
            };

            $scope.showNewScenarioDialog = function () {
                newScenarioSvc.setSelectedWindowId($scope.window.id);
                newScenarioSvc.setSelectedWindowName($scope.window.name);
                navWithConfirm($scope.newScenario);
            };

            $scope.showNewDeploymentDialog = function () {
                newDeploymentSvc.setInitWindowIds([$scope.window.id]);
                navWithConfirm($scope.newDeployment);
            };

            $scope.showHomeScreen = function () {
                navWithConfirm($scope.showHome);
            };

            $scope.showOpenScreen = function () {
                navWithConfirm($scope.showOpen);
            };

            $scope.selectScenario = function (scenario) {
                navWithConfirm($scope.showScenario, scenario.id);
            };

            $scope.openWithVae = function () {
                // TODO: text
                $scope.showLoadingMask($scope.resources.main_loadingVae_msg_txt);
                windowSvc.openWithVae($scope.window);
            };

            $scope.onHeaderRMB = function (column) {
                var menuItems = [];
                //If it's a protected column, return an empty array of items
                if (column.executionOrder === 0) {
                    return menuItems;
                }
                //Add delete menu entry
                if (!column.usedInScenario) {
                    var menuItem = {
                        label: $scope.resources.scWindow_Preview_deleteColumnLbl_text,
                        iconClass: "sasDeleteIcon",
                        onClick: function (evt) {
                            $scope.$apply(function () {
                                $scope.deleteColumn(column);
                            });
                        }
                    };

                    menuItems.push(menuItem);
                }

                //Create reordering menu
                menuItems.push(previewReorderMenuGenerator(getPreviewColIndexByName(column.name)));

                return menuItems;
            };

            $scope.window = window;
            if ($scope.window.transactionKeyOptions) {
                $scope.options.transactionKeyColumns = $orderBy($scope.window.transactionKeyOptions, ['label']);
            } else {
                if (!$scope.window.inputTable) {
                    return;
                }
                // Filter partitionKey columns
                var temp = _.reject($scope.window.inputTable.columns, function (col) {
                    return _.find($scope.window.partitionKeyColumns, function (partitionKey) {
                        return partitionKey.name === col.name;
                    });
                });

                // Filter date columns
                $scope.options.transactionKeyColumns = _.filter(temp, function (dateColumns) {
                    return dateColumns.anchorDateCandidate !== true;
                });
            }


            $scope.data.operatorMap = operators;
            //Set operator for preview filter to full object
            $scope.previewProps.filterCond.operator = _.findWhere(operators, {name: '='});

            $scope.windowProps.dateKeyColumn = $scope.window.anchorSourceColumn || {};

            $scope.dirty = function () {
                setDirty(true);
            };

            $scope.defaultNumericAggregationChanged = function () {
                window.defaultNumericAggregation = $scope.options.defaultNumericAggregation.value;
                $scope.windowNeedsRerun();
            };

            $scope.windowNeedsRerun = function () {
                $scope.window.needsRerun = true;
                disableSave();
            };

            $scope.displayHierarchicalView = function () {
                prepareHierarchicalDisplay();
                $scope.data.showHierarchicalView = true;
            };

            $scope.anchorSourceColumnChangeHandler = function () {
                var window = $scope.window;
                windowSvc.getTimeUnits().then(function (value) {
                    $scope.options.entityTimeUnits = value;
                    if (window.anchorSourceColumn) {

                        var dayUnitList = $filterBy(value, {granularityUnit: "DAYS"});
                        if (!$scope.window.anchorSourceColumn.sasDateTime) {
                            $scope.options.entityTimeUnits = dayUnitList;
                        }
                        if (!window.entityTimeUnit) {
                            window.entityTimeUnit = dayUnitList[0];
                        }
                        getInputTableDateRange();
                    }
                    else {
                        clearDateRange();
                        window.entityTimeUnit = null;
                    }
                });
                $scope.dirty();
            };

            $scope.options.dateColumns = $orderBy($filterBy(window.inputTable.columns, {anchorDateCandidate: true}), ['label']);

            if (window.anchorSourceColumn) {
                windowSvc.getTimeUnits().then(function (value) {
                    if (!window.anchorSourceColumn.sasDateTime) {
                        $scope.options.entityTimeUnits = $filterBy(value, {granularityUnit: "DAYS"});
                    }
                    else {
                        $scope.options.entityTimeUnits = value;
                    }
                });
            }

            $scope.runExport = function () {
                // did not work with .then or return statements. Users will not be notified of success or failure.
                windowSvc.exportIframe($scope.window.id, "WINDOW");
            };

            $scope.options.nonDateBasedNumericAggregationTypes =
                [
                    {value: null, label: $scope.resources.scWindow_AggBuilder_functions_UPP_txt},
                    {value: 'SUM', label: $scope.resources.scWindow_AggBuilder_functions_SUM_txt},
                    {value: 'MEAN', label: $scope.resources.scWindow_AggBuilder_functions_MEAN_txt},
                    {value: 'STD', label: $scope.resources.scWindow_AggBuilder_functions_STD_txt},
                    {value: 'N', label: $scope.resources.scWindow_AggBuilder_functions_COUNT_txt},
                    {value: 'NDISTINCT', label: $scope.resources.scWindow_AggBuilder_functions_NDISTINCT_txt},
                    {value: 'NMISS', label: $scope.resources.scWindow_AggBuilder_functions_NMISS_txt},
                    {value: 'MAX', label: $scope.resources.scWindow_AggBuilder_functions_MAX_txt},
                    {value: 'MIN', label: $scope.resources.scWindow_AggBuilder_functions_MIN_txt},
                    {value: 'MODE', label: $scope.resources.scWindow_AggBuilder_functions_MODE_txt}
                ];

            $scope.options.dateBasedNumericAggregationTypes =
                [
                    {value: 'FIRST', label: $scope.resources.scWindow_AggBuilder_functions_FIRST_txt},
                    {value: 'LAST', label: $scope.resources.scWindow_AggBuilder_functions_LAST_txt}
                ];

            $scope.options.defaultNumericAggregationTypes = [];

            setupWindow();

            setDirty(!window.id);


        }]);


});

},
'vsd/window/WindowSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

//Window CRUD services
angular.module('vsd')
    .factory('windowSvc', [ '$http', '$window', 'dojoModules', function ($http, $window, dojoModules) {

      var urlBase = 'action/';
      var windowSvc = {};

      windowSvc.getWindowInputs = function () {
        return $http.get(urlBase + 'windowInputs').then(function (resp) {
          angular.forEach(resp.data, function (input) {
            if (input.window) {
              input.table = input.window.outputTable;
            }
          });
          return resp.data;
        }, function (e) {
          return [];
        });
      };

        //tableWithDistinctCounts

        windowSvc.getTableWithDistinctCounts = function (tableId, getCounts) {
            return $http.post(urlBase + 'tableWithDistinctCounts', tableId,
                {headers: {'Content-Type': 'application/text;charset=utf-8'}}).then(function (resp) {
                    return resp.data;
                });
        };

      windowSvc.getTable = function (tableId, getCounts) {
        return $http.post(urlBase + 'table', tableId,
            {headers: {'Content-Type': 'application/text;charset=utf-8'}}).then(function (resp) {
              return resp.data;
            });
      };

      windowSvc.getTimeUnits = function () {
        return $http.get(urlBase + 'entityTimeUnits', {cache: true}).then(function (resp) {
            return _.reject(resp.data, function (entityTimeUnits) {
                return entityTimeUnits.id === -1;
            });
        });
      };

      windowSvc.getEntityTypes = function () {
        return $http.get(urlBase + 'entityTypes', {cache: true}).then(function (resp) {
          return resp.data;
        });
      };

      windowSvc.getOutputTypes = function () {
          return $http.get(urlBase + 'windowOutputTypes', {cache: true}).then(function (resp) {
              return _.reject(resp.data, function (windowOutputTypes) {
                  return windowOutputTypes.id === -1;
              });
          });
      };


      windowSvc.getWindowList = function () {
        var url = urlBase + '/win';
        return $http.get(url);
      };

      windowSvc.getWindow = function (id, getDistinctCounts) {

          var url = urlBase + 'win/' + id;
          url = getDistinctCounts ? url  + "/getDistinctCounts": url;
          windowSvc.gotDistinctCounts = getDistinctCounts ? true: false;

        return $http.get(url).then(function (results) {
          // Fixme: remove padding from defaultNumericAggregation during window save until data base is updated
          if (results.data.defaultNumericAggregation && typeof results.data.defaultNumericAggregation === "string") {
            results.data.defaultNumericAggregation = results.data.defaultNumericAggregation.trim();
          }
          return results.data;
        });
      };

      windowSvc.deleteWindow = function (id, version) {
        return $http['delete'](urlBase + 'win/' + id + '/' + version).then(function (response) {
          return response.data;
        });
      };

      windowSvc.createWindow = function (inWindow, useCache) {
        //doherk - uncomment to log the JSON that's will be sent to the midtier
        //console.log("inWindow = " + JSON.stringify(inWindow));

        var url = urlBase + 'createWindow/' + useCache;
        return $http.post(url, inWindow).then(function (results) {
				return results.data;
        });

      };

      windowSvc.runWindowById = function (windowId, scenarioCount) {
        var uri = "action/runWindow/" + windowId + "/" + scenarioCount;
        return $http.post(uri);
      };

      // FIXME:  handle duplicate names
      windowSvc.save = function (window) {
        return $http.put(urlBase + 'win', window).then(function (response) {
          return response.data;
        });
      };

      windowSvc.previewData = function (outputCols, orderByCols, inWhere, inStart, inNbr) {
        var where = inWhere || null,
            start = inStart || 0,
            nbr = inNbr || 100,
            columnNames = _.pluck(outputCols, "name"),
            orderByColumnNames = _.pluck(orderByCols, "name"),
            previewPayload = {"columnNames": columnNames, "orderByColumnNames": orderByColumnNames, "whereClause": where},
            uri = 'previewData/' + start + '/' + nbr;
        return $http.post(urlBase + uri, previewPayload).then(function (results) {
          return results.data;
        });
      };

      windowSvc.getDateRange = function (inputTable, dateColumn) {
        var options = {"table": inputTable, "column": dateColumn},
            uri = 'getTableDateRange/';

        return $http.post(urlBase + uri, options).then(function (results) {
          return results.data;
        });
      };

      windowSvc.getDateRangeForTables = function (inputTables, dateColumns) {
          var options = {"tables": inputTables, "columns": dateColumns},
              uri = 'getTablesDateRange/';

          return $http.post(urlBase + uri, options).then(function (results) {
              return results.data;
          });
      };

      windowSvc.getDateRangeForWindow = function (inputWindowId, inputTable, dateColumn) {
          var options = {table: inputTable, "column": dateColumn},
              uri = 'getWindowDateRange/' + inputWindowId;

          return $http.post(urlBase + uri, options).then(function (results) {
              return results.data;
          });
      };

        /**
         * Had to use a synchronous post call to return this data immediately for
         * a validator, which cannot use the "then" to access that info from the promise.
         * TODO: need a more consistent and dependable method of validation that wouldn't rely on this one off behavior
         * */
        windowSvc.validateTransactionKeyColSync = function (inputWindowId, inputTable, column) {
            if (!column) {
                return;
            }
            var idToPost = inputWindowId ? inputWindowId : -1;

            var options = {"table": inputTable, "column": column},
                uri = 'validateTransactionKeyCol/'+ idToPost;

            var returnVal = undefined;
            var postedData = JSON.stringify(options);

            dojo.xhrPost({
                url: urlBase + uri,
                timeout: 3000, // give up after 3 seconds
                postData: postedData, // creates ?part=one&another=part with GET, Sent as POST data when using xhrPost
                headers: { 'Content-Type': 'application/json' },
                handleAs: 'json',
                sync: true,
                load: function (data) {
                    returnVal = true;
                },
                error:function (data) {
                    returnVal = false;
                }
            });

            return returnVal;
        };

      windowSvc.getDistributionData = function (columns, endOnOpt) {
        if (!endOnOpt) {
          endOnOpt = 0;
        }
        return $http.post(urlBase + 'distributions/' + endOnOpt, columns).then(function (resp) {
          return resp.data;
        });
      };

      windowSvc.getSummaryStats = function (column) {
        return $http.get(urlBase + 'summaryStats/' + column.name).then(function (results) {
          return results.data;
        });
      };

      windowSvc.getSummaryStatsAndDistributions = function (columns, endOnOpt) {
        if (!endOnOpt) {
          endOnOpt = 0;
        }
        return $http.post(urlBase + 'summaryStatsAndDistributions/' + endOnOpt, columns).then(function (resp) {
          return resp.data;
        });
      };

      windowSvc.getOutputColumns = function (id) {
        return $http.get(urlBase + '/win/' + id + '/outputColumns');
      };

      windowSvc.getOutputTableColumnsForWindow = function (id) {
        var url = urlBase + '/win/' + id + '/getOutputTableColumnsForWindow';
        return $http.get(url).then(function (response) {
          var out = [];
          angular.forEach(response.data, function (entry) {
            var outEntry = entry.column;
            outEntry.columnType = entry.columnType;
            out.push(outEntry);
          });
          return out;
        });
      };

      windowSvc.getOutputTableColumnsWithClassification = function (windowId, reqScenarioExecOrder) {
        var url = urlBase + '/win/' + windowId + '/' + reqScenarioExecOrder + '/getOutputTableColumnsWithClassification';
        return $http.get(url).then(function (response) {
          var out = [];
          angular.forEach(response.data, function (entry) {
            var outEntry = entry.column;
            outEntry.columnType = entry.columnType;
            outEntry.classification = entry.classification;
            outEntry.distinctCount = entry.distinctCount;
            outEntry.earliestAssignment = entry.earliestAssignment;
            out.push(outEntry);
          });
          return out;
        });
      };

      windowSvc.getActionColumns = function (windowId, includeScenarioDeps) {
        var incDepsStr = includeScenarioDeps ? "true" : "false";
        var uri = urlBase + "win/" + windowId + "/actionColumns/" + incDepsStr;
        return $http.get(uri);
      };

      windowSvc.addUpdateActionColumns = function (windowId, addsUpdates) {
        var url = urlBase + "win/" + windowId + "/outputTableColumn";
        return $http.put(url, addsUpdates).then(function (response) {
          return response.data;
        });
      };

      windowSvc.deleteActionColumns = function (windowId, deletes) {
        var url = urlBase + "win/" + windowId + "/outputTableColumn";
        return $http.post(url, deletes).then(function (response) {
          return response.data;
        });
      };

      windowSvc.saveScenarioExecutionOrder = function (scenarios) {
        return $http.post(urlBase + "scenario/executionOrder", scenarios).then(function (response) {
          return response.data;
        });
      };

      windowSvc.openWithVae = function (window) {
        $http.post(urlBase + "promoteWindow", window).then(function () {
          angular.element($window).off("beforeunload");
          $window.location = 'transfer.jsp?target=EXPLORER&tablePath=' + window.outputTable.folderPath + '/' + window.outputTable.name + '(Table)';
        });
      };

      windowSvc.isUniqueName = function (id, name) {

        return $http.get(urlBase + 'win/validateName/' + id + '/' + name).then(function (response) {
          var result = {isValid: false, invalidMessage: ''};
          result.isValid = (response.data === 'OK');
          if (!result.isValid) {
            result.invalidMessage = dojoModules.resources.scWindow_nameInUse_txt;
          }
          return result;
        });
      };

      windowSvc.exportIframe = function (objId, objType) {
          if (objType === 'WINDOW') {
            // this is a jQuery call. The native saveas dialog does not work with XHR due to an AngularJS bug.
            $('#exportIframe').attr('src', urlBase + 'export/window/' + objId);
        }
        else if (objType === 'DEPLOYMENT') {
            var objIdNum = objId.replace("d","");
            // this is a jQuery call. The native saveas dialog does not work with XHR due to an AngularJS bug.
            $('#exportIframe').attr('src', urlBase + 'export/deployment/' + objIdNum);
        }
      };

      windowSvc.listScenarioNames = function (windowId) {
        return $http.get(urlBase + "window/" + windowId + "/listScenarioNames").then(function (response) {
          return response.data;
        });
      };

      function getUniqueName(nameHash, name, maxLength, uCaseFlg) {
        var suffix = 1,
            suffixLength,
            newName = name.substring(0, maxLength),
            baseName = newName;

        function uCase(val) {
          return uCaseFlg ? val.toUpperCase() : val;
        }

        // Mark name as first one to match
        if (!nameHash[uCase(newName)]) {
          nameHash[uCase(newName)] = 1;
          return newName;
        }
        else { // collision, name already in use
          do {
            suffixLength = suffix.toString().length;
            if (baseName.length + suffixLength <= maxLength) { // just add to the end since there is room
              newName = baseName + suffix;
            }
            else { // ran out of room, truncate and add suffix
              newName = baseName.substring(0, (maxLength - suffixLength)) + suffix;
            }
            suffix++;
          } while (nameHash[uCase(newName)]);
          // unique name found; store
          nameHash[uCase(newName)] = 1;
          // update column

          return newName;
        }
      }

      windowSvc.getUniqueColumnName = function (columns, newName) {
        var nameHash = {},
            maxLength = 32;
        // Record all taken names
        angular.forEach(columns, function (col) {
          nameHash[col.name.substring(0, maxLength).toUpperCase()] = 1;
        });

        return getUniqueName(nameHash, newName, maxLength, true);
      };

      windowSvc.getUniqueColumnLabel = function (columns, newLabel) {
        var nameHash = {},
            maxLength = 255;
        // Record all taken names
        angular.forEach(columns, function (col) {
          nameHash[col.label.substring(0, maxLength)] = 1;
        });

        return getUniqueName(nameHash, newLabel, maxLength);
      };

      windowSvc.handleNameCollisions = function (columns) {
        var nameHash = {},
            maxLength = 32,
            len;

        // Record all taken names
        angular.forEach(columns, function (col) {
          nameHash[col.name.substring(0, maxLength).toUpperCase()] = 0;
        });

        // Check for duplicate names
        angular.forEach(columns, function (col) {
          len = col.name.length > maxLength ? maxLength : col.name.length;
          col.name = getUniqueName(nameHash, col.name, len, true);
        });
      };

      return windowSvc;
    }]);

});

},
'vsd/window/columnsEditor/columnProperties':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.directive('columnProperties', ['dojoModules','operatorSvc', 'vsdUtil' , 'fieldParserSvc', 'columnNameValidationSvc', function (dojoModules, operatorSvc, vsdUtil, fieldParserSvc, columnNameValidationSvc) {
		return {
			templateUrl: 'vsd/window/columnsEditor/columnProperties.html',
			replace: true,
			restrict: 'E',
			priority: 1010,
			scope: true,
			controller: ['$scope','$element','$attrs',function($scope, $element, $attrs) {

				var currCol = $scope.data.outputColumnMappings[$scope.column.name.toLowerCase()],
					resources = dojoModules.resources;
			 	operatorSvc.getOperators().then( function(data) {
					$scope.operatorMap = data;
				});

				$scope.resources = resources;
				$scope.colPropPrimitives={};

				var asnFldParser=  fieldParserSvc.fieldParser($scope.column, $scope.data.inputColumnMappings, $scope.data.outputColumnMappings);
				var fields = asnFldParser.extractFields();
				if(fields.length)
					$scope.aggregateMenu = $scope.editPrimitives.aggregValues[fields[0].dataType];
				else
					$scope.aggregateMenu = $scope.editPrimitives.aggregValues.NUMERIC;

				var entityTimeUnit = $scope.window.entityTimeUnit;
				$scope.hasEntityTimeUnit = !!entityTimeUnit;

				if ($scope.hasEntityTimeUnit) {
					$scope.lookbackTimeUnit = resources['scWindow_AggBuilder_' + entityTimeUnit.granularityUnit + 'Label_txt'];
				}

				$scope.isNotAggregated = !$scope.column.aggrFunction;

				$scope.defaultAggreg =  $scope.editPrimitives.defaultAggregation[$scope.column.dataType];

                $scope.$watch('column.dataLength', function(newVal, oldVal) {
                    if (newVal !== oldVal) {
                        var col = $scope.column;
                        if (col.dataType === 'CHARACTER') {
                            col.columnFormat = col.columnFormat.replace(oldVal, newVal);
                        }
                    }
                });

                $scope.$watch('column.aggrFunction', function (newVal, oldVal) {

                    var col = $scope.column;

                    if (col.aggrFunction === 'NDISTINCT' || col.aggrFunction === 'N' || col.aggrFunction === 'NMISS') {
                        col.dataType = 'NUMERIC';
                        col.derived = true;
                        col.dataLength = 8;
                        col.columnFormat = vsdUtil.defaultFormats.integer;
                    }

                    /*
                    else if (col.parentColumns.length && col.parentColumns[0].dataType === 'CHARACTER') {
                        col.dataType = col.parentColumns[0].dataType;
                        col.derived = !!col.aggrFunction && (col.executionOrder === 1);
                        col.dataLength = col.parentColumns[0].dataLength;
                        col.columnFormat = col.parentColumns[0].columnFormat;
                    }
                    */

                    var aggrFunctionSuffix;
                    var newLabelTxt;
                    if (col.aggrFunction === 'SUM') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_sum_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_SUM_txt;
                    }
                    else if (col.aggrFunction === 'MAX') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_max_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_MAX_txt;
                    }
                    else if (col.aggrFunction === 'MIN') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_min_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_MIN_txt;
                    }
                    else if (col.aggrFunction === 'MEAN') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_mean_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_MEAN_txt;
                    }
                    else if (col.aggrFunction === 'N') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_count_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_COUNT_txt;
                    }
                    else if (col.aggrFunction === 'STD') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_std_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_STD_txt;
                    }
                    else if (col.aggrFunction === 'MODE') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_mode_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_MODE_txt;
                    }
                    else if (col.aggrFunction === 'NDISTINCT') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_ndistinct_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_NDISTINCT_txt;
                    }
                    else if (col.aggrFunction === 'NMISS') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_nmiss_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_NMISS_txt;
                    }
                    else if (col.aggrFunction === 'FIRST') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_first_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_FIRST_txt;
                    }
                    else if (col.aggrFunction === 'LAST') {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_last_suffix;
                        newLabelTxt = $scope.resources.scWindow_AggBuilder_functions_LAST_txt;
                    }
                    else {
                        aggrFunctionSuffix = $scope.resources.scWindow_aggrFunction_upp;
                        newLabelTxt = col.name;
                    }

                    var oldSuffix;
                    var oldLabelTxt;
                    if (oldVal === 'MEAN') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_mean_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_MEAN_txt;
                    }
                    else if (oldVal === 'N') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_count_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_COUNT_txt;
                    }
                    else if (oldVal === 'MODE') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_mode_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_MODE_txt;
                    }
                    else if (oldVal === 'NDISTINCT') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_ndistinct_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_NDISTINCT_txt;
                    }
                    else if (oldVal === 'NMISS') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_nmiss_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_NMISS_txt;
                    }
                    else if (oldVal === 'FIRST') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_first_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_FIRST_txt;
                    }
                    else if (oldVal === 'LAST') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_last_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_LAST_txt;
                    }
                    else if (oldVal === 'MAX') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_max_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_MAX_txt;
                    }
                    else if (oldVal === 'MIN') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_min_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_MIN_txt;
                    }
                    else if (oldVal === 'STD') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_std_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_STD_txt;
                    }
                    else if (oldVal === 'SUM') {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_sum_suffix;
                        oldLabelTxt = $scope.resources.scWindow_AggBuilder_functions_SUM_txt;
                    }
                    else {
                        oldSuffix = $scope.resources.scWindow_aggrFunction_upp;
                        oldLabelTxt = col.label;
                    }

                    var temp1 = '_' + oldSuffix;
                    var temp2 = oldSuffix + '_';
                    var temp3 = $scope.resources.scWindow_aggrFunction_first_prefix + '_';
                    var tempName = col.name;
                    var tempLabel = col.label;

                    if (col.aggrFunction !== null && col.aggrFunction !== 'UPP' && col.aggrFunction !== "") { //Not unique per partition
                        if (oldSuffix !== $scope.resources.scWindow_aggrFunction_upp) { //old not unique per partition
                            if (col.name.indexOf(temp1) >= 0) {
                                col.name = tempName.replace(temp1, ('_' + aggrFunctionSuffix));
                            }
                            else if (col.name.indexOf(temp2) >= 0) {
                                if (aggrFunctionSuffix !== $scope.resources.scWindow_aggrFunction_first_suffix)
                                    col.name = tempName.replace(temp2, (aggrFunctionSuffix + '_'));
                                else
                                    col.name = tempName.replace(temp2, temp3);
                            }
                            if (oldSuffix === $scope.resources.scWindow_aggrFunction_first_suffix) {
                                if (col.name.indexOf(temp3) >= 0 )
                                    col.name = tempName.replace(temp3, (aggrFunctionSuffix + '_'));
                            }

                            if (col.label.indexOf(temp1) >= 0)
                                col.label = tempLabel.replace(temp1, ('_' + aggrFunctionSuffix));
                            else if (col.label.indexOf(temp2) >= 0)
                                col.label = tempLabel.replace(temp2, (aggrFunctionSuffix + '_'));
                            else if (col.label.indexOf(oldLabelTxt) >= 0)
                                col.label = tempLabel.replace(oldLabelTxt, newLabelTxt);
                            else {
                                if (oldLabelTxt !== newLabelTxt) {
                                    col.label = newLabelTxt + ' ' + col.label;
                                }
                            }
                        } else { //old was unique per partition
                            col.name = col.name + '_' + aggrFunctionSuffix;
                            col.label = newLabelTxt + ' ' + col.label;
                        }
                    } else { //unique per partition
                        if(!$scope.isTransformation()) {  //leave columns from Transformation window alone
                            if (col.name.indexOf(temp1) >= 0) {
                                col.name = tempName.replace(temp1, (''));
                            }
                            else if (col.name.indexOf(temp2) >= 0) {
                                col.name = tempName.replace(temp2, (''));
                            }
                            else if (col.name.indexOf(temp3) >= 0) {
                                col.name = tempName.replace(temp3, (''));
                            }

                            if (col.label.indexOf(temp1) >= 0)
                                col.label = tempLabel.replace(temp1, (''));
                            else if (col.label.indexOf(temp2) >= 0)
                                col.label = tempLabel.replace(temp2, (''));
                            else if (col.label && col.label.indexOf(oldLabelTxt) >= 0) {
                                if (tempLabel.replace(oldLabelTxt, '') === '') {
                                    col.label = oldLabelTxt;
                                } else {
                                    col.label = tempLabel.replace(oldLabelTxt, '');
                                }
                            }
                            else
                                col.label = col.name;
                        }
                    }

				});


                $scope.colNameIsTransKey = function() {
                    var column = $scope.column;
                    var isTransKey = (($scope.window.transactionKeyColumn !== null) && (column.name.toLowerCase() === $scope.window.transactionKeyColumn.name.toLowerCase()));
                    return isTransKey;
                };

				$scope.nameValidator = function (value) {

					var column = $scope.column,
						lcName = column && column.name ? column.name.toLowerCase() : null,
						valid = true,
						invalidMessage = "",
						outputColumns = $scope.data.outputColumnMappings,
						foundCol = outputColumns[lcName],
						checkName = vsdUtil.validateDomainObjName(column.name);

					if (column && column.name.length === 0) {
						valid = false; // not setting message; using Dojo's default message for required property
					}
					else if(!checkName.isValid){
						valid = checkName.isValid;
						invalidMessage = resources.app_object_name_specCharValidationMsg_txt;
					}
					else if(outputColumns[lcName] && foundCol !== currCol){
							valid = false;
							invalidMessage = resources.scWindow_columnProperties_nameNotUnique_txt;
					}
                    else if(columnNameValidationSvc.isReserved(value)){
                        valid = false;
                        invalidMessage = dojoModules.string.substitute(resources.scWindow_columnProperties_reservedName_fmt_txt,[value]);
                    }
                    else if($scope.window.anchorSourceColumn && (value.trim().toLowerCase() === $scope.window.anchorSourceColumn.name.trim().toLowerCase()) && $scope.window.type !== vsmCommonConst.WINDOW_TYPE_AGGREGATION) {
                            valid = false;
                            invalidMessage = resources.scWindow_columnProperties_nameNotAnchor_txt;
                    }
                    else { // new column
						if (currCol) { // rename so delete old reference
							delete outputColumns[currCol.name.toLowerCase()];
						}
						currCol = {
							name:           column.name,
							executionOrder: column.executionOrder,
							dataType:       column.dataType,
							columnFormat:   column.columnFormat,
							dataLength:     column.dataLength
						};
						outputColumns[currCol.name.toLowerCase()] = currCol;
					}

					return { isValid: valid, invalidMessage: invalidMessage };
				};

                $scope.expressionIsTransKey = function() {
                    var column = $scope.column;
                    var isTransKey = (($scope.window.transactionKeyColumn !== null) && (column.expression.toLowerCase() === $scope.window.transactionKeyColumn.name.toLowerCase()));
                    return isTransKey;
                };

				$scope.expressionValidator = function(value) {
					var column = $scope.column,
						asnFldParser,
						invalidMessage="",
						newDataType,
						outputColumns = $scope.data.outputColumnMappings,
						outCol;

					if (!value) {
						return {isValid: false}; // not setting message; using Dojo's default message for required property
					}

					// Prohibit string literals
					if(value.indexOf("'") > -1 || value.indexOf('"') > -1){
						return {isValid: false, invalidMessage: resources.scWindow_formula_no_string_literals_txt};
					}

                    //Prohibit expression from containing column_name on non-aggregated windows
                     if ($scope.isNotAggregated) {
                         if (value.toLowerCase().indexOf(column.name.toLowerCase()) !== -1 &&
                         value.trim().length !== column.name.trim().length ) {
                            return {isValid: false, invalidMessage: resources.scWindow_formula_no_column_rename_txt};
                         }
                     }

 					asnFldParser=  fieldParserSvc.fieldParser(column, $scope.data.inputColumnMappings, outputColumns);
					var parseRet = asnFldParser.parseAndAssignExecutionOrder();

					if (parseRet.valid) {

						if(column.parentColumns.length === 1 && column.parentColumns[0].name.toLowerCase() === column.expression.toLowerCase()){
							newDataType = column.parentColumns[0].dataType;
							if(column.dataType !== newDataType && column.aggrFunction !== 'NDISTINCT' && column.aggrFunction !== 'N' && column.aggrFunction !== 'NMISS') {
								column.derived = newDataType === 'NUMERIC';
								column.dataType = newDataType;
								column.dataLength = column.parentColumns[0].dataLength;
								column.columnFormat = newDataType === 'NUMERIC' ?
																vsdUtil.getSASNumericFormat(column.parentColumns[0].columnFormat) :
																column.parentColumns[0].columnFormat;
								column.aggrFunction = null;
								column.condition = null;
							}
						}

						if (column.parentColumns.length) {
							$scope.aggregateMenu = $scope.editPrimitives.aggregValues[column.parentColumns[0].dataType];
						}
						else {
							$scope.aggregateMenu = $scope.editPrimitives.aggregValues.NUMERIC;
						}

						if(column.executionOrder > 1){
							column.aggrFunction = null;
						}
						else if (column.dataType === 'CHARACTER' && column.executionOrder === 1) {
							column.derived = !!column.aggrFunction;
						}

						$scope.isNotAggregated = !$scope.column.aggrFunction;

					}
					else {
						angular.forEach( parseRet.parseErrors, function(err){
							invalidMessage += err.text;
						});
					}

					return { isValid:parseRet.valid, invalidMessage: invalidMessage };
				};

			}],

			link: function(scope, element, attrs) {

				scope.colPropPrimitives.showFilters= !!scope.column.condition;

				scope.deleteClick = function() {
					scope.deleteRowClick(scope.column);
				};

				scope.toggleFilters= function() {
					var defaultOper = _.findWhere(scope.operatorMap, {name: 'AND'});
					var defaultCondOper = _.findWhere(scope.operatorMap, {name: '='});
					// replace null entry in first position with an actual nest operator and empty condition
					scope.column.condition = { operator: defaultOper, displayOrder: 0,
						operand1ColumnInfo: {},  operand2Constant: null,
						childConditions: [ { operator: defaultCondOper,
							                   operand1ColumnInfo: {name: '', label: ''},
							                   operand2Constant: "", childConditions: []} ]
					};
				}

				// if condition changes to a null value then go back to showing "addConditions" button
				scope.$watch('column.condition', function(value) {
					if  (value == null) {
						//scope.column.condition = null;
						scope.colPropPrimitives.showFilters= false;
					}
					else
						scope.colPropPrimitives.showFilters = true;

				});



/*
				scope.expressionValidator =  function(value, row) {
					if(column.expression !== "") {
						var valRet = vsdUtil.valExprConstByDataType(value,scope.condition.name);
						if(!valRet.isValid) {
							this.set('invalidMessage',getNameValidationErrMsg(valRet.reason)); //This is the widget that called the validator
						}
						return valRet.isValid;
					}
					else {
						return true;
					}
				}*/


				function getNameValidationErrMsg(reason) {
					var msg;
					switch(reason) {
						case  vsmCommonConst.EXPR_VAL_REASON_INVALID_DATE_CONST:
							msg = scope.resources.scenario_conditionEditor_invalidDateFormatErrMsg_txt;
							break;
						case vsmCommonConst.EXPR_VAL_REASON_NON_NUMERIC:
							msg = scope.resources.scenario_conditionEditor_nonNumericContentErrMsg_txt;
							break;
						case vsmCommonConst.EXPR_VAL_REASON_NULL:
							msg = scope.resources.scenario_conditionEditor_nullErrMsg_text;
							break;
					}
					return msg;
				}


				if(scope.column.scrollIntoView) {
					var top = element.closest('.dgrid-scroller').scrollTop() + element.position().top;
					element.closest('.dgrid-scroller').animate({scrollTop: top }, {queue:false});
					scope.column.scrollIntoView = false;
				}
			}
		};
	}]);

});

},
'vsd/window/columnsEditor/WindowEditCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.controller('WindowEditCtrl', function ($scope, vsdUtil, windowSvc, dojoModules) {

		var resources = dojoModules.resources;
		function createUniqueColName(name) {
			return windowSvc.getUniqueColumnName($scope.editPrimitives.columnProperties, name);
		}
		function createUniqueColLabel(label) {
			return windowSvc.getUniqueColumnLabel($scope.editPrimitives.columnProperties, label);
		}


		//region *** Scoped Vars  ***
		$scope.editPrimitives = {
			dirtyFlag:          false,
			columnProperties:   [],
			defaultAggregation: {CHARACTER: 'UPP', NUMERIC: !$scope.window.defaultNumericAggregation ? 'UPP' : $scope.window.defaultNumericAggregation },
			aggregValues:       {NUMERIC: [
				{value: null, label: resources.scWindow_AggBuilder_functions_UPP_txt},
                {value: 'SUM', label: resources.scWindow_AggBuilder_functions_SUM_txt, suffix: resources.scWindow_aggrFunction_sum_suffix},
				{value: 'MEAN', label: resources.scWindow_AggBuilder_functions_MEAN_txt, suffix: resources.scWindow_aggrFunction_mean_suffix},
                {value: 'STD', label: resources.scWindow_AggBuilder_functions_STD_txt, suffix: resources.scWindow_aggrFunction_std_suffix},
				{value: 'N', label: resources.scWindow_AggBuilder_functions_COUNT_txt, suffix: resources.scWindow_aggrFunction_count_suffix},
                {value: 'NDISTINCT', label: resources.scWindow_AggBuilder_functions_NDISTINCT_txt, suffix: resources.scWindow_aggrFunction_ndistinct_suffix},
                {value: 'NMISS', label: resources.scWindow_AggBuilder_functions_NMISS_txt, suffix: resources.scWindow_aggrFunction_nmiss_suffix},
				{value: 'MAX', label: resources.scWindow_AggBuilder_functions_MAX_txt, suffix: resources.scWindow_aggrFunction_max_suffix},
				{value: 'MIN', label: resources.scWindow_AggBuilder_functions_MIN_txt, suffix: resources.scWindow_aggrFunction_min_suffix},
                {value: 'MODE', label: resources.scWindow_AggBuilder_functions_MODE_txt, suffix: resources.scWindow_aggrFunction_mode_suffix}
			    ],

				CHARACTER: [
					{value: null, label: resources.scWindow_AggBuilder_functions_UPP_txt},
                    {value: 'N', label: resources.scWindow_AggBuilder_functions_COUNT_txt, suffix: resources.scWindow_aggrFunction_count_suffix},
                    {value: 'NDISTINCT', label: resources.scWindow_AggBuilder_functions_NDISTINCT_txt, suffix: resources.scWindow_aggrFunction_ndistinct_suffix},
                    {value: 'NMISS', label: resources.scWindow_AggBuilder_functions_NMISS_txt, suffix: resources.scWindow_aggrFunction_nmiss_suffix},
					{value: 'MAX', label: resources.scWindow_AggBuilder_functions_MAX_txt, suffix: resources.scWindow_aggrFunction_max_suffix},
					{value: 'MIN', label: resources.scWindow_AggBuilder_functions_MIN_txt, suffix: resources.scWindow_aggrFunction_min_suffix},
                    {value: 'MODE', label: resources.scWindow_AggBuilder_functions_MODE_txt, suffix: resources.scWindow_aggrFunction_mode_suffix}
				]},
			operatorMap:        $scope.data.operatorMap
		};
		var defaultCharacterAggrSize = $scope.editPrimitives.aggregValues.CHARACTER.length;
		$scope.data.outputColumnMappings = {};

        //First and Last need anchorSourceColumn to exist.
		$scope.$watch('window.anchorSourceColumn', function(newVal){
			if (newVal && $scope.editPrimitives.aggregValues.CHARACTER.length === defaultCharacterAggrSize) {
				$scope.editPrimitives.aggregValues.CHARACTER.push(
					{value: 'FIRST', label:resources.scWindow_AggBuilder_functions_FIRST_txt, suffix: resources.scWindow_aggrFunction_first_suffix},
					{value: 'LAST', label:resources.scWindow_AggBuilder_functions_LAST_txt, suffix: resources.scWindow_aggrFunction_last_suffix} );

                $scope.editPrimitives.aggregValues.NUMERIC.push(
                    {value: 'FIRST', label: resources.scWindow_AggBuilder_functions_FIRST_txt, suffix: resources.scWindow_aggrFunction_first_suffix},
                    {value: 'LAST', label: resources.scWindow_AggBuilder_functions_LAST_txt, suffix: resources.scWindow_aggrFunction_last_suffix} );

            }
			else if(!newVal){
				$scope.editPrimitives.aggregValues.CHARACTER.splice(defaultCharacterAggrSize,2);
			}
		});

		//endregion

            //region *** ViewModel Methods ***

		$scope.$watch('data.windowState', function (value) {
			if (value === 'EDIT') {
				$scope.editPrimitives.columnProperties = [];
				$scope.data.outputColumnMappings = {};
				angular.forEach($scope.window.outputColumns, function(col,idx){
					var colCopy = angular.copy(col);
					if(colCopy.dataType === 'NUMERIC'){
						colCopy.columnFormat = vsdUtil.getSASNumericFormat(colCopy.columnFormat);
					}
					$scope.data.outputColumnMappings[col.name.toLowerCase()] = {name: col.name, executionOrder: col.executionOrder, dataType: col.dataType, dataLength: col.dataLength, columnFormat: col.columnFormat};
					$scope.editPrimitives.columnProperties[idx] = colCopy;
				});
			}
		});


		$scope.newColumn = function () {
			var newcol = $scope.assignNewColumnDefaults({
				name:     '',
				label:    '',
				derived:  true,
				dataType: 'NUMERIC',
				dataLength: 8
			});

			newcol.scrollIntoView = true;
			$scope.editPrimitives.columnProperties.push(newcol);
		};

		$scope.deleteColumn = function () {
			deleteColumnProperty($scope.selectedRowColumn);
		};

		$scope.reCalcPreview = function () {
			if (dojoModules.registry.byId('editColumnsForm').validate()) {
				$scope.dirty();
				$scope.window.outputColumns = angular.copy($scope.editPrimitives.columnProperties);
				$scope.toggleWindowState();
				if($scope.window.outputColumns.length > 1)
					$scope.runWindow();
				else
					$scope.refreshPreview();
			}
		};


		$scope.cancelChanges = function () {
			$scope.editPrimitives.columnProperties = null;
			$scope.toggleWindowState();
		};

		$scope.columnSelect = function (rowIndex) {
			$scope.selectedRow = rowIndex;
			$scope.selectedRowColumn = $scope.editPrimitives.columnProperties[rowIndex];
		};


		$scope.onColumnDrop = function (source, nodes, copy, target) {
			angular.forEach(nodes, function (node) {
				var newcol = $scope.assignNewColumnDefaults(node.custNodeData);

				newcol.name = createUniqueColName(newcol.name);
				newcol.label = createUniqueColLabel(newcol.label);


				newcol.scrollIntoView = true;
				$scope.editPrimitives.columnProperties.push(newcol);
			});
		};

		$scope.duplicateClick = function (column) {
			var copyColumn = copyOutputColumn(column);
			copyColumn.usedInScenario = false;
			var index = $scope.editPrimitives.columnProperties.indexOf(column);
			$scope.editPrimitives.columnProperties.splice(index + 1, 0, copyColumn);
		};



		$scope.deleteRowClick = function (column) {
			deleteColumnProperty(column);
		};
		//endregion

		//region *** Private Functions ***
		function deleteColumnProperty(column){
			var index = $scope.editPrimitives.columnProperties.indexOf(column);
			if(index > -1){
				$scope.editPrimitives.columnProperties.splice(index, 1);
				delete $scope.data.outputColumnMappings[column.name.toLowerCase()];
				$scope.selectedRowColumn = null;
				$scope.selectedRow = -1;
			}

		}

		function copyOutputColumn(column){
			var copyColumn = {};
			angular.copy(column, copyColumn);
			// beware of new columns that have no name or label
			copyColumn.name = column.name ? createUniqueColName(copyColumn.name) : column.name;
			copyColumn.label = column.label ? createUniqueColLabel(copyColumn.label) : column.label;
			copyColumn.bins = [];
			copyColumn.columnOutputColumns = null;
			copyColumn.childOuputColumns = null;
			copyColumn.parentColumns = null;
			copyColumn.parentOutputColumnInfo = null;

			vsdUtil.clearIds(copyColumn, ['operator', 'operand1ColumnInfo', 'operand2ColumnInfo']);

			return copyColumn;
		}

		//endregion


	});


});

},
'vsd/window/new/NewWindowCtrl':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd').
    controller('NewWindowCtrl', ['$scope', '$location', '$route', '$filter', '$q', 'windowSvc', 'newWindowSvc', 'dojoModules', 'vsdUtil', 'configSvc', 'Util',
        function ($scope, $location, $route, $filter, $q, windowSvc, newWindowSvc, dojoModules, vsdUtil, configSvc, Util) {

            var earliestPossibleDate = moment().subtract(100, 'years').toDate();
            var latestPossibleDate = moment().add(100, 'years').toDate();

            var initialWindow = {
                name: '',
                desc: '',
                needsRerun: false,
                entityTimeUnit: null,
                inputTable: {columns: []},
                anchorSourceColumn: null,
                anchorEnd: undefined,
                anchorStart: undefined,
                transactionKeyColumn: null,
                outputColumns: [],
                windowInputs: [],
                columnLookbackPeriods: null, //Object with one or more columnName/period pairs
                scenarios: [],
                type: vsmCommonConst.WINDOW_TYPE_AGGREGATION,
                startDate: null,
                minStartDate: null,
                maxStartDate: null,
                includeMissing: true,
                duration: null,
                interval: 1,
                partitionKeyColumns: [],
                defaultNumericAggregation: {
                    value: 'SUM',
                    label: $scope.resources.scWindow_AggBuilder_functions_SUM_txt
                },
                defaultLookBackPeriod: 0,
                outputType: null
            };

            var filterBy = $filter('filter');

            $scope.window = {};
            $scope.validTransactionKeys = {};
            $scope.options = {
                entityKeyTables: [],
                entityKeyColumns: [],
                selectedJoinInputs: [],
                partitionKeys: [],
                selectedCandidatePartitionKeyNames: [],  //The sng-multi-select model; those that are highlighted
                candidatePartitionKeys: [],
                selectedPartitionKeyNames: [],
                windowInputs: [],
                joinInputs: [],
                endDate: null,
                hasPartitionKeys: false,
                duration: null,

                //These are the minimum and maximum *possible* start and end dates
                minPossibleAnchorStartDate: earliestPossibleDate,
                maxPossibleAnchorStartDate: latestPossibleDate,
                minPossibleAnchorEndDate: earliestPossibleDate,
                maxPossibleAnchorEndDate: latestPossibleDate,

                //These are the minimum and maximum *selectable* start and end dates,
                //given the currently selected dates
                minSelectableAnchorStartDate: null,
                maxSelectableAnchorStartDate: null,
                minSelectableAnchorEndDate: null,
                maxSelectableAnchorEndDate: null,

                //doherk: These are bound to a Dojo NumberSpinner to constrain the duration values
                durationConstraints: {
                    min: 1
                },

                lookbackPeriodConstraints: {
                    min: 0,
                    max: 2000000000
                },

                //These are the dates that are currently selected
                anchorStartDate: null,
                anchorEndDate: null,

                tableMaxDate: null,
                tableMaxTime: null,
                tableMinDate: null,
                tableMinTime: null,

                timeUnitRequired: false,
                selectedWindowType: null,
                includeMissing: true,
                outputLevels: [
                    {
                        label: $scope.resources.windowOutputLevelGroup_window_txt,
                        value: vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUP
                    },
                    {
                        label: $scope.resources.windowOutputLevelGroupDate_window_txt,
                        value: vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUPDATE
                    },
                    {
                        label: $scope.resources.windowOutputLevelTransaction_window_txt,
                        value: vsmCommonConst.WINDOW_OUTPUTLEVEL_TRANSACTION
                    }
                ],

                nonDateBasedNumericAggregationTypes: [
                    {value: null, label: $scope.resources.scWindow_AggBuilder_functions_UPP_txt},
                    {value: 'SUM', label: $scope.resources.scWindow_AggBuilder_functions_SUM_txt},
                    {value: 'MEAN', label: $scope.resources.scWindow_AggBuilder_functions_MEAN_txt},
                    {value: 'STD', label: $scope.resources.scWindow_AggBuilder_functions_STD_txt},
                    {value: 'N', label: $scope.resources.scWindow_AggBuilder_functions_COUNT_txt},
                    {value: 'NDISTINCT', label: $scope.resources.scWindow_AggBuilder_functions_NDISTINCT_txt},
                    {value: 'NMISS', label: $scope.resources.scWindow_AggBuilder_functions_NMISS_txt},
                    {value: 'MAX', label: $scope.resources.scWindow_AggBuilder_functions_MAX_txt},
                    {value: 'MIN', label: $scope.resources.scWindow_AggBuilder_functions_MIN_txt},
                    {value: 'MODE', label: $scope.resources.scWindow_AggBuilder_functions_MODE_txt}
                ],
                dateBasedNumericAggregationTypes: [
                    {value: 'FIRST', label: $scope.resources.scWindow_AggBuilder_functions_FIRST_txt},
                    {value: 'LAST', label: $scope.resources.scWindow_AggBuilder_functions_LAST_txt}
                ],
                defaultNumericAggregationTypes: [],
                defaultNumericAggregation: {
                    value: 'SUM',
                    label: $scope.resources.scWindow_AggBuilder_functions_SUM_txt
                },
                defaultLookBackPeriod: 0,
                outputLevel: {
                    label: $scope.resources.windowOutputLevelGroup_window_txt,
                    value: vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUP
                },
                transactionKey: null
            };

            // This structure is a "side-model" to the sngDateTextBoxes used in the window screen.  This means that
            // this structure is kept "off to the side" of the model used by the directive due to its truncation
            // of time granularity.  The need for this workaround will be moot when we use another component/directive
            // that does not do this truncation.
            //
            // The *real* model for this controller is $scope.window.anchorStart and
            // $scope.window.anchorEnd.  Those properties are kept updated via the sngDateTextBox's sng-change
            // invocation of anchorStartDateTimeChanged() and anchorEndDateTimeChanged().
            $scope.anchorDateTimeViews = {
                viewAnchorStartDateTime: null,  //The sngDateTextBox model
                oldAnchorStartDateTime: null,   //The old value of the window's start date/time
                viewAnchorEndDateTime: null,    //The sngDateTextBox model
                oldAnchorEndDateTime: null      //The old value of the window's end date/time
            };

            $scope.data = {
                entityKeyTable: null,
                entityKeyColumn: null,
                createEnabled: false,
                nextEnabled: false
            };

            $scope.invalidWindowNameMsg = null;
            $scope.invalidTransactionKeyColumnMsg = null;

            $scope.currentWizardIdx = 0;

            $scope.enableWizardControls = enableWizardControls;
            $scope.validate = validate;
            $scope.filterEntityTimeUnitsAndUpdateDates = filterEntityTimeUnitsAndUpdateDates;
            $scope.resetForm = resetForm;
            $scope.updateStartDate = updateStartDate;
            $scope.updateDurationAndEnableWizardControls = updateDurationAndEnableWizardControls;
            $scope.updateValidDateColumns = updateValidDateColumns;
            $scope.validateDefaultLookbackPeriodAndDuration = validateDefaultLookbackPeriodAndDuration;
            $scope.validateAnchorEndDate = validateAnchorEndDate;

            var checkDisableGroupAndDate = function () {
                //see if we need to remove 'Group and date' output level
                if (configSvc.getBoolOption(vsmCommonConst.CONFIG_OPTION_WINDOW_OUTPUT_LEVEL_DISABLE_GROUP_AND_DATE)) {
                    $scope.options.outputLevels = _.reject($scope.options.outputLevels, function (level) {
                        return level.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUPDATE;
                    });
                }
            };

            function resetForm() {
                checkDisableGroupAndDate();
                $scope.invalidWindowNameMsg = null;
                $scope.invalidTransactionKeyColumnMsg = null;
                $scope.currentWizardIdx = 0;
                $scope.window = angular.copy(initialWindow);
                $scope.options.selectedWindowType = null;
                $scope.window.type = null;
                clearOptionsDatesAndDuration();
                $scope.window.interval = 1;
                $scope.options.entityKeyTables = [];
                $scope.options.selectedCandidatePartitionKeyNames = [];
                $scope.options.candidatePartitionKeys = [];
                $scope.options.selectedPartitionKeyNames = [];
                $scope.options.defaultNumericAggregation = {
                    value: 'SUM',
                    label: $scope.resources.scWindow_AggBuilder_functions_SUM_txt
                };
                $scope.options.defaultLookBackPeriod = 0;
                $scope.options.hasPartitionKeys = false;
                $scope.options.outputLevel = {
                    label: $scope.resources.windowOutputLevelGroup_window_txt,
                    value: vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUP
                };
                $scope.options.transactionKey = null;
                $scope.validTransactionKeys = {};
                $scope.invalidTransactionKeyColumnMsg = null;
                $scope.options.includeMissing = true;
            }

            function loadLasrTables() {
                $scope.showLoadingMask($scope.resources.scWindow_NewWindowDlg_lasrTableLoadingMsg);
                return windowSvc.getWindowInputs().then(function (windowInputs) {
                        if (!windowInputs.length) {
                            $scope.showErrorMessage($scope.resources.scWindow_window_inputs_none_found_txt);
                        }
                        var preString = '<div class="tableWinSeparator"><div class="cell"><hr class="lines"></div><div class="cell mid">',
                            postString = '</div><div class="cell"><hr class="lines"></div></div>';

                        $scope.options.windowInputs = [];
                        $scope.options.windowInputs.push({
                            id: -1,
                            name: ' ',
                            htmlLabel: preString + $scope.resources.scWindow_NewWindowDlg_tables_txt + postString
                        });
                        var firstWindow = true;
                        angular.forEach(windowInputs, function (input) {
                            var table = angular.copy(input);
                            table.htmlLabel = table.name;
                            if (table.type === 'window' && firstWindow) {
                                $scope.options.windowInputs.push({
                                    id: -2,
                                    name: ' ',
                                    htmlLabel: preString + $scope.resources.scWindow_NewWindowDlg_windows_txt + postString
                                });
                                firstWindow = false;
                            }
                            $scope.options.windowInputs.push(table);
                        });

                        $scope.hideLoadingMask();
                    }
                );
            }

            function updateValidDateColumns() {
                //LASR fails if partition key is used as date column
                $scope.options.dateColumns = _.reject($scope.options.allDateColumns, function (col) {
                    return _.find($scope.options.partitionKeys, function (partitionKey) {
                        return partitionKey.name === col.name;
                    });
                });
            }

            function updateValidTransactionKeyColumns() {
                if (!$scope.options.selectedPrimaryInput) {
                    return;
                }

                // Filter partitionKey columns
                var temp = _.reject($scope.options.selectedPrimaryInput.table.columns, function (col) {
                    return _.find($scope.options.partitionKeys, function (partitionKey) {
                        return partitionKey.name === col.name;
                    });
                });

                // Filter date columns
                $scope.options.transactionKeyOptions = _.reject(temp, function (col) {
                    return _.find($scope.options.allDateColumns, function (dateColumns) {
                        return dateColumns.name === col.name;
                    });
                });
            }

            function validateDefaultLookbackPeriodAndDuration() {

                // Flush any previous selection for transaction key if outputLevel changes
                if(!numberSpinnerHasFocus()) {
                    $scope.options.transactionKey = null;
                    $scope.invalidTransactionKeyColumnMsg = null;
                    $scope.window.transactionKey = null;
                }

                var timeUnitSelector = dojoModules.registry.byId("transactionTimeUnitSelector");
                if ($scope.options.outputLevel && ($scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUPDATE ||
                    $scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_TRANSACTION)) {

                    //For Sliding, lookbackPeriod cannot exceed 32766
                    //For Rolling, lookbackPeriod must be between 0 - 1893455999
                    if ($scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUPDATE) {
                        if ($scope.options.defaultLookBackPeriod > 32766) {
                            $scope.options.defaultLookBackPeriod = 32766;
                        }
                    }

                    if ($scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_TRANSACTION) {
                        if ($scope.window.entityTimeUnit === "SECONDS") {
                            if ($scope.options.defaultLookBackPeriod > 1893455999) {
                                $scope.options.defaultLookBackPeriod = 1893455999;
                            }
                        }
                        if ($scope.window.entityTimeUnit === "MINUTES") {
                            if ($scope.options.defaultLookBackPeriod > 31557599) {
                                $scope.options.defaultLookBackPeriod = 31557599;
                            }
                        }
                        if ($scope.window.entityTimeUnit === "HOURS") {
                            if ($scope.options.defaultLookBackPeriod > 525959) {
                                $scope.options.defaultLookBackPeriod = 525959;
                            }
                        }
                        if ($scope.window.entityTimeUnit === "DAYS") {
                            if ($scope.options.defaultLookBackPeriod > 21914) {
                                $scope.options.defaultLookBackPeriod = 21914;
                            }
                        }
                        if ($scope.options.defaultLookBackPeriod < 0) {
                            $scope.options.defaultLookBackPeriod = 0;
                        }
                    }

                    //For Sliding and Rolling, duration must be > 1
                    if ($scope.options.duration < 1) {
                        //Determine what time units can be supported by input table
                        var endDateTime = moment($scope.options.anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).add($scope.options.anchorEndTime, 'milliseconds').toDate();
                        var startDate = moment($scope.options.minPossibleAnchorStartDate).clone().toDate();

                        if (endDateTime && startDate) {
                            if (specifiedDateTime()) {
                                if (((moment(endDateTime).clone().milliseconds(999) - moment(startDate).clone().milliseconds(0)) / 1000) > 1) {
                                    var seconds_valid = true;
                                }
                                if (((moment(endDateTime).clone().seconds(59).milliseconds(999) - moment(startDate).clone().seconds(0).milliseconds(0)) / 60000) > 1) {
                                    var minutes_valid = true;
                                }
                                if (((moment(endDateTime).clone().minutes(59).seconds(59).milliseconds(999) - moment(startDate).clone().minutes(0).seconds(0).milliseconds(0)) / 3600000) > 1) {
                                    var hours_valid = true;
                                }
                            }
                            if (((moment(endDateTime).clone().hours(23).minutes(59).seconds(59).milliseconds(999) - moment(startDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)) / 86400000) > 1) {
                                var days_valid = true;
                            }
                        }

                        //Walk through valid entityTimeUnits
                        if (days_valid) {
                            $scope.window.entityTimeUnit = _.find($scope.options.entityTimeUnits, function (entityTimeUnits) {
                                return entityTimeUnits.granularityName === 'DAYS';
                            });
                        } else if (hours_valid) {
                            $scope.window.entityTimeUnit = _.find($scope.options.entityTimeUnits, function (entityTimeUnits) {
                                return entityTimeUnits.granularityName === 'HOURS';
                            });
                        } else if (minutes_valid) {
                            $scope.window.entityTimeUnit = _.find($scope.options.entityTimeUnits, function (entityTimeUnits) {
                                return entityTimeUnits.granularityName === 'MINUTES';
                            });
                        } else if (seconds_valid) {
                            $scope.window.entityTimeUnit = _.find($scope.options.entityTimeUnits, function (entityTimeUnits) {
                                return entityTimeUnits.granularityName === 'SECONDS';
                            });
                        } else {
                            timeUnitSelector.set("state", "Error");
                            timeUnitSelector.set("invalidMessage", $scope.resources.scWindow_NewWindowDlg_validEntityTimeUnitLbl_txt);
                            timeUnitSelector.set("displayedValue", getTimeUnits().toUpperCase() + ' ');  //Appended space is needed for validation to function correctly
                        }
                    }
                } else { //Not group and date or transaction
                    timeUnitSelector.set("state", "");
                    timeUnitSelector.set("invalidMessage", "");
                }
            }

            function getTimeUnits() {
                //Use "days" as the time unit by default
                return $scope.window.entityTimeUnit ? $scope.window.entityTimeUnit.granularityUnit.toLowerCase() : $scope.resources.timeUnits_2_txt.toLowerCase();
            }

            function updateDuration(whatChanged) {

                //if there is no anchorDate or if the active widget (the one the user has just changed) is the
                //durationNumberSpinner, bail
                if (!$scope.options.anchorEndDate || numberSpinnerHasFocus() ) {
                    return;
                }

                var anchorEndDate = $scope.options.anchorEndDate;

                if (moment($scope.options.anchorEndDate).clone().startOf('day').isSame(moment($scope.options.tableMaxDate).clone().startOf('day'))) {
                    if (getTimeUnits() === 'days') {
                        anchorEndDate = moment($scope.options.anchorEndDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999).toDate();
                    } else if (getTimeUnits() === 'hours') {
                        anchorEndDate = moment($scope.options.anchorEndDate).clone().minutes(59).seconds(59).milliseconds(999).toDate();
                    } else if (getTimeUnits() === 'minutes') {
                        anchorEndDate = moment($scope.options.anchorEndDate).clone().seconds(59).milliseconds(999).toDate();
                    } else if (getTimeUnits() === 'seconds') {
                        anchorEndDate = moment($scope.options.anchorEndDate).clone().milliseconds(999).toDate();
                    }
                } else {
                    anchorEndDate = moment($scope.options.anchorEndDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999).toDate();
                }

                if (whatChanged === 'changedTimeUnit') {
                    // If user changed time unit, reset the anchorEndDate to maximum value on the table
                    if (getTimeUnits() === 'days') {
                        anchorEndDate = moment($scope.options.tableMaxDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999).toDate();
                    } else if (getTimeUnits() === 'hours') {
                        anchorEndDate = moment($scope.options.tableMaxDate).clone().minutes(59).seconds(59).milliseconds(999).toDate();
                    } else if (getTimeUnits() === 'minutes') {
                        anchorEndDate = moment($scope.options.tableMaxDate).clone().seconds(59).milliseconds(999).toDate();
                    } else if (getTimeUnits() === 'seconds') {
                        anchorEndDate = moment($scope.options.tableMaxDate).clone().milliseconds(999).toDate();
                    }
                }

//                $scope.options.anchorEndDate = moment(anchorEndDate).clone();
                setEndDateValue(new Date(anchorEndDate));
                setDisplayedEndTime(moment(moment(anchorEndDate).clone() - moment(anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)).utc().format('HH:mm:ss.SSS'));
                setOptionsEndDateTimeValue(moment.duration(moment(anchorEndDate).clone() - moment(anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)).asMilliseconds());

                var anchorStartDate;
                if (whatChanged === 'changedTimeUnit') {
                    anchorStartDate = moment($scope.options.minPossibleAnchorStartDate).clone().toDate();
                } else {
                    anchorStartDate = moment($scope.options.anchorStartDate).clone().toDate();
                }
                setOptionsStartDateValue(moment(anchorStartDate).clone().toDate());
                setOptionsStartDateTimeValue( moment.duration(moment(anchorStartDate).clone() - moment(anchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)).asMilliseconds());


                var duration;

                if (anchorEndDate && anchorStartDate) {
                    if (getTimeUnits() === 'seconds') {
                        duration = (moment(anchorEndDate).clone().milliseconds(999) - moment(anchorStartDate).clone().milliseconds(0)) / 1000;
                    } else if (getTimeUnits() === 'minutes') {
                        duration = (moment(anchorEndDate).clone().seconds(59).milliseconds(999) - moment(anchorStartDate).clone().seconds(0).milliseconds(0)) / 60000;
                    } else if (getTimeUnits() === 'hours') {
                        duration = (moment(anchorEndDate).clone().minutes(59).seconds(59).milliseconds(999) - moment(anchorStartDate).clone().minutes(0).seconds(0).milliseconds(0)) / 3600000;
                    } else if (getTimeUnits() === 'days') {
                        duration = (moment(anchorEndDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999) - moment(anchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)) / 86400000;
                    }
                }

                if (getTimeUnits() !== 'seconds') {
                    if (duration && Math.floor(duration) !== $scope.options.duration) {
                        $scope.options.duration = Math.floor(duration);
                    }
                } else {
                    if (duration && Math.ceil(duration) !== $scope.options.duration) {
                        $scope.options.duration = Math.ceil(duration);
                    }
                }

                updateDurationRange();

                if (whatChanged !== 'changedStartDate') {
                    updateStartDate();
                }
                updateSelectableDates();

                if ($scope.options.anchorEndTime) {
                    setStartDateValue(moment($scope.options.anchorStartDate).clone().toDate());
                    setDisplayedStartTime(moment($scope.options.anchorStartTime).clone().utc().format("HH:mm:ss.SSS"));
                    setDisplayedEndTime( moment(moment(anchorEndDate).clone() - moment(anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)).utc().format('HH:mm:ss.SSS'));
                }
            }

            function updateDurationAndEnableWizardControls(whatChanged) {
                updateDuration(whatChanged);
                updateSelectableDates();
                enableWizardControls();
            }

            function updateStartDate() {
                if ( numberSpinnerHasFocus() ) {

                    if (!$scope.options.anchorEndDate) {
                        return;
                    }

                    var anchorEndDate = moment($scope.options.anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).add($scope.options.anchorEndTime, 'milliseconds').toDate();
                    var minAnchorStartDate = moment($scope.options.minPossibleAnchorStartDate).clone().toDate();
                    var earliestPossibleStartDate = new Date(minAnchorStartDate.getTime());
                    var anchorEndMinusDuration = new Date(minAnchorStartDate.getTime());
                    if (minAnchorStartDate && anchorEndDate) {
                        //The anchor start date is the maximum of the earliest anchor date possible, or
                        //the computed start date = (anchor end date minus the duration)
                        if ($scope.options.duration > 0) {
                            if (getTimeUnits() === 'seconds') {
                                anchorEndMinusDuration = moment(anchorEndDate).clone().milliseconds(999).subtract($scope.options.duration, getTimeUnits()).add('1', 'milliseconds').toDate();
                                earliestPossibleStartDate = moment(minAnchorStartDate).clone().milliseconds(0).toDate();
                            } else if (getTimeUnits() === 'minutes') {
                                anchorEndMinusDuration = moment(anchorEndDate).clone().seconds(59).milliseconds(999).subtract($scope.options.duration + 1, getTimeUnits()).add('1', 'milliseconds').toDate();
                                earliestPossibleStartDate = moment(minAnchorStartDate).clone().seconds(0).milliseconds(0).toDate();
                            } else if (getTimeUnits() === 'hours') {
                                anchorEndMinusDuration = moment(anchorEndDate).clone().minutes(59).seconds(59).milliseconds(999).subtract($scope.options.duration + 1, getTimeUnits()).add('1', 'milliseconds').toDate();
                                earliestPossibleStartDate = moment(minAnchorStartDate).clone().minutes(0).seconds(0).milliseconds(0).toDate();
                            } else if (getTimeUnits() === 'days') {
                                anchorEndMinusDuration = moment(anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).subtract($scope.options.duration, getTimeUnits()).toDate();
                                earliestPossibleStartDate = moment(minAnchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).toDate();
                            }
                        } else {
                            anchorEndDate = minAnchorStartDate;
                        }

                        var maxDate = moment.max(moment(earliestPossibleStartDate).clone(), moment(anchorEndMinusDuration).clone()).toDate();
                        setStartDateValue(new Date(maxDate));
                        setOptionsStartDateValue(new Date(maxDate));
                        setOptionsStartDateTimeValue(moment.duration(moment(maxDate).clone() - moment(maxDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)).asMilliseconds());
                        setDisplayedStartTime(moment(moment(maxDate).clone() - moment(maxDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)).utc().format('HH:mm:ss.SSS'));
                    }

                    updateSelectableDates();
                    updateDurationRange();
                    validateDefaultLookbackPeriodAndDuration();

                }
            }

            function enableWizardControls() {
                enableNext();
                enableDateAndAggregationOptions();
                enableCreate();
            }

            function enableNext() {
                if ($scope.currentWizardIdx !== 0) {
                    $scope.data.nextEnabled = false;
                } else {
                    $scope.data.nextEnabled = $scope.window.name && $scope.options.selectedPrimaryInput;
                }
            }

            function hasValidTransactionKeyIfRolling() {
                return ($scope.isRolling() ? ($scope.options.transactionKey && $scope.validTransactionKeys[$scope.options.transactionKey.name]) : true);
            }

            function enableCreate() {
                $scope.data.createEnabled = $scope.window.name && $scope.options.selectedPrimaryInput && ($scope.options.candidatePartitionKeys.length !== 0) &&
                hasValidTransactionKeyIfRolling();
            }

            function enableDateAndAggregationOptions() {
                $scope.options.hasPartitionKeys = $scope.options.partitionKeys.length > 0;
            }


            $scope.isSliding = function () {
                return ($scope.options.outputLevel &&
                ($scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUPDATE || $scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_TRANSACTION));
            };

            $scope.isRolling = function () {
                return ($scope.options.outputLevel &&
                ($scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_TRANSACTION));
            };

            $scope.isGroupAndDate = function () {
                return ($scope.options.outputLevel &&
                ($scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUPDATE));
            };

            $scope.validateName = function (name) {
                var result = vsdUtil.validateDomainObjName(name, []);
                if (!result.isValid) {
                    result.invalidMessage = $scope.resources.app_object_name_specCharValidationMsg_txt;
                }
                return result;

            };


            $scope.addJoinInput = function () {
                $scope.options.selectedJoinInputs.push({});
                $scope.options.joinInputs.push(filterJoinInputs($scope.options.joinInputs.length));
                $('#joinWrapper').animate({scrollTop: $('#joinInputsBottom').offset().top}, 500);

                enableWizardControls();
            };

            $scope.removeJoinInput = function (index) {
                $scope.options.selectedJoinInputs.splice(index, 1);
                $scope.options.joinInputs.splice(index, 1);
                for (var i = 0; i < $scope.options.joinInputs.length; i++) {
                    $scope.options.joinInputs[i] = filterJoinInputs(i);
                }
                $scope.options.entityKeyTables.splice(1);
                angular.forEach($scope.options.selectedJoinInputs, function (input, idx) {
                    if (input.dimension) {
                        input.dimension.index = idx;
                        input.dimension.alias = 't' + $scope.options.entityKeyTables.length + '_';
                        $scope.options.entityKeyTables.push(input.dimension);
                    }
                });
                enableWizardControls();
            };

            $scope.updateJoinInput = function (index) {
                if (!$scope.options.selectedJoinInputs[index].dimension || $scope.options.selectedJoinInputs[index].dimension.id < 0) {
                    $scope.options.entityKeyTables.splice(1);
                    $scope.data.entityKeyTable = $scope.options.entityKeyTables[0];
                    $scope.options.entityKeyColumns = $scope.data.entityKeyTable.table.columns;
                    return;
                }

                enableWizardControls();
                for (var i = 0; i < $scope.options.joinInputs.length; i++) {
                    $scope.options.joinInputs[i] = filterJoinInputs(i);
                }
                var dim = $scope.options.selectedJoinInputs[index].dimension;

                if (dim && !dim.table) {
                    $scope.showLoadingMask($scope.resources.scWindow_NewWindowDlg_lasrColLoadingMsg);

                    windowSvc.getTable(dim.tableId).then(function (table) {
                        dim.table = table;
                        $scope.hideLoadingMask();
                    });
                }

                $scope.options.entityKeyTables.splice(1);
                angular.forEach($scope.options.selectedJoinInputs, function (input, idx) {
                    if (input.dimension) {
                        input.dimension.index = idx;
                        input.dimension.alias = 't' + $scope.options.entityKeyTables.length + '_';
                        $scope.options.entityKeyTables.push(input.dimension);
                    }
                });
            };

            function filterJoinInputs(index) {
                var selectedPrimaryInput = $scope.options.selectedPrimaryInput || {};
                var windowInputs = $scope.options.windowInputs;
                return filterBy(windowInputs, function (windowInput) {
                    if (windowInput.id === selectedPrimaryInput.id) {
                        return false;
                    }
                    for (var i = 0; i < $scope.options.selectedJoinInputs.length; i++) {
                        if (i !== index) {
                            var dim = $scope.options.selectedJoinInputs[i].dimension;
                            if (dim && windowInput.id === dim.id && dim.id >= 0) {
                                return false;
                            }
                        }
                    }
                    return true;
                });
            }

            function clearOptionsDatesAndDuration() {
                $scope.options.minPossibleAnchorStartDate = earliestPossibleDate;
                $scope.options.maxPossibleAnchorStartDate = latestPossibleDate;
                $scope.options.minPossibleAnchorEndDate = earliestPossibleDate;
                $scope.options.maxPossibleAnchorEndDate = latestPossibleDate;

                $scope.options.minSelectableAnchorStartDate = null;
                $scope.options.maxSelectableAnchorStartDate = null;
                $scope.options.minSelectableAnchorEndDate = null;
                $scope.options.maxSelectableAnchorEndDate = null;

                $scope.options.durationConstraints = {
                    min: 1
                };
                setOptionsStartDateValue(null);
                setOptionsEndDateValue(null);

                $scope.options.duration = null;
            }


            $scope.updateColumns = function () {
                if (!$scope.options.selectedPrimaryInput || $scope.options.selectedPrimaryInput.id < 0) {
                    return;
                }

                var i,
                    tableLoaded = {};

                for (i = $scope.options.selectedJoinInputs.length - 1; i >= 0; i--) {
                    if ($scope.options.selectedJoinInputs[i].dimension === $scope.options.selectedPrimaryInput) {
                        $scope.options.selectedJoinInputs.splice(i, 1);
                        $scope.options.joinInputs.splice(i, 1);
                    }
                }

                for (i = 0; i < $scope.options.joinInputs.length; i++) {
                    $scope.options.joinInputs[i] = filterJoinInputs(i);
                }

                $scope.window.needsRerun = false;
                $scope.window.anchorSourceColumn = null;
                $scope.window.entityTimeUnit = null;
                $scope.window.anchorEndDate = null;
                $scope.window.startDate = null;
                $scope.window.endDate = null;
                $scope.window.duration = null;
                $scope.window.outputLevel = null;
                $scope.options.outputLevel = null;
                $scope.options.transactionKey = null;
                $scope.validTransactionKeys = {};
                $scope.invalidTransactionKeyColumnMsg = null;
                $scope.window.type = null;
                if (!$scope.options.selectedPrimaryInput) {
                    return;
                }

                // Load table if not already loaded
                if (!$scope.options.selectedPrimaryInput.table) {
                    $scope.showLoadingMask($scope.resources.scWindow_NewWindowDlg_lasrTableLoadingMsg);

                        tableLoaded = windowSvc.getTable($scope.options.selectedPrimaryInput.tableId).then(function (table) {
                            $scope.options.selectedPrimaryInput.table = table;
                            $scope.hideLoadingMask();
                        });

                }

                $q.when(tableLoaded).then(function () {
                    var columns = $scope.options.selectedPrimaryInput.table.columns;
                    var entityKeyColumnName = $scope.options.selectedPrimaryInput.entityKeyColumnName;
                    $scope.options.dateColumns = filterBy(columns, {anchorDateCandidate: true});
                    $scope.options.allDateColumns = $scope.options.dateColumns;

                    $scope.window.entityKeyColumn = filterBy(columns, function (column) {
                        return (column.partitionKey || column.name === entityKeyColumnName);
                    })[0];

                    $scope.options.entityKeyTables[0] = $scope.options.selectedPrimaryInput;
                    $scope.data.entityKeyTable = $scope.options.selectedPrimaryInput;
                    $scope.options.entityKeyTables[0].index = 0;
                    $scope.options.entityKeyTables[0].alias = '';
                    $scope.options.entityKeyColumns = columns;
                    clearOptionsDatesAndDuration();
                    $scope.options.partitionKeys = [];
                    $scope.options.selectedPartitionKeyNames = [];
                    $scope.options.candidatePartitionKeys = $scope.options.entityKeyColumns.slice();  //shallow copy

                        getWindowTimeUnits().then(function () {
                            enableWizardControls();
                        });
                });
            };

            $scope.loadDistinctValuesAndCreateWindow = function () {

                $scope.showLoadingMask($scope.resources.scWindow_NewWindowDlg_creatingWindowMsg);
                var tableId = $scope.options.selectedPrimaryInput.tableId;
                    //$scope.window.inputTable.id
                $scope.createWindow();
/*                var tableLoaded = windowSvc.getTableWithDistinctCounts($scope.options.selectedPrimaryInput.tableId).then(function (table) {
                    $scope.options.selectedPrimaryInput.table = table;
                    $scope.createWindow();
                });*/
            }

            function validateAnchorEndDate() {

                if ($scope.options.outputLevel && ($scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUP ||
                    $scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUPDATE ||
                    $scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_TRANSACTION)) {

                    var transDateSelector = dojoModules.registry.byId("transactionDateSelector");

                    if ($scope.options.anchorEndDate === null) {
                        setDisplayedEndTime(' ');
                        setOptionsStartDateValue(null);
                        transDateSelector.set("state", "Error");
                        transDateSelector.set("invalidMessage", $scope.resources.scWindow_NewWindowDlg_validTransactionDateLbl_txt);
                        transDateSelector.set("displayedValue", $scope.window.anchorSourceColumn.label + ' ');  //Appended space is needed for validation to function correctly
                    } else {
                        Util.clearError("transactionDateSelector");
                        transDateSelector.set("displayedValue", $scope.window.anchorSourceColumn.label);
                    }
                }
            }

            function filterEntityTimeUnitsAndUpdateDates() {
                setOptionsEndDateValue(null);
                setOptionsStartDateValue(null);
                $scope.options.duration = null;
                $scope.options.minPossibleAnchorEndDate = earliestPossibleDate;
                $scope.options.maxPossibleAnchorEndDate = latestPossibleDate;
                $scope.options.minPossibleAnchorStartDate = earliestPossibleDate;
                $scope.options.maxPossibleAnchorStartDate = latestPossibleDate;

                getWindowTimeUnits();
                if ($scope.options.outputLevel === null) {
                    $scope.options.outputLevel = {
                        label: $scope.resources.windowOutputLevelGroup_window_txt,
                        value: vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUP
                    };
                }
                if ($scope.options.defaultLookBackPeriod === null) {
                    $scope.options.defaultLookBackPeriod = 0;
                }
                enableWizardControls();
            }

            function getWindowTimeUnits() {
                return windowSvc.getTimeUnits().then(function (value) {
                    $scope.options.entityTimeUnits = value;
                    if ($scope.window.anchorSourceColumn) {

                        $scope.options.timeUnitRequired = true;
                        var dayUnitList = filterBy(value, {granularityUnit: "DAYS"});
                        if (!specifiedDateTime()) {
                            $scope.options.entityTimeUnits = dayUnitList;
                        }
                        $scope.window.entityTimeUnit = dayUnitList[0];
                        getInputTableDateRange();
                    }
                    else {
                        $scope.options.timeUnitRequired = false;
                    }
                });
            }

            function initializePossibleDatesAndDuration(minPossibleStartDate, maxPossibleEndDate) {
                if (!minPossibleStartDate || !maxPossibleEndDate) {
                    return;
                }

                //doherk: These dates should be immutable, so clone them from the inputs
                $scope.options.minPossibleAnchorStartDate = moment(minPossibleStartDate).clone().milliseconds(0).toDate();
                $scope.options.maxPossibleAnchorStartDate = moment(maxPossibleEndDate).clone().milliseconds(0).subtract('1', 'seconds').toDate();

                $scope.options.maxPossibleAnchorEndDate = moment(maxPossibleEndDate).clone().milliseconds(999).toDate();
                $scope.options.minPossibleAnchorEndDate = moment(minPossibleStartDate).clone().milliseconds(999).add('1', 'seconds').toDate();

            }

            function updateSelectableDates() {
                //doherk: The use of clone() in the following statements is because Moment.js add() and subtract() methods mutate the original
                var newStartDate;
                var newEndDate;

                if (!$scope.options.anchorEndDate) {
                    return;
                }

                if (getTimeUnits() === 'seconds') {
                    newStartDate = moment($scope.options.anchorStartDate).clone().milliseconds(0).toDate();
                    newEndDate = moment($scope.options.anchorEndDate).clone().milliseconds(999).toDate();
                } else if (getTimeUnits() === 'minutes') {
                    newStartDate = moment($scope.options.anchorStartDate).clone().seconds(0).milliseconds(0).toDate();
                    newEndDate = moment($scope.options.anchorEndDate).clone().seconds(59).milliseconds(999).toDate();
                } else if (getTimeUnits() === 'hours') {
                    newStartDate = moment($scope.options.anchorStartDate).clone().minutes(0).seconds(0).milliseconds(0).toDate();
                    newEndDate = moment($scope.options.anchorEndDate).clone().minutes(59).seconds(59).milliseconds(999).toDate();
                } else if (getTimeUnits() === 'days') {
                    newStartDate = moment($scope.options.anchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).toDate();
                    newEndDate = moment($scope.options.anchorEndDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999).toDate();
                }

                if (newStartDate < moment($scope.options.minPossibleAnchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)) {
                    newStartDate = moment($scope.options.minPossibleAnchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).toDate();
                }
                if (newEndDate > moment($scope.options.maxPossibleAnchorEndDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999)) {
                    newEndDate = moment($scope.options.maxPossibleAnchorEndDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999).toDate();
                }

                $scope.options.maxSelectableAnchorStartDate = $scope.isSliding() ? newStartDate : moment($scope.options.anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).toDate();
                $scope.options.minSelectableAnchorStartDate = moment($scope.options.minPossibleAnchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).toDate();

                $scope.options.maxSelectableAnchorEndDate = moment($scope.options.maxPossibleAnchorEndDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999).toDate();
                $scope.options.minSelectableAnchorEndDate = $scope.isSliding() ? newEndDate : moment($scope.options.anchorStartDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999).toDate();
            }


            function updateDurationRange() {

                $scope.options.durationConstraints = {
                    min: 1
                };

                if (!$scope.options.anchorEndDate) {
                    return;
                }

                var anchorEndDate = moment($scope.options.anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).add($scope.options.anchorEndTime, 'milliseconds').toDate();

                if (anchorEndDate && $scope.options.minPossibleAnchorStartDate) {
                    var dateDiff;
                    if (getTimeUnits() === 'seconds') {
                        dateDiff = (moment(anchorEndDate).clone().milliseconds(999) - moment($scope.options.minPossibleAnchorStartDate).clone().milliseconds(0)) / 1000;
                    } else if (getTimeUnits() === 'minutes') {
                        dateDiff = (moment(anchorEndDate).clone().seconds(59).milliseconds(999) - moment($scope.options.minPossibleAnchorStartDate).clone().seconds(0).milliseconds(0)) / 60000;
                    } else if (getTimeUnits() === 'hours') {
                        dateDiff = (moment(anchorEndDate).clone().minutes(59).seconds(59).milliseconds(999) - moment($scope.options.minPossibleAnchorStartDate).clone().minutes(0).seconds(0).milliseconds(0)) / 3600000;
                    } else if (getTimeUnits() === 'days') {
                        dateDiff = (moment(anchorEndDate).clone().hours(23).minutes(59).seconds(59).milliseconds(999) - moment($scope.options.minPossibleAnchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0)) / 86400000;
                    }

                    if (getTimeUnits() !== 'seconds') {
                        $scope.options.durationConstraints.max = Math.floor(dateDiff);
                    } else {
                        $scope.options.durationConstraints.max = Math.ceil(dateDiff);
                    }
                }
            }

            function truncateToStartDate(date) {
                date.setHours(0);
                date.setMinutes(0);
                date.setSeconds(0);
                date.setMilliseconds(0);
                return date;
            }

            function truncateToEndDate(date) {
                date.setHours(23);
                date.setMinutes(59);
                date.setSeconds(59);
                date.setMilliseconds(999);
                return date;
            }

            function getInputTableDateRange() {
                $scope.showLoadingMask($scope.resources.scWindow_calculatingMaxDateMsg_txt);
                windowSvc.getDateRangeForWindow($scope.options.selectedPrimaryInput.window ? $scope.options.selectedPrimaryInput.window.id : -1, $scope.options.selectedPrimaryInput.table,
                    $scope.window.anchorSourceColumn).then(function (results) {
                        if (results.startDate) { //initialize the dates from the input table
                            var endDate = new Date(results.endDate);
                            var startDate = new Date(results.startDate);
                            setOptionsEndDateValue(specifiedDateTime() ? endDate : truncateToStartDate(endDate));
                            setOptionsStartDateValue(specifiedDateTime() ? startDate : truncateToStartDate(startDate));
                            $scope.options.tableMaxDate = new Date(results.endDate);
                            $scope.options.tableMinDate = new Date(results.startDate);
                            // Directive will remove time portion from anchorEndDate & anchorStartDate.  Save off time portion so can be used in dateTime calcs
                            setOptionsEndDateTimeValue(specifiedDateTime() ? moment.duration((moment($scope.options.anchorEndDate).clone() - moment($scope.options.anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0))).asMilliseconds() : 0);
                            setOptionsStartDateTimeValue(specifiedDateTime() ? moment.duration((moment($scope.options.anchorStartDate).clone() - moment($scope.options.anchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0))).asMilliseconds() : 0);
                            $scope.options.tableMaxTime = $scope.options.anchorEndTime;
                            $scope.options.tableMinTime = $scope.options.anchorStartTime;

                            setEndDateValue(new Date(results.endDate));
                            $scope.anchorDateTimeViews.oldAnchorEndDateTime = new Date(results.endDate);
                            setStartDateValue(new Date(results.startDate));
                            $scope.anchorDateTimeViews.oldAnchorStartDateTime = new Date(results.startDate);

                            initializePossibleDatesAndDuration($scope.options.anchorStartDate, $scope.options.anchorEndDate);

                            updateDurationRange();
                        }
                        validateAnchorEndDate();
                        $scope.hideLoadingMask();
                    });
            }


            function copyColumn(col, alias, table) {
                var aliasName = (alias + '_' + col.name).substring(0, 32);
                return {
                    originalColName: col.name,
                    tableFqid: table.fqid,
                    name: alias ? aliasName : col.name,
                    label: alias ? alias + '_' + col.label : col.label,
                    dataType: col.dataType,
                    dataLength: col.dataLength,
                    nullsAllowed: col.nullsAllowed,
                    columnFormat: col.columnFormat,
                    sasDate: col.sasDate,
                    sasDateTime: col.sasDateTime,
                    cnt: col.cnt
                };
            }

            function createWindowInput(primaryInput, joinInput, order) {
                var dim = (joinInput || {}).dimension;

                if (dim && dim.table) {
                    updateColumns(dim.table);
                }

                var wi = {
                    leftHandTable: primaryInput.type === 'table' ? primaryInput.table : null,
                    leftHandWindow: primaryInput.type !== 'table' ? primaryInput.window : null,
                    rightHandTable: dim && dim.type === 'table' ? dim.table : null,
                    rightHandWindow: dim && dim.type !== 'table' ? dim.window : null,
                    rightHandAlias: dim ? 't' + order : null,
                    joinType: dim ? {id: 2} : {id: 1},
                    order: order || 1,
                    joinColumns: []
                };

                if (wi.leftHandWindow) {
                    wi.leftHandWindow.outputTable = primaryInput.table;
                }
                if (wi.rightHandWindow) {
                    wi.rightHandWindow.outputTable = dim.table;
                }

                if (dim) {
                    wi.joinColumns[0] = {
                        leftColumn: joinInput.leftHandColumn,
                        rightColumn: joinInput.rightHandColumn
                    };
                }

                return wi;
            }

            function createJoinOutputTable(window) {
                var windowInputs = window.windowInputs,
                    primaryTable = windowInputs[0].leftHandTable || windowInputs[0].leftHandWindow.outputTable,
                    joinOutputTable = {
                        name: window.name + '_IN',
                        physicalName: window.name + '_IN',
                        libraryName: primaryTable.libraryName,
                        libraryFqid: primaryTable.libraryFqid,
                        folderPath: primaryTable.folderPath,
                        host: primaryTable.host,
                        port: primaryTable.port,
                        installLoc: primaryTable.installLoc,
                        lasrTag: primaryTable.lasrTag,
                        smpMode: primaryTable.smpMode,
                        role: 'JOIN_OUTPUT',
                        rowCount: primaryTable.rowCount,
                        columns: []
                    },
                    columns = joinOutputTable.columns;

                // Copy primary table columns
                angular.forEach(primaryTable.columns, function (col) {
                    columns.push(copyColumn(col, undefined, primaryTable));
                });

                // Copy columns from each right hand table
                angular.forEach(windowInputs, function (windowInput) {

                    var rightTable = windowInput.rightHandTable || windowInput.rightHandWindow.outputTable,
                        alias = windowInput.rightHandAlias;

                    angular.forEach(rightTable.columns, function (col) {
                        if (windowInput.joinColumns[0].rightColumn !== col) {
                            columns.push(copyColumn(col, alias, rightTable));
                        }
                    });
                });

                windowSvc.handleNameCollisions(columns);

                return joinOutputTable;
            }

            function validate() {
                $scope.showLoadingMask($scope.resources.scWindow_NewWindowDlg_validatingWindowMsg);

                $scope.invalidWindowNameMsg = null;

                if ($scope.window.name) {
                    return windowSvc.isUniqueName(0, $scope.window.name)
                        .then(function (result) {
                            var isValid = false;
                            // This highlights the validation widgets containing errors
                            if (result.isValid) {
                                isValid = dojoModules.registry.byId("newWindowForm").validate();
                            }
                            else {
                                $scope.invalidWindowNameMsg = result.invalidMessage;
                            }
                            return isValid;
                        });
                }
                else {
                    return dojoModules.registry.byId("newWindowForm").validate();
                }
            }

            /**
             * Determines if the window has an end anchored to a date and time
             * @param window
             * @returns {*|Window.anchorEnd|boolean}
             */
            function endAnchoredToDateTime(window) {
                return window.anchorSourceColumn && window.anchorSourceColumn.sasDateTime && window.anchorEnd && (window.anchorEnd.getTime() <= $scope.options.maxPossibleAnchorEndDate.getTime());
            }

            /**
             * Determines if the window has a start anchored to a date and time
             * @param window
             * @returns {*|Window.anchorStart|boolean}
             */
            function startAnchoredToDateTime(window) {
                return window.anchorSourceColumn && window.anchorSourceColumn.sasDateTime && window.anchorStart && (window.anchorStart.getTime() >= $scope.options.minPossibleAnchorStartDate.getTime());
            }

            function determineWindowType() {
                var windowType = '';
                if (!$scope.options.hasPartitionKeys) {
                    windowType = vsmCommonConst.WINDOW_TYPE_TRANSFORMATION;
                } else if ($scope.options.outputLevel && $scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_GROUPDATE) {
                    windowType = vsmCommonConst.WINDOW_TYPE_SLIDING;
                } else if ($scope.options.outputLevel && $scope.options.outputLevel.value === vsmCommonConst.WINDOW_OUTPUTLEVEL_TRANSACTION) {
                    windowType = vsmCommonConst.WINDOW_TYPE_ROLLING;
                } else {
                    windowType = vsmCommonConst.WINDOW_TYPE_AGGREGATION;
                }
                return windowType;
            }

            function updateColumns(table){

                var fqid = table.fqid;

                for(var i = 0; i < table.columns.length; i++){
                    table.columns[i].tableFqid = fqid;
                    table.columns[i].originalColumnName = table.columns[i].name;
                }
            }


            $scope.createWindow = function () {

                if ($scope.options.selectedJoinInputs.length === 1 && !$scope.options.selectedJoinInputs[0].dimension && !$scope.options.selectedJoinInputs[0].leftHandColumn && !$scope.options.selectedJoinInputs[0].rightHandColumn) {
                    $scope.options.selectedJoinInputs = [];
                }

                $q.when(validate()).then(function (isValid) {
                    if (isValid) { //Scrape and synthesize window property values from the options the user's chosen
                        $scope.showLoadingMask($scope.resources.scWindow_NewWindowDlg_creatingWindowMsg);
                        var window = angular.copy($scope.window);

                        window.type = determineWindowType();

                        // Clear any values not needed/relevant for the selected Window Type
                        switch (window.type) {
                            case vsmCommonConst.WINDOW_TYPE_TRANSFORMATION:
                                $scope.options.selectedPartitionVariables = null;
                                window.partitionKeyColumns = [];
                                window.anchorSourceColumn = null;
                                window.needsRerun = false;
                                window.entityTimeUnit = null;
                                window.anchorEnd = null;
                                window.startDate = null;
                                window.transactionKeyColumn = null;
                                setOptionsEndDateValue(null);
                                setOptionsStartDateValue(null);
                                $scope.options.duration = null;
                                $scope.options.includeMissing = true;
                                $scope.options.transactionKey = null;
                                break;
                            case vsmCommonConst.WINDOW_TYPE_AGGREGATION:
                                setOptionsStartDateValue(null);
                                $scope.options.duration = null;
                                break;
                            case vsmCommonConst.WINDOW_TYPE_ROLLING:
                                $scope.options.includeMissing = false;
                        }

                        window.anchorEnd = moment($scope.options.anchorEndDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).add($scope.options.anchorEndTime, 'milliseconds').toDate();
                        /*
                         if (endAnchoredToDateTime(window)) {
                         //The user has selected an end date/time whose default time is midnight (00:00:00.000),
                         //which has been determined to be something other than the max date of the dataset.
                         //We have to be sure to set the date/time to the very end of the day (hr, min, sec, ms).
                         window.anchorEnd.setHours(23);
                         window.anchorEnd.setMinutes(59);
                         window.anchorEnd.setSeconds(59);
                         window.anchorEnd.setMilliseconds(999);
                         }

                         if (startAnchoredToDateTime(window)) {
                         //The user has selected a start date/time
                         //which has been determined to be something other than the min date/time of the dataset.
                         //We have to be sure to set the date/time to the beginning of the day - midnight.
                         window.anchorStart.setHours(0);
                         window.anchorStart.setMinutes(0);
                         window.anchorStart.setSeconds(0);
                         window.anchorStart.setMilliseconds(0);
                         }
                         */
                        window.anchorStart = moment($scope.options.anchorStartDate).clone().hours(0).minutes(0).seconds(0).milliseconds(0).add($scope.options.anchorStartTime, 'milliseconds').toDate();

                        if ($scope.options && $scope.options.selectedPrimaryInput && $scope.options.selectedPrimaryInput.table) {
                            updateColumns($scope.options.selectedPrimaryInput.table);
                        }

                        var primaryInput = angular.copy($scope.options.selectedPrimaryInput);

                        if ($scope.options.selectedJoinInputs.length) {
                            angular.forEach($scope.options.selectedJoinInputs, function (joinInput, idx) {
                                var windowInput = createWindowInput(primaryInput, joinInput, idx + 1);
                                window.windowInputs.push(windowInput);
                            });
                            // join output
                            window.inputTable = createJoinOutputTable(window);
                        }
                        else {
                            var windowInput = createWindowInput(primaryInput);
                            window.windowInputs.push(windowInput);
                            window.inputTable = primaryInput.table;
                        }

                        $scope.hideLoadingMask();

                        var displayOrder = 0;
                        _($scope.options.partitionKeys).forEach(function (selectedPartitionKey) {
                            selectedPartitionKey.partitionKey = true;
                            window.partitionKeyColumns.push(selectedPartitionKey);
                            window.outputColumns.push(createOutputColumn(selectedPartitionKey, displayOrder++, true));
                        });

                        if ($scope.isSliding()) {
                            //Add the anchor date as an output column
                            var anchorDateCol = createOutputColumn($scope.window.anchorSourceColumn, displayOrder++, false);
                            window.outputColumns.push(anchorDateCol);
                        }

                        window.transactionKeyOptions = $scope.options.transactionKeyOptions;

                        window.transactionKeyColumn = $scope.options.transactionKey;

                        if (window.transactionKeyColumn) {
                            window.transactionKeyColumn.transactionKey = true;
                        }

                        if ($scope.isRolling()) {
                            //Add the transactionKeyColumn as an output column
                            var transKeyCol = createOutputColumn(window.transactionKeyColumn, displayOrder++, false);
                            window.outputColumns.push(transKeyCol);
                        }

                        if ($scope.isRolling()) {
                            window.includeMissing = false;
                        } else {
                            window.includeMissing = $scope.options.includeMissing;
                        }

                        window.duration = $scope.options.duration;

                        window.defaultNumericAggregation = $scope.options.defaultNumericAggregation ? $scope.options.defaultNumericAggregation.value : null;

                        window.defaultLookBackPeriod = $scope.options.defaultLookBackPeriod;

                        newWindowSvc.setWindow(window);

                        $scope.dialogFlags.showNewWindowDialog = false;

                        if ($location.path() === '/window/') {
                            $route.reload();
                        }
                        else {
                            $location.path('/window/');
                        }
                    }

                    $scope.hideLoadingMask();
                });
            };

            $scope.cancelDialog = function () {
                $scope.options.selectedJoinInputs = [];
                $scope.dialogFlags.showNewWindowDialog = false;
                $scope.options.selectedPartitionKeyNames = [];
                $scope.options.candidatePartitionKeys = [];
                $scope.options.selectedPartitionKeyNames = [];
                $scope.options.partitionKeys = [];
            };

            $scope.prevStep = function () {
                if ($scope.options.selectedJoinInputs.length === 1 && !$scope.options.selectedJoinInputs[0].dimension && !$scope.options.selectedJoinInputs[0].leftHandColumn && !$scope.options.selectedJoinInputs[0].rightHandColumn) {
                    --$scope.currentWizardIdx;
                }
                else if (validateVisibleFormElements()) {
                    --$scope.currentWizardIdx;
                }

                enableWizardControls();
            };

            $scope.nextStep = function () {
                var uniqueName = null,
                    deferred = null;

                if ($scope.currentWizardIdx === 0) {
                    uniqueName = validateUniqueWindowName();
                }
                else {
                    deferred = $q.defer();
                    deferred.resolve(true);
                    uniqueName = deferred.promise;
                }

                $q.when(uniqueName).then(function (isValid) {
                    if (isValid) {

                        if (!validateVisibleFormElements()) {
                            return;
                        }

                        if ($scope.currentWizardIdx === 0 && $scope.options.selectedJoinInputs.length === 0) {
                            $scope.addJoinInput();
                        }

                        ++$scope.currentWizardIdx;

                        enableWizardControls();
                    }
                });
            };

            function createOutputColumn(partitionKey, displayOrder, isPartitionKey) {
                var outputColumn = {};
                outputColumn.partitionKey = isPartitionKey;
                outputColumn.name = partitionKey.name;
                outputColumn.label = partitionKey.label;
                outputColumn.expression = partitionKey.name;
                outputColumn.derived = false;
                outputColumn.dataType = partitionKey.dataType;
                outputColumn.columnFormat = partitionKey.columnFormat;
                outputColumn.dataLength = partitionKey.dataLength;
                outputColumn.displayOrder = displayOrder;
                outputColumn.executionOrder = 0;
                outputColumn.lookbackPeriod = 0;
                outputColumn.sasDate = partitionKey.sasDate;
                outputColumn.sasDateTime = partitionKey.sasDateTime;
                return outputColumn;
            }

            function validateUniqueWindowName() {
                $scope.invalidWindowNameMsg = null;

                if ($scope.window.name) {
                    return windowSvc.isUniqueName(0, $scope.window.name)
                        .then(function (result) {
                            // This highlights the validation widgets containing errors
                            if (!result.isValid) {
                                $scope.invalidWindowNameMsg = result.invalidMessage;
                            }

                            return result.isValid;
                        });
                }
                else {
                    return true;
                }
            }


            function validateVisibleFormElements() {
                var isValid = true,
                    validWidget;
                var formWidgets = dojoModules.registry.byId("newWindowForm").getChildren();

                angular.forEach(formWidgets, function (widget) {
                    if (widget.domNode.offsetHeight !== 0 && widget.validate) {
                        validWidget = widget.validate();
                        if (!validWidget) {
                            widget.focus();
                        }
                        isValid = isValid && validWidget;
                    }
                });

                return isValid;
            }

            $scope.rightHandColumnChanged = function () {
                filterEntityKeyColumns();
            };

            $scope.validateJoinColumns = function (rightHandCol, idx) {
                var result = {isValid: false, invalidMessage: null};
                var leftHandCol = $scope.options.selectedJoinInputs[idx].leftHandColumn;
                if (leftHandCol && rightHandCol) {
                    if (leftHandCol.dataType === rightHandCol.dataType) {
                        if (leftHandCol.dataType === 'CHARACTER') {
                            if (leftHandCol.dataLength === rightHandCol.dataLength) {
                                result.isValid = true; // same length character
                            }
                            else {
                                result.isValid = false;
                                result.invalidMessage = dojoModules.resources.scWindow_newJoinColumnLengthMismatch_txt;
                            }
                        }
                        else {
                            result.isValid = true; // numeric
                        }
                    }
                    else {
                        result.isValid = false;
                        result.invalidMessage = dojoModules.resources.scWindow_newJoinColumnTypeMismatch_txt;
                    }
                }
                else if (rightHandCol) {
                    result.isValid = true;
                }

                if (!result.isValid) {
                    return result;
                }

                //Validate that selected key is valid
                //TODO: Make validation a property that can be set on config table
                /*
                $scope.showLoadingMask($scope.resources.scWindow_ValidateKey_txt);
                if (rightHandCol && $scope.options.selectedJoinInputs[idx]) {
                    var windowId = $scope.options.selectedJoinInputs[idx].dimension.window ? $scope.options.selectedJoinInputs[idx].dimension.window.id : undefined;
                    if (windowSvc.validateTransactionKeyColSync(windowId, $scope.options.selectedJoinInputs[idx].dimension.table, rightHandCol)) {
                        result.isValid = true;
                        result.invalidMessage = null;
                    } else {
                        result.isValid = false;
                        result.invalidMessage = dojoModules.resources.scWindow_InvalidKey_txt;
                    }
                } else {
                    $scope.hideLoadingMask();
                    return result;
                }
                $scope.hideLoadingMask();
                */
                return result;
            };


            function specifiedDateTime() {
                return $scope.window.anchorSourceColumn && $scope.window.anchorSourceColumn.sasDateTime;
            }

            function filterEntityKeyColumns() {
                if ($scope.data.entityKeyTable) {
                    angular.forEach($scope.options.selectedJoinInputs, function (joinInput) {
                        if (joinInput.dimension && joinInput.dimension.name === $scope.data.entityKeyTable.name) {
                            $scope.options.entityKeyColumns = _.filter($scope.data.entityKeyTable.table.columns, function (col) {
                                return col.name !== (joinInput.rightHandColumn || {}).name;
                            });
                        }
                    });
                }
            }

            $scope.updateEntityKeyColumns = function () {
                filterEntityKeyColumns();
            };

            $scope.$watch('dialogFlags.showNewWindowDialog', function (show) {
                if (show) {
                    resetForm();
                    loadLasrTables();
                }
            });

            $scope.$watch('options.partitionKeys', function () {
                enableWizardControls();
            });

            $scope.$watch('options.candidatePartitionKeys', function () {
                updateValidDateColumns();
                updateValidTransactionKeyColumns();
                enableWizardControls();
            });

            $scope.$watch('window.entityTimeUnit', function (new_val, old_val) {
                if (new_val && (new_val !== old_val) && $scope.options.anchorEndDate && $scope.options.anchorStartDate) {
                    updateDuration('changedTimeUnit');
                }
            });

            $scope.$watch('window.anchorSourceColumn', function (new_val, old_val) {
                $scope.options.defaultNumericAggregationTypes = $scope.options.nonDateBasedNumericAggregationTypes.slice(0);
                if (new_val) {
                    $scope.options.defaultNumericAggregationTypes.push.apply($scope.options.defaultNumericAggregationTypes, $scope.options.dateBasedNumericAggregationTypes);
                }
                if (new_val === null && ($scope.options.defaultNumericAggregation.value === 'FIRST' || $scope.options.defaultNumericAggregation.value === 'LAST')) {
                    $scope.options.defaultNumericAggregation = $scope.options.defaultNumericAggregationTypes[_.indexOf($scope.options.defaultNumericAggregationTypes, _.findWhere($scope.options.defaultNumericAggregationTypes, {'value': 'SUM'}))];
                }
            });

            /**
             * Invoked when the user selects or types a value for the window's end date.
             *
             * The "side-model" used to work around the sndDateTextBox's truncation of time is updated properly
             * with time granularity if needed.
             */
            $scope.anchorEndDateTimeChanged = function () {
                var new_val = $scope.anchorDateTimeViews.viewAnchorEndDateTime;  //This is the sngDateTextBox's model, *without* a time component
                if (new_val <= $scope.options.tableMaxDate &&
                    greaterThan(new_val, $scope.options.tableMinDate) &&
                    new_val >= $scope.anchorDateTimeViews.viewAnchorStartDateTime) {
                    //Then this is an acceptable date/time value
                    if (moment(new_val).startOf('day').isSame(moment($scope.options.tableMaxDate).startOf('day'))) {
                        //Insure that the window's anchor end time component is reset to include the time component (if any)
                        //of the original max anchor date
                        setOptionsEndDateValue(new Date($scope.options.tableMaxDate));
                    } else {
                        setOptionsEndDateValue(moment(new_val).endOf('day').toDate());
                    }
                    updateDurationAndEnableWizardControls('changedEndDate');
                } else {
                    //The user may have typed in a value, and circumvented the popup's legal selections
                    //Reset the anchor end date to the old value
                    setOptionsEndDateValue(new Date($scope.anchorDateTimeViews.oldAnchorEndDateTime.getTime()));
                    setEndDateValue(new Date($scope.anchorDateTimeViews.oldAnchorEndDateTime.getTime()));
                }
                $scope.anchorDateTimeViews.oldAnchorEndDateTime = $scope.options.anchorEndDate;
            };

            function greaterThan(newVal, minDateTime) {
                var newValDatePart = moment(newVal).startOf('day');
                var minDatePart = moment(minDateTime).startOf('day');
                return newValDatePart >= minDatePart;
            }

            function numberSpinnerHasFocus(){
                return $scope.activeWidget == "durationNumberSpinner";
            }

            //
            function setDisplayedStartTime(newValue){
                $scope.options.displayedStartTime = newValue;
            }

            function setDisplayedEndTime(newValue){
                $scope.options.displayedEndTime = newValue;
            }

            function setStartDateValue(newValue){
                    $scope.anchorDateTimeViews.viewAnchorStartDateTime = newValue;
            }

            function setOptionsEndDateValue(newValue){
                $scope.options.anchorEndDate = newValue;
            }

            function setOptionsStartDateValue(newValue){
                $scope.options.anchorStartDate = newValue;
            }

            function setOptionsStartDateTimeValue(newValue){
                $scope.options.anchorStartTime = newValue;
            }

            function setOptionsEndDateTimeValue(newValue){
                $scope.options.anchorEndTime = newValue;
            }

            function setEndDateValue(newValue){
                    $scope.anchorDateTimeViews.viewAnchorEndDateTime = newValue;
            }

            /**
             * Invoked when the user selects or types a value for the window's start date.
             *
             * The "side-model" used to work around the sndDateTextBox's truncation of time is updated properly
             * with time granularity if needed.
             */
            $scope.anchorStartDateTimeChanged = function () {
                if (!numberSpinnerHasFocus()  ) {
                    var new_val = $scope.anchorDateTimeViews.viewAnchorStartDateTime;  //This is the sngDateTextBox's model, *without* a time component
                    var selectionValid = true;
                    if ((new_val <= $scope.options.tableMaxDate) &&
                        greaterThan(new_val, $scope.options.tableMinDate) &&
                        new_val <= $scope.anchorDateTimeViews.viewAnchorEndDateTime) {
                        //Then this is an acceptable date/time value
                        if (moment(new_val).startOf('day').isSame(moment($scope.options.tableMinDate).startOf('day'))) {
                            //Insure that the window's anchor start time component is reset to include the time component (if any)
                            //of the original min anchor date
                            setOptionsStartDateValue(new Date($scope.options.tableMinDate));
                        } else {
                            setOptionsStartDateValue(moment(new_val).startOf('day').toDate());
                        }
                    } else {
                        //The user may have typed in a value, and circumvented the popup's legal selections
                        //Reset the anchor start date to the old value
                        setOptionsStartDateValue(new Date($scope.anchorDateTimeViews.oldAnchorStartDateTime.getTime()));
                        setStartDateValue(new Date($scope.anchorDateTimeViews.oldAnchorStartDateTime.getTime()));
                        selectionValid = false;
                    }
                    if (new_val && !numberSpinnerHasFocus()&& selectionValid) {
                        updateDurationAndEnableWizardControls('changedStartDate');
                    }
                    $scope.anchorDateTimeViews.oldAnchorStartDateTime = $scope.options.anchorStartDate;
                }
            };

            $scope.$watch('options.duration', function (new_val, old_val) {
                if (new_val !== old_val && new_val > 0 && new_val <= $scope.options.durationConstraints.max) {
                    updateStartDate();
                }
            });

            $scope.$watch('options.transactionKey', function (new_val, old_val) {
                if (new_val !== old_val) {
                    enableWizardControls();
                }
            });

            $scope.validateTransactionKeyColumn = function (new_val) {
                //TODO: Make validation a property that can be set on config table
                if(new_val) {
                    $scope.validTransactionKeys[new_val.name] = true;
                    return {isValid: true, invalidMessage: null};
                }

                if(!new_val) {
                    return {isValid: true, invalidMessage: null};
                }
/*
                function clearValidatorState() {
                    $scope.invalidTransactionKeyColumnMsg = null;
                    var transactionKeyValidator = dojoModules.registry.byId("windowTransactionKeyTextBox");
                    transactionKeyValidator.set('invalidMessage', '');
                    transactionKeyValidator._set('state', '');
                }

                if (new_val && $scope.validTransactionKeys.hasOwnProperty(new_val.name)) {
                    return {isValid: true, invalidMessage: null};
                }
                //TODO:  Investigate why loadingMask does not appear.
                $scope.showLoadingMask($scope.resources.scWindow_ValidateKey_txt);
                if (new_val && $scope.options.selectedPrimaryInput) {
                    var windowId = $scope.options.selectedPrimaryInput.window ? $scope.options.selectedPrimaryInput.window.id : undefined;
                    if (windowSvc.validateTransactionKeyColSync(windowId, $scope.options.selectedPrimaryInput.table, new_val)) {
                        clearValidatorState();
                        $scope.validTransactionKeys[new_val.name] = true;
                        $scope.hideLoadingMask();
                        return {isValid: true, invalidMessage: null};
                    } else {
                        $scope.hideLoadingMask();
                        return {isValid: false, invalidMessage: dojoModules.resources.scWindow_InvalidKey_txt};
                    }
                } else {
                    clearValidatorState();
                    $scope.hideLoadingMask();
                    return {isValid: true, invalidMessage: null};
                }
*/
            };
        }]
);



});

},
'vsd/window/new/newWindowSvc':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';

angular.module('vsd')
	.factory('newWindowSvc', function () {
		var window = null;
		return {
			setWindow:    function (w) {
				window = w;
			},
			getWindow:    function () {
				return window;
			},
			deleteWindow: function () {
				window = null;
			}
		};
	});

});

},
'vsd/appConfig':function(){
// wrapped by build app
define(["dojo","dijit","dojox"], function(dojo,dijit,dojox){
'use strict';
(function(sessionPeriod){
	var resetAutoSignoutTimer = angular.noop;

angular.module('vsd')
	.config([ '$httpProvider', '$parseProvider', function ($httpProvider, $parseProvider) {
		$httpProvider.interceptors.push(function () {
			return {
				'request': function (config) {
					resetAutoSignoutTimer();
					return config;
				}
			};
		});
		// FIXME: deprecated
		$parseProvider.unwrapPromises(true);
	}])
	.run([ '$rootScope','$http', '$timeout', '$window', 'dojoModules', 'confirmDialogSvc',
   function ($rootScope, $http, $timeout, $window, dojoModules, confirmDialogSvc) {

       $rootScope.$on("$locationChangeStart", function (event, next, current) {
           var nodes = $('.vsdEmptyContainerOnExit');
           nodes.empty();
           nodes.remove();
       });

	   var _noop = angular.noop,
		   _sessionPeriod = sessionPeriod || 1800, // 30 minutes
		   _warningPeriod = 300, // 5 minutes
		   _notifyUserPeriod = 0,
		   _timeout,
		   _autoSignoutTimeout,
		   _resources = dojoModules.resources;

	   resetAutoSignoutTimer = function () {
			$timeout.cancel(_timeout);
			_timeout = $timeout(showAutoSignoutDialog, _notifyUserPeriod);
		};

		if (_sessionPeriod && (_sessionPeriod - _warningPeriod) > 0) {
			_notifyUserPeriod = (_sessionPeriod - _warningPeriod) * 1000;
			_timeout = $timeout(showAutoSignoutDialog, _notifyUserPeriod);
		}

	   function showAutoSignoutDialog() {
		   confirmDialogSvc
			   .setTitle(_resources.app_title_txt)
			   .setMsg(_resources.app_session_timeout_txt)
			   .setConfirmLabel(_resources.session_extend_txt)
			   .setCancelLabel(_resources.session_signout_txt)
			   .setConfirmCallback(extendSession)
			   .setCancelCallback(signout)
			   .setCloseCallback(extendSession)
			   .showDialog();
		   _autoSignoutTimeout = $timeout(signout, 60000);
	   }

		function signout(){
			angular.element($window).off("beforeunload");
			$window.location = 'Logoff';
		}

		function extendSession(){
			$timeout.cancel(_autoSignoutTimeout);
			$http.get("action/extendSession");
		}

		angular.element($window).on("beforeunload", function(){
			return _resources.app_leaving_msg_txt;
		});

		// Disable backspace on non text elements to prevent back button behavior
		angular.element(document).on("keydown", function (evt) {
			if (evt.which === 8) { // backspace key code
				var nodeName = evt.target.nodeName.toUpperCase();
				if (nodeName === "INPUT") {
					var inputType = evt.target.type.toUpperCase();
					if (inputType !== "TEXT") {
						evt.preventDefault();
					}
				}
				else if (nodeName !== "TEXTAREA") {
					evt.preventDefault();
				}
			}
		});
}]);

})(sessionPeriod);

});

},
'url:dijit/templates/Tooltip.html':"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\"\n\t><div class=\"dijitTooltipConnector\" data-dojo-attach-point=\"connectorNode\"></div\n\t><div class=\"dijitTooltipContainer dijitTooltipContents\" data-dojo-attach-point=\"containerNode\" role='alert'></div\n></div>\n",
'url:dijit/templates/Menu.html':"<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" role=\"menu\" tabIndex=\"${tabIndex}\"\n\t   cellspacing=\"0\">\n\t<tbody class=\"dijitReset\" data-dojo-attach-point=\"containerNode\"></tbody>\n</table>\n",
'url:dijit/templates/MenuItem.html':"<tr class=\"dijitReset dijitMenuItem\" data-dojo-attach-point=\"focusNode\" role=\"menuitem\" tabIndex=\"-1\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<span role=\"presentation\" class=\"dijitInline dijitIcon dijitMenuItemIcon\" data-dojo-attach-point=\"iconNode\"></span>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" data-dojo-attach-point=\"containerNode,textDirNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" data-dojo-attach-point=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">\n\t\t<span data-dojo-attach-point=\"arrowWrapper\" style=\"visibility: hidden\">\n\t\t\t<span class=\"dijitInline dijitIcon dijitMenuExpand\"></span>\n\t\t\t<span class=\"dijitMenuExpandA11y\">+</span>\n\t\t</span>\n\t</td>\n</tr>\n",
'url:dijit/form/templates/TextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\" id=\"widget_${id}\" role=\"presentation\"\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n",
'url:dijit/form/templates/ValidationTextBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\" role=\"presentation\"\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" data-dojo-attach-point='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n",
'url:dijit/form/templates/DropDownBox.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\"\n\trole=\"combobox\"\n\taria-haspopup=\"true\"\n\tdata-dojo-attach-point=\"_popupStateNode\"\n\t><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer'\n\t\tdata-dojo-attach-point=\"_buttonNode\" role=\"presentation\"\n\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"button presentation\" aria-hidden=\"true\"\n\t\t\t${_buttonInputDisabled}\n\t/></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' ${!nameAttrSetting} type=\"text\" autocomplete=\"off\"\n\t\t\tdata-dojo-attach-point=\"textbox,focusNode\" role=\"textbox\"\n\t/></div\n></div>\n",
'url:dijit/templates/Calendar.html':"<table cellspacing=\"0\" cellpadding=\"0\" class=\"dijitCalendarContainer\" role=\"grid\" aria-labelledby=\"${id}_mddb ${id}_year\" data-dojo-attach-point=\"gridNode\">\n\t<thead>\n\t\t<tr class=\"dijitReset dijitCalendarMonthContainer\" valign=\"top\">\n\t\t\t<th class='dijitReset dijitCalendarArrow' data-dojo-attach-point=\"decrementMonth\" scope=\"col\">\n\t\t\t\t<span class=\"dijitInline dijitCalendarIncrementControl dijitCalendarDecrease\" role=\"presentation\"></span>\n\t\t\t\t<span data-dojo-attach-point=\"decreaseArrowNode\" class=\"dijitA11ySideArrow\">-</span>\n\t\t\t</th>\n\t\t\t<th class='dijitReset' colspan=\"5\" scope=\"col\">\n\t\t\t\t<div data-dojo-attach-point=\"monthNode\">\n\t\t\t\t</div>\n\t\t\t</th>\n\t\t\t<th class='dijitReset dijitCalendarArrow' scope=\"col\" data-dojo-attach-point=\"incrementMonth\">\n\t\t\t\t<span class=\"dijitInline dijitCalendarIncrementControl dijitCalendarIncrease\" role=\"presentation\"></span>\n\t\t\t\t<span data-dojo-attach-point=\"increaseArrowNode\" class=\"dijitA11ySideArrow\">+</span>\n\t\t\t</th>\n\t\t</tr>\n\t\t<tr role=\"row\">\n\t\t\t${!dayCellsHtml}\n\t\t</tr>\n\t</thead>\n\t<tbody data-dojo-attach-point=\"dateRowsNode\" data-dojo-attach-event=\"ondijitclick: _onDayClick\" class=\"dijitReset dijitCalendarBodyContainer\">\n\t\t\t${!dateRowsHtml}\n\t</tbody>\n\t<tfoot class=\"dijitReset dijitCalendarYearContainer\">\n\t\t<tr>\n\t\t\t<td class='dijitReset' valign=\"top\" colspan=\"7\" role=\"presentation\">\n\t\t\t\t<div class=\"dijitCalendarYearLabel\">\n\t\t\t\t\t<span data-dojo-attach-point=\"previousYearLabelNode\" class=\"dijitInline dijitCalendarPreviousYear\" role=\"button\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"currentYearLabelNode\" class=\"dijitInline dijitCalendarSelectedYear\" role=\"button\" id=\"${id}_year\"></span>\n\t\t\t\t\t<span data-dojo-attach-point=\"nextYearLabelNode\" class=\"dijitInline dijitCalendarNextYear\" role=\"button\"></span>\n\t\t\t\t</div>\n\t\t\t</td>\n\t\t</tr>\n\t</tfoot>\n</table>\n",
'url:dijit/form/templates/Button.html':"<span class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\n\t\tdata-dojo-attach-event=\"ondijitclick:__onClick\" role=\"presentation\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"titleNode,focusNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" data-dojo-attach-point=\"iconNode\"></span\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t></span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\n\t\tdata-dojo-attach-event=\"onclick:_onClick\"\n\t\ttabIndex=\"-1\" role=\"presentation\" data-dojo-attach-point=\"valueNode\"\n/></span>\n",
'url:dijit/form/templates/DropDownButton.html':"<span class=\"dijit dijitReset dijitInline\"\n\t><span class='dijitReset dijitInline dijitButtonNode'\n\t\tdata-dojo-attach-event=\"ondijitclick:__onClick\" data-dojo-attach-point=\"_buttonNode\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdata-dojo-attach-point=\"focusNode,titleNode,_arrowWrapperNode,_popupStateNode\"\n\t\t\trole=\"button\" aria-haspopup=\"true\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\n\t\t\t\tdata-dojo-attach-point=\"iconNode\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tdata-dojo-attach-point=\"containerNode\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdata-dojo-attach-event=\"onclick:_onClick\"\n\t\tdata-dojo-attach-point=\"valueNode\" role=\"presentation\"\n/></span>\n",
'url:vsd/common/message/VSDMessagesTemplate.html':"<div class='messagesClip globalDijit' data-dojo-attach-point='clipNode'>\n\t<div class='messagesContainer' data-dojo-attach-point='containerNode'>\n\t\t<div class='messageDiv' data-dojo-attach-point='messageDiv'>\n\t\t\t<span data-dojo-type='dijit/form/Button'\n\t\t\t      data-dojo-attach-event=\"onClick:closeMessage\"\n\t\t\t      data-dojo-attach-point='closeButton'\n\t\t\t      label=\"${resources.VSDMessages_closeBtn_txt}\"\n\t\t\t      showLabel='false'\n\t\t\t      iconClass='messageCloseButtonIcon'\n\t\t\t      class='messageCloseButton'></span>\n\t\t\t<div class='messageIcon' data-dojo-attach-point='messageIcon'></div>\n\t\t\t<div class='messageText' data-dojo-attach-point='messageText'></div>\n\t\t</div>\n\t</div>\n</div>\n",
'url:dijit/templates/TooltipDialog.html':"<div role=\"alertdialog\" tabIndex=\"-1\">\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\n\t\t<div class=\"dijitTooltipContents dijitTooltipFocusNode\" data-dojo-attach-point=\"containerNode\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\" data-dojo-attach-point=\"connectorNode\"></div>\n</div>\n",
'url:dijit/templates/MenuSeparator.html':"<tr class=\"dijitMenuSeparator\" role=\"separator\">\n\t<td class=\"dijitMenuSeparatorIconCell\">\n\t\t<div class=\"dijitMenuSeparatorTop\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n\t<td colspan=\"3\" class=\"dijitMenuSeparatorLabelCell\">\n\t\t<div class=\"dijitMenuSeparatorTop dijitMenuSeparatorLabel\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n</tr>\n",
'url:dijit/templates/Dialog.html':"<div class=\"dijitDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\">\n\t<div data-dojo-attach-point=\"titleBar\" class=\"dijitDialogTitleBar\">\n\t\t<span data-dojo-attach-point=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"\n\t\t\t\trole=\"heading\" level=\"1\"></span>\n\t\t<span data-dojo-attach-point=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" data-dojo-attach-event=\"ondijitclick: onCancel\" title=\"${buttonCancel}\" role=\"button\" tabindex=\"0\">\n\t\t\t<span data-dojo-attach-point=\"closeText\" class=\"closeText\" title=\"${buttonCancel}\">x</span>\n\t\t</span>\n\t</div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\n</div>\n",
'url:dijit/form/templates/CheckBox.html':"<div class=\"dijit dijitReset dijitInline\" role=\"presentation\"\n\t><input\n\t \t${!nameAttrSetting} type=\"${type}\" role=\"${type}\" aria-checked=\"false\" ${checkedAttrSetting}\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\n\t\tdata-dojo-attach-point=\"focusNode\"\n\t \tdata-dojo-attach-event=\"ondijitclick:_onClick\"\n/></div>\n",
'url:dijit/templates/TreeNode.html':"<div class=\"dijitTreeNode\" role=\"presentation\"\n\t><div data-dojo-attach-point=\"rowNode\" class=\"dijitTreeRow\" role=\"presentation\"\n\t\t><span data-dojo-attach-point=\"expandoNode\" class=\"dijitInline dijitTreeExpando\" role=\"presentation\"></span\n\t\t><span data-dojo-attach-point=\"expandoNodeText\" class=\"dijitExpandoText\" role=\"presentation\"></span\n\t\t><span data-dojo-attach-point=\"contentNode\"\n\t\t\tclass=\"dijitTreeContent\" role=\"presentation\">\n\t\t\t<span role=\"presentation\" class=\"dijitInline dijitIcon dijitTreeIcon\" data-dojo-attach-point=\"iconNode\"></span\n\t\t\t><span data-dojo-attach-point=\"labelNode,focusNode\" class=\"dijitTreeLabel\" role=\"treeitem\" tabindex=\"-1\" aria-selected=\"false\"></span>\n\t\t</span\n\t></div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitTreeNodeContainer\" role=\"presentation\" style=\"display: none;\"></div>\n</div>\n",
'url:dijit/templates/Tree.html':"<div role=\"tree\">\n\t<div class=\"dijitInline dijitTreeIndent\" style=\"position: absolute; top: -9999px\" data-dojo-attach-point=\"indentDetector\"></div>\n\t<div class=\"dijitTreeExpando dijitTreeExpandoLoading\" data-dojo-attach-point=\"rootLoadingIndicator\"></div>\n\t<div data-dojo-attach-point=\"containerNode\" class=\"dijitTreeContainer\" role=\"presentation\">\n\t</div>\n</div>\n",
'url:dijit/form/templates/Spinner.html':"<div class=\"dijit dijitReset dijitInline dijitLeft\"\n\tid=\"widget_${id}\" role=\"presentation\"\n\t><div class=\"dijitReset dijitButtonNode dijitSpinnerButtonContainer\"\n\t\t><input class=\"dijitReset dijitInputField dijitSpinnerButtonInner\" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t/><div class=\"dijitReset dijitLeft dijitButtonNode dijitArrowButton dijitUpArrowButton\"\n\t\t\tdata-dojo-attach-point=\"upArrowNode\"\n\t\t\t><div class=\"dijitArrowButtonInner\"\n\t\t\t\t><input class=\"dijitReset dijitInputField\" value=\"&#9650; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t\t\t${_buttonInputDisabled}\n\t\t\t/></div\n\t\t></div\n\t\t><div class=\"dijitReset dijitLeft dijitButtonNode dijitArrowButton dijitDownArrowButton\"\n\t\t\tdata-dojo-attach-point=\"downArrowNode\"\n\t\t\t><div class=\"dijitArrowButtonInner\"\n\t\t\t\t><input class=\"dijitReset dijitInputField\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t\t\t${_buttonInputDisabled}\n\t\t\t/></div\n\t\t></div\n\t></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' data-dojo-attach-point=\"textbox,focusNode\" type=\"${type}\" data-dojo-attach-event=\"onkeydown:_onKeyDown\"\n\t\t\trole=\"spinbutton\" autocomplete=\"off\" ${!nameAttrSetting}\n\t/></div\n></div>\n",
'url:dijit/form/templates/HorizontalSlider.html':"<table class=\"dijit dijitReset dijitSlider dijitSliderH\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\" data-dojo-attach-event=\"onkeydown:_onKeyDown, onkeyup:_onKeyUp\"\n\trole=\"presentation\"\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"topDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationT dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderDecrementIconH\" style=\"display:none\" data-dojo-attach-point=\"decrementButton\"><span class=\"dijitSliderButtonInner\">-</span></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderLeftBumper\" data-dojo-attach-event=\"press:_onClkDecBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><input data-dojo-attach-point=\"valueNode\" type=\"hidden\" ${!nameAttrSetting}\n\t\t\t/><div class=\"dijitReset dijitSliderBarContainerH\" role=\"presentation\" data-dojo-attach-point=\"sliderBarContainer\"\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"progressBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderProgressBar dijitSliderProgressBarH\" data-dojo-attach-event=\"press:_onBarClick\"\n\t\t\t\t\t><div class=\"dijitSliderMoveable dijitSliderMoveableH\"\n\t\t\t\t\t\t><div data-dojo-attach-point=\"sliderHandle,focusNode\" class=\"dijitSliderImageHandle dijitSliderImageHandleH\" data-dojo-attach-event=\"press:_onHandleClick\" role=\"slider\"></div\n\t\t\t\t\t></div\n\t\t\t\t></div\n\t\t\t\t><div role=\"presentation\" data-dojo-attach-point=\"remainingBar\" class=\"dijitSliderBar dijitSliderBarH dijitSliderRemainingBar dijitSliderRemainingBarH\" data-dojo-attach-event=\"press:_onBarClick\"></div\n\t\t\t></div\n\t\t></td\n\t\t><td class=\"dijitReset\"\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitSliderBumperH dijitSliderRightBumper\" data-dojo-attach-event=\"press:_onClkIncBumper\"></div\n\t\t></td\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitSliderButtonContainerH\"\n\t\t\t><div class=\"dijitSliderIncrementIconH\" style=\"display:none\" data-dojo-attach-point=\"incrementButton\"><span class=\"dijitSliderButtonInner\">+</span></div\n\t\t></td\n\t></tr\n\t><tr class=\"dijitReset\"\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t\t><td data-dojo-attach-point=\"containerNode,bottomDecoration\" class=\"dijitReset dijitSliderDecoration dijitSliderDecorationB dijitSliderDecorationH\"></td\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\n\t></tr\n></table>\n",
'url:dijit/layout/templates/TabContainer.html':"<div class=\"dijitTabContainer\">\n\t<div class=\"dijitTabListWrapper\" data-dojo-attach-point=\"tablistNode\"></div>\n\t<div data-dojo-attach-point=\"tablistSpacer\" class=\"dijitTabSpacer ${baseClass}-spacer\"></div>\n\t<div class=\"dijitTabPaneWrapper ${baseClass}-container\" data-dojo-attach-point=\"containerNode\"></div>\n</div>\n",
'url:dijit/layout/templates/_TabButton.html':"<div role=\"presentation\" data-dojo-attach-point=\"titleNode,innerDiv,tabContent\" class=\"dijitTabInner dijitTabContent\">\n\t<span role=\"presentation\" class=\"dijitInline dijitIcon dijitTabButtonIcon\" data-dojo-attach-point=\"iconNode\"></span>\n\t<span data-dojo-attach-point='containerNode,focusNode' class='tabLabel'></span>\n\t<span class=\"dijitInline dijitTabCloseButton dijitTabCloseIcon\" data-dojo-attach-point='closeNode'\n\t\t  role=\"presentation\">\n\t\t<span data-dojo-attach-point='closeText' class='dijitTabCloseText'>[x]</span\n\t\t\t\t></span>\n</div>\n",
'url:dijit/layout/templates/ScrollingTabController.html':"<div class=\"dijitTabListContainer-${tabPosition}\" style=\"visibility:hidden\">\n\t<div data-dojo-type=\"dijit.layout._ScrollingTabControllerMenuButton\"\n\t\t class=\"tabStripButton-${tabPosition}\"\n\t\t id=\"${id}_menuBtn\"\n\t\t data-dojo-props=\"containerId: '${containerId}', iconClass: 'dijitTabStripMenuIcon',\n\t\t\t\t\tdropDownPosition: ['below-alt', 'above-alt']\"\n\t\t data-dojo-attach-point=\"_menuBtn\" showLabel=\"false\" title=\"\">&#9660;</div>\n\t<div data-dojo-type=\"dijit.layout._ScrollingTabControllerButton\"\n\t\t class=\"tabStripButton-${tabPosition}\"\n\t\t id=\"${id}_leftBtn\"\n\t\t data-dojo-props=\"iconClass:'dijitTabStripSlideLeftIcon', showLabel:false, title:''\"\n\t\t data-dojo-attach-point=\"_leftBtn\" data-dojo-attach-event=\"onClick: doSlideLeft\">&#9664;</div>\n\t<div data-dojo-type=\"dijit.layout._ScrollingTabControllerButton\"\n\t\t class=\"tabStripButton-${tabPosition}\"\n\t\t id=\"${id}_rightBtn\"\n\t\t data-dojo-props=\"iconClass:'dijitTabStripSlideRightIcon', showLabel:false, title:''\"\n\t\t data-dojo-attach-point=\"_rightBtn\" data-dojo-attach-event=\"onClick: doSlideRight\">&#9654;</div>\n\t<div class='dijitTabListWrapper' data-dojo-attach-point='tablistWrapper'>\n\t\t<div role='tablist' data-dojo-attach-event='onkeydown:onkeydown'\n\t\t\t data-dojo-attach-point='containerNode' class='nowrapTabStrip'></div>\n\t</div>\n</div>",
'url:dijit/layout/templates/_ScrollingTabControllerButton.html':"<div data-dojo-attach-event=\"ondijitclick:_onClick\" class=\"dijitTabInnerDiv dijitTabContent dijitButtonContents\"  data-dojo-attach-point=\"focusNode\" role=\"button\">\n\t<span role=\"presentation\" class=\"dijitInline dijitTabStripIcon\" data-dojo-attach-point=\"iconNode\"></span>\n\t<span data-dojo-attach-point=\"containerNode,titleNode\" class=\"dijitButtonText\"></span>\n</div>",
'*now':function(r){r(['dojo/i18n!*preload*dojo/nls/vsd*["ar","ca","cs","da","de","de-de","el","en-gb","en-us","es-es","fi-fi","fr","fr-fr","he-il","hu","it-it","ja","ja-jp","ko","ko-kr","nl-nl","no","pl","pl-pl","pt-br","pt-pt","ru","sk","sl","sv","th","tr","zh-hk","zh-tw","zh-cn","zh-cn-xx","ROOT"]']);}
}});
(function(){
	// must use this.require to make this work in node.js
	var require = this.require;
	// consume the cached dojo layer
	require({cache:{}});
	!require.async && require(["dojo"]);
	require.boot && require.apply(null, require.boot);
})();
